/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	cvtaudio.c - test jig for auido library
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/cvtaudio.c,v 1.16 1995/08/21 04:04:07 toddb Exp $
 */
#define NeedFunctionPrototypes 1
#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include "nvrTypes.h"
#include "osio.h"
#include "nvrAlloc.h"
#include "nvrTimer.h"
#include "nvrString.h"
#include "longjump.h"
#include "nvrReason.h"
#include "nvrAudio.h"
#include "nvrDebug.h"
#include "process.h"
#include "NvrLmFeatures.h"

#define UsageMacro(args)	{ Msg args; Usage(); }

#if DEBUG_ENABLED

extern FlagNames AmpegdecompDebugNames;
extern FlagMacroList AmpegdecompDebugMacros;

FlagLabels DebugFlagDesc[] = {
    { &AmpegdecompDebugNames, &AmpegdecompDebugMacros },
    { NULL },
};

#else /* DEBUG_ENABLED */

FlagLabels DebugFlagDesc[] = { NULL };

#endif /* DEBUG_ENABLED */

char *ProgramName;

typedef struct _cvtParams {
    /*
     * input params
     */
    char *inputFile;
    AudioQuality aQuality;	/* used if rawInConf.type != FT_Unknown */
    AudioFileConfig rawInConf;	/* used if rawInConf.type == FT_Unknown */

    /*
     * output params
     */
    char *outputFile;
    AudioFileConfig outConf;
} CvtParams, *CvtParamsPtr;

static void
Usage(void)
{
    Msg("Usage: %s [attribute] ... input output\n", ProgramName);
    Msg("    %s -usage\n", ProgramName);
    Msg("will show a complete usage\n");
    NvrLmExit(1);
}

static void
LicenseDeath()
{
    if (TIME_TO_DIE) {
	ProgMsg("This alpha/beta version has expired.\n");
	ProgMsg("Please contact North Valley Research for a new copy.\n");
    }
    NvrLmExit (0);
}

static void
Help(void)
{
    Msg("Usage: %s [attribute] ... input output\n", ProgramName);
    Msg("\n");
    Msg("where the attributes are any of rate, channels, bits, type\n");
    Msg("There are various arguments for each attribute and are listed\n");
    Msg("below.\n");
    Msg("    rate=N\n");
    Msg("	N is the number of audio frames per second; e.g.\n");
    Msg("         rate=44100\n");
    Msg("         rate=22500\n");
    Msg("    bits=N\n");
    Msg("	N is the number of bits per sample; e.g. bits=16.\n");
    Msg("	A special form of this is \"ulaw\" or \"mulaw\",\n");
    Msg("	which means 8-bit mu-law.  The channels must be 1.\n");
    Msg("    channels=N\n");
    Msg("	N is the number of channels; e.g. channels=2\n");
    Msg("    quality=Q\n");
    Msg("	Q is real number between 0.0 and 1.0; e.g. quality=.50\n");
    Msg("	The default quality is 1.0\n");
    Msg("    type=Name\n");
    Msg("	Name is one of \"aiff\", \"au\".\n");
    Msg("\n");
    Msg("Input files may be aiff, aifc (not compressed), au (sun audio),\n");
    Msg("mpeg audio, mpeg system (with audio inside), or parallax movie\n");
    Msg("files.\n");
    NvrLmExit(0);
}

static void LicenseExceptionCB(int condition, char *feature);

static FeatureName Features[] = 
    { NVRLM_MPEGDECOMP, NVRLM_MPEG_COMP, NVRLM_MPEG_LITE, NVRLM_ALL_SOFTWARE };
static LmConfig LmConfigData = {
    Features,
    True,	/* do heartbeat */
    True,	/* shared */
    LicenseExceptionCB
};

/* ARGSUSED */
static void
LicenseExceptionCB(int condition,
		   char *feature)
{
    NvrLicenseError(condition, &LmConfigData, True);
}

static void
CheckLicense(StackInfo context, PrivPointer somethingImportant)
{
    TimeSpec now;
    int lm_code;

    TimeNow(&now);
    lm_code = NvrGetLicense(&LmConfigData);
    if (lm_code != LM_OK
     || (TIME_TO_DIE && now.seconds > TIME_TO_DIE)) {
	NvrLicenseError(lm_code, &LmConfigData, False);
	/*
	 * Here we should fill main() with noop instructructions so that
	 * if someone finds the LicenseDeath() routine and disables
	 * it, its too late.  Instead, we try to subtly screw up the
	 * data collected in mp.
	 */
	/* memcpy((char *)pipe, ((char *)pipe) + 1, sizeof(ImagePipeline)-1); */
	longjmp(context, 1);
    }
}

static AudioFileConfig InitialConfig = {
    -2,	/* unspecified */
    {
	AudioRateUnspecified,
	AudioChannelsUnspecified,
	AudioDataUnspecified
    }
};

typedef enum {
    AAUnknown,
    AAType,
    AARate,
    AAChannels,
    AABits,
    AAQuality
} AudioAttr;

NameEnumPair AudioAttrNames[] = {
    { "channels", AAChannels },
    { "rate",     AARate },
    { "bits",     AABits },
    { "type",     AAType },
    { "quality",  AAQuality },
    { NULL,       AAUnknown },
};

static InternalReasonPtr
ParseArgs(CvtParamsPtr params, int argc, char **argv)
{
    StringArgListPtr argList = NULL;
    boolean outputArgs = True;
    AudioAttr attrName;
    InternalReasonPtr reason = NULL;
    int i;

    argList = AllocStringArgList(argc, argv);
    if (argList == NULL)
	return &IRBadAllocation;
    if (argList->length == 0
     && argc >= 2
     && strcmp(argv[1], "-usage") == 0)
	Help();
    if (argList->length != argc - 1)
	UsageMacro(("Some flags were not recognized.\n"));
        
    for ( ; argList->index < argList->length; argList->index++) {
        i = argList->index;
        attrName = NameToEnum(argList->list[i].name, AudioAttrNames);
        switch (attrName) {
	case AAUnknown:
	    if (params->inputFile == NULL) {
		params->inputFile = argList->list[i].name;
		outputArgs = False;
	    } else if (params->outputFile == NULL) {
		params->outputFile = argList->list[i].name;
		outputArgs = True;
	    } else {
		reason = CreateReason("\"%s\": Is this a third file name?\n",
		    argList->list[i].name);
	    }
            break;

	case AAType:
            if (argList->list[i].value == NULL)
                goto missingArg;
	    if (outputArgs) {
		params->outConf.type =
		    NameToEnum(argList->list[i].value, AudioFileTypeNames);
	    } else {
		params->rawInConf.type =
		    NameToEnum(argList->list[i].value, AudioFileTypeNames);
	    }
	    break;

	case AARate:
            if (argList->list[i].value == NULL)
                goto missingArg;
	    if (outputArgs) {
		params->outConf.au.rate = 
		    NameToEnum(argList->list[i].value, AudioRateNames);
		/* undocumented 8k mpeg decode coersion */
		if (params->outConf.au.rate == Audio8Khz) {
		    params->aQuality.outputFormats =
			Avail8Khz | AvailBitMask | AvailChannelMask;
		}
	    } else {
		params->rawInConf.au.rate = 
		    NameToEnum(argList->list[i].value, AudioRateNames);
	    }
	    break;

	case AAChannels:
            if (argList->list[i].value == NULL)
                goto missingArg;
	    if (outputArgs) {
		params->outConf.au.channels = 
		    NameToEnum(argList->list[i].value, AudioChannelNames);
	    } else {
		params->rawInConf.au.channels = 
		    NameToEnum(argList->list[i].value, AudioChannelNames);
	    }
	    break;

	case AABits:
            if (argList->list[i].value == NULL) {
	    missingArg:
                reason = CreateReason("missing argument for \"%s\"\n",
		    argList->list[i].name);
		break;
	    }
	    if (outputArgs) {
		params->outConf.au.dataForm = 
		    NameToEnum(argList->list[i].value, AudioBitNames);
	    } else {
		params->rawInConf.au.dataForm = 
		    NameToEnum(argList->list[i].value, AudioBitNames);
	    }
	    break;

	case AAQuality:
	    if (argList->list[i].value == NULL
		|| !isdigit(argList->list[i].value[0]))
                goto missingArg;
            params->aQuality.decodeQuality = atof(argList->list[i].value);
        }

        if (reason) {
            break;
        }
    }
    return(reason);
}

void
main(int argc, char **argv)
{
    InternalReasonPtr reason = NULL;
    StackInfo context;
    AudioFilePtr input = NULL;
    AudioFilePtr output = NULL;
    long samplesOut, samplesIn;
    char fromForm[128], toForm[128];
    CvtParams params;
    AudioConversionPtr cvt;

    memset((char *)&params, 0, sizeof(params));
    params.aQuality.decodeQuality = 1.0;
    params.aQuality.outputFormats = ~0;
    params.outConf = InitialConfig;
    params.rawInConf = InitialConfig;

    SetDebugFileFlags(DebugFlags, DebugFlagDesc, "cvtaudio.debug");
    AcquireMacArgs("cvtaudio", &argc, &argv);
    ProgramName = argv[0];

    if (SaveContext(context)) {
	LicenseDeath();
    }

    NvrCopyrightMsg(  NvrCopyright
                    | LmCopyright
                    | CopyrightToTty,
                    0);

    if (reason = ParseArgs(&params, argc, argv))
	goto done;
    if (params.inputFile == NULL)
	UsageMacro(("Input file not specified.\n"));
    if (params.outputFile == NULL)
	UsageMacro(("Output file not specified.\n"));

    /*
     * Check for License
     */
    CheckLicense(context, 0);

    switch ((int)params.rawInConf.type) {
    case FT_Unknown:
	input = OpenRawAudioFile(params.inputFile,
				 &params.rawInConf.au, &reason);
	break;
    default:
	UsageMacro(("%s.\n%s.\n",
		    "Only raw may be specified for input files",
		    "Other file types are discovered from the header"));
	break;
    case -2:
	input = OpenAudioFile(params.inputFile, &params.aQuality, &reason);
	break;
    }
    if (input == NULL)
	goto done;
	
    switch (params.outConf.au.dataForm) {
    case AudioDataUnknown:
	UsageMacro(("unknown bits specified\n"));
	break;
    case AudioDataUnspecified:
	params.outConf.au.dataForm = input->au.dataForm;
	break;
    default:
	break;
    }

    switch (params.outConf.au.channels) {
    case AudioChannelsUnknown:
	UsageMacro(("unknown channels specified\n"));
	break;
    case AudioChannelsUnspecified:
	params.outConf.au.channels = input->au.channels;
	break;
    default:
	break;
    }

    switch (params.outConf.au.rate) {
    case AudioRateUnknown:
	UsageMacro(("unknown rate specified\n"));
	break;
    case AudioRateUnspecified:
	params.outConf.au.rate = input->au.rate;
	break;
    default:
	break;
    }

    switch (params.outConf.type) {
    case FT_Unknown:
	output = CreateRawAudioFile(params.outputFile,
				    &params.outConf.au, &reason);
	break;
    default:
	output = CreateAudioFile(params.outputFile, &params.outConf, &reason);
	break;
    }
    if (output == NULL)
	goto done;

    cvt = CreateAudioConversion(&input->au, &output->au, &reason);
    if (cvt == NULL)
	goto done;

    Msg("Converting from %s/%s to %s/%s\n",
	EnumToName(input->u.fileType, AudioFileTypeNames),
        AudioAttrName(&input->au, fromForm),
	EnumToName(output->u.fileType, AudioFileTypeNames),
        AudioAttrName(&output->au, toForm));

    if (reason = AllocMaxAudioBuffer(cvt, MicroSecPerSec/4))
	goto done;

    for (;;) {
	samplesIn = ReadAudioSamples(input, cvt->buffer, cvt->maxInSamples);
	if (samplesIn <= 0 || input->reason) {
	    reason = input->reason;
	    break;
	}

	if (cvt->convIndex != NoConversion) {
	    samplesIn =
		ConvertAudioBuffer(cvt, samplesIn, &reason);
	    if (reason)
		goto done;
	}

	samplesOut = WriteAudioSamples(output, cvt->buffer, samplesIn);
	if (samplesOut < 0 || output->reason) {
	    reason = output->reason;
	    break;
	}
    }
	
done:
    if (input)
	CloseAudioFile(input);
    if (output)
	CloseAudioFile(output);
    if (reason)
	TtyShowReason(reason);
    NvrLmExit (reason != NULL);
}
