/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	fileinfo.c - main program for printing file info
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/fileinfo.c,v 1.5 1995/09/07 01:39:11 russ Exp $
 */
#include "nvrTypes.h"
#include "osio.h"
#include "nvrReason.h"
#include "nvrAudio.h"
#include "nvrImage.h"
#include "nvrDebug.h"
#include "fileContent.h"
#include "NvrLmFeatures.h"
#include "process.h"

#define UsageMacro(args)                                \
    {                                                   \
        Msg args;                                       \
        Usage();                                        \
    }

#if defined(TIMER_ENABLED) || defined(DEBUG_ENABLED)

FlagLabels DebugFlagDesc[1];

#else /* TIMER_ENABLED || DEBUG_ENABLED */

FlagLabels DebugFlagDesc[1];

#endif /* TIMER_ENABLED || DEBUG_ENABLED */

char *ProgramName;
static int MaxPathLength;
static char PrefixString[256];

static void
Usage()
{
    Msg("Usage: %s [-a] file ...\n", ProgramName);
    exit(1);
}

static void
ShowAudio(char *file, FileContentType type)
{
    AudioFilePtr auFile;
    InternalReasonPtr reason;
    char *typeName = EnumToName(type, AllFileTypes);
    char formName[ 256 ];

    if (typeName == NULL)
	typeName = "?type";
    switch (type) {
    case FT_MpegSystem:
	NormalOutput("%s (audio), ");
	break;
    default:
	NormalOutput("%s, ", typeName);
	break;
    }

    auFile = OpenAudioFile(file, NULL, &reason);
    if (auFile == NULL) {
	TtyShowReason(reason);
	FreeReason(reason);
	return;
    }
    NormalOutput("%s\n", AudioAttrName(&auFile->au, formName));
    CloseAudioFile(auFile);
}

static void
ShowVideo(char *file, FileContentType type)
{
    char *typeName;
    ImageFileInfo imageInfo;
    InternalReasonPtr reason;

    reason = GetImageFileInfo(file, FT_Unknown, &imageInfo);
    if (reason) {
	TtyShowReason(reason);
	FreeReason(reason);
	return;
    }

    typeName = EnumToName(type, AllFileTypes);
    if (typeName == NULL)
	typeName = "?type";
    switch (type) {
    case FT_MpegSystem:
	NormalOutput("%s (video), ");
	break;
    default:
	NormalOutput("%s, ", typeName);
	break;
    }
    NormalOutput("%dx%d", imageInfo.width, imageInfo.height);
    if (imageInfo.interlaced)
	NormalOutput(", interlaced");
    if (imageInfo.defined & IFI_FrameCount)
	NormalOutput("\n%s%d frames", PrefixString, imageInfo.frameCount);
    if (imageInfo.defined & IFI_FrameRate)
	NormalOutput(", %.3ffps", imageInfo.framesPerSecond);
    if (imageInfo.defined & IFI_BitRate) {
	if (imageInfo.bitRate == -1)
	    NormalOutput(", variable bitrate");
	else
	    NormalOutput(", %.3fMbits/sec", imageInfo.bitRate / 1000000.0);
    }
    NormalOutput("\n");
}

void
main(int argc, char **argv)
{
    InternalReasonPtr reason = NULL;
    char *file;
    boolean allInfo = False;
    FileInfo fileInfo;
    int pathLength;
    int nFiles;
    int tmpArgc;
    char **tmpArgv;

    AcquireMacArgs("fileinfo", &argc, &argv);
    ProgramName = argv[0];

    NvrCopyrightMsg(  NvrCopyright
                    | CopyrightToTty,
                    0);

    tmpArgc = argc;
    tmpArgv = argv;
    for (tmpArgc--, tmpArgv++; tmpArgc; tmpArgc--, tmpArgv++) {
        if (strcmp(*tmpArgv, "-a") == 0) {
	    allInfo = True;
        } else if (**tmpArgv == '-') {
	    UsageMacro(("unrecognized flag: %s\n", *tmpArgv));
	} else {
	    pathLength = strlen(*tmpArgv);
	    if (pathLength > MaxPathLength)
		MaxPathLength = pathLength;
	}
    }
    sprintf(PrefixString, "%*s", MaxPathLength + 2, "");

    /*
     * Now process each file in turn.
     */
    nFiles = 0;
    tmpArgc = argc;
    tmpArgv = argv;
    for (tmpArgc--, tmpArgv++; tmpArgc; tmpArgc--, tmpArgv++) {
        if (**tmpArgv == '-')
	    continue;
	nFiles++;
	file = *tmpArgv;
	NormalOutput("%*s: ", MaxPathLength, file);

	reason = GetFileContentInfo(file, &fileInfo);
	if (reason) {
	    TtyShowReason(reason);
	    FreeReason(reason);
	    continue;
	}

	switch (fileInfo.efc) {
	case EFC_Compressed:    NormalOutput("(compressed) "); break;
	case EFC_GZipped:       NormalOutput("(gzipped) "); break;
	case EFC_NotCompressed: break;
	}

	switch (fileInfo.cClass) {
	case CC_AVMovie:
	    ShowVideo(file, fileInfo.type);
	    NormalOutput("%*s: ", MaxPathLength, "");
	    ShowAudio(file, fileInfo.type);
	    break;
	case CC_Video:
	    ShowVideo(file, fileInfo.type);
	    break;
	case CC_Audio:
	    ShowAudio(file, fileInfo.type);
	    break;
	case CC_Image:
	    ShowVideo(file, fileInfo.type);
	    break;
	default:
	    NormalOutput("\n");
	    break;
	}
    }
    if (nFiles == 0)
	UsageMacro(("No files specified\n"));

    exit(0);
}

