#include "alloc.h"
#include "osio.h"
#include <stdio.h>

typedef struct {
    int pixel;
    UChar red;
    UChar green;
    UChar blue;
    int count;
} RgbPixel;

char *ProgramName;

void Usage(void)
{
    Msg("Usage: %s width height rgbfile pix.c\n",
	ProgramName);
    exit(1);
}

main(int argc, char **argv)
{
    
    int width, height;
    int w, h, i;
    char *rgbFile, *pixFile;
    UChar *fileContents, *pInput;
    UChar *newPixmap, *pOutput;
    RgbPixel *pixels = NULL;
    int nUniquePixels = 0;
    int fd;
    int fileSize;
    FILE *fpOut;

    ProgramName = argv[0];
    if (argc != 5)
	Usage();

    width = atoi(argv[1]);
    height = atoi(argv[2]);
    rgbFile = argv[3];
    pixFile = argv[4];

    if ((fd = OpenFileForReading(rgbFile)) < 0)
	Fatal("can't open %s\n", rgbFile);

    if ((fpOut = fopen(pixFile, "w")) == NULL)
	Fatal("can't create %s\n", pixFile);

    fileSize = FdFileSize(fd);
    if (fileSize != width * height * 3)
	Fatal("File %s is %d bytes, but %dx%d implies %d bytes\n",
	    rgbFile, fileSize, width, height, width * height * 3);

    fileContents = (UChar *)SecureAlloc(fileSize);
    newPixmap = (UChar *)SecureAlloc(width*height);
    if (read(fd, fileContents, fileSize) != fileSize)
	Fatal("can't read all of %s\n", rgbFile);
    close(fd);

    pInput = fileContents;
    pOutput = newPixmap;
    for (h = 0; h < height; h++) {
	for (w = 0; w < width; w++) {
	    for (i = 0; i < nUniquePixels; i++) {
		if (pixels[i].red == pInput[0]
		 && pixels[i].green == pInput[1]
		 && pixels[i].blue == pInput[2]) {
		    pixels[i].count++;
		    goto foundPixel;
		}
	    }
	    if (++nUniquePixels >= 256) {
		Msg("\"%s\" (@%d,%d) has too many pixels (>256)\n%s\n",
		    rgbFile,
		    w, h,
		    "for the intended purpose\n");
		goto report;
	    }
	    pixels = (RgbPixel *)
		NullRealloc(pixels, nUniquePixels * sizeof(RgbPixel));
	    pixels[i].red = pInput[0];
	    pixels[i].green = pInput[1];
	    pixels[i].blue = pInput[2];
	    pixels[i].count = 1;

	foundPixel:
	    *pOutput++ = i;
	    pInput += 3;;
	}
    }

report:
    NormalOutput("%d pixel values:\n", nUniquePixels);
    for (i = 0; i < nUniquePixels; i++) {
	NormalOutput("%3d: %02x %02x %02x (%3d)\n",
	    i,
	    pixels[i].red,
            pixels[i].green,
            pixels[i].blue,
	    pixels[i].count);
    }

    fprintf(fpOut, "struct {\n");
    fprintf(fpOut, "  int nPixels;\n");
    fprintf(fpOut, "  int width;\n");
    fprintf(fpOut, "  int height;\n");
    fprintf(fpOut, "  struct {\n");
    fprintf(fpOut, "     unsigned char red;\n");
    fprintf(fpOut, "     unsigned char green;\n");
    fprintf(fpOut, "     unsigned char blue;\n");
    fprintf(fpOut, "     unsigned char pixel;\n");
    fprintf(fpOut, "  } colors[ %d ];\n", nUniquePixels);
    fprintf(fpOut, "  unsigned char pixels[ %d ];\n", width*height);
    fprintf(fpOut, "} %sPixels = {\n", pixFile);
    fprintf(fpOut, "  { %d, %d, %d,\n", nUniquePixels, width, height);
    fprintf(fpOut, "    {\n");
    for (i = 0; i < nUniquePixels; i++) {
	fprintf(fpOut, "      { %d, %d, %d },\n",
	    pixels[i].red,
            pixels[i].green,
            pixels[i].blue);
    }
    fprintf(fpOut, "    },\n");
    fprintf(fpOut, "    {\n");

    pOutput = newPixmap;
    for (i = 0; i < width*height; ) {
	fprintf(fpOut, "\t");
	if ((w = width*height - i) > 10)
	    w = 10;
	i += w;
	while (--w >= 0)
	    fprintf(fpOut, "%d,", *pOutput++);
	fprintf(fpOut, "\n");
    }
    fprintf(fpOut, "    }\n");
    fprintf(fpOut, "  }\n");
    fprintf(fpOut, "};\n");
    fclose(fpOut);
}
