/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	<file name> - <one-line description of file contents>
 *
 * DESCRIPTION
 *	<overall description of what the c code does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/MpegControl.c,v 1.4 1994/07/15 03:32:40 toddb Exp $
 */
#include "MpegControl.h"
#include "OnePlay.h"
#include "mcs-server.h"
#include "mpegdecomp.h"

#define DEFAULT_QUALITY 4

typedef struct _privMpegInfo {
    McsMpegDecompPtr	dMpeg;
    Boolean		running;
    Boolean		restart;
    Boolean		atEnd;
    Boolean		seenFirstReady;
    ULong		uninitFlags;
    ULong		changedFlags;
    Boolean		fileDefined;
    int			seqWidth;
    int			seqHeight;
    int			lastFrame;
    char		inPath[ MAX_PATH_LEN ];	/* audio/video input path */
    char		aInPath[ MAX_PATH_LEN ];/* audio-only input path */
    char		bitServer[ MAX_PATH_LEN ];
} PrivMpegInfo;

static PrivMpegInfo PrivMpeg;

static UChar DepthTypeToMcsSym[] =
      { 0,
	SYM_RGB_888,
	SYM_RGB_444,
	SYM_RGB_555,
	SYM_RGB_232,
	SYM_RGB_332,
	SYM_GRAY8,
	SYM_GRAY7,
	SYM_GRAY1,
	SYM_RGB_C555,
	SYM_RGB_C666,
      };

static void
MpegManageState(void)
{
    if (PrivMpeg.changedFlags || PrivMpeg.uninitFlags) {
	if (PrivMpeg.running) {
	    McsStopOperation(DevWidgetData.mcs.server);
	    return;
	}

	if (PrivMpeg.uninitFlags) {
	    McsUnsetParam(DevWidgetData.mcs.server, PrivMpeg.uninitFlags);
	    PrivMpeg.uninitFlags = 0;
	}

	if (PrivMpeg.changedFlags) {
	    McsSetParam(DevWidgetData.mcs.server,
			PrivMpeg.dMpeg,
			PrivMpeg.changedFlags);
	    PrivMpeg.changedFlags = 0;
	}
    }

    if (PrivMpeg.restart) {
	McsStartOperation(DevWidgetData.mcs.server);
	PrivMpeg.restart = False;
	PrivMpeg.running = True;
    }
}

static void
MpegHandleMcsEvent(XtCallbackProc quitCB, int *source, XtInputId* mcsID)
{
    char buf[ 1024 ];
    McsEventPtr evp;

    DevWidgetData.mcs.inputReady = False;
    while ((evp = McsGetNextEvent(DevWidgetData.mcs.server))) {
	switch(evp->eventType) {
	  case MCS_EVENT_PAUSED:
	  case MCS_EVENT_STOPPED:
	    DevWidgetData.mcs.incomplete = False;
	    DevWidgetData.mcs.lastKnownStopFrame = evp->u.number;
	    PrivMpeg.running = False;
	    PrivMpeg.atEnd = (evp->eventType == MCS_EVENT_STOPPED);
	    MpegManageState();
	    break;
	  case MCS_EVENT_FRAME:
	    break;
	  case MCS_EVENT_INFO:
	    McsFormatEvent(DevWidgetData.mcs.server, evp, buf);
	    BlindInformUser(DevWidgetData.mainWindow, buf);
	    break;
	  case MCS_EVENT_ICOMPRATIO:
	  case MCS_EVENT_BCOMPRATIO:
	  case MCS_EVENT_PCOMPRATIO:
	    break; /* we don't get these */
	  case MCS_EVENT_LOADING:
	  case MCS_EVENT_RUNNING:
	  case MCS_EVENT_BITRATE:
	    break; /* these don't do anything for us */
	  case MCS_EVENT_READY:
	    if (! PrivMpeg.seenFirstReady) {
		PrivMpeg.seenFirstReady = True;
		XtVaSetValues(DevWidgetData.videoDisplay,
			XnvrNactive, True,
			NULL);
	    }
	    break;
	  case MCS_EVENT_AUDIOBUSY:
	    sprintf(buf,
		"The audio device is in use.\n%s",
		"Try again when it is free.");
	    BlindInformUser(DevWidgetData.mainWindow, buf);
	    break;
	  case MCS_EVENT_SKIPPED:
	    /* skippedFrames = evp->u.number; */
	    break;
	  case MCS_EVENT_LATEFRAMES:
	    /* lateFrames = evp->u.number; */
	    break;
	  case MCS_EVENT_FRAMERATE:
	    /* frameRate = evp->u.number; */
	    break;
	  case MCS_BAD_FILE:
	    sprintf(buf, "That selected file is corrupt.\n%s",
		"Please remove it or choose another file.");
	    BlindInformUser(DevWidgetData.mainWindow, buf);
	    break;
	  case MCS_BAD_CONFIGURE:
	    FatalInformUser("Can't configure MCS correctly.", quitCB);
	    break;
	  case MCS_BAD_VALUE:
	  case MCS_BAD_WINDOW:
	  case MCS_BAD_DISPLAY:
	  case MCS_BAD_VISUAL:
	  case MCS_BAD_ASSIGNMENT:
	  case MCS_BAD_HARDWARE:
	  case MCS_BAD_WRITE:
	  case MCS_BAD_READ:
	  case MCS_BAD_RUNNING:
	  case MCS_BAD_ALLOC:
	  case MCS_BAD_SYNTAX:
	  case MCS_BAD_INTIME:
	  case MCS_BAD_AUDIO:
	  case MCS_BAD_PRELOAD:
	  case MCS_BAD_IMPLEMENTATION:
	  case MCS_BAD_MATCH:
	    PrivMpeg.running = False;
	    sprintf(buf, "The MCS server reported a problem:\n  ");
	    McsFormatEvent(DevWidgetData.mcs.server, evp, buf + strlen(buf));
	    FatalInformUser(buf, quitCB);
	    break;
	  case MCS_EVENT_SERVER_EXIT:
	    FatalInformUser("The MCS server has exited.", quitCB);
	    break;
	}
	McsFreeEvent(DevWidgetData.mcs.server, evp);
    }
}

static void
MpegMcsFdChange(McsServer server,
		int mcsFd,
		boolean needToAdd,
		PrivPointer unused)
{
    static XtInputId mcsId = 0;

    if (needToAdd) {
        if (mcsId) {
            ProgMsg("MpegMcsFdChange:: already connected to an fd\n");
        } else {
            /* ProgMsg("MpegMcsFdChange:: add %d\n", mcsFd); */
            mcsId = XtAppAddInput(DevWidgetData.appContext, mcsFd,
                    (XtPointer)XtInputReadMask,
                    (XtInputCallbackProc)MpegHandleMcsEvent,
                    (XtPointer)ReallyQuitCB);
        }
    } else {
        if (mcsId) {
            /* ProgMsg("MpegMcsFdChange:: remove %d\n", mcsFd); */
            XtRemoveInput(mcsId);
        } else {
            ProgMsg("MpegMcsFdChange:: asked to remove non-existant handler\n");
        }
    }
}

void
MpegInit(void)
{
    Visual	*visual;
    ULong	valueMask = 0;
    Position x;
    Position y;
    Dimension width;
    Dimension height;
    RgbDepthType depthType;
    Window decodeWindow;
    int availableDepths;

    DevWidgetData.mcs.inputReady = False;

    XtVaGetValues(DevWidgetData.videoDisplay,
	    XnvrNdepthType,	  &depthType,
	    XnvrNavailableDepths, &availableDepths,
	    NULL);

    /*
     * If a depth was specified in the resources and it is available
     * use it.  Same thing for the widget's resources.
     */
    if ((availableDepths & (1<<depthType)) == 0)
	depthType = DepthUnspecified;

    if (Resources.dMpegRsc.depth != DepthUnspecified
     && (availableDepths & (1<<Resources.dMpegRsc.depth)))
	depthType = Resources.dMpegRsc.depth;

    XtVaSetValues(DevWidgetData.videoDisplay,
		  XnvrNdepthType, depthType,
		  NULL);

    XtVaGetValues(DevWidgetData.videoDisplay,
	    XnvrNmcsServer,	  &DevWidgetData.mcs.server,
	    XmNx,		  &x,
	    XmNy, 		  &y,
	    XmNwidth,		  &width,
	    XmNheight,		  &height,
	    XmNvisual,		  &visual,
	    XnvrNdecodeWindow,	  &decodeWindow,
	    NULL);

    /*
     * Although the CreateWindow request has already returned
     * a window id, we have to make sure the X server knows
     * the id at the time the mcs server asks about it.
     */
    XFlush(XtDisplay(DevWidgetData.videoDisplay));

    PrivMpeg.dMpeg = &DevWidgetData.mcs.dMpeg;

    /*
     * Assign it, but don't hand it to SetParams, below.
     */
    PrivMpeg.dMpeg->inPath = PrivMpeg.inPath;
    PrivMpeg.dMpeg->aInPath = PrivMpeg.aInPath;
    PrivMpeg.dMpeg->bitServer = PrivMpeg.bitServer;

    if (Resources.bitServer[0]) {
	strcpy(PrivMpeg.bitServer, Resources.bitServer);
	valueMask |= MDBitServer;
    }

    PrivMpeg.dMpeg->x = 0;
    PrivMpeg.dMpeg->y = 0;
    if (Resources.dMpegRsc.mpegBig) {
	PrivMpeg.dMpeg->width = PrivMpeg.seqWidth * 2;
	PrivMpeg.dMpeg->height  = PrivMpeg.seqHeight * 2;
    } else {
	PrivMpeg.dMpeg->width = PrivMpeg.seqWidth;
	PrivMpeg.dMpeg->height  = PrivMpeg.seqHeight;
    }
    PrivMpeg.dMpeg->displayName =
	    DisplayString(XtDisplay(DevWidgetData.videoDisplay));
    PrivMpeg.dMpeg->windowId = decodeWindow;

    switch(Resources.dMpegRsc.destType) {
    case DestX:        PrivMpeg.dMpeg->destType = SYM_DPY_TO_WIN; break;
    case DestScreenFb: PrivMpeg.dMpeg->destType = SYM_DPY_TO_FB; break;
    case DestYcFb:     PrivMpeg.dMpeg->destType = SYM_DPY_TO_YCFB; break;
    case DestFile:     PrivMpeg.dMpeg->destType = SYM_YCTOFILE; break;
    }
    PrivMpeg.dMpeg->audioEnabled = True;
    PrivMpeg.dMpeg->videoEnabled = True;
    PrivMpeg.dMpeg->inputNotify = MpegMcsFdChange;
    PrivMpeg.dMpeg->audioDelay = Resources.audioDelay;
    PrivMpeg.dMpeg->color = DepthTypeToMcsSym[ depthType ];
    if( PrivMpeg.dMpeg->audioQuality == 0 ) {
        PrivMpeg.dMpeg->audioQuality = DEFAULT_QUALITY;
    }

    valueMask |= MDXcoord | MDYcoord | MDWidth | MDHeight |
		    MDDisplayName | MDWindowId | MDColor | MDAudioDelay |
		    MDDestType | MDAudioEnabled  |  MDVideoEnabled |
		    MDInputNotify | MDAudioQuality;

    if (PrivMpeg.dMpeg->preInit) {
	valueMask |= MDPreInit;
    }
    if (PrivMpeg.dMpeg->postInit) {
	valueMask |= MDPostInit;
    }
    McsDebugEnable(DevWidgetData.mcs.server,
		   Resources.showMcsProto,
		   Resources.showMcsProto);

    if (depthType == DepthUnspecified || decodeWindow == 0) {
	FatalInformUser("Can't configure for that depth.", ReallyQuitCB);
	return;
    }

    McsSetParam(DevWidgetData.mcs.server, PrivMpeg.dMpeg, valueMask);
}

void
MpegPlay()
{
    if (PrivMpeg.running)
	return;
    if ((PrivMpeg.restart = PrivMpeg.fileDefined) == 0)
	return;
    PrivMpeg.uninitFlags |= MDOutFrame | MDFrameRate;
    if (PrivMpeg.atEnd) {
	PrivMpeg.changedFlags |= MDInFrame;
	PrivMpeg.dMpeg->inFrame = 0;
    }
    MpegManageState();
}

static void
MpegCalcFrameSize()
{
    Dimension width, height;

    if (PrivMpeg.dMpeg->width != width
     || PrivMpeg.dMpeg->height != height) {
	PrivMpeg.changedFlags |= MDWidth | MDHeight;
	PrivMpeg.dMpeg->width = width;
	PrivMpeg.dMpeg->height = height;
    }
}

void
MpegSetFile(char *video, ImageFileInfo *vInfo, char *audio, FileInfo *aInfo)
{
    DevWidgetData.mcs.lastKnownStopFrame = 0;
    if (video) {
	strcpy(PrivMpeg.inPath, video);
	PrivMpeg.fileDefined = True;
	PrivMpeg.seqWidth = vInfo->width;
	PrivMpeg.seqHeight = vInfo->height;
	PrivMpeg.changedFlags |= MDInPath;
    } else {
	PrivMpeg.uninitFlags |= MDInPath;
    }

    if (audio) {
	strcpy(PrivMpeg.aInPath, audio);
	PrivMpeg.changedFlags |= MDAudioInPath;
    } else {
	PrivMpeg.uninitFlags |= MDAudioInPath;
    }
}

void
MpegSetDest(MpegDestination newDest)
{
    switch(newDest) {
    case DestX:        PrivMpeg.dMpeg->destType = SYM_DPY_TO_WIN; break;
    case DestScreenFb: PrivMpeg.dMpeg->destType = SYM_DPY_TO_FB; break;
    case DestYcFb:     PrivMpeg.dMpeg->destType = SYM_DPY_TO_YCFB; break;
    case DestFile:     PrivMpeg.dMpeg->destType = SYM_YCTOFILE; break;
    default:	       return;
    }
    
    PrivMpeg.changedFlags = MDDestType;
    MpegManageState();
}
