/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	OnePlay.c - play an mpeg file and quit
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/OnePlay.c,v 1.5 1994/10/03 02:00:35 toddb Exp $
 */
#include "OnePlay.h"
#include "process.h"

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/AtomMgr.h>
#include <Xm/Protocols.h>
#include <Xm/MwmUtil.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/DrawingA.h>

#ifndef MIN
#define MIN(x,y) ((x) > (y) ? (y) : (x))
#endif /* MIN */

#ifndef MAX
#define MAX(x,y) ((x) > (y) ? (x) : (y))
#endif /* MAX */

/* Define the application resources */
#define XtNgamma            "gamma"
#define XtCGamma            "Gamma"
#define XtNdestination      "destination"
#define XtCDestination      "Destination"
#define XtNshowMcsProto     "showMcs"
#define XtCShowMcsProto     "ShowMcs"
#define XtNaudioDelay       "audioDelay"
#define XtCAudioDelay       "AudioDelay"
#define XtNmpegBig	    "mpegBig"
#define XtCMpegBig	    "MpegBig"
#define XtNbitServer	    "bitServer"
#define XtCBitServer	    "BitServer"
#define XtNversion	    "version"
#define XtCVersion	    "Version"

DevWidgetInfo DevWidgetData;

char mpegDestName[MAX_STRING_LENGTH] =
#ifdef sun
    "screen frame buffer";
#else
    "X window";
#endif

static XtResource playerResources[] =
{
    {
        XtNshowMcsProto, XtCShowMcsProto, XtRBoolean, sizeof(Boolean),
        XtOffsetOf(ResourceInfo, showMcsProto), XtRImmediate, (XtPointer) False,
    },
    {
        XtNaudioDelay, XtCAudioDelay, XtRInt, sizeof(int),
        XtOffsetOf(ResourceInfo, audioDelay), XtRImmediate, (XtPointer) 1500,
    },
    {
        XtNmpegBig, XtCMpegBig, XtRBoolean, sizeof(Boolean),
        XtOffsetOf(ResourceInfo, dMpegRsc.mpegBig),
	XtRImmediate, (XtPointer) False,
    },
    {
        XtNbitServer, XtCBitServer, XtRString, sizeof(String),
        XtOffsetOf(ResourceInfo, bitServer), XtRString, (XtPointer) "",
    },
    {
        XtNversion, XtCVersion, XtRString, sizeof(String),
        XtOffsetOf(ResourceInfo, version), XtRString, (XtPointer) "",
    },
    {
        XnvrNdepthType, XnvrCDepthType, XtRInt, sizeof(int),
        XtOffsetOf(ResourceInfo, dMpegRsc.depth), XtRImmediate,
	(XtPointer) DepthUnspecified,
    },
    {
        XtNgamma, XtCGamma, XtRFloat, sizeof(float),
        XtOffsetOf(ResourceInfo, dMpegRsc.gamma), XtRString,
	(XtPointer) "1.116",
    },
    {
        XtNdestination, XtCDestination, XmRString, sizeof(String),
        XtOffsetOf(ResourceInfo, dMpegRsc.destName), XmRString,
	(XtPointer) mpegDestName
    },
    {
	"mcsPreInit", "McsPreInit", XmRString, sizeof(String),
	XtOffsetOf(ResourceInfo, mcsPreInit), XmRString,
	(XtPointer) NULL
    },
    {
	"mcsPostInit", "McsPostInit", XmRString, sizeof(String),
	XtOffsetOf(ResourceInfo, mcsPostInit), XmRString,
	(XtPointer) NULL
    },
};

static XrmOptionDescRec options[] = {
    {"-a", ".gamma", XrmoptionSepArg, NULL},
    {"-adelay", ".audioDelay", XrmoptionSepArg, NULL},
    {"-showmcs", ".showMcs", XrmoptionNoArg, "True"},
    {"-big", ".mpegBig", XrmoptionNoArg, "True"},
    {"-small", ".mpegBig", XrmoptionNoArg, "False"},

    {"-24bit", ".depthType", XrmoptionNoArg, "1"},
    {"-12bit", ".depthType", XrmoptionNoArg, "2"},
    {"-232",   ".depthType", XrmoptionNoArg, "4"},
    {"-332",   ".depthType", XrmoptionNoArg, "5"},
    {"-gray",  ".depthType", XrmoptionNoArg, "6"},
    {"-gray7", ".depthType", XrmoptionNoArg, "6"},
    {"-gray8", ".depthType", XrmoptionNoArg, "7"},
    {"-555",   ".depthType", XrmoptionNoArg, "9"},
    {"-666",   ".depthType", XrmoptionNoArg, "10"},

#ifdef broken
    {"-x", ".destination", XrmoptionNoArg, MpegDestNames[DestX] },
    {"-useX", ".destination", XrmoptionNoArg, MpegDestNames[DestX] },
    {"-useFB", ".destination", XrmoptionNoArg, MpegDestNames[DestScreenFb] },
    {"-useVideo", ".destination", XrmoptionNoArg, MpegDestNames[DestYcFb] },
#endif
};

typedef struct _privOnePlayInfo {
    char   	lmFeature[8];
    Widget	workWindow;
    int		seqWidth;
    int		seqHeight;
    char	bitServerName[ MAX_PATH_LEN ];	/* last known name */
} PrivOnePlayInfo;

static PrivOnePlayInfo OnePlayUI;

char *MpegDestNames[4] = {
    "X window",
    "screen frame buffer",
    "video frame buffer",
    "file",
};

/* Ugly globals that make so many things so much prettier */

ResourceInfo Resources;
char *ProgramName;

static String FallbackResources[] = {
    "NVROnePlay*videoDisplay.width : 320",
    "NVROnePlay*videoDisplay.height : 240",
    NULL
};

void
ReallyQuitCB(Widget w, XtPointer unused, XtPointer alsoUnused)
{
    exit(0);
}

void
FatalInformUser(char *msg, XtCallbackProc quit_cb)
{
    InformUser(DevWidgetData.mainWindow, msg,
        (XtCallbackProc)quit_cb,
        (XtPointer)DevWidgetData.videoDisplay,
        (XtCallbackProc)NULL, (XtPointer)NULL,
        (XtCallbackProc)NULL, (XtPointer)NULL);
}

static void
SetUIFrameSize()
{
    Dimension videoDpyWidth, videoDpyHeight;
    Dimension frameWidth, frameHeight;
    Dimension currentFrameWidth, currentFrameHeight;
    Dimension currentDpyWidth, currentDpyHeight;
    Boolean frameSizeChanged;

    if (Resources.dMpegRsc.mpegBig) {
        /* 4x mode */
        videoDpyWidth = OnePlayUI.seqWidth * 2;
        videoDpyHeight = OnePlayUI.seqHeight * 2;
    } else {
        /* 1x mode */
        videoDpyWidth = OnePlayUI.seqWidth;
        videoDpyHeight = OnePlayUI.seqHeight;
    }

    frameWidth = MAX(videoDpyWidth, MIN_VIDEODISPLAY_WIDTH);
    frameHeight = MAX(videoDpyHeight, MIN_VIDEODISPLAY_HEIGHT);
    XtVaGetValues(XtParent(DevWidgetData.videoDisplay),
            XmNwidth, &currentFrameWidth,
            XmNheight, &currentFrameHeight,
            NULL);
    XtVaGetValues(DevWidgetData.videoDisplay,
            XmNwidth, &currentDpyWidth,
            XmNheight, &currentDpyHeight,
            NULL);
    if (frameWidth != currentFrameWidth
     || frameHeight != currentFrameHeight) {
        frameSizeChanged = True;
        XtVaSetValues(XtParent(DevWidgetData.videoDisplay),
                XmNresizePolicy, XmRESIZE_ANY,
                XmNwidth, frameWidth,
                XmNheight, frameHeight,
                NULL);
        XtVaSetValues(XtParent(DevWidgetData.videoDisplay),
                XmNresizePolicy, XmRESIZE_NONE,
                NULL);
        XtVaGetValues(XtParent(DevWidgetData.videoDisplay),
                XmNwidth, &frameWidth,
                XmNheight, &frameHeight,
                NULL);
	/*
        if (Resources.dMpegRsc.debug)
            ProgMsg("frame %dx%d ==> %dx%d\n",
                currentFrameWidth, currentFrameHeight, frameWidth, frameHeight);
	 */
    } else
        frameSizeChanged = False;

    if (frameSizeChanged
     || videoDpyWidth != currentDpyWidth
     || videoDpyHeight != currentDpyHeight) {
        XtVaSetValues(DevWidgetData.videoDisplay,
                XmNx, (frameWidth - videoDpyWidth) / 2,
                XmNy, (frameHeight - videoDpyHeight) / 2,
                XmNwidth, videoDpyWidth,
                XmNheight, videoDpyHeight,
                NULL);
	/*
        if (Resources.dMpegRsc.debug)
            ProgMsg("video %dx%d ==> %dx%d+%d+%d\n",
                currentDpyWidth, currentDpyHeight,
                videoDpyWidth, videoDpyHeight,
                (frameWidth - videoDpyWidth) / 2,
                (frameHeight - videoDpyHeight) / 2);
	 */
    }
}

static void
ShowFileReason(InternalReasonPtr reason, char *path)
{
    char buf[1024];
    sprintf(buf, "Sorry,\n\n  %s\n\nappears to be bad:\n", path);
    FormatReason(reason, buf + strlen(buf));
    BlindInformUser(DevWidgetData.mainWindow, buf);
}

/* ARGSUSED */
static void
SpecifySourceFiles(char *video, char *audio)
{
    FileInfo audioInfo, videoInfo;
    ImageFileInfo imageInfo;
    char buf[1024];
    InternalReasonPtr reason;

    if (reason = GetFileContentInfo(video, &videoInfo)) {
	ShowFileReason(reason, video);
	return;
    }
    switch(videoInfo.type) {
    case FT_MpegVideo:
    case FT_MpegSystem:
	break;
    default:
	Fatal("Sorry,\n\n  %s\n\ndoesn't look like an mpeg video file\n",
	    video);
    }

    if (audio) {
	if (reason = GetFileContentInfo(audio, &audioInfo)) {
	    ShowFileReason(reason, audio);
	    return;
	}
	switch(audioInfo.type) {
        case FT_MpegAudio:
        case FT_MpegSystem:
        case FT_SunAudio:
        case FT_AiffAudio:
        case FT_AifcAudio:
	    break;
	default:
	    sprintf(buf, "Sorry,\n\n  %s\n\ndoesn't look like an audio file\n",
		audio);
	    BlindInformUser(DevWidgetData.mainWindow, buf);
	    return;
	}
    } else if (videoInfo.type == FT_MpegSystem) {
	audio = video;
	audioInfo = videoInfo;
    }

    if (reason = GetImageFileInfo(video, videoInfo.type, &imageInfo)) {
        ShowFileReason(reason, video);
        return;
    }

    OnePlayUI.seqWidth = imageInfo.width;
    OnePlayUI.seqHeight = imageInfo.height;
    SetUIFrameSize();
    MpegSetFile(video, &imageInfo, audio, &audioInfo);
}

static void
SetMpegColormapWindows()
{
    Window list[2];
    /*
     * We update the list of colormap windows here because
     * the mpeg widget's window may have changed.  We have to
     * use XSetWMColormapWindows() instead of XtSetWMColormapWindows()
     * because the decoder window is not a widget.
     */
    XtVaGetValues(DevWidgetData.videoDisplay,
	    XnvrNdecodeWindow, &list[0],
	    NULL);
    list[1] = XtWindow(DevWidgetData.topLevel);
    XSetWMColormapWindows(XtDisplay(DevWidgetData.videoDisplay),
			  XtWindow(DevWidgetData.topLevel),
			  list, XtNumber(list));
}

static void
InitCB(Widget w, XtPointer client_data, XtPointer alsoUnused)
{
    MpegInit();
    SetMpegColormapWindows();
    MpegPlay();
}

static void
MplayerVideoPaneCreate(Widget workWindow)
{
    Widget videoPane;
    Widget videoFrame;
    Widget videoDA;
    Widget mpegVideoDisplay;

    /*
     * Video Display window
     */
    videoPane = XtVaCreateWidget("mpegVideoPane",
	    xmFormWidgetClass,		workWindow,
	    XmNtopAttachment,		XmATTACH_FORM,
	    XmNrightAttachment,		XmATTACH_FORM,
	    XmNleftAttachment,		XmATTACH_FORM,
	    XmNbottomAttachment,	XmATTACH_FORM,
	    XmNrubberPositioning,	True,
	    NULL);
    videoFrame = XtVaCreateManagedWidget("videoFrame",
	    xmFrameWidgetClass,		videoPane,
	    XmNtopAttachment,		XmATTACH_FORM,
	    XmNleftAttachment,		XmATTACH_FORM,
	    XmNrightAttachment,		XmATTACH_FORM,
	    XmNshadowType,		XmSHADOW_IN,
	    XmNshadowThickness,		3,
	    /* if you don't turn off traversal, the motif mavigation
	     * mechanism gets stuck trying to traverse to the videoDisplay
	     * which, not being based on XmPrimitive, hasn't the 
	     * capability.
	     */
	    XmNtraversalOn,		False,
	    XmNnavigationType,		XmNONE,
	    NULL);
    videoDA = XtVaCreateManagedWidget("videoDA",
	    xmDrawingAreaWidgetClass,	videoFrame,
	    XmNmarginWidth,		0,
	    XmNmarginHeight,		0,
	    NULL);
    mpegVideoDisplay = XtVaCreateWidget("videoDisplay",
	    xnvrMpegDisplayWidgetClass,	videoDA,
	    XmNborderWidth,		0,
	    NULL);
    DevWidgetData.videoDisplay = mpegVideoDisplay;

    (void) XtVaCreateManagedWidget("nvrLabel",
	    xmLabelWidgetClass,		videoPane,
	    XmNrightAttachment,		XmATTACH_FORM,
	    XmNtopAttachment,		XmATTACH_WIDGET,
	    XmNtopWidget,		videoFrame,
	    NULL);

    XtAddCallback(mpegVideoDisplay, XnvrNrealizeCallback,
            (XtCallbackProc)InitCB,
            (XtPointer)NULL);
    XtManageChild(mpegVideoDisplay);
    XtManageChild(videoPane);
}

void
main (unsigned int argc, char **argv)
{
    Atom 	wmDeleteWindow;
    int nArgs;

    ProgramName = argv[0];

    DevWidgetData.topLevel =
	XtVaAppInitialize(
	    &DevWidgetData.appContext, "NVROnePlay",
	    options, XtNumber(options),
	    (int*)&argc, argv,
	    FallbackResources,
	    NULL);

    XtVaSetValues(DevWidgetData.topLevel,
	    XmNallowShellResize, True,
	    XmNnoResize, True,
	    NULL);
    XtGetApplicationResources(DevWidgetData.topLevel,
			      &Resources, playerResources,
                              XtNumber(playerResources), (ArgList)NULL, 0);

    /* Check Version */
    if (*Resources.version == '\0') {
        ProgMsg("You do not have the app-defaults file properly installed.\n");
        ProgMsg("This program may not correctly function without it.\n");
        Fatal("Please refer to your installation instructions.\n");
    } else if (strcmp(Resources.version, CURRENT_VERSION)) {
        ProgMsg("This program is version \"%s\" but your app-defaults\n", 
            CURRENT_VERSION);
        ProgMsg("file appears to be version \"%s\".  This program may\n",
            Resources.version);
        ProgMsg("not function correctly.  Please refer to the installation\n");
        ProgMsg("instructions for directions on proper installation.\n\n");
        ProgMsg("Continue at your own risk!!!\n");
    }

    DevWidgetData.mcs.dMpeg.preInit = Resources.mcsPreInit;
    DevWidgetData.mcs.dMpeg.postInit = Resources.mcsPostInit;

    /* Main window */
    DevWidgetData.mainWindow = XtVaCreateManagedWidget("mainWindow",
	    xmFormWidgetClass, DevWidgetData.topLevel,
	    NULL);

    /*
     * Application window
     */
    OnePlayUI.workWindow = XtVaCreateManagedWidget("workWindow",
	    xmFormWidgetClass,		DevWidgetData.mainWindow,
	    XmNmarginWidth,		10,
	    XmNmarginHeight,		10,
	    NULL);

    /*
     * Video Display window
     */
    MplayerVideoPaneCreate(OnePlayUI.workWindow);

    /* Link window manager close button to quit callback */
    XtVaSetValues(DevWidgetData.topLevel,
	    XmNdeleteResponse, XmDO_NOTHING,
	    NULL);
    wmDeleteWindow = XmInternAtom(XtDisplay(DevWidgetData.topLevel),
				  "WM_DELETE_WINDOW", False);
    XmAddWMProtocolCallback(DevWidgetData.topLevel,
			    wmDeleteWindow, ReallyQuitCB, NULL);

    /* disable resize handles */
    {
	int decor;
	decor = MWM_DECOR_BORDER | MWM_DECOR_TITLE | MWM_DECOR_MENU |
			MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE;
	XtVaSetValues(DevWidgetData.topLevel, XmNmwmDecorations, decor, NULL);
    }

    nArgs = argc-1;
    if (nArgs == 0)
	Fatal("no input file(s) specified\n");
    else if (nArgs == 1)
	SpecifySourceFiles(argv[1], NULL);
    else if (nArgs == 2)
	SpecifySourceFiles(argv[1], argv[2]);
    else {
	Msg("Usage: %s mpegFile [audioFile]\n", ProgramName);
	exit(0);
    }

    XtRealizeWidget(DevWidgetData.topLevel);
    XSync(XtDisplay(DevWidgetData.topLevel), False);

    XtAppMainLoop(DevWidgetData.appContext);
}
