/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	playaudio.c - play back audio
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/playaudio.c,v 1.10 1995/11/09 18:56:27 toddb Exp $
 */

#include "nvrTypes.h"
#include "osio.h"
#include "nvrAlloc.h"
#include "nvrTimer.h"
#include "nvrString.h"
#include "longjump.h"
#include "nvrReason.h"
#include "nvrAudio.h"
#include "nvrDebug.h"
#include "process.h"
#include "NvrLmFeatures.h"

#define UsageMacro(args)	{ Msg args; Usage(); }

char *ProgramName;

#ifdef DEBUG_ENABLED

extern FlagNames MpegSdemuxNames;
extern FlagMacroList MpegSdemuxMacros;

extern FlagNames AmpegdecompDebugNames;
extern FlagMacroList AmpegdecompDebugMacros;

FlagLabels DebugFlagDesc[] = {
    { &MpegSdemuxNames, &MpegSdemuxMacros },
    { &AmpegdecompDebugNames, &AmpegdecompDebugMacros },
    { NULL },
};

#else /* DEBUG_ENABLED */

FlagLabels DebugFlagDesc[1];

#endif /* DEBUG_ENABLED */

static void
Usage(void)
{
    Msg("Usage: %s [attribute] ... input output\n", ProgramName);
    Msg("    %s -usage\n", ProgramName);
    Msg("will show a complete usage\n");
    exit(1);
}

static void
Help()
{
    Msg("Usage: %s [quality=Q] file ...\n", ProgramName);
    Msg("\n");
    Msg("  where Q is real number between 0.0 and 1.0; e.g. quality=.50\n");
    Msg("  The default quality is 1.0 (This is only meaningful for MPEG\n");
    Msg("  files).  Input files may be aiff, aifc (not compressed),\n");
    Msg("  au (sun audio), mpeg audio, mpeg system (with audio inside),\n");
    Msg("  or parallax movie files.\n");
    exit(0);
}

void
main(int argc, char **argv)
{
    InternalReasonPtr reason = NULL;
    AudioFilePtr input = NULL;
    AudioFilePtr output = NULL;
    char *inputFile = NULL;
    long samplesOut, samplesIn;
    AudioConversionPtr cvt = NULL;
    char fromForm[128], toForm[128];
    AudioQuality quality;

    AcquireMacArgs("playaudio", &argc, &argv);
    ProgramName = argv[0];

    SetDebugFileFlags(DebugFlags, DebugFlagDesc, "playaudio.debug");

    NvrCopyrightMsg(  NvrCopyright
                    | CopyrightToTty,
                    0);

    quality.decodeQuality = 1.0;
    for (argc--, argv++; argc; argc--, argv++) {
        if (strncmp(*argv, "-usage", 5) == 0) {
	    Help();
	} else if (strncmp(*argv, "quality=", 8) == 0) {
	    quality.decodeQuality = atof(*argv+8);
	} else {
	    break;
	}
    }

    if (argc == 0)
	UsageMacro(("You didn't give any input file(s)!\n"));
    if (quality.decodeQuality < 0.0 || quality.decodeQuality > 1.0)
	UsageMacro(("Quality must be between 0.0 and 1.0\n"));

    output = OpenAudioDevice(NULL, &reason);
    if (output == NULL)
	goto done;

    quality.outputFormats = output->u.devFormats;


    /*
    * Now process each file in turn.
     */
    for (; argc; argc--, argv++) {
	inputFile = *argv;

	input = OpenAudioFile(inputFile, &quality, &reason);
	if (input == NULL)
	    goto nextFile;

	switch (input->u.fileType) {
	case FT_MpegAudio:
	case FT_MpegSystem:
	    /*
	     * first, loop through looking for the best quality at about
	     * 80% of the cpu.
	     */
	    break;
	default:
	    break;
	}
	    
	reason = ChooseAudioForm(&input->au,
				 output->u.devFormats,
				 &output->au);
	if (reason)
	    goto nextFile;
	cvt = CreateAudioConversion(&input->au, &output->au, &reason);
	if (cvt == NULL)
	    goto nextFile;

	if (!InitAudioDevice(output, &cvt->dstAu)) {
	    reason = output->reason;
	    goto nextFile;
	}

	if ( !AudioDeviceMode(output, DevModeBlocking) ) {
	    reason = output->reason;
	    goto nextFile;
	}

	Msg("%s:\n    Converting from %s to %s\n",
	    inputFile,
	    AudioAttrName(&input->au, fromForm),
	    AudioAttrName(&output->au, toForm));

	if (reason = AllocMaxAudioBuffer(cvt, MicroSecPerSec/8))
	    break;

	for (;;) {
	    samplesIn = ReadAudioSamples(input, cvt->buffer, cvt->maxInSamples);
	    if (samplesIn <= 0 || input->reason) {
		reason = input->reason;
		break;
	    }

	    if (cvt->convIndex != NoConversion) {
		samplesIn =
		    ConvertAudioBuffer(cvt, samplesIn, &reason);
		if (reason)
		    goto nextFile;
	    }

	    samplesOut = WriteAudioSamples(output, cvt->buffer, samplesIn);
	    if (samplesOut < 0 || output->reason) {
		break;
	    }
	}
	    
    nextFile:
	if (input) {
	    CloseAudioFile(input);
	    input = NULL;
	}
	if (reason) {
	    TtyShowReason(reason);
	    reason = NULL;
	}
	if (cvt) {
	    free((char *)cvt->buffer);
	    free((char *)cvt);
	    cvt = NULL;
	}
    }

done:
    if (input)
	CloseAudioFile(input);
    if (reason)
	TtyShowReason(reason);
    if (output)
	CloseAudioFile(output);
    exit(0);
}

