/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	sunaifc.c - sun functions for playing Aifc files
 *		    Actually this will only read Aiff files at present
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/aifcaudio.c,v 1.13 1994/07/14 23:46:00 russ Exp $
 */
#define NeedFunctionPrototypes 1

#include 	"vmpegdecomp.h"
#include 	"mu-law.h"
 
#define AUDIODEV		"/dev/audio"

/* State Variables */
static struct {
    BufFileHndl fp;
    float usecPerFrame;
    float bytesPerUsec;
    short bytesPerFrame;
    long hdrSize;
} AifcState;


/**********************************************************************
 *                      audio file routines                           *
 **********************************************************************/

#include <stdio.h>
#include <math.h>

/* Size field */
#define	BYTE	1
#define	WORD	2
#define	LONG	4
#define	FLOAT	5
#define DOUBLE	6
#define IEEE	7		/* IEEE 80-bit floats.  Is it necessary? */

/* Style field */
#define UNSIGNED	1	/* unsigned linear: Sound Blaster */
#define SIGN2		2	/* signed linear 2's comp: Mac */
#define	ULAW		3	/* U-law signed logs: US telephony, SPARC */
#define ALAW		4	/* A-law signed logs: non-US telephony */

#define UnsignedToFloat(u)      (((double)((long)(u - 2147483647L - 1))) \
				+ 2147483648.0)

static char IdValues[][4] = {
    { 'F', 'O', 'R', 'M' },
    { 'A', 'I', 'F', 'F' },
    { 'C', 'O', 'M', 'M' },
    { 'S', 'S', 'N', 'D' },
    { 'M', 'P', 'E', 'G' },
    { 'A', 'I', 'F', 'C' },
    { 'F', 'V', 'E', 'R' },
    { 'N', 'O', 'N', 'E' },
};

typedef union IDView_union {
    char    byte[4];
    ULong   word;
} IDView;

static IDView *Ids = (IDView *)IdValues;

#define  IFF_ID_FORM Ids[0].word
#define  IFF_ID_AIFF Ids[1].word
#define  IFF_ID_COMM Ids[2].word
#define  IFF_ID_SSND Ids[3].word
#define  IFF_ID_MPEG Ids[4].word
#define  IFF_ID_AIFC Ids[5].word
#define  IFF_ID_FVER Ids[6].word
#define  IFF_ID_NONE Ids[7].word


typedef struct _aiffPrefix {
    ULong form;
    ULong totalSize;
    ULong aiff;
} AiffPrefix;

typedef struct _aiffChunk {
    IDView name;
    ULong size;
} AiffChunk;

typedef struct _aiffCommInfo {
    UChar channels[2];
    UChar frames[4];
    UChar bits[2];
    UChar ieeeRate[ 10 ];
} AiffCommInfo;

typedef struct _aifcCommInfo {
    UChar channels[2];
    UChar frames[4];
    UChar bits[2];
    UChar ieeeRate[ 10 ];
    char compType[4];
    UChar length;
    char comment[1];	/* probably longer */
} AifcCommInfo;

typedef struct _aiffSsndInfo {
    long offset;
    long blocksize;
} AiffSsndInfo;

typedef union _aiffBuffer {
    AiffSsndInfo ssnd;
    AiffCommInfo aiffComm;
    AifcCommInfo aifcComm;
    long formatVersion;
    UChar bytes[ 1024 ];
} AiffBuffer;

static ULong
CvtLong(UChar bytes[4])
{
    return(  (bytes[0] << 24)
	   | (bytes[1] << 16)
	   | (bytes[2] << 8)
	   | (bytes[3])
	  );
}

static UShort
CvtShort(UChar bytes[2])
{
    return(bytes[0] << 8 | bytes[1]);
}

static double ConvertFromIeeeExtended(UChar *bytes)
{
    double    f;
    int    expon;
    unsigned long hiMant, loMant;
 
    expon = ((bytes[0] & 0x7F) << 8) | (bytes[1] & 0xFF);
    hiMant    =    ((unsigned long)(bytes[2] & 0xFF) << 24)
            |    ((unsigned long)(bytes[3] & 0xFF) << 16)
            |    ((unsigned long)(bytes[4] & 0xFF) << 8)
            |    ((unsigned long)(bytes[5] & 0xFF));
    loMant    =    ((unsigned long)(bytes[6] & 0xFF) << 24)
            |    ((unsigned long)(bytes[7] & 0xFF) << 16)
            |    ((unsigned long)(bytes[8] & 0xFF) << 8)
            |    ((unsigned long)(bytes[9] & 0xFF));
 
    if (expon == 0 && hiMant == 0 && loMant == 0) {
        f = 0;
    }
    else {
        if (expon == 0x7FFF) {    /* Infinity or NaN */
            f = HUGE_VAL;
        }
        else {
            expon -= 16383;
            f  = ldexp(UnsignedToFloat(hiMant), expon-=31);
            f += ldexp(UnsignedToFloat(loMant), expon-=32);
        }
    }
 
    if (bytes[0] & 0x80)
        return -f;
    else
        return f;
}

void
CloseAifcAudio(DisplayInfoPtr dpyInfo)
{
    if (AifcState.fp) {
	CloseBufferedFile(AifcState.fp);
	AifcState.fp = NULL; 
	if( dpyInfo->audioData.buffer ) {
	    free((char *)dpyInfo->audioData.buffer);
	    dpyInfo->audioData.buffer = NULL;
	    dpyInfo->audioData.bufferLength = 0;
	}
    }
}

/*
 * We know the audio File is opened before the audio device
 * This routine is responsible for closing any previously opened
 *	audio file.
 */
boolean
OpenAifcAudio(DisplayInfoPtr dpyInfo)
{
    AiffPrefix prefix;
    AiffChunk chunk;
    AudioInfoPtr audioData = &dpyInfo->audioData;
    char *errFunc = "OpenAifcAudio";
    AiffBuffer buffer;
    ULong frames, offset, blocksize;
    long result;
    short channels;
    short bits;
    double rate = 0.0;

    dpyInfo->pSeq->audioPicOverhead = 0;

    /* scrutinize the audio file */
    AifcState.fp = OpenBufferedFileForReading(dpyInfo->audioInput);
    if( AifcState.fp == NULL ) {
	SystemError("%s: open failed for %s\n", errFunc, dpyInfo->audioInput);
	return(False);
    }

    result = BufferedStdRead((char *)&prefix,
			     sizeof(prefix),
			     1,
			     AifcState.fp);
    if (result != 1) {
	ProgMsg("Premature EOF on %s\n", dpyInfo->audioInput);
	return(False);
    }
    if (prefix.form != IFF_ID_FORM
     || (prefix.aiff != IFF_ID_AIFF && prefix.aiff != IFF_ID_AIFC)) {
	ProgMsg("Corrupt AIFF header in %s\n", dpyInfo->audioInput);
	return(False);
    }

    /* Skip everything but the COMM chunk and the SSND chunk */
    /* The SSND chunk must be the last in the file */
    bits = channels = 0;
    while (1) {
	result = BufferedStdRead((char *)&chunk,
				 sizeof(chunk), 1, AifcState.fp);
	if (result != 1) {
	    ProgMsg("Missing SSND chunk in AIFF file");
	    return(False);
	}
	if (chunk.name.word == IFF_ID_SSND)
	    break;

	if (chunk.size > sizeof(buffer)) {
	    ProgMsg("AIFF file is corrupt\n");
	    return(False);
	}
	if (chunk.size
	 && BufferedStdRead(buffer.bytes, chunk.size, 1, AifcState.fp) != 1) {
	    ProgMsg("Premature eof on aiff file\n");
	    return(False);
	}
	
	if (chunk.name.word == IFF_ID_COMM && prefix.aiff == IFF_ID_AIFF) {
	    /* COMM chunk in an aiff file */
	    channels = CvtShort(buffer.aiffComm.channels);
	    frames = CvtLong(buffer.aiffComm.frames);
	    bits = CvtShort(buffer.aiffComm.bits);
	    rate = ConvertFromIeeeExtended(buffer.aiffComm.ieeeRate);
	}
	else if (chunk.name.word == IFF_ID_COMM && prefix.aiff == IFF_ID_AIFC) {
	    IDView compType;

	    /* COMM chunk in an aifc file */
	    channels = CvtShort(buffer.aifcComm.channels);
	    frames = CvtLong(buffer.aifcComm.frames);
	    bits = CvtShort(buffer.aifcComm.bits);
	    rate = ConvertFromIeeeExtended(buffer.aifcComm.ieeeRate);

	    /* gotta copy 'cause its not aligned */
	    memcpy(compType.byte, buffer.aifcComm.compType, 4);
	    if (compType.word != IFF_ID_NONE) {
		ProgMsg("Can't process aifc %c%c%c%c compression\n",
		    compType.byte[0], compType.byte[1],
		    compType.byte[2], compType.byte[3]);
		return(False);
	    }
	}
	else if (chunk.name.word == IFF_ID_FVER) {
	    /* ignore the version */
	}
	else {
	    /* unrecognized chunk */
	    DebugTask(N_AudioActivity,
		ProgMsg("unrecognized aiff chunk: %c%c%c%c, len=%d\n",
		    chunk.name.byte[0], chunk.name.byte[1],
		    chunk.name.byte[2], chunk.name.byte[3], chunk.size);
	    )
	}
    }
    
    /* SSND chunk just read */
    result = BufferedStdRead(buffer.bytes,
			     sizeof(buffer.ssnd),
			     1, AifcState.fp);
    if (result != 1) {
	ProgMsg("Corrupt SSND chunk in AIFF file\n");
	return(False);
    }
    offset = buffer.ssnd.offset;
    blocksize = buffer.ssnd.blocksize;

    if (blocksize != 0) {
	ProgMsg("AIFF header specifies nonzero blocksize?!?!");
	return(False);
    }
    while ((long) (--offset) >= 0) {
	if (GetBufferedChar(AifcState.fp) == EOF) {
	    ProgMsg("unexpected EOF while skipping AIFF offset");
	    return(False);
	}
    }

    switch (audioData->cvt.srcAu.channels = channels) {
    case AudioStereo:
    case AudioMono:
	break;
    default:
	ProgMsg("aiff channels undefined\n");
	break;
    }

    audioData->cvt.srcAu.rate = (int)rate;

    switch(bits) {
    case 8:
	dpyInfo->audioData.cvt.srcAu.dataForm = Audio8BitULaw;
	break;
    case 16:
	dpyInfo->audioData.cvt.srcAu.dataForm = Audio16BitPcm;
	break;
    default:
	ProgMsg("unsupported sample size in AIFF header");
	return(False);
    }

    AifcState.bytesPerFrame = DataFormToBytes[ audioData->cvt.srcAu.dataForm ]
			    * audioData->cvt.srcAu.channels;
    AifcState.bytesPerUsec = (DataFormToBytes[ audioData->cvt.srcAu.dataForm ]
			      * audioData->cvt.srcAu.channels
			      * audioData->cvt.srcAu.rate)
			   / (float) MicroSecPerSec;
    AifcState.usecPerFrame = (float)MicroSecPerSec / audioData->cvt.srcAu.rate;
    AifcState.hdrSize = TellBufferedFile(AifcState.fp);

    if (! PickHardwareAudio(dpyInfo))
	return(False);

    if (audioData->buffer == NULL
     && ! AllocAudioBuffer(dpyInfo, MicroSecPerSec))
	return(False);

    if (! PickHardwareAudio(dpyInfo))
	return(False);

    return(True);
}

boolean
SetQualityAifcAudio(DisplayInfoPtr dpyInfo)
{
    return(True);
}

boolean
PreloadAifcAudio(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    long skipBytes, timeSeek;

    InitTimerInc(&audioData->timer,
                 (double) audioData->schedRate,
                 MicroSecPerSec);

    if( (timeSeek = audioData->timeSeek) >= 0 ) {
	skipBytes = (long)(timeSeek / AifcState.usecPerFrame)
		* AifcState.bytesPerFrame;
	if( SeekBufferedFile(AifcState.fp,
			     skipBytes+AifcState.hdrSize, SK_Start) < 0 ) {
	    ProgMsg("PreloadAifcAudio: seek failed\n");
	    return(False);
	}
    }
    return( LoadNextAifcAudio(dpyInfo) );
}

boolean
LoadNextAifcAudio(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    long gatherBytes, gatherAuFrames;
    char *errFunc = "LoadNextAifcAudio";

    gatherAuFrames = (audioData->timeGather / AifcState.usecPerFrame);

    gatherBytes = BufferedReadData(AifcState.fp,
				   dpyInfo->audioData.buffer,
				   gatherAuFrames * AifcState.bytesPerFrame);
    if (gatherBytes < 0 ) {
	/* reached EOF */
	return(False);
    }
    audioData->bufferLength = gatherBytes;

    if (audioData->cvt.conversionIndex != NoConversion)
	ConvertAudio(dpyInfo);

    DebugTask(N_AudioActivity,
        Msg("%s: %d bytes in buffer after read (new offset=%d)",
	    errFunc,audioData->bufferLength,TellBufferedFile(AifcState.fp));
        if( audioData->timeSeek >= 0 )
	    Msg("(seek=%d)\n", AifcState.hdrSize);
	Msg("\n");
    )

    audioData->timeGather = TimerIncrement(&audioData->timer);

    return(True);
}
