/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	sunau.c - sun functions for playing au
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/anyaudio.c,v 1.4 1996/01/12 23:14:34 toddb Exp $
 */
#define NeedFunctionPrototypes 1

#include 	"vmpegdecomp.h"
 
#define AUDIODEV		"/dev/audio"

static struct {
    AudioFilePtr afp;
    long samplesPerCall;
    double samplesPerUsec;
} AnyAudioState;

/**********************************************************************
 *                      audio file routines                           *
 **********************************************************************/

static boolean
ShowAudioReason(InternalReasonPtr reason)
{
    TtyShowReason(reason);
    FreeReason(reason);
    if (AnyAudioState.afp)
	AnyAudioState.afp->reason = NULL;
    return(False);
}

void
CloseAnyAudio(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;

    if (AnyAudioState.afp) {
	CloseAudioFile(AnyAudioState.afp);
	AnyAudioState.afp = NULL;
	if( audioData->buffer ) {
	    free((char *)audioData->buffer);
	    audioData->buffer = NULL;
	    audioData->bufferLength = 0;
	}
    }
}

/*
 * We know the audio File is opened before the audio device
 * This routine is responsible for closing any previously opened
 *	audio file.
 */
boolean
OpenAnyAudio(DisplayInfoPtr dpyInfo)
{
    InternalReasonPtr reason;
    AudioFilePtr afp;
    AudioInfoPtr audioData = &dpyInfo->audioData;

    afp = OpenAudioFile(dpyInfo->audioInput, NULL, &reason);
    if (afp == NULL)
	return( ShowAudioReason(reason) );

    dpyInfo->pSeq->audioPicOverhead = 0;

    audioData->cvt.srcAu = afp->au;
    reason = ChooseAudioForm(&audioData->cvt.srcAu,
			     audioData->formatsAvailable,
			     &audioData->cvt.dstAu);
    if (reason) {
	CloseAudioFile(afp);
	return( ShowAudioReason(reason) );
    }

    reason = InitAudioConversion(&audioData->cvt,
				 &audioData->cvt.srcAu,
				 &audioData->cvt.dstAu);
    if (reason) {
	CloseAudioFile(afp);
	return( ShowAudioReason(reason) );
    }

    if (reason = AllocMaxAudioBuffer(&audioData->cvt, 1000000)) {
	CloseAudioFile(afp);
	ProgMsg("can't configure audio buffer\n");
	return(False);
    }
    dpyInfo->audioData.buffer = audioData->cvt.buffer;
    AnyAudioState.afp = afp;

    AnyAudioState.samplesPerUsec =
	  (audioData->cvt.srcAu.channels * audioData->cvt.srcAu.rate)
	/ (float) MicroSecPerSec;

    return(True);
}

boolean
SetQualityAnyAudio(DisplayInfoPtr dpyInfo)
{
    return(True);
}

static boolean
LoadSamples(DisplayInfoPtr dpyInfo, long nSamples)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    long samplesIn, samplesOut;
    InternalReasonPtr reason;

    samplesIn = ReadAudioSamples(AnyAudioState.afp,
				 audioData->buffer,
				 nSamples);

    if (samplesIn < 0)
	return( ShowAudioReason(AnyAudioState.afp->reason) );

    if (audioData->cvt.convIndex != NoConversion) {
	samplesOut = ConvertAudioBuffer(&audioData->cvt, samplesIn, &reason);
	if (samplesOut < 0)
	    return( ShowAudioReason(reason) );
	audioData->bufferLength =
	    samplesOut << DataFormToShift[ audioData->cvt.dstAu.dataForm ];
    } else {
	audioData->bufferLength =
	    samplesIn << DataFormToShift[ audioData->cvt.srcAu.dataForm ];
    }

    return(True);
}

boolean
PreloadAnyAudio(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    long sampleOffset, timeSeek, framesPerCall;
    InternalReasonPtr reason;

    framesPerCall = (float) audioData->cvt.srcAu.rate
		  / audioData->schedRate + 0.5;
    audioData->samplesPerCall = framesPerCall
			     << (audioData->cvt.srcAu.channels == AudioStereo);
    reason = InitTimerInc(&audioData->timer,
			  (double)audioData->schedRate,
			  MicroSecPerSec);
    if (reason)
	return( ShowAudioReason(reason) );

    if( (timeSeek = audioData->timeSeek) >= 0 ) {
	sampleOffset = timeSeek * AnyAudioState.samplesPerUsec;
	if (reason = SeekAudioOffset(AnyAudioState.afp, sampleOffset))
	    return( ShowAudioReason(reason) );
    }
    
    return( LoadSamples(dpyInfo,
			audioData->timeGather * AnyAudioState.samplesPerUsec)
	  );
}

boolean
LoadNextAnyAudio(DisplayInfoPtr dpyInfo)
{
    dpyInfo->audioData.timeGather = TimerIncrement(&dpyInfo->audioData.timer);
    return( LoadSamples(dpyInfo, dpyInfo->audioData.samplesPerCall) );
}
