/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	sunau.c - sun functions for playing au
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/auaudio.c,v 1.8 1994/07/13 21:03:41 russ Exp $
 */
#define NeedFunctionPrototypes 1

#include 	"vmpegdecomp.h"
 
#define AUDIODEV		"/dev/audio"

static struct {
    BufFileHndl fp;
    float usecPerFrame;
    float bytesPerUsec;
    short bytesPerFrame;
    long bytesPerCall;
    long hdrSize;
    Audio_filehdr auFileHdr;
} AuState;

/**********************************************************************
 *                      audio file routines                           *
 **********************************************************************/

void
CloseAuAudio(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;

    if (AuState.fp) {
	CloseBufferedFile(AuState.fp);
	AuState.fp = NULL; 
	if( audioData->buffer ) {
	    free((char *)audioData->buffer);
	    audioData->buffer = NULL;
	    audioData->bufferLength = 0;
	}
    }
}

/*
 * We know the audio File is opened before the audio device
 * This routine is responsible for closing any previously opened
 *	audio file.
 */
boolean
OpenAuAudio(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    Audio_filehdr *afh = &AuState.auFileHdr;
    int len;
    char *errFunc = "auOpen";

    dpyInfo->pSeq->audioPicOverhead = 0;

    /* scrutinize the audio file */
    AuState.fp = OpenBufferedFileForReading(dpyInfo->audioInput);
    if( AuState.fp == NULL ) {
	SystemError("%s: open failed: %s\n", errFunc, dpyInfo->audioInput);
	return(False);
    }
    len = sizeof(*afh);
    if( BufferedReadData(AuState.fp,(char *)afh,len) != len ||
       afh->magic != AUDIO_FILE_MAGIC ) {
	ProgMsg("%s: %s: not a sun audio file\n", errFunc, dpyInfo->audioInput);
	return(False);
    }

    switch(afh->encoding) {
    case AUDIO_FILE_ENCODING_MULAW_8:
    case AUDIO_FILE_ENCODING_LINEAR_8:
	audioData->cvt.srcAu.dataForm = Audio8BitULaw;
	break;
    case AUDIO_FILE_ENCODING_LINEAR_16:
	audioData->cvt.srcAu.dataForm = Audio16BitPcm;
	break;
    default:
	ProgMsg("%s: can't handle audio files of encoding type '0x%x'\n",
		errFunc, afh->encoding);
	return(False);
    }

    switch (audioData->cvt.srcAu.channels = afh->channels) {
    case AudioStereo:
    case AudioMono:
        break;
    default:
        ProgMsg("au channels undefined\n");
        break;
    }

    audioData->cvt.srcAu.rate = afh->sample_rate;
    AuState.bytesPerFrame = DataFormToBytes[ audioData->cvt.srcAu.dataForm ]
			  * audioData->cvt.srcAu.channels;
    AuState.usecPerFrame = (float)MicroSecPerSec / audioData->cvt.srcAu.rate;
    AuState.bytesPerUsec = (DataFormToBytes[ audioData->cvt.srcAu.dataForm ]
				* audioData->cvt.srcAu.channels
				* audioData->cvt.srcAu.rate)
		         / (float) MicroSecPerSec;
    AuState.hdrSize = afh->hdr_size;

    if (! PickHardwareAudio(dpyInfo))
        return(False);

    if (audioData->buffer == NULL && ! AllocAudioBuffer(dpyInfo, 1000000))
    {
	ProgMsg("%s: can't configure audio buffer\n",errFunc);
	return(False);
    }

    return(True);
}

boolean
SetQualityAuAudio(DisplayInfoPtr dpyInfo)
{
    return(True);
}

static boolean
LoadAudio(DisplayInfoPtr dpyInfo, long nBytes)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    long gatherBytes;
    char *errFunc = "LoadAudio";
    DebugDeclare(ULong readTime = 0;)

    DebugTask(N_AudioActivity, CheckRTClock(&readTime))
    gatherBytes = BufferedReadData(AuState.fp,
				   audioData->buffer,
				   nBytes);
    audioData->bufferLength = gatherBytes;

    if (audioData->cvt.conversionIndex != NoConversion)
        ConvertAudio(dpyInfo);

    DebugTask(N_AudioActivity,
	CheckRTClock(NULL);
        Msg("%s: asked %d, got %d, time=%d\n",
	    errFunc, nBytes, gatherBytes, readTime);
    )

    if (gatherBytes < 0 )
	return(False); /* reached EOF */

    audioData->timeGather = TimerIncrement(&audioData->timer);

    return(True);
}

boolean
PreloadAuAudio(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    long skipBytes, timeSeek, framesPerCall;

    framesPerCall = (float) audioData->cvt.srcAu.rate
		  / audioData->schedRate + 0.5;
    audioData->cvt.samplesPerCall = framesPerCall
			     << (audioData->cvt.srcAu.channels == AudioStereo);
    AuState.bytesPerCall = audioData->cvt.samplesPerCall
			 * DataFormToBytes[ audioData->cvt.srcAu.dataForm ];
    InitTimerInc(&audioData->timer,
		 (double)audioData->schedRate,
		 MicroSecPerSec);

    if( (timeSeek = audioData->timeSeek) >= 0 ) {
	skipBytes = timeSeek * AuState.bytesPerUsec;
	SeekBufferedFile(AuState.fp, skipBytes+AuState.hdrSize, SK_Start);
	DebugTask(N_AudioActivity, Msg("(seek=%d)\n", skipBytes))
    }

    
    return( LoadAudio(dpyInfo, audioData->timeGather * AuState.bytesPerUsec) );
}

boolean
LoadNextAuAudio(DisplayInfoPtr dpyInfo)
{
    boolean result;

    result = LoadAudio(dpyInfo, AuState.bytesPerCall);
    dpyInfo->audioData.timeGather = TimerIncrement(&dpyInfo->audioData.timer);

    return(result);
}
