/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	audiodecode.h - audio-specific information
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/audiodecode.h,v 1.13 1994/08/05 15:32:26 toddb Exp $
 */
#ifndef _AUDIODECODE_H_
#define _AUDIODECODE_H_

#include "nvrAudio.h"

typedef struct _audioFuncs {
    boolean (*fileOpen)(DisplayInfoPtr);
    void    (*fileClose)(DisplayInfoPtr);
    boolean (*filePreload)(DisplayInfoPtr);
    boolean (*fileLoadNext)(DisplayInfoPtr);
    boolean (*fileSetQuality)(DisplayInfoPtr);

    void (*devInit)(DisplayInfoPtr);
    boolean (*devOpen)(DisplayInfoPtr);
    void    (*devClose)(DisplayInfoPtr);
    boolean (*devUpdate)(DisplayInfoPtr);
} AudioFuncs, *AudioFuncsPtr;

typedef struct _audioInfo {
    /* AudioForm au;		set by fileOpen() */
    long formatsAvailable;	/* set by devInit() */

    AudioConversion cvt;	/* used by audio library for rate conversion */

    long samplesPerCall;	/* # of samples in per call to fileLoadNext() */

    /*
     * This is initially set by audiodecode.c before calling filePreload()
     * to the "initial" amount.  After that, filePreload() is expected
     * to change the value for subsequent calls to fileLoadNext() to an
     * appropriate amount
     */
    long timeGather;

    /*
     * Set by audiodecode.c before calling filePreload(), it indicates the
     * seek point.
     */
    long timeSeek;

    /*
     * set and used by audiodecode.c to decide whether it has already
     * performed a seek.
     */
    long seekPicture;

    /*
     * This is a copy of dpyInfo->audioQuality.  When dpyInfo->audioQuality
     * changes and filePreload() has already been called for a given
     * offset, fileSetQuality() is called.
     */
    short audioQuality;

    /*
     * This is filled in by the file-reading functions... The device
     * function only looks at this.
     */
    UChar *buffer;	/* storage for samples */

    /*
     * Again, filled in by file-reading functions as to the number
     * of available bytes in 'buffer'.  After delivering the
     * samples, the device function should clear this.
     */
    long bufferLength;

    /*
     * Where to start playing in 'buffer'.  Note that sampleOffset is
     * in terms of sample size, not bytes
     */
    long sampleOffset;

    /*
     * The number of audio Samples (not including the 'sampleOffset' samples
     * still in 'buffer') that we've seen and can't get back to without
     * seeking.
     */
    long pastAudioSamples;

    /*
     * This is a counter of zero samples to play.  This may happen if
     * the samples can't be produced.  XXX this should really be done
     * by the file functions.
     */
    long nullSamplesToPlay;

    /*
     * the set of functions that handle device delivery and file
     * decoding.  Assigned by audiodecode.c
     */
    AudioFuncsPtr funcs;

    /*
     * This is set by audiodecode.c to the # of times the scheduler
     * would like to call the audio per second
     */
    long schedRate;

    /*
     * This is set by the file reading routines to reflect:
     *  - the desires of the scheduler reflected in 'schedRate'
     *  - the requirements of the file reading routines
     * In general, it reflects the amount of time per chunk of audio.
     */
    TimerInc timer;

} AudioInfo, *AudioInfoPtr;

/*
 * These are portable audio decoder
 * fuctions
 */
extern boolean OpenMpegAudio(DisplayInfoPtr dpyInfo);
extern void    CloseMpegAudio(DisplayInfoPtr dpyInfo);
extern boolean PreloadMpegAudio(DisplayInfoPtr dpyInfo);
extern boolean LoadNextMpegAudio(DisplayInfoPtr dpyInfo);
extern boolean SetQualityMpegAudio(DisplayInfoPtr dpyInfo);

extern boolean OpenAnyAudio(DisplayInfoPtr dpyInfo);
extern void    CloseAnyAudio(DisplayInfoPtr dpyInfo);
extern boolean PreloadAnyAudio(DisplayInfoPtr dpyInfo);
extern boolean LoadNextAnyAudio(DisplayInfoPtr dpyInfo);
extern boolean SetQualityAnyAudio(DisplayInfoPtr dpyInfo);

extern boolean AllocAudioBuffer(DisplayInfoPtr dpyInfo, long usec);

extern AudioFuncs	AnyAudioFuncs;
extern AudioFuncs	MpegAudioFuncs;

#endif /* _AUDIODECODE_H_ */
