/*
 * Copyright (c) 1992,1993,1994, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	block.c - mpeg block-level parsing
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/block.c,v 4.14 1994/05/09 00:04:36 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"
#include "binarybits.h"
#include "huffman.h"
#include "reconstruct.h"
#include "motion.h"
#include "idct.h"

/*
 * This zero's out coefficients that are not already zeroed out, but
 * avoids writing it memory, if possible.  We also identify here
 * the maximum number of rows and columns that must be read.
 */
Inline static void ZeroBlock(MacroBlockInfo *pMb, ULong cells1, ULong cells2)
{
    ULong oldCells, newCells;
    ULong cellsToClear;
    long *dctCoeff;
    ULong *pCellsInBlock;
    OctWord zero;
    long i;

    zero.l.high = zero.l.low = 0;

    pCellsInBlock = &pMb->cellsInBlock2;
    newCells = cells2;
    dctCoeff = &pMb->dctCoeff[0];

    for (i = 1;;) {
	if ((oldCells = *pCellsInBlock) != newCells) {
	    *pCellsInBlock = newCells;
	    cellsToClear = oldCells & (~newCells);
	    while (cellsToClear) {
		while ((cellsToClear & 0xf) == 0) {
		    dctCoeff += 4;
		    cellsToClear >>= 4;
		}
		switch(cellsToClear & 0xf) {
		case 0:                    break;
		case 1:                    dctCoeff[ 0 ] = 0; break;
		case 2:                    dctCoeff[ 1 ] = 0; break;
		case 3:                    *OctWordP(dctCoeff) = zero; break;

		case 4: dctCoeff[ 2 ] = 0; break;
		case 5: dctCoeff[ 2 ] = 0; dctCoeff[ 0 ] = 0; break;
		case 6: dctCoeff[ 2 ] = 0; dctCoeff[ 1 ] = 0; break;
		case 7: dctCoeff[ 2 ] = 0; *OctWordP(dctCoeff) = zero; break;

		case 8: dctCoeff[ 3 ] = 0; break;
		case 9: dctCoeff[ 3 ] = 0; dctCoeff[ 0 ] = 0; break;
		case 10:dctCoeff[ 3 ] = 0; dctCoeff[ 1 ] = 0; break;
		case 11:dctCoeff[ 3 ] = 0; *OctWordP(dctCoeff) = zero; break;

		case 12:*OctWordP(dctCoeff+2) = zero; break;
		case 13:*OctWordP(dctCoeff+2) = zero; dctCoeff[ 0 ] = 0; break;
		case 14:*OctWordP(dctCoeff+2) = zero; dctCoeff[ 1 ] = 0; break;
		case 15:*OctWordP(dctCoeff+2) = zero;
			*OctWordP(dctCoeff) = zero;
			break;
		}
		dctCoeff += 4;
		cellsToClear >>= 4;
	    }
	}

	if (i-- <= 0)
	    break;
	dctCoeff = &pMb->dctCoeff[4*8];
	pCellsInBlock = &pMb->cellsInBlock1;
	newCells = cells1;
    }
}

CacheReturnValue
IntraBlock(CacheArgsDecl,
	   int blockIndex,
	   SequenceInfoPtr pSeq)
{
    MacroBlockInfo *pMb = pSeq->macroBlk;
    BitStreamPtr mpegBuf = pSeq->mpegBuf;
    UChar dcSize;
    UChar dcDifferential;
    UShort *scaledIntraQMatrix = pSeq->scaledIntraQMatrix;
    boolean negative;
    ULong bits;
    ULong hashValue;
    ULong cellsInBlock1, cellsInBlock2;
    int bitLength;
    int run, dcLevel, level, dctZZIndex, dctIndex;

    ShowMark(mpegBuf, N_BlockMark,
	("decode intra %s\n", BlockNames[blockIndex]));

    if (blockIndex > CB) { /* protocol */
	NextBits(mpegBuf, 7, bits);
	hashValue = HuffLookup.sizeLuminance[ bits ];
	bitLength = SizeYCBits(hashValue);
	ConsumeBits(mpegBuf, bitLength, N_DcSizeLuminance);
	if (dcSize = SizeYCValue(hashValue)) {
	    ReadBits(mpegBuf, dcSize, dcDifferential, N_DcLumaDiff);
	    if (dcDifferential & (1 << (dcSize-1)) )
		dcLevel = dcDifferential;
	    else
		dcLevel = (~0 << dcSize) | (dcDifferential + 1);
	    dcLevel <<= 3; /* *8 as per protocol */;
	} else
	    dcLevel = 0;
    } else { /* chroma block */
	NextBits(mpegBuf, 8, bits);
	hashValue = HuffLookup.sizeChrominance[ bits ];
	bitLength = SizeYCBits(hashValue);
	ConsumeBits(mpegBuf, bitLength, N_DcSizeChrominance);
	if (dcSize = SizeYCValue(hashValue)) {
	    ReadBits(mpegBuf, dcSize, dcDifferential, N_DcChromaDiff);
	    if (dcDifferential & (1 << (dcSize-1)) )
		dcLevel = dcDifferential;
	    else
		dcLevel = (~0 << dcSize) | (dcDifferential + 1);
	    dcLevel <<= 3; /* *8 as per protocol */;
	} else
	    dcLevel = 0;
    }

    dctZZIndex = 0;
    cellsInBlock1 = 0;
    cellsInBlock2 = 1<<0;

    /* as per protocol for dctCoeff[0] */
    switch(blockIndex) {
    case Y0:
	if (pMb->mbAddr - pMb->pastIntraMbAddr > 1)
	    pMb->dctCoeff[0] = (dcLevel += 128 * 8) << FIXED_PT_BITS;
	else
	    pMb->dctCoeff[0] = (dcLevel += pMb->dcYPast) << FIXED_PT_BITS;
	pMb->dcYPast = dcLevel;
	break;

    case Y1: case Y2: case Y3:
	pMb->dctCoeff[0] = (dcLevel += pMb->dcYPast) << FIXED_PT_BITS;
	pMb->dcYPast = dcLevel;
	break;

    case CR:
	if (pMb->mbAddr - pMb->pastIntraMbAddr > 1)
	    pMb->dctCoeff[0] = (dcLevel += 128 * 8) << FIXED_PT_BITS;
	else
	    pMb->dctCoeff[0] = (dcLevel += pMb->dcCrPast) << FIXED_PT_BITS;
	pMb->dcCrPast = dcLevel;
	break;

    case CB:
	if (pMb->mbAddr - pMb->pastIntraMbAddr > 1)
	    pMb->dctCoeff[0] = (dcLevel += 128 * 8) << FIXED_PT_BITS;
	else
	    pMb->dctCoeff[0] = (dcLevel += pMb->dcCbPast) << FIXED_PT_BITS;
	pMb->dcCbPast = dcLevel;
	break;
    }

    for(;;) {
	/* do a dct_coeff_next */

	NextBits(mpegBuf, 28, bits);
	hashValue = HuffLookup.dctCoeff1[ bits>>(28-DCT_COEFF1_LOOKUP_BITS) ];

	/*
	 * If the level is 0, it may be an end_of_block,
	 * an escape or a long code.
	 */
	if ((level = DCTCoeffLevel(hashValue)) == 0) {
	    if (bits>>26 == Binary2(10,  2))
		break;	/* end_of_block */
	    if (bits>>22 == Binary6(0000,01,  6)) {
		DecodeDCTCoeffEscape(bits, level, run, bitlength, negative);
		
	    } else {
		DecodeLongDCTCoeff(bits, level, run, bitlength, negative);
	    }
	} else {
	    DecodeShortDCTCoeff(hashValue,
				bits, level, run, bitlength, negative);
	}
	ShowMark(mpegBuf, N_CoeffMark, ("#%d (run=%d), level=%d\n",
	    dctZZIndex+run+1, run, level));

	dctZZIndex += run + 1;
	StreamAssert(mpegBuf, dctZZIndex <= 63);

	dctIndex = pMb->zigZag[dctZZIndex];
	MarkCellNZ(cellsInBlock1, cellsInBlock2, dctIndex);
	ReconstructIntraCoeff(dctIndex, pMb->dctCoeff, scaledIntraQMatrix,
			      level, negative);
	ConsumeBits(mpegBuf, bitLength, N_DctCoeffNext);
    }

    ZeroBlock(pMb, cellsInBlock1, cellsInBlock2);

    ConsumeBits(mpegBuf, 2, N_EndOfBlock);

    CacheReturn(mpegBuf);
}

/*
 * Note that pFrom is the upper left-hand corner of the block of pixels
 * pointed to by the motion vector, and it is probably not long-word aligned.
 */
CacheReturnValue
Block(CacheArgsDecl, SequenceInfoPtr pSeq)
{
    int bitLength;
    ULong bits;
    ULong hashValue;
    boolean negative;
    BitStreamPtr mpegBuf = pSeq->mpegBuf;
    MacroBlockInfo *pMb = pSeq->macroBlk;
    int run, level, dctZZIndex, dctIndex;
    ULong cellsInBlock1, cellsInBlock2;
    UShort *scaledNonIntraQMatrix = pSeq->scaledNonIntraQMatrix;

    ShowMark(mpegBuf, N_BlockMark,
	     ("decode non-intra %s\n", BlockNames[pMb->blockIndex]));

    /* do a dct_coeff_first */
    NextBits(mpegBuf, 2, bits);
    if (bits & 0x2) { /* NOTE 2 for dct_coeff first */
	negative = (bits & 1);
	level = 1;
	ConsumeBits(mpegBuf, 2, N_DctCoeffFirst);
	ShowMark(mpegBuf, N_CoeffMark, ("#0 (first), level=%d\n", level));
	cellsInBlock2 = 1<<0;
	ReconstructNonIntraCoeff(0, pMb->dctCoeff, scaledNonIntraQMatrix,
				 level, negative);
	dctZZIndex = 0;
    } else {
	cellsInBlock2 = 0;
	dctZZIndex = -1;
    }

    cellsInBlock1 = 0;
    for(;;) {
	/* do a dct_coeff_next */

	NextBits(mpegBuf, 28, bits);
	hashValue = HuffLookup.dctCoeff1[ bits>>(28-DCT_COEFF1_LOOKUP_BITS) ];

	/*
	 * If the level is 0, it may be an end_of_block,
	 * an escape or a long code.
	 */
	if ((level = DCTCoeffLevel(hashValue)) == 0) {
	    if (bits>>26 == Binary2(10,  2))
		break;	/* end_of_block */
	    if (bits>>22 == Binary6(0000,01,  6)) {
		DecodeDCTCoeffEscape(bits, level, run, bitlength, negative);
	    } else {
		DecodeLongDCTCoeff(bits, level, run, bitlength, negative);
	    }
	} else {
	    DecodeShortDCTCoeff(hashValue,
				bits, level, run, bitlength, negative);
	}
	ShowMark(mpegBuf, N_CoeffMark, ("#%d (run=%d), level=%d\n",
	    dctZZIndex+run+1, run, negative ? (-level) : level));

	dctZZIndex += run + 1;
	dctIndex = pMb->zigZag[dctZZIndex];
	MarkCellNZ(cellsInBlock1, cellsInBlock2, dctIndex);
	ReconstructNonIntraCoeff(dctIndex, pMb->dctCoeff, scaledNonIntraQMatrix,
				 level, negative);
	ConsumeBits(mpegBuf, bitLength, N_DctCoeffNext);

	StreamAssert(mpegBuf, dctZZIndex <= 63);
    }

    ZeroBlock(pMb, cellsInBlock1, cellsInBlock2);

    ConsumeBits(mpegBuf, 2, N_EndOfBlock);

    CacheReturn(mpegBuf);
}
