/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              cbr.c
 *
 *      DESCRIPTION
 *              SGI constant bit rate server input function interface
 *
 *      REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/cbr.C,v 1.9 1994/12/19 19:53:17 toddb Exp $
 */

#include "nvrImage.h"
#include "bitstream.h"

#ifdef CBR_ENABLED

CExternal void		CallCMain(int argc, char ** argv);
#include "cbr.h"
#include "Cout.h"
#include "osio.h"
#include "fileContent.h"
#include "nvrString.h"

static char *bitHost;	/* yuck */

main(int argc, char **argv)
{
    CallCMain(argc, argv);
}

static FileHandle
CBR_open(char *name)
{
    FileHandle f;
    ImageFileInfo fInfo;
    long Bps = 250000;	/* 2 Megabits-per-sec default */
    size_t bufsiz = DFLT_BITSTREAM_BUF_SIZE << (L_LONG - L_BYTE);
    InternalReasonPtr reason;

    if (reason = GetImageFileInfo(name, FT_Unknown, &fInfo)) {
	TtyShowReason(reason);
	FreeReason(reason);
	return(-1);
    }
    if( fInfo.type == FT_MpegVideo ) {
	Bps = (fInfo.bitRate < 0) ? Bps : fInfo.bitRate / 8;
    }
    if ((f = (FileHandle)cbr_open(name,bufsiz,Bps)) < 0) {
	ProgMsg("Open of '%s' on bitserver '%s' failed\n",
		name,bitHost);
	cbr_done();
    }
    return(f);
}

static long
CBR_seek(FileHandle f, long offset, SeekWhere sk)
{
    if (sk != SK_Start)
	return(-1);
    cbr_seek((tdhand_t)f, offset);
    return(0);
}

static int
CBR_read(FileHandle f, char *buf, long count)
{
    return((int)cbr_read((tdhand_t)f, buf, (size_t)count));
}

static void
CBR_close(FileHandle f)
{
    cbr_done();
}

/*
** This following line oif code is so that the CBR library
**	won't spew out its various noises to stdout which
**	is interpreted by the decompressor as mcs protocol
**	commands
*/
Cout myCout("/dev/null");

CExternal ImageReasonPtr
CBR_init(BitStream *mpegBuf, char *hostname)
{
    static BitStreamOpenFunc  openSave;
    static BitStreamReadFunc  readSave;
    static BitStreamSeekFunc  seekSave;
    static BitStreamCloseFunc closeSave;

    if ((hostname == NULL) || !hostname[0]) {
	if (openSave != NULL) {
	    mpegBuf->openFunc  = openSave;
	    mpegBuf->readFunc  = readSave;
	    mpegBuf->seekFunc  = seekSave;
	    mpegBuf->closeFunc = closeSave;
	    openSave = NULL;
	}
    } else {
	openSave  = mpegBuf->openFunc;
	readSave  = mpegBuf->readFunc;
	seekSave  = mpegBuf->seekFunc;
	closeSave = mpegBuf->closeFunc;
	mpegBuf->openFunc  = CBR_open;
	mpegBuf->readFunc  = CBR_read;
	mpegBuf->seekFunc  = CBR_seek;
	mpegBuf->closeFunc = CBR_close;
	cbr_init(hostname);
	bitHost = hostname;
    }
    return(NULL);
}

#endif /* BITSERVER */
