/* $Header: /usr/local/src/nvrToMIT/RCS/checkfb.c,v 1.4 1994/12/17 12:58:10 russ Exp $ */


#define NeedFunctionPrototypes 1

#include "nvrTypes.h"
#include <stdio.h>
#include <signal.h>
#include <sys/errno.h>

#include    <pixrect/pr_planegroups.h>
#ifdef Solaris2
#include    <sys/fbio.h>
#else
#include    <sun/fbio.h>
#endif
#include    <sys/mman.h>
#include    <sys/types.h>
#include    <sys/file.h>
#include    <sys/ioctl.h>
#include    <sundev/cg6reg.h>
#include    <sundev/cg4reg.h>
#include    <sundev/cg8reg.h>

static char *SunFbTypes[] = {
    "SUN1BW",
    "SUN1COLOR",
    "SUN2BW",
    "SUN2COLOR",
    "SUN2GP",
    "SUN5COLOR",
    "SUN3COLOR",
    "MEMCOLOR",
    "SUN4COLOR",
    "NOTSUN1",
    "NOTSUN2",
    "NOTSUN3",
    "SUNFAST_COLOR",
    "SUNROP_COLOR",
    "SUNFB_VIDEO",
    "SUNGIFB",
    "SUNGPLAS",
    "SUNGP3",
    "SUNGT",
    "RESERVED1",
};

static void
ShowType(char *prelude, int type)
{
    Msg("%s%s (%d)\n",
	prelude,
	type < NumberOfCells(SunFbTypes)
	? SunFbTypes[ type ] : "?",
	type);
}

static void
ShowFbType(char *prelude, struct fbtype *type)
{
    Msg("%s", prelude);
    ShowType("type: ", type->fb_type);
    Msg("%sgeom: %dx%d\n", prelude, type->fb_width, type->fb_height);
    Msg("%sdepth: %d\n", prelude, type->fb_depth);
    Msg("%scolor map size: %d\n", prelude, type->fb_cmsize);
    Msg("%stotal size: %d\n", prelude, type->fb_size);
}

static void CheckSunFb(char *devName)
{
    struct fbtype type;
    struct fbgattr fbattr;
    struct cg6_info cg6attr;
    boolean sunFastColor = False;
    int fd;
    int i;
    extern int errno;

    Msg("%s:\n", devName);
    fd = open(devName, O_RDWR);
    if (fd < 0) {
	if (errno != ENXIO && errno != ENODEV)
	    Error("can't open %s\n", devName);
	return;
    }

    /*
     * Now get the frame buffer info
     */
    if (ioctl(fd, FBIOGTYPE, &type) < 0) {
        Error("can't get sun fb info\n");
    } else {
	Msg("--- FBIOGTYPE ---\n");
	ShowFbType("", &type);
	Msg("\n");
	sunFastColor |= (type.fb_type == FBTYPE_SUNFAST_COLOR);
    }

    if (ioctl(fd, FBIOGATTR, &fbattr) < 0) {
        Error("can't get sun fbattr\n");
    } else {
	sunFastColor |= (fbattr.real_type == FBTYPE_SUNFAST_COLOR);
	Msg("--- FBIOGATTR ---\n");
	ShowType("real type: ", fbattr.real_type);
	Msg("owner: %d\n", fbattr.owner);
	Msg("fbtype:\n");
	ShowFbType("     ", &fbattr.fbtype);

	Msg("fbsattr:\n");
	Msg("    flags: 0x%x\n", fbattr.sattr.flags);
	ShowType("    emu type: ", fbattr.sattr.emu_type);
	Msg("    dev specific: ");
	for (i = 0; i < FB_ATTR_NDEVSPECIFIC; i++)
	    Msg("%d ", fbattr.sattr.dev_specific[i]);
	Msg("\n");
	for (i = 0; i < FB_ATTR_NEMUTYPES; i++) {
	    if (fbattr.emu_types[i] >= 0) {
		ShowType("    + emu type: ", fbattr.emu_types[i]);
		sunFastColor |= (fbattr.emu_types[i] == FBTYPE_SUNFAST_COLOR);
	    }
	}
	Msg("\n");
    }

    if (sunFastColor) {
	if (ioctl(fd, FBIOGXINFO, &cg6attr) < 0) {
	    Error("can't get sun cg6 attr\n");
	    return;
	}
	Msg("--- FBIOGXINFO ---\n");
	Msg("accessible w,h: %d,%d\n",
	    cg6attr.accessible_width,
	    cg6attr.accessible_height);
	Msg("line bytes: %d\n", cg6attr.line_bytes);
	Msg("hw capable: %s (%d)\n",
	    cg6attr.hdb_capable ? "yes" : "no", cg6attr.hdb_capable);
	Msg("video mem: %dMB\n", cg6attr.vmsize);
	Msg("board rev: %d\n", cg6attr.boardrev);
	Msg("slot: %d\n", cg6attr.slot);
    }
}

char *ProgramName;

main(int argc, char **argv)
{
    ProgramName = *argv;

    for (argc--, argv++; argc; argc--, argv++) {
	CheckSunFb(*argv);
    }
}
