/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              convertluma.c
 *
 *      DESCRIPTION
 *              fast routines for conversion of luma
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/convertluma.c,v 4.2 1993/10/22 23:53:29 russ Exp $
 */


#define NeedFunctionPrototypes 1

#include "nvrTypes.h"
#include "bitstream.h"
#include "longjump.h"
#include "general.h"
#include "decoder.h"
#include "display.h"

UChar YDithered7to3Pixel[ 128 * 16 ];

void InitMonoDitherLookup()
{
    static boolean complete = False;
    int i, dither;
    UChar pixel;
    static UChar ditherVals[16] = {
	D00, D01, D02, D03,
	D10, D11, D12, D13,
	D20, D21, D22, D23,
	D30, D31, D32, D33
    };
    static UChar patterns[ 8 ] = {
     /* 0x00, 0x10, 0x02, 0x21, 0x12, 0x31, 0x23, 0x33 */
	0x33, 0x33, 0x31, 0x12, 0x21, 0x02, 0x00, 0x00
    }; 

    if (complete)
	return;
    for (i = 0; i < 128; i++)
	for (dither = 0; dither < 16; dither++) {
	    pixel = (i + (ditherVals[ dither ] >> 4)) >> 4;
	    if (pixel > 7)
		pixel = 7;
	    YDithered7to3Pixel[ dither*128 + i ] = patterns[ pixel ];
	}
    complete = True;
}

/*ARGSUSED*/
void
ConvertYChannel1Double(UChar *pFb, long fbWidth, UChar *pY, GeomRectangle *src,
		       GeomRectangle *dst)
{
    long width = (dst->width & (~0x1f));
    long height = dst->height;
    int pixStride, chanStride;
    int i, j, bigWidth;
    ULong *pLongPix1 = ULongP(pFb);
    ULong *pLongPix2 = pLongPix1 + (fbWidth>>2);
    ULong *pLongY = ULongP(pY);
    ULong yValues;
    ULong yPixel1, yPixel2;
    UChar pattern;
    UChar *ditherPattern;

    /*
     * Hide some of the grody details
     */
#define DitherFirstPix0							\
    pattern = ditherPattern[ 0*128 + (yValues >> 24) ];			\
    yPixel1 = (pattern >> 4);						\
    yPixel2 = (pattern & 0x3);

#define DitherPix0							\
    pattern = ditherPattern[ 0*128 + (yValues >> 24) ];			\
    yPixel1 = (yPixel1 << 2) | (pattern >> 4);				\
    yPixel2 = (yPixel2 << 2) | (pattern & 0x3);

#define DitherPix1							\
    pattern = ditherPattern[ 1*128 + ((yValues >> 16) & 0x7f) ];	\
    yPixel1 = (yPixel1 << 2) | (pattern >> 4);				\
    yPixel2 = (yPixel2 << 2) | (pattern & 0x3);

#define DitherPix2							\
    pattern = ditherPattern[ 2*128 + ((yValues >> 8) & 0x7f) ];		\
    yPixel1 = (yPixel1 << 2) | (pattern >> 4);				\
    yPixel2 = (yPixel2 << 2) | (pattern & 0x3);

#define DitherPix3							\
    pattern = ditherPattern[ 3*128 + (yValues & 0x7f) ];		\
    yPixel1 = (yPixel1 << 2) | (pattern >> 4);				\
    yPixel2 = (yPixel2 << 2) | (pattern & 0x3);

    pixStride = fbWidth + fbWidth - (width >> 3);
    chanStride = MAX_X - (width>>1);

    bigWidth = width >> (4+1);

    i = height >> 1;
    for (;;) {
	ditherPattern = &YDithered7to3Pixel[ (i&0x3) << (2+7) ];
	for (j = bigWidth; j-- > 0;) {
	    pattern = 0;

	    yValues = *pLongY++ >> 1;
	    DitherFirstPix0; DitherPix1; DitherPix2; DitherPix3;

	    yValues = *pLongY++ >> 1;
	    DitherPix0; DitherPix1; DitherPix2; DitherPix3;

	    yValues = *pLongY++ >> 1;
	    DitherPix0; DitherPix1; DitherPix2; DitherPix3;

	    yValues = *pLongY++ >> 1;
	    DitherPix0; DitherPix1; DitherPix2; DitherPix3;

	    *pLongPix1++ = yPixel1; *pLongPix2++ = yPixel2;
	}

	if (--i <= 0)
	    break;
	UCharP(pLongPix1) += pixStride;
	UCharP(pLongPix2) += pixStride;
	UCharP(pLongY) += chanStride;
    }
#undef DitherFirstPix0
#undef DitherPix0
#undef DitherPix1
#undef DitherPix2
#undef DitherPix3
}

/*ARGSUSED*/
void ConvertYChannel8Double(UChar *pFb, long fbWidth, UChar *pY, GeomRectangle *src,
			    GeomRectangle *dst)
    /* GeomRectangle *src;		XXX ignored */
    /* GeomRectangle *dst;		x and y already compensated for */
{
    Fatal("can't copy to 8-bit\n");
}

/*ARGSUSED*/
void ConvertYChannel8(UChar *pFb, long fbWidth, UChar *pY, GeomRectangle *src,
		      GeomRectangle *dst)
    /* GeomRectangle *src;		XXX ignored */
    /* GeomRectangle *dst;		x and y already compensated for */
{
    Fatal("can't copy to 8-bit\n");
}

/*ARGSUSED*/
void ConvertYChannel7(UChar *pFb, long fbWidth, UChar *pY, GeomRectangle *src,
		      GeomRectangle *dst)
    /* GeomRectangle *src;		XXX ignored */
    /* GeomRectangle *dst;		x and y already compensated for */
{
    long width = dst->width;
    long height = dst->height;
    OctWord in1, in2;
    long i, j;

    for (i = height;;) {
	for (j = width; j > 0; j -= 16) {
	    in1 = *OctWordP(pY)++;
            in2 = *OctWordP(pY)++;
	    in1.l.high = (in1.l.high >> 1) | 0x80808080;
	    in1.l.low  = (in1.l.low >> 1)  | 0x80808080;
	    in2.l.high = (in2.l.high >> 1) | 0x80808080;
	    in2.l.low  = (in2.l.low >> 1)  | 0x80808080;
            *OctWordP(pFb)++ = in1;
            *OctWordP(pFb)++ = in2;
	}
	if (--i <= 0)
	    break;
	pFb += fbWidth - width;
	pY += MAX_X - width;
    }
}

/*ARGSUSED*/
void ConvertYChannel7Double(UChar *pFb, long fbWidth, UChar *pY, GeomRectangle *src,
			    GeomRectangle *dst)
    /* GeomRectangle *src;		XXX ignored */
    /* GeomRectangle *dst;		x and y already compensated for */
{
    long width = dst->width;
    long height = dst->height;
    long i;
    UChar *pEndY;
    OctWord in;
    OctWord out;
    ULong pix;
    ULong highBit = 0x80808080;

    width >>= 4;	/* number of double word pairs (in the output) */
    i = height>>1;	/* number of double lines in the output */
    for (;;) {
	pEndY = pY + (width<<3);
	while (pY != pEndY) {
	    in = *OctWordP(pY)++;
	    in.l.high = (in.l.high >> 1 ) | highBit;

	    out.l.high = in.l.high & 0xff000000;
	    out.l.high |= out.l.high >> 8;
	    pix  = (in.l.high >> 16) & 0xff;
	    out.l.high |= pix | (pix << 8);

	    out.l.low = (in.l.high & 0xff00) << 8;
	    out.l.low |= out.l.low << 8;
	    pix = in.l.high & 0xff;
	    out.l.low |= pix | (pix << 8);

	    *OctWordP(pFb+fbWidth) = out;
	    *OctWordP(pFb)++ = out;

	    in.l.low  = (in.l.low >> 1 ) | highBit;
	    out.l.high = in.l.low & 0xff000000;
	    out.l.high |= out.l.high >> 8;
	    pix  = (in.l.low >> 16) & 0xff;
	    out.l.high |= pix | (pix << 8);

	    out.l.low = (in.l.low & 0xff00) << 8;
	    out.l.low |= out.l.low << 8;
	    pix = in.l.low & 0xff;
	    out.l.low |= pix | (pix << 8);

	    *OctWordP(pFb+fbWidth) = out;
	    *OctWordP(pFb)++ = out;
	}
	if (--i <= 0)
	    break;
	pFb += fbWidth*2 - (width<<4);
	pY += MAX_X - (width<<3);
    }
}

/*ARGSUSED*/
void ConvertYChannel8To24(UChar *pFb, long fbWidth, UChar *pY, GeomRectangle *src,
			  GeomRectangle *dst)
    /* GeomRectangle *src;		XXX ignored */
    /* GeomRectangle *dst;		x and y already compensated for */
{
    NormalOutput("output 8=>24 frame unimplemented\n");
}

/*ARGSUSED*/
void ConvertYChannel8To24Double(UChar *pFb, long fbWidth, UChar *pY, GeomRectangle *src,
				GeomRectangle *dst)
    /* GeomRectangle *src;		XXX ignored */
    /* GeomRectangle *dst;		x and y already compensated for */
{
    NormalOutput("output 8=>24 double size frame unimplemented\n");
}
