/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	copyblock.c - do motion compensation
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/copyblock.c,v 4.7 1994/02/05 00:26:06 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"
#include "motion.h"

#include "machdep.h"

static void
FCopyFwdBackMotion(UChar *pPast, UChar *pTo, UChar *pFuture)
{
    OctWord present;
    UChar *pEnd = pTo + 8*MAX_X;

    do {
	present.l.high  = (pPast[0] + pFuture[0]) >> 1;
	present.l.high <<= 8;
	present.l.high |= (pPast[1] + pFuture[1]) >> 1;
	present.l.high <<= 8;
	present.l.high |= (pPast[2] + pFuture[2]) >> 1;
	present.l.high <<= 8;
	present.l.high |= (pPast[3] + pFuture[3]) >> 1;

	present.l.low  = (pPast[4] + pFuture[4]) >> 1;
	present.l.low <<= 8;
	present.l.low |= (pPast[5] + pFuture[5]) >> 1;
	present.l.low <<= 8;
	present.l.low |= (pPast[6] + pFuture[6]) >> 1;
	present.l.low <<= 8;
	present.l.low |= (pPast[7] + pFuture[7]) >> 1;
	*OctWordP(pTo) = present;

	pTo += MAX_X;
	pPast += MAX_X;
	pFuture += MAX_X;
    } while (pTo != pEnd);
}

static void
FCopyHalfVertMotion(UChar *pTo, UChar *pTime)
{
    OctWord present;
    int i;

    for (i = 8; i > 0; i--) {
	present.l.high  = (pTime[0] + pTime[0+MAX_X]) >> 1;
	present.l.high <<= 8;
	present.l.high |= (pTime[1] + pTime[1+MAX_X]) >> 1;
	present.l.high <<= 8;
	present.l.high |= (pTime[2] + pTime[2+MAX_X]) >> 1;
	present.l.high <<= 8;
	present.l.high |= (pTime[3] + pTime[3+MAX_X]) >> 1;

	present.l.low  = (pTime[4] + pTime[4+MAX_X]) >> 1;
	present.l.low <<= 8;
	present.l.low |= (pTime[5] + pTime[5+MAX_X]) >> 1;
	present.l.low <<= 8;
	present.l.low |= (pTime[6] + pTime[6+MAX_X]) >> 1;
	present.l.low <<= 8;
	present.l.low |= (pTime[7] + pTime[7+MAX_X]) >> 1;
	*OctWordP(pTo) = present;

	pTo += MAX_X;
	pTime += MAX_X;
    }
}

static void
FCopyHalfHorizMotion(UChar *pTo, UChar *pTime)
{
    OctWord present;
    int i;

    for (i = 8; i > 0; i--) {
	present.l.high  = (pTime[0] + pTime[1]) >> 1;
	present.l.high <<= 8;
	present.l.high |= (pTime[1] + pTime[2]) >> 1;
	present.l.high <<= 8;
	present.l.high |= (pTime[2] + pTime[3]) >> 1;
	present.l.high <<= 8;
	present.l.high |= (pTime[3] + pTime[4]) >> 1;

	present.l.low  = (pTime[4] + pTime[5]) >> 1;
	present.l.low <<= 8;
	present.l.low |= (pTime[5] + pTime[6]) >> 1;
	present.l.low <<= 8;
	present.l.low |= (pTime[6] + pTime[7]) >> 1;
	present.l.low <<= 8;
	present.l.low |= (pTime[7] + pTime[8]) >> 1;
	*OctWordP(pTo) = present;

	pTo += MAX_X;
	pTime += MAX_X;
    }
}

static void
FCopyHalfMotion(UChar *pTo, UChar *pTime)
{
    OctWord present;
    int i;

    for (i = 8; i > 0; i--) {
	present.l.high  =
	    (pTime[0] + pTime[1] + pTime[MAX_X] + pTime[MAX_X+1]) >> 2;
	present.l.high <<= 8;
	present.l.high |=
	    (pTime[1] + pTime[2] + pTime[MAX_X+1] + pTime[MAX_X+2]) >> 2;
	present.l.high <<= 8;
	present.l.high |=
	    (pTime[2] + pTime[3] + pTime[MAX_X+2] + pTime[MAX_X+3]) >> 2;
	present.l.high <<= 8;
	present.l.high |=
	    (pTime[3] + pTime[4] + pTime[MAX_X+3] + pTime[MAX_X+4]) >> 2;

	present.l.low  =
	    (pTime[4] + pTime[5] + pTime[MAX_X+4] + pTime[MAX_X+5]) >> 2;
	present.l.low <<= 8;
	present.l.low |=
	    (pTime[5] + pTime[6] + pTime[MAX_X+5] + pTime[MAX_X+6]) >> 2;
	present.l.low <<= 8;
	present.l.low |=
	    (pTime[6] + pTime[7] + pTime[MAX_X+6] + pTime[MAX_X+7]) >> 2;
	present.l.low <<= 8;
	present.l.low |=
	    (pTime[7] + pTime[8] + pTime[MAX_X+7] + pTime[MAX_X+8]) >> 2;
	*OctWordP(pTo) = present;

	pTo += MAX_X;
	pTime += MAX_X;
    }
}

void
CopyYCBlock(UChar *pPast, UChar *pTo, UChar *pFuture, int halfPixel)
{
    UChar *pTime;
    OctWord present;

    if (pPast && pFuture) {
	DebugTask(N_YCValues,
		  printf("past:\n");
		  PrintPix(pPast));
	DebugTask(N_YCValues,
		  printf("future:\n");
		  PrintPix(pFuture));

	FCopyFwdBackMotion(pPast, pTo, pFuture);

	return;
    }

    GeneralAssert(pPast || pFuture);
    if (pPast) {
	halfPixel >>= FORWARD_TO_GENERIC_SHIFT;
	pTime = pPast;		/* a P or B picture */
	DebugTask(N_YCValues,
		  printf("past:\n");
		  PrintPix(pPast));
    } else {
	halfPixel >>= BACK_TO_GENERIC_SHIFT;
	pTime = pFuture;	/* a B picture */
	DebugTask(N_YCValues,
		  printf("future:\n");
		  PrintPix(pFuture));
    }

    switch(halfPixel) {
    default:
	Fatal("forward and back halfPixel bits: 0x%x\n", halfPixel);
	/*NOTREACHED*/

    case HALF_HORIZ_PIXEL:
	FCopyHalfHorizMotion(pTo, pTime);
	break;

    case HALF_VERT_PIXEL:
	FCopyHalfVertMotion(pTo, pTime);
	break;

    case HALF_HORIZ_PIXEL | HALF_VERT_PIXEL:
	FCopyHalfMotion(pTo, pTime);
	break;

    case 0:
	switch( (ULong)(pTime) & 0x7 ) {
	case 0x0:
	    *OctWordP(pTo+MAX_X*0) = *OctWordP(pTime+MAX_X*0);
	    *OctWordP(pTo+MAX_X*1) = *OctWordP(pTime+MAX_X*1);
	    *OctWordP(pTo+MAX_X*2) = *OctWordP(pTime+MAX_X*2);
	    *OctWordP(pTo+MAX_X*3) = *OctWordP(pTime+MAX_X*3);
	    *OctWordP(pTo+MAX_X*4) = *OctWordP(pTime+MAX_X*4);
	    *OctWordP(pTo+MAX_X*5) = *OctWordP(pTime+MAX_X*5);
	    *OctWordP(pTo+MAX_X*6) = *OctWordP(pTime+MAX_X*6);
	    *OctWordP(pTo+MAX_X*7) = *OctWordP(pTime+MAX_X*7);
	    break;

	case 0x4:
	    present.l.high = *ULongP(pTime+MAX_X*0+0);
	    present.l.low  = *ULongP(pTime+MAX_X*0+4);
	    *OctWordP(pTo+MAX_X*0) = present;
	    present.l.high = *ULongP(pTime+MAX_X*1+0);
	    present.l.low  = *ULongP(pTime+MAX_X*1+4);
	    *OctWordP(pTo+MAX_X*1) = present;
	    present.l.high = *ULongP(pTime+MAX_X*2+0);
	    present.l.low  = *ULongP(pTime+MAX_X*2+4);
	    *OctWordP(pTo+MAX_X*2) = present;
	    present.l.high = *ULongP(pTime+MAX_X*3+0);
	    present.l.low  = *ULongP(pTime+MAX_X*3+4);
	    *OctWordP(pTo+MAX_X*3) = present;
	    present.l.high = *ULongP(pTime+MAX_X*4+0);
	    present.l.low  = *ULongP(pTime+MAX_X*4+4);
	    *OctWordP(pTo+MAX_X*4) = present;
	    present.l.high = *ULongP(pTime+MAX_X*5+0);
	    present.l.low  = *ULongP(pTime+MAX_X*5+4);
	    *OctWordP(pTo+MAX_X*5) = present;
	    present.l.high = *ULongP(pTime+MAX_X*6+0);
	    present.l.low  = *ULongP(pTime+MAX_X*6+4);
	    *OctWordP(pTo+MAX_X*6) = present;
	    present.l.high = *ULongP(pTime+MAX_X*7+0);
	    present.l.low  = *ULongP(pTime+MAX_X*7+4);
	    *OctWordP(pTo+MAX_X*7) = present;
	    break;

	case 0x1:
	case 0x5:
	    CopyMotion(pTime, pTo, Load2ULongsAligned1);
	    break;

	case 0x2:
	case 0x6:
	    CopyMotion(pTime, pTo, Load2ULongsAligned2);
	    break;

	case 0x3:
	case 0x7:
	    CopyMotion(pTime, pTo, Load2ULongsAligned3);
	    break;
	}
	break;
    }
}

