/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	decoder.c - main control for mpeg decoder
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/decoder.c,v 4.43 1995/05/06 01:38:14 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"
#include "binarybits.h"
#include "huffman.h"

static void
BadPadBits(CacheArgsDecl, BitStream *buf)
{
    BitByteOffset offset = BitStreamOffset(buf, 0);

    buf->nCacheBits = _nCacheBits;
    buf->pbuf = _pCacheBuf;
    ProgMsg("pad bits not 0; @ %d+%d\n", offset.bytes, offset.bits);
}

CacheReturnValue
NextStartCode(CacheArgsDecl, BitStream *buf, int checkPadBits)
{
    register ULong padBits;
    register ULong tmp;

    if (tmp =_nCacheBits & 0x7) {
        FReadBits(buf, tmp, padBits, N_PadBits);
        if (checkPadBits && padBits != 0)
	    BadPadBits(CacheArgs, buf);
    }
    FReadBits(buf, 24, padBits, N_PadAndStartcodeBytes);
    while (padBits != B_STARTCODE) {
        if (checkPadBits && (padBits & 0xff0000))
	    BadPadBits(CacheArgs, buf);
        ReadBits(buf, 8, tmp, N_PadByte);
        padBits = (KeepBits(16, padBits) << 8) | tmp;
    }

    CacheReturn(buf);
}

static void
InvalidateScaledMatrix(SequenceInfo *pSeq)
{
    int i;

    for (i = 0; i < 31; i++) {
	pSeq->cachedIntraQMatrix[i][0] = 0;
    }
}

void
ScaleQuantMatrix(SequenceInfo *pSeq, int qScale)
{
    UChar *pInput;
    UShort *pOutput;
    int pass;
    int i;

    qScale--;	/* value falls from 1 to 31 */

    pSeq->scaledIntraQMatrix = &pSeq->cachedIntraQMatrix[qScale][0];
    pSeq->scaledNonIntraQMatrix = &pSeq->cachedNonIntraQMatrix[qScale][0];

    if (pSeq->cachedIntraQMatrix[qScale][0])
	return;

    qScale++;

    pInput = pSeq->intraQMatrix;
    pOutput = pSeq->scaledIntraQMatrix;
    for (pass = 1; pass <= 2; pass++) {
	for (i = 0; i < 8; i++) {
	    Unroll8(*pOutput++ = (UShort)*pInput++ * qScale);
	}
	pInput = pSeq->nonIntraQMatrix;
	pOutput = pSeq->scaledNonIntraQMatrix;
    }
}

CacheReturnValue
SequenceHeader(CacheArgsDecl, SequenceInfoPtr pSeq)
{
    BitStreamPtr mpegBuf = pSeq->mpegBuf;
    DisplayInfoPtr dpyInfo = pSeq->dpyInfo;
    int code, i, pelAspectBits;
    boolean loadIQM, loadNIQM, marker;
    ULong *pFrom, *pTo;
    UInt horizontalSize, verticalSize;
    ULong pictureRateCode;
    int bits;

    /*
     * this must be the sequence header code, which is actually 32 bits,
     * but NextStartCode already consumed the first 24.
     */
    FReadBits(mpegBuf, 8, code, N_SequenceHeaderCode);
    StreamAssert(mpegBuf, code == B_SEQUENCE_HEADER_CODE);

    FReadBits(mpegBuf, 12, horizontalSize, N_HorizontalSize);
    FReadBits(mpegBuf, 12, verticalSize, N_VerticalSize);
    FReadBits(mpegBuf, 4, pelAspectBits, N_PelAspectRatio);
    FReadBits(mpegBuf, 4, pictureRateCode, N_PictureRate);
    FReadBits(mpegBuf, 18, bits, N_BitRate);
    pSeq->bitRate = bits;
    FReadBits(mpegBuf, 1, marker, N_Marker);
    FReadBits(mpegBuf, 10, bits, N_VbvBufSize);
    pSeq->VBVBufSize = bits;
    FReadBits(mpegBuf, 1, bits, N_Constrained);
    pSeq->constrained = bits;

    dpyInfo->new.src.x = dpyInfo->new.src.x = 0;
    dpyInfo->new.src.width = horizontalSize;
    dpyInfo->new.src.height = verticalSize;
    dpyInfo->new.changed |= DPY_SrcRect;

    StreamAssert(mpegBuf, marker == 1);
    StreamAssert(mpegBuf,
	pelAspectBits != 0 && pelAspectBits != Binary4(1111, 4));

    switch(pictureRateCode) {
    case PICTURE_RATE_23_976:	break;
    case PICTURE_RATE_24:	break;
    case PICTURE_RATE_25:	break;
    case PICTURE_RATE_29_97:	break;
    case PICTURE_RATE_30:	break;
    case PICTURE_RATE_50:	break;
    case PICTURE_RATE_59_94:	break;
    case PICTURE_RATE_60:	break;

    /* NVR picture rates */
    case PICTURE_RATE_5:	break;
    case PICTURE_RATE_6:	break;
    case PICTURE_RATE_10:	break;
    case PICTURE_RATE_15:	break;
    case PICTURE_RATE_20:	break;

    /* a guess at XING rates */
    case PICTURE_RATE_XING_30:	break;

    default:
	break;
    }

    pSeq->pelAspectRatio = PelAspectRatio[ pelAspectBits ];

    /*
     * Intra quantizer matrix
     */
    FReadBits(mpegBuf, 1, loadIQM, N_LoadIQM);
    if (pSeq->loadIQM = loadIQM) {
	for (i = -1; ++i < N_BLOCK_CELLS;) {
	    FReadBits(mpegBuf, 8, bits, N_NoLabel);
	    pSeq->intraQMatrix[ ZigZag[i] ] = bits;
	    StreamAssert(mpegBuf, bits != 0);
	}
	StreamAssert(mpegBuf, pSeq->intraQMatrix[ 0 ] == 8);
	DebugTask(N_LoadIQM,
	    Msg("Intra matrix:\n");
	    PrintQMatrix(pSeq->intraQMatrix);
	)
    } else {
	pFrom = (ULong *) DefaultIntraQMatrix;
	pTo = (ULong *) pSeq->intraQMatrix;
	Unroll16(*pTo++ = *pFrom++);
    }


    /*
     * Non-intra quantizer matrix
     */
    FReadBits(mpegBuf, 1, loadNIQM, N_LoadNIQM);
    if (pSeq->loadNIQM = loadNIQM) {
	for (i = 0; i < N_BLOCK_CELLS; i++) {
	    FReadBits(mpegBuf, 8, bits, N_NoLabel);
	    pSeq->nonIntraQMatrix[ ZigZag[i] ] = bits;
	    StreamAssert(mpegBuf, bits != 0);
	}
	DebugTask(N_LoadNIQM,
	    Msg("Non-intra matrix:\n");
	    PrintQMatrix(pSeq->nonIntraQMatrix);
	)
    } else {
	pFrom = (ULong *) DefaultNonIntraQMatrix;
	pTo = (ULong *) pSeq->nonIntraQMatrix;
	Unroll16(*pTo++ = *pFrom++);
    }

    InvalidateScaledMatrix(pSeq);

    ShowMark(mpegBuf, N_SequenceMark,
	("%dx%d, pel=%.4f:1,%.2ffps(%d),%dbps,vbv=%d,%s,%sIQM,%sNIQM\n",
	horizontalSize, verticalSize, FixedToFloat(pSeq->pelAspectRatio),
	FrameRateTable[ pictureRateCode ], pictureRateCode,
	pSeq->bitRate, pSeq->VBVBufSize,
	pSeq->constrained ? "constr" : "unconstr",
	loadIQM ? "" : "no ",
	loadNIQM ? "" : "no "));

    RestoreCacheAfterCall(mpegBuf, NextStartCode(CacheArgs, mpegBuf, True) );

    /*
     * Get the extension data
     */
    FNextBits(mpegBuf, 8, code);
    if (code == B_EXTENSION_START_CODE) {
	FReadBits(mpegBuf, 8, code, N_ExtensionData);
	ReadBitstreamData(mpegBuf, data /* a noop */);
	/* ReadBitstreamData(pSeq->extData[ pSeq->extDataLen++ ]); */

	FNextBits(mpegBuf, 8, code);  /* get ready for the next if statement */
    }

    /*
     * Get the user data
     */
    if (code == B_USER_DATA_START_CODE) {
	FReadBits(mpegBuf, 8, code, N_UserData);
	FNextBits(mpegBuf, 24, code);
	if( code == B_STARTCODE ) {
	    /*
	     * The XING player will always read a byte of user data if
	     *	B_USER_DATA_START_CODE is encountered
	     */
	    ProgMsg("0-length user data. Ignoring start code\n");
	    FReadBits(mpegBuf, 8, code, N_UserData);
	}
	ReadBitstreamData(mpegBuf, data /* a noop */);
	/* ReadBitstreamData(pSeq->userData[ pSeq->userDataLen++ ]); */
    }

    /*
     * Initialize data if necessary
     */
    if (horizontalSize != pSeq->horizontalSize
     || verticalSize != pSeq->verticalSize
     || pictureRateCode != pSeq->pictureRateCode) {
	pSeq->horizontalSize = horizontalSize;
	pSeq->verticalSize = verticalSize;
	pSeq->mbWidth = (horizontalSize+15) >> 4;
	pSeq->mbHeight = (verticalSize+15) >> 4;
	pSeq->lastMacroBlock = pSeq->mbWidth * pSeq->mbHeight - 1;
	pSeq->pictureRateCode = pictureRateCode;
	InitMBAddressLookup(pSeq);
    }

    CacheReturn(mpegBuf);
}

Inline static
CacheReturnValue GroupOfPictures(CacheArgsDecl, SequenceInfoPtr pSeq)
{
    BitStreamPtr mpegBuf = pSeq->mpegBuf;
    TimeCode tc;
    boolean closedGop;
    boolean brokenLink;
    register int code;

    /*
     * this must be the group header code, which is actually 32 bits,
     * but NextStartCode already consumed the first 24.
     */
    FReadBits(mpegBuf, 8, code, N_GroupStartCode);
    StreamAssert(mpegBuf, code == B_GROUP_START_CODE);

    FReadBits(mpegBuf, 25, tc.composite, N_Timecode);
    /* XXX XING StreamAssert(mpegBuf, tc.discrete.marker == 1); */
    pSeq->tc = tc;
    WhenDebugEnabled(
	tc.discrete.groupOfPictures = pSeq->tc.discrete.groupOfPictures+1
    );

    StreamAssert(mpegBuf, tc.discrete.hours    <= 23);
    StreamAssert(mpegBuf, tc.discrete.minutes  <= 59);
    StreamAssert(mpegBuf, tc.discrete.seconds  <= 59);
    StreamAssert(mpegBuf, tc.discrete.pictures <= 59);

    FReadBits(mpegBuf, 2, code, N_GopInfo);
    pSeq->closedGop = code >> 1;  
    pSeq->brokenLink = code & 1;  

    pSeq->picGopStart += pSeq->picsInLastGop;  
    pSeq->picsInLastGop = 0;

    RestoreCacheAfterCall(mpegBuf, NextStartCode(CacheArgs, mpegBuf, True) );

    /*
     * Get the extension data
     */
    FNextBits(mpegBuf, 8, code);
    if (code == B_EXTENSION_START_CODE) {
	FReadBits(mpegBuf, 8, code, N_ExtensionData);
	ReadBitstreamData(mpegBuf, data /* a noop */);
	/* ReadBitstreamData(mpegBuf,
			     pSeq->grpExtData[ pSeq->grpExtDataLen++ ]); */

	FNextBits(mpegBuf, 8, code);  /* get ready for the next if statement */
    }

    /*
     * Get the user data
     */
    if (code == B_USER_DATA_START_CODE) {
	FReadBits(mpegBuf, 8, code, N_UserData);
	ReadBitstreamData(mpegBuf, data /* a noop */);
	/* ReadBitstreamData(mpegBuf,
			     pSeq->grpUserData[ pSeq->grpUserDataLen++ ]); */
    }

    do {
	RestoreCacheAfterCall(mpegBuf, (*pSeq->picture)(CacheArgs, pSeq) );
	FNextBits(mpegBuf, 8, code);
    } while (code == B_PICTURE_START_CODE);

    CacheReturn(mpegBuf);
}

static PictureInfoPtr
NewIPPicture(SequenceInfo *pSeq, ULong codes, ULong tc)
{
    pSeq->currentPict = (*pSeq->allocPicture)(pSeq, codes, tc);
    return (pSeq->futurePict = pSeq->currentPict);
}

static PictureInfoPtr
NewBPicture(SequenceInfo *pSeq, ULong codes, ULong tc)
{
    return (pSeq->currentPict = (*pSeq->allocPicture)(pSeq, codes, tc));
}

static CacheReturnValue
SkipPictures(CacheArgsDecl, SequenceInfoPtr pSeq, ULong codesComposite)
{
    BitStreamPtr mpegBuf = pSeq->mpegBuf;
    ULong code;
    ULong picsInLastGop = pSeq->picsInLastGop;
    UInt typeWanted;
    PicCodes current, new;

    current.composite = codesComposite;
    if (pSeq->currentPict)
	pSeq->currentPict->skipped = True;

    switch(current.s.type) {
    case B_B_PIC:
	typeWanted = 0;
	break; /* anything will do */
    case B_P_PIC:
    case B_I_PIC:
	if (pSeq->pastPict) {
	    ReleasePicture(pSeq->pastPict);
	    pSeq->pastPict = NULL;
	}
	typeWanted = B_I_PIC;
	break;
    }

    for(;;) {
	/*
	 * read through to the next start code.
	 */
	RestoreCacheAfterCall(mpegBuf,
			      NextStartCode(CacheArgs, mpegBuf, False) );

	/*
	 * peek at the start code (without actually consuming it) and
	 * see if this is a picture start, a group start or a sequence
	 * start.  If none of those, it must be a slice start and we
	 * are still in the middle of a picture.
	 *
	 * Note that we can only peek at 8 bits because this may be
	 * a sequence end code, followed by an end of file.
	 */
	FNextBits(mpegBuf, 8, code);
	switch (code) {
	case B_SEQUENCE_HEADER_CODE:
	case B_SEQUENCE_ERROR_CODE:
	case B_SEQUENCE_END_CODE:
	case B_GROUP_START_CODE:
	    DebugTask(N_SkippedFrames,
		Msg("skipped %d pics from %c pic %d to start code 0x%x\n",
		    1 + picsInLastGop - pSeq->picsInLastGop,
		    IPBDchars[ current.s.type ],
		    pSeq->picGopStart + current.s.timeRef,
		    code);
	    )

	    /*
	     * We are at the start of a group or sequence, then we
	     * can return because the next frame is an I frame.  In any
	     * case, we need to process the new sequece or gop data.
	     */
	    break;

	case B_PICTURE_START_CODE:
	    /*
	     * new picture... decide if this is the one we want.
	     */
	    FNextBits(mpegBuf, 21, new.composite);
	    if (typeWanted == 0 || new.s.type == typeWanted) {
		DebugTask(N_SkippedFrames,
		    Msg("skipped %d pics from %c pic %d to %c pic %d\n",
			1 + picsInLastGop - pSeq->picsInLastGop,
			IPBDchars[ current.s.type ],
			pSeq->picGopStart + current.s.timeRef,
			IPBDchars[ new.s.type ],
			pSeq->picGopStart + new.s.timeRef);
		)
		break;
	    }
	    picsInLastGop++;
	    continue;

	default:
	    /*
	     * This must be a slice start code.  We're
	     * still in the middle of a picture... look for the next one
	     */
	    FConsumeBits(mpegBuf, 8, N_SliceStartCode);
	    continue;
	}
	break;
    }

    pSeq->picsInLastGop = picsInLastGop;
    CacheReturn(mpegBuf);
}

Inline static ULong
ComputeTimeCode(SequenceInfo *pSeq, int timeRef)
{
    TimeCode tc;
    UShort unit, nextUnit;
    UChar pictureLimit;
    boolean checkDropFrame;

    tc = pSeq->tc;

    switch (pSeq->pictureRateCode) {
    case PICTURE_RATE_23_976:	pictureLimit = 24; break;
    case PICTURE_RATE_24:	pictureLimit = 24; break;
    case PICTURE_RATE_25:	pictureLimit = 25; break;
    case PICTURE_RATE_30:	pictureLimit = 30; break;
    case PICTURE_RATE_50:	pictureLimit = 50; break;
    case PICTURE_RATE_59_94:	pictureLimit = 60; break;
    case PICTURE_RATE_60:	pictureLimit = 60; break;

    /*
     * NVR picture rates... 
     */
    case PICTURE_RATE_5:	pictureLimit = 5; break;
    case PICTURE_RATE_6:	pictureLimit = 6; break;
    case PICTURE_RATE_10:	pictureLimit = 10; break;
    case PICTURE_RATE_15:	pictureLimit = 15; break;
    case PICTURE_RATE_20:	pictureLimit = 20; break;

    case PICTURE_RATE_29_97:
	checkDropFrame = tc.discrete.dropFrame;
	pictureLimit = 30;
	break;

    case PICTURE_RATE_XING_30:	pictureLimit = 15; break;

    default:
	Fatal("unknown encoded picture rate: 0x%x\n", pSeq->pictureRateCode);
	/* NOTREACHED */
    }

    /*
     * NVR picture rates... Our encoder still produces timecodes
     * and temporal references with respect to 30fps
     */

    unit = tc.discrete.pictures + timeRef;
    for (nextUnit = 0; unit >= pictureLimit; unit -= pictureLimit, nextUnit++) ;
    tc.discrete.pictures = unit;
    if (nextUnit == 0)
	goto timeCodeComplete;

    unit = tc.discrete.seconds + nextUnit;
    for (nextUnit = 0; unit >= 60; unit -= 60, nextUnit++) ;
    tc.discrete.seconds = unit;
    if (nextUnit == 0)
	goto timeCodeComplete;

    unit = tc.discrete.minutes + nextUnit;
    for (nextUnit = 0; unit >= 60; unit -= 60, nextUnit++) ;
    tc.discrete.minutes = unit;
    if (nextUnit == 0)
	goto timeCodeComplete;

    unit = tc.discrete.hours + nextUnit;
    for (nextUnit = 0; unit >= 24; unit -= 24, nextUnit++) ;
    tc.discrete.hours = unit;
    GeneralAssert(nextUnit == 0);

timeCodeComplete:
    return (tc.composite);
}

Inline static
CacheReturnValue Slice(CacheArgsDecl, SequenceInfoPtr pSeq)
{
    UInt code;
    UInt bits;
    int qScale, i, mbAddr;
    BitStreamPtr mpegBuf = pSeq->mpegBuf;
    DisplayInfoPtr dpyInfo = pSeq->dpyInfo;
    MacroBlockInfoPtr pMb = pSeq->macroBlk;
    WhenDebugEnabled(PictureInfoPtr pPic = pSeq->currentPict);

    /*
     * this must be the slice start code, which is actually 32 bits,
     * but NextStartCode already consumed the first 24.
     */
    FReadBits(mpegBuf, 8, code, N_SliceStartCode);
    StreamAssert(mpegBuf,
	code >= B_SLICE_START_MIN && code <= B_SLICE_START_MAX);

    /*
     * The vertical offset is usually redundant.  It points to the same
     * row that the first macroblock address increment in the slice
     * points to.  However, if a slice starts in the middle of a row,
     * the next address increment should assume that the mbaddr has
     * been reset to the beginning of the row, minus one.
     *
     * For example, if the the picture is 22 macroblocks wide and
     * the first macroblock goes from mb 0 through mb 26, then the
     * processed macroblocks looks like:
     *
     *    x x x x x x x x x x x x x x x x x x x x x
     *    x x x x x
     *
     * Since there is guaranteed to be no gap in the
     * macro block addresses between slices, the address of the first
     * macroblock in the next slice (27) is always one more
     * than the previous mbaddr (26).  But since we have to reset
     * the mb address to the beginning of the row, minus one (or 21),
     * the next address increment must put us back to the right place
     * i.e. mb addr must be 6 (21+6=27).
     */
    mbAddr = -1;
    for (i = code-1; i > 0; i--)
	mbAddr += pSeq->mbWidth;
    pMb->mbAddr = mbAddr;

    FReadBits(mpegBuf, 5, qScale, N_QuantizerScale);
    if (qScale != pSeq->qScale) {
	pSeq->qScale = qScale;
	ScaleQuantMatrix(pSeq, qScale);
    }

    ShowMark(mpegBuf, N_SliceMark, ("qscale=%d\n", pSeq->qScale));

    for (;;) {
	FReadBits(mpegBuf, 1, bits, N_ExtraSliceInfo);
	if (bits) {
	    FReadBits(mpegBuf, 8, bits, N_NoLabel);
	} else
	    break;
    }

    pMb->pastIntraMbAddr = -2;		/* per protocol */
    pMb->dcYPast = 128;			/* per protocol */
    pMb->dcCrPast = 128;		/* per protocol */
    pMb->dcCbPast = 128;		/* per protocol */
    pMb->pastHorizBakMotion = 0;	/* per protocol */
    pMb->pastVertBakMotion = 0;		/* per protocol */
    pMb->pastHorizFwdMotion = 0;	/* per protocol */
    pMb->pastVertFwdMotion = 0;		/* per protocol */
    pSeq->mbToCheckTime = pSeq->mbCheckInc;

    do {
	if (pMb->mbAddr >= pSeq->mbToCheckTime) {
	    pSeq->mbToCheckTime += pSeq->mbCheckInc;
	    CheckServices(dpyInfo);
	}
	RestoreCacheAfterCall(mpegBuf, MacroBlock(CacheArgs, pSeq) );
	FNextBits(mpegBuf, 23, code);
	pMb->pastMbAddr = pMb->mbAddr;
    } while (code != 0);

    pMb->firstMbInSlice = pMb->mbAddr + 1;

    RestoreCacheAfterCall(mpegBuf, NextStartCode(CacheArgs, mpegBuf, True) );

    CacheReturn(mpegBuf);
}

static void
AddIPicOffset(SequenceInfoPtr pSeq, long offset, long picNumber)
{
    int index = pSeq->iPics.nOffsets++;

    if (pSeq->iPics.listLength <= pSeq->iPics.nOffsets) {
	int listLength = pSeq->iPics.listLength += IPicOffsetListInc;
	int size = listLength * sizeof(IPicOffset);

	pSeq->iPics.listLength = listLength;
	pSeq->iPics.list =
	    (IPicOffsetPtr) NullRealloc((char *)pSeq->iPics.list, size);
    }
    pSeq->iPics.list[ index ].picNumber = picNumber;
    pSeq->iPics.list[ index ].offset = offset;

    DebugTask(N_SeekActivity, Msg("Add I%d @ off %d\n", picNumber, offset))
}

CacheReturnValue
ProcPicture(CacheArgsDecl, SequenceInfoPtr pSeq)
{
    BitStreamPtr mpegBuf = pSeq->mpegBuf;
    register int code;
    register PictureInfo *pPic;
    register ULong bits;
    int vbvDelay, displaySched;
    TimeCode tc;
    UChar fullPelBakVec;
    PicCodes codes;

    /*
     * this must be the picture start code, which is actually 32 bits,
     * but NextStartCode already consumed the first 24.
     */
    FReadBits(mpegBuf, 21, codes.composite, N_PictureStart);
    StreamAssert(mpegBuf, codes.s.code == B_PICTURE_START_CODE);

    /* only 01, 10 and 11 valid */
    StreamAssert(mpegBuf, codes.s.type && codes.s.type < 4);

    if( pSeq->xingSequence ) {
	pSeq->lastPicNumber++;
	codes.s.timeRef = pSeq->lastPicNumber - pSeq->picGopStart;
    }
    pSeq->picsInLastGop++;
    /*
     * Before we go any further, we need to update the reference picture
     * pointers so that things will be hunky-dory even if we skip
     * this picture.
     */
    switch( codes.s.type ) {
    case B_I_PIC:
	if (! pSeq->dpyInfo->continuousMedia) {
	    BitByteOffset offset;
	    FlushCacheValues(mpegBuf);
	    offset = BitStreamOffset(mpegBuf, 21);
	    AddIPicOffset(pSeq, offset.bytes,
			  pSeq->picGopStart + codes.s.timeRef);
	}
	/* fall through... */

    case B_P_PIC:
	if (pSeq->prevPict)
	    (*pSeq->pictureReady)(pSeq->prevPict, True);
	if (pSeq->futurePict) {
	    (*pSeq->pictureReady)(pSeq->futurePict, False);
	    if (pSeq->pastPict)
		ReleasePicture(pSeq->pastPict);
	    pSeq->pastPict = pSeq->futurePict;
	}
	break;

    case B_B_PIC:
	if (pSeq->prevPict)
	    (*pSeq->pictureReady)(pSeq->prevPict, True);
	break;
    }

    displaySched = (*pSeq->schedule)(pSeq, &codes.composite);
    if (displaySched == 0) {
	RestoreCacheAfterCall(mpegBuf,
			      SkipPictures(CacheArgs, pSeq, codes.composite) );
	pPic = NULL;
	goto pictureDone;
    }

    FReadBits(mpegBuf, 16, vbvDelay, N_VbvDelay);

    tc.composite = ComputeTimeCode(pSeq, codes.s.timeRef);

    /*
     * If we longjump back here, the value is non-zero, meaning
     * that we should skip the picture and move on.
     */
    if (SaveContext(pSeq->pictContext)) {
	RestoreCacheAfterCall(mpegBuf,
			      SkipPictures(CacheArgs, pSeq, codes.composite) );
	pPic = NULL;
	goto pictureDone;
    }

    if (codes.s.type == B_P_PIC || codes.s.type == B_B_PIC) {
	UChar forwardSize;
	UChar fullPelFwdVec;

	FReadBits(mpegBuf, 1, fullPelFwdVec, N_FullPelFwdVector);
	FReadBits(mpegBuf, 3, bits, N_ForwardFCode);
	StreamAssert(mpegBuf, bits >= 1 && bits <= 7);
	forwardSize = bits - 1;

	switch (codes.s.type) {
	case B_B_PIC:
	    pPic = NewBPicture(pSeq, codes.composite, tc.composite);
	    FReadBits(mpegBuf, 1, fullPelBakVec, N_FullPelBakVector);
	    FReadBits(mpegBuf, 3, bits, N_BackwardFCode);
	    StreamAssert(mpegBuf, bits >= 1 && bits <= 7);
	    pPic->backwardSize = bits - 1;
	    if (pPic->fullPelBakVec != fullPelBakVec)
		pPic->fullPelBakVec = fullPelBakVec;
	    break;

	case B_P_PIC:
	    pPic = (PictureInfo *)NewIPPicture(pSeq,
					       codes.composite,
					       tc.composite);
	    break;
	}
	if (pPic->fullPelFwdVec != fullPelFwdVec)
	    pPic->fullPelFwdVec = fullPelFwdVec;
	if (pPic->forwardSize != forwardSize)
	    pPic->forwardSize = forwardSize;
    } else if (codes.s.type == B_I_PIC) {
	pPic = (PictureInfo *)NewIPPicture(pSeq, codes.composite, tc.composite);
    } else
	Fatal("don't know how to handle D pictures\n");

    /*
     * We may have blocked for some time in New*Picture waiting for a
     * picture to become available after being displayed.  In particular,
     * when we entered New*Picture, we may have been in a stopped state, where
     * all picture schedules are '1'.  If so, when we return, the
     * state will be DisplayRunning, a picture would have been displayed,
     * subsequently freeing a picture for New*Picture to return.
     *
     * That's the background.  The problem is that the schedule we have
     * is bogus for a running state and we may have to reschedule.  The
     * pain of it all is that the picture may be cancelled and the
     * picture has to make the rounds through the display queue, back
     * to the picture queue.
     */
    pPic->type = codes.s.type;
    pPic->timeRef = codes.s.timeRef;
    pPic->picNumber = pSeq->picGopStart + codes.s.timeRef;
    if (pPic->reschedule && pSeq->dpyInfo->state == DisplayRunning) {
	displaySched = (*pSeq->schedule)(pSeq, &codes.composite);
	if (displaySched == 0) {
	    DebugTask(N_DisplayTimecode,
		Msg("pic %c #%d resched failed\n",
		    IPBDchars[ codes.s.type ], pPic->picNumber);
	    )

	    pPic->skipped = True;
	    pPic->estDecodeTime = 0;
	    RestoreCacheAfterCall(mpegBuf,
		SkipPictures(CacheArgs,pSeq,codes.composite) );
	    goto pictureDone;
	}
    }

    pPic->displaySched = displaySched;
    pSeq->qScale = 0;	/* force the very first check to fail in Slice() */
    pPic->estDecodeTime = pSeq->decodeTime[ codes.s.type ] + pSeq->displayTime;
    pSeq->workToDo += pPic->estDecodeTime;

    WhenDebugEnabled(
	pPic->vbvDelay = vbvDelay;
	if (tc.composite == BreakAtTc
	 || pPic->picNumber == BreakAtPic)
	    BreakHere();
    )
    DebugTask(N_Timecode,
	Msg("pic %c #%d, %d/%02d:%02d:%02d\n",
	    IPBDchars[ codes.s.type ],
	    pPic->picNumber,
	    tc.discrete.groupOfPictures,
	    tc.discrete.minutes,
	    tc.discrete.seconds,
	    tc.discrete.pictures)
    )

    /*
     * Process the picture normally
     */
    TimerTask(pSeq->timers, N_TimePictures, StartTimer());
    for (;;) {
	FReadBits(mpegBuf, 1, bits, N_ExtraPictureInfo);
	if (bits) {
	    FReadBits(mpegBuf, 9, bits, N_NoLabel);
	} else
	    break;
    }

    ShowMark(mpegBuf, N_PictureMark,
	("#%d, %s pict, vbv delay=%d\n",
	pPic->timeRef,
	codes.s.type == B_I_PIC ? "I" :
	    codes.s.type == B_P_PIC ? "P" :
		codes.s.type == B_B_PIC ? "B" : "D",
	vbvDelay));

    RestoreCacheAfterCall(mpegBuf, NextStartCode(CacheArgs, mpegBuf, False) );

    /*
     * Get the extension data
     */
    FNextBits(mpegBuf, 8, code);
    if (code == B_EXTENSION_START_CODE) {
	FReadBits(mpegBuf, 8, code, N_ExtensionData);
	ReadBitstreamData(mpegBuf, data /* a noop */);

	/* get ready for the next if statement */
	FNextBits(mpegBuf, 8, code);
    }

    /*
     * Get the user data
     */
    if (code == B_USER_DATA_START_CODE) {
	FReadBits(mpegBuf, 8, code, N_UserData);
	ReadBitstreamData(mpegBuf, data /* a noop */);
    }

    /*
     * Do the slices in this picture
     */
    pSeq->macroBlk->firstMbInSlice = 0;
    pSeq->macroBlk->pastMbAddr = -1;
    do {
	RestoreCacheAfterCall(mpegBuf, Slice(CacheArgs, pSeq) );
	FNextBits(mpegBuf, 8, code);
    } while (code >= B_SLICE_START_MIN && code <= B_SLICE_START_MAX);

    TimerTask(pSeq->timers, N_TimePictures,
	NormalOutput("pic %d/%02d:%02d:%02d (%c): %s\n",
	    pPic->tc.discrete.groupOfPictures,
	    pPic->tc.discrete.minutes,
	    pPic->tc.discrete.seconds,
	    pPic->tc.discrete.pictures,
	    IPBDchars[ pPic->type ],
	    FormatTimeUsage(StopTimer(), True));
    )

    /*
     * For the first frame of this type, use the unweighted time as
     * the value.  Otherwise it takes a while for the statistics to
     * gather for infrequent types (like I and P).
     */
    if (pSeq->decodeTime[ codes.s.type ] == 0) {
	pSeq->decodeTime[ codes.s.type ] = pPic->decodeTime;
    } else {
	pSeq->decodeTime[ codes.s.type ] =
	    ExpMovAverage(pSeq->decodeTime[ codes.s.type ],
			  pPic->decodeTime,
			  PictTimeAvg);
    }
    pSeq->dpyInfo->framesDecoded++;

pictureDone:
    pSeq->currentPict = NULL;	/* we're done with the current picture */
    if (codes.s.type == B_B_PIC)
	pSeq->prevPict = pPic;
    else {
	if (pPic == NULL)	/* we skipped the picture */
	    pSeq->futurePict = NULL;
	pSeq->prevPict = NULL;
    }

    CacheReturn(mpegBuf);
}

ImageReasonPtr
ConfigureDecoder(SequenceInfoPtr pSeq)
{
    return(NULL);
}

void
MpegDecode(SequenceInfoPtr pSeq)
{
    ULong bits;
    BitStream *mpegBuf = pSeq->mpegBuf;
    CheckOutCache(mpegBuf);

    /*
     * When we longjump back here, the value is non-zero, meaning
     * that we should terminate the sequence... XXX some pictures
     * may be left dangling!
     */
    if (SaveContext(pSeq->seqContext)) {
	return;
    }

    pSeq->currentPict = NULL;
    pSeq->prevPict = NULL;
    pSeq->pastPict = NULL;
    pSeq->futurePict = NULL;
    pSeq->picsInLastGop = 0;  

    for (;;) {
	do {
	    RestoreCacheAfterCall(mpegBuf, GroupOfPictures(CacheArgs, pSeq) );
	    FNextBits(mpegBuf, 8, bits);
	} while (bits == B_GROUP_START_CODE);

	/*
	 * The first sequence header is read when the file is first
	 * opened.  This read is here for subsequence sequence headers
	 * in the stream that do things like reset the quantizer matrices.
	 */
	if (bits == B_SEQUENCE_HEADER_CODE) {
	    RestoreCacheAfterCall(mpegBuf, SequenceHeader(CacheArgs, pSeq) );
	} else {
	    break;
	}
    }

    StreamAssert(mpegBuf, bits == B_SEQUENCE_END_CODE);
    FReadBits(mpegBuf, 8, bits, N_SequenceEndCode);

    /*
     * We normally don't display a picture until the next one arrives.
     * In this case, there is no next picture.  After reaching this point
     * we may have just finished decoding a B picture.  If so, display it.
     * Finally, we display the final P (or future) picture.
     */
    if (pSeq->prevPict) {
	(*pSeq->pictureReady)(pSeq->prevPict, True);
	pSeq->prevPict = NULL;
    }
    if (pSeq->futurePict) {
	(*pSeq->pictureReady)(pSeq->futurePict, True);
	pSeq->futurePict = NULL;
    }
    pSeq->pastPict = NULL;

    FlushCacheValues(mpegBuf);
}

