/*
 *	(c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		decoder.h
 *
 *	DESCRIPTION
 *		basic decoder definitions for mpeg
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/decoder.h,v 4.35 1995/08/18 03:42:27 toddb Exp $
 */
#ifndef _DECODER_H_
#define _DECODER_H_

#include "nvrTypes.h"
#include "bitstream.h"
#include "longjump.h"

#define MAX_X			352
#define MAX_LUMA_Y		480
#define MAX_CHROMA_Y		240
#define MAX_MB_PER_ROW	 	(MAX_X>>4)
#define MAX_MB_PER_COL	 	(MAX_LUMA_Y>>4)

#define YToMbRow(y)		((y)>>4)
#define XToMbColumn(x)		((x)>>4)

#define BITSTREAM_BUF_SIZE	(8*1024)	/* in longwords */
/*
 * yblk in the macros below should be one of Y0, Y1, Y2 or Y3.  As such
 * it has to be converted to an index.  Hence, 5-yblk, changes Y1
 * whose value is 4, into 1.
 */
#define YBlkToIndex(yblk)		(5-(yblk))

/*
 * This constant is the way we store cr and cb in a buffer that is
 * the same width as the luma: cr and cb are stored side-by-side.`
 * Let 'r' and 'b' represent cr and cb, respectively.  Then
 *    bbbbbbbbbbbrrrrrrrrrrr
 *    bbbbbbbbbbbrrrrrrrrrrr
 *    bbbbbbbbbbbrrrrrrrrrrr
 *    bbbbbbbbbbbrrrrrrrrrrr
 *    bbbbbbbbbbbrrrrrrrrrrr
 * is a picture of the cr/cb frame buffer.  The cr is (MAX_X>>1) to the
 * right of cb.
 */
#define CR_OFFSET			(MAX_X >> 1)
#define CbToCrAddr(cbAddr)		(cbAddr + CR_OFFSET)

#define MBLumaOffset(yblk)		((YBlkToIndex(yblk)>>1)*MAX_X*8 \
					 + (YBlkToIndex(yblk)&1)*8)

#define MBAddrToLuma(yblk, mbAddr)	((pMb->mbAddrMap[mbAddr] << 4)	\
					 + MBLumaOffset(yblk))

#define MBAddrToCb(mbAddr)		(pMb->mbAddrMap[mbAddr] << 3)
#define MBAddrToCr(mbAddr)		CbToCrAddr(MBAddrToCb(mbAddr))

#define MotionOffset(horiz, vert)	((vert)*MAX_X + (horiz))
#define CbMotionOffset(horiz, vert)	MotionOffset(horiz, vert)
#define CbToCrMotionOffset(crMotionOff)	(crMotionOff + CR_OFFSET)

/*
 * These constants define the flags for vertical and horizontal, past
 * and future motion vectors.
 */
#define HALF_HORIZ_BACK_SHIFT		0
#define HALF_VERT_BACK_SHIFT		1
#define HALF_HORIZ_FORWARD_SHIFT	2
#define HALF_VERT_FORWARD_SHIFT		3

#define HALF_HORIZ_BACK_PIXEL		(1<<HALF_HORIZ_BACK_SHIFT)
#define HALF_VERT_BACK_PIXEL		(1<<HALF_VERT_BACK_SHIFT)
#define HALF_HORIZ_FORWARD_PIXEL 	(1<<HALF_HORIZ_FORWARD_SHIFT)
#define HALF_VERT_FORWARD_PIXEL		(1<<HALF_VERT_FORWARD_SHIFT)

/*
 * When you have only one of future or motion, these are the generic
 * flags, and a shift value to convert forward and back flags to generic.
 */
#define HALF_HORIZ_PIXEL		(1<<0)
#define HALF_VERT_PIXEL			(1<<1)
#define BACK_TO_GENERIC_SHIFT		0
#define FORWARD_TO_GENERIC_SHIFT	2

#define B_I_PIC         1       /* (I) intra frame coded */
#define B_P_PIC         2       /* (P) predictive coded */
#define B_B_PIC         3       /* (B) bidirectionally predictively coded */
#define B_D_PIC         4       /* (D) DC intra frame coded */
#define B_PROCESSED_PIC 5       /* a preprocessed picture (internal) */

#define B_STARTCODE             0x000001/* 24-bit byte-align start code */
#define B_PICTURE_START_CODE    0x00    /* picture start code */
#define B_SLICE_START_MIN       0x01    /* Minimum slice start code */
#define B_SLICE_START_MAX       0xAF    /* Maximum slice start code */
#define B_USER_DATA_START_CODE  0xB2    /* user data start code */
#define B_SEQUENCE_HEADER_CODE  0xB3    /* sequence header code */
#define B_SEQUENCE_ERROR_CODE   0xB4    /* sequence error code */
#define B_EXTENSION_START_CODE  0xB5    /* extension start code */
#define B_SEQUENCE_END_CODE     0xB7    /* sequence end code */
#define B_GROUP_START_CODE      0xB8    /* group start code */

#define B_MACROBLOCK_STUFFING   0x00F   /* 11-bit code for padding */
#define B_MACROBLOCK_ESCAPE     0x008   /* 11-bit code for macroblock escape */

/*
 * Protocol picture rates
 */
#define PICTURE_RATE_23_976	Binary4(0001,4)
#define PICTURE_RATE_24		Binary4(0010,4)
#define PICTURE_RATE_25		Binary4(0011,4)
#define PICTURE_RATE_29_97	Binary4(0100,4)
#define PICTURE_RATE_30		Binary4(0101,4)
#define PICTURE_RATE_50		Binary4(0110,4)
#define PICTURE_RATE_59_94	Binary4(0111,4)
#define PICTURE_RATE_60		Binary4(1000,4)

/*
 * NVR picture rates
 */
#define PICTURE_RATE_XING_30	Binary4(1001,4)
#define PICTURE_RATE_6		Binary4(1010,4)
#define PICTURE_RATE_10		Binary4(1011,4)
#define PICTURE_RATE_15		Binary4(1100,4)
#define PICTURE_RATE_20		Binary4(1101,4)
#define PICTURE_RATE_5		Binary4(1110,4)

#define MBT_QUANT_BitPosition	0
#define MBT_QUANT	0x01	/* Override quantizer scale for this macroblk */
#define	MBT_MFWD	0x02	/* Macroblock motion forward */
#define	MBT_MBAK	0x04	/* Macroblock motion backward */
#define	MBT_PATTERN	0x08	/* DCT coefficients to follow */
#define	MBT_INTRA	0x10	/* Intra-frame coding */
#define	MBT_AVERAGE	0x20	/* Average fwd & bak motion estimates */

#define N_BLOCK_CELLS		64	/* number of cells in a block */

#define	Y0		5	/* Block index of Y0 */
#define	Y1		4	/* Block index of Y1 */
#define	Y2		3	/* Block index of Y2 */
#define	Y3		2	/* Block index of Y3 */
#define	CB		1	/* Block index of CB */
#define	CR		0	/* Block index of CR */

#define CBP_Y0		(1<<Y0)	/* Upper left Y block included in macroblock */
#define	CBP_Y1		(1<<Y1)	/* Upper right Y block included in macroblock */
#define CBP_Y2		(1<<Y2)	/* Lower left Y block included in macroblock */
#define	CBP_Y3		(1<<Y3)	/* Lower right Y block included in macroblock */
#define	CBP_CB		(1<<CB)	/* B-Y block included in macroblock */
#define	CBP_CR		(1<<CR)	/* R-Y block included in macroblock */

/*
 * These macros enumerate the different states of the timecodes.
 * See ComputeTimeCode() for details.
 */
#define TC_NORMAL	1
#define	TC_GOP_START	2
#define	TC_TRANSITION	3

/*
 * Constants and defines that handle computing an exponential moving
 * average (EMA).  This is used to keep a history of picture decoding
 * times which in turn help decide whether it is too late to decode
 * a particular picture.
 */
#define	PictTimeAvg	2	/* i.e. new * 1/4 + ave * 3/4 */
#define	DisplayTimeAvg	3	/* i.e. new * 1/8 + ave * 7/8 */

#define ExpMovAverage(ave, new, wieghtBits)				\
    ((((ave << wieghtBits) - ave) + new) >> wieghtBits)

/*
 **********************************************************************
 *
 * The following routines are functions that are expanded as macros
 * for speed.
 *
 **********************************************************************
 */

/*
 * This implements the escape codes in table B5g.  bits and level
 * are input; 28 bits are assumed to be in bits.  level, run and bitlength
 * are outputs.
 */ 
#define DecodeDCTCoeffEscape(bits, level, run, bitlength, negative)	\
{									\
    run = KeepBits(6, bits>>16);					\
    level = KeepBits(8, bits>>8);					\
    if (level == Binary8(1000,0000, 8)) {				\
	negative = True;						\
	level = 256 - KeepBits(8, bits);				\
	bitLength = 28;							\
    } else if (level == 0) {						\
	negative = False;						\
	level = KeepBits(8, bits);					\
	bitLength = 28;							\
    } else {								\
	if (negative = (level & 0x80)) {				\
	    level |= (~0)<<8;      /* make it negative */		\
	    level = -level;	/* and then positive again */		\
	}								\
	bitLength = 20;							\
    }									\
}

/*
 * we know that the top seven bits are 0
 */
#define DecodeLongDCTCoeff(bits, level, run, bitlength, negative)	\
{									\
    ULong _hashValue;							\
    _hashValue = HuffLookup.dctCoeff2[ bits>>(21-DCT_COEFF2_LOOKUP_BITS) ];\
    level = DCTCoeffLevel(_hashValue);					\
    bitLength = DCTCoeffBits(_hashValue);				\
    negative = (bits >> (28-bitLength)) & 1;				\
    run = DCTCoeffRun(_hashValue);					\
}


/*
 * Level has already been assigned, but its sign might be wrong
 * There are 28 bits in bits.
 */
#define DecodeShortDCTCoeff(hashValue, bits, level, run, bitlength, negative) \
{									\
    bitLength = DCTCoeffBits(hashValue);				\
    run = DCTCoeffRun(hashValue);					\
    negative = (bits >> (28-bitLength)) & 1;				\
}

/*************************************************************************
 *
 * Per-Macroblock information
 *
 *************************************************************************/
typedef struct _mbinfo {
    long	dctCoeff[8*8];	/* the dct coefficients */
    long	dctRecon[8*8];	/* the reconstructed dct blocks */
    short	mbAddr;		/* index to current macro block */
    short	pastIntraMbAddr;/* previous intra macroblock */
    short	pastMbAddr;	/* previous macroblock (intra or delta) */
    short	firstMbInSlice;	/* first macroblock in current slice */
    short	dcYPast;	/* dctRecon[0] of previous intra-coded Y */
    short	dcCrPast;	/* dctRecon[0] of previous intra-coded Cr */
    short	dcCbPast;	/* dctRecon[0] of previous intra-coded Cb */
    short	pastHorizFwdMotion;
    short	pastVertFwdMotion;
    short	pastHorizBakMotion;
    short	pastVertBakMotion;
    UChar	zigZag[64];	/* copy of global ZigZag: for speed */
    UChar	type;		/* contains 5 bits about the macroblock */
    int		bits;		/* Encoded bit length */
    UInt	maxerr;		/* Maximum pixel error */
    UInt	iterations;	/* Number of interations */
    UChar	codedBlockPattern;/* per protocol */

    /*
     * cached values
     */
    UChar	*pPast;		/* past picture pointer, if defined */
    UChar	*pTo;		/* current picture pointer */
    UChar	*pFuture;	/* future picture pointer, if defined */
    VoidFuncPtr	motionAndClamp;
    ULong	cellsInBlock1;
    ULong	cellsInBlock2;
    UShort	mbAddrMap[ MAX_MB_PER_ROW*MAX_MB_PER_COL ];

    SequenceInfoPtr pSeq;	/* back pointer */

#ifdef DEBUG_ENABLED
    short	blockIndex;	/* Y0, Y1, Y2, Y3, CR or CB */
    BitByteOffset start;	/* starting point of this macroblock */
#endif /* DEBUG_ENABLED */
} MacroBlockInfo, *MacroBlockInfoPtr;

#define MB_Y0		(1 << Y0)	/* Y0 coded */
#define MB_Y1		(1 << Y1)	/* Y1 coded */
#define MB_Y2		(1 << Y2)	/* Y2 coded */
#define MB_Y3		(1 << Y3)	/* Y3 coded */
#define MB_CB		(1 << CB)	/* CB coded */
#define MB_CR		(1 << CR)	/* CR coded */
#define MB_PIXELS0	(1 << 9)	/* the final pixels */
#define MB_PIXELS1	(1 << 8)	/* the final pixels */
#define MB_PIXELS2	(1 << 7)	/* the final pixels */
#define MB_PIXELS3	(1 << 6)	/* the final pixels */
#define MB_TYPE		0xC0		/* Macroblock type */
#define MB_INTRA	0x00		/* Intra coded */
#define MB_FWDMC	0x40		/* Forward motion compensated */
#define MB_BAKMC	0x80		/* Backward motion compensated */
#define MB_FBMC		0xC0		/* Forward & backward motion comp */

typedef union {
    struct {
	ULong groupOfPictures :7;	/* used to provide uniqueness */
	ULong dropFrame:1;
	ULong hours    :5;
	ULong minutes  :6;
	ULong marker   :1;
	ULong seconds  :6;
	ULong pictures :6;
    } discrete;
    ULong composite;
} TimeCode;


/*************************************************************************
 *
 * Decoder's picture structure
 *
 *************************************************************************/
struct _picinfo {
    /* This must be first... shared with PixelInfo (see display.h) */
    UChar	type;		/* Picture coding type (B_?_PIC) */

    boolean	releaseAfterDisplay; /* return to queue after display if true */
    boolean	notDisplayed;	/* true if picture is still in the queue */
    boolean	skipped;	/* true if this picture gets skipped */
    boolean	onTime;		/* true if on time for display */
    boolean	reschedule;	/* true if allocated during loading state */
    TimeCode	tc;		/* timecode derived from gop tc and timeref */
    UShort	timeRef;	/* temporal reference */
    UShort	vbvDelay;	/* initial occupancy of the decoder's buffer */
    UChar	fullPelFwdVec;	/* if true, then vectors are integer offsets */
    UChar	fullPelBakVec;	/* if true, then vectors are integer offsets */
    UChar	forwardSize;	/* size in bits of forward motion numbers */
    UChar	backwardSize;	/* size in bits of backward motion numbers */

    ULong	displaySched;	/* display time relative to pSeq->seqClock */
    ULong	decodeTime;	/* accumulated wall time to decode this pic */
    ULong	estDecodeTime;	/* estimated decode time */
    ULong	picNumber;	/* the display-order picture number */

    /*
     * the pixels here are
     * used by I pictures and the picture contructed for P and B
     */
    UChar	*y;		/* Luma image */
    UChar	*cb;		/* cb image */

    SequenceInfoPtr pSeq;	/* back pointer */

#ifdef STATISTICS_ENABLED
    UShort	skippedMB;	/* number of skipped macro blocks */
    UShort	dctBlocksFwd;	/* number of dct blocks w/ fwd motion */
    UShort	dctBlocksBak;	/* number of dct blocks w/ bak motion */
    UShort	dctBlocksBoth;	/* number of dct blocks w/ fwd/bak motion */
    UShort	copyBlocksFwd;	/* number of blocks copied w/ fwd motion */
    UShort	copyBlocksBak;	/* number of blocks copied w/ bak motion */
    UShort	copyBlocksBoth;	/* number of blocks copied w/ fwd/bak motion */
    UShort	dctBlocks;	/* number of intra-coded blocks */
#endif /* STATISTICS_ENABLED */
#ifdef DEBUG_ENABLED
    BitByteOffset start;	/* starting point of this picture */
#endif /* DEBUG_ENABLED */
}; /* typedef: PictureInfo, *PictureInfoPtr */


/*************************************************************************
 *
 * Info assicuated with the sequence being played.
 *
 *************************************************************************/
typedef enum {
    DecodingSequence,
    DecodingPicture,
    MainLoopIdle
} DecoderState;

typedef struct _iPicOffset {
    long offset;
    long picNumber;
} IPicOffset, *IPicOffsetPtr;

#define IPicOffsetListInc	128

typedef struct _iPicOffsetList {
    IPicOffsetPtr list;		/* list of offsets */
    int nOffsets;		/* number of cells filled with offsets */
    int listLength;		/* number of cells */
} IPicOffsetList;

struct _sequence {
    short	horizontalSize;	/* horizontal width of picture */
    short	verticalSize;	/* vertical width of picture */
    short	mbWidth;	/* width in macroblocks */
    short	mbHeight;	/* height in macroblocks */
    short	lastMacroBlock;	/* the last possible macroblock address */
    short	bitRate;	/* flow rate in bits; 1 unit = 400 bits/sec */
    float	pelAspectRatio;	/* aspect ratio of a single pixel*/
    int		VBVBufSize;	/* size of "Video Buffering Verifier" buffer */
				/* measured in 16k bit chunks */
    UChar	pictureRateCode;/* protocol (or private) picture rate */
    UChar	qScale;		/* current quantizer scale (matches scaled*) */
    boolean	constrained;	/* true if picture size, rate, etc. are small */
    boolean	loadIQM;	/* true if we load intraQMatrix */
    boolean	loadNIQM;	/* true if we load nonIntraQMatrix */
    long	dctLimit;	/* if >0 defines # of coefficients in x & y */

    TimerInc	timerInc;	/* clock increment for each frame */
    ULong	seqClock;	/* time relative to start of seq (microsecs) */
    ULong	idle;		/* idle time while blocked on schedule */
    ULong	*active;	/* points to total time for current activity */
    TimeSpec	wallClock;	/* current system time */	
    short	mbToCheckTime;	/* check the time at this mb */
    short	mbCheckInc;	/* # of mb's to go before checking the time */

    ULong	decodeTime[5];	/* exponential moving avg. for each pic type */
    ULong	displayTime;	/* exponential moving avg. of display time */

    ULong	audioPicOverhead;	/* fixed au overhead per picture */
    ULong	audioChunkOverhead;	/* fixed au overhead per au time slice */

    ULong	picGopStart;
    ULong	refPictDisplaySched;/* this helps bridge Gops in scheduling */
    ULong	iPictDisplaySched;/* this helps bridge Gops in scheduling */
    long	refPicNumber;	/* this helps bridge Gops in scheduling */
    UShort	picsInLastGop;
    long	lastPicNumber;	/* picNumber of last I-frame */
    boolean	xingSequence;	/* I-frame only, broken temporal_reference */
    long	workToDo;	/* total scheduled work to do */

    boolean	showEveryFrame;	/* user specified: real-time or not */
    boolean	adaptSchedule;	/* if true, adapt the fps to the processor */
    boolean	inputExhausted;	/* true if nothing left => drain the queue */
    DecoderState state;

    PictureInfoPtr currentPict;
    PictureInfoPtr prevPict;
    PictureInfoPtr futurePict;
    PictureInfoPtr pastPict;

    IPicOffsetList	iPics;	/* list of I picture offsets, as discovered */

    /*
     * Group of pictures data kept here
     */
    long	nGopStarts;
    TimeCode	tc;		/* current time code */
    boolean	closedGop;	/* true if vectors only point in this group */
    boolean	brokenLink;	/* true if editing has broken picture groups */

    UChar	preloadMask;	/* mask of picture types to preload */

    boolean clientTerminated;	/* true of client has closed connection */

    /* scheduler for pictures */
    ULong (*schedule)(SequenceInfoPtr, ULong *codesComposite);

    /* processor for pictures */
    CacheReturnValue (*picture)(CacheProtoArgs, SequenceInfoPtr);

    /* allocator for pictures */
    PictureInfoPtr (*allocPicture)(SequenceInfoPtr, ULong, ULong);

    /* call when picture is ready for display */
    void	(*pictureReady)(PictureInfoPtr , boolean);

    StackInfo	decodeContext;	/* use this to jump out of the decode process */
    StackInfo	seqContext;	/* use this to jump out of a sequence */
    StackInfo	pictContext;	/* use this to jump out of a picture */
    StackInfo	licenseContext;	/* use this when the license has expired */

    /* force longword alignment here. */
    UChar	intraQMatrix[64];	/* intra quantizer matrix */
    UChar	nonIntraQMatrix[64];	/* non-intra quantizer matrix */
    UShort	*scaledIntraQMatrix;	/* intraQMatrix scaled by q factor */
    UShort	*scaledNonIntraQMatrix;	/* nonIntraQMatrix scaled by q */
    UShort	cachedIntraQMatrix[31][64];	/* all scaled intra matrices */
    UShort	cachedNonIntraQMatrix[31][64];	/* all scaled intra matrices */
    
    ParseInfoPtr pParse;		/* mcs parse info */
    BitStreamPtr mpegBuf;		/* bitstream context */
    DisplayInfoPtr dpyInfo;		/* display context */
    MacroBlockInfoPtr macroBlk;		/* current macroblock */

#ifdef notdef
    char	*extData;
    int		extDataLen 
    char	*userData;
    int		userDataLen;
#endif

#ifdef TIMING_ENABLED
    ULong	timers[1];
#endif

#ifdef notdef
    char	*grpExtData;
    int		grpExtDataLen;
    char	*grpUserData;
    int		grpUserDataLen;
#endif
}; /* typedef: SequenceInfo, *SequenceInfoPtr */

/**************************************************************************
 *
 * Where they occur, multiple fixed length codes should be read all
 * at once (as long as they fit in a 32 bit word).
 * These are unions/bitfields that define them
 *
 **************************************************************************/
typedef union _picCodes {
    struct {
	ULong pad    :11;
	ULong code   : 8;
	ULong timeRef:10;
	ULong type   : 3;
    } s;	/* s for "struct" */
    ULong composite;
} PicCodes;

extern float    FrameRateTable[16];
extern UChar    DefaultIntraQMatrix[];
extern UChar    DefaultNonIntraQMatrix[];
extern UChar    ZigZag[64];
extern float    PelAspectRatio[];
extern TimeSpec QuarterSec;

extern void MpegDecode(SequenceInfoPtr);
extern void ScaleQuantMatrix(SequenceInfoPtr, int);
extern void IDct(MacroBlockInfoPtr, UChar *, UChar *, UChar *, int);

extern CacheReturnValue NextStartCode(CacheProtoArgs, BitStreamPtr, int);
extern CacheReturnValue SequenceHeader(CacheProtoArgs, SequenceInfoPtr);
extern CacheReturnValue ProcPicture(CacheProtoArgs, SequenceInfoPtr);
extern CacheReturnValue PreloadPicture(CacheProtoArgs, SequenceInfoPtr);
extern CacheReturnValue MacroBlock(CacheProtoArgs, SequenceInfoPtr);
extern CacheReturnValue IntraBlock(CacheProtoArgs, int, SequenceInfoPtr);
extern CacheReturnValue Block(CacheProtoArgs, SequenceInfoPtr);

extern void InitStructures(SequenceInfoPtr);
extern void InitMBAddressLookup(SequenceInfoPtr);
extern void InitMBAddressInc(void);
extern void InitMBType(void);
extern void InitMotionVector(void);
extern void InitCodedBlock(void);
extern void InitSizeYC(void);
extern void InitGeneral(SequenceInfoPtr, int, char **);
extern void InitVerify(void);
extern void AnnounceCopyright(SequenceInfoPtr);
extern ImageReasonPtr ConfigureDecoder(SequenceInfoPtr);
extern ImageReasonPtr CBR_init(BitStream *mpegBuf, char *serverName);

extern void CopyYCBlock(UChar *, UChar *, UChar *, int);

#ifdef DEBUG_ENABLED

CExternal void PrintMbPix(MacroBlockInfo *pMb, int mbAddr, int mask);
CExternal void PrintMb(MacroBlockInfo *, int mask,
                       ULong cellsInBlock1, ULong cellsInBlock2);
CExternal void PrintPix(UChar *pixels);
CExternal void PrintQMatrix(UChar *matrix);

#endif /* DEBUG_ENABLED */

#endif /* _DECODER_H_ */
