/*
 *      (c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              display.c
 *
 *      DESCRIPTION
 *              handle display activity
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/display.c,v 4.59 1994/10/09 18:38:25 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"
#include "binarybits.h"
#include "process.h"

void
InitPictureBuffers(SequenceInfoPtr pSeq)
{
    PictureInfoPtr picList;
    UChar *pixels;
    int i, count;
    int picSize = sizeof(PictureInfo)
		+ MAX_X * MAX_LUMA_Y
		+ MAX_X * MAX_CHROMA_Y;

    count = pSeq->dpyInfo->queueSize;

    picList = (PictureInfoPtr) SecureAlloc(picSize * count);
    memset((char *)picList, 0, picSize * count);
    pixels = (UChar *)(picList + count);
    for (i = 0; i < count; i++) {
	picList[i].y = pixels;
	pixels += MAX_X * MAX_LUMA_Y;
	picList[i].cb = pixels;
	pixels += MAX_X * MAX_CHROMA_Y;
	picList[i].pSeq = pSeq;
    }

    PictBuffer.buffer = picList;
}

void
InitPictureQueues(SequenceInfoPtr pSeq)
{
    PictureInfo *ppic;
    DisplayInfoPtr dpyInfo = pSeq->dpyInfo;

    InitQueue(&PictBuffer.displayQueue, dpyInfo->queueSize);
    InitQueue(&PictBuffer.pictureQueue, dpyInfo->queueSize);

    ppic = PictBuffer.buffer;
    while (! QueueFull(&PictBuffer.pictureQueue)) {
	InsertQueueHead(&PictBuffer.pictureQueue, ppic++);
    }
}

#ifdef SHOW_QUEUES
static void
ShowQueue(Queue *q, PictureInfo *ppic, char *banner)
{
    long picIndex;
    char *qName;

    picIndex = ppic - PictBuffer.buffer;
    if (q == &PictBuffer.displayQueue)
	qName = "dq";
    else
	qName = "pq";

    Msg("%s%cpic(%d) in %s hd=%d,tl=%d,occ=%d\n",
	banner,
	IPBDchars[ ppic->type ],
	picIndex,
	qName,
	q->head - q->array,
	q->tail - q->array,
	QueueOccupancy(q));
}

#else /* SHOW_QUEUES */

#define ShowQueue(q, ppic, banner)

#endif /* SHOW_QUEUES */

/*
 * Find the picture designated by picNumber and return it.
 */
PictureInfoPtr
FindPicture(SequenceInfo *pSeq, int picNumber)
{
    QItemPtr item;
    Queue *q;
    PictureInfoPtr pic;

    q = &PictBuffer.displayQueue;
    if (! QueueEmpty(q) ) {
	item = q->tail;
	do {
	    pic = ((PictureInfoPtr)*item);
	    if (pic->picNumber == picNumber)
		return(pic);
	    if (item++ >= q->last)
		item = q->array;
	} while (item != q->head);
    }

    q = &PictBuffer.pictureQueue;
    if (! QueueEmpty(q) ) {
	item = q->tail;
	do {
	    pic = ((PictureInfoPtr)*item);
	    if (pic->picNumber == picNumber)
		return(pic);
	    if (item++ >= q->last)
		item = q->array;
	} while (item != q->head);
    }
    if (pSeq->currentPict->picNumber == picNumber)
	return(pSeq->currentPict);
    return (NULL);
}

/*
 * Used for the loading phase of the decoder
 */
PictureInfoPtr
NewPreloadPicture(SequenceInfo *pSeq, ULong codesComposite, ULong tcComposite)
{
    PictureInfo *p;
    Queue *pq = &PictBuffer.pictureQueue;
    Queue *dq = &PictBuffer.displayQueue;
    ULong spins = 0;
    DebugDeclare(ULong spinStart)
    PicCodes codes;
    DisplayInfo *dpyInfo = pSeq->dpyInfo;

    codes.composite = codesComposite;

    DebugTask(N_QueueInfo,
	Msg("nplp: %c %d", IPBDchars[ codes.s.type ],
	    pSeq->picGopStart + codes.s.timeRef);
    )

    pSeq->active = &pSeq->idle;
    spins = 0;
    while (QueueEmpty(pq) || dpyInfo->state != DisplayRunning) {
	CheckServices(dpyInfo);

	if (dpyInfo->state == DisplayLoading) {
	    if (QueueOccupancy(dq) > NLoadedPictures) {
		EventToClient(pSeq->pParse, MCS_EVENT_READY,
			      NULL, MCS_TYPE_NONE);
		
		p = TailOfQueue(dq, PictureInfo *);
		dpyInfo->savedPic = p;
		DebugTask(N_DisplayTimecode,
		    Msg("NewPreloadPicture last loaded: pic %d\n",
			pSeq->picGopStart + codes.s.timeRef);
		)		
		(void) NewState(dpyInfo, DisplayStopped);
	    } else {
		if (! QueueEmpty(pq))
		    break;
	    }
	}

	if (dpyInfo->refresh) {
	    if (dpyInfo->rerenderRefresh) {
		dpyInfo->rerenderRefresh = False;
		(*dpyInfo->saveRefreshFrame)(dpyInfo, NULL);
	    }
	    /*
	     * clear this after (->saveRefreshFrame) because it may
	     * cause it to be set again.
	     */
	    dpyInfo->refresh = False;
	    (*dpyInfo->refresher)(dpyInfo);
	}

	if (dpyInfo->state == DisplayRunning) {
	    DebugTask(N_DisplayTimecode,
		if (spins++ == 0)
		    spinStart = pSeq->seqClock;
	    )
	} else {
	    WaitForSomething(&QuarterSec);
	}
    }
    DebugTask(N_DisplayTimecode,
	if (spins > 1)
	    Msg("NewPreloadPic: %dms/%d spins waiting for alloc of pic %d\n",
		pSeq->seqClock - spinStart, spins,
		pSeq->picGopStart + codes.s.timeRef);
    )

    p = TailOfQueue(pq, PictureInfo *);
    DebugTask(N_QueueInfo,
	Msg(" (bf%d)\n", p - PictBuffer.buffer);
	ShowQueue(pq, p, "before rm: ");
    )

    RemoveQueueTail(pq);

    p->tc.composite = tcComposite;
    p->skipped = False;
    p->onTime = True;
    p->decodeTime = 0;
    p->reschedule = True;
    pSeq->active = &p->decodeTime;

    return (p);
}

/*ARGSUSED*/
PictureInfoPtr
NewPicture(SequenceInfo *pSeq, ULong codesComposite, ULong tcComposite)
{
    PictureInfo *p;
    Queue *pq = &PictBuffer.pictureQueue;
    ULong spins = 0;
    DebugDeclare(ULong spinStart)
    PicCodes codes;

    codes.composite = codesComposite;

    DebugTask(N_QueueInfo,
	Msg("np: %c %d", IPBDchars[ codes.s.type ],
	    pSeq->picGopStart + codes.s.timeRef);
    )

    while (QueueEmpty(pq)) {
	DisplayInfo *dpyInfo = pSeq->dpyInfo;

	CheckServices(dpyInfo);
	if (dpyInfo->state != DisplayRunning)
	    return( NewPreloadPicture(pSeq, codesComposite, tcComposite) );

	if (spins++ == 0) {
	    DebugTask(N_DisplayTimecode, spinStart = pSeq->seqClock );
	    pSeq->active = &pSeq->idle;
	}
    }

    DebugTask(N_DisplayTimecode,
	if (spins)
	    Msg("NewPic: clk=%d-%d: %d spins waiting for alloc of pic %d\n",
		spinStart, pSeq->seqClock, spins,
		pSeq->picGopStart + codes.s.timeRef);
    )

    p = TailOfQueue(pq, PictureInfo *);
    DebugTask(N_QueueInfo,
	Msg(" (bf%d)\n", p - PictBuffer.buffer);
	p->type = codes.s.type;
	ShowQueue(pq, p, "before rm: ");
    )

    RemoveQueueTail(pq);

    p->tc.composite = tcComposite;
    p->skipped = False;
    p->onTime = True;
    p->decodeTime = 0;
    p->reschedule = False;
    pSeq->active = &p->decodeTime;

    return (p);
}

/*ARGSUSED*/
void
ShowMb(PictureInfo *ppic, int mbAddr)
{
    /* XShowMb(ppic, mbAddr); */
}

void
PictureReady(PictureInfoPtr ppic, boolean releaseAfterDisplay)
{
    Queue *dq = &PictBuffer.displayQueue;
    DisplayInfo *dpyInfo = ppic->pSeq->dpyInfo;
    PictureInfoPtr p;

    DebugTask(N_QueueInfo,
	Msg("pr: %c %d/%02d:%02d:%02d (%d)\n",
	    IPBDchars[ ppic->type ],
	    ppic->tc.discrete.groupOfPictures,
	    ppic->tc.discrete.minutes,
	    ppic->tc.discrete.seconds,
	    ppic->tc.discrete.pictures,
	    ppic - PictBuffer.buffer);
	ShowQueue(dq, ppic, "before ins: ");
    )

    if (dpyInfo->state == DisplayLoading && QueueFull(dq)) {
	WhenDebugEnabled( dq->lowWaterMark = QueueOccupancy(dq) );
	(void) NewState(dpyInfo, DisplayStopped);
	p = TailOfQueue(dq, PictureInfoPtr);
	dpyInfo->savedPic = p;

	while (QueueFull(dq)) {
	    CheckServices(dpyInfo);
	    if (dpyInfo->state != DisplayRunning)
		WaitForSomething(&QuarterSec);
	}
    }

    GeneralAssert(! QueueFull(dq));
    InsertQueueHead(dq, ppic);
    ppic->notDisplayed = True;
    ppic->releaseAfterDisplay = releaseAfterDisplay;
}

void
ReleasePicture(PictureInfoPtr ppic)
{
    GeneralAssert(! ppic->releaseAfterDisplay);

    if (ppic->notDisplayed) {
	ppic->releaseAfterDisplay = True;
	return;
    }

    DebugTask(N_QueueInfo,
	ShowQueue(&PictBuffer.pictureQueue, ppic, "before ins: "))

    GeneralAssert (! QueueFull(&PictBuffer.pictureQueue));
    InsertQueueHead(&PictBuffer.pictureQueue, ppic);
}

void
SetFrameRate(SequenceInfoPtr pSeq)
{
    double divisor;

    divisor = FrameRateTable[ pSeq->pictureRateCode ];

    pSeq->showEveryFrame = False;
    if (IsInitialized(pSeq->dpyInfo, SYM_FRAME_RATE)) {
	if (pSeq->dpyInfo->frameRate)
	    divisor = pSeq->dpyInfo->frameRate;
	else
	    pSeq->showEveryFrame = True;
    }

    InitTimerInc(&pSeq->timerInc, divisor, MicroSecPerSec);

    /* Check the clock 4 times per picture */
    pSeq->mbCheckInc = pSeq->lastMacroBlock / 4;

    DebugTask(N_DisplayTimecode,
	ProgMsg("timer: inc=%d, denom=%d, remainder=%d\n",
	    pSeq->timerInc.increment,
	    pSeq->timerInc.denom,
	    pSeq->timerInc.remainder);
    )
}

void
DisplayFrame(DisplayInfo *dpyInfo)
{
    SequenceInfoPtr pSeq = dpyInfo->pSeq;
    PictureInfo *ppic;
    int currentFrame;
    Queue *dq = &PictBuffer.displayQueue;
    Queue *pq = &PictBuffer.pictureQueue;
    ParseValue value;

    dpyInfo->u.s.timeForDpy = False;
    if (dpyInfo->state != DisplayRunning)
	return;

    GeneralAssert( !QueueEmpty(dq) );

    ppic = TailOfQueue(dq, PictureInfo *);

    WhenDebugEnabled(
	if (QueueOccupancy(dq) < dq->lowWaterMark) {
	    dq->lowWaterMark = QueueOccupancy(dq);
	    DebugTask(N_QueueInfo, Msg("-"));
	}
    )

    if (ppic->onTime && dpyInfo->windowState != WindowObscured) {
	TimerTask(pSeq->timers, N_TimeDisplay, StartTimer());
	(*dpyInfo->converter)(dpyInfo, ppic);
	TimerTask(pSeq->timers, N_TimeDisplay,
	    Msg("dpy %c %d/%02d:%02d:%02d time: %s\n",
		IPBDchars[ ppic->type ],
		ppic->tc.discrete.groupOfPictures,
		ppic->tc.discrete.minutes,
		ppic->tc.discrete.seconds,
		ppic->tc.discrete.pictures,
		FormatTimeUsage(StopTimer(), True));
	)
	dpyInfo->savedPic = ppic;
	dpyInfo->framesDisplayed++;
    }
    DebugTask(N_DisplayTimecode,
	Msg("clk=%d: %s %c pic #%d, dec=%d, sched=%d\n",
	    pSeq->seqClock, ppic->onTime ? "dpy" : "skp",
	    IPBDchars[ ppic->type ],
	    ppic->picNumber, ppic->decodeTime, ppic->displaySched);
    )

    currentFrame = dpyInfo->currentFrame = ppic->picNumber;
    if (currentFrame >= dpyInfo->outFrame) {
	value.intvalue = dpyInfo->currentFrame;
        EventToClient(pSeq->pParse, MCS_EVENT_PAUSED, &value, MCS_TYPE_INT);
	(void) NewState(dpyInfo, DisplayStopped);
    } else {
	/*
	 * Note that we only remove the picture from the queue
	 * if we are running.  Otherwise, we leave it there
	 * so that it can be used to refresh the display.  It should
	 * be removed by SchedPicutes() before scheduling anything.
	 */
	DebugTask(N_QueueInfo, ShowQueue(dq, ppic, "before rm: "));
	RemoveQueueTail(dq);
	if (ppic->releaseAfterDisplay) {
	    DebugTask(N_QueueInfo, ShowQueue(pq, ppic, "before ins: "))
	    InsertQueueHead(pq, ppic);
	}
	if (currentFrame >= dpyInfo->issueFrameEvent) {
	    value.intvalue = dpyInfo->currentFrame;
	    EventToClient(pSeq->pParse, MCS_EVENT_FRAME, &value, MCS_TYPE_INT);
	    dpyInfo->issueFrameEvent += dpyInfo->frameFrequency;
	}
    }

    pSeq->workToDo -= ppic->estDecodeTime;
    ppic->notDisplayed = False;
}

void
CheckServices(DisplayInfoPtr dpyInfo)
{
    SequenceInfoPtr pSeq = dpyInfo->pSeq;
    ULong displayTime = 0;
    ULong audioOverhead;

    UpdateClock(dpyInfo->pSeq, NULL);
    if (! dpyInfo->u.checkServices)
	return;

    do {
	if (dpyInfo->u.s.timeForAudio) {
	    audioOverhead = dpyInfo->audioOverhead;

	    UpdateAudioPlayback(dpyInfo);
	    UpdateClock(dpyInfo->pSeq, &dpyInfo->audioOverhead);

	    DebugTask(N_AudioActivity, 
		      audioOverhead = dpyInfo->audioOverhead - audioOverhead;
		      Msg("au overhead: %d\n", audioOverhead);
            )
	}
	if (dpyInfo->u.s.timeForDpy) {
	    DisplayFrame(dpyInfo);
	    UpdateClock(dpyInfo->pSeq, &displayTime);
	}
	if (dpyInfo->u.s.readXServer) {
	    HandleWinInput(dpyInfo);
	    UpdateClock(dpyInfo->pSeq, &dpyInfo->winOverhead);
	}
	if (dpyInfo->u.s.readClient) {
	    dpyInfo->u.s.readClient = False; /* HandleConn... may set again */
	    HandleConnectionInput(dpyInfo->pSeq->pParse);
	    UpdateClock(dpyInfo->pSeq, &dpyInfo->clientOverhead);
	    if (dpyInfo->u.s.restartDecoder) {
		dpyInfo->u.s.restartDecoder = False;
		RestoreContext(dpyInfo->pSeq->decodeContext, 1);
	    }
	}
	if (dpyInfo->u.s.writeClient) {
	    _McsFlushMessages(dpyInfo->pSeq->pParse);
	    UpdateClock(dpyInfo->pSeq, &dpyInfo->clientOverhead);
	    dpyInfo->u.s.writeClient = False;
	}
	if (dpyInfo->u.s.reconfigure) {
	    ConfigureDisplayForSequence(dpyInfo);
	    UpdateClock(dpyInfo->pSeq, &pSeq->idle);
	}
    } while (dpyInfo->u.checkServices);

    if (displayTime) {
	pSeq->displayTime = ExpMovAverage(pSeq->displayTime,
					  displayTime,
					  DisplayTimeAvg);
    }
}

void
UpdateClock(SequenceInfoPtr pSeq, ULong *active)
{
    PictureInfoPtr pic;
    ULong seconds, microSeconds;
    long changeInSeconds, changeInMicroSecs;
    Queue *dq;
    DisplayInfoPtr dpyInfo;

    YieldProcessor();
    
    seconds      = pSeq->wallClock.seconds;
    microSeconds = pSeq->wallClock.microSeconds;
    TimeNow(&pSeq->wallClock);

    changeInSeconds   = (long)(pSeq->wallClock.seconds      - seconds);
    changeInMicroSecs = (long)(pSeq->wallClock.microSeconds - microSeconds);

    /*
     * Here we assume that the change in time is always small, hence, a loop
     * instead of a multiply.
     */
    if (changeInMicroSecs < 0) {
	changeInMicroSecs += MicroSecPerSec;
	changeInSeconds--;
    }
    while (--changeInSeconds >= 0) {
	changeInMicroSecs += MicroSecPerSec;
    }
    pSeq->seqClock += changeInMicroSecs;
    if (active)
	*active += changeInMicroSecs;
    else
	*pSeq->active += changeInMicroSecs;

    /*
     * Look at the display queue and see if a picture needs to be displayed
     * Look at the audio time and see if it is time to do some audio
     */
    dpyInfo = pSeq->dpyInfo;
    if (dpyInfo->state != DisplayRunning)
	return;

    if (dpyInfo->nextAudioTime <= pSeq->seqClock)
	pSeq->dpyInfo->u.s.timeForAudio = True;

    if (pSeq->dpyInfo->u.s.timeForDpy)
	return;
    dq = &PictBuffer.displayQueue;
    if (QueueEmpty(dq))
	return;
    pic = TailOfQueue(dq, PictureInfoPtr);
    if (pic->displaySched <= pSeq->seqClock) {
	if (! pSeq->adaptSchedule
	 && pSeq->seqClock > pic->displaySched + pSeq->timerInc.increment
	 && pic->picNumber != pSeq->dpyInfo->outFrame) {
	    pic->onTime = False;	/* more than a frame late */
	}
	pSeq->dpyInfo->u.s.timeForDpy = True;
    }
}

/*
 * Scheduler for the loading state
 */
static ULong
InitSchedPicture(SequenceInfoPtr pSeq, ULong *codesComposite)
{
    PicCodes codes;

    codes.composite = *codesComposite;

    /*
     * If we previously skipped to the next gop, we will no longer
     * have the past picture with which to decode the first B
     * pictures in the new gop.  skip them.
     */
    if (codes.s.type == B_B_PIC
     && (pSeq->pastPict == NULL || pSeq->pastPict->skipped)
     && ! pSeq->closedGop) {
	DebugTask(N_DisplayTimecode,
	    Msg("can't sched %c pic #%d\n",
		IPBDchars[ codes.s.type ], pSeq->picGopStart + codes.s.timeRef);
	)

	return( 0 );
    }

    if ( pSeq->lastPicNumber < 0 ) {
#if 0
	if( codes.s.timeRef != 0 ) {	/* XING sequence */
	    codes.s.timeRef = 0;
	    ((PicCodes *)codesComposite)->s.timeRef = 0;
	    pSeq->xingSequence = True;
	}
#endif
	pSeq->lastPicNumber = 0;
    }
    if ( pSeq->xingSequence && codes.s.type != B_I_PIC ) {
	ProgMsg("incorrectly set xingSequence flag\n");
    }

    DebugTask(N_DisplayTimecode,
	Msg("InitSched: %c pic #%d\n",
	    IPBDchars[ codes.s.type ], pSeq->picGopStart + codes.s.timeRef);
    )
    return(1);	/* a bogus, non-zero schedule */
}

/*
 * Adaptive scheduler for play-every-frame mode
 */
static ULong
AdaptSchedPicture(SequenceInfoPtr pSeq, ULong *codesComposite)
{
    PicCodes codes;
    long timeInc;
    long refPicNumber, late;
    ULong displaySched;
    int picNumber;

    codes.composite = *codesComposite;

    UpdateClock(pSeq, NULL);

    picNumber = pSeq->picGopStart + codes.s.timeRef;
    timeInc = pSeq->timerInc.increment;
    refPicNumber = pSeq->refPicNumber;
    displaySched = pSeq->refPictDisplaySched;

    /*
     * We either have to go up or down from the reference picture number
     */
    if (refPicNumber > picNumber) {
	while (refPicNumber > picNumber) {
	    if (timeInc <= displaySched)	/* don't go below zero */
		displaySched -= timeInc;
	    refPicNumber--;
	}
    } else {
	while (refPicNumber < picNumber) {
	    displaySched += timeInc;
	    refPicNumber++;
	}
    }

    /*
     * Adjust the time increment so that it adapts to the schedule.
     */
    late = (int)(pSeq->seqClock - displaySched)
	 + pSeq->decodeTime[ codes.s.type ]
	 + pSeq->displayTime
	 + pSeq->workToDo;

    if (late > 0)
	displaySched += late;
    if (codes.s.type <= B_P_PIC) {
	/* codes.s.type == B_I_PIC || codes.s.type == B_P_PIC */
	pSeq->refPictDisplaySched = displaySched;
	pSeq->refPicNumber = picNumber;
    }
    pSeq->timerInc.increment =
	ExpMovAverage(pSeq->timerInc.increment,
		      pSeq->decodeTime[ codes.s.type ],
		      PictTimeAvg);
    DebugTask(N_DisplayTimecode,
	Msg("timeinc %d ==> %d\n", timeInc, pSeq->timerInc.increment);
	)

    DebugTask(N_DisplayTimecode,
	Msg("clk=%d, sched %c pic #%d @%d, work=(%d,%d), %s by %d\n",
	    pSeq->seqClock, IPBDchars[ codes.s.type ], picNumber, displaySched,
	    pSeq->decodeTime[ codes.s.type ], pSeq->workToDo,
	    late > 0 ? "late" : "early",
	    late > 0 ? late: -late);
    )
    return( displaySched );
}

/*
 * Figure out the proper time to schedule a new picture.
 */
static ULong
SchedPicture(SequenceInfoPtr pSeq, ULong *codesComposite)
{
    PicCodes codes;
    long timeInc;
    long refPicNumber, late;
    ULong displaySched;
    boolean cancel;
    int picNumber;
    ULong audioOverhead, audioPicOverhead;

    codes.composite = *codesComposite;

    /*
     * If we previously skipped to the next gop, we will no longer
     * have the past picture with which to decode the first B
     * pictures in the new gop.  skip them.
     */
    if (codes.s.type == B_B_PIC
     && (pSeq->pastPict == NULL || pSeq->pastPict->skipped)) {
	DebugTask(N_DisplayTimecode,
	    Msg("can't sched %c pic #%d\n",
		IPBDchars[ codes.s.type ], pSeq->picGopStart + codes.s.timeRef);
	)

	return( 0 );
    }

    UpdateClock(pSeq, NULL);

    picNumber = pSeq->picGopStart + codes.s.timeRef;
    refPicNumber = pSeq->refPicNumber;
    displaySched = pSeq->refPictDisplaySched;

    /*
     * We either have to go up or down from the reference picture number
     */
    if (refPicNumber > picNumber) {
	timeInc = pSeq->timerInc.increment;
	while (refPicNumber > picNumber) {
	    displaySched -= timeInc;
	    refPicNumber--;
	}
    } else {
	audioOverhead = 0;
	audioPicOverhead = pSeq->audioPicOverhead;
	while (refPicNumber < picNumber) {
	    displaySched += TimerIncrement(&pSeq->timerInc);
	    refPicNumber++;
	    audioOverhead += audioPicOverhead;
	}
	pSeq->workToDo += audioOverhead;
	DebugTask(N_AudioActivity, Msg("pic %d +%d\n",
				       refPicNumber,
				       audioOverhead));
    }

    /*
     * if there is not enough time left to decode this before the
     * display time hits, we may cancel it
     */
    late = (int)(pSeq->seqClock - displaySched)
	 + pSeq->decodeTime[ codes.s.type ]
	 + pSeq->displayTime
	 + pSeq->workToDo;

    cancel = False;
    while (late > 0) {
	ULong timeBetweenAnchors;
	/*
	 * Don't drop the last frame (or anchor frames that follow it)
	 */
	if (picNumber >= pSeq->dpyInfo->outFrame)
	    break;

	/*
	 * Don't fret about B frames... nuke 'em.
	 */
	if (codes.s.type == B_B_PIC) {
	    cancel = True;	/* cancel it */
	    break;
	}

	/*
	 * Don't drop P and I frames if they can be decoded, but not displayed,
	 * in time
	 */
	if (codes.s.type == B_I_PIC)
	    timeBetweenAnchors = displaySched - pSeq->iPictDisplaySched;
	else
	    timeBetweenAnchors = displaySched - pSeq->refPictDisplaySched;
	/*
	 * Note: timeBetweenAnchors/3 although it reduced the decode frame rate
	 *	made the display frame rate about equal whereas before there was
	 *	a *large* difference (esp. w/ mpeg audio). This should be more
	 *	rigorously investigated
	 */
	if (late > timeBetweenAnchors/*/3*/
	 || pSeq->decodeTime[ codes.s.type ] > timeBetweenAnchors) {
	    DebugTask(N_DisplayTimecode,
		Msg("! %d>%d... ",
		    pSeq->decodeTime[ codes.s.type ], timeBetweenAnchors);
	    )
	    cancel = True;	/* cancel it */
	}
	break;
    }

    if (codes.s.type <= B_P_PIC) {
	/* codes.s.type == B_I_PIC || codes.s.type == B_P_PIC */
	pSeq->refPictDisplaySched = displaySched;
	if (codes.s.type == B_I_PIC)
	    pSeq->iPictDisplaySched = displaySched;
	pSeq->refPicNumber = picNumber;
    }

    DebugTask(N_DisplayTimecode,
	Msg("clk=%d, %s %c pic #%d @%d, work=(%d,%d), %s by %d\n",
	    pSeq->seqClock,
	    cancel ? "drop" : "sched",
	    IPBDchars[ codes.s.type ], picNumber, displaySched,
	    pSeq->decodeTime[ codes.s.type ], pSeq->workToDo,
	    late > 0 ? "late" : "early",
	    late > 0 ? late: -late);
    )
    return( cancel ? 0 : displaySched );
}

/*
 * Schedule all the pictures in the queue in anticipation of starting
 * the playback.
 */
static void
SchedPictures(SequenceInfoPtr pSeq)
{
    PictureInfoPtr pic;
    QItemPtr item;
    Queue *dq = &PictBuffer.displayQueue;
    long displaySched;
    int picNumber;

    if ( QueueEmpty(dq) )
	return;
    
    /*
     * if the first picture in the queue has been displayed, then
     * it was left there purely for refresh purposes.  Remove it
     * now.
     */
    pic = TailOfQueue(dq, PictureInfo *);
    if (! pic->notDisplayed) {
	DebugTask(N_QueueInfo, ShowQueue(dq, pic, "before rm: "));
	RemoveQueueTail(dq);
	if (pic->releaseAfterDisplay) {
	    Queue *pq = &PictBuffer.pictureQueue;
	    DebugTask(N_QueueInfo, ShowQueue(pq, pic, "before ins: "))
	    InsertQueueHead(pq, pic);
	}
    }

    if ( QueueEmpty(dq) )
	return;
    
    if( pSeq->xingSequence && pic->type != B_I_PIC ) {
	ProgMsg("incorrectly set xingSequence flag\n");
    }
    /*
     * When scheduling, the SchedulerStartDelay is the earliest anything
     * starts.  The audio scheduler does the same sort of thing.
     */
    displaySched = SchedulerStartDelay;
    if (pSeq->dpyInfo->audioDelay < 0)
	displaySched += pSeq->dpyInfo->absAudioDelay;
    item = dq->tail;
    picNumber = ((PictureInfoPtr)*item)->picNumber;
    do {
	pic = ((PictureInfoPtr)*item);
	while (picNumber < pic->picNumber) {
	    displaySched += TimerIncrement(&pSeq->timerInc);
	    picNumber++;
	}
	pic->displaySched = displaySched;
	pic->estDecodeTime = 0;

	DebugTask(N_DisplayTimecode,
	    Msg("sched %c pic #%d @ %d\n",
		IPBDchars[ pic->type ], pic->picNumber, displaySched);
	)
	if (pic->type == B_I_PIC || pic->type == B_P_PIC) {
	    pSeq->refPictDisplaySched = displaySched;
	    if (pic->type == B_I_PIC)
		pSeq->iPictDisplaySched = displaySched;
	    pSeq->refPicNumber = pic->picNumber;
	}

	if (item++ >= dq->last)
	    item = dq->array;
    } while (item != dq->head);

    /*
     * There may be a future picture that has not yet been put on the
     * display queue
     */
    if ((pic = pSeq->futurePict)) {
	while (picNumber < pic->picNumber) {
	    displaySched += TimerIncrement(&pSeq->timerInc);
	    picNumber++;
	}
	pic->displaySched = displaySched;
	pic->estDecodeTime = 0;

	if (pic->type == B_I_PIC || pic->type == B_P_PIC) {
	    pSeq->refPictDisplaySched = displaySched;
	    pSeq->refPicNumber = pic->picNumber;
	}
	DebugTask(N_DisplayTimecode,
	    Msg("sched %c pic #%d @ %d\n",
		IPBDchars[ pic->type ], pic->picNumber, displaySched);
	)
    }

}


void
CleanClock(DisplayInfoPtr dpyInfo)
{
    SequenceInfoPtr pSeq = dpyInfo->pSeq;

    dpyInfo->framesDecoded = 0;
    dpyInfo->framesDisplayed = 0;
    pSeq->idle = 0;
    dpyInfo->winOverhead = 0;
    dpyInfo->clientOverhead = 0;
    dpyInfo->audioOverhead = 0;
    pSeq->active = &pSeq->idle;
    pSeq->seqClock = 0;	/* reset the clock to 0 */
    UpdateClock(pSeq, NULL);
    pSeq->seqClock = 0;	/* reset the clock to 0 */
    pSeq->workToDo = 0;
    dpyInfo->u.s.timeForAudio = False;
    dpyInfo->u.s.timeForDpy = False;

}

DisplayState
NewState(DisplayInfoPtr dpyInfo, DisplayState state)
{
    DisplayState oldState;
    SequenceInfoPtr pSeq = dpyInfo->pSeq;

    oldState = dpyInfo->state;
    dpyInfo->state = state;

    switch (state) {
    case DisplayRunning:
	if (oldState != DisplayRunning) {
	    EventToClient(pSeq->pParse, MCS_EVENT_RUNNING, NULL, MCS_TYPE_NONE);
	    SchedPictures(pSeq);
	    SchedAudioPlayback(dpyInfo);
	    pSeq->schedule = SchedPicture;
	    pSeq->allocPicture = NewPicture;
	    if (pSeq->adaptSchedule) {
		/* pSeq->schedule = AdaptSchedPicture; */
		pSeq->schedule = InitSchedPicture;
	    }
	    if (dpyInfo->outFrame <= dpyInfo->currentFrame + PictureQueueSize)
		pSeq->schedule = AdaptSchedPicture;
	    CleanClock(dpyInfo);
	}
	break;

    case DisplayLoading:
	PreloadAudio(dpyInfo);
	SetParserState(pSeq->pParse, True);
	pSeq->schedule = InitSchedPicture;
	pSeq->allocPicture = NewPreloadPicture;
	break;

    case DisplayStopped:
	StopAudioPlayback(dpyInfo);
	SetParserState(pSeq->pParse, False);
	pSeq->schedule = InitSchedPicture;
	pSeq->allocPicture = NewPreloadPicture;

	if (oldState == DisplayRunning) {
	    ParseValue value;
	    double seconds, frameRate;
	    double dpyRate;

	    UpdateClock(pSeq, NULL);
	    seconds = pSeq->seqClock / 1000000.0;
	    frameRate =  dpyInfo->framesDecoded/seconds;
	    dpyRate = dpyInfo->framesDisplayed/seconds;

	    value.real = frameRate;
	    EventToClient(pSeq->pParse,
			  MCS_EVENT_FRAMERATE,
			  &value,
			  MCS_TYPE_REAL);
	    value.real = dpyRate;
	    EventToClient(pSeq->pParse,
			  MCS_EVENT_FRAMERATE,
			  &value,
			  MCS_TYPE_REAL);
	    DebugTask(N_PerformanceSummary,

		UpdateClock(pSeq, NULL);
		seconds = pSeq->seqClock / 1000000.0;
		Msg("stop @ %d, frm#%d: avg I,P,B,dpy=%d,%d,%d,%d;\n",
		    pSeq->seqClock,
		    dpyInfo->currentFrame,
		    pSeq->decodeTime[ B_I_PIC ],
		    pSeq->decodeTime[ B_P_PIC ],
		    pSeq->decodeTime[ B_B_PIC ],
		    pSeq->displayTime);
	        Msg("workToDo=%d\n", dpyInfo->pSeq->workToDo);
		Msg("idle=%d,win=%d,client=%d,audio=%d\n",
		    pSeq->idle, dpyInfo->winOverhead,
		    dpyInfo->clientOverhead, dpyInfo->audioOverhead);
		Msg("%d frames decoded over %.2f sec: %.2ffps\n",
		    dpyInfo->framesDecoded, seconds, frameRate);
		Msg("%d frames displayed over %.2f sec: %.2ffps\n",
		    dpyInfo->framesDisplayed, seconds, dpyRate);
	    )
	}

	DebugTask(N_SeekActivity,
	    int i;

	    Msg("I pic offsets:\n    ");
	    for (i = 0; i < pSeq->iPics.nOffsets; i++) {
		Msg("#%d@%5d  ", 
		    pSeq->iPics.list[ i ].picNumber,
		    pSeq->iPics.list[ i ].offset);
		if ((i & 0x3) == 0x3)
		    Msg("\n    ");
	    }
	    Msg("\n");
	)
	break;
    }

    return (oldState);
}
