/*
 *      (c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              display.h
 *
 *      DESCRIPTION
 *              <overall description of what the package does>
 *
 *      REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/display.h,v 4.42 1994/07/15 01:11:27 russ Exp $
 */
#include "nvrTypes.h"
#include "nvrQueue.h"
#include "rgblookup.h"
#include "nvrBitTest.h"
#include "audiodecode.h"

#define MAX_PIX_X		MAX_X
#define MAX_PIX_Y		MAX_LUMA_Y

/*
 * Defines for picture queueing
 */
#ifdef sgi
#define PictureQueueSize	  32
#endif

#ifdef sun
#define PictureQueueSize	  8
#endif

#ifdef THINK_C
#define PictureQueueSize	  6
#endif

#define NLoadedPictures		  3

/*
 * Delay from the time someone says start and when we actually hit
 * the ground running.
 */
#define SchedulerStartDelay	(MicroSecPerSec >> 5)

typedef enum {
    DisplayLoading,
    DisplayRunning,
    DisplayStopped
} DisplayState;

#define TICKS_PER_HEART_BEAT	(MicroSecPerSec*60)	/* for license mgr */

#define AllInitialized(dpyInfo, symbolBits)				\
	BM_ALLSET((BitMask *)(dpyInfo)->initialized,(symbolBits))
#define IsInitialized(dpyInfo, symbol)					\
	BM_ISSET((BitMask *)(dpyInfo)->initialized,(symbol))
#define MarkInitialized(dpyInfo, symbol)				\
	BM_SET((BitMask *)(dpyInfo)->initialized,(symbol));
#define UnmarkInitialized(dpyInfo, symbol)				\
	BM_UNSET((BitMask *)(dpyInfo)->initialized,(symbol));

typedef struct _copyInfo CopyInfo, *CopyInfoPtr;

typedef void (*ConvertController)(DisplayInfoPtr, PictureInfoPtr);
typedef void (*RefreshController)(DisplayInfoPtr);

struct _copyInfo {
    RefreshController winDisplay;	/* how to display DestWindow */
    RefreshController fbDisplay;	/* how to display DestWinFrameBuffer */
    ConvertController converter;	/* how to save the convert a frame */
    ConvertController saveFbFrame;	/* how to save the refresh picture */
    ConvertController saveWinFrame;	/* how to save the refresh picture */
    boolean clipToScreen;		/* true if we clip to screen */
    ULong maxDirectConvertPix;		/* area thresh. for 2-stage convert */
    UChar *fb;
    int lineBytes;
    int w, h;
}; /* typedef: CopyInfo, *CopyInfoPtr */

typedef enum {
    WindowObscured = 0,
    WindowPartiallyObscured = 1,
    WindowUnobscured = 2
} WindowState;

typedef enum {
    DPY_SrcRect     = 0x0001,
    DPY_DestRect    = 0x0002,
    DPY_Clip        = 0x0004,
    DPY_Destination = 0x0008,
    DPY_Color       = 0x0010
} DpyInfoFlags;

typedef enum {
    DestWindow,			/* use available window system */
    DestWinFrameBuffer,		/* use window, but bypass window system */
    DestYcFile,			/* output to a file */
    DestYcFrameBuffer		/* use video frame buffer */
} DpyDestType;

struct _dpyInfo {
    /*
     * Protocol accessible values
     */
    struct { /* changable geometry parameters before they take effect. */
	GeomRectangle src;
	GeomRectangle dst;
	GeomRectangle clip;
	RgbDepthType color;
	DpyDestType destType;
	ULong changed;
	boolean audioEnabled;	/* specified by the protocol */
    } new;

    /*
     * The changable parameters after the change takes effect.
     */
    GeomRectangle src;		/* current source geometry */
    GeomRectangle dst;		/* current display destination geometry */
    GeomRectangle clip;		/* current window offset and clip (raw mode) */
    RgbDepthType color;
    DpyDestType destType;

    char *windowSysName;	/* name for the window system */
    char *videoInput;		/* specified input path for mpeg video */
    char *videoOutput;		/* specified output path for decoded video */
    char *bitServer;		/* name of the BitStream server host */
    char *audioInput;		/* specified input path for audio */
    ULong windowId;		/* window for window system */
    TimeSpec inTime;		/* time to start*/
    TimeSpec outTime;		/* time when we are done */
    long tolerance;		/* tolerance of frame lateness (microsecs) */
    long preloadSize;		/* number of frames to read ahead */
    long audioDelay;		/* delay until start of audio */
    long frameFrequency;	/* frequency of frame events */
    boolean audioEnabled;	/* whether we are capturing/playing audio */
    boolean videoEnabled;	/* whether we are capturing/playing video */
    boolean reopenVideo;	/* if true, then reopen video at next config */
    boolean reopenAudio;	/* if true, then reopen video at next config */
    boolean audioOpened;	/* an audio file is open */
    long inFrame;		/* specified starting frame */
    long outFrame;		/* specified frame which we *don't* play */
				/* (compare this with ->lastFrameIndex */
    short frameRate;		/* ... in (integer) frames per second */
    short audioQuality;		/* 1 .. 32 */

    ULong initialized[2];	/* bitmask of initialized variables */

    /*
     * Other calculated, internal values
     */
    PrivPointer aInFp;		/* io pointer for audioInput */
    PrivPointer vOutFp;		/* io pointer for videoOutput */
    AudioInfo audioData;	/* for audio */
    int copyPixelStart;		/* frame buffer offset for CopyPixels() */
    ULong clientOverhead;	/* time spent in client io/parsing */
    ULong winOverhead;		/* time spent servicing the window system */
    ULong audioOverhead;	/* time spent servicing audio */
    ULong nextAudioTime;	/* time for next audio deliver to the device */
    boolean continuousMedia;	/* true if sequence is unbounded (broadcast) */
    boolean configured;		/* true if configuration is complete */
    boolean oneChannelOnly;	/* true if only doing luma */
    boolean useOverlay;		/* true if we use the overlay for 1-bit B&W */
    boolean doubleSize;		/* true => double picture size on output */
    boolean rerenderRefresh;	/* true => rerender the refresh frame */
    boolean refresh;		/* true => refresh the screen */
    boolean visualDisplay;	/* true if output to a screen (vs file) */
    boolean render444as888;	/* true hardware dithers 888 to 444 */
    boolean hardwareZoom;	/* true hardware zooms to 2x, etc. */
    boolean oneStageConvert;	/* false ==> convert offscreen, then display */
    int absFrameRate;		/* abs(frameRate) */
    long absAudioDelay;		/* abs(audioDelay) */
    int issueFrameEvent;	/* when to issue next event */
    int currentFrame;		/* current frame # being displayed */
    PictureInfoPtr savedPic;	/* pointer to last "saved picture" */
    int queueSize;		/* display queue size */
    WindowState windowState;	/* tracks the state of the window */
    DisplayState state;		/* current state (see above) */

    /*
     * debug variables
     */
    int framesDecoded;		/* count of frames decoded during last play */
    int framesDisplayed;	/* count of frames displayed during last play */

    /*
     * Internal variables
     */
    union _cvtUnion {
	YcToRgbInfoPtr ycToRgb;
	YcToGrayInfoPtr ycToGray;
    } cvtu, cvtuTmp;	/* for doing yc -> rgb conversions */
    RefreshController refresher;/* how to refresh */
    ConvertController converter;/* how to convert a frame on the fly */
    ConvertController saveRefreshFrame;	/* how to save the refresh picture */
    CopyInfo ycFileCopier;
    CopyInfo ycFbCopier;
    CopyInfo copiers[ NRgbDepthTypes ];
    CopyInfo *copier;		/* pointer to selected copier */

    /*
     * Offscreen copy buffer info
     */
    short pixelsPer32Bits;	/* used when copying pixels */
    short pixelSizeInBits;	/* used when copying pixels */
    ULong clippedPictureArea;	/* bytes per picture, after clipping */
    int bufferSize;		/* allocation size for each copyBuffer */
    int bufferHeight;		/* height of allocated buffer */
    UChar *copyBuffer;		/* offscreen buffer for conversion */
    long copyBufferWidth;	/* scanline width of offscreen buffer */

    /*
     * A single, quickly checked status word that says whether we
     * have something to do.
     */
    union {
	struct {
	    ULong reconfigure:8;/* true => reconfigure at next display time */
	    ULong timeForDpy :8;/* true => time to display a frame */
	    ULong timeForAudio :8;/* true => time to deliver audio */
	    ULong readXServer:2;/* true if events are waiting from X server */
	    ULong restartDecoder :2;/* true => time to longjump(decodeContext)*/
	    ULong readClient :2;/* true if input waiting from client */
	    ULong writeClient:2;/* true if output queued to client */
	} s;
	ULong checkServices;
    } u;

    char *deviceName;

    SequenceInfoPtr pSeq;	/* back pointer */
}; /* typedef: DisplayInfo, *DisplayInfoPtr */

typedef struct _pictBufferInfo {
    PictureInfoPtr buffer;
    long        nPictures;
    Queue       pictureQueue;
    Queue       displayQueue;
    Queue       releaseQueue;
} PictBufferInfo;

/*
 * A pre-processed picture
 */
typedef struct _pixinfo {
    /* This must be first... shared with PictureInfo */
    UChar	type;		/* Picture coding type (B_?_PIC) */
    double	pad;		/* force octword alignment */
    UChar	*pixels;	/* one frame of pixels */
} PixelInfo;

extern PictBufferInfo PictBuffer;

CExternal void PreloadPictureReady(
    PictureInfoPtr,
    boolean
);
CExternal void PictureReady(
    PictureInfoPtr,
    boolean
);
CExternal DisplayState NewState(
    DisplayInfoPtr,
    DisplayState
);
CExternal void CleanClock(
    DisplayInfoPtr
);
extern void InitPictureQueues(SequenceInfoPtr);
extern void DisplayFrame(DisplayInfoPtr);
extern void SunFbInit(DisplayInfoPtr);
extern void SgiFbInit(DisplayInfoPtr);
extern void FreeCopyBuffer(UChar *);
extern void SetFrameRate(SequenceInfoPtr);
extern void ClearPreloadQueue(DisplayInfoPtr);
extern void CheckServices(DisplayInfoPtr);
extern void UpdateClock(SequenceInfoPtr, ULong *active);

extern UChar *AllocateCopyBuffer(long size, int width, int height);

extern PictureInfoPtr NewPreloadPicture(SequenceInfoPtr, ULong, ULong);
extern PictureInfoPtr NewPicture(SequenceInfoPtr, ULong, ULong);
extern PictureInfoPtr FindPicture(SequenceInfoPtr, int);
extern void ReleasePicture(PictureInfoPtr);

extern void InitDisplay(DisplayInfoPtr);
extern void InitYcFb(DisplayInfoPtr);
extern void InitPictureQueues(SequenceInfoPtr);
extern void InitPictureBuffers(SequenceInfoPtr);

/*
 * display-related routines
 */

extern ImageReasonPtr ConfigureDisplayForSequence(DisplayInfoPtr);
extern ImageReasonPtr WinInit(DisplayInfoPtr);
extern boolean InitPictureQueue(SequenceInfoPtr);
extern void HandleWinInput(DisplayInfoPtr);
extern void HandleClientInput(DisplayInfoPtr);
extern ImageReasonPtr OpenAudio(DisplayInfoPtr);
extern void CloseAudio(DisplayInfoPtr);
extern void StopAudioPlayback(DisplayInfoPtr);
extern void SchedAudioPlayback(DisplayInfoPtr);
extern void UpdateAudioPlayback(DisplayInfoPtr);
extern void PreloadAudio(DisplayInfoPtr);
extern void SetWinClip(DisplayInfoPtr);
extern void SetWinBuffer(DisplayInfoPtr);
extern void CopyPixelsToFb(DisplayInfoPtr);
extern void SaveRgbRefreshFrame(DisplayInfoPtr, PictureInfoPtr);

