/*
 *      (c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              display.c
 *
 *      DESCRIPTION
 *              handle display activity
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/dpyconfig.c,v 1.30 1995/04/12 23:04:43 russ Exp $
 */

#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"
#include "binarybits.h"

static void
CopyYCToFile(DisplayInfoPtr dpyInfo, PictureInfo *ppic)
{
    SequenceInfoPtr pSeq = ppic->pSeq;
    FileHandle fd = (FileHandle)pSeq->dpyInfo->vOutFp;
    static UChar *buffer;
    static int bufferLength;
    UChar *pY, *pCr, *pCb, *pout;
    int width = pSeq->horizontalSize;
    int height = pSeq->verticalSize;
    int i;

    pY = ppic->y;

    if (bufferLength < pSeq->horizontalSize * pSeq->verticalSize) {
	if (buffer) {
	    free((char *)buffer);
	}
	bufferLength = pSeq->horizontalSize * pSeq->verticalSize;
	buffer = (UChar *)InsecureAlloc(bufferLength);
	if (buffer == NULL) {
	    ProgMsg("CopyYCToFile: can't allocate %d bytes\n", bufferLength);
	    bufferLength = 0;
	    return;
	}
    }

    pout = buffer;
    for (i = 0; i < height; i++) {
	memcpy(pout, pY, width);
	pout += width;
	pY += MAX_X;
    }
    (void) WriteData(fd, (char *)buffer, width*height);

    width >>= 1;
    height >>= 1;

    pout = buffer;
    pCr = CbToCrAddr(ppic->cb);
    for (i = 0; i < height; i++) {
	memcpy(pout, pCr, width);
	pout += width;
	pCr += MAX_X;
    }

    pCb = ppic->cb;
    for (i = 0; i < height; i++) {
	memcpy(pout, pCb, width);
	pout += width;
	pCb += MAX_X;
    }

    (void) WriteData(fd, (char *)buffer, width*height*2);
}

static void
DontConvert(DisplayInfoPtr dpyInfo, PictureInfoPtr ppic)
{
}

/*ARGSUSED*/
static void
DontDisplay(DisplayInfoPtr dpyInfo)
{
}

static void
NotConfigured(char *what)
{
    ProgMsg("Internal error: %s not configured\n", what);
}

static void
DisplayNotConfigured(DisplayInfoPtr dpyInfo)
{
    NotConfigured("display");
}

static void
ConvertAndDisplayLuma(DisplayInfoPtr dpyInfo, PictureInfoPtr ppic)
{
    dpyInfo->cvtu.ycToGray->y = ppic->y;
    (*dpyInfo->cvtu.ycToGray->lookupFrame)(dpyInfo->cvtu.ycToGray);
    if (! dpyInfo->oneStageConvert) {
        (*dpyInfo->refresher)(dpyInfo);
    }
}

static void
ConvertAndDisplayColor(DisplayInfoPtr dpyInfo, PictureInfoPtr ppic)
{
    YcToRgbInfoPtr ycToRgb = dpyInfo->cvtu.ycToRgb;

    ycToRgb->y = ppic->y;
    ycToRgb->cb = ppic->cb;
    ycToRgb->cr = CbToCrAddr(ppic->cb);

    (*ycToRgb->lookupFrame)(ycToRgb);
    if (! dpyInfo->oneStageConvert) {
        (*dpyInfo->refresher)(dpyInfo);
    }
}

#ifdef sun
static void
OctAddrPixelCopy(UChar *pPix, UChar *pFb,
		 long width, long height,
		 long pixWidth,
		 long fbLineBytes,
		 long widthDecrement)
{
    long accelToNextPixLine = pixWidth - width;
    long accelToNextFbLine = fbLineBytes - width;
    OctWord in1, in2;
    long w;

    while (--height >= 0) {
	/*
	 * We do two pairs of double longwords at a time because the
	 * processor will stall if you try to use a register immediately
	 * after a load.
	 */
	for (w = width; (w -= widthDecrement) >= 0; ) {
	    in1 = *OctWordP(pPix);	pPix += 8;
	    in2 = *OctWordP(pPix);	pPix += 8;
	    *OctWordP(pFb) = in1;	pFb += 8;
	    *OctWordP(pFb) = in2;	pFb += 8;
	}
	/*
	 * If the copy is not a multiple of 16, make up for it here.
	 */
	if (w != -widthDecrement) {
	    in1 = *OctWordP(pPix);	pPix += 8;
	    *OctWordP(pFb) = in1;	pFb += 8;
	}
	pPix += accelToNextPixLine;
	pFb += accelToNextFbLine;
    }
}

static void
OddAddrPixelCopy(UChar *pPix, UChar *pFb,
		 int width, int height,
		 int pixWidth,
		 int fbLineBytes)
{
    while (--height >= 0) {
	memcpy(pFb,pPix,width);
	pPix += pixWidth;
	pFb += fbLineBytes;
    }
}

/*
 * Assume that the copy buffer and the frame buffer are aligned on octword
 * boundaries, and that and dpyInfo->copier->lineBytes are expressed
 * in terms of longwords.
 */
/*ARGSUSED*/
void
CopyPixelsToFb(DisplayInfoPtr dpyInfo)
{
    long pixWidth = dpyInfo->copyBufferWidth;
    long width = dpyInfo->dst.width;
    long height = dpyInfo->dst.height;
    UChar *pFb = dpyInfo->copier->fb + dpyInfo->copyPixelStart;
    UChar *pPix = dpyInfo->copyBuffer;

    if( (int)pFb & 0x3 ) {
	OddAddrPixelCopy(pPix, pFb, width, height,
			 pixWidth, dpyInfo->copier->lineBytes);
    } else {
	OctAddrPixelCopy(pPix, pFb, width, height,
			 pixWidth, dpyInfo->copier->lineBytes,
			 dpyInfo->pixelsPer32Bits << 2);
    }
}

#else /* sun */
/*
 * Assume that the copy buffer and the frame buffer are aligned on octword
 * boundaries, and that and dpyInfo->copier->lineBytes are expressed
 * in terms of longwords.
 */
/*ARGSUSED*/
void
CopyPixelsToFb(DisplayInfoPtr dpyInfo)
{
    long pixWidth = dpyInfo->copyBufferWidth;
    long width = dpyInfo->dst.width;
    long height = dpyInfo->dst.height;
    UChar *pFb = dpyInfo->copier->fb + dpyInfo->copyPixelStart;
    UChar *pPix = dpyInfo->copyBuffer;
    OctWord in1, in2;
    long w, widthDecrement;
    long accelToNextFbLine;
    long accelToNextPixLine;

    accelToNextFbLine = dpyInfo->copier->lineBytes - width;
    accelToNextPixLine = pixWidth - width;
    widthDecrement = dpyInfo->pixelsPer32Bits << 2;
    while (--height >= 0) {
	/*
	 * We do two pairs of double longwords at a time because the
	 * processor will stall if you try to use a register immediately
	 * after a load.
	 */
	for (w = width; (w -= widthDecrement) >= 0; ) {
	    in1 = *OctWordP(pPix);	pPix += 8;
	    in2 = *OctWordP(pPix);	pPix += 8;
	    *OctWordP(pFb) = in1;	pFb += 8;
	    *OctWordP(pFb) = in2;	pFb += 8;
	}
	/*
	 * If the copy is not a multiple of 16, make up for it here.
	 */
	if (w != -widthDecrement) {
	    in1 = *OctWordP(pPix);	pPix += 8;
	    *OctWordP(pFb) = in1;	pFb += 8;
	}
	pPix += accelToNextPixLine;
	pFb += accelToNextFbLine;
    }
}
#endif /* sun */

static void
ConfigureClip(DisplayInfo *dpyInfo)
{
    ULong mask = (DPY_DestRect | DPY_Clip);
    SequenceInfoPtr pSeq = dpyInfo->pSeq;
    boolean doubleSize = dpyInfo->doubleSize;

    if ((dpyInfo->new.changed & mask) == 0)
	return;
    dpyInfo->new.changed &= ~mask;

    dpyInfo->dst = dpyInfo->new.dst;

    /*
     * calculate the display area
     */
    DebugTask(N_GeometryInfo,
	Msg("zip: clipBox %dx%d+%d+%d\n",
	    dpyInfo->new.clip.width, dpyInfo->new.clip.height,
	    dpyInfo->new.clip.x, dpyInfo->new.clip.y)
    )

    /*
     * move the destination box into to destination window and clip it
     */
    if (dpyInfo->destType == DestWinFrameBuffer
     && dpyInfo->copier->clipToScreen) {
	dpyInfo->clip = dpyInfo->new.clip;
	dpyInfo->dst.x += dpyInfo->clip.x;
	dpyInfo->dst.y += dpyInfo->clip.y;
	if (dpyInfo->dst.width > dpyInfo->clip.width)
	    dpyInfo->dst.width = dpyInfo->clip.width;
	if (dpyInfo->dst.height > dpyInfo->clip.height)
	    dpyInfo->dst.height = dpyInfo->clip.height;
    }

    /*
     * clip anything off the left and top of the physical display
     */ 
    if (dpyInfo->dst.x < 0) {
	dpyInfo->dst.width += dpyInfo->dst.x;
	dpyInfo->dst.x = 0;
    }
    if (dpyInfo->dst.y < 0) {
	dpyInfo->dst.height += dpyInfo->dst.y;
	dpyInfo->dst.y = 0;
    }

    /*
     * Decide if it is double-size or not.
     */
    if (dpyInfo->dst.width >= 2*pSeq->horizontalSize
     || dpyInfo->dst.height >= 2*pSeq->verticalSize) {
	dpyInfo->doubleSize = True;
	if (dpyInfo->dst.width > 2*pSeq->horizontalSize)
	    dpyInfo->dst.width = 2*pSeq->horizontalSize;
	if (dpyInfo->dst.height > 2*pSeq->verticalSize)
	    dpyInfo->dst.height = 2*pSeq->verticalSize;
    } else {
	dpyInfo->doubleSize = False;
	if (dpyInfo->dst.width > pSeq->horizontalSize)
	    dpyInfo->dst.width = pSeq->horizontalSize;
	if (dpyInfo->dst.height > pSeq->verticalSize)
	    dpyInfo->dst.height = pSeq->verticalSize;
    }

    DebugTask(N_GeometryInfo,
	Msg("zip: dst %dx%d+%d+%d, doublesize=%s\n",
	    dpyInfo->dst.width, dpyInfo->dst.height,
	    dpyInfo->dst.x, dpyInfo->dst.y,
	    dpyInfo->doubleSize ? "true" : "false")
    )

    if ( doubleSize != dpyInfo->doubleSize )
	dpyInfo->refresh = dpyInfo->rerenderRefresh = True;

    /*
     * Give the window system a chance to respond
     */
    SetWinClip(dpyInfo);
}

static void
ConfigureBuffer(DisplayInfoPtr dpyInfo)
{
    long copyBufferSize;
    int widthInPixels;

    /*
     * Calculate the off-screen buffer size.  This is used for double-buffering
     * as well as expose events.
     */
    widthInPixels = MAX_PIX_X;
    dpyInfo->copyBufferWidth = MAX_PIX_X * 4 / dpyInfo->pixelsPer32Bits;

    copyBufferSize = (MAX_PIX_X * MAX_PIX_Y * dpyInfo->pixelSizeInBits) / 8;
    if (dpyInfo->doubleSize && ! dpyInfo->hardwareZoom) {
	widthInPixels *= 2;
	copyBufferSize *= 4;
	dpyInfo->copyBufferWidth *= 2;
    }
    if (copyBufferSize != dpyInfo->bufferSize
     || dpyInfo->bufferHeight != dpyInfo->dst.height) {
	if (dpyInfo->copyBuffer)
	    FreeCopyBuffer(dpyInfo->copyBuffer);
	dpyInfo->bufferHeight = dpyInfo->dst.height;
	dpyInfo->bufferSize = copyBufferSize;
	dpyInfo->copyBuffer = AllocateCopyBuffer(copyBufferSize,
						 widthInPixels,
						 dpyInfo->dst.height);
    }
    SetWinBuffer(dpyInfo);
}

static char *
OutputName(DisplayInfoPtr dpyInfo)
{
    switch (dpyInfo->destType) {
    case DestWindow:
    case DestWinFrameBuffer:
	switch (dpyInfo->color) {
	case Rgb888:		return("24-bit color");
	case Rgb444:		return("12-bit color");
	case Rgb555:		return("16-bit color");
	case Rgb232:		return("7-bit color");
	case Rgb332:		return("8-bit color");
	case RgbC555:		return("8-bit 5-5-5 color");
	case RgbC666:		return("8-bit 6-6-6 color");
	case GrayScale8:	return("8-bit grayscale");
	case GrayScale7:	return("7-bit grayscale");
	case GrayScale1:	return("1-bit grayscale");
	default:		break;
	}
	break;
    case DestYcFrameBuffer:	return("raw yc");
    case DestYcFile:		return("file output");
    }
    return("unknown");
}
   
ImageReasonPtr
ConfigureDisplayForSequence(DisplayInfo *dpyInfo)
{
    GeomRectanglePtr convertDst;
    GeomRectangle unclippedDst;
    YcToGrayInfoPtr ycToGray;
    YcToRgbInfoPtr ycToRgb;
    ImageReasonPtr reason;
    RgbDepthType newColor;
    CopyInfo *copier;
    boolean visualDisplay;
    boolean outputTypeChanged;

    SetFrameRate(dpyInfo->pSeq);

    dpyInfo->u.s.reconfigure = False;
    dpyInfo->pSeq->adaptSchedule = dpyInfo->pSeq->showEveryFrame;
    if (IsInitialized(dpyInfo, SYM_FRAMEFREQ))
	dpyInfo->issueFrameEvent = dpyInfo->frameFrequency;
    else
	dpyInfo->issueFrameEvent = 0x7fffffff;
    if (!IsInitialized(dpyInfo, SYM_OUTFRAME))
	dpyInfo->outFrame = 0x7fffffff;

    if (dpyInfo->new.changed & DPY_SrcRect) {
	dpyInfo->src = dpyInfo->new.src;
	dpyInfo->new.changed &= ~DPY_SrcRect;
    }

    outputTypeChanged = False;
    if (dpyInfo->new.changed & (DPY_Destination | DPY_Color)) {
	newColor = dpyInfo->new.color;
	if (newColor == Rgb444 && dpyInfo->render444as888)
	    newColor = Rgb888;
	if (dpyInfo->new.destType != dpyInfo->destType
	 || newColor != dpyInfo->color) {
	    outputTypeChanged = True;
	    dpyInfo->destType = dpyInfo->new.destType;
	    dpyInfo->color = newColor;
	}
	dpyInfo->new.changed &= ~(DPY_Destination | DPY_Color);
    }

    visualDisplay = True;
    switch(dpyInfo->destType) {
    case DestYcFrameBuffer:
	/*
	 * special frame buffer output
	 */
	dpyInfo->windowState = WindowUnobscured;	/* force it */
	copier = &dpyInfo->ycFbCopier;
	InitYcFb(dpyInfo);
	break;
	
    case DestYcFile:
	/*
	 * file output
	 */
	copier = &dpyInfo->ycFileCopier;
	dpyInfo->windowState = WindowUnobscured;	/* force it */
	dpyInfo->pSeq->adaptSchedule = True;		/* force it */
	visualDisplay = False;
	break;

    case DestWindow:
    case DestWinFrameBuffer:
	copier = &dpyInfo->copiers[ dpyInfo->color ];
	break;
    }

    dpyInfo->visualDisplay = visualDisplay;
    dpyInfo->copier = copier;
    dpyInfo->converter = copier->converter;

    if (copier->fb == NULL
     && (dpyInfo->destType == DestWinFrameBuffer
      || dpyInfo->destType == DestYcFrameBuffer)) {
	reason = CreateReason("No frame buffer access in %s.  %s.\n",
			      OutputName(dpyInfo),
			      "Switch to X display");
	reason->hint = IR_BadDisplay;
	return (reason);
    }

    if (outputTypeChanged) {
	dpyInfo->rerenderRefresh = True;
	dpyInfo->refresh = True;
    }

    if (dpyInfo->destType == DestYcFile
     || dpyInfo->destType == DestYcFrameBuffer) {
	dpyInfo->refresher = copier->fbDisplay;
	dpyInfo->saveRefreshFrame = copier->saveFbFrame;
	return(NULL);	/* nothing more to do */
    }

    /*
     * From here on out, it is some kind of rgb or grayscale display
     */
    if (outputTypeChanged && (reason = WinInit(dpyInfo)))
	return(reason);

    ConfigureClip(dpyInfo);

    dpyInfo->pixelSizeInBits = LookupPixelSizeInBits(dpyInfo->color);
    dpyInfo->pixelsPer32Bits = 32 / dpyInfo->pixelSizeInBits;
    dpyInfo->clippedPictureArea = dpyInfo->dst.width
				* dpyInfo->dst.height
				* dpyInfo->pixelSizeInBits
				/ 8;

    /*
     * Choose the ->refresher routine.
     */
    dpyInfo->oneStageConvert = False;
    if (dpyInfo->destType == DestWindow) {
	dpyInfo->refresher = copier->winDisplay;
	dpyInfo->saveRefreshFrame = copier->saveWinFrame;
    } else {
	dpyInfo->refresher = copier->fbDisplay;
	dpyInfo->saveRefreshFrame = copier->saveFbFrame;
#ifdef sun
dpyInfo->copyPixelStart = dpyInfo->dst.x
+ dpyInfo->dst.y * copier->lineBytes;
#endif /* sun */
	if (dpyInfo->clippedPictureArea > copier->maxDirectConvertPix) {
#ifdef sun
#else /* sun */
	    dpyInfo->copyPixelStart = dpyInfo->dst.x
				    + dpyInfo->dst.y * copier->lineBytes;
#endif /* sun */
	    AlignConversionBox(dpyInfo->pixelSizeInBits,
			       8 /* always align on 8-byte boundaries */,
			       &dpyInfo->dst,
			       &dpyInfo->copyPixelStart);
	} else {
	    dpyInfo->oneStageConvert = True;
	}
    }

    if (dpyInfo->dst.width == 0 || dpyInfo->dst.height == 0) {
	dpyInfo->converter = DontConvert;
	return(NULL);
    }

    if (copier->clipToScreen && dpyInfo->oneStageConvert) {
	convertDst = &dpyInfo->dst;
    } else {
	unclippedDst = dpyInfo->new.dst; /* undo the effect of clipping */
	convertDst = &unclippedDst;
	unclippedDst.x = unclippedDst.y = 0;	/* only used during display */
    }

    ConfigureBuffer(dpyInfo);
    if (dpyInfo->copyBuffer == NULL) {
	return( &IRBadAllocation );
    }

    /*
     * Put together the right pointers and info for the lookup mechanism
     */
    reason = NULL;
    switch(dpyInfo->color) {
    case GrayScale1:
    case GrayScale7:
    case GrayScale8:
	dpyInfo->oneChannelOnly = True;
	ycToGray = dpyInfo->cvtu.ycToGray;
	if (dpyInfo->oneStageConvert) {
	    ycToGray->grayWidth = copier->lineBytes;
	    ycToGray->gray = copier->fb;
	} else {
	    ycToGray->grayWidth = dpyInfo->copyBufferWidth;
	    ycToGray->gray = dpyInfo->copyBuffer;
	}
	ycToGray->scale = (! dpyInfo->doubleSize || dpyInfo->hardwareZoom)
				     ? LookupScaleUnity : LookupScaleQuad;
	ycToGray->lumaWidth = MAX_PIX_X;
	ycToGray->depthType = dpyInfo->color;
	reason = InitYcToGrayInfo(&dpyInfo->cvtu.ycToGray,
				  &dpyInfo->src,
				  convertDst);
	break;

    case Rgb232:
    case Rgb332:
    case Rgb444:
    case Rgb555:
    case Rgb888:
    case RgbC555:
    case RgbC666:
	dpyInfo->oneChannelOnly = False;
	ycToRgb = dpyInfo->cvtu.ycToRgb;
	if (dpyInfo->oneStageConvert) {
	    ycToRgb->rgbWidth = copier->lineBytes;
	    ycToRgb->rgb = copier->fb;
	} else {
	    ycToRgb->rgbWidth = dpyInfo->copyBufferWidth;
	    ycToRgb->rgb = dpyInfo->copyBuffer;
	}

	ycToRgb->scale = (! dpyInfo->doubleSize || dpyInfo->hardwareZoom)
				     ? LookupScaleUnity : LookupScaleQuad;
	ycToRgb->lumaWidth = MAX_PIX_X;
	ycToRgb->chromaWidth = MAX_PIX_X;
	ycToRgb->depthType = dpyInfo->color;
	ycToRgb->tablesAreValid = (! outputTypeChanged);
	reason = InitYcToRgbInfo(&dpyInfo->cvtu.ycToRgb,
				 &dpyInfo->src,
				 convertDst);
	break;

    default:
	/* never happens */
	break;
    }

    return(reason);
}

static void
DontSaveRefreshFrame(DisplayInfoPtr dpyInfo, PictureInfoPtr ppic)
{
}

void
SaveRgbRefreshFrame(DisplayInfoPtr dpyInfo, PictureInfoPtr ppic)
{
    GeomRectanglePtr unclippedDst = &dpyInfo->new.dst;
    Queue *dq = &PictBuffer.displayQueue;
    InternalReasonPtr reason = NULL;

    while (ppic == NULL) {
	if ((ppic = dpyInfo->savedPic)
	 && ppic->picNumber == dpyInfo->currentFrame)
	    break;
	dq = &PictBuffer.displayQueue;
	if (QueueEmpty(dq)) {
	    ProgMsg("SaveRgbRefreshFrame: no current frame & q empty\n");
	    return;
	}
	ppic = TailOfQueue(dq, PictureInfo *);
	break;
    }

    /*
     * save just the structure, reconfigure, render the frame, then
     * replace the old configuration.
     */
    switch(dpyInfo->color) {
    case GrayScale1:
    case GrayScale7:
    case GrayScale8:
	{
	    YcToGrayInfoPtr ycToGrayTmp = dpyInfo->cvtuTmp.ycToGray;
	    YcToGrayInfoPtr ycToGray = dpyInfo->cvtu.ycToGray;

	    memcpy((char *)ycToGrayTmp,
		   (char *)ycToGray,
		   sizeof(YcToGrayInfo));
	    ycToGray->y = ppic->y;
	    ycToGray->grayWidth = dpyInfo->copyBufferWidth;
	    ycToGray->gray = dpyInfo->copyBuffer;
	    ycToGray->scale = (! dpyInfo->doubleSize || dpyInfo->hardwareZoom)
					 ? LookupScaleUnity : LookupScaleQuad;
	    ycToGray->lumaWidth = MAX_PIX_X;
	    ycToGray->depthType = dpyInfo->color;
#if 0
	    reason = InitYcToGrayInfo(&dpyInfo->cvtuTmp.ycToGray,
				      &dpyInfo->src,
				      unclippedDst);
	    ycToGray = dpyInfo->cvtuTmp.ycToGray;
#else /* 0 */
	    reason = InitYcToGrayInfo(&dpyInfo->cvtu.ycToGray,
				      &dpyInfo->src,
				      unclippedDst);
	    ycToGray = dpyInfo->cvtu.ycToGray;
#endif /* 0 */
	    if (reason == NULL)
		(*ycToGray->lookupFrame)(ycToGray);
	    memcpy((char *)ycToGray,
		   (char *)ycToGrayTmp,
		   sizeof(YcToGrayInfo));
	    break;

	}
    case Rgb232:
    case Rgb332:
    case Rgb444:
    case Rgb555:
    case Rgb888:
    case RgbC555:
    case RgbC666:
	{
	    YcToRgbInfoPtr ycToRgbTmp = dpyInfo->cvtuTmp.ycToRgb;
	    YcToRgbInfoPtr ycToRgb = dpyInfo->cvtu.ycToRgb;

	    memcpy((char *)ycToRgbTmp,
		   (char *)ycToRgb,
		   sizeof(YcToRgbInfo));
	    ycToRgb->y = ppic->y;
	    ycToRgb->cb = ppic->cb;
	    ycToRgb->cr = CbToCrAddr(ppic->cb);
	    ycToRgb->rgbWidth = dpyInfo->copyBufferWidth;
	    ycToRgb->rgb = dpyInfo->copyBuffer;
	    ycToRgb->scale = (! dpyInfo->doubleSize || dpyInfo->hardwareZoom)
					 ? LookupScaleUnity : LookupScaleQuad;
	    ycToRgb->lumaWidth = MAX_PIX_X;
	    ycToRgb->chromaWidth = MAX_PIX_X;
	    ycToRgb->depthType = dpyInfo->color;
#if 0
	    reason = InitYcToRgbInfo(&dpyInfo->cvtuTmp.ycToRgb,
				     &dpyInfo->src,
				     unclippedDst);
	    ycToRgb = dpyInfo->cvtuTmp.ycToRgb;
#else /* 0 */
	    reason = InitYcToRgbInfo(&dpyInfo->cvtu.ycToRgb,
				     &dpyInfo->src,
				     unclippedDst);
	    ycToRgb = dpyInfo->cvtu.ycToRgb;
#endif /* 0 */
	    if (reason == NULL)
		(*ycToRgb->lookupFrame)(ycToRgb);
	    memcpy((char *)ycToRgb,
		   (char *)ycToRgbTmp,
		   sizeof(YcToRgbInfo));
	    break;
	}
    default:
	/* never happens */
	break;
    }

    if (reason) {
	ShowReason(ProgMsg, reason);
	FreeReason(reason);
    }
}

void
InitDisplay(DisplayInfoPtr dpyInfo)
{
    short i;
    
    /*
     * XXX don't allow lots of pictures to be allocated.
     */
    dpyInfo->queueSize = PictureQueueSize;
    dpyInfo->new.changed = 0;
    dpyInfo->hardwareZoom = False;

    InitPictureBuffers(dpyInfo->pSeq);
    dpyInfo->cvtu.ycToRgb = AllocYcToRgbInfo(Rgb332);	/* pick one */
    dpyInfo->cvtuTmp.ycToRgb = AllocYcToRgbInfo(Rgb332);/* allocate minimum */
    if (dpyInfo->cvtu.ycToRgb == NULL)
	Fatal("can't allocate YcToRgbInfo\n");

    for (i = 0; i < NRgbDepthTypes; i++) {
        switch(i) {
        case GrayScale8:
        case GrayScale7:
        case GrayScale1:
            dpyInfo->copiers[ i ].converter = ConvertAndDisplayLuma;
            break;
        default:
            dpyInfo->copiers[ i ].converter = ConvertAndDisplayColor;
            break;
	}
	dpyInfo->copiers[ i ].saveWinFrame = SaveRgbRefreshFrame;
	dpyInfo->copiers[ i ].saveFbFrame = SaveRgbRefreshFrame;
	dpyInfo->copiers[ i ].winDisplay = DisplayNotConfigured;
	dpyInfo->copiers[ i ].fbDisplay = DisplayNotConfigured;
    }

    dpyInfo->ycFileCopier.saveFbFrame = DontSaveRefreshFrame;
    dpyInfo->ycFileCopier.fbDisplay = DontDisplay;
    dpyInfo->ycFileCopier.winDisplay = DontDisplay;
    dpyInfo->ycFileCopier.saveFbFrame = DontConvert;
    dpyInfo->ycFileCopier.converter = CopyYCToFile;

    dpyInfo->ycFileCopier.fb = (UChar*)"";	/* makes it look available */
    dpyInfo->ycFileCopier.maxDirectConvertPix = ~0;

    dpyInfo->saveRefreshFrame = DontSaveRefreshFrame;
    dpyInfo->refresher = DontDisplay;

    SunFbInit(dpyInfo);
    SgiFbInit(dpyInfo);
}

#ifndef SUNFB
void SunFbInit(DisplayInfoPtr dpyInfo) {}
#endif

#ifndef SGIFB
void SgiFbInit(DisplayInfoPtr dpyInfo) {}
void InitYcFb(DisplayInfoPtr dpyInfo) {}
#endif
