/* $Header: /usr/local/src/nvrToMIT/RCS/genidcttable.c,v 1.3 1993/10/22 23:53:29 russ Exp $ */

#define NeedFunctionPrototypes 1

#include <stdio.h>
#include <sys/types.h>

#include "nvrTypes.h"
#include "general.h"
#include "idct.h"
#include "binarybits.h"
#include "bitstream.h"
#include "longjump.h"
#include "decoder.h"

#define True	1
#define False	0

char	*ProgramName;
char LeftComment[3] = { '/', '*', '\0' };
char RightComment[3] = { '*', '/', '\0' };

/*
 * This list appears because sometimes the compiler is too stupid
 * to arrange for the final results (in the IDctRow macro) to be in
 * the proper registers for the std instruction generated by the OctWord
 * structure and actually generates extra instructions
 * to put them in the right place.
 */
UChar StupidRowFunctions[] = {
    BitsFromArgs(0,0,0,0,0,0,0,1),
    BitsFromArgs(0,0,0,0,0,0,1,0),
    BitsFromArgs(0,0,0,0,0,1,0,0),
    BitsFromArgs(0,0,0,0,0,1,0,1),
    BitsFromArgs(0,0,0,0,1,0,0,0),
    BitsFromArgs(0,0,0,0,1,0,1,0),
    BitsFromArgs(0,0,0,1,0,0,0,0),
    BitsFromArgs(0,0,0,1,0,1,0,0),
    BitsFromArgs(0,0,1,0,0,0,0,0),
    BitsFromArgs(0,0,1,0,1,0,0,0),
    BitsFromArgs(1,0,0,0,0,0,0,0),
    BitsFromArgs(1,0,0,0,0,0,1,0),
    BitsFromArgs(1,0,0,0,1,0,0,0),
    BitsFromArgs(1,0,1,0,0,0,0,0),
    BitsFromArgs(1,0,1,0,1,0,0,0),
};

/*
 * The FrequentIDctRow[] and FrequentIDctCol[] are a guess at the most
 * frequent dct operations.  This guess was compiled 8/29/92 and
 * represents 90% of the activity for IDctRow and 80% for IDctCol.
 */
UChar FrequentIDctRow[] = {
    BitsFromArgs(0,0,0,0,0,0,0,0),
    BitsFromArgs(0,0,0,0,0,0,0,1),
    BitsFromArgs(0,0,0,0,0,0,1,0),
    BitsFromArgs(0,0,0,0,0,0,1,1),
    BitsFromArgs(0,0,0,0,0,1,0,0),
    BitsFromArgs(0,0,0,0,0,1,0,1),
    BitsFromArgs(0,0,0,0,0,1,1,0),
    BitsFromArgs(0,0,0,0,1,0,0,0),
    BitsFromArgs(0,0,0,0,1,0,1,0),
    BitsFromArgs(0,0,0,0,1,1,0,0),
    BitsFromArgs(0,0,0,1,0,0,0,0),
    BitsFromArgs(0,0,0,1,0,1,0,0),
    BitsFromArgs(0,0,0,1,1,0,0,0),
    BitsFromArgs(0,0,1,0,0,0,0,0),
    BitsFromArgs(0,0,1,0,0,1,0,0),
    BitsFromArgs(0,0,1,0,1,0,0,0),
    BitsFromArgs(0,0,1,1,0,0,0,0),
    BitsFromArgs(0,1,0,0,0,0,0,0),
    BitsFromArgs(0,1,0,0,0,1,0,0),
    BitsFromArgs(0,1,0,0,1,0,0,0),
    BitsFromArgs(0,1,0,1,0,0,0,0),
    BitsFromArgs(0,1,1,0,0,0,0,0),
    BitsFromArgs(0,1,1,0,1,0,0,0),
    BitsFromArgs(0,1,1,1,0,0,0,0),
    BitsFromArgs(1,0,0,0,0,0,0,0),
    BitsFromArgs(1,0,0,0,0,0,1,0),
    BitsFromArgs(1,0,0,0,0,1,0,0),
    BitsFromArgs(1,0,0,0,1,0,0,0),
    BitsFromArgs(1,0,0,1,0,0,0,0),
    BitsFromArgs(1,0,1,0,0,0,0,0),
    BitsFromArgs(1,0,1,0,1,0,0,0),
    BitsFromArgs(1,0,1,1,0,0,0,0),
    BitsFromArgs(1,0,1,1,1,0,0,0),
    BitsFromArgs(1,1,0,0,0,0,0,0),
    BitsFromArgs(1,1,0,0,0,1,0,0),
    BitsFromArgs(1,1,0,0,1,0,0,0),
    BitsFromArgs(1,1,0,1,0,0,0,0),
    BitsFromArgs(1,1,0,1,1,0,0,0),
    BitsFromArgs(1,1,1,0,0,0,0,0),
    BitsFromArgs(1,1,1,0,0,1,0,0),
    BitsFromArgs(1,1,1,0,1,0,0,0),
    BitsFromArgs(1,1,1,0,1,1,0,0),
    BitsFromArgs(1,1,1,1,0,0,0,0),
    BitsFromArgs(1,1,1,1,0,1,0,0),
    BitsFromArgs(1,1,1,1,1,0,0,0),
    BitsFromArgs(1,1,1,1,1,1,0,0),
    BitsFromArgs(1,1,1,1,1,1,1,0),
    BitsFromArgs(1,1,1,1,1,1,1,1),
};

UChar FrequentIDctCol[] = {
    BitsFromArgs(0,0,0,0,0,0,0,0),
    BitsFromArgs(0,0,0,0,0,0,0,1),
    BitsFromArgs(0,0,0,0,0,0,1,0),
    BitsFromArgs(0,0,0,0,0,0,1,1),
    BitsFromArgs(0,0,0,0,0,1,0,0),
    BitsFromArgs(0,0,0,0,0,1,0,1),
    BitsFromArgs(0,0,0,0,0,1,1,0),
    BitsFromArgs(0,0,0,0,0,1,1,1),
    BitsFromArgs(0,0,0,0,1,0,0,0),
    BitsFromArgs(0,0,0,0,1,0,0,1),
    BitsFromArgs(0,0,0,0,1,0,1,0),
    BitsFromArgs(0,0,0,0,1,0,1,1),
    BitsFromArgs(0,0,0,0,1,1,0,0),
    BitsFromArgs(0,0,0,0,1,1,1,0),
    BitsFromArgs(0,0,0,0,1,1,1,1),
    BitsFromArgs(0,0,0,1,0,0,0,0),
    BitsFromArgs(0,0,0,1,1,0,0,0),
    BitsFromArgs(0,0,0,1,1,1,0,0),
    BitsFromArgs(0,0,0,1,1,1,1,1),
    BitsFromArgs(0,0,1,0,0,0,0,0),
    BitsFromArgs(0,0,1,0,1,0,0,0),
    BitsFromArgs(0,0,1,1,0,0,0,0),
    BitsFromArgs(0,0,1,1,1,1,1,1),
    BitsFromArgs(0,1,0,0,0,0,0,0),
    BitsFromArgs(0,1,0,1,0,0,0,0),
    BitsFromArgs(0,1,0,1,1,1,1,1),
    BitsFromArgs(0,1,1,0,0,0,0,0),
    BitsFromArgs(0,1,1,1,0,0,0,0),
    BitsFromArgs(0,1,1,1,1,0,0,0),
    BitsFromArgs(0,1,1,1,1,1,1,0),
    BitsFromArgs(0,1,1,1,1,1,1,1),
    BitsFromArgs(1,0,0,0,0,0,0,0),
    BitsFromArgs(1,0,0,0,0,0,0,1),
    BitsFromArgs(1,0,0,0,0,0,1,0),
    BitsFromArgs(1,0,0,0,0,1,0,0),
    BitsFromArgs(1,0,0,0,1,0,0,0),
    BitsFromArgs(1,0,0,1,0,0,0,0),
    BitsFromArgs(1,0,0,1,1,1,1,1),
    BitsFromArgs(1,0,1,0,0,0,0,0),
    BitsFromArgs(1,0,1,1,0,0,0,0),
    BitsFromArgs(1,0,1,1,1,1,1,1),
    BitsFromArgs(1,1,0,0,0,0,0,0),
    BitsFromArgs(1,1,0,0,1,0,0,0),
    BitsFromArgs(1,1,0,1,0,0,0,0),
    BitsFromArgs(1,1,0,1,1,0,0,0),
    BitsFromArgs(1,1,0,1,1,1,1,1),
    BitsFromArgs(1,1,1,0,0,0,0,0),
    BitsFromArgs(1,1,1,0,1,0,0,0),
    BitsFromArgs(1,1,1,0,1,1,0,0),
    BitsFromArgs(1,1,1,0,1,1,1,0),
    BitsFromArgs(1,1,1,0,1,1,1,1),
    BitsFromArgs(1,1,1,1,0,0,0,0),
    BitsFromArgs(1,1,1,1,0,0,1,0),
    BitsFromArgs(1,1,1,1,0,1,0,0),
    BitsFromArgs(1,1,1,1,0,1,1,0),
    BitsFromArgs(1,1,1,1,0,1,1,1),
    BitsFromArgs(1,1,1,1,1,0,0,0),
    BitsFromArgs(1,1,1,1,1,0,0,1),
    BitsFromArgs(1,1,1,1,1,0,1,0),
    BitsFromArgs(1,1,1,1,1,0,1,1),
    BitsFromArgs(1,1,1,1,1,1,0,0),
    BitsFromArgs(1,1,1,1,1,1,0,1),
    BitsFromArgs(1,1,1,1,1,1,1,0),
    BitsFromArgs(1,1,1,1,1,1,1,1),
};

UChar BitCounts[ 256 ];

void Usage();
void PrintMotionTable();
void BuildBitCounts();
void PrintIDctTable();
FILE *PipeTo();
FILE *PrintIDctFunc();

main(int argc, char **argv)
{
    FILE *fp;
    long i;
    long col0, col1, col2, col3, col4, col5, col6, col7;
    ProgramName = *argv;
    if (argc != 2)
	Usage();

    BuildBitCounts();

    if (strcmp(argv[1], "row") == 0) {
      fp = PrintIDctFunc(False);
      PrintIDctTable(fp, False);
    } else if (strcmp(argv[1], "col") == 0) {
      fp = PrintIDctFunc(True);
      PrintIDctTable(fp, True);
      PrintMotionTable(fp, "");
      PrintMotionTable(fp, "0Dc");
    } else {
	Usage();
    }
    fflush(fp);

    exit(0);
}

FILE *PipeTo(char *command, char *a0)
{
    FILE *fp;
    int pid;
    int pipeFd[2];

    if (pipe(pipeFd) < 0)
	Fatal("can't creat pipe\n");
    if ((pid = vfork()) < 0)
	Fatal("can't fork\n");
    if (pid == 0) {
	close(pipeFd[1]);
	dup2(pipeFd[0], 0);
	close(pipeFd[0]);
	execlp(command, command, a0, 0);
	Fatal("can't exec %s\n", command);
    }

    fp = fdopen(pipeFd[1], "w");
    close(pipeFd[0]);
    return (fp);
}

UChar ReplacementFunc(UChar *frequent, long nFrequent, UChar *bits)
{
    long i;
    UChar foundBits = bits;
    UChar nFoundBits = 9;

    for (i = 0; i < nFrequent; i++)
	if ((frequent[i] & bits) == bits) {
	    if (BitCounts[ frequent[i] ] < nFoundBits) {
		foundBits = frequent[i];
		nFoundBits = BitCounts[ foundBits ];
	    }
	}
    return(foundBits);
}

void PrintIDctTable(FILE *fp, boolean column)
{
    long b0, b1, b2, b3, b4, b5, b6, b7;
    int n = 0;
    UChar *frequent;

    fprintf(fp, "%s (*IDct%sTable[256])() = {\n",
      column ? "void" : "ULong",
      column ? "Col" : "Row");
    for (b7 = 0; b7 <= 1; b7++) {
      for (b6 = 0; b6 <= 1; b6++) {
	for (b5 = 0; b5 <= 1; b5++) {
	  for (b4 = 0; b4 <= 1; b4++) {
	    for (b3 = 0; b3 <= 1; b3++) {
	      for (b2 = 0; b2 <= 1; b2++) {
		for (b1 = 0; b1 <= 1; b1++) {
		  for (b0 = 0; b0 <= 1; b0++) {
		    fprintf(fp, "IDct%s%d%d%d%d%d%d%d%d, ",
		      column ? "Col" : "Row",
		      b0, b1, b2, b3, b4, b5, b6, b7);
		    if ((++n & 0x3) == 0x0)
		      fprintf(fp, "\n");
		  }
		}
	      }
	    }
	  }
	}
      }
    }
    fprintf(fp, "};\n");
}

void PrintMotionTable(FILE *fp, char *suffix)
{
    int i;

    fprintf(fp, "void (*MotionAndClamp%sTable[64])() = {\n", suffix);
    for(i = 0; i < NMotionAndClamp; i++) {
	switch (i & MotionAndClampToIndex(0, True, True)) {
	case MotionAndClampToIndex(0, False, False):
	    fprintf(fp, "NoMotionAndClamp%s,\n", suffix);
	    break;
	case MotionAndClampToIndex(0, False, True):
	case MotionAndClampToIndex(0, True, False):
	    switch (i & ~MotionAndClampToIndex(0, True, True)) {
	    case HALF_HORIZ_BACK_PIXEL:
	    case HALF_HORIZ_FORWARD_PIXEL:
		fprintf(fp, "MotionHalfHorizAndClamp%s,\n", suffix);
		break;
	    case HALF_VERT_BACK_PIXEL:
	    case HALF_VERT_FORWARD_PIXEL:
		fprintf(fp, "MotionHalfVertAndClamp%s,\n", suffix);
		break;
	    case HALF_HORIZ_BACK_PIXEL | HALF_VERT_BACK_PIXEL:
	    case HALF_HORIZ_FORWARD_PIXEL | HALF_VERT_FORWARD_PIXEL:
		fprintf(fp, "MotionHalfAndClamp%s,\n", suffix);
		break;
	    case 0:
		fprintf(fp, "MotionAndClamp%s,\n", suffix);
		break;
	    default:
		fprintf(fp, "IllegalMotionAndClamp,\n");
		break;
	    }
	    break;
	case MotionAndClampToIndex(0, True, True):
	    fprintf(fp, "FwdBackMotionAndClamp%s,\n", suffix);
	    break;
	}
    }
    fprintf(fp, "};\n");
}

void Usage()
{
    Msg("Usage: %s row|col\n", ProgramName);
    Msg("this generates tables for computing idct\n");
    exit (1);
}

void BuildBitCounts()
{
    long b0, b1, b2, b3, b4, b5, b6, b7;
    long nBits;

    for (b7 = 0; b7 <= 1; b7++) {
      for (b6 = 0; b6 <= 1; b6++) {
	for (b5 = 0; b5 <= 1; b5++) {
	  for (b4 = 0; b4 <= 1; b4++) {
	    for (b3 = 0; b3 <= 1; b3++) {
	      for (b2 = 0; b2 <= 1; b2++) {
		for (b1 = 0; b1 <= 1; b1++) {
		  for (b0 = 0; b0 <= 1; b0++) {
		    nBits  = b0 ? 1 : 0;
		    nBits += b1 ? 1 : 0;
		    nBits += b2 ? 1 : 0;
		    nBits += b3 ? 1 : 0;
		    nBits += b4 ? 1 : 0;
		    nBits += b5 ? 1 : 0;
		    nBits += b6 ? 1 : 0;
		    nBits += b7 ? 1 : 0;
		    BitCounts[ BitsFromArgs(b0,b1,b2,b3,b4,b5,b6,b7) ] = nBits;
		  }
		}
	      }
	    }
	  }
	}
      }
    }
}

void ZeroMissingCol(FILE *fp, UChar mmissingBits)
{
    long i;

    for (i = 0; i < 8; i++)
	if (missingBits & (1<<(7-i)))
	    fprintf(fp, "ptr[%d] = 0;\n", i);
}

void ZeroMissingRows(FILE *fp, UChar missingBits)
{
    long i;

    fprintf(fp, "OctWord zero;\n");
    fprintf(fp, "zero.l.high = zero.l.low = 0;\n");
    for (i = 0; i < 8; i++)
	if (missingBits & (1<<(7-i)))
	    fprintf(fp, "ZeroRow(ptr, %d)\n", i);
}

boolean IsStupidFunc(UChar bits)
{
    long i;
    for (i = 0; i < NumberOfCells(StupidRowFunctions); i++)
	if (StupidRowFunctions[i] == bits)
	    return (True);
    return (False);
}

FILE *PrintIDctFunc(boolean column)
{
    FILE *fp;
    long b0,b1,b2,b3,b4,b5,b6,b7;
    long bits, newBits, i;

    printf("#include \"types.h\"\n");
    printf("#include \"general.h\"\n");
    printf("#include \"dctdef.h\"\n");
    printf("#include \"binarybits.h\"\n");
    fflush(stdout);
    fp = PipeTo("/lib/cpp", "-C");
    fprintf(fp, "#include \"idct.h\"\n");
    for (b7 = 0; b7 <= 1; b7++) {
      for (b6 = 0; b6 <= 1; b6++) {
	for (b5 = 0; b5 <= 1; b5++) {
	  for (b4 = 0; b4 <= 1; b4++) {
	    for (b3 = 0; b3 <= 1; b3++) {
	      for (b2 = 0; b2 <= 1; b2++) {
		for (b1 = 0; b1 <= 1; b1++) {
		  for (b0 = 0; b0 <= 1; b0++) {
		    if ((bits = BitsFromArgs(b0,b1,b2,b3,b4,b5,b6,b7)) == 0)
			continue;	/* provided externally */

		    fprintf(fp, "static %s IDct%s%d%d%d%d%d%d%d%d(ptr)\n",
		      column ? "void" : "ULong",
		      column ? "Col" : "Row",
		      b0, b1, b2, b3, b4, b5, b6, b7);
		    fprintf(fp, "long *ptr;\n");
		    fprintf(fp, "{\n");

		    if (column) {
		      newBits = ReplacementFunc(FrequentIDctCol,
						NumberOfCells(FrequentIDctCol),
						bits);
		      if (bits == newBits) {
			fprintf(fp, "IDctCol(%s,%s,%s,%s,%s,%s,%s,%s,ptr)\n",
			  b0 ? "CellInCol(ptr, 0, 0, 8)" : "0",
			  b1 ? "CellInCol(ptr, 1, 0, 8)" : "0",
			  b2 ? "CellInCol(ptr, 2, 0, 8)" : "0",
			  b3 ? "CellInCol(ptr, 3, 0, 8)" : "0",
			  b4 ? "CellInCol(ptr, 4, 0, 8)" : "0",
			  b5 ? "CellInCol(ptr, 5, 0, 8)" : "0",
			  b6 ? "CellInCol(ptr, 6, 0, 8)" : "0",
			  b7 ? "CellInCol(ptr, 7, 0, 8)" : "0");
		      } else {
			fprintf(fp, "static void IDctCol%d%d%d%d%d%d%d%d();",
			  ArgsFromBits(newBits));
			ZeroMissingRows(fp, newBits & ~bits);
			fprintf(fp, "IDctCol%d%d%d%d%d%d%d%d(ptr);\n",
			  ArgsFromBits(newBits));
		      }
		    } else {
		      newBits = ReplacementFunc(FrequentIDctRow,
						NumberOfCells(FrequentIDctRow),
						bits);
		      if (bits == newBits) {
			fprintf(fp,
			  "IDctRow%s(%s,%s,%s,%s,%s,%s,%s,%s, ptr);\n",
			  IsStupidFunc(bits) ? "Stupid" : "",
			  b0 ? "ptr[0]" : "0",
			  b1 ? "ptr[1]" : "0",
			  b2 ? "ptr[2]" : "0",
			  b3 ? "ptr[3]" : "0",
			  b4 ? "ptr[4]" : "0",
			  b5 ? "ptr[5]" : "0",
			  b6 ? "ptr[6]" : "0",
			  b7 ? "ptr[7]" : "0");
			fprintf(fp, "return(1);\n");
		      } else {
			fprintf(fp, "static ULong IDctRow%d%d%d%d%d%d%d%d();",
			  ArgsFromBits(newBits));
			ZeroMissingCol(fp, newBits & ~bits);
			fprintf(fp, "return( IDctRow%d%d%d%d%d%d%d%d(ptr) );\n",
			  ArgsFromBits(newBits));
		      }
		    }
		    fprintf(fp, "}\n");
		  }
		}
	      }
	    }
	  }
	}
      }
    }
    return(fp);
}
