/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              genrgbtable.c
 *
 *      DESCRIPTION
 *              <overall description of what the package does>
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/genrgbtable.c,v 4.2 1993/10/25 21:51:18 russ Exp $
 */


#define NeedFunctionPrototypes 1

#include <stdio.h>
#include <sys/types.h>
#include <stdarg.h>

#include "general.h"
#include "decoder.h"
#include "display.h"

#define True	1
#define False	0

char	*ProgramName;
char LeftComment[3] = { '/', '*', '\0' };
char RightComment[3] = { '*', '/', '\0' };

typedef struct _bitCluster {
    int offset;
    int width;
} BitCluster;

BitCluster Clusters[16];

void
Msg(char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
}

void Usage()
{
    Msg("Usage: %s\n", ProgramName);
    Msg("this generates tables for looking up RGB from Y/Cr/Cb\n");
    exit (1);
}


main(int argc, char **argv)
{
    long cr, cb;
    long i, j;
    long lastIndex;
    long redIndex, greenIndex, blueIndex;
    float redAdd, greenAdd, blueAdd;
    long minRedIndex, minGreenIndex, minBlueIndex;
    long maxRedIndex, maxGreenIndex, maxBlueIndex;

    ProgramName = *argv;
    if (argc != 1)
	Usage();

    printf("#ifdef LOOKUP_RGB\n");
    printf("#include \"general.h\"\n");
    printf("#include \"decoder.h\"\n");
    printf("#include \"display.h\"\n");
    printf("\n");
    printf("RGBLookup RGBTable = {\n");
    GeneratePrimaryTable("232");
    GeneratePrimaryTable("332");
    GeneratePrimaryTable("888");

    printf("\n");
    printf("#define t(r,g,b) TripleFromRGB(r,g,b)\n"); /* reduce output size */
    printf("\n");
    printf(" {\n");

    maxRedIndex = 0;
    maxGreenIndex = 0;
    maxBlueIndex = 0;
    minRedIndex = NRedLookup;
    minGreenIndex = NGreenLookup;
    minBlueIndex = NBlueLookup;

    for (cr = 0; cr < 256; cr++) {
	for (cb = 0; cb < 256; cb++) {
	    /*
	     * These values in this table are indices that point to
	     * ready-made calculations of red, green or blue.
	     * Just add y.
	     */
            redAdd = YCrCbToRed(0,cr,cb);
            greenAdd = YCrCbToGreen(0,cr,cb);
            blueAdd = YCrCbToBlue(0,cr,cb);
	    if ((cb & 0x3f) == 0)
		printf("%s cr,cb=%d,%d %s\n ",
		    LeftComment, cr, cb, RightComment);
	    /*
	     * The added bias is to compensate for negative values
	     * in redAdd, greenAdd and blueAdd (e.g. blueAdd is
	     * -277 when cb is 0).
	     */
	    redIndex   = (redAdd + RedBias) / LookupStepSize + .5;
	    greenIndex = (greenAdd + GreenBias) / LookupStepSize + .5;
	    blueIndex  = (blueAdd + BlueBias) / LookupStepSize + .5;

	    if (redIndex < minRedIndex)
		minRedIndex = redIndex;
	    if (greenIndex < minGreenIndex)
		minGreenIndex = greenIndex;
	    if (blueIndex < minBlueIndex)
		minBlueIndex = blueIndex;
	    if (redIndex+255 > maxRedIndex)
		maxRedIndex = redIndex+255;
	    if (greenIndex+255 > maxGreenIndex)
		maxGreenIndex = greenIndex+255;
	    if (blueIndex+255 > maxBlueIndex)
		maxBlueIndex = blueIndex+255;
	    /*
	     * Make sure the generated indices don't exceed the allocated
	     * space.
	     */
	    if (redIndex + 255 > NRedLookup || redIndex < 0)
		Msg("red %d out of bounds (0-%d) @ cr,cb=(%d,%d)\n",
		    redIndex + 255, NRedLookup, cr, cb);
	    if (greenIndex + 255 > NGreenLookup || greenIndex < 0)
		Msg("green %d out of bounds (0-%d) @ cr,cb=(%d,%d)\n",
		    greenIndex + 255, NGreenLookup, cr, cb);
	    if (blueIndex + 255 > NBlueLookup || blueIndex < 0)
		Msg("blue %d out of bounds (0-%d) @ cr,cb=(%d,%d)\n",
		    blueIndex + 255, NBlueLookup, cr, cb);
	    printf("t(%d,%d,%d),", redIndex, greenIndex, blueIndex);
	    if ((cb & 0x3) == 0x3)
		printf("\n ");
	}
    }
    printf(" }\n");
    printf("};\n");

    printf("#endif %s LOOKUP_RGB %s\n", LeftComment, RightComment);

    Msg("Calculated indices:\n");
    Msg("red: %d - %d, calculated: %d - %d\n",
	0, NRedLookup, minRedIndex, maxRedIndex);
    Msg("green: %d - %d, calculated: %d - %d\n",
	0, NGreenLookup, minGreenIndex, maxGreenIndex);
    Msg("blue: %d - %d, calculated: %d - %d\n",
	0, NBlueLookup, minBlueIndex, maxBlueIndex);
    exit(0);
}

GeneratePrimaryTable(char *nameOfBits)
{
    long lastIndex;
    long i, j;
    char *macroName;
    long addBaseInt;
    float addBase;

    /*
     * These values are the results of redAdd or greenAdd or blueAdd
     * after having added in 1.16438*y.  
     */
    printf(" {\n");
    lastIndex = 0;
    j = 0;
    for (i = 0; i < 3; i++) {
	switch(i) {
	case 0:
	    printf("%s Primary lookup for %s %s\n",
		LeftComment, nameOfBits, RightComment);
	    addBase = -RedBias;
	    macroName = "r";
	    lastIndex = NRedLookup;
	    printf("  {\n");
	    printf("#define r(n) GetR%sBits(n)\n", nameOfBits);
	    printf("  %s red %s\n", LeftComment, RightComment);
	    printf("  %s0:%.1f%s ", LeftComment, addBase, RightComment);
	    break;
	case 1:
	    printf("\n#undef r\n");
	    printf("  },\n\n");
	    addBase = -GreenBias;
	    macroName = "g";
	    lastIndex = NGreenLookup;
	    printf("  {\n");
	    printf("#define g(n) GetG%sBits(n)\n", nameOfBits);
	    printf("  %s green %s\n", LeftComment, RightComment);
	    printf("  %s0:%.1f%s ", LeftComment, addBase, RightComment);
	    break;
	case 2:
	    printf("\n#undef g\n");
	    printf("  },\n\n");
	    lastIndex = NBlueLookup;
	    addBase = -BlueBias;
	    macroName = "b";
	    printf("  {\n");
	    printf("#define b(n) GetB%sBits(n)\n", nameOfBits);
	    printf("  %s blue %s\n", LeftComment, RightComment);
	    printf("  %s0:%.1f %s ", LeftComment, addBase, RightComment);
	    break;
	}

	for (j = 0; j < lastIndex; ) {
	    addBaseInt = addBase + .5;
	    if (addBaseInt < 0)
		addBaseInt = 0;
	    else if (addBaseInt > 255)
		addBaseInt = 255;
	    printf("%s(%d),", macroName, addBaseInt);
	    if ((j++ & 0x7) == 0x7)
		printf("\n  %s%d:%.1f%s ",
		    LeftComment, j, addBase, RightComment);
	    addBase += LookupStepSize;
	}
    }
    printf("\n#undef b\n", nameOfBits);
    printf("\n");
    printf("  }\n");
    printf(" },\n\n");
}

