/*
 *      (c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              display.c
 *
 *      DESCRIPTION
 *              handle display activity
 *		X-DEPENDENT
 *		OS-DEPENDENT
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/glvis.c,v 1.1 1994/03/12 19:00:00 toddb Exp $
 */

#include <X11/keysym.h>
#include <stdarg.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/XShm.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <GL/glx.h>

char *ProgramName;

void
Fatal(char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
    fflush(stderr);
}

static char *
GlAttrToString(int attr)
{
    switch(attr) {
    case GLX_USE_GL:		return("GLX_USE_GL");
    case GLX_BUFFER_SIZE:	return("GLX_BUFFER_SIZE");
    case GLX_LEVEL:		return("GLX_LEVEL");
    case GLX_RGBA:		return("GLX_RGBA");
    case GLX_DOUBLEBUFFER:	return("GLX_DOUBLEBUFFER");
    case GLX_STEREO:		return("GLX_STEREO");
    case GLX_AUX_BUFFERS :	return("GLX_AUX_BUFFERS ");
    case GLX_RED_SIZE:		return("GLX_RED_SIZE");
    case GLX_GREEN_SIZE:	return("GLX_GREEN_SIZE");
    case GLX_BLUE_SIZE:		return("GLX_BLUE_SIZE");
    case GLX_ALPHA_SIZE:	return("GLX_ALPHA_SIZE");
    case GLX_DEPTH_SIZE:	return("GLX_DEPTH_SIZE");
    case GLX_STENCIL_SIZE:	return("GLX_STENCIL_SIZE");
    case GLX_ACCUM_RED_SIZE:	return("GLX_ACCUM_RED_SIZE");
    case GLX_ACCUM_GREEN_SIZE:	return("GLX_ACCUM_GREEN_SIZE");
    case GLX_ACCUM_BLUE_SIZE:	return("GLX_ACCUM_BLUE_SIZE");
    case GLX_ACCUM_ALPHA_SIZE:	return("GLX_ACCUM_ALPHA_SIZE");
    default:			return("?");
    }
}
    
static void
ShowGlxConfig(Display *dpy, XVisualInfo *visInfo, int attrib, char *fmt)
{
    int value, result;

    printf("    %s: ", GlAttrToString(attrib));

    result = glXGetConfig(dpy, visInfo, attrib, &value);
    if (result) {
	switch(result) {
	case GLX_BAD_SCREEN:    printf("bad screen\n"); return;
	case GLX_BAD_ATTRIBUTE: printf("bad attribute\n"); return;
	case GLX_NO_EXTENSION:  printf("no extension\n"); return;
	case GLX_BAD_VISUAL:    printf("bad visual\n"); return;
	}
    } else {
	printf(fmt, value);
	printf("\n");
    }
}

static void
glxShowConfig(Display *dpy, VisualID visid)
{
    XVisualInfo visInfo;
    XVisualInfo template, *v;
    int n;
    static char *colorClasses[] = {
        "StaticGray",
        "GrayScale",
        "StaticColor",
        "PseudoColor",
        "TrueColor",
        "DirectColor" };

    visInfo.visualid = visid;
    visInfo.screen = DefaultScreen(dpy);
    v = XGetVisualInfo (dpy, VisualScreenMask|VisualIDMask, &visInfo, &n);
    printf("vis 0x%x, class=%s, depth=%d:\n",
	visid, colorClasses[ v->visual->class ], v->depth);

    ShowGlxConfig(dpy, &visInfo, GLX_USE_GL, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_BUFFER_SIZE, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_LEVEL, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_RGBA, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_DOUBLEBUFFER, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_STEREO, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_AUX_BUFFERS , "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_RED_SIZE, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_GREEN_SIZE, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_BLUE_SIZE, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_ALPHA_SIZE, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_DEPTH_SIZE, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_STENCIL_SIZE, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_ACCUM_RED_SIZE, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_ACCUM_GREEN_SIZE, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_ACCUM_BLUE_SIZE, "%d");
    ShowGlxConfig(dpy, &visInfo, GLX_ACCUM_ALPHA_SIZE, "%d");
}

main(int argc, char **argv)
{
    Display *dpy;
    XWindowAttributes winAttr;
    Colormap cmap;
    VisualID visid;
    int scrNumber;
    char *p;
    XVisualInfo *visInfo;
    Visual *vis;

    ProgramName = argv[0];

    if ( (dpy = XOpenDisplay(NULL)) == NULL)
	Fatal("can't open display\n");

    for (argc--,argv++; argc; argc--, argv++) {
	if (strncmp(*argv, "vis=", 4) == 0) {
	    p = *argv + 4;
	    if (p[0] == '0' && p[1] == 'x')
		sscanf(p, "0x%x", &visid);
	    else
		visid = atoi(p);
	    glxShowConfig(dpy, visid);
	}
    }
}
