/*
 *      (c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              glxdisplay.c
 *
 *      DESCRIPTION
 *              handle gl display activity
 *		GL-DEPENDENT
 *		X-DEPENDENT
 *		OS-DEPENDENT
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/glxdisplay.c,v 1.10 1994/05/18 00:02:14 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <gl/glws.h>

#include "xdef.h"

#define	Rgb888CrBits		6
#define	Rgb888CbBits		6
#define	Rgb888YBits		6
#define	Rgb888IndexShift	2

#define	Rgb444CrBits		6
#define	Rgb444CbBits		6
#define	Rgb444YBits		6
#define	Rgb444IndexShift	1

#define MaskOf(nbits)	((1 << (nbits)) - 1)

typedef struct _glxInfo {
    boolean doubleBuffered;
    boolean hardwareZoom;
    short x2, y2;		/* GL screen coords of upper right corner */
    Window window;
    GLXconfig *conf;
    boolean glManaged[NRgbDepthTypes];
} GlxInfo, *GlxInfoPtr;

static GlxInfo GlxData;

static struct {
    char *name;
    boolean hardwareZoom;
    char *realName;
} FeatureMap[] = {
     { "GL4D-",		False,	"B or G" },
     { "GL4DGT-",	False,	"GT" },
     { "GL4DGTX-",	True,	"GTX" },
     { "GL4DVGX-",	True,	"VGX" },
     { "GL4DVGXT-",	True,	"VGXT" },
     { "GL4DRE-",	True,	"RealityEngine" },
     { "GL4DPI2-",	False,	"Personal Iris" },
     { "GL4DPIT-",	False,	"Personal Iris with Turbo Graphics" },
     { "GL4DPI-",	False,	"Personal Iris (early serial numbers)" },
     { "GL4DLG-",	False,	"Entry" },
     { "GL4DXG-",	False,	"XS" },
     { "GL4DXG-",	False,	"XS24" },
     { "GL4DXG-",	False,	"XZ" },
     { "GL4DXG-",	False,	"Elan" },
     { "GL4DXG-",	False,	"Extreme" },
     { "GL4DNP-",	False,	"Indy" },
     { NULL },
};

/*
**  Used to get a value out of the configuration structure returned by
**  GLXgetconfig.  Scan through the GLX config structure, and, when we
**  find the proper buffer-mode combination, return the argument relevant
**  for that buffer type.
*/
static ULong
GLXgetvalue(GLXconfig* conf, int buffer, int mode)
{
    int i;
    for (i = 0; conf[i].buffer; i++)
        if (conf[i].buffer == buffer && conf[i].mode == mode)
            return conf[i].arg;
    return 0;
}

/*
 * Dorky little helper function used to build up a GLXconfig array.
 */
static void
SetGlEntry (GLXconfig* ptr, int b, int m, int a, boolean find)
{
    if (! find) {
	ptr->buffer = b;
	ptr->mode = m;
	ptr->arg = a;
	return;
    }

    while (ptr->buffer) {
	if (ptr->buffer == b && ptr->mode == m) {
	    ptr->arg = a;
	    return;
	}
	ptr++;
    }
}

static void
GLXRefreshLongPixels(DisplayInfoPtr dpyInfo)
{
    GlxInfoPtr glx = &GlxData;

    lrectwrite(0, 0,				/* x1, y1 */
	       glx->x2, glx->y2,		/* x2, y2 */
	       (ULong *)dpyInfo->cvtu.ycToRgb->rgb);
    if (glx->doubleBuffered)
        swapbuffers();
}

static void
GLXRefreshWordPixels(DisplayInfoPtr dpyInfo)
{
    GlxInfoPtr glx = &GlxData;

    rectwrite(0, 0,				/* x1, y1 */
	      glx->x2, glx->y2,			/* x2, y2 */
	      (UShort *)dpyInfo->cvtu.ycToRgb->rgb);
    if (glx->doubleBuffered)
        swapbuffers();
}

static boolean
GLXCheckVisual(XInfo *xInfo, GLXconfig *initConf)
{
    VisualID vis;
    DisplayInfoPtr dpyInfo = xInfo->dpyInfo;
    GLXconfig *conf;
    boolean doubleBuffered;
    boolean success;
    int depth;
    CopyInfo *copier;
    GlxInfoPtr glx = &GlxData;

    conf = GLXgetconfig(xInfo->dpy, xInfo->decode.scrNumber, initConf);
    if (conf == NULL)
	return(False);

    vis = GLXgetvalue(conf, GLX_NORMAL, GLX_VISUAL);
    depth = GLXgetvalue(conf, GLX_NORMAL, GLX_BUFSIZE);
    doubleBuffered = GLXgetvalue(conf, GLX_NORMAL, GLX_DOUBLE);

    if (vis == xInfo->decode.visual->visualid) {
	copier = &dpyInfo->copiers[ dpyInfo->color ];
	switch(dpyInfo->color) {
	case Rgb444:
	    copier->fbDisplay = GLXRefreshWordPixels;
	    copier->winDisplay = GLXRefreshWordPixels;
	    glx->glManaged[ dpyInfo->color ] = True;
	    break;

	case Rgb888:
	    copier->fbDisplay = GLXRefreshLongPixels;
	    copier->winDisplay = GLXRefreshLongPixels;
	    glx->glManaged[ dpyInfo->color ] = True;
	    break;

	default:
	    free(conf);
	    return(False);
	}

	glx->doubleBuffered = doubleBuffered;
	ProgMsg("using GLX %d-bit %s buffered\n",
	    depth, doubleBuffered ? "double" : "single");

	copier->clipToScreen = False;
	copier->maxDirectConvertPix = ~0; /* we always convert offscreen */
	dpyInfo->hardwareZoom = glx->hardwareZoom;

	if (glx->window)
	    GLXunlink(xInfo->dpy, glx->window);
	if (glx->conf)
	    free(glx->conf);
	glx->conf = conf;
	glx->window = xInfo->decode.w;

	SetGlEntry(conf, GLX_NORMAL, GLX_WINDOW, glx->window, True);
	GLXlink(xInfo->dpy, conf);
	GLXwinset(xInfo->dpy, glx->window);
	success = True;
    } else {
	dpyInfo->hardwareZoom = False;
	free(conf);
	success = False;
    }

    switch(dpyInfo->color) {
    case GrayScale1: case GrayScale7: case GrayScale8:
	dpyInfo->cvtu.ycToGray->reflectVertical = success;
	break;
    case Rgb232: case Rgb332: case Rgb444:
    case Rgb888: case RgbC555: case RgbC666:
	dpyInfo->cvtu.ycToRgb->reflectVertical = success;
	break;
    default:
	break;	/* never happens */
    }

    return(success);
}

void
GLXSetBuffer(XInfo *xInfo)
{
    GlxInfoPtr glx = &GlxData;
    DisplayInfoPtr dpyInfo = xInfo->dpyInfo;
    CopyInfo *copier;

    if (! glx->glManaged[ dpyInfo->color ])
	return;

    copier = &dpyInfo->copiers[ dpyInfo->color ];

    copier->fb = dpyInfo->copyBuffer;
    copier->lineBytes = dpyInfo->copyBufferWidth;
}

void
GLXSetWinClip(XInfo *xInfo)
{
    GlxInfoPtr glx = &GlxData;
    DisplayInfoPtr dpyInfo = xInfo->dpyInfo;

    if (! glx->glManaged[ dpyInfo->color ])
	return;

    if (dpyInfo->doubleSize) {
	/* glx->x2 = (2*dpyInfo->pSeq->horizontalSize)-1; */
	glx->x2 = (2*MAX_PIX_X)-1;
	glx->y2 = (2*dpyInfo->pSeq->verticalSize)-1;
	if (glx->hardwareZoom)
	    rectzoom(2.0, 2.0);
    } else {
	/* glx->x2 = dpyInfo->pSeq->horizontalSize-1; */
	glx->x2 = MAX_PIX_X-1;
	glx->y2 = dpyInfo->pSeq->verticalSize-1;
	rectzoom(1.0, 1.0);
    }
    viewport(0, glx->x2, 0, glx->y2);
}

static boolean
IsVowel(char *p)
{
    return(*p == 'a'
        || *p == 'e'
        || *p == 'i'
        || *p == 'o'
        || *p == 'u'
	|| *p == 'A'
        || *p == 'E'
        || *p == 'I'
        || *p == 'O'
        || *p == 'U');
}

void
GLXInit(XInfo *xInfo)
{
    GLXconfig params[50];
    GLXconfig* next;
    DisplayInfoPtr dpyInfo = xInfo->dpyInfo;
    static boolean firstTime = True;

    if (firstTime) {
	char version[100];
	int i, length;

	firstTime = False;
	(void)gversion(version);
	for (i = 0; FeatureMap[i].name; i++) {
	    length = strlen(FeatureMap[i].name);
	    if (strncmp(FeatureMap[i].name, version, length) == 0) {
		Msg("Platform is a%s \"%s\" (%s hardware zoom)\n",
		    IsVowel(FeatureMap[i].realName) ? "n" : "",
		    FeatureMap[i].realName,
		    FeatureMap[i].hardwareZoom ? "with" : "no");
		GlxData.hardwareZoom = FeatureMap[i].hardwareZoom;
		break;
	    }
	}
	if (FeatureMap[i].name == NULL)
	    ProgMsg("Unknown SGI hardware\n");
    }

    /*
     * This builds an array in "params" that describes for GLXgetconfig(3G)
     * the type of GL drawing that will be done.  Ask first for RGB double
     * buffered.
     */
    next = params;
    SetGlEntry(next++, GLX_NORMAL, GLX_RGB, True, False);
    SetGlEntry(next++, GLX_NORMAL, GLX_DOUBLE, True, False);
    SetGlEntry(next, 0, 0, 0, 0); /* input to GLXgetconfig is null terminated */
    if (GLXCheckVisual(xInfo, params))
	return;

    /*
     * Now ask first for RGB single buffered.
     */
    next = params;
    SetGlEntry(next++, GLX_NORMAL, GLX_RGB, True, False);
    SetGlEntry(next++, GLX_NORMAL, GLX_DOUBLE, False, False);
    SetGlEntry(next, 0, 0, 0, 0);

    if (GLXCheckVisual(xInfo, params))
	return;

    /*
     * ask first for index double buffered.
     */
    next = params;
    SetGlEntry(next++, GLX_NORMAL, GLX_RGB, False, False);
    SetGlEntry(next++, GLX_NORMAL, GLX_DOUBLE, True, False);
    SetGlEntry(next, 0, 0, 0, 0);

    if (GLXCheckVisual(xInfo, params))
	return;

    /*
     * ask first for index single buffered.
     */
    next = params;
    SetGlEntry(next++, GLX_NORMAL, GLX_RGB, False, False);
    SetGlEntry(next++, GLX_NORMAL, GLX_DOUBLE, False, False);
    SetGlEntry(next, 0, 0, 0, 0);

    (void) GLXCheckVisual(xInfo, params);

    GlxData.x2 = dpyInfo->pSeq->horizontalSize-1;
    GlxData.y2 = dpyInfo->pSeq->verticalSize-1;
}

void
SgiFbInit(DisplayInfoPtr dpyInfo)
{
}
