/* $Header: /usr/local/src/nvrToMIT/RCS/halfpel.c,v 3.1 1993/10/22 23:53:29 russ Exp $ */

#define NeedFunctionPrototypes 1

typedef unsigned char UChar;

typedef struct {
    UChar full;
    UChar sign;
    UChar bit1;
    UChar bit0;
    UChar isHalf;
} BitSet;

typedef int (*Func)();
BitSet Bits[] = {
 { 0,0,1,0,  1 },
 { 0,0,1,1,  1 },
 { 0,1,0,1,  1 },
 { 0,1,1,0,  1 },
 { 1,0,0,1,  1 },
 { 1,0,1,1,  1 },
 { 1,1,0,1,  1 },
 { 1,1,1,1,  1 }

 { 0,0,0,0,  0 },
 { 0,0,0,1,  0 },
 { 0,1,0,0,  0 },
 { 0,1,1,1,  0 },
 { 1,0,0,0,  0 },
 { 1,0,1,0,  0 },
 { 1,1,1,0,  0 },
 { 1,1,0,0,  0 },
};

#define NBitSets (sizeof(Bits) / sizeof(BitSet))

main()
{
    UChar fullPos, signPos, bit1Pos, bit0Pos;		/* positions */
    
    for (fullPos = 0; fullPos <= 3; fullPos++) {
	for (signPos = 0; signPos <= 3; signPos++) {
	    if (signPos == fullPos)
		continue;
	    for (bit1Pos = 0; bit1Pos <= 3; bit1Pos++) {
		if (bit1Pos == fullPos || bit1Pos == signPos)
		    continue;
		for (bit0Pos = 0; bit0Pos <= 3; bit0Pos++) {
		    if (bit0Pos == fullPos
		     || bit0Pos == signPos
		     || bit0Pos == bit1Pos)
			continue;
		    CheckOperandOrder(fullPos, signPos, bit1Pos, bit0Pos);
		}
	    }
	}
    }
}

OpAnd(val1, val2) { return val1 & val2; }
OpOr(val1, val2)  { return val1 | val2; }
OpXor(val1, val2) { return val1 ^ val2; }

Func OpList[] = {
    OpAnd, OpOr, OpXor
};

#define NOps (sizeof(OpList) / sizeof(Func))

/*
 * (((val1 op val2) op val3) op val4)
 */
Expr(int val1, Func op1, int val2, Func op2, int val3, Func op3, int val4)
{
    UChar value;
    value = (*op1)(val1, val2);
    value = (*op2)(value, val3);
    value = (*op3)(value, val4);
    return (value);
}

CheckOperandOrder(UChar fullPos, int signPos, int bit1Pos, int bit0Pos)
    /* UChar fullPos, signPos, bit1Pos, bit0Pos;		positions */
{
    UChar value;
    UChar compliments;
    int op1idx, op2idx, op3idx;

    for (op1idx = 0; op1idx < NOps; op1idx++) {
	for (op2idx = 0; op2idx < NOps; op2idx++) {
	    for (op3idx = 0; op3idx < NOps; op3idx++) {
		for (compliments = 0; compliments <= 0xf; compliments++) {
		    CheckOpOrder(compliments,
			 fullPos, signPos, bit1Pos, bit0Pos,
			 OpList[op1idx], OpList[op2idx], OpList[op3idx]);
		}
	    }
	}
    }
}

static void PrintOperand(int which, int fullPos, int signPos, int bit1Pos,
			 int bit0Pos, int compliments)
{
    if (which == fullPos)
	printf("%sfullPel", compliments & 1 ? "!" : "");
    if (which == signPos)
	printf("%ssign",    compliments & 2 ? "!" : "");
    if (which == bit1Pos)
	printf("%sbit1",    compliments & 4 ? "!" : "");
    if (which == bit0Pos)
	printf("%sbit0", compliments & 8 ? "!" : "");
}

static void PrintOp(Func op)
{
    if (op == OpAnd)
	printf(" & ");
    if (op == OpOr)
	printf(" | ");
    if (op == OpXor)
	printf(" ^ ");
}

CheckOpOrder(UChar compliments, UChar fullPos, UChar signPos, UChar bit1Pos,
	     UChar bit0Pos, Func op1, Func op2, Func op3)
    /* UChar fullPos, signPos, bit1Pos, bit0Pos;	positions */
{
    int set, position;
    UChar operand[4];
    UChar nCorrect;
    UChar value;

    printf("(((");
    PrintOperand(0, fullPos, signPos, bit1Pos, bit0Pos, compliments);
    PrintOp(op1);
    PrintOperand(1, fullPos, signPos, bit1Pos, bit0Pos, compliments);
    printf(")");
    PrintOp(op2);
    PrintOperand(2, fullPos, signPos, bit1Pos, bit0Pos, compliments);
    printf(")");
    PrintOp(op2);
    PrintOperand(3, fullPos, signPos, bit1Pos, bit0Pos, compliments);
    printf("):\t");

    nCorrect = 0;
    for (set = 0; set < NBitSets; set++) {
	for (position = 0; position < 4; position++) {
	    if (fullPos == position)
		operand[position] =
		    compliments & 1 ? ! Bits[set].full : Bits[set].full;
	    if (signPos == position)
		operand[position] =
		    compliments & 2 ? ! Bits[set].sign : Bits[set].sign;
	    if (bit1Pos == position)
		operand[position] =
		    compliments & 4 ? ! Bits[set].bit1 : Bits[set].bit1;
	    if (bit0Pos == position)
		operand[position] =
		    compliments & 8 ? ! Bits[set].bit0 : Bits[set].bit0;
	}

	value = Expr(operand[0], op1, operand[1],
			op2, operand[2],
			    op3, operand[3]);
	if (value == Bits[set].isHalf)
	    nCorrect++;
    }
    if (nCorrect == NBitSets)
	printf(" * ");
    printf("%d of %d\n", nCorrect, NBitSets);
}
