/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	huffman.h - support for huffman codes and processing
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/huffman.h,v 4.2 1994/01/27 23:20:22 toddb Exp $
 */
#ifndef _HUFFMAN_H_
#define _HUFFMAN_H_
/*
 * These macros extract data from a single byte for
 * huffman codes associated with the macroblock address increment.
 */
#define MBAIBits(value)			((value) >> 6)
#define MBAIncrementValue(value)	KeepBits(6, value)

#define MBAI_LOOKUP_BITS		11
#define MBAI_LOOKUP_CELLS		(1 << MBAI_LOOKUP_BITS)
#define MBAI_INIT_CELLS			33

#define MBTBits(value)			((value) >> 5)
#define MBTypeValue(value)		KeepBits(5, value)

#define MBT_B_LOOKUP_BITS		6
#define MBT_B_LOOKUP_CELLS		(1 << MBT_B_LOOKUP_BITS)
#define MBT_B_INIT_CELLS		11

#define MBT_P_LOOKUP_BITS		6
#define MBT_P_LOOKUP_CELLS		(1 << MBT_P_LOOKUP_BITS)
#define MBT_P_INIT_CELLS		7

#define MotionValue(value)		(-16 + KeepBits(6, value)) /* bias 16 */
#define MotionBits(value)		((value) >> 6)

#define MOTION_LOOKUP_BITS		11
#define MOTION_LOOKUP_CELLS		(1 << MOTION_LOOKUP_BITS)
#define MOTION_INIT_CELLS		33

#define CodedBlockBits(value)		((value) >> 6)
#define CodedBlockValue(value)		KeepBits(6, value)

#define CODED_BLOCK_LOOKUP_BITS		9
#define CODED_BLOCK_LOOKUP_CELLS	(1 << MOTION_LOOKUP_BITS)
#define CODED_BLOCK_INIT_CELLS		63

#define SizeYCBits(value)		((value) >> 4)
#define SizeYCValue(value)		KeepBits(4, value)

#define SIZE_LUMA_LOOKUP_BITS		7
#define SIZE_CHROMA_LOOKUP_BITS		8
#define SIZE_LUMA_LOOKUP_CELLS		(1 << SIZE_LUMA_LOOKUP_BITS)
#define SIZE_CHROMA_LOOKUP_CELLS	(1 << SIZE_CHROMA_LOOKUP_BITS)
#define SIZE_YC_INIT_CELLS		9

#define DCTCoeffBits(value)		((value) >> 11)
#define DCTCoeffRun(value)		KeepBits(5, (value) >> 6)
#define DCTCoeffLevel(value)		KeepBits(6, value)

#define DCT_COEFF1_LOOKUP_BITS		10
#define DCT_COEFF2_LOOKUP_BITS		9
#define DCT_COEFF1_LOOKUP_CELLS		(1 << DCT_COEFF1_LOOKUP_BITS)
#define DCT_COEFF2_LOOKUP_CELLS		(1 << DCT_COEFF2_LOOKUP_BITS)
#define DCT_COEFF1_INIT_CELLS		(34 + 2 /* flag entries */)
#define DCT_COEFF2_INIT_CELLS		80

/*
 * This structure is used to hold mappings of huffman codes to values.
 * These are rearranged and expanded at run time to require a single
 * memory access to discover the associated value.
 *
 * At run time, initialization tables contain the huffman code and
 * a value (perhaps comprised of several small values).  These are
 * sorted, and expanded so that the maximum number of huffman code
 * bits can be used as an index into this map.
 */
typedef struct _HuffCodeMap {
    long	code;		/* the huffman code */
    ULong	value;
} HuffCodeMap;

/*
 * this holds all the initialized values.
 */
typedef struct _HuffLookup {
    UShort   mbAddressInc[ MBAI_LOOKUP_CELLS ];
    UChar    mb_B_Type[ MBT_B_LOOKUP_CELLS ];
    UChar    mb_P_Type[ MBT_P_LOOKUP_CELLS ];
    UShort   motionVector[ MOTION_LOOKUP_CELLS ];
    UShort   codedBlock[ CODED_BLOCK_LOOKUP_CELLS ];
    UChar    sizeLuminance[ SIZE_LUMA_LOOKUP_CELLS ];
    UChar    sizeChrominance[ SIZE_CHROMA_LOOKUP_CELLS ];
    UShort   dctCoeff1[ DCT_COEFF1_LOOKUP_CELLS ];
    UShort   dctCoeff2[ DCT_COEFF2_LOOKUP_CELLS ];
} HuffLookupInfo;

extern HuffCodeMap MBAI_InitData[ MBAI_INIT_CELLS ];
extern HuffCodeMap MBT_B_InitData[ MBT_B_INIT_CELLS ];
extern HuffCodeMap MBT_P_InitData[ MBT_P_INIT_CELLS ];
extern HuffCodeMap MotionVectorInitData[ MOTION_INIT_CELLS ];
extern HuffCodeMap CodedBlockInitData[ CODED_BLOCK_INIT_CELLS ];
extern HuffCodeMap SizeLuminanceInitData[ SIZE_YC_INIT_CELLS ];
extern HuffCodeMap SizeChrominanceInitData[ SIZE_YC_INIT_CELLS ];
extern HuffCodeMap DCTCoeff1InitData[ DCT_COEFF1_INIT_CELLS ];
extern HuffCodeMap DCTCoeff2InitData[ DCT_COEFF2_INIT_CELLS ];

extern HuffLookupInfo HuffLookup;

#endif /* _FOO_H_ */
