/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	huffman.h - support for huffman codes and processing
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/idct.h,v 4.6 1994/12/17 12:58:16 russ Exp $
 */

#define NMotionAndClamp	64
#define MotionAndClampToIndex(halfPixel, pFuture, pPast)	\
    (halfPixel | ((pFuture != 0) << 4) | ((pPast != NULL) << 5))

/*
 * These two Zero macros are called in the IDct{Row,Col} functions
 * that are infrequent.  Hence, these are used to fill in enough of
 * the elements to call some other IDct{Row,Col} function.  Note
 * that the msb corresponds to the left-most element.
 */
#define ZeroRow(ptr, row)					\
    OctWordP(ptr)[ row*4+0 ] = zero;				\
    OctWordP(ptr)[ row*4+1 ] = zero;				\
    OctWordP(ptr)[ row*4+2 ] = zero;				\
    OctWordP(ptr)[ row*4+3 ] = zero;				\

/*
 * This is used to build a bitmask whose arguments correspond
 * left-to-right with the cells in a dct row, or top-to-bottom
 * with the cells in a dct column
 */
#define BitsFromArgs(b0,b1,b2,b3,b4,b5,b6,b7)			\
    ((b7|(b6<<1)|(b5<<2)|(b4<<3)|(b3<<4)|(b2<<5)|(b1<<6)|(b0<<7)))

#define ArgsFromBits(bits)					\
    (bits>>7)&1,						\
    (bits>>6)&1,						\
    (bits>>5)&1,						\
    (bits>>4)&1,						\
    (bits>>3)&1,						\
    (bits>>2)&1,						\
    (bits>>1)&1,						\
    newBits&1




/*
 * Macros to help keep track of which cells in the block are occupied.
 */
#define MarkCellNZ(word1, word2, index)				\
    {								\
	if (index >= 32)					\
	    word1 |= 1<<(index - 32);				\
	else							\
	    word2 |= 1<<index;					\
    }

#define ShiftCellMarks(word1, word2)				\
    {								\
	word2 = (word2 >> 8) | (word1 << 24);			\
	word1 >>= 8;						\
    }

/*
 * This macro is to load coefficients for the second pass 2d IDCT.
 */
#define CellInCol(pCell, row, col)  (pCell)[(row)*MAX_X+(col)]

/*
 * For intra-coded blocks, there is no future or past.
 * Hence, the final three arguments are ignored.
 */

#define IntraClampOp(storage, assignOp, value)			\
    {								\
	if ((value = RoundToInteger(value)) & ~255) {		\
	    if (value > 255)					\
		value = 255;					\
	    else						\
		value = 0;					\
	}							\
	storage assignOp value;					\
    }

#define IntraClamp(storage, value)	IntraClampOp(storage, =, value)

/*
 * For non intra-coded blocks, the (signed) difference is added
 * to the reference pel (forward, back, or both) to arrive at a
 * final value that is stored
 */
#define NonIntraClampOp(storage, assignOp, reference, value)	\
    {								\
	value = RoundToInteger(value);				\
	if ((value += (reference)) & ~255) {			\
	    if (value > 255)					\
		value = 255;					\
	    else						\
		value = 0;					\
	}							\
	storage assignOp value;					\
    }

#define NonIntraClamp(storage, reference, value)		\
   NonIntraClampOp(storage, =, reference, value)

/*
 * Round and clamp to +-255 for use with FastNonIntraClamp
 */
#define PreNonIntraClamp(value)					\
    ((RoundToInteger(value) & ~255)				\
      ? (RoundToInteger(value) > 255				\
	  ? 255 :						\
	    (RoundToInteger(value) < -256			\
	      ? -256						\
	      : RoundToInteger(value)				\
	    )							\
	)							\
      : RoundToInteger(value)					\
    )

/*
 * This is for when we know the value is already rounded and
 * clamped.
 */
#define FastNonIntraClamp(storage, assignOp, reference, value)		\
	storage assignOp SignedClamp[ value + (reference) ];


/*
 * This little ditty perform the inverse dct using integer arithmetic.
 * Adapted from code written by Bob Eifrig, TekLabs.
 *
 * For reference
 * see B. G. Lee, IEEE Transactions on Acoustics, Speech, and Signal
 * Processing, Vol 32, No 6, (Dec 1984) pgs 1243-1245.
 */

/*
 * This is designed to generate code used as the body of a function.
 */
#define IDctRow(in0,in1,in2,in3,in4,in5,in6,in7,out) 			\
    int p0,p1,p2,p3,p4,p5,p6,p7;					\
    int q0,q1,q2,q3,q4,q5,q6,q7;					\
    int tmp1, tmp2;							\
    OctWord result;							\
									\
    q0 = in0;								\
    HalfSqrt2_2_Times(p0, q0, tmp1);					\
    q1 = in1;								\
    p1 = q1 >> 1;			/* times .5 */			\
    q2 = in2;								\
    p2 = q2 >> 1;			/* times .5 */			\
    q3 = in3;								\
    p3 = (q0 = q1 + q3) >> 1;	/* times .5 */				\
    q4 = in4;								\
			 HalfSqrt2_2_Times(p4, q4, tmp1);		\
    q5 = in5;								\
    tmp1 = q5 + q3;      HalfSqrt2_2_Times(p5, tmp1, tmp2);		\
    q6 = in6;								\
    tmp1 = q2 + q6;      HalfSqrt2_2_Times(p6, tmp1, tmp2);		\
    q7 = in7;								\
    tmp1 = q0 + q5 + q7; HalfSqrt2_2_Times(p7, tmp1, tmp2);		\
									\
    q0 = p0 + p4;							\
    q1 = p1 + p5;							\
    tmp1 = p2 + p6; HalfCos2Pi_16_Times(q2, tmp1, tmp2);		\
    tmp1 = p3 + p7; HalfCos2Pi_16_Times(q3, tmp1, tmp2);		\
    q4 = p0 - p4;							\
    q5 = p1 - p5;							\
    tmp1 = p2 - p6; HalfCos6Pi_16_Times(q6, tmp1, tmp2);		\
    tmp1 = p3 - p7; HalfCos6Pi_16_Times(q7, tmp1, tmp2);		\
									\
    p0 = q0 + q2;							\
    tmp1 = q1 + q3; HalfCosPi_16_Times(p1, tmp1, tmp2);			\
    p2 = q4 + q6;							\
    tmp1 = q5 + q7; HalfCos3Pi_16_Times(p3, tmp1, tmp2);		\
									\
    result.l.high = p0 + p1;						\
    result.l.low  = p2 + p3;						\
    OctWordP(out)[0] = result;						\
    result.l.high = p2 - p3;						\
    result.l.low  = p0 - p1;						\
    OctWordP(out)[3] = result;						\
									\
    p4 = q4 - q6;							\
    tmp1 = q5 - q7; HalfCos5Pi_16_Times(p5, tmp1, tmp2);		\
    p6 = q0 - q2;							\
    tmp1 = q1 - q3; HalfCos7Pi_16_Times(p7, tmp1, tmp2);		\
									\
    result.l.high = p4 + p5;						\
    result.l.low  = p6 + p7;						\
    OctWordP(out)[1] = result;						\
    result.l.high = p6 - p7;						\
    result.l.low  = p4 - p5;						\
    OctWordP(out)[2] = result;

/*
 * This is designed to generate code used as the body of a function.
 * The function must have the following arguments.
 * func(ptr)
 *     long *ptr;
 * 
 */
#define IDctCol(in0, in1, in2, in3, in4, in5, in6, in7, ptr)		\
    int p0,p1,p2,p3,p4,p5,p6,p7;					\
    int q0,q1,q2,q3,q4,q5,q6,q7;					\
    int tmp1, tmp2, result;						\
    int i;								\
									\
    for (i = 8; i > 0; i--) {						\
	q0 = in0;							\
	HalfSqrt2_2_Times(p0, q0, tmp1);				\
	q1 = in1;							\
	p1 = q1 >> 1;			/* times .5 */			\
	q2 = in2;							\
	p2 = q2 >> 1;			/* times .5 */			\
	q3 = in3;							\
	p3 = (q0 = q1 + q3) >> 1;	/* times .5 */			\
	q4 = in4;							\
			     HalfSqrt2_2_Times(p4, q4, tmp1);		\
	q5 = in5;							\
	tmp1 = q5 + q3;      HalfSqrt2_2_Times(p5, tmp1, tmp2);		\
	q6 = in6;							\
	tmp1 = q2 + q6;      HalfSqrt2_2_Times(p6, tmp1, tmp2);		\
	q7 = in7;							\
	tmp1 = q0 + q5 + q7; HalfSqrt2_2_Times(p7, tmp1, tmp2);		\
									\
	q0 = p0 + p4;							\
	q1 = p1 + p5;							\
	tmp1 = p2 + p6; HalfCos2Pi_16_Times(q2, tmp1, tmp2);		\
	tmp1 = p3 + p7; HalfCos2Pi_16_Times(q3, tmp1, tmp2);		\
	q4 = p0 - p4;							\
	q5 = p1 - p5;							\
	tmp1 = p2 - p6; HalfCos6Pi_16_Times(q6, tmp1, tmp2);		\
	tmp1 = p3 - p7; HalfCos6Pi_16_Times(q7, tmp1, tmp2);		\
									\
	p0 = q0 + q2;							\
	tmp1 = q1 + q3; HalfCosPi_16_Times(p1, tmp1, tmp2);		\
	p6 = q0 - q2;							\
	tmp1 = q1 - q3; HalfCos7Pi_16_Times(p7, tmp1, tmp2);		\
	p2 = q4 + q6;							\
	tmp1 = q5 + q7; HalfCos3Pi_16_Times(p3, tmp1, tmp2);		\
	p4 = q4 - q6;							\
	tmp1 = q5 - q7; HalfCos5Pi_16_Times(p5, tmp1, tmp2);		\
									\
	CellInCol(ptr, 0, 0, 8) = p0 + p1;				\
	CellInCol(ptr, 7, 0, 8) = p0 - p1;				\
	CellInCol(ptr, 1, 0, 8) = p2 + p3;				\
	CellInCol(ptr, 6, 0, 8) = p2 - p3;				\
	CellInCol(ptr, 2, 0, 8) = p4 + p5;				\
	CellInCol(ptr, 5, 0, 8) = p4 - p5;				\
	CellInCol(ptr, 3, 0, 8) = p6 + p7;				\
	CellInCol(ptr, 4, 0, 8) = p6 - p7;				\
	ptr++;								\
    }

/*
 * This declares the needed local variables, does the basic 1d dct
 * and leaves (almost) final values in p0-p7;
 */
#define _IDctDeclarations						\
    int p0,p1,p2,p3,p4,p5,p6,p7;					\
    int q0,q1,q2,q3,q4,q5,q6,q7;					\
    int tmp1, tmp2

#define _IDctBody(in0, in1, in2, in3, in4, in5, in6, in7)		\
    _IDctDeclarations;							\
    _IDctCode(in0, in1, in2, in3, in4, in5, in6, in7)

#define _IDctCode(in0, in1, in2, in3, in4, in5, in6, in7)		\
    q0 = in0;								\
    HalfSqrt2_2_Times(p0, q0, tmp1);					\
    q1 = in1;								\
    p1 = q1 >> 1;			/* times .5 */			\
    q2 = in2;								\
    p2 = q2 >> 1;			/* times .5 */			\
    q3 = in3;								\
    p3 = (q0 = q1 + q3) >> 1;	/* times .5 */				\
    q4 = in4;								\
			 HalfSqrt2_2_Times(p4, q4, tmp1);		\
    q5 = in5;								\
    tmp1 = q5 + q3;      HalfSqrt2_2_Times(p5, tmp1, tmp2);		\
    q6 = in6;								\
    tmp1 = q2 + q6;      HalfSqrt2_2_Times(p6, tmp1, tmp2);		\
    q7 = in7;								\
    tmp1 = q0 + q5 + q7; HalfSqrt2_2_Times(p7, tmp1, tmp2);		\
									\
    q0 = p0 + p4;							\
    q1 = p1 + p5;							\
    tmp1 = p2 + p6; HalfCos2Pi_16_Times(q2, tmp1, tmp2);		\
    tmp1 = p3 + p7; HalfCos2Pi_16_Times(q3, tmp1, tmp2);		\
    q4 = p0 - p4;							\
    q5 = p1 - p5;							\
    tmp1 = p2 - p6; HalfCos6Pi_16_Times(q6, tmp1, tmp2);		\
    tmp1 = p3 - p7; HalfCos6Pi_16_Times(q7, tmp1, tmp2);		\
									\
    p0 = q0 + q2;							\
    tmp1 = q1 + q3; HalfCosPi_16_Times(p1, tmp1, tmp2);			\
    p6 = q0 - q2;							\
    tmp1 = q1 - q3; HalfCos7Pi_16_Times(p7, tmp1, tmp2);		\
    p2 = q4 + q6;							\
    tmp1 = q5 + q7; HalfCos3Pi_16_Times(p3, tmp1, tmp2);		\
    p4 = q4 - q6;							\
    tmp1 = q5 - q7; HalfCos5Pi_16_Times(p5, tmp1, tmp2)

/*
 * These defines are consistent with _IDctBody and provide the
 * final values: X0, ... X7
 */
#define DctX0	(p0 + p1)
#define DctX1	(p2 + p3)
#define DctX2	(p4 + p5)
#define DctX3	(p6 + p7)
#define DctX4	(p6 - p7)
#define DctX5	(p4 - p5)
#define DctX6	(p2 - p3)
#define DctX7	(p0 - p1)

/*
 * This calculates the constant value for the 8x8 idct when only
 * the dc constant is present.  Essentially it is
 * dctRecon[0][0] * ((sqrt(2)/2) ** 2)
 * The constant turns out to be 1/8.
 */
#define IDctDc(ptr)	(*(ptr) >> 3)

extern void NoMotionAndClamp();
extern void MotionHalfVertAndClamp();
extern void MotionHalfHorizAndClamp();
extern void MotionHalfAndClamp();
extern void MotionAndClamp();
extern void FwdBackMotionAndClamp();

extern void NoMotionAndClamp0Dc(/* UChar *, long, long, long, long, long */);
extern void MotionHalfVertAndClamp0Dc(/* UChar *, long, UChar *, long, long */);
extern void MotionHalfHorizAndClamp0Dc(/* UChar *,long,UChar *,long,long */);
extern void MotionHalfAndClamp0Dc(/* UChar *, long, UChar *, long, long */);
extern void MotionAndClamp0Dc(/* UChar *, long, UChar *, long, long */);
extern void FwdBackMotionAndClamp0Dc(/* UChar*,long,UChar*,UChar*,long,long */);

extern void IllegalMotionAndClamp();

extern void (*MotionAndClampTable[64])();
extern void (*MotionAndClamp0DcTable[64])();

/*
 * These are specifically optimized table entries for the idct tables.
 */
extern ULong IDctRow00000000();
extern void IDctCol00000000();
