/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	macMotion.c - mac - specific implementation of motion compensation
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mac-motion.c,v 1.3 1994/04/12 02:01:00 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"
#include "idct.h"
#include "motion.h"
#include "machdep.h"

void NoMotionAndClamp0Dc(MacroBlockInfo *pMb, long in, int row)
{
    long clamped;
    register ULong finalHigh, finalLow;
    register UChar *out = pMb->pTo;

    IntraClamp(clamped, in);
    finalLow = finalHigh =
	(clamped << 24) | (clamped << 16) | (clamped << 8) | clamped;
    StoreTwoLongs(out+MAX_X*0, finalHigh, finalLow);
    StoreTwoLongs(out+MAX_X*1, finalHigh, finalLow);
    StoreTwoLongs(out+MAX_X*2, finalHigh, finalLow);
    StoreTwoLongs(out+MAX_X*3, finalHigh, finalLow);
    StoreTwoLongs(out+MAX_X*4, finalHigh, finalLow);
    StoreTwoLongs(out+MAX_X*5, finalHigh, finalLow);
    StoreTwoLongs(out+MAX_X*6, finalHigh, finalLow);
    StoreTwoLongs(out+MAX_X*7, finalHigh, finalLow);
}

void NoMotionAndClamp(MacroBlockInfo *pMb, int row)
{
    /*
     * This is done in the idct function.
     */
    Fatal("NoMotionAndClamp: bad call!\n");
}

void MotionHalfVertAndClamp0Dc(MacroBlockInfo *pMb, long in)
{
    long value, i;
    register ULong finalHigh, finalLow;
    UChar *pTime = pMb->pFuture;
    UChar *out = pMb->pTo;

    /*
     * Process one row at a time
     */
    in = PreNonIntraClamp(in);
    for (i = 8; i > 0; i--) {
	value = in;
	MotionHalfVert(FastNonIntraClamp, finalHigh,  =, value, pTime);
	value = in; finalHigh <<= 8;
	MotionHalfVert(FastNonIntraClamp, finalHigh, |=, value, pTime+1);
	value = in; finalHigh <<= 8;
	MotionHalfVert(FastNonIntraClamp, finalHigh, |=, value, pTime+2);
	value = in; finalHigh <<= 8;
	MotionHalfVert(FastNonIntraClamp, finalHigh, |=, value, pTime+3);
	value = in;
	MotionHalfVert(FastNonIntraClamp, finalLow,  =, value, pTime+4);
	value = in; finalLow <<= 8;
	MotionHalfVert(FastNonIntraClamp, finalLow, |=, value, pTime+5);
	value = in; finalLow <<= 8;
	MotionHalfVert(FastNonIntraClamp, finalLow, |=, value, pTime+6);
	value = in; finalLow <<= 8;
	MotionHalfVert(FastNonIntraClamp, finalLow, |=, value, pTime+7);
	StoreTwoLongs(out, finalHigh, finalLow);
	out += MAX_X;
	pTime += MAX_X;
    }
}

void MotionHalfVertAndClamp(MacroBlockInfo *pMb, int row)
{
    UChar *pTime;
    register ULong finalHigh, finalLow;
    short *ptr = (short *)&pMb->dctRecon[8*row];
    long in;

    pTime = pMb->pFuture+row*MAX_X;

    in = ptr[0];
    MotionHalfVert(FastNonIntraClamp, finalHigh,  =, in, pTime);
    in = ptr[1]; finalHigh <<= 8;
    MotionHalfVert(FastNonIntraClamp, finalHigh, |=, in, pTime+1);
    in = ptr[2]; finalHigh <<= 8;
    MotionHalfVert(FastNonIntraClamp, finalHigh, |=, in, pTime+2);
    in = ptr[3]; finalHigh <<= 8;
    MotionHalfVert(FastNonIntraClamp, finalHigh, |=, in, pTime+3);

    in = ptr[4];
    MotionHalfVert(FastNonIntraClamp, finalLow,  =, in, pTime+4);
    in = ptr[5]; finalLow <<= 8;
    MotionHalfVert(FastNonIntraClamp, finalLow, |=, in, pTime+5);
    in = ptr[6]; finalLow <<= 8;
    MotionHalfVert(FastNonIntraClamp, finalLow, |=, in, pTime+6);
    in = ptr[7]; finalLow <<= 8;
    MotionHalfVert(FastNonIntraClamp, finalLow, |=, in, pTime+7);

    StoreTwoLongs(pMb->pTo+row*MAX_X, finalHigh, finalLow);
}

void MotionHalfHorizAndClamp0Dc(MacroBlockInfo *pMb, long in)
{
    long value, i;
    register ULong finalHigh, finalLow;
    UChar *pTime = pMb->pFuture;
    UChar *out = pMb->pTo;

    /*
     * Process one row at a time
     */
    in = PreNonIntraClamp(in);
    for (i = 8; i > 0; i--) {
	value = in;
	MotionHalfHoriz(FastNonIntraClamp, finalHigh, =, value, pTime)
	value = in; finalHigh <<= 8;
	MotionHalfHoriz(FastNonIntraClamp, finalHigh, |=, value, pTime+1)
	value = in; finalHigh <<= 8;
	MotionHalfHoriz(FastNonIntraClamp, finalHigh, |=, value, pTime+2)
	value = in; finalHigh <<= 8;
	MotionHalfHoriz(FastNonIntraClamp, finalHigh, |=, value, pTime+3)
	value = in;
	MotionHalfHoriz(FastNonIntraClamp, finalLow,  =, value, pTime+4)
	value = in; finalLow <<= 8;
	MotionHalfHoriz(FastNonIntraClamp, finalLow, |=, value, pTime+5)
	value = in; finalLow <<= 8;
	MotionHalfHoriz(FastNonIntraClamp, finalLow, |=, value, pTime+6)
	value = in; finalLow <<= 8;
	MotionHalfHoriz(FastNonIntraClamp, finalLow, |=, value, pTime+7)
	StoreTwoLongs(out, finalHigh, finalLow);
	out += MAX_X;
	pTime += MAX_X;
    }
}

void MotionHalfHorizAndClamp(MacroBlockInfo *pMb, int row)
{
    register ULong finalHigh, finalLow;
    UChar *pTime;
    short *ptr = (short *)&pMb->dctRecon[8*row];
    long in;

    pTime = pMb->pFuture+row*MAX_X;

    in = ptr[0];
    MotionHalfHoriz(FastNonIntraClamp, finalHigh,  =, in, pTime);
    in = ptr[1]; finalHigh <<= 8;
    MotionHalfHoriz(FastNonIntraClamp, finalHigh, |=, in, pTime+1);
    in = ptr[2]; finalHigh <<= 8;
    MotionHalfHoriz(FastNonIntraClamp, finalHigh, |=, in, pTime+2);
    in = ptr[3]; finalHigh <<= 8;
    MotionHalfHoriz(FastNonIntraClamp, finalHigh, |=, in, pTime+3);

    in = ptr[4];
    MotionHalfHoriz(FastNonIntraClamp, finalLow,   =, in, pTime+4);
    in = ptr[5]; finalLow <<= 8;
    MotionHalfHoriz(FastNonIntraClamp, finalLow,  |=, in, pTime+5);
    in = ptr[6]; finalLow <<= 8;
    MotionHalfHoriz(FastNonIntraClamp, finalLow,  |=, in, pTime+6);
    in = ptr[7]; finalLow <<= 8;
    MotionHalfHoriz(FastNonIntraClamp, finalLow,  |=, in, pTime+7);

    StoreTwoLongs(pMb->pTo+row*MAX_X, finalHigh, finalLow);
}

void MotionHalfAndClamp0Dc(MacroBlockInfo *pMb, long in)
{
    long value, i;
    register ULong finalHigh, finalLow;
    UChar *pTime = pMb->pFuture;
    UChar *out = pMb->pTo;

    /*
     * Process one row at a time
     */
    in = PreNonIntraClamp(in);
    for (i = 8; i > 0; i--) {
	value = in;
	MotionHalf(FastNonIntraClamp, finalHigh, =, value, pTime)
	value = in; finalHigh <<= 8;
	MotionHalf(FastNonIntraClamp, finalHigh, |=, value, pTime+1)
	value = in; finalHigh <<= 8;
	MotionHalf(FastNonIntraClamp, finalHigh, |=, value, pTime+2)
	value = in; finalHigh <<= 8;
	MotionHalf(FastNonIntraClamp, finalHigh, |=, value, pTime+3)
	value = in;
	MotionHalf(FastNonIntraClamp, finalLow,   =, value, pTime+4)
	value = in; finalLow <<= 8;
	MotionHalf(FastNonIntraClamp, finalLow,  |=, value, pTime+5)
	value = in; finalLow <<= 8;
	MotionHalf(FastNonIntraClamp, finalLow,  |=, value, pTime+6)
	value = in; finalLow <<= 8;
	MotionHalf(FastNonIntraClamp, finalLow,  |=, value, pTime+7)
	StoreTwoLongs(out, finalHigh, finalLow);
	out += MAX_X;
	pTime += MAX_X;
    }
}

void MotionHalfAndClamp(MacroBlockInfo *pMb, int row)
{
    register ULong finalHigh, finalLow;
    UChar *pTime;
    short *ptr = (short *)&pMb->dctRecon[8*row];
    long in;

    pTime = pMb->pFuture+row*MAX_X;

    in = ptr[0];
    MotionHalf(FastNonIntraClamp, finalHigh,  =, in, pTime);
    in = ptr[1]; finalHigh <<= 8;
    MotionHalf(FastNonIntraClamp, finalHigh, |=, in, pTime+1);
    in = ptr[2]; finalHigh <<= 8;
    MotionHalf(FastNonIntraClamp, finalHigh, |=, in, pTime+2);
    in = ptr[3]; finalHigh <<= 8;
    MotionHalf(FastNonIntraClamp, finalHigh, |=, in, pTime+3);

    in = ptr[4];
    MotionHalf(FastNonIntraClamp, finalLow,   =, in, pTime+4);
    in = ptr[5]; finalLow <<= 8;
    MotionHalf(FastNonIntraClamp, finalLow,  |=, in, pTime+5);
    in = ptr[6]; finalLow <<= 8;
    MotionHalf(FastNonIntraClamp, finalLow,  |=, in, pTime+6);
    in = ptr[7]; finalLow <<= 8;
    MotionHalf(FastNonIntraClamp, finalLow,  |=, in, pTime+7);

    StoreTwoLongs(pMb->pTo+row*MAX_X, finalHigh, finalLow);
}

void MotionAndClamp0Dc(MacroBlockInfo *pMb, long in)
{
    long value, i;
    register ULong finalHigh, finalLow;
    UChar *pTime = pMb->pFuture;
    UChar *out = pMb->pTo;

    /*
     * Process one row at a time
     */
    in = PreNonIntraClamp(in);
    for (i = 8; i > 0; i--) {
	value = in;
	FastNonIntraClamp(finalHigh, =, pTime[0], value);
	value = in; finalHigh <<= 8;
	FastNonIntraClamp(finalHigh, |=, pTime[1], value);
	value = in; finalHigh <<= 8;
	FastNonIntraClamp(finalHigh, |=, pTime[2], value);
	value = in; finalHigh <<= 8;
	FastNonIntraClamp(finalHigh, |=, pTime[3], value);
	value = in;
	FastNonIntraClamp(finalLow, =, pTime[4], value);
	value = in; finalLow <<= 8;
	FastNonIntraClamp(finalLow, |=, pTime[5], value);
	value = in; finalLow <<= 8;
	FastNonIntraClamp(finalLow, |=, pTime[6], value);
	value = in; finalLow <<= 8;
	FastNonIntraClamp(finalLow, |=, pTime[7], value);
	StoreTwoLongs(out, finalHigh, finalLow);
	out += MAX_X;
	pTime += MAX_X;
    }
}

void MotionAndClamp(MacroBlockInfo *pMb, int row)
{
    register ULong finalHigh, finalLow;
    UChar *pTime;
    short *ptr = (short *)&pMb->dctRecon[8*row];
    long in;

    pTime = pMb->pFuture+row*MAX_X;

    in = ptr[0]; FastNonIntraClamp(finalHigh,  =, pTime[0], in); finalHigh <<= 8;
    in = ptr[1]; FastNonIntraClamp(finalHigh, |=, pTime[1], in); finalHigh <<= 8;
    in = ptr[2]; FastNonIntraClamp(finalHigh, |=, pTime[2], in); finalHigh <<= 8;
    in = ptr[3]; FastNonIntraClamp(finalHigh, |=, pTime[3], in);

    in = ptr[4]; FastNonIntraClamp(finalLow,   =, pTime[4], in); finalLow <<= 8;
    in = ptr[5]; FastNonIntraClamp(finalLow,  |=, pTime[5], in); finalLow <<= 8;
    in = ptr[6]; FastNonIntraClamp(finalLow,  |=, pTime[6], in); finalLow <<= 8;
    in = ptr[7]; FastNonIntraClamp(finalLow,  |=, pTime[7], in);

    StoreTwoLongs(pMb->pTo+row*MAX_X, finalHigh, finalLow);
}

void FwdBackMotionAndClamp0Dc(MacroBlockInfo *pMb, long in)
{
    long value, i;
    register ULong finalHigh, finalLow;
    UChar *pFut = pMb->pFuture;
    UChar *pPast = pMb->pPast;
    UChar *out = pMb->pTo;

    /*
     * Process one row at a time
     */
    in = PreNonIntraClamp(in);
    for (i = 8; i > 0; i--) {
	value = in;
	FwdBackMotion(FastNonIntraClamp, finalHigh, =, value, pPast, pFut)
	value = in; finalHigh <<= 8;
	FwdBackMotion(FastNonIntraClamp, finalHigh, |=, value, pPast+1, pFut+1)
	value = in; finalHigh <<= 8;
	FwdBackMotion(FastNonIntraClamp, finalHigh, |=, value, pPast+2, pFut+2)
	value = in; finalHigh <<= 8;
	FwdBackMotion(FastNonIntraClamp, finalHigh, |=, value, pPast+3, pFut+3)
	value = in;
	FwdBackMotion(FastNonIntraClamp, finalLow,  =, value, pPast+4, pFut+4)
	value = in; finalLow <<= 8;
	FwdBackMotion(FastNonIntraClamp, finalLow, |=, value, pPast+5, pFut+5)
	value = in; finalLow <<= 8;
	FwdBackMotion(FastNonIntraClamp, finalLow, |=, value, pPast+6, pFut+6)
	value = in; finalLow <<= 8;
	FwdBackMotion(FastNonIntraClamp, finalLow, |=, value, pPast+7, pFut+7)
	StoreTwoLongs(out, finalHigh, finalLow);
	out += MAX_X;
	pPast += MAX_X;
	pFut += MAX_X;
    }
}

void FwdBackMotionAndClamp(MacroBlockInfo *pMb, int row)
{
    register ULong finalHigh, finalLow;
    UChar *pFuture;
    UChar *pPast;
    short *ptr = (short *)&pMb->dctRecon[8*row];
    long in;

    pFuture = pMb->pFuture+row*MAX_X;
    pPast = pMb->pPast+row*MAX_X;

    in = ptr[0];
    FwdBackMotion(FastNonIntraClamp, finalHigh,  =, in, pPast, pFuture);
    in = ptr[1]; finalHigh <<= 8;
    FwdBackMotion(FastNonIntraClamp, finalHigh, |=, in, pPast+1, pFuture+1);
    in = ptr[2]; finalHigh <<= 8;
    FwdBackMotion(FastNonIntraClamp, finalHigh, |=, in, pPast+2, pFuture+2);
    in = ptr[3]; finalHigh <<= 8;
    FwdBackMotion(FastNonIntraClamp, finalHigh, |=, in, pPast+3, pFuture+3);

    in = ptr[4];
    FwdBackMotion(FastNonIntraClamp, finalLow,   =, in, pPast+4, pFuture+4);
    in = ptr[5]; finalLow <<= 8;
    FwdBackMotion(FastNonIntraClamp, finalLow,  |=, in, pPast+5, pFuture+5);
    in = ptr[6]; finalLow <<= 8;
    FwdBackMotion(FastNonIntraClamp, finalLow,  |=, in, pPast+6, pFuture+6);
    in = ptr[7]; finalLow <<= 8;
    FwdBackMotion(FastNonIntraClamp, finalLow,  |=, in, pPast+7, pFuture+7);

    StoreTwoLongs(pMb->pTo+row*MAX_X, finalHigh, finalLow);
}

void IllegalMotionAndClamp(MacroBlockInfo *pMb, int row)
{
    Fatal("Illegal motion and clamp\n");
    /*NOTREACHED*/
}

void (*MotionAndClampTable[64])() = {
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    NoMotionAndClamp,
    MotionAndClamp,
    MotionHalfHorizAndClamp,
    MotionHalfVertAndClamp,
    MotionHalfAndClamp,
    MotionHalfHorizAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    MotionHalfVertAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    MotionHalfAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    MotionAndClamp,
    MotionHalfHorizAndClamp,
    MotionHalfVertAndClamp,
    MotionHalfAndClamp,
    MotionHalfHorizAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    MotionHalfVertAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    MotionHalfAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
    FwdBackMotionAndClamp,
};

void (*MotionAndClamp0DcTable[64])() = {
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    NoMotionAndClamp0Dc,
    MotionAndClamp0Dc,
    MotionHalfHorizAndClamp0Dc,
    MotionHalfVertAndClamp0Dc,
    MotionHalfAndClamp0Dc,
    MotionHalfHorizAndClamp0Dc,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    MotionHalfVertAndClamp0Dc,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    MotionHalfAndClamp0Dc,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    MotionAndClamp0Dc,
    MotionHalfHorizAndClamp0Dc,
    MotionHalfVertAndClamp0Dc,
    MotionHalfAndClamp0Dc,
    MotionHalfHorizAndClamp0Dc,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    MotionHalfVertAndClamp0Dc,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    MotionHalfAndClamp0Dc,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    IllegalMotionAndClamp,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
    FwdBackMotionAndClamp0Dc,
};
