/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              macdisplay.c
 *
 *      DESCRIPTION
 *              macintosh methods for displaying decoded mpeg video
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/macdisplay.c,v 1.6 1994/06/18 05:35:59 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"
#include "macEvent.h"

#include <QuickDraw.h>
#include <QDOffscreen.h>
#include <ToolUtils.h>
#include <Windows.h>
#include <Resources.h>
#include <Memory.h>

enum {
    CtPositionIsValue = 0x8000,		/* table position corresponds to index */
    CtUseDestinationColormap = 0x4000	/* use window's colormap, not the pixmap's */
};

typedef struct _macdata
{
    WindowPtr	decodeWindow;         /* really a CWindowPtr */
    PixMapPtr	pixmap;               /* offscreen pixmap */
    Rect	imageRect;
    DisplayInfoPtr dpyInfo;
    RGBColor	white, black;
    int		depth;
    CTabHandle	indexClut;
    CTabHandle	directClut;
} MacData, *MacDataPtr, **MacDataHandle;

static MacData ImageData;


void
RefreshWinPixels(DisplayInfoPtr dpyInfo)
{
    PicHandle myPicHandle;
    Rect picRect;
    CGrafPtr port = (CGrafPtr)ImageData.decodeWindow;	/* they are isometric */
    GrafPtr oldPort;
    RGBColor fore, back;
    
    GetPort((GrafPtr *)&oldPort);
    SetPort((GrafPtr)port);
    GetForeColor(&fore);
    GetBackColor(&back);
    RGBForeColor(&ImageData.black);
    RGBBackColor(&ImageData.white);
    CopyBits((BitMapPtr)(ImageData.pixmap),
    	     (BitMapPtr)(*port->portPixMap),
    	     &ImageData.pixmap->bounds,
    	     &ImageData.pixmap->bounds,
    	     srcCopy,
    	     (RgnHandle)NULL);
    RGBForeColor(&fore);
    RGBBackColor(&back);
    SetPort((GrafPtr)oldPort);
}

void
SetupWinImage(UChar *pixels, long width, long height)
{
    ProgMsg("SetupWinImage\n");
}

static void
InitIndexClut(CTabPtr clut)
{
    DisplayInfoPtr dpyInfo = ImageData.dpyInfo;
    ColorSpecPtr cs = clut->ctTable;
    short i, lastEntry;
    short maxRed, maxGreen, maxBlue;
    short red, green, blue;
    short oneBits;
    long stepSize;
    
    clut->ctSeed = GetCTSeed();
    clut->ctFlags = 0;

    memset(cs, 0, sizeof(ColorSpec) & 256);
    
    switch(dpyInfo->color) {
    default:	return;	/* never happens */
    case GrayScale1:
        lastEntry = 1;
        cs[0].value = 0;
        cs[0].rgb.red = 0;
        cs[0].rgb.green = 0;
        cs[0].rgb.blue = 0;
        cs[1].value = 1;
        cs[1].rgb.red = ~0;
        cs[1].rgb.green = ~0;
        cs[1].rgb.blue = ~0;
        break;

    case GrayScale7:
        oneBits = 0x80;
        lastEntry = 127;
        stepSize = 0xffff / 127;
        goto grayScale;
        
    case GrayScale8:
        oneBits = 0;
        lastEntry = 255;
        stepSize = 0xffff / 255;
    grayScale:
        for (i = 0; i <= lastEntry; cs++, i++) {
            cs->value = i | oneBits;
            cs->rgb.red = cs->rgb.green = cs->rgb.blue = 0xffff - i * stepSize;
        }
        break;
        
    case Rgb232:
        oneBits = 0x80;
        lastEntry = 127;
        maxRed = maxBlue = 3;
        maxGreen = 7;
        goto colorIndex;
        
    case Rgb332:
        oneBits = 0;
        lastEntry = 255;
        maxBlue = 3;
        maxRed = maxGreen = 7;
        goto colorIndex;
        
    case RgbC555:
        oneBits = 0;
        lastEntry = 5*5*5-1;
        maxGreen = maxRed = maxBlue = 4;
        goto colorIndex;
        
    case RgbC666:
        oneBits = 0;
        lastEntry = 6*6*6-1;
        maxGreen = maxRed = maxBlue = 5;
        
    colorIndex:
        for (red = 0; red <= maxRed; red++) {
            for (green = 0; green <= maxGreen; green++) {
                for (blue = 0; blue <= maxBlue; blue++) {
                    cs->rgb.red   = 0xffff -  red   * (0xffff / maxRed);
                    cs->rgb.green = 0xffff -  green * (0xffff / maxGreen);
                    cs->rgb.blue  = 0xffff -  blue  * (0xffff / maxBlue);
                    cs->value = red * (maxGreen+1) * (maxBlue+1)
                              + green * (maxBlue+1)
                              + blue
                              + oneBits;
                    cs++;
                }
            }
        }
        
        break;
    }
    clut->ctSize = lastEntry;
}

static void
InitPixmap(DisplayInfoPtr dpyInfo)
{
    int depth;

    switch(dpyInfo->color) {
    default:
    case GrayScale7:
    case GrayScale8:
    case Rgb232:
    case Rgb332:
    case RgbC555:
    case RgbC666:    depth = 8; break;
    case Rgb444:     depth = 16; break;
    case Rgb555:     depth = 16; break;
    case Rgb888:     depth = 32; break;
    case GrayScale1: depth = 1; break;
    }

    switch(depth) {
    case 32:
    case 16:
        ImageData.pixmap->pixelType = RGBDirect;
        ImageData.pixmap->pmTable = ImageData.directClut;
        ImageData.pixmap->pixelSize = 32;
        ImageData.pixmap->cmpCount = 3;
        ImageData.pixmap->cmpSize = (depth == 16 ? 5 : 8);
        (**ImageData.pixmap->pmTable).ctSeed = GetCTSeed();
        (**ImageData.pixmap->pmTable).ctFlags = 0x8000; /* ignore color table */
        (**ImageData.pixmap->pmTable).ctSize = 0;
        break;
    case 8:
        ImageData.pixmap->pixelSize = 8;
        goto indexedDepth;
        
    case 1:
        ImageData.pixmap->pixelSize = 1;
    indexedDepth:
        ImageData.pixmap->cmpCount = 1;
        ImageData.pixmap->cmpSize = ImageData.pixmap->pixelSize;
        InitIndexClut(*ImageData.indexClut);
        ImageData.pixmap->pixelType = 0;
        ImageData.pixmap->pmTable = ImageData.indexClut;
        break;
    }

    switch(dpyInfo->color) {
    case Rgb444:
    case Rgb555:
    case Rgb888:
        if (! dpyInfo->cvtu.ycToRgb->invertFinalRgb
         && dpyInfo->cvtu.ycToRgb->tablesAreValid)
            dpyInfo->cvtu.ycToRgb->tablesAreValid = False;
        dpyInfo->cvtu.ycToRgb->invertFinalRgb = True;
        dpyInfo->cvtu.ycToRgb->components = Pixel0RGB;
        break;
    }
}

UChar *
AllocateCopyBuffer(long size, int width, int height)
{
    UChar *copyBuffer;
    DisplayInfoPtr dpyInfo = ImageData.dpyInfo;

    copyBuffer = (UChar *)SecureAllocAligned(sizeof(OctWord), size);
    if (dpyInfo == NULL)
    	return(copyBuffer);

    SizeWindow(ImageData.decodeWindow,
    	       dpyInfo->dst.width, dpyInfo->dst.height, True);
               
    ImageData.pixmap->baseAddr = (Ptr)copyBuffer;
    ImageData.pixmap->rowBytes = dpyInfo->copyBufferWidth | 0x8000;
    ImageData.pixmap->bounds.top = 0;
    ImageData.pixmap->bounds.left = 0;
    ImageData.pixmap->bounds.bottom = height;
    ImageData.pixmap->bounds.right = width;
    
    InitPixmap(dpyInfo);
    
    return (copyBuffer);
}

void
SetWinBuffer(DisplayInfoPtr dpyInfo)
{
}

void
SetWinClip(DisplayInfoPtr dpyInfo)
{
}

void
FreeCopyBuffer(UChar *copyBuffer)
{
    free ((char *)copyBuffer);
}

void
HandleWinInput(DisplayInfoPtr dpyInfo)
{
    ProgMsg("HandleWinInput\n");
}

ImageReasonPtr
FailedWinInit(ImageReasonPtr reason)
{
    if (ImageData.decodeWindow) {
        DisposeWindow(ImageData.decodeWindow);
        ImageData.decodeWindow = NULL;
    }
    if (ImageData.pixmap) {
        free((char *)ImageData.pixmap);
        ImageData.pixmap = NULL;
    }
    if (ImageData.indexClut) {
    	DisposCTable(ImageData.indexClut);
    	ImageData.indexClut = NULL;
    }
    if (ImageData.directClut) {
    	DisposCTable(ImageData.directClut);
    	ImageData.directClut = NULL;
    }
}

static void
DecodeWindowHandler(EventRecord *event,	/* event, itself */
		    Ptr data,
		    WindowPtr wp,	/* window of the event */
		    int windowLocation)	/* FindWindow result for mouseUp/Down */
{
    DisplayInfoPtr dpyInfo = ImageData.dpyInfo;
    RgnHandle screen;

    switch (event->what) {
    case mouseDown:
    case mouseUp:
        switch(windowLocation) {
        case inDesk:
        case inMenuBar:
        case inSysWindow:
            break;
        case inDrag:
            screen = GetGrayRgn();
            DragWindow(wp, event->where, &(*screen)->rgnBBox);
            break;
            
        case inContent:
        case inGrow:
        case inGoAway:
        case inZoomIn:
        case inZoomOut:
            SelectWindow(wp);
            break;
        }
        break;
        
    case activateEvt:
    case keyDown:
    case keyUp:
    case autoKey:
    case diskEvt:
    case osEvt:
    case kHighLevelEvent:
    default:
        break;
    case updateEvt:
	if (dpyInfo->state != DisplayRunning)
	    dpyInfo->refresh = True;
        break;
    }
}

ImageReasonPtr
WinInit(DisplayInfoPtr dpyInfo)
{   
    OSErr error;
    short i;
   
    if (ImageData.dpyInfo != NULL) {
        InitPixmap(ImageData.dpyInfo);
        return(NULL);	/* already initialized */
    }
    
    ImageData.dpyInfo = dpyInfo;
    ImageData.imageRect.left = 0;
    ImageData.imageRect.right = dpyInfo->pSeq->horizontalSize;
    ImageData.imageRect.bottom = dpyInfo->pSeq->verticalSize;
    ImageData.imageRect.top = 0;
   
    ImageData.decodeWindow = GetNewCWindow(128, nil, (WindowPtr)-1);
    RegisterMacEventHandler(everyEvent, ImageData.decodeWindow, DecodeWindowHandler, NULL);

    SizeWindow(ImageData.decodeWindow,
    	       dpyInfo->dst.width, dpyInfo->dst.height, True);
   
    if (ImageData.decodeWindow == NULL)
        return( FailedWinInit(&IRBadAllocation) );
        
    SelectWindow(ImageData.decodeWindow);
    ShowWindow(ImageData.decodeWindow);

    ImageData.pixmap = (PixMapPtr)InsecureAlloc(sizeof(PixMap));
    if (ImageData.pixmap == NULL)
        return( FailedWinInit(&IRBadAllocation) );
    memset(ImageData.pixmap, 0, sizeof(PixMap));
    ImageData.pixmap->hRes = 72 << 16;	/* fixed point number */
    ImageData.pixmap->vRes = 72 << 16;	/* fixed point number */

    ImageData.directClut = (CTabHandle)
        NewHandle( sizeof(ColorTable) - sizeof(CSpecArray) );
    if (ImageData.directClut == NULL)
        return( FailedWinInit(&IRBadAllocation) );
    
    ImageData.indexClut = (CTabHandle)
        NewHandle( sizeof(ColorTable) + 255 * sizeof(ColorSpec) );
    if (ImageData.indexClut == NULL)
        return( FailedWinInit(&IRBadAllocation) );
    
    for (i = 0; i < NRgbDepthTypes; i++) {
	dpyInfo->copiers[ i ].winDisplay = RefreshWinPixels;
    }

    ImageData.white.red = 0;
    ImageData.white.green = 0;
    ImageData.white.blue = 0;
    ImageData.black.red = ~0;
    ImageData.black.green = ~0;
    ImageData.black.blue = ~0;
    
    dpyInfo->windowState = WindowUnobscured;
    return(NULL);
}

void
RawInit(DisplayInfoPtr dpyInfo)
{
    ProgMsg("RawInit\n");
}

XParseGeometry(char *spec, int x, int y, int width, int height)
{
    ProgMsg("XParseGeometry\n");
}