/*
 *      (c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              macroblock.c
 *
 *      DESCRIPTION
 *              macroblock-level processing
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/macroblock.c,v 4.17 1995/04/27 19:55:05 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"
#include "huffman.h"
#include "motion.h"

#ifdef DEBUG_ENABLED
char *MacroBlockTypeString();
#endif /* DEBUG_ENABLED */

static short ComputeMotion(short size, short code, short r, short *pPastMotion,
			   short maxMotion, short minMotion)	
{
    short motion;
    short complementR;
    short big, little;

    /*
     * compute the motion vector
     */
    if (size == 0) {        /* same as forward_f == 1 */
	complementR = 0;
	/*
	 * same code as below, except takes advantage of size==0
	 */
	if (little = code) {
	    if (little > 0)
		big = little - (1<<5);
	    else
		big = little + (1<<5);
	} else
	    big = 0;
    } else {
	if (code == 0)
	    complementR = 0;
	else
	    complementR = (1<<size) - 1 - r;

	if (little = code << size) {
	    if (little > 0) {
		little -= complementR;
		big = little - (1<<(size+5));
	    } else {
		little += complementR;
		big = little + (1<<(size+5));
	    }
	} else
	    big = 0;
    }

    motion = *pPastMotion + little;
    if (motion < minMotion || motion > maxMotion)
	motion += big - little;

    return (*pPastMotion = motion);
}

#ifdef DEBUG_ENABLED

CheckMotionBounds(int motionType,
		  SequenceInfo *pSeq,
		  MacroBlockInfo *pMb,
		  int mbAddr,
		  int halfLumaPixel,
		  int halfChromaPixel,
		  int horizYMotion,
		  int vertYMotion,
		  int horizCMotion,
		  int vertCMotion)
{ 
#define mbAddrToX(mbAddr)	((mbAddr % pSeq->mbWidth) * 16)
#define mbAddrToY(mbAddr)	((mbAddr / pSeq->mbWidth) * 16)

    int maxDim;
    boolean showMv = False;
    boolean xBad = False;
    boolean yBad = False;
    int horizMask, vertMask;
    int pastHorizMotion;
    int pastVertMotion;
    char *mvName;

    if (motionType == MBT_MFWD) {
	mvName = "fwd";
	pastHorizMotion = pMb->pastHorizFwdMotion;
	pastVertMotion = pMb->pastVertFwdMotion;
	horizMask = HALF_HORIZ_FORWARD_PIXEL;
	vertMask = HALF_VERT_FORWARD_PIXEL;
    } else {
	mvName = "bak";
	pastHorizMotion = pMb->pastHorizBakMotion;
	pastVertMotion = pMb->pastVertBakMotion;
	horizMask = HALF_HORIZ_BACK_PIXEL;
	vertMask = HALF_VERT_BACK_PIXEL;
    }
	
    /*
     * make sure the reference is not out of bounds
     */
    maxDim = (pSeq->mbWidth << 4)
	   - ((halfLumaPixel & horizMask) != 0);
    if (mbAddrToX(mbAddr) + horizYMotion + 15 >= maxDim
     || mbAddrToX(mbAddr) + horizYMotion < 0) {
	xBad = True;
    }

    maxDim = (pSeq->mbHeight << 4)
	   - ((halfLumaPixel & vertMask) != 0);
    if (mbAddrToY(mbAddr) + vertYMotion + 15 >= maxDim
     || mbAddrToY(mbAddr) + vertYMotion < 0) {
	yBad = True;
    }

    DebugTask(N_MotionVectors, showMv = True);
    if (xBad || yBad || showMv) {
	if (xBad || yBad)
	    Msg("*** mb %d@(%d,%d): ",
		mbAddr, mbAddrToX(mbAddr), mbAddrToY(mbAddr));

	Msg("%s motion: (%d,%d) or Y(%d%s%s,%d%s%s)",
	    mvName,
	    pastHorizMotion,
	    pastVertMotion,
	    horizYMotion,
	    halfLumaPixel & horizMask ? "+" : "",
	    xBad ? " oob!" : "",
	    vertYMotion,
	    halfLumaPixel & vertMask ? "+" : "",
	    yBad ? " oob!" : "");
	Msg(", C(%d%s,%d%s)\n",
	    horizCMotion,
	    halfChromaPixel & horizMask ? "+" : "",
	    vertCMotion,
	    halfChromaPixel & vertMask ? "+" : "");
    }
}

#else /* DEBUG_ENABLED */

#define CheckMotionBounds(type, pSeq, pMb, mba, halfY, halfC, hy,vy,yc,vc) \
    /* as nothing */

#endif /* DEBUG_ENABLED */

CacheReturnValue
MacroBlock(CacheArgsDecl, SequenceInfo *pSeq)
{
    ULong bits;
    ULong mbAddrInc;
    ULong hashValue;
    short pattern;
    short mbAddr;
    int bitLength, qScale;
    UChar picType, mbType;
    BitStreamPtr mpegBuf = pSeq->mpegBuf;
    PictureInfo *pPic = pSeq->currentPict;
    MacroBlockInfo *pMb = pSeq->macroBlk;
    UChar *pY, *pCb;
    UChar *pPastY, *pFutureY;
    UChar *pPastCb, *pFutureCb;
    int mbYIndex, mbCbIndex;	/* index to chroma and luma (and rgb) pixels */
    int halfLumaPixel, halfChromaPixel;

    DebugTask(N_MacroBlockMark,
	FlushCacheValues(mpegBuf);
	pMb->start = BitStreamOffset(mpegBuf, 0);
    )

    NextBits(mpegBuf, 11, bits);
    while (bits == B_MACROBLOCK_STUFFING) {
	ConsumeThenNextBits(mpegBuf, 11, 11, bits, N_MacroblockStuffing);
    }

    mbAddrInc = 0;
    while (bits == B_MACROBLOCK_ESCAPE) {
	mbAddrInc += 33;	/* as per section 3.2.6 */
	ConsumeThenNextBits(mpegBuf, 11, 11, bits, N_MacroblockEscape);
    }

    /* 11 bits are already in 'bits' */
    bitLength = MBAIBits(hashValue = HuffLookup.mbAddressInc[ bits ]);
    StreamAssert(mpegBuf, bitLength <= 11 && bitLength > 0);
    StreamAssert(mpegBuf, MBAIncrementValue(hashValue) <= 33);
    mbAddrInc += MBAIncrementValue(hashValue);
    ConsumeThenNextBits(mpegBuf, bitLength, 6, bits, N_MacroblockAddrInc);

    mbAddr = pMb->mbAddr + mbAddrInc;
    StreamAssert(mpegBuf, mbAddr <= pSeq->lastMacroBlock);

    /*
     * each skipped macro block is simply copied.
     * In this case, we can simply copy the final rgb pixels.
     */
    while (mbAddrInc > 1) {	/* this is only executed once */
	UChar *pFutureYBase, *pPastYBase;
	UChar *pFutureCbBase, *pPastCbBase;

	/*
	 * if the slice started in the middle of a row, mbAddrInc has
	 * to be > 1 to calculate an mb addr one greater than the previous.
	 * If this is the case, everything is fine.
	 */
	if (mbAddr == pMb->firstMbInSlice)
	    break;	/* slice started in the middle of a row */

	if (pMb->pastMbAddr + 1 == pMb->firstMbInSlice
	 || pPic->type == B_I_PIC) {
	    ProgMsg("%c pic %d: %d skipped macroblock starting @mb %d\n",
		IPBDchars[ pPic->type ], pPic->picNumber,
		mbAddrInc, pMb->mbAddr);
	}

	mbAddr -= mbAddrInc - 1;

	halfLumaPixel = 0;
	halfChromaPixel = 0;

	if (pPic->type == B_P_PIC) {
	    mbType = MBT_MFWD;	/* as per protocol */
	    pPastYBase = pSeq->pastPict->y;
	    pPastCbBase = pSeq->pastPict->cb;
	    pFutureYBase = NULL;
	    pFutureCbBase = NULL;
	    pMb->pastVertBakMotion = 0;
	    pMb->pastHorizBakMotion = 0;
	    pMb->pastVertFwdMotion = 0;
	    pMb->pastHorizFwdMotion = 0;
	    goto doCopyBlocks;
	} else
	    mbType = pMb->type;	/* use the last type (per protocol) */

	pPastCbBase = NULL;
	pPastYBase = NULL;
	if (mbType & MBT_MFWD) {
	    short horizFwdYMotion, vertFwdYMotion;
	    short horizFwdCMotion, vertFwdCMotion;
	    int motionOffset;

	    horizFwdYMotion = pMb->pastHorizFwdMotion;
	    vertFwdYMotion = pMb->pastVertFwdMotion;

	    AdjustPelMotion(pPic->fullPelFwdVec,
				HALF_HORIZ_FORWARD_SHIFT,
				HALF_VERT_FORWARD_SHIFT,
				horizFwdYMotion, vertFwdYMotion,
				horizFwdCMotion, vertFwdCMotion,
				halfLumaPixel, halfChromaPixel);

	    motionOffset = MotionOffset(horizFwdYMotion, vertFwdYMotion);
	    pPastYBase = pSeq->pastPict->y + motionOffset;
	    motionOffset = CbMotionOffset(horizFwdCMotion, vertFwdCMotion);
	    pPastCbBase = pSeq->pastPict->cb + motionOffset;

	    DebugTask(N_MotionVectors,
		Msg("skipped blk fwd motion: (%d,%d) or Y(%d%s,%d%s)",
		    pMb->pastHorizFwdMotion, pMb->pastVertFwdMotion,
		    horizFwdYMotion,
		    halfLumaPixel & HALF_HORIZ_FORWARD_PIXEL ? "+" : "",
		    vertFwdYMotion,
		    halfLumaPixel & HALF_VERT_FORWARD_PIXEL ? "+" : "");
		Msg(", C(%d%s,%d%s)\n",
		    horizFwdCMotion,
		    halfChromaPixel & HALF_HORIZ_FORWARD_PIXEL ? "+" : "",
		    vertFwdCMotion,
		    halfChromaPixel & HALF_VERT_FORWARD_PIXEL ? "+" : "");
	    )
	}

	pFutureYBase = NULL;
	pFutureCbBase = NULL;
	if (mbType & MBT_MBAK) {
	    short horizBakYMotion, vertBakYMotion;
	    short horizBakCMotion, vertBakCMotion;
	    int motionOffset;

	    horizBakYMotion = pMb->pastHorizBakMotion;
	    vertBakYMotion = pMb->pastVertBakMotion;

	    AdjustPelMotion(pPic->fullPelBakVec,
				HALF_HORIZ_BACK_SHIFT,
				HALF_VERT_BACK_SHIFT,
				horizBakYMotion, vertBakYMotion,
				horizBakCMotion, vertBakCMotion,
				halfLumaPixel,
				halfChromaPixel);

	    motionOffset = MotionOffset(horizBakYMotion, vertBakYMotion);
	    pFutureYBase = pSeq->futurePict->y + motionOffset;

	    motionOffset = CbMotionOffset(horizBakCMotion, vertBakCMotion);
	    pFutureCbBase = pSeq->futurePict->cb + motionOffset;

	    DebugTask(N_MotionVectors,
		Msg("skipped blk bak motion: (%d,%d) or Y(%d%s,%d%s)",
		    pMb->pastHorizBakMotion, pMb->pastVertBakMotion,
		    horizBakYMotion,
		    halfLumaPixel & HALF_HORIZ_BACK_PIXEL ? "+" : "",
		    vertBakYMotion,
		    halfLumaPixel & HALF_VERT_BACK_PIXEL ? "+" : "");
		Msg(", C(%d%s,%d%s)\n",
		    horizBakCMotion,
		    halfChromaPixel & HALF_HORIZ_BACK_PIXEL ? "+" : "",
		    vertBakCMotion,
		    halfChromaPixel & HALF_VERT_BACK_PIXEL ? "+" : "");
	    )
	}

	/* skipped block following intra-coded block in B-pic -- bad! */
	GeneralAssert(mbType & (MBT_MFWD | MBT_MBAK));

	/*
	 * If there are bits in the halfLumaPixel, then we must copy
	 * the y,cr,cb bits.  We could copy the rgb if:
	 *  - this is a B picture
	 *  - the motion vector is forward or back, not both
	 * Since this is rather unlikely, we omit the code.
	 */
   doCopyBlocks:
	while (--mbAddrInc > 0) {
	    mbCbIndex  = MBAddrToCb(mbAddr);
	    mbYIndex  = MBAddrToLuma(Y0, mbAddr++);
	    pY        = &pPic->y[ mbYIndex ];
	    pCb       = &pPic->cb[ mbCbIndex ];
	    if (pPastYBase) {
		pPastY    = pPastYBase  + mbYIndex;
		pPastCb   = pPastCbBase + mbCbIndex;
	    } else
		pPastY = pPastCb = NULL;
	    if (pFutureYBase) {
		pFutureY  = pFutureYBase + mbYIndex;
		pFutureCb = pFutureCbBase + mbCbIndex;
	    } else
		pFutureY = pFutureCb = NULL;

	    ShowMark(mpegBuf, N_MacroBlockMark, ("(skipped) addr=%d, type=%s\n",
		     mbAddr-1, MacroBlockTypeString(mbType)));
	    ShowMark(mpegBuf, N_BlockMark, ("copy y = 0x%x, 0x%x\n",
				   pPastY, pFutureY));
	    ShowMark(mpegBuf, N_BlockMark, ("copy cb = 0x%x, 0x%x\n",
				   pPastCb, pFutureCb));
	    ShowMark(mpegBuf, N_BlockMark, ("copy cr = 0x%x, 0x%x\n",
				   CbToCrAddr(pPastCb), CbToCrAddr(pFutureCb)));

	    CopyYCBlock(pPastY, pY, pFutureY, halfLumaPixel);

	    pPastY += pPastY ? MBLumaOffset(Y1) : 0;
	    pFutureY += pFutureY ? MBLumaOffset(Y1) : 0;
	    CopyYCBlock(pPastY, pY + MBLumaOffset(Y1), pFutureY,
			halfLumaPixel);

	    pPastY += pPastY ? MBLumaOffset(Y2) - MBLumaOffset(Y1) : 0;
	    pFutureY += pFutureY ? MBLumaOffset(Y2) - MBLumaOffset(Y1) : 0;
	    CopyYCBlock(pPastY, pY + MBLumaOffset(Y2), pFutureY,
			halfLumaPixel);

	    pPastY += pPastY ? MBLumaOffset(Y3) - MBLumaOffset(Y2) : 0;
	    pFutureY += pFutureY ? MBLumaOffset(Y3) - MBLumaOffset(Y2) : 0;
	    CopyYCBlock(pPastY, pY + MBLumaOffset(Y3), pFutureY,
			halfLumaPixel);

	    if (! pSeq->dpyInfo->oneChannelOnly) {
		CopyYCBlock(pPastCb, pCb, pFutureCb, halfChromaPixel);
		CopyYCBlock(pPastCb ? CbToCrAddr(pPastCb) : pPastCb,
			    CbToCrAddr(pCb),
			    pFutureCb ? CbToCrAddr(pFutureCb) : pFutureCb,
			    halfChromaPixel);
	    }

	    DebugTask(N_AnimateDstMB, ShowMb(pPic, mbAddr-1));
	}

	break;	/* only execute the loop once */
    }

    pMb->mbAddr = mbAddr;

    DebugTask(N_MacroBlockMark,
	ShowMark(mpegBuf, N_MacroBlockMark, ("start addr=%d\n",mbAddr)) );
    
    /*
     * decode the macroblock type depending on the picture type.  We check
     * first the most likely, then next most likely...
     */
    if ((picType = pPic->type) == B_B_PIC) {
	bitLength = MBTBits(hashValue = HuffLookup.mb_B_Type[ bits ]);
	StreamAssert(mpegBuf, bitLength >= 2 && bitLength <= 6);
	mbType = pMb->type = MBTypeValue(hashValue);
    }

    else if (picType == B_P_PIC) {
	bitLength = MBTBits(hashValue = HuffLookup.mb_P_Type[ bits ]);
	StreamAssert(mpegBuf, bitLength >= 1 && bitLength <= 6);
	mbType = pMb->type = MBTypeValue(hashValue);
    }

    else { /* B_I_PIC or B_D_PIC */
	/*
	 * For I pictures we only have to consider the first of the six bits.
	 * If its 1, then the bitlength is 1 and the type is MBT_INTRA; if 0,
	 * then the bitlength is 2 (code is 01) and the type is
	 * MBT_INTRA|MBT_QUANT.
	 * For D pictures, it is always a one bit and is yields the same
	 * type as an I picture with a one bit.
	 * This gobbledy gook code does that without branches.
	 */
	bitLength = 2 - (bits >>= 5);
	mbType = pMb->type =
	    MBT_INTRA | (((~bits) << MBT_QUANT_BitPosition) & MBT_QUANT);
	
    }
    ConsumeBits(mpegBuf, bitLength, N_MbType);

    if (mbType & MBT_QUANT) {
	ReadBits(mpegBuf, 5, qScale, N_MbQuant);
	if (qScale != pSeq->qScale) {
	    pSeq->qScale = qScale;
	    ScaleQuantMatrix(pSeq, qScale);
	}
    } else
	qScale = pSeq->qScale;

    /*
     * These values are used from here to the end
     */
    mbYIndex = MBAddrToLuma(Y0, mbAddr);
    mbCbIndex = MBAddrToCb(mbAddr);

    /*
     * if this is an intra coded macro block, then MBT_MFWD, MBT_MBAK and
     * MBT_PATTERN will never be set.
     */
    if (mbType & MBT_INTRA) {
	StreamAssert(mpegBuf, (mbType & (MBT_MFWD|MBT_MBAK|MBT_PATTERN)) == 0);

	RestoreCacheAfterCall(mpegBuf, IntraBlock(CacheArgs, Y0, pSeq) );
	IDct(pMb, NULL, pY = &pPic->y[ mbYIndex ], NULL, 0);

	RestoreCacheAfterCall(mpegBuf, IntraBlock(CacheArgs, Y1, pSeq) );
	IDct(pMb, NULL, pY + 8, NULL, 0);

	RestoreCacheAfterCall(mpegBuf, IntraBlock(CacheArgs, Y2, pSeq) );
	IDct(pMb, NULL, pY + 8*MAX_X, NULL, 0);

	RestoreCacheAfterCall(mpegBuf, IntraBlock(CacheArgs, Y3, pSeq) );
	IDct(pMb, NULL, pY + 8*MAX_X + 8, NULL, 0);

	RestoreCacheAfterCall(mpegBuf, IntraBlock(CacheArgs, CB, pSeq) );
	if (! pSeq->dpyInfo->oneChannelOnly)
	    IDct(pMb, NULL, pCb = &pPic->cb[ mbCbIndex ], NULL, 0);

	RestoreCacheAfterCall(mpegBuf, IntraBlock(CacheArgs, CR, pSeq) );
	if (! pSeq->dpyInfo->oneChannelOnly)
	    IDct(pMb, NULL, CbToCrAddr(pCb), NULL, 0);

	pMb->pastIntraMbAddr = mbAddr;		/* per protocol */
	pMb->pastVertBakMotion = 0;		/* per protocol */
	pMb->pastHorizBakMotion = 0;		/* per protocol */
	pMb->pastVertFwdMotion = 0;		/* per protocol */
	pMb->pastHorizFwdMotion = 0;		/* per protocol */
    } else {
	pMb->dcYPast = 128;			/* per protocol */
	pMb->dcCrPast = 128;			/* per protocol */
	pMb->dcCbPast = 128;			/* per protocol */

	halfLumaPixel = 0;
	halfChromaPixel = 0;
	if (mbType & MBT_MFWD) {
	    short fwdSize;
	    short horizFwdCode, vertFwdCode;
	    short horizFwdYMotion, vertFwdYMotion;
	    short horizFwdCMotion, vertFwdCMotion;
	    short horizFwdR, vertFwdR;
	    short maxMotion, minMotion;
	    int motionOffset;

	    NextBits(mpegBuf, 11, bits);
	    bitLength = MotionBits(hashValue = HuffLookup.motionVector[ bits ]);
	    ConsumeBits(mpegBuf, bitLength, N_HorizFwdCode);

	    fwdSize = pPic->forwardSize;
	    if ((horizFwdCode = MotionValue(hashValue))
	     && fwdSize > 0) {/* same as forward_f != 1 */
		ReadBits(mpegBuf, fwdSize, horizFwdR, N_MotionHorizFwdR);
	    } else {
		horizFwdR = 0;
	    }

	    NextBits(mpegBuf, 11, bits);
	    bitLength = MotionBits(hashValue = HuffLookup.motionVector[ bits ]);
	    ConsumeBits(mpegBuf, bitLength, N_VertFwdCode);

	    if ((vertFwdCode = MotionValue(hashValue))
	     && fwdSize > 0) {/* same as forward_f != 1 */
		ReadBits(mpegBuf, fwdSize, vertFwdR, N_MotionVertFwdR);
	    } else {
		vertFwdR = 0;
	    }

	    maxMotion = (1<<(fwdSize+4)) - 1;
	    minMotion = -(maxMotion+1);

	    horizFwdYMotion =
		ComputeMotion(fwdSize, horizFwdCode, horizFwdR,
			      &pMb->pastHorizFwdMotion, maxMotion, minMotion);
	    vertFwdYMotion = 
		ComputeMotion(fwdSize, vertFwdCode, vertFwdR,
			      &pMb->pastVertFwdMotion, maxMotion, minMotion);

	    AdjustPelMotion(pPic->fullPelFwdVec,
				HALF_HORIZ_FORWARD_SHIFT,
				HALF_VERT_FORWARD_SHIFT,
				horizFwdYMotion, vertFwdYMotion,
				horizFwdCMotion, vertFwdCMotion,
				halfLumaPixel,
				halfChromaPixel);

	    CheckMotionBounds(MBT_MFWD, pSeq, pMb, mbAddr,
			      halfLumaPixel, halfChromaPixel,
			      horizFwdYMotion, vertFwdYMotion,
			      horizFwdCMotion, vertFwdCMotion);

	    pPastY = pSeq->pastPict->y
		   + mbYIndex
		   + MotionOffset(horizFwdYMotion, vertFwdYMotion);
	    motionOffset = CbMotionOffset(horizFwdCMotion, vertFwdCMotion);
	    pPastCb = pSeq->pastPict->cb + mbCbIndex + motionOffset;

	} else { /* (mbType & MBT_MFWD) == 0 */
	    if (picType == B_P_PIC) {
		/*
		 * Non-intra coded macroblocks in P pictures without
		 * motion vectors have their forward motion vectors set to 0
		 */
		pPastY = pSeq->pastPict->y + mbYIndex;
		pPastCb = pSeq->pastPict->cb + mbCbIndex;
		pMb->pastVertFwdMotion = 0;		/* per protocol */
		pMb->pastHorizFwdMotion = 0;	/* per protocol */
		halfLumaPixel = halfChromaPixel = 0;
		/*
		 * The protocol did not mark the fwd bit, but we mark
		 * it here so that the motion compensation code will
		 * do its work.
		 */
		mbType = (pMb->type |= MBT_MFWD);
	    } else {
		pPastCb = NULL;
		pPastY = NULL;
	    }
	}

	if (mbType & MBT_MBAK) {
	    short bakSize;
	    short horizBakCode, vertBakCode;
	    short horizBakYMotion, vertBakYMotion;
	    short horizBakCMotion, vertBakCMotion;
	    short horizBakR, vertBakR;
	    short maxMotion, minMotion;
	    int motionOffset;

	    NextBits(mpegBuf, 11, bits);
	    bitLength = MotionBits(hashValue = HuffLookup.motionVector[ bits ]);
	    ConsumeBits(mpegBuf, bitLength, N_HorizBakCode);

	    if ((horizBakCode = MotionValue(hashValue))
	     && (bakSize = pPic->backwardSize) > 0) {/* backward_f != 1 */
		ReadBits(mpegBuf, bakSize, horizBakR, N_MotionHorizBakR);
	    } else {
		horizBakR = 0;
	    }

	    NextBits(mpegBuf, 11, bits);
	    bitLength = MotionBits(hashValue = HuffLookup.motionVector[ bits ]);
	    ConsumeBits(mpegBuf, bitLength, N_VertBakCode);

	    bakSize = pPic->backwardSize;
	    vertBakCode = MotionValue(hashValue);
	    if (vertBakCode && bakSize > 0) {/* backward_f != 1 */
		ReadBits(mpegBuf, bakSize, vertBakR, N_MotionVertBakR);
	    } else {
		vertBakR = 0;
	    }

	    maxMotion = (1<<(bakSize+4)) - 1;
	    minMotion = -(maxMotion+1);

	    horizBakYMotion =
		ComputeMotion(bakSize, horizBakCode, horizBakR,
			      &pMb->pastHorizBakMotion, maxMotion, minMotion);
	    vertBakYMotion =
		ComputeMotion(bakSize, vertBakCode, vertBakR,
			      &pMb->pastVertBakMotion, maxMotion, minMotion);

	    AdjustPelMotion(pPic->fullPelBakVec,
				HALF_HORIZ_BACK_SHIFT,
				HALF_VERT_BACK_SHIFT,
				horizBakYMotion, vertBakYMotion,
				horizBakCMotion, vertBakCMotion,
				halfLumaPixel, halfChromaPixel);

	    CheckMotionBounds(MBT_MBAK, pSeq, pMb, mbAddr,
			      halfLumaPixel, halfChromaPixel,
			      horizBakYMotion, vertBakYMotion,
			      horizBakCMotion, vertBakCMotion);

	    pFutureY = pSeq->futurePict->y
		     + mbYIndex
		     + MotionOffset(horizBakYMotion, vertBakYMotion);
	    motionOffset = CbMotionOffset(horizBakCMotion, vertBakCMotion);
	    pFutureCb = pSeq->futurePict->cb + mbCbIndex + motionOffset;

	} else { /* (mbType & MBT_MBAK) == 0 */
	    pFutureCb = NULL;
	    pFutureY = NULL;
	}

	if (mbType & MBT_PATTERN) {
	    NextBits(mpegBuf, 9, bits);
	    bitLength =
		CodedBlockBits(hashValue = HuffLookup.codedBlock[ bits ]);
	    pattern = Optional(pMb->codedBlockPattern =)
		CodedBlockValue(hashValue);
	    ConsumeBits(mpegBuf, bitLength, N_CodedBlockPattern);
	} else
	    pattern = Optional(pMb->codedBlockPattern =) 0;

	/*
	 * Each of the blocks in the protocol stream are either
	 * difference blocks (that are added to the block pointed
	 * at by the motion vector), or are simply motion compensated
	 * (copied from the block pointed at by the motion vector)
	 */
	pY = &pPic->y[ mbYIndex ];
	WhenDebugEnabled(pMb->blockIndex = Y0);
	if (pattern & CBP_Y0) {
	    RestoreCacheAfterCall(mpegBuf, Block(CacheArgs, pSeq) );
	    IDct(pMb, pPastY, pY, pFutureY, halfLumaPixel);
	} else {
	    ShowMark(mpegBuf, N_BlockMark,
		("copy luma0=0x%x,0x%x\n", pPastY, pFutureY));
	    CopyYCBlock(pPastY, pY, pFutureY, halfLumaPixel);
	}

	pPastY   += pPastY   ? MBLumaOffset(Y1) : 0;
	pFutureY += pFutureY ? MBLumaOffset(Y1) : 0;
	WhenDebugEnabled(pMb->blockIndex = Y1);
	if (pattern & CBP_Y1) {
	    RestoreCacheAfterCall(mpegBuf, Block(CacheArgs, pSeq) );
	    IDct(pMb, pPastY, pY + MBLumaOffset(Y1), pFutureY, halfLumaPixel);
	} else {
	    ShowMark(mpegBuf, N_BlockMark,
		("copy luma1=0x%x,0x%x\n", pPastY, pFutureY));
	    CopyYCBlock(pPastY, pY + MBLumaOffset(Y1), pFutureY, halfLumaPixel);
	}

	pPastY   += pPastY   ? MBLumaOffset(Y2)-MBLumaOffset(Y1) : 0;
	pFutureY += pFutureY ? MBLumaOffset(Y2)-MBLumaOffset(Y1) : 0;
	WhenDebugEnabled(pMb->blockIndex = Y2);
	if (pattern & CBP_Y2) {
	    RestoreCacheAfterCall(mpegBuf, Block(CacheArgs, pSeq) );
	    IDct(pMb, pPastY, pY + MBLumaOffset(Y2), pFutureY, halfLumaPixel);
	} else {
	    ShowMark(mpegBuf, N_BlockMark,
		("copy luma2=0x%x,0x%x\n", pPastY, pFutureY));
	    CopyYCBlock(pPastY, pY + MBLumaOffset(Y2), pFutureY, halfLumaPixel);
	}

	pPastY   += pPastY   ? MBLumaOffset(Y3)-MBLumaOffset(Y2) : 0;
	pFutureY += pFutureY ? MBLumaOffset(Y3)-MBLumaOffset(Y2) : 0;
	WhenDebugEnabled(pMb->blockIndex = Y3);
	if (pattern & CBP_Y3) {
	    RestoreCacheAfterCall(mpegBuf, Block(CacheArgs, pSeq) );
	    IDct(pMb, pPastY, pY + MBLumaOffset(Y3), pFutureY, halfLumaPixel);
	} else {
	    ShowMark(mpegBuf, N_BlockMark,
		("copy luma=0x%x,0x%x\n", pPastY, pFutureY));
	    CopyYCBlock(pPastY, pY + MBLumaOffset(Y3), pFutureY, halfLumaPixel);
	}

	pCb = &pPic->cb[ mbCbIndex ];

	WhenDebugEnabled(pMb->blockIndex = CB);
	if (pattern & CBP_CB) {
	    RestoreCacheAfterCall(mpegBuf, Block(CacheArgs, pSeq) );
	    if (! pSeq->dpyInfo->oneChannelOnly)
		IDct(pMb, pPastCb, pCb, pFutureCb, halfChromaPixel);
	} else if (! pSeq->dpyInfo->oneChannelOnly) {
	    ShowMark(mpegBuf, N_BlockMark,
		("copy cb=0x%x,0x%x\n", pPastCb, pFutureCb));
	    CopyYCBlock(pPastCb, pCb, pFutureCb, halfChromaPixel);
	}

	WhenDebugEnabled(pMb->blockIndex = CR);
	if (pattern & CBP_CR) {
	    RestoreCacheAfterCall(mpegBuf, Block(CacheArgs, pSeq) );
	    if (! pSeq->dpyInfo->oneChannelOnly)
		IDct(pMb, 
		     pPastCb ? CbToCrAddr(pPastCb) : pPastCb,
		     CbToCrAddr(pCb),
		     pFutureCb ? CbToCrAddr(pFutureCb) : pFutureCb,
		     halfChromaPixel);
	} else if (! pSeq->dpyInfo->oneChannelOnly) {
	    ShowMark(mpegBuf, N_BlockMark,
		("copy cr=0x%x,0x%x\n",
		CbToCrAddr(pPastCb), CbToCrAddr(pFutureCb)));
	    CopyYCBlock(pPastCb ? CbToCrAddr(pPastCb) : pPastCb,
			CbToCrAddr(pCb),
			pFutureCb ? CbToCrAddr(pFutureCb) : pFutureCb,
			halfChromaPixel);
	}
    }

    if (pPic->type == B_D_PIC) {
	ReadBits(mpegBuf, 1, bits, N_EndOfMacroblock);
	StreamAssert(mpegBuf, bits == 1);
    }

    DebugTask(N_MacroBlockMark,
	BitByteOffset end;

	FlushCacheValues(mpegBuf);
	end = BitStreamOffset(mpegBuf, 0);
	ShowMark(mpegBuf, N_MacroBlockMark,
	    ("done. addr=%d, type=%s, q=%d, nbits=%d\n",
		mbAddr, MacroBlockTypeString(mbType), qScale,
		BitDifference(&pMb->start, &end)) );
    )

    DebugTask(N_AnimateDstMB, ShowMb(pPic, mbAddr));
    CacheReturn(mpegBuf);
}
