/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	main.c - top-level control for mpeg decoding
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/main.c,v 4.70 1995/08/09 23:44:41 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"

#include "fileContent.h"
#include "process.h"

static BitMask FilePlayRequirements;
static BitMask ScreenPlayRequirements;
static BitMask YcFbPlayRequirements;

static void
RequirementsInit()
{
    BitMask *p;

    p = &FilePlayRequirements;
    memset((char *)p, 0, sizeof(*p));
    BM_SET(p, SYM_INPATH);
    BM_SET(p, SYM_OUTPATH);
    BM_SET(p, SYM_VIDEOENABLE);
    BM_SET(p, SYM_DESTTYPE);

    p = &ScreenPlayRequirements;
    memset((char *)p, 0, sizeof(*p));
    BM_SET(p, SYM_WIDTH);
    BM_SET(p, SYM_HEIGHT);
    BM_SET(p, SYM_X);
    BM_SET(p, SYM_Y);
    BM_SET(p, SYM_DISPLAY);
    BM_SET(p, SYM_WINDOW);
    BM_SET(p, SYM_INPATH);
    BM_SET(p, SYM_AUDIOENABLE);
    BM_SET(p, SYM_VIDEOENABLE);
    BM_SET(p, SYM_METHOD);
    BM_SET(p, SYM_DESTTYPE);
    BM_SET(p, SYM_COLOR);

    p = &YcFbPlayRequirements;
    memset((char *)p, 0, sizeof(*p));
    BM_SET(p, SYM_INPATH);
    BM_SET(p, SYM_AUDIOENABLE);
    BM_SET(p, SYM_VIDEOENABLE);
    BM_SET(p, SYM_DESTTYPE);
}

static void
StreamError(BitStream *buf, BitstreamCondition condition)
{
    SequenceInfoPtr pSeq = (SequenceInfoPtr)buf->seqPriv;
    PictureInfo *ppic;
    BitByteOffset offset;

    switch (condition) {
    case BitstreamError:
	break;

    case BitstreamEof:
	offset = BitStreamOffset(buf, 0);
        ProgMsg("Unexpected EOF @ byte/bit offset %d/%d\n",
	    offset.bytes, offset.bits);
	break;
    }
    if ((ppic = pSeq->currentPict)) {
	ProgMsg("    %c pic @ GOP #%d; %02d:%02d:%02d:%02d; mb %d\n",
		"?IPBD"[ ppic->type ],
		ppic->tc.discrete.groupOfPictures,
		ppic->tc.discrete.hours,
		ppic->tc.discrete.minutes,
		ppic->tc.discrete.seconds,
		ppic->tc.discrete.pictures, 
		pSeq->macroBlk->mbAddr);
	WhenDebugEnabled(ProgMsg("    tc comp=%d\n", ppic->tc.composite))
	ProgMsg("    skipping this picture...\n");

	ppic->releaseAfterDisplay = False;
	ppic->notDisplayed = False;
	ppic->skipped = True;
	ReleasePicture(ppic);
	if (ppic == pSeq->prevPict)
	    pSeq->prevPict = NULL;
	if (ppic == pSeq->futurePict)
	    pSeq->futurePict = NULL;
	
	if (condition == BitstreamError)
	    RestoreContext(pSeq->pictContext, 1);
    }
    RestoreContext(pSeq->seqContext, 1);
}

static ImageReasonPtr
OpenVideoInput(SequenceInfoPtr pSeq)
{
    BitStream *mpegBuf = pSeq->mpegBuf;
    FileInfo fileInfo;  
    char *file = pSeq->dpyInfo->videoInput;
    ImageReasonPtr irp;
    static ImageReason badSequenceHdr =
	{ "Bad sequence header\n", IR_BadIOFile };
    static ImageReason badFileType =
	{ "Can't open file or wrong type\n", IR_UnknowFileType };

    if (! IsInitialized(pSeq->dpyInfo, SYM_INPATH))
	return( NULL );
    if (pSeq->dpyInfo->reopenVideo) {
	if (mpegBuf->opened)
	    CloseBitStream(pSeq->mpegBuf);
	pSeq->dpyInfo->reopenVideo = False;
    }
    if (mpegBuf->opened)
	return(NULL);

    InitPictureQueues(pSeq);

    if( IsInitialized(pSeq->dpyInfo, SYM_BITSERVER)
     && (irp = CBR_init(mpegBuf, pSeq->dpyInfo->bitServer)) ) {
	return(irp);
    }
    if (irp = GetFileContentInfo(file, &fileInfo))
	return(irp);
    switch (fileInfo.type) {

    case FT_MpegVideo:
	if ((irp = OpenBitStreamInput(mpegBuf, file)))
	    return(irp);
	break;

    case FT_MpegSystem:
	if ((irp = OpenSystemStreamInput(mpegBuf, NULL, file, VideoBitstream)))
	    return(irp);
	break;

    default:
	return(&badFileType);
    }
    {
	CheckOutCache(mpegBuf);
	pSeq->iPics.nOffsets = 0;
	pSeq->lastPicNumber = -1;
	pSeq->xingSequence = False;
	pSeq->dpyInfo->refresh = pSeq->dpyInfo->visualDisplay;
	pSeq->dpyInfo->rerenderRefresh = pSeq->dpyInfo->refresh;
	pSeq->dpyInfo->u.s.restartDecoder = True;
	pSeq->dpyInfo->currentFrame = 0;

	/*
	 * If the call to SequenceHeader fails below, then clean up
	 * and say this is a bogus sequence
	 */
	if (SaveContext(pSeq->pictContext)) {
	    CloseFile(mpegBuf->fd);
	    mpegBuf->fd = -1;
	    return(&badSequenceHdr);
	}
	RestoreCacheAfterCall(mpegBuf,
			      NextStartCode(CacheArgs, mpegBuf, True) );
	RestoreCacheAfterCall(mpegBuf,
			      SequenceHeader(CacheArgs, pSeq) );

	if (pSeq->horizontalSize > MAX_X
	|| pSeq->verticalSize > MAX_LUMA_Y) {
	    irp = CreateReason("%s (%dx%d) %s (%dx%d)\n",
		"Sequence size", pSeq->horizontalSize, pSeq->verticalSize,
		"is greater than maximum", MAX_X, MAX_LUMA_Y);
	    irp->hint = IR_BadDimensions;
	    return(irp);
	}

	FlushCacheValues(mpegBuf);
    }
    return(NULL);
}

static ImageReasonPtr
OpenVideoOutput(SequenceInfoPtr pSeq)
{
    DisplayInfoPtr dpyInfo = pSeq->dpyInfo;
    FileHandle fd = (FileHandle)dpyInfo->vOutFp;

    if (! IsInitialized(dpyInfo, SYM_OUTPATH))
	return( NULL );
    if (fd >= 0)
	return( NULL );

    fd = OpenFileForWriting(dpyInfo->videoOutput);
    if (fd < 0)
	return( CreateSysReason("can't open %s\n", dpyInfo->videoOutput) );
    dpyInfo->vOutFp = (PrivPointer)fd;

    return( NULL );
}

static void
CloseVideoOutput(SequenceInfoPtr pSeq)
{
    FileHandle fd = (FileHandle)pSeq->dpyInfo->vOutFp;

    pSeq->dpyInfo->vOutFp = (PrivPointer)-1;
    if (fd < 0)
	return;
    CloseFile(fd);
}

static void
DecoderMainLoop(ParseInfoPtr pParse)
{
    int longJumped;
    SequenceInfoPtr pSeq = (SequenceInfoPtr)GetParsePrivateData(pParse);
    DisplayInfoPtr dpyInfo = pSeq->dpyInfo;
    Queue *dq;
    ParseValue value;

    DebugTask(N_Protocol, pParse->showInput = True )
    DebugTask(N_Protocol, pParse->showOutput = True )

    dpyInfo->aInFp = (PrivPointer)-1;
    dpyInfo->vOutFp = (PrivPointer)-1;
    dpyInfo->state = DisplayStopped;
    pSeq->allocPicture = NewPreloadPicture;
    pSeq->mpegBuf->fd = -1;

    InitPictureQueues(pSeq);
    while (! pSeq->clientTerminated) {
	/*
	 * When we longjump back here, the value is non-zero, so the while
	 * loop will save a new copy.
	 */
	do {
	    longJumped = SaveContext(pSeq->decodeContext);
	    if (longJumped) {
		pSeq->inputExhausted = False;
	    }
	} while (longJumped);

	/*
	 * At this point, we are in a stopped state.
	 */
	pSeq->idle = 0;
	pSeq->active = &pSeq->idle;
	for (;;) {
	    CheckServices(dpyInfo);
	    if (dpyInfo->state != DisplayLoading)
		WaitForSomething(&QuarterSec);
	    else
		break;
	}

	/*
	 * ... now loading state.  Set up and start the decoder.
	 * From there we will process stop and continue requests.
	 */
	pSeq->picture = ProcPicture;
	pSeq->pictureReady = PictureReady;
	pSeq->tc.composite = 0; /* reset the time code to 0 */
	pSeq->picGopStart = 0;	/* start counting at start of all sequences */
	
	CleanClock(dpyInfo);
	MpegDecode(pSeq);

	/*
	 * Decoding for this sequence is complete.  Finish up the
	 * display.
	 */
	dq = &PictBuffer.displayQueue;
	pSeq->inputExhausted = True;
	for (;;) {
	    CheckServices(dpyInfo);

	    /*
	     * A config command will set the state to loading,
	     * but there's nothing to load just now, so just 
	     * transition to stopped, and if there are commands
	     * left to process (like a "start" command), we'll
	     * transition nicely.
	     */
	    if (dpyInfo->state == DisplayLoading)
		(void) NewState(dpyInfo, DisplayStopped);

	    /*
	     * at this point, we are waiting for the display
	     * queue to drain.  When it is, go into the stopped
	     * state.
	     */
	    if (QueueEmpty(dq) && dpyInfo->state != DisplayStopped) {
		(void) NewState(dpyInfo, DisplayStopped);
		value.intvalue = dpyInfo->currentFrame;
		EventToClient(pSeq->pParse, MCS_EVENT_STOPPED,
			      &value, MCS_TYPE_INT);
	    }

	    if (dpyInfo->state != DisplayRunning) {
		if (dpyInfo->refresh) {
		    if (dpyInfo->rerenderRefresh) {
			dpyInfo->rerenderRefresh = False;
			(*dpyInfo->saveRefreshFrame)(dpyInfo, NULL);
		    }
		    /*
		     * clear this after (*dpyInfo->saveRefreshFrame)
		     * because it may cause it to be set again.
		     */
		    dpyInfo->refresh = False;
		    (*dpyInfo->refresher)(dpyInfo);
		}

		WaitForSomething(&QuarterSec);
	    }
	}
    }
}

static SequenceInfoPtr
AllocSequence()
{
    SequenceInfoPtr pSeq;

    /*
     * Allocate storage for the mpeg compression context.  This may be
     * inefficient to allocate each separately, but it helps in
     * tracking down memory reference problems.
     */
    pSeq = (SequenceInfoPtr)memalign(8, sizeof(SequenceInfo));
    memset((char*)pSeq, 0, sizeof(SequenceInfo));

    pSeq->mpegBuf = (BitStreamPtr)memalign(8, sizeof(BitStream));
    memset((char*)pSeq->mpegBuf, 0, sizeof(BitStream));
    WhenDebugEnabled(pSeq->mpegBuf->dbgFormat = VmpegDebugFormat);

    pSeq->macroBlk = (MacroBlockInfoPtr)memalign(8, sizeof(MacroBlockInfo));
    memset((char*)pSeq->macroBlk, 0, sizeof(MacroBlockInfo));

    pSeq->dpyInfo = (DisplayInfoPtr)memalign(8, sizeof(DisplayInfo));
    memset((char*)pSeq->dpyInfo, 0, sizeof(DisplayInfo));

    pSeq->macroBlk->pSeq = pSeq;
    pSeq->dpyInfo->pSeq = pSeq;
    InitBitStreamData(pSeq->mpegBuf);
    pSeq->mpegBuf->seqPriv = (PrivPointer)pSeq;
    pSeq->mpegBuf->errorFunc = StreamError;

    TimeNow(&pSeq->wallClock);
    pSeq->seqClock = 0;

    return(pSeq);
}

static void
ClientInputNotify(ParseInfoPtr pParse)
{
    SequenceInfoPtr pSeq = (SequenceInfoPtr)GetParsePrivateData(pParse);

    if (pSeq->dpyInfo->state != DisplayLoading)
	pSeq->dpyInfo->u.s.readClient = True;
}

static void
ClientOutputNotify(ParseInfoPtr pParse)
{
    SequenceInfoPtr pSeq = (SequenceInfoPtr)GetParsePrivateData(pParse);

    pSeq->dpyInfo->u.s.writeClient = True;
}

static void
ClientEofNotify(ParseInfoPtr pParse)
{
    SequenceInfoPtr pSeq = (SequenceInfoPtr)GetParsePrivateData(pParse);
    DisplayInfoPtr dpyInfo = pSeq->dpyInfo;

    pSeq->clientTerminated = True;
    /* free shared memory segments */
    FreeCopyBuffer(dpyInfo->copyBuffer);
    /* quietly go away */
    NvrLmExit(0);
}

static void
ExecuteCommand(ParseInfoPtr pParse, int id)
{
    SequenceInfoPtr pSeq = (SequenceInfoPtr)GetParsePrivateData(pParse);
    DisplayInfoPtr dpyInfo = pSeq->dpyInfo;
    ImageReasonPtr reason = NULL;
    BitMask *requirements;
    ParseValue value;

    switch (id) {
    case SYM_CONFIGURE:
	if (dpyInfo->state != DisplayStopped) {
	    RejectCommand(pParse, id, MCS_BAD_RUNNING);
	    break;
	}
	dpyInfo->configured = False;
	
	switch (dpyInfo->new.destType) {
	case DestWinFrameBuffer:requirements = &ScreenPlayRequirements; break;
	case DestWindow:        requirements = &ScreenPlayRequirements; break;
	case DestYcFile:        requirements = &FilePlayRequirements; break;
	case DestYcFrameBuffer: requirements = &YcFbPlayRequirements; break;
	}

	if (! IsInitialized(dpyInfo, SYM_DESTTYPE)
	 || ! AllInitialized(dpyInfo, requirements)) {
	    RejectCommand(pParse, id, MCS_BAD_CONFIGURE);
	    break;
	}

	if ((reason = OpenVideoInput(pSeq))) {
	    ReasonToClient(pSeq->pParse, reason);
	    break;
	}
	/*
	 * OpenAudio is called after OpenVideoInput and the
	 *	audio routines are responsible for using the same
	 *	MpegSystemInfo in the case of System Stream decoding
	 */
	if ((reason = OpenAudio(dpyInfo))) {
	    ReasonToClient(pSeq->pParse, reason);
	    break;
	}
	if (requirements == &FilePlayRequirements
	 && ((reason = OpenVideoOutput(pSeq)))) {
	    ReasonToClient(pSeq->pParse, reason);
	    break;
	}
	if (reason = ConfigureDisplayForSequence(dpyInfo)) {
	    ReasonToClient(pSeq->pParse, reason);
	    break;
	}
	if ((reason = ConfigureDecoder(pSeq))) {
	    ReasonToClient(pSeq->pParse, reason);
	    break;
	}
	dpyInfo->configured = True;
	(void) NewState(dpyInfo, DisplayLoading);

	break;
	
    case SYM_QUIT:
	if (dpyInfo->state != DisplayStopped) {
	    RejectCommand(pParse, id, MCS_BAD_RUNNING);
	    break;
	}
	NvrLmExit(0);
	/* NOTREACHED */

    case SYM_START:
	if (!dpyInfo->configured) {
	    RejectCommand(pParse, id, MCS_BAD_CONFIGURE);
	    break;
	}
	if (IsInitialized(dpyInfo, SYM_OUTFRAME)
	 && dpyInfo->currentFrame >= dpyInfo->outFrame) {
	    goto stopped;
	}

	if (dpyInfo->state == DisplayStopped)
	    (void) NewState(dpyInfo, DisplayRunning);
	else
	    Msg("start: currently %s\n",
		dpyInfo->state == DisplayRunning
		? "running" : "loading");
	break;

    case SYM_STOP:
	if (dpyInfo->state == DisplayRunning) {
	    (void) NewState(dpyInfo, DisplayStopped);
	}
	/*
	 * Even if we're already stopped, generate an event anyway.
	 */
    stopped:
	value.intvalue = dpyInfo->currentFrame;
	EventToClient(pSeq->pParse, MCS_EVENT_PAUSED,
		      &value, MCS_TYPE_INT);
	break;

    default:
	reason = CreateReason("unknown mcs command: <%d>\n", id);
	ReasonToClient(pSeq->pParse, reason);
	break;
    }

    if (reason)
	FreeReason(reason);
}

static void
AssignVar(ParseInfoPtr pParse, int id, ParseValuePtr value)
{
    SequenceInfoPtr pSeq = (SequenceInfoPtr)GetParsePrivateData(pParse);
    DisplayInfoPtr dpyInfo = pSeq->dpyInfo;
    boolean initialized = True;	/* usually the case */
    int badStatus = MCS_BAD_VALUE;
    char **ppath;

    if (dpyInfo->state == DisplayRunning) {
	RejectAssignment(pParse, id, MCS_BAD_RUNNING);
	return;
    }

    /*
     * First, do the assignment,
     * and for some, do something special.
     */
    switch(id) {
    case SYM_DESTTYPE:
	switch(value->intvalue) {
	case SYM_DPY_TO_YCFB: dpyInfo->new.destType = DestYcFrameBuffer; break;
	case SYM_DPY_TO_WIN:  dpyInfo->new.destType = DestWindow; break;
	case SYM_YCTOFILE:    dpyInfo->new.destType = DestYcFile; break;
	case SYM_DPY_TO_FB:   dpyInfo->new.destType = DestWinFrameBuffer; break;
	default:          initialized = False; break;
	}
	dpyInfo->new.changed |= DPY_Destination;
	break;

    case SYM_COLOR:
	switch(value->intvalue) {
	case SYM_GRAY1:   dpyInfo->new.color = GrayScale1; break;
	case SYM_GRAY7:   dpyInfo->new.color = GrayScale7; break;
	case SYM_GRAY8:   dpyInfo->new.color = GrayScale8; break;
	case SYM_RGB_232: dpyInfo->new.color = Rgb232; break;
	case SYM_RGB_332: dpyInfo->new.color = Rgb332; break;
	case SYM_RGB_888: dpyInfo->new.color = Rgb888; break;
	case SYM_RGB_444: dpyInfo->new.color = Rgb444; break;
	case SYM_RGB_555: dpyInfo->new.color = Rgb555; break;
	case SYM_RGB_C555:dpyInfo->new.color = RgbC555; break;
	case SYM_RGB_C666:dpyInfo->new.color = RgbC666; break;
	default:          initialized = False; break;
	}
	dpyInfo->new.changed |= DPY_Color;
	break;

    case SYM_FRAMEFREQ:
	if ((dpyInfo->frameFrequency = value->intvalue) < 0)
	    initialized = False;
	break;

    case SYM_OUTFRAME:
	if ((dpyInfo->outFrame = value->intvalue) < 0)
	    initialized = False;
	break;

    case SYM_INFRAME:
	if ((dpyInfo->inFrame = value->intvalue) < 0)
	    initialized = False;
	break;

    case SYM_VIDEOENABLE:
	dpyInfo->videoEnabled = (value->intvalue != 0);
	break;
    case SYM_AUDIOENABLE:
	dpyInfo->new.audioEnabled = (value->intvalue != 0);
	break;
    case SYM_AUDIODELAY:
	dpyInfo->audioDelay = value->intvalue;
	dpyInfo->absAudioDelay = abs(dpyInfo->audioDelay);
	break;
    case SYM_X:
	dpyInfo->new.dst.x = value->intvalue;
	dpyInfo->new.changed |= DPY_DestRect;
	break;
    case SYM_Y:
	dpyInfo->new.dst.y = value->intvalue;
	dpyInfo->new.changed |= DPY_DestRect;
	break;
    case SYM_WIDTH:
	if ((dpyInfo->new.dst.width = value->intvalue) <= 0)
	    initialized = False;
	else
	    dpyInfo->new.changed |= DPY_DestRect;
	break;
    case SYM_HEIGHT:
	if ((dpyInfo->new.dst.height = value->intvalue) <= 0)
	    initialized = False;
	else
	    dpyInfo->new.changed |= DPY_DestRect;
	break;
    case SYM_DISPLAY:
	if (dpyInfo->windowSysName)
	    free(dpyInfo->windowSysName);
	dpyInfo->windowSysName = value->string;
	break;
    case SYM_WINDOW:
	if ((dpyInfo->windowId = value->intvalue) <= 0) {
	    initialized = False;
	    badStatus = MCS_BAD_WINDOW;
	}
	break;
    case SYM_BITSERVER:

	if( dpyInfo->bitServer ) {
	    free(dpyInfo->bitServer);
	}
	CloseBitStream(pSeq->mpegBuf);
	dpyInfo->bitServer = value->string;
	break;
    case SYM_AUDIOINPATH:
	dpyInfo->reopenAudio = True;
	ppath = &dpyInfo->audioInput;
	goto processPath;
    case SYM_INPATH:
	dpyInfo->reopenVideo = True;
	ppath = &dpyInfo->videoInput;
	goto processPath;
    case SYM_OUTPATH:
	ppath = &dpyInfo->videoOutput;

    processPath:
	if (*ppath) {
	    /* even if the file is not different, we have to free one */
	    free(*ppath);
	}
	*ppath = value->string;
	break;

    case SYM_INTIME:
    case SYM_OUTTIME:
    {
	long timeSec, timeMicroSec;

	if (value->intvalue < 0)
	    break;
	timeSec = value->intvalue / MilliSecPerSecond;
	timeMicroSec = (value->intvalue - (timeSec * 1000)) * 1000;
	if (id == SYM_INTIME) {
	    dpyInfo->inTime.seconds = timeSec;
	    dpyInfo->inTime.microSeconds = timeMicroSec;
	} else {
	    dpyInfo->outTime.seconds = timeSec;
	    dpyInfo->outTime.microSeconds = timeMicroSec;
	}
	break;
    }
    case SYM_FRAME_RATE:
	if ((dpyInfo->frameRate = value->intvalue) < 0)
	    initialized = False;
	dpyInfo->absFrameRate = abs(dpyInfo->frameRate);
	break;
    case SYM_TOLERANCE:
	if (value->intvalue <= 0)
	    break;
	dpyInfo->tolerance = value->intvalue;
	break;
    case SYM_LOGFILE:
	if (value->string[0] != '/' || ! SetLogfile(value->string)) {
	    initialized = False;
	    badStatus = MCS_BAD_FILE;
	}
	free(value->string);
	break;

    case SYM_UNINIT:
	if (IsInitialized(dpyInfo, value->intvalue)) {
	    dpyInfo->configured = False;
	    UnmarkInitialized(dpyInfo, value->intvalue);
	}
	switch(id) {
	case SYM_AUDIOINPATH: CloseAudio(dpyInfo); break;
	case SYM_INPATH:      CloseBitStream(pSeq->mpegBuf); break;
	case SYM_OUTPATH:     CloseVideoOutput(pSeq); break;
	case SYM_BITSERVER:   CloseBitStream(pSeq->mpegBuf);
			      ResetBitstreamFuncs(pSeq->mpegBuf);
			      break;
	}
	break;

    case SYM_AUDIOQUALITY:
	if (value->intvalue < 1)
	    dpyInfo->audioQuality = 1;
	else if (value->intvalue > 32)
	    dpyInfo->audioQuality = 32;
	else
	    dpyInfo->audioQuality = value->intvalue;
	break;

    default:
	Fatal("unknown symbol assignment: <%d>=%d\n", id, value->intvalue);
	/* NOTREACHED */
    }

    if (initialized) {
	MarkInitialized(dpyInfo, id);
	dpyInfo->configured = False;
    } else {
	UnmarkInitialized(dpyInfo, id);
	RejectAssignment(pParse, id, badStatus);
    }
}

static void
LicenseDeath()
{
    if (TIME_TO_DIE) {
        ProgMsg("This alpha/beta version has expired.\n");
        ProgMsg("Please contact North Valley Research for a new copy.\n");
    }
    NvrLmExit (0);
}

#ifdef CBR_ENABLED
/*
 * Truely ugly.  The CBR library insists that c++ be used.  If we
 * can confirm that there are no static constructors, then the
 * main() function need not be written in c++ (see sgicbr.c)
 */
#define main CallCMain
#endif

void
main(int argc, char **argv)
{
    ParseConfig config;
    SequenceInfoPtr pSeq;
    DisplayInfoPtr dpyInfo;
    ParseInfoPtr pParse;
    extern read();

#ifdef GNU_MALLOC_DEBUG
    mcheck(0);	/* gnu malloc debugging */
#endif

    ProgramName = *argv;
    InitMacintosh("mpegdecomp");
    
    RequirementsInit();

    /*
     * Allocate storage for the mpeg compression context
     */
    pSeq = AllocSequence();
    dpyInfo = pSeq->dpyInfo;

    if (SaveContext(pSeq->licenseContext)) {
	LicenseDeath();
    }

    InitGeneral(pSeq, argc, argv);
    NewToken(STR_X,            INT_VARIABLE,      SYM_X);
    NewToken(STR_Y,            INT_VARIABLE,      SYM_Y);
    NewToken(STR_WIDTH,        INT_VARIABLE,      SYM_WIDTH);
    NewToken(STR_HEIGHT,       INT_VARIABLE,      SYM_HEIGHT);
    NewToken(STR_DISPLAY,      STRING_VARIABLE,   SYM_DISPLAY);
    NewToken(STR_WINDOW,       INT_VARIABLE,      SYM_WINDOW);
    NewToken(STR_INPATH,       STRING_VARIABLE,   SYM_INPATH);
    NewToken(STR_OUTPATH,      STRING_VARIABLE,   SYM_OUTPATH);
    NewToken(STR_AUDIOINPATH,  STRING_VARIABLE,   SYM_AUDIOINPATH);
    NewToken(STR_INTIME,       INT_VARIABLE,      SYM_INTIME);
    NewToken(STR_OUTTIME,      INT_VARIABLE,      SYM_OUTTIME);
    NewToken(STR_FRAME_RATE,   INT_VARIABLE,      SYM_FRAME_RATE);
    NewToken(STR_TOLERANCE,    INT_VARIABLE,      SYM_TOLERANCE);
    NewToken(STR_LOGFILE,      STRING_VARIABLE,   SYM_LOGFILE);
    NewToken(STR_METHOD,       SYMBOL_VARIABLE,   SYM_METHOD);
    NewToken(STR_INFRAME,      INT_VARIABLE,      SYM_INFRAME);
    NewToken(STR_OUTFRAME,     INT_VARIABLE,      SYM_OUTFRAME);
    NewToken(STR_FRAMEFREQ,    INT_VARIABLE,      SYM_FRAMEFREQ);
    NewToken(STR_AUDIOENABLE,  INT_VARIABLE,      SYM_AUDIOENABLE);
    NewToken(STR_VIDEOENABLE,  INT_VARIABLE,      SYM_VIDEOENABLE);
    NewToken(STR_AUDIODELAY,   INT_VARIABLE,      SYM_AUDIODELAY);
    NewToken(STR_DESTTYPE,     SYMBOL_VARIABLE,   SYM_DESTTYPE);
    NewToken(STR_COLOR,        SYMBOL_VARIABLE,   SYM_COLOR);
    NewToken(STR_UNINIT,       VARIABLE_VARIABLE, SYM_UNINIT);
    NewToken(STR_START,        COMMAND_TOKEN,     SYM_START);
    NewToken(STR_STOP,         COMMAND_TOKEN,     SYM_STOP);
    NewToken(STR_QUIT,         COMMAND_TOKEN,     SYM_QUIT);
    NewToken(STR_CONFIGURE,    COMMAND_TOKEN,     SYM_CONFIGURE);
    NewToken(STR_BITSERVER,    STRING_VARIABLE,   SYM_BITSERVER);
    NewToken(STR_AUDIOQUALITY, INT_VARIABLE,      SYM_AUDIOQUALITY);

    NewToken(STR_DPY_TO_FB,  SYMBOL_VALUE,    SYM_DPY_TO_FB);
    NewToken(STR_DPY_TO_WIN, SYMBOL_VALUE,    SYM_DPY_TO_WIN);
    NewToken(STR_DPY_TO_YCFB,SYMBOL_VALUE,    SYM_DPY_TO_YCFB);
    NewToken(STR_YCTOFILE,   SYMBOL_VALUE,    SYM_YCTOFILE);

    NewToken(STR_GRAY1,      SYMBOL_VALUE,    SYM_GRAY1);
    NewToken(STR_GRAY7,      SYMBOL_VALUE,    SYM_GRAY7);
    NewToken(STR_GRAY8,      SYMBOL_VALUE,    SYM_GRAY8);
    NewToken(STR_RGB_232,    SYMBOL_VALUE,    SYM_RGB_232);
    NewToken(STR_RGB_332,    SYMBOL_VALUE,    SYM_RGB_332);
    NewToken(STR_RGB_888,    SYMBOL_VALUE,    SYM_RGB_888);
    NewToken(STR_RGB_444,    SYMBOL_VALUE,    SYM_RGB_444);
    NewToken(STR_RGB_555,    SYMBOL_VALUE,    SYM_RGB_555);
    NewToken(STR_RGB_C555,   SYMBOL_VALUE,    SYM_RGB_C555);
    NewToken(STR_RGB_C666,   SYMBOL_VALUE,    SYM_RGB_C666);

    config.inFileHandle = 0;
    config.outFileHandle = 1;
    config.pointer = (PrivPointer)pSeq;
    config.asynchronous = True;
    config.server = True;
    config.inputNotify = ClientInputNotify;
    config.outputNotify = ClientOutputNotify;
    config.eofNotify = ClientEofNotify;
    config.u.server.assignNotify = AssignVar;
    config.u.server.commandNotify = ExecuteCommand;
    pParse = InitParsedConnection(&config);
    pSeq->pParse = pParse;

    AnnounceCopyright(pSeq);

    DecoderMainLoop(pParse);

    NvrLmExit(0);
}

#ifndef CBR_ENABLED

ImageReasonPtr
CBR_init(BitStream *mpegBuf, char *serverName)
{
    static ImageReason noBitServer = {
	"Constant bitrate server is not available with this decoder",
	IR_NotImplemented
    };

    return( &noBitServer );
}

#endif /* sgi */
