/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	motion.h
 *
 * DESCRIPTION
 *	declarations for motion compensation processing.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/motion.h,v 4.3 1994/01/27 23:20:22 toddb Exp $
 */
#ifndef _MOTION_H_
#define _MOTION_H_

/*
 * Adjust the x and y luma and chroma pel motion depending on whether this is
 * full or half pel.  The expression (x + ((x>>16)&1)) is equivalent
 * to x/2 and works for both negative and positive numbers.
 */
#define DivideByTwo(n)	(((n) + (((n)>>31)&1)) >> 1)

#define AdjustPelMotion(fullPel,xShft,yShft,x,y,cx,cy,halfY,halfC)	\
    {									\
	if (fullPel) {							\
	    cx = x >> 1;						\
	    cy = y >> 1;						\
	    halfC |= ((x&1) << xShft) | ((y&1) << yShft);		\
	} else {							\
	    cx = DivideByTwo(x);					\
	    cy = DivideByTwo(y);					\
	    halfC |= ((cx&1) << xShft) | ((cy&1) << yShft);		\
	    cx >>= 1;							\
	    cy >>= 1;							\
	    halfY |= ((x&1) << xShft) | ((y&1) << yShft);		\
	    x >>= 1;							\
	    y >>= 1;							\
	}								\
    }

/*
 * These motion macros compute the average of the past or future
 * or past/future pixels with rounding.  The rounding takes the form
 * of the "+ 1" in the expressions below.  They are intended for use
 * with IDct().
 */

#ifdef TRUNCATE_REAL
#define FwdBackMotion(clamp, out, assignOp, result, past, future) \
    {								\
	clamp(out, assignOp,					\
	      (*(past) + *(future)) >> 1,			\
	      result);						\
    }

#define MotionHalfVert(clamp, out, assignOp, result, time)	\
    {								\
	clamp(out, assignOp,					\
	       (CellInCol(time, 0, 0)				\
	      + CellInCol(time, 1, 0)) >> 1,			\
	      result);						\
    }

#define MotionHalfHoriz(clamp, out, assignOp, result, time)	\
    {								\
	clamp(out, assignOp,					\
	      ((time)[0] + (time)[1]) >> 1,			\
	      result);						\
    }

#define MotionHalf(clamp, out, assignOp, result, time)		\
    {								\
	clamp(out, assignOp,					\
	      (CellInCol(time,  0, 0)				\
	      + CellInCol(time, 1, 0)				\
	      + CellInCol(time, 0, 1)				\
	      + CellInCol(time, 1, 1)) >> 2,			\
	      result);						\
    }

#else /* TRUNCATE_REAL */

#define FwdBackMotion(clamp, out, assignOp, result, past, future) \
    {								\
	clamp(out, assignOp,					\
	      (*(past) + *(future) + 1) >> 1,			\
	      result);						\
    }

#define MotionHalfVert(clamp, out, assignOp, result, time)	\
    {								\
	clamp(out, assignOp,					\
	       (CellInCol(time, 0, 0)				\
	      + CellInCol(time, 1, 0)				\
	      + 1 /* effectively .5 */) >> 1,			\
	      result);						\
    }

#define MotionHalfHoriz(clamp, out, assignOp, result, time)	\
    {								\
	clamp(out, assignOp,					\
	      ((time)[0] + (time)[1] + 1) >> 1,			\
	      result);						\
    }

#define MotionHalf(clamp, out, assignOp, result, time)		\
    {								\
	clamp(out, assignOp,					\
	      (CellInCol(time,  0, 0)				\
	      + CellInCol(time, 1, 0)				\
	      + CellInCol(time, 0, 1)				\
	      + CellInCol(time, 1, 1)				\
	      + 2 /* effectively .5 */) >> 2,			\
	      result);						\
    }
#endif /* TRUNCATE_REAL */

/*
 * These macros perform loading from aligned/unaligned addresses
 * for 8 pels, or in the case of horizontal motion, 9 pels.
 * They are only used for copying blocks with the CopyMotion
 * macros down farther in this file.
 *
 * XXX this assumes big endian words
 *
 * with the given alignment the following macros accomplish
 * the stated load operation.
 *  - 'a' is word1 or word 11
 *  - 'b' is word2 or word 12
 *  - 'c' is word21
 *  - 'd' is word22
 *  - 'x' is a throwaway byte that is a byproduct of an unaligned memory read
 *
 * For vertical motion, word11 and word21 have the same byte alignment
 * but are in different rows:
 *
 * p&0x3 == 0	   aabb   word11
 *		   ccdd   word21
 * p&0x3 == 3	  xaabb   word11
 *		  xccdd   word21
 * p&0x3 == 2	 xxaabb   word11
 *		 xxccdd   word21
 * p&0x3 == 1	xxxaabb   word11
 *		xxxccdd   word21
 *
 * For horizontal motion, word11 and word21 are on the same row, but
 * word21 is one byte to the right of word11.
 *
 * p&0x3 == 0	   aabbx  word11
 *		   xccdd  word21
 * p&0x3 == 3	  xaabbx  word11
 *		  xxccdd  word21
 * p&0x3 == 2	 xxaabbx  word11
 *		 xxxccdd  word21
 * p&0x3 == 1	xxxaabbx  word11
 *		xxxxccdd  word21
 */
#define Load2ULongsAligned1(p, word1, word2)			\
    word1 = *ULongP(UCharP(p)-1);				\
    word2 = *ULongP(UCharP(p)+3);				\
    word1 = word1 << 8 | word2 >> 24;				\
    word2 = word2 << 8 | *(UCharP(p)+7);

#define Load2ULongsAligned2(p, word1, word2)			\
    word1 = UShortP(p)[0] << 16;				\
    word2 = *ULongP(UShortP(p)+1);				\
    word1 |= word2 >> 16;					\
    word2 = word2 << 16 | *UShortP(UCharP(p)+6);

#define Load2ULongsAligned3(p, word1, word2)			\
    word2 = *ULongP(UCharP(p)+1);				\
    word1 = *UCharP(p)<<24 | word2>>8;				\
    word2 = word2 << 24 | *ULongP(UCharP(p)+5)>>8;

#define Load2ULongsAligned0(p, word1, word2)			\
    word1 = ULongP(p)[0];					\
    word2 = ULongP(p)[1];

#define Load4ULongsAligned1(p, word11, word12, word21, word22)	\
    word11 = *ULongP(UCharP(p)-1);				\
    word12 = *ULongP(UCharP(p)+3);				\
    word21 = word11 << 16 | word12 >> 16;			\
    word11 = word11 << 8 | word12 >> 24;			\
    word22 = *UShortP(UCharP(p)+7) | word12 << 16;		\
    word12 = word12 << 8 | ((word22 >> 8) & 0xff);

#define Load4ULongsAligned2(p, word11, word12, word21, word22)	\
    word11 = UShortP(p)[0];					\
    word12 = *ULongP(UShortP(p)+1);				\
    word21 = word11 << 24 | word12 >> 8;			\
    word11 = word11 << 16 | word12 >> 16;			\
    word22 = *ULongP(UCharP(p)+6);				\
    word12 = word12 << 16 | word22 >> 16;			\
    word22 = word22 >> 8 | ((word12 << 8) & 0xff000000);

#define Load4ULongsAligned3(p, word11, word12, word21, word22)	\
    word21 = *ULongP(UCharP(p)+1);				\
    word22 = *ULongP(UCharP(p)+5);				\
    word11 = *UCharP(p)<<24 | word21>>8;			\
    word12 = word21 << 24 | word22>>8;

#define Load4ULongsAligned0(p, word11, word12, word21, word22)	\
    word11 = ULongP(p)[0];					\
    word12 = ULongP(p)[1];					\
    word21 = word11 << 8 | word12 >> 24;			\
    word22 = *(UCharP(p)+8) | word12 << 8;

/*
 * These two macros perform an average on two or for sets of numbers.
 * Each set of numbers are stored as four bytes in an unsigned long.
 * So one number from each set are added, divided by the size of the
 * set, and rounded to the nearest integer.  The result ends up
 * in an unsigned long.
 *
 * To cut computation time in half the four operations are split into
 * two: averaging the quantities in the first and third bytes, and
 * those in the second and fourth.  We can then add two longwords
 * that contain quantities separated by eight 0-bits and know
 * that overflow from the lower byte will not affect the higher.
 */
#define ComputeTwoSourceAverage(newValue, value1, value2)		\
    newValue = (((value1 & 0x00ff00ff)					\
	      + (value2 & 0x00ff00ff)					\
	      + 0x00010001)>>1) & 0x00ff00ff;				\
									\
    newValue |= (((value1 & 0xff00ff00)>>1)				\
	      + ((value2 & 0xff00ff00)>>1)				\
	      + 0x00800080) & 0xff00ff00;

#define ComputeFourSourceAverage(newValue, v1, v2, v3, v4)		\
    newValue = (((v1 & 0x00ff00ff)					\
	      + (v2 & 0x00ff00ff)					\
	      + (v3 & 0x00ff00ff)					\
	      + (v4 & 0x00ff00ff)					\
	      + 0x00020002)>>2) & 0x00ff00ff;				\
    newValue |= (((v1 & 0xff00ff00)>>2)					\
	      + ((v2 & 0xff00ff00)>>2)					\
	      + ((v3 & 0xff00ff00)>>2)					\
	      + ((v4 & 0xff00ff00)>>2)					\
	      + 0x00800080) & 0xff00ff00;

/*
 * These macros are for skipped blocks; i.e. the data needs only to
 * be copied from the future and/or past pictures.  Past, present,
 * and future are ULong quantities.
 */
#define CopyFwdBackMotion(pPast, pTo, pFuture, pastld, futureld)	\
    {									\
	int i;								\
	ULong past1, future1;						\
	ULong past2, future2;						\
	OctWord present;						\
	long index = 0;							\
									\
	for (i = 8;;) {							\
	    pastld(pPast+index, past1, past2);				\
	    futureld(pFuture+index, future1, future2);			\
	    ComputeTwoSourceAverage(present.l.high, past1, future1);	\
	    ComputeTwoSourceAverage(present.l.low, past2, future2);	\
	    *OctWordP(pTo+index) = present;				\
									\
	    if (--i <= 0)						\
		break;							\
	    index += MAX_X;						\
	}								\
    }

/*
 * XXX no longer used
 * this is a specific variant of CopyFwdBackMotion for skipped macroblocks
 * (to be distinguished from skipped blocks).  It assumes that
 * we are copying rgb pixels, all aligned on longword boundaries.
 */
#define MbCopyFwdBackMotion(pPast, pTo, pFuture)			\
    {									\
	int i;								\
	ULong past;							\
	ULong future;							\
	ULong present;							\
	int accel = MAX_X - 3*sizeof(ULong);				\
									\
	for (i = 16;;) {						\
	    past = *ULongP(pPast)++;					\
	    future = *ULongP(pFuture)++;				\
	    ComputeTwoSourceAverage(present, past, future);		\
	    *ULongP(pTo)++ = present;					\
									\
	    past = *ULongP(pPast)++;					\
	    future = *ULongP(pFuture)++;				\
	    ComputeTwoSourceAverage(present, past, future);		\
	    *ULongP(pTo)++ = present;					\
									\
	    past = *ULongP(pPast)++;					\
	    future = *ULongP(pFuture)++;				\
	    ComputeTwoSourceAverage(present, past, future);		\
	    *ULongP(pTo)++ = present;					\
									\
	    past = *ULongP(pPast);					\
	    future = *ULongP(pFuture);					\
	    ComputeTwoSourceAverage(present, past, future);		\
	    *ULongP(pTo) = present;					\
									\
	    if (--i <= 0)						\
		break;							\
									\
	    UCharP(pTo)     += accel;					\
	    UCharP(pPast)   += accel;					\
	    UCharP(pFuture) += accel;					\
	}								\
    }
	
#define CopyMotion(pTime, pTo, timeld)					\
    {									\
	OctWord time;							\
									\
	timeld(pTime +  0*MAX_X, time.l.high, time.l.low);		\
	*OctWordP(pTo + 0*MAX_X) = time;				\
	timeld(pTime +  1*MAX_X, time.l.high, time.l.low);		\
	*OctWordP(pTo + 1*MAX_X) = time;				\
	timeld(pTime +  2*MAX_X, time.l.high, time.l.low);		\
	*OctWordP(pTo + 2*MAX_X) = time;				\
	timeld(pTime +  3*MAX_X, time.l.high, time.l.low);		\
	*OctWordP(pTo + 3*MAX_X) = time;				\
	timeld(pTime +  4*MAX_X, time.l.high, time.l.low);		\
	*OctWordP(pTo + 4*MAX_X) = time;				\
	timeld(pTime +  5*MAX_X, time.l.high, time.l.low);		\
	*OctWordP(pTo + 5*MAX_X) = time;				\
	timeld(pTime +  6*MAX_X, time.l.high, time.l.low);		\
	*OctWordP(pTo + 6*MAX_X) = time;				\
	timeld(pTime +  7*MAX_X, time.l.high, time.l.low);		\
	*OctWordP(pTo + 7*MAX_X) = time;				\
    }

/*
 * XXX no longer used
 * Like MbCopyFwdBackMotion, above, this is a specific variant of
 * CopyMotion() for skipped macroblocks.  It assumes that
 * we are copying rgb pixels, all aligned on longword boundaries.
 */
#define MbCopyMotion(pTime, pTo)					\
    {									\
	int i;								\
	int accel = MAX_X - 3*sizeof(ULong);				\
									\
	for (i = 8;;) {							\
	    *ULongP(pTo)++ = *ULongP(pTime)++;				\
	    *ULongP(pTo)++ = *ULongP(pTime)++;				\
	    *ULongP(pTo)++ = *ULongP(pTime)++;				\
	    *ULongP(pTo)   = *ULongP(pTime);				\
									\
	    UCharP(pTo)   += accel;					\
	    UCharP(pTime) += accel;					\
									\
	    *ULongP(pTo)++ = *ULongP(pTime)++;				\
	    *ULongP(pTo)++ = *ULongP(pTime)++;				\
	    *ULongP(pTo)++ = *ULongP(pTime)++;				\
	    *ULongP(pTo)   = *ULongP(pTime);				\
									\
	    if (--i <= 0)						\
		break;							\
									\
	    UCharP(pTo)   += accel;					\
	    UCharP(pTime) += accel;					\
	}								\
    }

#define CopyMotionHalfVert(pTime, pTo, timeld)				\
    {									\
	int i;								\
	ULong time11, time12;						\
	ULong time21, time22;						\
	OctWord present;						\
	long index = 0;							\
									\
	timeld(pTime, time21, time22);					\
	for (i = 8;;) {							\
	    time11 = time21;						\
	    time12 = time22;						\
	    UCharP(pTime) += MAX_X;					\
	    timeld(pTime, time21, time22);				\
									\
	    ComputeTwoSourceAverage(present.l.high, time11, time21);	\
	    ComputeTwoSourceAverage(present.l.low, time12, time22);	\
	    *OctWordP(pTo) = present;					\
									\
	    if (--i <= 0)						\
		break;							\
									\
	    pTo += MAX_X;						\
	}								\
    }

#define CopyMotionHalfHoriz(pTime, pTo, timeld)				\
    {									\
	int i;								\
	ULong time11, time12;						\
	ULong time21, time22;						\
	OctWord present;						\
									\
	for (i = 8;;) {							\
	    timeld(pTime, time11, time12, time21, time22);		\
									\
	    ComputeTwoSourceAverage(present.l.high, time11, time21);	\
	    ComputeTwoSourceAverage(present.l.low, time12, time22);	\
	    *OctWordP(pTo) = present;					\
									\
	    if (--i <= 0)						\
		break;							\
	    UCharP(pTime) += MAX_X;					\
	    UCharP(pTo) += MAX_X;					\
	}								\
    }

#define CopyMotionHalf(pTime, pTo, timeld)				\
    {									\
	int i;								\
	ULong timeA11, timeA12;						\
	ULong timeA21, timeA22;						\
	ULong timeB11, timeB12;						\
	ULong timeB21, timeB22;						\
	OctWord present;						\
	long index = 0;							\
									\
	timeld(pTime, timeB11, timeB12, timeB21, timeB22);		\
	for (i = 8;;) {							\
	    timeA11 = timeB11;						\
	    timeA12 = timeB12;						\
	    timeA21 = timeB21;						\
	    timeA22 = timeB22;						\
									\
	    UCharP(pTime) += MAX_X;					\
	    timeld(pTime, timeB11, timeB12, timeB21, timeB22);		\
									\
	    ComputeFourSourceAverage(present.l.high,			\
				     timeA11, timeA21,			\
				     timeB11, timeB21);			\
	    ComputeFourSourceAverage(present.l.low,			\
				     timeA12, timeA22,			\
				     timeB12, timeB22);			\
	    *OctWordP(pTo) = present;					\
									\
	    if (--i <= 0)						\
		break;							\
	    UCharP(pTo) += MAX_X;					\
	}								\
    }

#define SIGNED_CLAMP_SIZE       (3*256)

extern UChar	*SignedClamp;

#endif /* _MOTION_H_ */
