/* $Header: /usr/local/src/nvrToMIT/RCS/mpegaudio.c,v 1.13 1996/01/10 23:21:35 toddb Exp $ */
/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpegaudio.c - functions for playing Mpeg files
 *
 * DESCRIPTION
 *
 */
#define NeedFunctionPrototypes 1

#include 	"vmpegdecomp.h"
#include	"MAcommon.h"
#include	"bitstream.h"

static struct {
    char *lastAudioInput;
    float samplePerUsec;	/* total sample rate - not per channel	*/
    float bytesPerUsec;		/* total byte rate - not per channel	*/
    MpegAudioInfoPtr mdec;
    TimerInc chunkOverhead;

    struct {
	float   decodeusec;
	ULong   minsamples;
    } quality[4];
} MpegState;
/*
 * 0:	Full quality
 * 1:	MonoSample only
 * 2:   QuarterSample only
 * 3:	MonoSample && QuarterSample
 */
#define minSamples \
        quality[  ((MpegState.mdec->quarterSample == True) << 1) \
		| (MpegState.mdec->monoSample == True)].minsamples
#define decodeUsecPerSample \
	quality[  ((MpegState.mdec->quarterSample == True) << 1) \
		| (MpegState.mdec->monoSample == True)].decodeusec

#define ShiftAudioBytesPast(ad, cBytes, dShift)				\
        {								\
	    int n = cBytes;						\
            if (n > (ad)->bufferLength) {				\
		n = (ad)->bufferLength;					\
	    }								\
	    (ad)->pastAudioSamples += n >> (dShift);			\
            if (n < (ad)->bufferLength) {				\
                memcpy((ad)->buffer,(ad)->buffer + n,			\
                       (ad)->bufferLength - n);				\
	    }								\
            (ad)->bufferLength -= n;					\
	    cBytes -= n;						\
            MpegState.mdec->nFifoEntries -= n >> (dShift);		\
	    WhenDebugEnabled(						\
                if (MpegState.mdec->nFifoEntries < 0)			\
                    Msg("ShiftAudioBytesPast botch\n");			\
           )								\
	}

#define ScaleStateMachine(ad, dShift, factor)				\
	{								\
	    (ad)->pastAudioSamples *= (factor);				\
	    (ad)->nullSamplesToPlay += (ad)->bufferLength >> (dShift);	\
	    (ad)->nullSamplesToPlay *= (factor);			\
	    (ad)->bufferLength = 0;					\
	    MpegState.mdec->nFifoEntries = 0;				\
	}

static boolean PickInitialMpegAudio(DisplayInfoPtr);

/*
 * Externs
 */
extern CacheReturnValue HeaderAndErrorCheck(CacheProtoArgs, MpegAudioInfoPtr);
extern Inline int FrameCheck(MpegAudioInfoPtr);


#define LAYERII_FRAME_SIZE	1152

/*
 * Currently the measurement for the audio overhead is taken once at
 * the very beginning of the sequence.  However, this tends to have
 * less work to do.  Hence we inflate the value here.
 */
#define AUDIO_OVERHEAD_INFLATE	1.00

/*
 * Number of 16-bit samples (1/2 second stereo @ 48KHz)
 *	rounded up to integral # of layerII framesize chunks
 */
#define FIFOSIZE		96768
#define EXTRAFIFOSIZE           (FIFOSIZE - FIFO_SIZE)

/*
 * Must be callable at least twice in a row without an 
 * intervening update() works until gather_samps samples are in the buffer,
 * so if some were already there it will only gather new samples up to the
 * difference
 */
static boolean
gatherSamples(long gather_samps)
{
    DisplayInfoPtr dpyInfo = (DisplayInfoPtr)MpegState.mdec->priv;
    AudioInfoPtr audioData = &dpyInfo->audioData;
    int dataShift = DataFormToShift[ audioData->cvt.srcAu.dataForm ];
    DebugDeclare(long lastNFifoEntries = 0;)

    if( audioData->bufferLength == 0 ) {
	/*
	 * audioUpdate() has been called and all samples were written out.
	 * Otherwise this is the from first PreloadMpegAudio() and we need more 
	 * audio data to satisfy the first request
	 */
	audioData->pastAudioSamples += MpegState.mdec->nFifoEntries;
	MpegState.mdec->nFifoEntries = 0;
    }

    DebugTask(N_AudioActivity, lastNFifoEntries = MpegState.mdec->nFifoEntries)

    if (MAgatherSamples(MpegState.mdec, gather_samps) == False) {
	return(False);
    }
    audioData->bufferLength = MpegState.mdec->nFifoEntries << dataShift;

    DebugTask(N_AudioActivity,
	      Msg("gather=%d, had=%d, total=%d\n",
		  gather_samps, lastNFifoEntries,
		  audioData->bufferLength >> dataShift);
    )

    return(True);
}


void
CloseMpegAudio(DisplayInfoPtr dpyInfo)
{
    if (MpegState.mdec) {
	CloseBitStream(MpegState.mdec->bitStream);
	free((char *)MpegState.mdec->bitStream);
	free((char *)MpegState.mdec);
	MpegState.mdec = NULL;
	if( dpyInfo->audioData.buffer ) {
	    dpyInfo->audioData.buffer = NULL;
	    dpyInfo->audioData.bufferLength = 0;
	}
    }
}

static void
AudioStreamError(BitStream*buf, BitstreamCondition condition)
{
    DisplayInfoPtr dpyInfo = (DisplayInfoPtr)MpegState.mdec->priv;
    RestoreContext(dpyInfo->pSeq->decodeContext, 0);
}

/*
 * We know the audio File is opened before the audio device
 * This routine is responsible for closing any previously opened
 *	audio file.
 */
boolean
OpenMpegAudio(DisplayInfoPtr dpyInfo)
{
    long i, fifoSize;
    AudioInfoPtr audioData = &dpyInfo->audioData;
    int dataShift = DataFormToShift[ audioData->cvt.srcAu.dataForm ];

    if (MpegState.mdec)
	Fatal("OpenMpegAudio: already open\n");

    DebugTask(N_AudioActivity, Msg("mpegOpen\n"))

    memset((char *)&MpegState.quality,0,sizeof(MpegState.quality));
    /*
     * make fifoSize an integral number of layerII framesizes
     *	and large enough to hold 1/2 sec of 48KHz stereo samples
     */
    fifoSize = FIFOSIZE;
    i = sizeof(MpegAudioInfo)
    	+ sizeof(short) * (EXTRAFIFOSIZE);
    MpegState.mdec = (MpegAudioInfoPtr) InsecureAlloc(i);
    if (MpegState.mdec == NULL) {
	ProgMsg("can't allocate space for mpeg audio\n");
	return(False);
    }

    MpegState.lastAudioInput = dpyInfo->audioInput;
    audioData->pastAudioSamples = 0;
    audioData->bufferLength = 0;
    audioData->nullSamplesToPlay = 0;

    memset((char *)MpegState.mdec, 0, sizeof(MpegAudioInfo));
    MpegState.mdec->priv = dpyInfo;
    MpegState.mdec->fifoSize = fifoSize;

    MpegState.mdec->bitStream = (BitStreamPtr)memalign(8, sizeof(BitStream));
    memset((char *)MpegState.mdec->bitStream, 0, sizeof(BitStream));

    /*
     * XXX Before we decode the first chunk, make a guess here whether
     * we are going to have to force low quality or not.  We only have
     * hardware that supports every format, or only support mu-law.
     * If we had hardware that supported only some of the formats need
     * for MPEG, then we would need to look at the bits stream before
     * anything was decoded to decide whether we should force 8kHz.
     */
    MpegState.mdec->frameHeader.s.mode = MODE_STEREO;
    MpegState.mdec->frameHeader.s.sampling_frequency = 0; /* 44.1 kHz */
    MpegState.mdec->bitStream->errorFunc = AudioStreamError;
    audioData->buffer = (UChar *)MpegState.mdec->fifo;

    if (! PickInitialMpegAudio(dpyInfo))
	return(False);

    if( ! NewMaSetupDecode(MpegState.mdec,
			   dpyInfo->audioInput,
			   dpyInfo->pSeq->mpegBuf) ) {
	return(False);
    }

    /* decode initial frame */
    if( gatherSamples(1) == False ) {
	return(False);
    }

    MpegState.minSamples = audioData->bufferLength >> dataShift;

    /*
     * Now set the quality again, after we have read the bitstream
     */
    if (! PickInitialMpegAudio(dpyInfo))
	return(False);

    return(True);
}




/*
 * Until the audio bitstream seeking is integrated, InitAndGather must
 *	callable N times in a row to gather timing/decode statistics for 
 *	N quality levels. Also for now the amount asked to decode must
 *	be kept small enough so that N * <decode time> will fit into
 *	fifosize. for 4 quality levels, max decode == 1/8 sec.
 */
static void
InitAndGather(DisplayInfoPtr dpyInfo, long gather)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    float tmp, videoTime, audioTime, playTime;
    ULong gatherSamps;
    int dataShift = DataFormToShift[ audioData->cvt.srcAu.dataForm ];
    DebugDeclare(TimerInc timer)
    DebugDeclare(char *errFunc = "InitAndGather")

    DebugTask(N_AudioActivity,
	    Msg("%s: ",errFunc);
    )

    /* if decode timing is unknown for this quality level, gather stats */
    if (   MpegState.decodeUsecPerSample == 0.0 
	|| MpegState.minSamples == 0) {
	ULong overHead = 0;
	ULong decodeSamps;

	decodeSamps = audioData->bufferLength >> dataShift;
	UpdateClock(dpyInfo->pSeq,NULL);
	gatherSamples(decodeSamps+1);
	UpdateClock(dpyInfo->pSeq,&overHead);
	decodeSamps = ((audioData->bufferLength >> dataShift)) - decodeSamps;

	/* usec audio decode time / audio sample */
	MpegState.decodeUsecPerSample = 1.0 * overHead / decodeSamps;
	
	MpegState.minSamples = decodeSamps;
    }

    /* usec to play 1 video frame */
    videoTime = MicroSecPerSec
		/ FrameRateTable[ dpyInfo->pSeq->pictureRateCode ];

    /* usec to decode audio for one video frame */
    dpyInfo->pSeq->audioPicOverhead = AUDIO_OVERHEAD_INFLATE *
				      MpegState.decodeUsecPerSample *
				      MpegState.samplePerUsec * videoTime;

    /*
     * discover the positive integral number of MpegState.minSamples 
     * for the requested decode frequency
     */
    tmp = 1.0 * audioData->cvt.srcAu.rate * audioData->cvt.srcAu.channels
    	  / (MpegState.minSamples * audioData->schedRate);
    if( tmp < 1.0 ) {
	tmp = 1.0;
    }
    audioData->samplesPerCall = ((long)tmp) * MpegState.minSamples;

    /* usec audio play time produced for each call */
    audioTime = 1.0 * audioData->samplesPerCall / MpegState.samplePerUsec;

    DebugTask(N_AudioActivity,
	      /* calculate Usec per call */
	      tmp = audioTime;
	      timer.increment = (long)tmp;
	      tmp -= timer.increment;
	      timer.denom = MicroSecPerSec;
	      timer.remainder = (long)(tmp * timer.denom);
	      timer.accumulator = 0;
    )

    /* audio timer - when we're called to decode audio */
    InitTimerInc(&audioData->timer,
		 (double) (1.0 * audioData->cvt.srcAu.rate
			   * audioData->cvt.srcAu.channels
			   / audioData->samplesPerCall),
		 MicroSecPerSec);

    DebugTask(N_AudioActivity,
	if (audioData->timer.increment != timer.increment
	 || audioData->timer.denom != timer.denom
	 || audioData->timer.remainder != timer.remainder
	 || audioData->timer.accumulator != timer.accumulator)
	    Msg("new/old timer:\n\tinc,remain,denom = %d/%d, %d/%d, %d/%d\n",
		audioData->timer.increment, timer.increment,
		audioData->timer.remainder, timer.remainder,
		audioData->timer.denom, timer.denom);
    )

    DebugTask(N_AudioActivity,
	      tmp = 1.0 * dpyInfo->pSeq->audioPicOverhead *
		    audioTime / videoTime;
	      timer.increment = (long)tmp;
	      tmp -= timer.increment;
	      timer.denom = MicroSecPerSec;
	      timer.remainder = (long)(tmp * timer.denom);
	      timer.accumulator = 0;
    )

    InitTimerInc(&MpegState.chunkOverhead,
	         (double) (1.0 * MicroSecPerSec * videoTime /
			   (dpyInfo->pSeq->audioPicOverhead * audioTime)),
                 MicroSecPerSec);

    DebugTask(N_AudioActivity,
	if (MpegState.chunkOverhead.increment != timer.increment
	 || MpegState.chunkOverhead.denom != timer.denom
	 || MpegState.chunkOverhead.remainder != timer.remainder
	 || MpegState.chunkOverhead.accumulator != timer.accumulator)
	    Msg("new/old chnk ovrhd:\n\tinc,rem,denom = %d/%d, %d/%d, %d/%d\n",
		audioData->timer.increment, timer.increment,
		audioData->timer.remainder, timer.remainder,
		audioData->timer.denom, timer.denom);
        Msg("%s: picOverhead=%d\n", errFunc, dpyInfo->pSeq->audioPicOverhead); 
        Msg("\taudioTimer incr=%d, denom=%d, rem=%d\n",
	    audioData->timer.increment,
	    audioData->timer.denom, audioData->timer.remainder);
        Msg("\tchunkOverhead inc=%d, denom=%d, rem=%d\n",
	    MpegState.chunkOverhead.increment, MpegState.chunkOverhead.denom,
	    MpegState.chunkOverhead.remainder);
    )

    gatherSamps = MpegState.samplePerUsec * gather;
    if( MpegState.mdec->nFifoEntries >= gatherSamps ) {
	DebugTask(N_AudioActivity,
	    Msg("%s: gather=%d, have=%d - ignoring request\n",errFunc,
		gatherSamps, MpegState.mdec->nFifoEntries);
	)
	return;
    }
    if ((audioData->bufferLength >> dataShift) < gatherSamps) {
	gatherSamples(gatherSamps);
    }

}


/*
 * Gather specified number of audio frames decribed in timeGather.
 * Always return the amount of data decoded.
 */
boolean
LoadNextMpegAudio(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    long gatherSamps;
    DebugDeclare(char *errFunc = "LoadNextMpegAudio";)

    dpyInfo->pSeq->workToDo -= TimerIncrementMacro(MpegState.chunkOverhead);

    if( audioData->nullSamplesToPlay > 0 ) {
	/*
	 * Had more null samples than would fit in the audio buffer
	 *	 at the last audioUpdate()
	 */
	return(True);
    }

    DebugTask(N_AudioActivity,
	Msg("%s: ", errFunc);
    )
    if( gatherSamples(audioData->samplesPerCall) == False ) {
	DebugTask(N_AudioActivity,
		  int dataShift =
			DataFormToShift[ audioData->cvt.srcAu.dataForm ];
		  Msg("can't gather, want=%d, have=%d\n",
		      gatherSamps, (audioData->bufferLength >> dataShift)))
	return(False);
    }

    audioData->timeGather = TimerIncrementMacro(audioData->timer);
    
    return(True);
}

/*
 * If timeSeek is specified, seek to that offset in the file before gathering.
 * If timeSeek is >=0, we are decoding after stop.
 * Otherwise, we are being called just after the
 * mpeg stream was (re)opened.
 * Always return the amount of data decoded.
 */
boolean
PreloadMpegAudio(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    int dataShift = DataFormToShift[ audioData->cvt.srcAu.dataForm ];
    long skipSamples;
    long gatherSamps;
    int where = -1;
    char *errFunc = "PreloadMpegAudio";

    /* Until seeking is implemented make all gathers <= 1/8 sec */
    if (audioData->timeGather > 125000) {
	audioData->timeGather = 125000;
    }
    WhenDebugEnabled (
        Msg("\n%s: seek=%d gather=%d; ",errFunc,audioData->timeSeek,
	    audioData->timeGather );
        Msg("\n\tIn: past=%d null=%d blen=%d spc=%d; ",
	    audioData->pastAudioSamples, audioData->nullSamplesToPlay, 
	    audioData->bufferLength, audioData->samplesPerCall);
    )


    if (audioData->timeSeek == 0) {
	if (audioData->pastAudioSamples == 0) {
	  /* We are in the preload phase after (re)opening the audio stream */
	    InitAndGather(dpyInfo, audioData->timeGather);
	    audioData->pastAudioSamples = 0;
	    WhenDebugEnabled(
	        Msg("\n\tOut: past=%d null=%d blen=%d spc=%d;\n",
		    audioData->pastAudioSamples, audioData->nullSamplesToPlay,
		    audioData->bufferLength, audioData->samplesPerCall);
            )
	    return(True);
	}
	/*
	 * we will be out-of-sync now because no pictures made it into
	 *	the display queue during last play, and we can't tell what the
	 *	next picture to be decoded will be, so guess based on
	 *	currentFrame or set to 0
	 */
	audioData->timeSeek = dpyInfo->currentFrame * MicroSecPerSec
		/ FrameRateTable[ dpyInfo->pSeq->pictureRateCode ];
	WhenDebugEnabled(
		ProgMsg("%s: lost audio sync due to decode starvation\n",
			errFunc);
		Msg("    resyncing on frame %d\n",dpyInfo->currentFrame);
        )
		
    } else if( audioData->pastAudioSamples == 0 ) {
	WhenDebugEnabled(
	    ProgMsg("%s: bad preload state(1) you're on your own\n", errFunc);
        )
    }

    /* we are in the preload phase in the middle of a sequence */
    /* or we were called from audioClose() to decode more data */
    skipSamples = audioData->timeSeek * MpegState.samplePerUsec;
    gatherSamps = audioData->timeGather * MpegState.samplePerUsec;
    where = skipSamples - audioData->pastAudioSamples;
    DebugTask(N_AudioActivity,
	      Msg("%s: get=%d have=%d skip=%d seen=%d null=%d\n",
		  errFunc, gatherSamps,
		  (audioData->bufferLength >> dataShift),
		  skipSamples, audioData->pastAudioSamples,
		  audioData->nullSamplesToPlay);
    )

    WhenDebugEnabled(
        Msg("\n\tnskip=%d ngather=%d bdelta=%d; ", skipSamples, gatherSamps,
	    where);
    )

    if (where < 0) {
	/*
	 * If we are to play somewhere past the beginning of the file
	 *	but before the samples we have, just play null samples
	 *	until we get to where we're supposed to be
	 */
	audioData->nullSamplesToPlay = -(where);
	DebugTask(N_AudioActivity,
	    Msg("%s: play %d null samples\n",
		errFunc, audioData->nullSamplesToPlay))
	/* still verify gatherSamps worth of data ... */
    } else if (where > 0) {
	/* copy the bytes */
	int copyBytes = where << dataShift;

	if( copyBytes > audioData->bufferLength ) {
	    WhenDebugEnabled(
	        Msg("lookahead decoding ... ");
            )
								      
	    while (copyBytes > 0) {
		ShiftAudioBytesPast(audioData, copyBytes, dataShift);
		if( ! gatherSamples(LAYERII_FRAME_SIZE)) {
                    return(False);
                }
	    }
	}
	/* still verify gatherSamps worth of data ... */
    }

    /* make sure there are gatherSamps worth in the buffer */
    if( ! gatherSamples(gatherSamps) ) {
	return(False);
    }
done:
    WhenDebugEnabled(
        Msg("\n\tOut: past=%d null=%d blen=%d spc=%d;\n",
	    audioData->pastAudioSamples, audioData->nullSamplesToPlay, 
	    audioData->bufferLength, audioData->samplesPerCall);
    )
    return(True);
}

/*
 * same as SetQualityMpegAudio() except without the
 *	modifications of state variables and audioPicoverhead
 */
static boolean
PickInitialMpegAudio(DisplayInfoPtr dpyInfo)
{
    long interestingFormats;
    AudioInfoPtr audioData = &dpyInfo->audioData;
    InternalReasonPtr irp;
    char *errFunc = "PickInitialMpegAudio";
    int srcShift, dstShift;

    /*
     * If we cross QUALITY_RATE_CHANGE, then we may need to
     * change rate, channels, dataForm
     */
    audioData->cvt.srcAu.rate = MA_SamplingFrequencyTable[
			MpegState.mdec->frameHeader.s.sampling_frequency];
    /*
     * MPEG decoder doesn't do ulaw conversion.
     */
    audioData->cvt.srcAu.dataForm = Audio16BitPcm;

    if ( dpyInfo->audioQuality > 24 ) {         /* full qual */
	MpegState.mdec->quarterSample = False;
	MpegState.mdec->monoSample = False;
    } else if ( dpyInfo->audioQuality > 16  ) {  /* full mono*/
	MpegState.mdec->quarterSample = False;
	MpegState.mdec->monoSample = True;
    } else if ( dpyInfo->audioQuality > 8 ) {  /* 1/4 stereo */
	MpegState.mdec->quarterSample = True;
	MpegState.mdec->monoSample = False;
    } else {                                      /* 1/4 mono */
	MpegState.mdec->quarterSample = True;
	MpegState.mdec->monoSample = True;
    }
    if (MpegState.mdec->quarterSample)
	audioData->cvt.srcAu.rate >>= 2;

    switch(MpegState.mdec->frameHeader.s.mode) {
    case MODE_STEREO:
    case MODE_JOINT_STEREO:
    case MODE_DUAL_CHANNEL:
	if (MpegState.mdec->monoSample)
	    audioData->cvt.srcAu.channels = AudioMono;
	else
	    audioData->cvt.srcAu.channels = AudioStereo;
	break;

    case MODE_SINGLE_CHANNEL:
	audioData->cvt.srcAu.channels = AudioMono;
	break;

    default:
	WhenDebugEnabled(
            ProgMsg("%s: unknown channel mode 0x%x\n",
		    errFunc, MpegState.mdec->frameHeader.s.mode);
	)
	return(False);
    }

    irp = ChooseAudioForm(&audioData->cvt.srcAu,
			  audioData->formatsAvailable,
			  &audioData->cvt.dstAu);
    if (irp) {
	ShowReason(Msg, irp);
	FreeReason(irp);
	return(False);
    }
    /* dataForm matches, but rate may just be close */

    irp = InitAudioConversion(&audioData->cvt,
			      &audioData->cvt.srcAu,
			      &audioData->cvt.dstAu);
    if (irp) {
	ShowReason(Msg, irp);
	FreeReason(irp);
	return(False);
    }

    /* initialize convert structure members */
    srcShift =  DataFormToShift[ audioData->cvt.srcAu.dataForm ];
    dstShift =  DataFormToShift[ audioData->cvt.dstAu.dataForm ];
    audioData->cvt.buffer = audioData->buffer;
    audioData->cvt.bufferLength = MpegState.mdec->fifoSize << 1;
    audioData->cvt.maxInSamples = audioData->cvt.bufferLength >> srcShift;
    audioData->cvt.maxOutSamples = audioData->cvt.bufferLength >> dstShift;
    audioData->cvt.outIsBiggerThanIn =
	 (  (audioData->cvt.dstAu.rate  /
	     (audioData->cvt.dstAu.channels << dstShift))
	  > (audioData->cvt.srcAu.rate  /
	     (audioData->cvt.srcAu.channels << srcShift)) );

    MpegState.samplePerUsec = 1.0 * audioData->cvt.srcAu.channels
			          * audioData->cvt.srcAu.rate
			          / MicroSecPerSec;
    MpegState.bytesPerUsec = 1.0 * audioData->cvt.srcAu.channels
                                 * audioData->cvt.srcAu.rate
                             * DataFormToBytes[audioData->cvt.srcAu.dataForm]
                                 / MicroSecPerSec;


    return(True);

}

boolean
SetQualityMpegAudio(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    AudioInfo oldAudioData;
    boolean oMonoSample = MpegState.mdec->monoSample;
    boolean oQuarterSample = MpegState.mdec->quarterSample;
    extern void SetupSynFilter(MpegAudioInfoPtr);
    char *errFunc = "SetQualityMpegAudio";

    oldAudioData = *audioData;

    if( ! PickInitialMpegAudio(dpyInfo) ) {
	return(False);
    }

    /*
     * If the sample characteristics have changed, then
     *	adjust the state variables
     *	pastAudioSamples, nullSamplesToPlay, bufferLength
     */
    if (   oMonoSample != MpegState.mdec->monoSample
	|| oQuarterSample != MpegState.mdec->quarterSample
	|| oldAudioData.cvt.srcAu.rate != audioData->cvt.srcAu.rate
	|| oldAudioData.cvt.srcAu.channels != audioData->cvt.srcAu.channels
	|| oldAudioData.cvt.srcAu.dataForm != audioData->cvt.srcAu.dataForm
    ) {
	int oDataShift = DataFormToShift[ oldAudioData.cvt.srcAu.dataForm ];
	int dataShift = DataFormToShift[ audioData->cvt.srcAu.dataForm ];
	float factor = (float)(audioData->cvt.srcAu.channels
			       * audioData->cvt.srcAu.rate)
			/ (oldAudioData.cvt.srcAu.channels
			   * oldAudioData.cvt.srcAu.rate);

	WhenDebugEnabled(
            Msg("\n%s: quality decoding change. ",errFunc);
            Msg("\n\tIn: past=%d null=%d blen=%d spc=%d; ",
		audioData->pastAudioSamples, audioData->nullSamplesToPlay, 
		audioData->bufferLength, audioData->samplesPerCall);
        )

	DebugTask(N_AudioActivity,
		  Msg("SetQualityMpegAudio: audio characteristics changed\n");
		  Msg("    adjusting state variables. factor = %.2f\n",
 		      factor);
        )

	/*
	 * modify pastAudioSamples, nullSamplesToPlay
	 * and bufferLength to reflect new frame size
	 */
	ScaleStateMachine(audioData, oDataShift, factor);

	/* audioData->samplesPerCall will be reset in InitAndGather */
	
	DebugTask(N_AudioActivity,
		   Msg("    past=%d->%d, have=%d->%d, null=%d->%d\n",
		       oldAudioData.pastAudioSamples,
		       audioData->pastAudioSamples, 
		       oldAudioData.bufferLength >> oDataShift,
		       audioData->bufferLength >> dataShift,
		       oldAudioData.nullSamplesToPlay,
		       audioData->nullSamplesToPlay);
		   )

	InitAndGather(dpyInfo, 1);
	SetupSynFilter(MpegState.mdec);

	WhenDebugEnabled(
            Msg("\n\tOut: past=%d null=%d blen=%d spc=%d;\n",
		audioData->pastAudioSamples, audioData->nullSamplesToPlay, 
		audioData->bufferLength, audioData->samplesPerCall);
        )

    }
    return (True);
}
