/* $Header: /usr/local/src/nvrToMIT/RCS/sgiaudio.c,v 1.12 1995/08/18 19:07:59 russ Exp $ */
/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	sunmpeg.c - sun functions for playing Mpeg files
 *
 * DESCRIPTION
 *
 */
#define NeedFunctionPrototypes 1
#define RUSS

#include	<errno.h>
#include	<fcntl.h>
#include 	<audio.h>
#include 	<audiofile.h>
#include 	<sys/schedctl.h>
#include 	<stdio.h>
#include 	<math.h>
#include 	"vmpegdecomp.h"
 
/* State Variables */

#define _ALwritesamps(fp,b,n) ALwritesamps(fp,b,n)

static struct {
    boolean rude;
    ALport fp;
} SgiAuState = { True };

static int
GetInputRate()
{
    long buf[6];

    buf[0] = AL_INPUT_RATE;
    buf[2] = AL_INPUT_SOURCE;
    buf[4] = AL_DIGITAL_INPUT_RATE;
    ALgetparams(AL_DEFAULT_DEVICE, buf, 6);

    /* we are clocked off digital input. find real input rate, if possible. */
    if	((buf[1] == AL_RATE_AES_1)||(buf[3] == AL_INPUT_DIGITAL)) {
	if (ALgetdefault(AL_DEFAULT_DEVICE, AL_DIGITAL_INPUT_RATE) >= 0) 
	    return (buf[5]);
    }
    /* input rate is Hz and we're using an analog input */
    else if (buf[1] > 0) 
	return (buf[1]);

    return (AL_RATE_UNDEFINED);
}

static int
GetOutputRate()
{
    long buf[4];

    buf[0] = AL_OUTPUT_RATE;
    buf[2] = AL_DIGITAL_INPUT_RATE;
    ALgetparams(AL_DEFAULT_DEVICE, buf, 4);

    /* output rate is in Hz -- return it */
    if (buf[1] > 0) 
	return (buf[1]);

    /* output rate is logical -- track down what it means */
    if (buf[1] == AL_RATE_AES_1) {
	/* we are clocked off of digital input. find
	 * real input rate, if system supports this ability.
	 * Otherwise, return AL_RATE_UNDEFINED */
	if (ALgetdefault(AL_DEFAULT_DEVICE,AL_DIGITAL_INPUT_RATE) >= 0) 
	    return (buf[3]);
    } else if (buf[1] == AL_RATE_INPUTRATE) 
	return (GetInputRate());
    return (AL_RATE_UNDEFINED);
}

static void
audioInit(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;

    /*
     * We can do just about anything except Avail8BitULaw
     */
    audioData->formatsAvailable = Avail24BitPcm
				| Avail16BitPcm
				| Avail8BitPcm
				| Avail48Khz
				| Avail44_1Khz
				| Avail32Khz
				| Avail16Khz
				| Avail8Khz
				| AvailStereo
				| AvailMono;
}

static void
audioClose(DisplayInfoPtr dpyInfo)
{
    if (SgiAuState.fp) {
	ALcloseport(SgiAuState.fp);
	SgiAuState.fp = NULL;
    }
}

static boolean
audioOpen(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    ALconfig audioDevFp_config;
    long pvbuf[4];
    long devOutputRate;
    int err;
    int	sampleWordsize;    	/* size of sample (AL_SAMPLE_*) */
    char *errFunc = "audioOpen";


    audioClose(dpyInfo);

    /*
     * need to determine whether audio is in use. if not, then we
     * can just go ahead and be "rude."
     */
    pvbuf[0] = AL_OUTPUT_COUNT;
    pvbuf[2] = AL_MONITOR_CTL;
    if (ALgetparams(AL_DEFAULT_DEVICE, pvbuf, 4) < 0) {
	if (oserror() == AL_BAD_DEVICE_ACCESS) {
	    ProgMsg("Can't play -- could not access audio hardware.\n");
	    return(False);
	}
    }
    
    
    if ((pvbuf[1] == 0) && (pvbuf[3] == AL_MONITOR_OFF)) {
        SgiAuState.rude = 1;
    }
    
    /* 
     * determine the output rate for the audio hardware
     */
    devOutputRate = GetOutputRate();
    
    /*
     * if the rates are the same, all is well. if not, then we need to proceed
     * in a either a "rude" or "polite" manner.
     */
    
    if (devOutputRate != audioData->cvt.dstAu.rate)
    {
        /*
         *  we set the rate to our value in Hz.  ALsetparams will set
	 *  the hardware to the closest rate to audioData->cvt.dstAu.rate
	 *  which the hardware supports.
         */
	
        if (SgiAuState.rude)
        {
            devOutputRate = audioData->cvt.dstAu.rate;
            pvbuf[0] = AL_OUTPUT_RATE;
            pvbuf[1] = devOutputRate;
            ALsetparams(AL_DEFAULT_DEVICE, pvbuf, 2);
	    if( GetOutputRate() != devOutputRate ) {
		ProgMsg("Could not set audio device output rate to %d\n",
			devOutputRate);
		return(False);
	    }
        }
        else                    /*polite*/
        {
	    DebugTask(N_AudioActivity,
		Msg("%s: '%s':  Adjust output rate ",
		    errFunc, dpyInfo->audioInput);
                if (devOutputRate < 0) 
		    Msg("[currently set to undetermined digital rate]\n");
                else 
		    Msg("[currently %d Hz]\n", devOutputRate);
            )
	    return(False);
        }
    }

    /*
     * configure and open audio port
     */
    audioDevFp_config = ALnewconfig();

    switch (audioData->cvt.dstAu.dataForm) {
    case Audio24BitPcm:	sampleWordsize = AL_SAMPLE_24; break;
    case Audio16BitPcm:	sampleWordsize = AL_SAMPLE_16; break;
    case Audio8BitPcm:	sampleWordsize = AL_SAMPLE_8; break;
    default:
	return(False);
    }

    ALsetwidth(audioDevFp_config, sampleWordsize);
    ALsetchannels(audioDevFp_config, audioData->cvt.dstAu.channels);
    
    /*
     * make the ring buffer large enough to hold 1 sec of audio samples
     */
    ALsetqueuesize(audioDevFp_config,
		   audioData->cvt.dstAu.rate * audioData->cvt.dstAu.channels);
    SgiAuState.fp = ALopenport(ProgramName, "w", audioDevFp_config);
    if (SgiAuState.fp == 0) {
	err = oserror();
	if (err == AL_BAD_NO_PORTS) {
	    ProgMsg("Can't play -- no audio ports available at the moment\n");
	} else if (err == AL_BAD_OUT_OF_MEM) {
	    ProgMsg("Can't play -- not enough memory to open audio port\n");
	}
	return(False);
    }

    DebugTask(N_AudioActivity,
        Msg("%s: samp w=%d, samp/frame=%d, q sz=%d rate=%d\n",
	    errFunc, sampleWordsize, audioData->cvt.dstAu.channels,
	    ALgetqueuesize(audioDevFp_config), devOutputRate);
    )
    
    /* Attempt to remove initial delay by writing 0 samples */
    _ALwritesamps(SgiAuState.fp, (void *)audioData->buffer, 0);
    return(True);
}

static boolean
audioUpdate(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    int fillable;
    int len;
    int dstBytesToWrite, srcBytesToWrite;
    boolean success = True;
    InternalReasonPtr reason = NULL;
    int dataShift = DataFormToShift[ audioData->cvt.dstAu.dataForm ];
    DebugDeclare(char *errFunc = "audioUpdate";)

    DebugTask(N_AudioActivity,
        if( ALgetfilled(SgiAuState.fp) == 0 ) {
	    Msg("%s: au buffer empty\n",errFunc);
	}
    )

    fillable = ALgetfillable(SgiAuState.fp);
    DebugTask(N_AudioActivity,
        Msg("%s: null=%d ply=%d fill=%d\n",
	    errFunc, audioData->nullSamplesToPlay,
	    (audioData->bufferLength >> dataShift),
	    fillable);
    )

    if( audioData->nullSamplesToPlay ) {
#ifdef	   RUSS
	int i;
	i = audioData->nullSamplesToPlay <= audioData->samplesPerCall
	  ? audioData->nullSamplesToPlay
	  : audioData->samplesPerCall;
	audioData->nullSamplesToPlay -= i;
	WhenDebugEnabled(
            Msg("nullSamples = %d\n",audioData->nullSamplesToPlay);
        )
	if (i < audioData->samplesPerCall) {
	   /* ??? */
	}
	return(True);
#else	/* RUSS */
	char nullSamples[1024];

	memset((char *)nullSamples,0,sizeof(nullSamples));
	while( fillable > 0 && audioData->nullSamplesToPlay > 0 ) {
	    len = audioData->nullSamplesToPlay << dataShift;
	    if (len > sizeof(nullSamples))
		len = sizeof(nullSamples);
	    _ALwritesamps(SgiAuState.fp, (void *)nullSamples, len >> dataShift);
	    audioData->nullSamplesToPlay -= len >> dataShift;
	    fillable -= len >> dataShift;
	}
	/*
	 * If there are null samples to play, that means this call is a
	 *	result of UpdateAudioPlayback() after a stop. In this case
	 *	don't play more than the null samples. If we play the
	 *	actual data then if the user issues several single-steps
	 *	we will play audio for pictures that don't happen and could
	 *	decode to the end of the audio stream while just displaying
	 *	a few pictures.
	 */
	return(True);
#endif	/* RUSS */
    }

    dstBytesToWrite = srcBytesToWrite = audioData->bufferLength;
    if (audioData->cvt.convIndex != NoConversion
     && audioData->funcs == &MpegAudioFuncs) {
	long dstSamples;

	dstSamples = ConvertAudioBuffer(&audioData->cvt,
			      audioData->bufferLength >> dataShift, &reason);
	if (reason) {
	    success = False; goto done;
	}
	dstBytesToWrite = dstSamples << dataShift;
    }
    if ( fillable < (dstBytesToWrite >> dataShift) ) {
	DebugTask(N_AudioActivity,
	    Msg("%s: no room for play samples\n",errFunc)
	)
	audioData->bufferLength = 0;
	return(True);
    }
    if( dstBytesToWrite ) {

	_ALwritesamps(SgiAuState.fp,
		     (void *)(audioData->buffer),
		     (dstBytesToWrite >> dataShift));
	audioData->bufferLength = 0;
    }
done:
    if (reason) {
	WhenDebugEnabled( TtyShowReason(reason); )
	FreeReason(reason);
    }
    return(success);
}

/*		External Linkage				*/

AudioFuncs MpegAudioFuncs = {
    OpenMpegAudio,
    CloseMpegAudio,
    PreloadMpegAudio,
    LoadNextMpegAudio,
    SetQualityMpegAudio,
    audioInit,
    audioOpen,
    audioClose,
    audioUpdate
};

AudioFuncs AnyAudioFuncs = {
    OpenAnyAudio,
    CloseAnyAudio,
    PreloadAnyAudio,
    LoadNextAnyAudio,
    SetQualityAnyAudio,
    audioInit,
    audioOpen,
    audioClose,
    audioUpdate
};
