/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	<file name> - <one-line description of file contents>
 *
 * DESCRIPTION
 *      only runs on video hardware that has a 
 *	video output port. It will not run on a VINO video 
 *      board on an Indy.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/sgiycfb.c,v 1.6 1994/05/18 00:02:40 toddb Exp $
 */
#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"

#include <vl/vl.h>

#define MIN(x,y) ((x>y)?y:x)

typedef struct _videoSvrInfo {
    VLServer svr;
    VLNode vnode;
    VLBuffer buf;
    VLPath vlPath;
    GeomDimension fbSize;/* the width and height of the frame buffer */

    UChar *y;		/* the y component from the refresh frame */
    UChar *cb;		/* the cb/cr components from the refresh frame */
    ULong yStart;	/* offset into the y picture */
    ULong cbStart;	/* offset into the cb picture */
    ULong crStart;	/* offset into the cr picture */
    ULong yAccel;	/* accelerator to next luma line */
    ULong cAccel;	/* accelerator to next chroma line */

    UChar *fb;		/* the video frame buffer */
    ULong fbStart;	/* offset into the video frame buffer for picture */
    ULong fbAccel;	/* accelerator to next video buffer line */

    GeomRectangle src;	/* the source rectangle to copy */
    boolean doubleSize;	/* true ==> double the source rectangle on output */
} VideoSrvInfo;

VideoSrvInfo SgiData;

#define VideoPixelSize	2
#define UseIndyVideo	1

/*
 * Straight copy
 */
static void
CopyYCToVideo(DisplayInfoPtr dpyInfo, PictureInfoPtr ppic)
{
    UChar *yFrom, *crFrom, *cbFrom;
    UChar *vBuf;
    ULong crcb, topPixels, bottomPixels;
    int h, w, height, width;
    int nextVideoLine = SgiData.fbSize.width * VideoPixelSize;

    if (ppic) {
	yFrom = ppic->y;
	cbFrom = ppic->cb;
	crFrom = CbToCrAddr(ppic->cb);
    } else {
	yFrom = SgiData.y;
	cbFrom = SgiData.cb;
	crFrom = CbToCrAddr(SgiData.cb);
    }
    yFrom += SgiData.yStart;
    crFrom += SgiData.crStart;
    cbFrom += SgiData.cbStart;
    vBuf = SgiData.fb + SgiData.fbStart;

    if (UseIndyVideo) {
	/* Begin the data transfer */
	vlBeginTransfer(SgiData.svr, SgiData.vlPath, 0, NULL);
    }

    /* process 8x2 pixels
     *
     *   y00 y01 y02 y03 y04 y05 y06 y07
     *   y10 y11 y12 y13 y14 y15 y16 y17
     */
    width = SgiData.src.width;
    height = SgiData.src.height;
    for (h = 0; h < height; h += 2) {
	for (w = 0; w < width; w += 8) {
	    /*
	     * do...  y00 y01
	     *        y10 y11
	     */
	    crcb = (*cbFrom++ << 24)
		 | (*crFrom++ << 8);

	    topPixels = crcb | (yFrom[0] << 16);
	    bottomPixels = crcb | (yFrom[MAX_X] << 16);
	    yFrom++;

	    topPixels |= yFrom[0];
	    bottomPixels |= yFrom[MAX_X];
	    yFrom++;

	    *ULongP(vBuf) = topPixels;
	    *ULongP(vBuf+nextVideoLine) = bottomPixels;
	    vBuf += 4;

	    /*
	     * do...  y02 y03
	     *        y12 y13
	     */
	    crcb = (*cbFrom++ << 24)
		 | (*crFrom++ << 8);

	    topPixels = crcb | (yFrom[0] << 16);
	    bottomPixels = crcb | (yFrom[MAX_X] << 16);
	    yFrom++;

	    topPixels |= yFrom[0];
	    bottomPixels |= yFrom[MAX_X];
	    yFrom++;

	    *ULongP(vBuf) = topPixels;
	    *ULongP(vBuf+nextVideoLine) = bottomPixels;
	    vBuf += 4;

	    /*
	     * do...  y04 y05
	     *        y14 y15
	     */
	    crcb = (*cbFrom++ << 24)
		 | (*crFrom++ << 8);

	    topPixels = crcb | (yFrom[0] << 16);
	    bottomPixels = crcb | (yFrom[MAX_X] << 16);
	    yFrom++;

	    topPixels |= yFrom[0];
	    bottomPixels |= yFrom[MAX_X];
	    yFrom++;

	    *ULongP(vBuf) = topPixels;
	    *ULongP(vBuf+nextVideoLine) = bottomPixels;
	    vBuf += 4;

	    /*
	     * do...  y06 y07
	     *        y16 y17
	     */
	    crcb = (*cbFrom++ << 24)
		 | (*crFrom++ << 8);

	    topPixels = crcb | (yFrom[0] << 16);
	    bottomPixels = crcb | (yFrom[MAX_X] << 16);
	    yFrom++;

	    topPixels |= yFrom[0];
	    bottomPixels |= yFrom[MAX_X];
	    yFrom++;

	    *ULongP(vBuf) = topPixels;
	    *ULongP(vBuf+nextVideoLine) = bottomPixels;
	    vBuf += 4;
	}

	vBuf        += SgiData.fbAccel;
	yFrom       += SgiData.yAccel;
	crFrom      += SgiData.cAccel;
	cbFrom      += SgiData.cAccel;
    }

    if (UseIndyVideo) {
	/* Put the data into the ring buffer  */
	vlPutValid(SgiData.svr, SgiData.buf);

	/* End the data transfer */
	vlEndTransfer(SgiData.svr, SgiData.vlPath);
    }
}

/*
 * Copy and double the size on output
 */
static void
CopyYCToQuadVideo(DisplayInfoPtr dpyInfo, PictureInfoPtr ppic)
{
    ULong crcb;
    UChar *yFrom, *crFrom, *cbFrom;
    UChar *vBuf;
    int h, w, height, width;
    ULong pixel;
    int nextVideoLine = SgiData.fbSize.width * VideoPixelSize;

    if (ppic) {
	yFrom = ppic->y;
	cbFrom = ppic->cb;
	crFrom = CbToCrAddr(ppic->cb);
    } else {
	yFrom = SgiData.y;
	cbFrom = SgiData.cb;
	crFrom = CbToCrAddr(SgiData.cb);
    }
    yFrom += SgiData.yStart;
    crFrom += SgiData.crStart;
    cbFrom += SgiData.cbStart;
    vBuf = SgiData.fb + SgiData.fbStart;

    if (UseIndyVideo) {
	/* Begin the data transfer */
	vlBeginTransfer(SgiData.svr, SgiData.vlPath, 0, NULL);
    }

    width = SgiData.src.width;
    height = SgiData.src.height;
    for (h = 0; h < height; h+=2) {
	for (w = 0; w < width; w+=2) {
	    /* expand four y pixels into 4x4 pixels
	     *
	     *   y0 y1    ==>    y0 y0 y1 y1
	     *   y2 y3           y0 y0 y1 y1
	     *                   y2 y2 y3 y3
	     *                   y2 y2 y3 y3
	     */

	    crcb = (*cbFrom++ << 24)
		 | (*crFrom++ << 8);

	    pixel = yFrom[0];
	    pixel |= crcb | pixel << 16;
	    ULongP(vBuf)[0]               = pixel;
	    vBuf += nextVideoLine;
	    ULongP(vBuf)[0] = pixel;
	    vBuf += nextVideoLine;

	    pixel = yFrom[MAX_X];
	    pixel |= crcb | pixel << 16;
	    ULongP(vBuf)[0]               = pixel;
	    ULongP(vBuf+nextVideoLine)[0] = pixel;

	    vBuf -= (nextVideoLine << 1) - 4;
	    yFrom++;

	    pixel = yFrom[0];
	    pixel |= crcb | pixel << 16;
	    ULongP(vBuf)[0] = pixel;
	    vBuf += nextVideoLine;
	    ULongP(vBuf)[0] = pixel;
	    vBuf += nextVideoLine;

	    pixel = yFrom[MAX_X];
	    pixel |= crcb | pixel << 16;
	    ULongP(vBuf)[0]               = pixel;
	    ULongP(vBuf+nextVideoLine)[0] = pixel;

	    vBuf -= (nextVideoLine << 1) - 4;
	    yFrom++;
	}

	vBuf        += SgiData.fbAccel;
	yFrom       += SgiData.yAccel;
	crFrom      += SgiData.cAccel;
	cbFrom      += SgiData.cAccel;
    }

    if (UseIndyVideo) {
	/* Put the data into the ring buffer  */
	vlPutValid(SgiData.svr, SgiData.buf);

	/* End the data transfer */
	vlEndTransfer(SgiData.svr, SgiData.vlPath);
    }
}

void
SaveYcRefreshFrame(DisplayInfoPtr dpyInfo, PictureInfoPtr ppic)
{
    if (ppic == NULL) {
	Queue *dq = &PictBuffer.displayQueue;
	GeneralAssert(! QueueEmpty(dq));
	ppic = TailOfQueue(dq, PictureInfo *);
    }

    memcpy(SgiData.y, ppic->y, MAX_X * MAX_LUMA_Y);
    memcpy(SgiData.cb, ppic->cb, MAX_X * MAX_CHROMA_Y);
}

static void
RefreshYcFbFrame(DisplayInfoPtr dpyInfo)
{
    (*dpyInfo->converter)(dpyInfo, NULL);
}

static boolean
SgiYcFbInit(DisplayInfoPtr dpyInfo)
{
    VLNode drn;
    VLInfoPtr info;
    ULong transferSize;
    int result;
    VLControlValue ctlValue;

    SgiData.y = InsecureAlloc(MAX_X * MAX_LUMA_Y
			    + MAX_X * MAX_CHROMA_Y);

    if (SgiData.y == NULL)
	return(False);
    SgiData.cb = SgiData.y + MAX_X * MAX_LUMA_Y;

    if (UseIndyVideo) {
	if ((SgiData.svr = vlOpenVideo("")) == NULL) {
	    free(SgiData.y);
	    ProgMsg("can't open video server\n");
	    return(False);
	}

	/* Set up a source node in memory */
	SgiData.vnode = vlGetNode(SgiData.svr, VL_SRC, VL_MEM, VL_ANY);
	
	/* Set up a video drain node on the first device that has one */
	drn = vlGetNode(SgiData.svr, VL_DRN, VL_VIDEO, VL_ANY); 
	
	/* Create a path using the selected devices */
	SgiData.vlPath = vlCreatePath(SgiData.svr, VL_ANY, SgiData.vnode, drn);

	/* Set up the hardware for and define the usage of the path */
	result = vlSetupPaths(SgiData.svr,
			      (VLPathList)&SgiData.vlPath,
			      1,	/* one path */
			      VL_SHARE,
			      VL_SHARE);
	if (result < 0) {
	    ProgMsg("can't setup path: %s\n", vlStrError(vlGetErrno()));
	lateFailure:
	    vlDestroyPath(SgiData.svr, SgiData.vlPath);
	    vlCloseVideo(SgiData.svr);
	    free(SgiData.y);
	    return(False);
	}

	result = vlGetControl(SgiData.svr,
			      SgiData.vlPath,
			      drn,
			      VL_SIZE,
			      &ctlValue);
	if (result < 0) {
	    ProgMsg("Can't get video size: %s\n", vlStrError(vlGetErrno()));
	    goto lateFailure;
	}
	SgiData.fbSize.width = ctlValue.xyVal.x;
	SgiData.fbSize.height = ctlValue.xyVal.y;

	/* Find out what size this path supports */
	transferSize = vlGetTransferSize(SgiData.svr,SgiData.vlPath);

	/* Create a ring buffer for the data transfers */
	SgiData.buf = vlCreateBuffer(SgiData.svr,
				     SgiData.vlPath,
				     SgiData.vnode, 1);

	/* Associate the ring buffer with the path */
	vlRegisterBuffer(SgiData.svr,
			 SgiData.vlPath,
			 SgiData.vnode,
			 SgiData.buf);
       
	/* Get the next free frame in the buffer, reserve it for the data */
	do {
	    info = vlGetNextFree(SgiData.svr, SgiData.buf, transferSize);
	} while (!info && !vlBufferDone(SgiData.buf)); 

	/* Get a pointer to where the data will go */
	SgiData.fb = vlGetActiveRegion(SgiData.svr, SgiData.buf, info);
    } else {
	SgiData.fbSize.width = 640;
	SgiData.fbSize.height = 480;
	SgiData.fb = SecureAlloc(SgiData.fbSize.width
			       * SgiData.fbSize.height
			       * VideoPixelSize);
    }

    dpyInfo->ycFbCopier.fb = SgiData.fb;   	/* its available */
    dpyInfo->ycFbCopier.fbDisplay = RefreshYcFbFrame;
    dpyInfo->ycFbCopier.winDisplay = NULL;	/* should never be called */
    dpyInfo->ycFbCopier.saveFbFrame = SaveYcRefreshFrame;
    dpyInfo->ycFbCopier.maxDirectConvertPix = ~0;

    return(True);
}

/*
 * Precalculate the frame buffer offsets for rendering.
 */
void
InitYcFb(DisplayInfoPtr dpyInfo)
{
    UChar *fb, *fbEnd;
    SequenceInfoPtr pSeq = dpyInfo->pSeq;
    int vWidth, vHeight, tooBig;
    ULong pixel;

    if (SgiData.svr == NULL) {
	if (! SgiYcFbInit(dpyInfo))
	    return;
    }

    if (SgiData.fbSize.width/2 >= pSeq->horizontalSize
     && SgiData.fbSize.height/2 >= pSeq->verticalSize) {
	vWidth = SgiData.fbSize.width/2;
	vHeight = SgiData.fbSize.height/2;
        SgiData.doubleSize = True;
	dpyInfo->ycFbCopier.converter = CopyYCToQuadVideo;
    } else {
	vWidth = SgiData.fbSize.width;
	vHeight = SgiData.fbSize.height;
        SgiData.doubleSize = False;
	dpyInfo->ycFbCopier.converter = CopyYCToVideo;
    }
    SgiData.src.width = pSeq->horizontalSize;
    SgiData.src.height = pSeq->verticalSize;
    if (SgiData.src.width & 0x1)
	SgiData.src.width -= 1;		/* don't allow odd widths */
    if (SgiData.src.height & 0x1)
	SgiData.src.height -= 1;	/* don't allow odd heights */
    SgiData.src.y = (vHeight - pSeq->verticalSize) / 2;

    /*
     * trim back the width and height if they are bigger than the
     * video size
     */
    tooBig = SgiData.src.width - vWidth;
    if (tooBig > 0) {
	SgiData.src.width -= tooBig;
	SgiData.src.x = (pSeq->horizontalSize - SgiData.src.width) / 2;
	SgiData.src.x &= (~1);		/* don't allow odd left edges */
	SgiData.fbStart = 0;
    } else {
	SgiData.src.x = 0;
	SgiData.fbStart = ((-tooBig) / 2) * VideoPixelSize;
    }

    tooBig = SgiData.src.height - vHeight;
    if (tooBig > 0) {
	SgiData.src.height -= tooBig;
	SgiData.src.y = (pSeq->verticalSize - SgiData.src.height) / 2;
	SgiData.src.y &= (~1);		/* don't allow odd start lines */
    } else {
	SgiData.src.y = 0;
	SgiData.fbStart += ((-tooBig) / 2)
			 * SgiData.fbSize.width
			 * VideoPixelSize;
    }
    SgiData.fbStart <<= SgiData.doubleSize;
    SgiData.fbStart &= (~3);	/* don't allow odd left edges */

    /*
     * Now that we know the source rectangle compute the accelerators
     */
    SgiData.yStart = SgiData.src.y * MAX_X + SgiData.src.x;
    SgiData.crStart = (SgiData.src.y/2) * MAX_X + SgiData.src.x/2;
    SgiData.cbStart = (SgiData.src.y/2) * MAX_X + SgiData.src.x/2;
    if (SgiData.doubleSize) {
	SgiData.fbAccel = SgiData.fbSize.width * 3
			- ((SgiData.fbSize.width>>1) - SgiData.src.width);
	SgiData.yAccel = 2 * MAX_X - SgiData.src.width;
	SgiData.cAccel = MAX_X - SgiData.src.width/2;
    } else {
	SgiData.fbAccel = SgiData.fbSize.width
			+ (SgiData.fbSize.width - SgiData.src.width);
	SgiData.yAccel = 2 * MAX_X - SgiData.src.width;
	SgiData.cAccel = MAX_X - SgiData.src.width/2;
    }
    SgiData.fbAccel *= VideoPixelSize;

    /*
     * Fill the entire buffer with black.
     */
    fb = SgiData.fb;
    fbEnd = fb + SgiData.fbSize.width * SgiData.fbSize.height * VideoPixelSize;
    pixel = (128 << 24)	/* cb */
	  | (0 << 16)	/* y0 */
	  | (128 << 8)	/* cr */
	  | (0 << 0);	/* y1 */
    while (fb < fbEnd) {
	*ULongP(fb) = pixel;
	fb += 2 * VideoPixelSize;
    }
}

#ifdef notdef
static void
SgiClose()
{
    /* Clean up and exit */
    vlDeregisterBuffer(SgiData.svr, SgiData.vlPath, SgiData.vnode, SgiData.buf);
    vlDestroyPath(SgiData.svr, SgiData.vlPath);
    vlDestroyBuffer(SgiData.svr, SgiData.buf);
    vlCloseVideo(SgiData.svr);
}
#endif
