/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	sunaifc.c - sun functions for playing Aifc files
 *		    Actually this will only read Aiff files at present
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/sunaudio.c,v 1.10 1995/08/18 00:32:22 russ Exp $
 */
#define NeedFunctionPrototypes 1

#include	<errno.h>
#include	<fcntl.h>
#include	<sys/ioccom.h>
#include	"audio_hdr.h"
#ifdef Solaris2
#include      <sys/audioio.h>
#else
#include      <sun/audioio.h>
#endif
#include 	"vmpegdecomp.h"
#include 	"mu-law.h"
 
#define AUDIODEV		"/dev/audio"

#ifndef Solaris2
extern int close(int);
extern int write(int fd, char *buf, int length);
extern int ioctl(int fd, int request, caddr_t arg);
#endif


/* Globals */

static struct {
    int fd;
} AudioState = { -1 };

/*
 * Function Prototypes
 */

static int
audioOpenDev(void)
{
    int fd;

    if( (fd = open(AUDIODEV, O_WRONLY | O_NDELAY)) < 0 ) {
	SystemError("Sun audio: %s: open failed: %s\n", AUDIODEV);
	return(-1);
    }
    return(fd);
}

static void
audioClose(DisplayInfoPtr dpyInfo)
{
    if (AudioState.fd > -1) {
	close(AudioState.fd);
	AudioState.fd = -1; 
    }
}

static void
audioInit(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    audio_info_t devData, oldData;

    audioData->formatsAvailable = Avail8BitULaw;

    if ((AudioState.fd = audioOpenDev()) < 0)
	return;

    if (ioctl(AudioState.fd, AUDIO_GETINFO, (caddr_t)&oldData) < 0) {
	SystemError("Can't read audio device state\n");
	return;
    }

    AUDIO_INITINFO(&devData);
    devData.play.channels = 1;
    devData.play.sample_rate = Audio16Khz;
    devData.play.encoding = AUDIO_ENCODING_LINEAR;
    devData.play.precision = 16;
 
    if( ioctl(AudioState.fd, AUDIO_SETINFO, (caddr_t)&devData) == 0 ) {
        audioData->formatsAvailable |= Avail16Khz;
        audioData->formatsAvailable |= Audio16BitPcm;
 
        AUDIO_INITINFO(&devData);
        devData.play.sample_rate = Audio48Khz;
        if( ioctl(AudioState.fd, AUDIO_SETINFO, (caddr_t)&devData) == 0 )
            audioData->formatsAvailable |= Avail48Khz;
 
        AUDIO_INITINFO(&devData);
        devData.play.sample_rate = Audio44_1Khz;
        if( ioctl(AudioState.fd, AUDIO_SETINFO, (caddr_t)&devData) == 0 )
            audioData->formatsAvailable |= Avail44_1Khz;
 
        AUDIO_INITINFO(&devData);
        devData.play.sample_rate = Audio32Khz;
        if( ioctl(AudioState.fd, AUDIO_SETINFO, (caddr_t)&devData) == 0 )
            audioData->formatsAvailable |= Avail32Khz;
 
        AUDIO_INITINFO(&devData);
        devData.play.sample_rate = Audio8Khz;
        if( ioctl(AudioState.fd, AUDIO_SETINFO, (caddr_t)&devData) == 0 )
            audioData->formatsAvailable |= Avail8Khz;
    }
 
    if (devData.play.channels == 1) {
        audioData->formatsAvailable |= AvailMono;
        AUDIO_INITINFO(&devData);
        devData.play.channels = 2;
        if( ioctl(AudioState.fd, AUDIO_SETINFO, (caddr_t)&devData) == 0 ) {
            if (devData.play.channels == 2) {
                audioData->formatsAvailable |= AvailStereo;
            }
        }
    } else {
        audioData->formatsAvailable |= AvailStereo;
        AUDIO_INITINFO(&devData);
        devData.play.channels = 1;
        if( ioctl(AudioState.fd, AUDIO_SETINFO, (caddr_t)&devData) == 0 ) {
            if (devData.play.channels == 1) {
                audioData->formatsAvailable |= AvailMono;
            }
        }
    }
 
    if( ioctl(AudioState.fd, AUDIO_SETINFO, (caddr_t)&oldData) == 0 ) {
        ; /* should we say something? */
    }

    audioClose(dpyInfo);
}

static boolean
audioOpen(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    audio_info_t devData, *ads = &devData;
    audio_prinfo_t *pds = &devData.play;
    char *errFunc = "audioOpen";
    int encoding, precision;
    boolean success = True;
    InternalReasonPtr reason = NULL;

    audioClose(dpyInfo);
    if ((AudioState.fd = audioOpenDev()) < 0)
	return(False);

    SetNonBlocking(AudioState.fd);

    ioctl(AudioState.fd,AUDIO_GETINFO, (caddr_t)ads);

    switch(audioData->cvt.dstAu.dataForm) {
    default:
	reason = CreateReason("%s: bad data encoding requested: %d\n",
			      audioData->cvt.dstAu.dataForm);
	success = False; goto done;

    case Audio8BitULaw:
        encoding = AUDIO_ENCODING_ULAW;
        precision = 8;
        break;
    case Audio16BitPcm:
        encoding = AUDIO_ENCODING_LINEAR;
        precision = 16;
        break;
    }

    /*
     * Check the audio device configuration and
     * attempt to match it to the audio file
     */
    if (pds->sample_rate != audioData->cvt.dstAu.rate
     || pds->channels != audioData->cvt.dstAu.channels
     || pds->encoding != encoding
     || pds->precision != precision)
    {
        AUDIO_INITINFO(ads);
        pds->sample_rate = audioData->cvt.dstAu.rate;
        pds->channels = audioData->cvt.dstAu.channels;
        pds->encoding = encoding;
        pds->precision = precision;
        ioctl(AudioState.fd,AUDIO_SETINFO, (caddr_t)ads);
    }
 
    if (pds->channels != audioData->cvt.dstAu.channels
     || pds->sample_rate != audioData->cvt.dstAu.rate
     || pds->encoding != encoding
     || pds->precision != precision)
    {
	reason =
	    CreateReason("%s: Cannot configure audio device appropriately\n",
			 errFunc);
	success = False; goto done;
    }

done:
    if (reason) {
	WhenDebugEnabled(TtyShowReason(reason);)
	FreeReason(reason);
    }
    return(success);
}

static boolean
audioUpdate(DisplayInfoPtr dpyInfo)
{
    AudioInfoPtr audioData = &dpyInfo->audioData;
    char *errFunc = "audioUpdate";
    UChar *buffer;
    int srcBytesToWrite = 0;	/* bytes to write base on src rate */
    int dstBytesToWrite;	/* bytes to write base on dst rate */
    int dstBytesWritten;
    int dataShift;
    boolean underWrite;
    InternalReasonPtr reason = NULL;
    int success = True;
    float cvtFactor;
    DebugDeclare(ULong writeTime = 0;)

/*
 * Note that nullSamplesToPlay > 0 => MPEG audio
 * Note that if convIndex != NoConversion then for MPEG audio
 *	conversion must be done here at the device level (until seeking
 *	is implemented), but for all other audio, conversion has
 *	already been performed.
 * Note that samplesPerCall and nullSamplesToPlay are based on src rate
 */

    dataShift = DataFormToShift[ audioData->cvt.dstAu.dataForm ];
    if (audioData->nullSamplesToPlay) {
	int i;
	i = audioData->nullSamplesToPlay <= audioData->samplesPerCall
	  ? audioData->nullSamplesToPlay
	  : audioData->samplesPerCall;
	audioData->nullSamplesToPlay -= i;
	WhenDebugEnabled(
            Msg("nullSamples = %d\n",audioData->nullSamplesToPlay);
        )
	if (i < audioData->samplesPerCall) {
	   /* ??? */
	}
	return(True);
    }

    if (audioData->bufferLength == 0)
	return(True);

    buffer = audioData->buffer;

    DebugTask(N_AudioActivity, CheckRTClock(&writeTime));

    dstBytesToWrite = srcBytesToWrite = audioData->bufferLength;
    if (audioData->cvt.convIndex != NoConversion
     && audioData->funcs == &MpegAudioFuncs) {
	long dstSamples;

	dstSamples = ConvertAudioBuffer(&audioData->cvt,
			      audioData->bufferLength >> dataShift, &reason);
	if (reason) {
	    success = False; goto done;
	}
	dstBytesToWrite = dstSamples << dataShift;
    }

    /* dst -> src bytes conversion */
    cvtFactor = 1.0 * srcBytesToWrite / dstBytesToWrite; 

    dstBytesWritten = write(AudioState.fd, audioData->buffer, dstBytesToWrite);
    DebugTask(N_AudioActivity,
	      CheckRTClock(NULL);
	      Msg("%s: %d bytes out, time=%d\n",
		  errFunc, dstBytesWritten, writeTime);
    )

    if (dstBytesWritten < 0) {
	reason = CreateSysReason("%s: write of %d bytes failed\n",
				 errFunc, dstBytesToWrite);
	goto done;
    }
    if (dstBytesWritten < dstBytesToWrite) {
	reason = CreateReason("%s: wrote %d (of %d) bytes requested\n",
			      errFunc, dstBytesWritten, dstBytesToWrite);
       /*
	*  if we shift up unwritten samples and decrement bufferLength
	*	then MPEG audio will be screwed up because we'd need to
	*	adjust nFifoEntries or the delicate state machine
	*	will be out of sync. So for MPEG audio, and underwrite
	*	means audio is out of sync.
	*/
    }
    audioData->bufferLength = 0;
    
done:
    if (reason) {
	WhenDebugEnabled(TtyShowReason(reason);)
	FreeReason(reason);
    }
    return(success);
}

/*		External Linkage				*/

AudioFuncs MpegAudioFuncs = {
    OpenMpegAudio,
    CloseMpegAudio,
    PreloadMpegAudio,
    LoadNextMpegAudio,
    SetQualityMpegAudio,
    audioInit,
    audioOpen,
    audioClose,
    audioUpdate
};

AudioFuncs AnyAudioFuncs = {
    OpenAnyAudio,
    CloseAnyAudio,
    PreloadAnyAudio,
    LoadNextAnyAudio,
    SetQualityAnyAudio,
    audioInit,
    audioOpen,
    audioClose,
    audioUpdate
};
