/*
 *      (c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              raw.c
 *
 *      DESCRIPTION
 *              figure out where the frame buffer memory is
 *		OS-DEPENDENT
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/sunfb.c,v 4.18 1995/04/12 23:04:46 russ Exp $
 */

#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"

#include <signal.h>
#include <sys/errno.h>

#ifdef SUNFB
#include    <pixrect/pr_planegroups.h>

#ifdef Solaris2
#include    <sys/fbio.h>
#include    <sys/mman.h>
#include    <sys/types.h>
#include    <sys/file.h>
#include    <sys/ioctl.h>
#include    <sys/cg6reg.h>
#include    <sys/cg4reg.h>
#include    <sys/cg8reg.h>
#include    <fcntl.h>
#include    <unistd.h>
#endif

#ifdef Solaris1
#include    <sun/fbio.h>
#include    <sys/mman.h>
#include    <sys/types.h>
#include    <sys/file.h>
#include    <sys/ioctl.h>
#include    <sundev/cg6reg.h>
#include    <sundev/cg4reg.h>
#include    <sundev/cg8reg.h>
#endif

#include "nvrString.h"

static UChar *SunOverlay;
static UChar *SunEnable;
static DisplayInfoPtr RawDpyInfoCopy;	/* XXX blech! */

typedef enum {
    DV_Unknown,
    DV_BW2,
    DV_CG8,
    DV_CG4,
    DV_CG6,
    DV_CG3,
    DV_TV2
} FrameBufferType;

static NameEnumPair fbNames[] = {
#ifdef Solaris2
    { "/dev/fbs/bwtwo0",	DV_BW2 },
    { "/dev/fbs/cgeight0",	DV_CG8 },
    { "/dev/fbs/cgfour0",	DV_CG4 },
    { "/dev/fbs/cgsix0",	DV_CG6 },
    { "/dev/fbs/cgthree0",	DV_CG3 },
    { "/dev/fbs/tvtwo0",	DV_TV2 },
#else /* Solaris2 */
    { "/dev/bwtwo0",		DV_BW2 },
    { "/dev/cgeight0",		DV_CG8 },
    { "/dev/cgfour0",		DV_CG4 },
    { "/dev/cgsix0",		DV_CG6 },
    { "/dev/cgthree0",		DV_CG3 },
    { "/dev/tvtwo0",		DV_TV2 },
#endif /* Solaris2 */
    { NULL,			DV_Unknown }
};
    

static void
SunFillMonoFb(UChar *fb, UChar value)
{
    DisplayInfoPtr dpyInfo = RawDpyInfoCopy;
    UChar *p = fb + dpyInfo->dst.y * 144 + (dpyInfo->dst.x >> 3);
    int width = dpyInfo->dst.width >> 3;
    int height = dpyInfo->dst.height;
    int h, w;

    for (h = height; h > 0; h--) {
	for (w = width; w > 0; w--)
	    *p++ = value;
	p += 144-width;
    }
}

static void
CleanupOverlay()
{
    NormalOutput("cleanup\n");
    SunFillMonoFb(SunOverlay, 0);
    SunFillMonoFb(SunEnable, 0);
    exit(0);
}

static void
ScheduleOverlayCleanup(DisplayInfoPtr dpyInfo)
{
    RawDpyInfoCopy = dpyInfo;
    if (signal(SIGINT, CleanupOverlay) == (void (*)())-1)
	Fatal("can't set signal handler (INT)\n");
    if (signal(SIGTERM, CleanupOverlay) == (void (*)())-1)
	Fatal("can't set signal handler (TERM)\n");
}

static UChar *MapFb(int fd, char *devName, int offset, int fbSize)
{
    int mapsize;
    int pagemask;
    caddr_t base;

    /*
     * It's not precisely clear that we have to round up
     * fb_size to the nearest page boundary but there are
     * rumors that this is a good idea and that it shouldn't
     * hurt anything.
     */
#ifdef Solaris2
    pagemask = sysconf(_SC_PAGESIZE) - 1;
#else
    pagemask = getpagesize() - 1;
#endif
    mapsize = (fbSize + pagemask) & ~pagemask;

    /*
     * map it
     */
    base = mmap((caddr_t) 0, mapsize,
		 PROT_READ | PROT_WRITE, MAP_SHARED, fd, offset);
    if ((int)base == -1) {
        Error("can't map %d bytes @ offset %d for %s\n",
	    mapsize, offset, devName);
	base = NULL;
    }
    return ((UChar *)base);
}

static char *SunFbTypes[] = {
    "SUN1BW",
    "SUN1COLOR",
    "SUN2BW",
    "SUN2COLOR",
    "SUN2GP",
    "SUN5COLOR",
    "SUN3COLOR",
    "MEMCOLOR",
    "SUN4COLOR",
    "NOTSUN1",
    "NOTSUN2",
    "NOTSUN3",
    "SUNFAST_COLOR",
    "SUNROP_COLOR",
    "SUNFB_VIDEO",
    "SUNGIFB",
    "SUNGPLAS",
    "SUNGP3",
    "Sungt",
    "RESERVED1",
};

static void
CheckSunFb(DisplayInfoPtr dpyInfo, char *devName)
{
    struct fbtype type;
    struct fbgattr fbattr;
    boolean fbattrValid = False;
    struct cg6_info cg6attr;
    int fd, i;
    CopyInfo *cp;
    extern int errno;

    fd = open(devName, O_RDWR);
    if (fd < 0) {
	if (errno != ENXIO && errno != ENODEV && errno != ENOENT)
	    Error("can't open %s\n", devName);
	return;
    }

    /*
     * Now get the frame buffer info
     */
    if (ioctl(fd, FBIOGTYPE, &type) < 0) {
        /* Error("can't get sun fb info from %s\n", devName); */
	return;
    }
    if (ioctl(fd, FBIOGATTR, &fbattr) >= 0) {
	fbattrValid = True;
	type = fbattr.fbtype;
    }

    /*
     * XXX - adjust fb sizes.  Without doing this, the mmap won't work.
     */

standardType:

    switch (type.fb_type) {
    case FBTYPE_SUN2BW:
	/* Msg("sun bw2 present (%s)\n", devName); */
	break;

    case FBTYPE_SUNFAST_COLOR:
	/* Msg("sun cg6 present (%s)\n", devName); */
	if (ioctl(fd, FBIOGXINFO, &cg6attr) >= 0)
	    type.fb_size = cg6attr.accessible_width * cg6attr.accessible_height;
	break;

    case FBTYPE_SUN4COLOR:
    case FBTYPE_SUN3COLOR:
	/* Msg("sun cg%c present (%s)\n",
	    type.fb_type == FBTYPE_SUN4COLOR ? '4' : '3', devName); */
	type.fb_size = type.fb_height * type.fb_width;
	break;

    case FBTYPE_NOTSUN1:
	if (! fbattrValid)
	    goto unknownType;
	switch (fbattr.sattr.emu_type) {
	case FBTYPE_SUN2BW:
	case FBTYPE_SUNFAST_COLOR:
	case FBTYPE_SUN4COLOR:
	case FBTYPE_SUN3COLOR:
	    type.fb_type = fbattr.sattr.emu_type;
	    goto standardType;
	}

	for (i = 0; i < FB_ATTR_NEMUTYPES; i++) {
	    switch (fbattr.emu_types[i]) {
	    case FBTYPE_SUN2BW:
	    case FBTYPE_SUNFAST_COLOR:
	    case FBTYPE_SUN4COLOR:
	    case FBTYPE_SUN3COLOR:
		type.fb_type = fbattr.emu_types[i];
		goto standardType;
	    }
	}

	/* fall through... */

    default:
    unknownType:
	if (type.fb_type > 0 && type.fb_type < NumberOfCells(SunFbTypes))
	    Msg("don't know how to handle fb type FBTYPE_%s (%d)\n",
		SunFbTypes[ type.fb_type ], type.fb_type);
	else
	    Msg("don't know how to handle fb type %d\n", type.fb_type);
#ifdef Solaris2
	Msg("See /usr/include/sys/fbio.h for FBTYPE_*\n");
#else
	Msg("See /usr/include/sun/fbio.h for FBTYPE_*\n");
#endif
	return;
    }

    switch (type.fb_type) {
    case FBTYPE_SUN2BW:
	cp = &dpyInfo->copiers[ GrayScale1 ];
	if ((cp->fb = MapFb(fd, devName, 0, type.fb_size)) == NULL)
	    return;
	cp->w = type.fb_width;
	cp->h = type.fb_height;
	cp->lineBytes = type.fb_width >> 3;
	break;

    case FBTYPE_SUN3COLOR:
    case FBTYPE_SUNFAST_COLOR:
    case FBTYPE_SUN4COLOR:
	if (dpyInfo->useOverlay) {
	    SunEnable = MapFb(fd, devName, 0, (128*1024) + (128*1024));
	    if (SunEnable == NULL)
		return;
	    SunOverlay = SunEnable + (128*1024);
	    SunFillMonoFb(SunEnable, (UChar)~0);
	    ScheduleOverlayCleanup(dpyInfo);

	    cp = &dpyInfo->copiers[ GrayScale1 ];
	    cp->fb = SunOverlay;
	    cp->w = type.fb_width;
	    cp->h = type.fb_height;
	    cp->lineBytes = type.fb_width >> 3;
	} else {
	    cp = &dpyInfo->copiers[ Rgb332 ];
	    cp->fb = MapFb(fd, devName, (128*1024) + (128*1024), type.fb_size);
	    if (cp->fb == NULL)
		return;
	    cp->w = type.fb_width;
	    cp->h = type.fb_height;
	    cp->lineBytes = type.fb_width;
	}
	break;
    }
    close(fd);
}

void
CloneCopier(CopyInfo *src, CopyInfo *dst)
{
    dst->fb        = src->fb;
    dst->w         = src->w;
    dst->h         = src->h;
    dst->lineBytes = src->lineBytes;
}

void
SunFbInit(DisplayInfoPtr dpyInfo)
{
    int i;

    if (dpyInfo->deviceName)
	CheckSunFb(dpyInfo, dpyInfo->deviceName);
    else {
	CheckSunFb(dpyInfo, EnumToName(DV_BW2,fbNames));
	CheckSunFb(dpyInfo, EnumToName(DV_CG8,fbNames));
	CheckSunFb(dpyInfo, EnumToName(DV_CG4,fbNames));
	CheckSunFb(dpyInfo, EnumToName(DV_CG6,fbNames));
	CheckSunFb(dpyInfo, EnumToName(DV_CG3,fbNames));
	CheckSunFb(dpyInfo, EnumToName(DV_TV2,fbNames));
    }

    for (i = 0; i < NRgbDepthTypes; i++) {
	dpyInfo->copiers[ i ].fbDisplay = CopyPixelsToFb;
	dpyInfo->copiers[ i ].clipToScreen = True;
	dpyInfo->copiers[ i ].maxDirectConvertPix = 352 * 240;
    }

    /*
     * XXX this should check whats available and the speed of
     * the machine.
     */
    dpyInfo->copiers[ Rgb888 ].maxDirectConvertPix = 160 * 120 * 4;
    dpyInfo->copiers[ Rgb444 ].maxDirectConvertPix = 352 * 240 * 2;

    /*
     * If Rgb332 was initialized, then clone that onto
     * Rgb232, GrayScale7, GrayScale8, RgbC555 and RgbC666
     */
    CloneCopier(&dpyInfo->copiers[ Rgb332 ], &dpyInfo->copiers[ Rgb232 ]);
    CloneCopier(&dpyInfo->copiers[ Rgb332 ], &dpyInfo->copiers[ RgbC555 ]);
    CloneCopier(&dpyInfo->copiers[ Rgb332 ], &dpyInfo->copiers[ RgbC666 ]);
    CloneCopier(&dpyInfo->copiers[ Rgb332 ], &dpyInfo->copiers[ GrayScale7 ]);
    CloneCopier(&dpyInfo->copiers[ Rgb332 ], &dpyInfo->copiers[ GrayScale8 ]);
}

#endif /* SUNFB */

#ifdef RASTEROPS
#include    <pixrect/cg8var.h>
#include    <pixrect/pr_planegroups.h>
#include    <sys/mman.h>
#include    <sys/types.h>
#include    <sys/file.h>

static void
RoInit(DisplayInfoPtr dpyInfo)
{
    Pipio_Fb_Info roFbInfo;
    caddr_t base;
    int i;
    extern int errno;
    CopyInfo *cp;
    int roDevFd;
    char *roDevName = EnumToName(DV_CG8,fbNames);

    roDevFd = open(roDevName, O_RDWR);
    if (roDevFd < 0) {
	if (errno != ENODEV)
	    Error("can't open %s\n", roDevName);
	return;
    }

    /*
     * Now get the frame buffer info
     */
    if (ioctl(roDevFd, PIPIO_G_FB_INFO, &roFbInfo) < 0) {
        Error("can't get rasterops fb info\n");
	return;
    }

    /*
     * map it
     */
    base = mmap((caddr_t) 0, roFbInfo.total_mmap_size,
		 PROT_READ | PROT_WRITE, MAP_SHARED, roDevFd, 0);
    if ((int)base == -1) {
        Error("can't map %d bytes for %s\n",
	    roFbInfo.total_mmap_size, roDevName);
	return;
    }

    for (i = 0; i < roFbInfo.frame_buffer_count; i++) {
        switch (roFbInfo.fb_descriptions[ i ].group) {
        case PIXPG_8BIT_COLOR:
	    cp = &dpyInfo->copiers[ Rgb332 ];
	    cp->fb = (UChar *)(base + roFbInfo.fb_descriptions[i].mmap_offset);
	    cp->w = roFbInfo.fb_descriptions[ i ].width;
	    cp->h = roFbInfo.fb_descriptions[ i ].height;
	    cp->lineBytes = roFbInfo.fb_descriptions[ i ].linebytes;
	    cp->maxDirectConvertPix = 352 * 240;
	    cp->clipToScreen = True;
	    break;

        case PIXPG_24BIT_COLOR:
	    cp = &dpyInfo->copiers[ Rgb888 ];
	    cp->fb = (UChar *)(base + roFbInfo.fb_descriptions[i].mmap_offset);
	    cp->w = roFbInfo.fb_descriptions[ i ].width;
	    cp->h = roFbInfo.fb_descriptions[ i ].height;
	    cp->lineBytes = roFbInfo.fb_descriptions[ i ].linebytes;
	    cp->maxDirectConvertPix = 160 * 120 * 4;
	    cp->clipToScreen = True;
	    break;
        }
    }
}

#endif /* RASTEROPS */
