/*
 *      (c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              display.c
 *
 *      DESCRIPTION
 *              handle display activity
 *		X-DEPENDENT
 *		OS-DEPENDENT
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/testgl.c,v 1.1 1994/03/12 19:00:00 toddb Exp $
 */

#include <X11/keysym.h>
#include <stdarg.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/XShm.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <gl/glws.h>

char *ProgramName;

void
Fatal(char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
    fflush(stderr);
}

/*
**  Used to get a value out of the configuration structure returned by
**  GLXgetconfig.  Scan through the GLX config structure, and, when we
**  find the proper buffer-mode combination, return the argument relevant
**  for that buffer type.
*/
unsigned long
GLXgetvalue(GLXconfig* conf, int buffer, int mode)
{
    int i;
    for (i = 0; conf[i].buffer; i++)
        if (conf[i].buffer == buffer && conf[i].mode == mode)
            return conf[i].arg;
    return 0;
}

/* extract_visual:
 *  a visual is uniquely identified by a viusal identifier descibing the
 *  visual and screen.  this function is used to get back the
 *  GLX_NORMAL-GLX_VISUAL and the GLX_NORMAL-GLX_COLORMAP fields out of
 *  the configuration data for use in creating a window using these data.
 *  XGetVisualInfo() returns a list of visual structures that match the
 *  attributes explicitly specified in the template structure.
 */
static XVisualInfo*
extract_visual(Display* D, int S, GLXconfig *conf, int buffer)
{
    XVisualInfo template, *v;
    int n;

    template.screen = S;
    template.visualid = GLXgetvalue(conf, buffer, GLX_VISUAL);
    return XGetVisualInfo (D, VisualScreenMask|VisualIDMask, &template, &n);
}

static void
GLXShowConfig(Display *dpy, GLXconfig* conf, char *name)
{
    int i;
    XVisualInfo *visInfo;
    static char *colorClasses[] = {
        "StaticGray",
        "GrayScale",
        "StaticColor",
        "PseudoColor",
        "TrueColor",
        "DirectColor" };

    printf("%s:\n", name);

    for (i = 0; conf[i].buffer; i++) {
	printf("    ");
	switch(conf[i].buffer) {
	case GLX_NORMAL:	printf("GLX_NORMAL:   "); break;
	case GLX_OVERLAY:	printf("GLX_OVERLAY:  "); break;
	case GLX_POPUP:		printf("GLX_POPUP:    "); break;
	case GLX_UNDERLAY:	printf("GLX_UNDERLAY: "); break;
	}
	switch(conf[i].mode) {
	case GLX_RGB:
	    printf("GLX_RGB = %d\n", conf[i].arg);
	    break;
	case GLX_DOUBLE:
	    printf("GLX_DOUBLE = %d\n", conf[i].arg);
	    break;
	case GLX_ZSIZE:
	    printf("GLX_ZSIZE = %d\n", conf[i].arg);
	    break;
	case GLX_BUFSIZE:
	    printf("GLX_BUFSIZE = %d\n", conf[i].arg);
	    break;
	case GLX_STENSIZE:
	    printf("GLX_STENSIZE = %d\n", conf[i].arg);
	    break;
	case GLX_ACSIZE:
	    printf("GLX_ACSIZE = %d\n", conf[i].arg);
	    break;
	case GLX_VISUAL:
	    visInfo = extract_visual(dpy,
				     DefaultScreen(dpy),
				     conf,
				     GLX_NORMAL);
	    printf("GLX_VISUAL = 0x%x", conf[i].arg);
	    if (conf[i].arg == 0)
		printf("\n");
	    else
		printf(", class=%s, depth=%d\n",
		    colorClasses[ visInfo->visual->class ],
		    visInfo->depth);
	    break;
	case GLX_COLORMAP:
	    printf("GLX_COLORMAP = 0x%x\n", conf[i].arg);
	    break;
	case GLX_WINDOW:
	    printf("GLX_WINDOW = %d\n", conf[i].arg);
	    break;
	case GLX_MSSAMPLE:
	    printf("GLX_MSSAMPLE = %d\n", conf[i].arg);
	    break;
	case GLX_MSZSIZE:
	    printf("GLX_MSZSIZE = %d\n", conf[i].arg);
	    break;
	case GLX_MSSSIZE:
	    printf("GLX_MSSSIZE = %d\n", conf[i].arg);
	    break;
	case GLX_STEREOBUF:
	    printf("GLX_STEREOBUF = %d\n", conf[i].arg);
	    break;
	case GLX_RGBSIZE:
	    printf("GLX_RGBSIZE = %d\n", conf[i].arg);
	    break;
	default:
	    printf("mode=0x%x, arg=%d\n",
		conf[i].mode, conf[i].arg);
	    break;
	}
    }
}

/*
 * Dorky little helper function used to build up a GLXconfig array.
 */

static void set_entry (GLXconfig* ptr, int b, int m, int a)
{
    ptr->buffer = b;
    ptr->mode = m;
    ptr->arg = a;
}

main(int argc, char **argv)
{
    Display *dpy;
    XWindowAttributes winAttr;
    Colormap cmap;
    Window win = 0;
    char *p;
    Visual *vis;
    GLXconfig   initConfig[50];
    GLXconfig   *retconfig, *next;

    ProgramName = argv[0];

    next = &initConfig[0];
    for (argc--,argv++; argc; argc--, argv++) {
	if (strcmp(*argv, "index") == 0) {
	    set_entry(next++, GLX_NORMAL, GLX_RGB, FALSE);
	} else if (strcmp(*argv, "rgb") == 0) {
	    set_entry(next++, GLX_NORMAL, GLX_RGB, TRUE);
	} else if (strcmp(*argv, "double") == 0) {
	    set_entry(next++, GLX_NORMAL, GLX_DOUBLE, TRUE);
	} else if (strcmp(*argv, "single") == 0) {
	    set_entry(next++, GLX_NORMAL, GLX_DOUBLE, FALSE);
	} else if (strncmp(*argv, "rgbsize=", 8) == 0) {
	    set_entry(next++, GLX_NORMAL, GLX_RGBSIZE, atoi(*argv+8));
	} else if (strncmp(*argv, "depth=", 6) == 0) {
	    set_entry(next++, GLX_NORMAL, GLX_BUFSIZE, atoi(*argv+6));
	}
    }
    set_entry(next, 0, 0, 0); /* The input to GLXgetconfig is null terminated */

    if ( (dpy = XOpenDisplay(NULL)) == NULL)
	Fatal("can't open display\n");

    GLXShowConfig(dpy, initConfig, "input");

    if ((retconfig = GLXgetconfig(dpy, DefaultScreen(dpy), initConfig)) ==  0)
        Fatal("Hardware doesn't support that window type\n");

    GLXShowConfig(dpy, retconfig, "returned");
}
