/* $Header: /usr/local/src/nvrToMIT/RCS/testidct.c,v 1.6 1994/12/17 12:58:05 russ Exp $ */

#define NeedFunctionPrototypes 1

#include <math.h>
#include "vmpegdecomp.h"

#define TestDimension	24

unsigned long   DebugFlags[3];		/* for idct.o */
UChar *SignedClamp;			/* for motion.o */

MacroBlockInfo MbInfo;
UChar PastChannel[ MAX_X*TestDimension ];
UChar FutureChannel[ MAX_X*TestDimension ];
UChar NowChannel[ MAX_X*TestDimension ];
double FloatBlock[64];
char *ProgramName;


PrintMb()	/* for idct.o */
{
    Fatal("PrintMb called\n");
}

void ShowChannel(UChar *channel, char *banner)
{
    int i, j;
    UChar c;

    printf("%s:\n+", banner);
    for (i = 4; i < TestDimension-4; i++)
	printf("----");
    printf("+\n");
    for (i = 4; i < TestDimension-4; i++) {
	printf("|");
	for (j = 4; j < TestDimension-4; j++) {
	    c = channel[i*MAX_X + j];
	    if (c)
		printf("%4d", c);
	    else
		printf("   .");
	}
	printf("|\n");
    }
    printf("+");
    for (i = 4; i < TestDimension-4; i++)
	printf("----");
    printf("+\n");
}

void ShowFixedBlock(char *banner)
{
    long c;
    int i, j;

    printf("%s:\n+", banner);
    for (i = 0; i < 8; i++)
	printf("------");
    printf("+\n");
    for (i = 0; i < 8; i++) {
	printf("|");
	for (j = 0; j < 8; j++) {
	    c = MbInfo.dctCoeff[i*8+j];
	    if (c == 0)
		printf("     .");
	    else
		printf("%6d", RoundToInteger(c));
	}
	printf("|\n");
    }
    printf("+");
    for (i = 0; i < 8; i++)
	printf("------");
    printf("+\n");
}

void ShowFloatBlock(char *banner)
{
    float c;
    int i, j;

    printf("%s:\n+", banner);
    for (i = 0; i < 8; i++)
	printf("------");
    printf("+\n");
    for (i = 0; i < 8; i++) {
	printf("|");
	for (j = 0; j < 8; j++) {
	    c = FloatBlock[i*8+j];
	    if (c == 0)
		printf("     .");
	    else
		printf("%6.0f", c);
	}
	printf("|\n");
    }
    printf("+");
    for (i = 0; i < 8; i++)
	printf("------");
    printf("+\n");
	    
}

static void Dct2d8x8(double *);
static double RefIDctCoeffTable[64];

static void
InitDctReference()
{
    int ij, uv;
    double sqrtOf2Reciprocal;

    sqrtOf2Reciprocal = 1.0/sqrt(2.0);
    for (ij = 0; ij < 8; ij++) {
	for (uv = 0; uv < 8; uv++) {
	    RefIDctCoeffTable[ij*8+uv] = cos(M_PI*(2*ij+1)*uv/16.0) / 2.0;
	}
	RefIDctCoeffTable[ij*8+0] *= sqrtOf2Reciprocal;
    }
}

#define IDctCoeff(ij, uv)	RefIDctCoeffTable[ij*8 + uv]

#define MaxError	.00001		/* maximum tolerable error */

/*
 * Reference inverse Dct.
 *                7   7                       (2i+1)u*PI      (2j+1)v*PI
 * x(i,j) = (1/4) Sum Sum X(u,v) C(u)C(v) cos(----------) cos(-----------)
 *                u=0 v=0                          16              16
 */
static void
ReferenceIDct2d8x8(double *x)
{
    double X[8*8];
    int i, j, u, v;
    double pixel;

    memcpy(X, x, sizeof(X));
    for(i = 0; i < 8; i++) {
	for(j = 0; j < 8; j++) {
	    pixel = 0.0;
	    for(u = 0; u < 8; u++) {
		for(v = 0; v < 8; v++) {
		    pixel += X[8*u+v] * IDctCoeff(i,u) * IDctCoeff(j,v);
		}
	    }
	    x[8*i+j] = pixel;
	}
    }
}

static void
ShowDiffDct(char *name, double *orig, double *approx, double *reference)
{
    int i, j;
    double error;

    for (i = 0; i < 64; i++) {
	error = approx[i] - reference[i];
	if (error < 0 && error < -MaxError)
	    goto showDiff;
    }
    return;

showDiff:
    NormalOutput("original:\n");
    for (i = 0; i < 64; i++) {
	NormalOutput(" %9.4f", orig[i]);
	if ((i & 0x7) == 0x7)
	    NormalOutput("\n");
    }
    NormalOutput("reference %s:\n", name);
    for (i = 0; i < 64; i++) {
	NormalOutput(" %9.4f", reference[i]);
	if ((i & 0x7) == 0x7)
	    NormalOutput("\n");
    }
    NormalOutput("fast %s:\n", name);
    for (i = 0; i < 64; i++) {
	NormalOutput(" %9.4f", approx[i]);
	if ((i & 0x7) == 0x7)
	    NormalOutput("\n");
    }
    NormalOutput("difference in %s:\n", name);
    for (i = 0; i < 64; i++) {
	error = reference[i]-approx[i];
	if (error < 0 && error < -MaxError)
	    NormalOutput(" %9.4f", reference[i]-approx[i]);
	else
	    NormalOutput("     .    ");
	if ((i & 0x7) == 0x7)
	    NormalOutput("\n");
    }
}

void
ReferenceIdct()
{
    int i;

    ReferenceIDct2d8x8(FloatBlock);

    /* ShowFloatBlock("before clamp"); */
    memset(NowChannel, 0, sizeof(NowChannel));
    MbInfo.pTo = NowChannel + 8*MAX_X+8;
    for (i = 0; i < 8; i++) {
	Pass9NameFunc(NoMotionAndClamp,
		      (long)(FloatBlock[i*8+0] * FloatToIntScale),
		      (long)(FloatBlock[i*8+1] * FloatToIntScale),
		      (long)(FloatBlock[i*8+2] * FloatToIntScale),
		      (long)(FloatBlock[i*8+3] * FloatToIntScale),
		      (long)(FloatBlock[i*8+4] * FloatToIntScale),
		      (long)(FloatBlock[i*8+5] * FloatToIntScale),
		      (long)(FloatBlock[i*8+6] * FloatToIntScale),
		      (long)(FloatBlock[i*8+7] * FloatToIntScale),
		      i);
    }
}
void
FastLeeIdct()
{
    int i;

    memset(NowChannel, 0, sizeof(NowChannel));
    IDct(&MbInfo, NULL, NowChannel + 8*MAX_X+8, NULL, 0);
}

#define FIXED_PT_BITS 12
#define FloatToIntScale		(1<<FIXED_PT_BITS)
#define RealValue(real)		((real) * FloatToIntScale)
#define FixedToFloat(fixed)	((float)(fixed) / FloatToIntScale)

float FloatOf(long fixed)
{
    return( FixedToFloat(fixed) );
}

void
LoadCoeff(char *file, boolean integerLoad)
{
    FILE *fp;
    long c0, c1, c2, c3, c4, c5, c6, c7;
    ULong cells1 = 0, cells2 = 0;
    int i;

    if (file) {
	fp = fopen(file, "r");
	if (fp == NULL)
	    Fatal("Can't open %s\n", file);
    } else {
	fp = stdin;
    }

    for (i = 0; i < 8; i++) {
	if (fp == stdin)
	    printf("col %d: ", i);
	fscanf(fp, "%d%d%d%d%d%d%d%d",
	    &c0, &c1, &c2, &c3, &c4, &c5, &c6, &c7);

	if (integerLoad) {
	    FloatBlock[i*8+0] = c0;
	    FloatBlock[i*8+1] = c1;
	    FloatBlock[i*8+2] = c2;
	    FloatBlock[i*8+3] = c3;
	    FloatBlock[i*8+4] = c4;
	    FloatBlock[i*8+5] = c5;
	    FloatBlock[i*8+6] = c6;
	    FloatBlock[i*8+7] = c7;

	    MbInfo.dctCoeff[i*8+0] = c0 << FIXED_PT_BITS;
	    MbInfo.dctCoeff[i*8+1] = c1 << FIXED_PT_BITS;
	    MbInfo.dctCoeff[i*8+2] = c2 << FIXED_PT_BITS;
	    MbInfo.dctCoeff[i*8+3] = c3 << FIXED_PT_BITS;
	    MbInfo.dctCoeff[i*8+4] = c4 << FIXED_PT_BITS;
	    MbInfo.dctCoeff[i*8+5] = c5 << FIXED_PT_BITS;
	    MbInfo.dctCoeff[i*8+6] = c6 << FIXED_PT_BITS;
	    MbInfo.dctCoeff[i*8+7] = c7 << FIXED_PT_BITS;
	} else {
	    FloatBlock[i*8+0] = FixedToFloat(c0);
	    FloatBlock[i*8+1] = FixedToFloat(c1);
	    FloatBlock[i*8+2] = FixedToFloat(c2);
	    FloatBlock[i*8+3] = FixedToFloat(c3);
	    FloatBlock[i*8+4] = FixedToFloat(c4);
	    FloatBlock[i*8+5] = FixedToFloat(c5);
	    FloatBlock[i*8+6] = FixedToFloat(c6);
	    FloatBlock[i*8+7] = FixedToFloat(c7);

	    MbInfo.dctCoeff[i*8+0] = c0;
	    MbInfo.dctCoeff[i*8+1] = c1;
	    MbInfo.dctCoeff[i*8+2] = c2;
	    MbInfo.dctCoeff[i*8+3] = c3;
	    MbInfo.dctCoeff[i*8+4] = c4;
	    MbInfo.dctCoeff[i*8+5] = c5;
	    MbInfo.dctCoeff[i*8+6] = c6;
	    MbInfo.dctCoeff[i*8+7] = c7;
	}
	if (c0) MarkCellNZ(cells1, cells2, i*8+0);
	if (c1) MarkCellNZ(cells1, cells2, i*8+1);
	if (c2) MarkCellNZ(cells1, cells2, i*8+2);
	if (c3) MarkCellNZ(cells1, cells2, i*8+3);
	if (c4) MarkCellNZ(cells1, cells2, i*8+4);
	if (c5) MarkCellNZ(cells1, cells2, i*8+5);
	if (c6) MarkCellNZ(cells1, cells2, i*8+6);
	if (c7) MarkCellNZ(cells1, cells2, i*8+7);
    }
    MbInfo.cellsInBlock1 = cells1;
    MbInfo.cellsInBlock2 = cells2;
    if (fp != stdin)
	fclose(fp);
}

GlobalRegister(MacroBlockInfo *Pmb, "%g7");

main(int argc, char **argv)
{
    boolean integerLoad = True;

    ProgramName = *argv;
    Pmb = &MbInfo;

    InitDctReference();

    for (argc--, argv++; argc; argc--, argv++) {
	if (**argv == '-' || **argv == '+') {
	    switch(argv[0][1]) {
	    case 'i':
		integerLoad = (**argv == '+');
		break;
	    default:
		Fatal("arg %s\n", *argv);
	    }
	    continue;
	}
	LoadCoeff(*argv, integerLoad);

	/* ShowFixedBlock("fixed, before transform"); */
	ShowFloatBlock("float coefficients, before transform");
	ShowFixedBlock("fixed coefficients, before transform");
	ReferenceIdct();
	ShowChannel(NowChannel, "reference idct");
	FastLeeIdct();
	ShowChannel(NowChannel, "Fast Lee idct");
    }
}
