
#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"

#define DefaultNowPixVal	32
#define DefaultFuturePixVal	8
#define DefaultPastPixVal	48

#define TestDimension	24

UChar *SignedClamp;

GlobalRegister(MacroBlockInfo *Pmb, "%g7");
MacroBlockInfo MbInfo;
UChar PastChannel[ MAX_X*TestDimension ];
UChar FutureChannel[ MAX_X*TestDimension ];
UChar NowChannel[ MAX_X*TestDimension ];

extern void NoMotionAndClamp0Dc();
extern void NoMotionAndClamp();
extern void MotionHalfVertAndClamp0Dc();
extern void MotionHalfVertAndClamp();
extern void MotionHalfHorizAndClamp0Dc();
extern void MotionHalfHorizAndClamp();
extern void MotionHalfAndClamp0Dc();
extern void MotionHalfAndClamp();
extern void MotionAndClamp0Dc();
extern void MotionAndClamp();
extern void FwdBackMotionAndClamp0Dc();
extern void FwdBackMotionAndClamp();

typedef struct {
    char *name;
    VoidFuncPtr func;
    boolean time;	/* whether fwd and back are useful */
    boolean fwdBack;	/* whether fwd and back are both required */
    boolean zeroDc;
} TestInfo;

TestInfo TestList[] = {
    { "nomotion",  NoMotionAndClamp0Dc,        False, False, True },
    { "nomotion",  NoMotionAndClamp,           False, False, False },
    { "halfvert",  MotionHalfVertAndClamp0Dc,  True,  False, True },
    { "halfvert",  MotionHalfVertAndClamp,     True,  False, False },
    { "halfhoriz", MotionHalfHorizAndClamp0Dc, True,  False, True },
    { "halfhoriz", MotionHalfHorizAndClamp,    True,  False, False },
    { "half",      MotionHalfAndClamp0Dc,      True,  False, True },
    { "half",      MotionHalfAndClamp,         True,  False, False },
    { "motion",    MotionAndClamp0Dc,          True,  False, True },
    { "motion",    MotionAndClamp,             True,  False, False },
    { "fwdback",   FwdBackMotionAndClamp0Dc,   True,  True,  True },
    { "fwdback",   FwdBackMotionAndClamp,      True,  True,  False },
};

char *ProgramName;
boolean HorizTimeGradient;
boolean HorizNowGradient;
boolean VertTimeGradient;
boolean VertNowGradient;

main(int argc, char **argv)
{
    int i;
    int pastPixValue = DefaultPastPixVal;
    int futurePixValue = DefaultFuturePixVal;
    int nowPixValue = DefaultNowPixVal;
    boolean zeroDc = False;
    char *testName = "nomotion";
    VoidFuncPtr funcToTest = NoMotionAndClamp0Dc;

    ProgramName = argv[0];

    InitDCTClampLookup();

    Pmb = &MbInfo;
    for (argc--, argv++; argc; argc--, argv++) {
	if (**argv != '-') {
	    if (strcmp(*argv, "hgrad-time") == 0)
		HorizTimeGradient = True;
	    else if (strcmp(*argv, "vgrad-time") == 0)
		VertTimeGradient = True;
	    else if (strcmp(*argv, "hgrad-now") == 0)
		HorizNowGradient = True;
	    else if (strcmp(*argv, "vgrad-now") == 0)
		VertNowGradient = True;
	    else
		testName = *argv;
	    continue;
	}
	switch (argv[0][1]) {
	case 'p':
	    pastPixValue = atoi(argv[0]+2);
	    break;
	case 'f':
	    futurePixValue = atoi(argv[0]+2);
	    break;
	case 'n':
	    nowPixValue = atoi(argv[0]+2);
	    break;
	case 'z':
	    zeroDc = True;
	    break;
	default:
	    Fatal("arg %s?\n", *argv);
	    break;
	}
    }
    if (nowPixValue > 255 || pastPixValue > 255 || futurePixValue > 255)
	Fatal("Can't test pel values > 255\n");

    if (strcmp("all", testName) == 0) {
	for (i = 0; i < NumberOfCells(TestList); i++) {
	    Test(i, nowPixValue, pastPixValue, futurePixValue);
	}
    } else {
	for (i = 0; i < NumberOfCells(TestList); i++) {
	    if (strcmp(testName, TestList[i].name) == 0
	     && zeroDc == TestList[i].zeroDc) {
		Test(i, nowPixValue, pastPixValue, futurePixValue);
	    }
	}
    }
    exit (0);
}

UChar *InitChannel(UChar *channel, UChar value, boolean ignoreGradient)
{
#define XOFF 8
#define YOFF 8

    boolean horizGradient = ignoreGradient ? False : HorizTimeGradient;
    boolean vertGradient = ignoreGradient ? False : VertTimeGradient;
    int x, y;
    UChar pel = value;
    int shift = 0;

    for (y = 0; y < TestDimension; y++) {
	if (horizGradient)
	    pel = value-1;
	if (vertGradient)
	    pel++;
	for (x = 0; x < TestDimension; x++) {
	    if (horizGradient)
		pel++;
	    if (y < YOFF || y >= 8+YOFF || x < XOFF || x >= 8+XOFF) 
		shift = 1;
	    else
		shift = 0;
	    channel[ y*MAX_X + x ] = pel << shift;
	}
    }
    return (channel + YOFF*MAX_X + XOFF);

#undef XOFF
#undef YOFF
}

void ShowChannel(UChar *channel, UChar normal, char *banner)
{
    int i, j;
    UChar c;

    printf("%s, norm=%d:\n+", banner, normal);
    for (i = 0; i < TestDimension; i++)
	printf("---");
    printf("+\n");
    for (i = 0; i < TestDimension; i++) {
	printf("|");
	for (j = 0; j < TestDimension; j++) {
	    c = channel[i*MAX_X + j];
	    if (c == 0)
		printf("   ");
	    else if (c == normal)
		printf("  .");
	    else
		printf("%3d", c);
	}
	printf("|\n");
    }
    printf("+");
    for (i = 0; i < TestDimension; i++)
	printf("---");
    printf("+\n");
}

Test(int index, int nowPixValue, int pastPixValue, int futurePixValue)
{
    int i, j;
    VoidFuncPtr motionAndClamp = TestList[index].func;
    long c = nowPixValue << FIXED_PT_BITS;

    NormalOutput("testing %s, now=%d", TestList[index].name, nowPixValue);
    if (TestList[index].zeroDc)
	NormalOutput(", zero DC");
    if (TestList[index].fwdBack)
	NormalOutput(", past=%d, future=%d",
	    pastPixValue, futurePixValue);
    else if (TestList[index].time)
	NormalOutput(", time=%d", futurePixValue);
    NormalOutput("...\n");

    Pmb->pTo = InitChannel(NowChannel, 0, True);

    if (TestList[index].fwdBack || TestList[index].time) {
	Pmb->pFuture = InitChannel(FutureChannel, futurePixValue, False);
	ShowChannel(FutureChannel,
		    futurePixValue,
		    TestList[index].fwdBack ? "Future" : "Time");
    }
    if (TestList[index].fwdBack) {
	Pmb->pPast = InitChannel(PastChannel, pastPixValue, False);
	ShowChannel(PastChannel,
		    pastPixValue,
		    "Past");
    }

    if (TestList[index].zeroDc) {
	(*motionAndClamp)(c);
    } else {
	if (HorizNowGradient) {
	    long c1 = c+(1<<FIXED_PT_BITS);
	    long c2 = c+(2<<FIXED_PT_BITS);
	    long c3 = c+(3<<FIXED_PT_BITS);
	    long c4 = c+(4<<FIXED_PT_BITS);
	    long c5 = c+(5<<FIXED_PT_BITS);
	    long c6 = c+(6<<FIXED_PT_BITS);
	    long c7 = c+(7<<FIXED_PT_BITS);
	    Pass9VarFunc((*motionAndClamp), c,c1,c2,c3,c4,c5,c6,c7, 0);
	    Pass9VarFunc((*motionAndClamp), c,c1,c2,c3,c4,c5,c6,c7, 1);
	    Pass9VarFunc((*motionAndClamp), c,c1,c2,c3,c4,c5,c6,c7, 2);
	    Pass9VarFunc((*motionAndClamp), c,c1,c2,c3,c4,c5,c6,c7, 3);
	    Pass9VarFunc((*motionAndClamp), c,c1,c2,c3,c4,c5,c6,c7, 4);
	    Pass9VarFunc((*motionAndClamp), c,c1,c2,c3,c4,c5,c6,c7, 5);
	    Pass9VarFunc((*motionAndClamp), c,c1,c2,c3,c4,c5,c6,c7, 6);
	    Pass9VarFunc((*motionAndClamp), c,c1,c2,c3,c4,c5,c6,c7, 7);
	} else if (VertNowGradient) {
	    long c1 = c+(1<<FIXED_PT_BITS);
	    long c2 = c+(2<<FIXED_PT_BITS);
	    long c3 = c+(3<<FIXED_PT_BITS);
	    long c4 = c+(4<<FIXED_PT_BITS);
	    long c5 = c+(5<<FIXED_PT_BITS);
	    long c6 = c+(6<<FIXED_PT_BITS);
	    long c7 = c+(7<<FIXED_PT_BITS);
	    Pass9VarFunc((*motionAndClamp), c,c,c,c,c,c,c,c, 0);
	    Pass9VarFunc((*motionAndClamp), c1,c1,c1,c1,c1,c1,c1,c1, 1);
	    Pass9VarFunc((*motionAndClamp), c2,c2,c2,c2,c2,c2,c2,c2, 2);
	    Pass9VarFunc((*motionAndClamp), c3,c3,c3,c3,c3,c3,c3,c3, 3);
	    Pass9VarFunc((*motionAndClamp), c4,c4,c4,c4,c4,c4,c4,c4, 4);
	    Pass9VarFunc((*motionAndClamp), c5,c5,c5,c5,c5,c5,c5,c5, 5);
	    Pass9VarFunc((*motionAndClamp), c6,c6,c6,c6,c6,c6,c6,c6, 6);
	    Pass9VarFunc((*motionAndClamp), c7,c7,c7,c7,c7,c7,c7,c7, 7);
	} else {
	    Pass9VarFunc((*motionAndClamp), c,c,c,c,c,c,c,c, 0);
	    Pass9VarFunc((*motionAndClamp), c,c,c,c,c,c,c,c, 1);
	    Pass9VarFunc((*motionAndClamp), c,c,c,c,c,c,c,c, 2);
	    Pass9VarFunc((*motionAndClamp), c,c,c,c,c,c,c,c, 3);
	    Pass9VarFunc((*motionAndClamp), c,c,c,c,c,c,c,c, 4);
	    Pass9VarFunc((*motionAndClamp), c,c,c,c,c,c,c,c, 5);
	    Pass9VarFunc((*motionAndClamp), c,c,c,c,c,c,c,c, 6);
	    Pass9VarFunc((*motionAndClamp), c,c,c,c,c,c,c,c, 7);
	}
    }
    ShowChannel(NowChannel, nowPixValue, "Results");
}

/*
 * Prepare tables indexed by signed numbers in the range -255 to 511 and
 * unsigned numbers in the range 0 to 511.  The indexed cell will
 * yield a value clamped to -128 to 127 and 0 to 256, respectively.
 * XXX we are assuming that 8-bits signed are enough to hold any
 * difference block generated
 */
InitDCTClampLookup()
{
    long intValue;

    SignedClamp = (UChar *)SecureAlloc(SIGNED_CLAMP_SIZE);
    SignedClamp += 255;	/* this makes negative indices work */

    for (intValue = -255; intValue <= 256+255; intValue++) {
	if (intValue < 0)
	    SignedClamp[ intValue ] = 0; 
	else if (intValue > 255)
	    SignedClamp[ intValue ] = 255; 
	else
	    SignedClamp[ intValue ] = intValue; 
    }
}

