/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              testrgb.c
 *
 *      DESCRIPTION
 *              <overall description of what the package does>
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/testrgb.c,v 3.5 1993/10/22 23:57:30 russ Exp $
 */


#define NeedFunctionPrototypes 1

#include <stdio.h>
#include "nvrTypes.h"
#include "general.h"
#include "display.h"

ULong Picture[ 256*256 ];

#define True	1
#define False	0

char	*Program;

float Tolerance = .5;

void Error();
void Fatal();
void Usage();
void Msg();
void PrepareYCr();
double fabs();
double atof();

#define YWidth	256
#define YHeight	256
#define CWidth	128
#define CHeight	128

struct {
    double pad1;			/* force alignment to 8 bytes */
    UChar  y[YWidth * YHeight];
    double pad2;			/* force alignment to 8 bytes */
    UChar cr[CWidth * CHeight];
    double pad3;			/* force alignment to 8 bytes */
    UChar cb[CWidth * CHeight];
    double pad4;
    UChar  rgb[YWidth * YHeight];
} Frame;

main(int argc, char **argv)
{
    long miny = 16, maxy = 240;
    long mincr = 16, maxcr = 240;
    long mincb = 16, maxcb = 240;
    boolean doTime = False;

    for (argc--, argv++; argc; argc--, argv++) {
	if (strncmp("miny=", *argv, 5) == 0)
	    miny = atoi(*argv+5);
	else if (strncmp("maxy=", *argv, 5) == 0)
	    maxy = atoi(*argv+5);
	else if (strncmp("mincb=", *argv, 6) == 0)
	    mincb = atoi(*argv+6);
	else if (strncmp("maxcb=", *argv, 6) == 0)
	    maxcb = atoi(*argv+6);
	else if (strncmp("mincr=", *argv, 6) == 0)
	    mincr = atoi(*argv+6);
	else if (strncmp("maxcr=", *argv, 6) == 0)
	    maxcr = atoi(*argv+6);
        else if (strcmp("time", *argv) == 0)
	    doTime = True;
        else if (strcmp("compare", *argv) == 0)
	    doTime = False;
        else if (strncmp("tolerance=", *argv, 10) == 0)
	    Tolerance = atof(*argv+10);;
    }

    if (miny < 0 || maxy > 255)
	Fatal("y range bad\n");
    if (mincr < 0 || maxcr > 255)
	Fatal("cr range bad\n");
    if (mincb < 0 || maxcb > 255)
	Fatal("cb range bad\n");

    if (doTime) {
	TimeCalculateRGB(miny, maxy, mincr, maxcr, mincb, maxcb);
	TimeLookupRGB(YWidth, YHeight, 256);
    } else {
	CompareCalculateRGB(miny, maxy, mincr, maxcr, mincb, maxcb);
	CompareLookupRGB(miny, maxy, mincr, maxcr, mincb, maxcb);
    }
}

void NewCbValue()
{
    static int cbValue;
    ULong value;
    OctWord data, *pCb, *pCbEnd;

    value = cbValue<<24 + cbValue<<16 + cbValue<<8 + cbValue;
    data.l.high = data.l.low = value;
    pCb = (OctWord *)Frame.cb; 
    pCbEnd = (OctWord *)(&Frame.cb[CWidth * CHeight]);
    while (pCb < pCbEnd)
	*pCb++ = data;
    cbValue++;
}

void PrepareYCr()
{
    UChar *p;
    int i, j;

    for (p = Frame.y; p < &Frame.y[YWidth * YHeight]; ) {
	for (i = 0; i < 256; i++)
	    *p++ = i;
    }

    for (p = Frame.cr; p < &Frame.cr[CWidth * CHeight]; ) {
	for (i = 16; i < 240; i++)
	    for (j = 0; j < 64; j++)
		*p++ = i;
    }
}

#ifdef CALCULATE_RGB

#include "rgbdef.h"

TimeCalculateRGB(long miny, long maxy, long mincr, mlong axcr, long mincb,
		 long maxcb)
{
    long cr, cb, y;
    long p;	/* p for primary */
    ULong final, *pPict;
    int tmp, cbToRGB, crToRGB, yToRGB;
    long rAdd, gAdd, bAdd;

    for (cr = mincr; cr <= maxcr; cr++) {
	pPict = Picture;
	for (cb = mincb; cb <= maxcb; cb++) {
	    for (y = miny; y <= maxy; y++) {
		CrToR_Times(crToRGB, cr, tmp);
		rAdd = - FixedRConstant + crToRGB;
		CrToG_Times(crToRGB, cr, tmp);
		CbToG_Times(cbToRGB, cb, tmp);
		gAdd = FixedGConstant - cbToRGB - crToRGB;
		CbToB_Times(cbToRGB, cb, tmp);
		bAdd = - FixedBConstant + cbToRGB;

		YToRGB_Times(yToRGB, y, tmp);
		p = RDitherAdd(rAdd + yToRGB, D00);
		TruncateRGBBits(final, p, RDitherMask, RNormalizeShift, 24);
		p = GDitherAdd(gAdd + yToRGB, D00);
		TruncateRGBBits(final, p, GDitherMask, GNormalizeShift, 24);
		p = BDitherAdd(bAdd + yToRGB, D00);
		TruncateRGBBits(final, p, BDitherMask, BNormalizeShift, 24);
		*pPict++ = final;
	    }
	}
    }
}

CompareCalculateRGB(long miny, long maxy, long mincr, long maxcr, long mincb,
		    long maxcb)
{
    long cr, cb, y;
    long redInt, greenInt, blueInt;
    float redFloat, greenFloat, blueFloat;
    float redError, greenError, blueError;
    float maxRedError = 0, maxGreenError = 0, maxBlueError = 0;
    long p;
    int tmp, cbToRGB, crToRGB, yToRGB;
    long rAdd, gAdd, bAdd;

    mincr <<= FIXED_PT_BITS;
    mincb <<= FIXED_PT_BITS;
    miny <<= FIXED_PT_BITS;
    maxcr <<= FIXED_PT_BITS;
    maxcb <<= FIXED_PT_BITS;
    maxy <<= FIXED_PT_BITS;
    for (cr = mincr; cr <= maxcr; cr += (1<<FIXED_PT_BITS)) {
	for (cb = mincb; cb <= maxcb; cb += (1<<FIXED_PT_BITS)) {
	    for (y = miny; y <= maxy; y += (1<<FIXED_PT_BITS)) {
		CrToR_Times(crToRGB, cr, tmp);
		rAdd = - FixedRConstant + crToRGB;
		CrToG_Times(crToRGB, cr, tmp);
		CbToG_Times(cbToRGB, cb, tmp);
		gAdd = FixedGConstant - cbToRGB - crToRGB;
		CbToB_Times(cbToRGB, cb, tmp);
		bAdd = - FixedBConstant + cbToRGB;

		YToRGB_Times(yToRGB, y, tmp);
		redInt = (rAdd+yToRGB+(1<<(FIXED_PT_BITS-1)))>>FIXED_PT_BITS;
		greenInt = (gAdd+yToRGB+(1<<(FIXED_PT_BITS-1)))>>FIXED_PT_BITS;
		blueInt = (bAdd+yToRGB+(1<<(FIXED_PT_BITS-1)))>>FIXED_PT_BITS;
		RealYUVtoRGB(y>>FIXED_PT_BITS,
			     cr>>FIXED_PT_BITS,
			     cb>>FIXED_PT_BITS,
			     &redFloat, &greenFloat, &blueFloat);

		redError = fabs((float)redInt - redFloat);
		greenError = fabs((float)greenInt - greenFloat);
		blueError = fabs((float)blueInt - blueFloat);
		if (redError > maxRedError)
		    maxRedError = redError;
		if (greenError > maxGreenError)
		    maxGreenError = greenError;
		if (blueError > maxBlueError)
		    maxBlueError = blueError;
		if (redError < Tolerance
		 && greenError < Tolerance
		 && blueError < Tolerance)
		    continue;
		printf("y,cr,cb=%d,%d,%d, lookup: %d,%d,%d, calc: ",
		    y, cr, cb, redInt, greenInt, blueInt);
		printf("%.2f,%.2f, %.2f\n", redFloat, greenFloat, blueFloat);
	    }
	}
    }
}
#else /* CALCULATE_RGB */
CompareCalculateRGB() {}
TimeCalculateRGB() {}
#endif /* CALCULATE_RGB */

#ifdef LOOKUP_RGB

TimeLookupRGB(long width, long height, long frames)
{
    UChar *pY;
    UChar *pCr, *pCb;
    UChar *pRgb;
    long frame;

    PrepareYCr();
    for (frame = frames; --frame >= 0;) {
	pRgb = Frame.rgb;
	pY = Frame.y;
	pCr = Frame.cr;
	pCb = Frame.cb;

	NewCbValue();

	Lookup232RGBFrame(pRgb, pY, pCr, pCb, width, height, YWidth);
    }
    printf("%d pixels\n", width*height*frames);
}

CompareLookupRGB(long miny, long maxy, long mincr, long maxcr, long mincb,
		 long maxcb)
{
    long crShifted, cb, y, yIndex;
    long redInt, greenInt, blueInt;
    float redFloat, greenFloat, blueFloat;
    float redError, greenError, blueError;
    float maxRedError = 0, maxGreenError = 0, maxBlueError = 0;
    RGBLookup *rgbTable = &RGBTable;
    ULong rgbTriple;
    UChar *rBase, *gBase, *bBase, *primaryBase;

    mincr <<= 8;
    maxcr <<= 8;
    for (crShifted = mincr; crShifted <= maxcr; crShifted += 1<<8) {
	for (cb = mincb; cb <= maxcb; cb++) {
	    for (y = miny; y <= maxy; y++) {
		rgbTriple = rgbTable->crCbToRGB[ crShifted | cb ];
		rBase = rgbTable->lookup888.red + RIndexFromTriple(rgbTriple);
		gBase = rgbTable->lookup888.green + GIndexFromTriple(rgbTriple);
		bBase = rgbTable->lookup888.blue + BIndexFromTriple(rgbTriple);

		yIndex = (int)(y*PrimaryScaleFactor + .5);
		redInt = rBase[yIndex];
		greenInt = gBase[yIndex];
		blueInt = bBase[yIndex];
		RealYUVtoRGB(y, crShifted>>8, cb,
			     &redFloat, &greenFloat, &blueFloat);

		redError = fabs((float)redInt - redFloat);
		greenError = fabs((float)greenInt - greenFloat);
		blueError = fabs((float)blueInt - blueFloat);
		if (redError > maxRedError)
		    maxRedError = redError;
		if (greenError > maxGreenError)
		    maxGreenError = greenError;
		if (blueError > maxBlueError)
		    maxBlueError = blueError;
		if (redError < Tolerance
		 && greenError < Tolerance
		 && blueError < Tolerance)
		    continue;
		printf("y,cr,cb=%d,%d,%d, lookup: %d,%d,%d, calc: ",
		    y, crShifted>>8, cb, redInt, greenInt, blueInt);
		printf("%.2f,%.2f, %.2f\n", redFloat, greenFloat, blueFloat);
	    }
	}
    }
    printf("max red error=%.2f\n", maxRedError);
    printf("max green error=%.2f\n", maxGreenError);
    printf("max blue error=%.2f\n", maxGreenError);
}

#else /* LOOKUP_RGB */
TimeLookupRGB() {}
CompareLookupRGB() {}
#endif /* LOOKUP_RGB */

void Msg(char *fmt,int x0,int x1,int x2,int x3,int x4,int x5,int x6,
	 int x7,int x8,int x9)
{
    fprintf(stderr, fmt,x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
}

void Usage()
{
    Msg("Usage: %s\n", Program);
    exit (1);
}

void Error(char *fmt,int x0,int x1,int x2,int x3,int x4,int x5,int x6,
	   int x7,int x8,int x9)
{
    extern char	    *sys_errlist[];
    extern int	    errno;

    Msg("%s: ", Program);
    if (errno)
	Msg("%s: ", sys_errlist[ errno ]);
    Msg(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
}

void Fatal(char *fmt,int x0,int x1,int x2,int x3,int x4,int x5,int x6,
	   int x7,int x8,int x9)
{
    Error(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
    exit(1);
}
