/*
 * (c) Copyright 1992,1993,1994 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	xdef.h
 *
 * DESCRIPTION
 *	definitions for the X window system.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/xdef.h,v 4.10 1994/12/17 12:58:13 russ Exp $
 */
#ifndef _XDEF_H_
#define _XDEF_H_

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/XShm.h>
#include <sys/ipc.h>
#include <sys/shm.h>

typedef enum {
    X_UnknownState,
    X_InInit,
    X_InConfigure,
    X_ReadingEvents,
    X_InPutImage,
    X_InAllocBuffer
} XState;

typedef struct _imageInfo {
    XImage *image;
    Window w;
    int depth;
    int scrNumber;
    Visual *visual;		/* the associated visual structure */
    GeomRectangle src;		/* geometry within the image */
} ImageInfo;

typedef struct _timeImageInfo {
    ImageInfo img;
    UShort timeRef;		/* from the picture info */
    int zoomFactor;
    GeomRectangle src;		/* geometry within the image */
    ULong mbShowing[ (MAX_MB_PER_ROW*MAX_MB_PER_COL+31) / 32 ];
    boolean mbSubset;		/* true if subset of macro blocks showing */
} TimeImageInfo;

typedef struct _xInfo {
    Display *dpy;		/* connection to the server */
    GC gc;			/* gc for the current window/depth */
    char *displayName;		/* string name of X server */
    ImageInfo decode;		/* the main decoder window */
    XState state;		/* current state */
    Window *decodeParents;	/* all ancestors of the decoder's window */
    long nDecodeParents;	/* count of windows in decodeParents */
    TimeImageInfo *past;
    TimeImageInfo *current;
    TimeImageInfo *future;
    int depth;			/* the depth to use for X */
    boolean useMitShm;		/* whether to use share mem extension */
    XShmSegmentInfo *shm;	/* shared mem info */
    int (*putImage)();		/* XPutImage function to use */
    DisplayInfoPtr dpyInfo;	/* private copy */

#   if defined(sgi)
	PrivPointer glInfo;
#   endif

} XInfo;

#define MbShowing(xpic, mbAddr)	\
	    ((xpic)->imgInfo.mbShowing[ (mbAddr)>>5 ] & (1<<(mbAddr&0x1f)))
#define MarkMbShowing(xpic, mbAddr)	\
	    (xpic)->imgInfo.mbShowing[ (mbAddr)>>5 ] |= (1<<(mbAddr&0x1f))
#define ClearAllMbShowing(xpic)						\
    {									\
	int _i;								\
	for (_i = 0; _i < (MB_PER_ROW*MB_PER_COL+31) / 32; _i++)	\
	    (xpic)->imgInfo.mbShowing[ _i ] = 0;			\
    }

#define NextPicture 1	/* returned by HandleEvent when user hits 'n' */
#define NextBlock 2	/* returned by HandleEvent when user hits 'b' */

#define MAXXCOLORS	((1<<16)-1)
/*
 * These are indicies for the different windows
 */
#define XVIDEOPICT	0
#define XFIRSTPICT	1
#define XPASTPICT	1
#define XFUTUREPICT	2
#define XBPICT		3

#ifdef notdef
#define XPASTSRCMB	4
#define XFUTURESTSRCMB	5
#define XDESTMB		6
#endif

#define NXPICTS		4   /* number of simultaneous windows */

#   if defined(sgi)
	extern void GLXInit(XInfo *);
	extern void GLXSetWinClip(XInfo *);
	extern void GLXSetBuffer(XInfo *);
#   else
#	define GLXInit(xInfo)	/* as nothing */
#	define GLXSetWinClip(xInfo)	/* as nothing */
#	define GLXSetBuffer(xInfo)	/* as nothing */
#	ifndef Solaris2
		extern int   shmctl(int, int, struct shmid_ds *);
		extern char *shmat(int, char *, int);
		extern int   shmdt(char *);
		extern int   shmget(key_t, int, int);
#	endif
#   endif

extern Bool  XShmQueryExtension (Display *);

#endif /* _XDEF_H_ */
