/*
 *      (c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              display.c
 *
 *      DESCRIPTION
 *              handle display activity
 *		X-DEPENDENT
 *		OS-DEPENDENT
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/xdisplay.c,v 4.42 1995/08/18 00:32:22 russ Exp $
 */

#define NeedFunctionPrototypes 1

#include "vmpegdecomp.h"

#include "xdef.h"

static char *NoSharedMemSegments = "A shared memory segment is not available.\n\
    you may continue at a degraded performance, or you can try\n\
    to free them and restart(refer to 'ipcs' and ipcrm' man pages).";

static XInfo XDisplayInfo;

char *WindowStateNames[] = {
    "obscured",
    "part obscured",
    "unobscured"
};

/*ARGSUSED*/
static int
Never(Display *dpy, XEvent *event, char *ignore)
{
    return (False);
}

static void
XServerInputNotify(int fd, PrivPointer data)
{
    DisplayInfoPtr dpyInfo = (DisplayInfoPtr)data;

    dpyInfo->u.s.readXServer = True;
    BlockAsyncNotification(ConnectionNumber(XDisplayInfo.dpy));
}

static void
CheckClipBox(XInfo *xInfo, int x, int y, UInt width, UInt height)
{
    XSetWindowAttributes xswa;
    DisplayInfoPtr dpyInfo = xInfo->dpyInfo;
    unsigned long mask;
    Visual visual;
    Window win;
    GeomRectangle box;

    if (dpyInfo->clip.x != x
     || dpyInfo->clip.y != y
     || dpyInfo->clip.width != width
     || dpyInfo->clip.height != height) {
	DebugTask(N_GeometryInfo,
	    Msg("new g=%dx%d+%d+%d\n", width, height, x, y)
	)

	if (dpyInfo->destType == DestWinFrameBuffer
	 && dpyInfo->state == DisplayRunning) {
	    /*
	     * If we are running have have been moved, then we need to
	     * do what xrefresh does to clean up the display.  If not,
	     * then everything will be taken care of by X.
	     */
	    box = dpyInfo->dst;
	    if (width > box.width)
		box.width = width;
	    if (height > box.height)
		box.height = height;
	    if (x < box.x) {
		box.width += box.x - x;
		box.x = x;
	    }
	    if (y < box.y) {
		box.height += box.y - y;
		box.y = y;
	    }
	    xswa.override_redirect = True;
	    xswa.backing_store = NotUseful;
	    xswa.save_under = False;
	    xswa.background_pixmap = ParentRelative;
	    mask = (CWBackPixmap
		  | CWOverrideRedirect
		  | CWBackingStore
		  | CWSaveUnder);
	    visual.visualid = CopyFromParent;
	    win = XCreateWindow(xInfo->dpy,
		    DefaultRootWindow(xInfo->dpy),
		    box.x, box.y, box.width, box.height,
		    0,	/* border width */
		    DefaultDepth(xInfo->dpy, DefaultScreen(xInfo->dpy)),
		    InputOutput, &visual, mask, &xswa);
	    XMapWindow (xInfo->dpy, win);
	    XDestroyWindow(xInfo->dpy, win);
	}

	dpyInfo->new.clip.x = x;
	dpyInfo->new.clip.y = y;
	dpyInfo->new.clip.width = width;
	dpyInfo->new.clip.height = height;
	dpyInfo->new.changed |= DPY_Clip;

	/*
	 * XXX this should really come from the client 
	 *
	 * dpyInfo->new.dst.width = width;
	 * dpyInfo->new.dst.height = height;
	 * dpyInfo->new.changed |= DPY_DestRect;
	 */
	dpyInfo->u.s.reconfigure = True;
    }
}

static void
SelectEvents(XInfo *xInfo, int mask)
{
    int i;

    for (i = 0; i < xInfo->nDecodeParents; i++) {
	XSelectInput (xInfo->dpy, xInfo->decodeParents[i], mask);
	DebugTask(N_GeometryInfo,
	   Msg("Select(0x%x,0x%x)\n", xInfo->decodeParents[i], mask);
	)
    }

    if (mask)
	mask |= (VisibilityChangeMask | ExposureMask);
    XSelectInput (xInfo->dpy, xInfo->decode.w, mask);
}

static boolean
FindDecoderAncestors(XInfo *xInfo)
{
    Window root, current, parent, *children;
    unsigned int nchildren;
    int nParents = 0;
    Display *dpy = xInfo->dpy;

    parent = xInfo->decode.w;
    root = 0;
    for (;;) {
	current = parent;
	if (! XQueryTree(dpy, current, &root, &parent, &children, &nchildren))
	    return(False);
	XFree((void *)children);
	if (parent == root)
	    break;
	if (parent == 0) /* the output window *is* the root window */
	    break;
	nParents++;
	xInfo->decodeParents = (Window *)
	    NullRealloc((char *)xInfo->decodeParents,
			nParents * sizeof(Window));
	xInfo->decodeParents[nParents-1] = parent;
    }
    xInfo->nDecodeParents = nParents;
    return(True);
}

void
HandleWinInput(DisplayInfoPtr dpyInfo)
{
    XInfo *xInfo = &XDisplayInfo;
    boolean reconfigured = False;
    int rootX, rootY;
    int width, height;
    Window child;
    XEvent event;
    Display *dpy;
    XState state;

    state = xInfo->state;
    xInfo->state = X_ReadingEvents;
    dpy = xInfo->dpy;
    dpyInfo->u.s.readXServer = False;

    /*
     * use the unmodified dimensions as default
     */
    width = dpyInfo->new.clip.width;
    height = dpyInfo->new.clip.height;

    (void) XEventsQueued(dpy, QueuedAfterFlush);
    AllowAsyncNotification(ConnectionNumber(XDisplayInfo.dpy));
    while (dpy->qlen) {
	XNextEvent(dpy, &event);
	switch(event.type) {
	case VisibilityNotify:
	    switch(event.xvisibility.state) {
	    case VisibilityUnobscured:
		dpyInfo->windowState = WindowUnobscured;
		break;
	    case VisibilityPartiallyObscured:
		dpyInfo->windowState = WindowPartiallyObscured;
		break;
	    case VisibilityFullyObscured:
		dpyInfo->windowState = WindowObscured;
		break;
	    }
	    break;

	case ReparentNotify:
	    DebugTask(N_GeometryInfo, Msg("X reparent w=0x%x, +%d+%d\n",
		event.xreparent.window,
		event.xreparent.x, event.xreparent.y)
	    )
	    reconfigured = True;
	    (void) FindDecoderAncestors(xInfo);
	    SelectEvents(xInfo, StructureNotifyMask);
	    break;

	case ConfigureNotify:
	    DebugTask(N_GeometryInfo, Msg("X config w=0x%x%s, %dx%d\n",
		event.xconfigure.window,
		(xInfo->decode.w == event.xconfigure.window) ? "*" : "",
		event.xconfigure.width, event.xconfigure.height)
	    )
	    if (xInfo->decode.w == event.xconfigure.window) {
		width = event.xconfigure.width;
		height = event.xconfigure.height;
	    }
	    reconfigured = True;
	    break;

	case Expose:
	    DebugTask(N_GeometryInfo, Msg("X expose w=0x%x%s, %dx%d+%d+%d\n",
		event.xexpose.window,
		(xInfo->decode.w == event.xexpose.window) ? "*" : "",
		event.xexpose.width, event.xexpose.height,
		event.xexpose.x, event.xexpose.y)
	    )
	    if (xInfo->decode.w == event.xexpose.window) {
		if (dpyInfo->state != DisplayRunning)
		    dpyInfo->refresh = True;
		if (dpyInfo->windowState == WindowObscured)
		    dpyInfo->windowState = WindowPartiallyObscured;
	    }
	    break;
	}
    }

    if (reconfigured) {
	XTranslateCoordinates(xInfo->dpy, xInfo->decode.w,
                              DefaultRootWindow(xInfo->dpy),
                              0, 0, &rootX, &rootY, &child);
	CheckClipBox(xInfo, rootX, rootY, width, height);
    }

    XFlush(xInfo->dpy);
    xInfo->state = state;
}

static void
TrackWindowChanges(XInfo *xInfo, boolean start)
{
    int x, y;
    int rootX, rootY;
    UInt width, height;
    UInt borderWidth;
    UInt depth;
    Window root, child;
    
    if (start) {
	AllocAsyncHandler(ConnectionNumber(xInfo->dpy),
			  XServerInputNotify,
			  False,
			  (PrivPointer)xInfo->dpyInfo);
	SelectEvents(xInfo, StructureNotifyMask);
	XGetGeometry(xInfo->dpy, xInfo->decode.w,
	    &root, &x, &y, &width, &height, &borderWidth, &depth);
	XTranslateCoordinates(xInfo->dpy, xInfo->decode.w,
                              DefaultRootWindow(xInfo->dpy),
                              0, 0, &rootX, &rootY, &child);
	CheckClipBox(xInfo, rootX, rootY, width, height);
    } else {
	FreeAsyncHandler(ConnectionNumber(xInfo->dpy));
	SelectEvents(xInfo, 0);
    }
}

static void
CloseWinConnection(XInfo *xInfo)
{
    if (xInfo->displayName) {
	FreeAsyncHandler(ConnectionNumber(xInfo->dpy));
	XCloseDisplay(xInfo->dpy);
	free((char *)xInfo->displayName);
	xInfo->displayName = NULL;
    }
}

static int
WinIOHandler(Display *dpy)
{
    ParseValue value;
    XInfo *xInfo = &XDisplayInfo;
    SequenceInfoPtr pSeq = xInfo->dpyInfo->pSeq;
    char *reason;

    switch (xInfo->state) {
    case X_UnknownState:  reason = "X IO error"; break;
    case X_InInit:        reason = "X IO error during X init"; break;
    case X_InConfigure:   reason = "X IO error during X config"; break;
    case X_ReadingEvents: reason = "X IO error while reading X events"; break;
    case X_InPutImage:    reason = "X IO error during XPutImage"; break;
    case X_InAllocBuffer: reason = "X IO error allocating buffer"; break;
    }
    value.string = reason;
    EventToClient(pSeq->pParse, MCS_BAD_DISPLAY, &value, MCS_TYPE_STRING);
    CloseWinConnection(xInfo);
    RestoreContext(pSeq->decodeContext, 1);
    return(0);
}

static int
WinErrorHandler(Display *dpy, XErrorEvent *errorEvent)
{
    XInfo *xInfo = &XDisplayInfo;
    char *reason;

    switch (xInfo->state) {
    case X_UnknownState:  reason = "X error"; break;
    case X_InInit:        reason = "X error during X init"; break;
    case X_InConfigure:   reason = "X error during X config"; break;
    case X_ReadingEvents: reason = "X error while reading X events"; break;
    case X_InPutImage:    reason = "X error during XPutImage"; break;
    case X_InAllocBuffer:
	if (xInfo->useMitShm)
	    reason = "X error in XSHM init";
	else
	    reason = "X error in buffer allocation";
	xInfo->useMitShm = False;
	break;
    }
    Msg("%s\n", reason);
    return(0);
}

static int
_XPutImage(Display *dpy, Drawable dr, GC gc, XImage *image,
	   int src_x, int src_y, int dst_x, int dst_y,
	   unsigned int width, unsigned int height)
{
    return( XShmPutImage(dpy,dr,gc,image,src_x,src_y,dst_x,dst_y,
			 width,height,True) );
}

/*ARGSUSED*/
static void
RefreshWinPixels(DisplayInfoPtr dpyInfo)
{
    XInfo *xInfo = &XDisplayInfo;
    GeomRectanglePtr src = &dpyInfo->src;
    GeomRectanglePtr dst = &dpyInfo->dst;
    XEvent event;
    XState state;

    state = xInfo->state;
    xInfo->state = X_InPutImage;
    /*
     * If the normal mode is *not* to use the window system,
     * we have to undo clipping
     */
    if (dpyInfo->destType == DestWinFrameBuffer)
	dst = &dpyInfo->new.dst;

    /*
     * Turn off notification while we are sending the image, because
     * we get all kinds of SIGIO signals.
     *
     * BlockAsyncNotification(ConnectionNumber(XDisplayInfo.dpy));
     */

    /*
     * Note that we assume the scaling implied in the difference
     * between dst->{width,height} and src->{width,height} has
     * already been taken care of during the conversion step
     */
    
    (*xInfo->putImage)(xInfo->dpy, xInfo->decode.w, xInfo->gc,
		      xInfo->decode.image,
		      src->x, src->y, dst->x, dst->y, dst->width, dst->height);

    /*
     * We really don't care if the window gets reconfigured when
     * we use X... the window system does the work.
     */
    if (dpyInfo->destType == DestWindow)
	(void) XCheckIfEvent(xInfo->dpy, &event, Never, NULL);
    else
	XFlush(xInfo->dpy);
    /* AllowAsyncNotification(ConnectionNumber(XDisplayInfo.dpy)); */
    xInfo->state = state;
}

void
SetWinBuffer(DisplayInfoPtr dpyInfo)
{
    GLXSetBuffer(&XDisplayInfo);
}

void
SetWinClip(DisplayInfoPtr dpyInfo)
{
    GLXSetWinClip(&XDisplayInfo);
}

static void
InitXCopiers(DisplayInfoPtr dpyInfo)
{
    static boolean firstTime = True;

    if (!firstTime)
	return;

    dpyInfo->copiers[ Rgb888 ].winDisplay = RefreshWinPixels;
    dpyInfo->copiers[ Rgb444 ].winDisplay = RefreshWinPixels;
    dpyInfo->copiers[ Rgb555 ].winDisplay = RefreshWinPixels;
    dpyInfo->copiers[ Rgb332 ].winDisplay = RefreshWinPixels;
    dpyInfo->copiers[ Rgb232 ].winDisplay = RefreshWinPixels;
    dpyInfo->copiers[ RgbC555 ].winDisplay = RefreshWinPixels;
    dpyInfo->copiers[ RgbC666 ].winDisplay = RefreshWinPixels;
    dpyInfo->copiers[ GrayScale8 ].winDisplay = RefreshWinPixels;
    dpyInfo->copiers[ GrayScale7 ].winDisplay = RefreshWinPixels;
    dpyInfo->copiers[ GrayScale1 ].winDisplay = RefreshWinPixels;
}

/*
** This is called every time the window geometry changes
*/
ImageReasonPtr
WinInit(DisplayInfoPtr dpyInfo)
{
    XInfo *xInfo = &XDisplayInfo;
    Display *dpy;
    extern char *getenv();
    XWindowAttributes winAttr;
    XState state;
    ImageReasonPtr reason = NULL;
    int newDepth;

    InitXCopiers(dpyInfo);

    state = xInfo->state;
    xInfo->state = X_InInit;

    if (xInfo->displayName == NULL
     || strcmp(xInfo->displayName, dpyInfo->windowSysName) != 0) {
	xInfo->dpyInfo = dpyInfo;

	/*
	 * Reconnect to the server only if it is different than before.
	 */
	CloseWinConnection(xInfo);
	XSetErrorHandler(WinErrorHandler);
	XSetIOErrorHandler(WinIOHandler);
	if ( (dpy = XOpenDisplay(dpyInfo->windowSysName)) == NULL) {
	    reason = &IRBadDisplay;
	    goto xInitDone;
	}
	xInfo->dpy = dpy;
	xInfo->displayName = CopyOfString(dpyInfo->windowSysName);
	xInfo->state = X_InConfigure;
	xInfo->decode.w = ~dpyInfo->windowId;	/* force it to be different */
	if( XShmQueryExtension(dpy) ) {
	    xInfo->putImage = _XPutImage;
	    xInfo->useMitShm = True;
	} else{
	    xInfo->putImage = XPutImage;
	    xInfo->useMitShm = False;
	}
    } else {
	/*
	 * If the window id changed (but we are on the same server),
	 * then we should unselect events that were selected before.
	 */
	if (xInfo->decode.w != dpyInfo->windowId) {
	    TrackWindowChanges(xInfo, False);
	}
	dpy = xInfo->dpy;
    }

    if (xInfo->decode.w != dpyInfo->windowId) {
	xInfo->state = X_InConfigure;
	xInfo->decode.w = dpyInfo->windowId;
	if (! FindDecoderAncestors(xInfo)) {
	    reason = &IRBadWindow;
	    goto xInitDone;
	}

	/*
	 * XXX we really can't tell here if the window is mapped but
	 * partially obscured.
	 */
	XGetWindowAttributes(dpy, dpyInfo->windowId, &winAttr);
	xInfo->decode.visual = winAttr.visual;
	xInfo->decode.depth = winAttr.depth;
	xInfo->decode.scrNumber = XScreenNumberOfScreen(winAttr.screen);
	switch (winAttr.map_state) {
	case IsUnmapped:	dpyInfo->windowState = WindowObscured; break;
	case IsUnviewable:	dpyInfo->windowState = WindowObscured; break;
	case IsViewable:	dpyInfo->windowState = WindowUnobscured; break;
	}

	TrackWindowChanges(xInfo, True);
    }

    switch(dpyInfo->color) {
    case GrayScale1:
	newDepth = 1;
	break;

    default:
    case GrayScale7:
    case GrayScale8:
	newDepth = 8;
        break;

    case Rgb232:
    case Rgb332:
    case RgbC555:
    case RgbC666:
        dpyInfo->cvtu.ycToRgb->components = Pixel0RGB;
	newDepth = 8;
        break;

    case Rgb555:
    case Rgb444:
    case Rgb888:
	if (xInfo->decode.visual->class == PseudoColor)
	    dpyInfo->cvtu.ycToRgb->components = Pixel0RGB;
	else
	    dpyInfo->cvtu.ycToRgb->components = Pixel0BGR;
	switch(dpyInfo->color) {
	case Rgb555: newDepth = 16; break;
	case Rgb444: newDepth = 12; break;
	case Rgb888: newDepth = (xInfo->decode.depth == 32 ? 32 : 24); break;
	}
	break;
    }

    if (newDepth != xInfo->decode.depth) {
	return(& IRBadVisual);
	goto xInitDone;
    }

    if (newDepth != xInfo->depth) {
	if (xInfo->gc)
	    XFreeGC(xInfo->dpy, xInfo->gc);
	xInfo->gc = XCreateGC(xInfo->dpy,
			      xInfo->decode.w,
			      0,	/* value mask */
			      (XGCValues *)NULL);
	xInfo->depth = newDepth;
    }
    GLXInit(xInfo);
    XFlush(xInfo->dpy);

xInitDone:
    xInfo->state = state;
    return (reason);
}

/*
 * must be called before exit or else the shared memory segment
 *	(if one was allocated) won't be freed up 
 */
void
FreeCopyBuffer(UChar *copyBuffer)
{
    XInfo *xInfo = &XDisplayInfo;

    if( xInfo->useMitShm ) {
	shmctl(xInfo->shm->shmid,IPC_RMID,NULL);
	shmdt(xInfo->shm->shmaddr);
	free((char *)xInfo->shm);
    } else
	free ((char *)copyBuffer);
}

UChar *
AllocateCopyBuffer(long size, int width, int height)
{
    XInfo *xInfo = &XDisplayInfo;
    UChar *copyBuffer;
    XState state;
    IntFuncPtr oldSync;

    state = xInfo->state;
    xInfo->state = X_InAllocBuffer;

    if(xInfo->displayName == NULL)
	goto plainMemory;

    oldSync = XSynchronize(xInfo->dpy, 1);
    
    if( xInfo->useMitShm ) {
	xInfo->shm = (XShmSegmentInfo *) InsecureAlloc(sizeof(XShmSegmentInfo));
	if (xInfo->shm == NULL) {
	    Msg("can't allocate shared mem\n");
	    goto plainMemory;
	}
	xInfo->shm->shmid = shmget(IPC_PRIVATE, size, IPC_CREAT|0777);
	if( xInfo->shm->shmid < 0 ) {
	    EventToClient(xInfo->dpyInfo->pSeq->pParse, MCS_EVENT_INFO,
			  (ParseValuePtr)&NoSharedMemSegments, MCS_TYPE_STRING);
	    goto plainMemory;
	}
	/* This memory should be page aligned */
	xInfo->shm->shmaddr = (char *)shmat(xInfo->shm->shmid,0,0);
	if( xInfo->shm->shmaddr == (char *)-1 ) {
	    SystemError("shmat failure\n");
	    goto plainMemory;
	}
	xInfo->shm->readOnly = False;
	copyBuffer = xInfo->shm->shmaddr;
	xInfo->decode.image =
	    XShmCreateImage(xInfo->dpy,
			    NULL,		/* visual (used for rgb mask) */
			    xInfo->depth,
			    ZPixmap,
			    (char *)copyBuffer,
			    xInfo->shm,
			    width, height);
	if (xInfo->decode.image == NULL) {
	    SystemError("can't allocate XShm image\n");
	    goto plainMemory;
	}
	if( !XShmAttach(xInfo->dpy, xInfo->shm) || ! xInfo->useMitShm) {
	    (*xInfo->decode.image->f.destroy_image)(xInfo->decode.image);
	    xInfo->decode.image = NULL;
	    ProgMsg("XShmAttach failed, switching to X transport\n");
	    goto plainMemory;
	}
    } else {
    plainMemory:
	copyBuffer = (UChar *)SecureAllocAligned(sizeof(OctWord), size);
	if (xInfo->displayName == NULL)
	    goto nonXAllocation;

	xInfo->useMitShm = False;
	xInfo->putImage = XPutImage;
	if (xInfo->shm)
	    free((char *)xInfo->shm);
	xInfo->shm = NULL;

	if (xInfo->decode.image != NULL)
	    XFree((void *)xInfo->decode.image);

	xInfo->decode.image =
	    XCreateImage(xInfo->dpy,
			 NULL,			/* visual (used for rgb mask) */
			 xInfo->depth,
			 ZPixmap,
			 0,			/* offset */
			 (char *)copyBuffer,
			 width, height,
			 32, 0);		/* bitmap pad, bytes/line */
	if (xInfo->decode.image == NULL) {
	    FreeCopyBuffer(copyBuffer);
	    copyBuffer = NULL;
	    ProgMsg("unable to create image");
	}
    }

    xInfo->state = state;
    XSetAfterFunction(xInfo->dpy, oldSync);

nonXAllocation:
    return (copyBuffer);
}
