/*
 *	Copyright (c) 1993  North Valley Research, Inc.
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research, Inc. has obtained a licensing right,
 *	and is considered by North Valley Research, Inc. to be confidential.
 *	It is protected by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		NvrLmFeatures.h
 *
 *	DESCRIPTION
 *		
 *		
 *	DESCRIPTION
 *		$Header: /usr/local/src/nvrToMIT/RCS/NvrLmFeatures.h,v 1.17 1995/05/27 00:13:35 russ Exp $
 *
 */
#ifndef _NVRLMFEATURES_H_
#define _NVRLMFEATURES_H_

#include "nvrTypes.h"
#include "nvrCrypt.h"

/*
 * These bits define a hex strring that goes in the "vendor data".  This
 * way we can prevent someone who buys a sun license from using sgi
 * version, etc.  For demo licenses we use no bits
 */
#define LIC_ANY_ARCHITECTURE	0x0000
#define LIC_SGI_ARCHITECTURE	0x0001
#define LIC_SOL1_ARCHITECTURE	0x0002
#define LIC_SOL2_ARCHITECTURE	0x0004
#define LIC_HP_ARCHITECTURE	0x0008

#ifdef sun
# ifdef Solaris2
#  define LIC_THIS_ARCHITECTURE	LIC_SOL2_ARCHITECTURE
# else
#  define LIC_THIS_ARCHITECTURE	LIC_SOL1_ARCHITECTURE
# endif
#endif

#ifdef sgi
#define LIC_THIS_ARCHITECTURE	LIC_SGI_ARCHITECTURE
#endif

#ifdef _HPUX_SOURCE
#define LIC_THIS_ARCHITECTURE	LIC_HP_ARCHITECTURE
#endif

/*
 * These numbers should be incremented at major releases so that
 * customers with old licenses can't use the new software.
 */

/*
 * Non-Shared License Feature for product components
 * This type of license allows a user to use only the specified
 * component.
 */
#define NVRLM_MPEGDECOMP	"0011"
#define NVRLM_PLXJPEG		"0012"
#define NVRLM_AMPEGCOMP		"0013"
#define NVRLM_VMPEGCOMP		"0014"
#define NVRLM_SMPEGCOMP		"0015"
#define NVRLM_IMAGEOP		"0016"
#define NVRLM_SPATIALFILTER	"0017"
#define NVRLM_MULTIPROCESSOR	"0018"
#define NVRLM_MPEGDEMUX		"0019"

/*
 * Feature groups for product configurations
 */
#define NVRLM_MPEG_COMP		"1001"
#define NVRLM_MPEG_LITE		"1002"

/*
 * Feature to cover all nvr software
 */
#define NVRLM_ALL_SOFTWARE	"2001"

/*
 * ERROR_CODES
 */

#define LM_OK				0
#define LM_ERROR			(-1)
#define LM_WAIT				(-2)
#define LM_NO_LICENSES			(-3)
#define LM_SRV_DOWN			(-4)
#define LM_AUTH_FAIL			(-5)
#define LM_EXPIRED			(-6)
#define LM_NOHOST			(-7)
#define LM_SOCKFAIL			(-8)
#define LM_SENDFAIL			(-9)
#define LM_LOST				(-10)
#define LM_NOKEY			(-11)
#define LM_NOT_LICENSED			(-12)
#define LM_NO_SERVICE			(-13)
#define LM_CODE_CORRUPT			(-14)
#define LM_NO_SUCH_FEATURE		(-15)
#define LM_INVALID_CLIENT		(-16)
#define LM_INSUFFICIENT_SERVERS		(-17)
#define LM_CANT_WRITE			(-18)
#define LM_CANT_READ			(-19)


    /*
     * MISC
     */
#define BEAT		60
#define DEFAULT_KEYDIR	"/usr/lib/elm"
#define NVRKEYDIR_ENV	"NVRKEYDIR"
#define NVRHOST_ENV	"SYNC_ELMHOST"
#define NVRHOME_ENV     "NVRHOME"
#define MAXNAMLEN	255

/*
 * These bits define what to include in the copyright notice, and
 * where to display it
 */
typedef enum {
    NvrCopyright      = 0x0001,
    LmCopyright       = 0x0002,
    TekCopyright      = 0x0004,

    CopyrightToScreen = 0x0010,
    CopyrightToTty    = 0x0020
} CopyrightBits;

typedef void (*LmExceptionProc)(
#if NeedFunctionPrototypes
    int		/* condition */,
    char*	/* feature */
#endif
);

typedef char FeatureName[8];
typedef FeatureName *FeatureNamePtr;

typedef struct _lmConfig {
    FeatureName *features;
    boolean doHeartbeat;
    boolean shared;
    LmExceptionProc exceptionCB;

    /*
     * returned state
     */
    char *featureUsed;
    int licenseError;		/* ... if any */
} LmConfig, *LmConfigPtr;

typedef struct _keyData {
    UShort nodeLocked: 1;
    UShort demoKey   : 1;	/* true=> expires 'week'; else issued 'week' */
    UShort netValid  : 1;	/* true=> netAddress if valid */
    UShort hostValid : 1;	/* true=> hostId if valid */
    UShort random    :12;	/* random bits to vary the encoding */
    UShort week;		/* week number counting from Jan 1, 1990 */
    ULong feature;
    ULong hostId;
    ULong netAddress;
} KeyData, *KeyDataPtr;

/*
 * We may not want to include Intrinsic.h
 */
#ifndef _XtIntrinsic_h
typedef void *XtAppContext;
#endif

#ifdef LM_DISABLED

# define NvrGetLicense(config)	LM_OK
# define NvrXtGetLicense(app_context, config)	LM_OK
# define NvrLmExit(code)		exit(code)
# define NvrLmClose()				/* as nothing */
# define NvrLmXtClose()				/* as nothing */
# define NvrLmHeartBeat(feature)		/* as nothing */
# define NvrLicenseError(error, config, die)	/* as nothing */

#else /* LM_DISABLED */

/*
 *      EXTERNS
 */

extern CryptSalt LicenseSalt;

CExternal int NvrGetLicense(
    LmConfigPtr
);

CExternal void NvrLmExit(
    int code
);

CExternal void NvrLmClose(
    void
);

CExternal int NvrXtGetLicense(
    XtAppContext,	/* app_context */
    LmConfigPtr
);

CExternal void NvrLmXtClose(
    void
);

CExternal void NvrLmHeartBeat(
    char*               /* feature */
);

CExternal void NvrLicenseError(
    int                 /* error code */,
    LmConfigPtr,
    boolean		/* whether we die or not */
);

#endif /* LM_DISABLED */

CExternal void NvrCopyrightMsg(
    long bitMask,
    ULong timeBomb
);

CExternal boolean EncodeKey(
    KeyDataPtr,
    char *key
);

CExternal boolean EncodeData(
    UChar *data,
    long dataLength
);

CExternal long DecodeKey(
    char *password,
    KeyDataPtr keyData
);

CExternal boolean DecodeData(
    UChar *data,
    long dataLength
);

CExternal int ValidateKeyData(
    KeyDataPtr keyData,
    long length
);

CExternal UShort CurrentLicenseWeek(
    void
);

CExternal int EnterNewKey(
    KeyDataPtr keyData,
    long *pLength
);

#endif /* _NVRLMFEATURES_H_ */
