/*
 * Copyright (c) 1993  North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research, Inc. has obtained a licensing right, and is
 * considered by North Valley Research, Inc. to be confidential.
 * It is protected by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 * 	NvrMacLm.c - functions to express mac license manager.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/NvrMacLm.c,v 1.2 1994/12/18 00:42:34 toddb Exp $
 */
#define NeedFunctionPrototypes 1

#include "NvrLmFeatures.h"
#include "nvrTimer.h"
#include "nvrString.h"
#include "macDialogs.h"
#include "macResources.h"
#include "macUtils.h"
#include "osio.h"
#include "process.h"
#include "nvrCrypt.h"

#include <OSUtils.h>
#include <Resources.h>
#include <Memory.h>
#include <Errors.h>

typedef struct _macLicenseInfo {
    long keyLength;
    long random;
    KeyData key;
    char path[64];
    char user[32];
} MacLicenseInfo, *MacLicenseInfoPtr;

static UChar *LicenseFile = "\pNVR license keys";

void
NvrCopyrightMsg(long bitMask, ULong timebomb)
{
}

static boolean
GetLicensePath(char path[64])
{
    FSSpec fsSpec;

    if (! GetProcessFSSpec(&fsSpec))
        return(False);
    memcpy((char *)fsSpec.name, (char *)LicenseFile, LicenseFile[0]+1);
    FSSpecToString(path, &fsSpec);
    
    return(True);
}

static void
GetUserName(char user[ 32 ])
{
    Handle hndl;
    
    memset(user, 0, 32);
    hndl = GetResource('STR ', -16096);	/* page 1-127, IM; More Mac Toolbox */
    if (hndl == nil)
        return;
    PascalStringToC(*hndl, user);
}

static int
ValidateMacLicense(MacLicenseInfoPtr lic)
{
    char path[ 64 ];
    char user[ 32 ];
    int code;
    
    code = ValidateKeyData(&lic->key, lic->keyLength);
    if (code)
        return(code);
    
    if (lic->random == 0)
        return(LM_CODE_CORRUPT);
    if (! GetLicensePath(path))
        return(LM_CODE_CORRUPT);
    if (strcmp(path, lic->path) != 0)
        return(LM_CODE_CORRUPT);
    GetUserName(user);
    if (strcmp(user, lic->user) != 0)
        return(LM_CODE_CORRUPT);
    return(LM_OK);
}

static int
UpdateLicenseResource(KeyDataPtr keyData, long keyLength)
{
    Handle hndl;
    MacLicenseInfoPtr lic;
    FeatureName name;
    short resId;
    
    sprintf(name, "%04ld", keyData->feature);
    hndl = Get1NamedResource('NVRl', (StringPtr)CStringToPascal(name, name));
    if (hndl == nil && (hndl = NewHandle(sizeof(MacLicenseInfo)))) {
        resId = UniqueID('NVRl');
        AddResource(hndl, 'NVRl', resId, (StringPtr)name);
    } else if (hndl) {
        SetHandleSize(hndl, sizeof(MacLicenseInfo));
        if (GetHandleSize(hndl) != sizeof(MacLicenseInfo)) {
            DisposHandle(hndl);
            hndl = nil;
        }
    }
    if (hndl == nil)
        return(LM_ERROR);
    HLock(hndl);	/* must be locked, because crypt lib allocates mem */
    lic = (MacLicenseInfoPtr) *hndl;
    lic->keyLength = keyLength;
    lic->key = *keyData;
    GetDateTime((ULong *)&lic->random);
    GetUserName(lic->user);
    if (GetLicensePath(lic->path)
     && EncodeData((UChar *)lic, sizeof(MacLicenseInfo))) {
        ChangedResource(hndl);
        WriteResource(hndl);
        ReleaseResource(hndl);
        return(LM_ERROR);
    }
    return(LM_OK);
}

static int
GetFeatureLicense(FeatureNamePtr feature)
{
    MacLicenseInfoPtr lic;
    Handle hndl;
    FeatureName name;
    int code = LM_OK;

    hndl = Get1NamedResource('NVRl', (StringPtr)CStringToPascal((char *)feature, name));
    if (hndl == nil) {
        if (ResError() == resNotFound)
            return(LM_NOKEY);
        return(LM_ERROR);
    }
    HLock(hndl);	/* must be locked, because crypt lib allocates mem */
    
    if (GetHandleSize(hndl) != sizeof(MacLicenseInfo)) {
        code = LM_ERROR;
        goto complete;
    }
    
    lic = (MacLicenseInfoPtr) *hndl;
    if (! DecodeData((UChar *)lic, sizeof(MacLicenseInfo))) {
        code = LM_ERROR;
        goto complete;
    }
        
    code = ValidateMacLicense(lic);

complete:
    ReleaseResource(hndl);
    return(code);
}

int
NvrGetLicense(LmConfigPtr config)
{
    KeyData keyData;
    long length;
    int code, bestCode;
    short licenseFd;
    FSSpec fsSpec;
    FeatureNamePtr pFeature;
    
    if (!GetProcessFSSpec(&fsSpec))
        return(config->licenseError = LM_ERROR);
    HCreateResFile(fsSpec.vRefNum, fsSpec.parID, LicenseFile);
    licenseFd = HOpenResFile(fsSpec.vRefNum,
                             fsSpec.parID,
                             LicenseFile,
                             fsRdWrPerm);
    if (licenseFd < 0) {
        short error = ResError();
        if (error == fnfErr)
            return(config->licenseError = LM_CANT_WRITE);
        else
            return(config->licenseError = LM_CANT_READ);
    }
    
    for (bestCode = LM_WAIT; bestCode != LM_OK;) {
        bestCode = LM_NOKEY;
        for (pFeature = config->features; **pFeature; pFeature++) {
            switch (code = GetFeatureLicense(pFeature)) {
            case LM_OK:
                bestCode = LM_OK;
                goto processCode;
            case LM_CANT_READ:
            case LM_CANT_WRITE:
            case LM_ERROR:
                if (bestCode == LM_WAIT || LM_CODE_CORRUPT)
                    bestCode = code;
                break;
            case LM_CODE_CORRUPT:
                if (bestCode == LM_WAIT)
                    bestCode = code;
                break;
            case LM_EXPIRED:
                bestCode = code;
                break;
            }
        }
            
    processCode:
        if (bestCode == LM_EXPIRED) {
            CautionDialog("%s. %s.",
                "The key for this software appears to have expired",
                "Please contact North Valley Research for a new key");
            bestCode = LM_NOKEY;
        }
        
        if (bestCode == LM_CODE_CORRUPT || bestCode == LM_NOKEY) {
            bestCode = EnterNewKey(&keyData, &length);
            switch (bestCode) {
            case LM_ERROR:
            case LM_CODE_CORRUPT:
            case LM_WAIT:
                goto complete;
            }
            if (! UpdateLicenseResource(&keyData, length)) {
                bestCode = LM_ERROR;
                goto complete;
            }
            
            /*
             * Check that this code is the one we needed.
             */
            for (pFeature = config->features; **pFeature; pFeature++) {
                if (atoi((char *)pFeature) == keyData.feature)
                    goto complete;
            }
            CautionDialog("%s. %s %s.",
                "That's a valid key, but not for this product",
                "Please enter the key for this product",
                "or contact North Valley Research for assistance");
            bestCode = LM_NOKEY;
        } else
            break;
    }

complete:
    UpdateResFile(licenseFd);
    CloseResFile(licenseFd);
    return(config->licenseError = bestCode);
}

UShort
CurrentLicenseWeek(void)
{
    DateTimeRec licenseBaseDate;
    ULong nowTime;
    ULong licenseBaseTime;
    ULong secsPerWeek = 60*60*24*7;

    licenseBaseDate.year   = 1990;
    licenseBaseDate.month  =    1;
    licenseBaseDate.day    =    1;
    licenseBaseDate.hour   =    0;
    licenseBaseDate.minute =    0;
    licenseBaseDate.second =    0;
    Date2Secs(&licenseBaseDate, &licenseBaseTime);
    GetDateTime(&nowTime);
    
    return( (nowTime - licenseBaseTime) / secsPerWeek );
}

void
NvrLicenseError(int err_code, LmConfigPtr config, boolean die)
{
    int i;
    char msg[ 1024 ], *pMsg;

    pMsg = msg;
    sprintf(pMsg, "For feature(s):");
    pMsg += strlen(pMsg);
    for (i = 0; config->features[i][0]; i++) {
	sprintf(pMsg, "%s %s", i > 0 ? "," : "", config->features[i]);
	pMsg += strlen(pMsg);
    }
    *pMsg++ = '.';
    *pMsg++ = ' ';
    *pMsg = '\0';
    
    switch (err_code) {
    case LM_ERROR:
        sprintf(pMsg, "Internal error.");
        break;
    case LM_CANT_WRITE:
        sprintf(pMsg, "Can't create the license file.");
        break;
    case LM_CANT_READ:
        sprintf(pMsg, "%s. %s.",
            "The license file is present but unreadable",
            "Perhaps ResEdit has it open");
        break;
    case LM_CODE_CORRUPT:
        /* The feature's license is invalid. */
        sprintf(pMsg, "Your license key appears to be corrupt.");
        break;
    case LM_NOKEY:
    case LM_AUTH_FAIL:
        /* The feature's license is invalid. */
        sprintf(pMsg, "Sorry, invalid license.");
        break;
    case LM_WAIT:
        break;
    case LM_EXPIRED:
        /* License for requested feature has expired. */
        sprintf(pMsg, "Sorry, license has expired.");
        break;
    default:
	sprintf(pMsg, "Unknown license failure code=%d", err_code);
	break;
    }
    
    if (err_code != LM_WAIT)
        CautionDialog("%s", msg);
    if (die)
        exit(0);
}

void
NvrLmExit(int code)
{
    exit(code);
}
