/*
 * Copyright (c) 1993  North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research, Inc. has obtained a licensing right, and is
 * considered by North Valley Research, Inc. to be confidential.
 * It is protected by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 * 	macLmEntry.c - functions to support license entry
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/macLmEntry.c,v 1.2 1994/12/18 00:42:52 toddb Exp $
 */
#define NeedFunctionPrototypes 1

#include "NvrLmFeatures.h"
#include "macEvent.h"
#include "macDialogs.h"
#include "macUtils.h"
#include "macMenus.h"
#include "macUITools.h"
#include "macUIEvents.h"
#include "nvrCrypt.h"
#include "nvrString.h"

#include <OSUtils.h>
#include <Resources.h>
#include <Dialogs.h>
#include <Fonts.h>

typedef enum {
    kLicenseOK = 1,
    kLicenseQuit,
    kLicenseSn,
    kLicenseSnLabel,
    kLicenseDesc1,
    kLicenseDesc2,
    kLicenseDesc3
} DialogItems;

static void ButtonCB(WindowPtr, WinItemInfoPtr itemPtr, long value);

static RGBColor kBlack     = { 0x0000, 0x0000, 0x0000};
static char *LicenceEntryDlogName = "License Entry";
static ButtonParam DfltButtonClick = { ButtonCB, True };
static ButtonParam ButtonClick = { ButtonCB };
static TextParam LeftFont = {12, srcOr, systemFont, normal, kLeftJustified };
static TextParam CenterFont = {12, srcOr, systemFont, normal, kCentered };

static UserItemMap LicenseItemMap[] = {
    { kLicenseOK,            WTButtonItem,  (WIParamPtr)&DfltButtonClick },
    { kLicenseQuit,          WTButtonItem,  (WIParamPtr)&ButtonClick },

    { kLicenseDesc1,         WTTextItem,    (WIParamPtr)&CenterFont, &kBlack },
    { kLicenseDesc2,         WTTextItem,    (WIParamPtr)&CenterFont, &kBlack },
    { kLicenseDesc3,         WTTextItem,    (WIParamPtr)&CenterFont, &kBlack },
    { kLicenseSnLabel,       WTTextItem,    (WIParamPtr)&LeftFont, &kBlack },
};

static struct {
    boolean complete;
    boolean success;
} LicenseEntryState;

static void
ButtonCB(WindowPtr wp, WinItemInfoPtr itemPtr, long value)
{
    switch(itemPtr->itemNumber) {
    case kLicenseOK:
        LicenseEntryState.complete = True;
        LicenseEntryState.success = True;
        break;

    case kLicenseQuit:
        LicenseEntryState.complete = True;
        break;
    }
}

int
EnterNewKey(KeyDataPtr keyData, long *pLength)
{
    short dlogId;
    int code;
    WindowPtr wp;
    MenuSystemPtr msys;
    WinMgrInfoPtr winMgrInfo;
    NamedMenuItemParam DlogEditParam;
    char password[ 256 ];
    WinItemInfoPtr itemPtr;
        
    if ((msys = GetRegisteredMenuSystem()) == nil) {
        CautionDialog("No registered menu system.");
        return(LM_ERROR);
    }

    if ((dlogId = DialogNameToId(LicenceEntryDlogName)) == 0)
        return(LM_ERROR);
    
    wp = GetNewDialog(dlogId, nil, (WindowPtr)-1);
    if (wp == nil)
        return(LM_ERROR);

    winMgrInfo = WinMgrInfoFromTemplate(wp,
                                        dlogId,
                                        LicenseItemMap,
                                        NumberOfCells(LicenseItemMap));
    RegisterMacEventHandler(1L << UpdateEvent, wp,
                            StdUpdateEventCb, (Ptr)winMgrInfo);

    RegisterMacEventHandler(1L << WindowContentEvent, wp,
                            StdWindowContentEventCb, (Ptr)winMgrInfo);

    RegisterMacEventHandler(1L << KeyDownEvent, wp,
                            StdKeydownEventCb, (Ptr)winMgrInfo);
    
    PushDialog(wp);

    DlogEditParam = StdDlogEditMenuParams;
    DlogEditParam.callbackData = (Ptr)msys;
    DlogEditParam.wp = wp;
    PushNamedMenuState(msys, &DlogEditParam, 1);
    
    code = LM_WAIT;
    for (;;) {
        LicenseEntryState.complete = False;
        LicenseEntryState.success = False;
        while (! LicenseEntryState.complete) {
            RecursiveProcessEvents();
        }
        if (! LicenseEntryState.success) {
            code = LM_WAIT;
            break;
        }

        itemPtr = FindWinItem(winMgrInfo->itemList, kLicenseSn);
        GetIText(itemPtr->u.textEdit->itemHndl, (StringPtr)password);
        PascalStringToC((char *)password, (char *)password);
        if ((*pLength = DecodeKey(password, keyData)) == 0
         || (code = ValidateKeyData(keyData, *pLength)) == LM_CODE_CORRUPT) {
            CautionDialog("That key appears to be incorrect.  %s",
                "Please check the serial number carefully and try again.");
        } else
            break;
    }
    
    (void) PopDialog();
    FreeWinMgrInfo(winMgrInfo);
    DisposDialog(wp);
    RemoveMacEventHandlers(wp);
    PopMenuState(msys);
    
    return(code);
}

