/*
 * Copyright (c) 1993  North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research, Inc. has obtained a licensing right, and is
 * considered by North Valley Research, Inc. to be confidential.
 * It is protected by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 * 	nodeLicense.c - functions to decode node-locked licenses
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nodeLicense.c,v 1.2 1994/12/18 00:43:13 toddb Exp $
 */
#define NeedFunctionPrototypes 1

#include "NvrLmFeatures.h"
#include "nvrCrypt.h"
#include "nvrString.h"

long
DecodeKey(char *password, KeyDataPtr keyData)
{
    CryptKey key;
    ULong nibble;
    long shift;
    long keyDataSize;
    KeyData encrypted;
    char *pFrom, *pTo;

    pFrom = password;
    pTo = (char *)&encrypted;
    shift = 4;
    *pTo = '\0';
    for (; *pFrom; pFrom++) {
        if (pTo - (char *)&encrypted > sizeof(encrypted))
            return(0);	/* too big */
            
        switch (*pFrom) {
        case '0': case '1': case '2': case '3':
        case '4': case '5': case '6': case '7':
        case '8': case '9':
            nibble = *pFrom - '0';
            break;
        case 'a': case 'b': case 'c': case 'd':  case 'e': case 'f':
            nibble = *pFrom - 'a' + 10;
            break;
        case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
            nibble = *pFrom - 'A' + 10;
            break;
        case ' ':
            continue;
        default:
            return(0);
        }
        *pTo |= nibble << shift;
        shift -= 4;
        if (shift < 0) {
            shift = 4;
            *++pTo = '\0';
        }
    }

    if (shift != 4)
    	return(0);	/* missing nibble */

    keyDataSize = pTo - (char *)&encrypted;
    if (! CryptInit(0))
    	return(0);
    SetCryptKey(key, LicenseSalt);
    DecryptData(key, (char *)&encrypted, (char *)keyData, keyDataSize);
    CryptComplete();

    return(keyDataSize);
}

boolean
DecodeData(UChar *data, long dataLength)
{
    CryptKey key;

    if (! CryptInit(0))
    	return(False);
    SetCryptKey(key, LicenseSalt);
    DecryptData(key, (char *)data, (char *)data, dataLength);
    CryptComplete();
    return(True);
}

boolean
EncodeKey(KeyDataPtr keyDataIn, char *password)
{
    CryptKey key;
    long i;
    KeyData keyDataOut;
    UShort *pkeyData;
    long keyDataSize;

    keyDataSize = sizeof(KeyData);
    if (! keyDataIn->netValid)
        keyDataSize -= sizeof(ULong);
    if (! keyDataIn->hostValid)
        keyDataSize -= sizeof(ULong);

    if (! CryptInit(0))
    	return(False);
    SetCryptKey(key, LicenseSalt);
    if (! EncryptData(key, (char *)keyDataIn, (char *)&keyDataOut, keyDataSize))
        return(False);
    CryptComplete();
    
    pkeyData = (UShort *)&keyDataOut;
    for (i = 0; i < keyDataSize; i += sizeof(UShort), pkeyData++) {
        sprintf(password, "%04x ", *pkeyData);
        password += 5;
    }
    password[-1] = '\0';
    
    return(True);
}

boolean
EncodeData(UChar *data, long dataLength)
{
    CryptKey key;

    if (! CryptInit(0))
    	return(False);
    SetCryptKey(key, LicenseSalt);
    if (! EncryptData(key, (char *)data, (char *)data, dataLength))
    	return(False);
    CryptComplete();
    return(True);
}

int
ValidateKeyData(KeyDataPtr keyData, long length)
{
    long keyDataSize;

    keyDataSize = sizeof(KeyData);
    if (! keyData->netValid)
        keyDataSize -= sizeof(ULong);
    if (! keyData->hostValid)
        keyDataSize -= sizeof(ULong);
    if (length != keyDataSize)
        return(LM_CODE_CORRUPT);
    if (keyData->random == 0)
        return(LM_CODE_CORRUPT);
    if (keyData->feature == 0)
        return(LM_CODE_CORRUPT);
    if (keyData->netValid && keyData->netAddress == 0)
        return(LM_CODE_CORRUPT);
    if (keyData->hostValid && keyData->hostId == 0)
        return(LM_CODE_CORRUPT);
    if (keyData->demoKey && CurrentLicenseWeek() > keyData->week)
        return(LM_EXPIRED);
    return(LM_OK);
}
