/*
 * Copyright (c) 1992, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	MAcommon.c -- MPEG Audio Encoder/Decoder common code
 *
 * DESCRIPTION
 *	This module contains function common to the MPEG
 *	audio encoder and decoder.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/MAcommon.c,v 1.9 1995/08/09 00:32:53 toddb Exp $
 */

#include        "MAcommon.h"
#include        <math.h>

#ifdef DEBUG_ENABLED

static char *DebugNames[] = {
    "syncword",
    "ID",
    "layer",
    "protection_bit",
    "bitrate_index",
    "sampling_frequency",
    "padding_bit",
    "private_bit",
    "mode",
    "mode_extension",
    "copyright",
    "original_home",
    "emphasis",
    "crc_check",
    "allocation",
    "scale_factor",
    "sample",
    "pad",

    "sample data",
    "allocation data",
    "scale data",
    "timer",
    "pad data",
    "granules",
    "reset sb samples",
    "audio sequence info",

    "audio frame",
};

static UShort BitsBits[] = {
    N_syncword,
    N_ID,
    N_layer,
    N_protection_bit,
    N_bitrate_index,
    N_sampling_frequency,
    N_padding_bit,
    N_private_bit,
    N_mode,
    N_mode_extension,
    N_copyright,
    N_original_home,
    N_emphasis,
    N_crc_check,
    N_allocation,
    N_scale_factor,
    N_sample,
    N_pad,
};

static UShort MarkBits[] = {
    N_SampleData,
    N_AllocationData,
    N_ScaleData,
    N_Timer,
    N_Granules,
    N_ResetSbSamples,
    N_AuSequenceInfo,
    N_AuFrameMark
};

FlagNames AmpegdecompDebugNames = {
    DBI_Ampegdecomp,
    DebugNames,
    NumberOfCells(DebugNames)
};

static FlagMacro DebugMacros[] = {
    { "bits", BitsBits, NumberOfCells(BitsBits) },
    { "marks", MarkBits, NumberOfCells(MarkBits) },
};

FlagMacroList AmpegdecompDebugMacros = {
    DebugMacros,
    NumberOfCells(DebugMacros)
};

#endif /* DEBUG_ENABLED */

/*
 *      GLOBALS
 *              Variables declared in this package that are allowed
 *		to be used globally.
 */

short     MA_BitRateTable[4][15] = {
/* Layer Number
   |     bit_rate_index -->
   | 	 0   1   2   3    4    5    6    7    8    9   10   11   12   13   14 
   V     =  ==  ==  ==  ===  ===  ===  ===  ===  ===  ===  ===  ===  ===  === */
/* r */	{0,  0,  0,  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0}, 
/* 3 */	{0, 32, 40, 48,  56,  64,  80,  96, 112, 128, 160, 192, 224, 256, 320},
/* 2 */	{0, 32, 48, 56,  64,  80,  96, 112, 128, 160, 192, 224, 256, 320, 384}, 
/* 1 */	{0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448}, 
};

int  MA_SamplingFrequencyTable[4] = {
	44100,	/* 00 */
	48000,	/* 01 */
	32000,	/* 10 */
	0000	/* 11 = reserved */
};

int  MA_Bound[4] = {
	4,	/* MODE_EXT_0 */
	8,	/* MODE_EXT_1 */
	12,	/* MODE_EXT_2 */
	16	/* MODE_EXT_3 */
};

char *AbbrevAudioModeNames[] = {
    "S ",
    "JS",
    "DC",
    "SC"
};

char *JointStereoBandNames[] = {
    " 4-31",
    " 8-31",
    "12-31",
    "16-31"
};

char *EmphasisNames[] = {
    "no emphasis",
    "50/15 ms emphasis",
    "reserved",
    "CCITT J.17",
};

#include "I_II_ScaleFactor.h"
