/*
 *	(c) Copyright 1992 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		MAcommon.h - Includes common to the MPEG audio encoder & decoder
 *
 *	DESCRIPTION
 *		
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/MAcommon.h,v 1.23 1995/08/16 20:14:48 russ Exp $
 */
#ifndef _MPEG_AUDIO_COMMON_H_
#define _MPEG_AUDIO_COMMON_H_

#define	NeedFunctionPrototypes	1

#include "nvrAlloc.h"
#include "osio.h"
#include "bitstream.h"
#include "longjump.h"
#include "MAdebug.h"

/*
 *	DEFINES
 */


/***********************************************************************
*
*  Global Definitions
*
***********************************************************************/

/*
 * General Definitions
 */
#if 0
typedef double		FloatType;
#else
typedef float		FloatType;
#endif

#ifndef FALSE
#define	FALSE		0
#define	TRUE		1
#endif

#define	NULL_CHAR	'\0'
#define	MAX_U_32_NUM	0xFFFFFFFF
#define	LN_TO_LOG10	0.2302585093
#define	VOL_REF_NUM	0
#define	MPEG_AUDIO_ID	1
#define	MAC_WINDOW_SIZE	24
#define	MONO		1
#define	STEREO		2
#define	BITS_IN_A_BYTE	8
#define	WORD		16
#define	MAX_NAME_SIZE	81
#define	SBLIMIT		32
#define	FFT_SIZE	1024
#define	HAN_SIZE	512
#define	SCALE_BLOCK	12
#define	SCALE_RANGE	64
#define	SCALE		32768
#define	CRC16_POLYNOMIAL	0x8005


/*
 * Define for MPEG Audio Frame Header
 */

/* MPEG Audio MODE values */
#define	MODE_STEREO		0
#define	MODE_JOINT_STEREO	1	/* intensity_stereo  and/or ms_stereo */
#define	MODE_DUAL_CHANNEL	2
#define	MODE_SINGLE_CHANNEL	3

/* MPEG Audio MODE Extention values for Intensity Stereo */
#define	MODE_EXT_0		0	/* subband  4-31, bound =  4 */
#define	MODE_EXT_1		1	/* subband  8-31, bound =  8 */
#define	MODE_EXT_2		2	/* subband 12-31, bound = 12 */
#define	MODE_EXT_3		3	/* subband 16-31, bound = 16 */

/* MPEG Audio Emphasis */
#define	EMPHASIS_NONE		0
#define	EMPHASIS_50DIV15_MSEC	1
#define	EMPHASIS_RESERVED	2
#define	EMPHASIS_CCITT_J17	3

/* MPEG Samping Rate Indices */
#define	INDEX_44100HZ		0
#define	INDEX_48000HZ		1
#define	INDEX_32000HZ		2
#define	INDEX_RESERVED		3

/* Misc Macros */
#define LAYER_NUMBER(encoding)		(4 - (encoding))
#define LAYER_I				3
#define LAYER_II			2
#define LAYER_III			1
#define LAYER_RESERVED			0
#define P_INTENSITY_STEREO(encoding)	((encoding) & 0x2)
#define P_MS_STEREO(encoding)		((encoding) & 0x1)
#define P_NO_REDUNDANCY(hdr)		((hdr).protection_bit)
#define P_REDUNDANCY(hdr)		(!((hdr).protection_bit))
#define UPDATE_CRC(mask, nbits,data,crc) \
    {								\
	register UShort _data = data;				\
        mask = 1 << nbits;					\
	while(mask >>= 1) {					\
	    crc = (!((crc) & 0x8000) ^ !(_data & mask))		\
		?						\
		((crc) << 1) ^ 0x8005				\
		:						\
		(crc) << 1;					\
        }							\
    }

#define   VAL_SYNCWORD		0xfffL
#define   BITS_HEADER		32

#define FIFO_SIZE		2304	/* 1152 * 2 */

#define LAZY_AUDIO_LEVEL	4	/* level below which we do lazy decoding */

/*
 * The fifo will contain all the samples for a frame:
 *		 384 for Layer I
 *		1152 for Layer II
 *		1152 for Layer III
 */
#define NMonoSamplesPerFrame(audioInfo)					\
    ((audioInfo)->quarterSample? 8 : 32)

#define NStereoSamplesPerFrame(audioInfo)				\
    (((audioInfo)->quarterSample ? 8 : 32)				\
   * ((audioInfo)->monoSample ? 1 : 2))

/*
 *	TYPEDEFS
 */
typedef struct _FrameInfo {
    /*
     * Audio Frame Decode Context
     */
    short layer;		/* 3 = Layer1, 2 = 2, 1 = 3, 0 = reserved */
    short sampling_frequency;	/* index into MA_SamplingFrequencyTable */
    short mode;			/* MODE_STEREO, MODE_JOINT_STEREO, */
				/*   MODE_DUAL_CHANNEL, MODE_SINGLE_CHANNEL */
    short mode_extension;	/* Layer I & II: for Joint Stereo mode */
				/*	see MODE_EXT_# and MA_Bound[]  */
				/* Layer III: encoding for intensity_stereo */
				/*   (bit 1) and ms_stereo (bit 0). 	*/
    unsigned char *nbal_table;	/* Layer II */
    char *alloc_table;		/* Layer II */
    int	sblimit;		/* Layer II */
} FrameInfo, *FrameInfoPtr;

typedef union _FrameHeader {
    struct {
	ULong syncWord         :12;	/* 0xfff */

	/*
	 * Audio Frame Header Information Structure
	 */
	ULong ID                :1;	/* 1 = MPEG, 0 = reserved */
	ULong layer             :2;	/* 3,2,1 for layer 1,2,3; 0=reserved */
	ULong protection_bit    :1;	/* 0 ==> includes redundancy */
	ULong bitrate_index     :4;	/* MA_BitRateTable index */
	ULong sampling_frequency:2;	/* MA_SamplingFrequencyTable index */
	ULong padding_bit	:1;	/* 1==> has added slot to adjust mean */
				    	/*  bitrate to sampling frequency  */
					/* only necessary for 44.1 kHz */
	ULong private_bit	:1;	/* for private use */
	ULong mode		:2;	/* MODE_{STEREO,JOINT_STEREO, */
					/* DUAL_CHANNEL,SINGLE_CHANNEL} */
	ULong mode_extension    :2;	/* Layer I & II: Joint Stereo mode */
					/* see MODE_EXT_# and MA_Bound[]  */
					/* Layer III: encoding for */
					/* intensity_stereo */
					/* (bit 1) and ms_stereo (bit 0). */
	ULong copyright		:1;	/* 0 = no copyright, 1 = copyrighted */
	ULong original_home	:1;	/* 0 = is a copy, 1 = original */
	ULong emphasis		:2;	/* See EMPHASIS_* */
    } s;
    ULong composite;
} FrameHeader, *FrameHeaderPtr;

enum {
    FHsyncWord,
    FHId,
    FHlayer,
    FHprotectionBit,
    FHbitrateIndex,
    FHsamplingFrequency,
    FHpaddingBit,
    FHprivateBit,
    FHmode,
    FHmodeExtension,
    FHcopyright,
    FHoriginalHome,
    FHemphasis
};

/*
 * This is the central context structure, carrying most of the
 * decoding information.
 */
typedef struct _mpegAudioInfo MpegAudioInfo, *MpegAudioInfoPtr;
struct _mpegAudioInfo {
    BitStreamPtr bitStream;
    boolean	crcFrameError;	/* true==> current frame has an error */
    UShort	crcCheck;	/* crc check value */
    ULong	nCrcErrors;	/* number of errors */

    StackInfo	decodeContext;	/* where to longjump for terrible errors */
    PrivPointer	priv;	

    boolean  	quarterSample;	/* decode to 1/4 of the original KHz */
    boolean  	monoSample;  	/* convert stereo to mono */

    CacheReturnValue (*decodeProc)(CacheProtoArgs, MpegAudioInfoPtr);

    ULong	frame;		/* (audio) frame number */
    FrameInfo	frameInfo;	/* frame decoding context */
    FrameHeader frameHeader;	/* current frame header */
    FrameHeader lastHeader;	/* previous frame header */
    short       frameSize;	/* calculated from the header */

    union {
	FloatType	monoSbSample[2048];
	FloatType	stereoSbSample[2][1024];
    } u;

    FloatType I_RequantizationTable[ ((1<<16)-1) ];

    long	nFifoEntries;	/* # of samples to deliver */

    /*
     * fifo[] must be last element. fifo[] may be larger than FIFO_SIZE
     *	UShorts by mallocing extra space at the end. fifoSize must be set.
     */
    long	fifoSize;
    UShort	fifo[ FIFO_SIZE ]; /* decoded data */

}; /* typedef: MpegAudioInfo, *MpegAudioInfoPtr */

typedef struct _II_AllocRec {
    unsigned short	n_steps;
    unsigned short	n_bits_per_sample;
    unsigned short	n_bits_per_codeword;
    unsigned short	n_samples_per_codeword;
    FloatType		C;
    FloatType		D;
    FloatType		mult_coeff;
} II_AllocRec, *p_II_AllocRec;

typedef unsigned int ErrorCheck;

extern int	MA_Bound[];
extern FloatType	I_II_ScaleFactor[];
extern UChar	 SignedPCMtoULawTbl[];
extern int	MA_Bound[];
extern short	MA_BitRateTable[4][15];
extern UChar	II_Table0_nbal[];
extern UChar	II_Table1_nbal[];
extern UChar	II_Table2_nbal[];
extern UChar	II_Table3_nbal[];
extern char	II_AllocTable0[];
extern char	II_AllocTable1[];
extern char	II_AllocTable2[];
extern char	II_AllocTable3[];
extern II_AllocRec II_AllocRecords[];
extern int	MA_SamplingFrequencyTable[];

CExternal CacheReturnValue DecodeLayerI_SingleChannel(
    CacheProtoArgs,
    MpegAudioInfoPtr
);

CExternal CacheReturnValue DecodeLayerII_SingleChannel(
    CacheProtoArgs,
    MpegAudioInfoPtr
);

CExternal CacheReturnValue DecodeLayerI_Stereo(
    CacheProtoArgs,
    MpegAudioInfoPtr
);

CExternal CacheReturnValue DecodeLayerI_IntensityStereo(
    CacheProtoArgs,
    MpegAudioInfoPtr
);

CExternal CacheReturnValue DecodeLayerII_Stereo(
    CacheProtoArgs,
    MpegAudioInfoPtr
);

CExternal CacheReturnValue DecodeLayerII_IntensityStereo(
    CacheProtoArgs,
    MpegAudioInfoPtr
);


CExternal void InitLayerI_Tables(MpegAudioInfoPtr);
CExternal boolean MAgatherSamples(MpegAudioInfoPtr, long);
CExternal boolean NewMaSetupDecode(MpegAudioInfoPtr, char *, BitStreamPtr);
CExternal void crcError(MpegAudioInfoPtr, boolean stereo);

/***************************************************************************
 *
 * Berlin University of Technology routines
 *
 ***************************************************************************/
CExternal void BUTInitSynthesisFilter(
    ULong ch,
    FloatType factor,
    int quarterSample
);


CExternal void BUTMonoSubbandFilter(
    FloatType requantized_sample[32],
    UShort *output_sample
);


CExternal void BUTStereoSubbandFilter(
    FloatType requantized_sample[2][32],
    UShort *output_sample
);


CExternal void BUTStereoToMonoSubbandFilter(
    FloatType requantized_sample[2][32],
    UShort *output_sample
);




extern char *AbbrevAudioModeNames[];
extern char *JointStereoBandNames[];
extern char *EmphasisNames[];

#endif /* _MPEG_AUDIO_COMMON_H_ */
