/*
 *	Copyright (c) 1992, North Valley Research, Inc.
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		MAlayerII.c -- MPEG Audio Decoder
 *
 *	DESCRIPTION
 *		This module contains the core of the decoder ie all the
 *		computational routines. (Layer I and II only)
 *		Functions are common to both layer unless
 *		otherwise specified.
 *
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/MAlayerII.c,v 1.25 1995/08/16 20:14:48 russ Exp $
 *
 */

/*
 *      INTERNAL INCLUDES
 *              Include files for local use only, therefore, NOT exported
 *		to any package or program using this package.
 */
#define NeedFunctionPrototypes 1

#include        "MAcommon.h"

/*
 * this table contains 3 requantized samples for each legal codeword
 * when grouped in 5 bits, i.e. 3 quantizationsteps per sample
 */
static FloatType grouping_5bits[27 * 3] =
{
  -2.0/3.0, -2.0/3.0, -2.0/3.0,
       0.0, -2.0/3.0, -2.0/3.0,
   2.0/3.0, -2.0/3.0, -2.0/3.0,
  -2.0/3.0,      0.0, -2.0/3.0,
       0.0,      0.0, -2.0/3.0,
   2.0/3.0,      0.0, -2.0/3.0,
  -2.0/3.0,  2.0/3.0, -2.0/3.0,
       0.0,  2.0/3.0, -2.0/3.0,
   2.0/3.0,  2.0/3.0, -2.0/3.0,
  -2.0/3.0, -2.0/3.0,      0.0,
       0.0, -2.0/3.0,      0.0,
   2.0/3.0, -2.0/3.0,      0.0,
  -2.0/3.0,      0.0,      0.0,
       0.0,      0.0,      0.0,
   2.0/3.0,      0.0,      0.0,
  -2.0/3.0,  2.0/3.0,      0.0,
       0.0,  2.0/3.0,      0.0,
   2.0/3.0,  2.0/3.0,      0.0,
  -2.0/3.0, -2.0/3.0,  2.0/3.0,
       0.0, -2.0/3.0,  2.0/3.0,
   2.0/3.0, -2.0/3.0,  2.0/3.0,
  -2.0/3.0,      0.0,  2.0/3.0,
       0.0,      0.0,  2.0/3.0,
   2.0/3.0,      0.0,  2.0/3.0,
  -2.0/3.0,  2.0/3.0,  2.0/3.0,
       0.0,  2.0/3.0,  2.0/3.0,
   2.0/3.0,  2.0/3.0,  2.0/3.0
};

/*
 * this table contains 3 requantized samples for each legal codeword
 * when grouped in 7 bits, i.e. 5 quantizationsteps per sample
 */
static FloatType grouping_7bits[125 * 3] =
{
  -0.8, -0.8, -0.8,   -0.4, -0.8, -0.8,    0.0, -0.8, -0.8,    0.4, -0.8, -0.8,    0.8, -0.8, -0.8,
  -0.8, -0.4, -0.8,   -0.4, -0.4, -0.8,    0.0, -0.4, -0.8,    0.4, -0.4, -0.8,    0.8, -0.4, -0.8,
  -0.8,  0.0, -0.8,   -0.4,  0.0, -0.8,    0.0,  0.0, -0.8,    0.4,  0.0, -0.8,    0.8,  0.0, -0.8,
  -0.8,  0.4, -0.8,   -0.4,  0.4, -0.8,    0.0,  0.4, -0.8,    0.4,  0.4, -0.8,    0.8,  0.4, -0.8,
  -0.8,  0.8, -0.8,   -0.4,  0.8, -0.8,    0.0,  0.8, -0.8,    0.4,  0.8, -0.8,    0.8,  0.8, -0.8,
  -0.8, -0.8, -0.4,   -0.4, -0.8, -0.4,    0.0, -0.8, -0.4,    0.4, -0.8, -0.4,    0.8, -0.8, -0.4,
  -0.8, -0.4, -0.4,   -0.4, -0.4, -0.4,    0.0, -0.4, -0.4,    0.4, -0.4, -0.4,    0.8, -0.4, -0.4,
  -0.8,  0.0, -0.4,   -0.4,  0.0, -0.4,    0.0,  0.0, -0.4,    0.4,  0.0, -0.4,    0.8,  0.0, -0.4,
  -0.8,  0.4, -0.4,   -0.4,  0.4, -0.4,    0.0,  0.4, -0.4,    0.4,  0.4, -0.4,    0.8,  0.4, -0.4,
  -0.8,  0.8, -0.4,   -0.4,  0.8, -0.4,    0.0,  0.8, -0.4,    0.4,  0.8, -0.4,    0.8,  0.8, -0.4,
  -0.8, -0.8,  0.0,   -0.4, -0.8,  0.0,    0.0, -0.8,  0.0,    0.4, -0.8,  0.0,    0.8, -0.8,  0.0,
  -0.8, -0.4,  0.0,   -0.4, -0.4,  0.0,    0.0, -0.4,  0.0,    0.4, -0.4,  0.0,    0.8, -0.4,  0.0,
  -0.8,  0.0,  0.0,   -0.4,  0.0,  0.0,    0.0,  0.0,  0.0,    0.4,  0.0,  0.0,    0.8,  0.0,  0.0,
  -0.8,  0.4,  0.0,   -0.4,  0.4,  0.0,    0.0,  0.4,  0.0,    0.4,  0.4,  0.0,    0.8,  0.4,  0.0,
  -0.8,  0.8,  0.0,   -0.4,  0.8,  0.0,    0.0,  0.8,  0.0,    0.4,  0.8,  0.0,    0.8,  0.8,  0.0,
  -0.8, -0.8,  0.4,   -0.4, -0.8,  0.4,    0.0, -0.8,  0.4,    0.4, -0.8,  0.4,    0.8, -0.8,  0.4,
  -0.8, -0.4,  0.4,   -0.4, -0.4,  0.4,    0.0, -0.4,  0.4,    0.4, -0.4,  0.4,    0.8, -0.4,  0.4,
  -0.8,  0.0,  0.4,   -0.4,  0.0,  0.4,    0.0,  0.0,  0.4,    0.4,  0.0,  0.4,    0.8,  0.0,  0.4,
  -0.8,  0.4,  0.4,   -0.4,  0.4,  0.4,    0.0,  0.4,  0.4,    0.4,  0.4,  0.4,    0.8,  0.4,  0.4,
  -0.8,  0.8,  0.4,   -0.4,  0.8,  0.4,    0.0,  0.8,  0.4,    0.4,  0.8,  0.4,    0.8,  0.8,  0.4,
  -0.8, -0.8,  0.8,   -0.4, -0.8,  0.8,    0.0, -0.8,  0.8,    0.4, -0.8,  0.8,    0.8, -0.8,  0.8,
  -0.8, -0.4,  0.8,   -0.4, -0.4,  0.8,    0.0, -0.4,  0.8,    0.4, -0.4,  0.8,    0.8, -0.4,  0.8,
  -0.8,  0.0,  0.8,   -0.4,  0.0,  0.8,    0.0,  0.0,  0.8,    0.4,  0.0,  0.8,    0.8,  0.0,  0.8,
  -0.8,  0.4,  0.8,   -0.4,  0.4,  0.8,    0.0,  0.4,  0.8,    0.4,  0.4,  0.8,    0.8,  0.4,  0.8,
  -0.8,  0.8,  0.8,   -0.4,  0.8,  0.8,    0.0,  0.8,  0.8,    0.4,  0.8,  0.8,    0.8,  0.8,  0.8
};

/*
 *  this table contains 3 requantized samples for each legal codeword
 * when grouped in 10 bits, i.e. 9 quantizationsteps per sample
 */
static FloatType grouping_10bits[729 * 3] =
{
  -8.0/9.0, -8.0/9.0, -8.0/9.0,   -6.0/9.0, -8.0/9.0, -8.0/9.0,   -4.0/9.0, -8.0/9.0, -8.0/9.0,
  -2.0/9.0, -8.0/9.0, -8.0/9.0,        0.0, -8.0/9.0, -8.0/9.0,    2.0/9.0, -8.0/9.0, -8.0/9.0,
   4.0/9.0, -8.0/9.0, -8.0/9.0,    6.0/9.0, -8.0/9.0, -8.0/9.0,    8.0/9.0, -8.0/9.0, -8.0/9.0,
  -8.0/9.0, -6.0/9.0, -8.0/9.0,   -6.0/9.0, -6.0/9.0, -8.0/9.0,   -4.0/9.0, -6.0/9.0, -8.0/9.0,
  -2.0/9.0, -6.0/9.0, -8.0/9.0,        0.0, -6.0/9.0, -8.0/9.0,    2.0/9.0, -6.0/9.0, -8.0/9.0,
   4.0/9.0, -6.0/9.0, -8.0/9.0,    6.0/9.0, -6.0/9.0, -8.0/9.0,    8.0/9.0, -6.0/9.0, -8.0/9.0,
  -8.0/9.0, -4.0/9.0, -8.0/9.0,   -6.0/9.0, -4.0/9.0, -8.0/9.0,   -4.0/9.0, -4.0/9.0, -8.0/9.0,
  -2.0/9.0, -4.0/9.0, -8.0/9.0,        0.0, -4.0/9.0, -8.0/9.0,    2.0/9.0, -4.0/9.0, -8.0/9.0,
   4.0/9.0, -4.0/9.0, -8.0/9.0,    6.0/9.0, -4.0/9.0, -8.0/9.0,    8.0/9.0, -4.0/9.0, -8.0/9.0,
  -8.0/9.0, -2.0/9.0, -8.0/9.0,   -6.0/9.0, -2.0/9.0, -8.0/9.0,   -4.0/9.0, -2.0/9.0, -8.0/9.0,
  -2.0/9.0, -2.0/9.0, -8.0/9.0,        0.0, -2.0/9.0, -8.0/9.0,    2.0/9.0, -2.0/9.0, -8.0/9.0,
   4.0/9.0, -2.0/9.0, -8.0/9.0,    6.0/9.0, -2.0/9.0, -8.0/9.0,    8.0/9.0, -2.0/9.0, -8.0/9.0,
  -8.0/9.0,      0.0, -8.0/9.0,   -6.0/9.0,      0.0, -8.0/9.0,   -4.0/9.0,      0.0, -8.0/9.0,
  -2.0/9.0,      0.0, -8.0/9.0,        0.0,      0.0, -8.0/9.0,    2.0/9.0,      0.0, -8.0/9.0,
   4.0/9.0,      0.0, -8.0/9.0,    6.0/9.0,      0.0, -8.0/9.0,    8.0/9.0,      0.0, -8.0/9.0,
  -8.0/9.0,  2.0/9.0, -8.0/9.0,   -6.0/9.0,  2.0/9.0, -8.0/9.0,   -4.0/9.0,  2.0/9.0, -8.0/9.0,
  -2.0/9.0,  2.0/9.0, -8.0/9.0,        0.0,  2.0/9.0, -8.0/9.0,    2.0/9.0,  2.0/9.0, -8.0/9.0,
   4.0/9.0,  2.0/9.0, -8.0/9.0,    6.0/9.0,  2.0/9.0, -8.0/9.0,    8.0/9.0,  2.0/9.0, -8.0/9.0,
  -8.0/9.0,  4.0/9.0, -8.0/9.0,   -6.0/9.0,  4.0/9.0, -8.0/9.0,   -4.0/9.0,  4.0/9.0, -8.0/9.0,
  -2.0/9.0,  4.0/9.0, -8.0/9.0,        0.0,  4.0/9.0, -8.0/9.0,    2.0/9.0,  4.0/9.0, -8.0/9.0,
   4.0/9.0,  4.0/9.0, -8.0/9.0,    6.0/9.0,  4.0/9.0, -8.0/9.0,    8.0/9.0,  4.0/9.0, -8.0/9.0,
  -8.0/9.0,  6.0/9.0, -8.0/9.0,   -6.0/9.0,  6.0/9.0, -8.0/9.0,   -4.0/9.0,  6.0/9.0, -8.0/9.0,
  -2.0/9.0,  6.0/9.0, -8.0/9.0,        0.0,  6.0/9.0, -8.0/9.0,    2.0/9.0,  6.0/9.0, -8.0/9.0,
   4.0/9.0,  6.0/9.0, -8.0/9.0,    6.0/9.0,  6.0/9.0, -8.0/9.0,    8.0/9.0,  6.0/9.0, -8.0/9.0,
  -8.0/9.0,  8.0/9.0, -8.0/9.0,   -6.0/9.0,  8.0/9.0, -8.0/9.0,   -4.0/9.0,  8.0/9.0, -8.0/9.0,
  -2.0/9.0,  8.0/9.0, -8.0/9.0,        0.0,  8.0/9.0, -8.0/9.0,    2.0/9.0,  8.0/9.0, -8.0/9.0,
   4.0/9.0,  8.0/9.0, -8.0/9.0,    6.0/9.0,  8.0/9.0, -8.0/9.0,    8.0/9.0,  8.0/9.0, -8.0/9.0,
  -8.0/9.0, -8.0/9.0, -6.0/9.0,   -6.0/9.0, -8.0/9.0, -6.0/9.0,   -4.0/9.0, -8.0/9.0, -6.0/9.0,
  -2.0/9.0, -8.0/9.0, -6.0/9.0,        0.0, -8.0/9.0, -6.0/9.0,    2.0/9.0, -8.0/9.0, -6.0/9.0,
   4.0/9.0, -8.0/9.0, -6.0/9.0,    6.0/9.0, -8.0/9.0, -6.0/9.0,    8.0/9.0, -8.0/9.0, -6.0/9.0,
  -8.0/9.0, -6.0/9.0, -6.0/9.0,   -6.0/9.0, -6.0/9.0, -6.0/9.0,   -4.0/9.0, -6.0/9.0, -6.0/9.0,
  -2.0/9.0, -6.0/9.0, -6.0/9.0,        0.0, -6.0/9.0, -6.0/9.0,    2.0/9.0, -6.0/9.0, -6.0/9.0,
   4.0/9.0, -6.0/9.0, -6.0/9.0,    6.0/9.0, -6.0/9.0, -6.0/9.0,    8.0/9.0, -6.0/9.0, -6.0/9.0,
  -8.0/9.0, -4.0/9.0, -6.0/9.0,   -6.0/9.0, -4.0/9.0, -6.0/9.0,   -4.0/9.0, -4.0/9.0, -6.0/9.0,
  -2.0/9.0, -4.0/9.0, -6.0/9.0,        0.0, -4.0/9.0, -6.0/9.0,    2.0/9.0, -4.0/9.0, -6.0/9.0,
   4.0/9.0, -4.0/9.0, -6.0/9.0,    6.0/9.0, -4.0/9.0, -6.0/9.0,    8.0/9.0, -4.0/9.0, -6.0/9.0,
  -8.0/9.0, -2.0/9.0, -6.0/9.0,   -6.0/9.0, -2.0/9.0, -6.0/9.0,   -4.0/9.0, -2.0/9.0, -6.0/9.0,
  -2.0/9.0, -2.0/9.0, -6.0/9.0,        0.0, -2.0/9.0, -6.0/9.0,    2.0/9.0, -2.0/9.0, -6.0/9.0,
   4.0/9.0, -2.0/9.0, -6.0/9.0,    6.0/9.0, -2.0/9.0, -6.0/9.0,    8.0/9.0, -2.0/9.0, -6.0/9.0,
  -8.0/9.0,      0.0, -6.0/9.0,   -6.0/9.0,      0.0, -6.0/9.0,   -4.0/9.0,      0.0, -6.0/9.0,
  -2.0/9.0,      0.0, -6.0/9.0,        0.0,      0.0, -6.0/9.0,    2.0/9.0,      0.0, -6.0/9.0,
   4.0/9.0,      0.0, -6.0/9.0,    6.0/9.0,      0.0, -6.0/9.0,    8.0/9.0,      0.0, -6.0/9.0,
  -8.0/9.0,  2.0/9.0, -6.0/9.0,   -6.0/9.0,  2.0/9.0, -6.0/9.0,   -4.0/9.0,  2.0/9.0, -6.0/9.0,
  -2.0/9.0,  2.0/9.0, -6.0/9.0,        0.0,  2.0/9.0, -6.0/9.0,    2.0/9.0,  2.0/9.0, -6.0/9.0,
   4.0/9.0,  2.0/9.0, -6.0/9.0,    6.0/9.0,  2.0/9.0, -6.0/9.0,    8.0/9.0,  2.0/9.0, -6.0/9.0,
  -8.0/9.0,  4.0/9.0, -6.0/9.0,   -6.0/9.0,  4.0/9.0, -6.0/9.0,   -4.0/9.0,  4.0/9.0, -6.0/9.0,
  -2.0/9.0,  4.0/9.0, -6.0/9.0,        0.0,  4.0/9.0, -6.0/9.0,    2.0/9.0,  4.0/9.0, -6.0/9.0,
   4.0/9.0,  4.0/9.0, -6.0/9.0,    6.0/9.0,  4.0/9.0, -6.0/9.0,    8.0/9.0,  4.0/9.0, -6.0/9.0,
  -8.0/9.0,  6.0/9.0, -6.0/9.0,   -6.0/9.0,  6.0/9.0, -6.0/9.0,   -4.0/9.0,  6.0/9.0, -6.0/9.0,
  -2.0/9.0,  6.0/9.0, -6.0/9.0,        0.0,  6.0/9.0, -6.0/9.0,    2.0/9.0,  6.0/9.0, -6.0/9.0,
   4.0/9.0,  6.0/9.0, -6.0/9.0,    6.0/9.0,  6.0/9.0, -6.0/9.0,    8.0/9.0,  6.0/9.0, -6.0/9.0,
  -8.0/9.0,  8.0/9.0, -6.0/9.0,   -6.0/9.0,  8.0/9.0, -6.0/9.0,   -4.0/9.0,  8.0/9.0, -6.0/9.0,
  -2.0/9.0,  8.0/9.0, -6.0/9.0,        0.0,  8.0/9.0, -6.0/9.0,    2.0/9.0,  8.0/9.0, -6.0/9.0,
   4.0/9.0,  8.0/9.0, -6.0/9.0,    6.0/9.0,  8.0/9.0, -6.0/9.0,    8.0/9.0,  8.0/9.0, -6.0/9.0,
  -8.0/9.0, -8.0/9.0, -4.0/9.0,   -6.0/9.0, -8.0/9.0, -4.0/9.0,   -4.0/9.0, -8.0/9.0, -4.0/9.0,
  -2.0/9.0, -8.0/9.0, -4.0/9.0,        0.0, -8.0/9.0, -4.0/9.0,    2.0/9.0, -8.0/9.0, -4.0/9.0,
   4.0/9.0, -8.0/9.0, -4.0/9.0,    6.0/9.0, -8.0/9.0, -4.0/9.0,    8.0/9.0, -8.0/9.0, -4.0/9.0,
  -8.0/9.0, -6.0/9.0, -4.0/9.0,   -6.0/9.0, -6.0/9.0, -4.0/9.0,   -4.0/9.0, -6.0/9.0, -4.0/9.0,
  -2.0/9.0, -6.0/9.0, -4.0/9.0,        0.0, -6.0/9.0, -4.0/9.0,    2.0/9.0, -6.0/9.0, -4.0/9.0,
   4.0/9.0, -6.0/9.0, -4.0/9.0,    6.0/9.0, -6.0/9.0, -4.0/9.0,    8.0/9.0, -6.0/9.0, -4.0/9.0,
  -8.0/9.0, -4.0/9.0, -4.0/9.0,   -6.0/9.0, -4.0/9.0, -4.0/9.0,   -4.0/9.0, -4.0/9.0, -4.0/9.0,
  -2.0/9.0, -4.0/9.0, -4.0/9.0,        0.0, -4.0/9.0, -4.0/9.0,    2.0/9.0, -4.0/9.0, -4.0/9.0,
   4.0/9.0, -4.0/9.0, -4.0/9.0,    6.0/9.0, -4.0/9.0, -4.0/9.0,    8.0/9.0, -4.0/9.0, -4.0/9.0,
  -8.0/9.0, -2.0/9.0, -4.0/9.0,   -6.0/9.0, -2.0/9.0, -4.0/9.0,   -4.0/9.0, -2.0/9.0, -4.0/9.0,
  -2.0/9.0, -2.0/9.0, -4.0/9.0,        0.0, -2.0/9.0, -4.0/9.0,    2.0/9.0, -2.0/9.0, -4.0/9.0,
   4.0/9.0, -2.0/9.0, -4.0/9.0,    6.0/9.0, -2.0/9.0, -4.0/9.0,    8.0/9.0, -2.0/9.0, -4.0/9.0,
  -8.0/9.0,      0.0, -4.0/9.0,   -6.0/9.0,      0.0, -4.0/9.0,   -4.0/9.0,      0.0, -4.0/9.0,
  -2.0/9.0,      0.0, -4.0/9.0,        0.0,      0.0, -4.0/9.0,    2.0/9.0,      0.0, -4.0/9.0,
   4.0/9.0,      0.0, -4.0/9.0,    6.0/9.0,      0.0, -4.0/9.0,    8.0/9.0,      0.0, -4.0/9.0,
  -8.0/9.0,  2.0/9.0, -4.0/9.0,   -6.0/9.0,  2.0/9.0, -4.0/9.0,   -4.0/9.0,  2.0/9.0, -4.0/9.0,
  -2.0/9.0,  2.0/9.0, -4.0/9.0,        0.0,  2.0/9.0, -4.0/9.0,    2.0/9.0,  2.0/9.0, -4.0/9.0,
   4.0/9.0,  2.0/9.0, -4.0/9.0,    6.0/9.0,  2.0/9.0, -4.0/9.0,    8.0/9.0,  2.0/9.0, -4.0/9.0,
  -8.0/9.0,  4.0/9.0, -4.0/9.0,   -6.0/9.0,  4.0/9.0, -4.0/9.0,   -4.0/9.0,  4.0/9.0, -4.0/9.0,
  -2.0/9.0,  4.0/9.0, -4.0/9.0,        0.0,  4.0/9.0, -4.0/9.0,    2.0/9.0,  4.0/9.0, -4.0/9.0,
   4.0/9.0,  4.0/9.0, -4.0/9.0,    6.0/9.0,  4.0/9.0, -4.0/9.0,    8.0/9.0,  4.0/9.0, -4.0/9.0,
  -8.0/9.0,  6.0/9.0, -4.0/9.0,   -6.0/9.0,  6.0/9.0, -4.0/9.0,   -4.0/9.0,  6.0/9.0, -4.0/9.0,
  -2.0/9.0,  6.0/9.0, -4.0/9.0,        0.0,  6.0/9.0, -4.0/9.0,    2.0/9.0,  6.0/9.0, -4.0/9.0,
   4.0/9.0,  6.0/9.0, -4.0/9.0,    6.0/9.0,  6.0/9.0, -4.0/9.0,    8.0/9.0,  6.0/9.0, -4.0/9.0,
  -8.0/9.0,  8.0/9.0, -4.0/9.0,   -6.0/9.0,  8.0/9.0, -4.0/9.0,   -4.0/9.0,  8.0/9.0, -4.0/9.0,
  -2.0/9.0,  8.0/9.0, -4.0/9.0,        0.0,  8.0/9.0, -4.0/9.0,    2.0/9.0,  8.0/9.0, -4.0/9.0,
   4.0/9.0,  8.0/9.0, -4.0/9.0,    6.0/9.0,  8.0/9.0, -4.0/9.0,    8.0/9.0,  8.0/9.0, -4.0/9.0,
  -8.0/9.0, -8.0/9.0, -2.0/9.0,   -6.0/9.0, -8.0/9.0, -2.0/9.0,   -4.0/9.0, -8.0/9.0, -2.0/9.0,
  -2.0/9.0, -8.0/9.0, -2.0/9.0,        0.0, -8.0/9.0, -2.0/9.0,    2.0/9.0, -8.0/9.0, -2.0/9.0,
   4.0/9.0, -8.0/9.0, -2.0/9.0,    6.0/9.0, -8.0/9.0, -2.0/9.0,    8.0/9.0, -8.0/9.0, -2.0/9.0,
  -8.0/9.0, -6.0/9.0, -2.0/9.0,   -6.0/9.0, -6.0/9.0, -2.0/9.0,   -4.0/9.0, -6.0/9.0, -2.0/9.0,
  -2.0/9.0, -6.0/9.0, -2.0/9.0,        0.0, -6.0/9.0, -2.0/9.0,    2.0/9.0, -6.0/9.0, -2.0/9.0,
   4.0/9.0, -6.0/9.0, -2.0/9.0,    6.0/9.0, -6.0/9.0, -2.0/9.0,    8.0/9.0, -6.0/9.0, -2.0/9.0,
  -8.0/9.0, -4.0/9.0, -2.0/9.0,   -6.0/9.0, -4.0/9.0, -2.0/9.0,   -4.0/9.0, -4.0/9.0, -2.0/9.0,
  -2.0/9.0, -4.0/9.0, -2.0/9.0,        0.0, -4.0/9.0, -2.0/9.0,    2.0/9.0, -4.0/9.0, -2.0/9.0,
   4.0/9.0, -4.0/9.0, -2.0/9.0,    6.0/9.0, -4.0/9.0, -2.0/9.0,    8.0/9.0, -4.0/9.0, -2.0/9.0,
  -8.0/9.0, -2.0/9.0, -2.0/9.0,   -6.0/9.0, -2.0/9.0, -2.0/9.0,   -4.0/9.0, -2.0/9.0, -2.0/9.0,
  -2.0/9.0, -2.0/9.0, -2.0/9.0,        0.0, -2.0/9.0, -2.0/9.0,    2.0/9.0, -2.0/9.0, -2.0/9.0,
   4.0/9.0, -2.0/9.0, -2.0/9.0,    6.0/9.0, -2.0/9.0, -2.0/9.0,    8.0/9.0, -2.0/9.0, -2.0/9.0,
  -8.0/9.0,      0.0, -2.0/9.0,   -6.0/9.0,      0.0, -2.0/9.0,   -4.0/9.0,      0.0, -2.0/9.0,
  -2.0/9.0,      0.0, -2.0/9.0,        0.0,      0.0, -2.0/9.0,    2.0/9.0,      0.0, -2.0/9.0,
   4.0/9.0,      0.0, -2.0/9.0,    6.0/9.0,      0.0, -2.0/9.0,    8.0/9.0,      0.0, -2.0/9.0,
  -8.0/9.0,  2.0/9.0, -2.0/9.0,   -6.0/9.0,  2.0/9.0, -2.0/9.0,   -4.0/9.0,  2.0/9.0, -2.0/9.0,
  -2.0/9.0,  2.0/9.0, -2.0/9.0,        0.0,  2.0/9.0, -2.0/9.0,    2.0/9.0,  2.0/9.0, -2.0/9.0,
   4.0/9.0,  2.0/9.0, -2.0/9.0,    6.0/9.0,  2.0/9.0, -2.0/9.0,    8.0/9.0,  2.0/9.0, -2.0/9.0,
  -8.0/9.0,  4.0/9.0, -2.0/9.0,   -6.0/9.0,  4.0/9.0, -2.0/9.0,   -4.0/9.0,  4.0/9.0, -2.0/9.0,
  -2.0/9.0,  4.0/9.0, -2.0/9.0,        0.0,  4.0/9.0, -2.0/9.0,    2.0/9.0,  4.0/9.0, -2.0/9.0,
   4.0/9.0,  4.0/9.0, -2.0/9.0,    6.0/9.0,  4.0/9.0, -2.0/9.0,    8.0/9.0,  4.0/9.0, -2.0/9.0,
  -8.0/9.0,  6.0/9.0, -2.0/9.0,   -6.0/9.0,  6.0/9.0, -2.0/9.0,   -4.0/9.0,  6.0/9.0, -2.0/9.0,
  -2.0/9.0,  6.0/9.0, -2.0/9.0,        0.0,  6.0/9.0, -2.0/9.0,    2.0/9.0,  6.0/9.0, -2.0/9.0,
   4.0/9.0,  6.0/9.0, -2.0/9.0,    6.0/9.0,  6.0/9.0, -2.0/9.0,    8.0/9.0,  6.0/9.0, -2.0/9.0,
  -8.0/9.0,  8.0/9.0, -2.0/9.0,   -6.0/9.0,  8.0/9.0, -2.0/9.0,   -4.0/9.0,  8.0/9.0, -2.0/9.0,
  -2.0/9.0,  8.0/9.0, -2.0/9.0,        0.0,  8.0/9.0, -2.0/9.0,    2.0/9.0,  8.0/9.0, -2.0/9.0,
   4.0/9.0,  8.0/9.0, -2.0/9.0,    6.0/9.0,  8.0/9.0, -2.0/9.0,    8.0/9.0,  8.0/9.0, -2.0/9.0,
  -8.0/9.0, -8.0/9.0,      0.0,   -6.0/9.0, -8.0/9.0,      0.0,   -4.0/9.0, -8.0/9.0,      0.0,
  -2.0/9.0, -8.0/9.0,      0.0,        0.0, -8.0/9.0,      0.0,    2.0/9.0, -8.0/9.0,      0.0,
   4.0/9.0, -8.0/9.0,      0.0,    6.0/9.0, -8.0/9.0,      0.0,    8.0/9.0, -8.0/9.0,      0.0,
  -8.0/9.0, -6.0/9.0,      0.0,   -6.0/9.0, -6.0/9.0,      0.0,   -4.0/9.0, -6.0/9.0,      0.0,
  -2.0/9.0, -6.0/9.0,      0.0,        0.0, -6.0/9.0,      0.0,    2.0/9.0, -6.0/9.0,      0.0,
   4.0/9.0, -6.0/9.0,      0.0,    6.0/9.0, -6.0/9.0,      0.0,    8.0/9.0, -6.0/9.0,      0.0,
  -8.0/9.0, -4.0/9.0,      0.0,   -6.0/9.0, -4.0/9.0,      0.0,   -4.0/9.0, -4.0/9.0,      0.0,
  -2.0/9.0, -4.0/9.0,      0.0,        0.0, -4.0/9.0,      0.0,    2.0/9.0, -4.0/9.0,      0.0,
   4.0/9.0, -4.0/9.0,      0.0,    6.0/9.0, -4.0/9.0,      0.0,    8.0/9.0, -4.0/9.0,      0.0,
  -8.0/9.0, -2.0/9.0,      0.0,   -6.0/9.0, -2.0/9.0,      0.0,   -4.0/9.0, -2.0/9.0,      0.0,
  -2.0/9.0, -2.0/9.0,      0.0,        0.0, -2.0/9.0,      0.0,    2.0/9.0, -2.0/9.0,      0.0,
   4.0/9.0, -2.0/9.0,      0.0,    6.0/9.0, -2.0/9.0,      0.0,    8.0/9.0, -2.0/9.0,      0.0,
  -8.0/9.0,      0.0,      0.0,   -6.0/9.0,      0.0,      0.0,   -4.0/9.0,      0.0,      0.0,
  -2.0/9.0,      0.0,      0.0,        0.0,      0.0,      0.0,    2.0/9.0,      0.0,      0.0,
   4.0/9.0,      0.0,      0.0,    6.0/9.0,      0.0,      0.0,    8.0/9.0,      0.0,      0.0,
  -8.0/9.0,  2.0/9.0,      0.0,   -6.0/9.0,  2.0/9.0,      0.0,   -4.0/9.0,  2.0/9.0,      0.0,
  -2.0/9.0,  2.0/9.0,      0.0,        0.0,  2.0/9.0,      0.0,    2.0/9.0,  2.0/9.0,      0.0,
   4.0/9.0,  2.0/9.0,      0.0,    6.0/9.0,  2.0/9.0,      0.0,    8.0/9.0,  2.0/9.0,      0.0,
  -8.0/9.0,  4.0/9.0,      0.0,   -6.0/9.0,  4.0/9.0,      0.0,   -4.0/9.0,  4.0/9.0,      0.0,
  -2.0/9.0,  4.0/9.0,      0.0,        0.0,  4.0/9.0,      0.0,    2.0/9.0,  4.0/9.0,      0.0,
   4.0/9.0,  4.0/9.0,      0.0,    6.0/9.0,  4.0/9.0,      0.0,    8.0/9.0,  4.0/9.0,      0.0,
  -8.0/9.0,  6.0/9.0,      0.0,   -6.0/9.0,  6.0/9.0,      0.0,   -4.0/9.0,  6.0/9.0,      0.0,
  -2.0/9.0,  6.0/9.0,      0.0,        0.0,  6.0/9.0,      0.0,    2.0/9.0,  6.0/9.0,      0.0,
   4.0/9.0,  6.0/9.0,      0.0,    6.0/9.0,  6.0/9.0,      0.0,    8.0/9.0,  6.0/9.0,      0.0,
  -8.0/9.0,  8.0/9.0,      0.0,   -6.0/9.0,  8.0/9.0,      0.0,   -4.0/9.0,  8.0/9.0,      0.0,
  -2.0/9.0,  8.0/9.0,      0.0,        0.0,  8.0/9.0,      0.0,    2.0/9.0,  8.0/9.0,      0.0,
   4.0/9.0,  8.0/9.0,      0.0,    6.0/9.0,  8.0/9.0,      0.0,    8.0/9.0,  8.0/9.0,      0.0,
  -8.0/9.0, -8.0/9.0,  2.0/9.0,   -6.0/9.0, -8.0/9.0,  2.0/9.0,   -4.0/9.0, -8.0/9.0,  2.0/9.0,
  -2.0/9.0, -8.0/9.0,  2.0/9.0,        0.0, -8.0/9.0,  2.0/9.0,    2.0/9.0, -8.0/9.0,  2.0/9.0,
   4.0/9.0, -8.0/9.0,  2.0/9.0,    6.0/9.0, -8.0/9.0,  2.0/9.0,    8.0/9.0, -8.0/9.0,  2.0/9.0,
  -8.0/9.0, -6.0/9.0,  2.0/9.0,   -6.0/9.0, -6.0/9.0,  2.0/9.0,   -4.0/9.0, -6.0/9.0,  2.0/9.0,
  -2.0/9.0, -6.0/9.0,  2.0/9.0,        0.0, -6.0/9.0,  2.0/9.0,    2.0/9.0, -6.0/9.0,  2.0/9.0,
   4.0/9.0, -6.0/9.0,  2.0/9.0,    6.0/9.0, -6.0/9.0,  2.0/9.0,    8.0/9.0, -6.0/9.0,  2.0/9.0,
  -8.0/9.0, -4.0/9.0,  2.0/9.0,   -6.0/9.0, -4.0/9.0,  2.0/9.0,   -4.0/9.0, -4.0/9.0,  2.0/9.0,
  -2.0/9.0, -4.0/9.0,  2.0/9.0,        0.0, -4.0/9.0,  2.0/9.0,    2.0/9.0, -4.0/9.0,  2.0/9.0,
   4.0/9.0, -4.0/9.0,  2.0/9.0,    6.0/9.0, -4.0/9.0,  2.0/9.0,    8.0/9.0, -4.0/9.0,  2.0/9.0,
  -8.0/9.0, -2.0/9.0,  2.0/9.0,   -6.0/9.0, -2.0/9.0,  2.0/9.0,   -4.0/9.0, -2.0/9.0,  2.0/9.0,
  -2.0/9.0, -2.0/9.0,  2.0/9.0,        0.0, -2.0/9.0,  2.0/9.0,    2.0/9.0, -2.0/9.0,  2.0/9.0,
   4.0/9.0, -2.0/9.0,  2.0/9.0,    6.0/9.0, -2.0/9.0,  2.0/9.0,    8.0/9.0, -2.0/9.0,  2.0/9.0,
  -8.0/9.0,      0.0,  2.0/9.0,   -6.0/9.0,      0.0,  2.0/9.0,   -4.0/9.0,      0.0,  2.0/9.0,
  -2.0/9.0,      0.0,  2.0/9.0,        0.0,      0.0,  2.0/9.0,    2.0/9.0,      0.0,  2.0/9.0,
   4.0/9.0,      0.0,  2.0/9.0,    6.0/9.0,      0.0,  2.0/9.0,    8.0/9.0,      0.0,  2.0/9.0,
  -8.0/9.0,  2.0/9.0,  2.0/9.0,   -6.0/9.0,  2.0/9.0,  2.0/9.0,   -4.0/9.0,  2.0/9.0,  2.0/9.0,
  -2.0/9.0,  2.0/9.0,  2.0/9.0,        0.0,  2.0/9.0,  2.0/9.0,    2.0/9.0,  2.0/9.0,  2.0/9.0,
   4.0/9.0,  2.0/9.0,  2.0/9.0,    6.0/9.0,  2.0/9.0,  2.0/9.0,    8.0/9.0,  2.0/9.0,  2.0/9.0,
  -8.0/9.0,  4.0/9.0,  2.0/9.0,   -6.0/9.0,  4.0/9.0,  2.0/9.0,   -4.0/9.0,  4.0/9.0,  2.0/9.0,
  -2.0/9.0,  4.0/9.0,  2.0/9.0,        0.0,  4.0/9.0,  2.0/9.0,    2.0/9.0,  4.0/9.0,  2.0/9.0,
   4.0/9.0,  4.0/9.0,  2.0/9.0,    6.0/9.0,  4.0/9.0,  2.0/9.0,    8.0/9.0,  4.0/9.0,  2.0/9.0,
  -8.0/9.0,  6.0/9.0,  2.0/9.0,   -6.0/9.0,  6.0/9.0,  2.0/9.0,   -4.0/9.0,  6.0/9.0,  2.0/9.0,
  -2.0/9.0,  6.0/9.0,  2.0/9.0,        0.0,  6.0/9.0,  2.0/9.0,    2.0/9.0,  6.0/9.0,  2.0/9.0,
   4.0/9.0,  6.0/9.0,  2.0/9.0,    6.0/9.0,  6.0/9.0,  2.0/9.0,    8.0/9.0,  6.0/9.0,  2.0/9.0,
  -8.0/9.0,  8.0/9.0,  2.0/9.0,   -6.0/9.0,  8.0/9.0,  2.0/9.0,   -4.0/9.0,  8.0/9.0,  2.0/9.0,
  -2.0/9.0,  8.0/9.0,  2.0/9.0,        0.0,  8.0/9.0,  2.0/9.0,    2.0/9.0,  8.0/9.0,  2.0/9.0,
   4.0/9.0,  8.0/9.0,  2.0/9.0,    6.0/9.0,  8.0/9.0,  2.0/9.0,    8.0/9.0,  8.0/9.0,  2.0/9.0,
  -8.0/9.0, -8.0/9.0,  4.0/9.0,   -6.0/9.0, -8.0/9.0,  4.0/9.0,   -4.0/9.0, -8.0/9.0,  4.0/9.0,
  -2.0/9.0, -8.0/9.0,  4.0/9.0,        0.0, -8.0/9.0,  4.0/9.0,    2.0/9.0, -8.0/9.0,  4.0/9.0,
   4.0/9.0, -8.0/9.0,  4.0/9.0,    6.0/9.0, -8.0/9.0,  4.0/9.0,    8.0/9.0, -8.0/9.0,  4.0/9.0,
  -8.0/9.0, -6.0/9.0,  4.0/9.0,   -6.0/9.0, -6.0/9.0,  4.0/9.0,   -4.0/9.0, -6.0/9.0,  4.0/9.0,
  -2.0/9.0, -6.0/9.0,  4.0/9.0,        0.0, -6.0/9.0,  4.0/9.0,    2.0/9.0, -6.0/9.0,  4.0/9.0,
   4.0/9.0, -6.0/9.0,  4.0/9.0,    6.0/9.0, -6.0/9.0,  4.0/9.0,    8.0/9.0, -6.0/9.0,  4.0/9.0,
  -8.0/9.0, -4.0/9.0,  4.0/9.0,   -6.0/9.0, -4.0/9.0,  4.0/9.0,   -4.0/9.0, -4.0/9.0,  4.0/9.0,
  -2.0/9.0, -4.0/9.0,  4.0/9.0,        0.0, -4.0/9.0,  4.0/9.0,    2.0/9.0, -4.0/9.0,  4.0/9.0,
   4.0/9.0, -4.0/9.0,  4.0/9.0,    6.0/9.0, -4.0/9.0,  4.0/9.0,    8.0/9.0, -4.0/9.0,  4.0/9.0,
  -8.0/9.0, -2.0/9.0,  4.0/9.0,   -6.0/9.0, -2.0/9.0,  4.0/9.0,   -4.0/9.0, -2.0/9.0,  4.0/9.0,
  -2.0/9.0, -2.0/9.0,  4.0/9.0,        0.0, -2.0/9.0,  4.0/9.0,    2.0/9.0, -2.0/9.0,  4.0/9.0,
   4.0/9.0, -2.0/9.0,  4.0/9.0,    6.0/9.0, -2.0/9.0,  4.0/9.0,    8.0/9.0, -2.0/9.0,  4.0/9.0,
  -8.0/9.0,      0.0,  4.0/9.0,   -6.0/9.0,      0.0,  4.0/9.0,   -4.0/9.0,      0.0,  4.0/9.0,
  -2.0/9.0,      0.0,  4.0/9.0,        0.0,      0.0,  4.0/9.0,    2.0/9.0,      0.0,  4.0/9.0,
   4.0/9.0,      0.0,  4.0/9.0,    6.0/9.0,      0.0,  4.0/9.0,    8.0/9.0,      0.0,  4.0/9.0,
  -8.0/9.0,  2.0/9.0,  4.0/9.0,   -6.0/9.0,  2.0/9.0,  4.0/9.0,   -4.0/9.0,  2.0/9.0,  4.0/9.0,
  -2.0/9.0,  2.0/9.0,  4.0/9.0,        0.0,  2.0/9.0,  4.0/9.0,    2.0/9.0,  2.0/9.0,  4.0/9.0,
   4.0/9.0,  2.0/9.0,  4.0/9.0,    6.0/9.0,  2.0/9.0,  4.0/9.0,    8.0/9.0,  2.0/9.0,  4.0/9.0,
  -8.0/9.0,  4.0/9.0,  4.0/9.0,   -6.0/9.0,  4.0/9.0,  4.0/9.0,   -4.0/9.0,  4.0/9.0,  4.0/9.0,
  -2.0/9.0,  4.0/9.0,  4.0/9.0,        0.0,  4.0/9.0,  4.0/9.0,    2.0/9.0,  4.0/9.0,  4.0/9.0,
   4.0/9.0,  4.0/9.0,  4.0/9.0,    6.0/9.0,  4.0/9.0,  4.0/9.0,    8.0/9.0,  4.0/9.0,  4.0/9.0,
  -8.0/9.0,  6.0/9.0,  4.0/9.0,   -6.0/9.0,  6.0/9.0,  4.0/9.0,   -4.0/9.0,  6.0/9.0,  4.0/9.0,
  -2.0/9.0,  6.0/9.0,  4.0/9.0,        0.0,  6.0/9.0,  4.0/9.0,    2.0/9.0,  6.0/9.0,  4.0/9.0,
   4.0/9.0,  6.0/9.0,  4.0/9.0,    6.0/9.0,  6.0/9.0,  4.0/9.0,    8.0/9.0,  6.0/9.0,  4.0/9.0,
  -8.0/9.0,  8.0/9.0,  4.0/9.0,   -6.0/9.0,  8.0/9.0,  4.0/9.0,   -4.0/9.0,  8.0/9.0,  4.0/9.0,
  -2.0/9.0,  8.0/9.0,  4.0/9.0,        0.0,  8.0/9.0,  4.0/9.0,    2.0/9.0,  8.0/9.0,  4.0/9.0,
   4.0/9.0,  8.0/9.0,  4.0/9.0,    6.0/9.0,  8.0/9.0,  4.0/9.0,    8.0/9.0,  8.0/9.0,  4.0/9.0,
  -8.0/9.0, -8.0/9.0,  6.0/9.0,   -6.0/9.0, -8.0/9.0,  6.0/9.0,   -4.0/9.0, -8.0/9.0,  6.0/9.0,
  -2.0/9.0, -8.0/9.0,  6.0/9.0,        0.0, -8.0/9.0,  6.0/9.0,    2.0/9.0, -8.0/9.0,  6.0/9.0,
   4.0/9.0, -8.0/9.0,  6.0/9.0,    6.0/9.0, -8.0/9.0,  6.0/9.0,    8.0/9.0, -8.0/9.0,  6.0/9.0,
  -8.0/9.0, -6.0/9.0,  6.0/9.0,   -6.0/9.0, -6.0/9.0,  6.0/9.0,   -4.0/9.0, -6.0/9.0,  6.0/9.0,
  -2.0/9.0, -6.0/9.0,  6.0/9.0,        0.0, -6.0/9.0,  6.0/9.0,    2.0/9.0, -6.0/9.0,  6.0/9.0,
   4.0/9.0, -6.0/9.0,  6.0/9.0,    6.0/9.0, -6.0/9.0,  6.0/9.0,    8.0/9.0, -6.0/9.0,  6.0/9.0,
  -8.0/9.0, -4.0/9.0,  6.0/9.0,   -6.0/9.0, -4.0/9.0,  6.0/9.0,   -4.0/9.0, -4.0/9.0,  6.0/9.0,
  -2.0/9.0, -4.0/9.0,  6.0/9.0,        0.0, -4.0/9.0,  6.0/9.0,    2.0/9.0, -4.0/9.0,  6.0/9.0,
   4.0/9.0, -4.0/9.0,  6.0/9.0,    6.0/9.0, -4.0/9.0,  6.0/9.0,    8.0/9.0, -4.0/9.0,  6.0/9.0,
  -8.0/9.0, -2.0/9.0,  6.0/9.0,   -6.0/9.0, -2.0/9.0,  6.0/9.0,   -4.0/9.0, -2.0/9.0,  6.0/9.0,
  -2.0/9.0, -2.0/9.0,  6.0/9.0,        0.0, -2.0/9.0,  6.0/9.0,    2.0/9.0, -2.0/9.0,  6.0/9.0,
   4.0/9.0, -2.0/9.0,  6.0/9.0,    6.0/9.0, -2.0/9.0,  6.0/9.0,    8.0/9.0, -2.0/9.0,  6.0/9.0,
  -8.0/9.0,      0.0,  6.0/9.0,   -6.0/9.0,      0.0,  6.0/9.0,   -4.0/9.0,      0.0,  6.0/9.0,
  -2.0/9.0,      0.0,  6.0/9.0,        0.0,      0.0,  6.0/9.0,    2.0/9.0,      0.0,  6.0/9.0,
   4.0/9.0,      0.0,  6.0/9.0,    6.0/9.0,      0.0,  6.0/9.0,    8.0/9.0,      0.0,  6.0/9.0,
  -8.0/9.0,  2.0/9.0,  6.0/9.0,   -6.0/9.0,  2.0/9.0,  6.0/9.0,   -4.0/9.0,  2.0/9.0,  6.0/9.0,
  -2.0/9.0,  2.0/9.0,  6.0/9.0,        0.0,  2.0/9.0,  6.0/9.0,    2.0/9.0,  2.0/9.0,  6.0/9.0,
   4.0/9.0,  2.0/9.0,  6.0/9.0,    6.0/9.0,  2.0/9.0,  6.0/9.0,    8.0/9.0,  2.0/9.0,  6.0/9.0,
  -8.0/9.0,  4.0/9.0,  6.0/9.0,   -6.0/9.0,  4.0/9.0,  6.0/9.0,   -4.0/9.0,  4.0/9.0,  6.0/9.0,
  -2.0/9.0,  4.0/9.0,  6.0/9.0,        0.0,  4.0/9.0,  6.0/9.0,    2.0/9.0,  4.0/9.0,  6.0/9.0,
   4.0/9.0,  4.0/9.0,  6.0/9.0,    6.0/9.0,  4.0/9.0,  6.0/9.0,    8.0/9.0,  4.0/9.0,  6.0/9.0,
  -8.0/9.0,  6.0/9.0,  6.0/9.0,   -6.0/9.0,  6.0/9.0,  6.0/9.0,   -4.0/9.0,  6.0/9.0,  6.0/9.0,
  -2.0/9.0,  6.0/9.0,  6.0/9.0,        0.0,  6.0/9.0,  6.0/9.0,    2.0/9.0,  6.0/9.0,  6.0/9.0,
   4.0/9.0,  6.0/9.0,  6.0/9.0,    6.0/9.0,  6.0/9.0,  6.0/9.0,    8.0/9.0,  6.0/9.0,  6.0/9.0,
  -8.0/9.0,  8.0/9.0,  6.0/9.0,   -6.0/9.0,  8.0/9.0,  6.0/9.0,   -4.0/9.0,  8.0/9.0,  6.0/9.0,
  -2.0/9.0,  8.0/9.0,  6.0/9.0,        0.0,  8.0/9.0,  6.0/9.0,    2.0/9.0,  8.0/9.0,  6.0/9.0,
   4.0/9.0,  8.0/9.0,  6.0/9.0,    6.0/9.0,  8.0/9.0,  6.0/9.0,    8.0/9.0,  8.0/9.0,  6.0/9.0,
  -8.0/9.0, -8.0/9.0,  8.0/9.0,   -6.0/9.0, -8.0/9.0,  8.0/9.0,   -4.0/9.0, -8.0/9.0,  8.0/9.0,
  -2.0/9.0, -8.0/9.0,  8.0/9.0,        0.0, -8.0/9.0,  8.0/9.0,    2.0/9.0, -8.0/9.0,  8.0/9.0,
   4.0/9.0, -8.0/9.0,  8.0/9.0,    6.0/9.0, -8.0/9.0,  8.0/9.0,    8.0/9.0, -8.0/9.0,  8.0/9.0,
  -8.0/9.0, -6.0/9.0,  8.0/9.0,   -6.0/9.0, -6.0/9.0,  8.0/9.0,   -4.0/9.0, -6.0/9.0,  8.0/9.0,
  -2.0/9.0, -6.0/9.0,  8.0/9.0,        0.0, -6.0/9.0,  8.0/9.0,    2.0/9.0, -6.0/9.0,  8.0/9.0,
   4.0/9.0, -6.0/9.0,  8.0/9.0,    6.0/9.0, -6.0/9.0,  8.0/9.0,    8.0/9.0, -6.0/9.0,  8.0/9.0,
  -8.0/9.0, -4.0/9.0,  8.0/9.0,   -6.0/9.0, -4.0/9.0,  8.0/9.0,   -4.0/9.0, -4.0/9.0,  8.0/9.0,
  -2.0/9.0, -4.0/9.0,  8.0/9.0,        0.0, -4.0/9.0,  8.0/9.0,    2.0/9.0, -4.0/9.0,  8.0/9.0,
   4.0/9.0, -4.0/9.0,  8.0/9.0,    6.0/9.0, -4.0/9.0,  8.0/9.0,    8.0/9.0, -4.0/9.0,  8.0/9.0,
  -8.0/9.0, -2.0/9.0,  8.0/9.0,   -6.0/9.0, -2.0/9.0,  8.0/9.0,   -4.0/9.0, -2.0/9.0,  8.0/9.0,
  -2.0/9.0, -2.0/9.0,  8.0/9.0,        0.0, -2.0/9.0,  8.0/9.0,    2.0/9.0, -2.0/9.0,  8.0/9.0,
   4.0/9.0, -2.0/9.0,  8.0/9.0,    6.0/9.0, -2.0/9.0,  8.0/9.0,    8.0/9.0, -2.0/9.0,  8.0/9.0,
  -8.0/9.0,      0.0,  8.0/9.0,   -6.0/9.0,      0.0,  8.0/9.0,   -4.0/9.0,      0.0,  8.0/9.0,
  -2.0/9.0,      0.0,  8.0/9.0,        0.0,      0.0,  8.0/9.0,    2.0/9.0,      0.0,  8.0/9.0,
   4.0/9.0,      0.0,  8.0/9.0,    6.0/9.0,      0.0,  8.0/9.0,    8.0/9.0,      0.0,  8.0/9.0,
  -8.0/9.0,  2.0/9.0,  8.0/9.0,   -6.0/9.0,  2.0/9.0,  8.0/9.0,   -4.0/9.0,  2.0/9.0,  8.0/9.0,
  -2.0/9.0,  2.0/9.0,  8.0/9.0,        0.0,  2.0/9.0,  8.0/9.0,    2.0/9.0,  2.0/9.0,  8.0/9.0,
   4.0/9.0,  2.0/9.0,  8.0/9.0,    6.0/9.0,  2.0/9.0,  8.0/9.0,    8.0/9.0,  2.0/9.0,  8.0/9.0,
  -8.0/9.0,  4.0/9.0,  8.0/9.0,   -6.0/9.0,  4.0/9.0,  8.0/9.0,   -4.0/9.0,  4.0/9.0,  8.0/9.0,
  -2.0/9.0,  4.0/9.0,  8.0/9.0,        0.0,  4.0/9.0,  8.0/9.0,    2.0/9.0,  4.0/9.0,  8.0/9.0,
   4.0/9.0,  4.0/9.0,  8.0/9.0,    6.0/9.0,  4.0/9.0,  8.0/9.0,    8.0/9.0,  4.0/9.0,  8.0/9.0,
  -8.0/9.0,  6.0/9.0,  8.0/9.0,   -6.0/9.0,  6.0/9.0,  8.0/9.0,   -4.0/9.0,  6.0/9.0,  8.0/9.0,
  -2.0/9.0,  6.0/9.0,  8.0/9.0,        0.0,  6.0/9.0,  8.0/9.0,    2.0/9.0,  6.0/9.0,  8.0/9.0,
   4.0/9.0,  6.0/9.0,  8.0/9.0,    6.0/9.0,  6.0/9.0,  8.0/9.0,    8.0/9.0,  6.0/9.0,  8.0/9.0,
  -8.0/9.0,  8.0/9.0,  8.0/9.0,   -6.0/9.0,  8.0/9.0,  8.0/9.0,   -4.0/9.0,  8.0/9.0,  8.0/9.0,
  -2.0/9.0,  8.0/9.0,  8.0/9.0,        0.0,  8.0/9.0,  8.0/9.0,    2.0/9.0,  8.0/9.0,  8.0/9.0,
   4.0/9.0,  8.0/9.0,  8.0/9.0,    6.0/9.0,  8.0/9.0,  8.0/9.0,    8.0/9.0,  8.0/9.0,  8.0/9.0
};

static FloatType *table_ab1_groupingtables[16] =
/* pointer to sample grouping table, or NULL-pointer if ungrouped */
{ 0, grouping_5bits, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

/* subbands 3-... tables 3-B.2a and 2b: */
static FloatType *table_ab234_groupingtables[16] =
{ 0, grouping_5bits, grouping_7bits, 0, grouping_10bits, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

static FloatType *table_cd_groupingtables[16] =
{ 0, grouping_5bits, grouping_7bits, grouping_10bits, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

static FloatType table_ab1_factor[16] =
/* factor for requantization: (real)sample * factor - 1.0 gives requantized sample */
{ 0.0, 1.0/2.0, 1.0/4.0, 1.0/8.0, 1.0/16.0, 1.0/32.0, 1.0/64.0,
  1.0/128.0, 1.0/256.0, 1.0/512.0, 1.0/1024.0, 1.0/2048.0,
  1.0/4096.0, 1.0/8192.0, 1.0/16384.0, 1.0/32768.0 };

static FloatType table_ab2_factor[16] =
{ 0.0, 1.0/2.0, 1.0/4.0, 1.0/4.0, 1.0/8.0, 1.0/8.0, 1.0/16.0,
  1.0/32.0, 1.0/64.0, 1.0/128.0, 1.0/256.0, 1.0/512.0,
  1.0/1024.0, 1.0/2048.0, 1.0/4096.0, 1.0/32768.0 };

static FloatType table_ab3_factor[8] =
{ 0.0, 1.0/2.0, 1.0/4.0, 1.0/4.0, 1.0/8.0, 1.0/8.0, 1.0/16.0, 1.0/32768.0 };

static FloatType table_ab4_factor[8] = { 0.0, 1.0/2.0, 1.0/4.0, 1.0/32768.0 };

static FloatType table_cd_factor[16] =
{ 0.0, 1.0/2.0, 1.0/4.0, 1.0/8.0, 1.0/8.0, 1.0/16.0, 1.0/32.0, 1.0/64.0,
  1.0/128.0, 1.0/256.0, 1.0/512.0, 1.0/1024.0, 1.0/2048.0, 1.0/4096.0,
  1.0/8192.0, 1.0/16384.0 };


/************************************************************************
 *									*
 *			PRIVATE ROUTINES		       		*
 *									*
 *		       Private  routines				*
 *									*
 ************************************************************************/
static void
PrepareSampleReading(int sb, int mode, int allocation, int chBitrate,
		     FloatType **pGroup, FloatType *pFactor)
{
    FloatType factor;
    FloatType *grouping;

    if (mode != MODE_SINGLE_CHANNEL) {
	if( chBitrate == 4 ) {
	    chBitrate = 1;
	} else {
	    chBitrate -= 4;
	}
    }
    if (chBitrate == 1 || chBitrate == 2) {
	/* table 3-B.2c or 3-B.2d */
	grouping = table_cd_groupingtables[allocation];
	factor = table_cd_factor[allocation];
    } else {
	if (sb <= 2) {
	    grouping = table_ab1_groupingtables[allocation];
	    factor = table_ab1_factor[allocation];
	} else {
	    grouping = table_ab234_groupingtables[allocation];
	    if (sb <= 10) {
		factor = table_ab2_factor[allocation];
	    } else if (sb <= 22) {
		factor = table_ab3_factor[allocation];
	    } else {
		factor = table_ab4_factor[allocation];
	    }
	}
    }
    *pGroup = grouping;
    *pFactor = factor;
}


/************************************************************************
 *									*
 *			PUBLIC ROUTINES					*
 *									*
 *		Publically accessible routines				*
 *									*
 ************************************************************************/

/*
 *	NAME
 *		II_CheckCRC
 *
 *	SYNOPSIS
 */
static UShort
II_CheckCRC(MpegAudioInfoPtr audioInfo, void *alloc_index, void *scfsi)
{
    register int	sb;
    UInt		mask;
    int			sblimit;
    int			bound;
    UChar		*nbal_tbl;
    UShort		crc = ~0;
    int			*pIndex, *pIndex2;
    short		*pScfsi, *pScfsi2;

    UPDATE_CRC(mask, 4, audioInfo->frameHeader.s.bitrate_index, crc);
    UPDATE_CRC(mask, 2, audioInfo->frameHeader.s.sampling_frequency, crc);
    UPDATE_CRC(mask, 1, audioInfo->frameHeader.s.padding_bit, crc);
    UPDATE_CRC(mask, 1, audioInfo->frameHeader.s.private_bit, crc);
    UPDATE_CRC(mask, 2, audioInfo->frameHeader.s.mode, crc);
    UPDATE_CRC(mask, 2, audioInfo->frameHeader.s.mode_extension, crc);
    UPDATE_CRC(mask, 1, audioInfo->frameHeader.s.copyright, crc);
    UPDATE_CRC(mask, 1, audioInfo->frameHeader.s.original_home, crc);
    UPDATE_CRC(mask, 2, audioInfo->frameHeader.s.emphasis, crc);

    sblimit = audioInfo->frameInfo.sblimit;
    nbal_tbl = audioInfo->frameInfo.nbal_table;
    switch(audioInfo->frameHeader.s.mode) {
    case MODE_SINGLE_CHANNEL:
	pIndex = (int *)alloc_index;
	pScfsi = (short *)scfsi;
	for (sb = 0; sb < sblimit; sb++ ) {
	    UPDATE_CRC(mask, nbal_tbl[sb], pIndex[sb], crc);
	    if (pIndex[sb]) {
		UPDATE_CRC(mask, 2, pScfsi[sb], crc);
	    }
	}
	break;
    case MODE_STEREO:
    case MODE_DUAL_CHANNEL:
	pIndex = (int *)alloc_index;
	pIndex2 = pIndex + 32;
	pScfsi = (short *)scfsi;
	pScfsi2 = pScfsi + 32;
	for (sb = 0; sb < sblimit; sb++) {
	    UPDATE_CRC(mask, nbal_tbl[sb], pIndex[sb], crc);
	    UPDATE_CRC(mask, nbal_tbl[sb], pIndex2[sb], crc);
	}
	for (sb = 0; sb < sblimit; sb++) {
	    if (pIndex[sb]) {
		UPDATE_CRC(mask, 2, pScfsi[sb], crc);
	    }
	    if (pIndex2[sb]) {
		UPDATE_CRC(mask, 2, pScfsi2[sb], crc);
	    }
	}
	break;
    case MODE_JOINT_STEREO:
	pIndex = (int *)alloc_index;
	pIndex2 = pIndex + 32;
	pScfsi = (short *)scfsi;
	pScfsi2 = pScfsi + 32;
	bound = MA_Bound[audioInfo->frameHeader.s.mode_extension];
	for (sb = 0; sb < bound; sb++) {
	    UPDATE_CRC(mask, nbal_tbl[sb], pIndex[sb], crc);
	    UPDATE_CRC(mask, nbal_tbl[sb], pIndex2[sb], crc);
	}
	for (sb = bound; sb < sblimit; sb++) {
	    UPDATE_CRC(mask, nbal_tbl[sb], pIndex[sb], crc);
	}
	for (sb = 0; sb < bound; sb++) {
	    if (pIndex[sb]) {
		UPDATE_CRC(mask, 2, pScfsi[sb], crc);
	    }
	    if (pIndex2[sb]) {
		UPDATE_CRC(mask, 2, pScfsi2[sb], crc);
	    }
	}
	for (sb = bound; sb < sblimit; sb++) {
	    if (pIndex[sb]) {
		UPDATE_CRC(mask, 2, pScfsi[sb], crc);
		UPDATE_CRC(mask, 2, pScfsi2[sb], crc);
	    }
	}
	break;
    }
    return(crc);
}

/*
 *	NAME
 *		DecodeLayerII_SingleChannel
 *
 *	SYNOPSIS
 *		This function decodes audio data in Layer I, single
 *		channel format.
 */
CacheReturnValue
DecodeLayerII_SingleChannel(CacheArgsDecl, MpegAudioInfoPtr audioInfo)
{
    BitStreamPtr	bitStream = audioInfo->bitStream;
    register int	sb, granule;
    unsigned int	nb;
    short		scfsi[32];
    unsigned int 	scalefactor[32][3];
    int 		alloc_index[32];
    UShort		input_sample;
    UShort		crc;
    FloatType		requantized_sample[3][32];
    UShort		*pSample;
    int			nSamples;
    int			sblimit;
    unsigned char	*nbal_tbl;
    char		*alloc_tbl;
    II_AllocRec		*p_rec;
    int			i_part;
    unsigned int	c;
    FloatType		*groupingtable[32];
    FloatType		factortable[32];

    sblimit = audioInfo->frameInfo.sblimit;
    nbal_tbl = audioInfo->frameInfo.nbal_table;
    alloc_tbl = audioInfo->frameInfo.alloc_table;

    /*
     * DECODE ALLOCATION
     */
    for (sb = 0; sb < sblimit; sb++) {
	ReadBits(bitStream, nbal_tbl[sb], alloc_index[sb], N_allocation);
    }

    DebugTask(N_AllocationData,
        Msg("\nallocation:    ");
        for (sb = 0; sb < sblimit; sb++) {
            if (alloc_index[sb]) Msg(" %2d", alloc_index[sb]);
            else                 Msg(" .");
        }
        Msg("\n");
    )

    /*
     * DECODE SCALE FACTOR INDICES AND SCALE FACTORS
     */
    for (sb = 0; sb < sblimit; sb++) {
	if (alloc_index[sb]) {
	    ReadBits(bitStream, 2, scfsi[sb], N_scale_factor);
	}
    }

    for (sb = 0; sb < sblimit; sb++) {
	if (alloc_index[sb]) {
	    switch (scfsi[sb]) {
	      case 0:
		/*
		 * three scalefactors transmitted ...
		 *	for parts 0,1,2 respectively
		 */
		ReadBits(bitStream, 6, scalefactor[sb][0], N_scale_factor);
		ReadBits(bitStream, 6, scalefactor[sb][1], N_scale_factor);
		ReadBits(bitStream, 6, scalefactor[sb][2], N_scale_factor);
		break;
	      case 1:
		/*
		 * two scalefactors transmitted ...
		 *	first for parts 0 and 1
		 *	second for part 2
		 */
		ReadBits(bitStream, 6, scalefactor[sb][0], N_scale_factor);
		ReadBits(bitStream, 6, scalefactor[sb][2], N_scale_factor);
		scalefactor[sb][1] = scalefactor[sb][0];
		break;
	      case 2:
		/*
		 * one scalefactor transmitted ...
		 *	valid for all three parts
		 */
		ReadBits(bitStream, 6, scalefactor[sb][0], N_scale_factor);
		scalefactor[sb][2] = scalefactor[sb][1] = scalefactor[sb][0];
		break;
	      case 3:
		/*
		 * two scalefactors transmitted ...
		 *	first for part 0
		 *	second for parts 1 and 2
		 */
		ReadBits(bitStream, 6, scalefactor[sb][0], N_scale_factor);
		ReadBits(bitStream, 6, scalefactor[sb][1], N_scale_factor);
		scalefactor[sb][2] = scalefactor[sb][1];
		break;
	      default:
		break;
	    }

	    PrepareSampleReading(sb, audioInfo->frameHeader.s.mode,
				 alloc_index[sb],
				 audioInfo->frameHeader.s.bitrate_index,
				 &groupingtable[sb],
				 &factortable[sb]);

	}
    }


    /*
     * CRC CHECK
     */
    if (P_REDUNDANCY(audioInfo->frameHeader.s)) {
	crc = II_CheckCRC(audioInfo, alloc_index, scfsi);
	if (crc != audioInfo->crcCheck) {
	    crcError(audioInfo, False);
	    CacheReturn(bitStream);
	}
    }



    /*
     * DECODE SAMPLE
     */
    for (granule = 0; granule < 12; granule++) {
	/*
	 * BUFFER AND REQUANTIZE SAMPLES
	 */
	i_part = granule / 4;
	for (sb = 0; sb < sblimit; sb++) {
	    if (alloc_index[sb]) {
		p_rec = &II_AllocRecords[*(alloc_tbl + sb*16 + alloc_index[sb])];
		nb = p_rec->n_bits_per_codeword;
		if (p_rec->n_samples_per_codeword == 1)  {
		    ReadBits(bitStream, nb, input_sample, N_sample);
		    WhenDebugEnabled(
		       if (input_sample == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
		       }
		    )

		    requantized_sample[0][sb] =
			(FloatType) ((input_sample * factortable[sb] - 1.0
				      + p_rec->D) * p_rec->C);

		    ReadBits(bitStream, nb, input_sample, N_sample);
		    WhenDebugEnabled(
		       if (input_sample == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
		       }
		    )
		    requantized_sample[1][sb] =
			(FloatType) ((input_sample * factortable[sb] - 1.0
				      + p_rec->D) * p_rec->C);

		    ReadBits(bitStream, nb, input_sample, N_sample);
		    WhenDebugEnabled(
		       if (input_sample == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
		       }
		    )
		    requantized_sample[2][sb] =
			(FloatType) ((input_sample * factortable[sb] - 1.0
				      + p_rec->D) * p_rec->C);

		} else {
		   ReadBits(bitStream, nb, c, N_sample);
		   WhenDebugEnabled(
		       if (c == (1 << nb) - 1) {
		 Msg("WARNING: stream contains an illegal subband sample!\n");
		       }
		   )
		   c += c << 1;
		   requantized_sample[0][sb] = groupingtable[sb][c];
		   requantized_sample[1][sb] = groupingtable[sb][c+1];
		   requantized_sample[2][sb] = groupingtable[sb][c+2];
		}
		requantized_sample[0][sb] *=
			I_II_ScaleFactor[scalefactor[sb][i_part]];
		requantized_sample[1][sb] *=
			I_II_ScaleFactor[scalefactor[sb][i_part]];
		requantized_sample[2][sb] *=
			I_II_ScaleFactor[scalefactor[sb][i_part]];
		WhenDebugEnabled(
		    int i;
		    for(i = 0; i < 3; i++) {
			if (   requantized_sample[i][sb] < -1.0
			    || requantized_sample[i][sb] > 1.0) {
	       Msg("WARNING: rescaled subband sample is not in [-1.0, 1.0]\n");
			}
		    }
                )

	    } else {
		requantized_sample[0][sb] = 0.0;
		requantized_sample[1][sb] = 0.0;
		requantized_sample[2][sb] = 0.0;
	    }
	}

	/*
	 * Initialize other subband samples to 0.0
	 */
	for (sb = sblimit; sb < 32; sb++) {
	    requantized_sample[0][sb] = 0.0;
	    requantized_sample[1][sb] = 0.0;
	    requantized_sample[2][sb] = 0.0;
	}

	/*
	 * SYNTHESIS SUBBAND FILTER
	 */
	nSamples = NMonoSamplesPerFrame(audioInfo);
	pSample = &audioInfo->fifo[ audioInfo->nFifoEntries ];
	for (i_part = 0; i_part < 3; i_part++) {
	    BUTMonoSubbandFilter(requantized_sample[i_part], pSample);
	    pSample += nSamples;
	    audioInfo->nFifoEntries += nSamples;
	}
	DebugTask(N_Granules, Msg(".%d", granule) )
    }
    CacheReturn(bitStream);
}


/*
 *	NAME
 *		DecodeLayerII_Stereo
 *
 *	SYNOPSIS
 *		This function decodes audio data in Layer I, single
 *		channel format.
 */
CacheReturnValue
DecodeLayerII_Stereo(CacheArgsDecl, MpegAudioInfoPtr audioInfo)
{

    BitStreamPtr	bitStream = audioInfo->bitStream;
    register int	sb, granule, ch;
    unsigned int	nb;
    short		scfsi[2][32];
    unsigned int 	scalefactor[2][32][3];
    int 		alloc_index[2][32];
    UShort		input_sample;
    UShort		crc;
    FloatType		requantized_sample[3][2][32];
    UShort		*pSample;
    int			nSamples;
    int			sblimit;
    unsigned char	*nbal_tbl;
    char		*alloc_tbl;
    II_AllocRec		*p_rec;
    int			i_part;
    unsigned int	c;
    FloatType		*groupingtable[2][32];
    FloatType		factortable[2][32];

    sblimit = audioInfo->frameInfo.sblimit;
    nbal_tbl = audioInfo->frameInfo.nbal_table;
    alloc_tbl = audioInfo->frameInfo.alloc_table;

    /*
     * DECODE ALLOCATION
     */
    for (sb = 0; sb < sblimit; sb++) {
	ReadBits(bitStream, nbal_tbl[sb], alloc_index[0][sb], N_allocation);
	ReadBits(bitStream, nbal_tbl[sb], alloc_index[1][sb], N_allocation);
    }

    /*
     * DECODE SCALE FACTOR INDICES AND SCALE FACTORS
     */
    for (sb = 0; sb < sblimit; sb++) {
	if (alloc_index[0][sb]) {
	    ReadBits(bitStream, 2, scfsi[0][sb], N_scale_factor);
	}
	if (alloc_index[1][sb]) {
	    ReadBits(bitStream, 2, scfsi[1][sb], N_scale_factor);
	}
    }
    for (sb = 0; sb < sblimit; sb++) {
	for (ch = 0; ch < 2; ch++) {
	    if (alloc_index[ch][sb]) {
		switch (scfsi[ch][sb]) {
		case 0:
		    /*
		     * three scalefactors transmitted ...
		     *	for parts 0,1,2 respectively
		     */
		    ReadBits(bitStream, 6, scalefactor[ch][sb][0],
			     N_scale_factor);
		    ReadBits(bitStream, 6, scalefactor[ch][sb][1],
			     N_scale_factor);
		    ReadBits(bitStream, 6, scalefactor[ch][sb][2],
			     N_scale_factor);
		    break;
		case 1:
		    /*
		     * two scalefactors transmitted ...
		     *	first for parts 0 and 1
		     *	second for part 2
		     */
		    ReadBits(bitStream, 6, scalefactor[ch][sb][0],
			     N_scale_factor);
		    ReadBits(bitStream, 6, scalefactor[ch][sb][2],
			     N_scale_factor);
		    scalefactor[ch][sb][1] = scalefactor[ch][sb][0];
		    break;
		case 2:
		    /*
		     * one scalefactor transmitted ...
		     *	valid for all three parts
		     */
		    ReadBits(bitStream, 6, scalefactor[ch][sb][0],
			     N_scale_factor);
		    scalefactor[ch][sb][2] = scalefactor[ch][sb][1]
		    = scalefactor[ch][sb][0];
		    break;
		case 3:
		    /*
		     * two scalefactors transmitted ...
		     *	first for part 0
		     *	second for parts 1 and 2
		     */
		    ReadBits(bitStream, 6, scalefactor[ch][sb][0],
			     N_scale_factor);
		    ReadBits(bitStream, 6, scalefactor[ch][sb][1],
			     N_scale_factor);
		    scalefactor[ch][sb][2] = scalefactor[ch][sb][1];
		    break;
		default:
		    break;
		}

		PrepareSampleReading(sb, audioInfo->frameHeader.s.mode,
				     alloc_index[ch][sb],
				     audioInfo->frameHeader.s.bitrate_index,
				     &groupingtable[ch][sb],
				     &factortable[ch][sb]);
	    }
	}
    }


    /*
     * CRC CHECK
     */
    if (P_REDUNDANCY(audioInfo->frameHeader.s)) {
        crc = II_CheckCRC(audioInfo, alloc_index, scfsi);
        if (crc != audioInfo->crcCheck) {
	    crcError(audioInfo, True);
            CacheReturn(bitStream);
        }
    }

    /*
     * DECODE SAMPLE
     */
    for (granule = 0; granule < 12; granule++) {
	/*
	 * BUFFER AND REQUANTIZE SAMPLES
	 */
	i_part = granule / 4;
	for (sb = 0; sb < sblimit; sb++) {
	    for (ch = 0; ch < 2 ; ch++) {
		if (alloc_index[ch][sb]) {
		    p_rec = &II_AllocRecords[*(alloc_tbl + sb*16
			    + alloc_index[ch][sb])];
		    nb = p_rec->n_bits_per_codeword;
		    if (p_rec->n_samples_per_codeword == 1)  {
			ReadBits(bitStream, nb, input_sample, N_sample);
			WhenDebugEnabled(
			    if (input_sample == (1 << nb) - 1) {
		   Msg("WARNING: stream contains an illegal subband sample!\n");
			    }
		        )

			requantized_sample[0][ch][sb] =
			(FloatType) ((input_sample * factortable[ch][sb] - 1.0
				      + p_rec->D) * p_rec->C);

			ReadBits(bitStream, nb, input_sample, N_sample);
			WhenDebugEnabled(
		           if (input_sample == (1 << nb) - 1) {

		  Msg("WARNING: stream contains an illegal subband sample!\n");
		           }
		        )
			requantized_sample[1][ch][sb] =
			(FloatType) ((input_sample * factortable[ch][sb] - 1.0
				      + p_rec->D) * p_rec->C);

			ReadBits(bitStream, nb, input_sample, N_sample);
			WhenDebugEnabled(
		           if (input_sample == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
		           }
		        )
			requantized_sample[2][ch][sb] =
			(FloatType) ((input_sample * factortable[ch][sb] - 1.0
				      + p_rec->D) * p_rec->C);

		    } else {
			ReadBits(bitStream, nb, c, N_sample);
			WhenDebugEnabled(
			   if (c == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
		           }
		       )
			c += c << 1;
			requantized_sample[0][ch][sb] =
				groupingtable[ch][sb][c];
			requantized_sample[1][ch][sb] =
				groupingtable[ch][sb][c+1];
			requantized_sample[2][ch][sb] =
				groupingtable[ch][sb][c+2];
		    }
		    requantized_sample[0][ch][sb] *=
			I_II_ScaleFactor[scalefactor[ch][sb][i_part]];
		    requantized_sample[1][ch][sb] *=
			I_II_ScaleFactor[scalefactor[ch][sb][i_part]];
		    requantized_sample[2][ch][sb] *=
			I_II_ScaleFactor[scalefactor[ch][sb][i_part]];
		    
		    WhenDebugEnabled(
		        int i;
		        for(i = 0; i < 3; i++) {
			    if (   requantized_sample[i][ch][sb] < -1.0
				|| requantized_sample[i][ch][sb] > 1.0) {
	 
		Msg("WARNING: rescaled subband sample is not in [-1.0, 1.0]\n");
			    }
		        }
                    )
		    
		} else {
		    requantized_sample[0][ch][sb] = 0.0;
		    requantized_sample[1][ch][sb] = 0.0;
		    requantized_sample[2][ch][sb] = 0.0;
		}
	    }
	}

	for (sb = sblimit; sb < 32; sb++) {
	    requantized_sample[0][0][sb] = 0.0;
	    requantized_sample[0][1][sb] = 0.0;
	    requantized_sample[1][0][sb] = 0.0;
	    requantized_sample[1][1][sb] = 0.0;
	    requantized_sample[2][0][sb] = 0.0;
	    requantized_sample[2][1][sb] = 0.0;
	}

	nSamples = NStereoSamplesPerFrame(audioInfo);
	pSample = &audioInfo->fifo[ audioInfo->nFifoEntries ];
	for (i_part = 0; i_part < 3; i_part++) {
	    if( audioInfo->monoSample ) {
                BUTStereoToMonoSubbandFilter(requantized_sample[i_part],
                                             pSample);
            } else {
                BUTStereoSubbandFilter(requantized_sample[i_part], pSample);
            }
	    pSample += nSamples;
	    audioInfo->nFifoEntries += nSamples;
	}
	
	DebugTask(N_Granules, Msg(".%d", granule) )
    }
    CacheReturn(bitStream);
}


/*
 *	NAME
 *		DecodeLayerII_IntensityStereo
 *
 *	SYNOPSIS
 *		This function decodes audio data in Layer I, single
 *		channel format.
 */
CacheReturnValue
DecodeLayerII_IntensityStereo(CacheArgsDecl, MpegAudioInfoPtr audioInfo)
{
    BitStreamPtr	bitStream = audioInfo->bitStream;
    register int	sb, granule, ch;
    int			bound;
    unsigned int	nb;
    short		scfsi[2][32];
    unsigned int 	scalefactor[2][32][3];
    int 		alloc_index[2][32];
    UShort		input_sample;
    UShort		crc;
    FloatType		requantized_sample[3][2][32];
    UShort		*pSample;
    int			nSamples;
    int			sblimit;
    unsigned char	*nbal_tbl;
    char		*alloc_tbl;
    II_AllocRec		*p_rec;
    int			i_part;
    unsigned int	c;
    FloatType		*groupingtable[2][32];
    FloatType		factortable[2][32];

    sblimit = audioInfo->frameInfo.sblimit;
    nbal_tbl = audioInfo->frameInfo.nbal_table;
    alloc_tbl = audioInfo->frameInfo.alloc_table;

    /*
     * DECODE ALLOCATION
     */
    bound = MA_Bound[audioInfo->frameInfo.mode_extension];
    for (sb = 0; sb < bound; sb++) {
	ReadBits(bitStream, nbal_tbl[sb], alloc_index[0][sb], N_allocation);
	ReadBits(bitStream, nbal_tbl[sb], alloc_index[1][sb], N_allocation);
    }
    for (sb = bound; sb < sblimit; sb++) {
	ReadBits(bitStream, nbal_tbl[sb], alloc_index[0][sb], N_allocation);
	alloc_index[1][sb] = alloc_index[0][sb];
    }

    /*
     * DECODE SCALE FACTOR INDICES AND SCALE FACTORS
     */
    for (sb = 0; sb < bound; sb++) {
	if (alloc_index[0][sb]) {
	    ReadBits(bitStream, 2, scfsi[0][sb], N_scale_factor);
	}
	if (alloc_index[1][sb]) {
	    ReadBits(bitStream, 2, scfsi[1][sb], N_scale_factor);
	}
    }
    for (sb = bound; sb < sblimit; sb++) {
	if (alloc_index[0][sb]) {
	    ReadBits(bitStream, 2, scfsi[0][sb], N_scale_factor);
	    ReadBits(bitStream, 2, scfsi[1][sb], N_scale_factor);
	}
    }
    for (sb = 0; sb < bound; sb++) {
	for (ch = 0; ch < 2; ch++) {
	    if (alloc_index[ch][sb]) {
		switch (scfsi[ch][sb]) {
		  case 0:
		    /*
		     * three scalefactors transmitted ...
		     *	for parts 0,1,2 respectively
		     */
		    ReadBits(bitStream, 6, scalefactor[ch][sb][0],
			    N_scale_factor);
		    ReadBits(bitStream, 6, scalefactor[ch][sb][1],
			    N_scale_factor);
		    ReadBits(bitStream, 6, scalefactor[ch][sb][2],
			    N_scale_factor);
		    break;
		  case 1:
		    /*
		     * two scalefactors transmitted ...
		     *	first for parts 0 and 1
		     *	second for part 2
		     */
		    ReadBits(bitStream, 6, scalefactor[ch][sb][0],
			    N_scale_factor);
		    ReadBits(bitStream, 6, scalefactor[ch][sb][2],
			    N_scale_factor);
		    scalefactor[ch][sb][1] = scalefactor[ch][sb][0];
		    break;
		  case 2:
		    /*
		     * one scalefactor transmitted ...
		     *	valid for all three parts
		     */
		    ReadBits(bitStream, 6, scalefactor[ch][sb][0],
			    N_scale_factor);
		    scalefactor[ch][sb][2] = scalefactor[ch][sb][1]
			    = scalefactor[ch][sb][0];
		    break;
		  case 3:
		    /*
		     * two scalefactors transmitted ...
		     *	first for part 0
		     *	second for parts 1 and 2
		     */
		    ReadBits(bitStream, 6, scalefactor[ch][sb][0],
			    N_scale_factor);
		    ReadBits(bitStream, 6, scalefactor[ch][sb][1],
			    N_scale_factor);
		    scalefactor[ch][sb][2] = scalefactor[ch][sb][1];
		    break;
		  default:
		    break;
		}

		PrepareSampleReading(sb, audioInfo->frameHeader.s.mode,
				     alloc_index[ch][sb],
				     audioInfo->frameHeader.s.bitrate_index,
				     &groupingtable[ch][sb],
				     &factortable[ch][sb]);
	    }
	}
    }

    for (sb = bound; sb < sblimit; sb++) {
	if (alloc_index[0][sb]) {
	    for (ch = 0; ch < 2; ch++) {
		switch (scfsi[ch][sb]) {
		  case 0:
		    /*
		     * three scalefactors transmitted ...
		     *	for parts 0,1,2 respectively
		     */
		    ReadBits(bitStream, 6, scalefactor[ch][sb][0],
			    N_scale_factor);
		    ReadBits(bitStream, 6, scalefactor[ch][sb][1],
			    N_scale_factor);
		    ReadBits(bitStream, 6, scalefactor[ch][sb][2],
			    N_scale_factor);
		    break;
		  case 1:
		    /*
		     * two scalefactors transmitted ...
		     *	first for parts 0 and 1
		     *	second for part 2
		     */
		    ReadBits(bitStream, 6, scalefactor[ch][sb][0],
			    N_scale_factor);
		    ReadBits(bitStream, 6, scalefactor[ch][sb][2],
			    N_scale_factor);
		    scalefactor[ch][sb][1] = scalefactor[ch][sb][0];
		    break;
		  case 2:
		    /*
		     * one scalefactor transmitted ...
		     *	valid for all three parts
		     */
		    ReadBits(bitStream, 6, scalefactor[ch][sb][0],
			    N_scale_factor);
		    scalefactor[ch][sb][2] = scalefactor[ch][sb][1]
			    = scalefactor[ch][sb][0];
		    break;
		  case 3:
		    /*
		     * two scalefactors transmitted ...
		     *	first for part 0
		     *	second for parts 1 and 2
		     */
		    ReadBits(bitStream, 6, scalefactor[ch][sb][0],
			    N_scale_factor);
		    ReadBits(bitStream, 6, scalefactor[ch][sb][1],
			    N_scale_factor);
		    scalefactor[ch][sb][2] = scalefactor[ch][sb][1];
		    break;
		  default:
		    break;
		}

	    }
	    PrepareSampleReading(sb, audioInfo->frameHeader.s.mode,
				 alloc_index[0][sb],
				 audioInfo->frameHeader.s.bitrate_index,
				 &groupingtable[0][sb],
				 &factortable[0][sb]);

	}
    }


    /*
     * CRC CHECK
     */
    if (P_REDUNDANCY(audioInfo->frameHeader.s)) {
        crc = II_CheckCRC(audioInfo, alloc_index, scfsi);
        if (crc != audioInfo->crcCheck) {
	    crcError(audioInfo, True);
            CacheReturn(bitStream);
        }
    }


    /*
     * DECODE SAMPLE
     */
    for (granule = 0; granule < 12; granule++) {
	/*
	 * BUFFER AND REQUANTIZE SAMPLES
	 */
	i_part = granule / 4;
	for (sb = 0; sb < bound; sb++) {
	    for (ch = 0; ch < 2 ; ch++) {
		if (alloc_index[ch][sb]) {
		    p_rec = &II_AllocRecords[*(alloc_tbl + sb*16
			    + alloc_index[ch][sb])];
		    nb = p_rec->n_bits_per_codeword;
		    if (p_rec->n_samples_per_codeword == 1)  {
			ReadBits(bitStream, nb, input_sample, N_sample);
			WhenDebugEnabled(
			    if (input_sample == (1 << nb) - 1) {
		   Msg("WARNING: stream contains an illegal subband sample!\n");
			    }
		        )

			requantized_sample[0][ch][sb] =
			(FloatType) ((input_sample * factortable[ch][sb] - 1.0
				      + p_rec->D) * p_rec->C);

			ReadBits(bitStream, nb, input_sample, N_sample);
			WhenDebugEnabled(
		           if (input_sample == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
		           }
		        )
			requantized_sample[1][ch][sb] =
			(FloatType) ((input_sample * factortable[ch][sb] - 1.0
				      + p_rec->D) * p_rec->C);

			ReadBits(bitStream, nb, input_sample, N_sample);
			WhenDebugEnabled(
		           if (input_sample == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
		           }
		        )
			requantized_sample[2][ch][sb] =
			(FloatType) ((input_sample * factortable[ch][sb] - 1.0
				      + p_rec->D) * p_rec->C);

		    } else {
			ReadBits(bitStream, nb, c, N_sample);
			WhenDebugEnabled(
			   if (c == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
		           }
		       )
			c += c << 1;
			requantized_sample[0][ch][sb] =
				groupingtable[ch][sb][c];
			requantized_sample[1][ch][sb] =
				groupingtable[ch][sb][c+1];
			requantized_sample[2][ch][sb] =
				groupingtable[ch][sb][c+2];
		    }
		    requantized_sample[0][ch][sb] *=
			I_II_ScaleFactor[scalefactor[ch][sb][i_part]];
		    requantized_sample[1][ch][sb] *=
			I_II_ScaleFactor[scalefactor[ch][sb][i_part]];
		    requantized_sample[2][ch][sb] *=
			I_II_ScaleFactor[scalefactor[ch][sb][i_part]];
		    
		    WhenDebugEnabled(
		        int i;
		        for(i = 0; i < 3; i++) {
			    if (   requantized_sample[i][ch][sb] < -1.0
				|| requantized_sample[i][ch][sb] > 1.0) {
	 
		Msg("WARNING: rescaled subband sample is not in [-1.0, 1.0]\n");
			    }
		        }
                    )
		    
		} else {
		    requantized_sample[0][ch][sb] = 0.0;
		    requantized_sample[1][ch][sb] = 0.0;
		    requantized_sample[2][ch][sb] = 0.0;
		}
	    }
	}
 

	for (sb = bound; sb < sblimit; sb++) {
	    if (alloc_index[0][sb]) {
		p_rec = &II_AllocRecords[*(alloc_tbl + sb*16
					   + alloc_index[0][sb])];
		nb = p_rec->n_bits_per_codeword;
		if (p_rec->n_samples_per_codeword == 1)  {
		    ReadBits(bitStream, nb, input_sample, N_sample);
		    WhenDebugEnabled(
		        if (input_sample == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
			}
		    )
		    requantized_sample[0][0][sb] =
		    requantized_sample[0][1][sb] =
		    (FloatType) ((input_sample * factortable[0][sb] - 1.0
				  + p_rec->D) * p_rec->C);

		    ReadBits(bitStream, nb, input_sample, N_sample);
		    WhenDebugEnabled(
		        if (input_sample == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
			}
		    )
		    requantized_sample[1][0][sb] =
		    requantized_sample[1][1][sb] =
		    (FloatType) ((input_sample * factortable[0][sb] - 1.0
				  + p_rec->D) * p_rec->C);

		    ReadBits(bitStream, nb, input_sample, N_sample);
		    WhenDebugEnabled(
		        if (input_sample == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
			}
		    )
		    requantized_sample[2][0][sb] =
		    requantized_sample[2][1][sb] =
		    (FloatType) ((input_sample * factortable[0][sb] - 1.0
				  + p_rec->D) * p_rec->C);

		} else {
		    ReadBits(bitStream, nb, c, N_sample);
		    WhenDebugEnabled(
		        if (c == (1 << nb) - 1) {
		 Msg("WARNING: stream contains an illegal subband sample!\n");
			}
		    )
		    c += c << 1;
		    requantized_sample[0][0][sb] =
		    requantized_sample[0][1][sb] = groupingtable[0][sb][c];
		    requantized_sample[1][0][sb] = 
		    requantized_sample[1][1][sb] = groupingtable[0][sb][c+1];
		    requantized_sample[2][0][sb] = 
		    requantized_sample[2][1][sb] = groupingtable[0][sb][c+2];
		}

		requantized_sample[0][0][sb] *=
			I_II_ScaleFactor[scalefactor[0][sb][i_part]];
		requantized_sample[0][1][sb] *=
			I_II_ScaleFactor[scalefactor[1][sb][i_part]];
		requantized_sample[1][0][sb] *=
			I_II_ScaleFactor[scalefactor[0][sb][i_part]];
		requantized_sample[1][1][sb] *=
			I_II_ScaleFactor[scalefactor[1][sb][i_part]];
		requantized_sample[2][0][sb] *=
			I_II_ScaleFactor[scalefactor[0][sb][i_part]];
		requantized_sample[2][1][sb] *=
			I_II_ScaleFactor[scalefactor[1][sb][i_part]];
		    
		WhenDebugEnabled(
		    int i;
		    for(i = 0; i < 3; i++) {
			if (   requantized_sample[i][0][sb] < -1.0
			    || requantized_sample[i][0][sb] > 1.0
			    || requantized_sample[i][1][sb] < -1.0
			    || requantized_sample[i][1][sb] > 1.0 ) {
	      Msg("WARNING: rescaled subband sample is not in [-1.0, 1.0]\n");
			}
		    }
		)
		    
	    } else {
		requantized_sample[0][0][sb] = 0.0;
		requantized_sample[0][1][sb] = 0.0;
		requantized_sample[1][0][sb] = 0.0;
		requantized_sample[1][1][sb] = 0.0;
		requantized_sample[2][0][sb] = 0.0;
		requantized_sample[2][1][sb] = 0.0;
	    }
	}

	/*
	 * Initialize other subband samples to 0.0
	 */
	for (sb = sblimit; sb < 32; sb++) {
	    requantized_sample[0][0][sb] = 0.0;
	    requantized_sample[0][1][sb] = 0.0;
	    requantized_sample[1][0][sb] = 0.0;
	    requantized_sample[1][1][sb] = 0.0;
	    requantized_sample[2][0][sb] = 0.0;
	    requantized_sample[2][1][sb] = 0.0;
	}

	/*
	 * SYNTHESIS SUBBAND FILTER
	 */
	nSamples = NStereoSamplesPerFrame(audioInfo);
	pSample = &audioInfo->fifo[ audioInfo->nFifoEntries ];
	for (i_part = 0; i_part < 3; i_part++) {
	    if( audioInfo->monoSample ) {
                BUTStereoToMonoSubbandFilter(requantized_sample[i_part],
                                             pSample);
            } else {
                BUTStereoSubbandFilter(requantized_sample[i_part], pSample);
            }
	    pSample += nSamples;
	    audioInfo->nFifoEntries += nSamples;
	}
	DebugTask(N_Granules, Msg(".%d", granule) )
    }
    CacheReturn(bitStream);
}
