#define USE_TIMER
/*
    Testbed program for either comparing numerical output or
    execution time of NVR original SubbandFilter_Mono code with
    the current Bennett Battaile version.
    
    kind of scrappy, but useful.  #define USE_TIMER to run
    a timing comparison; if it's not defined, the program
    prints out the calculated samples side-by-side for
    accuracy checks.  NUM_OUTPUTS below controls how many
    samples are printed.
    
    Bennett Battaile, April-June 1995, under contract to NVR.
*/
 
#define BB_SITE
#ifdef BB_SITE
    /* nothing */
    #define difftime(a,b) (a-b)
#else
    #define difftime(a,b) (a-b)
#endif
 
 
#ifdef USE_TIMER
    #define NVR_COUNT 2000
    #define BB_COUNT  2000
 
    #include <sys/types.h>
    #include <sys/timeb.h>
    #include <time.h>
#endif /* USE_TIMER */
 
 
 
 
#include <math.h> 
#include <stdio.h>
 
#ifndef M_PI
    #define M_PI  3.141592653589793238
#endif
 
typedef float FloatType;
 
#define DECLARE(name) \
    extern void name ( void *scrap, \
                       FloatType requantized_sample[32], \
                       FloatType N[64][32], \
                       FloatType V[2048], \
                       FloatType D[512], \
                       short *output_sample, \
                       int lastSbIdx );
 
DECLARE(   SubbandFilter_Mono);
DECLARE(BB_SubbandFilter_Mono);
 
 
#define NUMRQSAMPLES (1024 )
#define NUM_OUTPUTS 1024 /*NUMRQSAMPLES */
 
 
 
FloatType rqsamples[NUMRQSAMPLES];
FloatType N[64][32];
FloatType D[512];
FloatType NVRworkSpace[2048];
FloatType BBworkSpace[2048];
 
void initializeVariables()
{
    int i, k;
    
    for ( i = 0; i < 64; i++ )
    {
        for ( k = 0; k < 32; k++ )
        {
             /* NOTE!  the scaling by 32K, per NVR current practice */
             N[i][k] = (FloatType)cos( (i+16)*(2*k+1) * M_PI/ 64 )
                     * 0x8000 ;
        }
    }
    
    for ( i = 0; i < 512; i++ )
    {
        D[i] = (FloatType)(cos( i*i) );
        /* some smallish random numbers; assume D has no pattern */
    }
    
    for ( i = 0; i < 2048; i++ )
    {
        NVRworkSpace[i] = (FloatType)0.0;
        BBworkSpace[i] = (FloatType)0.0;
    }
    
    for ( i = 0; i < NUMRQSAMPLES; i++ )
    {
       /* random numbers in range 0-1 */
       rqsamples[i] = (FloatType) (.5 + .5*cos(i*i));
    }
}
 
 
 
 
 
#ifdef USE_TIMER
 
 
 
 
void
main()
{
    short NVRoutputs[NUMRQSAMPLES];
    short BBoutputs[NUMRQSAMPLES];
    int block;
    int i;
 
    time_t t0;
    time_t t1;
    time_t t2;
    double nvrtime;
    double bbtime;
    double speedratio;
 
    
    printf("starting\n");
    
    initializeVariables();
    
    printf("did initialization\n");
    time(&t0);
 
    for ( i = 0; i < NVR_COUNT; i++ )
    {
        for ( block = 0; block < NUMRQSAMPLES; block+=32 )
        {
            SubbandFilter_Mono(0,
                               &rqsamples[block],
                               N,
                               NVRworkSpace,
                               D,
                               &NVRoutputs[block],
                               31);
        }
    }
    time(&t1);
    for ( i = 0; i < BB_COUNT; i++ )
    {
 
        for ( block = 0; block < NUMRQSAMPLES; block+=32 )
        {
            BB_SubbandFilter_Mono(
                           0,
                           &rqsamples[block],
                           N,
                           BBworkSpace,
                           D,
                           &BBoutputs[block],
                           31);
        }
    }
    time(&t2);
    nvrtime = difftime(t1, t0);
    bbtime = difftime(t2, t1);
    printf("elapsed time:  nvr %f  bb %f\n", nvrtime, bbtime);
    printf("number loops nvr %d bb %d\n", NVR_COUNT, BB_COUNT);
    speedratio = ( (float) BB_COUNT ) / ( (float) NVR_COUNT) * nvrtime / bbtime;
    printf("speed ratio bb/nvr %f\n", speedratio);
    
    printf("all done\n");
    
    
}
 
 
 
 
 
 
#else
 
 
 
 
void
main()
{
    short NVRoutputs[NUMRQSAMPLES];
    short BBoutputs[NUMRQSAMPLES];
    int block;
    int i;
          
          
    printf("starting\n");
    
    initializeVariables();
    
    printf("did initialization\n");
 
    for ( block = 0; block < NUMRQSAMPLES; block+=32 )
    {
        SubbandFilter_Mono(0,
                           &rqsamples[block],
                           N,
                           NVRworkSpace,
                           D,
                           &NVRoutputs[block],
                           31);
 
        BB_SubbandFilter_Mono(
                           0,
                           &rqsamples[block],
                           N,
                           BBworkSpace,
                           D,
                           &BBoutputs[block],
                           31);
    }
    
    printf(     "i         diff            real             bb\n");
    
    for ( i = 0;
          i <   NUM_OUTPUTS ;
          i++ )
    {
    
        printf( /* for FloatType "%4i %13.7f %13.7f \n", */
                "%4i%7d %8d\n",
                i,
                NVRoutputs[i] - BBoutputs[i],
                NVRoutputs[i]
                /*BBoutputs[i] */
                 );
        
    }
    
    printf("all done\n");
    
    
}
 
#endif  /* USE_TIMER */
