/*
 *	(c) Copyright 1992 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		SunAudioStruct.h
 *
 *	DESCRIPTION
 *		Defines for Sun:
 *			1. Header for Sun audio files
 *			2. Sun audio data structure
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/SunAudioStruct.h,v 1.2 1995/04/12 23:09:17 russ Exp $
 */
#ifndef _sun_audio_struct_h
#define _sun_audio_struct_h

#define	AUDIO_SUCCESS		(0)		/* no error */
#define	AUDIO_ERR_BADHDR	(1)		/* bad audio header structure */
#define	AUDIO_ERR_BADFILEHDR	(2)		/* bad file header format */
#define	AUDIO_ERR_BADARG	(3)		/* bad subroutine argument */
#define	AUDIO_ERR_NOEFFECT	(4)		/* device control ignored */
#define	AUDIO_ERR_ENCODING	(5)		/* unknown encoding format */
#define	AUDIO_ERR_INTERRUPTED	(6)		/* operation was interrupted */

/*
 * The following defines and typedefs is specifically for the header in
 * Sun audio files (which differ from the Sun audio structure defined later).
 */

#define	AUDIO_FILE_MAGIC		((u_32)0x2e736e64)
#define	AUDIO_FILE_ENCODING_MULAW_8	(1)	/* 8-bit ISDN u-law */
#define	AUDIO_FILE_ENCODING_LINEAR_8	(2)	/* 8-bit linear PCM */
#define	AUDIO_FILE_ENCODING_LINEAR_16	(3)	/* 16-bit linear PCM */
#define	AUDIO_FILE_ENCODING_LINEAR_24	(4)	/* 24-bit linear PCM */
#define	AUDIO_FILE_ENCODING_LINEAR_32	(5)	/* 32-bit linear PCM */
#define	AUDIO_FILE_ENCODING_FLOAT	(6)	/* 32-bit IEEE floating point */
#define	AUDIO_FILE_ENCODING_DOUBLE	(7)	/* 64-bit IEEE floating point */
#ifdef Solaris2
#define AUDIO_FILE_ENCODING_ADPCM_G721  (23)    /* 4-bit CCITT g.721 ADPCM */
#define AUDIO_FILE_ENCODING_ADPCM_G722  (24)    /* CCITT g.722 ADPCM */
#define AUDIO_FILE_ENCODING_ADPCM_G723_3 (25)   /* CCITT g.723 3-bit ADPCM */
#define AUDIO_FILE_ENCODING_ADPCM_G723_5 (26)   /* CCITT g.723 5-bit ADPCM */
#define AUDIO_FILE_ENCODING_ALAW_8      (27)    /* 8-bit ISDN A-law */
#endif /* Solaris2 */

typedef unsigned u_32;
typedef struct {
	u_32		magic;		/* magic number */
	u_32		hdr_size;	/* size of this header */
	u_32		data_size;	/* length of data (optional) */
	u_32		encoding;	/* data encoding format */
	u_32		sample_rate;	/* samples per second */
	u_32		channels;	/* number of interleaved channels */
} Audio_filehdr;


/*
 * The following defines and typedef is the Sun audio data structure.
 * (not to be confused with the Sun audio header file).
 */


#define	AUDIO_ENCODING_ULAW	(1)	/* ISDN u-law */
#define	AUDIO_ENCODING_ALAW	(2)	/* ISDN A-law */
#define	AUDIO_ENCODING_LINEAR	(3)	/* PCM 2's-complement (0-center) */
#ifdef Solaris2
#define AUDIO_ENCODING_NONE     (0)     /* No encoding specified ... */
#define AUDIO_ENCODING_FLOAT    (100)   /* IEEE float (-1. <-> +1.) */
#define AUDIO_ENCODING_G721     (101)   /* CCITT g.721 ADPCM */
#define AUDIO_ENCODING_G722     (102)   /* CCITT g.722 ADPCM */
#define AUDIO_ENCODING_G723     (103)   /* CCITT g.723 ADPCM */
#define AUDIO_ENCODING_DVI      (104)   /* DVI ADPCM */

/*
 * Define the possible endian types.
 */
#define AUDIO_ENDIAN_BIG 0    /* SPARC, 68000, etc. */
#define AUDIO_ENDIAN_SMALL 1  /* Intel              */
#else /* Solaris2 */
#define	AUDIO_ENCODING_FLOAT	(4)	/* IEEE float (-1. <-> +1.) */
#endif /* Solaris2 */

/* Value used for indeterminate size (e.g., data passed through a pipe) */
#define AUDIO_UNKNOWN_SIZE      ((unsigned)(~0))

typedef struct {
	unsigned	sample_rate;		/* samples per second */
	unsigned	samples_per_unit;	/* samples per unit */
	unsigned	bytes_per_unit;		/* bytes per sample unit */
	unsigned	channels;		/* # of interleaved channels */
	unsigned	encoding;		/* data encoding format */
#ifdef Solaris2
	unsigned	endian;			/* byte order */
#endif
	unsigned	data_size;		/* length of data (optional) */
} Audio_hdr;


#endif /* _sun_audio_struct_h */
