/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	sunaifc.c - sun functions for playing Aifc files
 *		    Actually this will only read Aiff files at present
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/aifcio.c,v 1.12 1994/12/18 03:32:16 toddb Exp $
 */
#define NeedFunctionPrototypes 1

#include 	"nvrAudioI.h"
#include 	"nvrAlloc.h"
#include 	"nvrString.h"
#include 	<math.h>
 
/**********************************************************************
 *                      audio file routines                           *
 **********************************************************************/

static char IdValues[][4] = {
    { 'F', 'O', 'R', 'M' },
    { 'A', 'I', 'F', 'F' },
    { 'C', 'O', 'M', 'M' },
    { 'S', 'S', 'N', 'D' },
    { 'M', 'P', 'E', 'G' },
    { 'A', 'I', 'F', 'C' },
    { 'F', 'V', 'E', 'R' },
    { 'N', 'O', 'N', 'E' },
    { '(', 'c', ')', ' ' },
    { 'M', 'A', 'R', 'K' },
    { 'I', 'N', 'S', 'T' },
};

typedef union IDView_union {
    char    byte[4];
    ULong   word;
} IDView;

static IDView *Ids = (IDView *)IdValues;

#define  IFF_ID_FORM Ids[0].word
#define  IFF_ID_AIFF Ids[1].word
#define  IFF_ID_COMM Ids[2].word
#define  IFF_ID_SSND Ids[3].word
#define  IFF_ID_MPEG Ids[4].word
#define  IFF_ID_AIFC Ids[5].word
#define  IFF_ID_FVER Ids[6].word
#define  IFF_ID_NONE Ids[7].word
#define  IFF_ID_CPYR Ids[8].word
#define  IFF_ID_MARK Ids[9].word
#define  IFF_ID_INST Ids[10].word

#define AIFF_HEADER_SIZE	 (sizeof(AiffPrefix) \
				+ sizeof(AiffChunk) /* COMM chunk header */ \
				+ sizeof(AiffCommInfo) \
				+ sizeof(AiffChunk) /* SSND chunk header */ \
				+ sizeof(AiffSsndInfo))

typedef struct _aiffPrefix {
    ULong form;
    ULong totalSize;
    ULong aiff;
} AiffPrefix, *AiffPrefixPtr;

typedef struct _aiffChunk {
    IDView name;
    ULong size;
} AiffChunk;

typedef struct _aiffCommInfo {
    UChar channels[2];
    UChar frames[4];
    UChar bits[2];
    UChar ieeeRate[ 10 ];
} AiffCommInfo;

typedef struct _aifcCommInfo {
    UChar channels[2];
    UChar frames[4];
    UChar bits[2];
    UChar ieeeRate[ 10 ];
    char compType[4];
    UChar length;
    char comment[1];	/* probably longer */
} AifcCommInfo;

typedef struct _aiffSsndInfo {
    long offset;
    long blocksize;
} AiffSsndInfo;

typedef union _aiffBuffer {
    AiffSsndInfo ssnd;
    AiffCommInfo aiffComm;
    AifcCommInfo aifcComm;
    long formatVersion;
    UChar bytes[ 1024 ];
} AiffBuffer;

typedef struct _aiffState {
    long nFrames;
    long totalSize;
} AiffState, *AiffStatePtr;

static InternalReason CantEncode = {
    "Can't encode that kind of data in an aiff file.\n", IR_NotImplemented
};

static void
SetLong(ULong value, UChar bytes[4])
{
    bytes[0] = value >> 24;
    bytes[1] = value >> 16;
    bytes[2] = value >> 8;
    bytes[3] = value;
}

static ULong
CvtLong(UChar bytes[4])
{
    return(  (bytes[0] << 24)
	   | (bytes[1] << 16)
	   | (bytes[2] << 8)
	   | (bytes[3])
	  );
}

static void
SetShort(UShort value, UChar bytes[2])
{
    bytes[0] = value >> 8;
    bytes[1] = value;
}

static UShort
CvtShort(UChar bytes[2])
{
    return(bytes[0] << 8 | bytes[1]);
}

#define UnsignedToFloat(u)      (((double)((long)(u - 2147483647L - 1))) \
				+ 2147483648.0)

typedef struct  IEEE_DBL_struct {
             ULong      hi;
             ULong      lo;
} IEEE_DBL;

typedef struct  SANE_EXT_struct {
             ULong      l1;
             ULong      l2;
             unsigned short     s1;
} SANE_EXT;

/****************************************************************************
*
*  Routines to convert between the Apple SANE extended floating point format
*  and the IEEE double precision floating point format.  These routines are
*  called from within the Audio Interchange File Format (AIFF) routines.
*
*****************************************************************************/

/*
*** Apple's 80-bit SANE extended has the following format:

 1       15      1            63
+-+-------------+-+-----------------------------+
|s|       e     |i|            f                |
+-+-------------+-+-----------------------------+
  msb        lsb   msb                       lsb

The value v of the number is determined by these fields as follows:
If 0 <= e < 32767,              then v = (-1)^s * 2^(e-16383) * (i.f).
If e == 32767 and f == 0,       then v = (-1)^s * (infinity), regardless of i.
If e == 32767 and f != 0,       then v is a NaN, regardless of i.

*** IEEE Draft Standard 754 Double Precision has the following format:

MSB
+-+---------+-----------------------------+
|1| 11 Bits |           52 Bits           |
+-+---------+-----------------------------+
 ^     ^                ^
 |     |                |
 Sign  Exponent         Mantissa
*/

/*****************************************************************************
*
*  ConvertToIeeeExtended()
*
*  Purpose:     Convert from IEEE double precision format to SANE extended
*               format.
*
*  Passed:      Pointer to the double precision number and a pointer to what
*               will hold the Apple SANE extended format value.
*
*  Outputs:     The SANE extended format pointer will be filled with the
*               converted value.
*
*  Returned:    Nothing.
*
*****************************************************************************/
void
ConvertToIeeeExtended(double d, char ps[10])
{
    register ULong  top2bits;

    register unsigned short *ps2;
    register IEEE_DBL       *p_dbl;
    register SANE_EXT       *p_ext;

    p_dbl = (IEEE_DBL *) &d;
    p_ext = (SANE_EXT *) ps;
    top2bits = p_dbl->hi & 0xc0000000;
    p_ext->l1 = ((p_dbl->hi >> 4) & 0x3ff0000) | top2bits;
    p_ext->l1 |= ((p_dbl->hi >> 5) & 0x7fff) | 0x8000;
    p_ext->l2 = (p_dbl->hi << 27) & 0xf8000000;
    p_ext->l2 |= ((p_dbl->lo >> 5) & 0x07ffffff);
    ps2 = (unsigned short *) & (p_dbl->lo);
    ps2++;
    p_ext->s1 = (*ps2 << 11) & 0xf800;
}

#if 0
/*****************************************************************************
*
*  ConvertFromIeeeExtended()
*
*  Purpose:     Convert from SANE extended format to IEEE double precision
*               format.
*
*  Passed:      Pointer to the Apple SANE extended format value and a pointer
*               to what will hold the the IEEE double precision number.
*
*  Outputs:     Nothing.
*
*  Returned:    The IEEE double precision format pointer will be filled with
*               the converted value.
*
*****************************************************************************/

static double ConvertFromIeeeExtended(UChar ps[10])
{
    register ULong  top2bits;
    double d;

    register IEEE_DBL       *p_dbl;
    register SANE_EXT       *p_ext;

    p_dbl = (IEEE_DBL *) &d;
    p_ext = (SANE_EXT *) ps;
    top2bits = p_ext->l1 & 0xc0000000;
    p_dbl->hi = ((p_ext->l1 << 4) & 0x3ff00000) | top2bits;
    p_dbl->hi |= (p_ext->l1 << 5) & 0xffff0;
    p_dbl->hi |= (p_ext->l2 >> 27) & 0x1f;
    p_dbl->lo = (p_ext->l2 << 5) & 0xffffffe0;
    p_dbl->lo |= (ULong) ((p_ext->s1 >> 11) & 0x1f);
    
    return(d);
}

#else /* THINK_C */

static double ConvertFromIeeeExtended(UChar *bytes)
{
    double    f;
    int    expon;
    unsigned long hiMant, loMant;
 
    expon = ((bytes[0] & 0x7F) << 8) | (bytes[1] & 0xFF);
    hiMant    =    ((unsigned long)(bytes[2] & 0xFF) << 24)
            |    ((unsigned long)(bytes[3] & 0xFF) << 16)
            |    ((unsigned long)(bytes[4] & 0xFF) << 8)
            |    ((unsigned long)(bytes[5] & 0xFF));
    loMant    =    ((unsigned long)(bytes[6] & 0xFF) << 24)
            |    ((unsigned long)(bytes[7] & 0xFF) << 16)
            |    ((unsigned long)(bytes[8] & 0xFF) << 8)
            |    ((unsigned long)(bytes[9] & 0xFF));
 
    if (expon == 0 && hiMant == 0 && loMant == 0) {
        f = 0;
    }
    else {
        if (expon == 0x7FFF) {    /* Infinity or NaN */
            f = HUGE_VAL;
        }
        else {
            expon -= 16383;
            f  = ldexp(UnsignedToFloat(hiMant), expon-=31);
            f += ldexp(UnsignedToFloat(loMant), expon-=32);
        }
    }
 
    if (bytes[0] & 0x80)
        return -f;
    else
        return f;
}
#endif /* THINK_C */

static boolean
WriteAiffChunk(AudioFilePtr auFile, char *data, long size)
{
    if (BufferedStdWrite(data, size, 1, auFile->io->fp) != 1) {
        SetAudioReason(auFile, &IRCantWrite);
        return(False);
    }
    return(True);
}

static boolean
WriteAiffHeader(AudioFilePtr auFile)
{
    AiffPrefix prefix;
    AiffCommInfo comm;
    AiffChunk chunk;
    AiffSsndInfo ssndInfo;
    short bits;
    AiffStatePtr aiffState = (AiffStatePtr) auFile->io->state;

    prefix.form = IFF_ID_FORM;
    prefix.totalSize = aiffState->totalSize - sizeof(chunk);
    prefix.aiff = IFF_ID_AIFF;
    if (! WriteAiffChunk(auFile, (char *)&prefix, sizeof(prefix)))
        return(False);

    /*
     * put in an empty copyright
     */
    chunk.name.word = IFF_ID_CPYR;
    chunk.size = 0;
    if (! WriteAiffChunk(auFile, (char *)&chunk, sizeof(chunk)))
        return(False);

    chunk.name.word = IFF_ID_COMM;
    chunk.size = sizeof(comm);
    if (! WriteAiffChunk(auFile, (char *)&chunk, sizeof(chunk)))
        return(False);

    switch(auFile->au.dataForm) {
    case Audio24BitPcm: bits = 24; break;
    case Audio16BitPcm: bits = 16; break;
    case Audio8BitPcm:  bits =  8; break;
    default:
        SetAudioReason(auFile, &CantEncode);
        return(False);
    }

    SetShort(auFile->au.channels, comm.channels);
    SetLong(aiffState->nFrames, comm.frames);
    SetShort(bits, comm.bits);
    ConvertToIeeeExtended((double)auFile->au.rate, (char *)comm.ieeeRate);

    if (! WriteAiffChunk(auFile, (char *)&comm, sizeof(comm)))
        return(False);

    chunk.name.word = IFF_ID_SSND;
    chunk.size = aiffState->nFrames
	       * auFile->au.channels
	       * DataFormToBytes[ auFile->au.dataForm ];
    chunk.size += sizeof(AiffSsndInfo);

    if (! WriteAiffChunk(auFile, (char *)&chunk, sizeof(chunk)))
        return(False);

    ssndInfo.offset = 0;
    ssndInfo.blocksize = 0;
    if (! WriteAiffChunk(auFile, (char *)&ssndInfo, sizeof(ssndInfo)))
        return(False);
    
    auFile->io->hdrSize = TellBufferedFile(auFile->io->fp);

    return(True);
}

static void
CloseAifcAudio(AudioFilePtr auFile)
{
    BufFileHndl fp = auFile->io->fp;
    AiffStatePtr aiffState = (AiffStatePtr) auFile->io->state;
    long dataShift = DataFormToShift[ auFile->au.dataForm ];
    long nDataBytes;

    if (! auFile->io->input) {
	FlushBufferedFile(fp);
	if (SeekBufferedFile(fp, 0, SK_End) < 0)
	    return;
	aiffState->totalSize = TellBufferedFile(fp);
	nDataBytes = aiffState->totalSize - auFile->io->hdrSize;
	aiffState->nFrames = nDataBytes / (auFile->au.channels << dataShift);
	if (SeekBufferedFile(fp, 0, SK_Start) < 0)
	    return;
	(void) WriteAiffHeader(auFile);
    }

#if 0
    Msg("%s %s:\n   size=%d, frames=%d\n",
	auFile->io->input ? "input" : "output",
	auFile->io->name, aiffState->totalSize, aiffState->nFrames);
#endif
}

static boolean
CreateAifcAudio(AudioFilePtr auFile)
{
    AiffStatePtr aiffState;

    if (! AllocAudioState(auFile, sizeof(*aiffState)) )
        return(False);
    aiffState = (AiffStatePtr) auFile->io->state;

    if (auFile->u.fileType == FT_AifcAudio)
	auFile->u.fileType = FT_AiffAudio;	/* we only write aiff */

    if (! WriteAiffHeader(auFile))
        return(False);

    return(True);
}

static boolean
OpenAifcAudio(AudioFilePtr auFile, AudioQualityPtr quality)
{
    AiffPrefix prefix;
    AiffChunk chunk;
    AiffBuffer buffer;
    ULong offset, blocksize;
    long result;
    short channels;
    short bits;
    double rate = 0.0;
    BufFileHndl fp = auFile->io->fp;
    AiffStatePtr aiffState;

    if (! AllocAudioState(auFile, sizeof(*aiffState)) )
        return(False);
    aiffState = (AiffStatePtr) auFile->io->state;

    result = BufferedStdRead((char *)&prefix, sizeof(prefix), 1, fp);
    if (result != 1) {
	SetAudioReason(auFile, &IRPrematureEof);
	return(False);
    }
    if (prefix.form != IFF_ID_FORM
     || (prefix.aiff != IFF_ID_AIFF && prefix.aiff != IFF_ID_AIFC)) {
	SetAudioReason(auFile,
            CreateReason("Corrupt AIFF header in %s\n", auFile->io->name));
	return(False);
    }

    /* Skip everything but the COMM chunk and the SSND chunk */
    /* The SSND chunk must be the last in the file */
    bits = channels = 0;
    while (1) {
	result = BufferedStdRead((char *)&chunk,
				 sizeof(chunk), 1, fp);
	if (result != 1) {
	    SetAudioReason(auFile,
		CreateReason("Missing SSND chunk in AIFF file"));
	    return(False);
	}
	if (chunk.name.word == IFF_ID_SSND)
	    break;

	if (chunk.size > sizeof(buffer)) {
	    SetAudioReason(auFile, CreateReason("AIFF file is corrupt\n"));
	    return(False);
	}
	if (chunk.size
	 && BufferedStdRead((char *)buffer.bytes, chunk.size, 1, fp) != 1) {
	    SetAudioReason(auFile, &IRPrematureEof);
	    return(False);
	}
	
	if (chunk.name.word == IFF_ID_COMM && prefix.aiff == IFF_ID_AIFF) {
	    /* COMM chunk in an aiff file */
	    channels = CvtShort(buffer.aiffComm.channels);
	    aiffState->nFrames = CvtLong(buffer.aiffComm.frames);
	    bits = CvtShort(buffer.aiffComm.bits);
	    rate = ConvertFromIeeeExtended(buffer.aiffComm.ieeeRate);
	}
	else
	if (chunk.name.word == IFF_ID_COMM && prefix.aiff == IFF_ID_AIFC) {
	    IDView compType;

	    /* COMM chunk in an aifc file */
	    channels = CvtShort(buffer.aifcComm.channels);
	    aiffState->nFrames = CvtLong(buffer.aifcComm.frames);
	    bits = CvtShort(buffer.aifcComm.bits);
	    rate = ConvertFromIeeeExtended(buffer.aifcComm.ieeeRate);

	    /* gotta copy 'cause its not aligned */
	    memcpy(compType.byte, buffer.aifcComm.compType, 4);
	    if (compType.word != IFF_ID_NONE) {
		SetAudioReason(auFile,
		    CreateReason("Can't process aifc %c%c%c%c compression\n",
			compType.byte[0], compType.byte[1],
			compType.byte[2], compType.byte[3]));
		return(False);
	    }
	}
	else if (chunk.name.word == IFF_ID_CPYR) {
	    /* ignore copyright */
	}
	else if (chunk.name.word == IFF_ID_MARK) {
	    /* ?? */
	}
	else if (chunk.name.word == IFF_ID_INST) {
	    /* ?? */
	}
	else if (chunk.name.word == IFF_ID_FVER) {
	    /* ignore the version */
	}
	else {
	    ProgMsg("unrecognized aiff chunk: %c%c%c%c, len=%d\n",
		chunk.name.byte[0], chunk.name.byte[1],
		chunk.name.byte[2], chunk.name.byte[3], chunk.size);
	}
    }
    
    /* SSND chunk just read */
    result = BufferedStdRead((char *)buffer.bytes,
			     sizeof(buffer.ssnd),
			     1, fp);
#if 0
    Msg("rate: %f\n", rate);
    Msg("bits: %d\n", bits);
#endif

    if (result != 1) {
	SetAudioReason(auFile,
            CreateReason("Corrupt SSND chunk in AIFF file\n"));
	return(False);
    }
    offset = buffer.ssnd.offset;
    blocksize = buffer.ssnd.blocksize;

    if (blocksize != 0) {
	SetAudioReason(auFile,
            CreateReason("AIFF header specifies nonzero blocksize?"));
	return(False);
    }
    while ((long) (--offset) >= 0) {
	if (GetBufferedChar(fp) == -1) {
	    SetAudioReason(auFile,
		CreateReason("unexpected EOF while skipping AIFF offset"));
	    return(False);
	}
    }
    auFile->io->hdrSize = TellBufferedFile(fp);

    switch (auFile->au.channels = channels) {
    case AudioStereo:
    case AudioMono:
	break;
    default:
	ProgMsg("aiff channels undefined\n");
	break;
    }

    auFile->au.rate = (int)rate;

    switch(bits) {
    case 8:
	auFile->au.dataForm = Audio8BitPcm;
	break;
    case 16:
	auFile->au.dataForm = Audio16BitPcm;
	break;
    default:
	SetAudioReason(auFile,
            CreateReason("unsupported sample size in AIFF header"));
	return(False);
    }

    auFile->nSamples = aiffState->nFrames
                     << DataFormToShift[ auFile->au.dataForm ];
    return(True);
}

AudioOpFunc AudioLibAiffFuncs = {
    OpenAifcAudio,
    CreateAifcAudio,
    StdAudioReadSamples,
    StdAudioWriteSamples,
    StdAudioSeekSamples,
    CloseAifcAudio
};
