/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	auio.c - functions for reading/writing sun audio
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/auio.c,v 1.7 1994/09/11 20:58:56 toddb Exp $
 */
#define NeedFunctionPrototypes 1

#include 	"nvrAudioI.h"
#include 	"SunAudioStruct.h"
#include 	"nvrAlloc.h"

static InternalReason CantEncode = {
    "Can't encode that kind of data in a sun au file.\n", IR_NotImplemented
};
 
static void
CloseAuAudio(AudioFilePtr auFile)
{
    ULong totalSize;
    BufFileHndl fp = auFile->io->fp;
    Audio_filehdr *afh = auFile->io->state;

    if (auFile->io->input)
        return;
    FlushBufferedFile(fp);
    if (SeekBufferedFile(fp, 0, SK_End) < 0)
        return;
    totalSize = TellBufferedFile(fp);
    if (totalSize == afh->data_size)
        return;         /* nothing to fix */

    /*
     * Rewrite the file size into the header.
     */
    afh->data_size = totalSize - sizeof(*afh);
    if (SeekBufferedFile(fp, 0, SK_Start) < 0)
        return;
    (void) BufferedStdWrite((char *)afh, sizeof(*afh), 1, fp);
}

static boolean
CreateAuAudio(AudioFilePtr auFile)
{
    Audio_filehdr *afh;
    BufFileHndl fp = auFile->io->fp;

    if (! AllocAudioState(auFile, sizeof(*afh)) )
	return(False);
    afh = (Audio_filehdr *) auFile->io->state;

    afh->magic       = AUDIO_FILE_MAGIC;
    afh->hdr_size    = sizeof(*afh);
    afh->data_size   = 0;			/* for now */
    afh->sample_rate = auFile->au.rate;
    afh->channels    = auFile->au.channels;

    switch(auFile->au.dataForm) {
    case Audio24BitPcm: afh->encoding = AUDIO_FILE_ENCODING_LINEAR_24; break;
    case Audio16BitPcm: afh->encoding = AUDIO_FILE_ENCODING_LINEAR_16; break;
    case Audio8BitPcm:  afh->encoding = AUDIO_FILE_ENCODING_LINEAR_8; break;
    case Audio8BitULaw: afh->encoding = AUDIO_FILE_ENCODING_MULAW_8; break;
    default:
	SetAudioReason(auFile, &CantEncode);
	return(False);
    }

    if( BufferedWriteData(fp, (char *)afh, sizeof(*afh)) != sizeof(*afh)) {
	SetAudioReason(auFile, &IRCantWrite);
	return(False);
    }
    auFile->io->hdrSize = afh->hdr_size;

    return(True);
}

static boolean
OpenAuAudio(AudioFilePtr auFile, AudioQualityPtr quality)
{
    Audio_filehdr *afh;
    int len;
    char *errFunc = "OpenAuAudio";
    BufFileHndl fp = auFile->io->fp;

    if (! AllocAudioState(auFile, sizeof(*afh)) )
	return(False);
    afh = (Audio_filehdr *) auFile->io->state;

    len = sizeof(*afh);
    if( BufferedReadData(fp,(char *)afh,len) != len ||
       afh->magic != AUDIO_FILE_MAGIC ) {
	SetAudioReason(auFile,
	    CreateReason("%s: %s: not a sun audio file\n",
		errFunc, auFile->io->name)
	);
	return(False);
    }

    switch(afh->encoding) {
    case AUDIO_FILE_ENCODING_MULAW_8:
    case AUDIO_FILE_ENCODING_LINEAR_8:
	auFile->au.dataForm = Audio8BitULaw;
	break;
    case AUDIO_FILE_ENCODING_LINEAR_16:
	auFile->au.dataForm = Audio16BitPcm;
	break;
    default:
	SetAudioReason(auFile,
	    CreateReason("%s: can't handle audio encoding type '0x%x'\n",
		errFunc, afh->encoding)
	);
	return(False);
    }

    switch (auFile->au.channels = afh->channels) {
    case AudioStereo:
    case AudioMono:
        break;
    default:
        ProgMsg("au channels undefined\n");
        break;
    }

    auFile->au.rate = afh->sample_rate;
    if (afh->hdr_size == 0)
	afh->hdr_size = sizeof(*afh);
    auFile->io->hdrSize = afh->hdr_size;
    auFile->nSamples = afh->data_size >> DataFormToShift[ auFile->au.dataForm ];
    return(True);
}

AudioOpFunc AudioLibAuFuncs = {
    OpenAuAudio,
    CreateAuAudio,
    StdAudioReadSamples,
    StdAudioWriteSamples,
    StdAudioSeekSamples,
    CloseAuAudio
};
