/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	prlxAudio.c - The audio part for a Parallax board
 *
 * DESCRIPTION
 *	This is the guts of the audio extractor for the Parallax board.
 *	It translates the 8-bit ulaw audio into 16-bit linear audio.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/jmovieio.c,v 1.7 1995/08/18 17:57:57 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "osio.h"
#include "nvrAudio.h"
#include "SunAudioStruct.h"
#include "nvrPlx.h"
#include "nvrString.h"

typedef struct _jmovieState {
    int frameIndex;
    int sliceIndex;
    int sampleShift;
    int *frameOffsets;	/* Where we record the offsets	*/
    nvrPlxMovieHdr header;
    int (*readAuFrame)(BufFileHndl fp, char *buffer);
} JmovieState, *JmovieStatePtr;

#include 	"nvrAudioI.h"
#include 	"nvrAlloc.h"
 
static InternalReason CantReadOffsets = {
    "Can't read frame offsets.\n", IR_ShortRead
};

static InternalReason CantCreate = {
    "Can`t create jmovie files files with just audio.\n", IR_NotImplemented
};

static int
ReadFrame1Byte(BufFileHndl fp, char *buffer)
{
    int byte1;

    if ((byte1 = GetBufferedChar(fp)) == EndOfFile)
	return(0);
    buffer[0] = byte1;
    return(1);
}

static int
ReadFrame2Bytes(BufFileHndl fp, char *buffer)
{
    int byte1, byte2;

    if ((byte1 = GetBufferedChar(fp)) == EndOfFile)
	return(0);
    if ((byte2 = GetBufferedChar(fp)) == EndOfFile)
	return(0);
    buffer[0] = byte1;
    buffer[1] = byte2;
    return(2);
}

static int
ReadFrame4Bytes(BufFileHndl fp, char *buffer)
{
    int byte1, byte2, byte3, byte4;

    if ((byte1 = GetBufferedChar(fp)) == EndOfFile)
	return(0);
    if ((byte2 = GetBufferedChar(fp)) == EndOfFile)
	return(0);
    if ((byte3 = GetBufferedChar(fp)) == EndOfFile)
	return(0);
    if ((byte4 = GetBufferedChar(fp)) == EndOfFile)
	return(0);
    buffer[0] = byte1;
    buffer[1] = byte2;
    buffer[2] = byte3;
    buffer[3] = byte4;
    return(4);
}

static void
CloseJmovieAudio(AudioFilePtr auFile)
{
}

static boolean
CreateJmovieAudio(AudioFilePtr auFile)
{
    SetAudioReason(auFile, &CantCreate);
    return(False);
}

static boolean
OpenJmovieAudio(AudioFilePtr auFile, AudioQualityPtr quality)
{
    nvrPlxMovieHdr header;
    long nItems, size;
    JmovieStatePtr state;
    BufFileHndl fp = auFile->io->fp;
    int (*readAuFrame)(BufFileHndl fp, char *buffer);

    nItems = BufferedStdRead((char *)&header, sizeof(header), 1, fp);

    if (nItems != 1
     || strcmp(header.magic, "j_movie") != 0) {
	SetAudioReason(auFile,
	    CreateReason("%s: not a jmovie file\n", auFile->io->name)
	);
	return(False);
    }

    /*
     * Hack for nvr-produced movie files
     */
    if (header.audio.channels == 0
     && header.audio.encoding == 0
     && header.audio.sample_rate == 0) {
	header.audio.encoding = AUDIO_FILE_ENCODING_MULAW_8;
	header.audio.channels = 1;
	header.audio.sample_rate = 8000;
    }

    switch (auFile->au.channels = header.audio.channels) {
    case AudioStereo: break;
    case AudioMono:   break;
    default:          ProgMsg("au channels undefined\n"); break;
    }

    switch(header.audio.encoding) {
    case AUDIO_FILE_ENCODING_MULAW_8:
    case AUDIO_FILE_ENCODING_LINEAR_8:
	if (auFile->au.channels == 1)
	    readAuFrame = ReadFrame1Byte;
	else
	    readAuFrame = ReadFrame2Bytes;
	auFile->au.dataForm = Audio8BitULaw;
	break;
    case AUDIO_FILE_ENCODING_LINEAR_16:
	auFile->au.dataForm = Audio16BitPcm;
	if (auFile->au.channels == 1)
	    readAuFrame = ReadFrame2Bytes;
	else
	    readAuFrame = ReadFrame4Bytes;
	break;
    default:
	SetAudioReason(auFile,
	    CreateReason("can't handle audio encoding type '0x%x'\n",
		header.audio.encoding)
	);
	return(False);
    }

    auFile->au.rate = header.audio.sample_rate;
    auFile->nSamples = (header.frames * header.audioslice)
                     >> DataFormToShift[ auFile->au.dataForm ];

    size = sizeof(JmovieState) + header.frames*sizeof(int);
    if (! AllocAudioState(auFile, size))
	return(False);
    state = (JmovieStatePtr) auFile->io->state;
    memcpy((char *)&state->header, (char *)&header, sizeof(header));
    state->frameOffsets = (int *)(state+1);
    state->readAuFrame = readAuFrame;
    state->sampleShift = DataFormToShift[ auFile->au.dataForm ];

    if (SeekBufferedFile(fp, header.indexbuf, 0) < 0)
	Fatal("Couldn't seek to frame offsets @ %d\n", header.indexbuf);
    nItems = BufferedStdRead((char *)state->frameOffsets,
		   sizeof(int) * header.frames, 1, fp);
    
    if (nItems != 1) {
	SetAudioReason(auFile, &CantReadOffsets);
	return(False);
    }

    return(True);
}

static long
ReadJmovieSamples(AudioFilePtr auFile, UChar *buffer, ULong nSamples)
{
    int	length, command, sampleIndex, nBytes;
    JmovieStatePtr state = (JmovieStatePtr)auFile->io->state;
    nvrPlxMovieHdr *header = &state->header;
    int *frameOffsets = state->frameOffsets;
    BufFileHndl fp = auFile->io->fp;

    sampleIndex = 0;
    for (; state->frameIndex < header->frames; state->frameIndex++) {
	if (state->sliceIndex == 0	/* frame not yet in progress */
	 && SeekBufferedFile(fp, frameOffsets[state->frameIndex], SK_Start) < 0)
	{
	    SetAudioReason(auFile,
		CreateReason("Can't find frame %d at offset %d in \"%s\"\n",
		    state->frameIndex, frameOffsets[state->frameIndex],
		    auFile->io->name);
	    );
	    return(sampleIndex);
	}

	/*
	 * Now, keep on reading the byte stream until we get
	 * an END_FRAME indicator.
	 */
	for (;;) {
	    if (state->sliceIndex)
		command = NVRPLXLOAD_AUDIO_0;
	    else {
		command = GetBufferedChar(fp);
		if (command == EndOfFile)
		    break;
	    }

	    switch (command) {
	    case NVRPLXLOAD_AUDIO_0:
	    case NVRPLXLOAD_AUDIO_1:
	    case NVRPLXLOAD_AUDIO_2:
	    case NVRPLXLOAD_AUDIO_3:
		while (sampleIndex < nSamples
		 && state->sliceIndex < header->audioslice) {
		    nBytes = (*state->readAuFrame)(fp, (char *)buffer);
		    if (nBytes == 0)
			goto gotEof;
		    buffer += nBytes;
		    state->sliceIndex += nBytes;
		    sampleIndex += nBytes >> state->sampleShift;
		}
		if (state->sliceIndex == header->audioslice)
		    state->sliceIndex = 0;
		if (sampleIndex == nSamples) {
		    if (state->sliceIndex == 0)
			state->frameIndex++;
		    return(sampleIndex);
		}
		continue;

	    case NVRPLXLOAD_JPEG:
		length = 0; /* lint */
		if (BufferedStdRead((char *)&length,sizeof(length),1,fp) != 1) {
		    SystemError("Can't read JPEG frame length.\n");
		    goto nextFrame;
		}

		if (SeekBufferedFile(fp, length, SK_Relative) != 0) {
		    SystemError("Can't seek %d bytes past JPEG frame.\n",
			length);
		    goto nextFrame;
		}

		continue;

	    case NVRPLXEND_FRAME:
		goto nextFrame;

	    default:
		Msg("%s: Frame %5d: Unknown movie command byte 0x%x %d\n",
		    auFile->io->name, state->frameIndex, command, command);
		/*
		 * Don't know what this frame is;
		 * go onto next frame.
		 */
		goto nextFrame;
	    }
	}
    nextFrame:
	;
    }

gotEof:
    return(sampleIndex);
}

static long
WriteJmovieSamples(AudioFilePtr auFile, UChar *buffer, ULong nSamples)
{
    return(-1); /* we can't */
}

static InternalReasonPtr
SeekJmovieSamples(AudioFilePtr auFile, ULong nSamples)
{
    JmovieStatePtr state = (JmovieStatePtr)auFile->io->state;
    nvrPlxMovieHdr *header = &state->header;
    long samplesNeeded, sampleOffset, bufferSize;
    UChar buffer[ 1024 ];

    state->frameIndex = nSamples / (header->audioslice >> state->sampleShift);
    sampleOffset      = nSamples % (header->audioslice >> state->sampleShift);
    state->sliceIndex = 0;

    if (sampleOffset == 0)
	return(NULL);		/* nothing more to do */

    bufferSize = sizeof(buffer) >> state->sampleShift;
    while (sampleOffset > 0) {
	samplesNeeded = sampleOffset << state->sampleShift;
	if (samplesNeeded > bufferSize)
	    samplesNeeded = bufferSize;
	if (ReadJmovieSamples(auFile, buffer, samplesNeeded) < 0)
	    return(auFile->reason);
	sampleOffset -= samplesNeeded;
    }

    return(NULL);
}

AudioOpFunc AudioLibJmovieFuncs = {
    OpenJmovieAudio,
    CreateJmovieAudio,
    ReadJmovieSamples,
    WriteJmovieSamples,
    SeekJmovieSamples,
    CloseJmovieAudio
};
