/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *      macQTio.c - functions for reading/writing quicktime audio
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/macQTio.c,v 1.4 1995/03/10 08:56:40 toddb Exp $
 */

#include "nvrAudioI.h"

static InternalReason CantSeek = {
    "Seek on quicktime audio Failed.\n", IR_InternalProblem
};
static InternalReason CantRead = {
    "Can't read quicktime audio yet.\n", IR_NotImplemented
};
static InternalReason CantCreate = {
    "Can't write quicktime audio yet.\n", IR_NotImplemented
};

#if defined(THINK_C) || defined(__MWERKS__)
#include "nvrAlloc.h"
#include "nvrString.h"
#include "fileAccess.h"
#include "macDialogs.h"
#include "macUtils.h"

#include <Movies.h>
#include <Memory.h>

#ifndef kFix1
	#define kFix1			(0x00010000)
#endif

typedef struct _macQtState {
    Movie movie;
    Media media;
    Handle audioHndl;        /* handle to hold the audio data */
    long sampleOffset;       /* current (sample) offset within the audio data */
    long numberOfSamples;    /* current count of samples in audio data */
    double scaleToMovie;
    double scaleToMedia;
    TimeValue mediaTimeScale;
    TimeValue movieTimeScale;
    TimeValue currentMovieTime;
    TimeValue movieDuration;
} MacQtState, *MacQtStatePtr;

static InternalReason CantDoMAC = {
    "Can't handle MAC3 or MAC6 yet.\n", IR_NotImplemented
};
 
static void
CloseQTAudio(AudioFilePtr auFile)
{
    MacQtStatePtr qts = (MacQtStatePtr) auFile->io->state;
    
    if (qts->movie) {
        DisposeMovie(qts->movie);
        qts->movie = nil;
    }
    
    if (qts->audioHndl) {
        DisposeHandle(qts->audioHndl);
        qts->audioHndl = nil;
    }
}

static boolean
CreateQTAudio(AudioFilePtr auFile)
{
    SetAudioReason(auFile, &CantCreate);
    return(False);
}

static boolean
CantOpenMovie(AudioFilePtr auFile)
{
    InternalReasonPtr   reason;

    reason = CreateSysReason("Can't open movie %s",
                             ExtractPathSuffix(auFile->io->name));
    SetAudioReason(auFile, reason);
    CloseQTAudio(auFile);
    return(False);
}

static boolean
GetMovieStats (AudioFilePtr auFile)
{
    OSType  trackType;
    long    numTracks, numAudioTracks;
    long    i;
    long    numDescriptions;
    Track   thisTrack;
    Media   media, audioMedia;
    SoundDescriptionHandle descH;
    SoundDescriptionPtr desc;
    MacQtStatePtr qts = (MacQtStatePtr) auFile->io->state;

    numTracks = GetMovieTrackCount(qts->movie);
    numAudioTracks = 0;
    audioMedia = nil;

    for (i = 1; i <= numTracks; ++i)
    {
        thisTrack = GetMovieIndTrack (qts->movie, i);
        media = GetTrackMedia(thisTrack);
        GetMediaHandlerDescription(media, &trackType, nil, nil);

        if (trackType == SoundMediaType) {
            audioMedia = media;
            numAudioTracks++;
        }
    }
    
    if (audioMedia) {
        qts->media = audioMedia;
        qts->movieTimeScale = GetMovieTimeScale(qts->movie);
        qts->mediaTimeScale = GetMediaTimeScale(qts->media);
        qts->scaleToMovie = (double) qts->movieTimeScale / qts->mediaTimeScale;
        qts->scaleToMedia = (double) qts->mediaTimeScale / qts->movieTimeScale;
        qts->movieDuration = GetMovieDuration(qts->movie);
        descH = (SoundDescriptionHandle) NewHandle(1);
        if (descH == nil) {
            SetAudioReason(auFile, &IRBadAllocation);
            return(False);
        }

        numDescriptions = GetMediaSampleDescriptionCount(audioMedia);
        if (numDescriptions > 1)
            CautionDialog("%d descriptions for audio media!\n", numDescriptions);
        GetMediaSampleDescription(audioMedia, 1, (SampleDescriptionHandle)descH);
        desc = *descH;
        
        switch (desc->dataFormat) {
        case 'raw ':
            switch (desc->sampleSize) {
            case 8:  auFile->au.dataForm = AudioUSign8BitPcm; break;
            case 16: auFile->au.dataForm = AudioUSign16BitPcm; break;
            }
            break;
        case 'twos':
            switch (desc->sampleSize) {
            case 8:  auFile->au.dataForm = Audio8BitPcm; break;
            case 16: auFile->au.dataForm = Audio16BitPcm; break;
            }
            break;
        case 'MAC3':
            SetAudioReason(auFile, &CantDoMAC);
            return(False);
        }
        
        switch (desc->sampleSize) {
        case 8:
        case 16:
            break;
        default:
            SetAudioReason(auFile,
                CreateReason("unknown sample size: %d", desc->sampleSize));
            return(False);
        }

        switch (auFile->au.rate = ((ULong)desc->sampleRate >> 16)) {
        case 44100: break;
        case 11127: break;
        case 22254: break;
        default:
            SetAudioReason(auFile,
                CreateReason("can't handle sample rate of %.3f",
                             desc->sampleRate / (double)0x10000));
            return(False);
        }
        
        auFile->au.channels = desc->numChannels;
        auFile->io->hdrSize = 0;        /* not meaningful here */
        
        auFile->nSamples = (qts->movieDuration / (double) qts->movieTimeScale)
                         * (auFile->au.rate * auFile->au.channels);

        DisposeHandle((Handle)descH);
        return(True);
    }
    return(False);
}

static boolean
OpenQTAudio(AudioFilePtr auFile, AudioQualityPtr quality)
{
    MacQtStatePtr       qts;
    FSSpec              fsSpec;
    OSErr               err;
    extern int          errno;
    short               movieResID;
    short               movieResFile;
    Str255              movieName;
    Boolean             wasChanged;

    if (! AllocAudioState(auFile, sizeof(*qts)) )
        return(False);
    qts = (MacQtStatePtr) auFile->io->state;

    qts->currentMovieTime = 0;
    qts->sampleOffset = 0;
    qts->audioHndl = NewHandle(1);
    if (qts->audioHndl == nil) {
        SetAudioReason(auFile, &IRBadAllocation);
        return(False);
    }

    PathnameToFSSpec(auFile->io->name, &fsSpec);
    if (errno = OpenMovieFile(&fsSpec, &movieResFile, fsRdPerm)) 
        return( CantOpenMovie(auFile) );
        
    movieResID = 0;
    err = NewMovieFromFile(&qts->movie, movieResFile,
                           &movieResID, movieName,
                           newMovieActive, &wasChanged);
    CloseMovieFile(movieResFile);
    if (errno = err)
        return( CantOpenMovie(auFile) );

    if (! GetMovieStats(auFile)) {
        CloseQTAudio(auFile);
        return(False);
    }

    return(True);
}

/*
 * Read the next set of samples from the movie.
 */
static boolean
ReadNextSampleSet(AudioFilePtr auFile)
{
    MacQtStatePtr qts = (MacQtStatePtr) auFile->io->state;
    TimeValue movieFrameStart, movieFrameDuration, mediaFrameStart;
    TimeValue currentMediaTime;
    TimeValue actualStart, durationPerFrame;
    long size, numberOfFrames;
    short flags = nextTimeMediaSample
                | nextTimeIgnoreActiveSegment
                | nextTimeEdgeOK;
    const OSType whichMediaTypes = AudioMediaCharacteristic;
    OSErr err;

    qts->sampleOffset = 0;
    qts->numberOfSamples = 0;

    /*
     * Starting at the beginning of the movie,
     * find each frame (media sample) and display it in the window
     */

    GetMovieNextInterestingTime(qts->movie, flags, 1, &whichMediaTypes,
        qts->currentMovieTime, kFix1, &movieFrameStart, &movieFrameDuration);

    /* Break if an error has occurred */

    if (movieFrameStart < 0 || movieFrameDuration <= 0)
        return(False);
    mediaFrameStart = movieFrameStart * qts->scaleToMedia + 0.5;

    err = GetMediaSample(qts->media,
                         qts->audioHndl,     /* dataOut */
                         0,                  /* maxSizeToGrow */
                         &size,              /* *size */
                         mediaFrameStart,    /* time */
                         &actualStart,       /* *sampleTime */
                         &durationPerFrame, /* *durationPerSample */
                         nil,                /* sampleDescriptionH */
                         nil,                /* *sampleDescriptionIndex */
                         0,                  /* maxNumberOfSamples */
                         &numberOfFrames,    /* *numberOfSamples */
                         nil);               /* *sampleFlags */

    if (err)
        return(False);
    qts->numberOfSamples = size >> DataFormToShift[ auFile->au.dataForm ];
    currentMediaTime = actualStart
                     + numberOfFrames * durationPerFrame;
    /*
     * Advance the current movie time to the beginning of the next frame
     */
    qts->currentMovieTime = currentMediaTime * qts->scaleToMovie + 0.5;

    return( qts->numberOfSamples > 0 );
}

static long
ReadQTSamples(AudioFilePtr auFile, UChar *buffer, ULong nSamples)
{
    MacQtStatePtr qts = (MacQtStatePtr) auFile->io->state;
    long samplesNeeded = nSamples;
    long count, available, i;
    short sampleSize = DataFormToBytes[ auFile->au.dataForm ];
    AudioDataType from, to;

    while (samplesNeeded > 0) {
        available = qts->numberOfSamples - qts->sampleOffset;
        if (available == 0) {
            if (! ReadNextSampleSet(auFile))
                break;
            available = qts->numberOfSamples - qts->sampleOffset;
        }
        if ((count = samplesNeeded) > available)
            count = available;
        
        to.anyType = buffer;
        from.pcm8 = *qts->audioHndl;
        switch (sampleSize) {
        case 1:
            from.pcm8 += qts->sampleOffset;
            for (i = count; --i >= 0; )
                to.pcm8[ i ] = from.pcm8[ i ];
            buffer += count;
            break;
        case 2:
            from.pcm16 += qts->sampleOffset;
            for (i = count; --i >= 0; )
                to.pcm16[ i ] = from.pcm16[ i ];
            buffer += count << 1;
            break;
        case 4:
            from.pcm24 += qts->sampleOffset;
            for (i = count; --i >= 0; )
                to.pcm24[ i ] = from.pcm24[ i ];
            buffer += count << 2;
            break;
        }
        qts->sampleOffset += count;
        samplesNeeded -= count;
    }
    
    return(nSamples - samplesNeeded);	/* true if samples were read */
}

static InternalReasonPtr
SeekQTSamples(AudioFilePtr auFile, ULong nSamples)
{
    MacQtStatePtr qts = (MacQtStatePtr) auFile->io->state;
    double movieFrac;
    TimeValue targetMediaTime;
    TimeValue targetMovieTime;
    TimeValue mediaTime;
    long sampleNum, sampleOffset;
    
    if (nSamples == 0) {
        /*
         * Reset to the beginning of the file
         */
        qts->currentMovieTime = 0;
        qts->sampleOffset = 0;
        return(NULL);
    }
    
    movieFrac = nSamples / (double) auFile->nSamples;
    targetMediaTime = movieFrac * qts->movieDuration * qts->scaleToMedia + .5;
    MediaTimeToSampleNum(qts->media, targetMediaTime, &sampleNum,
                         &mediaTime, nil);
    targetMovieTime = mediaTime * qts->scaleToMovie + .5;
    qts->currentMovieTime = targetMovieTime;
    if (! ReadNextSampleSet(auFile))
        return(&CantSeek);

    sampleNum = (targetMovieTime / (double)qts->movieDuration)
              * auFile->nSamples
              + .5;
    sampleNum &= ~1;	/* make it even so we don't swap channels */
    sampleOffset = nSamples - sampleNum;
    if (sampleOffset >= qts->numberOfSamples)
        return(&CantSeek);
    qts->sampleOffset = sampleOffset;
    return(NULL);
}

#else /* THINK_C or __MWERKS__ */

static boolean
CreateQTAudio(AudioFilePtr auFile)
{
    SetAudioReason(auFile, &CantCreate);
    return(False);
}

static boolean
OpenQTAudio(AudioFilePtr auFile, AudioQualityPtr quality)
{
    SetAudioReason(auFile, &CantRead);
    return(False);
}

static long
ReadQTSamples(AudioFilePtr auFile, UChar *buffer, ULong nSamples)
{
    SetAudioReason(auFile, &CantRead);
    return(-1);
}

static void
CloseQTAudio(AudioFilePtr auFile)
{
}

static InternalReasonPtr
SeekQTSamples(AudioFilePtr auFile, ULong nSamples)
{
    return(&CantSeek);
}

#endif /* THINK_C or __MWERKS__ */

static long
WriteQTSamples(AudioFilePtr auFile, UChar *buffer, ULong nSamples)
{
    SetAudioReason(auFile, &CantCreate);
    return(-1);
}

AudioOpFunc AudioLibQTFuncs = {
    OpenQTAudio,
    CreateQTAudio,
    ReadQTSamples,
    WriteQTSamples,
    SeekQTSamples,
    CloseQTAudio
};
