/*
 * Copyright (c) 1992, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	sunaudio.c
 *
 * DESCRIPTION
 *	Programming interface to SparcStation's audio device.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/noaudio.c,v 1.3 1994/01/10 18:48:13 toddb Exp $
 */
#ifndef LINT
static char *copy_notice = "Copyright 1992 North Valley Research, Inc.";
#endif /* LINT */

/*
 *      EXTERNAL INCLUDES
 *              Include files that must be exported to any package or
 *              program using this package.
 */
#include "sunaudio.h"


/*
 *      INTERNAL INCLUDES
 *              Include files for local use only, therefore, NOT exported
 *		to any package or program using this package.
 */
#include <errno.h>
#include <fcntl.h>
#include <math.h>

#include "nvrTypes.h"
typedef long Audio_hdr;
typedef long Audio_info;

/*
 *      EXTERNS
 *              External declarations required locally to this package
 *              that are not already declared in any of the included header
 *		files (external includes or internal includes).
 */


/*
 *      GLOBALS
 *              Variables declared in this package that are allowed
 *		to be used globally.
 */


/*
 *      LOCAL DEFINES
 *		#define declarations local to this package.
 */
#define MIN(x,y) ((x) > (y) ? (y) : (x))
#define GAIN_TO_FLOAT(value)						\
	((double)(value) / (double)AUDIO_MAX_GAIN)

/*
 *      LOCAL TYPEDEFS
 *              typedefs local to this package (for use with local vars).
 *
 */

typedef struct _AudioCallbackRec {
    unsigned		which_changes;
    AudioCallbackProc	callback_proc;
    void *		client_data;
    struct _AudioCallbackRec *next;
} AudioCallbackRec, *AudioCallbackList;


/*
 *      LOCALS VARIABLES
 *		Variables local to this package.
 */

static int		AudioFd = -1;
static int		AudioCtlFd = -1;
static int		AudioFdMode = 0;
static audio_info_t	AudioState;
static Audio_hdr	AudioPlayConfig;
static Audio_hdr	AudioRecordConfig;
static AudioCallbackList AudioCallbacks = (AudioCallbackList)NULL;



/************************************************************************
 *									*
 *			PRIVATE ROUTINES				*
 *									*
 *	Routines private to this package therefore declared static	*
 *									*
 ************************************************************************/

/*
 *	NAME
 *		HandleControlChanges
 *
 *	SYNOPSIS
 */
static void
HandleControlChanges()
/*
 *	DESCRIPTION
 *		Add a callback for the indicated audio changes.
 *
 *	RETURNS
 *		
 */
{
}


/************************************************************************
 *									*
 *			PUBLIC ROUTINES					*
 *									*
 *		Publically accessible routines				*
 *									*
 ************************************************************************/

/*
 *
 *	Interfaces for Audio CONTROL Device Interfaces
 *
 */



/*
 *	NAME
 *		AudioGetState
 *
 *	SYNOPSIS
 */
int
AudioGetState(ap)
    audio_info_t	*ap;
/*
 *	DESCRIPTION
 *		Obtains the current control parameters of the audio device.
 *
 *	RETURNS
 *		0 on error; otherwise non-zero.
 *
 */
{
    return(0);
}


/*
 *	NAME
 *		AudioSetMonitorVolume
 *
 *	SYNOPSIS
 */
void
AudioSetMonitorVolume(value)
    double value;
/*
 *	DESCRIPTION
 *		Sets the audio device's monitor volume.
 *		Valid values are any floating point values between 0.0 and 1.0.
 *
 *	RETURNS
 *		The new value used by the audio device.
 *
 *	NOTES
 *		Updating actual value is done via
 *		SIGIO ... see AudioAddControlChangeCallback().
 */
{
}


/*
 *	NAME
 *		AudioSetMonitorState
 *
 *	SYNOPSIS
 */
int
AudioSetMonitorState(control, ratio)
    int control;
    double ratio;
/*
 *	DESCRIPTION
 *		Sets the audio device's monitor volume
 *
 *	RETURNS
 *		1 on success, otherwise 0;
 *
 *	NOTES
 *		Updating actual set values are done via
 *		SIGIO ... see AudioAddControlChangeCallback().
 */
{
    return(0);
}


/*
 *	NAME
 *		AudioSetPlayVolume
 *
 *	SYNOPSIS
 */
void
AudioSetPlayVolume(value)
    double value;
/*
 *	DESCRIPTION
 *		Sets the audio device's play volume
 *		Valid values are any floating point values between 0.0 and 1.0.
 *
 *	RETURNS
 *		The new value used by the audio device.
 *
 *	NOTES
 *		Updating actual value is done via
 *		SIGIO ... see AudioAddControlChangeCallback().
 */
{
}


/*
 *	NAME
 *		AudioSetPlayState
 *
 *	SYNOPSIS
 */
int
AudioSetPlayState(control, ratio)
    int control;
    double ratio;
/*
 *	DESCRIPTION
 *		Sets the audio device's play state.  Ratio ignored unless
 *		control is LOWER.
 *
 *	RETURNS
 *		1 on success, otherwise 0;
 *
 *	NOTES
 *		Updating actual set values are done via
 *		SIGIO ... see AudioAddControlChangeCallback().
 */
{
    return(0);
}


/*
 *	NAME
 *		AudioSetRecordState
 *
 *	SYNOPSIS
 */
int
AudioSetRecordState(control, ratio)
    int control;
    double ratio;
/*
 *	DESCRIPTION
 *		Sets the audio device's play state.  Ratio ignored unless
 *		control is LOWER.
 *
 *	RETURNS
 *		1 on success, otherwise 0;
 *
 *	NOTES
 *		Updating actual set values are done via
 *		SIGIO ... see AudioAddControlChangeCallback().
 */
{
    return(0);
}


/*
 *	NAME
 *		AudioSetRecordVolume
 *
 *	SYNOPSIS
 */
void
AudioSetRecordVolume(value)
    double value;
/*
 *	DESCRIPTION
 *		Sets the audio device's record volume.
 *		Valid values are any floating point values between 0.0 and 1.0.
 *
 *	RETURNS
 *		The new value used by the audio device.
 *
 *	NOTES
 *		Updating actual value is done via
 *		SIGIO ... see AudioAddControlChangeCallback().
 */
{
}


/*
 *	NAME
 *		AudioControlOpen
 *
 *	SYNOPSIS
 */
int
AudioControlOpen(cbs)
    AudioCallbackStruct	*cbs;
/*
 *	DESCRIPTION
 *		Opens the audio control device for read and write permissions.
 *
 *	RETURNS
 *		-1 on error; otherwise the file descriptor
 *
 */
{
    return(0);
}



/*
 *	NAME
 *		AudioSetMaxGain
 *
 *	SYNOPSIS
 */
int
AudioSetSpeakerOutput()
/*
 *	DESCRIPTION
 *		Attempts to set output to the speaker
 *
 *	RETURNS
 *		1 on success; otherwise 0;
 */
{
    return(0);
}


/*
 *	NAME
 *		AudioSetMaxGain
 *
 *	SYNOPSIS
 */
int
AudioSetHeadphoneOutput()
/*
 *	DESCRIPTION
 *		Close the audio i/o device.
 *
 *	RETURNS
 *		Attempts to set output to the headphone jack
 */
{
    return(0);
}


/*
 *	NAME
 *		AudioAddControlChangeCallback
 *
 *	SYNOPSIS
 */
int
AudioAddControlChangeCallback(which_changes, callback_proc, client_data)
    unsigned which_changes;
    AudioCallbackProc callback_proc;
    void *client_data;
/*
 *	DESCRIPTION
 *		Add a callback for the indicated audio changes.
 *
 *	RETURNS
 *		
 */
{
    return(0);
}

/*
 *
 *	Interfaces for Audio I/O Device Interfaces
 *
 */


/*
 *	NAME
 *		AudioOpen
 *
 *	SYNOPSIS
 */
int
AudioOpen(mode, p_fd)
    int mode; /* PLAY or RECORD */
    int *p_fd;
/*
 *	DESCRIPTION
 *		Opens the audio i/o device for read or write permissions.
 *
 *	RETURNS
 *		AUDIO_SUCCESS	on successful open
 *		AUDIO_BUSY	if audio device busy
 *		AUDIO_ERROR	if error
 */
{
    return(0);
}


/*
 *	NAME
 *		AudioFlushAndClose
 *
 *	SYNOPSIS
 */
void
AudioFlushAndClose(p_fd)
    int *p_fd;
/*
 *	DESCRIPTION
 *		Flush then close the audio i/o device.
 *
 *	RETURNS
 *		void
 */
{
}


/*
 *	NAME
 *		AudioClose
 *
 *	SYNOPSIS
 */
void
AudioClose(p_fd)
    int *p_fd;
/*
 *	DESCRIPTION
 *		Close the audio i/o device.
 *
 *	RETURNS
 *		void
 */
{
}
