/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	nvrAudio.h - audio-specific information
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrAudio.h,v 1.21 1996/01/18 20:50:46 toddb Exp $
 */
#ifndef _NVRAUDIO_H_
#define _NVRAUDIO_H_

#include "fileContent.h"
#include "nvrReason.h"
#include "nvrTimer.h"
#include "nvrString.h"

typedef enum {
    AudioRateUnknown = -1,
    AudioRateUnspecified = 0,
    Audio48Khz     = 48000,
    Audio44_1Khz   = 44100,
    Audio32Khz     = 32000,
    Audio22_254Khz = 22254,
    Audio22_05Khz  = 22050,
    Audio16Khz     = 16000,
    Audio12Khz     = 12000,
    Audio11_025Khz = 11025,
    Audio8Khz      =  8000
} AudioSampleRate;

typedef enum {
    AudioChannelsUnknown = -1,
    AudioChannelsUnspecified = 0,
    AudioMono = 1,
    AudioStereo = 2
} AudioChannelType;

typedef enum {
    AudioDataUnknown = -1,
    AudioDataUnspecified = 0,
    Audio24BitPcm,
    Audio16BitPcm,
    AudioUSign16BitPcm,
    Audio8BitPcm,
    AudioUSign8BitPcm,
    Audio8BitULaw
} AudioDataForm;

typedef enum {
    Avail24BitPcm      = 0x0001,
    Avail16BitPcm      = 0x0002,
    AvailUSign16BitPcm = 0x0004,
    Avail8BitPcm       = 0x0008,
    AvailUSign8BitPcm  = 0x0010,
    Avail8BitULaw      = 0x0020,	/* this implies mono, 8 bit, 8 kHz */
    AvailBitMask       = 0x003f,

    Avail48Khz         = 0x0040,
    Avail44_1Khz       = 0x0080,
    Avail32Khz         = 0x0100,
    Avail22_254Khz     = 0x0200,
    Avail22_05Khz      = 0x0400,
    Avail16Khz         = 0x0800,
    Avail11_025Khz     = 0x1000,
    Avail8Khz          = 0x2000,
    AvailRateMask      = 0x3fc0,

    AvailStereo        = 0x4000,
    AvailMono          = 0x8000,
    AvailChannelMask   = 0xc000
} AvailableAudioFormats;

enum {
    MonoToMono         = 0x00,
    MonoToStereo       = 0x01,
    StereoToStereo     = 0x02,
    StereoToMono       = 0x03,
    ChannelCnvMask     = 0x03,

    DownRate           = 0x00 << 2,
    UpRate             = 0x01 << 2,
    SameRate           = 0x02 << 2,
    RateCnvMask        = 0x03 << 2,
    
    Bits8To16          = 0x00 << 4,
    Bits16To8          = 0x01 << 4,
    Bits16ToULaw       = 0x02 << 4,
    BitsULawTo16       = 0x03 << 4,
    Bits8ToULaw        = 0x04 << 4,
    BitsULawTo8        = 0x05 << 4,
    Bits8To8           = 0x06 << 4,
    Bits16To16         = 0x07 << 4,
    BitsULawToULaw     = 0x08 << 4,
    BitCnvMask         = 0x0f << 4,

    SameSign           = 0x00 << 8,
    SignedToUnsigned   = 0x01 << 8,
    UnsignedToSigned   = 0x02 << 8,
    SignChangeMask     = 0x03 << 8,
    
    NoConversion   = 0xffff,
    CantDoConversion = -1
};

typedef enum {
    DevModeBlocking,
    DevModeNonBlocking
} AudioDevMode;


typedef union _audioDataType {
    long   *pcm24;
    short  *pcm16;
    UShort *upcm16;
    char   *pcm8;
    UChar  *upcm8;
    UChar  *ulaw;
    UChar  *anyType;
} AudioDataType, *AudioDataTypePtr;

typedef struct _audioQuality {
    float decodeQuality;
    long outputFormats;
} AudioQuality, *AudioQualityPtr;

typedef struct {
    AudioSampleRate rate;	/* audio sample rate */
    AudioChannelType channels;	/* # of audio channels */
    AudioDataForm dataForm;	/* audio sample encoding */
} AudioForm, *AudioFormPtr;

typedef struct _audioIO *AudioIOPtr;

typedef struct _audioFileRec {
    AudioForm au;
    AudioIOPtr io;
    long nSamples;				/* # of samples for this file */
    union {
	FileContentType fileType;		/* if this is a file */
	AvailableAudioFormats devFormats;	/* if this is a device */
    } u;
    InternalReasonPtr reason;	/* reason for failure if non-null */
} AudioFileRec, *AudioFilePtr;

typedef struct _audioFileConfig {
    FileContentType type;
    AudioForm au;
} AudioFileConfig, *AudioFileConfigPtr;

typedef struct _audioConversion {
    AudioForm srcAu;
    AudioForm dstAu;
    long convIndex;
    UChar *buffer;
    long bufferLength;
    long maxInSamples;	/* maximum input samples */
    long maxOutSamples;	/* maximum output samples */
    boolean outIsBiggerThanIn;	/* a conversion will increase data size */
    boolean bufferAllocated;	/* true if ->buffer is allocated */

    /*
     * These are changed during the conversion process
     * to reflect the state
     */
    TimerInc counter;
    long skipNextSrcSamples;	/* useful during down-rate conversion */
} AudioConversion, *AudioConversionPtr;

/*
 * External data declarations
 */
extern short DataFormToBytes[];
extern short DataFormToShift[];
extern NameEnumPair AudioRateNames[];
extern NameEnumPair AudioBitNames[];
extern NameEnumPair AudioChannelNames[];
extern NameEnumPair AudioFileTypeNames[];

/*
 * Function interface
 */
CExternal AudioFilePtr OpenAudioFile(
    char *path,
    AudioQualityPtr quality,
    InternalReasonPtr *reasonReturn
);

CExternal AudioFilePtr OpenRawAudioFile(
    char *path,
    AudioFormPtr au,
    InternalReasonPtr *reasonReturn
);

CExternal AudioFilePtr OpenAudioDevice(
    char *path,
    InternalReasonPtr *reasonReturn
);

CExternal void FlushAudio(
    AudioFilePtr
);

CExternal long AudioDelayMs(
    AudioFilePtr
);

CExternal long AudioSamplesQueued(
    AudioFilePtr
);

CExternal AudioFilePtr CreateRawAudioFile(
    char *path,
    AudioFormPtr,
    InternalReasonPtr *reasonReturn
);

CExternal AudioFilePtr CreateAudioFile(
    char *path,
    AudioFileConfigPtr,
    InternalReasonPtr *reasonReturn
);

CExternal boolean InitAudioDevice(
    AudioFilePtr,
    AudioFormPtr au
);

CExternal boolean AudioDeviceMode(
    AudioFilePtr auFile,
    long devCommand
);

CExternal long ReadAudioSamples(
    AudioFilePtr,
    UChar *buffer,
    ULong nSamples
);

CExternal long WriteAudioSamples(
    AudioFilePtr,
    UChar *buffer,
    ULong nSamples
);

CExternal InternalReasonPtr SeekAudioOffset(
    AudioFilePtr,
    ULong nSamples
);

CExternal void CloseAudioFile(
    AudioFilePtr
);

CExternal char *AudioAttrName(
    AudioFormPtr au,
    char *buffer
);

/*
 * This interface is for callers that need the space for an AudioConversion
 * allocated.  A buffer is not allocated.
 */
CExternal AudioConversionPtr CreateAudioConversion(
    AudioFormPtr srcAu,
    AudioFormPtr dstAu,
    InternalReasonPtr *pReason
);

/*
 * This interface is for callers that declare the space for an AudioConversion
 * and need only to initialize it.  A buffer is not allocated.
 */
CExternal InternalReasonPtr InitAudioConversion(
    AudioConversionPtr cvt,
    AudioFormPtr srcAu,
    AudioFormPtr dstAu
);

/*
 * This interface assumes that the samples are already present in
 * cvt->buffer, and converts 'nSamples' samples in place.
 */
CExternal long ConvertAudioBuffer(
    AudioConversionPtr cvt,
    long nSamples,
    InternalReasonPtr *pReason
);

/*
 * This interface is intended for callers that know their input buffer
 * size, and need the values in the AudioConversionPtr initialized
 * correctly.  No space is allocated.  This should be used along
 * with ConvertNewAudioBuffer(), below.  The value returned is
 * the value of cvt->bufferLength;
 */
CExternal long CalcAudioConversionSizes(
    AudioConversionPtr cvt,
    long nSamples
);

/*
 * This interface is a method to specify a new audio buffer and
 * perform the conversion at the same time.  It returns the
 * number of samples in the buffer after the conversion.
 */
CExternal long ConvertNewAudioBuffer(
    AudioConversionPtr cvt,
    UChar *buffer,
    long nSamples,
    InternalReasonPtr *pReason
);

CExternal InternalReasonPtr ChooseAudioForm(
    AudioFormPtr srcAu,
    long formats,
    AudioFormPtr dstAu
);

CExternal ULong AudioRateToAvailBit(
    AudioSampleRate rate
);

CExternal InternalReasonPtr AllocMaxAudioBuffer(
    AudioConversionPtr cvt,
    long usec
);

/*
 * Platform independent audio UI
 */
CExternal boolean SaveAudioPreview(
    AudioFilePtr input,
    char *previewFile
);

CExternal InternalReasonPtr PlayAudio(
    AudioFilePtr input,
    boolean (*callback)(PrivPointer, long nSamples),
    PrivPointer data
);

#endif /* _NVRAUDIO_H_ */
