/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 *      All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *      nvrAudioI.h - private declarations for audio library
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrAudioI.h,v 1.16 1996/01/18 20:50:46 toddb Exp $
 */
#ifndef _NVRAUDIOI_H_
#define _NVRAUDIOI_H_

#include "osio.h"
#include "nvrAudio.h"

#define SetAudioReason(auFile, _reason)					\
    {									\
	if ((auFile)->reason && (auFile)->reason->hint == IR_Warning) {	\
	    FreeReason((auFile)->reason);				\
	    (auFile)->reason = NULL;					\
	}								\
	if ((auFile)->reason == NULL) {					\
	    (auFile)->reason = _reason;					\
	}								\
    }

#define SetAudioWarning(auFile, _reason)				\
    {									\
	if ((auFile)->reason == NULL) {					\
	    (auFile)->reason = _reason;					\
	    (auFile)->reason->hint = IR_Warning;			\
	}								\
    }

typedef struct _audioDevOpFuncs {
    boolean (*openDev)     (AudioFilePtr);
    boolean (*initDev)     (AudioFilePtr);
    boolean (*modeDev)     (AudioFilePtr, long devCommand);
    long    (*delayMs)     (AudioFilePtr);
    long    (*readSamples) (AudioFilePtr, UChar *buffer, ULong sampleCount);
    long    (*writeSamples)(AudioFilePtr, UChar *buffer, ULong sampleCount);
    long    (*samplesQueued)(AudioFilePtr);
    void    (*flushSamples)(AudioFilePtr);
    void    (*closeDev)    (AudioFilePtr);
} AudioDevOpFunc, *AudioDevOpFuncPtr;

typedef struct _audioOpFuncs {
    boolean (*openFile)    (AudioFilePtr, AudioQualityPtr);
    boolean (*createFile)  (AudioFilePtr);
    long    (*readSamples) (AudioFilePtr, UChar *buffer, ULong sampleCount);
    long    (*writeSamples)(AudioFilePtr, UChar *buffer, ULong sampleCount);
    InternalReasonPtr
	    (*seekSamples) (AudioFilePtr, ULong sampleOffset);
    void    (*closeFile)   (AudioFilePtr);
} AudioOpFunc, *AudioOpFuncPtr;

typedef enum {
    AudioFileOutput,
    AudioFileInput,
    AudioDeviceOutput,
    AudioDeviceInput,
    AudioCddaInput
} AudioIoOpClass;

typedef struct _cddaTime {
    UChar min;
    UChar sec;
    UChar frame;
    UChar sample;
} CddaTime, *CddaTimePtr;

/*
 * This structure is used to represent a CDDA track address and length.
 */
typedef struct _cddaDesc {
    int track;		/* track number */
    CddaTime start;	/* start address, relative to track number */
    CddaTime length;	/* length of the selection */
    int device;		/* cd device number */
} CddaDesc, *CddaDescPtr;

struct _audioIO {
    BufFileHndl fp;		/* buffered io file pointer */
    long hdrSize;		/* used by StdAudioSeekSamples() */
    boolean input;		/* true if we are reading this file */
    char *name;			/* private copy of the name of the file */
    AudioIoOpClass ioClass;	/* what we do with this thing */
    union {
	AudioOpFunc fops;	/* file operation function pointers */
	AudioDevOpFunc dops;	/* device operation function pointers */
    } u;
    CddaDescPtr cdda;	/* optional cdda info; if ioClass==AudioCddaInput */
    PrivPointer state;	/* data private to each file/device type */
};

typedef struct _audioIO AudioIORec;

extern AudioOpFunc AudioLibAiffFuncs;
extern AudioOpFunc AudioLibAuFuncs;
extern AudioOpFunc AudioLibJmovieFuncs;
extern AudioOpFunc AudioLibMpegFuncs;
extern AudioOpFunc AudioLibRawFuncs;
extern AudioOpFunc AudioLibQTFuncs;
extern AudioOpFunc AudioLibCddaFuncs;
extern InternalReason CantSeekAudioDevice;

CExternal boolean AllocAudioState(
    AudioFilePtr,
    long nBytes
);

CExternal AudioFilePtr AllocAudioFile(
    char *path,
    AudioIoOpClass ioClass
);

CExternal long StdAudioReadSamples(
    AudioFilePtr auFile,
    UChar *buffer,
    ULong nSamples
);

CExternal long StdAudioWriteSamples(
    AudioFilePtr auFile,
    UChar *buffer,
    ULong nSamples
);

CExternal InternalReasonPtr StdAudioSeekSamples(
    AudioFilePtr auFile,
    ULong nSamples
);

CExternal void ConvertToIeeeExtended(
    double d,
    char ps[10]
);

#endif /* _NVRAUDIOI_H_ */

