/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	rawio.c - functions for reading/writing raw audio
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/rawio.c,v 1.4 1994/10/02 17:59:01 toddb Exp $
 */
#define NeedFunctionPrototypes 1

#include "nvrAudioI.h"
#include "nvrAlloc.h"
#include "fileAccess.h"

static InternalReason CantOpen =
    { "Can't open input file\n", IR_BadIOFile };
static InternalReason CantCreate =
    { "Can't create output file\n", IR_BadIOFile };

static void
CloseRawAudio(AudioFilePtr auFile)
{
}

static boolean
CreateRawAudio(AudioFilePtr auFile)
{
    return(True);
}

static boolean
OpenRawAudio(AudioFilePtr auFile, AudioQualityPtr quality)
{
    return(True);
}

AudioOpFunc AudioLibRawFuncs = {
    OpenRawAudio,
    CreateRawAudio,
    StdAudioReadSamples,
    StdAudioWriteSamples,
    StdAudioSeekSamples,
    CloseRawAudio
};

AudioFilePtr
OpenRawAudioFile(char *path,
                 AudioFormPtr au,
                 InternalReasonPtr *reasonReturn)
{
    AudioFilePtr auFile;
    BufFileHndl fp;

    if (! FileReadable(path)) {
        *reasonReturn = &CantOpen;
        return(NULL);
    }

    if (path)
	fp = OpenBufferedFileForReading(path);
    else
	fp = GetBufferedIn();
    if (fp == NULL) {
        *reasonReturn = &CantOpen;
        return(NULL);
    }

    /*
     * Allocate storage
     */
    if ((auFile = AllocAudioFile(path, AudioFileInput)) == NULL) {
        *reasonReturn = &IRBadAllocation;
        CloseBufferedFile(fp);
        return(auFile);
    }
    auFile->u.fileType = FT_Unknown;
    auFile->io->input = True;
    auFile->io->u.fops = AudioLibRawFuncs;
    auFile->io->fp = fp;
    auFile->au = *au;
    auFile->nSamples = FileSize(path) >> DataFormToShift[ auFile->au.dataForm ];

    return(auFile);
}

AudioFilePtr
CreateRawAudioFile(char *path,
                   AudioFormPtr au,
                   InternalReasonPtr *reasonReturn)
{
    BufFileHndl fp;
    AudioFilePtr auFile;

    if (path)
	fp = OpenBufferedFileForWriting(path);
    else
	fp = GetBufferedOut();
    if (fp == NULL) {
        *reasonReturn = &CantCreate;
        return(NULL);
    }

    /*
     * Allocate storage
     */
    if ((auFile = AllocAudioFile(path, AudioFileOutput)) == NULL) {
        *reasonReturn = &IRBadAllocation;
        CloseBufferedFile(fp);
        UnlinkPath(path, False);
        return(auFile);
    }
    auFile->u.fileType = FT_Unknown;
    auFile->au = *au;
    auFile->io->input = False;
    auFile->io->u.fops = AudioLibRawFuncs;
    auFile->io->fp = fp;

    return(auFile);
}

