/*
 * (c) Copyright 1992 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	sunaudio.h
 *
 * DESCRIPTION
 *	defines for use with sunaudio.c
 *	OS-DEPENDENT
 *
 * REVISION
 *	$Header: /usr/local/src/nvrToMIT/RCS/sgiaudio.h,v 1.1 1994/02/19 00:19:29 toddb Exp $
 */
#ifndef _SGIAUDIO_H_
#define _SGIAUDIO_H_

#include "nvrTypes.h"

#ifdef sgi
typedef long audio_info_t;
#define	AUDIO_SUCCESS		(0)		/* no error */
#define	AUDIO_UNIXERROR		(-1)		/* check errno for error code */
#define AUDIO_BUSY	1
#define AUDIO_ERROR	-1
#include <audio.h>
#else
#define	AUDIO_SPEAKER	1	/* output to built-in speaker */
#define	AUDIO_HEADPHONE	2	/* output to headphone jack */

/*
 * to fake out routines until we abstract this
 */

#define	AUDIO_ERR_BADHDR	(1)		/* bad audio header structure */
#define	AUDIO_ERR_BADFILEHDR	(2)		/* bad file header format */
#define	AUDIO_ERR_BADARG	(3)		/* bad subroutine argument */
#define	AUDIO_ERR_NOEFFECT	(4)		/* device control ignored */
#define	AUDIO_ERR_ENCODING	(5)		/* unknown encoding format */
#define	AUDIO_ERR_INTERRUPTED	(6)		/* operation was interrupted */

#endif /* sgi */

/*
 *	DEFINES
 */
#define PLAY	1
#define RECORD	2

#define MUTE	0
#define RESUME	1
#define LOWER	2
#define PAUSE	3
#define FLUSH	4

/* #define AUDIO_SUCCESS 0 ... defined in audio_errno.h */

#define AUDIO_CHANGED_PLAY_GAIN		(1<<0)
#define AUDIO_CHANGED_PLAY_PAUSE	(1<<1)
#define AUDIO_CHANGED_PLAY_PORT		(1<<2)
#define AUDIO_CHANGED_RECORD_GAIN	(1<<3)
#define AUDIO_CHANGED_MONITOR_GAIN	(1<<4)

/*
 *	TYPEDEFS
 */
typedef void (*AudioCallbackProc)(
#ifdef NeedFunctionPrototypes
    void *	/* client_data */,
    void *	/* call_data */
#endif
);

typedef struct {
    unsigned		changes;
    double		play_gain;
    boolean		play_pause;
    unsigned		play_port;
    double		record_gain;
    double		monitor_gain;
} AudioCallbackStruct;

/*
 *	EXTERNS
 */
CExternal int AudioControlOpen(
#if NeedFunctionPrototypes
    AudioCallbackStruct *
#endif
);
CExternal int AudioOpen(
#if NeedFunctionPrototypes
    int,
    int *
#endif
);
CExternal void AudioFlushAndClose(
#if NeedFunctionPrototypes
    int *
#endif
);
CExternal void AudioClose(
#if NeedFunctionPrototypes
    int *
#endif
);
CExternal int AudioGetState(
#if NeedFunctionPrototypes
    audio_info_t *
#endif
);
CExternal void AudioSetPlayVolume(
#if NeedFunctionPrototypes
    double
#endif
);
CExternal int AudioSetPlayState(
#if NeedFunctionPrototypes
    int,
    double
#endif
);
CExternal int AudioSetRecordState(
#if NeedFunctionPrototypes
    int,
    double
#endif
);
CExternal void AudioSetMonitorVolume(
#if NeedFunctionPrototypes
    double
#endif
);
CExternal int AudioSetMonitorState(
#if NeedFunctionPrototypes
    int,
    double
#endif
);
CExternal void AudioSetRecordVolume(
#if NeedFunctionPrototypes
    double
#endif
);
CExternal int AudioAddControlChangeCallback(
#if NeedFunctionPrototypes
    unsigned,
    AudioCallbackProc,
    void *
#endif
);
CExternal int AudioSetSpeakerOutput(
#if NeedFunctionPrototypes
    void
#endif
);
CExternal int AudioSetHeadphoneOutput(
#if NeedFunctionPrototypes
    void
#endif
);

#endif /* _SGIAUDIO_H_ */
