/*
 * Copyright (c) 1992, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	sunaudio.c
 *
 * DESCRIPTION
 *	Programming interface to SparcStation's audio device.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/sgicontrol.c,v 1.1 1994/02/19 00:19:29 toddb Exp $
 */
#ifndef LINT
static char *copy_notice = "Copyright 1992 North Valley Research, Inc.";
#endif /* LINT */

/*
 *      EXTERNAL INCLUDES
 *              Include files that must be exported to any package or
 *              program using this package.
 */
#include "sgiaudio.h"


/*
 *      INTERNAL INCLUDES
 *              Include files for local use only, therefore, NOT exported
 *		to any package or program using this package.
 */
#include <errno.h>
#include <fcntl.h>
#include <math.h>
#include <audio.h>

#include "nvrTypes.h"
typedef long Audio_hdr;
typedef long Audio_info;

/*
 *      EXTERNS
 *              External declarations required locally to this package
 *              that are not already declared in any of the included header
 *		files (external includes or internal includes).
 */


/*
 *      GLOBALS
 *              Variables declared in this package that are allowed
 *		to be used globally.
 */


/*
 *      LOCAL DEFINES
 *		#define declarations local to this package.
 */
#define MIN(x,y) ((x) > (y) ? (y) : (x))
#define GAIN_TO_FLOAT(value)						\
	((double)(value) / (double)AUDIO_MAX_GAIN)

#define ERROR(number) { return -1; }

/*
 *      LOCAL TYPEDEFS
 *              typedefs local to this package (for use with local vars).
 *
 */

typedef struct _AudioCallbackRec {
    unsigned		which_changes;
    AudioCallbackProc	callback_proc;
    void *		client_data;
    struct _AudioCallbackRec *next;
} AudioCallbackRec, *AudioCallbackList;


/*
 *      LOCALS VARIABLES
 *		Variables local to this package.
 */

static int		AudioFd = -1;
static int		AudioCtlFd = -1;
static int		AudioFdMode = 0;
static audio_info_t	AudioState;
static Audio_hdr	AudioPlayConfig;
static Audio_hdr	AudioRecordConfig;
static AudioCallbackList AudioCallbacks = (AudioCallbackList)NULL;



/************************************************************************
 *									*
 *			PRIVATE ROUTINES				*
 *									*
 *	Routines private to this package therefore declared static	*
 *									*
 ************************************************************************/

/*
 *	NAME
 *		HandleControlChanges
 *
 *	SYNOPSIS
 */
static void
HandleControlChanges()
/*
 *	DESCRIPTION
 *		Add a callback for the indicated audio changes.
 *
 *	RETURNS
 *		
 */
{
}

static ALerrfunc originalErrorHandler;

/*_____________________________________________________________________________
 *
 * silentALerror();
 *
 *____________________________________________________________________________
 */

static
void silentALerror(long errnum,     const char* fmt, ...)
{
/*     printf ("silent error\n"); */
}


/************************************************************************
 *									*
 *			PUBLIC ROUTINES					*
 *									*
 *		Publically accessible routines				*
 *									*
 ************************************************************************/

/*
 *
 *	Interfaces for Audio CONTROL Device Interfaces
 *
 */



/*
 *	NAME
 *		AudioGetState
 *
 *	SYNOPSIS
 */
int
AudioGetState(ap)
    audio_info_t	*ap;
/*
 *	DESCRIPTION
 *		Obtains the current control parameters of the audio device.
 *
 *	RETURNS
 *		0 on error; otherwise non-zero.
 *
 */
{
    return(0);
}


/*
 *	NAME
 *		AudioSetMonitorVolume
 *
 *	SYNOPSIS
 */
void
AudioSetMonitorVolume(value)
    double value;
/*
 *	DESCRIPTION
 *		Sets the audio device's monitor volume.
 *		Valid values are any floating point values between 0.0 and 1.0.
 *
 *	RETURNS
 *		The new value used by the audio device.
 *
 *	NOTES
 *		Updating actual value is done via
 *		SIGIO ... see AudioAddControlChangeCallback().
 */
{
}


/*
 *	NAME
 *		AudioSetMonitorState
 *
 *	SYNOPSIS
 */
int
AudioSetMonitorState(control, ratio)
    int control;
    double ratio;
/*
 *	DESCRIPTION
 *		Sets the audio device's monitor volume
 *
 *	RETURNS
 *		1 on success, otherwise 0;
 *
 *	NOTES
 *		Updating actual set values are done via
 *		SIGIO ... see AudioAddControlChangeCallback().
 */
{
    return(0);
}


/*
 *	NAME
 *		AudioSetPlayVolume
 *
 *	SYNOPSIS
 */
void
AudioSetPlayVolume(value)
    double value;
/*
 *	DESCRIPTION
 *		Sets the audio device's play volume
 *		Valid values are any floating point values between 0.0 and 1.0.
 *
 *	RETURNS
 *		The new value used by the audio device.
 *
 *	NOTES
 *		Updating actual value is done via
 *		SIGIO ... see AudioAddControlChangeCallback().
 */
{
    long buf[4];
    int newVol;

    newVol = 100 * value;

    if (newVol < 0 || newVol > 99) return;

    buf[0] = AL_LEFT_SPEAKER_GAIN;
    buf[2] = AL_RIGHT_SPEAKER_GAIN;

    originalErrorHandler = ALseterrorhandler(silentALerror);
    ALgetparams(AL_DEFAULT_DEVICE, buf,4);
    ALseterrorhandler(originalErrorHandler);

    if (buf[1] > 255 ||
	buf[3] > 255 ||
	buf[1] < 0 ||
	buf[3] < 0) {

	buf[1] = buf[3] = 0;
    }

    buf[1] = buf[3] = newVol*255/100;

    printf ("vol set to: %d\n",buf[1]);

    originalErrorHandler = ALseterrorhandler(silentALerror);
    ALsetparams(AL_DEFAULT_DEVICE, buf,4);
    ALseterrorhandler(originalErrorHandler);
}


/*
 *	NAME
 *		AudioSetPlayState
 *
 *	SYNOPSIS
 */
int
AudioSetPlayState(control, ratio)
    int control;
    double ratio;
/*
 *	DESCRIPTION
 *		Sets the audio device's play state.  Ratio ignored unless
 *		control is LOWER.
 *
 *	RETURNS
 *		1 on success, otherwise 0;
 *
 *	NOTES
 *		Updating actual set values are done via
 *		SIGIO ... see AudioAddControlChangeCallback().
 */
{
    return(1);
}


/*
 *	NAME
 *		AudioSetRecordState
 *
 *	SYNOPSIS
 */
int
AudioSetRecordState(control, ratio)
    int control;
    double ratio;
/*
 *	DESCRIPTION
 *		Sets the audio device's record state.  Ratio ignored unless
 *		control is LOWER.
 *
 *	RETURNS
 *		1 on success, otherwise 0;
 *
 *	NOTES
 *		Updating actual set values are done via
 *		SIGIO ... see AudioAddControlChangeCallback().
 */
{
    return(1);
}


/*
 *	NAME
 *		AudioSetRecordVolume
 *
 *	SYNOPSIS
 */
void
AudioSetRecordVolume(value)
    double value;
/*
 *	DESCRIPTION
 *		Sets the audio device's record volume.
 *		Valid values are any floating point values between 0.0 and 1.0.
 *
 *	RETURNS
 *		The new value used by the audio device.
 *
 *	NOTES
 *		Updating actual value is done via
 *		SIGIO ... see AudioAddControlChangeCallback().
 */
{
}


/*
 *	NAME
 *		AudioControlOpen
 *
 *	SYNOPSIS
 */
int
AudioControlOpen(cbs)
    AudioCallbackStruct	*cbs;
/*
 *	DESCRIPTION
 *		Opens the audio control device for read and write permissions.
 *
 *	RETURNS
 *		-1 on error; otherwise the file descriptor
 *
 */
{
    return(-1);
}



/*
 *	NAME
 *		AudioSetMaxGain
 *
 *	SYNOPSIS
 */
int
AudioSetSpeakerOutput()
/*
 *	DESCRIPTION
 *		Attempts to set output to the speaker
 *
 *	RETURNS
 *		1 on success; otherwise 0;
 */
{
    return(1);
}


/*
 *	NAME
 *		AudioSetMaxGain
 *
 *	SYNOPSIS
 */
int
AudioSetHeadphoneOutput()
/*
 *	DESCRIPTION
 *		Close the audio i/o device.
 *
 *	RETURNS
 *		Attempts to set output to the headphone jack
 */
{
    return(1);
}


/*
 *	NAME
 *		AudioAddControlChangeCallback
 *
 *	SYNOPSIS
 */
int
AudioAddControlChangeCallback(which_changes, callback_proc, client_data)
    unsigned which_changes;
    AudioCallbackProc callback_proc;
    void *client_data;
/*
 *	DESCRIPTION
 *		Add a callback for the indicated audio changes.
 *
 *	RETURNS
 *		
 */
{
    return(0);
}

/*
 *
 *	Interfaces for Audio I/O Device Interfaces
 *
 */


static ALport aud_port;

/*
 *	NAME
 *		AudioOpen
 *
 *	SYNOPSIS
 */
int
AudioOpen(mode, p_fd)
    int mode; /* PLAY or RECORD */
    int *p_fd;
/*
 *	DESCRIPTION
 *		Opens the audio i/o device for read or write permissions.
 *
 *	RETURNS
 *		AUDIO_SUCCESS	on successful open
 *		AUDIO_BUSY	if audio device busy
 *		AUDIO_ERROR	if error
 */
{
    ALconfig aconfig;
    long pvbuf[2];
    long buflen;
    char portname[21];
    ALerrfunc olderrfunc;
    int rate = 8000;

    olderrfunc = ALseterrorhandler(NULL);

    /*
     * configure audio port
     */
    if (( aconfig = ALnewconfig() ) == NULL ) {
	ALseterrorhandler(olderrfunc);
	ERROR( oserror() );
    }
    if ( ALsetwidth(aconfig, AL_SAMPLE_8) < 0) {
	ALseterrorhandler(olderrfunc);
	ERROR( oserror() );
    }
    if ( ALsetchannels(aconfig, 1) < 0) {
	ALseterrorhandler(olderrfunc);
	ERROR( oserror() );
    }
    if ( ALsetsampfmt(aconfig, AL_SAMPFMT_TWOSCOMP) < 0) {
	ALseterrorhandler(olderrfunc);
	ERROR( oserror() );
    }
    
    /*
    ** This queue size stuff is a hack.  The audio library has
    ** minimum and maximum queue sizes.  Because we use the audio
    ** queue to time the presentation of images in the image track
    ** and because we don't use fill points, the timing of the image
    ** thread will be off if the number of audio samples per frame
    ** is very small or very large.
    */
    
    {
	int queueSize = 267 + 2;
	if ( queueSize < 510 ) {
	    queueSize = 510;
	}
	if ( queueSize > 131000) {
	    queueSize = 131000;
	}
	if ( ALsetqueuesize( aconfig, queueSize ) < 0 ) {
	    ALseterrorhandler(olderrfunc);
	    ERROR( oserror() );
	}
    }
    
    /*
     * cook up a name for this audio port, using the process id
     * and the group ID.
     * 
     * XXX will fail if combined number of digits in process ID and
     * timer ID are greater than 15
     */
    sprintf(portname, "mpeg%d:%d", (int)getpid(), (int)getgid());

    /*
     * attempt to open audio port.
     */
    if ((aud_port = ALopenport(portname, "w", aconfig)) == 0) {
	ALseterrorhandler(olderrfunc);
	ERROR( oserror() );
    }
    ALfreeconfig(aconfig);
 
    /*
     * initialize output sample rate, if no other process owns the audio
     * We need to make sure that the rate we give is one of the
     * supported rates.  The breaking points are the geometric means
     * between adjacent rates.
     */

    if      ( rate > 46009 ) rate = 48000;
    else if ( rate > 37566 ) rate = 44100;
    else if ( rate > 26563 ) rate = 32000;
    else if ( rate > 18783 ) rate = 22050;
    else if ( rate > 13282 ) rate = 16000;
    else if ( rate > 9391 )  rate = 11025;
    else                     rate =  8000;

    pvbuf[0] = AL_OUTPUT_RATE;
    pvbuf[1] = rate;
    buflen = 2;
    if ( ALsetparams(AL_DEFAULT_DEVICE, pvbuf, buflen) < 0 ) {
	ALseterrorhandler(olderrfunc);
	ERROR( oserror() );
    }

    ALsetfillpoint(aud_port, 0);

    ALseterrorhandler(olderrfunc);
    return AUDIO_SUCCESS;
}


/*
 *	NAME
 *		AudioFlushAndClose
 *
 *	SYNOPSIS
 */
void
AudioFlushAndClose(p_fd)
    int *p_fd;
/*
 *	DESCRIPTION
 *		Flush then close the audio i/o device.
 *
 *	RETURNS
 *		void
 */
{
    ALcloseport(aud_port);
}


/*
 *	NAME
 *		AudioClose
 *
 *	SYNOPSIS
 */
void
AudioClose(p_fd)
    int *p_fd;
/*
 *	DESCRIPTION
 *		Close the audio i/o device.
 *
 *	RETURNS
 *		void
 */
{
    ALcloseport(aud_port);
}
