/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	sgidevice.c - sgi functions for playing audio
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/sgidevice.c,v 1.12 1996/01/18 20:50:46 toddb Exp $
 *
 */
#include 	"nvrAudioI.h"

#include	<errno.h>
#include	<fcntl.h>
#include 	<audio.h>
#include 	<audiofile.h>
#include 	<sys/schedctl.h>
#include 	<stdio.h>
#include 	<unistd.h>
#include 	<math.h>
 
/* State Variables */

typedef struct _sgiState {
    boolean rude;
    boolean blocking;
    long delayMs;
    ALport fp;
} SgiDevState, *SgiDevStatePtr;

static InternalReason CantRead = {
    "Can't read sgi device audio yet.\n", IR_NotImplemented
};

static int
GetInputRate(void)
{
    long buf[6];

    buf[0] = AL_INPUT_RATE;
    buf[2] = AL_INPUT_SOURCE;
    buf[4] = AL_DIGITAL_INPUT_RATE;
    ALgetparams(AL_DEFAULT_DEVICE, buf, 6);

    /* we are clocked off digital input. find real input rate, if possible. */
    if	((buf[1] == AL_RATE_AES_1)||(buf[3] == AL_INPUT_DIGITAL)) {
	if (ALgetdefault(AL_DEFAULT_DEVICE, AL_DIGITAL_INPUT_RATE) >= 0) 
	    return (buf[5]);
    }
    /* input rate is Hz and we're using an analog input */
    else if (buf[1] > 0) 
	return (buf[1]);

    return (AL_RATE_UNDEFINED);
}

static int
GetOutputRate(void)
{
    long buf[4];

    buf[0] = AL_OUTPUT_RATE;
    buf[2] = AL_DIGITAL_INPUT_RATE;
    ALgetparams(AL_DEFAULT_DEVICE, buf, 4);

    /* output rate is in Hz -- return it */
    if (buf[1] > 0) 
	return (buf[1]);

    /* output rate is logical -- track down what it means */
    if (buf[1] == AL_RATE_AES_1) {
	/* we are clocked off of digital input. find
	 * real input rate, if system supports this ability.
	 * Otherwise, return AL_RATE_UNDEFINED */
	if (ALgetdefault(AL_DEFAULT_DEVICE,AL_DIGITAL_INPUT_RATE) >= 0) 
	    return (buf[3]);
    } else if (buf[1] == AL_RATE_INPUTRATE) 
	return (GetInputRate());
    return (AL_RATE_UNDEFINED);
}

static boolean
sgiAudioOpen(AudioFilePtr auFile)
{
    SgiDevStatePtr sgiState;

    if (! AllocAudioState(auFile, sizeof(SgiDevState)) )
        return(False);
    sgiState = (SgiDevStatePtr) auFile->io->state;
    sgiState->rude = True;

    /*
     * Derived from running tools/calaudio/calaudio on 100mhz indy
     * XXX this may be different for different platforms.
     */
    sgiState->delayMs = 55;

    /*
     * We can do just about anything except Avail8BitULaw
     */
    auFile->u.devFormats = Avail24BitPcm
			 | Avail16BitPcm
			 | Avail8BitPcm
			 | Avail48Khz
			 | Avail44_1Khz
			 | Avail32Khz
			 | Avail22_05Khz
			 | Avail16Khz
			 | Avail11_025Khz
			 | Avail8Khz
			 | AvailStereo
			 | AvailMono;
    return(True);
}

static void
sgiAudioClose(AudioFilePtr auFile)
{
    SgiDevStatePtr sgiState = (SgiDevStatePtr) auFile->io->state;

    if (sgiState->fp) {
	while (ALgetfilled(sgiState->fp))
	    sginap(1);
	ALcloseport(sgiState->fp);
	sgiState->fp = NULL;
    }
}

static long
sgiAudioSamplesQueued(AudioFilePtr auFile)
{
    SgiDevStatePtr sgiState = (SgiDevStatePtr) auFile->io->state;
    return( ALgetfilled(sgiState->fp) );
}

static void
sgiAudioFlush(AudioFilePtr auFile)
{
    /* not implemented */
}

static long
sgiAudioDelayMs(AudioFilePtr auFile)
{
    SgiDevStatePtr sgiState = (SgiDevStatePtr) auFile->io->state;

    return(sgiState->delayMs);
}

static boolean
sgiAudioMode(AudioFilePtr auFile, long devCommand)
{
    SgiDevStatePtr sgiState = (SgiDevStatePtr) auFile->io->state;

    switch(devCommand) {
    case DevModeNonBlocking:
        sgiState->blocking = False;
        break;
    case DevModeBlocking:
        sgiState->blocking = True;
        break;
    default:
        return(False);
    }
    return(True);
}

static boolean
sgiAudioInit(AudioFilePtr auFile)
{
    SgiDevStatePtr sgiState = (SgiDevStatePtr) auFile->io->state;
    ALconfig audioDevFp_config;
    long pvbuf[4];
    long devOutputRate;
    int err;
    int	sampleWordsize;    	/* size of sample (AL_SAMPLE_*) */

    if (sgiState->fp) {
	while (ALgetfilled(sgiState->fp))
	    sginap(1);
    }
    /*
     * need to determine whether audio is in use. if not, then we
     * can just go ahead and be "rude."
     */
    pvbuf[0] = AL_OUTPUT_COUNT;
    pvbuf[2] = AL_MONITOR_CTL;
    if (ALgetparams(AL_DEFAULT_DEVICE, pvbuf, 4) < 0) {
	if (oserror() == AL_BAD_DEVICE_ACCESS) {
	    ProgMsg("Can't play -- could not access audio hardware.\n");
	    return(False);
	}
    }
    
    
    if ((pvbuf[1] == 0) && (pvbuf[3] == AL_MONITOR_OFF)) {
        sgiState->rude = True;
    }
    
    /* 
     * determine the output rate for the audio hardware
     */
    devOutputRate = GetOutputRate();
    
    /*
     * if the rates are the same, all is well. if not, then we need to proceed
     * in a either a "rude" or "polite" manner.
     */
    
    if (devOutputRate != auFile->au.rate)
    {
        /*
         *  we set the rate to our value in Hz.  ALsetparams will set
	 *  the hardware to the closest rate to auFile->au.rate which the
	 *  hardware supports.
         */
	
        if (sgiState->rude)
        {
            devOutputRate = auFile->au.rate;
            pvbuf[0] = AL_OUTPUT_RATE;
            pvbuf[1] = devOutputRate;
            ALsetparams(AL_DEFAULT_DEVICE, pvbuf, 2);
	    if( GetOutputRate() != devOutputRate ) {
		ProgMsg("Could not set audio device output rate to %d\n",
			devOutputRate);
		return(False);
	    }
        }
        else                    /*polite*/
        {
	    return(False);
        }
    }

    /*
     * configure and open audio port
     */
    audioDevFp_config = ALnewconfig();

    /*
     * From the man page: "ALsetconfig(3dm) will not change the channel
     * setting.  You can't change the channel setting of an ALport after
     * it has been opened."  Therefore, we must close it.
     */
    if (auFile->au.channels != ALgetchannels(audioDevFp_config)
     && sgiState->fp) {
	ALcloseport(sgiState->fp);
	sgiState->fp = NULL;
    }

    switch (auFile->au.dataForm) {
    case Audio24BitPcm:	sampleWordsize = AL_SAMPLE_24; break;
    case Audio16BitPcm:	sampleWordsize = AL_SAMPLE_16; break;
    case Audio8BitPcm:	sampleWordsize = AL_SAMPLE_8; break;
    default:
	return(False);
    }

    ALsetwidth(audioDevFp_config, sampleWordsize);
    ALsetchannels(audioDevFp_config, auFile->au.channels);

    /*
     * make the ring buffer large enough to hold 1 sec of audio samples
     */
    ALsetqueuesize(audioDevFp_config,
		   auFile->au.rate * auFile->au.channels);
    if (sgiState->fp == NULL)
	sgiState->fp = ALopenport("audiolib", "w", audioDevFp_config);
    if (sgiState->fp == NULL) {
	err = oserror();
	if (err == AL_BAD_NO_PORTS) {
	    ProgMsg("Can't play -- no audio ports available at the moment\n");
	} else if (err == AL_BAD_OUT_OF_MEM) {
	    ProgMsg("Can't play -- not enough memory to open audio port\n");
	}
	return(False);
    }
    return(True);
}

static long
sgiAudioReadSamples(AudioFilePtr auFile, UChar *buffer, ULong nSamples)
{
    SetAudioReason(auFile, &CantRead);
    return(-1);
}

static long
sgiAudioWriteSamples(AudioFilePtr auFile, UChar *buffer, ULong nSamples)
{
    SgiDevStatePtr sgiState = (SgiDevStatePtr) auFile->io->state;
    int fillable, remaining, dataShift;
    dataShift = DataFormToShift[ auFile->au.dataForm ];

    remaining = nSamples;
    while (remaining) {
        fillable = ALgetfillable(sgiState->fp);
        if (fillable > remaining) {
            fillable = remaining;
	} else if (fillable < remaining) {
            if (! sgiState->blocking)
                return(0);
            sginap(1);
            continue;
        }
        if (ALwritesamps(sgiState->fp, (void *)buffer, fillable) < 0)
	    Msg("writesamps < 0\n");
        buffer += fillable << dataShift;
        remaining -= fillable;
    }

    return(nSamples - remaining);
}


/*		External Linkage				*/

AudioDevOpFunc SgiDevFuncs = {
    sgiAudioOpen,
    sgiAudioInit,
    sgiAudioMode,
    sgiAudioDelayMs,
    sgiAudioReadSamples,
    sgiAudioWriteSamples,
    sgiAudioSamplesQueued,
    sgiAudioFlush,
    sgiAudioClose
};
