/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	sunaifc.c - sun functions for playing Aifc files
 *		    Actually this will only read Aiff files at present
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/sundevice.c,v 1.11 1996/01/18 21:04:18 toddb Exp $
 */
#define NeedFunctionPrototypes 1

#include	<errno.h>
#include	<fcntl.h>
#include        <unistd.h>
#include        <sys/ioccom.h>
#include        <sys/filio.h>
#ifdef Solaris2
# include       <sys/audioio.h>
#else
# include       <sun/audioio.h>
#endif
#include 	"SunAudioStruct.h"
#include 	"nvrAudioI.h"
#include 	"mu-law.h"
 

#define AUDIODEV		"/dev/audio"

typedef struct _sunDevState {
    int fd;
    long delayMs;
} SunDevState, *SunDevStatePtr;

static InternalReason CantRead = {
    "Can't read sun device audio yet.\n", IR_NotImplemented
};
InternalReason IRCantOpen =
    { "can't open sun audio device\n", IR_CantWrite };

/*
 * Function Prototypes
 */

static boolean
sunOpenDevice(AudioFilePtr auFile)
{
    SunDevStatePtr sunState = (SunDevStatePtr) auFile->io->state;
    int fd;

    if (sunState->fd >= 0)
	return(True);
    if( (fd = open(auFile->io->name, O_WRONLY | O_NDELAY)) < 0 ) {
	SetAudioReason(auFile,
	    CreateSysReason("Sun audio: %s: open failed\n", auFile->io->name));
	return(False);
    }
    sunState->fd = fd;
    return(True);
}

static void
sunAudioClose(AudioFilePtr auFile)
{
    SunDevStatePtr sunState = (SunDevStatePtr) auFile->io->state;

    if (sunState->fd > -1) {
	close(sunState->fd);
	sunState->fd = -1; 
    }
}

static long
sunAudioSamplesQueued(AudioFilePtr auFile)
{
    return(0);	/* don't know how to do this yet */
}

static void
sunAudioFlush(AudioFilePtr auFile)
{
    /* not implemented */
}

static boolean
sunAudioOpen(AudioFilePtr auFile)
{
    audio_info_t devData, oldData;
    SunDevStatePtr sunState;

    if (! AllocAudioState(auFile, sizeof(SunDevState)) )
        return(False);
    sunState = (SunDevStatePtr) auFile->io->state;
    sunState->fd = -1;

    auFile->u.devFormats = 0;

    if (auFile->io->name[0] == '\0' )
	auFile->io->name = AUDIODEV;
    if (! sunOpenDevice(auFile))
	return(False);

    if (ioctl(sunState->fd, AUDIO_GETINFO, (caddr_t)&oldData) < 0) {
	SetAudioReason(auFile,
	    CreateSysReason("Can't read audio device state\n"));
	return(False);
    }

    AUDIO_INITINFO(&devData);
    devData.play.encoding = AUDIO_ENCODING_ULAW;
    devData.play.channels = 1;
    devData.play.sample_rate = 8000;
    if( ioctl(sunState->fd, AUDIO_SETINFO, (caddr_t)&devData) == 0 ) {
	if (devData.play.encoding == AUDIO_ENCODING_ULAW) {
	    auFile->u.devFormats |= Avail8BitULaw;
	}
    }

    AUDIO_INITINFO(&devData);
    devData.play.channels = 1;
    devData.play.sample_rate = Audio16Khz;
    devData.play.encoding = AUDIO_ENCODING_LINEAR;
    devData.play.precision = 16;

    if( ioctl(sunState->fd, AUDIO_SETINFO, (caddr_t)&devData) == 0 
     && devData.play.encoding == AUDIO_ENCODING_LINEAR
     && devData.play.sample_rate == Audio16Khz
     && devData.play.precision == 16) {
	auFile->u.devFormats |= Avail16Khz;
	auFile->u.devFormats |= Audio16BitPcm;

	/*
	 * Derived from running tools/calaudio/calaudio on a sparc 10.
	 * other platforms may be different .
	 */
	sunState->delayMs = 45;

	AUDIO_INITINFO(&devData);
	devData.play.sample_rate = Audio48Khz;
	if( ioctl(sunState->fd, AUDIO_SETINFO, (caddr_t)&devData) == 0 
	 && devData.play.sample_rate == Audio48Khz)
	    auFile->u.devFormats |= Avail48Khz;

	AUDIO_INITINFO(&devData);
	devData.play.sample_rate = Audio44_1Khz;
	if( ioctl(sunState->fd, AUDIO_SETINFO, (caddr_t)&devData) == 0
	 && devData.play.sample_rate == Audio44_1Khz)
	    auFile->u.devFormats |= Avail44_1Khz;

	AUDIO_INITINFO(&devData);
	devData.play.sample_rate = Audio32Khz;
	if( ioctl(sunState->fd, AUDIO_SETINFO, (caddr_t)&devData) == 0
	 && devData.play.sample_rate == Audio32Khz)
	    auFile->u.devFormats |= Avail32Khz;

	AUDIO_INITINFO(&devData);
	devData.play.sample_rate = Audio11_025Khz;
	if( ioctl(sunState->fd, AUDIO_SETINFO, (caddr_t)&devData) == 0
	 && devData.play.sample_rate == Audio11_025Khz)
	    auFile->u.devFormats |= Avail11_025Khz;

	AUDIO_INITINFO(&devData);
	devData.play.sample_rate = Audio8Khz;
	if( ioctl(sunState->fd, AUDIO_SETINFO, (caddr_t)&devData) == 0
	 && devData.play.sample_rate == Audio8Khz)
	    auFile->u.devFormats |= Avail8Khz;
    } else {
	/*
	 * Derived from running tools/calaudio/calaudio on a sparc 2.
	 * other platforms may be different .
	 */
	sunState->delayMs = 42;
    }

    if (devData.play.channels == 1) {
	auFile->u.devFormats |= AvailMono;
	AUDIO_INITINFO(&devData);
	devData.play.channels = 2;
	if( ioctl(sunState->fd, AUDIO_SETINFO, (caddr_t)&devData) == 0
	 && devData.play.channels == 2)
	    auFile->u.devFormats |= AvailStereo;
    } else {
	auFile->u.devFormats |= AvailStereo;
	AUDIO_INITINFO(&devData);
	devData.play.channels = 1;
	if( ioctl(sunState->fd, AUDIO_SETINFO, (caddr_t)&devData) == 0
	 && devData.play.channels == 1)
	    auFile->u.devFormats |= AvailMono;
    }

    if( ioctl(sunState->fd, AUDIO_SETINFO, (caddr_t)&oldData) == 0 ) {
	; /* should we say something? */
    }

    sunAudioClose(auFile);
    return(True);
}

static long
sunAudioDelayMs(AudioFilePtr auFile)
{
    SunDevStatePtr sunState = (SunDevStatePtr) auFile->io->state;

    return(sunState->delayMs);
}

static boolean
sunAudioMode(AudioFilePtr auFile, long devCommand)
{
    SunDevStatePtr sunState = (SunDevStatePtr) auFile->io->state;
    int true;

    switch(devCommand) {
    case DevModeNonBlocking:
	true = True;
	break;
    case DevModeBlocking:
	true = False;
	break;
    default:
	return(False);
    }
    if (ioctl(sunState->fd, FIONBIO, &true) < 0) {
	SetAudioReason(auFile,
	    CreateSysReason("can't set %s io on fd %d\n",
			    devCommand == DevModeNonBlocking ?
			    "non-blocking" : "blocking",
			    sunState->fd));
	return(False);
    }
    return(True);
}


static boolean
sunAudioInit(AudioFilePtr auFile)
{
    SunDevStatePtr sunState = (SunDevStatePtr) auFile->io->state;
    audio_info_t devData, *ads = &devData;
    audio_prinfo_t *pds = &devData.play;
    char *errFunc = "audioOpen";
    int encoding, precision;

    if (! sunOpenDevice(auFile))
	return(False);

    ioctl(sunState->fd,AUDIO_GETINFO, (caddr_t)ads);

    switch(auFile->au.dataForm) {
    default:
	SetAudioReason(auFile,
	    CreateReason("bad data encoding requested: %d\n",
		auFile->au.dataForm));
	return(False);

    case Audio8BitULaw:
	encoding = AUDIO_ENCODING_ULAW;
	precision = 8;
	break;
    case Audio16BitPcm:
	encoding = AUDIO_ENCODING_LINEAR;
	precision = 16;
	break;
    }

    /*
     * Check the audio device configuration and
     * attempt to match it to the audio file
     */
    if (pds->sample_rate != auFile->au.rate
     || pds->channels != auFile->au.channels
     || pds->encoding != encoding
     || pds->precision != precision)
    {
	AUDIO_INITINFO(ads);
	pds->sample_rate = auFile->au.rate;
	pds->channels = auFile->au.channels;
	pds->encoding = encoding;
	pds->precision = precision;
	ioctl(sunState->fd,AUDIO_SETINFO, (caddr_t)ads);
    }

    if (pds->channels != auFile->au.channels
     || pds->sample_rate != auFile->au.rate
     || pds->encoding != encoding
     || pds->precision != precision)
    {
	ProgMsg("%s: Cannot configure audio device appropriately\n", errFunc);
	return(False);
    }

    return(True);
}

static long
sunAudioReadSamples(AudioFilePtr auFile, UChar *buffer, ULong nSamples)
{
    SetAudioReason(auFile, &CantRead);
    return(-1);
}

static long
sunAudioWriteSamples(AudioFilePtr auFile, UChar *buffer, ULong nSamples)
{
    SunDevStatePtr sunState = (SunDevStatePtr) auFile->io->state;
    int bytesToWrite, bytesWritten, dataShift;

    dataShift = DataFormToShift[ auFile->au.dataForm ];
    if( (bytesToWrite = nSamples << dataShift) == 0)
	return(True);

    bytesWritten = write(sunState->fd, buffer, bytesToWrite);

    if (bytesWritten < 0) {
	SetAudioReason(auFile, CreateSysReason("Can't write\n"));
    } else if (bytesWritten < bytesToWrite) {
	SetAudioReason(auFile, 
	    CreateSysReason("short write (%d of %d)\n",
		bytesWritten, bytesToWrite));
    }

    return(bytesWritten);
}

/*		External Linkage				*/

AudioDevOpFunc SunDevFuncs = {
    sunAudioOpen,
    sunAudioInit,
    sunAudioMode,
    sunAudioDelayMs,
    sunAudioReadSamples,
    sunAudioWriteSamples,
    sunAudioSamplesQueued,
    sunAudioFlush,
    sunAudioClose
};
