/*
 * (c) Copyright 1992 North Valley Research, Inc.
 *         All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 * 	osio.h
 *
 * DESCRIPTION
 * 	I/O functions that hide os dependencies
 *	OS-DEPENDENT
 *
 *      REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/asyncio.c,v 1.4 1994/12/17 13:02:42 russ Exp $
 */
#include <sys/types.h>
#include <sys/errno.h>
#include <sys/time.h>

#define NeedFunctionPrototypes 1

#include "nvrTypes.h"
#include "osio.h"
#include "nvrDebug.h"
#include "nvrAlloc.h"
#include "nvrString.h"

#ifdef sun

#ifdef Solaris2
# include <sys/syscall.h>
#else
# include <syscall.h>
#endif
#include <sys/asynch.h>

extern aio_result_t *aiowait(struct timeval *);

typedef struct _AsyncIoState {
    boolean allocated;		/* true if this is waiting for completion */
    boolean complete;		/* true if OS said this operation is complete */
    short id;			/* identifier returned to caller */
    aio_result_t result;	/* sun-specific: result of io */
} AsyncIoState, *AsyncIoStatePtr;

typedef struct _AsyncIoInfo {
    AsyncIoStatePtr *pending;	/* array of used/unused AsyncIoState */
    int nStates;		/* number of cells in 'pending' */
    int nPending;		/* number of outstanding io requests */
    short id;			/* counter for identifier */
    struct timeval noWait;	/* convience 0-value timeval struct */
} AsyncIoInfo;

AsyncIoInfo AsyncIoData;

static AsyncIoStatePtr
AllocAsyncIoOperation(void)
{
    AsyncIoStatePtr pState;
    AsyncIoStatePtr *pStates, *pEnd;

    if (AsyncIoData.nPending == AsyncIoData.nStates) {
	AsyncIoStatePtr *newStates;

	newStates = (AsyncIoStatePtr *)
	    malloc((AsyncIoData.nStates+1) * sizeof(AsyncIoStatePtr));
	if (newStates == NULL)
	    return(NULL);

	pState = (AsyncIoStatePtr) malloc(sizeof(AsyncIoState));
	if (pState == NULL) {
	    free(newStates);
	    return(NULL);
	}

	memcpy(newStates,
	       AsyncIoData.pending,
	       AsyncIoData.nStates * sizeof(AsyncIoStatePtr));
	free(AsyncIoData.pending);
	AsyncIoData.pending = newStates;
	AsyncIoData.pending[ AsyncIoData.nStates++ ] = pState;
    } else {
	pStates = AsyncIoData.pending;
	pEnd = &pStates[ AsyncIoData.nStates ];
	while(pStates != pEnd)
	    if (! (pState = pStates[0])->allocated)
		goto foundOne;
	Fatal("AllocAsyncIoOperation: lost an op!\n");
    }

foundOne:
    pState->complete = False;
    pState->allocated = True;
    AsyncIoData.nPending++;
    pState->id = AsyncIoData.id++;
    return(pState);
}

int
AsyncWrite(int fd, char *buf, int length, long offset)
{
    AsyncIoStatePtr pState;
    int status;
    extern int errno;

    if ((pState = AllocAsyncIoOperation()) == NULL) {
	errno = ENOBUFS;
	return(-1);
    }
    status = aiowrite(fd, buf, length, offset, 0, &pState->result);
    if (status < 0) {
	pState->allocated = False;
	return(-1);
    }

    return(pState->id);
}

int
FinishAsyncIo(int id)
{
    AsyncIoStatePtr *pStates, *pEnd;
    AsyncIoStatePtr pThisState, pState;
    extern int errno;
    aio_result_t *pResult;

    pEnd = &AsyncIoData.pending[ AsyncIoData.nStates ];

    pStates = AsyncIoData.pending;
    for(; pStates != pEnd; pStates++) {
	if (!(pThisState = pStates[0])->allocated)
	    continue;
	if (pThisState->id == id) {
	    if (pThisState->complete)
		goto operationComplete;

	    goto operationIncomplete;
	}
    }
    errno = ESRCH;
    return(-1);

    /*
     * Wait for this operation and possibly others.  Note that
     * we block here if there are any outstanding requests, but
     * after we find the one we're looking for, we look no further.
     */
operationIncomplete:
    for (;;) {
	pResult = aiowait(NULL);
	if ((int)pResult <= 0)
	    Fatal("Operation not running?\n");

	pStates = AsyncIoData.pending;
	while(pStates != pEnd) {
	    pState = pStates[0];
	    if (pResult == &pState->result)
		pState->complete = True;
	    if (! pState->allocated)
		Fatal("op @0x%x returned twice?\n", pState);
	    if (pState == pThisState)
		goto operationComplete;
	    pStates++;
	}
    }

operationComplete:
    errno = pThisState->result.aio_errno;
    pThisState->allocated = False;
    return (pThisState->result.aio_return);
}

#else /* sun */
int
AsyncWrite(int fd, char *buf, int length, long offset)
{
    Fatal("Async write called\n");
    /* NOTREACHED */
}

int
FinishAsyncIo(int id)
{
    Fatal("Finish Async called\n");
    /* NOTREACHED */
}

#endif /* sun */
