/*
 *	(c) Copyright 1992 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		binarybits.h
 *
 *	DESCRIPTION
 *		<overall description of what the package does>
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/binarybits.h,v 3.3 1993/09/09 23:03:23 toddb Exp $
 */
#ifndef _BINARYBITS_H_
#define _BINARYBITS_H_


/*
 * The Binary?() macros are used to allow a visually correct representation
 * of binary numbers.  Each of the n? arguments are nibbles.  The N in
 * BinaryN() is the number of bits present in the n1,n2,...  The nBits
 * must be >= than N and provides a width of the bit field in which
 * to locate the final number, left-justified.  For example,
 * Binary5(0101,1, 8) produces the binary number 0101 1000.
 */

#define  Binary1(n1,nBits)	 ((Concat(0,n1) &     01) <<  (nBits-1))

#define  Binary2(n1,nBits)	 ((((Concat(0,n1) &   010) >> 2) \
				  | (Concat(0,n1) &    01)) << (nBits-2))

#define  Binary3(n1,nBits)	 ((((Concat(0,n1) &  0100) >> 4) \
				  |((Concat(0,n1) &   010) >> 2) \
				  | (Concat(0,n1) &    01)) << (nBits-3))

#define  Binary4(n1,nBits)	 ((((Concat(0,n1) & 01000) >> 6) \
				 | ((Concat(0,n1) &  0100) >> 4) \
				 | ((Concat(0,n1) &   010) >> 2) \
				 |  (Concat(0,n1) &    01)) <<  (nBits-4))

#define  Binary5(n1,n2,nBits)	(Binary4(n1,nBits) | Binary1(n2,nBits-4))
#define  Binary6(n1,n2,nBits)	(Binary4(n1,nBits) | Binary2(n2,nBits-4))
#define  Binary7(n1,n2,nBits)	(Binary4(n1,nBits) | Binary3(n2,nBits-4))
#define  Binary8(n1,n2,nBits)	(Binary4(n1,nBits) | Binary4(n2,nBits-4))

#define  Binary9(n1,n2,n3,nBits) (Binary8(n1,n2,nBits) | Binary1(n3,nBits-8))
#define Binary10(n1,n2,n3,nBits) (Binary8(n1,n2,nBits) | Binary2(n3,nBits-8))
#define Binary11(n1,n2,n3,nBits) (Binary8(n1,n2,nBits) | Binary3(n3,nBits-8))
#define Binary12(n1,n2,n3,nBits) (Binary8(n1,n2,nBits) | Binary4(n3,nBits-8))

#define Binary13(n1,n2,n3,n4,nBits) (Binary12(n1,n2,n3,nBits) \
				   | Binary1(n4,nBits-12))
#define Binary14(n1,n2,n3,n4,nBits) (Binary12(n1,n2,n3,nBits) \
				   | Binary2(n4,nBits-12))
#define Binary15(n1,n2,n3,n4,nBits) (Binary12(n1,n2,n3,nBits) \
				   | Binary3(n4,nBits-12))
#define Binary16(n1,n2,n3,n4,nBits) (Binary12(n1,n2,n3,nBits) \
				   | Binary4(n4,nBits-12))

#define Binary17(n1,n2,n3,n4,n5,nBits) (Binary16(n1,n2,n3,n4,nBits) \
				   | Binary1(n5,nBits-16))
#define Binary18(n1,n2,n3,n4,n5,nBits) (Binary16(n1,n2,n3,n4,nBits) \
				   | Binary2(n5,nBits-16))
#define Binary19(n1,n2,n3,n4,n5,nBits) (Binary16(n1,n2,n3,n4,nBits) \
				   | Binary3(n5,nBits-16))
#define Binary20(n1,n2,n3,n4,n5,nBits) (Binary16(n1,n2,n3,n4,nBits) \
				   | Binary4(n5,nBits-16))

#define Binary21(n1,n2,n3,n4,n5,n6,nBits) (Binary20(n1,n2,n3,n4,n5,nBits) \
				   | Binary1(n6,nBits-20))
#define Binary22(n1,n2,n3,n4,n5,n6,nBits) (Binary20(n1,n2,n3,n4,n5,nBits) \
				   | Binary2(n6,nBits-20))
#define Binary23(n1,n2,n3,n4,n5,n6,nBits) (Binary20(n1,n2,n3,n4,n5,nBits) \
				   | Binary3(n6,nBits-20))
#define Binary24(n1,n2,n3,n4,n5,n6,nBits) (Binary20(n1,n2,n3,n4,n5,nBits) \
				   | Binary4(n6,nBits-20))

#endif /* _BINARYBITS_H_ */
