/*
 *	(c) Copyright 1992 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		bitstream.h
 *
 *	DESCRIPTION
 *		<overall description of what the package does>
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/bitstream.h,v 3.37 1995/08/09 00:15:50 toddb Exp $
 */
#ifndef _BITSTREAM_H_
#define _BITSTREAM_H_
#include "nvrReason.h"
#include "mpegSysDecode.h"

#include "nvrTypes.h"
#include "machdep.h"
#include "nvrDebug.h"
#include "osio.h"

#define KeepBits(nBits, value)  ((value) & ((1<<(nBits)) - 1))

/*
 * These macros provide some syntactic sugar for managing the cache
 * register variables used by NextBits and ReadBits.
 */

#ifdef NeedFunctionPrototypes
#define CacheArgsDecl	int _nCacheBits, ULong *_pCacheBuf
#define CacheProtoArgs	int, ULong *
#endif
#define CacheDeclarations				\
	ULong *_pCacheBuf;				\
	int _nCacheBits

#define CacheArgs					\
	_nCacheBits, _pCacheBuf

#define InitCacheVariables(buf)				\
	_nCacheBits = (buf)->nCacheBits;		\
	_pCacheBuf = (buf)->pbuf;			\

#define CheckOutCache(buf)				\
	CacheDeclarations; InitCacheVariables(buf)

#define FlushCacheValues(buf)				\
	(buf)->nCacheBits = _nCacheBits;		\
	(buf)->pbuf = _pCacheBuf;

/*
 * In the NextBits() and ReadBits() macros, the cache and nbits arguments
 * can be local (register) variables or the MpegBuf.{cache,nBits} elements.
 * These cells must be carefully checked in and out of the structure
 * such that they are always "in" when calling a subroutine or returning
 * from one.
 *
 * Further, the cache is only filled when there are not enough bits to
 * satisfy the request.  If NextBits is short any bits, the entire cache
 * is filled.  If ReadBits doesn't find enough enough bits in the cache,
 * the cache is filled after the request is satisfied; thus the bits
 * in each buffered longword only have to read from memory once.
 * ConsumeBits() is an efficient alternate to ReadBits() if you
 * have already seen the bits.
 */
#define NextBits(buf, nBits, value)					\
    {									\
	register bitsLeft;						\
	if ((bitsLeft = _nCacheBits - (nBits)) >= 0) {			\
	    value = KeepBits((nBits), _pCacheBuf[0] >> bitsLeft);	\
	} else if (_pCacheBuf < (buf)->pbufLast) {			\
	    value = KeepBits((nBits),					\
			(_pCacheBuf[0] << -bitsLeft)			\
		      | (_pCacheBuf[1] >> (32 + bitsLeft)));		\
	} else {							\
	    _pCacheBuf = FillBitStream(buf, _nCacheBits, nBits);	\
	    _nCacheBits = (buf)->nCacheBits;				\
	    value = (buf)->retValue;					\
	}								\
    }

#if defined(SPARC_GCC) || defined(MIPS_GCC)

#define FNextBits(buf, nBits, value)					\
    Accept3(_nCacheBits, _pCacheBuf, value,				\
	    _FNextBits(_nCacheBits, _pCacheBuf, buf, nBits)		\
    )

#define ReturnFromFNextBits(buf, value)					\
    Return3 (_nCacheBits, _pCacheBuf, value)

#else /* defined(SPARC_GCC, MIPS_GCC) */

#define FNextBits(buf, nBits, value)					\
    value = _FNextBits(_nCacheBits, _pCacheBuf, buf, nBits);		\
    InitCacheVariables(buf)

#define ReturnFromFNextBits(buf, value)					\
    FlushCacheValues(buf);						\
    return(value)

#endif /* defined(SPARC_GCC, MIPS_GCC) */

#define ReadBits(buf, nBits, value, whatIsIt)				\
    {									\
	StreamAssert(buf, nBits > 0);					\
	if ((_nCacheBits -= (nBits)) >= 0) {				\
	    value = KeepBits((nBits), _pCacheBuf[0] >> _nCacheBits);	\
	} else if (_pCacheBuf < (buf)->pbufLast) {			\
	    value = KeepBits((nBits),					\
			(_pCacheBuf[0] << -_nCacheBits)			\
		      | (_pCacheBuf[1] >> (32 + _nCacheBits)));		\
	    _pCacheBuf++;						\
	    _nCacheBits += 32;						\
	} else {							\
	    _pCacheBuf = FillBitStream(buf, _nCacheBits, nBits);	\
	    _nCacheBits = (buf)->nCacheBits;				\
	    value = (buf)->retValue;					\
	}								\
	ShowBits(buf, value, nBits, whatIsIt);				\
    }

#define Read32Bits(buf, value, whatIsIt)				\
    {									\
	if ((_nCacheBits -= (32)) == 0) {				\
	    value = _pCacheBuf[0];					\
	} else if (_pCacheBuf < (buf)->pbufLast) {			\
	    if (_nCacheBits == -32) {					\
		_pCacheBuf++;						\
		value = _pCacheBuf[0];					\
		_nCacheBits = 0;					\
	    } else {							\
		value = (_pCacheBuf[0] << -_nCacheBits)			\
		      | (_pCacheBuf[1] >> (32 + _nCacheBits));		\
		_pCacheBuf++;						\
		_nCacheBits += 32;					\
	    }								\
	} else {							\
	    _pCacheBuf = FillBitStream(buf, _nCacheBits, 32);		\
	    _nCacheBits = (buf)->nCacheBits;				\
	    value = (buf)->retValue;					\
	}								\
	ShowBits(buf, value, 32, whatIsIt);				\
    }

#if defined(SPARC_GCC) || defined(MIPS_GCC)

#define FReadBits(buf, nBits, value, whatIsIt)				\
    Accept3(_nCacheBits, _pCacheBuf, value,				\
	    _FReadBits(_nCacheBits, _pCacheBuf, buf, nBits, whatIsIt)	\
    )

#define ReturnFromFReadBits(buf, value)					\
    Return3 (_nCacheBits, _pCacheBuf, value)

#else /* defined(SPARC_GCC, MIPS_GCC) */

#define FReadBits(buf, nBits, value, whatIsIt)				\
    value = _FReadBits(_nCacheBits, _pCacheBuf, buf, nBits, whatIsIt);	\
    InitCacheVariables(buf)

#define ReturnFromFReadBits(buf, value)					\
    FlushCacheValues(buf);						\
    return(value)

#endif /* defined(SPARC_GCC, MIPS_GCC) */

#define ConsumeBits(buf, nConsume, whatWasIt)				\
    {									\
	StreamAssert(buf, nConsume > 0);				\
	if ((_nCacheBits -= (nConsume)) >= 0) {				\
	    ShowBits(buf,						\
		     KeepBits(nConsume, _pCacheBuf[0] >> _nCacheBits),	\
		     nConsume, whatWasIt);				\
	} else if (_pCacheBuf < (buf)->pbufLast) {			\
	    ShowBits(buf,						\
		KeepBits((nConsume),					\
		    (_pCacheBuf[0] << -_nCacheBits)			\
		  | (_pCacheBuf[1] >> 32+_nCacheBits)),			\
		 nConsume, whatWasIt);					\
	    _pCacheBuf++;						\
	    _nCacheBits += 32;						\
	} else {							\
	    Msg("ConsumeBits fills buffer\n");				\
	    _pCacheBuf = FillBitStream(buf, _nCacheBits, nConsume);	\
	    _nCacheBits = buf->nCacheBits;				\
	    ShowBits(buf, buf->retValue, nConsume, whatWasIt);		\
	}								\
    }

#if defined(SPARC_GCC) || defined(MIPS_GCC)

#define FConsumeBits(buf, nConsume, whatIsIt)				\
    Accept2(_nCacheBits, _pCacheBuf,					\
	_FConsumeBits(_nCacheBits, _pCacheBuf, buf, nConsume, whatIsIt)	\
    )

#define ReturnFromFConsumeBits(buf)					\
    Return2 (_nCacheBits, _pCacheBuf)

#define FScanForAlignedZeroByte(buf, whatIsIt)				\
    Accept2(_nCacheBits, _pCacheBuf,					\
	_FScanForAlignedZeroByte(_nCacheBits,_pCacheBuf,buf,whatIsIt)	\
    )

#define ReturnFromFScanForAlignedZeroByte(buf)				\
    Return2 (_nCacheBits, _pCacheBuf)


#else /* defined(SPARC_GCC, MIPS_GCC) */

#define FConsumeBits(buf, nConsume, whatIsIt)				\
    _FConsumeBits(_nCacheBits, _pCacheBuf, buf, nConsume, whatIsIt);	\
    InitCacheVariables(buf)

#define ReturnFromFConsumeBits(buf)					\
    FlushCacheValues(buf)						\
    return;

#define FScanForAlignedZeroByte(buf, whatIsIt)				\
    _FScanForAlignedZeroByte(_nCacheBits,_pCacheBuf,buf,whatIsIt);	\
    InitCacheVariables(buf)

#define ReturnFromFScanForAlignedZeroByte(buf)				\
    FlushCacheValues(buf)						\
    return;

#endif /* defined(SPARC_GCC, MIPS_GCC) */

/*
 * This macro allows you to do a NextBits but consume other bits
 * that you have already seen.
 */
#define ConsumeThenNextBits(buf, nReadBits, nBits, value, whatWasIt)	\
    ConsumeBits(buf, nReadBits, whatWasIt)				\
    NextBits(buf, nBits, value);

/*
 * This macro allows you to do a ReadBits but consume other bits
 * that you have already seen.
 */
#define ConsumeThenReadBits(buf, nReadBits, nBits, value, whatWasIt, whatIsIt)\
    ConsumeBits(buf, nReadBits, whatWasIt)				\
    ReadBits(buf, nBits, value, whatIsIt);

/*
 * As a subset of the cache management macros, these macrose
 * provide the calling mechanism for a small set of routines
 * that perform the mpeg decoding.
 */

#if defined(SPARC_GCC) || defined(MIPS_GCC)

#define	RestoreCacheAfterCall(buf, funcAndArgs)				\
    Accept2(_nCacheBits, _pCacheBuf, funcAndArgs)

#define	CacheReturn(buf)						\
    Return2(_nCacheBits, _pCacheBuf)
	
typedef TwoValues   CacheReturnValue;
typedef ThreeValues ReadBitsReturn;
typedef ThreeValues NextBitsReturn;
typedef TwoValues   ConsumeBitsReturn;

#else

#define	RestoreCacheAfterCall(buf, funcAndArgs)				\
    funcAndArgs;							\
    InitCacheVariables(buf);

#define	CacheReturn(buf)						\
    FlushCacheValues(buf)						\
    return;

typedef void CacheReturnValue;
typedef void ConsumeBitsReturn;
typedef ULong ReadBitsReturn;
typedef ULong NextBitsReturn;

#endif /* defined(SPARC_GCC, MIPS_GCC) */

/*
 * ReadBitstreamData() is used to read the various extra data junk in the
 * sequence, such as {extension,user,group,picture} data
 * By the time we have finished, we have effectively performed a
 * NextStartCode().
 */
#define ReadBitstreamData(buf, lhsExpression)				\
    {									\
	register ULong data, nextBits;					\
	FReadBits(buf, 24, nextBits, N_DataAndStartcode);		\
	data = nextBits >> 16;						\
	while (nextBits != B_STARTCODE) {				\
	    lhsExpression = data;					\
	    FReadBits(buf, 8, data, N_NoLabel);				\
	    nextBits = (KeepBits(16, nextBits) << 8) | data;		\
	}								\
    }

#define DFLT_BITSTREAM_BUF_SIZE      (8*1024)        /* in longwords */

/*
 * Values passed to error handler to signal condition
 */
typedef enum {
    BitstreamError,
    BitstreamEof
} BitstreamCondition;

typedef struct _bitStream BitStream, *BitStreamPtr;

typedef void (*BitstreamErrorFunc)(BitStreamPtr, BitstreamCondition);
typedef int  (*BitStreamReadFunc)(FileHandle, char *, long);
typedef void (*BitStreamCloseFunc)(FileHandle);
typedef long  (*BitStreamSeekFunc)(FileHandle, long offset, SeekWhere sk);
typedef FileHandle (*BitStreamOpenFunc)(char *);
typedef boolean (*DebugFormatter)(char *, int whatIsIt, ULong value);

struct _bitStream {
    BitstreamErrorFunc errorFunc;	/* function called on bitstream error */
    BitStreamOpenFunc  openFunc;	/* Raw bitstream open function */
    BitStreamReadFunc  readFunc;	/* Raw bitstream read function */
    BitStreamCloseFunc closeFunc;	/* Raw bitstream close function */
    BitStreamSeekFunc  seekFunc;	/* Raw bitstream seek function */

    DebugFormatter     dbgFormat;	/* optional token formatter function */

    FileHandle	       fd;		/* File descriptor for raw video */
    char              *file;		/* name of file */
    ULong             *pbufLast;	/* pointer to last whole longword */
    ULong	      *buffer;		/* 1st ULong addr in current buffer */
    ULong	       pos;		/* File byte position of .buffer[0] */
    ULong	       tmp[3];		/* Tmp for re-alignement on EOF */
    MpegStreamInfo     mstr;		/* System stream data structure */
    MpegSystemInfoPtr  msys;		/* System layer data structure */
    PrivPointer        seqPriv;		/* private storage for the sequence */
    int		       bufSize;		/* Buffer size for raw video (bytes) */
    char	       leftover;	/* Nbr bytes in partial last word */
    boolean            eof;		/* true if source eof reached */
    boolean	       system;		/* true if reading from MPEG system */
    boolean	       eofok;		/* T if EOK is OK, set F if EOF hit */
    boolean	       opened;		/* T if currently opened, else False */
    ULong             *pbuf;		/* TMP: pointer to buffer[] */
    ULong	       retValue;	/* TMP: FillBitStream value return */
    char               nCacheBits;	/* TMP: number of bits in the cache */
};

typedef struct _bitByteOffset {
    long bytes;			/* number of bytes */
    char bits; 			/* number of bits (never more than 7) */
} BitByteOffset, *BitByteOffsetPtr;

#define AtEndOfStream(buf)	(((buf)->pbuf+1 >= (buf)->pbufLast) 	\
			       && (buf)->eof)
#define CacheAtEndOfStream(buf)	((_pCacheBuf+1 >= (buf)->pbufLast) 	\
			       && (buf)->eof)

#ifdef DEBUG_ENABLED

#define ShowBits(buf, value, nBits, whatIsIt)				\
    {									\
	if (GetDebugBit(whatIsIt)) {					\
	    (buf)->nCacheBits = _nCacheBits;				\
	    (buf)->pbuf = _pCacheBuf;					\
	    PrintBits(buf, value, nBits, whatIsIt);			\
	}								\
    }

#define BitsConsumed  ((32-_nCacheBits)%8)

#define ShowMark(buf, whatIsIt, args)					\
    if (GetDebugBit(whatIsIt)) {					\
	(buf)->nCacheBits = _nCacheBits;				\
	(buf)->pbuf = _pCacheBuf;					\
	PrintLabel(buf, 0, whatIsIt);					\
	Msg args;							\
    }									\

#else

#define ShowBits(buf, value, nBits, whatIsIt) 		/* nothing */
#define ShowMark(buf, whatIsIt, args)			/* nothing */

#endif

#ifdef STREAM_ASSERT
#define StreamAssert(buf, mustBeTrue)					\
        if (! (mustBeTrue)) {						\
	    FailedAssert(_nCacheBits, _pCacheBuf, buf,			\
		"\"mustBeTrue\"", __FILE__, __LINE__);			\
	}
#else
#define StreamAssert(buf, mustBeTrue)        /* nothing */
#endif

CExternal void FailedAssert(
    int,	/* _nCacheBits */
    ULong *,	/* _pCacheBuf */
    BitStream *,
    char *,	/* mustBeTrue */
    char *,	/* fileName */
    int line	/* line number */
);

CExternal NextBitsReturn _FNextBits(
    int _nCacheBits,
    ULong *_pCacheBuf,
    BitStream *buf,
    int nBits
);

CExternal ReadBitsReturn _FReadBits(
    int _nCacheBits,
    ULong *_pCacheBuf,
    BitStream *buf,
    int nBits,
    int whatIsIt
);

CExternal ConsumeBitsReturn _FConsumeBits(
    int _nCacheBits,
    ULong *_pCacheBuf,
    BitStream *buf,
    int nConsume,
    int whatIsIt
);

CExternal ConsumeBitsReturn _FScanForAlignedZeroByte(
    int _nCacheBits,
    ULong *_pCacheBuf,
    BitStream *buf,
    int whatIsIt
);

CExternal ULong	*FillBitStream(
    BitStream *,
    int,			/* Nbr bits left in cache after fld removal */
    int				/* Total nbr of bits in field */
);

CExternal int SeekBitStream(
    BitStreamPtr buf,
    long offset,
    SeekWhere sk
);

CExternal void CloseBitStream(
    BitStreamPtr buf
);

CExternal void InitBitStreamData(
    BitStreamPtr mpegBuf
);

CExternal void ResetBitstreamFuncs(
    BitStreamPtr mpegBuf
);

CExternal InternalReasonPtr OpenSystemStreamInput(
    BitStreamPtr mpegB,
    BitStreamPtr firstMpegB,
    char *file,
    BitstreamType type
);

CExternal InternalReasonPtr OpenBitStreamInput(
    BitStream *,
    char *			/* File name */
);

CExternal long BitDifference(
    BitByteOffsetPtr,   /* start */
    BitByteOffsetPtr    /* end */
);

CExternal BitByteOffset BitStreamOffset(
    BitStream *,        /* pBuf */
    int                 /* newBits */
);

CExternal void PrintLabel(
    BitStream *,	/* pBuf */
    int,		/* nBits */
    int			/* whatIsIt */
);

CExternal void PrintBits(
    BitStream *,	/* pBuf */
    ULong,		/* value */
    ULong,		/* nBits */
    int			/* whatIsIt */
);

CExternal void CloseBitStream(
    BitStreamPtr buf
);

#endif /* _BITSTREAM_H_ */
