/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	catprog.c - test program
 *
 * DESCRIPTION
 *	OS-DEPENDENT
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/catprog.c,v 1.4 1994/01/10 18:48:13 toddb Exp $
 */
#define DEBUG_ENABLED
#define NeedFunctionPrototypes 1

#include "process.h"
#include "fileAccess.h"

#include <sys/types.h>

char ProgramNameBuffer[ 100];
char *ProgramName = ProgramNameBuffer;

void
Transfer(int infd, int outfd, fd_set *pReadFds)
{
    int bytesIn, bytesOut;
    char buf[ 100 ], *p;

    bytesIn = read(infd, buf, 100);
    if (bytesIn <= 0) {
	if (bytesIn < 0)
	    Fatal("error reading fd %d\n", infd);
	Error("Eof on fd %d\n", infd);
	FD_CLR(infd, pReadFds);
	return;
    }
    Error("got %d bytes on fd %d, sending to fd %d\n", bytesIn, infd, outfd);
    p = buf;
    while (bytesIn > 0) {
	if ((bytesOut = write(outfd, p, bytesIn)) <= 0)
	    Fatal("can't write out fd %d\n", outfd);
	p += bytesOut;
	bytesIn -= bytesOut;
    }
}

main(int argc, char **argv)
{
    int infd = 0, outfd = 1;
    fd_set readFds, fdsReady;
    int nfdsReady, nReadFds;

    sprintf(ProgramName, "%s (pid %d)", argv[0], getpid());

    ShowOpenFileDescriptors();

    for (argc--, argv++; argc; argc--, argv++) {
	if (argv[0][0] != '-')  {
	    Error("ignoring arg %s\n", *argv);
	    continue;
	}
	switch(argv[0][1]) {
	case 'i': infd = atoi(argv[0]+2); break;
	case 'o': outfd = atoi(argv[0]+2); break;
	default: Error("ignoring arg %s\n", *argv); break;
	}
    }
    FD_ZERO(&readFds);
    FD_SET(infd, &readFds);
    FD_SET(0, &readFds);
    nReadFds = infd+1;
    Error("input: %d%s, output: %d\n",
	infd,
	(infd != 0) ? " & 0" : "",
	outfd);

    while (FD_ISSET(0, &readFds) || FD_ISSET(infd, &readFds)) {
	fdsReady = readFds;
	nfdsReady = select(nReadFds, &fdsReady, 0,0,0);
	if (nfdsReady < 0)
	    continue;

	if (infd != 0 && FD_ISSET(0, &fdsReady))
	    Transfer(0, outfd, &readFds);
	if (FD_ISSET(infd, &fdsReady))
	    Transfer(infd, outfd, &readFds);
    }
    Error("no more input, closing %d\n", outfd);
    close(outfd);
    sleep(1);
    exit(0);
}
