/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	debug.c - basic debug support routines
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/debug.c,v 1.18 1994/10/09 18:48:30 toddb Exp $
 */

/*
 * We always turn this on so that the routines are defined.
 */
#define DEBUG_ENABLED
#define NeedFunctionPrototypes 1

#include "nvrTypes.h"
#include "nvrDebug.h"
#include "nvrString.h"
#include "bitstream.h"
#include "osio.h"
#include <stdio.h>

ULong DebugFlags[ (DBI_Last + 31) >> 5 ];

static void
CheckDebugNameSpace(FlagLabelsPtr labelList)
{
    UShort *bitNumbers;
    FlagNamesPtr flags;
    FlagMacroListPtr macros;
    long i, j, overlap;

    for (; labelList->flags; labelList++) {
	flags = labelList->flags;
	if (flags->nNames > DBI_Span)
	    Msg("Warning: debug bits %d-%d overlap %d bits in next set!\n%s\n",
		flags->baseIndex, flags->baseIndex + flags->nNames-1,
		flags->nNames - DBI_Span,
		"Increast the value for DBI_Span in nvrDebug.h");

	/*
	 * Macro flags
	 */
	macros = labelList->macros;
	if (macros == NULL)
	    continue;
	for (i = 0; i < macros->nMacros; i++) {
	    bitNumbers = macros->list[i].bitNumbers;
	    for (j = macros->list[i].nBits - 1; j >= 0; j--) {
		overlap = bitNumbers[j] - flags->baseIndex - DBI_Span;
		if (overlap > 0)
		    Msg("%s%s macro overlap %d bits in next set!\n%s\n",
			"Warning: debug bits in ",
			macros->list[ i ].name, overlap,
			"Increast the value for DBI_Span in nvrDebug.h");
	    }
	}
    }
}

char *
FindDebugFlagName(int index)
{
    FlagLabelsPtr labelList = DebugFlagDesc;
    FlagNamesPtr flags;

    for (; labelList->flags; labelList++) {
        flags = labelList->flags;
	if (index >= flags->baseIndex
	 && index < flags->baseIndex + flags->nNames)
	    return(flags->names[ index - flags->baseIndex ]);
    }
    return("???");
}

void
SetDebugFileFlags(ULong *debugBits, FlagLabelsPtr labelList, char *fileName)
{
    BufFileHndl fp;
    char buf[ BUFSIZ ];
    int lineLength;

    CheckDebugNameSpace(labelList);

    if ((fp = OpenBufferedFileForReading(fileName)) == NULL)
	return;
    while (BufferedReadLine(buf, BUFSIZ, fp)) {
	if ((lineLength = strlen(buf) - 1) <= 0)
	    continue;
	if (buf[ lineLength ] == '\n')
	    buf[ lineLength ] = '\0';
	SetFlagName(debugBits, labelList, buf);
    }
    CloseBufferedFile(fp);
}

void
SetFlagName(ULong *debugBits, FlagLabelsPtr labelList, char *name)
{
    long i, j, bit;
    UShort *bitNumbers;
    FlagNamesPtr flags;
    FlagMacroListPtr macros;

    for (; labelList->flags; labelList++) {
	flags = labelList->flags;
	for (i = 0; i < flags->nNames; i++)
	    if (strcmp(name, flags->names[ i ]) == 0) {
		bit = i + flags->baseIndex;
		debugBits[bit>>5] |= (1<<(bit&0x1f));
		Msg("mark \"%s\": bit %d, (%d/1<<%d)\n",
		    name, bit, bit>>5, bit&0x1f);
		return;
	    }

	/*
	 * Macro flags
	 */
	if ((macros = labelList->macros) == NULL)
	    continue;
	for (i = 0; i < macros->nMacros; i++) {
	    if (strcmp(name, macros->list[ i ].name) == 0) {
		Msg("mark macro \"%s\": %d bits\n",
		    name, macros->list[i].nBits);
		bitNumbers = macros->list[i].bitNumbers;
		for (j = macros->list[i].nBits - 1; j >= 0; j--)
		    debugBits[bitNumbers[j]>>5] |= (1<<(bitNumbers[j]&0x1f));
		return;
	    }
	}
    }
}

char *FixedPointToA(int value, int fixedPointBits)
{
#define MAXBUFS	8
    static char buf[ MAXBUFS ][ 30 ];
    static long bufIndex;
    char *p, *end;
    unsigned long fraction, bitValue;
    int i;

    p = buf[ bufIndex++ ];
    if (bufIndex >= MAXBUFS)
	bufIndex = 0;
    bitValue = 500000000; /* this is our .5 */
    fraction = 0;
    for (i = fixedPointBits-1; i >= 0; i--) {
	if (value & (1<<i))
	    fraction += bitValue;
	bitValue >>= 1;
    }

    sprintf(p, "%d.%09d", value >> fixedPointBits, fraction);

    /*
     * trim trailing 0's
     */
    end = &p[ strlen(p)-1 ];
    while (end[0] == '0' && end[-1] != '.')
	*end-- = '\0';
	
    return (p);
#undef MAXBUFS
}

char *BitString(int value, int nBits)
{
    int i, bitsPrinted;
    static char buf[ 100 ];
    char *p = buf;

    bitsPrinted = 0;
    for (i = nBits-1; i >= 0; i--) {
	*p++ = '0' + ((value & (1<<i)) != 0);
	if ((++bitsPrinted & 0x3) == 0)
	    *p++ = ' ';
    }
    *p = '\0';
    return (buf);
}
