/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	fileContent.c - determine the content of a file
 *
 * DESCRIPTION
 *	<overall description of what the c code does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/fileContent.c,v 1.47 1995/11/16 01:49:50 russ Exp $
 */

#define NeedFunctionPrototypes 1

#include "fileAccess.h"
#include "fileContent.h"
#include "nvrString.h"
#include "nvrPlx.h"
#include "nvrReason.h"
#include "osio.h"

typedef struct {
    char magic[NVRTYPEHDR];
    char mask[NVRTYPEHDR];
    int magicLength;
    ULong type;
} MagicType;

typedef struct {
    char magic[NVRTYPETRAILER];
    char mask[NVRTYPETRAILER];
    int magicLength;
    ULong type;
} MagicTrailerType;

static MagicType MagicTypesMap[] = {
    {
	{ 'j','_','m','o','v','i','e' },
	{0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff},
	8,
	FT_Jmovie
    },
    {
	{  'n', 'v', 'r', '_', 'p', 'l', 'a', 'y', '_', 'l', 'i', 's', 't' },
	{ 0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff },
	13,
	FT_NvrPlayList
    },
    {
	{ 'n','v','r','_','y','u','v' },
	{0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
	7,
	FT_NvrYuv
    },
    {
	{ 'n','v','r','_','i','d','x' },
	{0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
	7,
	FT_NvrIdx
    },
    {
	{ 0,0,1,0xba },
	{ 0xff,0xff,0xff,0xff },
	4,
	FT_MpegSystem
    },
    {
	{ 0,0,1,0xb3 },
	{ 0xff,0xff,0xff,0xff },
	4,
	FT_MpegVideo
    },
    {
	{ 0x2e,0x73,0x6e,0x64 },
	{ 0xff,0xff,0xff,0xff },
	4,
	FT_SunAudio
    },
    {
	{ 0xff,0xf0 },
	{ 0xff,0xf0 },
	2,
	FT_MpegAudio
    },
    {
	{  'V',  'C', 'R', '\n' },
	{ 0xff, 0xff, 0xff, 0xff },
	4,
	FT_SunVideo
    },
    {
	{  0, 0, 0, 0, 0, 0, 0, 0,  'A',   'I', 'F',  'C' },
	{  0, 0, 0, 0, 0, 0, 0, 0, 0xff, 0xff, 0xff, 0xff },
	12,
	FT_AifcAudio
    },
    {
	{  0,  0,  0,  0,  0,  0,  0,  0, 'A',   'I',  'F',  'F' },
	{  0,  0,  0,  0,  0,  0,  0,  0, 0xff, 0xff, 0xff, 0xff },
	12,
	FT_AiffAudio
    },
    {
	{ 'R', 'I', 'F', 'F',  0, 0, 0, 0,  'A',  'V',  'I',  ' ' },
	{ 0xff,0xff,0xff,0xff, 0, 0, 0, 0, 0xff, 0xff, 0xff, 0xff },
	12,
	FT_Avi
    },
    {
	{  'G', 'I', 'F', '8', '7' },
	{  0xff, 0xff, 0xff, 0xff, 0xff },
	5,
	FT_GIF
    },
    {
	{   'G',  'I',  'F',  '8',  '9' },
	{  0xff, 0xff, 0xff, 0xff, 0xff },
	5,
	FT_GIF
    },
    {
	{  '\377', '\330', '\377' },
	{   0xff,   0xff,   0xff },
	3,
	FT_JPEG
    },
    {
	{ 'J',   'P',  'E',  'G' },
	{  0xff, 0xff, 0xff, 0xff },
	4,
	FT_Miff
    },
    {
	{ 'J',  'P',  'G' },
	{ 0xff, 0xff, 0xff },
	3,
	FT_Miff
    },
    {
	{ 'P', '0' },
	{ 0xff, 0xff },
	2,
	FT_Pnm
    },
    {
	{ 'P', '1' },
	{ 0xff, 0xff },
	2,
	FT_Pnm
    },
    {
	{ 'P', '2' },
	{ 0xff, 0xff },
	2,
	FT_Pnm
    },
    {
	{ 'P', '3' },
	{ 0xff, 0xff },
	2,
	FT_Pnm
    },
    {
	{ 'P', '4' },
	{ 0xff, 0xff },
	2,
	FT_Pnm
    },
    {
	{ 'P', '5' },
	{ 0xff, 0xff },
	2,
	FT_Pnm
    },
    {
	{ 'P', '6' },
	{ 0xff, 0xff },
	2,
	FT_Pnm
    },
    {
	{ 'P', '7' },
	{ 0xff, 0xff },
	2,
	FT_Pnm
    },
    {
	{ 'P', '8' },
	{ 0xff, 0xff },
	2,
	FT_Pnm
    },
    {
	{ 'P', '9' },
	{ 0xff, 0xff },
	2,
	FT_Pnm
    },
    {
	{ '\131', '\246', '\152', '\225' },
	{ 0xff, 0xff, 0xff, 0xff },
	4,
	FT_SunRaster
    },
    {
	{ '\115', '\115' },
	{ 0xff, 0xff },
	2,
	FT_Tiff
    },
    {
	{ '\111', '\111' },
	{ 0xff, 0xff },
	2,
	FT_Tiff
    },
    {
	{ '\122', '\314' },
	{ 0xff, 0xff },
	2,
	FT_Rle
    },
    {
	{ 'L', 'B', 'L', 'S', 'I', 'Z', 'E' },
	{ 0xff, 0xff ,0xff, 0xff, 0xff, 0xff, 0xff },
	7,
	FT_Vicar
    },
    {
	{ 'N', 'J', 'P', 'L', '1', 'I' },
	{ 0xff, 0xff ,0xff, 0xff, 0xff, 0xff },
	6,
	FT_Vicar
    },
    {
	{ 0, 0, 0, '\110', 0, 0, 0, '\007' },
	{ 0xff, 0xff ,0xff, 0xff, 0xff, 0xff, 0xff, 0xff },
	8,
	FT_XWD
    },
    {
	{ 0x01, 0xda },
	{ 0xff, 0xff },
	2,
	FT_SgiRgb
    },
    {
	{ 0xda, 0x01 },
	{ 0xff, 0xff },
	2,
	FT_SgiRgb
    },
    {
        {  'M',  'O',  'V',  'I' },
	{ 0xff, 0xff, 0xff, 0xff },
	4,
	FT_SgiMovie
    },
    {
        {    0,    0,    0,    0,  'm',  'd',  'a',  't' },
        { 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff },
	8,
	FT_QuickTime
    }

};

/*
 * Note that the MagicTypes which occur at the end of the file
 *	are treated slightly differently in that we read NVRTYPETRAILER
 *	number of bytes at  the end of the file for comparison. So
 *	if the value of NVRTYPETRAILER changes the entries in 
 *	the following array must change to match
 */
static MagicTrailerType MagicTrailerTypesMap[] = {
    {
	{ '\244', 'C',  'M', '\245',  'H',   'd', 'r', '\327',
               0,   0,    0,      0,    0,     0,   0,      0,
               0,   0,    0,      0,    0,     0,   0,      0 },
	{  0xff, 0xff, 0xff,   0xff, 0xff, 0xff, 0xff,   0xff,
               0,   0,    0,      0,    0,     0,   0,      0,
               0,   0,    0,      0,    0,     0,   0,      0 },
	24,
	FT_Omf
    }
};

NameEnumPair AllFileTypes[] = {
    { "mpeg_video",      FT_MpegVideo },
    { "mpeg",            FT_MpegVideo },
    { "vmpeg",           FT_MpegVideo },
    { "mpeg_system",     FT_MpegSystem },
    { "jmovie",          FT_Jmovie },
    { "quicktime",       FT_QuickTime },
    { "sgimovie",        FT_SgiMovie },
    { "omf",             FT_Omf },
    { "avi",             FT_Avi },
    { "nvr_yuv",         FT_NvrYuv },
    { "nvr_idx",         FT_NvrIdx },
    { "nvr_play_list",   FT_NvrPlayList },
    { "sun_video",	 FT_SunVideo },

    { "avs",             FT_AVS },
    { "vicar",           FT_Vicar },
    { "xwd",             FT_XWD },
    { "gif",             FT_GIF },
    { "rle",             FT_Rle },
    { "jpeg",            FT_JPEG },
    { "miff",            FT_Miff },
    { "mtv",             FT_MTV },
    { "pnm",             FT_Pnm },
    { "sun_raster",      FT_SunRaster },
    { "sun",             FT_SunRaster },
    { "tiff",            FT_Tiff },
    { "sgirgb",          FT_SgiRgb },

    { "mpeg_audio",      FT_MpegAudio },
    { "sun_audio",       FT_SunAudio },
    { "au",              FT_SunAudio },
    { "aiff",            FT_AiffAudio },
    { "aifc",            FT_AifcAudio },

    { NULL,              FT_Unknown }
};

/*
 * Determine the files type by the data at the beginning of the file
 *	buf contains the first len bytes of the file
 * If FT_Unknown is returned it could mean that not enough data 
 *	was provided to ascertain the file type
 */
FileContentType
GetFileType(char *buf, int len)
{
    int i, j, diff;
    char tmp[NVRTYPEHDR];

    if( len < NVRTYPEHDR ) {
	return FT_Unknown;
    }
    for (i = 0; i < NumberOfCells(MagicTypesMap); i++) {
	for (j = 0; j < MagicTypesMap[i].magicLength; j++) {
	   tmp[j] = buf[j] &  MagicTypesMap[i].mask[j];
        }
	diff = memcmp(tmp,
		      MagicTypesMap[i].magic,
		      MagicTypesMap[i].magicLength);
	if (diff == 0) {
	    return(MagicTypesMap[i].type);
	}
    }
    for (i = 0; i < NumberOfCells(MagicTrailerTypesMap); i++) {
	for (j = 0; j < MagicTrailerTypesMap[i].magicLength; j++) {
	   tmp[j] = buf[j] &  MagicTrailerTypesMap[i].mask[j];
        }
	diff = memcmp(tmp,
		      MagicTrailerTypesMap[i].magic,
		      MagicTrailerTypesMap[i].magicLength);
	if (diff == 0) {
	    return(MagicTrailerTypesMap[i].type);
	}
    }
    return(FT_Unknown);
}

ContentClass
GetClassFromType(FileContentType type)
{
    switch (type) {
    default:
	return (CC_Unknown);

    /* movie types */
    case FT_MpegSystem:
    case FT_Jmovie:
    case FT_QuickTime:
	return (CC_AVMovie);

    case FT_MpegVideo:
    case FT_Avi:
    case FT_NvrYuv:
    case FT_NvrIdx:
    case FT_SunVideo:
    case FT_SgiMovie:	/* will migrate to CC_AVMovie */
    case FT_Omf:	/* will migrate to CC_AVMovie */
	return (CC_Video);

    /* image types */
    case FT_AVS:
    case FT_Vicar:
    case FT_XWD:
    case FT_GIF:
    case FT_Rle:
    case FT_JPEG:
    case FT_Miff:
    case FT_MTV:
    case FT_Pnm:
    case FT_SunRaster:
    case FT_Tiff:
    case FT_SgiRgb:
	return (CC_Image);

    /* audio types */
    case FT_MpegAudio:
    case FT_SunAudio:
    case FT_AiffAudio:
    case FT_AifcAudio:
	return (CC_Audio);
    }
}

