/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	fileContent.h
 *
 * DESCRIPTION
 *	Public include file for fileContent routines.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/fileContent.h,v 1.38 1995/08/30 02:16:36 russ Exp $
 */
#ifndef _fileContent_H_
#define _fileContent_H_

#include "nvrTypes.h"
#include "nvrString.h"
#include "nvrReason.h"

/*
 *	DEFINES
 *
 * These values are returned by FileContentType()
 */
typedef enum {
    FT_Unknown = 0,		/* nothing that we know of (or care about) */

    /* movie types */
    FT_MpegVideo,
    FT_MpegSystem,
    FT_Jmovie,
    FT_QuickTime,
    FT_SgiMovie,
    FT_Avi,
    FT_NvrYuv,
    FT_NvrIdx,
    FT_NvrPlayList,
    FT_SunVideo,
    FT_Omf,

    /* image types */
    FT_AVS,
    FT_Vicar,
    FT_XWD,
    FT_GIF,
    FT_Rle,
    FT_JPEG,
    FT_Miff,
    FT_MTV,
    FT_Pnm,
    FT_SunRaster,
    FT_Tiff,
    FT_SgiRgb,

    /* audio types */
    FT_MpegAudio,
    FT_SunAudio,
    FT_AiffAudio,
    FT_AifcAudio
} FileContentType;

typedef enum {
    EFC_NotCompressed,		/* not compressed */
    EFC_Compressed,		/* compressed with 'compress' */
    EFC_GZipped			/* compressed with 'gzip' */
} ExternalFileCompression;

typedef enum {
    CC_Unknown,
    CC_AVMovie,			/* movie with video and/or audio */
    CC_Video,			/* video sequence */
    CC_Audio,			/* audio sequence */
    CC_Image			/* image file */
} ContentClass;

/*
 * #chars from start of file needed to
 * 	determine various NVR file types
 */
#define NVRTYPEHDR		16
#define NVRTYPETRAILER		24

/*
 *	TYPEDEFS
 */
typedef struct _FileInfo {
    FileContentType type;		/* one of FT_* */
    ContentClass cClass;		/* one of CC_* */
    ExternalFileCompression efc;
} FileInfo, *FileInfoPtr;

/*
 *	EXTERNS
 */

CExternal InternalReasonPtr GetFileContentInfo(
    char *path,
    FileInfoPtr info
);

CExternal ContentClass GetClassFromType(
    FileContentType type
);

CExternal FileContentType GetFileType(
    char *buf,
    int len
);

extern NameEnumPair AllFileTypes[];

#endif /* _fileContent_H_ */
