/*
 * Copyright (c) 1992, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	ioutil.c - RS-232 control routines
 *
 * DESCRIPTION
 *	Library of routines for opening and operating an RS-232 connection.
 *	OS-DEPENDENT
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/ioutil.c,v 1.10 1994/12/17 13:02:53 russ Exp $
 */

/*
 *      INCLUDES
 */
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/errno.h>
#include <signal.h>
#include <setjmp.h>
#include <fcntl.h>
#include <assert.h>

#ifdef _HPUX_SOURCE
# include <termio.h>
#endif

#ifdef sgi
# define SYSV_TTY
#endif

#ifdef Solaris2
# include <unistd.h>
# include <termios.h>
# include <termio.h>
# include <sys/filio.h>
# define SYSV_TTY
#endif

#include "nvrAlloc.h"
#include "nvrString.h"
#include "osioI.h"
#include "ioutil.h"


/*
 *      EXTERNS
 */

extern int errno;


/*
 *      GLOBALS
 */


/*
 *      LOCAL DEFINES
 *		#define declarations local to this package.
 */


/*
 *      LOCAL TYPEDEFS
 *              typedefs local to this package (for use with local vars).
 *
 */


/*
 *      LOCALS VARIABLES
 *		Variables local to this package.
 */


static jmp_buf vmi_time_out_env;
void
vmi_time_out()
{
	longjmp(vmi_time_out_env, 1);
}


/*
 * Convert integer baud rate to system value
 */
#ifdef SYSV_TTY
#define SPEED_19_2	B19200
#define SPEED_38_4	B38400
#else
#define SPEED_19_2	EXTA
#define SPEED_38_4	EXTA
#endif

static int iouMakebaud(baudspec)
int baudspec;
{
	switch( baudspec ) {
		case 0:
			return (B0);
			break;
		case 50:
			return (B50);
			break;
		case 75:
			return (B75);
			break;
		case 110:
			return (B110);
			break;
		case 134:
			return (B134);
			break;
		case 150:
			return (B150);
			break;
		case 200:
			return (B200);
			break;
		case 300:
			return (B300);
			break;
		case 600:
			return (B600);
			break;
		case 1200:
			return (B1200);
			break;
		case 1800:
			return (B1800);
			break;
		case 2400:
			return (B2400);
			break;
		case 4800:
			return (B4800);
			break;
		case 9600:
			return (B9600);
			break;
		case 19200:
			return (SPEED_19_2);
			break;
		case 38400:
			return (SPEED_38_4);
			break;
		default:
			return (-1);
			break;		
		}
	/*NOTREACHED*/
}



int iouOpenTty(iouInfo, tty, baudrate, flags)  /* return fd */
IOUInfo* iouInfo;
char *tty;
int baudrate, flags;
{
	int ttyflag;
	int len = strlen(tty);
	char *ttydev;
#ifdef SYSV_TTY
	struct termio ttyold, ttynew;
#else
	struct sgttyb   tty_params;
#endif
	int speed;
	struct itimerval value, ovalue, junkvalue;
	void (*prev_signal_fn)();

	if((speed = iouMakebaud(baudrate)) < 0 )
		return(-1);

	if((ttydev = SecureAlloc(len + 1)) == NULL)
	{
		return(-1);
	}
	strcpy(ttydev, tty);
	iouInfo->dev_path = ttydev;

	/*
	 * Disable timer and set signal handler for SIGALRM to "time_out".
	 */
	value.it_interval.tv_sec = 0;
	value.it_interval.tv_usec = 0;
	value.it_value.tv_sec = 0;
	value.it_value.tv_usec = 0;
	if (setitimer(ITIMER_REAL, &value, &ovalue))
		assert(IOU_FALSE);
	prev_signal_fn = (void (*)())signal (SIGALRM, vmi_time_out);
	if ((int) prev_signal_fn == -1)
		assert(IOU_FALSE);
	/*
	 * "time_out" will longjmp to here if the call to "open(2)" times-out
	 */
	if (setjmp(vmi_time_out_env)) {
		(void) signal (SIGALRM, prev_signal_fn);
		if (setitimer(ITIMER_REAL, &ovalue, &junkvalue))
			assert(IOU_FALSE);
		return(-1);
	}
	value.it_value.tv_sec = iouInfo->timeout_sec;
	value.it_value.tv_usec = iouInfo->timeout_usec;
	if (setitimer(ITIMER_REAL, &value, &junkvalue))
		assert(IOU_FALSE);

#ifdef UTEKV
    iouInfo->fd = open( tty, O_RDWR, 0);
#else
    iouInfo->fd = open( tty, O_NDELAY | O_RDWR | O_EXCL, 0777 );
#endif /* UTEKV */

    /*
     * If we get here, the open has succeeded without timing out.
     * Re-establish the old signal handler and reset the old time value
     * in the timer.
     */
    (void) signal (SIGALRM, prev_signal_fn);
    if (setitimer(ITIMER_REAL, &ovalue, &junkvalue))
        assert(IOU_FALSE);

    if (iouInfo->fd < 0)
    {
        iouInfo->fd = 0;
        return(-1);
    }
#ifdef SYSV_TTY
    ioctl(iouInfo->fd, TCGETA, &ttyold);
    ioctl(iouInfo->fd, TCGETA, &ttynew);
    if(flags) {
        ttynew.c_lflag &= ~( ICANON | ECHO);
        if(flags & IOU_17BIT)
            ttyflag = CS7;
        else
            ttyflag = CS8;
        if(flags & IOU_ODDPARITY)
            ttyflag |= (PARODD | PARENB);
        if(flags & IOU_EVENPARITY)
            ttyflag |= PARENB;
        ttynew.c_cflag = (unsigned short) (speed | CLOCAL | CREAD) |
                         (unsigned short)ttyflag;
        ttynew.c_cc[4] = 1; /* Set to return after single char entered */
    }
    else /* default RAW, 8 bit, no parity */
    {
        ttynew.c_lflag &= ~( ICANON | ECHO);
        ttynew.c_cflag = (unsigned short) (speed | CS8 | CLOCAL | CREAD);
        ttynew.c_cc[4] = 1; /* Set to return after single char entered */
    }
    ioctl(iouInfo->fd, TCSETAW, &ttynew);
#else
	/* gtty is an outdated interface */
    if (gtty(iouInfo->fd, &tty_params) == -1)
    /* if (ioctl(iouInfo->fd, TIOCGETP, &tty_params) == -1) */
    {
		fprintf(stderr,"iouOpenTty: ioctl (gtty) failed\n");
		return(-1);
    }
    if( !flags ) 
		ttyflag = RAW|TANDEM; /* default: no processing (no parity) */
    else {
		ttyflag = CBREAK;	/* semi-cooked */
		if(flags & IOU_ODDPARITY)
		{
			ttyflag |= ODDP;
		}
		else if(flags & IOU_EVENPARITY)
		{
			ttyflag |= EVENP;
		}
    }
    tty_params.sg_ospeed = speed;	/* the system setting */
    tty_params.sg_ispeed = speed;
    tty_params.sg_flags = ttyflag;

	/* stty is an outdated interface */
    if (stty(iouInfo->fd,&tty_params) == -1)
    /* if (ioctl(iouInfo->fd, TIOCSETP, &tty_params) == -1) */
    {
		perror("iouOpenTty: ioctl (stty)");
		return(-1);
    }
#endif

    iouInfo->protocol = IOU_TTY;
/*
    vdev->SendOK = IOU_TRUE;
    vdev->ReceiveOK = IOU_TRUE;
*/

    return(iouInfo->fd);
}

void iouClose(iouInfo)
IOUInfo* iouInfo;
{
    close(iouInfo->fd);
}


/* This is just like iouSend EXCEPT you send the number of bytes instead
 * of it being inferred from the string length.  This was necessary to allow
 * binary data to be written (a binary 0 falsely triggered end-of-string).
 */
int iouSendBuf(iouInfo, msg, len)  /* return status */
IOUInfo* iouInfo;
char *msg;
int len;
{
    char *p;
    int wrote, bytesToWrite = len;
    struct timeval timeout;
    int nFound;
    fd_set writefds;

    timeout.tv_sec = iouInfo->timeout_sec;
    timeout.tv_usec = iouInfo->timeout_usec;
    FD_ZERO (&writefds);
    FD_SET(iouInfo->fd, &writefds);
    for(;;)
    {
	nFound = select(iouInfo->fd+1,
			(SelectDataPtr)NULL,
			(SelectDataPtr)&writefds,
			(SelectDataPtr)NULL,
			&timeout);
	if (nFound <= 0)
	{
	    if (errno == EINTR)
		continue;
	    Error("Cannot send message to %s after %d seconds: \"%s\"\n",
		iouInfo->fd, iouInfo->timeout_sec, msg);
	    return(-1);
	}
	break;
    }

    bytesToWrite = len;
    p = msg;
    while (bytesToWrite) {
	wrote = write(iouInfo->fd, p, bytesToWrite);
	if (wrote < 0)
	{
	    Error("Only able to send %d out of %d bytes to %s: \"%s\"\n",
		wrote, len, msg);
	    return(-1);
	}
	bytesToWrite -= wrote;
	p += wrote;
    }
    return(0);
}

int iouSend(iouInfo, msg)  /* return status */
IOUInfo* iouInfo;
char *msg;
{
    return( iouSendBuf(iouInfo, msg, strlen(msg)) );
}



/*
 * Flush the input queue.
 */
int iouFlushInput(iouInfo)  /* return bytes flushed ? */
IOUInfo* iouInfo;
{
    int     i;
    int     num;
    char    c;

    if(ioctl(iouInfo->fd, FIONREAD, &num) == -1)
        return (-1);
    for(i=num; i>0; i--) {
        if(read(iouInfo->fd,&c,1) == -1)
            return(-1);
    }
    return (num);
}



/*
 * Read up to num_bytes, buff is the pointer to the input buffer.
 * If fill is True , return only if buffer has num_bytes or timeout occurrs
 * Return either number of bytes read or -1 for timeout or error.
 */
int iouReceive(iouInfo, buf, num_bytes, fill)
IOUInfo* iouInfo;
char *buf;
int  num_bytes;
int  fill;
{
    struct timeval timeout;
    int nFound;
    int bytesIn;
    int nRead = 0;
    int gotIt = IOU_FALSE;
    fd_set readfds;
    char data[ IOU_BUFSIZE ]; /* Temporary data holder for telnet protocol */

    if (fill)
    {
	timeout.tv_sec = iouInfo->timeout_sec;
	timeout.tv_usec = iouInfo->timeout_usec;
    }
    else
    {
	/* wait just long enough that RS-232 data that is already on its
	 * way here can be read
	 */
	timeout.tv_sec = 0;
	timeout.tv_usec = 50;	
    }
    FD_ZERO (&readfds);
    while (! gotIt)
    {
	for (;;) {
	    FD_SET (iouInfo->fd, &readfds);
	    nFound = select(iouInfo->fd+1,
			    (SelectDataPtr)&readfds,
			    (SelectDataPtr)NULL,
			    (SelectDataPtr)NULL,
			    &timeout);
	    if (nFound > 0)
		break;
	    if (nFound < 0)
	    {
		if (errno == EINTR)
		    continue;
		Error("iouReceive select\n");
		return(-1);
	    }
	    if (nFound == 0 && nRead == 0)
		return(-1);	/* timed out */
	}
	if (nFound == 0)
	    break; /* nothing more to read */
        if ((bytesIn = read(iouInfo->fd, &buf[nRead], num_bytes - nRead)) <= 0)
            return(-1);

#ifdef INTENTIONALLY_LEFT_BLANK
        if(iouInfo->protocol == IOU_TELNET)
        {
            /*
             * Strip out Telnet Protocol data.
             */
            bytesIn = TelnetReceive(vdev, &buf[nRead], data, bytesIn);
            strncpy(data, &buf[nRead], bytesIn);
        }
#endif
        nRead += bytesIn;
        if(fill) {
            if(nRead == num_bytes)
                gotIt = IOU_TRUE;
        }
        else
            gotIt = IOU_TRUE;
    }
    return(nRead);	/* no error */
}


void iouSendChar(iouInfo, c)
IOUInfo* iouInfo;
unsigned char c;
{
    char buf[2];

    buf[0] = c;
    buf[1] = '\0';	/* make a message */

    /* vmiDevSend(iouInfo, buf); */
	printf("iouSendChar:: not yet implemented\n");
}


