/*
 *	(c) Copyright 1992 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		machdep.h
 *
 *	DESCRIPTION
 *		machine dependent macros for doing things fast.  Also included
 *		are corresponding portable, machine independent methods.
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/machdep.h,v 1.11 1994/12/18 07:47:58 toddb Exp $
 */
#ifndef _MACHEP_H_
#define _MACHEP_H_

#ifdef sun		/* big endian */
#include "sparcmachdep.h"
#define Pixel0Shift	24
#define Pixel1Shift	16
#define Pixel2Shift	 8
#define Pixel3Shift	 0
#endif

#ifdef sgi		/* big endian */
#include "mipsmachdep.h"
#define Pixel0Shift	24
#define Pixel1Shift	16
#define Pixel2Shift	 8
#define Pixel3Shift	 0
#endif

#ifdef THINK_C		/* big endian */
#include "macmachdep.h"
#define Pixel0Shift	24
#define Pixel1Shift	16
#define Pixel2Shift	 8
#define Pixel3Shift	 0
#endif

#ifdef __MWERKS__		/* big endian */
#define Inline inline
#define Pixel0Shift	24
#define Pixel1Shift	16
#define Pixel2Shift	 8
#define Pixel3Shift	 0
#endif

#ifdef _HPUX_SOURCE	/* big endian */
#define Pixel0Shift	24
#define Pixel1Shift	16
#define Pixel2Shift	 8
#define Pixel3Shift	 0
#endif

#if defined(SPARC_GCC) || defined(MIPS_GCC)

/**************************************************************************
 *
 * Find here a GCC-specific, but processor independent mechanism for
 * passing and returning several values with subroutines.
 *
 **************************************************************************/

typedef void TenValues;
typedef void NineValues;
typedef void EightValues;
typedef void SevenValues;
typedef void SixValues;
typedef void FiveValues;
typedef void FourValues;
typedef void ThreeValues;
typedef void TwoValues;

#undef Inline
#define Inline inline

/* force a register variable 'var' to be filled */
#define ForceVariableLive(var)						\
	asm volatile (MCOMMENT "(%0) var is live" : : "r" ((long)var))

/*
 * Input registers in a called function (used for return values, too)
 */
#define Declare2InRegisters(r1, r2)					\
    register long r1 asm(InReg1);					\
    register long r2 asm(InReg2);

#define Declare3InRegisters(r1, r2, r3)					\
    Declare2InRegisters(r1, r2)						\
    register long r3 asm(InReg3);

#define Declare4InRegisters(r1, r2, r3, r4)				\
    Declare3InRegisters(r1, r2, r3)					\
    register long r4 asm(InReg4);

#define Declare5InRegisters(r1, r2, r3, r4, r5)				\
    Declare4InRegisters(r1, r2, r3, r4)					\
    register long r5 asm(InReg5);

#define Declare6InRegisters(r1, r2, r3, r4, r5, r6)			\
    Declare5InRegisters(r1, r2, r3, r4, r5)				\
    register long r6 asm(InReg6);

#define Declare7InRegisters(r1, r2, r3, r4, r5, r6, r7)			\
    Declare6InRegisters(r1, r2, r3, r4, r5, r6)				\
    register long r7 asm(InReg7);

#define Declare8InRegisters(r1, r2, r3, r4, r5, r6, r7, r8)		\
    Declare7InRegisters(r1, r2, r3, r4, r5, r6, r7)			\
    register long r8 asm(InReg8);

#define Declare9InRegisters(r1, r2, r3, r4, r5, r6, r7, r8, r9)		\
    Declare8InRegisters(r1, r2, r3, r4, r5, r6, r7, r8)			\
    register long r9 asm(InReg9);

#define Declare10InRegisters(r1, r2, r3, r4, r5, r6, r7, r8, r9, r10)	\
    Declare9InRegisters(r1, r2, r3, r4, r5, r6, r7, r8, r9)		\
    register long r10 asm(InReg10);

/*
 * Output registers in a calling function (used to recieve returned
 * values, too.
 */
#define Declare2OutRegisters(r1, r2)					\
    register long r1 asm(OutReg1);					\
    register long r2 asm(OutReg2);

#define Declare3OutRegisters(r1, r2, r3)				\
    Declare2OutRegisters(r1, r2)					\
    register long r3 asm(OutReg3);

#define Declare4OutRegisters(r1, r2, r3, r4)				\
    Declare3OutRegisters(r1, r2, r3)					\
    register long r4 asm(OutReg4);

#define Declare5OutRegisters(r1, r2, r3, r4, r5)			\
    Declare4OutRegisters(r1, r2, r3, r4)				\
    register long r5 asm(OutReg5);

#define Declare6OutRegisters(r1, r2, r3, r4, r5, r6)			\
    Declare5OutRegisters(r1, r2, r3, r4, r5)				\
    register long r6 asm(OutReg6);

#define Declare7OutRegisters(r1, r2, r3, r4, r5, r6, r7)		\
    Declare6OutRegisters(r1, r2, r3, r4, r5, r6)			\
    register long r7 asm(OutReg7);

#define Declare8OutRegisters(r1, r2, r3, r4, r5, r6, r7, r8)		\
    Declare7OutRegisters(r1, r2, r3, r4, r5, r6, r7)			\
    register long r8 asm(OutReg8);

#define Declare9OutRegisters(r1, r2, r3, r4, r5, r6, r7, r8, r9)	\
    Declare8OutRegisters(r1, r2, r3, r4, r5, r6, r7, r8)		\
    register long r9 asm(OutReg9);

#define Declare10OutRegisters(r1, r2, r3, r4, r5, r6, r7, r8, r9, r10)	\
    Declare9OutRegisters(r1, r2, r3, r4, r5, r6, r7, r8, r9)		\
    register long r10 asm(OutReg10);

#define Get9Arguments(a1t,a1,a2t,a2,a3t,a3,a4t,a4,a5t,a5,a6t,a6,a7t,a7,a8t,a8,a9t,a9) \
    a1t a1;								\
    a2t a2;								\
    a3t a3;								\
    a4t a4;								\
    a5t a5;								\
    a6t a6;								\
    a7t a7;								\
    a8t a8;								\
    a9t a9;								\
    {									\
	register long reg1 asm(InReg1);					\
	register long reg2 asm(InReg2);					\
	register long reg3 asm(InReg3);					\
	register long reg4 asm(InReg4);					\
	register long reg5 asm(InReg5);					\
	register long reg6 asm(InReg6);					\
	register long reg7 asm(InReg7);					\
	register long reg8 asm(InReg8);					\
	register long reg9 asm(InReg9);					\
									\
	asm volatile (MCOMMENT "reg1-9 are %0,%1,%2,%3,%4,%5,%6,%7,%8"	\
		      : "=r" (reg1), "=r" (reg2), "=r" (reg3),		\
		        "=r" (reg4), "=r" (reg5), "=r" (reg6),		\
		        "=r" (reg7), "=r" (reg8), "=r" (reg9));		\
	a1 = (a1t)reg1;							\
	a2 = (a1t)reg2;							\
	a3 = (a1t)reg3;							\
	a4 = (a1t)reg4;							\
	a5 = (a1t)reg5;							\
	a6 = (a1t)reg6;							\
	a7 = (a1t)reg7;							\
	a8 = (a1t)reg8;							\
	a9 = (a1t)reg9;							\
	asm volatile (MCOMMENT "now: a1-9 in %0,%1,%2,%3,%4,%5,%6,%7,%8"\
		      :: "r" (a1), "r" (a2), "r" (a3),			\
		         "r" (a4), "r" (a5), "r" (a6),			\
		         "r" (a7), "r" (a8), "r" (a9));			\
    }

#define _Accept2(r1,r2)							\
    {									\
	asm volatile ("" ::: OutReg1, OutReg2);	/* clobber */		\
	{								\
	    Declare2OutRegisters(reg1, reg2);				\
	    (long)r1 = reg1;						\
	    (long)r2 = reg2;						\
	    asm volatile (MCOMMENT "r1,r2 in %0,%1"			\
		:: "r" (r1), "r" (r2));					\
	}								\
    }

#define Accept2(r1,r2, functionCall)					\
    functionCall;							\
    _Accept2(r1,r2)

#define _Accept3(r1,r2,r3)						\
    {									\
	asm volatile ("" ::: OutReg1,OutReg2,OutReg3);			\
	{								\
	    Declare3OutRegisters(reg1, reg2, reg3)			\
	    (long)r1 = reg1;						\
	    (long)r2 = reg2;						\
	    (long)r3 = reg3;						\
	}								\
	asm volatile (MCOMMENT "r1,r2,r3 in %0,%1,%2"			\
	    :: "r" (r1), "r" (r2), "r" (r3));				\
    }

#define Accept3(r1,r2,r3, functionCall)					\
    functionCall;							\
    _Accept3(r1,r2,r3)

#define _Accept4(r1,r2,r3,r4)						\
    {									\
	asm volatile ("" ::: OutReg1,OutReg2,OutReg3,OutReg4);			\
	{								\
	    Declare4OutRegisters(reg1, reg2, reg3, reg4)		\
	    (long)r1 = reg1;						\
	    (long)r2 = reg2;						\
	    (long)r3 = reg3;						\
	    (long)r4 = reg4;						\
	}								\
	asm volatile (MCOMMENT "1,r2,r3,r4 in %0,%1,%2,%3"		\
	    :: "r" (r1), "r" (r2), "r" (r3), "r" (r4));			\
    }

#define Accept4(r1,r2,r3,r4, functionCall)				\
    functionCall;							\
    _Accept4(r1,r2,r3,r4)

#define _Accept5(r1,r2,r3,r4,r5)					\
    {									\
	asm volatile ("" ::: OutReg1,OutReg2,OutReg3,OutReg4,OutReg5);		\
	{								\
	    Declare5OutRegisters(reg1, reg2, reg3, reg4, reg5)		\
	    (long)r1 = reg1;						\
	    (long)r2 = reg2;						\
	    (long)r3 = reg3;						\
	    (long)r4 = reg4;						\
	    (long)r5 = reg5;						\
	}								\
	asm volatile (MCOMMENT "1,r2,r3,r4,r5 in %0,%1,%2,%3,%4"	\
	    :: "r" (r1), "r" (r2), "r" (r3), "r" (r4), "r" (r5));	\
    }

#define Accept5(r1,r2,r3,r4,r5, functionCall)				\
    functionCall;							\
    _Accept5(r1,r2,r3,r4,r5)

#define _Accept6(r1,r2,r3,r4,r5,r6)					\
    {									\
	asm volatile ("" ::: OutReg1,OutReg2,OutReg3,OutReg4);		\
	asm volatile ("" ::: OutReg5,OutReg6,OutReg7,OutReg8);		\
	{								\
	    Declare6OutRegisters(reg1,reg2,reg3,reg4,reg5,reg6)		\
	    (long)r1 = reg1;						\
	    (long)r2 = reg2;						\
	    (long)r3 = reg3;						\
	    (long)r4 = reg4;						\
	    (long)r5 = reg5;						\
	    (long)r6 = reg6;						\
	}								\
	asm volatile (MCOMMENT "1,r2,r3,r4,r5,r6, in");			\
	asm volatile (MCOMMENT "0,%1,%2,%3,%4,%5"			\
	    :: "r" (r1), "r" (r2), "r" (r3), "r" (r4),			\
	       "r" (r5), "r" (r6));					\
    }

#define Accept8(r1,r2,r3,r4,r5,r6,r7,r8, functionCall)			\
    functionCall;							\
    _Accept8(r1,r2,r3,r4,r5,r6,r7,r8)

#define _Accept8(r1,r2,r3,r4,r5,r6,r7,r8)				\
    {									\
	{								\
	    Declare8OutRegisters(reg1,reg2,reg3,reg4,reg5,reg6,reg7,reg8)\
	    asm volatile (MCOMMENT "eturn values in %0,%1,%2,%3,%4,%5,%6,%7"\
		: "=r" (reg1), "=r" (reg2), "=r" (reg3), "=r" (reg4),	\
		  "=r" (reg5), "=r" (reg6), "=r" (reg7), "=r" (reg8));	\
	    (long)r1 = reg1;						\
	    (long)r2 = reg2;						\
	    (long)r3 = reg3;						\
	    (long)r4 = reg4;						\
	    (long)r5 = reg5;						\
	    (long)r6 = reg6;						\
	    (long)r7 = reg7;						\
	    (long)r8 = reg8;						\
	}								\
	asm volatile (MCOMMENT "r1,r2,r3,r4,r5,r6,r7,r8 in");		\
	asm volatile (MCOMMENT "%0,%1,%2,%3,%4,%5,%6,%7"		\
	    :: "r" (r1), "r" (r2), "r" (r3), "r" (r4),			\
	       "r" (r5), "r" (r6), "r" (r7), "r" (r8));			\
    }

#define Accept8(r1,r2,r3,r4,r5,r6,r7,r8, functionCall)			\
    functionCall;							\
    _Accept8(r1,r2,r3,r4,r5,r6,r7,r8)

#define _Accept10(r1,r2,r3,r4,r5,r6,r7,r8,r9,r10)			\
    {									\
	asm volatile ("" ::: OutReg1,OutReg2,OutReg3,OutReg4,OutReg5);	\
	asm volatile ("" ::: OutReg6,OutReg7,OutReg8,OutReg9,OutReg10);	\
	{								\
	    Declare10OutRegisters(reg1,reg2,reg3,reg4,reg5,		\
				  reg6,reg7,reg8,reg9,rega)		\
	    (long)r1 = reg1;						\
	    (long)r2 = reg2;						\
	    (long)r3 = reg3;						\
	    (long)r4 = reg4;						\
	    (long)r5 = reg5;						\
	    (long)r6 = reg6;						\
	    (long)r7 = reg7;						\
	    (long)r8 = reg8;						\
	    (long)r9 = reg9;						\
	    (long)r10 = rega;						\
	}								\
	asm volatile (MCOMMENT "r1,r2,r3,r4,r5,r6,r7,r8,r9,ra in");	\
	asm volatile (MCOMMENT "%0,%1,%2,%3,%4,%5,%6,%7,%8"		\
	    :: "r" (r1), "r" (r2), "r" (r3), "r" (r4),			\
	       "r" (r5), "r" (r6), "r" (r7), "r" (r8),			\
	       "r" (r9), "r" (r10));					\
    }

#define Accept10(r1,r2,r3,r4,r5,r6,r7,r8,r9,r10, functionCall)		\
    functionCall;							\
    _Accept10(r1,r2,r3,r4,r5,r6,r7,r8,r9,r10)

#define Return2(r1,r2)							\
    {									\
	Declare2InRegisters(reg1, reg2)					\
	reg1 = (long)(r1);						\
	reg2 = (long)(r2);						\
	asm volatile (MCOMMENT "return r1,r2 in %0,%1"			\
	    :: "r" (reg1), "r" (reg2));					\
	return;								\
    }

#define Return4(r1,r2,r3,r4)					\
    {									\
	Declare4InRegisters(reg1, reg2, reg3, reg4)			\
	reg1 = (long)(r1);						\
	reg2 = (long)(r2);						\
	reg3 = (long)(r3);						\
	reg4 = (long)(r4);						\
	asm volatile (MCOMMENT "return r1,r2,r3,r4 in %0,%1,%2,%3"	\
	    :: "r" (reg1), "r" (reg2), "r" (reg3), "r" (reg4));		\
	return;								\
    }

#define Return3(r1,r2,r3)						\
    {									\
	Declare3InRegisters(reg1, reg2, reg3)				\
	reg1 = (long)(r1);						\
	reg2 = (long)(r2);						\
	reg3 = (long)(r3);						\
	asm volatile (MCOMMENT "return r1,r2,r3 in %0,%1,%2"		\
	    :: "r" (reg1), "r" (reg2), "r" (reg3));			\
	return;								\
    }

#define Return5(r1,r2,r3,r4,r5)						\
    {									\
	Declare5InRegisters(reg1, reg2, reg3, reg4, reg5)		\
	reg1 = (long)(r1);						\
	reg2 = (long)(r2);						\
	reg3 = (long)(r3);						\
	reg4 = (long)(r4);						\
	reg5 = (long)(r5);						\
	asm volatile (MCOMMENT "return r1,r2,r3,r4,r5 in %0,%1,%2,%3,%4"\
	    :: "r" (reg1), "r" (reg2), "r" (reg3), "r" (reg4),		\
	       "r" (reg5));						\
	return;								\
    }

#define Return6(r1,r2,r3,r4,r5,r6)					\
    {									\
	Declare6InRegisters(reg1, reg2, reg3, reg4, reg5, reg6)		\
	reg1 = (long)(r1);						\
	reg2 = (long)(r2);						\
	reg3 = (long)(r3);						\
	reg4 = (long)(r4);						\
	reg5 = (long)(r5);						\
	reg6 = (long)(r6);						\
	asm volatile (MCOMMENT "return r1,r2,r3,r4,r5,r6 in %0,%1,%2,%3,%4,%5"\
	    :: "r" (reg1), "r" (reg2), "r" (reg3), "r" (reg4),		\
	       "r" (reg5), "r" (reg6));					\
	return;								\
    }

#define Return8(r1,r2,r3,r4,r5,r6,r7,r8)				\
    {									\
	Declare8InRegisters(reg1,reg2,reg3,reg4,reg5,reg6,reg7,reg8)	\
	reg1 = (long)(r1);						\
	reg2 = (long)(r2);						\
	reg3 = (long)(r3);						\
	reg4 = (long)(r4);						\
	reg5 = (long)(r5);						\
	reg6 = (long)(r6);						\
	reg7 = (long)(r7);						\
	reg8 = (long)(r8);						\
	asm volatile (MCOMMENT "ret r1,r2,r3,r4,r5,r6,r7,r8 in");	\
	asm volatile (MCOMMENT "%0,%1,%2,%3,%4,%5,%6,%7"		\
	    :: "r" (reg1), "r" (reg2), "r" (reg3), "r" (reg4),		\
	       "r" (reg5), "r" (reg6), "r" (reg7), "r" (reg8));		\
	return;								\
    }

#define Return10(r1,r2,r3,r4,r5,r6,r7,r8,r9,r10)			\
    {									\
	Declare10InRegisters(reg1,reg2,reg3,reg4,reg5,reg6,reg7,reg8,reg9,rega)\
	reg1 = (long)(r1);						\
	reg2 = (long)(r2);						\
	reg3 = (long)(r3);						\
	reg4 = (long)(r4);						\
	reg5 = (long)(r5);						\
	reg6 = (long)(r6);						\
	reg7 = (long)(r7);						\
	reg8 = (long)(r8);						\
	reg9 = (long)(r9);						\
	rega = (long)(r10);						\
	asm volatile (MCOMMENT "ret r1,r2,r3,r4,r5,r6,r7,r8,r9,ra in");	\
	asm volatile (MCOMMENT "%0,%1,%2,%3,%4,%5,%6,%7,%8,%9"			\
	    :: "r" (reg1), "r" (reg2), "r" (reg3), "r" (reg4),		\
	       "r" (reg5), "r" (reg6), "r" (reg7), "r" (reg8),		\
	       "r" (reg9), "r" (rega));					\
	return;								\
    }

#define Pass8NameFunc(f, a1, a2, a3, a4, a5, a6, a7, a8)		\
    _Pass8(f, "g", a1, a2, a3, a4, a5, a6, a7, a8)

#define Pass8VarFunc(f, a1, a2, a3, a4, a5, a6, a7, a8)			\
    _Pass8(f, "r", a1, a2, a3, a4, a5, a6, a7, a8)

#define Pass9Mem12NameFunc(f, a1, a2, a3, a4, a5, a6, a7, a8, a9)	\
    _Pass9Mem12(f, "g", a1, a2, a3, a4, a5, a6, a7, a8, a9)

#define Pass9Mem12VarFunc(f, a1, a2, a3, a4, a5, a6, a7, a8, a9)	\
    _Pass9Mem12(f, "r", a1, a2, a3, a4, a5, a6, a7, a8, a9)

#define Pass9Mem1234NameFunc(f, m1, m2, m3, m4, a5, a6, a7, a8, a9)	\
    _Pass9Mem1234(f, "g", m1, m2, m3, m4, a5, a6, a7, a8, a9)

#define Pass9Mem1234VarFunc(f, m1, m2, m3, m4, a5, a6, a7, a8, a9)	\
    _Pass9Mem1234(f, "r", m1, m2, m3, m4, a5, a6, a7, a8, a9)

#define Pass9NameFunc(f, a1, a2, a3, a4, a5, a6, a7, a8, a9)		\
    _Pass9(f, "g", a1, a2, a3, a4, a5, a6, a7, a8, a9)

#define Pass9VarFunc(f, a1, a2, a3, a4, a5, a6, a7, a8, a9)		\
    _Pass9(f, "r", a1, a2, a3, a4, a5, a6, a7, a8, a9)

#define Pass10NameFunc(f, a1, a2, a3, a4, a5, a6, a7, a8, a9, aa)	\
    _Pass10(f, "g", a1, a2, a3, a4, a5, a6, a7, a8, a9, aa)

#define Pass10VarFunc(f, a1, a2, a3, a4, a5, a6, a7, a8, a9)		\
    _Pass10(f, "r", a1, a2, a3, a4, a5, a6, a7, a8, a9)

#define Pass8NameFuncAccept6(f,a1,a2,a3,a4,a5,a6,a7,a8,r1,r2,r3,r4,r5,r6) \
    Pass8NameFunc(f,a1,a2,a3,a4,a5,a6,a7,a8)				\
    _Accept6(r1,r2,r3,r4,r5,r6)

#define Pass9Mem1234NameFuncAccept8(f, m1,m2,m3,m4,a5,a6,a7,a8,a9, r1,r2,r3,r4,r5,r6,r7,r8) \
    Pass9Mem1234NameFunc(f, m1,m2,m3,m4,a5,a6,a7,a8,a9)			\
    _Accept8(r1,r2,r3,r4,r5,r6,r7,r8)

#define Pass10NameFuncAccept8(f,a1,a2,a3,a4,a5,a6,a7,a8,a9,aa,r1,r2,r3,r4,r5,r6,r7,r8) \
    Pass10NameFunc(f,a1,a2,a3,a4,a5,a6,a7,a8,a9,aa)			\
    _Accept8(r1,r2,r3,r4,r5,r6,r7,r8)

#endif /* SPARC_GCC || MIPS_GCC */

#endif /* _MACHEP_H_ */
