/*
 * Copyright (c) 1992, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	nvrAlloc.c
 *
 * DESCRIPTION
 *	various functions for allocating memory
 *	OS-DEPENDENT
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrAlloc.c,v 1.26 1995/06/19 04:59:37 toddb Exp $
 */
#define NeedFunctionPrototypes 1

#include "nvrTypes.h"
#include "nvrAlloc.h"
#include "nvrString.h"
#include "osio.h"


#if defined(sgi)||defined(_HPUX_SOURCE)||defined(VXWORKS)

char *memalign(int alignment, int size)
{
    ULong ptr;

    size += alignment - 1;
    if ((ptr = (ULong)malloc(size)) != 0) {
	/* XXX if this changes ptr, and is later freed, this will die! */
	ptr = (ptr + alignment - 1) & -alignment;
    }

    return ((char *)ptr);
}
#endif /* defined(sgi, _HPUX_SOURCE, VXWORKS */

char *SecureAllocAligned(int alignment, int size)
{
    char *memalign();
    char *ptr;

    if (size == 0)
	size++;
    ptr = memalign(alignment, size);
    if (ptr == NULL)
        Fatal("can't allocate %d bytes, aligned on %d\n", size, alignment);
    return (ptr);
}

char *InsecureAlloc(int size)
{
    char *ptr;

    if (size == 0)
	size++;
    ptr = (char *)malloc(size);
    return (ptr);
}

char *
AllocNamedMem(int size, char *name, InternalReasonPtr *pIrp)
{
    char *ptr;

    if (size == 0)
	size++;
    ptr = (char *)malloc(size);
    if (ptr == NULL)
	*pIrp = AllocReason(name, size);
    else
	memset(ptr, 0, size);
    
    return (ptr);
}

char *SecureAlloc(int size)
{
    char *ptr;

    if (size == 0)
	size++;
    ptr = (char *)malloc(size);
    if (ptr == NULL)
        Fatal("can't allocate %d bytes\n", size);
    return (ptr);
}

char *CopyOfString(char *string)
{
    char *newString = InsecureAlloc(strlen(string)+1);

    if (newString)
	(void) strcpy(newString, string);
    return (newString);
}

char *InsecureNullRealloc(char *ptr, unsigned size)
{

    if (size == 0)
	size++;
    if (ptr == NULL)
        ptr = (char *)malloc(size);
    else
	ptr = (char *)realloc(ptr, size);
    return (ptr);
}

char *NullRealloc(char *ptr, unsigned size)
{

    ptr = InsecureNullRealloc(ptr, size);
    if (ptr == NULL)
	Fatal("can't reallocate %d bytes\n", size);
    return (ptr);
}

#include <signal.h>
#include "longjump.h"

StackInfo BeforeLoop;

/*
 * Make sure all data and text pages are in memory.
 * Stack is ignored.  The return value is meaningless, but is used
 * to prevent the compiler from optimizing away the accesses to memory.
 */
#ifdef sun
# ifdef Solaris2
#  define TEXT_BASE 0x2000
#  define SignalFunc sigset
#  define ReinstallHandler NO
#  define getpagesize() sysconf(_SC_PAGESIZE)
   extern long sysconf();
# else
#  define TEXT_BASE 0x2000
#  define SignalFunc signal
#  define ReinstallHandler NO
   extern int getpagesize();
# endif
#endif

#ifdef sgi
# define SignalFunc signal
# define ReinstallHandler YES
# define TEXT_BASE	0
  extern int getpagesize();
#endif

#ifdef _HPUX_SOURCE
# define SignalFunc signal
# define ReinstallHandler YES
# define getpagesize() 4096
# define TEXT_BASE	0x1000
#endif

static void
BadMemoryAddress(int signum)
{
#if ReinstallHandler
    SignalFunc(signum,BadMemoryAddress);
#endif
    longjmp(BeforeLoop, 1);
}

#ifdef TEXT_BASE
int
PageInMemory(void)
{
#if 0
    extern UChar *sbrk(int increment);
#endif /* 0 */
    static UChar *baseAddress = UCharP(TEXT_BASE);
    int pageSize;
    int junk;
    UChar *start, *end; 
    VoidFuncPtr oldBusHandler;
    VoidFuncPtr oldFaultHandler;

    oldBusHandler = SignalFunc(SIGBUS, BadMemoryAddress);
    if (oldBusHandler == (VoidFuncPtr)-1) {
	Error("can't set bus error handler\n");
	return(0);
    }

    oldFaultHandler = SignalFunc(SIGSEGV, BadMemoryAddress);
    if (oldFaultHandler == (VoidFuncPtr)-1) {
	(void) SignalFunc(SIGSEGV, oldBusHandler);
	Error("can't set seg fault error handler\n");
	return(0);
    }

    pageSize = getpagesize();
    start = baseAddress;
    end = (UChar *)sbrk(0);

    while (SaveContext(BeforeLoop) != 0) {
	baseAddress = (start += pageSize);
    }
	
    junk = 0;
    while (start < end) {
	junk += *start;
	start += pageSize;
    }

    (void) SignalFunc(SIGBUS, oldBusHandler);
    (void) SignalFunc(SIGSEGV, oldFaultHandler);

#   ifdef sun
    {
	char buf[1024];
	sprintf(buf, "%f", 1.0);	/* long time for the first time */
    }
#   endif
    return(junk);
}

#else /* TEXT_BASE */

int PageInMemory(void) { return(0); }

#endif /* TEXT_BASE */
