/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	<nvrAsyncIO.h> - os-independent async IO interface
 *
 * DESCRIPTION
 *      The intent is for the user to repeatedly call AsyncRead; the reads
 *      are returned in the order requested through the
 *      user's AsyncIOCB function.  When the user has finished with
 *      The AsyncIONotifyPtr, ReleaseAsyncNotify() should be called to
 *      release it for future use by the susystem.
 *
 *      If the service does not exist for the OS, QueryAsyncIO() will
 *      return false.
 * 
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrAsyncIO.h,v 1.1 1995/08/12 19:06:02 toddb Exp $
 */
#ifndef _NVRASYNCIO_H_
#define _NVRASYNCIO_H_

#include "osio.h"
#include "nvrReason.h"

typedef struct _asyncIONotify AsyncIONotify, *AsyncIONotifyPtr;

typedef void (*AsyncIOCB)(AsyncIONotifyPtr);

struct _asyncIONotify {
    FileHandle fd;		/* file descriptor for request */
    char *buf;			/* buffer must be at least 'length' bytes */
    int reqLength;		/* requested length of this request */
    int length;			/* returned length of this request */
    InternalReasonPtr error;	/* if length==0, then error contains error */
};

CExternal boolean QueryNvrAsyncIO(
    void
);

CExternal InternalReasonPtr NvrAsyncRead(
    FileHandle,
    char *buf,
    int length,
    AsyncIOCB cb
);

CExternal void ReleaseNvrAsyncNotify(
    AsyncIONotifyPtr
);

#endif /* _NVRASYNCIO_H_ */
