/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              nvrClock.h - clock interfaces (nvrTimer.h is old interface)
 *
 *      DESCRIPTION
 *              clock handling stuff
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/nvrClock.h,v 1.3 1996/01/03 02:34:41 toddb Exp $
 */
#ifndef _NVRCLOCK_H_
#define _NVRCLOCK_H_

#include "nvrTypes.h"
#include "nvrReason.h"

/*
 *      DEFINES
 */
#ifndef MicroSecPerSec
#define MicroSecPerSec          1000000
#define MilliSecPerSecond          1000
#endif

typedef struct _clockKeep ClockKeep, *ClockKeepPtr;
struct _clockKeep {
    ULong resolution;		/* read-only: clock resolution in ticks/sec */
    double secPerTick;		/* read-only: 1 / resolution */
    short curClock;		/* read-only: current clock index */

    /*
     * member functions
     */
    void (*init)(ClockKeepPtr);	/* zero all the clocks */
    ULong (*gettime)(ClockKeepPtr);/* get time since call to (*->init)() */
    void (*update)(ClockKeepPtr, /* update current clock, switch to new one */
		   short clock);
    void (*free)(ClockKeepPtr);	/* destroy this structure */

    /*
     * first clock in a list of clocks.  After a call to (*->update)() or
     * (*->switch)(), the sum of all the clocks is supposed to be <=
     * the value returned from (*gettime)(), different only by the time between
     * calls to update, switch and gettime.  Everthing is reset
     * after a call to (*->init)().
     */
    ULong clocks[1];
};

/*
 * Create a new structure
 */
CExternal ClockKeepPtr CreateClockKeep(
    short nClocks,
    long resolution,
    InternalReasonPtr *pIrp
);

#endif /* _NVRCLOCK_H_ */
