/*
 *	(c) Copyright 1992 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		nvrDebug.h
 *
 *	DESCRIPTION
 *		<overall description of what the package does>
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/nvrDebug.h,v 3.20 1996/01/02 22:18:29 toddb Exp $
 */
#ifndef _DEBUG_H_
#define _DEBUG_H_

/*
 * These values define the base and range of each allocated
 * debugging name space.  DBI: Debug Base Index
 */

#define DBI_Span	100
#define DBI_Vmpegcomp	  0
#define DBI_Vmpegdecomp	  (DBI_Vmpegcomp + DBI_Span)
#define DBI_Ampegcomp	  (DBI_Vmpegdecomp + DBI_Span)
#define DBI_Ampegdecomp	  (DBI_Ampegcomp + DBI_Span)
#define DBI_MpegSmux	  (DBI_Ampegdecomp + DBI_Span)
#define DBI_MpegSdemux	  (DBI_MpegSmux + DBI_Span)
#define DBI_Audio	  (DBI_MpegSdemux + DBI_Span)
#define DBI_Image	  (DBI_Audio + DBI_Span)
#define DBI_Packet	  (DBI_Image + DBI_Span)
#define DBI_SgiJpeg	  (DBI_Packet + DBI_Span)
#define DBI_SgiVl	  (DBI_SgiJpeg + DBI_Span)
#define DBI_SanyoGVR	  (DBI_SgiVl + DBI_Span)
#define DBI_IDct	  (DBI_SanyoGVR + DBI_Span)
#define DBI_MpegSched	  (DBI_IDct + DBI_Span)
#define DBI_Last	  (DBI_MpegSched + DBI_Span)

typedef struct _flagNames FlagNames, *FlagNamesPtr;
typedef struct _flagMacro FlagMacro, *FlagMacroPtr;
typedef struct _flagMacroList FlagMacroList, *FlagMacroListPtr;
typedef struct _flagLables FlagLabels, *FlagLabelsPtr;

struct _flagMacro {
    char *name;		/* name of the macro */
    UShort *bitNumbers;	/* a list of bit numbers */
    long nBits;		/* number of entries in bitNumbers */
};

struct _flagMacroList {
    FlagMacroPtr list; /* list of macros */
    UChar nMacros;	  /* number of macros */
};

struct _flagNames {
    long baseIndex;	  /* Base index, from DBI_* */
    char **names;	  /* list of flag names (name index==bit number) */
    UChar nNames;	  /* number of names */
};

struct _flagLables {
    FlagNamesPtr flags;	  /* list of flag names (name index==bit number) */
    FlagMacroListPtr macros; /* list of macros */
    char *title;	  /* used for UI representation */
};

#define _DoAssert(mustBeTrue)						\
	if (! (mustBeTrue))						\
	    Fatal("Assert failed:\n  %s\n  in \"%s\", line %d\n",	\
		"\"mustBeTrue\"", __FILE__, __LINE__)

#ifdef DEBUG_ENABLED

#define Optional(statement)	statement

#define DebugTask(whatIsIt, task)					\
    if (GetDebugBit(whatIsIt)) {					\
	task;								\
    }									\

#define WhenDebugEnabled(task) { task; }
#define DebugDeclare(declarations) declarations;

#define SetDebugBit(bitnum) DebugFlags[(bitnum)>>5] |= (1<<((bitnum) & 0x1f))
#define GetDebugBit(bitnum) (DebugFlags[(bitnum)>>5] & (1<<((bitnum) & 0x1f)))

extern ULong DebugFlags[ (DBI_Last + 31) >> 5 ];
extern FlagLabels DebugFlagDesc[];


CExternal void SetDebugFileFlags(
    ULong *flags,
    FlagLabels *list,	/* null-terminated */
    char *fileName
);

CExternal char *FindDebugFlagName(
    int bit
);

CExternal void SetFlagName(
    ULong *flags,
    FlagLabels *list,	/* null-terminated */
    char *flagName
);

CExternal char *FixedPointToA(
    int,		/* value */
    int			/* fixedPointBits */
);

CExternal char *BitString(
    int,		/* value */
    int			/* nBits */
);

#else

#define SetFlagName(flags, labels, name)		/* nothing */
#define FindDebugFlagName(bitnum)			/* nothing */
#define SetDebugFileFlags(flags, labels, file)		/* nothing */
#define Optional(statement)				/* nothing */
#define DebugTask(whatIsIt, task)			/* nothing */
#define WhenDebugEnabled(task)				/* nothing */
#define DebugDeclare(task)				/* nothing */
#define SetDebugBit(bitnum)				/* nothing */
#define GetDebugBit(bitnum)				/* nothing */

#endif

#endif /* _DEBUG_H_ */
