/*
 *	(c) Copyright 1992 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		<name> - <one-line description of package>
 *
 *	DESCRIPTION
 *		Queue management macros.
 *		We put items on the head and take them off the tail.
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/nvrQueue.h,v 3.8 1995/06/19 05:00:17 toddb Exp $
 */
#ifndef _QUEUE_H_
#define _QUEUE_H_

#include "nvrTypes.h"
#include "nvrReason.h"

/*
 *	TYPEDEFS
 */

typedef ULong *QItemPtr;
typedef ULong QItem;

typedef struct {
    long nitems;		/* number if items in array */
    long lowWaterMark;		/* smallest number in queue */
    long nInserted;		/* total # of items placed the queue */
    long nRemoved;		/* total # of items removed from the queue */
    QItemPtr array;		/* The queue's buffer; its base */
    QItemPtr last;		/* the last entry in the buffer */
    QItemPtr head;		/* put new items here, then increment */
    QItemPtr tail;		/* take old items from here */
} Queue, *QueuePtr;



/*
 *	DEFINES
 */

#define QueuedPrevious(pitem, q)				\
    ((QItemPtr)(pitem) == (q)->array	? (q)->last : ((QItemPtr)(pitem))-1)

#define QueueOccupancy(q)	((q)->nInserted - (q)->nRemoved)
#define QueueFull(q)		(QueueOccupancy(q) >= (q)->nitems)
#define QueueEmpty(q)		(QueueOccupancy(q) <= 0)
#define TailOfQueue(q, type)	((type)*(q)->tail)
#define HeadOfQueue(q, type)	((type)*QueuedPrevious((q)->head,q))

/*
 * Queue operations.
 * Note that if there are two agents that operate on the queue,
 * one should operate on the head (insert) and one on the tail (remove).
 */

#define RemoveQueueTail(q)					\
    {								\
	if ((q)->tail == (q)->last)				\
	    (q)->tail = (q)->array;				\
	else							\
	    (q)->tail++;					\
	(q)->nRemoved++;					\
    }

#define InsertQueueHead(q, item)				\
    {								\
	*(q)->head = (QItem)(item);				\
	if ((q)->head == (q)->last)				\
	    (q)->head = (q)->array;				\
	else							\
	    (q)->head++;					\
	(q)->nInserted++;					\
    }

#define ReplaceQueueHead(q, item)				\
	(*QueuedPrevious((q)->head,q) = (QItem) item)

/*
 *	EXTERNS
 */

CExternal InternalReasonPtr InitQueue(
    QueuePtr,	/* pointer to queue */
    long	/* queue length */
);

CExternal InternalReasonPtr StretchQueue(
    QueuePtr,	/* pointer to queue */
    long	/* queue length */
);

CExternal void UninitQueue(
    QueuePtr
);

#endif /* _QUEUE_H_ */
