/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	<nvrReason.h> - internal reason return values
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrReason.h,v 1.12 1995/11/15 00:20:41 toddb Exp $
 */
#ifndef _NVRREASON_H_
#define _NVRREASON_H_

/*
 **************************************************************************
 *
 * If any part of the some software fails, it may return one of these.  It is
 * intended to contain enough information for the UI at the top level
 * to convey the problem; i.e., it is incomplete :-)
 *
 **************************************************************************
 */
typedef enum {
    IR_InternalProblem,		/* internal error */
    IR_NotImplemented,		/* feature not yet implememented */
    IR_Unknown,			/* too complicated */
    IR_PrematureEof,		/* Unexpected End of file */
    IR_CantWrite,		/* can't write output */
    IR_UnknowFileType,		/* unknown file type */
    IR_ShortRead,
    IR_IoBlocked,		/* non-blocking I/O returned failure */
    IR_BadSampleRate,		/* e.g., not 4:2:2 or 4:2:0, etc */
    IR_BadColor,		/* bad color space */
    IR_BadAlloc,		/* malloc gone bad */
    IR_BadConversion,		/* bad color conversion */
    IR_BadDimensions,		/* bad width or height */
    IR_BadIOFile,		/* bad file name,type, etc */
    IR_BadDisplay,
    IR_BadWindow,
    IR_BadValue,
    IR_BadVisual,
    IR_BadHardware,
    IR_BadSyntax,		/* syntax error */

    /*
     * These are a bit special
     */
    IR_Eof,			/* End of file */
    IR_Warning,			/* Non-fatal problem */
    IR_Canceled			/* Non-fatal problem */
} InternalReasonHint;

typedef struct _internalReason InternalReason, *InternalReasonPtr;
struct _internalReason {
    char *msg;			/* reason for failure */
    InternalReasonHint hint;	/* encoded reason */
    char *nodeName;		/* name of node */
    int nodeIndex;		/* index of node that generated reason */
    boolean allocated;		/* true if this structure should be freed */
    InternalReasonPtr next;	/* linked list of reasons */
};

CExternal InternalReasonPtr CreateSysReason(char *, ...);
CExternal InternalReasonPtr CreateReason(char *, ...);
CExternal InternalReasonPtr InternalProblem(char *, ...);
CExternal InternalReasonPtr BadValueReason(char *, ...);
CExternal InternalReasonPtr BadSyntaxReason(char *, ...);
CExternal InternalReasonPtr BadFileReason(char *, ...);
CExternal InternalReasonPtr AllocReason(char *what, int nBytes);

CExternal void AppendReason(InternalReasonPtr *list, InternalReasonPtr newIrp);
CExternal void FreeReason(InternalReasonPtr);
CExternal void TtyShowReason(InternalReasonPtr);
CExternal void FormatReason(InternalReasonPtr reason, char *msg);
CExternal void ShowReason(void (*dpyFunc)(char *, ...), InternalReasonPtr);

/*
 * common failure reasons
 */
extern InternalReason IRCanceled;
extern InternalReason IRPrematureEof;
extern InternalReason IRPipelineEof;
extern InternalReason IRBadFileHeader;
extern InternalReason IRNoSuchFile;
extern InternalReason IRShortRead;
extern InternalReason IRBadIoFileType;
extern InternalReason IRBadAllocation;
extern InternalReason IRBadTranslation;
extern InternalReason IRCantWrite;
extern InternalReason IRInternalProblem;
extern InternalReason IRBadDisplay;
extern InternalReason IRBadWindow;
extern InternalReason IRBadVisual;
extern InternalReason IRBadValue;
extern InternalReason IRBadHardware;
extern InternalReason IRBadInclip;
extern InternalReason IRBadOutclip;
extern InternalReason IRBadSample;
extern InternalReason IRBadColor;
extern InternalReason IRBadDimensions;

#endif /* _NVRREASON_H_ */
