/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	imageToken.c - parse tokens associated with the image library
 *
 * DESCRIPTION
 *	<overall description of what the c code does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrString.c,v 1.6 1995/04/12 23:09:34 russ Exp $
 */

#define NeedFunctionPrototypes 1

#include "nvrString.h"
#include "nvrAlloc.h"
#include "osio.h"

/*
 * This allocates string storage from the string storage described below.
 */
static char *
LocalCopyOfString(char *string, char *stringSpace)
{
    strcpy(stringSpace, string);
    return(stringSpace + strlen(string) + 1);
}

/*
 * This allocator creates a single chunk of space that holds an
 * argument list.  Memory is laid out as follows:
 *      -----------------
 *      |               | <-- returned argp
 *      | StringArgList |
 *      |   storage     |
 *      -----------------
 *      |  list[0]      |
 *      -----------------
 *      |  list[1]      |
 *      -----------------
 *      |  ...          |
 *      -----------------
 *      | list[length-1]|
 *      -----------------
 *      |  valueList    | \
 *      |   pointers    |  \
 *      |      .        |   > valueListBytes
 *      |      .        |  /
 *      |      .        | /
 *      -----------------
 *      |   string      | \
 *      |  storage      |  \
 *      |      .        |   > stringBytes 
 *      |      .        |  /
 *      |      .        | /
 *      -----------------
 */
StringArgListPtr
AllocStringArgList(int argc, char **argv)
{
    StringArgListPtr argp;
    long i, nStringBytes;
    char *stringSpace, *p;
    char **valueList;

    /*
     * Count the bytes in the strings.  We'll need string storage no more 
     * than this big.
     */
    nStringBytes = 0;
    for (i = 1; i < argc; i++)
	nStringBytes += strlen(argv[i]) + 1;

    /*
     * The idea here is that there are no more than argc elements
     * in argp->list[], and there are no more than argc valueList
     * pointers needed.
     *
     * Note: this creates wasted space which can multiply when using
     * CombineStringArgLists().
     */
    i = sizeof(StringArgList)
    	+ sizeof(argp->list[0]) * argc  /* for argp->list[] */
    	+ sizeof(char *) * argc		/* for possible valueList */
    	+ nStringBytes;	
    argp = (StringArgListPtr) InsecureAlloc(i);
    if (argp == NULL)
        return(NULL);
    argp->size = i;
    argp->list = (NameValuePairPtr) (argp+1);
    argp->stringBytes = nStringBytes;
    argp->valueListBytes = sizeof(char *) * argc;
    valueList = (char **)(argp->list + argc);
    stringSpace = (char *)(valueList + argc );

    i = 0;
    for (argc--, argv++; argc; argc--, argv++) {
	p = *argv;
	if (*p == '-')
	    continue;
	while (*p && *p != '=')
	    p++;
	if (*p == '=') {
	    *p = '\0';
	    argp->list[ i ].name = stringSpace;
	    stringSpace = LocalCopyOfString(*argv, stringSpace);

	    if( *(p+1) == '{' ) {	/* Explicit list */
	        argp->list[ i ].valueList = valueList;
	        while (--argc) {
	            argv++;
	            if (argv[0][0] == '}')
	            	break;
	            *valueList++ = stringSpace;
	            stringSpace = LocalCopyOfString(*argv, stringSpace);
	        }
	        *valueList++ = NULL;
		argp->list[ i ].value = argp->list[ i ].valueList[0];
		if (argc == 0)
		    break;
	    } else {			/* Single list item */
		argp->list[ i ].valueList = NULL;
		argp->list[ i ].value = stringSpace;
		stringSpace = LocalCopyOfString(p+1, stringSpace);
	    }
	    *p = '=';
	} else {
	    argp->list[ i ].name = stringSpace;
	    stringSpace = LocalCopyOfString(*argv, stringSpace);
	    argp->list[ i ].value = NULL;
	    argp->list[ i ].valueList = NULL;
	}
	i++;
    }
    argp->list[i].name = argp->list[i].value = NULL;
    argp->length = i;
    argp->index = 0;
    
    if (stringSpace > (char *)argp + argp->size
     || (char *)valueList > (char *)argp + argp->size)
        Fatal("AllocStringArgList: argp: %x-%x, ptrs %x or %x out of range\n",
            argp, (char *)argp + argp->size, stringSpace, valueList);

    return (argp);
}

typedef struct _strListCopyParam {
    StringArgListPtr src;
    StringArgListPtr dst;
    char *stringSpace;
    char **valueList;
    long start;
    long stop;
} StrListCopyParam, *StrListCopyParamPtr;

/*
 * For internal use: copy elements from 'from' beginning with
 * index 'start' to index 'stop'.  Elements are copied onto the end
 * 
 */
static void
CopyStringArgList(StrListCopyParamPtr cp)
{
    StringArgListPtr src = cp->src;
    StringArgListPtr dst = cp->dst;
    long srcIdx, dstIdx, j;
    
    for(srcIdx = cp->start; srcIdx <= cp->stop; srcIdx++) {
        dstIdx = dst->index;
	dst->list[ dstIdx ].name = cp->stringSpace;
	cp->stringSpace = LocalCopyOfString(src->list[ srcIdx ].name,
					    cp->stringSpace);
	if( src->list[ srcIdx ].valueList ) {
	    dst->list[ dstIdx ].valueList = cp->valueList;
	    for(j = 0; src->list[ srcIdx ].valueList[j]; j++) {
		*cp->valueList++ = cp->stringSpace;
		cp->stringSpace = LocalCopyOfString(src->list[ srcIdx ].valueList[j],
						    cp->stringSpace);
	    }
	    *cp->valueList++ = NULL;
	    dst->list[ dstIdx ].value = dst->list[ dstIdx ].valueList[0];
	} else if( src->list[ srcIdx ].value ) {
	    dst->list[ dstIdx ].valueList = NULL;
	    dst->list[ dstIdx ].value = cp->stringSpace;
	    cp->stringSpace = LocalCopyOfString(src->list[ srcIdx ].value,
	    				       cp->stringSpace);
	} else {
	    dst->list[ dstIdx ].valueList = NULL;
	    dst->list[ dstIdx ].value = NULL;
	}
	dst->index++;
    }
}

/*
 * Create a new StringArgListPtr that appears as if all of the "from"
 *	parameter (from->index through from->length) was inserted into
 *	the "to" parameter *before* to->index.
 *
 *      Note: this does not attempt to remove the wasted space caused
 *      by AllocStringArgList() in the first place.
 */
StringArgListPtr
CombineStringArgLists(StringArgListPtr to, StringArgListPtr from)
{
    StringArgListPtr newTo;
    long size;
    StrListCopyParam cp;
    
    size = to->size + from->size;
    newTo = (StringArgListPtr) InsecureAlloc(size);
    if (newTo == NULL)
        return(NULL);
    newTo->index = 0;
    newTo->length = to->length + from->length;
    newTo->list = (NameValuePairPtr) (newTo+1);
    newTo->size = size;
    newTo->stringBytes = from->stringBytes + to->stringBytes;
    newTo->valueListBytes = from->valueListBytes + to->valueListBytes;
    cp.valueList = (char **)(newTo->list + newTo->length);
    cp.stringSpace = (char *)((long)cp.valueList + newTo->valueListBytes);
    cp.dst = newTo;
    
    /*
     * copy first to->index - 1 entries from "to"
     */
    cp.src = to;
    cp.start = 0;
    cp.stop = to->index-1;
    CopyStringArgList( &cp );
    
    /*
     * copy all of "from"
     */
    cp.src = from;
    cp.start = 0;
    cp.stop = from->length-1;
    CopyStringArgList( &cp );
    
    /*
     * copy the rest of "to"
     */
    cp.src = to;
    cp.start = to->index;
    cp.stop = to->length-1;
    CopyStringArgList( &cp );

    /*
     * Set the index for the newly created StringArgList to point to
     * the insertion point.  This is sort of arbitrary, but we note
     * the fact here.
     */
    newTo->index = to->index;

    if (cp.stringSpace > (char *)newTo + newTo->size
     || (char *)cp.valueList > (char *)newTo + newTo->size)
        Fatal("AllocStringArgList: argp: %x-%x, ptrs %x or %x out of range\n",
            newTo, (char *)newTo + newTo->size, cp.stringSpace, cp.valueList);

    return(newTo);
}

void
FreeStringArgList(StringArgListPtr argp)
{
    free((char *)argp); /* allocated in a single chunk */
}

int
StringToInt(char *string)
{
    int value = 0;

    if (string[0] == '0' && string[1] == 'x') {
	sscanf(string, "0x%x", &value);
    } else {
	value = atoi(string);
    }
    return(value);
}

/*
 * This assumes an array of pairs, the last of which must
 * have a null name and its corresponding enumVal will be the Unknown
 * value.
 */
int NameToEnum(char *name, NameEnumPairPtr pairs)
{
    while (pairs->name) {
	if (*name == *pairs->name) {
	    if (strcmp(name, pairs->name) == 0)
		return(pairs->enumVal);
	}
	pairs++;
    }
    return(pairs->enumVal);
}

char *EnumToName(int enumVal, NameEnumPairPtr pairs)
{
    while (pairs->name) {
	if (enumVal == pairs->enumVal)
	    return(pairs->name);
	pairs++;
    }
    return(pairs->name);
}

char **
AllocStringList(char *strings, int separator)
{
    int nStrings, allocSize;
    char *p, **pList, **list, *newStrings;
    
    allocSize = 1;	/* the null byte at the end */
    nStrings = 2;	/* the first pointer & the null pointer at the end */
    if (strings) {
        for (p = strings; *p; p++) {
            if (*p == separator)
                nStrings++;
            allocSize++;
        }
    }
    
    allocSize += nStrings * sizeof(char *);
    list = (char **)InsecureAlloc(allocSize);
    if (list == NULL)
        return(NULL);
    
    pList = list;
    if (strings) {
        newStrings = (char *)(list + nStrings);
        *pList++ = newStrings;
        for (; *strings; strings++) {
            if (*strings == separator) {
                *newStrings++ = '\0';
                *pList++ = newStrings;
            } else
                *newStrings++ = *strings;
        }
        *newStrings++ = '\0';
    }
    
    *pList = NULL;
    return(list);
}






