/*
 * (c) Copyright 1992 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	nvrString.h
 *
 * DESCRIPTION
 *	String function prototypes
 *	OS-DEPENDENT
 *
 * REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/nvrString.h,v 1.26 1995/12/05 17:51:21 toddb Exp $
 */
#ifndef _STRING_FORMAT_H_
#define _STRING_FORMAT_H_

#include "nvrTypes.h"

#ifndef __cplusplus

#if defined(sgi) || defined(__MWERKS__)
    CExternal int sprintf(char *, const char *, ...);
    CExternal int sscanf(const char *, const char *, ...);
    CExternal int scanf(const char *, ...);
    CExternal int atoi(const char *);
    CExternal double atof(const char *);
    CExternal long int strtol(const char *, char **, int);
#endif
    


#if defined(_HPUX_SOURCE) || defined(THINK_C)
    CExternal int scanf(const char *, ...);
    CExternal int atoi(const char *);
    CExternal double atof(const char *);
    CExternal long int strtol(const char *, char **, int);
    CExternal int sprintf(char *, const char *,...);
    CExternal int sscanf(const char *, const char *, ...);
#endif

#if defined(sun)

# ifdef Solaris1

    CExternal int scanf(const char *, ...);
    CExternal int atoi(const char *);
    CExternal double atof(const char *);
    CExternal long int strtol(const char *, char **, int);
#  if defined(FILE)
    CExternal int vfprintf(FILE *, const char *, void *);
    CExternal int fflush(FILE *);
    CExternal int _flsbuf(unsigned char, FILE *);
    CExternal int _filbuf(FILE *);
#  else
    CExternal char *sprintf(char *, const char *, ...);
#  endif
    CExternal int printf(const char *, ...);
    CExternal int vprintf(const char *, void *);
    CExternal int vsprintf(char *, const char *, void *);
    CExternal int sscanf(const char *, const char *, ...);

# endif

# ifdef Solaris2

#if 0
#  if defined(FILE)
    CExternal int      fprintf(FILE *, const char *, ...);
    CExternal int      vfprintf(FILE *, const char *, void *);
    CExternal int      fscanf(FILE *, const char *, ...);
#  endif
    CExternal int      printf(const char *, ...);
    CExternal int      sprintf(char *, const char *, ...);
    CExternal int      vprintf(const char *, void *);
    CExternal int      vsprintf(char *, const char *, void *);
    CExternal int      scanf(const char *, ...);
    CExternal int      sscanf(const char *, const char *, ...);
    CExternal double   atof(const char *);
    CExternal int      atoi(const char *);
    CExternal long int atol(const char *);
#endif /* 0 */

# include <stdio.h>
# include <stdlib.h>
# endif

#endif /* sun */

#if defined(VXWORKS)
    CExternal int sscanf(char *, const char *, ...);
#endif
#endif /* __cplusplus */
 
#include <string.h>

/*
 **************************************************************************
 *
 * This is used to hold argument lists specified to the individual
 * pipeline members
 *
 **************************************************************************
 */
typedef struct _nameValuePair {
    char *name;		/* value name */
    char *value;	/* single value, first value in list or NULL */
    char **valueList;	/* list of values (null terminated), or NULL */
} NameValuePair, *NameValuePairPtr;

typedef struct _stringArgList {
    long index;		/* current index */
    long length;	/* length of list */
    NameValuePairPtr list;
    long size;		/* total bytes allocated including string space */
    long stringBytes;	/* number of bytes for string storage */
    long valueListBytes;/* number of bytes used in valueList Pointers */
} StringArgList, *StringArgListPtr;

typedef struct _nameEnumPair {
    char *name;
    long enumVal;
} NameEnumPair, *NameEnumPairPtr;


/*
 * Token parsing
 */
CExternal int NameToEnum(char *, NameEnumPairPtr);
CExternal char *EnumToName(int, NameEnumPairPtr);
CExternal int StringToInt(char *);

CExternal StringArgListPtr AllocStringArgList(
    int,
    char **argv
);

CExternal char **AllocStringList(
    char *strings,	/* <spearator>-separated list */
    int separator
);

CExternal void FreeStringArgList(
    StringArgListPtr
);

CExternal StringArgListPtr CombineStringArgLists(
    StringArgListPtr to,
    StringArgListPtr from
);

#endif /* _STRING_FORMAT_H_ */
