/*
 * (c) Copyright 1992 North Valley Research, Inc.
 *         All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	osio.c
 *
 * DESCRIPTION
 * 	I/O functions that hide os dependencies
 *	OS-DEPENDENT
 *
 * REVISION
 *	$Header: /usr/local/src/nvrToMIT/RCS/osio.c,v 1.77 1995/02/13 04:48:36 toddb Exp $
 */
#include <stdio.h>
#include <stdarg.h>
#ifdef Solaris2
# include <unistd.h>
  extern long tell(int);
#endif

#if defined(THINK_C) || defined(__MWERKS__)
#include "macUnix.h"
#else
#include <fcntl.h>
#endif

#define NeedFunctionPrototypes 1

#include "nvrTimer.h"
#include "nvrDebug.h"
#include "nvrAlloc.h"
#include "nvrString.h"
#include "osioI.h"

#define GeneralAssert(mustBeTrue) _DoAssert(mustBeTrue)

#ifdef VXWORKS

# define InitLogFile()	{ if (LogFile == NULL) LogFile = stderr; }
# define LogFileAutoInitialization	NULL
# define AuxLogFileAutoInitialization

static void
InitAuxLogFiles()
{
    static boolean initialized = False;
    int i;

    if (initialized)
	return;
    for (i = 0; i < MAX_AUX_LOGFILES; i++)
	AuxLogFiles[i] = stderr;
}

#else /* !VXWORKS */

# define InitLogFile()
# define LogFileAutoInitialization	stderr
# define AuxLogFileAutoInitialization		\
  = { stderr, stderr, stderr, stderr, stderr,	\
      stderr, stderr, stderr, stderr, stderr }
# define InitAuxLogFiles()

#endif /* !VXWORKS */

CExternal void exit(int);

extern char *ProgramName;
IoInfo IoData;
static FatalHandler FatalFunction;
static PrivPointer FatalFuncArg;
static FILE *AuxLogFiles[ MAX_AUX_LOGFILES ] AuxLogFileAutoInitialization;
static FILE *LogFile = LogFileAutoInitialization;

boolean
SetAuxLogfile(int index, char *name)
{
    FileHandle fd;
    FILE *fp;

    InitAuxLogFiles();
    if (index > NumberOfCells(AuxLogFiles)) {
	Msg("SetAuxLogfile(%d, \"%s\"): index > max(%d)\n",
	    index, name, NumberOfCells(AuxLogFiles));
	return (False);
    }

    fd = OpenFileForWriting(name);
    if (fd < 0) {
	SystemError("Can't reopen aux logfile #%don %s\n", index, name);
	return(False);
    }
    if ((fp = fdopen(fd, "w")) == NULL) {
	close(fd);
	return(False);
    }
    if (AuxLogFiles[index] != stderr)
	fclose(AuxLogFiles[index]);
    AuxLogFiles[index] = fp;
    return (True);
}
    
boolean
SetLogfile(char *name)
{
    FILE *fp;

    InitLogFile();
    fp = fopen(name, "w");
    if (fp == NULL) {
	SystemError("Can't reopen logfile on %s\n", name);
	return(False);
    }
    if (LogFile != stderr)
	fclose(LogFile);
    LogFile = fp;
    return (True);
}
    
void
RegisterFatalHandler(FatalHandler handler, PrivPointer arg)
{
    FatalFunction = handler;
    FatalFuncArg = arg;
}

/*VARARGS1*/
void
NormalOutput(char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    vprintf(fmt, ap);
    fflush(stdout);
    va_end(ap);
}

static void
VaMsg(char *fmt, va_list ap)
{
    InitLogFile();
    vfprintf(LogFile, fmt, ap);
    fflush(LogFile);
}

void
Msg(char *fmt, ...)
{
    va_list ap;

    InitLogFile();
    va_start(ap, fmt);
    vfprintf(LogFile, fmt, ap);
    va_end(ap);
    fflush(LogFile);
}

void
AuxMsg(int index, char *fmt, ...)
{
    va_list ap;

    if (index >= NumberOfCells(AuxLogFiles))
	return;
    InitAuxLogFiles();
    va_start(ap, fmt);
    vfprintf(AuxLogFiles[ index ], fmt, ap);
    va_end(ap);
    fflush(AuxLogFiles[ index ]);
}

static char *
StripLeadingPath(char *path)
{
    /*
     * Strip the leading path components
     */
    char *p = path + strlen(path) - 1;
    while (*p != '/')
	p--;
    return(p+1);
}

void
ProgMsg(char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    if (*ProgramName == '/')
	ProgramName = StripLeadingPath(ProgramName);
    Msg("%s: ", ProgramName);
    VaMsg(fmt, ap);
    va_end(ap);
}

static void
VaSystemError(char *fmt, va_list ap)
{
    extern int	    errno;

    if (*ProgramName == '/')
	ProgramName = StripLeadingPath(ProgramName);
    Msg("%s: ", ProgramName);
    if (errno) {
	Msg("%s: ", LookupSysError(errno));
	errno = 0;
    }
    VaMsg(fmt, ap);
}

void
SystemError(char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    VaSystemError(fmt, ap);
    va_end(ap);
}

/*
 * Obsolete
 */
void
Error(char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    VaSystemError(fmt, ap);
    va_end(ap);
}

/*VARARGS1*/
void
Fatal(char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    VaSystemError(fmt, ap);
    va_end(ap);
    if (FatalFunction)
	(*FatalFunction)(FatalFuncArg);
    exit(1);
}

void
BufPrintf(BufFileHndl fp, char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    if (fp == NULL)
    	fp = (BufFileHndl)stdout;
    vfprintf(fp, fmt, ap);
    va_end(ap);
}

boolean
BufferedReadLine(char *buf, int length, BufFileHndl fp)
{
    if (fp == NULL)
    	fp = (BufFileHndl)stdin;
    return(fgets(buf, length, (FILE *)fp) != NULL);
}

void
FlushBufferedFile(BufFileHndl fp)
{
    fflush((FILE *)fp);
}

void
CloseBufferedFile(BufFileHndl fp)
{
    fclose((FILE *)fp);
}

int
UngetBufferedChar(int c, BufFileHndl fp)
{
    return( ungetc(c, (FILE *)fp) );
}

int
GetBufferedChar(BufFileHndl fp)
{
    return( getc((FILE *)fp) );
}

int
PutBufferedChar(int c, BufFileHndl fp)
{
    return( putc(c,(FILE *)fp) );
}

int
BufferedStdRead(char *buffer, int elemSize, int count, BufFileHndl fp)
{
    return( fread(buffer, elemSize, count, (FILE *)fp) );
}

int
BufferedStdWrite(char *buffer, int elemSize, int count, BufFileHndl fp)
{
    return( fwrite(buffer, elemSize, count, (FILE *)fp) );
}

int
BufferedWriteData(BufFileHndl fp, char *buffer, int length)
{
    int bytesOut;
    int totalOut = 0;

    while (length > 0) {
	bytesOut = fwrite(buffer, 1, length, (FILE *)fp);
	if (bytesOut < 0)
	    return(-1);
	if (bytesOut == 0)
	    break;
	totalOut += bytesOut;
	length -= bytesOut;
	buffer += bytesOut;
    }

    return(totalOut);
}

int
BufferedReadData(BufFileHndl fp, char *buffer, int length)
{
    int bytesIn;
    int totalIn = 0;

    while (length > 0) {
	bytesIn = fread(buffer, 1, length, (FILE *)fp);
	if (bytesIn < 0)
	    return(-1);
	if (bytesIn == 0)
	    break;
	totalIn += bytesIn;
	length -= bytesIn;
	buffer += bytesIn;
    }

    return(totalIn);
}

BufFileHndl
GetBufferedIn() {
    return((BufFileHndl) stdin);
}

BufFileHndl
GetBufferedOut() {
    return((BufFileHndl) stdout);
}

BufFileHndl
GetBufferedErr() {
    return((BufFileHndl) stderr);
}

char *
GetBufferedString(char *buf, int size, BufFileHndl fp)
{
    return( fgets(buf,size,fp) );
}

int
TellBufferedFile(BufFileHndl fp)
{
    return( ftell((FILE *)fp) );
}

int
TellFile(FileHandle fd)
{
    return( tell(fd) );
}

int
EndOfBufferedFile(BufFileHndl fp)
{
    return( feof((FILE *)fp) );
}

void
CloseFile(FileHandle fd)
{
    close(fd);
}

long
SeekBufferedFile(BufFileHndl fp, long offset, SeekWhere sk)
{
    switch(sk) {
    case SK_Start:
	return( fseek((FILE *)fp,offset,0) );
    case SK_Relative:
	return( fseek((FILE *)fp,offset,1) );      
    case SK_End:
	return( fseek((FILE *)fp,offset,2) );
    }
    return(-1);
}

long
SeekFile(FileHandle fd, long offset, SeekWhere sk)
{
    switch(sk) {
    case SK_Start:
	return( lseek((int)fd,offset,0) );
    case SK_Relative:
	return( lseek((int)fd,offset,1) );      
    case SK_End:
	return( lseek((int)fd,offset,2) );
    }
    return(-1);
}

boolean
WriteData(FileHandle fd, char *buffer, long length)
{
    int bytesOut;

    while (length) {
	bytesOut = write((int)fd, buffer, (int)length);
	if (bytesOut <= 0)
	    return(False);
	buffer += bytesOut;
	length -= bytesOut;
    }
    return(True);
}

int
ReadData(FileHandle fd, char *buffer, long length)
{
    int bytesIn, notSent = length;

    while (notSent) {
	bytesIn = read((int)fd, buffer, notSent);
	if (bytesIn == 0)
	    break;
	if (bytesIn < 0) {
	    if (notSent == length)
		return(-1);
	    break;
	}
	buffer += bytesIn;
	notSent -= bytesIn;
    }

    return( length - notSent );
}

boolean
SetFileOffset(int fd, ULong offset)
{
    return( lseek(fd, offset, 0) < 0 ? (boolean)False : (boolean)True );
}

void
FreeAsyncHandler(int fd)
{
    int i;

    GeneralAssert(IoData.list[fd].handler);
    IoData.list[fd].handler = NULL;
    FD_CLR(fd, &IoData.inReady);
    if (fd == IoData.lastFd) {
	for (i = IoData.lastFd-1; i >= 0; i--) {
	    if (IoData.list[i].handler)
		break;
	}
	IoData.lastFd = i;
    }
}

void
AllocHandler(int fd,
	     VoidFuncPtr handler,
	     boolean bypassSelect,
	     void *pointer)
{
    int i;

    if (IoData.list == NULL) {
	IoData.list = (FileDescriptorInfo *)NullRealloc((char *) IoData.list,
					      FD_SETSIZE * sizeof(FileDescriptorInfo));
	for (i = 0; i < FD_SETSIZE; i++) {
	    IoData.list[ i ].handler = NULL;
	    IoData.list[ i ].pointer = NULL;
	    IoData.list[ i ].bypassSelect = False;
	    IoData.list[ i ].isAtty = False;
	    IoData.list[ i ].async = False;
	}
	IoData.pid = getpid();
    }
    GeneralAssert(fd < FD_SETSIZE);

    if (IoData.lastFd < fd)
	IoData.lastFd = fd;
    IoData.list[fd].handler = handler;
    IoData.list[fd].bypassSelect = bypassSelect;
    IoData.list[fd].pointer = pointer;
    IoData.list[fd].isAtty = isatty(fd);
    IoData.list[fd].async = False;
    FD_SET(fd, &IoData.inReady);
}

void
AllocSelectHandler(int fd,
		   VoidFuncPtr handler,
		   char *pointer)
{
    AllocHandler(fd, handler, False, pointer);
}

