/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              osio.h
 *
 *      DESCRIPTION
 *              declare external osio routines
 *
 *      REVISION
 * 		$Header: /usr/local/src/nvrToMIT/RCS/osio.h,v 1.57 1995/10/27 05:32:55 toddb Exp $
 */

#ifndef _OSIO_H
#define _OSIO_H

#include "nvrTypes.h"
#include "nvrTimer.h"

#define MAX_AUX_LOGFILES	10
#define DEFAULT_UDP_BUFSIZ	2048	/* for broadCaast, multiCast */
#define DEFAULT_TTL		32	/* for multiCast */
#define EndOfFile		(-1)

typedef enum { SK_Start, SK_Relative, SK_End } SeekWhere;
typedef enum { uniCast, broadCast, multiCast } CastType;

/* routines with variable arguments or no arguments */
CExternal int BlockAsyncHandler(void);
CExternal int AllowAsyncHandler(void);

typedef PrivPointer BufFileHndl;
typedef int FileHandle;

typedef void (*FatalHandler)(
    PrivPointer	/* argument to fatal handler */
);

CExternal boolean TtyAskUser(
    char *fmt,	/* format */
    ...
);

CExternal char *EnvironmentValue(
    char *name
);

CExternal void NormalOutput(
    char *,	/* format */
    ...
);

CExternal void AuxMsg(
    int,	/* index */
    char *,	/* format */
    ...
);

CExternal void Msg(
    char *,	/* format */
    ...
);

CExternal void Error(
    char *,	/* format */
    ...
); /* obsolete */

CExternal void SystemError(
    char *,	/* format */
    ...
);

CExternal void ProgMsg(
    char *,	/* format */
    ...
);

CExternal void RegisterFatalHandler(
    FatalHandler,	/* handler function */
    PrivPointer		/* argument */
);

CExternal void Fatal(
    char *,	/* format */
    ...
);

CExternal boolean SetAuxLogfile(
    int,	/* logfile index */
    char *	/* path */
);

CExternal boolean SetLogfile(
    char *	/* path */
);

CExternal boolean ChangeDirectory(
    char *		/* path */
);

CExternal boolean BufferedReadLine(
    char *,		/* path */
    int,		/* length */
    BufFileHndl		/* file handle */
);

CExternal BufFileHndl OpenBufferedFileForAppending(
    char *	/* path */
);

CExternal BufFileHndl OpenBufferedFileForWriting(
    char *	/* path */
);

CExternal void FlushBufferedFile(
    BufFileHndl fp
);

CExternal BufFileHndl OpenBufferedFileForReading(
    char *	/* path */
);

CExternal BufFileHndl ReopenBufferedFileForWriting(
    int		/* file descriptor */
);

CExternal FileHandle GetBufferedFileHandle(
    BufFileHndl	/* file pointer */
);

CExternal BufFileHndl ReopenBufferedFileForReading(
    int		/* file descriptor */
);

CExternal boolean WriteData(
    FileHandle,	/* file descriptor */
    char *,	/* buffer pointer */
    long	/* length */
);

CExternal int ReadData(
    FileHandle, /* file descriptor */
    char *,     /* buffer pointer */
    long        /* length */
);

CExternal int
BufferedStdWrite(
    char *,	/* buffer */
    int,	/* size of element */
    int,	/* count of elements */
    BufFileHndl	/* buffered file pointer */
);

CExternal int
BufferedStdRead(
    char *,	/* buffer */
    int,	/* size of element */
    int,	/* count of elements */
    BufFileHndl	/* buffered file pointer */
);

CExternal int BufferedWriteData(
    BufFileHndl,/* file pointer */
    char *,	/* buffer pointer */
    int		/* length */
);

CExternal int BufferedReadData(
    BufFileHndl,/* file pointer */
    char *,	/* buffer pointer */
    int		/* length */
);

CExternal void CloseBufferedFile(
    BufFileHndl
);

CExternal TellBufferedFile(
    BufFileHndl
);

CExternal TellFile(
    FileHandle
);

CExternal int UngetBufferedChar(
    int,		/* char to put back */
    BufFileHndl
);

CExternal int GetBufferedChar(
    BufFileHndl
);

CExternal PutBufferedChar(
    int,
    BufFileHndl
);

CExternal BufFileHndl GetBufferedIn(
    void
);

CExternal BufFileHndl GetBufferedOut(
    void
);

CExternal BufFileHndl GetBufferedErr(
    void
);

CExternal void SetBufferSize(
    BufFileHndl fp,
    int size
);

CExternal char *GetBufferedString(
    char *,
    int,
    BufFileHndl
);

CExternal void BufPrintf(
    BufFileHndl,
    char *,
    ...    
);

CExternal EndOfBufferedFile(
    BufFileHndl
);

CExternal FileHandle OpenFileForAppending(
    char *	/* path */
);

CExternal FileHandle OpenFileForWriting(
    char *	/* path */
);

CExternal FileHandle OpenFileForReading(
    char *	/* path */
);

CExternal void CloseFile(
    FileHandle
);

CExternal long SeekBufferedFile(
    BufFileHndl,
    long,
    SeekWhere
);

CExternal long SeekFile(
    FileHandle,
    long,
    SeekWhere
);

CExternal void AllowAsyncNotification(
    int		/* file descriptor */
);

CExternal void BlockAsyncNotification(
    int		/* file descriptor */
);
    
CExternal void FreeAsyncHandler(
    int		/* file descriptor */
);

CExternal void SetNonBlocking(
    int		/* file descriptor */
);
    
CExternal boolean SetFileOffset(
    int,	/* file descriptor */
    ULong	/* offset */
);

typedef void (*AsyncHandlerFunc)(int, PrivPointer);

CExternal void AllocAsyncHandler(
    int,	/* file descriptor */
    AsyncHandlerFunc,/* handler */
    boolean,	/* bypassSelect */
    PrivPointer	/* data passed to handler */
);

void AllocSelectHandler(
    int,	/* file descriptor */
    VoidFuncPtr,/* handler */
    char *	/* data pointer */
);

CExternal long BytesAvailable(
    int		/* file descriptor */
);

CExternal int NonBlockingRead(
    int,	/* file descriptor */
    char *,	/* buffer for read */
    long	/* byte count to read */
);

CExternal boolean FileHandleIsTty(
    int		/* file descriptor */
);

CExternal void CloseOpenFileDescriptors(
    ULong
);

CExternal boolean WaitForFileOutput(
    int,	/* file descriptor */
    TimeSpecPtr	/* max timeout */
);

CExternal boolean WaitForFileInput(
    int,	/* file descriptor */
    TimeSpecPtr	/* max timeout */
);

CExternal void WaitForSomething(
    TimeSpecPtr	/* max timeout */
);

CExternal void ShowOpenFileDescriptors(
    void
);

CExternal int AsyncWrite(
    int,		/* file descriptor */
    char *,		/* write buffer */
    int,		/* length of write buffer */
    long		/* file offset */
);

CExternal int AsyncRead(
    int,		/* file descriptor */
    char *,		/* read buffer */
    int,		/* length of read buffer */
    long		/* file offset */
);

CExternal void WaitForSignal(
    void
);

CExternal int FinishAsyncIo(
    int			/* identifier returned by AsyncWrite or AsyncRead */
);

CExternal int BlockSignals(
    int,
    ...
);

CExternal int UnblockSignals(
    int,
    ...
);

CExternal FileHandle FileNo(
    BufFileHndl
);

CExternal int CreatePipe(
    FileHandle[2]
);

CExternal FileHandle MakeTcpConnection(
    char *,
    int,
    InternalReasonPtr *
);

CExternal FileHandle MakeUdpConnection(
    char *,
    int,
    InternalReasonPtr *
);

CExternal FileHandle MakeUdpListenSocket(
    int,
    CastType,
    void *,
    InternalReasonPtr *
);

extern char *ProgramName;

#ifdef Solaris1
# ifndef __cplusplus
#  include <unistd.h>
# endif
#endif
 
#ifdef Solaris2
# include <unistd.h>
  extern long tell(int);
#endif

#if defined(sgi) || defined(_HPUX_SOURCE) || defined(sun)

    extern char *sys_errlist[];
#   define LookupSysError(errno) sys_errlist[ errno ]

#else /* defined(THINK_C, __MWERKS__, VXWORKS) */

    CExternal char *LookupSysError(
	int			/* errno */
    );

#endif /* defined(sgi) || defined(_HPUX_SOURCE) || defined(THINK_C) */

#endif /* _OSIO_H */
