/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	pipetest.c - test program
 *
 * DESCRIPTION
 *	OS-DEPENDENT
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/pipetest.c,v 1.4 1994/01/10 18:48:13 toddb Exp $
 */
#define NeedFunctionPrototypes 1
#include "process.h"
#include "fileAccess.h"
#include <sys/types.h>

extern int errno;

char *ProgramName;

/*
 * This test program implements the ipc setup shown below.  In our current
 * use, the user app is the mpeg compressor GUI, proc2 is the mpeg compressor, 
 * and proc1 is a filter for translating movie files into yuv and writing
 * them on stdout.  This compressor recieves commands on the file descriptor
 * specified by the -i arg, and gives back events on the fd specified by -o.
 * It is most convenient for the compressor to recieve yuv input on stdin
 * (file descriptor 0).
 *
 * ---------       ---------------
 * |       |       |             |
 * | USER  |<------|-o?          |
 * | APP   |------>|-i?  Proc 2  |
 * | ----  |       |     ------  |
 * |       |   +-->|(fd 0)       |
 * |       |   |   |             |
 * ---------   |   -------------
 * ---------   |
 * | (fd 1)|   |
 * |       |   |
 * | Proc1 |---+
 * | ----- |
 * |       |
 * | fd 0  |
 * | closed|
 * ---------
 */

main(int argc, char **argv)
{
    int bytesIn, bytesOut;
    char buf[ 100 ], *p;
    Argvec proc1Args, proc2Args;
    TwoWayPipe proc2Pipe;
    TwoWayPipe proc1Pipe;
    fd_set readFds, fdsReady;
    int nfdsReady, nReadFds;

    ProgramName = argv[0];

    proc1Args.length = proc2Args.length = 0;
    proc1Args.argv = proc2Args.argv = NULL;
    proc1Args.path = proc2Args.path = "catprog";
    AddArgv(&proc1Args, "catprog", False);
    AddArgv(&proc2Args, "catprog", False);

    proc1Pipe = PipeToAndFrom(&proc1Args, NULL);
    sleep(1); errno = 0;

    /*
     * Normally we would close proc1's stdin, but we use this to simulate
     * data input to proc1 (which then gets transfered to proc2)
     *
     * close(proc1Pipe.outFd);
     */
    Error("proc1 (%d): input to us on %d, output to it on %d\n",
	proc1Pipe.pid, proc1Pipe.inFd, proc1Pipe.outFd);
    NormalOutput("    "); PrintArgv(&proc1Args);

    proc2Pipe = PipeToAndFromWithStdin(&proc2Args,
				       proc1Pipe.inFd,
				       DefaultDuplexPipeRewriter);
    close(proc1Pipe.inFd); /* we can't use this */
    sleep(1); errno = 0;
    Error("proc2 (%d): input to us on %d, output to it on %d\n",
	proc2Pipe.pid, proc2Pipe.inFd, proc2Pipe.outFd);
    NormalOutput("    "); PrintArgv(&proc2Args);

    ShowOpenFileDescriptors();

    /*
     * As the commands come in, send data to proc1 or proc2, or close
     * inputs to proc1 or proc2.
     */
    FD_ZERO(&readFds);
    FD_SET(proc2Pipe.inFd, &readFds);	/* events from the compressor */
    FD_SET(0, &readFds);		/* commands for this processes */
    nReadFds = proc2Pipe.inFd+1;

    while (proc1Pipe.outFd >= 0 || proc2Pipe.outFd >= 0) {
	fdsReady = readFds;
        nfdsReady = select(nReadFds, &fdsReady, 0,0,0);
        if (nfdsReady < 0)
            continue;

	if (FD_ISSET(0, &fdsReady)) {
	    if (gets(buf) == NULL) {
		FD_CLR(0, &readFds);
		Error("eof on stdin\n");
		continue;
	    }
	    if (strcmp(buf, "data1") == 0)
		write(proc1Pipe.outFd, buf, 5);
	    else if (strcmp(buf, "data2") == 0)
		write(proc2Pipe.outFd, buf, 5);
	    else if (strcmp(buf, "close1") == 0) {
		close(proc1Pipe.outFd);
		proc1Pipe.outFd = -1;
	    } else if (strcmp(buf, "close2") == 0) {
		close(proc2Pipe.outFd);
		proc2Pipe.outFd = -1;
	    } else
		Error("huh?\n");
	}
	if (FD_ISSET(proc2Pipe.inFd, &fdsReady)) {
	    bytesIn = read(proc2Pipe.inFd, buf, 100);
	    if (bytesIn < 0)
		Fatal("Error reading fd %d\n", proc2Pipe.inFd);
	    if (bytesIn == 0) {
		Error("eof on fd %d\n", proc2Pipe.inFd);
		close(proc2Pipe.inFd);
		FD_CLR(proc2Pipe.inFd, &readFds);
	    } else
		Error("got %d bytes from %d\n", bytesIn, proc2Pipe.pid);
	}
    }
    if (bytesIn < 0)
	Fatal("Error on in fd %d (pid %d)\n", proc2Pipe.inFd, proc2Pipe.pid);
    sleep(2); errno = 0;
    Error("done\n");
    exit(0);
}
