/*
 * (c) Copyright 1992 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	process.c
 *
 * DESCRIPTION
 *	Management of sub-processes
 *
 * REVISION
 *	$Header: /usr/local/src/nvrToMIT/RCS/process.c,v 1.18 1995/09/05 21:38:13 russ Exp $
 */

#define NeedFunctionPrototypes 1

#include <stdio.h>
#include <stdarg.h>
#include "process.h"
#include "osio.h"
#include "nvrAlloc.h"

#define Max(a,b)	((a) > (b) ? (a) : (b))

static void
StretchArgv(Argvec *argv, int add)
{
    long newLength = add;
    long oldLength = argv->length;

    if (oldLength == 0) {
	argv->index = 0;
	argv->argv = NULL;
	argv->allocated = NULL;
    } else
	newLength += argv->index;

    if (newLength > oldLength) {
        argv->length = Max(oldLength+ARGVINCREMENT, newLength);
        argv->argv = (char **)
            NullRealloc((char *) argv->argv,
                        (unsigned) (sizeof(char *) * (argv->length+1)));
        argv->allocated = (boolean *)
            NullRealloc((char *) argv->allocated,
                        (unsigned) (sizeof(boolean) * argv->length));
    }
    argv->argv[ argv->index+add ] = NULL;
}

void
PrintArgv(Argvec *argv)
{
    long i;

    for (i = 0; i < argv->index; i++)
        Msg("%s ", argv->argv[i]);
    Msg("\n");
}
    
void
AddArgv(Argvec *argv, char *arg, boolean copy)
{
    StretchArgv(argv, 1);
    argv->allocated[ argv->index ] = copy;
    argv->argv[ argv->index++ ] = copy ? CopyOfString(arg) : arg;
}

/*VARARGS2*/
void
AddFmtArgv(Argvec *argv, char *fmt, ...)
{
    char arg[ BUFSIZ ];
    va_list ap;

    va_start(ap, fmt);
    (void) vsprintf(arg, fmt, ap);
    AddArgv(argv, arg, True);
    va_end(ap);
}

void
TrimArgv(Argvec *argv, int length)
{
    int    i;

    if (argv->length == 0) {
	return;
    }
    for (i = length; i < argv->index; i++) {
        if (argv->allocated[ i ]) {
            free(argv->argv[ i ]);
	    argv->allocated[ i ] = False;
	}
    }
    argv->index = length;
    argv->argv[ argv->index+1 ] = NULL;
}

void
FreeArgv(Argvec *argv)
{
    TrimArgv(argv, 0);
    if (argv->length > 0) {
	free((void *)argv->argv);
	free((void *)argv->allocated);
    }
}

void
DuplexPipeRewriterToStdin(Argvec *argv, int inFd, int outFd)
{
    char buf[ BUFSIZ ];

    StretchArgv(argv, 2);
    sprintf(buf, "-i%d", inFd);
    argv->allocated[ argv->index ] = True;
    argv->argv[ argv->index++ ] = CopyOfString(buf);
    sprintf(buf, "-o%d", outFd);
    argv->allocated[ argv->index ] = True;
    argv->argv[ argv->index++ ] = CopyOfString(buf);
}

void
DefaultDuplexPipeRewriter(Argvec *argv, int inFd, int outFd)
{
    long i;
    char buf[ BUFSIZ ];

    StretchArgv(argv, 2);
    argv->index += 2;

    argv->argv[ argv->index ] = NULL;
    for (i = argv->index-1; i > 2; i--) {
	argv->argv[i] = argv->argv[i-2];
	argv->allocated[i] = argv->allocated[i-2];
    }
    sprintf(buf, "-i%d", inFd);
    argv->argv[1] = CopyOfString(buf);
    argv->allocated[1] = True;
    sprintf(buf, "-o%d", outFd);
    argv->argv[2] = CopyOfString(buf);
    argv->allocated[2] = True;
}

void
PipeRewriterToStdout(Argvec *argv, int newStdOut)
{
    long i;
    char buf[ BUFSIZ ];

    StretchArgv(argv, 1);
    argv->index++;

    argv->argv[ argv->index ] = NULL;
    for (i = argv->index-1; i > 1; i--) {
	argv->argv[i] = argv->argv[i-1];
	argv->allocated[i] = argv->allocated[i-1];
    }
    sprintf(buf, "-o%d", newStdOut);
    argv->argv[1] = CopyOfString(buf);
    argv->allocated[1] = True;
}

void
DefaultPipeRewriter(Argvec *argv, int outFd)
{
    long i;
    char buf[ BUFSIZ ];

    StretchArgv(argv, 1);
    argv->index++;

    argv->argv[ argv->index ] = NULL;
    for (i = argv->index-1; i > 1; i--) {
	argv->argv[i] = argv->argv[i-1];
	argv->allocated[i] = argv->allocated[i-1];
    }
    sprintf(buf, "-i%d", outFd);
    argv->argv[1] = CopyOfString(buf);
    argv->allocated[1] = True;
}
