/*
 *	(c) Copyright 1992 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		process.h
 *
 *	DESCRIPTION
 *		structures for process management
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/process.h,v 1.27 1995/10/27 22:38:52 russ Exp $
 */
#ifndef _PROCESS_H_
#define _PROCESS_H_

#include "nvrTypes.h"

#ifdef	SYSV
#define	vfork	fork
#endif	/* SYSV */

typedef enum {
    ProcIsDead = 0,
    ProcIsSuspended,
    ProcIsRunning
} ProcessState;
    
#define ARGVINCREMENT	10

typedef struct _argvec {
    char   *path;		/* pointer to arg1 in execv(arg1, arg2) */
    char   **argv;		/* pointer to arg2 in execv(arg1, arg2) */
    boolean *allocated;		/* True if matching elem in argv is allocated*/
    long    index;		/* index to the last+1 argument */
    long    length;		/* actual length of the **argv element */
} Argvec;

typedef struct {
    long pid;
    long inFd;
} InputPipe;

typedef struct {
    long pid;
    long outFd;
} OneWayPipe;

typedef struct {
    long pid;
    long inFd;
    long outFd;
} TwoWayPipe;

CExternal void AddArgv(
    Argvec *, char *, boolean
);

CExternal void PrintArgv(
    Argvec *
);
CExternal void AddFmtArgv(
    Argvec *, char *, ...
);
CExternal void TrimArgv(
    Argvec *, int
);
CExternal void FreeArgv(
    Argvec *
);
CExternal void DuplexPipeRewriterToStdin(
    Argvec *, int, int
);
CExternal void DefaultDuplexPipeRewriter(
    Argvec *, int, int
);
CExternal void PipeRewriterToStdout(
    Argvec *, int
);
CExternal void DefaultPipeRewriter(
    Argvec *, int
);
CExternal OneWayPipe PipeToWithStdout(
    Argvec *, int
);
CExternal OneWayPipe AltPipeToWithStdout(
    Argvec *, int
);
CExternal OneWayPipe PipeTo(
    Argvec *, void (*)()
);
CExternal OneWayPipe AltPipeTo(
    Argvec *, void (*)()
);
CExternal TwoWayPipe PipeToAndFrom(
    Argvec *, void(*)()
);
CExternal TwoWayPipe AltPipeToAndFrom(
    Argvec *, void(*)()
);
CExternal TwoWayPipe PipeToAndFromWithStdin(
    Argvec *,
    int,	/* new stdin */
    void(*)()
);
CExternal TwoWayPipe AltPipeToAndFromWithStdin(
    Argvec *,
    int,	/* new stdin */
    void(*)()
);
CExternal InputPipe PipeFromWithStdin(
    Argvec *,
    int		/* newStdin */
);
CExternal InputPipe AltPipeFromWithStdin(
    Argvec *,
    int		/* newStdin */
);
CExternal void Execute(
    Argvec *
);
CExternal ProcessState WaitForPid(
    long pid,
    boolean verbose
);
CExternal void SuspendProcess(
    long		/* -STOP in unix */
);
CExternal void ResumeProcess(
    long		/* -CONT in unix */
);
CExternal void KillProcess(
    long		/* -KILL in unix */
);
CExternal void TerminateProcess(
    long		/* -TERM in unix */
);

CExternal void exit(
    int
);

CExternal void CatchTerminateSignal(
    VoidFuncPtr func
);

CExternal void DefaultTerminateSignal(
    void
);

#if defined(THREAD_SUPPORT) || defined(THINK_C) || defined(__MWERKS__)
    extern void YieldProcessor(void);
#else
#   define YieldProcessor()	/* as nothing */
#endif

#if defined(THINK_C) || defined(__MWERKS__)
    extern void AcquireMacArgs(char *program, int *argc, char ***argv);
    extern void InitMacintosh(char *program);
    extern void MacintoshStartup(int *pArgc, char ***pArgv);
    extern void MacintoshCleanup(void);
#else
#   define AcquireMacArgs(program, argc, argv)	/* as nothing */
#   define InitMacintosh(program)		/* as nothing */
#   define MacintoshStartup(pArgc, pArgv)       /* as nothing */
#   define MacintoshCleanup()			/* as nothing */
#endif /* THINK_C */

#endif /* _PROCESS_H_ */
