/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	<file name> - <one-line description of file contents>
 *
 * DESCRIPTION
 *	<overall description of what the c code does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/testaio.c,v 1.4 1994/12/17 13:02:01 russ Exp $
 */
#define NeedFunctionPrototypes 1

#include "osio.h"
#include "nvrAlloc.h"

char *ProgramName;

typedef struct {
    long totalLength;
    boolean finished;
    char *buf;
} Data, *DataPtr;

typedef struct {
    long recordSize;
    UChar byteValue;
    int operationId;
    DataPtr data;
} RecordInfo, *RecordInfoPtr;

typedef struct {
    char *fileName;
    int nRecords;
    int maxActive;
    int recordSize;
} TestParam, *TestParamPtr;

TestParam DefaultParam = { "foo", 100, 6, 9000 };

void
StretchData(DataPtr record, UChar byteValue, int newLength)
{
    int i;

    if (newLength > record->totalLength) {
	record->buf = NullRealloc(record->buf, newLength);
	record->totalLength = newLength;
    }

    for (i = 0; i < newLength; i++)
	record->buf[i] = byteValue;
    record->finished = False;
}

void
TestFile(TestParamPtr param)
{
    RecordInfoPtr recordHistory;
    RecordInfoPtr incomplete, unused, last;
    long currentOffset = 0;
    DataPtr records;
    UChar byteValue = 0;
    int i;
    int fd;
    int result;

    if ((fd = OpenFileForWriting(param->fileName)) < 0)
	Fatal("can't create %s\n", param->fileName);
    (void) ftruncate(fd, 0);

    recordHistory =
	(RecordInfoPtr)SecureAlloc(sizeof(RecordInfo) * param->nRecords);
    records = (DataPtr)SecureAlloc(sizeof(Data) * param->maxActive);
    memset(records, 0, sizeof(Data) * param->maxActive);

    incomplete = unused = recordHistory;
    last = recordHistory + param->nRecords;
    for (i = 0; i < param->maxActive && i < param->nRecords; i++) {
	unused->byteValue = byteValue++;
	unused->recordSize = param->recordSize;
	unused->data = &records[i];
	StretchData(unused->data, unused->byteValue, unused->recordSize);
	unused->operationId =
	    AsyncWrite(fd,
		       unused->data->buf,
		       param->recordSize,
		       currentOffset);
	Msg("async write(%d) @ %d=%d\n",
	    param->recordSize, currentOffset, unused->operationId);
	if (unused->operationId < 0)
	    Fatal("can't write record %d\n", i);
	currentOffset += param->recordSize;
	unused++;
    }

    while (incomplete < last || unused < last) {
	if (incomplete < last) {
	    if ((result = FinishAsyncIo(incomplete->operationId)) < 0)
		Error("record %d failed completion\n",
		    incomplete - recordHistory);
	    incomplete->data->finished = True;
	    incomplete++;
	}

	if (unused < last) {
	    unused->byteValue = byteValue++;
	    unused->recordSize = param->recordSize;
	    /*
	     * look for an unused buffer (this is more complicated than
	     * necessary).
	     */
	    for (i = 0; i < param->maxActive; i++) {
		if (records[i].finished) {
		    unused->data = &records[i];
		    break;
		}
	    }
	    if (i == param->maxActive)
		Fatal("can't find an open record\n");

	    StretchData(unused->data, unused->byteValue, param->recordSize);
	    unused->operationId =
		AsyncWrite(fd,
			   unused->data->buf,
			   param->recordSize,
			   currentOffset);
	    Msg("async write(%d) @ %d=%d\n",
		param->recordSize, currentOffset, unused->operationId);
	    currentOffset += param->recordSize;
	    unused++;
	}
    }

    for (i = 0; i < param->maxActive && i < param->nRecords; i++)
	free(records[i].buf);
    free(recordHistory);
    free(records);
}

main(argc, argv)
    int argc;
    char **argv;
{
    TestParam param;

    ProgramName = *argv;
    param = DefaultParam;

    if (argc == 1) {
	Msg("Usage: %s -a<n> -n<n> -s<n> file ...\n", ProgramName);
	Msg("    where -a specifies the # of active writes    (default: %d)\n",
	    DefaultParam.maxActive);
	Msg("          -n specifies the # of records to write (default: %d)\n",
	    DefaultParam.nRecords);
	Msg("          -s specifies the record size           (default: %d)\n",
	    DefaultParam.recordSize);
    }

    for (argc--, argv++; argc; argc--, argv++) {
	if (**argv != '-') {
	    param.fileName = *argv;
	    TestFile(&param);
	    continue;
	}
	switch(argv[0][1]) {
	case 'a':
	    param.maxActive = atoi(argv[0]+2);
	    break;
	case 'n':
	    param.nRecords = atoi(argv[0]+2);
	    break;
	case 's':
	    param.recordSize = atoi(argv[0]+2);
	    break;
	default:
	    Fatal("arg %s?\n", *argv);
	}
    }
}
