/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	testlwp.c - test program
 *
 * DESCRIPTION
 *	OS-DEPENDENT
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/testlwp.c,v 1.5 1994/01/10 18:48:13 toddb Exp $
 */
#ifdef sun
#define NeedFunctionPrototypes 1

#include "osio.h"
#include "nvrAlloc.h"
#include <lwp/lwp.h>
#include <lwp/stackdep.h>
#include <lwp/lwperror.h>
#include <lwp/lwpmachdep.h>

#define HIGH_PRIORITY	10
#define BUFFER_SIZE	(4*1024)

char *ProgramName;

typedef struct {
    boolean complete;
    UChar data[ BUFFER_SIZE ];
    int dataLength;
    int fd;
} OutputInfo;

char *ProgressGraphic = "\b/\b\\\b-\b/\b\\\b-\b/\b\\\b-\b/\b\\\b-\b/\b\\\b-";

char *ThreadBlockNames[] = {
    "not",
    "cond. var.",
    "mon",
    "thr. or agent",
};

int ServerSpins;
boolean ServerWriting;

void ServerFunction(OutputInfo *data)
{
    thread_t self;
    char *p;

    lwp_self(&self);
    p = ProgressGraphic;

    for (;;) {
	if (data->complete) {
	    ServerSpins++;
	    lwp_suspend(&self);
	    continue;
	}
	ServerSpins++;
	write(1, p, 2);
	lwp_setpri(&self, HIGH_PRIORITY-1);
	ServerWriting = True;
	write(data->fd, data->data, data->dataLength);
	ServerWriting = False;

	p += 2;
	if ((p - ProgressGraphic) >= sizeof(ProgressGraphic))
	    p = ProgressGraphic;

	data->complete = True;
    }
}

main(argc, argv)
    int argc;
    char **argv;
{
    statvec_t lwpStatus;
    stkalign_t *srvStack;
    machstate_t machState;
    thread_t server;
    thread_t self;
    int status;
    OutputInfo data;
    int clientSpins;
    int i;
    char *fileName = "foo";
    struct timeval timeout;

    ProgramName = *argv;

    for (argc--, argv++; argc; argc--, argv++) {
	if (**argv != '-') {
	    fileName = *argv;
	    continue;
	}
	switch(argv[0][1]) {
	case 'a':
	    break;
	case 'n':
	    break;
	case 's':
	    break;
	default:
	    Fatal("arg %s?\n", *argv);
	}
    }

    pod_setmaxpri(HIGH_PRIORITY);

    lwp_self(&self);
    printf("lwp_self() = id,key: %p,%d\n", self.thread_id, self.thread_key);

    if (lwp_setpri(&self, HIGH_PRIORITY) < 0)
	Fatal("lwp_setpri() fails\n");

    printf("pod_getmaxpri() = %d\n", pod_getmaxpri());

    printf("lwp_setstkcache(%d,1)=%d\n",
	MINSIGSTKSZ, lwp_setstkcache(MINSIGSTKSZ, 1));

    srvStack = lwp_newstk();
    printf("lwp_newstk() = %p\n", srvStack);

    data.complete = True;
    data.dataLength = BUFFER_SIZE;

    if ((data.fd = OpenFileForWriting(fileName)) < 0)
	Fatal("can't open %s\n", fileName);
    ftruncate(data.fd, 0);

    status = lwp_create(&server,
			ServerFunction,
			HIGH_PRIORITY,
			LWPSERVER,
			srvStack,
			1, &data);
    if (status < 0)
	Fatal("can't create lwp: %s\n", lwp_errstr()[ lwp_geterr() ]);
    printf("lwp_create() = id,key: %p,%d\n",
	server.thread_id, server.thread_key);

    timeout.tv_sec = 0;
    timeout.tv_usec = 1;
    clientSpins = 0;
    for (i = 0; i < 1000; i++) {
	data.complete = False;
	lwp_setpri(&server, HIGH_PRIORITY);
	do {
	    lwp_getstate(&server, &lwpStatus);
	    if (ServerWriting)
		Msg("--%d\n", ServerSpins);
	    lwp_resume(&server);
	    if (lwpStatus.stat_prio == HIGH_PRIORITY)
		lwp_resched(HIGH_PRIORITY);
	    else
		lwp_sleep(&timeout);
	    clientSpins++;
	} while (! data.complete);
    }
    printf("\nserver/client spins = %d/%d\n", ServerSpins, clientSpins);
}
#endif sun
