/*
 * (c) Copyright 1992 North Valley Research, Inc.
 *         All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	unixFileAcc.c
 *
 * DESCRIPTION
 *	OS-specific access to files (permissions, etc).
 *	OS-DEPENDENT
 *
 * REVISION
 *	$Header: /usr/local/src/nvrToMIT/RCS/unixFileAcc.c,v 1.6 1994/12/17 13:01:51 russ Exp $
 */

#define NeedFunctionPrototypes 1

#include "fileAccessI.h"
#include "osio.h"
#include "nvrAlloc.h"
#include "nvrString.h"

char *
ExtractPathPrefix(char *path)
{
    char *filePart, *prefix;
    int prefixLength;

    for (filePart = path + strlen(path) - 1; filePart >= path; filePart--) {
	if (*filePart == '/') {
	    filePart++;
	    prefixLength = filePart - path;
	    prefix = SecureAlloc(prefixLength + 1);
	    memcpy(prefix, path, prefixLength);
	    prefix[ prefixLength ] = '\0';
	    return(prefix);
	}
    }
    return (NULL);
}

char *
ExtractPathSuffix(char *path)
{
    char *filePart;
    
    for (filePart = path + strlen(path) - 1; filePart >= path; filePart--) {
	if (*filePart == '/')
	    return(filePart + 1);
    }
    return (path);
}


boolean
FileReadable(char *path)
{
    return (access(path, R_OK) == 0);
}

boolean
FileCreatable(char *path)
{
    char *p;
    char directory[ BUFSIZ ];

    strcpy(directory, path);
    p = directory+strlen(directory);
    if (p == directory || (directory[0] == '/' && directory[1] == '\0'))
	return(False);	/* empty path or just "/" */
    while (p != directory) {
	if (*--p == '/')
	    break;
    }
	
    if (*p == '/') {
	if (p == directory)	 /* its just "/something" */
	    p++;
	*p = '\0';
    } else {
	/* its just "something", so assume "." */
	directory[0] = '.';
	directory[1] = '\0';
    }
    return( access(directory, W_OK) == 0 );
}

boolean
FileWritable(char *path)
{
    return (access(path, W_OK) == 0);
}

boolean GetLinkName(char *path, char *buf, int size)
{
    int    cnt;

    cnt = readlink(path, buf, size);
    if (cnt < 0) {
        Error("Cannot readlink(%s,0x%x,%d)\n",
            path, buf, size);
        return(False);
    }
    if (cnt >= size)
        Fatal("Oops: readlink() returned %d bytes\n", cnt);
    buf[ cnt ] = '\0';
    return(True);
}

void *
OpenDir(char *path)
{
    DIR   *dirp;
    char  *cp;

    if( (dirp = opendir(path)) == NULL ) {
	if( errno == ENOENT && (cp = strrchr(path,'/')) ) {
	    *cp = '\0';
	    dirp = opendir(path);
	}
    }
    return ((void *)dirp);
}


void
CloseDir(void *dirp)
{
    closedir((DIR *)dirp);
    return;
}

char *
ReadDir(void *dirp)
{

    DirPtr dp;

    dp = readdir((DIR *)dirp);
    if( dp )
        return (dp->d_name);
    return (NULL);
}

char *
GetCurrentDir(void)
{
    static char pathname[MAXPATHLEN];
    char *getwd(char *);

    if (getwd(pathname) == NULL) {
	pathname[0] = '/';
	pathname[1] = '\0';
    }
    return(pathname);
}

#if defined(Solaris2) || ( defined(SYSV) && !defined(sgi) )
/*
 *  getwd():  Remap BSD getwd() to POSSIX getcwd().
 *
 *  getcwd(buf, size)
 *  char   *buf;
 *  int    size;
 *
 *  This assumes that cwd is of size BUFSIZ.
 *
 */
char *getwd(cwd)
char    *cwd;
{
    char    *getcwd();

    return (getcwd(cwd, BUFSIZ));
}
#endif /* SYSV */
