static char *rcsid="$Header: /usr/local/src/nvrToMIT/RCS/unixNetwork.c,v 1.4 1995/08/21 20:47:11 russ Exp $";
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>
#include "osio.h"

#if defined(Solaris2) || defined(sgi)
# define HaveIpMulticast
#endif

FileHandle
MakeTcpConnection(char *host, int port, InternalReasonPtr *pReasonPtr)
{
    int netFd, failure;
    struct sockaddr_in netSocket;
    UShort netPort = port;
    struct hostent *hPtr;


    memset(&netSocket,0,sizeof(netSocket));
    /*
     * Ascertain host 
     */
    hPtr = gethostbyname(host);
    if (hPtr == NULL) {
	netSocket.sin_family = AF_INET;
	netSocket.sin_addr.s_addr = htonl(inet_addr(host));
    } else {
	netSocket.sin_family = hPtr->h_addrtype;
	netSocket.sin_addr.s_addr =
		htonl(((struct in_addr *)(hPtr->h_addr))->s_addr);
    }
    netSocket.sin_port = htons(netPort);
    
    /*
     * Create socket
     */
    netFd = socket(netSocket.sin_family, SOCK_STREAM, 0);
    if (netFd < 0) {
	*pReasonPtr = CreateSysReason("socket failure\n");
	return(-1);
    }
    
    /*
     * Establish a connectionto client
     */
    failure = connect(netFd, (struct sockaddr *)&netSocket,
		      sizeof(netSocket));
    if (failure) {
	*pReasonPtr = CreateSysReason("connect failure\n");
	return(-1);
    }

    *pReasonPtr = NULL;
    return((FileHandle)netFd);
}

FileHandle
MakeUdpConnection(char *host, int port, InternalReasonPtr *pReasonPtr)
{
    int netFd, failure;
    struct sockaddr_in netSocket;
    UShort netPort = port;
    ULong netAddr;
    struct hostent *hPtr;
    int intArg;


    memset(&netSocket,0,sizeof(netSocket));
    /*
     * Ascertain host 
     */
    hPtr = gethostbyname(host);
    if (hPtr == NULL) {
	netSocket.sin_family = AF_INET;
	netAddr = inet_addr(host);
    } else {
	netSocket.sin_family = hPtr->h_addrtype;
	netAddr = ((struct in_addr *)(hPtr->h_addr))->s_addr;
    }
    netSocket.sin_addr.s_addr = htonl(netAddr);
    netSocket.sin_port = htons(netPort);
    
    /*
     * Create socket
     */
    netFd = socket(netSocket.sin_family, SOCK_DGRAM, 0);
    if (netFd < 0) {
	*pReasonPtr = CreateSysReason("socket failure\n");
	return(-1);
    }
    
    /*
     * set ttl for multicast addresses
     */
    if (IN_MULTICAST(netAddr)) {
#ifdef HaveIpMulticast
	ULong ttl = DEFAULT_TTL;

	failure = setsockopt(netFd, IPPROTO_IP, IP_MULTICAST_TTL,
		             (char *) &ttl, sizeof(ttl));
#else
	*pReasonPtr = CreateReason("IP multicasting not available\n");
	return(-1);
#endif
    }
    
#if	   0
    intArg = DEFAULT_UDP_BUFSIZ;
    setsockopt(netFd, SOL_SOCKET, SO_SNDBUF, (char *) &intArg, sizeof(intArg));
#endif	/* 0 */
    
	
    /*
     * Establish a connectionto client
     */
    failure = connect(netFd, (struct sockaddr *)&netSocket,
		      sizeof(netSocket));
    if (failure) {
	*pReasonPtr = CreateSysReason("connect failure\n");
	return(-1);
    }

    *pReasonPtr = NULL;
    return((FileHandle)netFd);
}

FileHandle
MakeUdpListenSocket(int port, CastType castType, void *info,
		    InternalReasonPtr *pReasonPtr)
{
#define HOSTLEN 64
    int netFd, failure;
    struct sockaddr_in netSocket;
    UShort netPort = port;
    struct hostent *hPtr;


    netFd = socket(AF_INET, SOCK_DGRAM, 0);
    if (netFd < 0) {
	*pReasonPtr = CreateSysReason("socket failure\n");
	return(-1);
    }

    if (castType == multiCast) {
#ifdef HaveIpMulticast
	char hostname[HOSTLEN];
	struct hostent *hPtr;
	struct ip_mreq imr;
	char *multiAddr = (char *)info;

	gethostname(hostname,HOSTLEN);
	hPtr = gethostbyname(hostname);
	if (hPtr == NULL) {
	    *pReasonPtr = CreateSysReason("gethostname failure");
	    return(-1);
	}
	memcpy((char *)&imr.imr_interface.s_addr,
	       hPtr->h_addr, hPtr->h_length);
	hPtr = gethostbyname(multiAddr);
	if (hPtr == NULL) {
	    imr.imr_multiaddr.s_addr = htonl(inet_addr(multiAddr));
	} else {
	    imr.imr_multiaddr.s_addr =
		htonl(((struct in_addr *)(hPtr->h_addr))->s_addr);
	}

	failure = setsockopt(netFd, IPPROTO_IP, IP_ADD_MEMBERSHIP,
			     (char *) &imr, sizeof(imr));
	if (failure) {
	    *pReasonPtr =
		CreateSysReason("Cannot join multicast group %s\n",multiAddr);
	    return(-1);
       }
#else
        *pReasonPtr = CreateReason("IP multicasting not available\n");
        return(-1);
#endif
    }

    if (castType == broadCast || castType == multiCast) {
	int arg;

	arg = 1;
	setsockopt(netFd, SOL_SOCKET, SO_REUSEADDR, (char *) &arg, sizeof(arg));
#if	   0
	arg = DEFAULT_UDP_BUFSIZ;
	setsockopt(netFd, SOL_SOCKET, SO_RCVBUF, (char *) &arg, sizeof(arg));
#endif	/* 0 */
    }

    memset(&netSocket,0,sizeof(netSocket));
    netSocket.sin_family = AF_INET;
    netSocket.sin_port = htons(netPort);
    netSocket.sin_addr.s_addr = INADDR_ANY;

    failure = bind(netFd, (struct sockaddr *)&netSocket, sizeof(netSocket));
    if (failure) {
	*pReasonPtr =
	     CreateSysReason("bind failure: cannot bind to addr %s port %d\n",
			     inet_ntoa(netSocket.sin_addr), port);
	return(-1);
    }

    
    *pReasonPtr = NULL;
    return(netFd);
    
#undef HOSTLEN
}
	
