/*
 * (c) Copyright 1992 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	process.c
 *
 * DESCRIPTION
 *	Management of sub-processes
 *	OS-DEPENDENT
 *
 * REVISION
 *	$Header: /usr/local/src/nvrToMIT/RCS/unixProcess.c,v 1.14 1995/10/27 22:38:52 russ Exp $
 */

#define NeedFunctionPrototypes 1

#include "process.h"
#include "osio.h"
#include "nvrString.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>
 
 
#ifdef _HPUX_SOURCE
  typedef int WaitStatus;
  typedef int *WaitStatusPtr;
# define usleep(count)        getpid()        /* this at least reschedules */
#endif

#ifdef sgi
  typedef union wait WaitStatus;
  typedef union wait *WaitStatusPtr;
# define usleep(count)        getpid()        /* this at least reschedules */
#endif

#ifdef sun
# ifdef Solaris2
#  define usleep(count)       getpid()        /* this at least reschedules */
   typedef int WaitStatus;
    typedef int *WaitStatusPtr;
# else
   typedef union wait WaitStatus;
   typedef union wait *WaitStatusPtr;
# endif
#endif

int
CreatePipe(FileHandle ToFrom[2]) 
{
    return( pipe(ToFrom) );
}

static int
CreateSocketPair(FileHandle ToFrom[2]) 
{
    return( socketpair(AF_UNIX,SOCK_STREAM,0,ToFrom) );
}

static OneWayPipe
_PipeTo(Argvec *argv, void (*argumentRewriter)(),
       int (*createPair)(FileHandle[2]))
{
    int pid;
    int pipeFd[2];
    OneWayPipe result;

    if ((*createPair)(pipeFd) < 0)
        Fatal("can't creat pipe\n");
    if (argumentRewriter)
	(*argumentRewriter)(argv, pipeFd[0]);
    if ((pid = vfork()) < 0)
        Fatal("can't fork\n");
    if (pid == 0) {
	if (argumentRewriter == NULL) {
	    close(pipeFd[1]);
	    dup2(pipeFd[0], 0);
	    close(pipeFd[0]);
	    CloseOpenFileDescriptors(0x7);
	} else {
	    close(pipeFd[1]);
	    CloseOpenFileDescriptors(0x6 | (1<<pipeFd[0]));
	}
        execvp(argv->path, argv->argv);
        Fatal("can't exec %s\n", argv->path);
    }

    close(pipeFd[0]);

    result.pid = pid;
    result.outFd = pipeFd[1];
    return(result);
}

/* uses CreateSocketPair() */
OneWayPipe
PipeTo(argv, argumentRewriter)
    Argvec *argv;
    void (*argumentRewriter)();
{
    int (*createPair)(FileHandle[2]);
    createPair = CreateSocketPair;

    return( _PipeTo(argv, argumentRewriter, createPair) );
}

/* uses CreatePipe() */
OneWayPipe
AltPipeTo(argv, argumentRewriter)
    Argvec *argv;
    void (*argumentRewriter)();
{
    int (*createPair)(FileHandle[2]);
    createPair = CreatePipe;

    return( _PipeTo(argv, argumentRewriter, createPair) );
}

static OneWayPipe
_PipeToWithStdout(Argvec *argv, int newStdout, int (*createPair)(FileHandle[2]))
{
    int pid;
    int pipeFd[2];
    OneWayPipe result;

    if (CreateSocketPair(pipeFd) < 0)
        Fatal("can't creat pipe\n");
    if ((pid = vfork()) < 0)
        Fatal("can't fork\n");
    if (pid == 0) {
	close(pipeFd[1]);
	if (pipeFd[0] != 0) {
	    dup2(pipeFd[0], 0);
	    close(pipeFd[0]);
	}
	if (newStdout != 1) {
	    dup2(newStdout, 1);
	    close(newStdout);
	}
	CloseOpenFileDescriptors(0x7);
        execvp(argv->path, argv->argv);
        Fatal("can't exec %s\n", argv->path);
    }

    close(pipeFd[0]);

    result.pid = pid;
    result.outFd = pipeFd[1];
    return(result);
}

/* uses CreateSocketPair() */
OneWayPipe
PipeToWithStdout(Argvec *argv, int newStdout)
{
    int (*createPair)(FileHandle[2]);
    createPair = CreateSocketPair;

    return( _PipeToWithStdout(argv, newStdout, createPair) );
}

/* uses CreatePipe() */
OneWayPipe
AltPipeToWithStdout(Argvec *argv, int newStdout)
{
    int (*createPair)(FileHandle[2]);
    createPair = CreatePipe;

    return( _PipeToWithStdout(argv, newStdout, createPair) );
}

static TwoWayPipe
_PipeToAndFrom(Argvec *argv, void (*argumentRewriter)(),
	       int (*createPair)(FileHandle[2]))
{
    int pid;
    int pipeFdTo[2];
    int pipeFdFrom[2];
    TwoWayPipe result;

    if (CreateSocketPair(pipeFdTo) < 0 || CreateSocketPair(pipeFdFrom) < 0)
        Fatal("can't creat pipes\n");
    if (argumentRewriter)
	(*argumentRewriter)(argv, pipeFdTo[0], pipeFdFrom[1]);
    if ((pid = vfork()) < 0)
        Fatal("can't fork\n");
    if (pid == 0) {
	close(pipeFdTo[1]);
	close(pipeFdFrom[0]);
	if (argumentRewriter == NULL) {
	    dup2(pipeFdTo[0], 0);
	    close(pipeFdTo[0]);
	    dup2(pipeFdFrom[1], 1);
	    close(pipeFdFrom[1]);

	    CloseOpenFileDescriptors(0x7);
	} else {
	    CloseOpenFileDescriptors(0x6 |(1<<pipeFdTo[0])|(1<<pipeFdFrom[1]));
	}
        execvp(argv->path, argv->argv);
        Fatal("can't exec %s\n", argv->path);
    }

    close(pipeFdTo[0]);
    close(pipeFdFrom[1]);

    result.pid = pid;
    result.outFd = pipeFdTo[1];
    result.inFd = pipeFdFrom[0];
    return(result);
}

/* uses CreateSocketPair() */
TwoWayPipe
PipeToAndFrom(Argvec *argv, void (*argumentRewriter)())
{
    int (*createPair)(FileHandle[2]);
    createPair = CreateSocketPair;

    return( _PipeToAndFrom(argv, argumentRewriter, createPair) );
}

/* uses CreatePipe() */
TwoWayPipe
AltPipeToAndFrom(Argvec *argv, void (*argumentRewriter)())
{
    int (*createPair)(FileHandle[2]);
    createPair = CreatePipe;

    return( _PipeToAndFrom(argv, argumentRewriter, createPair) );
}

static InputPipe
_PipeFromWithStdin(Argvec *argv, int newStdin, int (*createPair)(FileHandle[2]))
{
    int pid;
    int pipeFdFrom[2];
    InputPipe result;

    if (CreateSocketPair(pipeFdFrom) < 0)
        Fatal("can't creat pipe\n");

    if ((pid = vfork()) < 0)
        Fatal("can't fork\n");
    if (pid == 0) {
	close(pipeFdFrom[0]);
	if (newStdin > 0) {
	    dup2(newStdin, 0);
	    close(newStdin);
	}
	if (pipeFdFrom[1] != 1) {
	    dup2(pipeFdFrom[1], 1);
	    close(pipeFdFrom[1]);
	}
	CloseOpenFileDescriptors(0x7 | (1<<pipeFdFrom[1]));
        execvp(argv->path, argv->argv);
        Fatal("can't exec %s\n", argv->path);
    }

    close(pipeFdFrom[1]);

    result.pid = pid;
    result.inFd = pipeFdFrom[0];
    return(result);
}

/* uses CreateSocketPair() */
InputPipe
PipeFromWithStdin(Argvec *argv, int newStdin)
{
    int (*createPair)(FileHandle[2]);
    createPair = CreateSocketPair;

    return( _PipeFromWithStdin(argv, newStdin, createPair) );
}

/* uses CreatePipe() */
InputPipe
AltPipeFromWithStdin(Argvec *argv, int newStdin)
{
    int (*createPair)(FileHandle[2]);
    createPair = CreatePipe;

    return( _PipeFromWithStdin(argv, newStdin, createPair) );
}

static TwoWayPipe
_PipeToAndFromWithStdin(Argvec *argv, int newStdin,
			VoidFuncPtr argumentRewriter,
			int (*createPair)(FileHandle[2]))
{
    int pid;
    int pipeFdTo[2];
    int pipeFdFrom[2];
    TwoWayPipe result;

    if (argumentRewriter == NULL)
        Fatal("PipeToAndFromWithStdin: must supply an argumentRewriter\n");
    if (newStdin == 0)
	return( PipeToAndFrom(argv, argumentRewriter) );

    if (CreateSocketPair(pipeFdTo) < 0 || CreateSocketPair(pipeFdFrom) < 0)
        Fatal("can't creat pipes\n");
    /*
     * if stdin was closed, pipeFdTo would end up with 0 as one
     * of the file descriptors.  If so, move it.
     */
    if (pipeFdTo[0] == 0) {
	pipeFdTo[0] = dup(pipeFdTo[0]);
	close(0);
    } else if (pipeFdTo[1] == 0) {
	pipeFdTo[1] = dup(pipeFdTo[1]);
	close(0);
    }

    (*argumentRewriter)(argv, pipeFdTo[0], pipeFdFrom[1]);
    if ((pid = vfork()) < 0)
        Fatal("can't fork\n");
    if (pid == 0) {
	close(pipeFdTo[1]);
	close(pipeFdFrom[0]);
	dup2(newStdin, 0);
	close(newStdin);
	CloseOpenFileDescriptors(0x7 | (1<<pipeFdTo[0]) | (1<<pipeFdFrom[1]));
        execvp(argv->path, argv->argv);
        Fatal("can't exec %s\n", argv->path);
    }

    close(pipeFdTo[0]);
    close(pipeFdFrom[1]);

    result.pid = pid;
    result.outFd = pipeFdTo[1];
    result.inFd = pipeFdFrom[0];
    return(result);
}

TwoWayPipe
PipeToAndFromWithStdin(Argvec *argv, int newStdin,
		       VoidFuncPtr argumentRewriter)
{
    int (*createPair)(FileHandle[2]);
    createPair = CreateSocketPair;

    return( _PipeToAndFromWithStdin(argv, newStdin, argumentRewriter,
				    createPair) );

}

TwoWayPipe
AltPipeToAndFromWithStdin(Argvec *argv, int newStdin,
			  VoidFuncPtr argumentRewriter)
{
    int (*createPair)(FileHandle[2]);
    createPair = CreatePipe;

    return( _PipeToAndFromWithStdin(argv, newStdin, argumentRewriter,
				    createPair) );
}

SendSignal(long processID, int sigNumber)
{
    extern int errno;

    if (sigNumber != SIGSTOP)
        (void) kill(processID, SIGCONT);

    errno = 0;
    if (kill(processID, sigNumber) < 0 && errno != ESRCH)
	Error("Couldn't kill process ID %d\n", processID);
}

static void
CatchSignal(int sigNumber, VoidFuncPtr func)
{
    if( signal(sigNumber, func) == (VoidFuncPtr)-1 ) {
	Error("Couldn't catch signal %d\n", sigNumber);
    }
}

static void
DefaultSignal(int sigNumber)
{
    if( signal(sigNumber, SIG_DFL) == (VoidFuncPtr)-1 ) {
	Error("Couldn't restor signal %d to default action\n", sigNumber);
    }
}

void KillProcess(long processID)
{
    SendSignal(processID, SIGKILL);
}

void TerminateProcess(long processID)
{
    SendSignal(processID, SIGTERM);
}

void CatchTerminateSignal(VoidFuncPtr func)
{
    CatchSignal(SIGTERM,func);
}

void DefaultTerminateSignal(void)
{
    DefaultSignal(SIGTERM);
}

void SuspendProcess(long processID)
{
    SendSignal(processID, SIGSTOP);
}

void ResumeProcess(long processID)
{
    (void) kill(processID, SIGCONT);
}

static void
ShowWaitStatus(long pid, WaitStatusPtr status)
{
    Msg("pid %d ", pid);
    if (WIFSTOPPED(*status)) {
	Msg("stopped\n");
    }
    if (WIFEXITED(*status)) {
	Msg("exited with code %d.\n", WEXITSTATUS(*status));
    }
    if (WIFSIGNALED(*status)) {
	Msg("exited with signal %d.\n", WTERMSIG(*status));
    }
}

ProcessState
WaitForPid(long pid, boolean verbose)
{
    WaitStatus reapedStatus;
    boolean foundIt = False;
    long reapedPid;
    ProcessState state = ProcIsRunning;

    for (;;) {
	memset((char *)&reapedStatus, 0, sizeof(WaitStatus));

#ifdef Solaris2
	reapedPid = waitpid(-1, &reapedStatus, WNOHANG | WUNTRACED);
#else
	reapedPid = wait3(&reapedStatus, WNOHANG | WUNTRACED, NULL);
#endif

	if (verbose)
	    ShowWaitStatus(reapedPid, &reapedStatus);

	if (reapedPid == pid) {
	    foundIt = True;
	    if (WIFSTOPPED(reapedStatus)) {
		state = ProcIsSuspended;
	    } else if (WIFEXITED(reapedStatus)) {
		state = ProcIsDead;
	    } else if (WIFSIGNALED(reapedStatus)) {
		state = ProcIsDead;
	    }
	}
	
	if (foundIt) {
	    if (reapedPid <= 0)
	    	break;
	} else if (reapedPid < 0) {
	    Error("can't wait for pid %d\n", pid);
	    state = ProcIsDead;
	    break;
	} else {
	    usleep(1);
	    continue;	/* not done yet */
	}
    }
    return(state);
}

void
Execute(argv)
    Argvec *argv;
{
    int pid;
    int stdInput;

    if ((pid = fork()) < 0)
        Fatal("can't fork\n");
    if (pid != 0) {
	/* this is the main program... wait for the child that exits below */
	WaitForPid(pid, False);
	return;
    }

    if ((pid = vfork()) < 0)
        Fatal("can't fork\n");
    if (pid != 0)
	exit(0);	/* this is the child program */

    /* this is the grand child */
    stdInput = OpenFileForReading("/dev/null");
    if (stdInput < 0)
	CloseFile(0);
    else if (stdInput > 0) {
	dup2(stdInput, 0);
	CloseFile(stdInput);
    }
    CloseOpenFileDescriptors(0x7);
    execvp(argv->path, argv->argv);
    Fatal("can't exec %s\n", argv->path);
}
