/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	xextension.c - get information about known X extensions
 *
 * DESCRIPTION
 *	For extensions themselves, NVR applications don't want to
 *	deal with strings that represent extensions, so here we
 *	reduce them to a set of bits.
 *	X-DEPENDENT
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/xextension.c,v 1.6 1994/02/27 22:18:05 toddb Exp $
 */
#include <stdio.h>
#include "nvrTypes.h"
#include "xextension.h"
#include "nvrPlx.h"
#include "nvrMvex.h"


static struct {
    char *name;
    ULong flag;
} ExtensionMap[] = {
    { "SUN_DGA",		VDPY_SUN_DGA },
    { "SunWindowGrabber",	VDPY_SUN_WINDOW_GRABBER },
    { "SUN_ALLPLANES",		VDPY_SUN_ALLPLANES },
    { "ParallaxVideo",		VDPY_PARALLAX },
    { "MVEX",			VDPY_MVEX },
};


static ULong ConfirmMpeg(Display *dpy, int screenIndex, ULong abilities)
{
    /*
     * This should be done in the widget
     */
    abilities |= VDPY_SWMPEG;
    return (abilities);
}

extern ULong FindVideoAbilities(dpy, screenIndex)
    Display *dpy;
    int screenIndex;
{
    int n = 0;
    int i, j;
    char **extlist = XListExtensions (dpy, &n);
    ULong abilities = 0;
    ULong mask;
    

    if (extlist == NULL)
	return (0);

    for (i = 0; i < n; i++) {
	for (j = 0 ; j < NumberOfCells(ExtensionMap); j++) {
	    if (strcmp(ExtensionMap[j].name, extlist[i]) == 0)
		abilities |= ExtensionMap[j].flag;
	}
    }
    XFreeExtensionList (extlist);

    mask = abilities;
    for (i = 0; i < 32; i++) {
	switch((1<<i) & mask) {
	case VDPY_SUN_DGA:
	case VDPY_SUN_WINDOW_GRABBER:
	case VDPY_SUN_ALLPLANES:
	    /* so far, this is as much as we want to know */
	    break;

	case VDPY_PARALLAX:
	    abilities = ConfirmParallax(dpy, screenIndex, abilities);
	    break;
	case VDPY_MVEX:
	    abilities = ConfirmMVEX(dpy, screenIndex, abilities);
	    break;
	}
    }

    abilities = ConfirmMpeg(dpy, screenIndex, abilities);

    return (abilities);
}
