/*
 *	(c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		idctMotion.c - motion-related idct functions
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/idctMotion.c,v 1.19 1995/10/20 20:09:19 toddb Exp $
 */
#include "nvrIdctI.h"
#include "idctMotionI.h"

static void
ShowPostMotionValue(UChar *pPixel, int lineLength)
{
    int row, col;

    for (row = 0; row < 8; row++) {
        Msg("                 %c ", row == 0 ? '=' : ' ');
        for (col = 0; col < 8; col++)
            Msg("%4d", pPixel[ col ]);
        pPixel += lineLength;
        Msg("\n");
    }
    Msg("\n");
}

enum {
    MotionHalf      = 0x01,
    HorizShift      = 0,
    VertShift       = 1,
    LumaShift       = 0,
    ChromaShift     = 2,
    MotionHalfHoriz = MotionHalf << HorizShift,
    MotionHalfVert  = MotionHalf << VertShift,
    LumaMotionMask  = (MotionHalfHoriz | MotionHalfVert),
    HorizYShift     = HorizShift + LumaShift,
    HorizCShift     = HorizShift + ChromaShift,
    VertYShift      = VertShift + LumaShift,
    VertCShift      = VertShift + ChromaShift,
    MotionHalfYHoriz = MotionHalf << HorizYShift,
    MotionHalfCHoriz = MotionHalf << HorizCShift,
    MotionHalfYVert  = MotionHalf << VertYShift,
    MotionHalfCVert  = MotionHalf << VertCShift
};

#define DivideByTwo(n)  (((n) + (((n)>>31)&1)) >> 1)

#define AdjustPelMotion(_fullPel, _x, _y, _cx, _cy, halfFlags)		\
    if (_x) {								\
	_x <<= _fullPel;						\
	_cx = DivideByTwo(_x);						\
	halfFlags = ((_cx&1) << HorizCShift);				\
	_cx >>= 1;							\
	halfFlags |= ((_x&1) << HorizYShift);				\
	_x >>= 1;							\
    } else {								\
	halfFlags = 0;							\
	_cx = 0;							\
    }									\
    if (_y) {								\
	_y <<= _fullPel;						\
	_cy = DivideByTwo(_y);						\
	halfFlags |= ((_cy&1) << VertCShift);				\
	_cy >>= 1;							\
	halfFlags |= ((_y&1) << VertYShift);				\
	_y >>= 1;							\
    } else {								\
	_cy = 0;							\
    }

/*
 * Here, we initialize priv->pmc to point to the appropriate McSource
 * for the motion type at hand.  Then we do the motion compensation.
 */
void
Idct420MotionCompensate(IdctParamPrivPtr priv, DctMacroblockPtr idctMb)
{
    int cx, cy, x, y, blocksMask;
    int picOffset, picType, offset, blkIndex;
    int srcPicWidth, dstPicWidth, row;
    ULong t0,t1,t2,t3,t4,t5,t6,t7,t8;
    boolean fullPel, allHalfFlags, halfFlags;
    boolean allHalfFwdFlags, halfFwdFlags;
    boolean allHalfBakFlags, halfBakFlags;
    int mcOffsets[ NDctPicTypes ];
    int fwdMcOffsets[ NDctPicTypes ], bakMcOffsets[ NDctPicTypes ];
    McSourcePtr pmc;
    UChar *pTo, *pTime, *pTimeNxtRow, *pFut, *pFutNxtRow, *pPast, *pPastNxtRow;
    DebugDeclare( YcImagePtr yci )
    DebugDeclare( boolean timeSwapped )

    switch (idctMb->flags.composite & DctMotionFwdBak) {
    case DctMotionBak:
        x = idctMb->motion[ kIndexBakX ];
        y = idctMb->motion[ kIndexBakY ];
        picOffset = priv->futureOffset;
	fullPel = priv->bakFullPel;
        DebugTask(DCTD_MotionMbAddr,
	    Msg("  raw motion: bak: (%d,%d)\n", x, y);
	    yci = priv->future;
	)
        goto singleMotion;
        
    case DctMotionFwd:
        x = idctMb->motion[ kIndexFwdX ];
        y = idctMb->motion[ kIndexFwdY ];
        picOffset = priv->pastOffset;
	fullPel = priv->fwdFullPel;
        DebugTask(DCTD_MotionMbAddr,
	    Msg("  raw motion: fwd: (%d,%d)\n", x, y);
	    yci = priv->past;
	)

    singleMotion:
	AdjustPelMotion(fullPel, x, y, cx, cy, allHalfFlags);

	offset = picOffset + x;
	if (y)
	    offset += y * priv->geom.pic[ Dct444Pic ].width;
	mcOffsets[ Dct444Pic ] = offset;

	offset = picOffset + cx;
	if (cy)
	    offset += cy * priv->geom.pic[ Dct420Pic ].width;
	mcOffsets[ Dct420Pic ] = offset;

	DebugTask(DCTD_MotionMbAddr,
	    UChar *target;
	    Msg("  present y(%3d,%3d): ",
		(priv->super.nextMbAddr % priv->geom.mbWidth) * 16,
		(priv->super.nextMbAddr / priv->geom.mbWidth) * 16);
	    target = priv->pTo[ Dct444Pic ]
		   + priv->geom.blkOffset[ priv->geom.nBlocks-1 ];
	    ShowPicXY(priv->current, target);
	    Msg("  time y  (%3d%c,%3d%c):",
		x, (allHalfFlags & MotionHalfYHoriz) ? 'h' : ' ',
		y, (allHalfFlags & MotionHalfYVert) ? 'h' : ' ');
	    ShowPicXY(yci, target + mcOffsets[ Dct444Pic ]);
	    target = priv->pTo[ Dct420Pic ]
		   + priv->geom.blkOffset[ 0 ];
	    Msg("  time c  (%3d%c,%3d%c):",
		cx, (allHalfFlags & MotionHalfCHoriz) ? 'h' : ' ',
		cy, (allHalfFlags & MotionHalfCVert) ? 'h' : ' ');
	    ShowPicXY(yci, target + mcOffsets[ Dct420Pic ]);
	)

	if (idctMb->flags.b.blocksMask == priv->allBlocksMask) {
	    if (allHalfFlags == 0) {
		/*
		 * All blocks have dct and all of the motion is on full-pel
		 * boundaries.  This can be done simply by telling the
		 * inverse dct functions where in the reference picture
		 * to get the pixels.
		 */
		pmc = priv->pmc = &priv->singleSrcMc;
		pmc->pSrc[ Dct444Pic ] = priv->pTo[ Dct444Pic ]
				       + mcOffsets[ Dct444Pic ];
		pmc->pSrc[ Dct420Pic ] = priv->pTo[ Dct420Pic ]
				       + mcOffsets[ Dct420Pic ];
		return;
	    }
	    if ((allHalfFlags & LumaMotionMask) == 0) {
		/*
		 * All blocks have dct and all of the *luma* motion is on
		 * full-pel boundaries.  for this, we will do the motion
		 * for the chroma and simply inform the
		 * inverse dct functions where in the reference picture
		 * to get the pixels for both.
		 */
		pmc = priv->pmc = &priv->lumaSingleSrcMc;
		pmc->pSrc[ Dct444Pic ] = priv->pTo[ Dct444Pic ]
				       + mcOffsets[ Dct444Pic ];
		pmc->pSrc[ Dct420Pic ] = priv->mcSrc;
		blocksMask = priv->blocksMask[ Dct420Pic ];
		blkIndex = priv->firstChromaBlock;
	    } else {
		/*
		 * all blocks have dct and half-pixel motion.
		 */
		pmc = priv->pmc = &priv->multiSrcMc;
		blocksMask = priv->allBlocksMask;
		blkIndex = 0;
	    }
	} else if (idctMb->flags.b.blocksMask == 0) {
	    /*
	     * The entire macroblock consists of motion without dct.
	     * do it all here.
	     */
	    pmc = priv->pmc = &priv->singleSrcMc;
	    pmc->pSrc[ Dct444Pic ] = priv->pTo[ Dct444Pic ];
	    pmc->pSrc[ Dct420Pic ] = priv->pTo[ Dct420Pic ];
	    blkIndex = 0;
	    blocksMask = priv->allBlocksMask;
	} else {
	    int blk, blkMask;
	    int picHalfFlags[ NDctPicTypes ];

	    /*
	     * Some blocks are copied, some have dct deltas.  Copy only
	     * what we have to, and do motion only where we have to.
	     * Here's the truth table:
	     *    dct    half      do mc    pmc->offset
	     *  present  pel         ?
	     * ----------------  -----------------------
	     * 1.   no   |  no        yes       cur picture
	     * 2.  yes   |  no         no       ref picture
	     * 3.   no   | yes        yes       cur picture
	     * 4.  yes   | yes        yes       priv->mcPix
	     */
	    pmc = priv->pmc = &priv->mixedSrcMc;
	    pmc->pSrc[ Dct444Pic ] = priv->pTo[ Dct444Pic ];
	    pmc->pSrc[ Dct420Pic ] = priv->pTo[ Dct420Pic ];
	    blk = 0;
	    blkMask = 1;
	    picHalfFlags[ Dct444Pic ] = allHalfFlags & LumaMotionMask;
	    picHalfFlags[ Dct420Pic ] = allHalfFlags >> ChromaShift;

	    /*
	     * we already know that we'll do work here for any of
	     * the blocks that have no idct (hence the compliment
	     * of the blocks mask).  We may also do those that
	     * have half-pel motion.
	     */
	    blocksMask = (~idctMb->flags.fastBlocksMask) & priv->allBlocksMask;

	    while (blkMask & priv->allBlocksMask) {
		picType = priv->geom.picType[ blk ];
		if ((blkMask & idctMb->flags.fastBlocksMask)
		 && picHalfFlags[ picType ]) {
		    /*
		     * #1
		     */
		    pmc->picWidth[ blk ] = 8;
		    pmc->offset[ blk ] = (priv->mcSrc + blk * 64)
				       - pmc->pSrc[ picType ];
		    blocksMask |= blkMask;
		} else {
		    /*
		     * #2, #3 & #4.
		     */
		    offset = priv->geom.blkOffset[ blk ];
		    
		    /*
		     * this if statement handles the difference for #3.
		     */
		    if (idctMb->flags.fastBlocksMask & blkMask)
			offset += mcOffsets[ picType ];
		    pmc->offset[ blk ] = offset;
		    pmc->picWidth[ blk ] = priv->geom.pic[ picType ].width;
		}
		blkMask <<= 1;
		blk++;
	    }
	    blkIndex = 0;
	}

	/*
	 * blocksMask is initialized above
	 */
	for (; blocksMask; blkIndex++) {
	    if (((1<<blkIndex) & blocksMask) == 0)
		continue;
	    blocksMask &= ~(1<<blkIndex);

            picType = priv->geom.picType[ blkIndex ];
            pTo = priv->pTo[ picType ]
                + priv->geom.blkOffset[ blkIndex ];
            pTime = pTo + mcOffsets[ picType ];

	    dstPicWidth = pmc->picWidth[ blkIndex ];
            srcPicWidth = priv->geom.pic[ picType ].width;

	    pTo = pmc->pSrc[ picType ] + pmc->offset[ blkIndex ];
	    if (picType == Dct444Pic)
		halfFlags = allHalfFlags & LumaMotionMask;
	    else    
		halfFlags = allHalfFlags >> ChromaShift;

	    DebugTask(DCTD_MotionBlkAddr,
                if (pmc->picWidth[ blkIndex ] != 8) {
		    Msg("    blk%2d present:    ", blkIndex);
		    ShowPicXY(priv->current, pTo);
		}
		Msg("    blk%2d time:       ", blkIndex);
		ShowPicXY(yci, pTime);
	    )

	    switch (halfFlags) {
	    case 0:
		/*
		 * time: x . . . (only used when copying blocks with no dct)
		 *       . . . .
		 */
		if (((ULong)pTime & 0x3) == 0) {
		    t0 = ULongP(pTime)[0];
		    t1 = ULongP(pTime)[1];
		    pTime += srcPicWidth;
		    t2 = ULongP(pTime)[0];
		    t3 = ULongP(pTime)[1];
		    pTime += srcPicWidth;
		    t4 = ULongP(pTime)[0];
		    t5 = ULongP(pTime)[1];
		    pTime += srcPicWidth;
		    t6 = ULongP(pTime)[0];
		    t7 = ULongP(pTime)[1];
		    pTime += srcPicWidth;

		    ULongP(pTo)[0] = t0;
		    ULongP(pTo)[1] = t1;
		    pTo += dstPicWidth;
		    ULongP(pTo)[0] = t2;
		    ULongP(pTo)[1] = t3;
		    pTo += dstPicWidth;
		    ULongP(pTo)[0] = t4;
		    ULongP(pTo)[1] = t5;
		    pTo += dstPicWidth;
		    ULongP(pTo)[0] = t6;
		    ULongP(pTo)[1] = t7;
		    pTo += dstPicWidth;

		    t0 = ULongP(pTime)[0];
		    t1 = ULongP(pTime)[1];
		    pTime += srcPicWidth;
		    t2 = ULongP(pTime)[0];
		    t3 = ULongP(pTime)[1];
		    pTime += srcPicWidth;
		    t4 = ULongP(pTime)[0];
		    t5 = ULongP(pTime)[1];
		    pTime += srcPicWidth;
		    t6 = ULongP(pTime)[0];
		    t7 = ULongP(pTime)[1];

		    ULongP(pTo)[0] = t0;
		    ULongP(pTo)[1] = t1;
		    pTo += dstPicWidth;
		    ULongP(pTo)[0] = t2;
		    ULongP(pTo)[1] = t3;
		    pTo += dstPicWidth;
		    ULongP(pTo)[0] = t4;
		    ULongP(pTo)[1] = t5;
		    pTo += dstPicWidth;
		    ULongP(pTo)[0] = t6;
		    ULongP(pTo)[1] = t7;
		} else {
                    int align = (ULong)pTime & 0x3;
                    int lshift = align << 3;
                    int rshift = 32 - lshift;
                    pTime -= align;
		    for (row = 4; row > 0; row--) {
		        t0 = ULongP(pTime)[0];
		        t1 = ULongP(pTime)[1];
		        t2 = ULongP(pTime)[2];
		        pTime += srcPicWidth;
		        t3 = ULongP(pTime)[0];
		        t4 = ULongP(pTime)[1];
		        t5 = ULongP(pTime)[2];
		        t6 = t0 << lshift;
		        t7 = t1 >> rshift;
		        t8 = t6 | t7;
		        t0 = t1 << lshift;
		        t2 = t2 >> rshift;
		        t0 = t0 | t2;
		        ULongP(pTo)[0] = t8;
		        ULongP(pTo)[1] = t0;
                        pTo += dstPicWidth;
		        
		        t0 = t3 << lshift;
		        t1 = t4 >> rshift;
		        t2 = t0 | t1;
		        t3 = t4 << lshift;
		        t4 = t5 >> rshift;
		        t6 = t3 | t4;
		        ULongP(pTo)[0] = t2;
		        ULongP(pTo)[1] = t6;
                        
                        pTo += dstPicWidth;
                        pTime += srcPicWidth;
                    }
                }
		break;

	    case MotionHalfHoriz:
		/*
		 * time: x x . .
		 *       . . . .
		 */
		for (row = 8; row > 0; row--) {
		    t0 = pTime[0];
		    t1 = pTime[1];
		    t2 = pTime[2];
		    t3 = pTime[3];
		    t4 = pTime[4];
		    t5 = (t0 + t1 + Rnd1) >> 1;
		    t0 = pTime[5];
		    t6 = (t1 + t2 + Rnd1) >> 1;
		    t1 = pTime[6];
		    t7 = (t2 + t3 + Rnd1) >> 1;
		    t2 = pTime[7];
		    t8 = (t3 + t4 + Rnd1) >> 1;
		    t3 = pTime[8];
                    StoreAligned4b(pTo, t5,t6,t7,t8);
		    t5 = (t4 + t0 + Rnd1) >> 1;
		    t6 = (t0 + t1 + Rnd1) >> 1;
		    t7 = (t1 + t2 + Rnd1) >> 1;
		    t8 = (t2 + t3 + Rnd1) >> 1;
                    StoreAligned4b(pTo+4, t5,t6,t7,t8);

		    pTo += dstPicWidth;
		    pTime += srcPicWidth;
		}
		break;

	    case MotionHalfVert:
		/*
		 * time: x . . .
		 *       x . . .
		 */
		pTimeNxtRow = pTime + srcPicWidth;
		for (row = 8; row > 0; row--) {
		    t0 = pTime[0];
		    t1 = pTimeNxtRow[0];
		    t2 = pTime[1];
		    t3 = pTimeNxtRow[1];
		    t4 = pTime[2];
		    t5 = pTimeNxtRow[2];
		    t0 = (t0 + t1 + Rnd1) >> 1;
		    t6 = pTime[3];
		    t7 = pTimeNxtRow[3];
		    t1 = (t2 + t3 + Rnd1) >> 1;
		    t2 = (t4 + t5 + Rnd1) >> 1;
		    t3 = (t6 + t7 + Rnd1) >> 1;
                    StoreAligned4b(pTo, t0,t1,t2,t3);

		    t0 = pTime[4];
		    t1 = pTimeNxtRow[4];
		    t2 = pTime[5];
		    t3 = pTimeNxtRow[5];
		    t4 = pTime[6];
		    t5 = pTimeNxtRow[6];
		    t0 = (t0 + t1 + Rnd1) >> 1;
		    t6 = pTime[7];
		    t7 = pTimeNxtRow[7];
		    t1 = (t2 + t3 + Rnd1) >> 1;
		    t2 = (t4 + t5 + Rnd1) >> 1;
		    t3 = (t6 + t7 + Rnd1) >> 1;
                    StoreAligned4b(pTo+4, t0,t1,t2,t3);

		    pTo += dstPicWidth;
		    pTime = pTimeNxtRow;
		    pTimeNxtRow += srcPicWidth;
		}
		break;

	    case MotionHalfVert | MotionHalfHoriz:
		/*
		 * time: x x . .
		 *       x x . .
		 */
		pTimeNxtRow = pTime + srcPicWidth;
		for (row = 8; row > 0; row--) {
		    t0 = pTime[0];
		    t1 = pTimeNxtRow[0];
		    t2 = pTime[1];
		    t3 = pTimeNxtRow[1];
		    t4 = pTime[2];
		    t5 = pTimeNxtRow[2];
		    t6 = pTime[3];
		    t7 = pTimeNxtRow[3];
		    
		    t8 = t0 + t1;		/* col 0 */
		    t1 = t2 + t3;		/* col 1 */
		    t0 = (t8 + t1 + Rnd2) >> 2;
		    
		    t2 = pTime[4];
		    t3 = pTimeNxtRow[4];
		    
		    t4 = t4 + t5;		/* col 2 */
		    t1 = (t1 + t4 + Rnd2) >> 2;
		    
		    t5 = t6 + t7;		/* col 3 */
		    t6 = (t4 + t5 + Rnd2) >> 2;
		    
		    t7 = t2 + t3;		/* col 4 */
		    t8 = (t5 + t7 + Rnd2) >> 2;

		    t2 = pTime[5];
		    t3 = pTimeNxtRow[5];
                    StoreAligned4b(pTo, t0,t1,t6,t8);
		    
		    t4 = pTime[6];
		    t5 = pTimeNxtRow[6];
		    t6 = pTime[7];
		    t8 = pTimeNxtRow[7];
		    
		    t1 = t2 + t3;		/* col 5 */
		    t0 = (t7 + t1 + Rnd2) >> 2;
		    
		    t2 = pTime[8];
		    t3 = pTimeNxtRow[8];
		    
		    t4 = t4 + t5;		/* col 6 */
		    t1 = (t1 + t4 + Rnd2) >> 2;
		    
		    t5 = t6 + t8;		/* col 7 */
		    t6 = (t4 + t5 + Rnd2) >> 2;
		    
		    t7 = t2 + t3;		/* col 8 */
		    t8 = (t5 + t7 + Rnd2) >> 2;
                    StoreAligned4b(pTo+4, t0,t1,t6,t8);
		    
		    pTo += dstPicWidth;
		    pTime = pTimeNxtRow;
		    pTimeNxtRow += srcPicWidth;
		}
		break;
	    }

            DebugTask(DCTD_MotionMbAddr,
                if (pmc->picWidth[ blkIndex ] != 8
                 && CheckPicXY(priv->current, pTo+7)) {
                    Msg("after MC; present, blk %d: ", blkIndex);
                    ShowPicXY(priv->current, pTo);
                }
                if (CheckPicXY(yci, pTime)) {
                    Msg("after MC; time,    blk %d: ", blkIndex);
                    ShowPicXY(yci, pTime);
                }
            )
            DebugTask(DCTD_DeltaValues,
                if (pmc->picWidth[ blkIndex ] != 8) {
		    pTo = priv->pTo[ picType ]
			+ priv->geom.blkOffset[ blkIndex ];
		    ShowPostMotionValue(pTo, pmc->picWidth[ blkIndex ]);
		}
	    )

        }       
	break;

    case DctMotionFwdBak:
	if (idctMb->flags.b.blocksMask == priv->allBlocksMask) {
	    /*
	     * all blocks have dct.
	     */
	    pmc = priv->pmc = &priv->multiSrcMc;
	} else if (idctMb->flags.b.blocksMask == 0) {
	    /*
	     * The entire macroblock consists of motion without dct.
	     */
	    pmc = priv->pmc = &priv->singleSrcMc;
	    pmc->pSrc[ Dct444Pic ] = priv->pTo[ Dct444Pic ];
	    pmc->pSrc[ Dct420Pic ] = priv->pTo[ Dct420Pic ];
	} else {
	    int blk, blkMask;

	    /*
	     * Some blocks are copied, some have dct deltas.
	     * Here's the truth table:
	     *    dct        do mc    pmc->offset
	     *  present        ?
	     * ----------   ---------------------
	     * 1. no          yes     cur picture
	     * 2.yes          yes     priv->mcPix
	     */
	    pmc = priv->pmc = &priv->mixedSrcMc;
	    pmc->pSrc[ Dct444Pic ] = priv->pTo[ Dct444Pic ];
	    pmc->pSrc[ Dct420Pic ] = priv->pTo[ Dct420Pic ];
	    blk = 0;
	    blkMask = 1;
	    while (blkMask & priv->allBlocksMask) {
		picType = priv->geom.picType[ blk ];
		if (blkMask & idctMb->flags.fastBlocksMask) {
		    /*
		     * #1
		     */
		    pmc->picWidth[ blk ] = 8;
		    pmc->offset[ blk ] = (priv->mcSrc + blk * 64)
				       - pmc->pSrc[ picType ];
		} else {
		    pmc->picWidth[ blk ] = priv->geom.pic[ picType ].width;
		    pmc->offset[ blk ] = priv->geom.blkOffset[ blk ];
		}
		blk++;
		blkMask <<= 1;
	    }
	}

	/*
	 * figure out offsets for backward motion.
	 */
        x = idctMb->motion[ kIndexBakX ];
        y = idctMb->motion[ kIndexBakY ];
        
	AdjustPelMotion(priv->bakFullPel, x, y, cx, cy, allHalfBakFlags);

	offset = priv->futureOffset + x;
	if (y)
	    offset += y * priv->geom.pic[ Dct444Pic ].width;
	bakMcOffsets[ Dct444Pic ] = offset;

	offset = priv->futureOffset + cx;
	if (cy)
	    offset += cy * priv->geom.pic[ Dct420Pic ].width;
	bakMcOffsets[ Dct420Pic ] = offset;

	DebugTask(DCTD_MotionMbAddr,
	    UChar *target;
	    Msg("  raw motion; bak: (%d,%d), fwd: (%d,%d)\n",
		idctMb->motion[ kIndexBakX ],
		idctMb->motion[ kIndexBakY ],
		idctMb->motion[ kIndexFwdX ],
		idctMb->motion[ kIndexFwdY ]);
	    Msg("  present y(%3d,%3d): ",
		(priv->super.nextMbAddr % priv->geom.mbWidth) * 16,
		(priv->super.nextMbAddr / priv->geom.mbWidth) * 16);
	    target = priv->pTo[ Dct444Pic ]
		   + priv->geom.blkOffset[ priv->geom.nBlocks-1 ];
	    ShowPicXY(priv->current, target);
	    Msg("  future y(%3d%c,%3d%c):",
		x, (allHalfBakFlags & MotionHalfYHoriz) ? 'h' : ' ',
		y, (allHalfBakFlags & MotionHalfYVert) ? 'h' : ' ');
	    ShowPicXY(priv->future, target + bakMcOffsets[ Dct444Pic ]);
	    target = priv->pTo[ Dct420Pic ]
		   + priv->geom.blkOffset[ 0 ];
	    Msg("  future c(%3d%c,%3d%c):",
		cx, (allHalfBakFlags & MotionHalfCHoriz) ? 'h' : ' ',
		cy, (allHalfBakFlags & MotionHalfCVert) ? 'h' : ' ');
	    ShowPicXY(priv->future, target + bakMcOffsets[ Dct420Pic ]);
	)

	/*
	 * figure out offsets for foreward motion.
	 */
        x = idctMb->motion[ kIndexFwdX ];
        y = idctMb->motion[ kIndexFwdY ];
        
	AdjustPelMotion(priv->fwdFullPel, x, y, cx, cy, allHalfFwdFlags);

	offset = priv->pastOffset + x;
	if (y)
	    offset += y * priv->geom.pic[ Dct444Pic ].width;
	fwdMcOffsets[ Dct444Pic ] = offset;

	offset = priv->pastOffset + cx;
	if (cy)
	    offset += cy * priv->geom.pic[ Dct420Pic ].width;
	fwdMcOffsets[ Dct420Pic ] = offset;

	DebugTask(DCTD_MotionMbAddr,
	    UChar *target;
	    target = priv->pTo[ Dct444Pic ]
		   + priv->geom.blkOffset[ priv->geom.nBlocks-1 ];
	    Msg("  past y  (%3d%c,%3d%c):",
		x, (allHalfFwdFlags & MotionHalfYHoriz) ? 'h' : ' ',
		y, (allHalfFwdFlags & MotionHalfYVert) ? 'h' : ' ');
	    ShowPicXY(priv->past, target + fwdMcOffsets[ Dct444Pic ]);
	    target = priv->pTo[ Dct420Pic ]
		   + priv->geom.blkOffset[ 0 ];
	    Msg("  past c  (%3d%c,%3d%c):",
		cx, (allHalfFwdFlags & MotionHalfCHoriz) ? 'h' : ' ',
		cy, (allHalfFwdFlags & MotionHalfCVert) ? 'h' : ' ');
	    ShowPicXY(priv->past, target + fwdMcOffsets[ Dct420Pic ]);
	)

        for (blkIndex = priv->geom.nBlocks-1; blkIndex >= 0; blkIndex--) {
            picType = priv->geom.picType[ blkIndex ];
            pTo = priv->pTo[ picType ]
                + priv->geom.blkOffset[ blkIndex ];
            pFut = pTo + bakMcOffsets[ picType ];
            pPast = pTo + fwdMcOffsets[ picType ];

	    dstPicWidth = pmc->picWidth[ blkIndex ];
            srcPicWidth = priv->geom.pic[ picType ].width;

	    pTo = pmc->pSrc[ picType ] + pmc->offset[ blkIndex ];
	    if (picType == Dct444Pic) {
		halfFwdFlags = allHalfFwdFlags & LumaMotionMask;
		halfBakFlags = allHalfBakFlags & LumaMotionMask;
	    } else {
		halfFwdFlags = allHalfFwdFlags >> ChromaShift;
		halfBakFlags = allHalfBakFlags >> ChromaShift;
	    }

	    DebugTask(DCTD_MotionBlkAddr,
                if (pmc->picWidth[ blkIndex ] != 8) {
		    Msg("    blk%2d present:  ", blkIndex);
		    ShowPicXY(priv->current, pTo);
		}
		Msg("    blk%2d past:     ", blkIndex);
		ShowPicXY(priv->past, pPast);
		Msg("    blk%2d future:   ", blkIndex);
		ShowPicXY(priv->future, pFut);
	    )

	    WhenDebugEnabled(timeSwapped = False)
	    switch (halfFwdFlags) {
	    case 0:
		switch (halfBakFlags) {
		case 0:
		    /*
		     * past: x . . .    future: x . . .
		     *       . . . .            . . . .
		     * We take the worst alignment available and work
		     * within its constraints.
		     */
		    t8 = 0x7f7f7f7f;
		    t7 = 0x01010101;
		    if ((((ULong)pFut | (ULong)pPast) & 0x3) == 0) {
			t0 = ULongP(pPast)[0];
			t1 = ULongP(pFut)[0];
			t2 = ULongP(pPast)[1];
			t3 = ULongP(pFut)[1];
			pFut += srcPicWidth;
			pPast += srcPicWidth;
			
			t4 = t0 >> 1;	/* preshift past */
			t5 = t1 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t0 | t1;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[0] = t1;

			t4 = t2 >> 1;	/* preshift past */
			t5 = t3 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t2 | t3;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[1] = t1;
			pTo += dstPicWidth;

			t0 = ULongP(pPast)[0];
			t1 = ULongP(pFut)[0];
			t2 = ULongP(pPast)[1];
			t3 = ULongP(pFut)[1];
			pFut += srcPicWidth;
			pPast += srcPicWidth;
			
			t4 = t0 >> 1;	/* preshift past */
			t5 = t1 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t0 | t1;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[0] = t1;

			t4 = t2 >> 1;	/* preshift past */
			t5 = t3 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t2 | t3;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[1] = t1;
			pTo += dstPicWidth;

			t0 = ULongP(pPast)[0];
			t1 = ULongP(pFut)[0];
			t2 = ULongP(pPast)[1];
			t3 = ULongP(pFut)[1];
			pFut += srcPicWidth;
			pPast += srcPicWidth;
			
			t4 = t0 >> 1;	/* preshift past */
			t5 = t1 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t0 | t1;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[0] = t1;

			t4 = t2 >> 1;	/* preshift past */
			t5 = t3 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t2 | t3;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[1] = t1;
			pTo += dstPicWidth;

			t0 = ULongP(pPast)[0];
			t1 = ULongP(pFut)[0];
			t2 = ULongP(pPast)[1];
			t3 = ULongP(pFut)[1];
			pFut += srcPicWidth;
			pPast += srcPicWidth;
			
			t4 = t0 >> 1;	/* preshift past */
			t5 = t1 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t0 | t1;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[0] = t1;

			t4 = t2 >> 1;	/* preshift past */
			t5 = t3 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t2 | t3;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[1] = t1;
			pTo += dstPicWidth;

			t0 = ULongP(pPast)[0];
			t1 = ULongP(pFut)[0];
			t2 = ULongP(pPast)[1];
			t3 = ULongP(pFut)[1];
			pFut += srcPicWidth;
			pPast += srcPicWidth;
			
			t4 = t0 >> 1;	/* preshift past */
			t5 = t1 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t0 | t1;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[0] = t1;

			t4 = t2 >> 1;	/* preshift past */
			t5 = t3 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t2 | t3;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[1] = t1;
			pTo += dstPicWidth;

			t0 = ULongP(pPast)[0];
			t1 = ULongP(pFut)[0];
			t2 = ULongP(pPast)[1];
			t3 = ULongP(pFut)[1];
			pFut += srcPicWidth;
			pPast += srcPicWidth;
			
			t4 = t0 >> 1;	/* preshift past */
			t5 = t1 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t0 | t1;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[0] = t1;

			t4 = t2 >> 1;	/* preshift past */
			t5 = t3 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t2 | t3;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[1] = t1;
			pTo += dstPicWidth;

			t0 = ULongP(pPast)[0];
			t1 = ULongP(pFut)[0];
			t2 = ULongP(pPast)[1];
			t3 = ULongP(pFut)[1];
			pFut += srcPicWidth;
			pPast += srcPicWidth;
			
			t4 = t0 >> 1;	/* preshift past */
			t5 = t1 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t0 | t1;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[0] = t1;

			t4 = t2 >> 1;	/* preshift past */
			t5 = t3 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t2 | t3;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[1] = t1;
			pTo += dstPicWidth;

			t0 = ULongP(pPast)[0];
			t1 = ULongP(pFut)[0];
			t2 = ULongP(pPast)[1];
			t3 = ULongP(pFut)[1];
			
			t4 = t0 >> 1;	/* preshift past */
			t5 = t1 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t0 | t1;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[0] = t1;

			t4 = t2 >> 1;	/* preshift past */
			t5 = t3 >> 1;	/* preshift future */
			t4 &= t8;	/* mask off top bit */
			t5 &= t8;	/* mask off top bit */
			t0 = t2 | t3;	/* get all the bottom bits */
			t0 &= t7;	/* keep only the 0th bits */
			t1 = t4 + t5;	/* add eight sources */
			t1 = t1 + t0;	/* add in bottom carry bit */
			ULongP(pTo)[1] = t1;
		    } else {
			for (row = 8; row > 0; row--) {
			    t0 = pFut[0];
			    t4 = pPast[0];
			    t1 = pFut[1];
			    t5 = pPast[1];
			    t2 = pFut[2];
			    t6 = pPast[2];
			    t3 = pFut[3];
			    t7 = pPast[3];
			    t8 = t0 + t4;
			    t0 = (t8+Rnd1) >> 1;
			    t8 = t1 + t5;
			    t1 = (t8+Rnd1) >> 1;
			    t8 = t2 + t6;
			    t2 = (t8+Rnd1) >> 1;
			    t8 = t3 + t7;
			    t3 = (t8+Rnd1) >> 1;
			    StoreAligned4b(pTo, t0,t1,t2,t3);
			    
			    t0 = pFut[4];
			    t4 = pPast[4];
			    t1 = pFut[5];
			    t5 = pPast[5];
			    t2 = pFut[6];
			    t6 = pPast[6];
			    t3 = pFut[7];
			    t7 = pPast[7];
			    t8 = t0 + t4;
			    t0 = (t8+Rnd1) >> 1;
			    t8 = t1 + t5;
			    t1 = (t8+Rnd1) >> 1;
			    t8 = t2 + t6;
			    t2 = (t8+Rnd1) >> 1;
			    t8 = t3 + t7;
			    t3 = (t8+Rnd1) >> 1;
			    StoreAligned4b(pTo+4, t0,t1,t2,t3);

			    pTo += dstPicWidth;
			    pFut += srcPicWidth;
			    pPast += srcPicWidth;
			}
		    }
		    break;

		case MotionHalfHoriz:
		pastPlusHalfHorizFuture:
		    /*
		     * past: x . . .    future: x x . .
		     *       . . . .            . . . .
		     */
		    for (row = 8; row > 0; row--) {
			t0 = pFut[0];
			t1 = pFut[1];
			t2 = pPast[0];

			t3 = pFut[2];
			t4 = pPast[1];
			t5 = (((t0 + t1 + Rnd1) >> 1) + t2 + Rnd1) >> 1;

			t0 = pFut[3];
			t2 = pPast[2];
			t6 = (((t1 + t3 + Rnd1) >> 1) + t4 + Rnd1) >> 1;

			t1 = pFut[4];
			t4 = pPast[3];
			t7 = (((t3 + t0 + Rnd1) >> 1) + t2 + Rnd1) >> 1;

			t3 = pFut[5];
			t2 = pPast[4];
			t8 = (((t0 + t1 + Rnd1) >> 1) + t4 + Rnd1) >> 1;
			StoreAligned4b(pTo, t5,t6,t7,t8);

			t0 = pFut[6];
			t4 = pPast[5];
			t5 = (((t1 + t3 + Rnd1) >> 1) + t2 + Rnd1) >> 1;

			t1 = pFut[7];
			t2 = pPast[6];
			t6 = (((t3 + t0 + Rnd1) >> 1) + t4 + Rnd1) >> 1;

			t3 = pFut[8];
			t4 = pPast[7];
			t7 = (((t0 + t1 + Rnd1) >> 1) + t2 + Rnd1) >> 1;
			t8 = (((t1 + t3 + Rnd1) >> 1) + t4 + Rnd1) >> 1;
			StoreAligned4b(pTo+4, t5,t6,t7,t8);

			pTo += dstPicWidth;
			pFut += srcPicWidth;
			pPast += srcPicWidth;
		    }
		    break;

		case MotionHalfVert:
		pastPlusHalfVertFuture:
		    /*
		     * past: x . . .    future: x . . .
		     *       . . . .            x . . .
		     */
		    pFutNxtRow = pFut + srcPicWidth;
		    for (row = 8; row > 0; row--) {
			t0 = pFut[0];
			t1 = pFutNxtRow[0];
			t2 = pPast[0];

			t3 = pFut[1];
			t4 = pFutNxtRow[1];
			t5 = pPast[1];
			t6 = (((t0 + t1 + Rnd1) >> 1) + t2 + Rnd1) >> 1;

			t0 = pFut[2];
			t1 = pFutNxtRow[2];
			t2 = pPast[2];
			t7 = (((t3 + t4 + Rnd1) >> 1) + t5 + Rnd1) >> 1;

			t3 = pFut[3];
			t4 = pFutNxtRow[3];
			t5 = pPast[3];
			t8 = (((t0 + t1 + Rnd1) >> 1) + t2 + Rnd1) >> 1;

			t0 = pFut[4];
			t1 = pFutNxtRow[4];
			t2 = pPast[4];
			t3 = (((t3 + t4 + Rnd1) >> 1) + t5 + Rnd1) >> 1;
			StoreAligned4b(pTo, t6,t7,t8,t3);

			t3 = pFut[5];
			t4 = pFutNxtRow[5];
			t5 = pPast[5];
			t6 = (((t0 + t1 + Rnd1) >> 1) + t2 + Rnd1) >> 1;

			t0 = pFut[6];
			t1 = pFutNxtRow[6];
			t2 = pPast[6];
			t7 = (((t3 + t4 + Rnd1) >> 1) + t5 + Rnd1) >> 1;

			t3 = pFut[7];
			t4 = pFutNxtRow[7];
			t5 = pPast[7];
			t8 = (((t0 + t1 + Rnd1) >> 1) + t2 + Rnd1) >> 1;

			t0 = pFut[8];
			t1 = pFutNxtRow[8];
			t2 = pPast[8];
			t3 = (((t3 + t4 + Rnd1) >> 1) + t5 + Rnd1) >> 1;
			StoreAligned4b(pTo+4, t6,t7,t8,t3);

			pTo += dstPicWidth;
			pFut = pFutNxtRow;
			pFutNxtRow += srcPicWidth;
			pPast += srcPicWidth;
		    }
		    break;

		case MotionHalfVert | MotionHalfHoriz:
		pastPlusFutureHalf:
		    /*
		     * past: x . . .    future: x x . .
		     *       . . . .            x x . .
		     */
		    pFutNxtRow = pFut + srcPicWidth;
		    for (row = 8; row > 0; row--) {
			t0 = pFut[0];
			t1 = pFutNxtRow[0];
			t2 = pFut[1];
			t3 = pFutNxtRow[1];
			t4 = pPast[0];
			t5 = (((t0+t1+t2+t3+Rnd2) >> 2) + t4 + Rnd1) >> 1;
			
			t0 = pFut[2];
			t1 = pFutNxtRow[2];
			t4 = pPast[1];
			t6 = (((t2+t3+t0+t1+Rnd2) >> 2) + t4 + Rnd1) >> 1;

			t2 = pFut[3];
			t3 = pFutNxtRow[3];
			t4 = pPast[2];
			t7 = (((t0+t1+t2+t3+Rnd2) >> 2) + t4 + Rnd1) >> 1;
			
			t0 = pFut[4];
			t1 = pFutNxtRow[4];
			t4 = pPast[3];
			t8 = (((t2+t3+t0+t1+Rnd2) >> 2) + t4 + Rnd1) >> 1;
			StoreAligned4b(pTo, t5,t6,t7,t8);

			t2 = pFut[5];
			t3 = pFutNxtRow[5];
			t4 = pPast[4];
			t5 = (((t0+t1+t2+t3+Rnd2) >> 2) + t4 + Rnd1) >> 1;
			
			t0 = pFut[6];
			t1 = pFutNxtRow[6];
			t4 = pPast[5];
			t6 = (((t2+t3+t0+t1+Rnd2) >> 2) + t4 + Rnd1) >> 1;

			t2 = pFut[7];
			t3 = pFutNxtRow[7];
			t4 = pPast[6];
			t7 = (((t0+t1+t2+t3+Rnd2) >> 2) + t4 + Rnd1) >> 1;
			
			t0 = pFut[8];
			t1 = pFutNxtRow[8];
			t4 = pPast[7];
			t8 = (((t2+t3+t0+t1+Rnd2) >> 2) + t4 + Rnd1) >> 1;
			StoreAligned4b(pTo+4, t5,t6,t7,t8);

			pTo += dstPicWidth;
			pFut = pFutNxtRow;
			pFutNxtRow += srcPicWidth;
			pPast += srcPicWidth;
		    }
		    break;
		}
		break;

	    case MotionHalfHoriz:
		switch (halfBakFlags) {
		case 0:
		    /*
		     * past: x x . .    future: x . . .
		     *       . . . .            . . . .
		     */
		    pTime = pFut;
		    pFut = pPast;
		    pPast = pTime;
		    WhenDebugEnabled(timeSwapped = True)
		    goto pastPlusHalfHorizFuture;

		case MotionHalfHoriz:
		    /*
		     * past: x x . .    future: x x . .
		     *       . . . .            . . . .
		     */
		    if (MotionRounding == RoundMotion) {
			for (row = 8; row > 0; row--) {
			    t0 = pPast[0];
			    t1 = pPast[1];
			    t2 = pFut[0];
			    t3 = pFut[1];
			    t5 = (((t0+t1+Rnd1)>>1)
			       + ((t2+t3+Rnd1)>>1)
			       + Rnd1) >> 1;

			    t0 = pPast[2];
			    t2 = pFut[2];
			    t6 = (((t2+t3+Rnd1)>>1)
			       + ((t0+t1+Rnd1)>>1)
			       + Rnd1) >> 1;

			    t1 = pPast[3];
			    t3 = pFut[3];
			    t7 = (((t0+t1+Rnd1)>>1)
			       + ((t2+t3+Rnd1)>>1)
			       + Rnd1) >> 1;

			    t0 = pPast[4];
			    t2 = pFut[4];
			    t8 = (((t2+t3+Rnd1)>>1)
			       + ((t0+t1+Rnd1)>>1)
			       + Rnd1) >> 1;
			    StoreAligned4b(pTo, t5,t6,t7,t8);

			    t1 = pPast[5];
			    t3 = pFut[5];
			    t5 = (((t0+t1+Rnd1)>>1)
			       + ((t2+t3+Rnd1)>>1)
			       + Rnd1) >> 1;

			    t0 = pPast[6];
			    t2 = pFut[6];
			    t6 = (((t2+t3+Rnd1)>>1)
			       + ((t0+t1+Rnd1)>>1)
			       + Rnd1) >> 1;

			    t1 = pPast[7];
			    t3 = pFut[7];
			    t7 = (((t0+t1+Rnd1)>>1)
			       + ((t2+t3+Rnd1)>>1)
			       + Rnd1) >> 1;

			    t0 = pPast[8];
			    t2 = pFut[8];
			    t8 = (((t2+t3+Rnd1)>>1)
			       + ((t0+t1+Rnd1)>>1)
			       + Rnd1) >> 1;
			    StoreAligned4b(pTo+4, t5,t6,t7,t8);

			    pTo += dstPicWidth;
			    pFut += srcPicWidth;
			    pPast += srcPicWidth;
			}
		    } else {	/* just come close */
			for (row = 8; row > 0; row--) {
			    t0 = pPast[0];
			    t1 = pFut[0];
			    t2 = pPast[1];
			    t3 = pFut[1];
			    t5 = (t0 + t1 + t2 + t3) >> 2;

			    t0 = pPast[2];
			    t1 = pFut[2];
			    t6 = (t2 + t3 + t0 + t1) >> 2;

			    t2 = pPast[3];
			    t3 = pFut[3];
			    t7 = (t0 + t1 + t2 + t3) >> 2;

			    t0 = pPast[4];
			    t1 = pFut[4];
			    t8 = (t2 + t3 + t0 + t1) >> 2;
			    StoreAligned4b(pTo, t5,t6,t7,t8);

			    t2 = pPast[5];
			    t3 = pFut[5];
			    t5 = (t0 + t1 + t2 + t3) >> 2;

			    t0 = pPast[6];
			    t1 = pFut[6];
			    t6 = (t2 + t3 + t0 + t1) >> 2;

			    t2 = pPast[7];
			    t3 = pFut[7];
			    t7 = (t0 + t1 + t2 + t3) >> 2;

			    t0 = pPast[8];
			    t1 = pFut[8];
			    t8 = (t2 + t3 + t0 + t1) >> 2;
			    StoreAligned4b(pTo+4, t5,t6,t7,t8);

			    pTo += dstPicWidth;
			    pFut += srcPicWidth;
			    pPast += srcPicWidth;
			}
		    }
		    break;

		case MotionHalfVert:
		pastHalfHorizPlusfutureHalfVert:
		    /*
		     * past: x x . .    future: x . . .
		     *       . . . .            x . . .
		     */
		    pFutNxtRow = pFut + srcPicWidth;
		    if (MotionRounding == RoundMotion) {
			for (row = 8; row > 0; row--) {
			    t0 = pPast[0];
			    t1 = pPast[1];
			    t2 = pFut[0];
			    t3 = pFutNxtRow[0];
			    t4 = (((t0+t1+Rnd1)>>1)
			       +  ((t2+t3+Rnd1)>>1)
			       +  Rnd1) >> 1;

			    t0 = pPast[2];
			    t2 = pFut[1];
			    t3 = pFutNxtRow[1];
			    t5 = (((t0+t1+Rnd1)>>1)
			       +  ((t2+t3+Rnd1)>>1)
			       +  Rnd1) >> 1;
			    
			    t1 = pPast[3];
			    t2 = pFut[2];
			    t3 = pFutNxtRow[2];
			    t6 = (((t0+t1+Rnd1)>>1)
			       +  ((t2+t3+Rnd1)>>1)
			       +  Rnd1) >> 1;
			    
			    t0 = pPast[4];
			    t2 = pFut[3];
			    t3 = pFutNxtRow[3];
			    t7 = (((t0+t1+Rnd1)>>1)
			       +  ((t2+t3+Rnd1)>>1)
			       +  Rnd1) >> 1;
			    
			    StoreAligned4b(pTo, t4,t5,t6,t7);

			    t1 = pPast[5];
			    t2 = pFut[4];
			    t3 = pFutNxtRow[4];
			    t4 = (((t0+t1+Rnd1)>>1)
			       +  ((t2+t3+Rnd1)>>1)
			       +  Rnd1) >> 1;

			    t0 = pPast[6];
			    t2 = pFut[5];
			    t3 = pFutNxtRow[5];
			    t5 = (((t0+t1+Rnd1)>>1)
			       +  ((t2+t3+Rnd1)>>1)
			       +  Rnd1) >> 1;
			    
			    t1 = pPast[7];
			    t2 = pFut[6];
			    t3 = pFutNxtRow[6];
			    t6 = (((t0+t1+Rnd1)>>1)
			       +  ((t2+t3+Rnd1)>>1)
			       +  Rnd1) >> 1;
			    
			    t0 = pPast[8];
			    t2 = pFut[7];
			    t3 = pFutNxtRow[7];
			    t7 = (((t0+t1+Rnd1)>>1)
			       +  ((t2+t3+Rnd1)>>1)
			       +  Rnd1) >> 1;
			    
			    StoreAligned4b(pTo+4, t4,t5,t6,t7);

			    pTo += dstPicWidth;
			    pFut = pFutNxtRow;
			    pFutNxtRow += srcPicWidth;
			    pPast += srcPicWidth;
			}
		    } else { /* just come close */
			for (row = 8; row > 0; row--) {
			    t0 = pFut[0] + pFutNxtRow[0] + pPast[0];
			    t1 = pPast[1];
			    pTo[ 0 ] = (t0 + t1) >> 2;

			    t0 = t1 + pFut[1] + pFutNxtRow[1];
			    t1 = pPast[2];
			    pTo[ 1 ] = (t0 + t1) >> 2;

			    t0 = t1 + pFut[2] + pFutNxtRow[2];
			    t1 = pPast[3];
			    pTo[ 2 ] = (t0 + t1) >> 2;

			    t0 = t1 + pFut[3] + pFutNxtRow[3];
			    t1 = pPast[4];
			    pTo[ 3 ] = (t0 + t1) >> 2;

			    t0 = t1 + pFut[4] + pFutNxtRow[4];
			    t1 = pPast[5];
			    pTo[ 4 ] = (t0 + t1) >> 2;

			    t0 = t1 + pFut[5] + pFutNxtRow[5];
			    t1 = pPast[6];
			    pTo[ 5 ] = (t0 + t1) >> 2;

			    t0 = t1 + pFut[6] + pFutNxtRow[6];
			    t1 = pPast[7];
			    pTo[ 6 ] = (t0 + t1) >> 2;

			    t0 = t1 + pFut[7] + pFutNxtRow[7];
			    t1 = pPast[8];
			    pTo[ 7 ] = (t0 + t1) >> 2;

			    pTo += dstPicWidth;
			    pFut = pFutNxtRow;
			    pFutNxtRow += srcPicWidth;
			    pPast += srcPicWidth;
			}
		    }
		    break;

		case MotionHalfVert | MotionHalfHoriz:
		pastHalfHorizPlusFutureHalf:
		    /*
		     * past: x x . .    future: x x . .
		     *       . . . .            x x . .
		     */
		    pFutNxtRow = pFut + srcPicWidth;
		    for (row = 8; row > 0; row--) {
			t0 = pFut[0]; t1 = pFutNxtRow[0];
		        t2 = pFut[1]; t3 = pFutNxtRow[1];
			t4 = pPast[0];t5 = pPast[1];
			t6 = t0 + t1;
			t7 = t2 + t3;
			pTo[ 0 ] = (((t6 + t7 + Rnd2) >> 2)
				  + ((t4 + t5 + Rnd1) >> 1)
				  + Rnd1) >> 1;
			
			t0 = pFut[2]; t1 = pFutNxtRow[2];
			t4 = pPast[2];
			t6 = t0 + t1;
			pTo[ 1 ] = (((t6 + t7 + Rnd2) >> 2)
				  + ((t4 + t5 + Rnd1) >> 1)
				  + Rnd1) >> 1;
			
			t2 = pFut[3]; t3 = pFutNxtRow[3];
			t5 = pPast[3];
			t7 = t2 + t3;
			pTo[ 2 ] = (((t6 + t7 + Rnd2) >> 2)
				  + ((t4 + t5 + Rnd1) >> 1)
				  + Rnd1) >> 1;
			
			t0 = pFut[4]; t1 = pFutNxtRow[4];
			t4 = pPast[4];
			t6 = t0 + t1;
			pTo[ 3 ] = (((t6 + t7 + Rnd2) >> 2)
				  + ((t4 + t5 + Rnd1) >> 1)
				  + Rnd1) >> 1;
			
			t2 = pFut[5]; t3 = pFutNxtRow[5];
			t5 = pPast[5];
			t7 = t2 + t3;
			pTo[ 4 ] = (((t6 + t7 + Rnd2) >> 2)
				  + ((t4 + t5 + Rnd1) >> 1)
				  + Rnd1) >> 1;
			
			t0 = pFut[6]; t1 = pFutNxtRow[6];
			t4 = pPast[6];
			t6 = t0 + t1;
			pTo[ 5 ] = (((t6 + t7 + Rnd2) >> 2)
				  + ((t4 + t5 + Rnd1) >> 1)
				  + Rnd1) >> 1;
			
			t2 = pFut[7]; t3 = pFutNxtRow[7];
			t5 = pPast[7];
			t7 = t2 + t3;
			pTo[ 6 ] = (((t6 + t7 + Rnd2) >> 2)
				  + ((t4 + t5 + Rnd1) >> 1)
				  + Rnd1) >> 1;
			
			t0 = pFut[8]; t1 = pFutNxtRow[8];
			t4 = pPast[8];
			t6 = t0 + t1;
			pTo[ 7 ] = (((t6 + t7 + Rnd2) >> 2)
				  + ((t4 + t5 + Rnd1) >> 1)
				  + Rnd1) >> 1;
			
			pTo += dstPicWidth;
			pFut = pFutNxtRow;
			pFutNxtRow += srcPicWidth;
			pPast += srcPicWidth;
		    }
		    break;
		}
		break;

	    case MotionHalfVert:
		switch (halfBakFlags) {
		case 0:
		    /*
		     * past: x . . .    future: x . . .
		     *       x . . .            . . . .
		     */
		    pTime = pFut;
		    pFut = pPast;
		    pPast = pTime;
		    WhenDebugEnabled(timeSwapped = True)
		    goto pastPlusHalfVertFuture;

		case MotionHalfHoriz:
		    /*
		     * past: x . . .    future: x x . .
		     *       x . . .            . . . .
		     */
		    pTime = pFut;
		    pFut = pPast;
		    pPast = pTime;
		    WhenDebugEnabled(timeSwapped = True)
		    goto pastHalfHorizPlusfutureHalfVert;

		case MotionHalfVert:
		    /*
		     * past: x . . .    future: x . . .
		     *       x . . .            x . . .
		     */
		    pFutNxtRow = pFut + srcPicWidth;
		    pPastNxtRow = pPast + srcPicWidth;
		    for (row = 8; row > 0; row--) {
			t0 = pPast[0]; t1 = pPastNxtRow[0];
			t2 = pFut[0];  t3 = pFutNxtRow[0];
			t4 = (((t0+t1+Rnd1)>>1) + ((t2+t3+Rnd1)>>1)
			   +  Rnd1) >> 1;

			t0 = pPast[1]; t1 = pPastNxtRow[1];
			t2 = pFut[1];  t3 = pFutNxtRow[1];
			t5 = (((t0+t1+Rnd1)>>1) + ((t2+t3+Rnd1)>>1)
			   +  Rnd1) >> 1;

			t0 = pPast[2]; t1 = pPastNxtRow[2];
			t2 = pFut[2];  t3 = pFutNxtRow[2];
			t6 = (((t0+t1+Rnd1)>>1) + ((t2+t3+Rnd1)>>1)
			   +  Rnd1) >> 1;

			t0 = pPast[3]; t1 = pPastNxtRow[3];
			t2 = pFut[3];  t3 = pFutNxtRow[3];
			t7 = (((t0+t1+Rnd1)>>1) + ((t2+t3+Rnd1)>>1)
			   +  Rnd1) >> 1;

			StoreAligned4b(pTo, t4,t5,t6,t7);

			t0 = pPast[4]; t1 = pPastNxtRow[4];
			t2 = pFut[4];  t3 = pFutNxtRow[4];
			t4 = (((t0+t1+Rnd1)>>1) + ((t2+t3+Rnd1)>>1)
			   +  Rnd1) >> 1;

			t0 = pPast[5]; t1 = pPastNxtRow[5];
			t2 = pFut[5];  t3 = pFutNxtRow[5];
			t5 = (((t0+t1+Rnd1)>>1) + ((t2+t3+Rnd1)>>1)
			   +  Rnd1) >> 1;

			t0 = pPast[6]; t1 = pPastNxtRow[6];
			t2 = pFut[6];  t3 = pFutNxtRow[6];
			t6 = (((t0+t1+Rnd1)>>1) + ((t2+t3+Rnd1)>>1)
			   +  Rnd1) >> 1;

			t0 = pPast[7]; t1 = pPastNxtRow[7];
			t2 = pFut[7];  t3 = pFutNxtRow[7];
			t7 = (((t0+t1+Rnd1)>>1) + ((t2+t3+Rnd1)>>1)
			   +  Rnd1) >> 1;

			StoreAligned4b(pTo+4, t4,t5,t6,t7);

			pTo += dstPicWidth;
			pFut = pFutNxtRow;
			pFutNxtRow += srcPicWidth;
			pPast = pPastNxtRow;
			pPastNxtRow += srcPicWidth;
		    }
		    break;

		case MotionHalfVert | MotionHalfHoriz:
		pastHalfVertPlusFutureHalf:
		    /*
		     * past: x . . .    future: x x . .
		     *       x . . .            x x . .
		     */
		    pFutNxtRow = pFut + srcPicWidth;
		    pPastNxtRow = pPast + srcPicWidth;
		    for (row = 8; row > 0; row--) {
			t0 = pFut[0];  t1 = pFutNxtRow[0];
			t2 = pFut[1];  t3 = pFutNxtRow[1];
			t4 = pPast[0]; t5 = pPastNxtRow[0];
			t6 = t0 + t1;
			t7 = t2 + t3;
			t8 = t4 + t5;
			pTo[ 0 ] = (((t6 + t7 + Rnd2) >> 2)
				 +  ((t8 + Rnd1) >> 1)
				 +   Rnd1) >> 1;

			t2 = pFut[2];  t3 = pFutNxtRow[2];
			t4 = pPast[1]; t5 = pPastNxtRow[1];
			t6 = t2 + t3;
			t8 = t4 + t5;
			pTo[ 1 ] = (((t6 + t7 + Rnd2) >> 2)
				 +  ((t8 + Rnd1) >> 1)
				 +   Rnd1) >> 1;

			t2 = pFut[3];  t3 = pFutNxtRow[3];
			t4 = pPast[2]; t5 = pPastNxtRow[2];
			t7 = t2 + t3;
			t8 = t4 + t5;
			pTo[ 2 ] = (((t6 + t7 + Rnd2) >> 2)
				 +  ((t8 + Rnd1) >> 1)
				 +   Rnd1) >> 1;

			t2 = pFut[4];  t3 = pFutNxtRow[4];
			t4 = pPast[3]; t5 = pPastNxtRow[3];
			t6 = t2 + t3;
			t8 = t4 + t5;
			pTo[ 3 ] = (((t6 + t7 + Rnd2) >> 2)
				 +  ((t8 + Rnd1) >> 1)
				 +   Rnd1) >> 1;

			t2 = pFut[5];  t3 = pFutNxtRow[5];
			t4 = pPast[4]; t5 = pPastNxtRow[4];
			t7 = t2 + t3;
			t8 = t4 + t5;
			pTo[ 4 ] = (((t6 + t7 + Rnd2) >> 2)
				 +  ((t8 + Rnd1) >> 1)
				 +   Rnd1) >> 1;

			t2 = pFut[6];  t3 = pFutNxtRow[6];
			t4 = pPast[5]; t5 = pPastNxtRow[5];
			t6 = t2 + t3;
			t8 = t4 + t5;
			pTo[ 5 ] = (((t6 + t7 + Rnd2) >> 2)
				 +  ((t8 + Rnd1) >> 1)
				 +   Rnd1) >> 1;

			t2 = pFut[7];  t3 = pFutNxtRow[7];
			t4 = pPast[6]; t5 = pPastNxtRow[6];
			t7 = t2 + t3;
			t8 = t4 + t5;
			pTo[ 6 ] = (((t6 + t7 + Rnd2) >> 2)
				 +  ((t8 + Rnd1) >> 1)
				 +   Rnd1) >> 1;

			t2 = pFut[8];  t3 = pFutNxtRow[8];
			t4 = pPast[7]; t5 = pPastNxtRow[7];
			t6 = t2 + t3;
			t8 = t4 + t5;
			pTo[ 7 ] = (((t6 + t7 + Rnd2) >> 2)
				 +  ((t8 + Rnd1) >> 1)
				 +   Rnd1) >> 1;

			pTo += dstPicWidth;
			pFut = pFutNxtRow;
			pFutNxtRow += srcPicWidth;
			pPast = pPastNxtRow;
			pPastNxtRow += srcPicWidth;
		    }
		    break;
		}
		break;

	    case MotionHalfVert | MotionHalfHoriz:
		switch (halfBakFlags) {
		case 0:
		    /*
		     * past: x x . .    future: x . . .
		     *       x x . .            . . . .
		     */
		    pTime = pFut;
		    pFut = pPast;
		    pPast = pTime;
		    WhenDebugEnabled(timeSwapped = True)
		    goto pastPlusFutureHalf;

		case MotionHalfHoriz:
		    /*
		     * past: x x . .    future: x x . .
		     *       x x . .            . . . .
		     */
		    pTime = pFut;
		    pFut = pPast;
		    pPast = pTime;
		    WhenDebugEnabled(timeSwapped = True)
		    goto pastHalfHorizPlusFutureHalf;

		case MotionHalfVert:
		    /*
		     * past: x x . .    future: x . . .
		     *       x x . .            x . . .
		     */
		    pTime = pFut;
		    pFut = pPast;
		    pPast = pTime;
		    WhenDebugEnabled(timeSwapped = True)
		    goto pastHalfVertPlusFutureHalf;
		    break;

		case MotionHalfVert | MotionHalfHoriz:
		    /*
		     * past: x x . .    future: x x . .
		     *       x x . .            x x . .
		     */
		    pFutNxtRow = pFut + srcPicWidth;
		    pPastNxtRow = pPast + srcPicWidth;
		    if (MotionRounding == RoundMotion) {
			for (row = 8; row > 0; row--) {
			    t0 = pFut[0];  t1 = pFutNxtRow[0];
			    t2 = pPast[0]; t3 = pPastNxtRow[0]; t0 = t0 + t1;
			    t4 = pFut[1];  t5 = pFutNxtRow[1];  t2 = t2 + t3;
			    t6 = pPast[1]; t7 = pPastNxtRow[1]; t4 = t4 + t5;
								t6 = t6 + t7;
			    t1 = (t0 + t4 + Rnd2) >> 2;
			    t5 = (t2 + t6 + Rnd2) >> 2;
			    pTo[ 0 ] = (t1 + t5 + Rnd1) >> 1;

			    t0 = pFut[2];  t1 = pFutNxtRow[2];
			    t2 = pPast[2]; t3 = pPastNxtRow[2]; t0 = t0 + t1;
								t2 = t2 + t3;
			    t1 = (t0 + t4 + Rnd2) >> 2;
			    t5 = (t2 + t6 + Rnd2) >> 2;
			    pTo[ 1 ] = (t1 + t5 + Rnd1) >> 1;

			    t4 = pFut[3];  t5 = pFutNxtRow[3];
			    t6 = pPast[3]; t7 = pPastNxtRow[3]; t4 = t4 + t5;
								t6 = t6 + t7;
			    t1 = (t0 + t4 + Rnd2) >> 2;
			    t5 = (t2 + t6 + Rnd2) >> 2;
			    pTo[ 2 ] = (t1 + t5 + Rnd1) >> 1;

			    t0 = pFut[4];  t1 = pFutNxtRow[4];
			    t2 = pPast[4]; t3 = pPastNxtRow[4]; t0 = t0 + t1;
								t2 = t2 + t3;
			    t1 = (t0 + t4 + Rnd2) >> 2;
			    t5 = (t2 + t6 + Rnd2) >> 2;
			    pTo[ 3 ] = (t1 + t5 + Rnd1) >> 1;

			    t4 = pFut[5];  t5 = pFutNxtRow[5];
			    t6 = pPast[5]; t7 = pPastNxtRow[5]; t4 = t4 + t5;
								t6 = t6 + t7;
			    t1 = (t0 + t4 + Rnd2) >> 2;
			    t5 = (t2 + t6 + Rnd2) >> 2;
			    pTo[ 4 ] = (t1 + t5 + Rnd1) >> 1;

			    t0 = pFut[6];  t1 = pFutNxtRow[6];
			    t2 = pPast[6]; t3 = pPastNxtRow[6]; t0 = t0 + t1;
								t2 = t2 + t3;
			    t1 = (t0 + t4 + Rnd2) >> 2;
			    t5 = (t2 + t6 + Rnd2) >> 2;
			    pTo[ 5 ] = (t1 + t5 + Rnd1) >> 1;

			    t4 = pFut[7];  t5 = pFutNxtRow[7];
			    t6 = pPast[7]; t7 = pPastNxtRow[7]; t4 = t4 + t5;
								t6 = t6 + t7;
			    t1 = (t0 + t4 + Rnd2) >> 2;
			    t5 = (t2 + t6 + Rnd2) >> 2;
			    pTo[ 6 ] = (t1 + t5 + Rnd1) >> 1;

			    t0 = pFut[8];  t1 = pFutNxtRow[8];
			    t2 = pPast[8]; t3 = pPastNxtRow[8]; t0 = t0 + t1;
								t2 = t2 + t3;
			    t1 = (t0 + t4 + Rnd2) >> 2;
			    t5 = (t2 + t6 + Rnd2) >> 2;
			    pTo[ 7 ] = (t1 + t5 + Rnd1) >> 1;

			    pTo         += dstPicWidth;
			    pFut         = pFutNxtRow;
			    pFutNxtRow  += srcPicWidth;
			    pPast        = pPastNxtRow;
			    pPastNxtRow += srcPicWidth;
			}
		    } else { /* just come close */
			for (row = 8; row > 0; row--) {
			    t0 = pFut[0];  t1 = pFutNxtRow[0];
			    t2 = pPast[0]; t3 = pPastNxtRow[0]; t4 = t0 + t1;
			    t0 = pFut[1];  t1 = pFutNxtRow[1];  t4 += t2 + t3;
			    t2 = pPast[1]; t3 = pPastNxtRow[1]; t5 = t0 + t1;
								t5 += t2 + t3;
			    pTo[ 0 ] = (t4 + t5) >> 3;

			    t0 = pFut[2];  t1 = pFutNxtRow[2];
			    t2 = pPast[2]; t3 = pPastNxtRow[2]; t4 = t0 + t1;
								t4 += t2 + t3;
			    pTo[ 1 ] = (t5 + t4) >> 3;

			    t0 = pFut[3];  t1 = pFutNxtRow[3];
			    t2 = pPast[3]; t3 = pPastNxtRow[3]; t5 = t0 + t1;
								t5 += t2 + t3;
			    pTo[ 2 ] = (t4 + t5) >> 3;

			    t0 = pFut[4];  t1 = pFutNxtRow[4];
			    t2 = pPast[4]; t3 = pPastNxtRow[4]; t4 = t0 + t1;
								t4 += t2 + t3;
			    pTo[ 3 ] = (t5 + t4) >> 3;

			    t0 = pFut[5];  t1 = pFutNxtRow[5];
			    t2 = pPast[5]; t3 = pPastNxtRow[5]; t5 = t0 + t1;
								t5 += t2 + t3;
			    pTo[ 4 ] = (t4 + t5) >> 3;

			    t0 = pFut[6];  t1 = pFutNxtRow[6];
			    t2 = pPast[6]; t3 = pPastNxtRow[6]; t4 = t0 + t1;
								t4 += t2 + t3;
			    pTo[ 5 ] = (t5 + t4) >> 3;

			    t0 = pFut[7];  t1 = pFutNxtRow[7];
			    t2 = pPast[7]; t3 = pPastNxtRow[7]; t5 = t0 + t1;
								t5 += t2 + t3;
			    pTo[ 6 ] = (t4 + t5) >> 3;

			    t0 = pFut[8];  t1 = pFutNxtRow[8];
			    t2 = pPast[8]; t3 = pPastNxtRow[8]; t4 = t0 + t1;
								t4 += t2 + t3;
			    pTo[ 7 ] = (t5 + t4) >> 3;

			    pTo         += dstPicWidth;
			    pFut         = pFutNxtRow;
			    pFutNxtRow  += srcPicWidth;
			    pPast        = pPastNxtRow;
			    pPastNxtRow += srcPicWidth;
			}
		    }
		    break;
		}
		break;
	    }

            DebugTask(DCTD_MotionMbAddr,
                if (pmc->picWidth[ blkIndex ] != 8
		 && CheckPicXY(priv->current, pTo+7)) {
                    Msg("after MC; present, blk %d: ", blkIndex);
                    ShowPicXY(priv->current, pTo);
                }
		pTime = timeSwapped ? pPast : pFut;
                if (CheckPicXY(priv->future, pTime+7)) {
                    Msg("after MC; future, blk %d: ", blkIndex);
                    ShowPicXY(priv->future, pTime);
                }
		pTime = timeSwapped ? pFut : pPast;
                if (CheckPicXY(priv->past, pTime+7)) {
                    Msg("after MC; past,    blk %d: ", blkIndex);
                    ShowPicXY(priv->past, pTime);
                }
            )
            DebugTask(DCTD_DeltaValues,
                if (pmc->picWidth[ blkIndex ] != 8) {
		    pTo = priv->pTo[ picType ]
			+ priv->geom.blkOffset[ blkIndex ];
		    ShowPostMotionValue(pTo, pmc->picWidth[ blkIndex ]);
		}
	    )

        }

	break;
    }
}

