/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	intLeeIdct.c - fixed point implementation of Lee's idct
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/intLeeIdct.c,v 1.1 1995/08/01 22:21:54 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "integerIdctI.h"
#include "integerIdctConst.h"

#define DebugP(args)	/* nothing */

#define SaveColumnCoeff(c0, c1, c2, c3, c4, c5, c6, c7, index)	\
    pMb->dctRecon[index+0*8] = c0;				\
    pMb->dctRecon[index+1*8] = c1;				\
    pMb->dctRecon[index+2*8] = c2;				\
    pMb->dctRecon[index+3*8] = c3;				\
    pMb->dctRecon[index+4*8] = c4;				\
    pMb->dctRecon[index+5*8] = c5;				\
    pMb->dctRecon[index+6*8] = c6;				\
    pMb->dctRecon[index+7*8] = c7


static void
IDctOneRow(MacroBlockInfo *pMb,
	   int index,
	   int columns)
{
    long *ptr = &pMb->dctRecon[8*index];
    short *result;
    UChar *pixels;
    long c0, c1, c2, c3, c4, c5, c6, c7;
    _IDctDeclarations;
    
    switch (columns) {
    case 0x1:
	c0 = ptr[ 0 ];
	_IDctCode(c0, 0,0,0,0,0,0,0);
	break;

    case 0x2:
	c1 = ptr[ 1 ];
	_IDctCode(0,c1,0,0,0,0,0,0);
	break;

    case 0x3:
	c0 = ptr[ 0 ];
	c1 = ptr[ 1 ];
	_IDctCode(c0,c1,0,0,0,0,0,0);
	break;

    case 0x4: case 0x5: case 0x6: case 0x7:
	c2 = ptr[ 2 ];
	c3 = 0;
	goto fourCoeff;

    case 0x8: case 0x9: case 0xa: case 0xb:
	c2 = 0;
	c3 = ptr[ 3 ];
	goto fourCoeff;

    case 0xc: case 0xd: case 0xe: case 0xf:
	c2 = ptr[ 2 ];
	c3 = ptr[ 3 ];

    fourCoeff:
	c0 = (columns & (1<<0)) ? ptr[ 0 ] : 0;
	c1 = (columns & (1<<1)) ? ptr[ 1 ] : 0;
	_IDctCode(c0,c1,c2,c3,0,0,0,0);
	break;

    default:
	c0 = (columns & (1<<0)) ? ptr[ 0 ] : 0;
	c1 = (columns & (1<<1)) ? ptr[ 1 ] : 0;
	c2 = (columns & (1<<2)) ? ptr[ 2 ] : 0;
	c3 = (columns & (1<<3)) ? ptr[ 3 ] : 0;
	c4 = (columns & (1<<4)) ? ptr[ 4 ] : 0;
	c5 = (columns & (1<<5)) ? ptr[ 5 ] : 0;
	c6 = (columns & (1<<6)) ? ptr[ 6 ] : 0;
	c7 = (columns & (1<<7)) ? ptr[ 7 ] : 0;
	_IDctCode(c0,c1,c2,c3,c4,c5,c6,c7);
	break;
    }
    result = (short *)ptr;
    if (pMb->type & (MBT_MBAK | MBT_MFWD)) {
        c0 = p0 + p1; result[0] = PreNonIntraClamp(c0);
        c0 = p2 + p3; result[1] = PreNonIntraClamp(c0);
        c0 = p4 + p5; result[2] = PreNonIntraClamp(c0);
        c0 = p6 + p7; result[3] = PreNonIntraClamp(c0);
        c0 = p6 - p7; result[4] = PreNonIntraClamp(c0);
        c0 = p4 - p5; result[5] = PreNonIntraClamp(c0);
        c0 = p2 - p3; result[6] = PreNonIntraClamp(c0);
        c0 = p0 - p1; result[7] = PreNonIntraClamp(c0);
	(*pMb->motionAndClamp)(pMb, index);
    } else {
        pixels = pMb->pTo + index*MAX_X;
        c0 = p0 + p1; IntraClamp(pixels[0], c0);
        c0 = p2 + p3; IntraClamp(pixels[1], c0);
        c0 = p4 + p5; IntraClamp(pixels[2], c0);
        c0 = p6 + p7; IntraClamp(pixels[3], c0);
        c0 = p6 - p7; IntraClamp(pixels[4], c0);
        c0 = p4 - p5; IntraClamp(pixels[5], c0);
        c0 = p2 - p3; IntraClamp(pixels[6], c0);
        c0 = p0 - p1; IntraClamp(pixels[7], c0);
    }
}

static void IDctOneCol(MacroBlockInfo *pMb, int index)
{
    long *ptr = &pMb->dctCoeff[index];
    long c0, c1, c2, c3, c4, c5, c6, c7;
    _IDctDeclarations;

    c0 = ptr[0*8];
    c1 = ptr[1*8];
    c2 = ptr[2*8];
    c3 = ptr[3*8];
    c4 = ptr[4*8];
    c5 = ptr[5*8];
    c6 = ptr[6*8];
    c7 = ptr[7*8];

    if (c1+c2+c3+c4+c5+c6+c7 == 0) {
	_IDctCode(c0, 0,0,0,0,0,0,0);
    } else if (c0+c2+c3+c4+c5+c6+c7 == 0) {
	_IDctCode(0,c1,0,0,0,0,0,0);
    } else if (c0+c1+c3+c4+c5+c6+c7 == 0) {
	_IDctCode(0,0,c2,0,0,0,0,0);
    } else if (c2+c3+c4+c5+c6+c7 == 0) {
	_IDctCode(c0, c1, 0,0,0,0,0,0);
    } else if (c3+c4+c5+c6+c7 == 0) {
	_IDctCode(c0, c1, c2, 0,0,0,0,0);
    } else {
	_IDctCode(c0, c1, c2, c3, c4, c5, c6, c7);
    }
    SaveColumnCoeff(p0 + p1, p2 + p3, p4 + p5, p6 + p7,
		    p6 - p7, p4 - p5, p2 - p3, p0 - p1, index);
}

static void
IntegerIdct(IdctParamPtr param, int blockIndex)
{
    ULong cells1, cells2;

    /*
     * First create a mask corresponding to the non-zero coefficients.
     * This shouldn't take long, cause there probably are not very many.
     */
    {
	DctCoeffPtr pCoeff, lastCoeff;
	int index;

	pCoeff = param->coeffList;
	cells1 = cells2 = index = 0;

	switch (param->nCoeff) {
	default:
	    lastCoeff = pCoeff + param->nCoeff - 7;
	    while (pCoeff < lastCoeff) {
		index += (pCoeff++)->preIncrement;
		MarkCellNZ(cells1, cells2, index);
	    }
	    /* fall through and finish up... */
	    
	case 7:
	    index += (pCoeff++)->preIncrement;
	    MarkCellNZ(cells1, cells2, index);
	    /* fall through */
	case 6:
	    index += (pCoeff++)->preIncrement;
	    MarkCellNZ(cells1, cells2, index);
	    /* fall through */
	case 5:
	    index += (pCoeff++)->preIncrement;
	    MarkCellNZ(cells1, cells2, index);
	    /* fall through */
	case 4:
	    index += (pCoeff++)->preIncrement;
	    MarkCellNZ(cells1, cells2, index);
	    /* fall through */
	case 3:
	    index += (pCoeff++)->preIncrement;
	    MarkCellNZ(cells1, cells2, index);
	    /* fall through */
	case 2:
	    index += (pCoeff++)->preIncrement;
	    MarkCellNZ(cells1, cells2, index);
	    /* fall through */
	case 1:
	    index += (pCoeff++)->preIncrement;
	    MarkCellNZ(cells1, cells2, index);
	    /* fall through */
	case 0:	/* doesn't happen */
	    break;
	}
    }
    

--------------------------
    UChar motionAndClampIndex;
    ULong cells1, cells2;
    ULong columns;

    motionAndClampIndex = MotionAndClampToIndex(halfPixel,pFuture,pPast);

    DebugTask(N_CoeffValues, PrintMb(1<<blockIndex, cells1, cells2));

    /*
     * Very special case.  Catch the 8x8 matrix with only the
     * dc constant.
     */
    if (cells1 == 0 && cells2 == 1) {
	long dc;

	/*
	 * this repeats any inaccuracy in the calculations normally
	 * done in the idct routines.  Otherwise, a more "correct"
	 * calculation done here sticks out like a sore thumb on the screen
	 */
	dc = *pMb->dctCoeff;
	HalfSqrt2_2_Times(dc, dc, );
	HalfSqrt2_2_Times(dc, dc, );
	/*
	 * Finally, clamp it and do motion compensation.
	 */
	motionAndClamp = MotionAndClamp0DcTable[ motionAndClampIndex ];
	(*motionAndClamp)(pMb, dc);
	return;
    }

    /*
     * do the regular IDCT
     */
    pMb->motionAndClamp = MotionAndClampTable[ motionAndClampIndex ];

    /*
     * Find out the occupancy of each column...
     */
    columns = cells1 | cells2;
    columns |= columns>>16;
    columns |= columns>>8;
    columns &= 0xff;

    if (columns & 0x01) IDctOneCol(pMb, 0);
    if (columns & 0x02) IDctOneCol(pMb, 1);
    if (columns & 0x04) IDctOneCol(pMb, 2);
    if (columns & 0x08) IDctOneCol(pMb, 3);
    if (columns & 0x10) IDctOneCol(pMb, 4);
    if (columns & 0x20) IDctOneCol(pMb, 5);
    if (columns & 0x40) IDctOneCol(pMb, 6);
    if (columns & 0x80) IDctOneCol(pMb, 7);
	
    IDctOneRow(pMb, 0, columns);
    IDctOneRow(pMb, 1, columns);
    IDctOneRow(pMb, 2, columns);
    IDctOneRow(pMb, 3, columns);
    IDctOneRow(pMb, 4, columns);
    IDctOneRow(pMb, 5, columns);
    IDctOneRow(pMb, 6, columns);
    IDctOneRow(pMb, 7, columns);
}

typedef struct _integerMethodSet {
    IntMethod dcMotion;
    IntMethod motion;
    IntMethod copy;
} IntegerMethodSet, *IntegerMethodSetPtr;

static IntegerMethodSet MethodSet[] = {
 /*0000*/ { DcNoMotionAndClamp,     NoMotionAndClamp,     NULL },
 /*0001*/ { NULL,                   NULL,                 NULL },
 /*0010*/ { NULL,                   NULL,                 NULL },
 /*0011*/ { NULL,                   NULL,                 NULL },

 /*0100*/ { DcMotion,               Motion,               Copy },
 /*0101*/ { DcMotionHalfHoriz,      MotionHalfHoriz,      CopyHalfHoriz },
 /*0110*/ { DcMotionHalfVert,       MotionHalfVert,       CopyHalfVert },
 /*0111*/ { DcMotionHalf,           MotionHalf,           CopyHalf },

 /*1000*/ { DcMotion,               Motion,               Copy },
 /*1001*/ { DcMotionHalfHoriz,      MotionHalfHoriz,      CopyHalfHoriz },
 /*1010*/ { DcMotionHalfVert,       MotionHalfVert,       CopyHalfVert },
 /*1011*/ { DcMotionHalf,           MotionHalf,           CopyHalf },

 /*1100*/ { DcMotion,               Motion,               CopyBlock },
 /*1101*/ { DcMotionFwdBakHalfHoriz,MotionFwdBakHalfHoriz,CopyFwdBakHalfHoriz },
 /*1110*/ { DcMotionFwdBakHalfVert, MotionFwdBakHalfVert, CopyFwdBakHalfVert },
 /*1111*/ { DcMotionFwdBakHalf,     MotionFwdBakHalf,     CopyFwdBakHalf }
};

static InternalReasonPtr
IntegerIdctConfig(IdctParamPtr param, ULong configBits)
{
    ULong motionIndex;
    IntIdctParamPtr iparam = (IntIdctParamPtr)param;
    IntegerMethodSetPtr methodSet;
    ULong mask;

    mask = IC_Forward | IC_Backward | IC_HalfHoriz | IC_HalfVert;
    iparam->caller.configBits = motionIndex = configBits & mask;
    if (motionIndex != (iparam->caller.configBits & mask)) {
	methodSet = MethodSet[ motionIndex ];
	iparam->motion = methodSet->motion;
	iparam->motionDc = methodSet->dcMotion;
	iparam->copyBlock = methodSet->copy;
    }

    if (configBits & (IC_newQuant | IC_newMatrix)) {
	int i, coeff, value, qScale = iparam->caller.quantScale;
	short *lookup = iparam->qLookup;
	UChar *quant = iparam->caller.quant;

	/*
	 * calculate a lookup table for the first NPreScaled
	 * unsigned coefficient values.
	 */
	for (i = 0; i < 64; i++) {
	    for (coeff = 1; coeff < NPreScaled; coeff++) {
		value = coeff * quant[ i ] * qScale;
		table[ PreScaleIdx(i, 0, coeff) ] = value;
		table[ PreScaleIdx(i, 1, coeff) ] = -value;
	    }
	}
    }

    return (NULL);
}

static IdctParamPtr
CreateIntegerDct(InternalReasonPtr *pReason)
{
    IdctParamPtr param;
    int size = sizeof(IntIdctParam);

    param = (IntIdctParamPtr) InsecureAlloc(size);
    if (param == NULL)
	pReason = AllocReason("integer dct parameters", size);
    memset((char *)param, 0, size);

    param->inverseDct = IntegerIdct;
    param->configDct = IntegerIdctConfig;
}

static IdctMethod IntegerDctMethodData = { CreateIntegerDct };
IdctMethodPtr IntegerDctMethod = &IntegerDctMethodData;
