/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	intLeeIdctI.h - internal definitions for the fixed point
 *                      implementation of Lee's inverse dct operations
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/intLeeIdctI.h,v 1.1 1995/08/01 22:21:54 toddb Exp $
 */
#ifndef INTLEEIDCTDCTI_H
#define INTLEEIDCTDCTI_H

#include "nvrIdctI.h"

#define NMotionAndClamp	64
#define MotionAndClampToIndex(halfPixel, pFuture, pPast)	\
    (halfPixel | ((pFuture != 0) << 4) | ((pPast != NULL) << 5))

/*
 * For intra-coded blocks, there is no future or past.
 * Hence, the final three arguments are ignored.
 */

#define IntraClampOp(storage, assignOp, value)			\
    {								\
	if ((value = RoundToInteger(value)) & ~255) {		\
	    if (value > 255)					\
		value = 255;					\
	    else						\
		value = 0;					\
	}							\
	storage assignOp value;					\
    }

#define IntraClamp(storage, value)	IntraClampOp(storage, =, value)

/*
 * For non intra-coded blocks, the (signed) difference is added
 * to the reference pel (forward, back, or both) to arrive at a
 * final value that is stored
 */
#define NonIntraClampOp(storage, assignOp, reference, value)	\
    {								\
	value = RoundToInteger(value);				\
	if ((value += (reference)) & ~255) {			\
	    if (value > 255)					\
		value = 255;					\
	    else						\
		value = 0;					\
	}							\
	storage assignOp value;					\
    }

#define NonIntraClamp(storage, reference, value)		\
   NonIntraClampOp(storage, =, reference, value)

/*
 * Round and clamp to +-255 for use with FastNonIntraClamp
 */
#define PreNonIntraClamp(value)					\
    ((RoundToInteger(value) & ~255)				\
      ? (RoundToInteger(value) > 255				\
	  ? 255 :						\
	    (RoundToInteger(value) < -256			\
	      ? -256						\
	      : RoundToInteger(value)				\
	    )							\
	)							\
      : RoundToInteger(value)					\
    )

/*
 * This is for when we know the value is already rounded and
 * clamped.
 */
#define FastNonIntraClamp(storage, assignOp, reference, value)		\
	storage assignOp SignedClamp[ value + (reference) ];


/*
 * This little ditty perform the inverse dct using integer arithmetic.
 * Adapted from code written by Bob Eifrig, TekLabs.
 *
 * For reference
 * see B. G. Lee, IEEE Transactions on Acoustics, Speech, and Signal
 * Processing, Vol 32, No 6, (Dec 1984) pgs 1243-1245.
 *
 * This declares the needed local variables, does the basic 1d dct
 * and leaves (almost) final values in p0-p7;
 */
#define _IDctDeclarations						\
    int p0,p1,p2,p3,p4,p5,p6,p7;					\
    int q0,q1,q2,q3,q4,q5,q6,q7;					\
    int tmp1, tmp2

#define _IDctBody(in0, in1, in2, in3, in4, in5, in6, in7)		\
    _IDctDeclarations;							\
    _IDctCode(in0, in1, in2, in3, in4, in5, in6, in7)

#define _IDctCode(in0, in1, in2, in3, in4, in5, in6, in7)		\
    q0 = in0;								\
    HalfSqrt2_2_Times(p0, q0, tmp1);					\
    q1 = in1;								\
    p1 = q1 >> 1;			/* times .5 */			\
    q2 = in2;								\
    p2 = q2 >> 1;			/* times .5 */			\
    q3 = in3;								\
    p3 = (q0 = q1 + q3) >> 1;	/* times .5 */				\
    q4 = in4;								\
			 HalfSqrt2_2_Times(p4, q4, tmp1);		\
    q5 = in5;								\
    tmp1 = q5 + q3;      HalfSqrt2_2_Times(p5, tmp1, tmp2);		\
    q6 = in6;								\
    tmp1 = q2 + q6;      HalfSqrt2_2_Times(p6, tmp1, tmp2);		\
    q7 = in7;								\
    tmp1 = q0 + q5 + q7; HalfSqrt2_2_Times(p7, tmp1, tmp2);		\
									\
    q0 = p0 + p4;							\
    q1 = p1 + p5;							\
    tmp1 = p2 + p6; HalfCos2Pi_16_Times(q2, tmp1, tmp2);		\
    tmp1 = p3 + p7; HalfCos2Pi_16_Times(q3, tmp1, tmp2);		\
    q4 = p0 - p4;							\
    q5 = p1 - p5;							\
    tmp1 = p2 - p6; HalfCos6Pi_16_Times(q6, tmp1, tmp2);		\
    tmp1 = p3 - p7; HalfCos6Pi_16_Times(q7, tmp1, tmp2);		\
									\
    p0 = q0 + q2;							\
    tmp1 = q1 + q3; HalfCosPi_16_Times(p1, tmp1, tmp2);			\
    p6 = q0 - q2;							\
    tmp1 = q1 - q3; HalfCos7Pi_16_Times(p7, tmp1, tmp2);		\
    p2 = q4 + q6;							\
    tmp1 = q5 + q7; HalfCos3Pi_16_Times(p3, tmp1, tmp2);		\
    p4 = q4 - q6;							\
    tmp1 = q5 - q7; HalfCos5Pi_16_Times(p5, tmp1, tmp2)

/*
 * These defines are consistent with _IDctBody and provide the
 * final values: X0, ... X7
 */
#define DctX0	(p0 + p1)
#define DctX1	(p2 + p3)
#define DctX2	(p4 + p5)
#define DctX3	(p6 + p7)
#define DctX4	(p6 - p7)
#define DctX5	(p4 - p5)
#define DctX6	(p2 - p3)
#define DctX7	(p0 - p1)

/*
 * The pre-scaled lookup table is organized as follows:
 * If PreScaledBits is 2, then we will create a lookup table that
 * will accelerate inverse quantization for the values -3, -2, -1, 1, 2, 3.
 * Note that 0 never appears (because it is handled elsewhere) and
 * that two of eight slots are wasted.
 *
 * The indices to the table (assuming PreScaledBits==2) are the bits
 *
 *    iiiiiiscc
 *
 * Where 's' is the sign bit and 'cc' are the two bits from the quantized
 * coefficient.  The 'iiiiii' are the six bits from the index into
 * the 8x8 table.
 */
#define PreScaledBits   2       /* i.e. we pre-scale coefficients 1-3 */
#define NPreScaled      ((1<<PreScaledBits) - 1)

#define CanLookupCoeff(coeff)	(((coeff) >> PreScaledBits) == 0)

#define PreScaleIdx(i, s, c) 				\
    (((i) << (PreScaledBits+1)) | ((s) << PreScaledBits) | (c))

/*
 * Snag the initial coefficient.  Note that this assumes pCoeff
 * and pLookup points to a column and that move down one row each call.
 * Note that coeff is modified!!
 */
#define GetNextCoeffMacro(coeff, pLookup, pCoeff, row, reconstruct)	\
    {									\
	DctCoeff tmp;							\
	int level, index						\
									\
	tmp.composite = pCoeff[ row * 8 ];				\
	level = tmp.b.level;						\
	if ( CanLookupCoeff(level) ) {					\
	    coeff = pLookup[ PreScaleIdx(row*8, tmp.bsign, level) ];	\
	    if (coeff == 0)
		coeff = reconstruct(
	else {
	    coeff = level 
	}
	pLookup += PreScaleIdx(8, 0, 0);			\
    }

/*************************************************************************
 *
 * private version of IdctParam
 *
 *************************************************************************/

typedef struct _intIdctParam IntIdctParam, *IntIdctParamPtr;
typedef void (*IntMethod)(IntIdctParamPtr, int blockIndex);
struct _intIdctParam {
    IdctParam   caller;         /* caller supplied parameters */
    UChar       dctLookup[ 64 ];/* index location of coeff in caller.dctzz */
    UShort      midCeoff [ 64 ];/* the 8x8 block of intermediate results */
    short       qLookup  [ 64 * NPreScaled ]; /* lookup for pre-scaled quant */

    IntMethod	motion;
    IntMethod	motionDc;
}; /* IntIdctParam, *IntIdctParamPtr */

#endif /* INTLEEIDCTDCTI_H */
