/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *      refIdct.c - reference inverse dct.
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/refIdct.c,v 1.1 1995/08/01 22:21:54 toddb Exp $
 */

#include "nvrIdctI.h"
#include "nvrAlloc.h"
#include "nvrString.h"

static float RefIDctCoeffTable[64];

static void
InitDctReference()
{
    int ij, uv;
    float sqrtOf2Reciprocal;

    sqrtOf2Reciprocal = 1.0/sqrt(2.0);
    for (ij = 0; ij < 8; ij++) {
	for (uv = 0; uv < 8; uv++) {
	    RefIDctCoeffTable[ij*8+uv] = cos(M_PI*(2*ij+1)*uv/16.0) / 2.0;
	}
	RefIDctCoeffTable[ij*8+0] *= sqrtOf2Reciprocal;
    }
}

#define IDctCoeff(ij, uv)	RefIDctCoeffTable[ij*8 + uv]

#define MaxError	.00001		/* maximum tolerable error */

/*
 * Reference inverse Dct.
 *                7   7                       (2i+1)u*PI      (2j+1)v*PI
 * x(i,j) = (1/4) Sum Sum X(u,v) C(u)C(v) cos(----------) cos(-----------)
 *                u=0 v=0                          16              16
 */
static void
ReferenceIDct2d8x8(double *x)
{
    float X[8*8];
    int i, j, u, v;
    float pixel;

    memcpy(X, x, sizeof(X));
    for(i = 0; i < 8; i++) {
	for(j = 0; j < 8; j++) {
	    pixel = 0.0;
	    for(u = 0; u < 8; u++) {
		for(v = 0; v < 8; v++) {
		    pixel += X[8*u+v] * IDctCoeff(i,u) * IDctCoeff(j,v);
		}
	    }
	    x[8*i+j] = pixel;
	}
    }
}
