/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	<name> - <one-line description of header file>
 *
 * DESCRIPTION
 *	<overall description of what the header file does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/DeviceControl.H,v 1.26 1994/02/21 04:05:48 toddb Exp $
 */
#ifndef _DeviceControl_H_
#define _DeviceControl_H_

/*
 *	DEFINES
 */
#include "mcs-server.h"
#include "jpegcomp.h"
#include "mpegdecomp.h"

/*
 *	TYPEDEFS
 */
typedef enum {
    Baud1200,
    Baud2400,
    Baud4800,
    Baud9600,
    NumberOfBaudRates
} BaudRate;

#define DEFAULT_BAUD_RATE Baud4800

typedef enum {
    VideoInactive,
    VideoActive,
    VideoDeviceStopped
} VideoState;

typedef enum {
    DeviceControlInactive,
    DeviceControlActive,
    DeviceControlFailed
} DeviceControlState;

typedef struct _mcsData {
    McsServer server;
    union {
	McsJpegComp cJpeg;
	McsJpegDecomp dJpeg;
    } u;
    McsMpegDecomp dMpeg;
    boolean inputReady;
    Boolean incomplete;
    int lastKnownStopFrame;
} McsData;

/*
 * This structure is shared among all applications using device control
 * and the functions that provide feedback for the device control.
 */
typedef struct _DevWidgetInfo {
    XtAppContext appContext;
    UInt displayProtocol;
    McsData mcs;
    char destination[256];
    char pendingDestination[256];
    int sourceDevice;
    int sourceDeviceTtyBaud;
    char *sourceDeviceTty;
    int destFps;
    float sourceFps;
    int displayWhenStopped;
    Boolean recording;
    Boolean validDestination;
    Widget destinationDisplay;
    Widget videoPane;
    Widget videoDisplay;
    Widget currentDisplay;
    Widget inpointDisplay;
    Widget outpointDisplay;
    Widget sourcePanel;
    Widget cameraSourcePanel;
    Widget segmentPanel;
    Widget capturePanel;
    Widget topLevel;
    Widget mainWindow;
    Widget modeCreateButton;
    Widget modeAppendButton;
    Widget runningButton;
    Widget frameByFrameButton;
    Widget videoAndAudioButton;
    Widget videoOnlyButton;
    Widget audioOnlyButton;
    Widget destModeButton;
    Widget destButton;
    Widget setInpointButton;
    Widget setOutpointButton;
    Widget gotoInpointButton;
    Widget gotoOutpointButton;
    Widget segPreviewButton;
    Widget segRecordButton;
    Widget freeRecordButton;
    Widget stopAnyRecordButton;
} DevWidgetInfo;

/*
 * This structure holds the abstraction for the various actions
 * performed on a media device
 */
typedef struct _deviceControlFns {
    void (*initFn) (Widget w, Boolean *initOK, XtPointer alsoUnused);
    void (*closeFn) (Widget w, XtPointer unused, XtPointer alsoUnused);
    void (*playFn) (Widget w, XtPointer stuff, XtPointer unused);
    void (*stillFn) (Widget w, XtPointer stuff, XtPointer unused);
    void (*stopFn) (Widget w, XtPointer stuff, XtPointer unused);
    void (*singleForwardFn) (Widget w, XtPointer stuff, XtPointer unused);
    void (*fastForwardFn) (Widget w, XtPointer stuff, XtPointer unused);
    void (*singleReverseFn) (Widget w, XtPointer stuff, XtPointer unused);
    void (*fastReverseFn) (Widget w, XtPointer stuff, XtPointer unused);
    void (*updateCounterFn) (Widget w, Widget label, XtPointer unused);
    void (*searchFn) (Widget w, XtPointer stuff, XtPointer unused);
    void (*previewFn) (Widget w, XtPointer stuff, XtPointer unused);
    void (*recordRunningFn) (Widget w, XtPointer stuff, XtPointer unused);
    void (*recordTwoPassFn) (Widget w, XtPointer stuff, XtPointer unused);
    void (*stopRecordFn) (Widget w, XtPointer stuff, XtPointer unused);
    ULong hasInitFn : 1;
    ULong hasCloseFn : 1;
    ULong hasPlayFn : 1;
    ULong hasStillFn : 1;
    ULong hasStopFn : 1;
    ULong hasSingleForwardFn : 1;
    ULong hasFastForwardFn : 1;
    ULong hasSingleReverseFn : 1;
    ULong hasFastReverseFn : 1;
    ULong hasUpdateCounterFn : 1;
    ULong hasSearchFn : 1;
    ULong hasPreviewFn : 1;
    ULong hasRecordVideoFn : 1;
    ULong hasRecordAudioFn : 1;
    ULong hasStopRecordFn : 1;
} DeviceControlFns;

extern DevWidgetInfo DevWidgetData; /* filled in by the application */

#endif /* _DeviceControl_H_ */
