/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	nvrDeckCtl.h - abstract support for various VCR/VDP decks
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrDeckCntl.h,v 1.17 1995/08/22 20:07:12 toddb Exp $
 */
#ifndef _NVRDECKCTL_H_
#define _NVRDECKCTL_H_

#include "nvrTypes.h"
#include "nvrReason.h"
#include <Intrinsic.h>

typedef enum {
    kDeckFastRev       = 0x000001,
    kDeckStepRev       = 0x000002,
    kDeckPlayRev       = 0x000004,
    kDeckPlay          = 0x000008,
    kDeckPause         = 0x000010,
    kDeckStepFwd       = 0x000020,
    kDeckFastFwd       = 0x000040,
    kDeckSeekOk        = 0x000080,	/* can seek */
    kDeckTimecodeDpyOk = 0x000100,	/* device can show timecode on-screen */
    kDeckSeekCbOnDrag  = 0x000200,	/* can respond real-time to seek */
    kDeckIsDevice      = 0x000400,	/* otherwise it "software-only" */
    kDeckPrerollNeeded = 0x000800,	/* for recording, that is */
    kDeckCanRealTime   = 0x001000,	/* Can play every frame in real-time */
    kDeckCanRecord     = 0x002000,	/* can do "assemble" edits */
    kDeckCanSlave      = 0x004000,	/* deck can slave to source or sink */
    kDeckCanInsert     = 0x008000,	/* can do "insert" edits */
    kDeckFrameAccurate = 0x010000,	/* deck is frame accurate */
    kDeckMediaIsFile   = 0x020000,	/* Otherwise, its a "tape" */
    kDeckFrameRecord   = 0x040000,	/* deck can to single frame records */
    kDeckCanStripe     = 0x080000,	/* media is tape and can be "striped" */
    kDeckCanValidate   = 0x100000,	/* media is tape and can be "striped" */
    kDeckCanDubTimecode= 0x200000,	/* striped timecodes can be dubbed */
    kDeckCanEject      = 0x400000,	/* media can be ejected */
    kDeckVarSpeedOk    = 0x800000
} DeckAbilityBits;

typedef enum {
    kNoBaudAvail	= 0x000,
    kBaudAvail2400	= 0x001,
    kBaudAvail4800	= 0x002,
    kBaudAvail9600	= 0x004,
    kBaudAvail19200	= 0x008,
    kBaudAvail38400	= 0x010
} BaudAvailable;

typedef enum {
    kUnknownDeckStd,
    kNtscDeck,			/* 2nd field occupies top line */
    kPalDeck,			/* 1st field occupies top line */
    kNonVideoDeck		/* i.e., not interlaced */
} DeckStandard;

typedef enum {
    kHardwareDeck,		/* i.e., tape, video disk, D1, etc. */
    kJpegDeck,			/* disk storage, hw or sw playback */
    kMpegDeck,			/* disk storage, hw or sw playback */
    kFileSetDeck		/* files (tiff, gif, or ?) sw playback */
} DeckMedium;

typedef enum {
    kSerialTtyDeck,		/* connected via tty line */
    kTelnetDeck,		/* connected via ethernet */
    kIntegratedDeck		/* connected internally to some board */
} DeckProtocolTransport;

typedef enum {
    kNoTimecode,
    kSmpteDropFrame,
    kSmpteNonDropFrame
} DeckTimecodeType;

typedef struct _deckTimecode {
    char hour;
    char min;
    char sec;
    char frame;
} DeckTimecode, *DeckTimecodePtr;

typedef enum {
    kNoRecording,		/* no recording active */
    kRecording,			/* player is slaving to us */
    kSlaveRecording,		/* we slave to the player */
    kFrameRecording		/* frame-at-a-time recording */
} RecordType;

typedef enum {
    kUnknownConnector,		/* not connected */
    kScreenPlug,		/* screen or window */
    kAnalogVideoPlug,		/* video connector */
    kFilePlug,			/* file in a file system */
    kMemoryPlug,		/* memory (e.g. sw decoder) */
    kBlackPlug			/* Black, for video blanking */
} DeckConnectorType;

typedef struct _deckConnectorInfo DeckConnectorInfo, *DeckConnectorInfoPtr;
struct _deckConnectorInfo {
    DeckConnectorType type;	/* selected from enum, above */
    char *name;			/* symbolic name */
    int channel;		/* used by the deck to identify connector */
    DeckConnectorInfoPtr next;
}; /* typedef: DeckConnectorInfo, *DeckConnectorInfoPtr */

/*
 * This is what is used to represent each deck controller type
 */
typedef struct _playSegment *PlaySegmentPtr;
typedef struct _deckCntl DeckCntl, *DeckCntlPtr;
typedef struct _deckRunConfig DeckRunConfig, *DeckRunConfigPtr;
typedef struct _playerClass PlayerClassRec, *PlayerClass;
typedef struct _deckCable DeckCable, *DeckCablePtr;

/*
 * For device connections between decks, the system-level connection
 * mechanism may need a handle to remember connection details.  This is it.
 */
struct _deckCable {
    DeckConnectorType source;
    DeckConnectorType drain;
    void (*disconnect)(DeckCablePtr);
}; /* typedef: DeckCable, *DeckCablePtr */

struct _deckCntl {
    int fd;		/* if kDeckIsDevice, this is a selectable file desc */

    /*
     * possible operations.  All are non-null, but may not do anything
     * based on DeckAbilityBitsin 'ability'.
     */
    InternalReasonPtr (*fastRev)    (DeckCntlPtr);
    InternalReasonPtr (*stepRev)    (DeckCntlPtr);
    InternalReasonPtr (*playRev)    (DeckCntlPtr);
    InternalReasonPtr (*play)       (DeckCntlPtr, RecordType);
    InternalReasonPtr (*pause)      (DeckCntlPtr);
    InternalReasonPtr (*stepFwd)    (DeckCntlPtr);
    InternalReasonPtr (*fastFwd)    (DeckCntlPtr);
    InternalReasonPtr (*seek)       (DeckCntlPtr, long frame);
    void              (*close)      (DeckCntlPtr);
    InternalReasonPtr (*confDeck)   (DeckCntlPtr, DeckRunConfigPtr);
    InternalReasonPtr (*eject)      (DeckCntlPtr);

    /*
     * These functions will stripe a tape and validate it.
     */
    InternalReasonPtr (*stripe)     (DeckCntlPtr, PlaySegmentPtr,
				     boolean dub, int tapeId);
    InternalReasonPtr (*validate)   (DeckCntlPtr);

    /*
     * This function is called from a player when the 'record' parameter
     * is true.  The RecordType is determined at the player level
     * and is configured into the recorder before calling play().
     */
    InternalReasonPtr (*record)     (DeckCntlPtr, PlaySegmentPtr);

    /*
     * When preparing for a recording, the recorder calls this function
     * in the player to connect the player's output with the recorder's
     * input.
     */
    DeckCablePtr (*connect)(DeckCntlPtr player, DeckCntlPtr recorder,
			    InternalReasonPtr *);

    /*
     * This is the base level function to use as a filter for
     * file names if the kDeckIsDevice bit is *off*
     */
    boolean           (*fileok)     (DeckCntlPtr, char *path);

    /*
     * These functions are used by LoadPlayList() and CreatePlayList()
     * to load and unload a movie file.
     */
    InternalReasonPtr (*createMovie)(DeckCntlPtr, PlaySegmentPtr, char *media);
    InternalReasonPtr (*loadMovie)  (DeckCntlPtr, PlaySegmentPtr, char *media);
    void              (*unloadMovie)(DeckCntlPtr, PlaySegmentPtr);

    char *deckName;                     /* path name, device name, etc */
    char *prodName;			/* this can be used for widget names */
    char *mediaName;			/* current media id */
    long tapeId;			/* tape id from tapes with smpte TC */
    DeckStandard standard;		/* signal standard type */
    ULong standards;			/* 'or' of available standards */
    DeckTimecodeType tcType;		/* timecode type, if any */
    ULong tcTypes;			/* 'or' of available timecode types */
    ULong ability;                      /* mask of DeckAbilityBits */
    double frameRate;			/* frame rate of material */
    long width, height;			/* width, height of selected sequence */
    long frame;                         /* the current frame number */
    long minFrame, maxFrame;            /* range of frame numbers, if known */
    char *frameLabel;			/* the label used for the frame # */
    void (*frameFormat)(DeckCntlPtr, char *buf, int length);
    long speed;				/* relative deck speed */
    long msToRecord;			/* updated when master record deck */
    long minSpeed, maxSpeed;		/* range of relative speed */
    boolean running;
    boolean mediaAvailable;		/* true: tape in, file defined, etc */
    boolean selected;			/* true: selected as player/recorder */
    PlaySegmentPtr playList;		/* current play list for this deck */
    PlaySegmentPtr edit;		/* current play list for editing */
    int segSize;			/* private size for a PlaySegment */
    DeckConnectorInfoPtr recorderInput;	/* selected recorder input */
    DeckConnectorInfoPtr recorderInputs;/* available recorder inputs */
    DeckConnectorInfoPtr playerOutputs;	/* available player outputs */
    PlayerClass parent;			/* back pointer to parent player */
    DeckCntlPtr pDeck;			/* player for this recorder */
    DeckCntlPtr rDeck;			/* recorder for this player */
}; /* typedef: DeckCntl, *DeckCntlPtr */

typedef void (*DeckChangedCallback)(DeckCntlPtr, void *cbData);
typedef void (*RecordCallback)(DeckCntlPtr);

/*
 * This structure and enum set is used to do run-time configuration of
 * a deck.
 */
typedef enum {
    kChangeMedia      = 0x001,
    kChangeStandard   = 0x002,
    kChangeTimecode   = 0x004,
    kChangeRecorder   = 0x008,
    kChangePlayer     = 0x010,
    kChangeSelect     = 0x020,
    kChangeSpeed      = 0x040,
    kChangeSegment    = 0x080,
    kChangeRecordType = 0x100,
    kChangeRecordCb   = 0x200
} DeckRunConfigType;

struct _deckRunConfig {
    ULong configType;		/* set of DeckRunConfigType bits */
    char *path;			/* valid if kChangeMedia set */
    boolean newMedia;		/* valid if kChangeMedia set */
    DeckStandard standard;	/* valid if kChangeStandard set */
    DeckTimecodeType tcType;	/* valid if kChangeTimecode set */
    DeckCntlPtr recorder;	/* recorder for this player */
    DeckCntlPtr player;		/* player for this recorder */
    boolean select;		/* whether the player is selected */
    long speed;			/* relative speed */
    PlaySegmentPtr seg;		/* edit segment */
    RecordType recordType;	/* recording type */
    /* a recorder calls this to notify player after a frame record is done */
    RecordCallback recordCB;
}; /* typedef: DeckRunConfig, *DeckRunConfigPtr */

/*
 * This describes the interesting aspects of a deck that must be known
 * before selecting the deck.
 */
typedef struct _deckDesc DeckDesc, *DeckDescPtr;
typedef struct _deckConfig DeckConfig, *DeckConfigPtr;

struct _deckDesc {
    char *prodName;		/* this can be used for widget names */
    ULong ability;		/* kDeckIsDevice and/or kDeckCanRecord */
    ULong baudAvailable;	/* baud rates if kDeckIsDevice */
    DeckCntlPtr (*create)(DeckConfigPtr, InternalReasonPtr *);
    DeckMedium medium;		/* tape, jpeg, mpeg, etc */
    DeckProtocolTransport proto;/* how to talk to the deck */
}; /* typedef: DeckDesc, *DeckDescPtr */

/*
 * This is used to specify a configuration for a deck during creation
 * and is passed to DeckDescPtr->create().
 */
struct _deckConfig {
    DeckChangedCallback changedCB;	/* DeckCntlPtr changed */
    void *cbData;			/* data passed to changedCB */
    char *deckName;			/* path for kDeckIsDevice */
    long baudrate;			/* baud rate for kDeckIsDevice */
    PlayerClass player;			/* back pointer to parent player */
    DeckDescPtr desc;			/* reference */
}; /* typedef: DeckConfig, *DeckConfigPtr */

/*
 * A list of these composite structures is returned by ChooseDeck()
 * below and represents all decks that match the specified DeckMedium.
 */
typedef struct _deckChoice DeckChoice, *DeckChoicePtr;
struct _deckChoice {
    char **deckNames;		/* list of names for DeckConfigPtr->deckName */
    int nNames;			/* number of names */
    DeckDescPtr desc;		/* a deck matching specified DeckMedium */
    DeckChoicePtr next;		/* next choice in the list */
};

extern DeckDesc SanyoGVR950Desc;
extern DeckDesc SgiJpegMovieDesc;
extern DeckDesc ViscaDesc;
extern DeckDesc PioneerLdvDesc;

/*
 * These globals provide a default value for the deck->frameLabel
 * and deck->frameFormat.
 */
extern char *DefaultDeckFrameLabel;

CExternal void DeckFormatFrameAsNumber(
    DeckCntlPtr,
    char *buf,
    int length
);

CExternal void DeckFormatFrameAsSMPTE(
    DeckCntlPtr,
    char *buf,
    int length
);

/*
 * This is the platform-independent interface for picking a
 * deck of a certain type.
 */
CExternal DeckChoicePtr FindDeckChoices(
    DeckMedium,
    ULong ability,
    InternalReasonPtr *
);

CExternal void FreeDeckChoices(
    DeckChoicePtr
);

/*
 * Basic helper routines
 */
CExternal void FrameToTimecode(
    long frame,
    DeckTimecodeType type,
    DeckTimecodePtr tc
);

CExternal void FormatTimecode(
    DeckTimecodePtr tc,
    char *tcBuf,
    int length
);

CExternal DeckCntlPtr GetDeckFromPlayer(
    PlayerClass player
);

CExternal void CancelDeckRecord(
    DeckCntlPtr
);

#endif /* _NVRDECKCTL_H_ */
