/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	nvrDeckCtlI.h - Internal definitions for decks
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrDeckCntlI.h,v 1.7 1995/07/11 23:02:09 toddb Exp $
 */
#ifndef _NVRDECKCTLI_H_
#define _NVRDECKCTLI_H_

#include "nvrDeckCntl.h"
#include "nvrString.h"

/*
 * The parsing routines will only allow recursion on files to this depth
 */
#define MaxRecurseLevel	10

/*
 * This is used to express a single segment; a linked list of them
 * describes a play list.  They are maintained by the each individual
 * deck which may define a larger structure that encapsulates this.
 * Note that inFrame may be more than outFrame if the segment is
 * played backwards.  However, baseFrame and nFrames are always
 * non-negative.
 */
typedef struct _playSegment PlaySegment;
struct _playSegment {
    char *mediaName;		/* path name for files, media name for tapes */
    long baseFrame;		/* total frames for segments before this */
    long nFrames;		/* total frames for this segment */
    long inFrame;		/* first frame of defined playback */
    long outFrame;		/* the frame *after* the last playback frame */
    PlaySegmentPtr next;	/* next sequence definition */
}; /* PlaySegment, *PlaySegmentPtr */

/*
 * if a connection from a player to a recorder is made such that the
 * the destination is digital (kMemoryPlug or kFilePlug), then
 * GetRawCableFrame() must be called for every frame.  It returns
 * the structure below that defines an rgb frame in a very rigid form.
 * The intent is to finish this implementation quickly.
 *
 * The data pointed to by 'rgbFrame' is a continuous array of 0BGR,
 * 32 bits each.
 */
typedef struct _cableFrame {
    ULong *rgb;
    int width, height;
} CableFrame, *CableFramePtr;

/*
 * This is the model for implementation of cable connection makers.
 * Functions of this type (MakeConnection) are placed into a table
 * (CableEntry), and the function appropriate
 * for the target connection is called internally from SystemConnect().
 */
typedef DeckCablePtr (*MakeConnection)(
    DeckCntlPtr player,
    DeckConnectorType source,
    DeckCntlPtr recorder,
    DeckConnectorType drain,
    InternalReasonPtr *pIrp
);

typedef struct _cableEntry {
    MakeConnection maker;
    DeckConnectorType source;
    DeckConnectorType drain;
} CableEntry, *CableEntryPtr;

/***************************************************************************
 *
 * Functions for handling PlaySegmentPtr
 *
 ***************************************************************************/
CExternal void UnloadPlayList(
    DeckCntlPtr
);

CExternal PlaySegmentPtr CreatePlayList(
    DeckCntlPtr,
    char *file,
    InternalReasonPtr *pReason
);

CExternal PlaySegmentPtr LoadPlayList(
    DeckCntlPtr,
    char *file,
    InternalReasonPtr *pReason
);

CExternal void FreeSegmentList(
    DeckCntlPtr deck, 
    PlaySegmentPtr list
);

CExternal PlaySegmentPtr AppendSegmentToList(
    PlaySegmentPtr list,
    PlaySegmentPtr segment
);

CExternal PlaySegmentPtr ReversePlayList(
    PlaySegmentPtr list
);

CExternal PlaySegmentPtr LastPlaySegment(
    PlaySegmentPtr list
);

CExternal PlaySegmentPtr FrameToPlaySegment(
    PlaySegmentPtr list,
    long frame,
    InternalReasonPtr *pReason
);

CExternal void CopyDeckEditSegment(
    DeckCntlPtr deck,
    PlaySegmentPtr copy,
    boolean recorder
);

CExternal void UnionABDeckSegments(
    DeckCntlPtr pDeck,
    DeckCntlPtr rDeck,
    PlaySegmentPtr rUnion
);

/***************************************************************************
 *
 * Functions for handling DeckCablePtr and DeckConnectorType
 *
 ***************************************************************************/
CExternal InternalReasonPtr UnknownConnector(
    DeckConnectorType type
);

CExternal DeckCablePtr SystemBlackConnect(
    DeckCntlPtr recorder,
    InternalReasonPtr *pIrp
);

CExternal DeckCablePtr SystemConnect(
    DeckCntlPtr player,
    DeckCntlPtr recorder,
    InternalReasonPtr *pIrp
);

CExternal CableFramePtr GetRawCableFrame(
    DeckCablePtr,
    DeckStandard standard,
    InternalReasonPtr *pIrp
);

CExternal char *DeckRecordType(
    RecordType
);

CExternal char *DeckPlugType(
    DeckConnectorType
);

CExternal DeckCablePtr XScreenConnect(
    DeckCntlPtr player,
    DeckConnectorType source,
    DeckCntlPtr recorder,
    DeckConnectorType drain,
    InternalReasonPtr *pIrp
);

/***************************************************************************
 *
 * Miscellaneous
 *
 ***************************************************************************/
CExternal Widget PlayerDisplayWidget(
    PlayerClass
);

#endif /* _NVRDECKCTLI_H_ */
