#ifndef lint
static char SCCSid[] = "@(#)elmmulti.c	1.14 1/26/93 ELAN";
#endif

/*
 ****************************************************************************
 * Copyright (c) 1989-1992 Elan Computer Group, Inc.
 * All Rights Reserved.
 *
 * This is unpublished proprietary source code of Elan Computer Group, Inc.
 * The copyright notice above does not evidence any actual
 * or intended publication of such source code.
 *
 * ----------------------------------------
 *
 * elmmulti.c - Another sample Elan License Manager application.
 *		Demonstrates multiple client connections.
 *		1) Connects two clients with different userids to
 *		   one server.
 *		2) Connects another clients userid with another
 *		   vendors server with a different salt and port.
 *
 * This application is a teaching tool to demonstrate how to integrate
 * the Elan License Manager with an application.  For simplicity,
 * this application does little error recovery. It often simply prints
 * out any errors if and when they occur.
 ****************************************************************************
 */

#include <stdio.h>
#include <ctype.h>
#include <varargs.h>
#include <sys/signal.h>
#include "elm.h"

#define TRUE	1
#define FALSE	0

#define DEFAULT_FEATURE		"99"

/*
 * The heartbeat in a typical application is about 60 seconds.
 * We're using 10 seconds here just so that you can seen things happen 
 * a lot faster.  Use the -b option or change this define if you would 
 * like to change the heartbeat time.
 */
static unsigned int beat = 10;
char *progname;

extern char *ctime();
extern long atol();

#define NHANDLES	3
int handles[NHANDLES];

/*
 * Routine called when process exits.
 *	1. Disconnect from license server.
 *	2. Exit with error code "code".
 */
void
byebye(code)
{
	int i;

	printf ("\nPID %d disconnecting.\n", getpid());

	for (i = 0; i < NHANDLES; i++)
	{
		printf ("[H %d, CID %d] Closing connection.\n", 
			handles[i], elh_clientid (handles[i]));
		elh_bye(handles[i]);
	}

	printf ("PID %d exiting.\n", getpid());

	exit (code);
}

quit(va_alist)
va_dcl
{
	va_list args;
	char *fmt;

	va_start (args);
	fmt = va_arg(args, char *);

	vfprintf (stderr, fmt, args);
	fprintf (stderr, "\n");

	va_end (args);

	byebye(-1);
	exit (-1);
}

/*
 * Catch all normal signals (hangup, interrupt, quit, term)
 * and call "byebye" routine when any of these signals occur.
 */
void
catchem()
{
	signal (SIGHUP, byebye);
	signal (SIGINT, byebye);
	signal (SIGQUIT, byebye);
	signal (SIGTERM, byebye);
}

/*
 * Print an Elan License Manager error message.
 *	fnc	- Function that we have issued.
 *	feature	- Feature name that we have requested.
 *	x	- Error code returned by elm routine.
 */
message(handle, fnc, feature, x)
char *fnc, *feature;
{
	printf("[H %d, CID %d] %s", handle, elh_clientid(handle), fnc);

	printf(": %s\n", elm_message ((char *)NULL, feature, x));

	fflush (stdout);
}

/*
 * Do all the initialization stuff:
 *	1. Catch signals.
 *	2. Connect with the licenser server.
 *	3. Exit if any problems.
 */
do_init (feature, callback)
char *feature;
int (*callback)();
{
	char server[ELM_HOST_LEN+1], *h;
	int result;
	extern char *getenv();

	catchem();			/* Catch signals */

	/*
	 * Set ELMHOST (see elm_salt.c) in environnment to give 
	 * server name explicitly.
	 * If not set, system uses Server Address Resolution
	 * to find the license server automagically.
	 */
	server[0] = '\0';
	if ((h = getenv(elm_host)) && h[0])
		strcpy (server, h);

	result = elm_init(server, feature, ELMDIR, callback);

	if (result < 0)
	{
		printf ("%s\n", elm_message (server, feature, result));
		exit (1);
	}

	printf ("[H %d, CID %d] Connected with server \"%s\".\n", 
		result, elh_clientid (result), server);
	return (result);
}

/*
 * Here's the heartbeat to tell the license server that we're
 * still alive and kicking.  It is set to be called (via SIGALRM)
 * once a minute.
 */
void
heartbeat()
{
	char expired_feature[ELM_FEATURE_LEN+1];
	char user[ELM_USER_LEN+1], host[ELM_HOST_LEN+1];
	char display[ELM_DISPLAY_LEN+1];
	register int x;
	static int handle = -1;

	alarm(0);		/* Turn off */
	signal (SIGALRM, SIG_IGN);

	expired_feature[0] = '\0';

	if (++handle >= NHANDLES)
		handle = 0;
	x = elh_allalive(handles[handle], expired_feature);

	if (x != ELM_OK)
	{
		message(handles[handle], "alive", expired_feature, x);
		byebye(1);
	}
	if (x == ELM_OK && elm_status == ELM_SRV_DOWN)
		printf ("License server down (not-fatal)\n");
	elh_getuser (handles[handle], user, host, display);
printf ("[H %d, CID %d] %s@%s,%s, pid %d: %s.\n", 
		handles[handle], elh_clientid (handles[handle]), 
		user, host, display, getpid(),
		x == ELM_OK ? "Running":"Error detected");

	printf ("\n");

	signal (SIGALRM, heartbeat);	
	alarm (beat);		/* Trigger */
}

do_getlicense(handle, feature, option)
unsigned long option;
char *feature;
{
	struct elm_info info;
	int x;

	x = elh_getlicense(handle, feature, option, 1L);/* Get a license */

	if (x != ELM_OK)
	{
		if (option & ELM_QUEUE)
		{
			do
			{
				printf ("Number %d in the queue\n", x);
				sleep (15);
			} while ((x = elh_getlicense(handle, feature,
						ELM_GETQUEUED, 1L)) != ELM_OK);
		}
		else
		{
			message(handle, "getlicense", feature, x);
			byebye(1);
		}
	}

	printf ("[H %d, CID %d] %slicense for \"%s\" granted:\n",
		handle, elh_clientid (handle), 
			option & ELM_SHARED ? "shared ":"", feature);

	if (elh_getinfo (handle, &info, feature, 0))
	{
		printf ("\t%d licenses\n", info.numlic);
		printf ("\tVendordata = %s; ", 
			info.vendordata[0] ? info.vendordata:"(none)");
		printf ("\tExpires %.24s", 
		    (info.expiration == 0) ? "Never":ctime (&info.expiration));
	}
	printf ("\n\n");
}

/*
 * Establish a different salt and port to talk to another vendor's daemon...
 */
int
callback (handle)
ELM_HANDLE handle;
{
	elh_setsalt (handle, "anothersalt");
	elh_setport (handle, 0, "elmd2");
	return (ELM_OK);
}

/*
 ***************************************************************************
 * MAIN
 ***************************************************************************
 */
main(argc, argv)
char **argv;
{
	extern char *optarg;		/* in getopt() */
	extern int optind;		/* in getopt() */
	int i, c;
	char *feature = DEFAULT_FEATURE;
	long holdtime = 0L;
	unsigned long getoption = ELM_GETLIC;
	int SARusefeature = TRUE;

	progname = argv[0];

	while ((c = getopt(argc, argv, "f:h:b:qsFi")) != EOF)
	{
	    switch (c)
	    {
		case 'i':
			printf ("ELMMULTI Release %s [VID %lx]\n",
						elm_release, elm_vid());
			exit (0);
			break;
		case 'q':
			getoption |= ELM_QUEUE;
			break;
		case 's':
			getoption |= ELM_SHARED;
			break;
		case 'b':
			beat = atoi (optarg);
			break;
		case 'f':
			feature = optarg;
			break;
		case 'F':
			SARusefeature = FALSE;
			break;
		case 'h':
			holdtime = atol (optarg);
			break;
		case '?':
printf ("Usage: elmappl [-f feature -h holdtime -b beat -i -q -s -F]\n");
			exit (1);
	    }
	}

	handles[0] = do_init(SARusefeature ? "99":(char *)0, (int (*)())0);
	handles[1] = do_init(SARusefeature ? "99":(char *)0, (int (*)())0);
	handles[2] = do_init(SARusefeature ? "77":(char *)0, callback);
	printf ("\n");

	if (holdtime)
	{
		if (elh_control (handles[0], ELMC_SETHOLD, holdtime) != ELM_OK)
			printf ("SETHOLD failed\n");
		if (elh_control (handles[1], ELMC_SETHOLD, holdtime) != ELM_OK)
			printf ("SETHOLD failed\n");
		if (elh_control (handles[2], ELMC_SETHOLD, holdtime) != ELM_OK)
			printf ("SETHOLD failed\n");
	}

	elh_setuser (handles[0], "jack", "mars", "nebula:0");
	elh_setuser (handles[1], "jill", "venus", "nova:0");
	elh_setuser (handles[2], "justine", "saturn", "starfire:0");

	do_getlicense (handles[0], "99", getoption);	/* Get a license */
	do_getlicense (handles[1], "99", getoption);	/* Get a license */
	do_getlicense (handles[2], "77", getoption);	/* Get a license */

	heartbeat();			/* Set up heartbeat */

	/*
	 * Here's where an application does it's normal stuff.
	 * We just loop 10000 times.
	 * (We also exit if a keyboard interrupt or TERM signal is received.)
	 */
	for (i = 0; i < 10000; i++)
		pause();

	byebye(-1);
}
