#ifndef lint
static char SCCSid[] = "@(#)xtiny.c	1.10 1/26/93 ELAN";
#endif

/*
 * Copyright (c) 1992 Elan Computer Group, Inc.
 * All Rights Reserved.
 *
 * This is unpublished proprietary source code of Elan Computer Group, Inc.
 * The copyright notice above does not evidence any actual
 * or intended publication of such source code.
 */

/*
 * TINY - A Fancy Sample X Application Using Elan License Manager
 * --------------------------------------------------------------
 * 
 * Description
 * -----------
 * This sample application is a simple text editor, called tiny, that
 * operates under X-11.  Tiny should act as a handy example of how you
 * would use Elan License Manager in a real application, and, in
 * particular, an X application.  Please feel free to plagiarize this code
 * in your applications.
 * 
 * Two Versions for Xt and Xview
 * -----------------------------
 * There are actually two version of tiny and three files:
 * 
 * 	xtiny.c and xttiny.c	- Xt version of tiny "xttiny"
 * 	xtiny.c and xvtiny.c	- Xview version of tiny "xvtiny"
 * 
 * The file xtiny.c is common for both. The other files are unique
 * to their particular tool kit.  The biggest differences
 * to note regarding the Elan License manager are the way even handling
 * is done - see where elm_alive and CatchSignal are used.
 * 
 * You may only need to examine one version if you're only using one of Xt
 * or Xview.
 * 
 * Operation
 * ---------
 * Tiny brings up an edit window with two buttons and a message area.  The
 * user can type into the edit window, using all the standard keyboard and
 * mouse text editing functions.  Note that the Xt version uses control
 * characters for certain commands (such as Control-S for Search), while
 * the Xview one uses the right mouse button. (Not that it's very
 * important, but it is cool.)  The user can save the text at any time
 * into a pre-named file, "tiny.txt", via the SAVE button.
 * 
 * Error Conditions
 * ----------------
 * If while tiny is running, you can lose your right to use the product
 * for a number of reasons:
 * 
 * 	1) The license expires while you're running it (ELM_EXPIRED).
 * 	2) Redundant servers are running, but a majority are no longer active
 * 	   (ELM_SRV_DOWN).
 * 	3) Serious network failure (ELM_SENDFAIL).
 * 	4) Your license was returned by force via "elmadmin -b" (bury license)
 * 	   (ELM_ERROR).
 * 	5) You have lost your license, probably through some sneakery
 * 	   (ELM_LOST).
 * 
 * Under any of these conditions, tiny goes into read-only mode - it won't
 * allow you to modify your file any more, and says "Save you file and exit".
 * You should use a similar strategy within your applications.
 *
 */

#include "elm.h"
#include <varargs.h>
#include <sys/time.h>
#include <X11/Intrinsic.h>

#define plural(n)	((n) == 1 ? "":"s")

extern char *getenv();
char *progname;

char *FEATURE = "99";
char *Keydir = ELMDIR;          /* Default key directory */

/*
 * Print a message to stderr, call elm_bye(), and exit.
 */
void
quit(va_alist)
va_dcl
{
	va_list args;
	char *fmt;

	va_start (args);
	fmt = va_arg(args, char *);

	fprintf (stderr, "%s: ", progname);
	vfprintf (stderr, fmt, args);
	fprintf (stderr, "\n");

	va_end (args);

	elm_bye();
	exit (-1);
}

void
usage()
{
	fprintf (stderr,
	  "Usage: %s [ -b # -e dir -f feature -h # -i -q -s -t # -u user ]\n",
				progname);
	exit (1);
}

char *
howlong (seconds)
time_t seconds;
{
	static char msg[64];

	if (seconds > 70)
		sprintf (msg, "%.1f minutes", seconds / 60.0);
	else
		sprintf (msg, "%lu seconds", seconds);
	return (msg);
}

/*
 * Connect with license server.
 */
void
do_elm_init(feature)
char *feature;
{
	char *h;
	char server[ELM_HOST_LEN+1];
	int x;

	server[0] = '\0';

	/*
	 * If ELMHOST is set, try using it first as server host.
	 */
	if ((h = getenv(elm_host)) && h[0])
		elm_strncpy (server, h, ELM_HOST_LEN);

	elm_var = 0;
	x = elm_init(server, feature, Keydir, NULL);

	/*
	 * If the elm_init above failed, and we were using a user supplied
	 * ELMHOST, then just try again using Server Address Resolution.
	 */
	if (x != ELM_OK && h)
	{
		server[0] = '\0';
		x = elm_init(server, feature, Keydir, NULL);
	}

	if (x < 0)
		quit ("elm_init: %s", elm_message (server, feature, x));

	printf ("Connected with server %s\n", server);
}

int
do_getlicense(feature, options, ntokens)
char *feature;
unsigned long options;
unsigned long ntokens;
{
	struct elm_info info;
	int x;

	/* 
	 * Try to get a license
	 */
	x = elm_getlicense(feature, options, ntokens);

	if (x != ELM_OK)
	{
		if (x < 0)
		{
			printf ("elm_getlicense: %s\n",
				elm_message((char *)NULL, feature, x));
		}
		if (options & ELM_QUEUE)
		{
			printf ("[%d] No licenses available. Queueing...\n",
				elm_cid);
			options = (options & ~ELM_QUEUE) | ELM_GETQUEUED;
			while (x != 0)
			{
				printf ("[%d] Number %d in the queue\n",
					elm_cid, x);
				sleep (15);
				x = elm_getlicense(feature, options, ntokens);
			}
		}
		else
			exit(1);
	}

	printf ("[%d] %sLicense for \"%s\" granted: %ld token%s.\n",
	    elm_cid, 
	    options & ELM_SHARED ? "Shared ":"",
	    feature,
	    ntokens, 
	    plural(ntokens));

	if (elm_getinfo (&info, feature, 0))
	{
		EXTERN char *ctime	CARGS((time_t *));

		printf ("    Feature = \"%s\", VendorData = \"%s\"\n", 
		    info.feature, info.vendordata);
		printf ("    Licenses in use = %lu.\n",
		    info.inuse_resv + info.inuse_free);
		printf ("    Total licenses  = %lu (%lu free pool + %lu reserved).\n", 
		    info.numlic, info.numlic - info.numresv, info.numresv);
		printf ("    Expiration = %s\n", 
		    (info.expiration == 0) ? "Never":ctime (&info.expiration));
	}
	printf ("\n");
	return (0);
}

char *
basename(path)
char *path;
{
	register char *p;
	extern char *strrchr();

	if ((p = strrchr (path, '/')) && p[1])
		return (&p[1]);
	return (path);
}


void
main(argc, argv)
int argc;
char *argv[];
{
	int c, beat = 10;
	char *user = NULL;
	unsigned long getoption = ELM_GETLIC;
	unsigned long ntokens = 1;
	unsigned long holdtime = 0;
	extern char *optarg;
	extern optind;
	extern long atol();
	extern void heartbeat();
	extern void MainLoop();
	extern void CreateWidgets();
	extern void SetReadOnly();

	progname = basename (argv[0]);

	while ((c = getopt(argc, argv, "b:e:f:h:i:qst:u:")) != EOF)
	{
		switch (c)
		{

		case 'b':
			beat = atoi (optarg);
			break;
		case 'e':
			Keydir = optarg;
			break;
		case 'f':
			FEATURE = optarg;
			break;
		case 'h':
			holdtime = atol (optarg);
			break;
		case 'i':
			printf ("XELMDEMO Release %s [VID %lx]\n",
			    elm_release, elm_vid());
			exit (0);
			break;
		case 'q':
			getoption |= ELM_QUEUE;
			break;
		case 's':
			getoption |= ELM_SHARED;
			break;
		case 't':
			ntokens = atol (optarg);
			getoption |= ELM_COUNT;
			break;
		case 'u':
			user = optarg;
			break;

		case '?':
			usage();
		}			/* end switch */
	}

	do_elm_init(FEATURE);

	if (holdtime)
	{
		if (elm_control (ELMC_SETHOLD, holdtime) != ELM_OK)
			printf ("SETHOLD failed\n");
	}

	if (user)
		elm_setuser (user, (char *)NULL, (char *)NULL);

	do_getlicense (FEATURE, getoption, ntokens);

	CreateWidgets(argc, argv);

	heartbeat(beat);

	SetReadOnly(FALSE);

	MainLoop();

	elm_bye();

	exit(0);
}
