#ifndef lint
static char SCCSid[] = "@(#)xttiny.c	1.4 2/23/93 ELAN";
#endif

/*
 * Copyright (c) 1992 Elan Computer Group, Inc.
 * All Rights Reserved.
 *
 * This is unpublished proprietary source code of Elan Computer Group, Inc.
 * The copyright notice above does not evidence any actual
 * or intended publication of such source code.
 */

/*
 * xttiny.c - The "Xt" code for the "tiny" editor.
 *
 *            See xtiny.c for details.
 */

#define XAW_BC

#include <sys/types.h>
#include <varargs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Text.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Label.h>
#include <X11/StringDefs.h>
#include <sys/signal.h>
#include "elm.h"

#define INITIAL_WIDTH	400
#define INITIAL_HEIGHT	500

#define MESSAGE_WIDTH	230
#define DRAWAREA_WIDTH	(MESSAGE_WIDTH + 108)

#define FILENAME	"tiny.txt"

XtAppContext app;
Widget drawingA, statusW, messageW;

/*
 * Print a status message.
 * If X is initialized, the message goes to the message window.
 * Otherwise it goes to stdout.
 */
void
StatusMessage(str)
char *str;
{
	Arg arg;

	if (statusW != NULL)
	{
		XtSetArg(arg, XtNlabel, str);
		XtSetValues(statusW, arg, 1);
	}
	else
		printf ("%s\n", str);
}

/*
 * Set the application to "read-only" mode and update status message.
 */
void
SetReadOnly(ro)
int ro;
{
	Arg arg, darg;

	if (ro) 
	{
		XtSetArg (darg, XtNeditType, XawtextRead);  /* read-only */
		XtSetValues(drawingA, darg, 1);

		XtSetArg (arg, XtNlabel, "** READ ONLY **");
	}
	else
	{
		XtSetArg(arg, XtNlabel, "Elan's tiny text editor");
	}

	XtSetValues(messageW, arg, 1);
}

void
Exception(code, feature)
char *feature;
{
	if (code != ELM_OK) {
		SetReadOnly(TRUE);
		StatusMessage ("License lost: Save your file and exit!");
	}
	else
		SetReadOnly(FALSE);
}


void
heartbeat(seconds)
unsigned int seconds;
{
	elm_xtheartbeat(app, seconds, Exception);
}


/*
 * Action function for when the user types a character into the text widget.
 */
static void
typeAction(widget, client_data, call_data)
Widget widget;
caddr_t client_data;
caddr_t call_data;
{
	StatusMessage("");
}

/*
 * Action function for QUIT button.
 */
static void
quitAction(widget, client_data, call_data)
Widget widget;
caddr_t client_data;
caddr_t call_data;
{
	elm_bye();
	exit(0);
}

/*
 * Action function for SAVE button.
 */
static void
saveAction(widget, client_data, call_data)
Widget widget;
caddr_t client_data;
caddr_t call_data;
{
	FILE *fp;
	char *text;
	Arg arg;
	char msg[80];

	if ((fp = fopen (FILENAME, "w")) == NULL) {
		StatusMessage("File problem: Not saving.");
		return;
	}
	sprintf (msg, "Saving \"%s\" ...", FILENAME);
	StatusMessage(msg);

	XtSetArg(arg, XtNstring, &text);
	XtGetValues(drawingA, arg, 1);

	fputs(text, fp); /* Write the entire buffer from widget */
	fclose(fp);

	sprintf (msg, "File \"%s\" written.", FILENAME);
	StatusMessage(msg);
}

/*
 * Creates all the widgets and X goodies.
 */
void
CreateWidgets(argc, argv)
int argc;
char *argv[];
{
	Widget toplevel, printbutton, quitbutton;
	Widget form, box, messagebox;
	Display *display;
	XGCValues gcv;
	GC gc;
	Arg arg;
	Font font;

	toplevel = XtVaAppInitialize(&app, "Tiny", NULL, 0,
	    &argc, argv, NULL, 
	    XtNheight, INITIAL_HEIGHT,
	    XtNwidth, INITIAL_WIDTH,
	    NULL);

	/* create a drawing area widget */
	form = XtVaCreateWidget("form",
	    formWidgetClass, toplevel,
	    NULL);

	/* command button stuff */
	box = XtVaCreateWidget("box",
	    formWidgetClass, form,
	    XtNleft, XtChainLeft,
	    XtNtop, XtChainTop,
	    XtNx, 0,
	    XtNy, 0,
	    NULL);

	printbutton = XtVaCreateWidget("save",
	    commandWidgetClass, box,
	    XtNx, 0,
	    XtNy, 0,
	    /* these numbers just looked right */
	XtNheight, 10,
	    NULL);
	XtAddCallback(printbutton, XtNcallback, saveAction, 0);

	quitbutton = XtVaCreateWidget("quit",
	    commandWidgetClass, box,
	    XtNfromHoriz, printbutton,
	    /* these numbers just looked right */
	XtNheight, 10,
	    NULL);
	XtAddCallback(quitbutton, XtNcallback, quitAction, 0);


	/* message box stuff */
	messagebox = XtVaCreateWidget("messagebox",
	    formWidgetClass, form,
	    XtNright, XtChainRight,
	    XtNtop, XtChainTop,
	    XtNfromHoriz, box,
	    NULL);

	messageW = XtVaCreateWidget("messageWidget",
	    labelWidgetClass, messagebox,
	    XtNfromHoriz, box,
	    XtNborderWidth, 0,
	    XtNvertDistance, 0,
	    XtNinternalHeight, 0,
	    XtNjustify, XtJustifyCenter,
	    /* these numbers just looked right */
	    XtNheight, 8,
	    XtNwidth, MESSAGE_WIDTH,
	    XtNx, 0,
	    XtNlabel, "",
	    NULL);

	statusW = XtVaCreateWidget("messageWidget",
	    labelWidgetClass, messagebox,
	    XtNfromVert, messageW,
	    XtNvertDistance, 0,
	    XtNborderWidth, 0,
	    XtNjustify, XtJustifyLeft,
	    XtNinternalHeight, 0,
	    /* these numbers just looked right */
	    XtNheight, 8,
	    XtNwidth, MESSAGE_WIDTH,
	    XtNx, 0,
	    XtNlabel, "",
	    NULL);


	/* text widget stuff */
	drawingA = XtVaCreateWidget("drawingA",
	    asciiTextWidgetClass, form,
	    XtNeditType, XawtextEdit,
	    XtNwrap, XawtextWrapWord,
	    XtNfromVert, messagebox,
	    XtNright, XtChainRight,
	    XtNleft, XtChainLeft,
	    XtNbottom, XtChainBottom,
#if defined(editable) && defined(scrollVertical) && defined(scrollOnOverflow)
	    XtNtextOptions, editable|scrollVertical|scrollOnOverflow|wordBreak,
#endif
	    XtNx, 0,
	    XtNy, 30,
	    XtNheight, 210,
	    XtNwidth, DRAWAREA_WIDTH,
	    NULL);

	display =  XtDisplay(drawingA);

	/* load the font */
	font = XLoadFont(display, "-*-helvetica-medium-r-normal--10-*");

	/* create GC and attach to drawing area's XmNuserData */
	gcv.foreground = BlackPixelOfScreen(XtScreen(drawingA));

	gc = XCreateGC(display,
	    RootWindowOfScreen(XtScreen(drawingA)), GCForeground, &gcv);

	XSetFont(display, gc, font);

	XtManageChild(printbutton);
	XtManageChild(quitbutton);
	XtManageChild(box);
	XtManageChild(messageW);
	XtManageChild(statusW);
	XtManageChild(messagebox);
	XtManageChild(drawingA);
	XtManageChild(form);
	XtRealizeWidget(toplevel);

	XFlush(display);
}

void
CatchSignal()
{
	elm_bye();
	exit (1);
}

static void
SetSignalHandlers()
{
	signal(SIGHUP, CatchSignal);
	signal(SIGINT, CatchSignal);
	signal(SIGQUIT, CatchSignal);
	signal(SIGILL, CatchSignal);
	signal(SIGTRAP, CatchSignal);
	signal(SIGIOT, CatchSignal);
	signal(SIGABRT, CatchSignal);
	signal(SIGEMT, CatchSignal);
	signal(SIGFPE, CatchSignal);
	signal(SIGBUS, CatchSignal);
	signal(SIGSEGV, CatchSignal);
	signal(SIGSYS, CatchSignal);
#ifdef SIGPIPE
	signal(SIGPIPE, CatchSignal);
#endif
#ifdef SIGXCPU
	signal(SIGXCPU, CatchSignal);
#endif
#ifdef SIGXFSZ
	signal(SIGXFSZ, CatchSignal);
#endif
#ifdef SIGLOST
	signal(SIGLOST, CatchSignal);
#endif
	signal(SIGTERM, CatchSignal);
}

void
MainLoop()
{
	SetSignalHandlers();
	XtAppMainLoop(app);
}
