#ifndef lint
static char SCCSid[] = "@(#)xvtiny.c	1.4 1/4/93 ELAN";
#endif

/*
 * Copyright (c) 1992 Elan Computer Group, Inc.
 * All Rights Reserved.
 *
 * This is unpublished proprietary source code of Elan Computer Group, Inc.
 * The copyright notice above does not evidence any actual
 * or intended publication of such source code.
 */

/*
 * xvtiny.c - The "Xview" code for the "tiny" editor.
 *            See xtiny.c for details.
 */

#include "elm.h"
#include <sys/types.h>
#include <X11/Intrinsic.h>
#include <xview/xview.h>
#include <xview/frame.h>
#include <xview/panel.h>
#include <xview/textsw.h>

#define INITIAL_WIDTH	400
#define INITIAL_HEIGHT	500
#define FILENAME	"tiny.txt"

Panel_item statusW, messageW;
Textsw drawingA;
Frame toplevel;

/*
 * Print a status message.
 * If X is initialized, the message goes to the message window.
 * Otherwise it goes to stdout.
 */
void
StatusMessage(str)
char *str;
{
	if (statusW != XV_NULL)
		xv_set(statusW, PANEL_LABEL_STRING, str, NULL);
	else
		printf ("%s\n", str);
}

/*
 * Set the application to "read-only" mode and update status message.
 */
void
SetReadOnly(ro)
int ro;
{
	if (ro)
	{
		xv_set(drawingA, TEXTSW_READ_ONLY, TRUE, NULL);/* read-only */

		xv_set(messageW, PANEL_LABEL_STRING, "** READ ONLY **", NULL);
	}
	else
	{
		xv_set(drawingA, TEXTSW_READ_ONLY, FALSE, NULL);
		xv_set(messageW, PANEL_LABEL_STRING, 
		    "Elan's tiny text editor", NULL);
	}
}

void
Exception(code, feature)
char *feature;
{
	if (code != ELM_OK) {
		SetReadOnly(TRUE);
		StatusMessage ("License lost: Save your file and exit!");
	}
	else
		SetReadOnly(FALSE);
}

void
heartbeat(seconds)
int seconds;
{
	elm_xvheartbeat(toplevel, seconds, Exception);
}


/*
 * Action function for when the user types a character into the text widget.
 */
static void
typeAction(item, event)
Panel_item item;
Event *event;
{
	StatusMessage("");
}

/*
 * Action function for QUIT button.
 */
static void
quitAction(item, event)
Panel_item item;
Event *event;
{
	elm_bye();
	exit(0);
}

/*
 * Action function for SAVE button.
 */
static void
saveAction(item, event)
Panel_item item;
Event *event;
{
	int status;
	char msg[80];

	sprintf (msg, "Saving \"%s\" ...", FILENAME);
	StatusMessage(msg);

	status = (int) textsw_store_file(drawingA, FILENAME, 0, 0);
	if (status != 0)
		StatusMessage("File problem: Not saved.");
	else
	{
		sprintf (msg, "File \"%s\" written.", FILENAME);
		StatusMessage(msg);
	}
}

/*
 * Creates all the widgets and X goodies.
 */
void
CreateWidgets(argc, argv)
int argc;
char *argv[];
{
	Panel form, messageform;
	Panel_item printbutton, quitbutton;

	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

	toplevel = (Frame) xv_create(NULL, FRAME,
	    FRAME_LABEL, "tiny",
	    XV_HEIGHT, INITIAL_HEIGHT,
	    XV_WIDTH, INITIAL_WIDTH,
	    NULL);

	/* create a panel area widget */
	form = (Panel) xv_create(toplevel, PANEL,
	    PANEL_LAYOUT, PANEL_HORIZONTAL,
	    XV_HEIGHT, 30,
	    NULL);

	/* command button stuff */
	printbutton = (Panel_item)xv_create(form, PANEL_BUTTON,
	    PANEL_NOTIFY_PROC, saveAction,
	    PANEL_LABEL_STRING, "Save",
	    NULL);

	quitbutton = (Panel_item)xv_create(form, PANEL_BUTTON,
	    PANEL_NOTIFY_PROC, quitAction,
	    PANEL_LABEL_STRING, "Quit",
	    NULL);


	messageW = (Panel_item)xv_create(form, PANEL_MESSAGE,
	    PANEL_LABEL_STRING, "",
	    XV_Y, 1,
	    NULL);

	statusW = (Panel_item)xv_create(form, PANEL_MESSAGE,
	    PANEL_LABEL_STRING, "",
	    XV_Y, 15,
	    NULL);


	/* text widget stuff */
	drawingA = (Textsw) xv_create(toplevel, TEXTSW,
	    TEXTSW_NOTIFY_PROC, typeAction,
	    NULL);
	frame_fit_all(toplevel);
}

Notify_func
Bye()
{
	elm_bye();
	exit (1);
}

static void
SetSignalHandlers(client, frame)
Notify_client client;
Frame frame;
{
	notify_set_signal_func(client, (Notify_func)Bye, SIGHUP, NOTIFY_ASYNC);
	notify_set_signal_func(client, (Notify_func)Bye, SIGINT, NOTIFY_ASYNC);
	notify_set_signal_func(client, (Notify_func)Bye, SIGQUIT, NOTIFY_ASYNC);
	notify_set_signal_func(client, (Notify_func)Bye, SIGILL, NOTIFY_ASYNC);
	notify_set_signal_func(client, (Notify_func)Bye, SIGTRAP, NOTIFY_ASYNC);
	notify_set_signal_func(client, (Notify_func)Bye, SIGIOT, NOTIFY_ASYNC);
	notify_set_signal_func(client, (Notify_func)Bye, SIGABRT, NOTIFY_ASYNC);
	notify_set_signal_func(client, (Notify_func)Bye, SIGEMT, NOTIFY_ASYNC);
	notify_set_signal_func(client, (Notify_func)Bye, SIGFPE, NOTIFY_ASYNC);
	notify_set_signal_func(client, (Notify_func)Bye, SIGBUS, NOTIFY_ASYNC);
	notify_set_signal_func(client, (Notify_func)Bye, SIGSEGV, NOTIFY_ASYNC);
	notify_set_signal_func(client, (Notify_func)Bye, SIGSYS, NOTIFY_ASYNC);
#ifdef SIGPIPE
	notify_set_signal_func(client, (Notify_func)Bye, SIGPIPE, NOTIFY_ASYNC);
#endif
#ifdef SIGXCPU
	notify_set_signal_func(client, (Notify_func)Bye, SIGXCPU, NOTIFY_ASYNC);
#endif
#ifdef SIGXFSZ
	notify_set_signal_func(client, (Notify_func)Bye, SIGXFSZ, NOTIFY_ASYNC);
#endif
#ifdef SIGLOST
	notify_set_signal_func(client, (Notify_func)Bye, SIGLOST, NOTIFY_ASYNC);
#endif

	notify_set_destroy_func(client, (Notify_func)Bye);	/* SIGTERM */
}

void
MainLoop()
{
	SetSignalHandlers(statusW, toplevel);
	xv_main_loop(toplevel);
}
