#ifndef lint
static char SCCSid[] =
"@(#)wrapper.c	1.5 2/4/93 18:04:09 - Elan Computer Group, Inc. (c) 1991";
#endif

/*
 ****************************************************************************
 * Copyright (c) 1991 Elan Computer Group, Inc.
 * All Rights Reserved.
 *
 * This is unpublished proprietary source code of Elan Computer Group, Inc.
 * The copyright notice above does not evidence any actual
 * or intended publication of such source code.
 *
 ****************************************************************************
 *
 * wrapper.c - Protecting applications with Elan License Manager.
 *
 * How to set up:
 *
 * 1) Create a new group in /etc/group called "secure" with a unique 
 *    group number and no one as a member.
 *
 * 2) Create a directory, such as /usr/lib/secure.  The applications will
 *    go here. (See SECURE_DIR define below.)
 *
 * 3) Make this directory readable by the group "secure" only:
 *
 *			chown root   /usr/lib/secure
 *			chgrp secure /usr/lib/secure
 *			chmod 070    /usr/lib/secure
 *
 * 4) Move the applications you wish to protect to this directory.
 *    For example:
 *
 *			mv wings /usr/lib/secure
 *
 * 5) Edit this application "wrapper.c" and change the defines 
 *    for SECURE_DIR and KEYDIR if you have chosen different directory names.
 *
 *    Also edit the "features" structure to equate application
 *    names with feature names. (Feature names are numeric if numeric
 *    keys are to be used.)
 *
 * 6) Install this program wrapper in /usr/bin (or /usr/local/bin, etc.)
 *    as the same same as the program you are protecting.  For example,
 *
 *			mv wrapper /usr/bin/wings
 *
 * 7) Set the owner and modes for this program (/usr/bin/wings) as follows:
 *
 *			chown secure /usr/bin/wings
 *			chmod 111 /usr/bin/wings
 *			chnod g+s /usr/bin/wings
 *
 * 8) Proceed with the standard Elan License Manager installation.
 *
 * 9) Generate a key using Ean License Manager for the feature name 
 *    that is the same as the program name ("wings" in our example above.)
 *
 * 10) You're ready to run!
 *
 ****************************************************************************
 */

/*
 * Change these to whatever you want...
 */

#define SECURE_DIR	"/usr/lib/secure"	/* Real prog location */
#define KEYDIR		"/usr/lib/elm"		/* Key directory */

/****************************************************************************/


#include <stdio.h>
#include <ctype.h>
#include <sys/signal.h>
#include "../lib/elm.h"

#define equal(a,b)	(strcmp(a,b)==0)

extern char *ctime();

static char *progname;
static char server[ELM_HOST_LEN+1];

static int debug = 1;

struct features
{
	char *appname;
	char *featurename;
} features[] =
{
	/* CHANGE THIS LIST AS DESIRED */

	/* Application		Numeric Feature Name */

	"wings",		"01",
	"wordcool",		"02",
	"waycool",		"03",
	"wrapper",		"99",

	(char *)0,		(char *)0	/* End of list */
};

/*VARARGS1*/
quit(fmt, a, b, c, d, e)
char *fmt, *a, *b, *c, *d, *e;
{
	fprintf (stderr, fmt, a, b, c, d, e);
	fprintf (stderr, "\n");
	elm_bye();
	exit (-1);
}

char *
featurename (appname)
char *appname;
{
	register int i;

	for (i = 0; features[i].appname; i++)
		if (equal (appname, features[i].appname))
			return (features[i].featurename);
	return (NULL);
}

char *
basename(path)
char *path;
{
	register char *p;
	extern char *strrchr();

	if ((p = strrchr (path, '/')) == NULL)
		return (path);
	return (&p[1]);
}

/*
 * Routine called when process exits.
 *	1. Disconnect from license server (elm_bye).
 *	2. Exit with error code "code".
 */
void
byebye(code)
{
	elm_bye();
	exit (code);
}

/*
 * Catch all normal signals (hangup, interrupt, quit, term).
 * Call function "fnc" when any of these signals occur.
 */
void
catchem(fnc)
void (*fnc)();
{
	signal (1, fnc);
	signal (2, fnc);
	signal (3, fnc);
	signal (15, fnc);
}

/*
 * Do all the initialization stuff:
 *	1. Connect with the licenser server (elm_init).
 *	2. Exit with a message if any problems.
 */
do_init (appname, numfeature)
char *appname, *numfeature;
{
	extern char *getenv();
	char *h;
	int x;

	/*
	 * Set elm_host (see elm_salt.c) in environnment to give 
	 * server name explicitly.
	 * If not set, system uses Server Address Resolution
	 * to find the license server automagically.
	 */
	server[0] = '\0';
	if ((h = getenv(elm_host)) && h[0])
		strcpy (server, h);

	x = elm_init(server, numfeature, KEYDIR, (int (*)())NULL);
	if (x < 0)
		quit ("%s\n", elm_message (server, appname, x));
	if (debug)
		printf ("Connected with server \"%s\".\n", server);
}

/*
 * Here's the heartbeat to tell the license daemon that we're
 * still alive and kicking.  It is set to be called (via SIGALRM)
 * once a minute.
 */
void
heartbeat()
{
	char expired_feature[ELM_FEATURE_LEN+1];
	register int x;
	static unsigned int alarm_time = 60;

	alarm(0);		/* Turn off */
	signal (SIGALRM, SIG_IGN);

	expired_feature[0] = '\0';

	x = elm_alive(expired_feature);

	if (x == ELM_OK)
		alarm_time = 60;
	else
	{
		/*
		 * Get progressively obnoxious...
		 * First 5 minutes about 1 message a minute.
		 * More frequent messages each next minute.
		 * A message every 5 seconds by 10 minutes;
		 */
		if (alarm_time > 50)
			alarm_time -= 2;
		else if (alarm_time > 10)
			alarm_time -= 10;
		else if (alarm_time > 5)
			alarm_time -= 5;
		printf ("                                     \n");
		printf ("                                     \n");
		printf ("                                     \n");
		printf ("                                     \n");
		printf ("   *******************************   \n");
		printf ("   * LICENSE LOST! PLEASE EXIT!! *   \n");
		printf ("   *******************************   \n");
		printf ("                                     \n");
		printf ("                                     \n");
		printf ("                                     \n");
		printf ("                                     \n");
	}

	signal (SIGALRM, heartbeat);	
	alarm (alarm_time);		/* Trigger */
}

do_getlicense(appname, numfeature)
char *appname, *numfeature;
{
	int x;

	x = elm_getlicense(numfeature, ELM_GETLIC|ELM_COUNT, 1L);

	if (debug && x == ELM_OK)
		printf ("License for %s [%s] granted.\n", appname, numfeature);

	if (x < 0)
	{
		printf ("%s\n", elm_message (server, appname, x));
		exit (1);
	}
}

run (feature, argc, argv, envp)
char *feature, **argv, **envp;
{
	int pid, status;
	char path[256];

	sprintf (path, "%s/%s", SECURE_DIR, feature);

	if (debug)
		printf ("Running \"%s\" ...\n", path);

	argv[0] = path;
	argv[argc] = NULL;

	catchem(SIG_IGN);	/* Ignore signals in parent */

	pid = fork();

	switch (pid)
	{
		case -1:
			quit ("Unable to fork process!");
			break;
		case 0:				/* CHILD */
			alarm (0);		/* Turn off alarm */
			catchem(SIG_DFL);	/* Default signal handling */
			execve (path, argv, envp);
			perror (path);
			_exit (1);
			break;
		default:			/* PARENT */
			while (wait (&status) != -1);
	}
}

/*
 ***************************************************************************
 * MAIN
 ***************************************************************************
 */
main(argc, argv, envp)
char **argv, **envp;
{
	char *appname, *numfeature;

	progname = argv[0];
	appname = basename (argv[0]);

	catchem(byebye);	/* Catch signals until we run real prog */

	if ((numfeature = featurename (appname)) == NULL)
		quit ("Application %s has no feature assigned.\n", appname);

	if (debug)
		printf ("Application %s requires feature %s\n",
			appname, numfeature);

	do_init(appname, numfeature);	/* Initialization */

	do_getlicense (appname, numfeature);/* Get a license */

	heartbeat();		/* Set up once a minute heartbeat */

	run (appname, argc, argv, envp);/* Run the real thing */

	elm_bye();		/* Disconect from server */
}
