#ifndef lint
static char SCCSid[] =
"@(#)client.c	4.62 1/17/94 Copyright (c) 1992 Elan Computer Group, Inc.";
#endif

/*
 ****************************************************************************
 * Copyright (c) 1989-1992 Elan Computer Group, Inc.
 * All Rights Reserved.
 *
 * This is unpublished proprietary source code of Elan Computer Group, Inc.
 * The copyright notice above does not evidence any actual
 * or intended publication of such source code.
 *
 * -------------------------------------------------------------------------
 *
 * elmappl.c - A sample Elan License Manager application.
 *
 * This application is a teaching tool to demonstrate how to integrate
 * the Elan License Manager with an application.  For simplicity,
 * this application does little error recovery. It often simply prints
 * out any errors if and when they occur.
 ****************************************************************************
 */

#include <stdio.h>
#include <ctype.h>
#include <varargs.h>
#include <signal.h>
#include "elm.h"

#ifdef WIN32
#  include <process.h>
#endif

#define TRUE	1
#define FALSE	0
#define plural(n)	((n) == 1 ? "":"s")

#define DEFAULT_FEATURE		"99"

/*
 * The heartbeat in a typical application is about every 60 seconds.
 * We're using 10 seconds here just so that you can seen things happen 
 * a lot faster.  Use the -b option or change this define if you would 
 * like to change the heartbeat time.
 */
static unsigned int beat = 10;

static unsigned long ntokens = 1;	/* Number of tokens: default 1 */
static char *progname;
static long pid;
static char *Keydir = elm_keydir;
static char server[ELM_HOST_LEN+1];

extern char *ctime();
extern long atol();
EXTERN int   getopt		CARGS((int, char **, char *));

/*VARARGS*/
void quit(va_alist)
va_dcl
{
	va_list args;
	char *fmt;

	va_start (args);
	fmt = va_arg(args, char *);

	vfprintf (stderr, fmt, args);
	fprintf (stderr, "\n");

	va_end (args);

	elm_bye();
	exit (-1);
}

/*
 * Routine called when process exits.
 *	1. Disconnect from license server (elm_bye).
 *	2. Exit with error code "code".
 */
/*VARARGS*/
void byebye(code)
{
	printf ("\n[%d] PID %ld disconnecting.\n", elm_clientid(), pid);

	elm_bye();

	printf ("[%d] PID %ld exiting.\n", elm_clientid(), pid);

	exit (code);
}

/*
 * Catch all normal signals (hangup, interrupt, quit, term)
 * and call "byebye" routine when any of these signals occur.
 */
void
catchem()
{
#ifdef SIGHUP
	signal (SIGHUP, byebye);
#endif
#ifdef SIGINT
	signal (SIGINT, byebye);
#endif
#ifdef SIGQUIT
	signal (SIGQUIT, byebye);
#endif
#ifdef SIGTERM
	signal (SIGTERM, byebye);
#endif
}

/*
 * Do all the initialization stuff:
 *	1. Catch signals.
 *	2. Connect with the licenser server (elm_init).
 *	3. Exit if any problems.
 */
void
do_init (feature)
char *feature;
{
	int handle;
	extern char *getenv();

	catchem();			/* Catch signals */

	server[0] = '\0';
	handle = elm_init(server, feature, Keydir, (int (*)())NULL);

	if (handle < 0)
	{
		printf ("%s.\n", elm_message (server, feature, handle));
		exit (1);
	}

	printf ("[%d] Connected with server \"%s\".\n", elm_clientid(), server);
}

void
Exception(code, feature)
char *feature;
{
	if (code == ELM_OK)		/* Not a serious problem */
	{
		if (elm_getstatus() == ELM_SRV_DOWN)
			printf ("License server down (not-fatal)\n");
	}
	else
	{
		/*
		 * We just exit - You will want to do something fancier.
		 */
		printf ("%s.\n", elm_message(server, feature, code));
		byebye(1);
	}
}

void
do_getlicense(feature, options)
char *feature;
unsigned long options;
{
	struct elm_info info;
	int x;

	/* 
	 * Try to get a license
	 */
	x = elm_getlicense(feature, options, ntokens);

	if (x != ELM_OK)
	{
		if (x < 0)
			printf ("%s.\n", elm_message(server, feature, x));
		if (options & ELM_QUEUE)
		{
			printf ("[%d] No licenses available. Queueing...\n", elm_clientid());
			options = (options & ~ELM_QUEUE) | ELM_GETQUEUED;
			while (x != 0)
			{
				printf ("[%d] Number %d in the queue\n", elm_clientid(), x);
#ifndef WINDOWS_31
				elm_sleep (15);
#endif
				x = elm_getlicense(feature, options, ntokens);
			} 
		}
		else
			byebye(1);
	}

	printf ("[%d] %sLicense for \"%s\" granted: %ld token%s.\n",
		elm_clientid(), 
		options & ELM_SHARED ? "Shared ":"",
		feature,
		ntokens, 
		plural(ntokens));

	if (elm_getinfo (&info, feature, 0))
	{
		printf ("    Feature = \"%s\", VendorData = \"%s\"\n", 
			info.feature, info.vendordata);
		printf ("    Licenses in use = %ld.\n",
			info.inuse_resv + info.inuse_free);
		printf ("    Total licenses  = %ld (%ld free pool + %ld reserved).\n", 
			info.numlic, info.numlic - info.numresv, info.numresv);
		if (info.node_locked)
			printf ("    This is a node locked feature.\n");
		printf ("    Expiration = %s\n", 
		    (info.expiration == 0) ? "Never":ctime (&info.expiration));
	}
	printf ("\n");
}

void
sethold(holdval)
long holdval;
{
	if (elm_control (ELMC_SETHOLD, holdval) == ELM_OK)
		printf ("Setting hold time to %ld seconds.\n", holdval);
	else
		printf ("SETHOLD failed\n");
}

void
gethold()
{
	long holdval;

	if (elm_control (ELMC_GETHOLD, (long)&holdval) == ELM_OK)
		printf ("Hold time currently set to %ld seconds.\n", holdval);
	else
		printf ("GETHOLD failed\n");
}

/*
 ***************************************************************************
 * MAIN
 ***************************************************************************
 */
void
main(argc, argv)
char **argv;
{
	extern char *optarg;		/* in getopt() */
	extern int optind;		/* in getopt() */
	int c;
	char *feature = DEFAULT_FEATURE;
	char *user = NULL;
	long holdtime = -1L;
	unsigned long getoption = ELM_GETLIC;
	int SARusefeature = TRUE;
	int non_blocking_io = FALSE;

	progname = argv[0];
	pid = getpid();

	while ((c = getopt(argc, argv, "f:h:b:qsFit:u:e:n")) != EOF)
	{
	    switch (c)
	    {
		case 'n':
			non_blocking_io = TRUE;
			break;
		case 'e':
			Keydir = optarg;
			break;
		case 'u':
			user = optarg;
			break;
		case 't':
			ntokens = atol (optarg);
			getoption |= ELM_COUNT;
			break;
		case 'i':
			printf ("ELMAPPL Release %s [VID %lx]\n",
						elm_getrelease(), elm_vid());
			exit (0);
			break;
		case 'q':
			getoption |= ELM_QUEUE;
			break;
		case 's':
			getoption |= ELM_SHARED;
			break;
		case 'b':
			beat = atoi (optarg);
			break;
		case 'f':
			feature = optarg;
			break;
		case 'F':
			SARusefeature = FALSE;
			break;
		case 'h':
			holdtime = atol (optarg);
			break;
		case '?':
	printf("Usage: %s [ -e keydir -f name -h # -b # -q -s -t # -F -i -u name ]\n",
				progname);
			exit (1);
	    }
	}

	if (non_blocking_io)
	{
		printf ("Using NON-BLOCKING I/O instead of SELECT.\n");
		elm_control (ELMC_HEARTBEAT_IO, ELM_USE_NON_BLOCKING_IO);
	}

	do_init(SARusefeature ? feature:(char *)0);	/* Initialization */

	if (user)
		elm_setuser (user, (char *)NULL, (char *)NULL);

	/* 
	 * Set a default minimum hold time for this process.
	 */
	if (holdtime > 0)
		sethold (holdtime);

	do_getlicense (feature, getoption);	/* Get a license */

	/*
	 * For demonstration purposes, we've implemented a hold time 
	 * of 0 here to mean turn of any holding, even if holding 
	 * is specified in the resource file.
	 */
	if (holdtime == 0)
		sethold (0L);

	elm_heartbeat (beat, Exception);	/* Set up the heartbeat */

	/*
	 * Here's where an application does it's normal stuff.
	 * We just loop forever.
	 * (We also exit if a keyboard interrupt or TERM signal is received.)
	 */

	for (;;) {
		printf ("[%d] PID %ld is running.\n", elm_clientid(), pid);
#ifdef WINDOWS_31
		for (c = -1; c != 0; c--);
#else
		elm_sleep (5);
#endif
	}

	/*
	 * If we did not have an infinite loop above, prior
	 * to exiting we would issue...
	 *	elm_bye();
	 */
	 
}
