#ifndef lint
static char SCCSid[] = "@(#)nodelock.c	1.6 1/3/94 ELAN";
#endif

/*
 * Copyright (c) 1993 Elan Computer Group, Inc.
 * All Rights Reserved.
 *
 * This is unpublished proprietary source code of Elan Computer Group, Inc.
 * The copyright notice above does not evidence any actual
 * or intended publication of such source code.
 */

#include <stdio.h>
#include "elm.h"

#define FEATURE                "99" 
static int badnews = 1;

/*
 * Sample program to demonstrate node-locking *without* 
 * using the license server elmd.
 */

int
callback()
{
#ifndef _WINDOWS
	printf ("Callback function called OK.\n");
#endif
	badnews = 0;
	return (0);
}

#ifndef _WINDOWS
#define TRUE			1
#define FALSE			0

EXTERN int   getopt		CARGS((int, char **, char *));

static char *Keydir = elm_keydir;
static char *feature = FEATURE;
static char *progname;


main(argc, argv)
char **argv;
{
	extern char *optarg;		/* in getopt() */
	extern int optind;		/* in getopt() */
	int x, c;

	progname = argv[0];

	printf ("Node Lock test program started.\n");

	while ((c = getopt(argc, argv, "f:ie:")) != EOF)
	{
	    switch (c)
	    {
		case 'e':
			Keydir = optarg;
			break;
		case 'i':
			printf ("NODELOCK Release %s [VID %lx]\n",
						elm_getrelease(), elm_vid());
			exit (0);
			break;
		case 'f':
			feature = optarg;
			break;
		case '?':
			printf("Usage: %s [ -e keydir -f name -i ]\n",
				progname);
			exit (1);
	    }
	}

	/*
	 * Validate feature on current node.
	 * The callback function will be called upon success.
	 */
	printf ("Checking for feature %s in %s ...\n", feature, Keydir);
	if ((x = elm_valid (Keydir, feature, callback)) < 0)
	{
		printf ("%s\n", elm_message ((char *)0, feature, x));
		exit (1);
	}
	else
		printf ("Feature is valid.\n");

	if (badnews)
	{
		printf ("Failed to call callback function - exiting.\n");
		exit (1);
	}

	printf ("Program ran successfully.\n");
	return (0);
}
#else

#include <windows.h>
#if !defined (APIENTRY) // Windows NT defines APIENTRY, but 3.x doesn't
#define APIENTRY far pascal
#endif

char szAppName[] = "NClient";   // The name of this application
char szTitle[]   = "Nodelocked Client";  
#define BUFSIZE 128
char keydir[BUFSIZE];

/****************************************************************************/

BOOL InitInstance(
    HINSTANCE          hInstance,
    int             nCmdShow)
{
    int status;
    HWND hWnd; 
    /* Create a main window for this application instance. */

    if (!(hWnd = CreateWindow(szAppName, szTitle, WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL)))
        return (FALSE);

    /* contact license manager and acquire permission to run a new copy.  */

    GetProfileString((LPSTR)elm_icon_name, "KeyDirectory", elm_keydir, keydir, BUFSIZE);
    if ((status = elm_valid (keydir, FEATURE, callback)) < 0)
    {
	wsprintf (keydir, "%s\n", elm_message ((char *)0, FEATURE, status));
	MessageBox(NULL, keydir, elm_icon_name, MB_OK);
	return(FALSE);
    }

    ShowWindow(hWnd, nCmdShow); 
    UpdateWindow(hWnd);        

    return (TRUE); 

}

/****************************************************************************/

LRESULT CALLBACK WndProc( HWND hWnd, UINT message, WPARAM uParam, LPARAM lParam)  
{
    switch (message) {

        case WM_COMMAND:
            return (DefWindowProc(hWnd, message, uParam, lParam));
            break;

        case WM_DESTROY: 
            PostQuitMessage(0);
            break;

        case WM_PAINT:
            {
		PAINTSTRUCT      ps;
		HDC              hDC;
                char msg[64];
                hDC = BeginPaint(hWnd, &ps);
                wsprintf (msg, "Nodelocking validated ");
                TextOut ( hDC, 100 , 100, msg, strlen(msg));
                if (badnews)
			TextOut(hDC, 100, 130, "callback failed", 20);
		else
			TextOut(hDC, 100, 130, "callback successful!", 15);
               
                EndPaint(hWnd, &ps);
            }
            break ;
        default:          // Passes it on if unproccessed
            return (DefWindowProc(hWnd, message, uParam, lParam));
    }
    return (0);
}
/****************************************************************************/

BOOL InitApplication(HINSTANCE hInstance)
{
    WNDCLASS  wc;
    wc.style         = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc   = (WNDPROC)WndProc;      
    wc.cbClsExtra    = 0;                    
    wc.cbWndExtra    = 0;                   
    wc.hInstance     = hInstance;          
    wc.hIcon         = LoadIcon (hInstance, szAppName);
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wc.lpszMenuName  = szAppName;              
    wc.lpszClassName = szAppName;             
    return (RegisterClass(&wc));
}

/****************************************************************************/

int APIENTRY WinMain(
    HINSTANCE hInstance,
    HINSTANCE hPrevInstance,
    LPSTR lpCmdLine,
    int nCmdShow)

{
    MSG msg;
    
    if (!hPrevInstance) {       // Other instances of app running?
        if (!InitApplication(hInstance))  // Initialize shared things
            return (FALSE);     // Exits if unable to initialize
    }

    /* Perform license manager initializations  */

    if (!InitInstance(hInstance, nCmdShow)) 
        return (FALSE);

    while (GetMessage(&msg, NULL, 0, 0))  {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    return (msg.wParam); 
    lpCmdLine; 
}
#endif
