/* $Header: /usr/local/src/nvrToMIT/RCS/elm.h,v 1.1 1994/06/27 23:40:39 toddb Exp $ */
/*
 * @(#)elm.h	4.123 1/27/94 Elan Computer Group, Inc.
 *
 * Copyright (c) 1989-1992 Elan Computer Group, Inc.
 * All Rights Reserved.
 *
 * This is unpublished proprietary source code of Elan Computer Group, Inc.
 * The copyright notice above does not evidence any actual
 * or intended publication of such source code.
 */

#ifndef _ELM_H
#define _ELM_H

#include <stdio.h>
#ifdef vms
#  include <types.h>
#else
#  ifdef SVR3
#    ifdef M88K
#	include <netinet/types.h>
#    else
#	include <sys/bsdtypes.h>
#    endif
#  endif
#  include <sys/types.h>
#  if !defined(NEXT) && !defined(APOLLO) && !defined(_WINDOWS) && !defined(_DOS)
#    include <unistd.h>
#  endif
#endif
#ifdef HAVE_STDLIB
#  include <stdlib.h>
#endif
#include <string.h>
#ifdef _WINDOWS
#include <time.h>
#endif

#define ELM_KEYFILE_MAGIC	"!<Elan-License-Manager-Key>"

/*
 * Values returned from ELM routines...
 */
#define ELM_OK				0
#define ELM_ERROR			(-1)
#define ELM_WAIT			(-2)
/* ELM 2.2: ELM_FAIL renamed ELM_NO_LICENSES */
#define ELM_NO_LICENSES			(-3)
#define ELM_SRV_DOWN			(-4)
#define ELM_AUTH_FAIL			(-5)
#define ELM_EXPIRED			(-6)
#define ELM_NOHOST			(-7)
#define ELM_SOCKFAIL			(-8)
#define ELM_SENDFAIL			(-9)
#define ELM_LOST			(-10)
#define ELM_NOKEY			(-11)
#define ELM_NOT_LICENSED		(-12)
#define ELM_NO_SERVICE			(-13)
#define ELM_CODE_CORRUPT		(-14)
#define ELM_NO_SUCH_FEATURE		(-15)
#define ELM_INVALID_CLIENT		(-16)
#define ELM_INSUFFICIENT_SERVERS	(-17)
#define ELM_CANT_WRITE			(-18)
#define ELM_CANT_READ			(-19)
#define ELM_SYS_NOPRIV			(-20)	/* Internal used only */
#define ELM_SUICIDE			(-21)	/* Internal used only */

#define ELM_NUMERROR			21

/*
 * Commands for elm_control...
 */
#define ELMC_ZEROLOG		1	/* Zero log file */
#define ELMC_DIE		2	/* Kill elmd */
#define ELMC_VERBOSE		3	/* Change message verbosity */
#define ELMC_DUMP		4	/* Cause elmd to dump system tables */
#define ELMC_SETVERSION		5	/* Set version for encryption type */
#define ELMC_SETKEYDIR		6	/* Set local keydir, like -e to elmd */
#define ELMC_SETHOLD		7	/* Set license hold time */
#define ELMC_SETBROADCAST	8	/* Set broadcast scope */
#define ELMC_BURYCLIENT		9	/* Bury a client */
#define ELMC_SETZOMBIE		10	/* Set time at which client is zombie */
#define ELMC_LOG		11	/* Copy message to log */
#define ELMC_LOG_WITH_DATE	12	/* Copy message to log w/ time stamp */
#define ELMC_LOG_WITH_DATE_USER	13	/* Copy message to log w/ time/user */
#define ELMC_HEARTBEAT_IO	14	/* Use select or non-blocking I/O */
#define ELMC_GETHOLD		15	/* Get license hold time */
#define ELMC_GET_VERBOSITY	16	/* Get current verbosity level */

/*
 * Arguments to ELMC_HEARTBEAT_IO:
 */
#define	ELM_USE_NON_BLOCKING_IO	((long)(1))
#define	ELM_USE_SELECT		((long)(2))

/*
 * Key types, set via elm_control (ELMC_SETVERSION, arg)...
 */
#define ELM_STDKEY_VERSION	'C'		/* Standard key Version */
#define ELM_NUMKEY_VERSION1	'D'		/* Before redundant servers */
#define ELM_NUMKEY_VERSION	'E'		/* ELM Release 2.0 */
#define ELM_VALID_VERSIONS	"ABCDE"		/* Legal key versions */

#define is_elm_numkey_version(v)	((v) == 'D' || (v) == 'E')
#define is_elm_redkey_id(v)		((v) == '1' || (v) == '2')

/*
 * Client <-> daemon packet versions
 *
 * Versions prior to 2.1 were the current key type, either C, D, or E.
 *
 * Ver	Description
 * ---	-----------
 * F	Does not use the port number in the encryption routines.
 */
#define ELM_VERSION_F		'F'		/* Release 2.1 */
#define ELM_VERSION_ID		ELM_VERSION_F	/* CURRENT VERSION */
#define ELMD_VALID_VERSIONS	"CDEF"		/* Versions elmd will accept */

#define crypt_use_port(v)	((v) != ELM_VERSION_F)

/*
 * Second argument to elm_gencode() and elm_hostid() ...
 * Max value is -1L=37777777777(oct)=ffffffff(hex)
 */
/* Maximum hostcode value ......... 037777777777L */
#define ELM_HOSTCODE_IDPROM_MASK    000000000077L
#define ELM_HOSTCODE_IPADDR         000000000100L      /* Use IP Address */
#define ELM_HOSTCODE_IDPROM_BIT     000000000200L
#define ELM_HOSTCODE_IDPROM(n)     (ELM_HOSTCODE_IDPROM_BIT|((n) > 077 ? 077:n))
#define ELM_HOSTCODE_IDPROM2       (ELM_HOSTCODE_IDPROM_BIT|2)   /* For DEC */
#define ELM_HOSTCODE_IDPROM_NUM(v) (v & ELM_HOSTCODE_IDPROM_MASK)
#define ELM_HOSTCODE_HOSTNAME       000000000400L      /* Use hostname */
#define ELM_HOSTCODE_ETHER          000000001000L      /* Ethernet address */
#define ELM_HOSTCODE_HISECURE       000000002000L /* Extended Redundant Key */
#define ELM_HOSTCODE_DISKID_BIT    (000000004000L)
#define ELM_HOSTCODE_DISKID_MASK    000000070000L
#define ELM_HOSTCODE_DISKID(n) \
	(((((n) >= 8 ? 7:((n)-1))<<12) & ELM_HOSTCODE_DISKID_MASK ) | \
	ELM_HOSTCODE_DISKID_BIT)
#define ELM_HOSTCODE_DISKID_NUM(v) ((((v) >> 12) & 07) + 1)
#ifdef vms
#  define ELM_HOSTCODE_STANDARD	(ELM_HOSTCODE_ETHER)
#else
#  define ELM_HOSTCODE_STANDARD	(ELM_HOSTCODE_IPADDR|ELM_HOSTCODE_IDPROM2)
#endif

#define ELM_HOSTCODE_DEFAULT	0L

/*
 * Second argument to elm_getlicense...
 */
#define ELM_GETLIC	0000L		/* Get a license */
#define ELM_QUEUE	0121L		/* Enter into queue (was 'Q') */
#define ELM_GETQUEUED	0103L		/* Get from queue (was 'C') */

/*
 * "or" these in with any of the above
 */
#define ELM_SHARED		0040L
#define ELM_COUNT		0010L
#define ELM_unusedbit2		0004L
#define ELM_SHARE_USER		(001L << 8)
#define ELM_SHARE_HOST		(002L << 8)
#define ELM_SHARE_DISPLAY	(004L << 8)

/*
 * All option bits in getlicense second argument...
 */
#define ELM_GLOPTMASK	(ELM_SHARED|ELM_COUNT|ELM_unusedbit2|ELM_SHARE_USER|ELM_SHARE_HOST|ELM_SHARE_DISPLAY)

#define ELM_UNQUEUE	0125L		/* Dequeue (was 'U') */

/*
 * Options to elm_retlicense...
 */

#define ELM_DECR	0001		/* Decrement license count */

/*
 * Options to elm_genkey/elm_genredkey...
 */
#define ELM_STDKEYS		000
#define ELM_SHORTKEYS		001
#define ELM_LITERAL_HOSTIDS	002

#define ELM_MASTER		1
#define ELM_SHADOW		2
#define ELM_NO_MASTER		3

/*
 * Maximum length of various strings.
 * These are true lengths, so a null terminated string
 * buffer should be declared [LEN+1] long.
 */
#define ELM_USER_LEN		10
#define ELM_HOST_LEN		64
#define ELM_DISPLAY_LEN		64
#define ELM_GROUP_LEN		16
#define ELM_FEATURE_LEN		32
#define ELM_DATETAG_LEN		256
#define ELM_NUMFEATURE_MINLEN	2
#define ELM_NUMFEATURE_MAXLEN	12
#define ELM_ALIAS_LEN		ELM_FEATURE_LEN
#define ELM_VERSION_LEN		8
#define ELM_KEYDIR_LEN		80
#define ELM_CODE_LEN		100
#define ELM_HOSTID_LEN		128
#define ELM_VENDORDATA_LEN	ELM_GROUP_LEN	/* Same field used for both */
#define ELM_PATHLEN		126

#define ELM_MAX_SERVERS	127
#define ELM_MAX_HOSTIDS	ELM_MAX_SERVERS

#define ELM_MULTIKEY_MAXNUMFEAT	11  /* Max number of features in a Multi-Key */
#define ELM_MULTIKEY_FEATMAXLEN	 8  /* Max digits in a feature in a Multi-Key */

/*
 * Convenient constants...
 */
#define ELM_JAN_1_1989	599644800L	/* time value of Jan 1st, 1989 00:00 */
#define ELM_MINUTE	(60L)		/* # of seconds in a minute */
#define ELM_HOUR	(60L*ELM_MINUTE)/* # of seconds in an hour */
#define ELM_DAY		(24L*ELM_HOUR)	/* # of seconds in a day */

/*
 * Non-network version code prefix
 */
#define ELM_NNCHR	'#'

typedef int elm_handle_t;
#define ELM_HANDLE	elm_handle_t
#define ELM_STDHANDLE	(ELM_HANDLE)0
#define ELM_NOHANDLE	(ELM_HANDLE)(-1)

#define elm_alive(f)			elh_alive(ELM_STDHANDLE, f)
#define elm_alllive(f)			elh_allalive(ELM_STDHANDLE, f)
#define elm_bye()			elh_bye(ELM_STDHANDLE)
#define elm_chklicense(f, n)		elh_chklicense(ELM_STDHANDLE, f, n)
#define elm_clientid()			elh_clientid(ELM_STDHANDLE)
#define elm_control(cmd, arg)		elh_control(ELM_STDHANDLE, cmd, arg)
#define elm_owner()			elh_owner(ELM_STDHANDLE)
#define elm_getinfo(info, f, id)	elh_getinfo(ELM_STDHANDLE, info, f, id)
#define elm_getulist(u, f, id)		elh_getulist(ELM_STDHANDLE, u, f, id)
#define elm_getuser(u, h, x)		elh_getuser(ELM_STDHANDLE, u, h, x)
#define elm_retlicense(f, opt)		elh_retlicense(ELM_STDHANDLE, f, opt)
#define elm_setport(p, s)		elh_setport(ELM_STDHANDLE, p, s)
#define elm_setsalt(u, s)		elh_setsalt(ELM_STDHANDLE, s)
#define elm_setuser(u, h, x)		elh_setuser(ELM_STDHANDLE, u, h, x)

struct elm_server
{
	char host[ELM_HOST_LEN+1];
	unsigned long licenses;
	time_t time_remaining;
	char status;
	int numpeers;
	unsigned long netaddr;
};

typedef struct elm_keyentry
{
	char key[ELM_CODE_LEN+1];
	char alias[ELM_ALIAS_LEN+1];
	char version[ELM_VERSION_LEN+1];
	char date_tag[ELM_DATETAG_LEN+1];
	int  numservers;
	char *servers[ELM_MAX_SERVERS+1];
} elm_keyentry_t;

struct elm_info
{
	char feature[ELM_FEATURE_LEN+1];/* Feature name */
	char alias[ELM_FEATURE_LEN+1];	/* Feature name alias */
	char vendordata[ELM_VENDORDATA_LEN+1];/* Optional Vendor Data */
	char domain;			/* Domain */
	char node_locked;		/* Non-zero if license is node locked */
	time_t expiration;		/* Date feature expires */
	time_t installed;		/* Date feature installed */
	unsigned long numlic;		/* Total # of licenses for feature */
	unsigned long numresv;		/* Total # of licenses reserved */
	unsigned long inuse_resv;	/* # of reserved licenses in use */
	unsigned long inuse_free;	/* # of free pool licenses in use */
	unsigned long avail_to_me;	/* # available to me */
};

struct elm_user
{
	char user[ELM_USER_LEN+1];	/* User name */
	char host[ELM_HOST_LEN+1];	/* Host name */
	char display[ELM_DISPLAY_LEN+1];/* X-11 DISPLAY */
	char group[ELM_GROUP_LEN+1];	/* Group (if a reserved license) */
	time_t start_time;		/* Time started feature */
	time_t alive_time;		/* Last "alive" time */
	time_t hold_time;		/* Time to hold to */
	unsigned long tokens;		/* Number of tokens allocated */
	int pid;			/* Client's PID */
	int handle;			/* Client's Handle */
	int cid;			/* Client's Elm ID */
	int lid;			/* Client's License ID */
	int shared;			/* shared count (0 if not) */
	char connected;			/* 'V', 'H', or 'C' */
};

#if (defined(__GNUG__) || defined(C_plusplus) || defined(__cplusplus))
						/* C++ */
#define EXTERN		extern "C"
#define ELM_CONST	const
#define CARGS(args)	args
#else

#if defined(__STDC__) || defined(_WINDOWS)
						/* ANSI-C */
#define EXTERN		extern
#define ELM_CONST	const
#define CARGS(args)	args

#else
						/* K&R C */
#define EXTERN		extern 
#define ELM_CONST
#define CARGS(args)	()
#endif
#endif


#ifdef _XtIntrinsic_h
#define XTAPPCONTEXT	XtAppContext
#else
#define XTAPPCONTEXT	void *
#endif

/*
 * Declaration format...
 *		EXTERN type name	CARGS((arglist))
 * Note: use double parens with CARGS.
 *
 * The declarations below may be sorted with "sort -b -d +2"
 */

#define ELM_UINT		unsigned int
#define ELM_UCHAR		unsigned char
#define ELM_ULONG		unsigned long

#if defined(_WINDOWS) && !defined(WIN32)
#  define WINDOWS_31
#endif

#if defined(_WINDOWS) && !defined(WINSOCK)
#  define NO_SOCKETS
#endif

#ifdef WINDOWS_31
#  define EEXPORT        FAR PASCAL  __export
#  define WINSTATIC static
#  define sprintf wsprintf
#else
#  define EEXPORT
#  define WINSTATIC
#endif

#ifdef WINDOWS_31
#include <windows.h>
#endif
#ifdef _WINDOWS
EXTERN int		getopt	CARGS((int, char **, char *));
#endif
EXTERN int	EEXPORT	elh_alive	CARGS((ELM_HANDLE, char *));
EXTERN int	EEXPORT	elh_bye		CARGS((ELM_HANDLE));
EXTERN int	EEXPORT	elm_checksum	CARGS((ELM_CONST char *));
EXTERN int	EEXPORT	elh_chklicense	CARGS((ELM_HANDLE, char *, int));
EXTERN int	EEXPORT	elh_clientid	CARGS((ELM_HANDLE));
EXTERN char *		elm_code	CARGS((int, ELM_CONST char *));
EXTERN int	EEXPORT	elm_codedecode 	CARGS((ELM_CONST char *, char *, char *));
EXTERN int	EEXPORT	elh_control	CARGS((ELM_HANDLE, int, long));
EXTERN char *	EEXPORT	elm_date_encode	CARGS((char *, time_t));
EXTERN time_t	EEXPORT	elm_date_decode	CARGS((char *, char *));
EXTERN int	EEXPORT	elm_decode	CARGS((ELM_CONST char *,char *,char *,int *,int *,char *));
EXTERN int		elm_event	CARGS((unsigned int, void (*)(), int));
EXTERN ELM_ULONG EEXPORT	elm_fexpdate	CARGS((ELM_CONST char *));
EXTERN void	EEXPORT	elm_free_array	CARGS((char **, int));
EXTERN char *	EEXPORT	elm_gencode	CARGS((ELM_CONST char *, ELM_ULONG));
EXTERN char * 	EEXPORT	elm_genkey	CARGS((ELM_CONST char *, char *, int, int, char *, int, int));
EXTERN char * 	EEXPORT	elm_genmultikey	CARGS((ELM_CONST char **, char *feature, int *nlic_set, int nset, int xdays, char *vdata, int domain, int options));
EXTERN char *	EEXPORT	elm_genredkey	CARGS((ELM_CONST char **, char *, int, int, char *, int, int));
EXTERN int	EEXPORT	elh_getinfo	CARGS((ELM_HANDLE, struct elm_info *, ELM_CONST char *, int));
EXTERN int	EEXPORT	elm_getkey	CARGS((ELM_CONST char *, char *, char *, char *));
EXTERN elm_keyentry_t * EEXPORT elm_getkeyfile	CARGS((ELM_CONST char *));
EXTERN int  	EEXPORT	elm_getlicense	CARGS((ELM_CONST char *, ELM_ULONG, ELM_ULONG));
EXTERN int	EEXPORT	elh_getlicense	CARGS((ELM_HANDLE, ELM_CONST char *, ELM_ULONG, ELM_ULONG));
EXTERN int	EEXPORT	elm_getservers	CARGS((struct elm_server *, int nservers, ELM_CONST char *feature, ELM_CONST char *keydir));
EXTERN char *		elm_getstr	CARGS((char **, int, char *));
EXTERN int	EEXPORT	elh_getulist	CARGS((ELM_HANDLE,struct elm_user *, ELM_CONST char *,int));
EXTERN int	EEXPORT	elh_getuser	CARGS((ELM_HANDLE, char *, char *, char *));
EXTERN int	EEXPORT	elm_getport	CARGS((char *));
EXTERN int	EEXPORT	elm_heartbeat	CARGS((unsigned int, void (*)()));
EXTERN int		elm_xtheartbeat	CARGS((XTAPPCONTEXT, unsigned int, void (*)(int, char *)));
EXTERN int		elm_xvheartbeat	CARGS((unsigned long,unsigned int, void (*)(int, char *)));
EXTERN char		elm_host[];
EXTERN ELM_ULONG EEXPORT	elm_hostaddr	CARGS((ELM_CONST char *));
EXTERN ELM_ULONG 	elm_hostcode;
EXTERN char * 	EEXPORT	elm_hostid	CARGS((ELM_CONST char *, ELM_ULONG));
EXTERN ELM_ULONG EEXPORT	elm_hostnetaddr	CARGS((ELM_CONST char *));
#ifdef _WINDOWS
EXTERN char		elm_icon_description[];
EXTERN char		elm_icon_name[];
#endif
EXTERN int	EEXPORT	elm_init	CARGS((char *, ELM_CONST char *, ELM_CONST char *, int (*)()));
EXTERN ELM_ULONG EEXPORT	elm_iptolong	CARGS((ELM_CONST char *));
EXTERN ELM_ULONG EEXPORT	elm_kexpdate	CARGS((ELM_CONST char *));
EXTERN int  	EEXPORT	elm_keydecode	CARGS((ELM_CONST char *, char **, char *, int *, int *, char *, char *domain));
EXTERN char		elm_keydir[];
EXTERN void  	EEXPORT	elm_longtoip	CARGS((ELM_ULONG, char *));
EXTERN char * 	EEXPORT	elm_message	CARGS((ELM_CONST char *, ELM_CONST char *, int));
EXTERN char *	EEXPORT	elm_mkcode	CARGS((ELM_CONST char *, ELM_CONST char *, ELM_CONST char *, ELM_CONST char *, int));
EXTERN int EEXPORT	elm_mkeydecode	CARGS((ELM_CONST char *key, char **hostids, char *feature, int *xdays, int *nlic_set, int *nsets, char *vdata, char *domain));
EXTERN char *	EEXPORT	elm_mkstr	CARGS((ELM_CONST char *));
EXTERN char * 	EEXPORT	elm_nngencode	CARGS((void));
EXTERN int	EEXPORT	elm_nnvalid	CARGS((char *, int (*)()));
EXTERN long	EEXPORT	elh_owner	CARGS((ELM_HANDLE));
EXTERN char		elm_object[];
EXTERN ELM_UINT		elm_port;
EXTERN int		elm_putkey	CARGS((ELM_CONST char *, ELM_CONST char *, ELM_CONST char *));
EXTERN int 	EEXPORT	elm_putkeyfile	CARGS((ELM_CONST char *, struct elm_keyentry *));
EXTERN char *	EEXPORT	elm_getrelease	CARGS((void));
EXTERN int	EEXPORT	elh_retlicense	CARGS((ELM_HANDLE, ELM_CONST char *, int));
EXTERN char		elm_salt[];
EXTERN char		elm_service[];
EXTERN int 	EEXPORT	elh_setport	CARGS((ELM_HANDLE, int, char *));
EXTERN int	EEXPORT	elh_setsalt	CARGS((ELM_HANDLE, char *));
EXTERN int 	EEXPORT	elh_setuser	CARGS((ELM_HANDLE, ELM_CONST char *, ELM_CONST char *, ELM_CONST char *));
EXTERN ELM_UINT		elm_sleep	CARGS((ELM_UINT));
EXTERN char		elm_srelease[];
EXTERN void	EEXPORT	elm_strncpy	CARGS((char *, ELM_CONST char *, int));
EXTERN int	EEXPORT	elm_val_feature	CARGS((ELM_CONST char *));
EXTERN int	EEXPORT	elm_val_licenses CARGS((int));
EXTERN int	EEXPORT	elm_val_vendordata CARGS((ELM_CONST char *));
EXTERN int	EEXPORT	elm_valid	CARGS((ELM_CONST char *, ELM_CONST char *feature, int (*fnc)()));
EXTERN ELM_ULONG EEXPORT	elm_vid	CARGS((void));
EXTERN int	EEXPORT	elm_winsockinit	CARGS((void));
EXTERN int	EEXPORT	elm_getstatus	CARGS((void));
EXTERN long	EEXPORT	elm_getvar	CARGS((void));
EXTERN void	EEXPORT	elm_setvar	CARGS((long));

EXTERN char	EEXPORT	elm_getversion	CARGS((void));/* Version ID 'A', 'B', 'C', or 'D' */
EXTERN int		elm_setversion	CARGS((char));
#endif
