static char *rcsid="$Header: /usr/local/src/nvrToMIT/RCS/extImageIo.c,v 1.9 1995/11/16 22:23:52 russ Exp $";
/*
 * Copyright (c) 1992-1995 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	ExtImageIo.c - NVR image I/O library interface
 *
 * DESCRIPTION
 *	implements the I/O interface to image files.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/extImageIo.c,v 1.9 1995/11/16 22:23:52 russ Exp $
 */

#include "nvrString.h"
#include "nvrReason.h"
#include "nvrBitTest.h"
#include "nvrDebug.h"
#include "nvrAlloc.h"
#include "process.h"
#include "extImageIoI.h"
#include "extImageProto.h"
#include "nvrImageI.h"

/*
 *			PRIVATE ROUTINES
 *         Support Functions for External File Readers
 */


/*
 * Initialize the MemImage with the given parameters and return
 * the pointer to the (calculated) next pixel set.  This
 * return value may be ignored if the image is interleaved
 */
static UChar *
extInitMemImage(MemImagePtr image,
	     UChar *pixels,
	     int channelName,
	     ColorSpace color,
	     int width, int height,
	     int nextPixel,
	     int nextLine,
	     MemImagePtr field2Image)
{
    image->pixels = pixels;
    image->size.width = width;
    image->size.height = height;
    image->nextPixel = nextPixel;
    image->nextLine = nextLine;
    switch(color) {
    case ColorYC:	 image->u.ycName = channelName; break;
    case ColorRGB:	 image->u.rgbName = channelName; break;
    case ColorRGBLookup: image->u.rgbName = channelName; break;
    default: break;
    }
    image->u.anyName = channelName;
    image->field2Image = field2Image;
    return(image->pixels + (height*nextLine));
}

static CompositeMemImagePtr
extNewCompositeMemImage(int dataSize,
		     int headerSize,
		     int nImage,
		     SampleType sample,
		     ColorSpace color,
		     InterlaceLayoutType iLayout)
{
    CompositeMemImagePtr newComp;
    int nAllocatedImages = nImage;

    if (iLayout)
	nAllocatedImages *= 2;
    newComp = (CompositeMemImagePtr)
        InsecureAlloc(dataSize
		      + sizeof(CompositeMemImage)
		      + sizeof(MemImage) * nAllocatedImages
		      + headerSize);

    if (newComp == NULL)
	return(NULL);
    newComp->pixelBytes = dataSize;
    newComp->nImage = nImage;
    newComp->images = (MemImagePtr)(newComp+1);
    if (iLayout)
	newComp->field2Images = newComp->images + nImage;
    else
	newComp->field2Images = NULL;
    if (headerSize) {
	newComp->header = (UChar *)(newComp->images + nAllocatedImages);
	newComp->pixels = newComp->header + headerSize;
    } else {
	newComp->header = NULL;
	newComp->pixels = (UChar *)(newComp->images + nAllocatedImages);
    }
    newComp->sample = sample;
    newComp->colorSpace = color;
    newComp->priv = NULL;
    return(newComp);
}

static CompositeMemImagePtr
createExtRgbImage(ExtImageInfoPtr info,
		  InternalReasonPtr *pReason)
{
    char	*errFunc = "createExtRgbImage";
    int		nDataChannels,
		nSubImages,
		firstOff,
		secondOff,
		thirdOff,
		alphaOff,
		interlaceMult = (info->iLayout == NotInterlaced) ? 1 : 2;
    long	nextPixel,
		nextLine,
		imageSize,
		width = info->width,
		height = info->height;
    UChar	*field1Pixels,
		*field2Pixels;
    CompositeMemImage *newComp;
    MemImagePtr field2Images;

    /*
     * nDataChannels describes the number of data channels that will be read in,
     *	*not* the number that will be available from the CompositeMemImage
     */
    nDataChannels = 0;

    /*
     * nSubImages describes the number of data channels available
     *	via the  CompositeMemImage
     */
    nSubImages = 0;

    /* relative offsets in memory for the nDataChannels */
    firstOff = secondOff = thirdOff = alphaOff = -1;

    field1Pixels = field2Pixels = NULL;
    nextPixel = nextLine = -1;
    imageSize = 0;

    
    if (info->color != ColorRGB) {
	*pReason = CreateReason("%s: bad colorspace '%s'\n", errFunc,
				EnumToName(info->color,ExtColorSpaceNames));
	return(NULL);
    }
    if (info->sample != Sample4_4_4) {
	*pReason = CreateReason("%s: bad sample type '%s'\n", errFunc,
				EnumToName(info->sample,ExtSampleTypeNames));
	return(NULL);
    }
    if (info->sOrg != SI_Mixed ) {
	*pReason =
		CreateReason("%s: cannot create discrete component subimages\n",
			     errFunc);
	return(NULL);
    }
    if (info->linePadding != NoPadding) {
	*pReason = CreateReason("%s: cannot create line-padded images\n",
				errFunc);
	return(NULL);
    }

    /*
     * we will not make the alpha channel available through
     *	the CompositeMemImage
     */
    nSubImages = 3;
    
    switch(info->pLayout) {
    case Pixel1230:
	nDataChannels = 4; firstOff = 0; secondOff = 1; thirdOff = 2;
	alphaOff = 3;
	break;
    case Pixel0123:
	nDataChannels = 4; firstOff = 1; secondOff = 2; thirdOff = 3;
	alphaOff = 0;
	break;
    case Pixel3210:
	nDataChannels = 4; firstOff = 2; secondOff = 1; thirdOff = 0;
	alphaOff = 3;
	break;
    case Pixel0321:
	nDataChannels = 4; firstOff = 3; secondOff = 2; thirdOff = 1;
	alphaOff = 0;
	break;
	
    case Pixel123:
	nDataChannels = 3; firstOff = 0; secondOff = 1; thirdOff = 2;
	break;
    case Pixel321:
	nDataChannels = 3; firstOff = 2; secondOff = 1; thirdOff = 0;
	break;
    case Pixel132:
	nDataChannels = 3; firstOff = 0; secondOff = 2; thirdOff = 1;
	break;
    case Pixel312:
	nDataChannels = 3; firstOff = 1; secondOff = 2; thirdOff = 0;
	break;
    default:
	*pReason = CreateReason("bad pixel layout '%s'\n",
				EnumToName(info->pLayout, ExtPixelLayoutNames));
	return(NULL);
    }
    
    imageSize = width * height * nDataChannels * interlaceMult;
    newComp = extNewCompositeMemImage(imageSize,   /* total frame size */
				   0,              /* header size */
				   nSubImages,     /* 3 channels */
				   Sample4_4_4,
				   ColorRGB,
				   info->iLayout);
    if (newComp == NULL) {
	*pReason = &IRBadAllocation;
	return(NULL);
    }
    newComp->size.width = width;
    newComp->size.height = height;
    
    switch (info->iLayout) {
    case NotInterlaced:
	field2Images = NULL;
	field1Pixels = newComp->pixels;
	nextPixel = nDataChannels;
	nextLine = width * nDataChannels;
	break;

    case InterlacedByLine:
	field1Pixels = newComp->pixels + width * nSubImages;
	field2Pixels = newComp->pixels;
	field2Images = newComp->field2Images;
	nextLine = width * nDataChannels * 2;
	nextPixel = nDataChannels;
	break;

    case InterlacedByPictEven:
	field1Pixels = newComp->pixels;
	field2Pixels = newComp->pixels + width * height * nSubImages;
	field2Images = newComp->field2Images;
	nextLine = width * nDataChannels;
	nextPixel = nDataChannels;
	break;

    case InterlacedByPictOdd:
	field2Pixels = newComp->pixels + width * height * nSubImages;
	field1Pixels = newComp->pixels;
	field2Images = newComp->field2Images;
	nextLine = width * nDataChannels;
	nextPixel = nDataChannels;
	break;

    default:
	*pReason = CreateReason("%s: bad interlace type '%s'\n", errFunc,
				EnumToName(info->iLayout,ExtInterlaceLayoutNames));
	return(NULL);
    }
    
    if (info->linesReversed) {
	if (field2Images) {
	    field2Pixels += (imageSize - nextLine);
	}
	field1Pixels += (imageSize - nextLine);
	nextLine = - nextLine;
	nextPixel = - nextPixel;
    }
    
    if(field2Images) {
	(void) extInitMemImage(&newComp->field2Images[RgbRed],
			    field2Pixels + firstOff, /* offset of 0 */
			    RgbRed, ColorRGB,
			    width, height, /* width, height */
			    nextPixel, /* next pixel */
			    nextLine, /* next line */
			    NULL); /* field 2 */
	(void) extInitMemImage(&newComp->field2Images[RgbGreen],
			    field2Pixels + secondOff, /* offset of 1 */
			    RgbGreen, ColorRGB,
			    width, height, /* width, height */
			    nextPixel, /* next pixel */
			    nextLine, /* next line */
			    NULL); /* field 2 */
	(void) extInitMemImage(&newComp->field2Images[RgbBlue],
			    field2Pixels + thirdOff, /* offset of 2 */
			    RgbBlue, ColorRGB,
			    width, height, /* width, height */
			    nextPixel, /* next pixel */
			    nextLine, /* next line */
			    NULL);  
	/* field 2 */
    }
    
    /*
     * Note that we create it interleaved...
     */
    (void) extInitMemImage(&newComp->images[RgbRed],
			field1Pixels + firstOff,
			RgbRed, ColorRGB,
			width, height, /* width, height */
			nextPixel, /* next pixel */
			nextLine, /* next line */
			field2Images
			? &field2Images[RgbRed] : NULL); /* field 2 */
    (void) extInitMemImage(&newComp->images[RgbGreen],
			field1Pixels + secondOff,
			RgbGreen, ColorRGB,
			width, height, /* width, height */
			nextPixel, /* next pixel */
			nextLine, /* next line */
			field2Images
			? &field2Images[RgbGreen] : NULL); /* field 2 */
    (void) extInitMemImage(&newComp->images[RgbBlue],
			field1Pixels + thirdOff,
			RgbBlue, ColorRGB,
			width, height, /* width, height */
			nextPixel, /* next pixel */
			nextLine, /* next line */
			field2Images
			? &field2Images[RgbBlue] : NULL); /* field 2 */
    
    info->type = IFT_RawRgb;
    return(newComp);
    
}

static CompositeMemImagePtr
createExtYcImage(ExtImageInfoPtr info,
		 InternalReasonPtr *pReason)
{
    char	*errFunc = "createExtYcImage";
    int width  = info->width;
    int height = info->height;
    CompositeMemImagePtr newComp;
    int lumaSize, chromaSize, imageSize;
    int chromaWidth, chromaHeight;
    UChar *nextPixels;
    boolean cbIsFirst;

    if (info->color != ColorYC) {
	*pReason = CreateReason("%s: bad colorspace '%s'\n", errFunc,
				EnumToName(info->color,ExtColorSpaceNames));
	return(NULL);
    }
    if (info->sOrg != SI_Separate ) {
	*pReason = CreateReason("%s: cannot create mixed component subimages\n",
				errFunc);
	return(NULL);
    }
    if (info->linePadding != NoPadding) {
	*pReason = CreateReason("%s: cannot create line-padded images\n",
				errFunc);
	return(NULL);
    }
    if (info->linesReversed != False) {
	*pReason = CreateReason("%s: cannot create line-reversed images\n",
				errFunc);
	return(NULL);
    }
    if (info->iLayout != NotInterlaced) {
	*pReason = CreateReason("%s: cannot create interlaced YC images.\n",
				errFunc);
	return(NULL);
    }

    lumaSize = width * height;
    switch(info->sample) {
    case Sample4_2_0: chromaWidth = width / 2; chromaHeight = height / 2; break;
    case Sample4_2_2: chromaWidth = width / 2; chromaHeight = height; break;
    case Sample4_4_4: chromaWidth = width; chromaHeight = height; break;
    default:
	*pReason = CreateReason("%s: bad sample type '%s'\n", errFunc,
				EnumToName(info->sample,ExtSampleTypeNames));
	return(NULL);
    }
    chromaSize = chromaWidth * chromaHeight;
    imageSize = lumaSize + 2*chromaSize;

    newComp = extNewCompositeMemImage(imageSize,	/* total frame size */
				      0,
				      3,		/* 3 channels */
				      info->sample,
				      ColorYC,
				      NotInterlaced);
    if (newComp == NULL) {
	*pReason = &IRBadAllocation;
	return(NULL);
    }
    newComp->size.width = width;
    newComp->size.height = height;

    switch (info->pLayout) {
    case Pixel123:
    case Pixel132:
	break;
    default:
	*pReason =
          CreateReason("%s: cannot create images of pixel organization '%s'\n",
		       errFunc, EnumToName(info->pLayout, ExtPixelLayoutNames));
	return(NULL);
    }

    nextPixels = extInitMemImage(&newComp->images[0],
			newComp->pixels,
			YcLuma, ColorYC,
			width, height,			/* width, height */
			1,				/* next pixel */
			width,				/* next line */
		        NULL);				/* field 2 */

    cbIsFirst = (info->pLayout == Pixel132);
    nextPixels = extInitMemImage(&newComp->images[1],
			nextPixels,
			cbIsFirst ? YcCb : YcCr, ColorYC,
			chromaWidth, chromaHeight,	/* width, height */
			1,				/* next pixel */
			chromaWidth,			/* next line */
		        NULL);				/* field 2 */
    (void) extInitMemImage(&newComp->images[2],
			nextPixels,
			cbIsFirst ? YcCr : YcCb, ColorYC,
			chromaWidth, chromaHeight,	/* width, height */
			1,				/* next pixel */
			chromaWidth,			/* next line */
		        NULL);				/* field 2 */
    info->type = cbIsFirst ? IFT_RawYCbCr : IFT_RawYCrCb;
    return(newComp);
}

static InternalReasonPtr
initExternConnection(PrivExtImageIoPtr xp, char *path, FileContentType fType)
{
    ImageReasonPtr reason;
    Argvec argv;
    char buffer[512];
    ExtParseReadConfigPtr rp = &xp->config.u.read.parse;

    memset((char *)&argv,0,sizeof(argv));
    sprintf(buffer,"%s/bin/%s",EnvironmentValue("NVRHOME"),NvrExternProcess);
    argv.path = buffer;
    AddArgv(&argv,NvrExternProcess,False);
    AddArgv(&argv,"-file",False);
    AddArgv(&argv,path,True);
    AddArgv(&argv,"-type",False);
    AddArgv(&argv,EnumToName(fType, AllFileTypes),False);
    if (rp->paramMask & XFP_ResampleFpsMask) {
	AddArgv(&argv,"-changefps",False);
	AddFmtArgv(&argv,"%f",rp->resampleFps);
    }
    if (rp->paramMask & XFP_SkipMask) {
	AddArgv(&argv,"-skip",False);
	AddFmtArgv(&argv,"%d",rp->nSkip);
    }
    if (rp->paramMask & XFP_NframesMask) {
	AddArgv(&argv,"-nframes",False);
	AddFmtArgv(&argv,"%d",rp->nFrames);
    }
    if (rp->paramMask & XFP_PrefixMask) {
	AddArgv(&argv,"-prefix",False);
	AddFmtArgv(&argv,"%s",rp->prefix);
    }
    if (rp->paramMask & XFP_SuffixMask) {
	AddArgv(&argv,"-suffix",False);
	AddFmtArgv(&argv,"%s",rp->suffix);
    }
    if (rp->paramMask & XFP_NdigitsMask) {
	AddArgv(&argv,"-ndigits",False);
	AddFmtArgv(&argv,"%d",rp->ndigits);
    }
    if (rp->paramMask & XFP_StartMask) {
	AddArgv(&argv,"-start",False);
	AddFmtArgv(&argv,"%d",rp->start);
    }
    if (rp->paramMask & XFP_StopMask) {
	AddArgv(&argv,"-stop",False);
	AddFmtArgv(&argv,"%d",rp->stop);
    }
    WhenDebugEnabled(
        PrintArgv(&argv);
    )
   
    if (reason = ExtInitProto(&xp->procCtl, &argv)) {
	goto done;
    }
    reason = ExtReadProto(xp->procCtl.fpError, &xp->imageInfo);

done:
    FreeArgv(&argv);
    return(reason);
}

static InternalReasonPtr
nullOpen(ExtImageIoPtr xp, char *path)
{
    return(CreateReason("null \"open\" method\n"));
}

static InternalReasonPtr
nullCreate(ExtImageIoPtr xp, char *path)
{
    return(CreateReason("null \"create\" method\n"));
}

static void
nullClose(ExtImageIoPtr xp)
{
    return;
}

static InternalReasonPtr
nullConfig(ExtImageIoPtr xp, ExtImageConfigPtr cp, ExtImageConfigType ct)
{
    return(CreateReason("null \"config\" method\n"));
}

static InternalReasonPtr
nullRead(ExtImageIoPtr xp, CompositeMemImagePtr memPtr)
{
    return(CreateReason("null \"read\" method\n"));
}

static InternalReasonPtr
nullWrite(ExtImageIoPtr xp, CompositeMemImagePtr memPtr)
{
    return(CreateReason("null \"write\" method\n"));
}

static InternalReasonPtr
nullSeek(ExtImageIoPtr xp, int nItems, SeekWhere where)
{
    return(CreateReason("null \"seek\" method\n"));
}

static InternalReasonPtr
nullGetInfo(ExtImageIoPtr xp, ExtImageInfoPtr info)
{
    return(CreateReason("null \"getinfo\" method\n"));
}

static void
nullFree(ExtImageIoPtr xp)
{
    return;
}

static InternalReason NotOpen =
{ "External file interface is not open\n" };

static InternalReason AlreadyOpen =
{ "External file interface is already open\n" };

		/* "getinfo" method */
static InternalReasonPtr
extGetInfoImageIo(ExtImageIoPtr io, ExtImageInfoPtr info)
{
    char *errFunc = "external file Getinfo";
    PrivExtImageIoPtr xp = (PrivExtImageIoPtr) io;

    if (xp->isOpen != True) {
	return (&NotOpen);
    }
    *info = xp->imageInfo;
    return(NULL);
}

/*
 *		"config" method
 * Any configuring must be done before opening the file.
 * The configuring is advisory only, the NvrXInfo structure returned
 *	from the open is the one that is valid
 */
static InternalReasonPtr
extConfigImageIo(ExtImageIoPtr io, ExtImageConfigPtr config,
		 ExtImageConfigType cType)
{
    char *errFunc = "external file Config";
    PrivExtImageIoPtr xp = (PrivExtImageIoPtr) io;

    if (xp->isOpen == True) {
	return (&AlreadyOpen);
    }
    xp->config.cType = cType;
    switch(cType) {
    case XC_Read:
	xp->config.u.read = config->u.read;
	return(NULL);
    case XC_Write:
	xp->config.u.write = config->u.write;
	return(NULL);
    default:
	return( CreateReason("Cannot configure External Image Io\n") );
    }
}

		/* "read" method */
static InternalReasonPtr
extReadImageIo(ExtImageIoPtr io, CompositeMemImagePtr image)
{
    char *errFunc = "external file Read";
    PrivExtImageIoPtr xp = (PrivExtImageIoPtr) io;
    int success;
    InternalReasonPtr reason;

    
    if (xp->isOpen != True) {
	return (&NotOpen);
    }
    if (NULL != (reason = ExtContinueProto(&xp->procCtl))) {
	return(reason);
    }
    if (   (   xp->imageInfo.frameCount > 0 
	    && xp->nImage++ >= xp->imageInfo.frameCount)
	|| EndOfBufferedFile(xp->procCtl.fpData)) {

	return (&IRPipelineEof);
    }

    success = BufferedReadData(xp->procCtl.fpData, (char *)image->pixels, 
			       image->pixelBytes);
    if (success < 0) {
	return (CreateSysReason("read failed on image %d\n", xp->nImage));
    }
    return(NULL);
}

		/* "close" method */
static void
extCloseImageIo(ExtImageIoPtr io)
{
    PrivExtImageIoPtr xp = (PrivExtImageIoPtr) io;

    if (xp->isOpen != True) {
	return;
    }
    ExtEndProto(&xp->procCtl);
    xp->isOpen = False;
    return;
}

		/* "open" method */
static InternalReasonPtr
extOpenImageIo(ExtImageIoPtr io, char *path)
{
    char *errFunc = "external file Open";
    InternalReasonPtr reason;
    FileInfo fileInfo;
    PrivExtImageIoPtr xp = (PrivExtImageIoPtr) io;

    if (xp->isOpen) {
	return(NULL);
    }
    if (reason = GetFileContentInfo(path, &fileInfo)) {
	return(reason);
    }
    if (fileInfo.type == FT_Unknown) {
	return (CreateReason("%s: cannot determine file type\n",errFunc));
    }
    xp->fType = fileInfo.type;
    if (reason = initExternConnection(xp, path, xp->fType)) {
	return(reason);
    }
    xp->isOpen = True;
    return (reason);
}


		/* "free" method */
static void
extFreeImageIo(ExtImageIoPtr xp)
{
    if (xp) {
	free(xp);
    }
}





/*
 *			PUBLIC ROUTINES
 *		    External File Interface
 */
ExtImageIoPtr
CreateExtImageIo(InternalReasonPtr *pReason)
{

    PrivExtImageIoPtr xp = (PrivExtImageIoPtr)
	AllocNamedMem(sizeof(PrivExtImageIo), "PrivExtImageIo", pReason);
    if (xp == NULL) {
	return(NULL);
    }
    memset((char *)xp,0,sizeof(PrivExtImageIo));

    /* initializations */
    xp->imageInfo.iLayout = UnknownInterlace;

    xp->pub.open = extOpenImageIo; xp->pub.fileOpsAvail |= XFopen;
    xp->pub.create = nullCreate;
    xp->pub.close = extCloseImageIo ;xp->pub.fileOpsAvail |= XFclose;
    xp->pub.config = extConfigImageIo; xp->pub.fileOpsAvail |= XFconfig;
    xp->pub.read = extReadImageIo; xp->pub.fileOpsAvail |= XFread;
    xp->pub.write = nullWrite;
    xp->pub.seek = nullSeek;
    xp->pub.getinfo = extGetInfoImageIo; xp->pub.fileOpsAvail |= XFgetinfo;
    xp->pub.free = extFreeImageIo; xp->pub.fileOpsAvail |= XFfree;
    
    return((ExtImageIoPtr)xp);
}


CompositeMemImagePtr
NewExtImage(ExtImageInfoPtr info,
	    InternalReasonPtr *pReason)
{
    if (info->color == ColorRGB) {
	return( createExtRgbImage(info, pReason) );
    }
    if (info->color == ColorYC) {
	return( createExtYcImage(info, pReason) );
    }
    return( NULL );
}

