/* $Header: /usr/local/src/nvrToMIT/RCS/extImageIo.h,v 1.3 1995/11/16 22:48:43 russ Exp $ */
/*
 * (c) Copyright 1992-1995 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	extImageIo.h - public interface to external file 
 *
 * DESCRIPTION
 *	Implements an external file reader
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/extImageIo.h,v 1.3 1995/11/16 22:48:43 russ Exp $
 */
#ifndef _EXTIMAGEIO_H_
#define _EXTIMAGEIO_H_

#include "nvrImage.h"

/*
 * bits for fileOpsAvail
 */
enum {
    XFconfig	= 0x001,
    XFopen	= 0x002,
    XFcreate	= 0x004,
    XFclose	= 0x008,
    XFread	= 0x010,
    XFwrite	= 0x020,
    XFseek	= 0x040,
    XFfree      = 0x080,
    XFgetinfo	= 0x100
};

/*
 * RGBRGBRGB			123
 * YYY...CrCrCr...CbCbCb...	123
 * XBGRXBGR			0321
 * YCbYCrYCbYCr			132
 * CbYCrYCbYCrY			312
 */
typedef enum {
    PixelUnspecified,
    PixelUnknown,
    Pixel1230,
    Pixel0123,
    Pixel3210,
    Pixel0321,
    Pixel123,
    Pixel321,
    Pixel132,
    Pixel312
} PixelLayoutType;

/*
 * Specifies how the image linesBytes are padded out
 *	       NoPadding => no padding
 *	 FourBytePadding => lines are padded to 4 byte boundaries
 *	EightBytePadding => lines are padded to 8 byte boundaries
 */
typedef enum {
    PaddingUnspecified,
    PaddingUnknown,
    NoPadding,
    FourBytePadding,
    EightBytePadding
} PaddingType;

typedef enum {
    XC_Unspecified,
    XC_Read,
    XC_Write
} ExtImageConfigType;

typedef enum {
    SI_Unspecified,
    SI_Unknown,
    SI_Mixed,
    SI_Separate
} SubImageOrg;

typedef struct {
    ImageFileType
		type;
    long	width;
    long	height;
    ColorSpace	color;		/* color space - Color{YC,RGB,etc...} */
    SampleType	sample;		/* sample type - 420, 422, 444  */
    InterlaceLayoutType
		iLayout;	/* whether the image is interlaced or not */
    PixelLayoutType
		pLayout;
    boolean	linesReversed;
    PaddingType	linePadding;
    SubImageOrg	sOrg;
    long	frameCount;
    float	framesPerSecond;
} ExtImageInfo, *ExtImageInfoPtr;

enum {
    XFP_SkipMask       = (1 << 0),
    XFP_NframesMask    = (1 << 1),
    XFP_ResampleFpsMask  = (1 << 2),
    XFP_PrefixMask     = (1 << 3),
    XFP_SuffixMask     = (1 << 4),
    XFP_NdigitsMask    = (1 << 5),
    XFP_StartMask      = (1 << 6),
    XFP_StopMask       = (1 << 7)
};
    
typedef struct {
    ColorSpace	color;		/* color space - Color{YC,RGB,etc...} */
    SampleType	sample;		/* sample type - 420, 422, 444  */
    InterlaceLayoutType iLayout;  /* whether the image is interlaced or not */
    boolean	linesReversed;
    PaddingType	linePadding;
    PixelLayoutType pLayout;
} ExtImageReadConfig, *ExtImageReadConfigPtr;

typedef struct {
    long paramMask;
    long nSkip;		/* number of frames to skip */
    long nFrames;		/* number of input frames to read */
    float resampleFps;	/* change frame rate */
    char *prefix;
    char *suffix;
    int ndigits;
    int start;
    int stop;
} ExtParseReadConfig, *ExtParseReadConfigPtr;

typedef union {
    ExtImageReadConfig image;
    ExtParseReadConfig parse;
} ExtReadConfig, *ExtReadConfigPtr;

typedef ExtImageInfo ExtWriteConfig, *ExtWriteConfigPtr;

typedef struct {
    ExtImageConfigType cType;
    union {
	ExtReadConfig  read;
	ExtWriteConfig write;
    } u;
} ExtImageConfig, *ExtImageConfigPtr;

typedef struct _extImageIo ExtImageIo, *ExtImageIoPtr;

/*
 * This interface can currently only deal with
 *	single file paths. It cannot read implicit or
 *	explicit file descriptions
 */
struct _extImageIo {
    int fileOpsAvail;
    InternalReasonPtr
	(*config)(ExtImageIoPtr, ExtImageConfigPtr, ExtImageConfigType);
    InternalReasonPtr
	(*open)(ExtImageIoPtr, char *path);
    InternalReasonPtr
	(*create)(ExtImageIoPtr, char *path);
    void
	(*close)(ExtImageIoPtr);
    InternalReasonPtr
	(*read)(ExtImageIoPtr, CompositeMemImagePtr);
    InternalReasonPtr
	(*write)(ExtImageIoPtr, CompositeMemImagePtr);
    InternalReasonPtr
	(*seek)(ExtImageIoPtr, int nItems, SeekWhere where);
    InternalReasonPtr
	(*getinfo)(ExtImageIoPtr, ExtImageInfoPtr);
    void
	(*free)(ExtImageIoPtr);
    
};


CExternal ExtImageIoPtr
CreateExtImageIo(
    InternalReasonPtr *
);

CExternal CompositeMemImagePtr
NewExtImage(
    ExtImageInfoPtr, 
    InternalReasonPtr *
);

#endif /* _EXTIMAGEIO_H_ */
