static char *rcsid="$Header: /usr/local/src/nvrToMIT/RCS/extImageProto.c,v 1.6 1995/09/12 18:19:02 russ Exp $";
/*
 * Copyright (c) 1992-1995 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	extImageProto.c - handles interaction with file process
 *
 * DESCRIPTION
 *	implements the protocol with an external file reader
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/extImageProto.c,v 1.6 1995/09/12 18:19:02 russ Exp $
 */

#include "nvrTypes.h"
#include "process.h"
#include "osio.h"
#include "nvrReason.h"
#include "nvrImage.h"
#include "nvrBitTest.h"
#include "extImageIo.h"
#include "extImageProto.h"

#define InitCompareBitmask(b)						\
{	BM_ZERO((b));							\
	BM_SET((b),Nwidth);         BM_SET((b),Nheight);		\
	BM_SET((b),Ncolor);         BM_SET((b),Nsample);		\
	BM_SET((b),NiLayout);       BM_SET((b),NpLayout);		\
	BM_SET((b),NlinesReversed); BM_SET((b),NlinePadding);		\
	BM_SET((b),NframeCount);    BM_SET((b),NframesPerSecond);	\
	BM_SET((b),NsOrg);						}

#define LocalBufSize 1024

NameEnumPair ExtPixelLayoutNames[] = {
    { "Pixel123",	Pixel123     },
    { "Pixel0123",	Pixel0123    },
    { "Pixel123",	Pixel123     },
    { "Pixel3210",	Pixel3210    },
    { "Pixel0321",	Pixel0321    },
    { "Pixel321",	Pixel321     },
    { "Pixel132",	Pixel132     },
    { "Pixel312",	Pixel312     },
    { NULL,		PixelUnknown }
};

NameEnumPair ExtFileParamNames[] = {
    { "type",          XFP_Type },
    { "sample",        XFP_Sample },
    { "size",          XFP_Size },
    { "colorspace",    XFP_ColorSpace },
    { "ilayout",       XFP_ILayout },
    { "iLayout",       XFP_ILayout },
    { "playout",       XFP_PLayout },
    { "pLayout",       XFP_PLayout },
    { "linepadding",   XFP_LinePadding },
    { "subimageorg",   XFP_SubImageOrg },
    { "linesreversed", XFP_LinesReversed },
    { "nframes",       XFP_FrameCount },
    { "framerate",     XFP_FramesPerSec },
    { NULL,	       XFP_Unknown }
};

NameEnumPair ExtPaddingNames[] = {
    { "none",		NoPadding },
    { "0",		NoPadding },
    { "fourbyte",       FourBytePadding },
    { "4",	        FourBytePadding },
    { "eightbyte",      EightBytePadding },
    { "8",	        EightBytePadding },
    { NULL,		PaddingUnknown }
};

NameEnumPair ExtSubImageOrgNames[] = {
    { "mixed",		SI_Mixed },
    { "separate",	SI_Separate },
    { NULL,		SI_Unknown }
};

NameEnumPair ExtColorSpaceNames[] = {
    { "rgb",       ColorRGB },
    { "yc",        ColorYC },
    { "rgblookup", ColorRGBLookup },
    { NULL,        ColorUnknown }
};

NameEnumPair ExtSampleTypeNames[] = {
    { "4:2:0",	   Sample4_2_0 },
    { "4:1:1",	   Sample4_2_0 },
    { "4:2:2",	   Sample4_2_2 },
    { "4:4:4",     Sample4_4_4 },
    { "420",	   Sample4_2_0 },
    { "411",	   Sample4_2_0 },
    { "422",	   Sample4_2_2 },
    { "444",       Sample4_4_4 },
    { NULL,        SampleUnknown }
};

NameEnumPair ExtInterlaceLayoutNames[] = {
    { "pict",     InterlacedByPict },
    { "pictOdd",  InterlacedByPictOdd },
    { "pictEven", InterlacedByPictEven },
    { "True",     InterlacedByLine },
    { "true",     InterlacedByLine },
    { "line",     InterlacedByLine },
    { "false",    NotInterlaced },
    { "none",     NotInterlaced },
    { "no",       NotInterlaced },
    { NULL,       UnknownInterlace }
};

NameEnumPair ExtBooleanNames[] = {
    { "True",  True },
    { "False", False },
    { "true",  True },
    { "false", False },
    { "1",     True },
    { "0",     False },
    { "yes",   True },
    { "no",    False },
    { NULL,    False }
};

static InternalReason LineTooLong =
    { "line returned by external file process is longer than 1024.", 
      IR_BadValue };
static InternalReason NoStartProto =
    { "The first line from external file process must contain only 'NVR_START_PROTO'\n",
      IR_BadValue };

enum {
    Ntype,
    Nwidth,
    Nheight,
    Ncolor,
    Nsample,
    NiLayout,
    NpLayout,
    NlinesReversed,
    NlinePadding,
    NsOrg,
    NframeCount,
    NframesPerSecond
};

InternalReasonPtr
ExtInitProto(ExtProcCtlPtr cp, Argvec *pArgv)
{
    int pipeFdTo[2];
    int pipeFdFrom[2];
    int pipeFdError[2];
    int pid;
    InternalReasonPtr reason;

    if (   CreatePipe(pipeFdTo) < 0
	|| CreatePipe(pipeFdFrom) < 0
	|| CreatePipe(pipeFdError) < 0) {
	return( CreateSysReason("Can't create pipe\n") );
    }
    if ((pid = vfork()) < 0) {
	return (CreateSysReason("Can't fork\n"));
    }
    if (pid == 0) {	/* child */
	extern char *ProgramName;

	close(pipeFdTo[1]);
	close(pipeFdFrom[0]);
	close(pipeFdError[0]);

	dup2(pipeFdTo[0], 0);
	close(pipeFdTo[0]);
	dup2(pipeFdFrom[1], 1);
	close(pipeFdFrom[1]);
	dup2(pipeFdError[1], 2);
	close(pipeFdError[1]);

	
	execvp(pArgv->path, pArgv->argv);
	ProgramName = pArgv->path;
	Fatal("exec failed\n");
    }

    close(pipeFdTo[0]);
    close(pipeFdFrom[1]);
    close(pipeFdError[1]);

    cp->pid = pid;
    cp->fpControl = ReopenBufferedFileForWriting( pipeFdTo[1]);
    cp->fpData = ReopenBufferedFileForReading(pipeFdFrom[0]);
    cp->fpError = ReopenBufferedFileForReading(pipeFdError[0]);
    return(NULL);
} 

/*
 * If this function returns a reason, it is the responsibility of the caller
 *	to insure that the Argvec is freed
 */
static InternalReasonPtr
parseAsciiImageDesc(BufFileHndl fp, Argvec *argv)
{
    InternalReasonPtr reason = NULL;
    int c, commChar;
    char buffer[ LocalBufSize ], *p;
    int line = 1;


    
    if (BufferedReadLine(buffer, LocalBufSize, fp) == False) {
	return (CreateReason("The file contains no images\n"));
    }
    if (memcmp(buffer, StartProto, strlen(StartProto)) != 0) {
	return (CreateReason(
  "The first line read from the external file process must contain only '%s'\n\
      instead read: %s", StartProto, buffer));			     
    }
    AddArgv(argv, StartProto, False);	/* make AllocStringArgList happy */
    commChar = 0;
    /*
     * Break the file up into an argument vector.
     * All spaces are removed.
     * All words after a '#' and before a '\n' are ignored
     */
    p = buffer;
    while (reason == NULL) {
	c = GetBufferedChar(fp);
        if (c == EndOfFile)
	    goto endOfLine;
	if (p - buffer >= sizeof(buffer) - 1) {
	    if (line == 1)
		reason = &NoStartProto;
	    else
	        reason = &LineTooLong;
	    break;
	}
	
	switch(c) {
	case ' ':
	case '\t':
	    break;

	case '\n':
	case '\r':	/* for the mac */
	endOfLine:
	    if(p != buffer) {
		*p = '\0';
		line++;
		if (strcmp(buffer, EndProto) == 0) {
		    goto out;
		}
		AddArgv(argv, buffer, True);
	    }
	    commChar = 0;
	    p = buffer;
	    if (c == EndOfFile)
		goto out;
	    break;

	case '#':
	    commChar++;
	    break;

	default:
	    if (!commChar)
		*p++ = c;
	    break;
	}
    }

out:
    return(reason);
}


/*
 * Here we expect *only* the status to be returned. Any other
 *	data implies an error has occured
 */
InternalReasonPtr
ExtReadProto(BufFileHndl fp, ExtImageInfoPtr info)
{
    char *errFunc = "ExtReadProto";
    Argvec argv;
    StringArgListPtr args = NULL;
    InternalReasonPtr reason = NULL;
    char buffer[1024];
    int width, height, i;
    ImageFileType ift;
    float fps;
    InterlaceLayoutType iLayout;
    BitMask compare;
    BitMask attributes;
    ExtImageFileParam name;

    argv.length = 0;
    if (reason = parseAsciiImageDesc(fp, &argv)) {
	goto done;
    }

    if (argv.length == 0) {
	reason = CreateReason("%s: parse failure: no tokens present\n",
			      errFunc);
	goto done;
    }
    if ((args = AllocStringArgList(argv.index, argv.argv)) == NULL) {
	reason = &IRBadAllocation;
	goto done;
    }

    BM_ZERO(&attributes);
    InitCompareBitmask(&compare);

    for (i = args->index; reason == NULL && i < args->length; i++) {
	name = NameToEnum(args->list[i].name, ExtFileParamNames);

	switch (name) {
	case XFP_Type:
	case XFP_ColorSpace:
	case XFP_Sample:
	case XFP_Size:
	case XFP_ILayout:
	case XFP_PLayout:
	case XFP_LinePadding:
	case XFP_SubImageOrg:
	case XFP_LinesReversed:
	case XFP_FrameCount:
	case XFP_FramesPerSec:
	    if (args->list[i].value == NULL) {
		reason = CreateReason("%s: missing argument for \"%s\"\n",
				      errFunc, args->list[i].name);
	    }
	    break;
	
	default:
	    reason = CreateReason("%s: bad parse token \"%s\"\n",
				  errFunc, args->list[i].name);
	    break;
	}

	if (reason == NULL)
	switch (name) {
	case XFP_Type:
	    /*
	     * Note that the info->type does not change. from IFT_NvrIdx
	     * to the type listed here.  However, it does get checked for
	     * correctness.
	     */
	    ift = NameToEnum(args->list[i].value, NULL);
	    if (ift == IFT_Unspecified) {
		reason = CreateReason("%s: unknown file type: %s\n",
				      errFunc, args->list[i].value);
	    }
	    BM_SET(&attributes,Ntype);
	    break;

	case XFP_ColorSpace:
	    info->color = NameToEnum(args->list[i].value, 
					  ExtColorSpaceNames);
	    if (info->color == ColorUnknown) {
		reason = CreateReason("%s: unknown color space: %s\n",
				      errFunc, args->list[i].value);
	    }
	    BM_SET(&attributes,Ncolor);
	    break;
	    
	case XFP_Sample:
	    info->sample = NameToEnum(args->list[i].value, ExtSampleTypeNames);
	    if (info->sample == SampleUnknown) {
		reason = CreateReason("%s: unknown sampling: %s\n",
				      errFunc, args->list[i].value);
	    }
	    BM_SET(&attributes,Nsample);
	    break;

	case XFP_Size:
	    if (sscanf(args->list[i].value, "%dx%d", &width, &height) != 2
		&& sscanf(args->list[i].value, "%dX%d", &width, &height) != 2) {
                reason = CreateReason("%s: invalid image size: %s\n",
                                      errFunc, args->list[i].value);
		break;
	    }
	    info->width = width;
	    info->height = height;
	    BM_SET(&attributes,Nwidth);
	    BM_SET(&attributes,Nheight);
	    break;

	case XFP_FrameCount:
	    if (sscanf(args->list[i].value, "%d", &width) != 1) {
                reason = CreateReason("%s: bad frame count value: %s\n",
                                      errFunc, args->list[i].value);
		break;
	    }
	    info->frameCount = width;
	    BM_SET(&attributes,NframeCount);
	    break;

	case XFP_FramesPerSec:
	    if (sscanf(args->list[i].value, "%f", &fps) != 1) {
                reason = CreateReason("%s: bad fps value: %s\n",
                                      errFunc, args->list[i].value);
		break;
	    }
	    info->framesPerSecond = fps;
	    BM_SET(&attributes,NframesPerSecond);
	    break;
	
	case XFP_ILayout:
	    iLayout = NameToEnum(args->list[i].value, ExtInterlaceLayoutNames);
            if (iLayout == UnknownInterlace) {
                reason = CreateReason("%s: unknown interlace layout: %s\n",
                                      errFunc, args->list[i].value);
            }
	    info->iLayout = iLayout;
	    BM_SET(&attributes,NiLayout);
	    break;

	case XFP_PLayout:
	    info->pLayout = NameToEnum(args->list[i].value, ExtPixelLayoutNames);
            if (info->pLayout == PixelUnknown) {
                reason = CreateReason("%s: unknown pixel layout: %s\n",
                                      errFunc, args->list[i].value);
            }
	    BM_SET(&attributes,NpLayout);
	    break;

	case XFP_LinesReversed:
	    info->linesReversed = NameToEnum(args->list[i].value, ExtBooleanNames);
	    BM_SET(&attributes,NlinesReversed);
	    break;

	case XFP_LinePadding:
	    info->linePadding = NameToEnum(args->list[i].value, ExtPaddingNames);
	    if ( info->linePadding == PaddingUnknown ) {
		reason = CreateReason("%s: bad padding designation: %s\n",
				      errFunc, args->list[i].value);
		break;
	    }
	    BM_SET(&attributes,NlinePadding);
	    break;

	case XFP_SubImageOrg:
	    info->sOrg = NameToEnum(args->list[i].value, ExtSubImageOrgNames);
	    if ( info->sOrg == SI_Unknown ) {
		reason = CreateReason("%s: bad subimage organization: %s\n",
				      errFunc, args->list[i].value);
		break;
	    }
	    BM_SET(&attributes,NsOrg);
	    break;

	default:
	    break;
	}
    }
    
done:
    FreeArgv(&argv);
    if( args ) {
	FreeStringArgList(args);
    }
    if (   !reason 
	&& memcmp((char *)&attributes, (char *)&compare,
		  sizeof(BitMask)) != 0) {
	reason = CreateReason("%s: parse failure: not all needed tokens seen\n",
			     errFunc);
    }
    return(reason);
}

InternalReasonPtr
ExtContinueProto(ExtProcCtlPtr cp)
{
    char *errFunc = "ExtContinueProto";
    int errBytes;
    char errBuf[256];
    InternalReasonPtr reason;
    TimeSpec maxWait;

    memset(&maxWait,0,sizeof(maxWait));
    if (WaitForFileInput(FileNo(cp->fpError), &maxWait) == True) {
	BufferedStdRead(errBuf, 1, 255, cp->fpError);
	errBuf[255] = 0;
	return( CreateReason("%s: %s\n", errFunc, errBuf) );
    }
    return(NULL);
}

void
ExtEndProto(ExtProcCtlPtr cp)
{

    CloseBufferedFile(cp->fpData);
    CloseBufferedFile(cp->fpControl);
    CloseBufferedFile(cp->fpError);
    KillProcess(cp->pid);
    memset((char *)cp,0,sizeof(ExtProcCtl));
}
