/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	imageContent.c - figure out the content type of a file
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/imageContent.c,v 1.30 1995/11/16 01:50:16 russ Exp $
 */
#define NeedFunctionPrototypes 1

#include "nvrImageI.h"
#include "osio.h"
#include "imageIoI.h"
#include "fileAccess.h"
#include "nvrPlx.h"
#include "nvrQuickTime.h"

typedef struct {
    ULong sequenceHeaderCode;
    ULong width:12;
    ULong height:12;
    ULong pelAspectRatio:4;
    ULong pictureRate:4;
    ULong bitRate:18;
} MpegVideoHeader;

typedef union {
    MpegVideoHeader mpeg;
    nvrPlxMovieHdr plxMovie;
    char raw[1024];
} FileHeader;

/*
** The NVR_YUV file has an ascii header terminated by a single ^L
**	The minimum header is "nvr_yuv\0^L". The default header values are:
**	ColorSpace: yc; SampleRate: 420; Width: 320; Height: 240;
**		FrameCount: 0; FramesPerSecond: 30
**	any of the defaults can be overidden by uttering them in the header
*/
static void
NvrYuvHdr(ImageFileInfoPtr imageInfo, char *buffer, int sizeBuffer)
{
    char *cp,*cpN;
    int strCnt;

    imageInfo->width = 320;
    imageInfo->height = 240;
    imageInfo->framesPerSecond = 30.0;
    imageInfo->color = ColorYC;
    imageInfo->sample = Sample4_2_0;
    imageInfo->frameCount = 0;
    
    strCnt = 0;
    for(cpN = buffer; cpN < buffer + sizeBuffer && *cpN != '\014'; cpN++) {
	if( *cpN == '\n') {
	    *cpN = '\0';
	    strCnt++;
	}
    }
    if( *cpN != '\014' ) {
	return;
    }
    *cpN++ = '\0';
    imageInfo->hdrSize = cpN - buffer;
    imageInfo->contentSize -= imageInfo->hdrSize;
    /* seek past 'nvr_yuv' */
    for(cp = buffer; *cp; cp++) {
	;
    }
    cp++;
    while( strCnt-- ) {
	switch(*cp) {
	case 'W':
	    if( strncmp(cp,"Width:",6) == 0 ) {
		cp += 6;
		while( *cp == ' ' || *cp == '\t' ) cp++;
		imageInfo->width = atoi(cp);
	    }
	    break;
	case 'H':
	    if( strncmp(cp,"Height:",7) == 0 ) {
		cp += 7;
		while( *cp == ' ' || *cp == '\t' ) cp++;
		imageInfo->height = atoi(cp);
	    }
	    break;
	case 'S':
	    if( strncmp(cp,"SampleRate:",11) == 0 ) {
		cp += 11;
		while( *cp == ' ' || *cp == '\t' ) cp++;
		imageInfo->sample = NameToEnum(cp,SampleTypeNames);
	    }
	    break;
	case 'C':
	    if( strncmp(cp,"ColorSpace:",11) == 0 ) {
		cp += 11;
		while( *cp == ' ' || *cp == '\t' ) cp++;
		imageInfo->color = NameToEnum(cp,ColorSpaceNames);
	    }
	    break;
	case 'F':
	    if( strncmp(cp,"FrameCount:",11) == 0 ) {
		cp += 11;
		while( *cp == ' ' || *cp == '\t' ) cp++;
		imageInfo->frameCount = atoi(cp);
	    } else if( strncmp(cp,"FramesPerSecond:",16) == 0 ) {
		cp += 16;
		while( *cp == ' ' || *cp == '\t' ) cp++;
		imageInfo->framesPerSecond = atof(cp);
	    }
	    break;
	}
	while(*cp != '\0') cp++;
	cp++;
    }
    
}

static InternalReasonPtr
CantSeekReason(void)
{
    return CreateSysReason("Can't seek\n");
}

#define  ntohl(x)        (x)

static InternalReasonPtr
SvjHeader(FileHandle fd, ImageFileInfoPtr imageInfo)
{
    long size;
    struct {
        UShort id;                  /* 0xffe1 */
        UShort length;
        char   name[16];
        ULong   xsize;
        ULong   ysize;
        ULong   frames;
        ULong   table;
        ULong   initsize;
        ULong   pad[12-5];
    } header;

    /*
     *    The file header is actually a tail.
     */

    if (SeekFile(fd, -sizeof(header), SK_End) < 0)
        return( CantSeekReason() );

    size = sizeof(header);
    if (ReadData(fd, (char *)&header, size) != size)
        return( NULL );

    if(strncmp(header.name, "DVS100", 5) != 0)
        return( NULL );

    imageInfo->type = IFT_Svj;
    /*
     *    Make sure that the data is read in big endian format
     */

    imageInfo->color      = ColorRGB;
    imageInfo->interlaced = True;
    imageInfo->sample     = Sample4_2_2;
    imageInfo->width      = ntohl(header.xsize);
    imageInfo->height     = ntohl(header.ysize) >> 1;
    imageInfo->frameCount = ntohl(header.frames);
    if (imageInfo->height <= 240)
	imageInfo->framesPerSecond = 29.97;
    else
	imageInfo->framesPerSecond = 25.0;
    imageInfo->contentSize -= sizeof(header);
    imageInfo->hdrSize = sizeof(header);
    imageInfo->bitRate = (imageInfo->contentSize / imageInfo->frameCount)
		       * (imageInfo->framesPerSecond * 8.0);
    imageInfo->defined    = IFI_FrameCount | IFI_FrameRate | IFI_BitRate;

    return(NULL);
}

static void
NvrGetPlxMovieHdr(ImageFileInfoPtr imageInfo, nvrPlxMovieHdr *plxMovie)
{
    imageInfo->width = plxMovie->width;
    imageInfo->height = plxMovie->height;
    imageInfo->framesPerSecond = (float)plxMovie->fps;
    imageInfo->frameCount = plxMovie->frames;
    imageInfo->contentSize =
	plxMovie->indexbuf + plxMovie->frames * sizeof(long);
    imageInfo->qFactor = plxMovie->qfactor;
    imageInfo->defined = IFI_FrameCount | IFI_FrameRate | IFI_QFactor;
    return;
}

typedef enum {
    CBadCmd,
    CVideo,
    CType,
    CFrames,
    CFps,
    CWidth,
    CHeight,
    CName,
    CEnd
} SunVideoCmdName;

static InternalReasonPtr
SunVideoReadHdr(ImageFileInfoPtr imageInfo, char *buffer,
		int bufferSize)
{

    ImageReasonPtr	reason = &IRBadFileHeader;
    boolean		inVideo = False;
    char		**Strs, **strs;
    char		*eq, *cp;
    static NameEnumPair SunVideoNames[] = {
	{ "VIDEO",		CVideo  },
	{ "type",		CType   },
	{ "frames",		CFrames },
	{ "fps",		CFps,   },
	{ "Width",		CWidth  },
	{ "Height",		CHeight },
	{ "name",		CName   },
	{ "END",		CEnd    },
	{ NULL,			CBadCmd }
    };


    buffer[bufferSize-1] = '\0';
    if ((Strs = (char **)AllocStringList(buffer,'\n')) == NULL) {
	return(reason);
    }
    strs = Strs;

    if (strcmp(*strs,"VCR") != 0) {
	goto hdrDone;
    }
    while(*++strs) {
	eq = strchr(*strs,'=');
	if (!eq) {
	    switch( NameToEnum(*strs,SunVideoNames) ) {
	    case CVideo:
		inVideo++;
		break;
	    case CEnd:
		inVideo = 0;
		break;
	    }
	    continue;
	}
	if (!inVideo) {
	    continue;
	}
	*eq = '\0';
	for(cp = eq-1; *cp == ' '; ) {
	    *cp-- = '\0';
	}
	for(cp = eq+1; *cp == ' '; ) {
	    *cp++ = '\0';
	}
	switch(NameToEnum(*strs,SunVideoNames)) {
	case CWidth:
	    imageInfo->width = StringToInt(cp);
	    break;
	case CHeight:
	    imageInfo->height = StringToInt(cp);
	    break;
	case CFrames:
	    imageInfo->frameCount = StringToInt(cp);
	    imageInfo->defined |= IFI_FrameCount;
	    break;
	case CFps:
	    sscanf(cp,"%f",&imageInfo->framesPerSecond);
	    imageInfo->defined |= IFI_FrameRate;
	    break;
	}
    }
    imageInfo->color = ColorYC;
    imageInfo->sample = Sample4_2_0;
    imageInfo->interlaced = False;
    reason = NULL;
hdrDone:
    FreeStringArgList((StringArgListPtr)Strs);
    return(reason);
}

ImageFileType
PerformExternMap(ImageFileType type)
{
    ImageFileType *iptr;
    for( iptr = ExternImageMap; *iptr != IFT_Unknown; iptr++) {
	if (*iptr == type) {
	    return(IFT_NvrExtern);
	}

    }
    return(type);
}

static ImageFileType
FileTypeToImageType(FileContentType fileType)
{
    char *name;
    ImageFileType iType;

    name = EnumToName(fileType, AllFileTypes);
    iType = PerformExternMap(NameToEnum(name, ImageFileTypes));
    return(iType);
}

/*
 * Get image info for a certain file type.
 * Note that the type argument is only a hint and may be specified as
 * the value FT_Unknown.  Providing the type, saves having to call
 * GetFileContentInfo().  The typical use is
 *     GetFileContentInfo(filename, &fileInfo);
 *     if (fileInfo.cClass == CC_Video
 *      || fileInfo.cClass == CC_AVMovie
 *      || fileInfo.cClass == CC_Image)
 *          GetImageFileInfo(filename, fileInfo.type, imageFileInfo);
 */
InternalReasonPtr
GetImageFileInfo(char *path,
		 FileContentType fileType,
		 ImageFileInfoPtr imageInfo)
{
    InternalReasonPtr reason = NULL;
    FileHeader hdrBuffer;
    int fd;
    int bytesIn;
    ImageSpecPtr ip;

    /* to start with... */
    memset((char *)imageInfo,0,sizeof(ImageFileInfo));
    imageInfo->interlaced = False;

    if ((fd = OpenFileForReading(path)) < 0)
        return( CreateSysReason("Can't open %s\n", path) );
    bytesIn = ReadData(fd, hdrBuffer.raw, sizeof(hdrBuffer.raw));
    if (bytesIn < NVRTYPEHDR)
        goto complete;

    if (fileType == FT_Unknown) {
	FileInfo fileInfo;

	reason = GetFileContentInfo(path, &fileInfo);
	if (reason)
	    goto complete;
	fileType = fileInfo.type;
    }
    imageInfo->type = FileTypeToImageType(fileType);
    imageInfo->contentSize = FdFileSize(fd);

    if( (ip = ImageTypeToImageSpec(imageInfo->type)) ) {
	imageInfo->interlaced = (ip->iLayout != NotInterlaced);
	imageInfo->color = ip->colorSpace;
	imageInfo->sample = ip->sample;
	if( ip->width > 0 ) {
	    imageInfo->width = ip->width;
	    imageInfo->height = ip->height;
	}
    }

    switch(imageInfo->type) {
    case IFT_QuickTime:
        {
            VTrackStatistics trackStats;
            double duration;
            
            trackStats.targetTrackNum = 1;
            if (reason = GetQuickTimeVInfo(path, &trackStats, NULL))
            	goto complete;
            imageInfo->defined = IFI_FrameRate | IFI_BitRate;
            imageInfo->frameCount = trackStats.frameCount;
            imageInfo->width = trackStats.width;
            imageInfo->height = trackStats.height;
            duration = (float)trackStats.movieDuration / trackStats.timeScale;
            imageInfo->bitRate = (imageInfo->contentSize / duration) * 8;
            imageInfo->framesPerSecond = trackStats.frameRate;
	    imageInfo->interlaced = (trackStats.iLayout != NotInterlaced);
	    imageInfo->color = ColorRGB;
            imageInfo->sample = Sample4_4_4;
        }
        break;

    case IFT_AVS:
    case IFT_Vicar:
    case IFT_XWD:
    case IFT_GIF:
    case IFT_Rle:
    case IFT_Miff:
    case IFT_MTV:
    case IFT_Pnm:
    case IFT_SunRaster:
    case IFT_Tiff:
    case IFT_Avi:
	break;

    case IFT_JPEG:
	if (reason = SvjHeader(fd, imageInfo))
	    goto complete;
	break;

    case IFT_SunVideo:
	reason = SunVideoReadHdr(imageInfo, hdrBuffer.raw,
				 sizeof(hdrBuffer.raw));
	if( reason ) {
	    goto complete;
	}
	break;

    case IFT_NvrIdx:
	if (reason = NvrIdxHdr(path, imageInfo))
	    goto complete;
	break;

    case IFT_NvrExtern:
	/*
	 * GetNvrExternInfo will reset the type to IFT_Unknown
	 *	if it fails to find an appropriate external reader
	 */
	reason = GetNvrExternInfo(path, imageInfo);
	if (! reason && imageInfo->type != IFT_NvrExtern) {
	    reason = CreateReason("file type '%s' unavailable\n",
				  EnumToName(fileType, AllFileTypes));
	}
	break;

    case IFT_NvrYuv:
	NvrYuvHdr(imageInfo, hdrBuffer.raw, sizeof(hdrBuffer.raw));
	break;

    case IFT_Jmovie:
	NvrGetPlxMovieHdr(imageInfo,&hdrBuffer.plxMovie);
	break;

    case IFT_SgiRgb:
	(void) SgiRgbInitHdr(hdrBuffer.raw, sizeof(hdrBuffer.raw), imageInfo,
			     "info");
	break;

    case IFT_MpegSystem:
        while (bytesIn > 0) {
            UChar *p;
            int bytesLeft;

            bytesLeft = bytesIn - sizeof(MpegVideoHeader);
            p = (UChar *)hdrBuffer.raw;
            while (bytesLeft >= 0) {
                if (p[0] == 0x00
                 && p[1] == 0x00
                 && p[2] == 0x01
                 && p[3] == 0xb3) {
                    memcpy((char *)&hdrBuffer,
                           (char *)p,
                           bytesLeft + sizeof(MpegVideoHeader));
                    goto foundVideoStream;
                }
                p++;
                bytesLeft--;
            }
            /*
             * Not in the first block of data... try reading again.
             */
            p--;
            memcpy(hdrBuffer.raw, (char *)p, sizeof(MpegVideoHeader));
            bytesIn = ReadData(fd,
                               hdrBuffer.raw + sizeof(MpegVideoHeader),
                               sizeof(hdrBuffer.raw) - sizeof(MpegVideoHeader));
            if (bytesIn <= 0)
                break;
            bytesIn += sizeof(MpegVideoHeader);
        }
        goto complete;  /* system file type, but no video.  Hmmmm. */

    case IFT_MpegVideo:
    foundVideoStream:
	imageInfo->defined = IFI_FrameRate | IFI_BitRate;
	imageInfo->frameCount = 0;	/* a cop-out, for now */
	imageInfo->width = hdrBuffer.mpeg.width;
	imageInfo->height = hdrBuffer.mpeg.height;
	imageInfo->bitRate = (hdrBuffer.mpeg.bitRate == 0x3ffff) ?
			      -1 : hdrBuffer.mpeg.bitRate * 400;
	switch(hdrBuffer.mpeg.pictureRate) {
	case 1: imageInfo->framesPerSecond = 23.976; break; /* 23.976 */
	case 2: imageInfo->framesPerSecond = 24; break;
	case 3: imageInfo->framesPerSecond = 25; break;
	case 4: imageInfo->framesPerSecond = 29.97; break; /* 29.97 */
	case 5: imageInfo->framesPerSecond = 30; break;
	case 6: imageInfo->framesPerSecond = 50; break;
	case 7: imageInfo->framesPerSecond = 59.94; break; /* 59.94 */
	case 8: imageInfo->framesPerSecond = 60; break;
	case 9: imageInfo->framesPerSecond = 30; break; /* XING 30fps? */
	case 10: imageInfo->framesPerSecond = 6; break; /* NVR private */
	case 11: imageInfo->framesPerSecond = 10; break; /* NVR private */
	case 12: imageInfo->framesPerSecond = 15; break; /* NVR private */
	case 13: imageInfo->framesPerSecond = 20; break; /* NVR private */
	case 14: imageInfo->framesPerSecond = 5; break; /* NVR private */
	default: imageInfo->framesPerSecond = 0; break;
	}
	break;

    default:
	reason = &IRBadIoFileType;
	break;

    }

complete:
    CloseFile(fd);
    return(reason);
}
