/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	imageData.c - global data for image processing library
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/imageData.c,v 1.29 1995/11/16 22:18:32 russ Exp $
 */

#include "nvrImageI.h"

NameEnumPair ImageFileTypes[] = {
    { "rawycrcb",   IFT_RawYCrCb },
    { "rawycbcr",   IFT_RawYCbCr },
    { "rawrgb",     IFT_RawRgb },
    { "mbstats",    IFT_MpegMbStats },
    { "abekas",     IFT_Abekas },
    { "c3filt",     IFT_C3Filt },
    { "cperfile",   IFT_CompPerFile },
    { "pix",        IFT_Pix },
    { "svj",        IFT_Svj },

    { "jmovie",     IFT_Jmovie },
    { "quicktime",  IFT_QuickTime },
    { "sgimovie",   IFT_SgiMovie },
    { "omf",	    IFT_Omf },
    { "extern",     IFT_NvrExtern },
    { "sgirgb",     IFT_SgiRgb },
    { "avs",        IFT_AVS },
    { "vicar",      IFT_Vicar },
    { "xwd",        IFT_XWD },
    { "gif",        IFT_GIF },
    { "rle",        IFT_Rle },
    { "jpeg",       IFT_JPEG },
    { "miff",       IFT_Miff },
    { "mtv",        IFT_MTV },
    { "pnm",        IFT_Pnm },
    { "sun_video",  IFT_SunVideo },
    { "sun_raster", IFT_SunRaster },
    { "sun",        IFT_SunRaster },
    { "tiff",       IFT_Tiff },
    { "mpeg_video", IFT_MpegVideo },
    { "mpeg",       IFT_MpegVideo },
    { "vmpeg",      IFT_MpegVideo },
    { "mpeg_system",IFT_MpegSystem },	/* XXX */
    { "avi",        IFT_Avi },
    { "nvr_yuv",    IFT_NvrYuv },
    { "nvr_idx",    IFT_NvrIdx },

    { NULL,         IFT_Unspecified }
};

/*
 * These types use the external file interface
 *	for reading
 */
ImageFileType ExternImageMap[] = {
    IFT_Omf,
    IFT_SgiMovie,
    IFT_MpegVideo,
    IFT_MpegSystem,
    IFT_QuickTime,
    IFT_NvrExtern,
    IFT_Unknown		/* must be last */
};

NameEnumPair FileParamNames[] = {
    { "prefix",     IFP_Prefix },
    { "suffix",     IFP_Suffix },
    { "ndigits",    IFP_NDigits },
    { "start",      IFP_StartIndex },
    { "stop",       IFP_StopIndex },

    { "type",       IFP_Type },
    { "sample",     IFP_Sample },
    { "size",       IFP_Size },

    { "read",	    IFP_Read },
    { "write",	    IFP_Write },

    { "nframes",    IFP_FrameCount },
    { "framerate",  IFP_FramesPerSec },
    { "changefps",  IFP_ResampleFps },
    { "colorspace", IFP_ColorSpace },
    { "interlace",  IFP_Interlace },

    { NULL,         IFP_Unknown }
};

NameEnumPair ColorSpaceNames[] = {
    { "rgb",       ColorRGB },
    { "yc",        ColorYC },
    { "rgblookup", ColorRGBLookup },
    { NULL,        ColorUnknown }
};

NameEnumPair SampleTypeNames[] = {
    { "4:2:0",	   Sample4_2_0 },
    { "4:1:1",	   Sample4_2_0 },
    { "4:2:2",	   Sample4_2_2 },
    { "4:4:4",     Sample4_4_4 },
    { "420",	   Sample4_2_0 },
    { "411",	   Sample4_2_0 },
    { "422",	   Sample4_2_2 },
    { "444",       Sample4_4_4 },
    { NULL,        SampleUnknown }
};

NameEnumPair InterlaceLayoutNames[] = {
    { "pict",     InterlacedByPict },
    { "pictOdd",  InterlacedByPictOdd },
    { "pictEven", InterlacedByPictEven },
    { "True",     InterlacedByLine },
    { "true",     InterlacedByLine },
    { "line",     InterlacedByLine },
    { "false",    NotInterlaced },
    { "none",     NotInterlaced },
    { "no",       NotInterlaced },
    { NULL,       UnknownInterlace }
};

NameEnumPair BooleanNames[] = {
    { "True",  True },
    { "False", False },
    { "true",  True },
    { "false", False },
    { "1",     True },
    { "0",     False },
    { "yes",   True },
    { "no",    False },
    { NULL,    False }
};

NameEnumPair ImageOpNames[] = {
    { "unknown",	ImgOpUnknown },
    { "unspecified",	ImgOpUnspecified },
    { "io",		ImgOpIO },
    { "resample",	ImgOpResample },
    { "color",		ImgOpColor },
    { "filter",		ImgOpFilter },
    { "grid",		ImgOpGrid },
    { "deinterlace",	ImgOpDeinterlace },
    { "generate",	ImgOpGenerate },
    { NULL,		ImgOpUnknown },
};

NameEnumPair ResampleMethodNames[] = {
    { "nearest",        ResampleNearest },
    { "bilinear",       ResampleBiLinear },
    { "cubic",          ResampleCubic },
    { NULL,		ResampleUnknown }
};

InternalReason IRBadImageFileType =
    { "That is not a valid image file type\n", IR_UnknowFileType };

