/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	imageUtil.c - support for the image library
 *
 * DESCRIPTION
 *	<overall description of what the c code does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/imageUtil.c,v 2.51 1995/11/15 01:49:19 russ Exp $
 */

#define NeedFunctionPrototypes 1

#include "nvrImageI.h"
#include "nvrString.h"
#include "nvrAlloc.h"
#include "process.h"
#include "osio.h"
#include "fileAccess.h"

static CompositeMemImagePtr
NullCreate(ImageSpecPtr spec,
	   CompositeMemImagePtr lastComp,
           InternalReasonPtr *pReason)
{
    static InternalReason NoCreateFunc =
	{ "No image create function internally defined.\n",
	  IR_InternalProblem };
    *pReason = &NoCreateFunc;
    return(NULL);
}

InternalReasonPtr
SimpleOpInit(ImageOpPtr op, CompositeMemImagePtr in)
{
    static InternalReason badRgb = { "rgb must be 4:4:4\n", IR_BadColor };
    static InternalReason badCreate =
	{ "create not specified\n", IR_InternalProblem };
    static InternalReason cantDo = { "can't process rgb lookup\n", IR_BadColor };
    static InternalReason badWidth =
	{ "yc at 4:2:2 and 4:2:0 must have an even width\n", IR_BadDimensions };
    static InternalReason badHeight =
	{ "yc at 4:2:0 must have an even height\n", IR_BadDimensions };
    /*
     * ensure everything is filled in
     */
    if (   op->u.any.spec.sample == SampleUnspecified
	|| op->u.any.spec.sample == SampleUnknown)
        return(&IRBadSample);
    if (   op->u.any.spec.colorSpace == ColorUnspecified
	|| op->u.any.spec.colorSpace == ColorUnknown)
        return(&IRBadColor);
    if (op->u.any.spec.colorSpace == ColorRGBLookup && ! op->rgbLookupOk)
	return(&cantDo);
    if (   op->u.any.spec.colorSpace == ColorRGBLookup
	&& op->u.any.spec.sample != Sample4_4_4)
	return(&badRgb);
    if (   op->u.any.spec.colorSpace == ColorRGB
	&& op->u.any.spec.sample != Sample4_4_4)
	return(&badRgb);
    if (   op->u.any.spec.sample == Sample4_2_0
	&& op->u.any.spec.height & 0x1)
	return(&badHeight);
    if (   (   op->u.any.spec.sample == Sample4_2_2
	    || op->u.any.spec.sample == Sample4_2_0)
	&& (   (op->u.any.spec.width & 0x1)
	    && op->oddWidthOk == False))
	return(&badWidth);
    if (   op->u.any.spec.width <= 0
	|| op->u.any.spec.height <= 0)
        return(&IRBadDimensions);
    if (op->u.any.spec.create == NullCreate)
        return(&badCreate);
    return(NULL);
}

InternalReasonPtr
FailProc(ImageOpPtr op,
        CompositeMemImagePtr in,
        CompositeMemImagePtr out)
{
    static InternalReason notYet = { "no proc procedure\n", IR_NotImplemented };
    return(&notYet);
}

void
NopFree(ImageOpPtr op)
{
    return;
}

InternalReasonPtr
NopFlush(ImageOpPtr op,
	 CompositeMemImagePtr in, 
	 CompositeMemImagePtr out)
{
    return(NULL);
}

#define AbekasYC	ColorYC, NewAbekasImage
#define YC		ColorYC, NewYcImage
#define RGB		ColorRGB, NewRgbImage
#define NoColor		ColorUnspecified, NullCreate

#if defined(SYSV)||defined(__STDC__)||defined(THINK_C)||defined(__MWERKS__)

#define Spec(fileType,width,height,interlace,sample,colorSpace)		\
    {	IFT_##fileType,							\
	width,height,							\
	interlace,							\
	Sample##sample,							\
	colorSpace							\
    }
#else
#define Spec(fileType,width,height,interlace,sample,colorSpace)		\
    {	IFT_/**/fileType,						\
	width,height,							\
	interlace,							\
	Sample/**/sample,						\
	colorSpace							\
    }
#endif

static ImageSpec ImageSpecTemplates[] = {
    Spec(Abekas,     720, 243, InterlacedByLine ,4_2_2,       AbekasYC),
    Spec(Jmovie,      -1,  -1, NotInterlaced    ,4_2_2,       YC),
    Spec(CompPerFile, -1,  -1, NotInterlaced    ,4_2_0,       YC),
    Spec(MpegVideo,   -1,  -1, NotInterlaced    ,4_2_0,       YC),
    Spec(RawYCrCb,    -1,  -1, NotInterlaced    ,Unspecified, YC),
    Spec(RawYCbCr,    -1,  -1, NotInterlaced    ,Unspecified, YC),
    Spec(C3Filt,      -1,  -1, NotInterlaced    ,Unspecified, YC),
    Spec(NvrYuv,      -1,  -1, NotInterlaced    ,Unspecified, YC),
    Spec(SunVideo,    -1,  -1, NotInterlaced    ,4_2_0,       YC),
    Spec(Avi,         -1,  -1, NotInterlaced    ,Unspecified, NoColor),
    Spec(QuickTime,   -1,  -1, NotInterlaced    ,Unspecified, NoColor),
    Spec(SgiMovie,    -1,  -1, NotInterlaced    ,Unspecified, NoColor),
    Spec(Omf,	      -1,  -1, NotInterlaced    ,Unspecified, NoColor),
    Spec(NvrExtern,   -1,  -1, NotInterlaced    ,Unspecified, NoColor),

    Spec(SgiRgb,      -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(RawRgb,      -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(MpegMbStats, -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(Vicar,       -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(AVS,         -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(XWD,         -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(GIF,         -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(Rle,         -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(JPEG,        -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(Miff,        -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(MTV,         -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(Pnm,         -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(SunRaster,   -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(Tiff,        -1,  -1, NotInterlaced    ,4_4_4,       RGB),
    Spec(Pix,         -1,  -1, NotInterlaced    ,4_4_4,       RGB),

    Spec(NvrIdx,      -1,  -1, NotInterlaced    ,Unspecified, NoColor),
    Spec(Svj,         -1,  -1, InterlacedByPict ,Unspecified, NoColor),
};

static ImageSpec DefSpecTemplate =
    Spec(Unspecified, -1,  -1, NotInterlaced    ,Unspecified, NoColor);

#undef AbekasYC
#undef YC
#undef RGB
#undef NoColor
#undef Spec

ImageSpecPtr
ImageTypeToImageSpec(ImageFileType iType)
{
    int itLen = sizeof(ImageSpecTemplates) / sizeof(ImageSpecTemplates[0]);
    while( --itLen >= 0 ) {
	if( ImageSpecTemplates[itLen].type == iType ) {
	    return(&ImageSpecTemplates[itLen]);
	}
    }
    return(NULL);
}

boolean
AddTemplateToSpec(char *templateName, ImageSpecPtr spec)
{
    int i;
    ImageSpecPtr tmpl;
    ImageFileType type = NameToEnum(templateName, ImageFileTypes);

    for (i = 0; i < NumberOfCells(ImageSpecTemplates); i++) {
        if (ImageSpecTemplates[i].type == type) {
            tmpl = &ImageSpecTemplates[i];
            if (tmpl->width > 0)
                spec->width = tmpl->width;
            if (tmpl->height > 0)
                spec->height = tmpl->height;
            if (tmpl->sample != SampleUnspecified)
                spec->sample = tmpl->sample;
            if (tmpl->colorSpace != ColorUnspecified)
                spec->colorSpace = tmpl->colorSpace;
	    if (tmpl->create != NullCreate)
		spec->create = tmpl->create;
	    if (tmpl->iLayout)
	        spec->iLayout = tmpl->iLayout;
            spec->type = tmpl->type;
            return (True);
        }
    }
    return (False);
}

static struct {
    ImageOpPtr *list;
    int count;
} OpTemplates;

InternalReasonPtr
AddOpTemplate(ImageOpPtr template)
{
    int i, size;
    
    /*
     * It already may be in the list.
     */
    for (i = 0; i < OpTemplates.count; i++) {
        if (OpTemplates.list[ i ] == template)
            return(NULL);
    }
    size = ++OpTemplates.count * sizeof(ImageOpPtr);
    OpTemplates.list = (ImageOpPtr *)
	InsecureNullRealloc((char *)OpTemplates.list, size);
    if (OpTemplates.list == NULL) {
        OpTemplates.count = 0;
        return( AllocReason("image templates", size) );
    }

    OpTemplates.list[ OpTemplates.count-1 ] = template;
    return(NULL);
}

PipelineNodePtr
NewNodeForPipeline(ImagePipelinePtr pipeline,
                   ImageOpName opName,
                   ImageReasonPtr *pReason)
{
    PipelineNodePtr node, newList;
    int nNodes = pipeline->nNodes + 1;
    int i, size;

    size = sizeof(PipelineNode) * nNodes;
    newList = (PipelineNodePtr) InsecureAlloc(size);
    if (newList == NULL) {
        *pReason = AllocReason("pipeline node", size);
        return(NULL);
    }
    
    if (pipeline->nodes) {
        memcpy((char *)newList,
               (char *)pipeline->nodes,
               size - sizeof(PipelineNode));
    }

    node = &newList[ pipeline->nNodes ];
    memset((char *)node, 0, sizeof(PipelineNode));
    node->op.name = opName;
    for (i = 0; i < OpTemplates.count; i++) {
    	if (opName == OpTemplates.list[i]->name) {
    	    node->op = *OpTemplates.list[i];
    	    break;
    	}
    }
    if (i == OpTemplates.count) {
	if (OpTemplates.count == 0)
	    *pReason = InternalProblem(
	        "no operator templates specified with AddOpTemplate()!\n");
	else 
    	    *pReason = InternalProblem("Unknown op: %d\n", opName);
    	free((char *)newList);
    	return(NULL);
    }
    
    if (pipeline->nodes)
        free((char *)pipeline->nodes);
    node->op.u.any.spec = DefSpecTemplate;
    pipeline->nodes = newList;
    pipeline->nNodes = nNodes;
    return (node);
}

/*
 * Initialize the MemImage with the given parameters and return
 * the pointer to the (calculated) next pixel set.  This
 * return value may be ignored if the image is interleaved
 */
UChar *
InitMemImage(MemImagePtr image,
	     UChar *pixels,
	     int channelName,
	     ColorSpace colorSpace,
	     int width, int height,
	     int nextPixel,
	     int nextLine,
	     MemImagePtr field2Image)
{
    image->pixels = pixels;
    image->size.width = width;
    image->size.height = height;
    image->nextPixel = nextPixel;
    image->nextLine = nextLine;
    switch(colorSpace) {
    case ColorYC:	 image->u.ycName = channelName; break;
    case ColorRGB:	 image->u.rgbName = channelName; break;
    case ColorRGBLookup: image->u.rgbName = channelName; break;
    default: break;
    }
    image->u.anyName = channelName;
    image->field2Image = field2Image;
    return(image->pixels + (height*nextLine));
}

CompositeMemImagePtr
NewCompositeMemImage(int dataSize,
		     int headerSize,
		     int nImage,
		     SampleType sample,
		     ColorSpace colorSpace,
		     InterlaceLayoutType iLayout)
{
    CompositeMemImagePtr newComp;
    int nAllocatedImages = nImage;

    if (iLayout)
	nAllocatedImages *= 2;
    newComp = (CompositeMemImagePtr)
        InsecureAlloc(dataSize
		      + sizeof(CompositeMemImage)
		      + sizeof(MemImage) * nAllocatedImages
		      + headerSize);

    if (newComp == NULL)
	return(NULL);
    newComp->pixelBytes = dataSize;
    newComp->nImage = nImage;
    newComp->images = (MemImagePtr)(newComp+1);
    if (iLayout)
	newComp->field2Images = newComp->images + nImage;
    else
	newComp->field2Images = NULL;
    if (headerSize) {
	newComp->header = (UChar *)(newComp->images + nAllocatedImages);
	newComp->pixels = newComp->header + headerSize;
    } else {
	newComp->header = NULL;
	newComp->pixels = (UChar *)(newComp->images + nAllocatedImages);
    }
    newComp->sample = sample;
    newComp->colorSpace = colorSpace;
    newComp->priv = NULL;
    return(newComp);
}

CompositeMemImagePtr
CopyImageByReference(ImageSpecPtr spec,
		     CompositeMemImagePtr lastComp,
		     InternalReasonPtr *pReason)
{
    return(lastComp);
}

static CompositeMemImagePtr
CantCreateImage(InternalReasonPtr reason, InternalReasonPtr *pReason)
{
    *pReason = reason;
    return (NULL);
}

CompositeMemImagePtr
NewLookupImage(ImageSpecPtr spec,
               CompositeMemImagePtr lastComp,
	       InternalReasonPtr *pReason)
{
    int width  = spec->width;
    int height = spec->height;
    CompositeMemImagePtr newComp;
    int imageSize;
    static InternalReason CantInterlace =
	{ "Rgb lookup images can't be interlaced.\n", IR_NotImplemented };

    if (spec->iLayout)
	return( CantCreateImage(&CantInterlace, pReason) );
	
    imageSize = width * height;

    newComp = NewCompositeMemImage(imageSize,		/* total frame size */
				   spec->headerSize,
				   1,			/* 1 channel */
				   Sample4_4_4,
				   ColorRGBLookup,
				   NotInterlaced);
    if (newComp == NULL)
	return( CantCreateImage(&IRBadAllocation, pReason) );
    newComp->size.width = width;
    newComp->size.height = height;

    (void) InitMemImage(&newComp->images[0],
			newComp->pixels,
			RgbAll, ColorRGB,
			width, height,			/* width, height */
			1,				/* next pixel */
			width,				/* next line */
		        NULL);				/* field 2 */
    return(newComp);
}

CompositeMemImagePtr
NewRgbImage(ImageSpecPtr spec,
            CompositeMemImagePtr lastComp,
	    InternalReasonPtr *pReason)
{
    int width  = spec->width;
    int height = spec->height;
    CompositeMemImagePtr newComp;
    MemImagePtr field2Images;
    int imageSize;
    UChar *field1Pixels;
    UChar *field2Pixels;
    int interlaceMult = spec->iLayout ? 2 : 1;

    imageSize = width * height * 3 * interlaceMult;

    newComp = NewCompositeMemImage(imageSize,		/* total frame size */
				   spec->headerSize,
				   3,			/* 3 channels */
				   Sample4_4_4,
				   ColorRGB,
				   spec->iLayout);
    if (newComp == NULL)
	return( CantCreateImage(&IRBadAllocation, pReason) );

    newComp->size.width = width;
    newComp->size.height = height;

    switch (spec->iLayout) {
    case InterlacedByPict:
	field1Pixels = newComp->pixels;
	field2Pixels = newComp->pixels + width*height*3;
	goto interlaced;

    case InterlacedByLine:
	field1Pixels = newComp->pixels;
	field2Pixels = newComp->pixels + width*3;
    interlaced:
	(void) InitMemImage(&newComp->field2Images[0],
			    field2Pixels + 0,		/* offset of 0 */
			    RgbRed, ColorRGB,
			    width, height,		/* width, height */
			    3,                          /* next pixel */
			    width*3,                    /* next line */
			    NULL);                      /* field 2 */
	(void) InitMemImage(&newComp->field2Images[1],
			    field2Pixels + 1,		/* offset of 1 */
			    RgbGreen, ColorRGB,
			    width, height,		/* width, height */
			    3,                          /* next pixel */
			    width*3,                    /* next line */
			    NULL);                      /* field 2 */
	(void) InitMemImage(&newComp->field2Images[2],
			    field2Pixels + 2,		/* offset of 2 */
			    RgbBlue, ColorRGB,
			    width, height,		/* width, height */
			    3,                          /* next pixel */
			    width*3,                    /* next line */
			    NULL);                      /* field 2 */
	break;

    default:
	field1Pixels = newComp->pixels;
	break;
    }

    /*
     * Note that we create it interleaved...
     */
    field2Images = newComp->field2Images;
    (void) InitMemImage(&newComp->images[0],
			newComp->pixels+0,
			RgbRed, ColorRGB,
			width, height,			/* width, height */
			3,				/* next pixel */
			width*3,			/* next line */
		        field2Images
			    ? &field2Images[0] : NULL); /* field 2 */
    (void) InitMemImage(&newComp->images[1],
			newComp->pixels+1,
			RgbGreen, ColorRGB,
			width, height,			/* width, height */
			3,				/* next pixel */
			width*3,			/* next line */
		        field2Images
			    ? &field2Images[1] : NULL); /* field 2 */
    (void) InitMemImage(&newComp->images[2],
			newComp->pixels+2,
			RgbBlue, ColorRGB,
			width, height,			/* width, height */
			3,				/* next pixel */
			width*3,			/* next line */
		        field2Images
			    ? &field2Images[2] : NULL); /* field 2 */

    return(newComp);
}

CompositeMemImagePtr
NewAviRgbImage(ImageSpecPtr spec,
               CompositeMemImagePtr lastComp,
               InternalReasonPtr *pReason)
{
    int width  = spec->width;
    int height = spec->height;
    CompositeMemImagePtr newComp;
    int imageSize, rowStorage;
    UChar *nextPixels;
    static InternalReason CantInterlace =
	{ "AVI images can't be interlaced.\n", IR_NotImplemented };

    if (spec->iLayout)
	return( CantCreateImage(&CantInterlace, pReason) );

    rowStorage = ((width * 3 + 3) & ~3);
    imageSize = rowStorage * height;

    newComp = NewCompositeMemImage(imageSize,		/* total frame size */
				   spec->headerSize,
				   3,			/* 3 channels */
				   Sample4_4_4,
				   ColorRGB,
				   NotInterlaced);
    if (newComp == NULL)
	return( CantCreateImage(&IRBadAllocation, pReason) );
    newComp->size.width = width;
    newComp->size.height = height;

    /*
     * Note that we create it interleaved...
     */
    nextPixels = InitMemImage(&newComp->images[0],
			newComp->pixels + imageSize - rowStorage,
			RgbBlue, ColorRGB,
			width, height,			/* width, height */
			3,				/* next pixel */
			- (rowStorage),  		/* next line */
		        NULL);				/* field 2 */
    nextPixels = InitMemImage(&newComp->images[1],
			nextPixels,
			RgbGreen, ColorRGB,
			width, height,			/* width, height */
			3,				/* next pixel */
			- (rowStorage),	    	   	/* next line */
		        NULL);				/* field 2 */
    (void) InitMemImage(&newComp->images[2],
			nextPixels,
			RgbRed, ColorRGB,
			width, height,			/* width, height */
			3,				/* next pixel */
			- (rowStorage),			/* next line */
		        NULL);				/* field 2 */
    return(newComp);
}

CompositeMemImagePtr
NewYcImage(ImageSpecPtr spec,
	   CompositeMemImagePtr lastComp,
	   InternalReasonPtr *pReason)
{
    int width  = spec->width;
    int height = spec->height;
    CompositeMemImagePtr newComp;
    int lumaSize, chromaSize, imageSize;
    int chromaWidth, chromaHeight;
    UChar *nextPixels;
    boolean cbIsFirst;
    static InternalReason CantInterlace =
	{ "can't handle interlaced YC images.\n", IR_NotImplemented };

    if (spec->iLayout)
	return( CantCreateImage(&CantInterlace, pReason) );

    lumaSize = width * height;
    switch(spec->sample) {
    case Sample4_2_0: chromaWidth = width / 2; chromaHeight = height / 2; break;
    case Sample4_2_2: chromaWidth = width / 2; chromaHeight = height; break;
    case Sample4_4_4: chromaWidth = width; chromaHeight = height; break;

    case SampleUnspecified: return(NULL); /* handled elsewhere */
    case SampleUnknown: return(NULL);     /* handled elsewhere */
    }
    chromaSize = chromaWidth * chromaHeight;
    imageSize = lumaSize + 2*chromaSize;

    newComp = NewCompositeMemImage(imageSize,		/* total frame size */
				   spec->headerSize,
				   3,			/* 3 channels */
				   spec->sample,
				   ColorYC,
				   NotInterlaced);
    if (newComp == NULL)
	return( CantCreateImage(&IRBadAllocation, pReason) );
    newComp->size.width = width;
    newComp->size.height = height;

    nextPixels = InitMemImage(&newComp->images[0],
			newComp->pixels,
			YcLuma, ColorYC,
			width, height,			/* width, height */
			1,				/* next pixel */
			width,				/* next line */
		        NULL);				/* field 2 */

    cbIsFirst = (spec->type == IFT_RawYCbCr);
    nextPixels = InitMemImage(&newComp->images[1],
			nextPixels,
			cbIsFirst ? YcCb : YcCr, ColorYC,
			chromaWidth, chromaHeight,	/* width, height */
			1,				/* next pixel */
			chromaWidth,			/* next line */
		        NULL);				/* field 2 */
    (void) InitMemImage(&newComp->images[2],
			nextPixels,
			cbIsFirst ? YcCr : YcCb, ColorYC,
			chromaWidth, chromaHeight,	/* width, height */
			1,				/* next pixel */
			chromaWidth,			/* next line */
		        NULL);				/* field 2 */
    return(newComp);
}

CompositeMemImagePtr
NewAbekasImage(ImageSpecPtr spec,
               CompositeMemImagePtr lastComp,
               InternalReasonPtr *pReason)
{
    int width  = spec->width;
    int height = spec->height;
    CompositeMemImagePtr newComp;
    UChar *field2Pixels;
    UChar *field1Pixels;
    MemImagePtr field1Images;
    MemImagePtr field2Images;
    int imageSize;
    int nextLine;

    imageSize = width * height * 2 * (spec->iLayout ? 2 : 1);
    newComp = NewCompositeMemImage(imageSize,		/* total frame size */
				   spec->headerSize,
				   3,			/* 3 channels */
				   Sample4_2_2,
				   ColorYC,
				   spec->iLayout);

    if (newComp == NULL)
	return( CantCreateImage(&IRBadAllocation, pReason) );

    newComp->size.width = width;
    newComp->size.height = height;

    switch (spec->iLayout) {
    case InterlacedByPictOdd:
	field1Pixels = newComp->pixels + width*2*height;
	field2Pixels = newComp->pixels;
	field1Images = newComp->images;
	field2Images = newComp->field2Images;
	nextLine = width*2;
	break;

    case InterlacedByPictEven:
	field1Pixels = newComp->pixels;
	field2Pixels = newComp->pixels + width*2*height;
	field1Images = newComp->images;
	field2Images = newComp->field2Images;
	nextLine = width*2;
	break;

    case InterlacedByLine:
	field1Pixels = newComp->pixels + width*2;
	field2Pixels = newComp->pixels;
	field1Images = newComp->images;
	field2Images = newComp->field2Images;
	nextLine = width*2*2;
	break;

    default:
	field1Pixels = newComp->pixels;
	field2Pixels = NULL;
	field1Images = newComp->images;
	field2Images = newComp->field2Images;
	nextLine = width*2;
	break;
    }

    if (field2Pixels) {
	(void) InitMemImage(&field2Images[0],
			    field2Pixels + 1,		/* offset of 1 */
			    YcLuma, ColorYC,
			    width, height,		/* width, height */
			    2,				/* next pixel */
			    nextLine,			/* next line */
			    NULL);			/* field 2 */
	(void) InitMemImage(&field2Images[1],
			    field2Pixels + 0,		/* offset of 0 */
			    YcCb, ColorYC,
			    width>>1, height,		/* width, height */
			    4,				/* next pixel */
			    nextLine,		/* next line */
			    NULL);			/* field 2 */
	(void) InitMemImage(&field2Images[2],
			    field2Pixels + 2,		/* offset of 2 */
			    YcCr, ColorYC,
			    width>>1, height,		/* width, height */
			    4,				/* next pixel */
			    nextLine,		/* next line */
			    NULL);			/* field 2 */
    }

    (void) InitMemImage(&field1Images[0],
			field1Pixels + 1,		/* offset of 1 */
			YcLuma, ColorYC,
			width, height,			/* width, height */
			2,				/* next pixel */
			nextLine,			/* next line */
		        field2Images
			? &field2Images[0] : NULL); /* field 2 */
    (void) InitMemImage(&field1Images[1],
			field1Pixels + 0,		/* offset of 0 */
			YcCb, ColorYC,
			width>>1, height,		/* width, height */
			4,				/* next pixel */
			nextLine,			/* next line */
		        field2Images
			? &field2Images[1] : NULL); /* field 2 */
    (void) InitMemImage(&field1Images[2],
			field1Pixels + 2,		/* offset of 2 */
			YcCr, ColorYC,
			width>>1, height,		/* width, height */
			4,				/* next pixel */
			nextLine,		/* next line */
		        field2Images
			? &field2Images[2] : NULL); /* field 2 */


    return(newComp);
}

static boolean
ImagesAreIsometric(CompositeMemImagePtr srcComp, CompositeMemImagePtr dstComp)
{
    int i;
    MemImagePtr srcImage;
    MemImagePtr dstImage;

    if (srcComp == dstComp)
    	return(True);

    for (i = 0; i < srcComp->nImage; i++) {
	if (dstComp->images[i].u.anyName != srcComp->images[i].u.anyName)
	    return(False);

	srcImage = &srcComp->images[i];
	dstImage = &dstComp->images[i];

	if (srcImage->size.width != dstImage->size.width
	 || srcImage->size.height != dstImage->size.height)
	     return(False);
    }
    return(True);
}

/*
 * This structure is used by InitImagePipeline() and CheckCreateFunc()
 * to make sure the
 * parameters in the op's spec structure are compatible with the selected
 * create routine.  See CheckCreateFunc() below
 */
static struct {
    SampleType sample;		/* how the components are sub-sampled */
    ColorSpace colorSpace;	/* colorspace of the image */
    CompositeMemImagePtr	/* function to create image */
	(*create)(ImageSpecPtr, CompositeMemImagePtr, InternalReasonPtr *);
} CreateConstraints[] = {
    { Sample4_2_2,       ColorYC,  NewAbekasImage  },
    { SampleUnspecified, ColorYC,  NewYcImage  },
    { Sample4_4_4,       ColorRGB, NewRgbImage },
    { Sample4_4_4,       ColorRGB, NewAviRgbImage },
    { Sample4_4_4,       ColorRGB, NewLookupImage },
};

static InternalReasonPtr
CheckCreateFunc(ImageOpPtr op)
{
    InternalReasonPtr reason = NULL;
    int i;
    
    for (i = 0; i < NumberOfCells(CreateConstraints); i++) {
        if (CreateConstraints[ i ].create != op->u.any.spec.create)
            continue;
        if (CreateConstraints[ i ].sample != SampleUnspecified
         && CreateConstraints[ i ].sample != op->u.any.spec.sample) {
	    reason = CreateReason("%s (%s) %s (%s).\n",
	    		"sample rate",
			EnumToName(op->u.any.spec.sample,
				   SampleTypeNames),
	    		"conflicts with rate implied by type",
			EnumToName(CreateConstraints[ i ].sample,
				   SampleTypeNames));
	    break;
	}

        if (CreateConstraints[ i ].colorSpace != ColorUnspecified
         && CreateConstraints[ i ].colorSpace != op->u.any.spec.colorSpace) {
	    reason = CreateReason("%s (%s) %s (%s)\n",
	    		"color space",
			EnumToName(op->u.any.spec.colorSpace,
				   ColorSpaceNames),
	    		"conflicts with that implied by type",
			EnumToName(CreateConstraints[ i ].colorSpace,
				   ColorSpaceNames));
	    break;
	}
    }
    return(reason);
}

InternalReasonPtr
InitImagePipeline(ImagePipelinePtr pipeline)
{
    int i;
    ImageOpPtr op = NULL, lastOp;
    CompositeMemImagePtr newImage, lastImage;
    InternalReasonPtr reason = NULL;
    CompositeMemImagePtr (*create)(ImageSpecPtr,
				   CompositeMemImagePtr,
				   InternalReasonPtr *);
    
    lastImage = NULL;
    lastOp = NULL;
    for (i = 0; i < pipeline->nNodes; i++) {
        op = &pipeline->nodes[ i ].op;
	if (lastImage != NULL) {
	    if (op->dimensionInherited) {
		op->u.any.spec.width = lastImage->size.width;
		op->u.any.spec.height = lastImage->size.height;
	    }
	    if (op->u.any.spec.type == IFT_Unspecified)
		op->u.any.spec.type = lastOp->u.any.spec.type;
	    if (op->u.any.spec.sample == SampleUnspecified)
		op->u.any.spec.sample = lastImage->sample;
	    if (op->u.any.spec.colorSpace == ColorUnspecified)
		op->u.any.spec.colorSpace = lastImage->colorSpace;
	    if (op->u.any.spec.create == NullCreate) {
		ImageSpecPtr spec;

		spec = ImageTypeToImageSpec(op->u.any.spec.type);
		op->u.any.spec.create = spec ? spec->create
		                             : lastOp->u.any.spec.create;
	    }
	}
        if (reason = (op->init)(op, lastImage))
            break;
    
        /*
         * Look at the previous op.u.any.spec.  If the parameters are the
         * same as this one, then no new image needs to be created.
         */
        if (op->canProcessInPlace
         && lastOp != NULL
         && op->u.any.spec.width == lastOp->u.any.spec.width
         && op->u.any.spec.height == lastOp->u.any.spec.height
         && op->u.any.spec.iLayout == lastOp->u.any.spec.iLayout
         && op->u.any.spec.sample == lastOp->u.any.spec.sample
         && op->u.any.spec.colorSpace == lastOp->u.any.spec.colorSpace
#if	   0
         && op->u.any.spec.create == lastOp->u.any.spec.create
#endif	/* 0 */
	    )
	    create = CopyImageByReference;
	else {
	    create = op->u.any.spec.create;
	    if (reason = CheckCreateFunc(op))
	    	break;
	}
	newImage = (*create)(&op->u.any.spec, lastImage, &reason);
	if (newImage == NULL)
            break;

	if (op->enforceIsometric
	 && lastImage != NULL
	 && ! ImagesAreIsometric(lastImage, newImage)) {
	    reason =
		CreateReason("nodes %d (%s) and %d (%s) must be isometric\n",
			     i, EnumToName(pipeline->nodes[i-1].op.name,
					   ImageOpNames),
			     i+1, EnumToName(op->name, ImageOpNames));
	    return(reason);
	}

	pipeline->nodes[ i ].out = newImage;
	lastImage = newImage;
	lastOp = op;
    }

    if (reason) {
	reason->nodeIndex = i;
	reason->nodeName = EnumToName(op->name, ImageOpNames);
    }
    return(reason);
}

InternalReasonPtr
ProcessPipeline(ImagePipelinePtr pipeline)
{
    int i;
    ImageOpPtr op;
    InternalReasonPtr reason;
    CompositeMemImagePtr opImage, lastImage;
    
    for(;;) {
        lastImage = NULL;
	if (pipeline->startPipe)
	    (*pipeline->startPipe)();
        for (i = 0; i < pipeline->nNodes; i++) {
            opImage = pipeline->nodes[ i ].out;
            op = &pipeline->nodes[ i ].op;

	    if (pipeline->startop)
		(*pipeline->startop)(op, lastImage, opImage);

            if (reason = (*op->proc)(op, lastImage, opImage))
                goto done;

	    if (pipeline->endop)
		(*pipeline->endop)(op, lastImage, opImage);
            
            lastImage = opImage;
        }
	if (pipeline->endPipe)
	    (*pipeline->endPipe)();
    }

done:
    if (reason) {
	reason->nodeIndex = i;
	reason->nodeName = EnumToName(op->name, ImageOpNames);
    }

    return(reason);
}

InternalReasonPtr
FlushPipeline(ImagePipelinePtr pipeline)
{
    int i;
    ImageOpPtr op;
    InternalReasonPtr reason;
    CompositeMemImagePtr opImage, lastImage;
    
    lastImage = NULL;
    for (i = 0; i < pipeline->nNodes; i++) {
        opImage = pipeline->nodes[ i ].out;
        op = &pipeline->nodes[ i ].op;
        if (reason = (*op->flush)(op, lastImage, opImage))
	    return(reason);        
        lastImage = opImage;
    }

    return(NULL);
}

void
FreeImagePipeline(ImagePipelinePtr pipeline)
{
    int i;
    ImageOpPtr op;
    CompositeMemImagePtr opImage, lastImage;

    lastImage = NULL;
    for (i = 0; i < pipeline->nNodes; i++) {
        opImage = pipeline->nodes[ i ].out;
        op = &pipeline->nodes[ i ].op;
        (*op->free)(op);
	if (opImage && opImage != lastImage) {
	    if (opImage->priv)
		free((char *)opImage->priv);
	    free((char *)opImage);
	}
	lastImage = opImage;
    }

    if (pipeline->argList)
        FreeStringArgList(pipeline->argList);
    if (pipeline->nNodes)
        free((char *)pipeline->nodes);
    free((char *)pipeline);
}

static void
ShowPtrToXY(UChar *ptr, MemImagePtr image)
{
    int offset = ptr - image->pixels;
    int x = (offset % image->nextLine) / image->nextPixel;
    int y = offset / image->nextLine;

    Msg("0x%x ==> (%d,%d) in image 0x%x\n", ptr, x, y, image->pixels);
}

boolean
ImageStreamAtEof(ImageIOParamPtr ioParam)
{
    int newIndex;
    int fileIndex = ioParam->fileIndex;
    char *path;
    char *ioName;

    switch(ioParam->u.type) {
    case FLT_Single:
	if (fileIndex >= 1) {
	    return (True);
	} else {
	    return(False);
	}
	break;

    case FLT_Explicit:
	if (fileIndex >= ioParam->u.explicit.length) {
	    return (True);
	} else {
	    return(False);
	}
	break;

    case FLT_Implicit:
	newIndex = fileIndex + ioParam->u.implicit.start;
	if (newIndex > ioParam->u.implicit.stop) {
	    return(True);
	} else {
	    return(False);
	}
	break;

    default:
	return(True);
    }
}

InternalReasonPtr
GenerateNextFile(ImageIOParamPtr ioParam)
{
    static InternalReason NotEnoughFiles =  { "not enough output files\n",
					   IR_Unknown };
    int newIndex, firstIndex;
    char *path;
    char *ioName;
    InternalReasonPtr reason;

    ioParam->fileIndex++;

    switch(ioParam->u.type) {
    case FLT_Single:
	if (ioParam->fileIndex >= 1) {
	    if (ioParam->name == ImageIOReadFile)
	        return( &IRPipelineEof );
	    else
	        return( &NotEnoughFiles );
	}
	if (ioParam->u.single.buffer) {
	    sprintf(ioParam->u.single.buffer, "%s%s",
		    ioParam->fullPathPrefix, ioParam->u.single.path);
	    path = ioParam->u.single.buffer;
	} else
	    path = ioParam->u.single.path;
	break;

    case FLT_Explicit:
	if (ioParam->fileIndex >= ioParam->u.explicit.length) {
	    if (ioParam->name == ImageIOReadFile)
	        return( &IRPipelineEof );
	    else
	        return( &NotEnoughFiles );
	}
	if (ioParam->u.single.buffer) {
	    sprintf(ioParam->u.explicit.buffer, "%s%s",
		    ioParam->fullPathPrefix,
		    ioParam->u.explicit.list[ioParam->fileIndex]);
	    path = ioParam->u.explicit.buffer;
	} else
	    path = ioParam->u.explicit.list[ioParam->fileIndex];
	break;

    case FLT_Implicit:
	firstIndex = newIndex = ioParam->fileIndex + ioParam->u.implicit.start;
	if (newIndex > ioParam->u.implicit.stop) {
	    if (ioParam->name == ImageIOReadFile)
	        return( &IRPipelineEof );
	    else
	        return( &NotEnoughFiles );
	}

	while (newIndex <= ioParam->u.implicit.stop) {
	    if (newIndex > MaxFileIndex)
		return(CreateReason("generated index exceeds max: %d\n",
				    MaxFileIndex) );
	    if (ioParam->u.implicit.ndigits)
		sprintf(ioParam->u.implicit.buffer, "%s%s%0*d%s",
			ioParam->fullPathPrefix,
			ioParam->u.implicit.prefix,
			ioParam->u.implicit.ndigits,
			newIndex,
			ioParam->u.implicit.suffix);
	    else
		sprintf(ioParam->u.implicit.buffer, "%s%s%d%s",
			ioParam->fullPathPrefix,
			ioParam->u.implicit.prefix,
			newIndex,
			ioParam->u.implicit.suffix);
	    path = ioParam->u.implicit.buffer;
	    if (ioParam->name == ImageIOReadFile && FileType(path) == 0)
		newIndex++;
	    else
		break;
	}
	if (firstIndex != newIndex) {
	    if (newIndex - firstIndex == 1)
		reason = CreateReason("file number %d missing\n", firstIndex);
	    else
		reason = CreateReason("file numbers %d-%d missing\n",
				      firstIndex, newIndex-1);
	    if (newIndex > ioParam->u.implicit.stop)
		reason->hint = IR_BadIOFile;
	    else
		reason->hint = IR_Warning;
	    ioParam->fileIndex = newIndex - 1 - ioParam->u.implicit.start;
	    return(reason);
	}
	break;

    case FLT_Unspecified:
    default:
	ioName = (ioParam->name == ImageIOReadFile ? "input" : "output");
	return( CreateReason("%s file not specified\n", ioName) );
    }

    ioParam->currentPath = path;

    return(NULL);
}

void
CloseImageStream(ImageIOParamPtr ioParam)
{
    char *path = ioParam->currentPath;
    boolean isStdio;

    isStdio = (ioParam->stdioOk && path && strcmp(path,"<stdout>") == 0);
    if( isStdio && ioParam->name == ImageIOWriteFile ) {
	return;
    }
    if (ioParam->fp) {
	CloseBufferedFile(ioParam->fp);
	ioParam->fp = NULL;
    } else if (ioParam->fd >= 0) {
	CloseFile(ioParam->fd);
    }
    
    if (ioParam->pipePid) {
	WaitForPid(ioParam->pipePid,False);
	ioParam->pipePid = 0;
    }
    ioParam->fd = -1;
    ioParam->currentPath = NULL;
}

static InternalReasonPtr
_ReopenImageStream(ImageIOParamPtr ioParam, boolean buffered)
{
    char *currentPath = ioParam->currentPath;
 
    CloseImageStream(ioParam);
    ioParam->currentPath = currentPath;
    return( buffered ? OpenBufferedImageStream(ioParam)
	   : OpenImageStream(ioParam) );
}

InternalReasonPtr
ReopenImageStream(ImageIOParamPtr ioParam)
{
    return(_ReopenImageStream(ioParam,False));
}

InternalReasonPtr
ReopenBufferedImageStream(ImageIOParamPtr ioParam)
{
    return(_ReopenImageStream(ioParam,True));
}


InternalReasonPtr
OpenImageStream(ImageIOParamPtr ioParam)
{
    char *path;
    boolean isStdio;
    int fd;
    char *command = NULL;
    char *ioName = (ioParam->name == ImageIOReadFile ? "input" : "output");

    if (ioParam->fd >= 0)
	return(NULL);		/* already open */

    path = ioParam->currentPath;
    isStdio = (ioParam->stdioOk && path[0] == '-' && path[1] == '\0');

    /*
     * Open the specified file
     */
    if (ioParam->name == ImageIOReadFile) {
	if (isStdio) {
	    ioParam->currentPath = "<stdin>";
	    ioParam->fd = 0;
	    return(NULL);
	}
	fd = OpenFileForReading(path);
    } else { /* ImageIOWriteFile */
	if (isStdio) {
	    ioParam->currentPath = "<stdout>";
	    ioParam->fd = 1;
	    return(NULL);
	}
	fd = OpenFileForWriting(path);
    }

    if (fd < 0)
	return( CreateSysReason("Can't open %s for %s\n", path, ioName) );

    ioParam->fd = fd;
    ioParam->fp = NULL;
    WhenCodingForUnix(
        if (! isStdio) {
	    int pathLength = strlen(path);
    
	    if (pathLength >= 3 && strcmp(path+pathLength-2,".Z") == 0) {
	        if (ioParam->name == ImageIOReadFile)
		    command = "uncompress";
	        else
		    command = "compress";
	    }
    
	    if ((pathLength >= 3 && strcmp(path+pathLength-2,".z") == 0)
	    || (pathLength >= 4 && strcmp(path+pathLength-3,".gz") == 0)) {
	        if (ioParam->name == ImageIOReadFile)
		    command = "gunzip";
	        else
		    command = "gzip";
	    }
    
	    if (command) {
	        Argvec args;
	        args.length = 0;
	        args.path = command;   
	        AddArgv(&args, command, False);
		if (ioParam->name == ImageIOReadFile) {
		    InputPipe result;
		    result = PipeFromWithStdin(&args, fd);
		    CloseFile(fd);
		    ioParam->pipePid = result.pid;
		    ioParam->fd = result.inFd;
		} else {
		    OneWayPipe result;
		    result = PipeToWithStdout(&args, fd);
		    CloseFile(fd);
		    ioParam->pipePid = result.pid;
		    ioParam->fd = result.outFd;
		}
		FreeArgv(&args);
    	    }
    	}
    )
    return(NULL);
}

InternalReasonPtr
OpenBufferedImageStream(ImageIOParamPtr ioParam)
{
    InternalReasonPtr reason;
    int i;

    if (ioParam->fp)
	return (NULL);

tryAgain:
    if (reason = OpenImageStream(ioParam))
	return (reason);
    if (ioParam->name == ImageIOReadFile)
	ioParam->fp = ReopenBufferedFileForReading(ioParam->fd);
    else
	ioParam->fp = ReopenBufferedFileForWriting(ioParam->fd);
    if (ioParam->fp == NULL)
	return(&IRBadAllocation);
    
    if (ioParam->dataLength) {
	WhenCodingForUnix(
	    int fileType = FdFileType(ioParam->fd);

	    if (fileType == FILE_REGULAR) {
		(void) SeekFile(ioParam->fd, 0, SK_Start); /* always succeeds */
		ioParam->dataLength = 0;
	    }
	)
	for (i = ioParam->dataLength-1; i >= 0; i--) {
	    if (UngetBufferedChar(ioParam->unreadData[ i ], ioParam->fp) < 0)
		break;
	}
	if (i < 0)
	    ioParam->dataLength = 0;
    }

    if (ioParam->dataLength) {
	ioParam->dataLength = 0;
	if (reason = ReopenImageStream(ioParam))
	    return (reason);
	goto tryAgain;
    }

    return (NULL);
}

InternalReasonPtr
ReadImageStream(ImageIOParamPtr ioParam, char *pbuf, int totalBytes)
{
    int bytesIn, needed;
    FileHandle fd = ioParam->fd;
    InternalReasonPtr reason;
#if defined(__MWERKS__)
    extern read(int fd, char *, int needed);
#endif /* 0 */

    /*
     * there may be saved up data.
     */
    needed = totalBytes;
    if (ioParam->dataLength) {
	if (ioParam->dataLength > needed)
	    bytesIn = needed;
	else
	    bytesIn = ioParam->dataLength;

	memcpy(pbuf, ioParam->unreadData, bytesIn);
	needed -= bytesIn;
	pbuf += bytesIn;

	/*
	 * Copy down any left over data.
	 */
	if (ioParam->dataLength -= bytesIn)
	    memcpy(ioParam->unreadData,
		   ioParam->unreadData+bytesIn,
		   ioParam->dataLength);
    }

    while (needed) {
	bytesIn = read(fd, (char *)pbuf, needed);
        if (bytesIn < 0)
            return( CreateSysReason("can't read input image\n") );
        if (bytesIn == 0) {
	    CloseImageStream(ioParam);
            if (needed < totalBytes)
                return( &IRPrematureEof );
	    if (reason = GenerateNextFile(ioParam))
		return(reason);
	    if (reason = OpenImageStream(ioParam))
		return(reason);
	    fd = ioParam->fd;
        }
	needed -= bytesIn;
	pbuf += bytesIn;
    }
    return(NULL);
}

InternalReasonPtr
UnreadImageStream(ImageIOParamPtr ioParam, char *pbuf, int notNeeded)
{
    int newDataLength = ioParam->dataLength + notNeeded;

    ioParam->unreadData = InsecureNullRealloc(ioParam->unreadData,
					      newDataLength);
    if (ioParam->unreadData == NULL)
	return( &IRBadAllocation );
    memcpy(ioParam->unreadData + ioParam->dataLength, pbuf, notNeeded);
    ioParam->dataLength += notNeeded;

    return(NULL);
}

static MaxNameWidth;

void
PipelineTiming(ImagePipelinePtr pipeline, boolean enable)
{
    int i, width;
    char *name;

    if (enable) {
	if (pipeline->nodeClocks == NULL) {
	    pipeline->nodeClocks = (ULong *)
		InsecureAlloc(sizeof(ULong) * pipeline->nNodes);
	    if (pipeline->nodeClocks == NULL)
	    	return;
	}
	for (i = 0; i < pipeline->nNodes; i++) {
	    pipeline->nodeClocks[i] = 0;
	    name = EnumToName(pipeline->nodes[ i ].op.name, ImageOpNames);
	    width = strlen(name);
	    if (width > MaxNameWidth)
		MaxNameWidth = width;
	}
    } else {
	if (pipeline->nodeClocks)
	    free((char *)pipeline->nodeClocks);
	pipeline->nodeClocks = NULL;
    }
    RegisterTimerSummary(PipelineTimingSummary, (PrivPointer)pipeline, enable);
}

void
PipelineTimingSummary(PrivPointer pipelinePtr, ULong totalTime)
{
    int i;
    ImageOpPtr op;
    ImagePipelinePtr pipeline = (ImagePipelinePtr) pipelinePtr;

    for (i = 0; i < pipeline->nNodes; i++) {
        op = &pipeline->nodes[ i ].op;
	Msg("%*s (%d) times: %.3f (%%%.1f)\n",
	    MaxNameWidth,
	    EnumToName(op->name, ImageOpNames),
	    i, pipeline->nodeClocks[i] / 1000000.0,
	    (100.0 * pipeline->nodeClocks[i]) / totalTime);
    }
}
