static char *rcsid="$Header: /usr/local/src/nvrToMIT/RCS/nvrExtern.c,v 1.4 1995/11/16 22:18:32 russ Exp $";
/*
 * Copyright (c) 1992-1995 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	nvrExtern.c - Access to external file reader processes
 *
 * DESCRIPTION
 *	Implements access to external file readers via the external
 *	file library
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrExtern.c,v 1.4 1995/11/16 22:18:32 russ Exp $
 */


#include "nvrImageI.h"
#include "extImageIo.h"

static InternalReason NoExternWrite =
	{ "Cannot write extern file types (yet)\n" };

static CompositeMemImagePtr
createNvrExternImage(ImageSpecPtr spec,
		     CompositeMemImagePtr lastComp,
		     InternalReasonPtr *pReason)
{
    ExtImageIoPtr io = (ExtImageIoPtr) ((ImageIOParamPtr) spec)->priv;
    ExtImageInfo info;
    InternalReasonPtr reason;
    CompositeMemImagePtr image;

     if (*pReason = (*io->getinfo)(io, &info)) {
	return (NULL);
    }
    image = NewExtImage(&info, pReason);
    /* info.type is not set until NewExtImage() is called */
    spec->type = info.type;
    return (image);
}

static InternalReasonPtr 
nvrExternFlush(ImageOpPtr op, CompositeMemImagePtr in,
	       CompositeMemImagePtr out)
{
    return(NULL);
}

static void
nvrExternFree(ImageOpPtr op)
{

    ImageIOParamPtr ioParam = &op->u.io;
    ExtImageIoPtr io = (ExtImageIoPtr)ioParam->priv;

    CloseImageStream(ioParam);
    if (ioParam->u.type == FLT_Implicit)
	free(ioParam->u.implicit.buffer);
    (*io->close)(io);
    (*io->free)(io);
}

static InternalReasonPtr
nvrExternRead(ImageOpPtr op, CompositeMemImagePtr in, CompositeMemImagePtr out)
{
    ImageIOParamPtr ioParam = &op->u.io;
    ExtImageIoPtr io = (ExtImageIoPtr)ioParam->priv;

    return ((*io->read)(io,out));
}

InternalReasonPtr
NvrExternInit(ImageOpPtr op, CompositeMemImagePtr image)
{
    ImageIOParamPtr	ioParam = &op->u.io;
    InternalReasonPtr	reason = NULL;
    ExtImageIoPtr	io;
    ExtImageInfo	info;
    ExtImageConfig	config;
    ExtParseReadConfigPtr rp;

    if( ioParam->name == ImageIOWriteFile ) {
	return(&NoExternWrite);
    }

    memset(&config, 0, sizeof(ExtImageConfig));
    rp = &config.u.read;
    if (ioParam->flags & ResampleFpsSpecified) {
	rp->resampleFps = ioParam->resampleFps;
	rp->paramMask |= XFP_ResampleFpsMask;
    }
    if( ioParam->u.type != FLT_Single ) {
	if (ioParam->u.implicit.params & (1<<IFP_Prefix)) {
	    rp->prefix = ioParam->u.implicit.prefix;
	    rp->paramMask |= XFP_PrefixMask;
	}
	if (ioParam->u.implicit.params & (1<<IFP_Suffix)) {
	    rp->suffix = ioParam->u.implicit.suffix;
	    rp->paramMask |= XFP_SuffixMask;
	}
	if (ioParam->u.implicit.params & (1<<IFP_NDigits)) {
	    rp->ndigits = ioParam->u.implicit.ndigits;
	    rp->paramMask |= XFP_NdigitsMask;
	}
	if (ioParam->u.implicit.params & (1<<IFP_StartIndex)) {
	    rp->ndigits = ioParam->u.implicit.start;
	    rp->paramMask |= XFP_StartMask;
	}
	if (ioParam->u.implicit.params & (1<<IFP_StopIndex)) {
	    rp->ndigits = ioParam->u.implicit.stop;
	    rp->paramMask |= XFP_StopMask;
	}
    }

    if ((io = CreateExtImageIo(&reason)) == NULL) {
	return(reason);
    }
    if (reason = (*io->config)(io, &config, XC_Read)) {
	(*io->free)(io);
	return(reason);
    }
    if (reason = (*io->open)(io, ioParam->currentPath)) {
	(*io->free)(io);
	return(reason);
    }
    if (reason = (*io->getinfo)(io, &info)) {
	(*io->close)(io);
	(*io->free)(io);
	return (reason);
    }
    ioParam->priv = (PrivPointer) io;
    ioParam->spec.width = info.width;
    ioParam->spec.height = info.height;
    ioParam->spec.iLayout = info.iLayout;
    ioParam->spec.sample = info.sample;
    ioParam->spec.colorSpace = info.color;
    ioParam->spec.linesReversed = info.linesReversed;
    ioParam->spec.create = createNvrExternImage;

    op->proc = nvrExternRead;
    op->flush = nvrExternFlush;
    op->free = nvrExternFree;
    return(NULL);
}

InternalReasonPtr
GetNvrExternInfo(char *path, ImageFileInfoPtr imageInfo)
{
    InternalReasonPtr reason = NULL;
    ExtImageIoPtr io;
    ExtImageInfo info;

    imageInfo->type = IFT_Unknown;

    if ((io = CreateExtImageIo(&reason)) == NULL) {
	goto done;
    }
    if (reason = (*io->open)(io, path)) {
	(*io->free)(io);
	goto done;
    }
    reason = (*io->getinfo)(io, &info);
    (*io->close)(io);
    (*io->free)(io);

    if (reason) {
	goto done;
    }
    
    imageInfo->type = IFT_NvrExtern;
    imageInfo->width = info.width;
    imageInfo->height = info.height;
    imageInfo->color = info.color;
    imageInfo->sample = info.sample;
    imageInfo->interlaced = (info.iLayout != NotInterlaced);
    imageInfo->frameCount = info.frameCount;
    imageInfo->framesPerSecond = info.framesPerSecond;
    imageInfo->defined |= (IFI_FrameCount | IFI_FrameRate);

done:
    return(reason);
}
