/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	<nvrImage.h> - structure definitions of picture image.
 *
 * DESCRIPTION
 *	include file for just picture images without the other cruft.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrImage.h,v 2.45 1995/11/15 01:49:19 russ Exp $
 */
#ifndef _NVRIMAGE_H_
#define _NVRIMAGE_H_

#include "osio.h"
#include "fileContent.h"
#include "nvrString.h"
#include "nvrReason.h"

/*
 *	Forward declarations of structure typedefs
 */

typedef struct _imageOp ImageOp, *ImageOpPtr;
typedef struct _imageSpec ImageSpec, *ImageSpecPtr;
typedef struct _memImage MemImage, *MemImagePtr;
typedef struct _deinterlaceParam DeinterlaceParam, *DeinterlaceParamPtr;
typedef struct _filterParam FilterParam, *FilterParamPtr;
typedef struct _imageIOParam ImageIOParam, *ImageIOParamPtr;
typedef struct _geomRectangle GeomRectangle, *GeomRectanglePtr;
typedef struct _resampleParam ResampleParam, *ResampleParamPtr;
typedef struct _gridParam GridParam, *GridParamPtr;
typedef struct _colorParam ColorParam, *ColorParamPtr;
typedef struct _pipelineNode PipelineNode, *PipelineNodePtr;
typedef struct _imagePipeline ImagePipeline, *ImagePipelinePtr;
typedef struct _generateParam GenerateParam, *GenerateParamPtr;


/*
 * Structure declarations.
 */

/*
 **************************************************************************
 * 
 * Basic geometric structures for representing shapes and sizes
 * 
 **************************************************************************
 */
struct _geomRectangle {
    int x, y;
    int width, height;
} /* see typedef above */;

typedef struct _resampleRectangle {
    double x, y;
    long width, height;
} ResampleRectangle, *ResampleRectanglePtr;

typedef struct _geomDimension {
    ULong width;
    ULong height;
} GeomDimension, *GeomDimensionPtr;

/*
 **************************************************************************
 * 
 * These enums represent each of the unique image/movie file types
 * recognized by this image library.
 * 
 **************************************************************************
 */
typedef enum {
    IFT_Unspecified,
    IFT_Unknown,
    /*
     * Raw file types
     */
    IFT_RawYCrCb,
    IFT_RawYCbCr,
    IFT_RawRgb,
    IFT_MpegMbStats,
    IFT_Abekas,
    IFT_C3Filt,
    IFT_CompPerFile,	/* like that used in Andy Hung's Stanford encoder */
    IFT_Pix,
    IFT_Svj,		/* DVC's scsi/video jpeg movie format (it has */
			/* a non-unique header that looks like JFIF) */
    IFT_SunVideo,

    /*
     * other file types with identifying magic numbers
     */
    IFT_Jmovie,
    IFT_QuickTime,
    IFT_AVS,
    IFT_Vicar,
    IFT_XWD,
    IFT_GIF,
    IFT_Rle,
    IFT_JPEG,
    IFT_Miff,
    IFT_MTV,
    IFT_Pnm,
    IFT_SunRaster,
    IFT_Tiff,
    IFT_MpegVideo,
    IFT_MpegSystem,
    IFT_SgiRgb,
    IFT_Avi,
    IFT_NvrYuv,
    IFT_NvrIdx,
    IFT_SgiMovie,
    IFT_Omf,
    IFT_NvrExtern
} ImageFileType;

/*
 **************************************************************************
 * 
 * These enums represent the individual components of various color
 * spaces, the color space names, and the sample rates.
 * 
 **************************************************************************
 */
typedef enum {
    RgbRed,
    RgbGreen,
    RgbBlue,
    RgbAll	/* for ColorRGBLookup */
} RgbChannelName;

typedef enum {
    YcLuma,
    YcCr,
    YcCb
} YcChannelName;

typedef enum {
    RgbRedOrYcLuma,
    RgbGreenOrYcCr,
    RgbBlueOrYcCb,
    RgbAllOnly
} AnyChannelName;

typedef enum {
    ColorUnspecified,
    ColorUnknown,
    ColorYC,
    ColorRGB,
    ColorRGBLookup
} ColorSpace;

typedef enum {
    SampleUnspecified,
    SampleUnknown,
    Sample4_2_0,
    Sample4_2_2,
    Sample4_4_4
} SampleType;

typedef enum {
    DepthUnspecified,
    Rgb888,		/* 24-bit color */
    Rgb444,		/* 12-bit color */
    Rgb555,		/* 15-bit color (as on the mac) */
    Rgb232,		/* 7-bit color 2 bits red, 3 bits green, 2 bits blue */
    Rgb332,		/* 8-bit color 3 bits red, 3 bits green, 2 bits blue */
    GrayScale8,
    GrayScale7,
    GrayScale1,
    RgbC555,		/* 8-bit color, 5 red, 5 green, 5 blue */
    RgbC666		/* 8-bit color, 6 red, 6 green, 6 blue */
} RgbDepthType;

#define NRgbDepthTypes	(RgbC666+1)

/*
 * For Rgb888, Rgb555 and Rgb444, the image may be padded from 24 bits out
 * to 32 bits or from 12 bits out to 16 bits.  The 0 bits appear either on
 * either on the left or on the right and the order of the components
 * is either RGB or BGR.
 */
typedef enum {
    PixelRGB0,
    Pixel0RGB,
    PixelRGB,
    PixelBGR0,
    Pixel0BGR,
    PixelBGR
} RgbPixelComponents;

typedef enum {
    IFP_Unknown,

    /*
     * Implicit file name parameters
     */
    IFP_Prefix,
    IFP_Suffix,
    IFP_NDigits,
    IFP_StartIndex,
    IFP_StopIndex,

    /*
     * Image spec parameters
     */
    IFP_Type,
    IFP_Sample,
    IFP_Size,
    IFP_ColorSpace,
    IFP_Interlace,

    /*
     * Io direction
     */
    IFP_Read,
    IFP_Write,

    /*
     * File attribute parameters
     */
    IFP_FrameCount,
    IFP_FramesPerSec,
    IFP_ResampleFps
} ImageFileParam;

/*
 **************************************************************************
 *
 * FileListType
 *
 **************************************************************************
 */
typedef enum {
    FLT_Unspecified,
    FLT_Single,
    FLT_Explicit,
    FLT_Implicit
} FileListType;

typedef struct {
    FileListType type;
    char *buffer;	/* buffer to hold constructed file name */
    char *path;
} SingleFileList, *SingleFileListPtr;

typedef struct {
    FileListType type;
    char *buffer;	/* buffer to hold constructed file name */
    char *prefix;	/* file prefix */
    char *suffix;	/* file suffix */
    long start;		/* file name start index */
    long stop;		/* file name stop index */
    long ndigits;	/* number of digits in the name (0 ==> variable) */
    ULong params;	/* mask of parameters present */
} ImplicitFileList, *ImplicitFileListPtr;

typedef struct {
    FileListType type;
    long length;	/* length of list[] */
    char *buffer;	/* buffer to hold constructed file name */
    char **list;
} ExplicitFileList, *ExplicitFileListPtr;



/*
 **************************************************************************
 *
 * The MemImage and CompositeMemImage comprise the core object operated
 * on by the image library.  The ImageSpec is used to prototype the
 * type of image used in each node of the pipeline.
 *
 **************************************************************************
 */
struct _memImage {
    UChar *pixels;		/* pointer to 0,0 of image */
    GeomDimension size;		/* width, height of image */
    long nextPixel;		/* accelerator to next pixel in image */
    long nextLine;		/* accelerator to next line in image */
    MemImagePtr field2Image;	/* the corresponding image in field2 (if any) */
    union {
	RgbChannelName rgbName;
	YcChannelName ycName;
	AnyChannelName anyName;		/* XXX never, ever assign this! */
    } u;
} /* typedef: MemImage, *MemImagePtr */;

typedef struct _compositeMemImage {
    UChar *header;		/* header, if any */
    UChar *pixels;		/* contains all images, null in field2 */
    long pixelBytes;		/* total bytes for all images */
    GeomDimension size;		/* (overall) width, height of image */
    long nImage;		/* # of MemImages in images (and field2Images)*/
    MemImagePtr images;		/* array of image descriptions */
    MemImagePtr field2Images;	/* field 2 (null if deinterlaced) */
    SampleType sample;		/* sample type */
    ColorSpace colorSpace;	/* color space for images */
    PrivPointer priv;		/* private storage for operators */
} CompositeMemImage, *CompositeMemImagePtr;

/*
 **************************************************************************
 *
 * constants for filtering, resampling and io
 *
 **************************************************************************
 */
typedef enum {
    ResampleUnspecified,
    ResampleUnknown,
    ResampleNearest,		/* drop or duplicate pixels */
    ResampleBiLinear,		/* weighted average of 4 nearest neighbors */
    ResampleCubic		/* unimplemented */
} ResampleMethodName;

typedef enum {
    FilterUnspecified,
    FilterUnknown,
    FilterIntMatrix,
    FilterRealMatrix
} FilterName;

typedef enum {
    FSNone,
    FSDecimate,
    FSDecimateHoriz,
    FSDecimateVert,
    FSInterpolateHoriz,
    FSInterpolateVert
} FilterScale;

typedef enum {
    ImageIOUnspecified,
    ImageIOUnknown,
    ImageIOReadFile,
    ImageIOWriteFile
} ImageIOName;

typedef enum {
    ImgOpUnspecified,
    ImgOpUnknown,
    ImgOpIO,
    ImgOpResample,
    ImgOpColor,
    ImgOpFilter,
    ImgOpGenerate,
    ImgOpGrid,
    ImgOpDeinterlace
} ImageOpName;

typedef enum {
    UnknownInterlace = -1,
    NotInterlaced = 0,
    InterlacedByLine,
    InterlacedByPictOdd,
    InterlacedByPictEven
} InterlaceLayoutType;
#define InterlacedByPict InterlacedByPictEven

/*
 * These define the basic filter operations
 */
typedef UChar (*PixelFilterFunc)(MemImagePtr, int, int, FilterParamPtr);
typedef void (*ImageFilterFunc)(MemImagePtr, MemImagePtr, GeomRectanglePtr,
				FilterParamPtr);

/*
 * This is historical
 */
typedef InternalReason ImageReason, *ImageReasonPtr;

enum {
    IFI_BitRate    = 0x01,
    IFI_FrameCount = 0x02,
    IFI_FrameRate  = 0x04,
    IFI_QFactor    = 0x08
};

typedef struct _ImageFileInfo {
    /*
     * These items are guaranteed to be defined by the image file
     * format.
     */
    ImageFileType type;
    ULong	hdrSize;
    ULong	contentSize;	/* size of movie data */
    long	width;
    long	height;
    ColorSpace	color;		/* color space - Color{YC,RGB,etc...} */
    SampleType	sample;		/* sample type - 420, 422, 444  */
    boolean	interlaced;	/* whether the image is interlaced or not */

    ULong	defined;	/* mask of which items below are defined */

    /*
     * These items may not be defined by the file format
     */
    long	bitRate;	/* -1 means variable bitrate */
    long	frameCount;
    float	framesPerSecond;
    long	qFactor;	/* defined for jpeg movies */
} ImageFileInfo, *ImageFileInfoPtr;

/*
 *	EXTERNS
 */

CExternal InternalReasonPtr CreateReason(
    char *,
    ...
);

CExternal InternalReasonPtr CreateSysReason(
    char *,
    ...
);

CExternal CompositeMemImagePtr NewCompositeMemImage(
    int,			/* data size */
    int,			/* header size */
    int,			/* nImage */
    SampleType,
    ColorSpace,
    InterlaceLayoutType
);

/*
 * Image/movie header scanning.
 */

CExternal InternalReasonPtr GetImageFileInfo(
    char *,
    FileContentType hint,
    ImageFileInfoPtr
);

/*
 * Pipeline processing
 */
CExternal InternalReasonPtr AddOpTemplate(ImageOpPtr);
CExternal InternalReasonPtr InitImagePipeline(ImagePipelinePtr);
CExternal InternalReasonPtr ProcessPipeline(ImagePipelinePtr);
CExternal InternalReasonPtr FlushPipeline(ImagePipelinePtr);

CExternal ImagePipelinePtr CreatePipelineFromArgs(
    int,			/* argc */
    char **,			/* argv */
    InternalReasonPtr *		/* failure reason */
);

CExternal void PipelineTiming(ImagePipelinePtr, boolean enable);
CExternal void PipelineTimingSummary(PrivPointer pipelinePtr, ULong totalTime);
CExternal void FreeImagePipeline(ImagePipelinePtr);

/*
 * Error recovery
 */
CExternal InternalReasonPtr CreateReason(char *, ...);
CExternal void FreeReason(InternalReasonPtr);
CExternal void TtyShowReason(InternalReasonPtr);

/*
 * Obsolete
 */
CExternal void ResizeOneChannel(
    UChar *,	/* source image */
    int,	/* source width */
    int,	/* source height */
    UChar *dst, /* destination image */
    int,	/* destination width */
    int		/* destination height */
);

CExternal void RGBtoYUV(
    int, int,			/* width, height */
    UChar *, UChar *, UChar *,	/* red, green blue */
    int,			/* next rgb pixel */
    UChar *, UChar *, UChar *,	/* y, cr, cb */
    int				/* next yuv pixel */
);

void ResampleCompositeChannel(
    MemImagePtr,
    MemImagePtr
);

/*
 * operator templates.  By referencing any of these in your program
 * that will bring in the corresponding code at link time.
 */
extern ImageOpPtr ImageIoTemplate;
extern ImageOpPtr ImageResampleTemplate;
extern ImageOpPtr ImageColorTemplate;
extern ImageOpPtr ImageGridTemplate;
extern ImageOpPtr ImageFilterTemplate;
extern ImageOpPtr ImageDeinterlaceTemplate;
extern ImageOpPtr ImageGenerateTemplate;
/*
 * Various, shared NameEnumPairs
 */
extern NameEnumPair ImageFileTypes[];
extern NameEnumPair ColorSpaceNames[];
extern NameEnumPair BooleanNames[];
extern NameEnumPair ImageOpNames[];
extern NameEnumPair SampleTypeNames[];
extern NameEnumPair RgbDepthTypeNames[];
extern NameEnumPair RgbPixelComponentsNames[];
extern NameEnumPair FileParamNames[];
extern NameEnumPair ResampleMethodNames[];
extern NameEnumPair InterlaceLayoutNames[];

/*
 * common failure reasons
 */
extern InternalReason IRBadImageFileType;

#endif /* _NVRIMAGE_H_ */
