/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	<nvrImageI.h> - structure definitions private to the image library.
 *
 * DESCRIPTION
 *	internal structures
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrImageI.h,v 1.47 1995/11/15 01:49:19 russ Exp $
 */
#ifndef _NVRIMAGEI_H_
#define _NVRIMAGEI_H_

#include "nvrImage.h"

/*
 **************************************************************************
 *
 * This is the basic specification for a prototype image before it
 * gets created.
 *
 **************************************************************************
 */

struct _imageSpec {
    ImageFileType type;		/* named abstraction; e.g., abekas */
    long width;			/* width of full image */
    long height;		/* height of full image */
    InterlaceLayoutType iLayout;/* describes interlacing for the frame */
    SampleType sample;		/* how the components are sub-sampled */
    ColorSpace colorSpace;	/* colorspace of the image */
    CompositeMemImagePtr	/* function to create image */
	(*create)(ImageSpecPtr, CompositeMemImagePtr, InternalReasonPtr *);
    long headerSize;		/* size of image header */
    union {
	RgbPixelComponents rgbLayout;
    } u;
    boolean linesReversed;
}; /* typedef: ImageSpec, *ImageSpecPtr */

/*
 **************************************************************************
 *
 * Internal structures for deinterlacing
 *
 **************************************************************************
 */

#define DefDeiFilterWidth	5

typedef enum {
    DI_Unspecified,
    DI_Unknown,
    DI_Copy,
    DI_CopyField2,
    DI_Black,
    DI_Spatial
} DeinterlaceType;

struct _deinterlaceParam {
    ImageSpec spec;	   	/* output image spec (must be first) */
    int width;			/* width of the sample filter */
    GeomRectangle inClip;	/* clipping on area filtered */
    int minXFullWidth;		/* min x value to do full-width filtering */
    int maxXFullWidth;		/* max x value to do full-width filtering */
    int startingTopIndex;	/* accelerator: precalculated starting index */
    DeinterlaceType method;
    void  (*deinterlaceMemImage)(MemImagePtr, MemImagePtr, GeomRectanglePtr,
				DeinterlaceParamPtr);
    UChar (*deinterlacePixel)(MemImagePtr, int, UChar *, DeinterlaceParamPtr);
}; /* typedef: DeinterlaceParam, *DeinterlaceParamPtr */

/*
 **************************************************************************
 *
 * Internal structures for filtering
 *
 **************************************************************************
 */
struct _resampleParam {
    ImageSpec spec;	   	   /* output image spec (must be first) */
    ResampleMethodName method;	   /* how to do the resampling */
    ResampleRectangle realInClip;  /* source rectangle */
    GeomRectangle outClip;	   /* destination rectangle */
    void (*resampler)(MemImagePtr,
		      MemImagePtr,
		      ResampleRectanglePtr,
		      GeomRectanglePtr);
}; /* typedef: ResampleParam, *ResampleParamPtr */

typedef enum {
    PFTooManyCoefficients,
    PFTooFewCoefficients,
    PFNoDenominator,
    PFCantParse,
    PFok
} MatrixParserResult;

/*
 **************************************************************************
 *
 * Internal structures for image generation
 *
 **************************************************************************
 */
typedef enum {
    NM_Unspecified,
    NM_Unknown,
    NM_Random,
    NM_Static
} NoiseMethod;

struct _generateParam {
    ImageSpec spec;			/* output image spec (must be first) */
    ULong seed;				/* seed for random number generator */
    ULong count;			/* number of frames to generate */
    ULong vertNoise;			/* vertical noise value */
    ULong horizNoise;			/* horizontal noise value */
    UChar rightShift;			/* for shifting */
    UChar mask;				/* for masking noise after shift */
    NoiseMethod method;			/* how to generate the noise */
    UShort flags;			/* to keep track of what is set */
    void (*generateProc)(GenerateParamPtr, MemImagePtr);
}; /* typedef: GenerateParam, *GenerateParamPtr */

/*
 **************************************************************************
 *
 * Internal structures for color conversion
 *
 **************************************************************************
 */
typedef enum {
    DM_Unspecified,
    DM_Unknown,
    DM_None,
    DM_Random,
    DM_MagicSquare
} DitherMethod;

struct _colorParam {
    ImageSpec spec;			/* output image spec (must be first) */
    ULong flags;			/* keeps track of whats specified */
    int nColors;			/* used with ColorRGBLookup */
    DitherMethod method;		/* how to dither */
    void (*rateProc)(MemImagePtr,MemImagePtr,MemImagePtr,
		     MemImagePtr,MemImagePtr,MemImagePtr);
}; /* typedef: ColorParam, *ColorParamPtr */

/*
 **************************************************************************
 *
 * Internal structures for putting a grid on a picture
 *
 **************************************************************************
 */
struct _gridParam {
    ImageSpec spec;			/* output image spec (must be first) */
    int lineWidth;			/* width of the line */
    GeomDimension gridSize;		/* size of each grid box */
    UChar lineColors[ 5 ];		/* array of colors for the line */
}; /* typedef: GridParam, *GridParamPtr */


/*
 **************************************************************************
 *
 * Internal structures for filtering
 *
 **************************************************************************
 */
#define MAX_RESAMPLE_DIM	9	/* matrix never bigger (in w & h) */

typedef struct _intMatrix {
    short denominator;
    short numerator[ MAX_RESAMPLE_DIM * MAX_RESAMPLE_DIM ];
} IntMatrix, *IntMatrixPtr;

struct _filterParam {
    ImageSpec spec;			/* output image spec (must be first) */
    FilterName name;
    FilterScale scale;			/* interpolate, decimate or none */
    GeomRectangle inClip;		/* clipping on area filtered */
    GeomDimension size;			/* matrix size */
    PixelFilterFunc pixelFilter;
    ImageFilterFunc imageFilter;
    /*
     * Which element of this union is used, if any, depends on 'name'.
     */
    union {
	IntMatrix iCoeff;
	float rCoeff[ MAX_RESAMPLE_DIM * MAX_RESAMPLE_DIM ];
    } u;
}; /* typedef: FilterParam, *FilterParamPtr */


/*
 **************************************************************************
 *
 * Internal structures for image io
 *
 **************************************************************************
 */

#define MaxImplicitDigits	6
#define MaxFileIndex		999999	/* must agree with MaxImplicitDigits */

typedef enum {
    EofSample,
    RepeatSample,
    UseSample,
    DropSample
} SampleState;

typedef enum {
    FrameRateSpecified = 0x0001,
    ResampleFpsSpecified = 0x0002
} IoParamFlags;

struct _imageIOParam {
    ImageSpec spec;		/* output image spec (must be first) */
    ImageIOName name;
    boolean stdioOk;
    union {
	FileListType type;
	SingleFileList single;
	ImplicitFileList implicit;
	ExplicitFileList explicit;
    } u;
    long fileIndex;		/* counter for files processed */
    char *fullPathPrefix;	/* an nvr_idx file may specify this */
    char *currentPath;		/* the current path name being operated on */
    FileHandle fd;		/* file descriptor */
    BufFileHndl fp;		/* optional buffered file pointer */
    int pipePid;
    char *unreadData;
    long dataLength;		/* length of unreadData */
    PrivPointer priv;		/* private to specific file translators */

    /*
     * These are used for calculating frame samples
     */
    float frameRate;		/* frame rate for this node */
    float resampleFps;		/* This node should changed fps to this */
    long inFrameIndex;		/* input frame index */
    long outFrameIndex;		/* output frame index */
    float dropRate;		/* rate at which frames are dropped/repeated */
    InternalReasonPtr (*ioFunc)	/* the *real* io func when doing changeFps io */
	(ImageOpPtr, CompositeMemImagePtr, CompositeMemImagePtr);
    ULong flags;		/* keep track of what's specified, what's not */
}; /* typedef: ImageIOParam, *ImageIOParamPtr */

/*
 **************************************************************************
 *
 * the ImageOp and the PipeLineNode comprise the mechanism for representing
 * operations and their placement withint the pipeline.
 *
 **************************************************************************
 */

typedef struct _anyParam {
    ImageSpec spec;		/* output image spec (must be first) */
} AnyParam, *AnyParamPtr;

struct _imageOp {
    ImageOpName name;
    InternalReasonPtr (*parse)	/* parse the arguments for the node */
	(PipelineNodePtr, StringArgListPtr *);
    InternalReasonPtr (*init)		/* init the image op */
	(ImageOpPtr, CompositeMemImagePtr);
    InternalReasonPtr (*proc)		/* process the node */
	(ImageOpPtr, CompositeMemImagePtr, CompositeMemImagePtr);
    InternalReasonPtr (*flush)		/* flush the node (at the end) */
	(ImageOpPtr, CompositeMemImagePtr, CompositeMemImagePtr);
    void (*free)(ImageOpPtr);		/* free the node */
	
    boolean canProcessInPlace;	/* true if the 'out' and 'in' can be same */
    boolean enforceIsometric;	/* true if in and out must be same size */
    boolean dimensionInherited;	/* true if width/height inherited */
    boolean rgbLookupOk;	/* true if rgb lookup images are ok */
    boolean oddWidthOk;		/* true if alloced width is even */
    /*
     * Which element of this union is used depends on 'name'.
     */
    union {
	AnyParam any;
	ResampleParam resample;
	FilterParam filter;
	DeinterlaceParam deinterlace;
	ImageIOParam io;
	GridParam grid;
	ColorParam color;
	GenerateParam generate;
    } u;
}; /* typedef: ImageOp, *ImageOpPtr */

/*
 * XXX - in the image create routines, the original ImageOp is not
 * available, but the ImageSpec is, and its within the ImageOp.  This
 * converts an ImageSpec pointer to am ImageOp pointer.
 */
#define ImageSpecToImageOp(specPtr)	\
    ((ImageOpPtr) (UCharP(specPtr) - (ULong)&((ImageOpPtr)0)->u.any))

struct _pipelineNode {
    ImageOp op;			/* description of operation */
    CompositeMemImagePtr out;	/* output image for this node */
}; /* typedef: PipelineNode, *PipelineNodePtr */

struct _imagePipeline {
    int nNodes;			/* count of nodes in the pipeline */
    StringArgListPtr argList;	/* allocated args for this pipeline */
    PipelineNodePtr nodes;	/* list of nodes */
    CompositeMemImagePtr in;	/* input image, possibly null */
    ULong *nodeClocks;		/* list of clocks to accumulate time */
    void (*startop)(ImageOpPtr, CompositeMemImagePtr, CompositeMemImagePtr);
    void (*endop)(ImageOpPtr, CompositeMemImagePtr, CompositeMemImagePtr);
    void (*startPipe)(void);
    void (*endPipe)(void);
}; /* typedef: ImagePipeline, *ImagePipelinePtr */


/*
 *	EXTERNS
 */

/*
 * Create routines
 */
CExternal CompositeMemImagePtr NewLookupImage(
    ImageSpecPtr,
    CompositeMemImagePtr,	/* the image from the previous pipline node */
    InternalReasonPtr *
);

CExternal CompositeMemImagePtr NewRgbImage(
    ImageSpecPtr,
    CompositeMemImagePtr,	/* the image from the previous pipline node */
    InternalReasonPtr *
);

CExternal CompositeMemImagePtr NewYcImage(
    ImageSpecPtr,
    CompositeMemImagePtr,	/* the image from the previous pipline node */
    InternalReasonPtr *
);

CExternal CompositeMemImagePtr NewAbekasImage(
    ImageSpecPtr,
    CompositeMemImagePtr,	/* the image from the previous pipline node */
    InternalReasonPtr *
);

CExternal CompositeMemImagePtr CopyImageByReference(
    ImageSpecPtr,
    CompositeMemImagePtr,
    InternalReasonPtr *
);

CExternal CompositeMemImagePtr NewAviRgbImage(
    ImageSpecPtr spec,
    CompositeMemImagePtr lastComp,
    InternalReasonPtr *
);


CExternal UChar *InitMemImage(
    MemImagePtr,	/* image	*/
    UChar *,		/* pixels	*/
    int,		/* channelName	*/
    ColorSpace,		/* colorSpace	*/
    int,		/* width	*/
    int,		/* height	*/
    int,		/* nextPixel	*/
    int,		/* nextLine	*/
    MemImagePtr		/* field2Image	*/
);

/*
 * For clip manipulation.
 */
CExternal GeomRectanglePtr ScaleClip(GeomRectanglePtr,
				     GeomDimensionPtr,
				     GeomDimensionPtr,
				     GeomRectanglePtr);

/*
 * Internal node manipulation routines
 */
CExternal PipelineNodePtr NewNodeForPipeline(
    ImagePipelinePtr,		/* the current pipeline */
    ImageOpName,		/* the operation for this node */
    InternalReasonPtr *pReason
);

CExternal boolean AddTemplateToSpec(
    char *,			/* template name */
    ImageSpecPtr		/* specification to modify */
);

/*
 * Misc.
 */
CExternal ImageSpecPtr ImageTypeToImageSpec(
    ImageFileType
);

/*
 * Internal token parsing
 */
CExternal SampleType ParseSampleType(char *);
CExternal ResampleMethodName ParseResampleMethodName(char *);
CExternal InternalReasonPtr NvrIdxParse(PipelineNodePtr, StringArgListPtr *);
CExternal InternalReasonPtr NvrIdxHdr(char *file, ImageFileInfoPtr);


/*
 * Default functions for parse, init, flush and free
 */
CExternal InternalReasonPtr FailParse(PipelineNodePtr, StringArgListPtr *);
CExternal InternalReasonPtr SimpleOpInit(ImageOpPtr, CompositeMemImagePtr);
CExternal InternalReasonPtr NopFlush(ImageOpPtr,
			    CompositeMemImagePtr, CompositeMemImagePtr);
CExternal void NopFree(ImageOpPtr);

/*
 * io functions
 */
CExternal InternalReasonPtr ReadImageStream(
    ImageIOParamPtr,		/* the io param data */
    char *,			/* pointer to the data */
    int);			/* length of the data */

CExternal InternalReasonPtr UnreadImageStream(
    ImageIOParamPtr,		/* the io param data */
    char *,			/* pointer to the data */
    int);			/* length of the data */

CExternal boolean ImageStreamAtEof( ImageIOParamPtr );
CExternal InternalReasonPtr GenerateNextFile( ImageIOParamPtr );
CExternal InternalReasonPtr OpenImageStream( ImageIOParamPtr );
CExternal InternalReasonPtr ReopenBufferedImageStream( ImageIOParamPtr );
CExternal InternalReasonPtr OpenBufferedImageStream( ImageIOParamPtr );
CExternal InternalReasonPtr ReopenImageStream( ImageIOParamPtr );
CExternal InternalReasonPtr ReopenBufferedImageStream( ImageIOParamPtr );
CExternal InternalReasonPtr InferTypeFromHeader(ImageOpPtr op);
CExternal void CloseImageStream( ImageIOParamPtr );
CExternal SampleState IncrementIoSample(ImageIOParamPtr);
CExternal ImageFileType PerformExternMap(ImageFileType type);

/*
 * Proc functions
 */
CExternal InternalReasonPtr FailProc(ImageOpPtr,
			    CompositeMemImagePtr, CompositeMemImagePtr);


extern ImageFileType ExternImageMap[];
#endif /* _NVRIMAGEI_H_ */
