/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	imageToken.c - parse tokens associated with the image library
 *
 * DESCRIPTION
 *	<overall description of what the c code does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/sgilibimage.c,v 1.5 1994/12/18 07:24:43 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "nvrImageI.h"
#include "nvrAlloc.h"
#include "nvrString.h"
#include "osio.h"
#include "sgilibimage.h"

/*
 * taken from stdio.h...
 */
#define	_IOREAD	0001
#define	_IOWRT	0002
#define	_IOEOF	0020
#define	_IOERR	0040
#define	_IORW	0400

#define EOF	(-1)

static void
cvtlongs(register long buffer[], register long n)
{
    register short i;
    register long nlongs = n>>2;
    register ULong lwrd;

    for(i=0; i<nlongs; i++) {
	lwrd = buffer[i];
	buffer[i] =     ((lwrd>>24) 		| 
	   		(lwrd>>8 & 0xff00) 	| 
	   		(lwrd<<8 & 0xff0000) 	| 
	   		(lwrd<<24) 		);
    }
}

static long
reverse(register ULong lwrd) 
{
    return ((lwrd>>24) 		| 
	   (lwrd>>8 & 0xff00) 	| 
	   (lwrd<<8 & 0xff0000) | 
	   (lwrd<<24) 		);
}

static void
cvtshorts(register UShort *buffer, register long n)
{
    register short i;
    register long nshorts = n>>1;
    register UShort swrd;

    for(i=0; i<nshorts; i++) {
	swrd = *buffer;
	*buffer++ = (swrd>>8) | (swrd<<8);
    }
}

void
cvtimage(register long *buffer)
{
    cvtshorts((UShort *)buffer,12L);
    cvtlongs(buffer+3,12L);
    cvtlongs(buffer+26,4L);
}

static UShort *
ibufalloc(SgiRgbImage *image)
{
    return (UShort *)InsecureAlloc(IBUFSIZE(image->hdr.xsize));
}

static int
img_badrow(SgiRgbImage *image, unsigned int y, unsigned int z)
{
    if(y>=image->hdr.ysize || z>=image->hdr.zsize)
	return 1;
    else
        return 0;
}

static int
img_write(SgiRgbImage *image, char *buffer,int count)
{
    int retval;

    retval =  WriteData(image->file,buffer,count);
    if(retval == count) 
	image->offset += count;
    else
	image->offset = -1;
    return retval;
}

static int
img_read(SgiRgbImage *image, char *buffer, int count)
{
    int retval;

    retval =  ReadData(image->file,buffer,count);
    if(retval == count) 
	image->offset += count;
    else
	image->offset = -1;
    return retval;
}

static ULong
img_optseek(SgiRgbImage *image, ULong offset)
{
    if(image->offset != offset) {
       image->offset = offset;
       return ((ULong) SeekFile(image->file,offset,SK_Start));
   }
   return offset;
}

static ULong
img_seek(SgiRgbImage *image, unsigned int y, unsigned int z)
{
    SgiRgbHeaderPtr hdr = &image->hdr;

    if(img_badrow(image,y,z)) {
	Msg("img_seek: row number out of range\n");
	return EOF;
    }
    image->x = 0;
    image->y = y;
    image->z = z;
    if(ISVERBATIM(hdr->type)) {
	switch(hdr->dim) {
	    case 1:
		return img_optseek(image, 512L);
	    case 2: 
		return img_optseek(image,512L+(y*hdr->xsize)*BPP(hdr->type));
	    case 3: 
		return img_optseek(image,
		    512L+(y*hdr->xsize+z*hdr->xsize*hdr->ysize)*
							BPP(hdr->type));
	    default:
		Msg("img_seek: weird dim\n");
		break;
	}
    } else if(ISRLE(hdr->type)) {
	switch(hdr->dim) {
	    case 1:
		return img_optseek(image, image->rowstart[0]);
	    case 2: 
		return img_optseek(image, image->rowstart[y]);
	    case 3: 
		return img_optseek(image, image->rowstart[y+z*hdr->ysize]);
	    default:
		Msg("img_seek: weird dim\n");
		break;
	}
    } else 
	Msg("img_seek: weird image type\n");
    return((ULong)-1);
}

static int
iflush(SgiRgbImage *image)
{
    UShort *base;

    if ( (image->flags&_IOWRT)
     && (base=image->base)!=NULL && (image->ptr-base)>0) {
	    if (putrow(image, base, image->y,image->z)!=image->hdr.xsize) {
		    image->flags |= _IOERR;
		    return(EOF);
	    }
    }
    return(0);
}

int
iclose(SgiRgbImage *image)
{
    long tablesize;
    SgiRgbHeaderPtr hdr = &image->hdr;

    iflush(image);
    img_optseek(image, 0);
    if (image->flags&_IOWRT) {
	if(image->dorev)
	    cvtimage((long *)image);
	if (img_write(image,(char *)image,sizeof(SgiRgbImage)) != sizeof(SgiRgbImage)) {
	    Msg("iclose: error on write of image header\n");
	    return EOF;
	}
	if(image->dorev)
	    cvtimage((long *)image);
	if(ISRLE(hdr->type)) {
	    img_optseek(image, 512L);
	    tablesize = hdr->ysize*hdr->zsize*sizeof(long);
	    if(image->dorev)
		cvtlongs((long*)image->rowstart,tablesize);
	    if (img_write(image,(char *)(image->rowstart),tablesize) != tablesize) {
		Msg("iclose: error on write of rowstart\n");
		return EOF;
	    }
	    if(image->dorev)
		cvtlongs(image->rowsize,tablesize);
	    if (img_write(image,(char *)(image->rowsize),tablesize) != tablesize) {
		Msg("iclose: error on write of rowsize\n");
		return EOF;
	    }
	}
    }
    if(image->base) {
	free((char *)image->base);
	image->base = 0;
    }
    if(image->tmpbuf) {
	free((char *)image->tmpbuf);
	image->tmpbuf = 0;
    }
    if(ISRLE(hdr->type)) {
	free((char *)image->rowstart);
	image->rowstart = 0;
	free((char *)image->rowsize);
	image->rowsize = 0;
    }

    CloseFile(image->file);
    return(0);
}

static int
ifilbuf(SgiRgbImage *image)
{
	int size;
	SgiRgbHeaderPtr hdr = &image->hdr;

	if ((image->flags&_IOREAD) == 0)
		return(EOF);
	if (image->base==NULL) {
		size = IBUFSIZE(hdr->xsize);
		if ((image->base = ibufalloc(image)) == NULL) {
			Msg("can't alloc image buffer\n");
			return EOF;
		}
	}
	image->cnt = getrow(image,image->base,image->y,image->z);
	image->ptr = image->base;
	if (--image->cnt < 0) {
		if (image->cnt == -1) {
			image->flags |= _IOEOF;
			if (image->flags & _IORW)
				image->flags &= ~_IOREAD;
		} else
			image->flags |= _IOERR;
		image->cnt = 0;
		return -1;
	}
	if(++image->y >= hdr->ysize) {
	    image->y = 0;
	    if(++image->z >= hdr->zsize) {
		image->z = hdr->zsize-1;
		image->flags |= _IOEOF;
		return -1;
	    }
	}
	return *image->ptr++ & 0xffff;
}

static unsigned int
iflsbuf(SgiRgbImage *image, unsigned int c)
{
	register UShort *base;
	register n, rn;
	int size;
	SgiRgbHeaderPtr hdr = &image->hdr;

	if ((image->flags&_IOWRT)==0)
		return(EOF);
	if ((base=image->base)==NULL) {
		size = IBUFSIZE(hdr->xsize);
		if ((image->base=base=ibufalloc(image)) == NULL) {
			Msg("flsbuf: error on buf alloc\n");
			return EOF;
		}
		rn = n = 0;
	} else if ((rn = n = image->ptr - base) > 0)  {
			n = putrow(image,base,image->y,image->z);
			if(++image->y >= hdr->ysize) {
			    image->y = 0;
			    if(++image->z >= hdr->zsize) {
				image->z = hdr->zsize-1;
				image->flags |= _IOEOF;
				return -1;
			    }
			}
 	}
	image->cnt = hdr->xsize-1;
	*base++ = c;
	image->ptr = base;
	if (rn != n) {
		image->flags |= _IOERR;
		return(EOF);
	}
	return(c);
}

static void
isetname(SgiRgbImage *image, char *name)
{
    strncpy(image->hdr.name,name,80);
}

static void
isetcolormap(SgiRgbImage *image, int colormap)
{
    image->hdr.colormap = colormap;
}

SgiRgbImage *
imgopen(int f, char *file, char *mode,
		unsigned int type, unsigned int dim,
		unsigned int xsize, unsigned int ysize, unsigned int zsize)
{
	SgiRgbImagePtr 	image;
	SgiRgbHeaderPtr	hdr;
	register rw;
	int tablesize;
	register int i, max;

	image = (SgiRgbImage*)calloc(1,sizeof(SgiRgbImage));
	if(!image ) {
	    Msg("iopen: error on image struct alloc\n");
	    return NULL;
	}
	hdr = &image->hdr;
	rw = mode[1] == '+';
	if(rw) {
	    Msg("iopen: read/write mode not supported\n");
		return NULL;
	}
	if (*mode=='w') {
		if (file)
		    f = OpenFileForWriting(file);
		if (f < 0) {
		    Msg("iopen: can't open output file %s\n",file);
		    return NULL;
		}
		hdr->imagic = IMAGIC;
		hdr->type = type;
		hdr->xsize = xsize;
		hdr->ysize = 1;
		hdr->zsize = 1;
		if (dim>1)
		    hdr->ysize = ysize;
		if (dim>2)
		    hdr->zsize = zsize;
		if(hdr->zsize == 1) {
		    hdr->dim = 2;
		    if(hdr->ysize == 1)
			hdr->dim = 1;
		} else {
		    hdr->dim = 3;
		}
		hdr->min = 10000000;
		hdr->max = 0;
		isetname(image,"no name"); 
		hdr->wastebytes = 0;
		image->dorev = 0;
		if (! WriteData(f, (char *)image, sizeof(SgiRgbImage))) {
		    Msg("iopen: error on write of image header\n");
		    return NULL;
		}
	} else {
		if (file)
		    f = OpenFileForReading(file);
		if (f < 0)
		    return(NULL);
		if (! ReadData(f, (char *)image, sizeof(SgiRgbImage))) {
		    Msg("iopen: error on read of image header\n");
		    return NULL;
		}
		if( ((hdr->imagic>>8) | ((hdr->imagic&0xff)<<8)) 
							     == IMAGIC ) {
		    image->dorev = 1;
		    cvtimage((long *)image);
		} else
		    image->dorev = 0;
		if (hdr->imagic != IMAGIC) {
			Msg("iopen: bad magic in image file %x\n",hdr->imagic);
		    return NULL;
		}
	}
	if (rw)
	    image->flags = _IORW;
	else if (*mode != 'r')
	    image->flags = _IOWRT;
	else
	    image->flags = _IOREAD;
	if(ISRLE(hdr->type)) {
	    tablesize = hdr->ysize*hdr->zsize*sizeof(long);
	    image->rowstart = (ULong *)malloc(tablesize);
	    image->rowsize = (long *)malloc(tablesize);
	    if( image->rowstart == 0 || image->rowsize == 0 ) {
		Msg("iopen: error on table alloc\n");
		return NULL;
	    }
	    image->rleend = 512L+2*tablesize;
	    if (*mode=='w') {
		max = hdr->ysize*hdr->zsize;
		for(i=0; i<max; i++) {
		    image->rowstart[i] = 0;
		    image->rowsize[i] = -1;
		}
	    } else {
		tablesize = hdr->ysize*hdr->zsize*sizeof(long);
		SeekFile(f, 512L, SK_Start);
		if (! ReadData(f, (char *)image->rowstart, tablesize)) {
		    Msg("iopen: error on read of rowstart\n");
		    return NULL;
		}
		if(image->dorev)
		    cvtlongs((long*)image->rowstart, tablesize);
		if (! ReadData(f, (char *)image->rowsize, tablesize)) {
		    Msg("iopen: error on read of rowsize\n");
		    return NULL;
		}
		if(image->dorev)
		    cvtlongs(image->rowsize,tablesize);
	    }
	}
	image->cnt = 0;
	image->ptr = 0;
	image->base = 0;
	if( (image->tmpbuf = ibufalloc(image)) == 0 ) {	
	    Msg("iopen: error on tmpbuf alloc %d\n",hdr->xsize);
	    return NULL;
	}
	image->x = image->y = image->z = 0;
	image->file = f;
	image->offset = 512L;			/* set up for img_optseek */
	SeekFile(f, 512L, SK_Start);
	return(image);
}


static long
img_getrowsize(SgiRgbImage *image)
{
    switch(image->hdr.dim) {
	case 1:
	    return image->rowsize[0];
	case 2:
	    return image->rowsize[image->y];
	case 3:
	    return image->rowsize[image->y+image->z*image->hdr.ysize];
    }
    return(0);
}

static void
img_setrowsize(SgiRgbImage *image, long cnt, long y, long z)
{
    long *sizeptr;
    SgiRgbHeaderPtr hdr = &image->hdr;

    if(img_badrow(image,y,z)) 
	return;
    switch(hdr->dim) {
	case 1:
	    sizeptr = &image->rowsize[0];
	    image->rowstart[0] = image->rleend;
	    break;
	case 2:
	    sizeptr = &image->rowsize[y];
	    image->rowstart[y] = image->rleend;
	    break;
	case 3:
	    sizeptr = &image->rowsize[y+z*hdr->ysize];
	    image->rowstart[y+z*hdr->ysize] = image->rleend;
    }	
    if(*sizeptr != -1) 
	hdr->wastebytes += *sizeptr;
    *sizeptr = cnt;
    image->rleend += cnt;
}

#define docompact 							\
	while(iptr<ibufend) {						\
	    sptr = iptr;						\
	    iptr += 2;							\
	    while((iptr<ibufend)&&((iptr[-2]!=iptr[-1])||(iptr[-1]!=iptr[0])))\
		iptr++;							\
	    iptr -= 2;							\
	    count = iptr-sptr;						\
	    while(count) {						\
		todo = count>126 ? 126:count; 				\
		count -= todo;						\
		*optr++ = 0x80|todo;					\
		while(todo--)						\
		    *optr++ = *sptr++;					\
	    }								\
	    sptr = iptr;						\
	    cc = *iptr++;						\
	    while( (iptr<ibufend) && (*iptr == cc) )			\
		iptr++;							\
	    count = iptr-sptr;						\
	    while(count) {						\
		todo = count>126 ? 126:count; 				\
		count -= todo;						\
		*optr++ = todo;						\
		*optr++ = cc;						\
	    }								\
	}								\
	*optr++ = 0;

static int
img_rle_compact(UShort *expbuf, int ibpp,
			UShort *rlebuf, int obpp, int cnt)
{
    if(ibpp == 1 && obpp == 1) {
	register UChar *iptr = (UChar *)expbuf;
	register UChar *ibufend = iptr+cnt;
	register UChar *sptr;
	register UChar *optr = (UChar *)rlebuf;
	register short todo, cc;
	register long count;

	docompact;
	return optr - (UChar *)rlebuf;
    } else if(ibpp == 1 && obpp == 2) {
	register UChar *iptr = (UChar *)expbuf;
	register UChar *ibufend = iptr+cnt;
	register UChar *sptr;
	register UShort *optr = rlebuf;
	register short todo, cc;
	register long count;

	docompact;
	return optr - rlebuf;
    } else if(ibpp == 2 && obpp == 1) {
	register UShort *iptr = expbuf;
	register UShort *ibufend = iptr+cnt;
	register UShort *sptr;
	register UChar *optr = (UChar *)rlebuf;
	register short todo, cc;
	register long count;

	docompact;
	return optr - (UChar *)rlebuf;
    } else if(ibpp == 2 && obpp == 2) {
	register UShort *iptr = expbuf;
	register UShort *ibufend = iptr+cnt;
	register UShort *sptr;
	register UShort *optr = rlebuf;
	register short todo, cc;
	register long count;

	docompact;
	return optr - rlebuf;
    } else  {
	Msg("rle_compact: bad bpp: %d %d\n",ibpp,obpp);
	return 0;
    }
}

#define doexpand				\
	while(1) {				\
	    pixel = *iptr++;			\
	    if ( !(count = (pixel & 0x7f)) )	\
		return;				\
	    if(pixel & 0x80) {			\
	       while(count--)			\
		    *optr++ = *iptr++;		\
	    } else {				\
	       pixel = *iptr++;			\
	       while(count--)			\
		    *optr++ = pixel;		\
	    }					\
	}

static void
img_rle_expand(UShort *rlebuf, int ibpp,
			UShort *expbuf, int obpp)
{
    if(ibpp == 1 && obpp == 1) {
	register UChar *iptr = (UChar *)rlebuf;
	register UChar *optr = (UChar *)expbuf;
	register UShort pixel,count;

	doexpand;
    } else if(ibpp == 1 && obpp == 2) {
	register UChar *iptr = (UChar *)rlebuf;
	register UShort *optr = expbuf;
	register UShort pixel,count;

	doexpand;
    } else if(ibpp == 2 && obpp == 1) {
	register UShort *iptr = rlebuf;
	register UChar  *optr = (UChar *)expbuf;
	register UShort pixel,count;

	doexpand;
    } else if(ibpp == 2 && obpp == 2) {
	register UShort *iptr = rlebuf;
	register UShort *optr = expbuf;
	register UShort pixel,count;

	doexpand;
    } else 
	Msg("rle_expand: bad bpp: %d %d\n",ibpp,obpp);
}

int
putrow(SgiRgbImage *image, UShort *buffer,
		unsigned int y, unsigned int z) 
{
    register UShort 	*sptr;
    register UChar      *cptr;
    register unsigned int x;
    register ULong min, max;
    register long cnt;
    register SgiRgbHeaderPtr hdr = &image->hdr;

    if( !(image->flags & (_IORW|_IOWRT)) )
	return -1;
    if(hdr->dim<3)
	z = 0;
    if(hdr->dim<2)
	y = 0;
    if(ISVERBATIM(hdr->type)) {
	switch(BPP(hdr->type)) {
	    case 1: 
		min = hdr->min;
		max = hdr->max;
		cptr = (UChar *)image->tmpbuf;
		sptr = buffer;
		for(x=hdr->xsize; x--;) { 
		    *cptr = *sptr++;
		    if (*cptr > max) max = *cptr;
		    if (*cptr < min) min = *cptr;
		    cptr++;
		}
		hdr->min = min;
		hdr->max = max;
		img_seek(image,y,z);
		cnt = hdr->xsize;
		if (img_write(image,(char *)(image->tmpbuf),cnt) != cnt) 
		    return -1;
		else
		    return cnt;
		/* NOTREACHED */

	    case 2: 
		min = hdr->min;
		max = hdr->max;
		sptr = buffer;
		for(x=hdr->xsize; x--;) { 
		    if (*sptr > max) max = *sptr;
		    if (*sptr < min) min = *sptr;
		    sptr++;
		}
		hdr->min = min;
		hdr->max = max;
		img_seek(image,y,z);
		cnt = hdr->xsize<<1;
		if(image->dorev)	
		    cvtshorts(buffer,cnt);
		if (img_write(image,(char *)(buffer),cnt) != cnt) {
		    if(image->dorev)	
			cvtshorts(buffer,cnt);
		    return -1;
		} else {
		    if(image->dorev)	
			cvtshorts(buffer,cnt);
		    return hdr->xsize;
		}
		/* NOTREACHED */

	    default:
		Msg("putrow: weird bpp\n");
	}
    } else if(ISRLE(hdr->type)) {
	switch(BPP(hdr->type)) {
	    case 1: 
		min = hdr->min;
		max = hdr->max;
		sptr = buffer;
		for(x=hdr->xsize; x--;) { 
		    if (*sptr > max) max = *sptr;
		    if (*sptr < min) min = *sptr;
		    sptr++;
		}
		hdr->min = min;
		hdr->max = max;
		cnt = img_rle_compact(buffer,2,image->tmpbuf,1,hdr->xsize);
		img_setrowsize(image,cnt,y,z);
		img_seek(image,y,z);
		if (img_write(image,(char *)(image->tmpbuf),cnt) != cnt) 
		    return -1;
		else
		    return hdr->xsize;
		/* NOTREACHED */

	    case 2: 
		min = hdr->min;
		max = hdr->max;
		sptr = buffer;
		for(x=hdr->xsize; x--;) { 
		    if (*sptr > max) max = *sptr;
		    if (*sptr < min) min = *sptr;
		    sptr++;
		}
		hdr->min = min;
		hdr->max = max;
		cnt = img_rle_compact(buffer,2,image->tmpbuf,2,hdr->xsize);
		cnt <<= 1;
		img_setrowsize(image,cnt,y,z);
		img_seek(image,y,z);
		if(image->dorev)
		    cvtshorts(image->tmpbuf,cnt);
		if (img_write(image,(char *)(image->tmpbuf),cnt) != cnt) {
		    if(image->dorev)
			cvtshorts(image->tmpbuf,cnt);
		    return -1;
		} else {
		    if(image->dorev)
			cvtshorts(image->tmpbuf,cnt);
		    return hdr->xsize;
		}
		/* NOTREACHED */

	    default:
		Msg("putrow: weird bpp\n");
	}
    } else 
	Msg("putrow: weird image type\n");
    return(-1);
}

int
getrow(SgiRgbImage *image, UShort *buffer,
		unsigned int y, unsigned int z) 
{
    register short i;
    register UChar *cptr;
    register UShort *sptr;
    register short cnt;
    register SgiRgbHeaderPtr hdr = &image->hdr;

    if( !(image->flags & (_IORW|_IOREAD)) )
	return -1;
    if(hdr->dim<3)
	z = 0;
    if(hdr->dim<2)
	y = 0;
    img_seek(image, y, z);
    if(ISVERBATIM(hdr->type)) {
	switch(BPP(hdr->type)) {
	    case 1: 
		if (img_read(image,(char *)image->tmpbuf,hdr->xsize) 
							    != hdr->xsize) 
		    return -1;
		else {
		    cptr = (UChar *)image->tmpbuf;
		    sptr = buffer;
		    for(i=hdr->xsize; i--;)
			*sptr++ = *cptr++;
		}
		return hdr->xsize;
		/* NOTREACHED */

	    case 2: 
		cnt = hdr->xsize<<1; 
		if (img_read(image,(char *)(buffer),cnt) != cnt)
		    return -1;
		else {
		    if(image->dorev)
			cvtshorts(buffer,cnt);
		    return hdr->xsize;
		}
		/* NOTREACHED */

	    default:
		Msg("getrow: weird bpp\n");
		break;
	}
    } else if(ISRLE(hdr->type)) {
	switch(BPP(hdr->type)) {
	    case 1: 
		if( (cnt = img_getrowsize(image)) == -1 )
		    return -1;
		if( img_read(image,(char *)(image->tmpbuf),cnt) != cnt )
		    return -1;
		else {
		    img_rle_expand(image->tmpbuf,1,buffer,2);
		    return hdr->xsize;
		}
		/* NOTREACHED */

	    case 2: 
		if( (cnt = img_getrowsize(image)) == -1 )
		    return -1;
		if( cnt != img_read(image,(char *)(image->tmpbuf),cnt) )
		    return -1;
		else {
		    if(image->dorev)
			cvtshorts(image->tmpbuf,cnt);
		    img_rle_expand(image->tmpbuf,2,buffer,2);
		    return hdr->xsize;
		}
		/* NOTREACHED */

	    default:
		Msg("getrow: weird bpp\n");
		break;
	}
    } else 
	Msg("getrow: weird image type\n");
	return(-1);
}
