/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	imageToken.c - parse tokens associated with the image library
 *
 * DESCRIPTION
 *	<overall description of what the c code does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/sgirgb.c,v 1.7 1995/03/10 07:00:10 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "nvrImageI.h"
#include "imageIoI.h"
#include "nvrString.h"
#include "nvrAlloc.h"
#include "osio.h"
#include "sgilibimage.h"

#if 0
/* colormap of images */
#define CM_NORMAL               0
#define CM_DITHERED             1
#define CM_SCREEN               2
#define CM_COLORMAP             3

#define TYPEMASK		0xff00
#define BPPMASK			0x00ff
#define ITYPE_VERBATIM		0x0000
#define ITYPE_RLE		0x0100
#define ISRLE(type)		(((type) & 0xff00) == ITYPE_RLE)
#define ISVERBATIM(type)	(((type) & 0xff00) == ITYPE_VERBATIM)
#define BPP(type)		((type) & BPPMASK)
#define RLE(bpp)		(ITYPE_RLE | (bpp))
#define VERBATIM(bpp)		(ITYPE_VERBATIM | (bpp))
#define IBUFSIZE(pixels)	( (pixels+(pixels>>6))*sizeof(long) )
#define RLE_NOP			0x00

/*
 * The magic number must be...
 */
#define IMAGIC  0732

/*
 * Taken from sgi, include files, this is the definition of the
 * header for sgi rgb files.
 */
typedef struct _sgiRgbHeader {
    unsigned            imagic:16;              /* stuff saved on disk . . */
    unsigned            type:16;
    unsigned            dim:16;
    unsigned            xsize:16;
    unsigned            ysize:16;
    unsigned            zsize:16;
    unsigned            min:32;
    unsigned            max:32;
    unsigned            wastebytes:32;
    char                name[80];
    unsigned            colormap:32;
} SgiRgbHeader;
#endif /* 0 */


typedef struct {
    SgiRgbHeader	header;
    SgiRgbImagePtr	image;
    ImageIOParamPtr	ioParam;
    InternalReasonPtr	reason;
} Private, *PrivPtr;


#ifdef notdef
SgiRgbImage *
iopen(char *file, char *mode, unsigned int type, unsigned int dim,
		unsigned int xsize, unsigned int ysize, unsigned int zsize)
{
    return(imgopen(0, file, mode, type, dim, xsize, ysize, zsize));
}

SgiRgbImage *
fiopen(int f, char *mode, unsigned int type, unsigned int dim,
		unsigned int xsize, unsigned int ysize, unsigned int zsize)
{
    return(imgopen(f, 0, mode, type, dim, xsize, ysize, zsize));
}
#endif /* notdef */

InternalReasonPtr
SgiRgbFlush(ImageOpPtr op, CompositeMemImagePtr in, CompositeMemImagePtr out)
{
    return(NULL);
}

void SgiRgbFree(ImageOpPtr op)
{
    ImageIOParamPtr ioParam = &op->u.io;

    if (ioParam->u.type == FLT_Implicit) {
	free(ioParam->u.implicit.buffer);
    }
    CloseImageStream(&op->u.io);
}

static InternalReasonPtr
SgiRgbWrite(ImageOpPtr op, CompositeMemImagePtr in, CompositeMemImagePtr out)
{
    return( &IRCantWrite );
#if 0
    ImageIOParamPtr	ioParam = &op->u.io;
    PrivPtr		privPtr = ioParam->priv;
    SgiRgbHeaderPtr	pHeader = &privPtr->header;
    register unsigned char
			*rp, *gp, *bp;

    if (ioParam->currentPath == NULL)
    {
	if( privPtr->reason = GenerateNextFile(ioParam) ) {
	    return (privPtr->reason);
	}
    }
    if (privPtr->reason = OpenImageStream(ioParam)) {
	return (privPtr->reason);
    }
    privPtr->image->file = ioParam->fd;

    int width, height, plnlen, rowlen;
    unsigned short *rowbuf;
    register unsigned short *rowp;
    register int i, j;

    [...]

    width = image->hdr.cols;
    height = image->hdr.rows;
    rowlen = image->hdr.rowlen;
    plnlen = image->hdr.plnlen;

    privPtr->image = imgopen(privPtr->fd,NULL,"w",pHeader->type,pHeader->dim,
			     pHeader->xsize,pHeader->ysize,pHeader->zsize);

    /* Write bytes */
    rp = &privPtr->image->bm[0];
    gp = rp + plnlen;
    bp = gp + plnlen;
    rowbuf = (unsigned short *)malloc(rowlen*sizeof(short));
    if (rowbuf == NULL) {
	fprintf (stderr, "Can't malloc rowbuf\n");
	return (0);
    }

    for (i=0; i<height; i++) {
	for (j=0,rowp=rowbuf; j<rowlen; j++)
	    *rowp++ = *rp++;
	putrow(pImage, rowbuf, height-1-i, 0);
	if (image->hdr.planes > 1) {
	    for (j=0,rowp=rowbuf; j<rowlen; j++)
		*rowp++ = *gp++;
	    putrow(pImage, rowbuf, height-1-i, 1);
	    for (j=0,rowp=rowbuf; j<rowlen; j++)
		*rowp++ = *bp++;
	    putrow(pImage, rowbuf, height-1-i, 2);
	}
    }

    free(rowbuf);

    iclose(pImage);
    free(pImage);
    return (1);
#endif /* 0 */
    
}

InternalReasonPtr
SgiRgbRead(ImageOpPtr op, CompositeMemImagePtr in, CompositeMemImagePtr out)
{
    ImageIOParamPtr	ioParam = &op->u.io;
    PrivPtr		privPtr = ioParam->priv;
    SgiRgbHeaderPtr	pHeader = &privPtr->header;
    char		*errorFunc = "SgiRgbRead";
    InternalReasonPtr	reason;

    int width, height, plnlen, rowlen, clrlen, depth;
    register unsigned char *rp, *gp, *bp;
    unsigned short *rowbuf;
    register unsigned short *rowp;
    register int i, j;


    if (ioParam->currentPath == NULL)
    {
	if (privPtr->reason = GenerateNextFile(ioParam)) {
	    return (privPtr->reason);
	}
    }
    if (privPtr->reason = OpenImageStream(ioParam)){
	return (privPtr->reason);
    }

    SeekFile(ioParam->fd, 0L, SK_Start);
    privPtr->image = imgopen(privPtr->ioParam->fd,NULL,"r",pHeader->type,
			     pHeader->dim,pHeader->xsize,pHeader->ysize,
			     pHeader->zsize);
    if (! privPtr->image) {
	reason = CreateReason("%s: Error: can't open image\n",errorFunc);
	return ( reason );
    }

    /* Initialize and allocate input image */  
    width = privPtr->image->hdr.xsize;
    height = privPtr->image->hdr.ysize;
    depth = 24;
    clrlen = 0;

    rowlen = width;
    if (rowlen & 1) rowlen++;
    plnlen = rowlen * height; /* Corrected 4-6-90 Gary W. Sherwin */

    /* Read bytes */
    rp = out->pixels;
    gp = rp + 1;
    bp = gp + 1;
    rowbuf = (unsigned short *)malloc(rowlen*sizeof(short));
    if (rowbuf == NULL) {
	return (&IRBadAllocation);
    }

    for (i=0; i<height; i++) {
	if (getrow(privPtr->image, rowbuf, height-1-i, 0) < 0) break;
	for (j=0,rowp=rowbuf; j<rowlen; j++) {
	    *rp = *rowp++; rp += out->images[0].nextPixel;
	}
	if (privPtr->image->hdr.zsize > 1 &&
	    getrow(privPtr->image, rowbuf, height-1-i, 1) < 0)
	{
	    break;
	}
	for (j=0,rowp=rowbuf; j<rowlen; j++) {
	    *gp = *rowp++; gp += out->images[1].nextPixel;
	}
	if (privPtr->image->hdr.zsize > 1 &&
	    getrow(privPtr->image, rowbuf, height-1-i, 2) < 0)
	{
	    break;
	}
	for (j=0,rowp=rowbuf; j<rowlen; j++) {
	    *bp = *rowp++; bp += out->images[2].nextPixel;
	}
    }

    free((char *)rowbuf);
    CloseImageStream(ioParam);
    iclose(privPtr->image);
    free((char *)privPtr->image);
    privPtr->image = NULL;
    if (i<height) {
	reason = CreateReason("Error: %s %d of %d pixels (%d bytes)\n",
			      "EOF on bitmap after",
			      i*width,
			      plnlen,
			      plnlen * ((depth == 24) ? 3 : 1));
	return ( reason );
    }

    return (NULL);
}


/*
** type: "spec" for ImageSpecPtr
**       "info" for ImageFileInfoPtr
*/
boolean
SgiRgbInitHdr(char *buf, int bufSize, void *ptr, char *type)
{
    SgiRgbHeaderPtr headerPtr = (SgiRgbHeaderPtr)buf;
    ImageSpecPtr iPtr;
    ImageFileInfoPtr fPtr;

    if( bufSize < sizeof(SgiRgbHeader) ) {
	return(False);
    }
    if( headerPtr->imagic == REV_IMAGIC ) {
	cvtimage((long *)headerPtr);
    }
    if( headerPtr->imagic != IMAGIC ) {
	return(False);
    }
    switch( *type ) {
    case 's':
	iPtr = (ImageSpecPtr) ptr;
	iPtr->sample = Sample4_4_4;
	iPtr->headerSize = sizeof(SgiRgbHeader);
	iPtr->colorSpace = ColorRGB;
	iPtr->width = headerPtr->xsize;
	iPtr->height = headerPtr->ysize;
	break;
    case 'i':
	fPtr = (ImageFileInfoPtr) ptr;
	fPtr->sample = (int)Sample4_4_4;
	fPtr->hdrSize = sizeof(SgiRgbHeader);
	fPtr->color = (int)ColorRGB;
	fPtr->width = (int)headerPtr->xsize;
	fPtr->height = (int)headerPtr->ysize;
	fPtr->frameCount = 1;
	break;
    }
    return(True);
}

InternalReasonPtr
SgiRgbInit(ImageOpPtr op, CompositeMemImagePtr in)
{
    ImageIOParamPtr ioParam = &op->u.io;
    InternalReasonPtr reason;
    PrivPtr privPtr;

    privPtr = (PrivPtr) InsecureAlloc(sizeof(*privPtr));
    if (privPtr == NULL)
        return(AllocReason("SGI RGB info", sizeof(*privPtr)) );
    memset((char *)privPtr, 0, sizeof(*privPtr));
    ioParam->priv = privPtr;
    privPtr->ioParam = ioParam;

    if (reason = ReadImageStream(ioParam, (char *)&privPtr->header,
				 sizeof(privPtr->header)))
	return(reason);
    
    if (reason = UnreadImageStream(ioParam, (char *)&privPtr->header,
				   sizeof(privPtr->header)))
	return(reason);

    if( SgiRgbInitHdr((char *)&privPtr->header, sizeof(privPtr->header),
		      &ioParam->spec, "spec") == False ) {
	return(&IRInternalProblem);
    }

    /*
     * Change from IFT_SgiRgb to IFT_RawRgb...
     * NewRgbImage() needs this to do the right thing.
     */
    ioParam->spec.type = IFT_RawRgb;

    return(NULL);
}

