/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              fileDpy.c
 *
 *      DESCRIPTION
 *              Unix methods for displaying an image in a file
 *
 *      REVISION
 *              $Header: /nvr/share/src/nvr/prod/library/imgdpy/RCS/fileDpy.
c,v 1.2 1995/08/01 22:40:17 toddb Exp $
 */

#include "fileImageDpyI.h"

#define AllInitialized(initialized)  ((initialized & ID_AllFile) == ID_AllFile)
 
static InternalReasonPtr
DisplayYcImageInYc(ImgDpyRegionPtr pubimg, YcImagePtr yc)
{
    FileImageRegionPtr fimg = (FileImageRegionPtr)pubimg;

    WriteData(fimg->outputFile, (char *)yc->y,  fimg->super.yArea);
    WriteData(fimg->outputFile, (char *)yc->cr, fimg->super.cArea);
    WriteData(fimg->outputFile, (char *)yc->cb, fimg->super.cArea);

    /*
     * Since we never put an image in fimg->super.lastYci, we
     * never need to free it.
     */
    (*yc->free)(yc);
    return(NULL);
}

static InternalReasonPtr
RefreshRgbFileImage(ImgDpyRegionPtr pubimg)
{
    FileImageRegionPtr fimg = (FileImageRegionPtr)pubimg;

    WriteData(fimg->outputFile,
	      (char *)fimg->super.copyBuffer,
	      fimg->super.copyBufferSize);

    return(NULL);
}

static InternalReason NotInitialized =
  { "First call to InitFileImageRegion() must have all params initialized.\n",
    IR_InternalProblem };
 

static InternalReasonPtr
InitFileImageRegion(PrivImgDpyRegionPtr img, /* structure to update */
		    ImgDpyParamPtr param,    /* parameters to update */
		    ULong paramMask)         /* mask of parameters to update */
{
    FileImageRegionPtr fimg = (FileImageRegionPtr)img;
    InternalReasonPtr reason = NULL;
    ULong initialized = img->initialized | paramMask;

    if (! AllInitialized(initialized)) {
        return( &NotInitialized );
    }
 
    if (paramMask & ID_DstInfo) {
        img->dstInfo = param->dst;
        img->initialized |= ID_DstInfo;
    }
 
    if (paramMask & ID_SrcInfo) {
        img->srcInfo = param->src;
        img->initialized |= ID_SrcInfo;
    }

    if (paramMask & ID_DepthType) {
        img->color = param->color;
        img->initialized |= ID_DepthType;
    }
 
    if (paramMask & (ID_DstInfo | ID_SrcInfo))  {
	if (img->color != DepthUnspecified) {
	    reason = InitImgPixmap(img);
	    if (reason) {
		return(reason);
	    }
	} else {
	    /* straight YC file output ... no conversion */
	}
	img->displayYcNative = DisplayYcImageInYc;
	img->oneStageConvert = False;   /* this is all we know just now */
	img->pub.refreshImage = RefreshRgbFileImage;
	reason = InitImgYcConversion(img);
	if (reason) {
	    return(reason);
	}
    }
    if (paramMask & ID_File) {
	if (fimg->outputFile > 0) {
	    CloseFile(fimg->outputFile);
	}
	fimg->outputFile = OpenFileForWriting(param->u.file.path);
    }
    return(NULL);
}


static InternalReasonPtr
AllocateFileDevCopyBuffer(PrivImgDpyRegionPtr img)
{
    UChar *copyBuffer;

    copyBuffer = (UChar *)
        InsecureNullRealloc((char *)img->copyBuffer,
                            img->copyBufferSize);
    if (copyBuffer == NULL) {
        return( CreateReason("can't allocate %d bytes for %dx%d picture.",
            img->copyBufferSize, img->srcInfo.width, img->srcInfo.height));
    }
 
    img->copyBuffer = copyBuffer;

    return(NULL);
}

static void
FreeFileImageRegion(PrivImgDpyRegionPtr img)
{
}

static ImgDpyRegionPtr
FileCreateImageRegion(ImageDestDescPtr desc, InternalReasonPtr *pIrp)
{
    long size;
    PrivImgDpyRegionPtr img;

    size = sizeof(FileImageRegion);
    img = (PrivImgDpyRegionPtr) AllocNamedMem(size, "file image region", pIrp);
    if (img == NULL)
        return(NULL);

    img->init = InitFileImageRegion;
    img->allocCopyBuffer = AllocateFileDevCopyBuffer;
    img->free = FreeFileImageRegion;

    return(&img->pub);
}

static ImageDestDesc FileOutputDesc = {
    FileCreateImageRegion
};
 
ImageDestDescPtr FileImageDesc = &FileOutputDesc;
