/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              graylookup.c
 *
 *      DESCRIPTION
 *              Fast conversion routines for y/cr/cb -> gray
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/graylookup.c,v 1.1 1995/10/18 23:14:27 russ Exp $
 */

#define NeedFunctionPrototypes 1

#include "rgblookup.h"

static void
LookupGray1Frame(YcToGrayInfoPtr ycToGray) { }

static void
LookupQuadGray1Frame(YcToGrayInfoPtr ycToGray)
{
    long fbWidth = ycToGray->grayWidth;
    long width = (ycToGray->box.width & (~0x1f));
    int pixStride, chanStride;
    int i, j, bigWidth;
    UChar *pPix1 = ycToGray->gray;
    UChar *pY = ycToGray->y;
    ULong yValues;
    ULong yPixel1, yPixel2;
    UChar pattern;
    UChar *ditherPattern;

    /*
     * Hide some of the grody details
     */
#define DitherFirstPix0							\
    pattern = ditherPattern[ 0*128 + (yValues >> 24) ];			\
    yPixel1 = (pattern >> 4);						\
    yPixel2 = (pattern & 0x3);

#define DitherPix0							\
    pattern = ditherPattern[ 0*128 + (yValues >> 24) ];			\
    yPixel1 = (yPixel1 << 2) | (pattern >> 4);				\
    yPixel2 = (yPixel2 << 2) | (pattern & 0x3);

#define DitherPix1							\
    pattern = ditherPattern[ 1*128 + ((yValues >> 16) & 0x7f) ];	\
    yPixel1 = (yPixel1 << 2) | (pattern >> 4);				\
    yPixel2 = (yPixel2 << 2) | (pattern & 0x3);

#define DitherPix2							\
    pattern = ditherPattern[ 2*128 + ((yValues >> 8) & 0x7f) ];		\
    yPixel1 = (yPixel1 << 2) | (pattern >> 4);				\
    yPixel2 = (yPixel2 << 2) | (pattern & 0x3);

#define DitherPix3							\
    pattern = ditherPattern[ 3*128 + (yValues & 0x7f) ];		\
    yPixel1 = (yPixel1 << 2) | (pattern >> 4);				\
    yPixel2 = (yPixel2 << 2) | (pattern & 0x3);

    pixStride = fbWidth + fbWidth - (width >> 3);
    chanStride = ycToGray->lumaWidth - (width>>1);

    bigWidth = width >> (4+1);

    i = ycToGray->box.height >> 1;
    for (;;) {
	ditherPattern = &ycToGray->yDithered7to3Pixel[ (i&0x3) << (2+7) ];
	for (j = bigWidth; j-- > 0;) {
	    pattern = 0;

	    yValues = *ULongP(pY) >> 1;
	    pY += 4;
	    DitherFirstPix0; DitherPix1; DitherPix2; DitherPix3;

	    yValues = *ULongP(pY) >> 1;
	    pY += 4;
	    DitherPix0; DitherPix1; DitherPix2; DitherPix3;

	    yValues = *ULongP(pY) >> 1;
	    pY += 4;
	    DitherPix0; DitherPix1; DitherPix2; DitherPix3;

	    yValues = *ULongP(pY) >> 1;
	    pY += 4;
	    DitherPix0; DitherPix1; DitherPix2; DitherPix3;

	    *ULongP(pPix1) = yPixel1;
	    *ULongP(pPix1+fbWidth) = yPixel2;
	    pPix1 += 4;
	}

	if (--i <= 0)
	    break;
	pPix1 += pixStride;
	pY += chanStride;
    }
#undef DitherFirstPix0
#undef DitherPix0
#undef DitherPix1
#undef DitherPix2
#undef DitherPix3
}

static void
LookupGray7Frame(YcToGrayInfoPtr ycToGray)
{
    long width = ycToGray->box.width;
    long height = ycToGray->box.height;
    UChar *pFb = ycToGray->gray + ycToGray->grayStart;
    UChar *pY = ycToGray->y + ycToGray->lumaStart;
    OctWord in1, in2;
    long i, j;

    for (i = height;;) {
	for (j = width; j > 0; j -= 16) {
	    in1 = *OctWordP(pY); pY += 8;
            in2 = *OctWordP(pY); pY += 8;
	    in1.l.high = (in1.l.high >> 1) | 0x80808080;
	    in1.l.low  = (in1.l.low >> 1)  | 0x80808080;
	    in2.l.high = (in2.l.high >> 1) | 0x80808080;
	    in2.l.low  = (in2.l.low >> 1)  | 0x80808080;
            *OctWordP(pFb) = in1; pFb += 8;
            *OctWordP(pFb) = in2; pFb += 8;
	}
	if (--i <= 0)
	    break;
	pFb += ycToGray->grayAccel;
	pY += ycToGray->lumaAccel;
    }
}

static void
LookupQuadGray7Frame(YcToGrayInfoPtr ycToGray)
{
    long fbWidth = ycToGray->grayWidth;
    long width = ycToGray->box.width;
    long height = ycToGray->box.height;
    UChar *pFb = ycToGray->gray + ycToGray->grayStart;
    UChar *pY = ycToGray->y + ycToGray->lumaStart;
    long i;
    UChar *pEndY;
    OctWord in;
    OctWord out;
    ULong pix;
    ULong highBit = 0x80808080;

    width >>= 4;	/* number of double word pairs (in the output) */
    i = height>>1;	/* number of double lines in the output */
    for (;;) {
	pEndY = pY + (width<<3);
	while (pY != pEndY) {
	    in = *OctWordP(pY);
	    pY += 8;
	    in.l.high = (in.l.high >> 1 ) | highBit;

	    out.l.high = in.l.high & 0xff000000;
	    out.l.high |= out.l.high >> 8;
	    pix  = (in.l.high >> 16) & 0xff;
	    out.l.high |= pix | (pix << 8);

	    out.l.low = (in.l.high & 0xff00) << 8;
	    out.l.low |= out.l.low << 8;
	    pix = in.l.high & 0xff;
	    out.l.low |= pix | (pix << 8);

	    *OctWordP(pFb+fbWidth) = out;
	    *OctWordP(pFb) = out;
	    pFb += 8;

	    in.l.low  = (in.l.low >> 1 ) | highBit;
	    out.l.high = in.l.low & 0xff000000;
	    out.l.high |= out.l.high >> 8;
	    pix  = (in.l.low >> 16) & 0xff;
	    out.l.high |= pix | (pix << 8);

	    out.l.low = (in.l.low & 0xff00) << 8;
	    out.l.low |= out.l.low << 8;
	    pix = in.l.low & 0xff;
	    out.l.low |= pix | (pix << 8);

	    *OctWordP(pFb+fbWidth) = out;
	    *OctWordP(pFb) = out;
	    pFb += 8;
	}
	if (--i <= 0)
	    break;
	pFb += ycToGray->grayAccel;
	pY += ycToGray->lumaAccel;
    }
}

static void
LookupGray8Frame(YcToGrayInfoPtr ycToGray)
{
    long width = ycToGray->box.width;
    long height = ycToGray->box.height;
    UChar *pFb = ycToGray->gray + ycToGray->grayStart;
    UChar *pY = ycToGray->y + ycToGray->lumaStart;
    OctWord in1, in2;
    long i, j;

    for (i = height;;) {
	for (j = width; j > 0; j -= 16) {
	    in1 = *OctWordP(pY); pY += 8;
            in2 = *OctWordP(pY); pY += 8;
            *OctWordP(pFb) = in1; pFb += 8;
            *OctWordP(pFb) = in2; pFb += 8;
	}
	if (--i <= 0)
	    break;
	pFb += ycToGray->grayAccel;
	pY += ycToGray->lumaAccel;
    }
}

static void
LookupQuadGray8Frame(YcToGrayInfoPtr ycToGray)
{
    long fbWidth = ycToGray->grayWidth;
    long width = ycToGray->box.width;
    long height = ycToGray->box.height;
    UChar *pFb = ycToGray->gray + ycToGray->grayStart;
    UChar *pY = ycToGray->y + ycToGray->lumaStart;
    long i;
    UChar *pEndY;
    OctWord out;
    ULong pix;

    width >>= 4;	/* number of double word pairs (in the output) */
    i = height>>1;	/* number of double lines in the output */
    for (;;) {
	pEndY = pY + (width<<3);
	while (pY != pEndY) {
	    pix = pY[0];
	    out.l.high = (pix << 8) | pix;
	    pix = pY[1];
	    out.l.high = (out.l.high << 16) | (pix << 8) | pix;

	    pix = pY[2];
	    out.l.low = (pix << 8) | pix;
	    pix = pY[3];
	    out.l.low = (out.l.low << 16) | (pix << 8) | pix;

	    *OctWordP(pFb+fbWidth) = out;
	    *OctWordP(pFb) = out;
	    pFb += 8;

	    pix = pY[4];
	    out.l.high = (pix << 8) | pix;
	    pix = pY[5];
	    out.l.high = (out.l.high << 16) | (pix << 8) | pix;

	    pix = pY[6];
	    out.l.low = (pix << 8) | pix;
	    pix = pY[7];
	    out.l.low = (out.l.low << 16) | (pix << 8) | pix;

	    *OctWordP(pFb+fbWidth) = out;
	    *OctWordP(pFb) = out;
	    pFb += 8;
	    pY += 8;
	}
	if (--i <= 0)
	    break;
	pFb += ycToGray->grayAccel;
	pY += ycToGray->lumaAccel;
    }
}

static void
InitGrayLookupData(YcToGrayInfoPtr ycToGray)
{
    UChar *table = ycToGray->yDithered7to3Pixel;
    static boolean complete = False;
    int i, dither;
    UChar pixel;
    static UChar ditherVals[16] = {
        D00, D01, D02, D03,
        D10, D11, D12, D13,
        D20, D21, D22, D23,
        D30, D31, D32, D33
    };
    static UChar patterns[ 8 ] = {
     /* 0x00, 0x10, 0x02, 0x21, 0x12, 0x31, 0x23, 0x33 */
        0x33, 0x33, 0x31, 0x12, 0x21, 0x02, 0x00, 0x00
    };

    if (complete)
        return;
    for (i = 0; i < 128; i++)
        for (dither = 0; dither < 16; dither++) {
            pixel = (i + (ditherVals[ dither ] >> 4)) >> 4;
            if (pixel > 7)
                pixel = 7;
            table[ dither*128 + i ] = patterns[ pixel ];
        }
    complete = True;
}

YcToGrayInfoPtr
AllocYcToGrayInfo(RgbDepthType depthType)
{
    return (YcToGrayInfoPtr) AllocYcToRgbInfo(depthType);
}

/*
 * This assumes that the elements
 *	lumaWidth, grayWidth, grayDepth
 * have been assigned.  It will
 *   - assign values for box
 *   - assign {luma and gray}Start according to x,y in box
 *   - assign {luma and gray}Accel according to lumaWidth
 *   - Initialize dither.
 * y and gray can be assigned at any time.
 */
InternalReasonPtr
InitYcToGrayInfo(YcToGrayInfoPtr *pYcToGray,
		 GeomRectanglePtr src,
		 GeomRectanglePtr dst)
{
    YcToGrayInfoPtr ycToGray = *pYcToGray;
    void (*func)(YcToGrayInfoPtr);
    int width, height;
    float sizeScale = 0.0;

    switch (ycToGray->scale) {
    case LookupScaleUnity:
	width = src->width;
	height = src->height;
	break;

    case LookupScaleQuad:
	width = src->width << 1;
	height = src->height << 1;
	break;
    }

    switch (ycToGray->scale) {
    case LookupScaleUnity:   sizeScale = 1.0; break;
    case LookupScaleQuad:    sizeScale = 2.0; break;
    }

    width = src->width * sizeScale;
    if (width > dst->width)
	width = dst->width;
    height = src->height * sizeScale;
    if (height > dst->height)
	height = dst->height;

    ycToGray->box.width = width;
    ycToGray->box.height = height;
    ycToGray->box.x = dst->x;
    ycToGray->box.y = dst->y;

    ycToGray->lumaStart = src->x + src->y * ycToGray->lumaWidth;
    ycToGray->grayStart = dst->x + dst->y * ycToGray->grayWidth;

    func = NULL;	/* gratuitous for lint */

    switch(ycToGray->depthType) {
    default:
	return(NULL);	/* never happens */

    case GrayScale1:
        ycToGray->destPixelsToAlign = 32;
	InitGrayLookupData(ycToGray);
	switch (ycToGray->scale) {
	case LookupScaleUnity:   func = LookupGray1Frame;        break;
	case LookupScaleQuad:    func = LookupQuadGray1Frame;    break;
	}
	break;

    case GrayScale7:
        ycToGray->destPixelsToAlign = 8;
	switch (ycToGray->scale) {
	case LookupScaleUnity:   func = LookupGray7Frame;        break;
	case LookupScaleQuad:    func = LookupQuadGray7Frame;    break;
	}
	break;

    case GrayScale8:
        ycToGray->destPixelsToAlign = 8;
	switch (ycToGray->scale) {
	case LookupScaleUnity:   func = LookupGray8Frame;        break;
	case LookupScaleQuad:    func = LookupQuadGray8Frame;    break;
	}
	break;
    }
    ycToGray->lookupFrame = func;

    /*
     * If the destination buffer is not aligned the way we want it,
     * (which only happens if the destination is a frame buffer)
     * move it to the right until it is and then trim the right-hand edge.
     */
    AlignConversionBox(LookupPixelSizeInBits(ycToGray->depthType),
		       ycToGray->destPixelsToAlign,
		       &ycToGray->box,
		       &ycToGray->grayStart);

    /*
     * Now that we know the final width and height, we can calculate
     * the accelerators.
     */
    width = ycToGray->box.width / sizeScale;
    ycToGray->lumaAccel = ycToGray->lumaWidth - width;

    switch(ycToGray->depthType) {
    case GrayScale1:
	ycToGray->grayAccel = ycToGray->grayWidth * sizeScale
			    - (ycToGray->box.width >> 5);
	break;

    default: /* never happens */
    case GrayScale8:
    case GrayScale7:
	ycToGray->grayAccel = ycToGray->grayWidth * sizeScale
			    - ycToGray->box.width;
	break;
    }

    /*
     * Currently we never need to increase the size
     */
    return(NULL);
}
