/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              macImageDpy.c
 *
 *      DESCRIPTION
 *              Mac methods for displaying an image in a window
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/imageDpy.c,v 1.8 1995/12/11 23:02:58 toddb Exp $
 */

#include "imageDpyI.h"
#include "nvrAlloc.h"

NameEnumPair RgbDepthTypeNames[] = {
    { "DepthUnspecified",       DepthUnspecified },
    { "Rgb888",                 Rgb888 },
    { "Rgb444",                 Rgb444 },
    { "Rgb555",                 Rgb555 },
    { "Rgb232",                 Rgb232 },
    { "Rgb332",                 Rgb332 },
    { "GrayScale8",             GrayScale8 },
    { "GrayScale7",             GrayScale7 },
    { "GrayScale1",             GrayScale1 },
    { "RgbC555",                RgbC555 },
    { "RgbC666",                RgbC666 },
};

NameEnumPair RgbPixelComponentsNames[] = {
    { "PixelRGB0",      PixelRGB0 },
    { "Pixel0RGB",      Pixel0RGB },
    { "PixelRGB",       PixelRGB },
    { "PixelBGR0",      PixelBGR0 },
    { "Pixel0BGR",      Pixel0BGR },
    { "PixelBGR",       PixelBGR },
};

static InternalReasonPtr
DisplayYcImageInRgb(ImgDpyRegionPtr pubimg, YcImagePtr yc)
{
    PrivImgDpyRegionPtr img = (PrivImgDpyRegionPtr)pubimg;
    YcToRgbInfoPtr ycToRgb = img->cvtu.ycToRgb;

    if (img->lastYci)
	(*img->lastYci->free)(yc);
    img->lastYci = yc;

    ycToRgb->y = yc->y;
    ycToRgb->cb = yc->cb;
    ycToRgb->cr = yc->cr;

    (*ycToRgb->lookupFrame)(ycToRgb);
    if (! img->oneStageConvert) {
        return( (*img->pub.refreshImage)(pubimg) );
    }
    return(NULL);
}

InternalReasonPtr
IllegalYcDisplay(ImgDpyRegionPtr pubimg, YcImagePtr yc)
{
    return( InternalProblem("Illegal yc display function\n") );
}

static InternalReasonPtr
DisplayYcImageInGray(ImgDpyRegionPtr pubimg, YcImagePtr yc)
{
    PrivImgDpyRegionPtr img = (PrivImgDpyRegionPtr)pubimg;
    YcToGrayInfoPtr ycToGray = img->cvtu.ycToGray;

    if (img->lastYci)
	(*img->lastYci->free)(yc);
    img->lastYci = yc;

    ycToGray->y = yc->y;

    (*ycToGray->lookupFrame)(ycToGray);
    if (! img->oneStageConvert) {
        return( (*img->pub.refreshImage)(pubimg) );
    }
    return(NULL);
}

static InternalReasonPtr
InitialYcDisplay(ImgDpyRegionPtr pubimg, YcImagePtr yc)
{
    PrivImgDpyRegionPtr img = (PrivImgDpyRegionPtr)pubimg;

    if (img->lastYci)
	(*img->lastYci->free)(yc);
    img->lastYci = yc;

    return(NULL);
}

static InternalReasonPtr
InitialRefresh(ImgDpyRegionPtr pubimg)
{
    return(NULL);
}

InternalReasonPtr
InitImgYcConversion(PrivImgDpyRegionPtr img)
{
    YcToRgbInfoPtr ycToRgb;
    YcToGrayInfoPtr ycToGray;
    InternalReasonPtr reason = NULL;

    /*
     * Put together the right pointers and info for the lookup mechanism
     */
    switch(img->color) {
    case GrayScale1:
    case GrayScale7:
    case GrayScale8:
        img->pub.displayYcImage = DisplayYcImageInGray;
        ycToGray = img->cvtu.ycToGray;
        if (ycToGray == NULL) {
            ycToGray = AllocYcToGrayInfo(img->color);
            if (ycToGray == NULL)
                return(&IRBadAllocation);
        }
        img->cvtu.ycToGray = ycToGray;
        ycToGray->depthType = img->color;
        ycToGray->depthType = img->color;
        ycToGray->lumaWidth = img->srcInfo.lineBytes;
        ycToGray->grayWidth = img->copyBufferWidth;
        ycToGray->gray = img->copyBuffer;
        ycToGray->scale = img->scale;
        reason = InitYcToGrayInfo(&img->cvtu.ycToGray, &img->src, &img->dst);
        break;

    case Rgb232:
    case Rgb332:
    case Rgb444:
    case Rgb555:
    case Rgb888:
    case RgbC555:
    case RgbC666:
        img->pub.displayYcImage = DisplayYcImageInRgb;
        ycToRgb = img->cvtu.ycToRgb;
        if (ycToRgb == NULL) {
            ycToRgb = AllocYcToRgbInfo(img->color);
            if (ycToRgb == NULL)
                return(&IRBadAllocation);
        }
        if (ycToRgb->depthType != img->color) {
            ycToRgb->tablesAreValid = False;
            ycToRgb->depthType = img->color;
        }
        img->cvtu.ycToRgb = ycToRgb;
        ycToRgb->rgbWidth = img->copyBufferWidth;
        ycToRgb->lumaWidth = img->srcInfo.lineBytes;
        ycToRgb->chromaWidth = img->srcInfo.chromaLineBytes;
        ycToRgb->rgb = img->copyBuffer;
        ycToRgb->scale = img->scale;
        ycToRgb->depthType = img->color;
        reason = InitYcToRgbInfo(&img->cvtu.ycToRgb, &img->src, &img->dst);
        break;

    case DepthUnspecified:
	/*
	 * This is used by file conversion to go straight to a file
	 */
        img->pub.displayYcImage = img->displayYcNative;
        break;
	
    default:
        /* never happens */
        break;
    }

    return (reason);
}

InternalReasonPtr
InitImgPixmap(PrivImgDpyRegionPtr img)
{
    long copyBufferSize;
    InternalReasonPtr reason;

    /*
     * Calculate the src and dst rectangle
     */
    img->src.x = img->src.y = 0;
    img->src.width = img->srcInfo.width;
    img->src.height = img->srcInfo.height;

    img->yArea = img->srcInfo.width * img->srcInfo.height;
    img->cArea = img->yArea >> 2;

    if (img->oneStageConvert) {
        img->dst.x = img->dstInfo.x;
        img->dst.y = img->dstInfo.y;
    } else {
        img->dst.x = img->dst.y = 0;
    }
    
    if (img->scale == LookupScaleQuad) {
        img->dst.width = img->srcInfo.width * 2;
        img->dst.height = img->srcInfo.height * 2;
    } else {
        img->dst.width = img->srcInfo.width;
        img->dst.height = img->srcInfo.height;
    }

    /*
     * Calculate the off-screen buffer size.  This is used for double-buffering
     * as well as expose events.
     */
    img->pixelSizeInBits = LookupPixelSizeInBits(img->color);
    img->pixelsPer32Bits = 32 / img->pixelSizeInBits;
    img->copyBufferWidth = (img->src.width * img->pixelSizeInBits) / 8;
    copyBufferSize = img->copyBufferWidth * img->src.height;
    if (img->dstInfo.doubleSize && ! img->hardwareZoom) {
        img->scale = LookupScaleQuad;
        copyBufferSize *= 4;
    } else {
        img->scale = LookupScaleUnity;
    }

    if (copyBufferSize != img->copyBufferSize) {
        img->copyBufferSize = copyBufferSize;
        reason = (*img->allocCopyBuffer)(img);
        if (reason)
            return(reason);
        /*
         * Make it a black frame to begin with.
         */
        memset(img->copyBuffer, 0, img->copyBufferSize);
    }
    return(NULL);
}

InternalReasonPtr
NullSaveYcImage(ImgDpyRegionPtr img, YcImagePtr yc)
{
    return(NULL);
}

InternalReasonPtr
InitImageRegion(ImgDpyRegionPtr pubimg,  /* structure to update */
                ImgDpyParamPtr param,    /* parameters to update */
                ULong paramMask)         /* mask of parameters to update */
{
    PrivImgDpyRegionPtr img = (PrivImgDpyRegionPtr)pubimg;

    return( (*img->init)(img, param, paramMask) );
}

ImgDpyRegionPtr
CreateImageRegion(ImageDestDescPtr desc, InternalReasonPtr *pIrp)
{
    ImgDpyRegionPtr pubimg = (*desc->create)(desc, pIrp);

    pubimg->displayYcImage = InitialYcDisplay;
    pubimg->refreshImage = InitialRefresh;
    return( pubimg );
}

void
FreeImageRegion(ImgDpyRegionPtr pubimg)
{
    PrivImgDpyRegionPtr img = (PrivImgDpyRegionPtr)pubimg;

    (img->free)(img);
    if (img->cvtu.ycToGray)
        free((char *)img->cvtu.ycToGray);
    if (img->cvtuTmp.ycToGray)
        free((char *)img->cvtuTmp.ycToGray);
    free((char *)img);
}
