/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              imageDpy.h
 *
 *      DESCRIPTION
 *              Universal methods for displaying an image in a window
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/imageDpy.h,v 1.8 1996/01/18 23:02:11 toddb Exp $
 */

#ifndef IMAGEDPY_H
#define IMAGEDPY_H

#include "nvrImage.h"

/*
 * For yc->rgb conversion, things must go very fast, so the internal
 * representation of a yc image is always the same.
 */
typedef struct _ycImage YcImage, *YcImagePtr;
struct _ycImage {
    SampleType sample;		/* currently only Sample4_2_0 is supported */
    long sched;			/* scheduled time for display */
    ULong frame;		/* frame number */
    UChar *y;
    UChar *cr;
    UChar *cb;
    int lumaWidth;
    int lumaHeight;
    int chromaWidth;
    YcImagePtr next;		/* for linked list of images */
    void (*free)(YcImagePtr);	/* freeing method */
}; /* typedef: YcImage, *YcImagePtr */


/*
 * Call displayYcImage() or displayCompImage() to display
 * these particular image types in the registered window.  Note that
 * the image may or may not have been saved (it is implementation and
 * system dependent).
 *
 * Call saveYcImage() or saveCompImage() to save
 * these particular image types in the ImgDpyRegionPtr. One of these functions
 * must have been called prior to calling refreshImage().
 *
 * Call refreshImage() in response to a window system expose event.
 */

typedef struct _imgDpyRegion ImgDpyRegion, *ImgDpyRegionPtr;

struct _imgDpyRegion {
    InternalReasonPtr
    	 (*displayYcImage)  (ImgDpyRegionPtr, YcImagePtr),
         (*displayCompImage)(ImgDpyRegionPtr, CompositeMemImagePtr),
         (*refreshImage)    (ImgDpyRegionPtr);
}; /* typedef: ImgDpyRegion, *ImgDpyRegionPtr */

typedef struct _imgSrcInfo {
    ColorSpace colorSpace;
    SampleType sample;
    int width;
    int height;
    int lineBytes;
    int chromaLineBytes;
} ImageSrcInfo, *ImageSrcInfoPtr;

typedef struct _imgDstInfo {
    int x;
    int y;
    boolean doubleSize;		/* whether the image is doubled on output */
    boolean winClipped;		/* whether window is partly clipped or not */
} ImageDstInfo, *ImageDstInfoPtr;

typedef struct _imgDpyParam {
    union {
	struct {
	    char *path;
	} file;
	struct {
	    void *win;		/* window sys specific window identifier */
	    void *dpy;		/* window sys specific display identifier */
	} wsys;
    } u;
    RgbDepthType color;		/* type of window image to use */
    ImageDstInfo dst; 		/* info about the destination */
    ImageSrcInfo src;		/* info about the source */
} ImgDpyParam, *ImgDpyParamPtr;

enum {
    ID_Window     = 0x01,
    ID_DepthType  = 0x02,
    ID_DstInfo    = 0x04,
    ID_SrcInfo    = 0x08,
    ID_File       = 0x10
};

#define ID_AllWin  (ID_Window \
                  | ID_DepthType \
                  | ID_DstInfo \
                  | ID_SrcInfo)

#define ID_AllFile (ID_DepthType \
		  | ID_DstInfo \
                  | ID_SrcInfo \
                  | ID_File)


typedef struct _imageDestDesc *ImageDestDescPtr;

extern ImageDestDescPtr MacWindowImageDesc;
extern ImageDestDescPtr XWindowImageDesc;
extern ImageDestDescPtr FileImageDesc;

/*
 * Call this to create storage for the image region.  This only allocates
 * memory (specific to the window system)
 */
CExternal ImgDpyRegionPtr CreateImageRegion(
    ImageDestDescPtr,
    InternalReasonPtr *
);

CExternal void FreeImageRegion(
    ImgDpyRegionPtr
);

/*
 * Call this to (re)initialize the image region any time you want to
 * change the window, the depth or the display rectangle within
 * the window.
 */
CExternal InternalReasonPtr InitImageRegion(
    ImgDpyRegionPtr,	/* structure to update */
    ImgDpyParamPtr,	/* parameters to update */
    ULong paramMask	/* mask of parameters to update */
);

extern NameEnumPair RgbDepthTypeNames[];
extern NameEnumPair RgbPixelComponentsNames[];

#endif /* IMAGEDPY_H */
