/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              imageDpyI.h
 *
 *      DESCRIPTION
 *              internal header for displaying an image in a window
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/imageDpyI.h,v 1.7 1995/12/06 18:29:24 toddb Exp $
 */

#ifndef IMAGEDPYI_H
#define IMAGEDPYI_H

#include "imageDpy.h"
#include "rgblookup.h"

typedef struct _imageDestDesc ImageDestDesc;

struct _imageDestDesc {
   ImgDpyRegionPtr (*create)(ImageDestDescPtr, InternalReasonPtr *);
   int otherInfo;
};

/*
 * Note that PrivImgDpyRegionPtr is isometric to ImgDpyRegion
 */
typedef struct _privImgDpyRegion PrivImgDpyRegion, *PrivImgDpyRegionPtr;

struct _privImgDpyRegion {
    ImgDpyRegion pub;

    /*
     * Called by InitImageRegion() to do destination-specific work
     */
    InternalReasonPtr (*init)(PrivImgDpyRegionPtr, ImgDpyParamPtr,
                              ULong paramMask);
    /*
     * Called by InitImgPixmap() to allocate destination-specific frame
     * buffer memory (or just plain memory).
     */
    InternalReasonPtr (*allocCopyBuffer)(PrivImgDpyRegionPtr);

    /*
     * Assigned to ImgDpyRegionPtr->displayYcImage when
     * the configuration specifies ImgDpyParamPtr->color == DepthUnspecified
     */
    InternalReasonPtr (*displayYcNative)(ImgDpyRegionPtr, YcImagePtr);

    /*
     * Called by FreeImageRegion() to do destination-specific freeing
     */
    void              (*free)(PrivImgDpyRegionPtr);

    ULong	initialized;		/* ImgDpyParam members initialized */
    
    ImageDstInfo dstInfo;		/* user-provided destination info*/
    ImageSrcInfo srcInfo;		/* user-provided source info */
    GeomRectangle dst;			/* calculated destination rect */
    GeomRectangle src;			/* calculated source rect */

    long	yArea;			/* area of YcImagePtr->y */
    long	cArea;			/* area of YcImagePtr->cr and ->cb */
    int		pixelSizeInBits;
    int		pixelsPer32Bits;
    long	copyBufferSize;		/* bytes in the copy buffer */
    int		copyBufferWidth;
    UChar	*copyBuffer;		/* the framebuffer or pixmap storage */
    boolean	hardwareZoom;
    boolean	oneStageConvert;
    LookupScale scale;
    RgbDepthType color;
    YcImagePtr  lastYci;		/* last known image, for refreshing */
    union _cvtUnion {
	YcToRgbInfoPtr ycToRgb;
	YcToGrayInfoPtr ycToGray;
    } cvtu, cvtuTmp;	/* for doing yc -> rgb conversions */
};

CExternal InternalReasonPtr InitImgYcConversion(PrivImgDpyRegionPtr img);
CExternal InternalReasonPtr InitImgPixmap(PrivImgDpyRegionPtr img);
CExternal InternalReasonPtr AllocateImgDevCopyBuffer(PrivImgDpyRegionPtr img);
CExternal InternalReasonPtr IllegalYcDisplay(ImgDpyRegionPtr, YcImagePtr);

CExternal InternalReasonPtr InitDevImageRegion(
    PrivImgDpyRegionPtr,/* structure to update */
    ImgDpyParamPtr,	/* parameters to update */
    ULong paramMask	/* mask of parameters to update */
);

/*
 * Common methods
 */
CExternal InternalReasonPtr NullSaveYcImage(ImgDpyRegionPtr, YcImagePtr);

#endif /* IMAGEDPYI_H */
