/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              macImageDpy.c
 *
 *      DESCRIPTION
 *              Mac methods for displaying an image in a window
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/macImageDpy.c,v 1.5 1995/10/20 21:33:21 russ Exp $
 */

#define usepixmap

#include "macImageDpyI.h"
#include "nvrAlloc.h"

#include <Memory.h>

#define AllInitialized(initialized)     ((initialized & ID_AllWin) == ID_AllWin)

static ImgDpyRegionPtr
FreeMacImageRegion(MacImageRegionPtr mimg)
{
    if (mimg->indexClut) {
        DisposCTable(mimg->indexClut);
    }
    if (mimg->directClut) {
        DisposCTable(mimg->directClut);
    }
    if (mimg->pixmap->baseAddr) {
        free((char *)mimg->pixmap->baseAddr);
    }
    if (mimg->gWorld) {
        DisposeGWorld(mimg->gWorld);
    }
    
    return(NULL);
}

void
FreeImageRegion(ImgDpyRegionPtr pubimg)
{
    PrivImgDpyRegionPtr img = (PrivImgDpyRegionPtr)pubimg;
    MacImageRegionPtr mimg = (MacImageRegionPtr)img->dev;

    (void) FreeMacImageRegion(mimg);
    if (img->cvtu.ycToGray)
        free((char *)img->cvtu.ycToGray);
    if (img->cvtuTmp.ycToGray)
        free((char *)img->cvtuTmp.ycToGray);
    free((char *)img);
}

static void
InitIndexClut(PrivImgDpyRegionPtr img)
{
    ColorSpecPtr cs;
    short i, lastEntry;
    short maxRed, maxGreen, maxBlue;
    short red, green, blue;
    short oneBits;
    long stepSize;
    CTabPtr clut;
    MacImageRegionPtr mimg = (MacImageRegionPtr)img->dev;
    
    clut = *mimg->indexClut;
    clut->ctSeed = GetCTSeed();
    clut->ctFlags = 0;
    cs = clut->ctTable;

    memset(cs, 0, sizeof(ColorSpec) & 256);
    
    switch(img->color) {
    default:    return; /* never happens */
    case GrayScale1:
        lastEntry = 1;
        cs[0].value = 0;
        cs[0].rgb.red = 0;
        cs[0].rgb.green = 0;
        cs[0].rgb.blue = 0;
        cs[1].value = 1;
        cs[1].rgb.red = ~0;
        cs[1].rgb.green = ~0;
        cs[1].rgb.blue = ~0;
        break;

    case GrayScale7:
        oneBits = 0x80;
        lastEntry = 127;
        stepSize = 0xffff / 127;
        goto grayScale;
        
    case GrayScale8:
        oneBits = 0;
        lastEntry = 255;
        stepSize = 0xffff / 255;
    grayScale:
        for (i = 0; i <= lastEntry; cs++, i++) {
            cs->value = i | oneBits;
            cs->rgb.red = cs->rgb.green = cs->rgb.blue = i * stepSize;
        }
        break;
        
    case Rgb232:
        oneBits = 0x80;
        lastEntry = 127;
        maxRed = maxBlue = 3;
        maxGreen = 7;
        goto colorIndex;
        
    case Rgb332:
        oneBits = 0;
        lastEntry = 255;
        maxBlue = 3;
        maxRed = maxGreen = 7;
        goto colorIndex;
        
    case RgbC555:
        oneBits = 0;
        lastEntry = 5*5*5-1;
        maxGreen = maxRed = maxBlue = 4;
        goto colorIndex;
        
    case RgbC666:
        oneBits = 0;
        lastEntry = 6*6*6-1;
        maxGreen = maxRed = maxBlue = 5;
        
    colorIndex:
        for (red = 0; red <= maxRed; red++) {
            for (green = 0; green <= maxGreen; green++) {
                for (blue = 0; blue <= maxBlue; blue++) {
                    cs->rgb.red   = 0xffff -  red   * (0xffff / maxRed);
                    cs->rgb.green = 0xffff -  green * (0xffff / maxGreen);
                    cs->rgb.blue  = 0xffff -  blue  * (0xffff / maxBlue);
                    cs->value = red * (maxGreen+1) * (maxBlue+1)
                              + green * (maxBlue+1)
                              + blue
                              + oneBits;
                    cs++;
                }
            }
        }
        
        break;
    }
    clut->ctSize = lastEntry;
}

static void
InitMacColorTables(PrivImgDpyRegionPtr img)
{
    MacImageRegionPtr mimg = (MacImageRegionPtr)img->dev;
    int depth;

    switch(img->color) {
    default:
    case GrayScale7:
    case GrayScale8:
    case Rgb232:
    case Rgb332:
    case RgbC555:
    case RgbC666:    depth = 8; break;
    case Rgb444:     depth = 16; break;
    case Rgb555:     depth = 16; break;
    case Rgb888:     depth = 32; break;
    case GrayScale1: depth = 1; break;
    }

    switch(mimg->depth = depth) {
    case 32:
    case 16:
        mimg->pixmap->pixelType = RGBDirect;
        mimg->pixmap->pmTable = mimg->directClut;
        mimg->pixmap->pixelSize = 32;
        mimg->pixmap->cmpCount = 3;
        mimg->pixmap->cmpSize = (depth == 16 ? 5 : 8);
        (**mimg->pixmap->pmTable).ctSeed = GetCTSeed();
        (**mimg->pixmap->pmTable).ctFlags = 0x8000; /* ignore color table */
        (**mimg->pixmap->pmTable).ctSize = 0;
        break;
    case 8:
        mimg->pixmap->pixelSize = 8;
        goto indexedDepth;
        
    case 1:
        mimg->pixmap->pixelSize = 1;
    indexedDepth:
        mimg->pixmap->cmpCount = 1;
        mimg->pixmap->cmpSize = mimg->pixmap->pixelSize;
        InitIndexClut(img);
        mimg->pixmap->pixelType = 0;
        mimg->pixmap->pmTable = mimg->indexClut;
        break;
    }

    switch(img->color) {
    case Rgb444:
    case Rgb555:
    case Rgb888:
        if (! img->cvtu.ycToRgb->invertFinalRgb
         && img->cvtu.ycToRgb->tablesAreValid)
            img->cvtu.ycToRgb->tablesAreValid = False;
        img->cvtu.ycToRgb->invertFinalRgb = True;
        img->cvtu.ycToRgb->components = Pixel0RGB;
        break;
    }
}

static void
InitImgPort(PrivImgDpyRegionPtr img)
{
    MacImageRegionPtr mimg = (MacImageRegionPtr)img->dev;
    CGrafPtr port = mimg->win;
    GrafPtr oldPort;
    RGBColor fore, back;
    
    GetPort((GrafPtr *)&oldPort);
    SetPort((GrafPtr)port);
    GetForeColor(&fore);
    GetBackColor(&back);
    RGBForeColor(&mimg->black);
    RGBBackColor(&mimg->white);
    RGBForeColor(&fore);
    RGBBackColor(&back);
    SetPort((GrafPtr)oldPort);
}

static InternalReasonPtr
MacImgDevCopyBuffer(PrivImgDpyRegionPtr img)
{
    MacImageRegionPtr mimg = (MacImageRegionPtr)img->dev;
    UChar *copyBuffer;

    copyBuffer = (UChar *)
        InsecureNullRealloc(img->copyBuffer,
                            img->copyBufferSize);
    if (copyBuffer == NULL) {
        return( CreateReason("can't allocate %d bytes for %dx%d picture.",
            img->copyBufferSize, img->src.width, img->src.height));
    }

    img->copyBuffer = copyBuffer;
    mimg->pixmap->baseAddr = (Ptr)copyBuffer;
    mimg->pixmap->rowBytes = img->copyBufferWidth | 0x8000;
    mimg->pixmap->bounds.top = 0;
    mimg->pixmap->bounds.left = 0;
    mimg->pixmap->bounds.bottom = img->src.height;
    mimg->pixmap->bounds.right = img->src.width;
    
    return(NULL);
}

#ifndef UseGWorld

static InternalReasonPtr
RefreshMacImage(ImgDpyRegionPtr pubimg)
{
    PrivImgDpyRegionPtr img = (PrivImgDpyRegionPtr)pubimg;
    MacImageRegionPtr mimg = (MacImageRegionPtr)img->dev;
    CGrafPtr port = mimg->win;
    CGrafPtr oldPort;
    
    GetPort((GrafPtr *)&oldPort);
    if (oldPort != port)
        SetPort((GrafPtr)port);

    CopyBits((BitMapPtr)(mimg->pixmap),
             (BitMapPtr)(*port->portPixMap),
             &mimg->pixmap->bounds,
             &mimg->dstRect,
             srcCopy,
             (RgnHandle)NULL);

    if (oldPort != port)
        SetPort((GrafPtr)oldPort);
    return(NULL);
}

static InternalReasonPtr
InitGWorld(PrivImgDpyRegionPtr img)
{
    return(NULL);
}

#else /* ! defined(UseGWorld) */

static InternalReasonPtr
RefreshMacImage(ImgDpyRegionPtr pubimg)
{
    PrivImgDpyRegionPtr img = (PrivImgDpyRegionPtr)pubimg;
    MacImageRegionPtr mimg = (MacImageRegionPtr)img->dev;
    CGrafPtr port = mimg->win;
    CGrafPtr oldPort;
    
    GetPort((GrafPtr *)&oldPort);
    if (oldPort != port)
        SetPort((GrafPtr)port);

    rect = &(*mimg->gWorld->portPixMap)->bounds;
    CopyBits((BitMapPtr)(*mimg->gWorld->portPixMap),
             (BitMapPtr)(*port->portPixMap),
             rect,
             rect,
             srcCopy,
             (RgnHandle)NULL);
    if (oldPort != port)
        SetPort((GrafPtr)oldPort);
    return(NULL);
}

static InternalReasonPtr
InitGWorld(PrivImgDpyRegionPtr img)
{
    MacImageRegionPtr mimg = (MacImageRegionPtr)img->dev;
    OSErr error;
    Rect bounds;

    bounds.top = 0;
    bounds.left = 0;
    bounds.bottom = img->src.height;
    bounds.right = img->src.width;
    if (mimg->gWorld) {
        error = UpdateGWorld(&mimg->gWorld,
                             mimg->depth,
                             &bounds,
                             nil,
                             nil,          /* (GDHandle) gdevice */
                             0);           /* flags */
    } else {
        error = NewGWorld(&mimg->gWorld,
                          mimg->depth,
                          &bounds,
                          nil,
                          nil,          /* (GDHandle) gdevice */
                          0);           /* flags */
    }
    if (error != noErr)
        return( CreateSysReason("Can't create GWorld for QuickTime\n") );
    LockPixels(mimg->gWorld->portPixMap);
    
    return(NULL);
}

#endif /* UseGWorld */

static InternalReason NotInitialized =
  { "First call to InitMacImageRegion() must have all params initialized.\n",
    IR_InternalProblem };
 
static InternalReasonPtr
InitMacImageRegion(PrivImgDpyRegionPtr img, /* structure to update */
                   ImgDpyParamPtr param,    /* parameters to update */
                   ULong paramMask)         /* mask of parameters to update */
{
    MacImageRegionPtr mimg = (MacImageRegionPtr)img->dev;
    InternalReasonPtr reason = NULL;
    ULong initialized = img->initialized | paramMask;
    
    if (! AllInitialized(initialized))
        return( &NotInitialized );

    if (paramMask & ID_DstInfo) {
        img->dst = param->dst;
        img->initialized |= ID_DstInfo;
    }
    
    if (paramMask & ID_SrcInfo) {
        img->src = param->src;
        img->initialized |= ID_SrcInfo;
    }
    
    if (paramMask & ID_Window) {
        /*
         * param->win must be squirrelled away by the device layer.
         */
        img->initialized |= ID_Window;
    }

    if (paramMask & ID_DepthType) {
        img->color = param->color;
        img->initialized |= ID_DepthType;
    }

    if (paramMask & ID_DstInfo) {
        mimg->dstRect.left = param->dst.x;
        mimg->dstRect.right = param->dst.x + param->src.width;
        mimg->dstRect.bottom = param->dst.y + param->src.height;
        mimg->dstRect.top = param->dst.y;
    }
    
    if (paramMask & ID_Window) {
        mimg->win = (CGrafPtr) param->win;
        InitImgPort(img);
    }
    if (paramMask & (ID_DepthType | ID_DstInfo | ID_SrcInfo)) {
        reason = InitImgPixmap(img);
        if (reason)
            return(reason);
        InitMacColorTables(img);
        img->oneStageConvert = False;	/* this is all we know just now */
        img->pub.saveYcImage = NullSaveYcImage;
        reason = InitImgYcConversion(img);
        if (reason)
            return(reason);
        reason = InitGWorld(img);
        if (reason)
            return(reason);
    }

    img->pub.refreshImage = RefreshMacImage;
    
    return(reason);
}

static ImgDpyRegionPtr
NullImageRegion(void)
{
	return(NULL);
}

static ImgDpyRegionPtr
MacCreateImageRegion(ImageDestDescPtr desc)
{
    long size;
    PrivImgDpyRegionPtr img;
    MacImageRegionPtr mimg;
    
    size = sizeof(PixMap) + sizeof(PrivImgDpyRegion) + sizeof(MacImageRegion);
    img = (PrivImgDpyRegionPtr)InsecureAlloc(size);
    if (img == NULL)
        return(NULL);
    memset(img, 0, size);

    mimg = (MacImageRegionPtr) (img + 1);
    mimg->pixmap = (PixMapPtr)(mimg + 1);
    img->dev = (PrivPointer) mimg;
    mimg->pixmap->hRes = 72 << 16;      /* fixed point number */
    mimg->pixmap->vRes = 72 << 16;      /* fixed point number */
    mimg->directClut = (CTabHandle)
        NewHandle( sizeof(ColorTable) - sizeof(CSpecArray) );
    mimg->indexClut = (CTabHandle)
        NewHandle( sizeof(ColorTable) + 255 * sizeof(ColorSpec) );
    if (mimg->directClut == NULL || mimg->indexClut == NULL)
        return( FreeMacImageRegion(mimg) );

    mimg->white.red = 0;
    mimg->white.green = 0;
    mimg->white.blue = 0;
    mimg->black.red = ~0;
    mimg->black.green = ~0;
    mimg->black.blue = ~0;
    
    img->init = InitMacImageRegion;
    img->allocCopyBuffer = MacImgDevCopyBuffer;

    return((ImgDpyRegionPtr) img);
}

static ImageDestDesc MacWinDesc = {
    MacCreateImageRegion
};

ImageDestDescPtr MacWin = &MacWinDesc;

