/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              rgb444lookup.c
 *
 *      DESCRIPTION
 *              Fast conversion routines for y/cr/cb -> rgb
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/rgb444lookup.c,v 1.4 1995/10/27 17:42:45 russ Exp $
 */


#define NeedFunctionPrototypes 1

#include "nvrAlloc.h"
#include "nvrString.h"
#include "colorSpace.h"
#include "nvrImageI.h"
#include "nvrTimer.h"

#include "rgblookupI.h"
#include "rgbdither.h"

#define Rgb444IndexShift        1

#ifdef CONFIGURE_444

static void
Lookup444QuadFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w;
    UChar *pY, *pCr, *pCb;
    UChar *table = ycToRgb->rgbLookupData.u.rgbEqual;
    UChar *targetRgb;
    int crCbIndex, tmp;
    UChar *pRgb;
    ULong pixel, y;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    width >>= 2;
    for (;;) {
        /*
         * Each time through the loop we expand 2x2 pixels to 4x4 pixels
         *
         *               x x x x
         *    x x        x x x x
         *    x x   ==>  x x x x
         *               x x x x
         */
	w = width;
	for (;;) {
	    /*
	     * do:  x .      x x . .
	     *      . . ==>  x x . .
	     *               . . . .
	     *               . . . .
	     */
            crCbIndex = *pCr++ >> (8-Rgb444CrBits);
	    crCbIndex <<= Rgb444IndexShift;
            tmp = *pCb++ >> (8-Rgb444CbBits);
            crCbIndex |= tmp << (Rgb444IndexShift+Rgb444CrBits);
	    targetRgb = table+crCbIndex;

            tmp = (y = pY[0]+Rgb444D00) >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel = *UShortP(targetRgb+tmp);
            pixel <<= 16;

	    y += Rgb444D01 - Rgb444D00;
            tmp = y >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel |= *UShortP(targetRgb+tmp);

	    *ULongP(pRgb) = pixel;

	    y += Rgb444D10 - Rgb444D01;
            tmp = y >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel = *UShortP(targetRgb+tmp);
            pixel <<= 16;

	    y += Rgb444D11 - Rgb444D10;
            tmp = y >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel |= *UShortP(targetRgb+tmp);

	    *ULongP(pRgb+rgbWidth) = pixel;
	    pRgb += 4;

	    /*
	     * do:  . x      . . x x
	     *      . . ==>  . . x x
	     *               . . . .
	     *               . . . .
	     */
            tmp = (y = pY[1]+Rgb444D02) >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel = *UShortP(targetRgb+tmp);
            pixel <<= 16;

	    y += Rgb444D03 - Rgb444D02;
            tmp = y >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel |= *UShortP(targetRgb+tmp);

	    *ULongP(pRgb) = pixel;

	    y += Rgb444D12 - Rgb444D03;
            tmp = y >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel = *UShortP(targetRgb+tmp);
            pixel <<= 16;

	    y += Rgb444D13 - Rgb444D12;
            tmp = y >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel |= *UShortP(targetRgb+tmp);

	    *ULongP(pRgb+rgbWidth) = pixel;
	    pRgb += (rgbWidth << 1) - 4;

	    /*
	     * do:  . .      . . . .
	     *      x . ==>  . . . .
	     *               x x . .
	     *               x x . .
	     */
            tmp = (y = pY[lumaWidth]+Rgb444D20) >> (8-Rgb444YBits);
	    pY++;
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel = *UShortP(targetRgb+tmp);
            pixel <<= 16;

	    y += Rgb444D21 - Rgb444D20;
            tmp = y >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel |= *UShortP(targetRgb+tmp);

	    *ULongP(pRgb) = pixel;

	    y += Rgb444D30 - Rgb444D21;
            tmp = y >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel = *UShortP(targetRgb+tmp);
            pixel <<= 16;

	    y += Rgb444D31 - Rgb444D30;
            tmp = y >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel |= *UShortP(targetRgb+tmp);

	    *ULongP(pRgb+rgbWidth) = pixel;
	    pRgb += 4;

	    /*
	     * do:  . .      . . . .
	     *      . x ==>  . . . .
	     *               . . x x
	     *               . . x x
	     */
            tmp = (y = pY[lumaWidth]+Rgb444D22) >> (8-Rgb444YBits);
	    pY++;
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel = *UShortP(targetRgb+tmp);
            pixel <<= 16;

	    y += Rgb444D23 - Rgb444D22;
            tmp = y >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel |= *UShortP(targetRgb+tmp);

	    *ULongP(pRgb) = pixel;

	    y += Rgb444D32 - Rgb444D23;
            tmp = y >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel = *UShortP(targetRgb+tmp);
            pixel <<= 16;

	    y += Rgb444D33 - Rgb444D32;
            tmp = y >> (8-Rgb444YBits);
            tmp <<= Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits;
            pixel |= *UShortP(targetRgb+tmp);

	    *ULongP(pRgb+rgbWidth) = pixel;
	    pRgb -= (rgbWidth << 1) - 4;

	    if (--w <= 0)
		break;
	}

	if ((height -= 4) <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

/*
 * Optimized to load variables ahead of the time they'll be needed.
 *	This should provide enough information for the compiler to
 *	minimized pipeline stalls
 */
static void
Lookup444Frame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w;
    UChar *pY, *pCr, *pCb;
    UChar *table = ycToRgb->rgbLookupData.u.rgbEqual;
    UChar *targetRgb;
    UChar *pRgb;
    UShort pixel0, pixel1;
    long t0, t1, t2, t3;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    width >>= 2;
    for (;;) {
	/*
	 * Each time through the loop does 4x4 pixels
	 * . . . .
	 * . . . .
	 * . . . .
	 * . . . .
	 * The initial value of w, below, must be even
	 */
	w = width;
	for (;;) {
	    /*
	     * do:  x x . .
	     *      x x . .
	     *      . . . .
	     *      . . . .
	     */
	    /* preload */
	    t0 = pCr[0];
	    t1 = pCb[0];
	    t2 = pY[0];

            t0 = t0 >> (8-Rgb444CrBits);
            t0 <<= Rgb444IndexShift;
            t1 = t1 >> (8-Rgb444CbBits);
            t0 |= t1 << (Rgb444IndexShift+Rgb444CrBits);
            targetRgb = table+t0;
 
	    /* preload */
	    t3 = pY[1];

            t0 = (t2+Rgb444D00) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel0 = *UShortP(targetRgb+t0);
 
	    /* preload */
	    t2 = pY[lumaWidth];

            t0 = (t3+Rgb444D01) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel1 = *UShortP(targetRgb+t0);
 
	    StoreAligned2h(pRgb, pixel0, pixel1);
 
	    /* preload */
	    t3 = pY[lumaWidth+1];
	    pY += 2;

            t0 = (t2+Rgb444D10) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel0 = *UShortP(targetRgb+t0);
 
            t0 = (t3+Rgb444D11) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel1 = *UShortP(targetRgb+t0);
 
	    /* preload */
	    t2 = pY[0];
	    t0 = pCr[1];
	    t1 = pCb[1];

	    StoreAligned2h(pRgb+rgbWidth, pixel0, pixel1);
            pRgb += 4;
 
            /*
             * do:  . . x x
             *      . . x x
             *      . . . .
             *      . . . .
             */
            t0 = t0 >> (8-Rgb444CrBits);
            t0 <<= Rgb444IndexShift;
            t1 = t1 >> (8-Rgb444CbBits);
	    t3 = pY[1];

            t0 |= t1 << (Rgb444IndexShift+Rgb444CrBits);
            targetRgb = table+t0;
 
            t0 = (t2+Rgb444D02) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);

	    /* preload */
	    t2 = pY[lumaWidth];

            pixel0 = *UShortP(targetRgb+t0);
 
            t0 = (t3+Rgb444D03) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel1 = *UShortP(targetRgb+t0);
 
	    /* preload */
	    t3 = pY[lumaWidth+1];
	    pY += (lumaWidth<<1) - 2;

	    StoreAligned2h(pRgb, pixel0, pixel1);
            pRgb += rgbWidth;
 
            t0 = (t2+Rgb444D12) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel0 = *UShortP(targetRgb+t0);
 
            t0 = (t3+Rgb444D13) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel1 = *UShortP(targetRgb+t0);
 
	    /* preload */
	    pCr += ycToRgb->chromaWidth;
	    pCb += ycToRgb->chromaWidth;
	    t0 = *pCr++;
	    t1 = *pCb++;
	    t2 = pY[0];

	    StoreAligned2h(pRgb, pixel0, pixel1);
            pRgb += rgbWidth - 4;
 
	    /*
	     * do:  . . . .
	     *      . . . .
	     *      x x . .
	     *      x x . .
	     */
            t0 >>= (8-Rgb444CrBits);
            t0 <<= Rgb444IndexShift;
            t1 >>= (8-Rgb444CbBits);
            t0 |= t1 << (Rgb444IndexShift+Rgb444CrBits);
            targetRgb = table+t0;
 
	    /* preload */
	    t3 = pY[1];

            t0 = (t2+Rgb444D20) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel0 = *UShortP(targetRgb+t0);

	    /* preload */
	    t2 = pY[lumaWidth];

            t0 = (t3+Rgb444D21) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel1 = *UShortP(targetRgb+t0);

	    /* preload */
	    t3 = pY[lumaWidth+1];
	    pY += 2;

	    StoreAligned2h(pRgb, pixel0, pixel1);

            t0 = (t2+Rgb444D30) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel0 = *UShortP(targetRgb+t0);

            t0 = (t3+Rgb444D31) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel1 = *UShortP(targetRgb+t0);

	    /* preload */
	    t0 =  pCr[0];
	    pCr -= ycToRgb->chromaWidth - 1;
	    t1 = pCb[0];
	    pCb -= ycToRgb->chromaWidth - 1;
	    t2 = pY[0];

	    StoreAligned2h(pRgb+rgbWidth, pixel0, pixel1);
	    pRgb += 4;

	    /*
	     * do:  . . . .
	     *      . . . .
	     *      . . x x
	     *      . . x x
	     */
            t0 >>= (8-Rgb444CrBits);
	    t0 <<= Rgb444IndexShift;
            t1 >>= (8-Rgb444CbBits);
            t0 |= t1 << (Rgb444IndexShift+Rgb444CrBits);
	    targetRgb = table+t0;

	    /* preload */
	    t3 = pY[1]; 

            t0 = (t2+Rgb444D22) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel0 = *UShortP(targetRgb+t0);

	    /* preload */
	    t2 = pY[lumaWidth];

            t0 = (t3+Rgb444D23) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel1 = *UShortP(targetRgb+t0);

	    /* preload */
	    t3 = pY[lumaWidth+1];
	    pY -= (lumaWidth<<1) - 2;

	    StoreAligned2h(pRgb, pixel0, pixel1);

            t0 = (t2+Rgb444D32) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel0 = *UShortP(targetRgb+t0);

            t0 = (t3+Rgb444D33) >> (8-Rgb444YBits);
            t0 <<= (Rgb444IndexShift+Rgb444CrBits+Rgb444CbBits);
            pixel1 = *UShortP(targetRgb+t0);

	    StoreAligned2h(pRgb+rgbWidth, pixel0, pixel1);
	    pRgb -= (rgbWidth<<1) - 4;

	    if (--w <= 0)
		break;
	}

	if ((height -= 4) <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

#else /* CONFIGURE_444 */

static void
Lookup444Frame(YcToRgbInfoPtr ycToRgb)
{ RgbLookupNotConfigured(ycToRgb); }

static void
Lookup444QuadFrame(YcToRgbInfoPtr ycToRgb)
{ RgbLookupNotConfigured(ycToRgb); }

#endif /* CONFIGURE_444 */

RgbLookupFunc
ChooseRgb444LookupFunc(YcToRgbInfoPtr ycToRgb)
{
    RgbLookupFunc func = NULL;
    
    switch (ycToRgb->scale)	{
    case LookupScaleUnity:	func =	Lookup444Frame; break;
    case LookupScaleQuad:	func =	Lookup444QuadFrame; break;
    }
    return( func );
}
