/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              rgbDeepLookup.c
 *
 *      DESCRIPTION
 *              Fast conversion routines for y/cr/cb -> rgb
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/rgb555lookup.c,v 1.2 1994/04/28 22:20:42 toddb Exp $
 */


#define NeedFunctionPrototypes 1

#include "nvrAlloc.h"
#include "nvrString.h"
#include "colorSpace.h"
#include "nvrImageI.h"
#include "nvrTimer.h"

#include "rgblookup.h"
#include "rgbdither.h"

#ifdef CONFIGURE_555RGB

static void
Lookup555RGBFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w;
    UChar *pY, *pCr, *pCb;
    UChar *table = ycToRgb->rgbLookupData.u.rgbEqual;
    int index, tmp;
    UChar *pRgb;
    ULong rgbTop, rgbBottom;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    width >>= 2;
    for (;;) {
	/*
	 * Each time through the loop does 4x4 pixels
	 * . . . .
	 * . . . .
	 * . . . .
	 * . . . .
	 * The initial value of w, below, must be even
	 */
	w = width;
	for (;;) {
	    /*
	     * do:  x x . .
	     *      x x . .
	     *      . . . .
	     *      . . . .
	     */
            index = pCr[0] >> (8-Rgb555CrBits);
            tmp = pCb[0] >> (8-Rgb555CbBits);
            index |= tmp << Rgb555CrBits;
            tmp = (pY[0]+Rgb555D00) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbTop = *(table+index);
            rgbTop <<= 8;

            index &= MaskOf(Rgb555CbBits+Rgb555CrBits);
            tmp = (pY[1]+Rgb555D01) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbTop |= *(table+index);
            rgbTop <<= 8;

            index &= MaskOf(Rgb555CbBits+Rgb555CrBits);
            tmp = (pY[lumaWidth]+Rgb555D10) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbBottom = *(table+index);
            rgbBottom <<= 8;

	    pY++;

            index &= MaskOf(Rgb555CbBits+Rgb555CrBits);
            tmp = (pY[lumaWidth]+Rgb555D11) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbBottom |= *(table+index);
            rgbBottom <<= 8;

	    /*
	     * do:  . . x x
	     *      . . x x
	     *      . . . .
	     *      . . . .
	     */
	    pY++;
            index = pCr[1] >> (8-Rgb555CrBits);
            tmp = pCb[1] >> (8-Rgb555CbBits);
            index |= tmp << Rgb555CrBits;
            tmp = (pY[0]+Rgb555D02) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbTop |= *(table+index);
            rgbTop <<= 8;

            index &= MaskOf(Rgb555CbBits+Rgb555CrBits);
            tmp = (pY[1]+Rgb555D03) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbTop |= *(table+index);

	    *ULongP(pRgb) = rgbTop;
	    pRgb += rgbWidth;

            index &= MaskOf(Rgb555CbBits+Rgb555CrBits);
            tmp = (pY[lumaWidth]+Rgb555D12) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbBottom |= *(table+index);
            rgbBottom <<= 8;

            index &= MaskOf(Rgb555CbBits+Rgb555CrBits);
            tmp = (pY[lumaWidth+1]+Rgb555D13) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbBottom |= *(table+index);
	    *ULongP(pRgb) = rgbBottom;

	    /*
	     * do:  . . . .
	     *      . . . .
	     *      x x . .
	     *      x x . .
	     */
	    pY += (lumaWidth<<1) - 2;
	    pCr += ycToRgb->chromaWidth;
	    pCb += ycToRgb->chromaWidth;
	    pRgb += rgbWidth;

            index = pCr[0] >> (8-Rgb555CrBits);
            tmp = pCb[0] >> (8-Rgb555CbBits);
            index |= tmp << Rgb555CrBits;
            tmp = (pY[0]+Rgb555D20) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbTop = *(table+index);
            rgbTop <<= 8;

            index &= MaskOf(Rgb555CbBits+Rgb555CrBits);
            tmp = (pY[1]+Rgb555D21) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbTop |= *(table+index);
            rgbTop <<= 8;

            index &= MaskOf(Rgb555CbBits+Rgb555CrBits);
            tmp = (pY[lumaWidth]+Rgb555D30) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbBottom = *(table+index);
            rgbBottom <<= 8;

	    pY++;

            index &= MaskOf(Rgb555CbBits+Rgb555CrBits);
            tmp = (pY[lumaWidth]+Rgb555D31) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbBottom |= *(table+index);
            rgbBottom <<= 8;

	    /*
	     * do:  . . . .
	     *      . . . .
	     *      . . x x
	     *      . . x x
	     */
	    pY++;
            index = pCr[1] >> (8-Rgb555CrBits);
            tmp = pCb[1] >> (8-Rgb555CbBits);
            index |= tmp << Rgb555CrBits;
            tmp = (pY[0]+Rgb555D22) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbTop |= *(table+index);
            rgbTop <<= 8;

            index &= MaskOf(Rgb555CbBits+Rgb555CrBits);
            tmp = (pY[1]+Rgb555D23) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbTop |= *(table+index);

	    *ULongP(pRgb) = rgbTop;

            index &= MaskOf(Rgb555CbBits+Rgb555CrBits);
            tmp = (pY[lumaWidth]+Rgb555D32) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbBottom |= *(table+index);
            rgbBottom <<= 8;

            index &= MaskOf(Rgb555CbBits+Rgb555CrBits);
            tmp = (pY[lumaWidth+1]+Rgb555D33) >> (8-Rgb555YBits);
            index |= tmp << (Rgb555CbBits+Rgb555CrBits);
            rgbBottom |= *(table+index);
	    *ULongP(pRgb+rgbWidth) = rgbBottom;

	    pY -= (lumaWidth<<1) - 2;
	    pCr -= ycToRgb->chromaWidth - 2;
	    pCb -= ycToRgb->chromaWidth - 2;
	    pRgb -= (rgbWidth<<1) - 4;

	    if (--w <= 0)
		break;
	}

	if ((height -= 4) <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

static void
Lookup555RGBQuadFrame(YcToRgbInfoPtr ycToRgb)  { RgbLookupNotImplemented(ycToRgb); }

#else /* CONFIGURE_555RGB */

static void
Lookup555RGBFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }
static void
Lookup555RGBQuadFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }

#endif /* CONFIGURE_555RGB */


RgbLookupFunc
ChooseRgb555LookupFunc(YcToRgbInfoPtr ycToRgb)
{
    RgbLookupFunc func = NULL;
    
    switch (ycToRgb->components) {
    case PixelRGB:
    case Pixel0RGB:
    case PixelRGB0:
    case PixelBGR:
    case Pixel0BGR:
    case PixelBGR0:
	switch (ycToRgb->scale)	{
	case LookupScaleUnity:	 func =	Lookup555RGBFrame;	   break;
	case LookupScaleQuad:	 func =	Lookup555RGBQuadFrame;	   break;
	}
	break;
    }
    return( func );
}
