/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              rgbDeepLookup.c
 *
 *      DESCRIPTION
 *              Fast conversion routines for y/cr/cb -> rgb
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/rgb888lookup.c,v 1.6 1995/03/10 06:57:47 toddb Exp $
 */


#define NeedFunctionPrototypes 1

#include "nvrAlloc.h"
#include "nvrString.h"
#include "colorSpace.h"
#include "nvrImageI.h"
#include "nvrTimer.h"

#include "rgblookup.h"
#include "rgbdither.h"

#ifdef CONFIGURE_888

#define Rgb888IndexShift        2

static void
Lookup888QuadFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w;
    UChar *pY, *pCr, *pCb;
    UChar *table = ycToRgb->rgbLookupData.u.rgbEqual;
    UChar *targetRgb;
    int crCbIndex, tmp;
    UChar *pRgb;
    ULong y;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    width >>= 2;
    for (;;) {
        /*
         * Each time through the loop we expand 2x2 pixels to 4x4 pixels
         *
         *               x x x x
         *    x x        x x x x
         *    x x   ==>  x x x x
         *               x x x x
         */
	w = width;
	for (;;) {
	    /*
	     * do:  x .      x x . .
	     *      . . ==>  x x . .
	     *               . . . .
	     *               . . . .
	     */
            crCbIndex = *pCr++ >> (8-Rgb888CrBits);
	    crCbIndex <<= Rgb888IndexShift;
            tmp = *pCb++ >> (8-Rgb888CbBits);
            crCbIndex |= tmp << (Rgb888IndexShift+Rgb888CrBits);
	    targetRgb = table+crCbIndex;

            tmp = (y = pY[0]+Rgb888D00) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

	    tmp = (y += Rgb888D10 - Rgb888D00) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);
	    pRgb += 4;

	    tmp = (y += Rgb888D01 - Rgb888D10) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

	    tmp = (y += Rgb888D11 - Rgb888D01) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);
	    pRgb += 4;

	    /*
	     * do:  . x      . . x x
	     *      . . ==>  . . x x
	     *               . . . .
	     *               . . . .
	     */
            tmp = (y = pY[1]+Rgb888D02) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

	    tmp = (y += Rgb888D12 - Rgb888D02) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);
	    pRgb += 4;

	    tmp = (y += Rgb888D03 - Rgb888D12) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);
	    pRgb += rgbWidth;

	    tmp = (y += Rgb888D13 - Rgb888D03) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);
	    pRgb += rgbWidth - 12;

	    /*
	     * do:  . .      . . . .
	     *      x . ==>  . . . .
	     *               x x . .
	     *               x x . .
	     */
            tmp = (y = pY[lumaWidth]+Rgb888D20) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
	    pY++;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

	    tmp = (y += Rgb888D30 - Rgb888D20) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);
	    pRgb += 4;

	    tmp = (y += Rgb888D21 - Rgb888D30) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

	    tmp = (y += Rgb888D31 - Rgb888D21) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);
	    pRgb += 4;

	    /*
	     * do:  . .      . . . .
	     *      . x ==>  . . . .
	     *               . . x x
	     *               . . x x
	     */
            tmp = (y = pY[lumaWidth]+Rgb888D22) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
	    pY++;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

	    tmp = (y += Rgb888D32 - Rgb888D22) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);
	    pRgb += 4;

	    tmp = (y += Rgb888D23 - Rgb888D32) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

	    tmp = (y += Rgb888D33 - Rgb888D23) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);

	    pRgb -= (rgbWidth << 1) - 4;

	    if (--w <= 0)
		break;
	}

	if ((height -= 4) <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

static void
Lookup888Frame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w;
    UChar *pY, *pCr, *pCb;
    UChar *table = ycToRgb->rgbLookupData.u.rgbEqual;
    UChar *targetRgb;
    long crCbIndex, tmp;
    UChar *pRgb;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    width >>= 2;
    for (;;) {
	/*
	 * Each time through the loop does 4x4 pixels
	 * . . . .
	 * . . . .
	 * . . . .
	 * . . . .
	 * The initial value of w, below, must be even
	 */
	w = width;
	for (;;) {
	    /*
	     * do:  x x . .
	     *      x x . .
	     *      . . . .
	     *      . . . .
	     */
            crCbIndex = pCr[0] >> (8-Rgb888CrBits);
	    crCbIndex <<= Rgb888IndexShift;
            tmp = pCb[0] >> (8-Rgb888CbBits);
            crCbIndex |= tmp << (Rgb888IndexShift+Rgb888CrBits);
	    targetRgb = table+crCbIndex;

            tmp = (pY[0]+Rgb888D00)>>(8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

            tmp = (pY[lumaWidth]+Rgb888D10) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
	    pY++;
            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);
	    pRgb += 4;

            tmp = (pY[0]+Rgb888D01) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

            tmp = (pY[lumaWidth]+Rgb888D11) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
	    pY++;
            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);
	    pRgb += 4;

	    /*
	     * do:  . . x x
	     *      . . x x
	     *      . . . .
	     *      . . . .
	     */
            crCbIndex = pCr[1] >> (8-Rgb888CrBits);
	    crCbIndex <<= Rgb888IndexShift;
            tmp = pCb[1] >> (8-Rgb888CbBits);
            crCbIndex |= tmp << (Rgb888IndexShift+Rgb888CrBits);
	    targetRgb = table+crCbIndex;

            tmp = (pY[0]+Rgb888D02) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

            tmp = (pY[lumaWidth]+Rgb888D12) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
	    pY++;
            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);
	    pRgb += 4;

            tmp = (pY[0]+Rgb888D03) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);
	    pRgb += rgbWidth;

            tmp = (pY[lumaWidth]+Rgb888D13) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            pY += (lumaWidth<<1) - 3;

            *ULongP(pRgb) = *ULongP(targetRgb+tmp);
	    pRgb += rgbWidth - 12;

	    /*
	     * do:  . . . .
	     *      . . . .
	     *      x x . .
	     *      x x . .
	     */
	    pCr += ycToRgb->chromaWidth;
	    pCb += ycToRgb->chromaWidth;

            crCbIndex = *pCr++ >> (8-Rgb888CrBits);
	    crCbIndex <<= Rgb888IndexShift;
            tmp = *pCb++ >> (8-Rgb888CbBits);
            crCbIndex |= tmp << (Rgb888IndexShift+Rgb888CrBits);
	    targetRgb = table+crCbIndex;

            tmp = (pY[0]+Rgb888D20) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

            tmp = (pY[lumaWidth]+Rgb888D30) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
	    pY++;
            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);
	    pRgb += 4;

            tmp = (pY[0]+Rgb888D21) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

            tmp = (pY[lumaWidth]+Rgb888D31) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
	    pY++;
            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);
	    pRgb += 4;

	    /*
	     * do:  . . . .
	     *      . . . .
	     *      . . x x
	     *      . . x x
	     */
            crCbIndex = pCr[0] >> (8-Rgb888CrBits);
	    pCr -= ycToRgb->chromaWidth - 1;
	    crCbIndex <<= Rgb888IndexShift;
            tmp = pCb[0] >> (8-Rgb888CbBits);
	    pCb -= ycToRgb->chromaWidth - 1;
            crCbIndex |= tmp << (Rgb888IndexShift+Rgb888CrBits);
	    targetRgb = table+crCbIndex;

            tmp = (pY[0]+Rgb888D22) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

            tmp = (pY[lumaWidth]+Rgb888D32) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
	    pY++;
            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);
	    pRgb += 4;

            tmp = (pY[0]+Rgb888D23) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            *ULongP(pRgb) = *ULongP(targetRgb+tmp);

            tmp = (pY[lumaWidth]+Rgb888D33) >> (8-Rgb888YBits);
            tmp <<= Rgb888IndexShift+Rgb888CrBits+Rgb888CbBits;
            pY -= (lumaWidth<<1) - 1;

            *ULongP(pRgb+rgbWidth) = *ULongP(targetRgb+tmp);
	    pRgb -= (rgbWidth<<1) - 4;

	    if (--w <= 0)
		break;
	}

	if ((height -= 4) <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

#else /* CONFIGURE_888 */

static void
Lookup888Frame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }
static void
Lookup888QuadFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }

#endif /* CONFIGURE_888 */

#ifdef CONFIGURE_888RGB
static void
Lookup888RGBFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w, h;
    UChar *pY, *pCr, *pCb;
    int redIndex, greenIndex, blueIndex;
    UChar *redTbl, *greenTbl, *blueTbl;
    UChar *pRgb;
    ULong y, cr, cb;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    /*
     * The initial value of h, below, must be even, because macroblocks
     * are 16=4x4 pixels tall.
     */
    h = height>>1;
    width >>= 1;
    for (;;) {
	/*
	 * Each time through the loop does 2x2 pixels
	 * . .
	 * . .
	 * The initial value of w, below, must be even
	 */
	w = width;
	for (;;) {
	    /*
	     * do:  x x
	     *      . .
	     */
	    cr = *pCr++;
	    redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
	    redTbl = &ycToRgb->rgbLookupData.u.rgb888.red[ redIndex ];

	    greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

	    cb = *pCb++;
	    greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
	    greenTbl = &ycToRgb->rgbLookupData.u.rgb888.green[ greenIndex ];

	    blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
	    blueTbl = &ycToRgb->rgbLookupData.u.rgb888.blue[ blueIndex ];

	    y = pY[0] << RGB888FracBits;
	    pRgb[0] =   redTbl[ y ];
	    pRgb[1] = greenTbl[ y ];
	    pRgb[2] =  blueTbl[ y ];

	    y = pY[1] << RGB888FracBits;
	    pRgb[3] =   redTbl[ y ];
	    pRgb[4] = greenTbl[ y ];
	    pRgb[5] =  blueTbl[ y ];

	    /*
	     * do:  . .
	     *      x x
	     */
	    pY += lumaWidth;
	    y = pY[0] << RGB888FracBits;
	    pRgb += rgbWidth;
	    pRgb[0] =   redTbl[ y ];
	    pRgb[1] = greenTbl[ y ];
	    pRgb[2] =  blueTbl[ y ];

	    y = pY[1] << RGB888FracBits;
	    pRgb[3] =   redTbl[ y ];
	    pRgb[4] = greenTbl[ y ];
	    pRgb[5] =  blueTbl[ y ];

	    pRgb += 6 - rgbWidth;
	    pY   += sizeof(UShort) - lumaWidth;

	    if (--w <= 0)
		break;
	}

	if (--h <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

static void
Lookup888RGBQuadFrame(YcToRgbInfoPtr ycToRgb)  { RgbLookupNotImplemented(ycToRgb); }

#else /* CONFIGURE_888RGB */

static void
Lookup888RGBFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }
static void
Lookup888RGBQuadFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }

#endif /* CONFIGURE_888RGB */

#ifdef CONFIGURE_8880RGB

static void
Lookup8880RGBFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w, h;
    UChar *pY, *pCr, *pCb;
    int redIndex, greenIndex, blueIndex;
    UChar *redTbl, *greenTbl, *blueTbl;
    UChar *pRgb;
    ULong y, cr, cb, rgb;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    /*
     * The initial value of h, below, must be even, because macroblocks
     * are 16=4x4 pixels tall.
     */
    h = height>>1;
    width >>= 1;
    for (;;) {
	/*
	 * Each time through the loop does 2x2 pixels
	 * . .
	 * . .
	 * The initial value of w, below, must be even
	 */
	w = width;
	for (;;) {
	    /*
	     * do:  x x
	     *      . .
	     */
	    cr = *pCr++;
	    redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
	    redTbl = &ycToRgb->rgbLookupData.u.rgb888.red[ redIndex ];

	    greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

	    cb = *pCb++;
	    greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
	    greenTbl = &ycToRgb->rgbLookupData.u.rgb888.green[ greenIndex ];

	    blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
	    blueTbl = &ycToRgb->rgbLookupData.u.rgb888.blue[ blueIndex ];

	    y = pY[0] << RGB888FracBits;
	    rgb  =   redTbl[ y ] << 24;
	    rgb |= greenTbl[ y ] << 16;
	    rgb |=  blueTbl[ y ] << 8;
	    ULongP(pRgb)[0] = rgb;

	    y = pY[1] << RGB888FracBits;
	    rgb  =   redTbl[ y ] << 24;
	    rgb |= greenTbl[ y ] << 16;
	    rgb |=  blueTbl[ y ] << 8;
	    ULongP(pRgb)[1] = rgb;

	    /*
	     * do:  . .
	     *      x x
	     */
	    pY += lumaWidth;
	    y = pY[0] << RGB888FracBits;
	    pRgb += rgbWidth;
	    rgb  =   redTbl[ y ] << 24;
	    rgb |= greenTbl[ y ] << 16;
	    rgb |=  blueTbl[ y ] << 8;
	    ULongP(pRgb)[0] = rgb;

	    y = pY[1] << RGB888FracBits;
	    rgb  =   redTbl[ y ] << 24;
	    rgb |= greenTbl[ y ] << 16;
	    rgb |=  blueTbl[ y ] << 8;
	    ULongP(pRgb)[1] = rgb;

	    pRgb += 8 - rgbWidth;
	    pY   += sizeof(UShort) - lumaWidth;

	    if (--w <= 0)
		break;
	}

	if (--h <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

static void
Lookup8880RGBQuadFrame(YcToRgbInfoPtr ycToRgb)  { RgbLookupNotImplemented(ycToRgb); }

#else /* CONFIGURE_8880RGB */

static void
Lookup8880RGBFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }
static void
Lookup8880RGBQuadFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }

#endif /* CONFIGURE_8880RGB */

#ifdef CONFIGURE_0888RGB

static void
Lookup0888RGBFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w, h;
    UChar *pY, *pCr, *pCb;
    int redIndex, greenIndex, blueIndex;
    UChar *redTbl, *greenTbl, *blueTbl;
    UChar *pRgb;
    ULong y, cr, cb, rgb;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    /*
     * The initial value of h, below, must be even, because macroblocks
     * are 16=4x4 pixels tall.
     */
    h = height>>1;
    width >>= 1;
    for (;;) {
	/*
	 * Each time through the loop does 2x2 pixels
	 * . .
	 * . .
	 * The initial value of w, below, must be even
	 */
	w = width;
	for (;;) {
	    /*
	     * do:  x x
	     *      . .
	     */
	    cr = *pCr++;
	    redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
	    redTbl = &ycToRgb->rgbLookupData.u.rgb888.red[ redIndex ];

	    greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

	    cb = *pCb++;
	    greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
	    greenTbl = &ycToRgb->rgbLookupData.u.rgb888.green[ greenIndex ];

	    blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
	    blueTbl = &ycToRgb->rgbLookupData.u.rgb888.blue[ blueIndex ];

	    y = pY[0] << RGB888FracBits;
	    rgb  =   redTbl[ y ] << 16;
	    rgb |= greenTbl[ y ] << 8;
	    rgb |=  blueTbl[ y ];
	    ULongP(pRgb)[0] = rgb;

	    y = pY[1] << RGB888FracBits;
	    rgb  =   redTbl[ y ] << 16;
	    rgb |= greenTbl[ y ] << 8;
	    rgb |=  blueTbl[ y ];
	    ULongP(pRgb)[1] = rgb;

	    /*
	     * do:  . .
	     *      x x
	     */
	    pY += lumaWidth;
	    y = pY[0] << RGB888FracBits;
	    pRgb += rgbWidth;
	    rgb  =   redTbl[ y ] << 16;
	    rgb |= greenTbl[ y ] << 8;
	    rgb |=  blueTbl[ y ];
	    ULongP(pRgb)[0] = rgb;

	    y = pY[1] << RGB888FracBits;
	    rgb  =   redTbl[ y ] << 16;
	    rgb |= greenTbl[ y ] << 8;
	    rgb |=  blueTbl[ y ];
	    ULongP(pRgb)[1] = rgb;

	    pRgb += 8 - rgbWidth;
	    pY   += sizeof(UShort) - lumaWidth;

	    if (--w <= 0)
		break;
	}

	if (--h <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

static void
Lookup0888RGBQuadFrame(YcToRgbInfoPtr ycToRgb)  { RgbLookupNotImplemented(ycToRgb); }

#else /* CONFIGURE_0888RGB */

static void
Lookup0888RGBFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }
static void
Lookup0888RGBQuadFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }

#endif /* CONFIGURE_0888RGB */

#ifdef CONFIGURE_888BGR
static void
Lookup888BGRFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w, h;
    UChar *pY, *pCr, *pCb;
    int redIndex, greenIndex, blueIndex;
    UChar *redTbl, *greenTbl, *blueTbl;
    UChar *pRgb;
    ULong y, cr, cb;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    /*
     * The initial value of h, below, must be even, because macroblocks
     * are 16=4x4 pixels tall.
     */
    h = height>>1;
    width >>= 1;
    for (;;) {
	/*
	 * Each time through the loop does 2x2 pixels
	 * . .
	 * . .
	 * The initial value of w, below, must be even
	 */
	w = width;
	for (;;) {
	    /*
	     * do:  x x
	     *      . .
	     */
	    cr = *pCr++;
	    redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
	    redTbl = &ycToRgb->rgbLookupData.u.rgb888.red[ redIndex ];

	    greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

	    cb = *pCb++;
	    greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
	    greenTbl = &ycToRgb->rgbLookupData.u.rgb888.green[ greenIndex ];

	    blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
	    blueTbl = &ycToRgb->rgbLookupData.u.rgb888.blue[ blueIndex ];

	    y = pY[0] << RGB888FracBits;
	    pRgb[2] =   redTbl[ y ];
	    pRgb[1] = greenTbl[ y ];
	    pRgb[0] =  blueTbl[ y ];

	    y = pY[1] << RGB888FracBits;
	    pRgb[5] =   redTbl[ y ];
	    pRgb[4] = greenTbl[ y ];
	    pRgb[3] =  blueTbl[ y ];

	    /*
	     * do:  . .
	     *      x x
	     */
	    pY += lumaWidth;
	    y = pY[0] << RGB888FracBits;
	    pRgb += rgbWidth;
	    pRgb[2] =   redTbl[ y ];
	    pRgb[1] = greenTbl[ y ];
	    pRgb[0] =  blueTbl[ y ];

	    y = pY[1] << RGB888FracBits;
	    pRgb[5] =   redTbl[ y ];
	    pRgb[4] = greenTbl[ y ];
	    pRgb[3] =  blueTbl[ y ];

	    pRgb += 6 - rgbWidth;
	    pY   += sizeof(UShort) - lumaWidth;

	    if (--w <= 0)
		break;
	}

	if (--h <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

static void
Lookup888BGRQuadFrame(YcToRgbInfoPtr ycToRgb)  { RgbLookupNotImplemented(ycToRgb); }

#else /* CONFIGURE_888BGR */

static void
Lookup888BGRFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }
static void
Lookup888BGRQuadFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }

#endif /* CONFIGURE_888BGR */

#ifdef CONFIGURE_8880BGR

static void
Lookup8880BGRFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w, h;
    UChar *pY, *pCr, *pCb;
    int redIndex, greenIndex, blueIndex;
    UChar *redTbl, *greenTbl, *blueTbl;
    UChar *pRgb;
    ULong y, cr, cb, rgb;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    /*
     * The initial value of h, below, must be even, because macroblocks
     * are 16=4x4 pixels tall.
     */
    h = height>>1;
    width >>= 1;
    for (;;) {
	/*
	 * Each time through the loop does 2x2 pixels
	 * . .
	 * . .
	 * The initial value of w, below, must be even
	 */
	w = width;
	for (;;) {
	    /*
	     * do:  x x
	     *      . .
	     */
	    cr = *pCr++;
	    redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
	    redTbl = &ycToRgb->rgbLookupData.u.rgb888.red[ redIndex ];

	    greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

	    cb = *pCb++;
	    greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
	    greenTbl = &ycToRgb->rgbLookupData.u.rgb888.green[ greenIndex ];

	    blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
	    blueTbl = &ycToRgb->rgbLookupData.u.rgb888.blue[ blueIndex ];

	    y = pY[0] << RGB888FracBits;
	    rgb  =   redTbl[ y ] << 8;
	    rgb |= greenTbl[ y ] << 16;
	    rgb |=  blueTbl[ y ] << 24;
	    ULongP(pRgb)[0] = rgb;

	    y = pY[1] << RGB888FracBits;
	    rgb  =   redTbl[ y ] << 8;
	    rgb |= greenTbl[ y ] << 16;
	    rgb |=  blueTbl[ y ] << 24;
	    ULongP(pRgb)[1] = rgb;

	    /*
	     * do:  . .
	     *      x x
	     */
	    pY += lumaWidth;
	    y = pY[0] << RGB888FracBits;
	    pRgb += rgbWidth;
	    rgb  =   redTbl[ y ] << 8;
	    rgb |= greenTbl[ y ] << 16;
	    rgb |=  blueTbl[ y ] << 24;
	    ULongP(pRgb)[0] = rgb;

	    y = pY[1] << RGB888FracBits;
	    rgb  =   redTbl[ y ] << 8;
	    rgb |= greenTbl[ y ] << 16;
	    rgb |=  blueTbl[ y ] << 24;
	    ULongP(pRgb)[1] = rgb;

	    pRgb += 8 - rgbWidth;
	    pY   += sizeof(UShort) - lumaWidth;

	    if (--w <= 0)
		break;
	}

	if (--h <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

static void
Lookup8880BGRQuadFrame(YcToRgbInfoPtr ycToRgb)  { RgbLookupNotImplemented(ycToRgb); }

#else /* CONFIGURE_8880BGR */

static void
Lookup8880BGRFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }
static void
Lookup8880BGRQuadFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }

#endif /* CONFIGURE_8880BGR */

#ifdef CONFIGURE_0888BGR

static void
Lookup0888BGRFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w, h;
    UChar *pY, *pCr, *pCb;
    int redIndex, greenIndex, blueIndex;
    UChar *redTbl, *greenTbl, *blueTbl;
    UChar *pRgb;
    ULong y, cr, cb, rgb;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    /*
     * The initial value of h, below, must be even, because macroblocks
     * are 16=4x4 pixels tall.
     */
    h = height>>1;
    width >>= 1;
    for (;;) {
	/*
	 * Each time through the loop does 2x2 pixels
	 * . .
	 * . .
	 * The initial value of w, below, must be even
	 */
	w = width;
	for (;;) {
	    /*
	     * do:  x x
	     *      . .
	     */
	    cr = *pCr++;
	    redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
	    redTbl = &ycToRgb->rgbLookupData.u.rgb888.red[ redIndex ];

	    greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

	    cb = *pCb++;
	    greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
	    greenTbl = &ycToRgb->rgbLookupData.u.rgb888.green[ greenIndex ];

	    blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
	    blueTbl = &ycToRgb->rgbLookupData.u.rgb888.blue[ blueIndex ];

	    y = pY[0] << RGB888FracBits;
	    rgb  =   redTbl[ y ];
	    rgb |= greenTbl[ y ] << 8;
	    rgb |=  blueTbl[ y ] << 16;
	    ULongP(pRgb)[0] = rgb;

	    y = pY[1] << RGB888FracBits;
	    rgb  =   redTbl[ y ];
	    rgb |= greenTbl[ y ] << 8;
	    rgb |=  blueTbl[ y ] << 16;
	    ULongP(pRgb)[1] = rgb;

	    /*
	     * do:  . .
	     *      x x
	     */
	    pY += lumaWidth;
	    y = pY[0] << RGB888FracBits;
	    pRgb += rgbWidth;
	    rgb  =   redTbl[ y ];
	    rgb |= greenTbl[ y ] << 8;
	    rgb |=  blueTbl[ y ] << 16;
	    ULongP(pRgb)[0] = rgb;

	    y = pY[1] << RGB888FracBits;
	    rgb  =   redTbl[ y ];
	    rgb |= greenTbl[ y ] << 8;
	    rgb |=  blueTbl[ y ] << 16;
	    ULongP(pRgb)[1] = rgb;

	    pRgb += 8 - rgbWidth;
	    pY   += sizeof(UShort) - lumaWidth;

	    if (--w <= 0)
		break;
	}

	if (--h <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

static void
Lookup0888BGRQuadFrame(YcToRgbInfoPtr ycToRgb)  { RgbLookupNotImplemented(ycToRgb); }

#else /* CONFIGURE_0888BGR */

static void
Lookup0888BGRFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }
static void
Lookup0888BGRQuadFrame(YcToRgbInfoPtr ycToRgb) { RgbLookupNotConfigured(ycToRgb); }

#endif /* CONFIGURE_0888BGR */

RgbLookupFunc
ChooseRgb888LookupFunc(YcToRgbInfoPtr ycToRgb)
{
    RgbLookupFunc func = NULL;
    
    if (ycToRgb->multiStepLookup) {
	switch (ycToRgb->components) {
	case PixelRGB:
	    switch (ycToRgb->scale)	{
	    case LookupScaleUnity: func = Lookup888RGBFrame;	  break;
	    case LookupScaleQuad:  func = Lookup888RGBQuadFrame;  break;
	    }
	    break;
	case Pixel0RGB:
	    switch (ycToRgb->scale)	{
	    case LookupScaleUnity: func = Lookup0888RGBFrame;	   break;
	    case LookupScaleQuad:  func = Lookup0888RGBQuadFrame;  break;
	    }
	    break;
	case PixelRGB0:
	    switch (ycToRgb->scale)	{
	    case LookupScaleUnity: func = Lookup8880RGBFrame;	   break;
	    case LookupScaleQuad:  func = Lookup8880RGBQuadFrame;  break;
	    }
	    break;
	case PixelBGR:
	    switch (ycToRgb->scale)	{
	    case LookupScaleUnity: func = Lookup888BGRFrame;	  break;
	    case LookupScaleQuad:  func = Lookup888BGRQuadFrame;  break;
	    }
	    break;
	case Pixel0BGR:
	    switch (ycToRgb->scale)	{
	    case LookupScaleUnity: func = Lookup0888BGRFrame;	   break;
	    case LookupScaleQuad:  func = Lookup0888BGRQuadFrame;  break;
	    }
	    break;
	case PixelBGR0:
	    switch (ycToRgb->scale)	{
	    case LookupScaleUnity: func = Lookup8880BGRFrame;	   break;
	    case LookupScaleQuad:  func = Lookup8880BGRQuadFrame;  break;
	    }
	    break;
	}
    } else {
	switch (ycToRgb->scale)	{
	case LookupScaleUnity: func = Lookup888Frame;	   break;
	case LookupScaleQuad:  func = Lookup888QuadFrame;  break;
	}
    }

    return( func );
}
