/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              rgblookup.c
 *
 *      DESCRIPTION
 *              Fast conversion routines for y/cr/cb -> rgb
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/rgblookup.c,v 1.22 1995/10/27 17:41:25 russ Exp $
 */


#define NeedFunctionPrototypes 1

#include "nvrAlloc.h"
#include "nvrString.h"
#include "colorSpace.h"
#include "nvrImageI.h"
#include "nvrTimer.h"

#include "rgblookup.h"
#include "rgbdither.h"

#ifdef DEBUG_ENABLED

static void SetRgbTableChecksum(YcToRgbInfoPtr);
static void CheckRgbTableChecksum(YcToRgbInfoPtr);

#else /* DEBUG_ENABLED */

#define SetRgbTableChecksum(YcToRgbInfoPtr)     /* as nothing */
#define CheckRgbTableChecksum(YcToRgbInfoPtr)   /* as nothing */

#endif /* DEBUG_ENABLED */

static void
Lookup332QuadRGBFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w;
    UChar *pY, *pCr, *pCb;
    int redIndex, greenIndex, blueIndex;
    UChar *redTbl, *greenTbl, *blueTbl;
    UChar *pRgb;
    ULong y, cr, cb;
    ULong rgbTop, rgbBottom;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    height >>= 2;
    width >>= 2;
    for (;;) {
        /*
         * Each time through the loop interpolates 2x2 pixels out to 4x4 pixels
         * . . . .
         * . . . .
         * . . . .
         * . . . .
         */
        w = width;
        for (;;) {
	    /*
	     * do:  x .      x x . .
	     *      . . ==>  x x . .
	     *               . . . .
	     *               . . . .
	     */
            cr = *pCr++;
            redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
            redTbl = &ycToRgb->rgbLookupData.u.rgb332.red[ redIndex ];

            greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

            cb = *pCb++;
            greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
            greenTbl = &ycToRgb->rgbLookupData.u.rgb332.green[ greenIndex ];

            blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
            blueTbl = &ycToRgb->rgbLookupData.u.rgb332.blue[ blueIndex ];

	    y = pY[0] + Rgb332D00;
	    rgbTop = greenTbl[ y ];
	    y += Rgb332D00;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];
	    rgbTop <<= 8;

	    y += Rgb332D01 - (2*Rgb332D00);
	    rgbTop |= greenTbl[ y ];
	    y += Rgb332D01;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];
	    rgbTop <<= 8;

	    y += Rgb332D10 - (2*Rgb332D01);;
	    rgbBottom = greenTbl[ y ];
	    y += Rgb332D10;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];
	    rgbBottom <<= 8;

	    y += Rgb332D11 - (2*Rgb332D10);
	    rgbBottom |= greenTbl[ y ];
	    y += Rgb332D11;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];
	    rgbBottom <<= 8;

	    /*
	     * do:  . x      . . x x
	     *      . . ==>  . . x x
	     *               . . . .
	     *               . . . .
	     */
	    y = pY[1] + Rgb332D02;
	    rgbTop |= greenTbl[ y ];
	    y += Rgb332D02;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];
	    rgbTop <<= 8;

	    y += Rgb332D03 - (2*Rgb332D02);
	    rgbTop |= greenTbl[ y ];
	    y += Rgb332D03;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];

	    *ULongP(pRgb) = rgbTop;
	    pRgb += rgbWidth;

	    y += Rgb332D12 - (2*Rgb332D03);
	    rgbBottom |= greenTbl[ y ];
	    y += Rgb332D12;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];
	    rgbBottom <<= 8;

	    y += Rgb332D13 - (2*Rgb332D12);
	    rgbBottom |= greenTbl[ y ];
	    y += Rgb332D13;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];

	    *ULongP(pRgb) = rgbBottom;
	    pRgb += rgbWidth;

	    /*
	     * do:  . .      . . . .
	     *      x . ==>  . . . .
	     *               x x . .
	     *               x x . .
	     */
	    y = pY[lumaWidth] + Rgb332D20;
	    pY++;
	    rgbTop = greenTbl[ y ];
	    y += Rgb332D20;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];
	    rgbTop <<= 8;

	    y += Rgb332D21 - (2*Rgb332D20);
	    rgbTop |= greenTbl[ y ];
	    y += Rgb332D21;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];
	    rgbTop <<= 8;

	    y += Rgb332D30 - (2*Rgb332D31);;
	    rgbBottom = greenTbl[ y ];
	    y += Rgb332D30;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];
	    rgbBottom <<= 8;

	    y += Rgb332D31 - (2*Rgb332D30);
	    rgbBottom |= greenTbl[ y ];
	    y += Rgb332D31;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];
	    rgbBottom <<= 8;

	    /*
	     * do:  . .      . . . .
	     *      . x ==>  . . . .
	     *               . . x x
	     *               . . x x
	     */
	    y = pY[lumaWidth] + Rgb332D22;
	    pY++;
	    rgbTop |= greenTbl[ y ];
	    y += Rgb332D22;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];
	    rgbTop <<= 8;

	    y += Rgb332D23 - (2*Rgb332D22);
	    rgbTop |= greenTbl[ y ];
	    y += Rgb332D23;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];

	    *ULongP(pRgb) = rgbTop;

	    y += Rgb332D32 - (2*Rgb332D33);
	    rgbBottom |= greenTbl[ y ];
	    y += Rgb332D32;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];
	    rgbBottom <<= 8;

	    y += Rgb332D33 - (2*Rgb332D32);
	    rgbBottom |= greenTbl[ y ];
	    y += Rgb332D33;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];

	    *ULongP(pRgb + rgbWidth) = rgbBottom;
	    pRgb -= (rgbWidth<<1) - 4;

            if (--w <= 0)
                break;
        }

        if (--height <= 0)
            break;

        pRgb += ycToRgb->rgbAccel;
        pY   += ycToRgb->lumaAccel;
        pCr  += ycToRgb->chromaAccel;
        pCb  += ycToRgb->chromaAccel;
    }
}

static void
LookupC666QuadRGBFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w;
    UChar *pY, *pCr, *pCb;
    UChar *table = ycToRgb->rgbLookupData.u.rgbEqual;
    UChar *targetRgb;
    long tmp;
    UChar *pRgb;
    ULong rgbTop, rgbBottom, crCbIndex, y;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    width >>= 2;
    for (;;) {
        /*
         * Each time through the loop we expand 2x2 pixels to 4x4 pixels
         *
         *               x x x x
         *    x x        x x x x
         *    x x   ==>  x x x x
         *               x x x x
         */
	w = width;
	for (;;) {
	    /*
	     * do:  x .      x x . .
	     *      . . ==>  x x . .
	     *               . . . .
	     *               . . . .
	     */
            crCbIndex = *pCr++ >> (8-RgbC666CrBits);
	    crCbIndex <<= RgbC666YBits;
            tmp = *pCb++ >> (8-RgbC666CbBits);
            crCbIndex |= tmp << (RgbC666CrBits+RgbC666YBits);
	    targetRgb = table+crCbIndex;

            tmp = (y = pY[0]+RgbC666D00) >> (8-RgbC666YBits+1);
            rgbTop = *(targetRgb+tmp);
            rgbTop <<= 8;

	    y += RgbC666D01 - RgbC666D00;
            tmp = y >> (8-RgbC666YBits+1);
            rgbTop |= *(targetRgb+tmp);
            rgbTop <<= 8;

	    y += RgbC666D10 - RgbC666D01;
            tmp = y >> (8-RgbC666YBits+1);
            rgbBottom = *(targetRgb+tmp);
            rgbBottom <<= 8;

	    y += RgbC666D11 - RgbC666D10;
            tmp = y >> (8-RgbC666YBits+1);
            rgbBottom |= *(targetRgb+tmp);
            rgbBottom <<= 8;

	    /*
	     * do:  . x      . . x x
	     *      . . ==>  . . x x
	     *               . . . .
	     *               . . . .
	     */
            tmp = (y = pY[1]+RgbC666D02) >> (8-RgbC666YBits+1);
            rgbTop |= *(targetRgb+tmp);
            rgbTop <<= 8;

	    y += RgbC666D03 - RgbC666D02;
            tmp = y >> (8-RgbC666YBits+1);
            rgbTop |= *(targetRgb+tmp);

	    *ULongP(pRgb) = rgbTop;
	    pRgb += rgbWidth;

	    y += RgbC666D12 - RgbC666D03;
            tmp = y >> (8-RgbC666YBits+1);
            rgbBottom |= *(targetRgb+tmp);
            rgbBottom <<= 8;

	    y += RgbC666D13 - RgbC666D12;
            tmp = y >> (8-RgbC666YBits+1);
            rgbBottom |= *(targetRgb+tmp);

	    *ULongP(pRgb) = rgbBottom;
	    pRgb += rgbWidth;

	    /*
	     * do:  . .      . . . .
	     *      x . ==>  . . . .
	     *               x x . .
	     *               x x . .
	     */

            tmp = (y = pY[lumaWidth]+RgbC666D20) >> (8-RgbC666YBits+1);
	    pY++;
            rgbTop = *(targetRgb+tmp);
            rgbTop <<= 8;

	    y += RgbC666D21 - RgbC666D20;
            tmp = y >> (8-RgbC666YBits+1);
            rgbTop |= *(targetRgb+tmp);
            rgbTop <<= 8;

	    y += RgbC666D30 - RgbC666D21;
            tmp = y >> (8-RgbC666YBits+1);
            rgbBottom = *(targetRgb+tmp);
            rgbBottom <<= 8;

	    y += RgbC666D31 - RgbC666D30;
            tmp = y >> (8-RgbC666YBits+1);
            rgbBottom |= *(targetRgb+tmp);
            rgbBottom <<= 8;

	    /*
	     * do:  . .      . . . .
	     *      . x ==>  . . . .
	     *               . . x x
	     *               . . x x
	     */
            tmp = (y = pY[lumaWidth]+RgbC666D22) >> (8-RgbC666YBits+1);
	    pY++;
            rgbTop |= *(targetRgb+tmp);
            rgbTop <<= 8;

	    y += RgbC666D23 - RgbC666D22;
            tmp = y >> (8-RgbC666YBits+1);
            rgbTop |= *(targetRgb+tmp);

	    *ULongP(pRgb) = rgbTop;

	    y += RgbC666D32 - RgbC666D23;
            tmp = y >> (8-RgbC666YBits+1);
            rgbBottom |= *(targetRgb+tmp);
            rgbBottom <<= 8;

	    y += RgbC666D33 - RgbC666D32;
            tmp = y >> (8-RgbC666YBits+1);
            rgbBottom |= *(targetRgb+tmp);

	    *ULongP(pRgb+rgbWidth) = rgbBottom;
	    pRgb -= (rgbWidth<<1) - 4;

	    if (--w <= 0)
		break;
	}

	if ((height -= 4) <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

static void
LookupC555QuadRGBFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w;
    UChar *pY, *pCr, *pCb;
    UChar *table = ycToRgb->rgbLookupData.u.rgbEqual;
    UChar *targetRgb;
    long tmp;
    UChar *pRgb;
    ULong rgbTop, rgbBottom, crCbIndex, y;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    width >>= 2;
    for (;;) {
        /*
         * Each time through the loop we expand 2x2 pixels to 4x4 pixels
         *
         *               x x x x
         *    x x        x x x x
         *    x x   ==>  x x x x
         *               x x x x
         */
	w = width;
	for (;;) {
	    /*
	     * do:  x .      x x . .
	     *      . . ==>  x x . .
	     *               . . . .
	     *               . . . .
	     */
            crCbIndex = *pCr++ >> (8-RgbC555CrBits);
	    crCbIndex <<= RgbC555YBits;
            tmp = *pCb++ >> (8-RgbC555CbBits);
            crCbIndex |= tmp << (RgbC555CrBits+RgbC555YBits);
	    targetRgb = table+crCbIndex;

            tmp = (y = pY[0]+RgbC555D00) >> (8-RgbC555YBits+1);
            rgbTop = *(targetRgb+tmp);
            rgbTop <<= 8;

	    y += RgbC555D01 - RgbC555D00;
            tmp = y >> (8-RgbC555YBits+1);
            rgbTop |= *(targetRgb+tmp);
            rgbTop <<= 8;

	    y += RgbC555D10 - RgbC555D01;
            tmp = y >> (8-RgbC555YBits+1);
            rgbBottom = *(targetRgb+tmp);
            rgbBottom <<= 8;

	    y += RgbC555D11 - RgbC555D10;
            tmp = y >> (8-RgbC555YBits+1);
            rgbBottom |= *(targetRgb+tmp);
            rgbBottom <<= 8;

	    /*
	     * do:  . x      . . x x
	     *      . . ==>  . . x x
	     *               . . . .
	     *               . . . .
	     */
            tmp = (y = pY[1]+RgbC555D02) >> (8-RgbC555YBits+1);
            rgbTop |= *(targetRgb+tmp);
            rgbTop <<= 8;

	    y += RgbC555D03 - RgbC555D02;
            tmp = y >> (8-RgbC555YBits+1);
            rgbTop |= *(targetRgb+tmp);

	    *ULongP(pRgb) = rgbTop;
	    pRgb += rgbWidth;

	    y += RgbC555D12 - RgbC555D03;
            tmp = y >> (8-RgbC555YBits+1);
            rgbBottom |= *(targetRgb+tmp);
            rgbBottom <<= 8;

	    y += RgbC555D13 - RgbC555D12;
            tmp = y >> (8-RgbC555YBits+1);
            rgbBottom |= *(targetRgb+tmp);

	    *ULongP(pRgb) = rgbBottom;
	    pRgb += rgbWidth;

	    /*
	     * do:  . .      . . . .
	     *      x . ==>  . . . .
	     *               x x . .
	     *               x x . .
	     */

            tmp = (y = pY[lumaWidth]+RgbC555D20) >> (8-RgbC555YBits+1);
	    pY++;
            rgbTop = *(targetRgb+tmp);
            rgbTop <<= 8;

	    y += RgbC555D21 - RgbC555D20;
            tmp = y >> (8-RgbC555YBits+1);
            rgbTop |= *(targetRgb+tmp);
            rgbTop <<= 8;

	    y += RgbC555D30 - RgbC555D21;
            tmp = y >> (8-RgbC555YBits+1);
            rgbBottom = *(targetRgb+tmp);
            rgbBottom <<= 8;

	    y += RgbC555D31 - RgbC555D30;
            tmp = y >> (8-RgbC555YBits+1);
            rgbBottom |= *(targetRgb+tmp);
            rgbBottom <<= 8;

	    /*
	     * do:  . .      . . . .
	     *      . x ==>  . . . .
	     *               . . x x
	     *               . . x x
	     */
            tmp = (y = pY[lumaWidth]+RgbC555D22) >> (8-RgbC555YBits+1);
	    pY++;
            rgbTop |= *(targetRgb+tmp);
            rgbTop <<= 8;

	    y += RgbC555D23 - RgbC555D22;
            tmp = y >> (8-RgbC555YBits+1);
            rgbTop |= *(targetRgb+tmp);

	    *ULongP(pRgb) = rgbTop;

	    y += RgbC555D32 - RgbC555D23;
            tmp = y >> (8-RgbC555YBits+1);
            rgbBottom |= *(targetRgb+tmp);
            rgbBottom <<= 8;

	    y += RgbC555D33 - RgbC555D32;
            tmp = y >> (8-RgbC555YBits+1);
            rgbBottom |= *(targetRgb+tmp);

	    *ULongP(pRgb+rgbWidth) = rgbBottom;
	    pRgb -= (rgbWidth<<1) - 4;

	    if (--w <= 0)
		break;
	}

	if ((height -= 4) <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

static void
Lookup232QuadRGBFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w;
    UChar *pY, *pCr, *pCb;
    int redIndex, greenIndex, blueIndex;
    UChar *redTbl, *greenTbl, *blueTbl;
    UChar *pRgb;
    ULong y, cr, cb;
    ULong rgbTop, rgbBottom;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    height >>= 2;
    width >>= 2;
    for (;;) {
        /*
         * Each time through the loop interpolates 2x2 pixels out to 4x4 pixels
         * . . . .
         * . . . .
         * . . . .
         * . . . .
         */
        w = width;
        for (;;) {
	    /*
	     * do:  x .      x x . .
	     *      . . ==>  x x . .
	     *               . . . .
	     *               . . . .
	     */
            cr = *pCr++;
            redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
            redTbl = &ycToRgb->rgbLookupData.u.rgb232.red[ redIndex ];

            greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

            cb = *pCb++;
            greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
            greenTbl = &ycToRgb->rgbLookupData.u.rgb232.green[ greenIndex ];

            blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
            blueTbl = &ycToRgb->rgbLookupData.u.rgb232.blue[ blueIndex ];

	    y = pY[0] + Rgb232D00;
	    rgbTop = greenTbl[ y ];
	    y += Rgb232D00;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];
	    rgbTop <<= 8;

	    y += Rgb232D01 - (2*Rgb232D00);
	    rgbTop |= greenTbl[ y ];
	    y += Rgb232D01;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];
	    rgbTop <<= 8;

	    y += Rgb232D10 - (2*Rgb232D01);;
	    rgbBottom = greenTbl[ y ];
	    y += Rgb232D10;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];
	    rgbBottom <<= 8;

	    y += Rgb232D11 - (2*Rgb232D10);
	    rgbBottom |= greenTbl[ y ];
	    y += Rgb232D11;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];
	    rgbBottom <<= 8;

	    /*
	     * do:  . x      . . x x
	     *      . . ==>  . . x x
	     *               . . . .
	     *               . . . .
	     */
	    y = pY[1] + Rgb232D02;
	    rgbTop |= greenTbl[ y ];
	    y += Rgb232D02;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];
	    rgbTop <<= 8;

	    y += Rgb232D03 - (2*Rgb232D02);
	    rgbTop |= greenTbl[ y ];
	    y += Rgb232D03;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];

	    *ULongP(pRgb) = rgbTop;
	    pRgb += rgbWidth;

	    y += Rgb232D12 - (2*Rgb232D03);
	    rgbBottom |= greenTbl[ y ];
	    y += Rgb232D12;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];
	    rgbBottom <<= 8;

	    y += Rgb232D13 - (2*Rgb232D12);
	    rgbBottom |= greenTbl[ y ];
	    y += Rgb232D13;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];

	    *ULongP(pRgb) = rgbBottom;
	    pRgb += rgbWidth;

	    /*
	     * do:  . .      . . . .
	     *      x . ==>  . . . .
	     *               x x . .
	     *               x x . .
	     */
	    y = pY[lumaWidth] + Rgb232D20;
	    pY++;
	    rgbTop = greenTbl[ y ];
	    y += Rgb232D20;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];
	    rgbTop <<= 8;

	    y += Rgb232D21 - (2*Rgb232D20);
	    rgbTop |= greenTbl[ y ];
	    y += Rgb232D21;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];
	    rgbTop <<= 8;

	    y += Rgb232D30 - (2*Rgb232D31);;
	    rgbBottom = greenTbl[ y ];
	    y += Rgb232D30;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];
	    rgbBottom <<= 8;

	    y += Rgb232D31 - (2*Rgb232D30);
	    rgbBottom |= greenTbl[ y ];
	    y += Rgb232D31;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];
	    rgbBottom <<= 8;

	    /*
	     * do:  . .      . . . .
	     *      . x ==>  . . . .
	     *               . . x x
	     *               . . x x
	     */
	    y = pY[lumaWidth] + Rgb232D22;
	    pY++;
	    rgbTop |= greenTbl[ y ];
	    y += Rgb232D22;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];
	    rgbTop <<= 8;

	    y += Rgb232D23 - (2*Rgb232D22);
	    rgbTop |= greenTbl[ y ];
	    y += Rgb232D23;
	    rgbTop |=   redTbl[ y ];
	    rgbTop |=  blueTbl[ y ];

	    *ULongP(pRgb) = rgbTop;

	    y += Rgb232D32 - (2*Rgb232D33);
	    rgbBottom |= greenTbl[ y ];
	    y += Rgb232D32;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];
	    rgbBottom <<= 8;

	    y += Rgb232D33 - (2*Rgb232D32);
	    rgbBottom |= greenTbl[ y ];
	    y += Rgb232D33;
	    rgbBottom |=   redTbl[ y ];
	    rgbBottom |=  blueTbl[ y ];

	    *ULongP(pRgb + rgbWidth) = rgbBottom;
	    pRgb -= (rgbWidth<<1) - 4;

            if (--w <= 0)
                break;
        }

        if (--height <= 0)
            break;

        pRgb += ycToRgb->rgbAccel;
        pY   += ycToRgb->lumaAccel;
        pCr  += ycToRgb->chromaAccel;
        pCb  += ycToRgb->chromaAccel;
    }
}

static void
Lookup332RGBFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w;
    UChar *pY, *pCr, *pCb;
    int redIndex, greenIndex, blueIndex;
    UChar *redTbl, *greenTbl, *blueTbl;
    UChar *pRgb;
    ULong y, cr, cb;
    ULong rgbTop, rgbBottom;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    height >>= 2;
    width >>= 2;
    for (;;) {
        /*
         * Each time through the loop does 4x4 pixels
         *     . . . .
         *     . . . .
         *     . . . .
         *     . . . .
         */
        w = width;
        for (;;) {
            /*
             * do:  x x . .
             *      x x . .
             *      . . . .
             *      . . . .
             */
            cr = pCr[0];
            redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
            redTbl = &ycToRgb->rgbLookupData.u.rgb332.red[ redIndex ];

            greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

            cb = pCb[0];
            greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
            greenTbl = &ycToRgb->rgbLookupData.u.rgb332.green[ greenIndex ];

            blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
            blueTbl = &ycToRgb->rgbLookupData.u.rgb332.blue[ blueIndex ];

            y = pY[0] + Rgb332D00;
            rgbTop = greenTbl[ y ];
            y += Rgb332D00;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
            rgbTop <<= 8;

            y = pY[1] + Rgb332D01;
            rgbTop |= greenTbl[ y ];
            y += Rgb332D01;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
            rgbTop <<= 8;

            y = pY[lumaWidth] + Rgb332D10;
            pY++;
            rgbBottom = greenTbl[ y ];
            y += Rgb332D10;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
            rgbBottom <<= 8;

            y = pY[lumaWidth] + Rgb332D11;
            pY++;
            rgbBottom |= greenTbl[ y ];
            y += Rgb332D11;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
            rgbBottom <<= 8;

            /*
             * do:  . . x x
             *      . . x x
             *      . . . .
             *      . . . .
             */
            cr = pCr[1];
	    pCr += ycToRgb->chromaWidth;
            redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
            redTbl = &ycToRgb->rgbLookupData.u.rgb332.red[ redIndex ];

            greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

            cb = pCb[1];
	    pCb += ycToRgb->chromaWidth;
            greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
            greenTbl = &ycToRgb->rgbLookupData.u.rgb332.green[ greenIndex ];

            blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
            blueTbl = &ycToRgb->rgbLookupData.u.rgb332.blue[ blueIndex ];

            y = pY[0] + Rgb332D02;
            rgbTop |= greenTbl[ y ];
            y += Rgb332D02;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
            rgbTop <<= 8;

            y = pY[1] + Rgb332D03;
	    pY += lumaWidth;
            rgbTop |= greenTbl[ y ];
            y += Rgb332D03;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
	    *ULongP(pRgb) = rgbTop;
	    pRgb += rgbWidth;

            y = pY[0] + Rgb332D12;
	    pY++;
            rgbBottom |= greenTbl[ y ];
            y += Rgb332D12;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
            rgbBottom <<= 8;

            y = pY[0] + Rgb332D13;
            pY += lumaWidth - 3;
            rgbBottom |= greenTbl[ y ];
            y += Rgb332D13;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
	    *ULongP(pRgb) = rgbBottom;
	    pRgb += rgbWidth;

            /*
             * do:  . . . .
             *      . . . .
             *      x x . .
             *      x x . .
             */
            cr = pCr[0];
            redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
            redTbl = &ycToRgb->rgbLookupData.u.rgb332.red[ redIndex ];

            greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

            cb = pCb[0];
            greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
            greenTbl = &ycToRgb->rgbLookupData.u.rgb332.green[ greenIndex ];

            blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
            blueTbl = &ycToRgb->rgbLookupData.u.rgb332.blue[ blueIndex ];

            y = pY[0] + Rgb332D20;
            rgbTop = greenTbl[ y ];
            y += Rgb332D20;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
            rgbTop <<= 8;

            y = pY[1] + Rgb332D21;
            rgbTop |= greenTbl[ y ];
            y += Rgb332D21;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
            rgbTop <<= 8;

            y = pY[lumaWidth] + Rgb332D30;
            pY++;
            rgbBottom = greenTbl[ y ];
            y += Rgb332D30;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
            rgbBottom <<= 8;

            y = pY[lumaWidth] + Rgb332D31;
            pY++;
            rgbBottom |= greenTbl[ y ];
            y += Rgb332D31;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
            rgbBottom <<= 8;

            /*
             * do:  . . . .
             *      . . . .
             *      . . x x
             *      . . x x
             */
            cr = pCr[1];
	    pCr -= ycToRgb->chromaWidth - 2;
            redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
            redTbl = &ycToRgb->rgbLookupData.u.rgb332.red[ redIndex ];

            greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

            cb = pCb[1];
	    pCb -= ycToRgb->chromaWidth - 2;
            greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
            greenTbl = &ycToRgb->rgbLookupData.u.rgb332.green[ greenIndex ];

            blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
            blueTbl = &ycToRgb->rgbLookupData.u.rgb332.blue[ blueIndex ];

            y = pY[0] + Rgb332D22;
            rgbTop |= greenTbl[ y ];
            y += Rgb332D22;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
            rgbTop <<= 8;

            y = pY[1] + Rgb332D23;
            rgbTop |= greenTbl[ y ];
            y += Rgb332D23;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
	    *ULongP(pRgb) = rgbTop;

            y = pY[lumaWidth] + Rgb332D32;
	    pY++;
            rgbBottom |= greenTbl[ y ];
            y += Rgb332D32;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
            rgbBottom <<= 8;

            y = pY[lumaWidth] + Rgb332D33;
            pY -= (lumaWidth<<1) - 1;
            rgbBottom |= greenTbl[ y ];
            y += Rgb332D33;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
	    *ULongP(pRgb + rgbWidth) = rgbBottom;
	    pRgb -= (rgbWidth<<1) - 4;

            if (--w <= 0)
                break;
        }

        if (--height <= 0)
            break;

        pRgb += ycToRgb->rgbAccel;
        pY   += ycToRgb->lumaAccel;
        pCr  += ycToRgb->chromaAccel;
        pCb  += ycToRgb->chromaAccel;
    }
}

static void
LookupC555RGBFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w;
    UChar *pY, *pCr, *pCb;
    UChar *table = ycToRgb->rgbLookupData.u.rgbEqual;
    UChar *targetRgb;
    long tmp;
    UChar *pRgb;
    ULong rgbTop, rgbBottom, crCbIndex;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    width >>= 2;
    for (;;) {
	/*
	 * Each time through the loop does 4x4 pixels
	 * . . . .
	 * . . . .
	 * . . . .
	 * . . . .
	 * The initial value of w, below, must be even
	 */
	w = width;
	for (;;) {
	    /*
	     * do:  x x . .
	     *      x x . .
	     *      . . . .
	     *      . . . .
	     */
            crCbIndex = pCr[0] >> (8-RgbC555CrBits);
	    crCbIndex <<= RgbC555YBits;
            tmp = pCb[0] >> (8-RgbC555CbBits);
            crCbIndex |= tmp << (RgbC555CrBits+RgbC555YBits);
	    targetRgb = table+crCbIndex;

            tmp = (pY[0]+RgbC555D00) >> (8-RgbC555YBits+1);
            rgbTop = *(targetRgb+tmp);
            rgbTop <<= 8;

            tmp = (pY[1]+RgbC555D01) >> (8-RgbC555YBits+1);
            rgbTop |= *(targetRgb+tmp);
            rgbTop <<= 8;

            tmp = (pY[lumaWidth]+RgbC555D10) >> (8-RgbC555YBits+1);
	    pY++;
            rgbBottom = *(targetRgb+tmp);
            rgbBottom <<= 8;

            tmp = (pY[lumaWidth]+RgbC555D11) >> (8-RgbC555YBits+1);
	    pY++;
            rgbBottom |= *(targetRgb+tmp);
            rgbBottom <<= 8;

	    /*
	     * do:  . . x x
	     *      . . x x
	     *      . . . .
	     *      . . . .
	     */
            crCbIndex = pCr[1] >> (8-RgbC555CrBits);
	    crCbIndex <<= RgbC555YBits;
            tmp = pCb[1] >> (8-RgbC555CbBits);
            crCbIndex |= tmp << (RgbC555CrBits+RgbC555YBits);
	    targetRgb = table+crCbIndex;

            tmp = (pY[0]+RgbC555D02) >> (8-RgbC555YBits+1);
            rgbTop |= *(targetRgb+tmp);
            rgbTop <<= 8;

            tmp = (pY[1]+RgbC555D03) >> (8-RgbC555YBits+1);
            rgbTop |= *(targetRgb+tmp);

	    *ULongP(pRgb) = rgbTop;
	    pRgb += rgbWidth;

            tmp = (pY[lumaWidth]+RgbC555D12) >> (8-RgbC555YBits+1);
	    pY++;
            rgbBottom |= *(targetRgb+tmp);
            rgbBottom <<= 8;

            tmp = (pY[lumaWidth]+RgbC555D13) >> (8-RgbC555YBits+1);
	    pY += (lumaWidth<<1) - 3;
            rgbBottom |= *(targetRgb+tmp);

	    *ULongP(pRgb) = rgbBottom;
	    pRgb += rgbWidth;

	    /*
	     * do:  . . . .
	     *      . . . .
	     *      x x . .
	     *      x x . .
	     */
	    pCr += ycToRgb->chromaWidth;
	    pCb += ycToRgb->chromaWidth;

            crCbIndex = *pCr++ >> (8-RgbC555CrBits);
	    crCbIndex <<= RgbC555YBits;
            tmp = *pCb++ >> (8-RgbC555CbBits);
            crCbIndex |= tmp << (RgbC555CrBits+RgbC555YBits);
	    targetRgb = table+crCbIndex;

            tmp = (pY[0]+RgbC555D20) >> (8-RgbC555YBits+1);
            rgbTop = *(targetRgb+tmp);
            rgbTop <<= 8;

            tmp = (pY[1]+RgbC555D21) >> (8-RgbC555YBits+1);
            rgbTop |= *(targetRgb+tmp);
            rgbTop <<= 8;

            tmp = (pY[lumaWidth]+RgbC555D30) >> (8-RgbC555YBits+1);
	    pY++;
            rgbBottom = *(targetRgb+tmp);
            rgbBottom <<= 8;

            tmp = (pY[lumaWidth]+RgbC555D31) >> (8-RgbC555YBits+1);
	    pY++;
            rgbBottom |= *(targetRgb+tmp);
            rgbBottom <<= 8;

	    /*
	     * do:  . . . .
	     *      . . . .
	     *      . . x x
	     *      . . x x
	     */
            crCbIndex = pCr[0] >> (8-RgbC555CrBits);
	    pCr -= ycToRgb->chromaWidth - 1;
	    crCbIndex <<= RgbC555YBits;
            tmp = pCb[0] >> (8-RgbC555CbBits);
	    pCb -= ycToRgb->chromaWidth - 1;
            crCbIndex |= tmp << (RgbC555CrBits+RgbC555YBits);
	    targetRgb = table+crCbIndex;

            tmp = (pY[0]+RgbC555D22) >> (8-RgbC555YBits+1);
            rgbTop |= *(targetRgb+tmp);
            rgbTop <<= 8;

            tmp = (pY[1]+RgbC555D23) >> (8-RgbC555YBits+1);
            rgbTop |= *(targetRgb+tmp);

	    *ULongP(pRgb) = rgbTop;

            tmp = (pY[lumaWidth]+RgbC555D32) >> (8-RgbC555YBits+1);
	    pY++;
            rgbBottom |= *(targetRgb+tmp);
            rgbBottom <<= 8;

            tmp = (pY[lumaWidth]+RgbC555D33) >> (8-RgbC555YBits+1);
	    pY -= (lumaWidth<<1) - 1;
            rgbBottom |= *(targetRgb+tmp);

	    *ULongP(pRgb+rgbWidth) = rgbBottom;
	    pRgb -= (rgbWidth<<1) - 4;

	    if (--w <= 0)
		break;
	}

	if ((height -= 4) <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}

/*
 * Optimized to load variables ahead of the time they'll be needed.
 *	This should provide enough information for the compiler to
 *	minimized pipeline stalls
 */
static void
LookupC666RGBFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w;
    UChar *pY, *pCr, *pCb;
    UChar *table = ycToRgb->rgbLookupData.u.rgbEqual;
    UChar *targetRgb;
    long t0, t1, t2;
    UChar *pRgb;
    ULong rgbTop, rgbBottom;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;
 
    width >>= 2;
    for (;;) {
	/*
	 * Each time through the loop does 4x4 pixels
	 * . . . .
	 * . . . .
	 * . . . .
	 * . . . .
	 * The initial value of w, below, must be even
	 */
	w = width;
	for(;;) {
	    /*
	     * do:  x x . .
	     *      x x . .
	     *      . . . .
	     *      . . . .
	     */

	    /* preload */
	    t0 = pCr[0];
	    t1 = pCb[0];
	    t2 = pY[0];

	    t0 >>= (8-RgbC666CrBits);
	    t0 <<= RgbC666YBits;
	    t1 >>= (8-RgbC666CbBits);
	    t0 |= t1 << (RgbC666CrBits+RgbC666YBits);
	    targetRgb = table+t0;

	    /* preload */
	    t0 = pY[1];

	    t2 = (t2+RgbC666D00) >> (8-RgbC666YBits+1);
            rgbTop = *(targetRgb+t2);
            rgbTop <<= 8;

	    /* preload */
	    t1 = pY[lumaWidth];

	    t0 = (t0+RgbC666D01) >> (8-RgbC666YBits+1);
            rgbTop |= *(targetRgb+t0);

	    /* preload */
	    t0 = pY[lumaWidth+1];
	    pY += 2;

	    t1 = (t1+RgbC666D10) >> (8-RgbC666YBits+1);
            rgbBottom = *(targetRgb+t1);
            rgbBottom <<= 8;

	    t0 = (t0+RgbC666D11) >> (8-RgbC666YBits+1);
            rgbBottom |= *(targetRgb+t0);

	    /*
	     * do:  . . x x
	     *      . . x x
	     *      . . . .
	     *      . . . .
	     */

	    /* preload */
	    t0 = pCr[1];
	    t1 = pCb[1];
	    t2 = pY[0];

	    t0 >>= (8-RgbC666CrBits);
	    t0 <<= RgbC666YBits;
	    t1 >>= (8-RgbC666CbBits);
	    t0 |= t1 << (RgbC666CrBits+RgbC666YBits);
	    targetRgb = table+t0;

	    /* deferred store */
	    ((UShort *)pRgb)[0] = rgbTop;
	    /* preload */
	    t0 = pY[1];

	    t2 = (t2+RgbC666D02) >> (8-RgbC666YBits+1);
	    rgbTop = *(targetRgb+t2);
            rgbTop <<= 8;

	    /* preload */
	    t1 = pY[lumaWidth];

	    t0 = (t0+RgbC666D03) >> (8-RgbC666YBits+1);
            rgbTop |= *(targetRgb+t0);

	    /* deferred store */
	    ((UShort *)pRgb)[1] = rgbTop;

	    pRgb += rgbWidth;

	    /* preload */
	    t0 = pY[lumaWidth+1];
	    pY += (lumaWidth<<1) - 2;
	    /* deferred store */
	    ((UShort *)pRgb)[0] = rgbBottom;
	    
	    t1 = (t1+RgbC666D12) >> (8-RgbC666YBits+1);
	    rgbBottom = *(targetRgb+t1);
            rgbBottom <<= 8;

	    t0 = (t0+RgbC666D13) >> (8-RgbC666YBits+1);
            rgbBottom |= *(targetRgb+t0);

	    /* preload */
	    pCr += ycToRgb->chromaWidth;
	    pCb += ycToRgb->chromaWidth;
	    t0 = *pCr++;
	    t1 = *pCb++;
	    t2 = pY[0];
	    /* deferred store */
	    ((UShort *)pRgb)[1] = rgbBottom;
	    pRgb += rgbWidth;

	    /*
	     * do:  . . . .
	     *      . . . .
	     *      x x . .
	     *      x x . .
	     */
	    t0 >>= (8-RgbC666CrBits);
	    t0 <<= RgbC666YBits;
	    t1 >>= (8-RgbC666CbBits);
	    t0 |= t1 << (RgbC666CrBits+RgbC666YBits);
	    targetRgb = table+t0;

	    /* preload */
	    t0 = pY[1];

	    t2 = (t2+RgbC666D20) >> (8-RgbC666YBits+1);
	    rgbTop = *(targetRgb+t2);
	    rgbTop <<= 8;

	    /* preload */
	    t1 = pY[lumaWidth];

	    t0 = (t0+RgbC666D21) >> (8-RgbC666YBits+1);
	    rgbTop |= *(targetRgb+t0);

	    /* preload */
	    t0 = pY[lumaWidth+1];
	    pY += 2;

	    t1 = (t1+RgbC666D30) >> (8-RgbC666YBits+1);
	    rgbBottom = *(targetRgb+t1);
	    rgbBottom <<= 8;

	    t0 = (t0+RgbC666D31) >> (8-RgbC666YBits+1);
	    rgbBottom |= *(targetRgb+t0);

	    /*
	     * do:  . . . .
	     *      . . . .
	     *      . . x x
	     *      . . x x
	     */

	    /* preload */
	    t0 = pCr[0];
	    pCr -= ycToRgb->chromaWidth - 1;
	    t1 = pCb[0];
	    pCb -= ycToRgb->chromaWidth - 1;
	    t2 = pY[0];

	    t0 >>= (8-RgbC666CrBits);
	    t0 <<= RgbC666YBits;
	    t1 >>= (8-RgbC666CbBits);
	    t0 |= t1 << (RgbC666CrBits+RgbC666YBits);
	    targetRgb = table+t0;

	    /* deferred store */
	    ((UShort *)pRgb)[0] = rgbTop;
	    /* preload */
	    t0 = pY[1];

	    t2 = (t2+RgbC666D22) >> (8-RgbC666YBits+1);
            rgbTop = *(targetRgb+t2);
            rgbTop <<= 8;

	    /* preload */
	    t1 = pY[lumaWidth];

	    t0 = (t0+RgbC666D23) >> (8-RgbC666YBits+1);
            rgbTop |= *(targetRgb+t0);

	    /* deferred store */
	    ((UShort *)pRgb)[1] = rgbTop;
	    ((UShort *)(pRgb+rgbWidth))[0] = rgbBottom;
	    /* preload */
	    t0 = pY[lumaWidth+1];
	    pY -= (lumaWidth<<1) - 2;

	    t1 = (t1+RgbC666D32) >> (8-RgbC666YBits+1);
            rgbBottom = *(targetRgb+t1);
            rgbBottom <<= 8;

	    t0 = (t0+RgbC666D33) >> (8-RgbC666YBits+1);
            rgbBottom |= *(targetRgb+t0);

	    /* deferred store */
	    ((UShort *)(pRgb+rgbWidth))[1] = rgbBottom;
	    pRgb -= (rgbWidth<<1) - 4;

	    if (--w <= 0)
	        break;
	}

	if ((height -= 4) <= 0)
	    break;

	pRgb += ycToRgb->rgbAccel;
	pY   += ycToRgb->lumaAccel;
	pCr  += ycToRgb->chromaAccel;
	pCb  += ycToRgb->chromaAccel;
    }
}


/*ARGSUSED*/
static void
Lookup232RGBFrame(YcToRgbInfoPtr ycToRgb)
{
    long width = ycToRgb->box.width;
    long height = ycToRgb->box.height;
    long lumaWidth, rgbWidth;
    long w;
    UChar *pY, *pCr, *pCb;
    int redIndex, greenIndex, blueIndex;
    UChar *redTbl, *greenTbl, *blueTbl;
    UChar *pRgb;
    ULong y, cr, cb;
    ULong rgbTop, rgbBottom;

    pY = ycToRgb->y + ycToRgb->lumaStart;
    pCr = ycToRgb->cr + ycToRgb->chromaStart;
    pCb = ycToRgb->cb + ycToRgb->chromaStart;
    pRgb = ycToRgb->rgb + ycToRgb->rgbStart;
    lumaWidth = ycToRgb->lumaWidth;
    rgbWidth = ycToRgb->rgbWidth;

    height >>= 2;
    width >>= 2;
    for (;;) {
        /*
         * Each time through the loop does 4x4 pixels
         *     . . . .
         *     . . . .
         *     . . . .
         *     . . . .
         */
        w = width;
        for (;;) {
            /*
             * do:  x x . .
             *      x x . .
             *      . . . .
             *      . . . .
             */
            cr = pCr[0];
            redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
            redTbl = &ycToRgb->rgbLookupData.u.rgb232.red[ redIndex ];

            greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

            cb = pCb[0];
            greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
            greenTbl = &ycToRgb->rgbLookupData.u.rgb232.green[ greenIndex ];

            blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
            blueTbl = &ycToRgb->rgbLookupData.u.rgb232.blue[ blueIndex ];

            y = pY[0] + Rgb232D00;
            rgbTop = greenTbl[ y ];
            y += Rgb232D00;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
            rgbTop <<= 8;

            y = pY[1] + Rgb232D01;
            rgbTop |= greenTbl[ y ];
            y += Rgb232D01;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
            rgbTop <<= 8;

            y = pY[lumaWidth] + Rgb232D10;
            pY++;
            rgbBottom = greenTbl[ y ];
            y += Rgb232D10;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
            rgbBottom <<= 8;

            y = pY[lumaWidth] + Rgb232D11;
            pY++;
            rgbBottom |= greenTbl[ y ];
            y += Rgb232D11;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
            rgbBottom <<= 8;

            /*
             * do:  . . x x
             *      . . x x
             *      . . . .
             *      . . . .
             */
            cr = pCr[1];
	    pCr += ycToRgb->chromaWidth;
            redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
            redTbl = &ycToRgb->rgbLookupData.u.rgb232.red[ redIndex ];

            greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

            cb = pCb[1];
	    pCb += ycToRgb->chromaWidth;
            greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
            greenTbl = &ycToRgb->rgbLookupData.u.rgb232.green[ greenIndex ];

            blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
            blueTbl = &ycToRgb->rgbLookupData.u.rgb232.blue[ blueIndex ];

            y = pY[0] + Rgb232D02;
            rgbTop |= greenTbl[ y ];
            y += Rgb232D02;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
            rgbTop <<= 8;

            y = pY[1] + Rgb232D03;
	    pY += lumaWidth;
            rgbTop |= greenTbl[ y ];
            y += Rgb232D03;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
	    *ULongP(pRgb) = rgbTop;
	    pRgb += rgbWidth;

            y = pY[0] + Rgb232D12;
	    pY++;
            rgbBottom |= greenTbl[ y ];
            y += Rgb232D12;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
            rgbBottom <<= 8;

            y = pY[0] + Rgb232D13;
            pY += lumaWidth - 3;
            rgbBottom |= greenTbl[ y ];
            y += Rgb232D13;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
	    *ULongP(pRgb) = rgbBottom;
	    pRgb += rgbWidth;

            /*
             * do:  . . . .
             *      . . . .
             *      x x . .
             *      x x . .
             */
            cr = pCr[0];
            redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
            redTbl = &ycToRgb->rgbLookupData.u.rgb232.red[ redIndex ];

            greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

            cb = pCb[0];
            greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
            greenTbl = &ycToRgb->rgbLookupData.u.rgb232.green[ greenIndex ];

            blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
            blueTbl = &ycToRgb->rgbLookupData.u.rgb232.blue[ blueIndex ];

            y = pY[0] + Rgb232D20;
            rgbTop = greenTbl[ y ];
            y += Rgb232D20;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
            rgbTop <<= 8;

            y = pY[1] + Rgb232D21;
            rgbTop |= greenTbl[ y ];
            y += Rgb232D21;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
            rgbTop <<= 8;

            y = pY[lumaWidth] + Rgb232D30;
            pY++;
            rgbBottom = greenTbl[ y ];
            y += Rgb232D30;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
            rgbBottom <<= 8;

            y = pY[lumaWidth] + Rgb232D31;
            pY++;
            rgbBottom |= greenTbl[ y ];
            y += Rgb232D31;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
            rgbBottom <<= 8;

            /*
             * do:  . . . .
             *      . . . .
             *      . . x x
             *      . . x x
             */
            cr = pCr[1];
	    pCr -= ycToRgb->chromaWidth - 2;
            redIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toRed;
            redTbl = &ycToRgb->rgbLookupData.u.rgb232.red[ redIndex ];

            greenIndex = ycToRgb->rgbLookupData.index.cru.index[ cr ].toGreen;

            cb = pCb[1];
	    pCb -= ycToRgb->chromaWidth - 2;
            greenIndex += ycToRgb->rgbLookupData.index.cbu.index[ cb ].toGreen;
            greenTbl = &ycToRgb->rgbLookupData.u.rgb232.green[ greenIndex ];

            blueIndex  = ycToRgb->rgbLookupData.index.cbu.index[ cb ].toBlue;
            blueTbl = &ycToRgb->rgbLookupData.u.rgb232.blue[ blueIndex ];

            y = pY[0] + Rgb232D22;
            rgbTop |= greenTbl[ y ];
            y += Rgb232D22;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
            rgbTop <<= 8;

            y = pY[1] + Rgb232D23;
            rgbTop |= greenTbl[ y ];
            y += Rgb232D23;
            rgbTop |=   redTbl[ y ];
            rgbTop |=  blueTbl[ y ];
	    *ULongP(pRgb) = rgbTop;

            y = pY[lumaWidth] + Rgb232D32;
	    pY++;
            rgbBottom |= greenTbl[ y ];
            y += Rgb232D32;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
            rgbBottom <<= 8;

            y = pY[lumaWidth] + Rgb232D33;
            pY -= (lumaWidth<<1) - 1;
            rgbBottom |= greenTbl[ y ];
            y += Rgb232D33;
            rgbBottom |=   redTbl[ y ];
            rgbBottom |=  blueTbl[ y ];
	    *ULongP(pRgb + rgbWidth) = rgbBottom;
	    pRgb -= (rgbWidth<<1) - 4;

            if (--w <= 0)
                break;
        }

        if (--height <= 0)
            break;

        pRgb += ycToRgb->rgbAccel;
        pY   += ycToRgb->lumaAccel;
        pCr  += ycToRgb->chromaAccel;
        pCb  += ycToRgb->chromaAccel;
    }
}

/*
 * This is the central part of the algorithm.  Here, the tables are
 * initialized with the values that make it all work.  Tweak with
 * care!
 */
static InternalReasonPtr
InitSingleStepLookupData(YcToRgbInfoPtr *pYcToRgb)
{
    YcToRgbInfoPtr ycToRgb = *pYcToRgb;
    RgbLookupInfoPtr rgbLookupData = &ycToRgb->rgbLookupData;
    int maxDither, tableSize;
    int structLength = sizeof(YcToRgbInfo);
    int yBits, crBits, cbBits;
    int yIndex, crIndex, cbIndex, bias;
    int redScale, greenScale, blueScale;
    float redBase, greenBase, blueBase;
    int index, indexShift, nColors;
    UChar oneBits = 0;
    int r, g, b, cr, cb, y, yStep, yMax;
    float colorStepRecip, yReal, yRealStep;
    UChar *byteTable;
    
    switch (ycToRgb->depthType) {
    case Rgb888:
        maxDither = MaxRgb888Dither;
        yBits = Rgb888YBits;
        crBits = Rgb888CrBits;
        cbBits = Rgb888CbBits;
        indexShift = 2;
        nColors = 256;
        break;
    case Rgb444:
        maxDither = MaxRgb444Dither;
        yBits = Rgb444YBits;
        crBits = Rgb444CrBits;
        cbBits = Rgb444CbBits;
        indexShift = 1;
        nColors = 16;
        break;
    case Rgb555:
        maxDither = MaxRgb555Dither;
        yBits = Rgb555YBits;
        crBits = Rgb555CrBits;
        cbBits = Rgb555CbBits;
        indexShift = 1;
        nColors = 32;
        break;
    case RgbC666:
        maxDither = MaxRgbC666Dither;
        yBits = RgbC666YBits;
        crBits = RgbC666CrBits;
        cbBits = RgbC666CbBits;
        indexShift = 0;
        nColors = 6;
        break;
    case RgbC555:
	oneBits = 0x80;
        maxDither = MaxRgbC555Dither;
        yBits = RgbC555YBits;
        crBits = RgbC555CrBits;
        cbBits = RgbC555CbBits;
        indexShift = 0;
        nColors = 5;
        break;
    default:
        return( InternalProblem("InitSingleStepLookupData: depthType=%d\n",
        		        ycToRgb->depthType));
    }

    /*
     * Discover the table size.  See the comments below, where the
     * table is created
     */
    tableSize = 0;	/* gratuitous for lint */
    cr = 255 >> (8-crBits);
    cb = 255 >> (8-cbBits);
    y = (255+maxDither) >> (8-yBits);

    switch (indexShift) {
    case 0:
	/* put Y on the bottom */
	y >>= 1;
	tableSize = y
		  + (cb << (yBits+crBits))
		  + (cr << yBits)
		  + 1 /* because above calc is maximum index value */;
	break;
    case 1:
    case 2:
	/* put Y on top */
	tableSize = (y << (crBits+cbBits+indexShift))
		  + (cb << (crBits+indexShift))
		  + (cr << (indexShift))
		  + 1 /* because above calc is maximum index value */;
	break;
    }

    structLength += tableSize - sizeof(rgbLookupData->u);
    if (structLength > ycToRgb->length) {
        *pYcToRgb = (YcToRgbInfoPtr)
            InsecureNullRealloc((char *)ycToRgb, structLength);
        if (*pYcToRgb == NULL)
            return(AllocReason("color tables", structLength));
        ycToRgb = *pYcToRgb;
        rgbLookupData = &ycToRgb->rgbLookupData;
    }

    /*
     * These lookup tables are single-stage and are different
     * than those below.
     */
    colorStepRecip = 1.0/ycToRgb->colorStepSize;
    byteTable = rgbLookupData->u.rgbEqual;

    switch(ycToRgb->components) {
    default:	/* never happens */
    case PixelRGB0:
    case Pixel0RGB:
    case PixelRGB:
	redScale = nColors * nColors;
	greenScale = nColors;
	blueScale = 1;
	break;

    case PixelBGR0:
    case Pixel0BGR:
    case PixelBGR:
	redScale = 1;
	greenScale = nColors;
	blueScale = nColors * nColors;
	break;
    }

    bias = maxDither >> 1;
    yStep = 1<<(8-yBits);
    yMax = 255 + maxDither;
    yRealStep = YToRGBConst * yStep * colorStepRecip;
    index = 0;
    for(cr = 0; cr < 256; cr += (1<<(8-crBits))) {
      for(cb = 0; cb < 256; cb += (1<<(8-cbBits))) {

        redBase = YCrCbToRed(-bias, cr, cb) * colorStepRecip + .5;
        greenBase = YCrCbToGreen(-bias, cr, cb) * colorStepRecip + .5;
        blueBase = YCrCbToBlue(-bias, cr, cb) * colorStepRecip + .5;

        for(y = 0, yReal = 0.0; y <= yMax; yReal += yRealStep, y += yStep) {
          r = redBase + yReal;
          if (r < 0) r = 0;
          else if (r >= nColors) r = (nColors-1) * redScale;
          else r *= redScale;
          
          g = greenBase + yReal;
          if (g < 0) g = 0;
          else if (g >= nColors) g = (nColors-1) * greenScale;
          else g *= greenScale;

          b = blueBase + yReal;
          if (b < 0) b = 0;
          else if (b >= nColors) b = (nColors-1) * blueScale;
	  else b *= blueScale;

	  /*
	   * Crucial!!!  There are two types of single-step lookups:
           * single-byte and multi-byte.
	   *
	   * The single-byte lookups
	   * benefit from having the y in the lowest bits of the index
	   * because it takes only a right shift to set them up.  The
	   * cost is that you must spend one of those bits purely for
	   * overflow when dither is added in.
	   *
	   * The multi-byte lookups do not benefit from having y in
	   * the lowest bits because there must still be a mask to
	   * make sure the lowest bits are 0 to prevent address alignment
	   * errors.  Instead, they can benefit from having the y
	   * in the top-most position because the table requires
	   * less memory.  This is because the overflow of y in the
	   * top position does not require a full twice the memory.
	   *
	   * We do different tables for the two types
	   */
	  crIndex = cr >> (8-crBits);
	  cbIndex = cb >> (8-cbBits);

          switch (indexShift) {
	  case 0:
            /* put Y on the bottom: cb/cr/y */
	    yIndex  = y >> (8-yBits+1);
	    index = yIndex
		  + (cbIndex << (yBits+crBits))
		  + (crIndex << yBits);
	    *(byteTable + index) = r + g + b + oneBits;
	    break;
	  case 1:
	  case 2:
            /* put Y on the top: y/cb/cr */
	    yIndex  = y >> (8-yBits);
	    index = (yIndex << (crBits+cbBits+indexShift))
		  + (cbIndex << (crBits+indexShift))
		  + (crIndex << (indexShift));
            switch (indexShift) {
	    case 1: *UShortP(byteTable + index) = r + g + b; break;
	    case 2: *ULongP(byteTable + index) = r + g + b; break;
	    }
	    break;
	  }
          if (index >= tableSize) {
              return( InternalProblem("init RgbC: index(%d)>=table size(%d)\n",
                                      index, tableSize) );
          }
        }
      }
    }
    if (index != tableSize-1) {
        return( InternalProblem("init RgbC: index(%d)!=table size-1(%d)\n",
                                index, tableSize-1));
    }
    return(NULL);
}

/*
 * This is the central part of the algorithm.  Here, the tables are
 * initialized with the values that make it all work.  Tweak with
 * care!
 */
static InternalReasonPtr
InitMultiStepLookupData(YcToRgbInfoPtr *pYcToRgb)
{
    YcToRgbInfoPtr ycToRgb = *pYcToRgb;
    RgbLookupInfoPtr rgbLookupData = &ycToRgb->rgbLookupData;
    int value, index;
    float redIndex, greenCrIndex, greenCbIndex, blueIndex;
    float lookup;
    UChar pixel;
    int nRedLookup, nGreenLookup, nBlueLookup;
    int fracBits;

    switch (ycToRgb->depthType) {
    case Rgb888:
        nRedLookup   = N888RedLookup;
        nGreenLookup = N888GreenLookup;
        nBlueLookup  = N888BlueLookup;
        fracBits     = RGB888FracBits;
        break;
    case Rgb232:
        nRedLookup   = N232RedLookup;
        nGreenLookup = N232GreenLookup;
        nBlueLookup  = N232BlueLookup;
        fracBits     = RGB232FracBits;
        break;
    case Rgb332:
        nRedLookup   = N332RedLookup;
        nGreenLookup = N332GreenLookup;
        nBlueLookup  = N332BlueLookup;
        fracBits     = RGB332FracBits;
        break;
    default:
        return( InternalProblem("InitMultiStepLookupData: depthType=%d\n",
                                ycToRgb->depthType) );
    }

    /*
     * Initialize the stage one lookup table, common to the above rgb depths.
     */
    for (value = 255; value >= 0; value--) {
        redIndex     = RedIndex(value);
        blueIndex    = BlueIndex(value);
        greenCrIndex = GreenCrIndex(value);
        greenCbIndex = GreenCbIndex(value);

        rgbLookupData->index.cru.index[ value ].toRed =
            (UShort) (redIndex * (1<<fracBits) + 0.5);
        rgbLookupData->index.cru.index[ value ].toGreen =
            (UShort) (greenCrIndex * (1<<fracBits) + 0.5);
        rgbLookupData->index.cbu.index[ value ].toBlue =
            (UShort) (blueIndex * (1<<fracBits) + 0.5);
        rgbLookupData->index.cbu.index[ value ].toGreen =
            (UShort) (greenCbIndex * (1<<fracBits) + 0.5);
    }

    for (index = nRedLookup - 1; index >= 0; index--) {
        lookup = ((float)index - ycToRgb->redDitherBias) / (1<<fracBits);
        lookup = (lookup - RedBias) * YToRGBConst + .5;
        if (lookup < 0.0)
            lookup = 0.0;
        else if (lookup > 255.0)
            lookup = 255.0;
        pixel = (UChar)lookup;
        switch (ycToRgb->depthType) {
        case Rgb888:
            if (ycToRgb->invertFinalRgb)
                pixel = 255 - pixel;
            rgbLookupData->u.rgb888.red[ index ] = pixel;
            break;
        case Rgb332:
            rgbLookupData->u.rgb332.red[ index ] =
                (pixel>>R332BitOffset) & R332Mask;
            break;
        case Rgb232:
            /*
             * The high bit is always set... so put it here, in the
             * red value.
             */
            rgbLookupData->u.rgb232.red[ index ] =
                0x80 | ((pixel>>R232BitOffset) & R232Mask);
            break;
        default:
            break;      /* never happens */
        }
    }

    for (index = nGreenLookup - 1; index >= 0; index--) {
        lookup = ((float)index - ycToRgb->greenDitherBias) / (1<<fracBits);
        lookup = (lookup - GreenBias) * YToRGBConst + .5;
        if (lookup < 0.0)
            lookup = 0.0;
        else if (lookup > 255.0)
            lookup = 255.0;
        pixel = (UChar)lookup;
        switch (ycToRgb->depthType) {
        case Rgb888:
            if (ycToRgb->invertFinalRgb)
                pixel = 255 - pixel;
            rgbLookupData->u.rgb888.green[ index ] = pixel;
            break;
        case Rgb332:
            rgbLookupData->u.rgb332.green[ index ] =
                (pixel>>G332BitOffset) & G332Mask;
            break;
        case Rgb232:
            rgbLookupData->u.rgb232.green[ index ] =
                (pixel>>G232BitOffset) & G232Mask;
            break;
        default:
            break;      /* never happens */
        }
    }

    for (index = nBlueLookup - 1; index >= 0; index--) {
        lookup = ((float)index - ycToRgb->blueDitherBias) / (1<<fracBits);
        lookup = (lookup - BlueBias) * YToRGBConst + .5;
        if (lookup < 0.0)
            lookup = 0.0;
        else if (lookup > 255.0)
            lookup = 255.0;
        pixel = (UChar)lookup;
        switch (ycToRgb->depthType) {
        case Rgb444:
        case Rgb888:
            if (ycToRgb->invertFinalRgb)
                pixel = 255 - pixel;
            rgbLookupData->u.rgb888.blue[ index ] = pixel;
            break;
        case Rgb332:
            rgbLookupData->u.rgb332.blue[ index ] =
                (pixel>>B332BitOffset) & B332Mask;
            break;
        case Rgb232:
            rgbLookupData->u.rgb232.blue[ index ] =
                (pixel>>B232BitOffset) & B232Mask;
            break;
        default:
            break;      /* never happens */
        }
    }
    return( NULL );
}

void
AlignConversionBox(int pixelSizeInBits,
                   int destinationAlignment,
                   GeomRectanglePtr box,
                   int *dstStart)
{
    long moveLeftPtr = 0;

    switch (pixelSizeInBits * destinationAlignment) {
    case 8: /* nothing to do */
        break;

    case 32:
        if ((moveLeftPtr = BytesOutOfMod4Alignment(*dstStart))) {
            box->x += moveLeftPtr * 8 / pixelSizeInBits;
            box->width -= moveLeftPtr * 8 / pixelSizeInBits;
        }
        box->width &= ~ 0x3;
        break;

    case 64:
        if ((moveLeftPtr = BytesOutOfMod8Alignment(*dstStart))) {
            box->x += moveLeftPtr * 8 / pixelSizeInBits;
            box->width -= moveLeftPtr * 8 / pixelSizeInBits;
        }
        box->width &= ~ 0x7;
        break;
    default:
        ProgMsg("internal alignment error=%d*%d\n",
            pixelSizeInBits, destinationAlignment);
    }

    *dstStart += moveLeftPtr;
}

int
LookupPixelSizeInBits(RgbDepthType depth)
{
    int pixelSizeInBits = 8;    /* default value */

    switch(depth) {
    case Rgb888: pixelSizeInBits = 32; break;
    case Rgb555: pixelSizeInBits = 16; break;
    case Rgb444: pixelSizeInBits = 16; break;
    case GrayScale1: pixelSizeInBits = 1; break;
    default: break;
    }
    return( pixelSizeInBits );
}

YcToRgbInfoPtr
AllocYcToRgbInfo(RgbDepthType depthType)
{
    YcToRgbInfoPtr ptr;
    int length;

    switch(depthType) {
    case GrayScale1:
    case GrayScale7:
    case GrayScale8:    length = sizeof(YcToGrayInfo); break;

    case Rgb444:
    case Rgb888:
    case Rgb555:
    case Rgb232:
    case Rgb332:
    case RgbC555:
    case RgbC666:       length = sizeof(YcToRgbInfo); break;

    default:            return(NULL);
    }

    ptr = (YcToRgbInfoPtr)InsecureAlloc(length);
    if (ptr == NULL)
        return(NULL);
    memset((char *)ptr, 0, length);
    ptr->depthType = depthType;
    ptr->length = length;

    return(ptr);
}

/*
 * This assumes that the elements
 *      lumaWidth, rgbWidth, depthType
 * have been assigned.  It will
 *   - assign values for box
 *   - assign {luma, chroma and rgb}Start according to x,y in box
 *   - assign {luma, chroma and rgb}Accel according to lumaWidth
 *     and chromaWidth 
 *   - Initialize dither.
 * y, cr, cb and rgb can be assigned at any time.
 */
InternalReasonPtr
InitYcToRgbInfo(YcToRgbInfoPtr *pYcToRgb,
                GeomRectanglePtr src,
                GeomRectanglePtr dst)
{
    YcToRgbInfoPtr ycToRgb = *pYcToRgb;
    InternalReasonPtr reason;
    int linesPerLoop, width, height, pixelSizeInBits, pixelSizeInBytes;
    float colorStepSize;
    float sizeScale;
    void (*func)(YcToRgbInfoPtr);

    switch(ycToRgb->depthType) {
    case Rgb888:
	if (ycToRgb->multiStepLookup)
	    colorStepSize = (1<<RGB888FracBits) - 1;
	else
	    colorStepSize = 1;
	break;
    case Rgb444: colorStepSize = 15; break;
    case Rgb555: colorStepSize = 255/15.0; break;
    case Rgb232: colorStepSize = (1<<(RGB232FracBits + 5)) - 1; break;
    case Rgb332: colorStepSize = (1<<(RGB332FracBits + 5)) - 1; break;
    case RgbC555:colorStepSize = 255/4.0; break;
    case RgbC666:colorStepSize = 255/5.0; break;
    default:     colorStepSize = 0; break;
    }
    ycToRgb->colorStepSize = colorStepSize;

    switch (ycToRgb->scale) {
    default:	/* should not happen */
    case LookupScaleUnity:   sizeScale = 1.0; break;
    case LookupScaleQuad:    sizeScale = 2.0; break;
    }

    width = src->width * sizeScale;
    if (width > dst->width)
        width = dst->width;
    height = src->height * sizeScale;
    if (height > dst->height)
        height = dst->height;

    ycToRgb->box.width = width;
    ycToRgb->box.height = height;
    ycToRgb->box.x = dst->x;
    ycToRgb->box.y = dst->y;

    ycToRgb->lumaStart = src->x + src->y * ycToRgb->lumaWidth;
    ycToRgb->chromaStart = (src->x>>1) + (src->y>.1) * (ycToRgb->lumaWidth>>1);
    if (ycToRgb->reflectVertical) {
        ycToRgb->rgbStart = dst->x
                          + (dst->y + dst->height - 1) * ycToRgb->rgbWidth;
    } else {
        ycToRgb->rgbStart = dst->x + dst->y * ycToRgb->rgbWidth;
    }

    /*
     * most of the algorithms do two or four input scan lines per loop.
     * A few do less.
     */
    linesPerLoop = 4 >> ycToRgb->scale;

    func = NULL;	/* gratuitous assignment for lint */

    switch(ycToRgb->depthType) {
    case Rgb444:
    case Rgb555:
        ycToRgb->destPixelsToAlign = 2;
        goto chooseDeepLookup;

    case Rgb888:
	if (ycToRgb->multiStepLookup)
	    linesPerLoop = 2;
        ycToRgb->destPixelsToAlign = 1;
    chooseDeepLookup:
        ycToRgb->redDitherBias = (int)colorStepSize >> 1;
        ycToRgb->greenDitherBias = (int)colorStepSize >> 1;
        ycToRgb->blueDitherBias = (int)colorStepSize >> 1;
        switch(ycToRgb->depthType) {
	default: /* never happens */
        case Rgb444: func = ChooseRgb444LookupFunc(ycToRgb); break;
        case Rgb555: func = ChooseRgb555LookupFunc(ycToRgb); break;
        case Rgb888: func = ChooseRgb888LookupFunc(ycToRgb); break;
        }
        break;

    case RgbC666:
        ycToRgb->destPixelsToAlign = 4;
        switch (ycToRgb->scale) {
        case LookupScaleUnity:   func = LookupC666RGBFrame;        break;
        case LookupScaleQuad:    func = LookupC666QuadRGBFrame;    break;
        }
        break;

    case RgbC555:
        ycToRgb->destPixelsToAlign = 4;
        switch (ycToRgb->scale) {
        case LookupScaleUnity:   func = LookupC555RGBFrame;        break;
        case LookupScaleQuad:    func = LookupC555QuadRGBFrame;    break;
        }
        break;

    case Rgb232:
        ycToRgb->redDitherBias = (int)colorStepSize;
        ycToRgb->greenDitherBias = (int)colorStepSize >> 1;
        ycToRgb->blueDitherBias = (int)colorStepSize;
        ycToRgb->destPixelsToAlign = 4;
        switch (ycToRgb->scale) {
        case LookupScaleUnity:   func = Lookup232RGBFrame;        break;
        case LookupScaleQuad:    func = Lookup232QuadRGBFrame;    break;
        }
        break;

    case Rgb332:
        ycToRgb->redDitherBias = (int)colorStepSize >> 2;
        ycToRgb->greenDitherBias = (int)colorStepSize >> 2;
        ycToRgb->blueDitherBias = (int)colorStepSize >> 1;
        ycToRgb->destPixelsToAlign = 4;
        switch (ycToRgb->scale) {
        case LookupScaleUnity:   func = Lookup332RGBFrame;        break;
        case LookupScaleQuad:    func = Lookup332QuadRGBFrame;    break;
        }
        break;

    default:
        return( InternalProblem("InitYcToRgbInfo: depth=%d\n",
                                ycToRgb->depthType) );
    }
    ycToRgb->lookupFrame = func;

    /*
     * If the destination buffer is not aligned the way we want it,
     * (which only happens if the destination is a frame buffer)
     * move it to the right until it is and then trim the right-hand edge.
     */
    pixelSizeInBits = LookupPixelSizeInBits(ycToRgb->depthType);
    pixelSizeInBytes = pixelSizeInBits >> 3;
    AlignConversionBox(pixelSizeInBits,
                       ycToRgb->destPixelsToAlign,
                       &ycToRgb->box,
                       &ycToRgb->rgbStart);

    /*
     * Now that we know the final width and height, we can calculate
     * the accelerators.
     */
    width = ycToRgb->box.width / sizeScale;
    ycToRgb->lumaAccel = ycToRgb->lumaWidth * linesPerLoop - width;
    ycToRgb->chromaAccel  = ycToRgb->chromaWidth * (linesPerLoop>>1)
                          - (width>>1);
    if (ycToRgb->reflectVertical) {
        ycToRgb->rgbAccel = -(ycToRgb->rgbWidth * linesPerLoop * sizeScale
                          + (ycToRgb->box.width * pixelSizeInBytes));
        ycToRgb->rgbWidth = - ycToRgb->rgbWidth;
    } else {
        ycToRgb->rgbAccel = ycToRgb->rgbWidth * linesPerLoop * sizeScale
                          - (ycToRgb->box.width * pixelSizeInBytes);
    }

    if (! ycToRgb->tablesAreValid) {
        switch (ycToRgb->depthType) {
	default:	/* never happens */
        case Rgb888:
	    if (ycToRgb->multiStepLookup)
		reason = InitMultiStepLookupData(pYcToRgb);
	    else
		reason = InitSingleStepLookupData(pYcToRgb);
	    ycToRgb = *pYcToRgb;
	    if (reason)
	        return(reason);
	    break;

        case Rgb555:
        case Rgb444:
        case RgbC555:
        case RgbC666:
            reason = InitSingleStepLookupData(pYcToRgb);
            if (reason)
                return(reason);
            ycToRgb = *pYcToRgb;
            break;
        case Rgb232:
        case Rgb332:
            reason = InitMultiStepLookupData(pYcToRgb);
            if (reason)
                return(reason);
            ycToRgb = *pYcToRgb;
            break;
        }
        ycToRgb->tablesAreValid = True;
        SetRgbTableChecksum(ycToRgb);
    } else {
        CheckRgbTableChecksum(ycToRgb);
    }

    return( NULL );
}

void
ShowLookupStatus(char *status, YcToRgbInfoPtr ycToRgb)
{
    char *name = EnumToName(ycToRgb->depthType, RgbDepthTypeNames);
    char *composite = EnumToName(ycToRgb->components, RgbPixelComponentsNames);
    ProgMsg("%s/%s not %s\n", name, composite, status);
}

void
RgbLookupNotConfigured(YcToRgbInfoPtr ycToRgb)
{
    ShowLookupStatus("configured", ycToRgb);
}

void
RgbLookupNotImplemented(YcToRgbInfoPtr ycToRgb)
{
    ShowLookupStatus("implemented", ycToRgb);
}


#ifdef DEBUG_ENABLED

static void
SetRgbTableChecksum(YcToRgbInfoPtr ycToRgb)
{
    ULong checksum = 0;
    UChar *p = (UChar *)&ycToRgb->rgbLookupData;
    int length = sizeof(ycToRgb->rgbLookupData);

    while (length-- > 0)
        checksum += *p++;
    ycToRgb->tableChecksum = checksum;
}

static void
CheckRgbTableChecksum(YcToRgbInfoPtr ycToRgb)
{
    ULong checksum = ycToRgb->tableChecksum;

    SetRgbTableChecksum(ycToRgb);
    if (checksum != ycToRgb->tableChecksum)
        ProgMsg("yc table checksum changed from 0x%x to 0x%x\n",
            checksum, ycToRgb->tableChecksum);
}

#endif /* DEBUG_ENABLED */
