/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	<name> - <one-line description of header file>
 *
 * DESCRIPTION
 *	<overall description of what the header file does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/rgblookup.h,v 1.2 1995/11/01 01:51:43 toddb Exp $
 */
#ifndef _RGBLOOKUP_H_
#define _RGBLOOKUP_H_

#include "nvrImageI.h"
#include "colorSpace.h"

/*
 *	DEFINES
 */
/*
 * These must be provided here for the declarations in rgblookup.h to work
 * right.
 */
#define RGB888FracBits  4
#define RGB444FracBits  0
#define RGB332FracBits  0
#define RGB232FracBits  0

/*
 * 4x4 Magic Squares for dithering
 *	 1,    6,   13,   10
 *	12,   11,    0,    7
 *	 2,    5,   14,    9
 *	15,    8,    3,    4
 * The shifting places the dithered constants in the top portion of the byte
 */
#define NDITHERBITS	4
#define D00	 1
#define D01	 6
#define D02	13
#define D03	10
#define D10	12
#define D11	11
#define D12	 0
#define D13	 7
#define D20	 2
#define D21	 5
#define D22	14
#define D23	 9
#define D30	15
#define D31	 8
#define D32	 3
#define D33	 4

#define MaxDitherValue	0xff

#define DitherTo5BitDither(d)		(((ULong)d)>>27)
#define DitherTo5BitDitherPair(d)	((d>>16) & 0x0e0f)

/*
 * These are the values of the maximum negative values for red, green
 * and blue.  They are used to define the size and bounds of the lookup tables.
 */

#define UnbiasedRedIndex(cr)	(YCrCbToRed(0,cr,0) / YToRGBConst)
#define UnbiasedGreenCrIndex(cr) ((YCrCbToGreen(0,cr,0)-YcToGConst) \
				 / YToRGBConst)
#define UnbiasedGreenCbIndex(cb) (YCrCbToGreen(0,0,cb) / YToRGBConst)
#define UnbiasedBlueIndex(cb)	(YCrCbToBlue(0,0,cb) / YToRGBConst)

#define RedBias			(- UnbiasedRedIndex(0))
#define GreenCrBias		(- UnbiasedGreenCrIndex(255))
#define GreenCbBias		(- UnbiasedGreenCbIndex(255))
#define GreenBias		(GreenCbBias + GreenCrBias)
#define BlueBias		(- UnbiasedBlueIndex(0))

#define RedIndex(cr)		(UnbiasedRedIndex(cr) + RedBias)
#define GreenCrIndex(cr)	(UnbiasedGreenCrIndex(cr) + GreenCrBias)
#define GreenCbIndex(cb)	(UnbiasedGreenCbIndex(cb) + GreenCbBias)
#define BlueIndex(cb)		(UnbiasedBlueIndex(cb) + BlueBias)

#define MaxRedIndex		RedIndex(255)
#define MaxGreenCrIndex		GreenCrIndex(0)
#define MaxGreenCbIndex		GreenCbIndex(0)
#define MaxGreenIndex		(MaxGreenCrIndex + MaxGreenCbIndex)
#define MaxBlueIndex		BlueIndex(255)

#define RedLookupSpace		(MaxRedIndex+256)
#define GreenLookupSpace	(MaxGreenCrIndex+MaxGreenCbIndex+256)
#define BlueLookupSpace		(MaxBlueIndex+256)

/*
 * These constants define the 888 lookup space.
 */
#define N888RedLookup		((int)(RedLookupSpace * (1<<RGB888FracBits)))
#define N888GreenLookup		((int)(GreenLookupSpace * (1<<RGB888FracBits)))
#define N888BlueLookup		((int)(BlueLookupSpace * (1<<RGB888FracBits)))

/*
 * Here we define 7-bit rgb lookup and dithering, which can be a 4x4
 * magic square, error diffusion or simply truncating.
 * The rgb bits are dithered to a 232 in the bottom seven bits of the
 * pixel.  The top bit of the pixel is always set.
 */
#define Rgb232DitherSpace	(MaxDitherValue>>B232Bits)

#define N232RedLookup		((int)((RedLookupSpace + Rgb232DitherSpace) \
				 * (1<<RGB232FracBits)))
#define N232GreenLookup		((int)((GreenLookupSpace+Rgb232DitherSpace) \
				 * (1<<RGB232FracBits)))
#define N232BlueLookup		((int)((BlueLookupSpace + Rgb232DitherSpace) \
				 * (1<<RGB232FracBits)))

#define R232Bits		2
#define G232Bits		3
#define B232Bits		2
#define RGB232Bits		(R232Bits+G232Bits+B232Bits)

#define R232BitOffset		1	/* offset from the top of the byte */
#define G232BitOffset		(R232BitOffset+R232Bits)
#define B232BitOffset		(G232BitOffset+G232Bits)

#define R232Mask		(((1<<R232Bits)-1)<<(8-R232BitOffset-R232Bits))
#define G232Mask		(((1<<G232Bits)-1)<<(8-G232BitOffset-G232Bits))
#define B232Mask		(((1<<B232Bits)-1)<<(8-B232BitOffset-B232Bits))

/*
 * Here we define 8-bit rgb lookup and dithering, which can be a 4x4
 * magic square or error diffusion or simply truncating.
 * The rgb bits are dithered to 332, filling an 8-bit pixel.
 */
#define Rgb332DitherSpace	(MaxDitherValue>>B332Bits)

#define N332RedLookup		((int)((RedLookupSpace + Rgb332DitherSpace) \
				 * (1<<RGB332FracBits)))
#define N332GreenLookup		((int)((GreenLookupSpace+Rgb332DitherSpace) \
				 * (1<<RGB332FracBits)))
#define N332BlueLookup		((int)((BlueLookupSpace + Rgb332DitherSpace) \
				 * (1<<RGB332FracBits)))

#define R332Bits		3
#define G332Bits		3
#define B332Bits		2
#define RGB332Bits		(R332Bits+G332Bits+B332Bits)

#define R332BitOffset		0	/* offset from the top of the byte */
#define G332BitOffset		(R332BitOffset+R332Bits)
#define B332BitOffset		(G332BitOffset+G332Bits)

#define R332Mask		(((1<<R332Bits)-1)<<(8-R332BitOffset-R332Bits))
#define G332Mask		(((1<<G332Bits)-1)<<(8-G332BitOffset-G332Bits))
#define B332Mask		(((1<<B332Bits)-1)<<(8-B332BitOffset-B332Bits))

#define BytesOutOfMod8Alignment(p)      ((8 - ((long)(p) & 0x7)) &0x7)
#define BytesOutOfMod4Alignment(p)      ((4 - ((long)(p) & 0x3)) &0x3)

#define RgbC666CrBits		6
#define RgbC666CbBits		6
#define RgbC666YBits		7

#define RgbC555CrBits		6
#define RgbC555CbBits		6
#define RgbC555YBits		6

#define Rgb555CrBits		6
#define Rgb555CbBits		6
#define Rgb555YBits		6

#define Rgb444CrBits		6
#define Rgb444CbBits		6
#define Rgb444YBits		6

#define Rgb888CrBits		6
#define Rgb888CbBits		6
#define Rgb888YBits		6

/*
 *	TYPEDEFS
 */

/****************************************************************************
 *
 * This section lays out the structures that hold the lookup tables
 * It is a two step process:
 *  - use cru and cbu to get the scaled, partial results
 *  - use the sum of step one and Y to look up the correct red,green,blue
 *
 ****************************************************************************/

#ifndef RGB888FracBits
#define RGB888FracBits 4
#endif

#ifndef RGB332FracBits
#define RGB332FracBits 0
#endif

#ifndef RGB232FracBits
#define RGB232FracBits 0
#endif

#define CompIndexToRed(index)	((ULong)(index) >> 16)
#define CompIndexToBlue(index)	((ULong)(index) >> 16)
#define CompIndexToGreen(index)	((ULong)(index) & 0xffff)

typedef struct _rgbIndex {
    union {
	ULong indexToRedAndGreen[ 256 ];
	struct {
	    UShort toRed;
	    UShort toGreen;
	} index[ 256 ];
    } cru;
    union {
	ULong indexToBlueAndGreen[ 256 ];
	struct {
	    UShort toBlue;
	    UShort toGreen;
	} index[ 256 ];
    } cbu;
} RgbIndex, *RgbIndexPtr;

typedef struct _rgbLookupInfo {
    RgbIndex index;
    union {
	struct {
	    UChar red[ N888RedLookup ];
	    UChar green[ N888GreenLookup ];
	    UChar blue[ N888BlueLookup ];
	} rgb888;
	struct {
	    UChar red[ N332RedLookup ];
	    UChar green[ N332GreenLookup ];
	    UChar blue[ N332BlueLookup ];
	} rgb332;
	struct {
	    UChar red[ N232RedLookup ];
	    UChar green[ N232GreenLookup ];
	    UChar blue[ N232BlueLookup ];
	} rgb232;
	UChar rgbEqual[1];	/* much longer */
    } u;
} RgbLookupInfo, *RgbLookupInfoPtr;


/****************************************************************************
 *
 * This section lays out a structure containing all you need for accelerated
 * y/cr/cb -> rgb conversion.
 *
 ****************************************************************************/
typedef struct _ycToRgbInfo YcToRgbInfo, *YcToRgbInfoPtr;
typedef struct _ycToGrayInfo YcToGrayInfo, *YcToGrayInfoPtr;

typedef enum {
    LookupScaleUnity,	/* don't scale */
    LookupScaleQuad	/* scale output picture by 4, spatially */
} LookupScale;

typedef void (*RgbLookupFunc)(YcToRgbInfoPtr);

/*
 * This structure defines the overall control for yc -> rgb conversion
 * including knowledge about the source and destination, dithering, etc.
 */
struct _ycToRgbInfo {
    RgbDepthType depthType;	/* must be first */
    int length;			/* must be second */

    LookupScale scale;
    boolean reflectVertical;	/* invert image vertically during conversion */

    boolean multiStepLookup;	/* only meaningful for Rgb888, 555 or 444 */
    boolean invertFinalRgb;	/* true for inverted rgb space (like the Mac) */
    RgbPixelComponents components;/* only meaningful for Rgb888, 555 or 444 */
    UChar *y;
    UChar *cr;
    UChar *cb;
    int lumaWidth;
    int chromaWidth;

    /* function called at end of line */
    void (*lineNotify)(YcToRgbInfoPtr, int height1, int height2);

    /* function called at end of frame */
    void (*frameNotify)(YcToRgbInfoPtr);

    UChar *rgb;
    int rgbWidth;

    GeomRectangle box;		/* source and destination box */

    int lumaStart;		/* accelerator to nw starting corner */
    int chromaStart;		/* accelerator to nw starting corner */
    int rgbStart;		/* accelerator to nw starting corner */

    int lumaAccel;		/* accelerator to next line after processing */
    int chromaAccel;		/* accelerator to next line after processing */
    int rgbAccel;		/* accelerator to next line after processing */

    boolean tablesAreValid;	/* true ==> tables are already valid */

    long destPixelsToAlign;	/* supplied internally: how to align */

    /*
     * These values are calculated internally
     */
    float colorStepSize;	/* RgbC types: the step size for each RGB */ 
    UChar redDitherBias;	/* internal bias for tables */
    UChar greenDitherBias;	/* internal bias for tables */
    UChar blueDitherBias;	/* internal bias for tables */

    RgbLookupFunc lookupFrame;

    ULong tableChecksum;	/* only used during debugging */

    RgbLookupInfo rgbLookupData;/* mustbe last: the lookup tables */

}; /* typedef: YcToRgbInfo, *YcToRgbInfoPtr */

struct _ycToGrayInfo {
    RgbDepthType depthType;	/* must be first */
    int length;			/* must be second */

    LookupScale scale;
    boolean reflectVertical;	/* invert image vertically during conversion */

    UChar *y;
    int lumaWidth;

    UChar *gray;
    int grayWidth;

    GeomRectangle box;		/* source and destination box */

    int lumaStart;		/* accelerator to nw starting corner */
    int grayStart;		/* accelerator to nw starting corner */

    int lumaAccel;		/* accelerator to next line after processing */
    int grayAccel;		/* accelerator to next line after processing */

    long destPixelsToAlign;	/* supplied internally: how to align */

    /*
     * These values are calculated internally
     */
    UChar yDithered7to3Pixel[ 128 * 16 ];

    void (*lookupFrame)(YcToGrayInfoPtr);

}; /* typedef: YcToGrayInfo, *YcToGrayInfoPtr */

/*
 *	EXTERNS
 */

CExternal YcToGrayInfoPtr AllocYcToGrayInfo(
    RgbDepthType depthType
);

CExternal InternalReasonPtr InitYcToGrayInfo(
    YcToGrayInfoPtr *,	/* previous YcToGrayInfoPtr or YcToRgbInfoPtr */
    GeomRectanglePtr,	/* source */
    GeomRectanglePtr	/* destination */
);

CExternal YcToRgbInfoPtr AllocYcToRgbInfo(
    RgbDepthType depthType
);

CExternal InternalReasonPtr InitYcToRgbInfo(
    YcToRgbInfoPtr *,	/* previous YcToGrayInfoPtr or YcToRgbInfoPtr */
    GeomRectanglePtr,	/* source */
    GeomRectanglePtr	/* destination */
);

CExternal void AlignConversionBox(
    int,		/* in: pixelSizeInBits */
    int,		/* in: destinationAlignment */
    GeomRectanglePtr,	/* in, out: dest bounding box */
    int *		/* in, out: dstStart */
);

CExternal RgbLookupFunc ChooseRgb888LookupFunc(YcToRgbInfoPtr);
CExternal RgbLookupFunc ChooseRgb444LookupFunc(YcToRgbInfoPtr);
CExternal RgbLookupFunc ChooseRgb555LookupFunc(YcToRgbInfoPtr);
CExternal int LookupPixelSizeInBits(RgbDepthType);
CExternal YcToRgbInfoPtr AllocYcToRgbInfo(RgbDepthType);
CExternal YcToGrayInfoPtr AllocYcToGrayInfo(RgbDepthType);

/*
 * Internal functions.
 */
CExternal void ShowLookupStatus(char *status, YcToRgbInfoPtr);
CExternal void RgbLookupNotConfigured(YcToRgbInfoPtr);
CExternal void RgbLookupNotImplemented(YcToRgbInfoPtr);


#endif /* _RGBLOOKUP_H_ */
