/* $Header: /usr/local/src/nvrToMIT/RCS/test.c,v 1.1 1996/01/14 20:14:02 toddb Exp $ */

#include "imageDpy.h"
#include "nvrString.h"
#include "nvrAlloc.h"

char *ProgramName;

enum {
    Cin    = 0x01,
    Cout   = 0x02,
    Cdepth = 0x04,
    Csize  = 0x08,
    CAll   = (Cin|Cout|Cdepth|Csize)
};

static void
Usage() {
    Msg("Usage: %s size=WxH input=[-|<file>] output=<file> depth=depthType\n",
        ProgramName);
    exit(1);
}

int
main(int argc, char **argv)
{
    ImgDpyRegionPtr dpy;
    ImgDpyParam param;
    InternalReasonPtr reason = NULL;
    YcImage yc;
    int ycSize;
    int i;
    char *name, *cp;
    FileHandle inputFd;
    StringArgListPtr argp;
    struct {
	long width;
	long height;
	char *input;
	char *output;
	FileHandle fdIn;
	RgbDepthType color;
    } config;
    int confMask = 0;


    memset(&config, 0, sizeof(config));
    ProgramName = (char *)ExtractPathSuffix(argv[0]);

    argp = AllocStringArgList(argc, argv);
    if (argp == NULL) {
	Fatal("Can't allocate initial argument list\n");
    }
    if (argp->length < 1) {
	Usage();
    }
    for (i = argp->index; i < argp->length; i++) {
	name = argp->list[ i ].name;
	if (strcmp(name,"size") == 0) {
	    name = argp->list[ i ].value;
	    cp = (char *)index(name,'x');
	    if (! cp) {
		Usage();
	    }
	    *cp++ = '\0';
	    config.width = atoi(name);
	    config.height = atoi(cp);
	    confMask |= Csize;
	} else if (strcmp(name,"input") == 0) {
	    cp = config.input = argp->list[ i ].value;
	    if (strcmp(cp,"-") == 0) {
		config.fdIn = 0;
	    } else {
		config.fdIn = OpenFileForReading(config.input);
	    }
	    confMask |= Cin;
	} else if (strcmp(name,"output") == 0) {
	    cp = config.output = argp->list[ i ].value;
	    if (strcmp(cp,"-") == 0) {
		Usage();
	    }
	    confMask |= Cout;
	} else if (strcmp(name,"depth") == 0) {
	    cp = argp->list[ i ].value;
	    config.color = (int) NameToEnum(cp, &RgbDepthTypeNames);
	    if (config.color == DepthUnspecified) {
		Usage();
	    }
	    confMask |= Cdepth;
	} else {
	    Usage();
	}
    }
		
    if ((confMask & CAll) != CAll) {
	Usage();
    }

    dpy = CreateImageRegion(FileImageDesc, &reason);
    if (!dpy) {
	goto done;
    }

    param.file = config.output;

    param.src.colorSpace = ColorYC;
    param.src.sample = Sample4_2_0;
    param.src.width = config.width;
    param.src.height = config.height;
    param.src.lineBytes = config.width;
    param.src.chromaLineBytes = config.width >> 1;

    param.dst.x = 0;
    param.dst.y = 0;
    param.dst.doubleSize = False;
    param.dst.winClipped = False;

    param.color = config.color;
    if (NULL != (reason = InitImageRegion(dpy, &param,  ID_DepthType
					              | ID_File
					              | ID_SrcInfo
					              | ID_DstInfo))) {
	goto done;
    }

    ycSize = param.src.width * param.src.height * 1.5;
    yc.y = (UChar *)AllocNamedMem(ycSize, "YC pixel buffer", &reason);
    if (yc.y == NULL) {
	goto done;
    }
    yc.cr = yc.y + param.src.width * param.src.height;
    yc.cb = yc.cr + ((param.src.width * param.src.height) >> 2);
    while(! reason &&
	  (i = ReadData(config.fdIn, (char *)yc.y, ycSize)) > 0 ) {
	reason = (*dpy->displayYcImage)(dpy, &yc);
    }

 done:
    if (reason) {
	TtyShowReason(reason);
	exit(1);
    }
    return(0);
}
