
/*
 *	(c) Copyright 1992 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		viewcmap.h
 *
 *	DESCRIPTION
 *		Include file for viewcmap.c
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/viewcmap.h,v 1.13 1994/04/17 17:57:22 toddb Exp $
 */
#ifndef _VIEWCMAP_H_
#define _VIEWCMAP_H_

/*
 *	INCLUDES
 */
#include <X11/Xlib.h>
#include <X11/Xcms.h>

#include "nvrImage.h"

/*
 *	DEFINES
 */

/*
 * Brightness is defined as a signed offset from the default brightness.
 * It is scaled to appropriate values in viewcmap.c
 */
#define	DEFAULT_BRIGHTNESS	0
#define	MAX_BRIGHTNESS		(DEFAULT_BRIGHTNESS+50)
#define	MIN_BRIGHTNESS		(DEFAULT_BRIGHTNESS-50)

/*
 * Contrast is defined as a percentage of the dynamic range left over
 * after brightness has been added in.  The values here represent an offset
 * from the default brightness.  The real percentage is calculated private
 * to viewcmap.c
 */
#define	DEFAULT_CONTRAST	0
#define	MIN_CONTRAST		(DEFAULT_CONTRAST-50)
#define	MAX_CONTRAST		(DEFAULT_CONTRAST+50)

#define GAMMA_TBL_BITS	12
#define GAMMA_TBL_MAX	((1<<GAMMA_TBL_BITS)-1)
#define GAMMA_TBL_SIZE	(GAMMA_TBL_MAX + 1)

#define HUE_MASK		((UInt)(1<<0))
#define SATURATION_MASK		((UInt)(1<<1))
#define CONTRAST_MASK		((UInt)(1<<2))
#define BRIGHTNESS_MASK		((UInt)(1<<3))
#define DISPLAY_GAMMA_MASK	((UInt)(1<<4))
#define LOAD_COLORS_MASK	((UInt)(1<<5))	/* force colormap load */

#ifdef USE_X11R4
    typedef XColor ColorDefType;
    typedef UShort GammaValueType;
    typedef char *XPointer;		/* missing from their Xlib.h */
#   include "Xfuncproto.h.x11r5"
#   define XStoreColorsFunc	XStoreColors
#else /* USE_X11R4 */
    typedef XcmsColor ColorDefType;
    typedef double GammaValueType;
#   define XStoreColorsFunc	XcmsStoreColors
#endif /* USE_X11R4 */

/*
 *	TYPEDEFS
 */

/*
 * Parts of this are private to viewcmap.c
 */
typedef struct {
    Display		*dpy;			/* in: display pointer */
    Visual		*visual;		/* in: visual to use */
    int			scrNumber;		/* in: screen number */
    RgbDepthType	depthType; 		/* in: depth type */
    double		dpyGamma;		/* in: gamma value */

    Colormap		cmap;			/* out: colormap */
    int			contrast;		/* out: current contrast */
    int			brightness;		/* out: current brightness */

    /*
     * Private entries
     */
    boolean		cmapWritable;		/* true if entries writable */
    boolean		cmapShared;		/* true if cmap shared */
    int			privateContrast;	/* spread of colors */
    int			privateBrightness;	/* color base */
    ULong		*allocated;		/* temporary storage */
    GammaValueType	gamma_tbl[GAMMA_TBL_SIZE];
    int			redColors, greenColors, blueColors;
} ViewCmapInfo, *ViewCmapInfoPtr;

/*
 * This is the public structure
 */
typedef struct {
    int		hue;
    int		saturation;
    int		contrast;	/* spread of colors */
    int		brightness;	/* color base */
    double	dpyGamma;
} ViewAttributes, *ViewAttributesPtr;


/*
 *	EXTERNS
 */

CExternal void SetViewAttributes(
    ViewCmapInfoPtr,
    ViewAttributesPtr,
    UInt
);

CExternal boolean InitViewCmapInfo(
    ViewCmapInfoPtr
);

#endif /* _VIEWCMAP_H_ */
