/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              xImageDpy.c
 *
 *      DESCRIPTION
 *              X methods for displaying an image in a window
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/xImageDpy.c,v 1.7 1996/01/17 00:10:01 toddb Exp $
 */

#include "xImageDpyI.h"
#include "nvrDebug.h"

#define AllInitialized(initialized)     ((initialized & ID_AllWin) == ID_AllWin)

static InternalReason NotInitialized =
  { "First call to InitXImageRegion() must have all params initialized.\n",
    IR_InternalProblem };
 
static InternalReasonPtr
PutBlackXImage(XImageRegionPtr ximg)
{
    XFillRectangle(ximg->render.dpy, ximg->render.win, ximg->render.gc,
		   0, 0, ximg->super.dst.width, ximg->super.dst.height);

    return(NULL);
}

static InternalReasonPtr
RefreshOneStageXImage(ImgDpyRegionPtr pubimg)
{
    XImageRegionPtr ximg = (XImageRegionPtr)pubimg;
    InternalReasonPtr irp;

    /*
     * This is usually called when a decoder is stopped and an
     * expose event happens, so it doesn't matter if this is slow.
     */
    if (ximg->super.lastYci)
	irp = (*pubimg->displayYcImage)(pubimg, ximg->super.lastYci);
    else
	irp = PutBlackXImage(ximg);

    return(irp);
}

static InternalReasonPtr
RefreshTwoStageXImage(ImgDpyRegionPtr pubimg)
{
    XImageRegionPtr ximg = (XImageRegionPtr)pubimg;

    (*ximg->render.putImage)(ximg->render.dpy, ximg->render.win,
			     ximg->render.gc, ximg->render.image,
			     ximg->super.src.x, ximg->super.src.y,
			     ximg->super.dst.x, ximg->super.dst.y,
			     ximg->super.dst.width, ximg->super.dst.height);
    XFlush(ximg->render.dpy);

    return(NULL);
}

static int
SharedMemPutImage(Display *dpy, Drawable dr, GC gc, XImage *image,
           int src_x, int src_y, int dst_x, int dst_y,
           unsigned int width, unsigned int height)
{
    return( XShmPutImage(dpy,dr,gc,image,src_x,src_y,dst_x,dst_y,
                         width,height,True) );
}

static int
NullPutImage(Display *dpy, Drawable dr, GC gc, XImage *image,
             int src_x, int src_y, int dst_x, int dst_y,
             unsigned int width, unsigned int height)
{
    XFillRectangle(dpy, dr, gc, 0, 0, width, height);
    return(0);
}

static void
InitXDisplay(PrivImgDpyRegionPtr img)
{
    XImageRegionPtr ximg = (XImageRegionPtr)img;
    XWindowAttributes winAttr;

    XGetWindowAttributes(ximg->render.dpy, ximg->render.win, &winAttr);
    ximg->render.visual = winAttr.visual;
    ximg->render.depth = winAttr.depth;
 
    img->oneStageConvert = False;   /* this is all we know just now */
    if (img->oneStageConvert)
	img->pub.refreshImage = RefreshOneStageXImage;
    else
	img->pub.refreshImage = RefreshTwoStageXImage;
    img->displayYcNative = IllegalYcDisplay;

    if (ximg->render.gc)
	XFreeGC(ximg->render.dpy, ximg->render.gc);
    ximg->render.gc = XCreateGC(ximg->render.dpy,
			        ximg->render.win,
			        0,        /* value mask */
			        (XGCValues *)NULL);
}

static InternalReasonPtr
InitXImageRegion(PrivImgDpyRegionPtr img, /* structure to update */
                   ImgDpyParamPtr param,    /* parameters to update */
                   ULong paramMask)         /* mask of parameters to update */
{
    InternalReasonPtr reason = NULL;
    ULong initialized = img->initialized | paramMask;
    XImageRegionPtr ximg = (XImageRegionPtr)img;
    

    if (! AllInitialized(initialized)) {
        return( &NotInitialized );
    }

    if (paramMask & ID_DstInfo) {
        img->dstInfo = param->dst;
        img->initialized |= ID_DstInfo;
    }

    if (paramMask & ID_SrcInfo) {
        img->srcInfo = param->src;
        img->initialized |= ID_SrcInfo;
    }

    if (paramMask & ID_Window) {
	ximg->render.win = (Window)param->u.wsys.win;
	ximg->render.dpy = (Display *)param->u.wsys.dpy;
        img->initialized |= ID_Window;
    }

    if (paramMask & ID_DepthType) {
        img->color = param->color;
        img->initialized |= ID_DepthType;
    }
 
    if (paramMask & (ID_DepthType | ID_DstInfo | ID_SrcInfo)) {
        InitXDisplay(img);
        reason = InitImgPixmap(img);
        if (reason)
            return(reason);

	/*
	 * The dst.{x,y} are set in InitImgPixmap()... override them here
	 */
	if (ximg->render.fbDirect) {
	    img->dst.x = ximg->fb.xOff + img->dstInfo.x;
	    img->dst.y = ximg->fb.yOff + img->dstInfo.y;
	}

        reason = InitImgYcConversion(img);
        if (reason)
            return(reason);
    }

    return (reason);
}

static void
FreeXCopyBuffer(XImageRegionPtr ximg)
{
    XShmSegmentInfo *shm = ximg->shm.shmInfo;

    if (shm) {
	if (shm->shmid >= 0)
	    shmctl(shm->shmid,IPC_RMID,NULL);
	if (shm->shmaddr) {
            shmdt(shm->shmaddr);
        }
	shm->shmid = -1;
	shm->shmaddr = NULL;
	free((char *)shm);
	ximg->shm.shmInfo = NULL;
    } else {
	if (ximg->super.copyBuffer)
	    free((char *)ximg->super.copyBuffer);
    }
    if (ximg->render.image != NULL) {
	ximg->render.image->data = NULL;
	(*ximg->render.image->f.destroy_image)(ximg->render.image);
    }

    ximg->render.image = NULL;
    ximg->super.copyBuffer = NULL;
}

static InternalReasonPtr
AllocXCopyBuffer(PrivImgDpyRegionPtr img)
{
    XImageRegionPtr ximg = (XImageRegionPtr)img;
    InternalReasonPtr reason = NULL;
    XShmSegmentInfo *shm;

    FreeXCopyBuffer(ximg);

    ximg->render.putImage = NullPutImage;

    if (ximg->shm.useShm) {
	shm = ximg->shm.shmInfo = (XShmSegmentInfo *)
	    AllocNamedMem(sizeof(XShmSegmentInfo),
			  "shm segment info", &reason);
	if (shm == NULL)
	    return(reason);

	shm->shmaddr = NULL;
	shm->shmid = shmget(IPC_PRIVATE, img->copyBufferSize, IPC_CREAT|0777);
	if (shm->shmid < 0) {
	    SystemError("shmget failure; clean up with ipcs and ipcrm\n");
	    FreeXCopyBuffer(ximg);
	    goto plainMemory;
	}
	shm->shmaddr = (char *)shmat(shm->shmid,0,0);
	if(shm->shmaddr == (char *)-1) {
	    WhenDebugEnabled(
	        SystemError("shmaddr failure\n")
            )
	    shm->shmaddr = NULL;
	    FreeXCopyBuffer(ximg);
	    goto plainMemory;
	}
	shm->readOnly = False;
	if (ximg->render.image) {
	    ximg->render.image->data = NULL;
	    (*ximg->render.image->f.destroy_image)(ximg->render.image);
	}
	img->copyBuffer = (UChar *)shm->shmaddr;
	ximg->render.image =
	    XShmCreateImage(ximg->render.dpy,
			    NULL,	/* visual (used for rgb mask) */
			    ximg->render.depth,
			    ZPixmap,
			    (char *)img->copyBuffer,
			    shm,
			    img->src.width, img->src.height);
	if (ximg->render.image == NULL) {
	    WhenDebugEnabled(
	        SystemError("XShmCreateImage failure\n")
            )
	    FreeXCopyBuffer(ximg);
	    goto plainMemory;
	}
        if( !XShmAttach(ximg->render.dpy, shm)) {
	    FreeXCopyBuffer(ximg);
            ProgMsg("XShmAttach failed, switching to X transport\n");
            goto plainMemory;
        }

	ximg->render.putImage = SharedMemPutImage;

    } else {
  plainMemory:
	ximg->shm.useShm = False;

	img->copyBuffer = (UChar *)
	    AllocNamedMem(img->copyBufferSize, "X RGB frame buffer", &reason);
	if (img->copyBuffer == NULL)
	    return(reason);

        ximg->render.image =
            XCreateImage(ximg->render.dpy,
                         NULL,                  /* visual (used for rgb mask) */
                         ximg->render.depth,
                         ZPixmap,
                         0,                     /* offset */
                         (char *)img->copyBuffer,
                         img->src.width, img->src.height,
                         32, 0);                /* bitmap pad, bytes/line */
        if (ximg->render.image == NULL) {
	    FreeXCopyBuffer(ximg);
	    return( AllocReason("X image", sizeof(*ximg->render.image)));
        }
	ximg->render.putImage = XPutImage;
    }

    return (NULL);
}


static void
FreeXImageRegion(PrivImgDpyRegionPtr img)
{
    FreeXCopyBuffer((XImageRegionPtr)img);
}

static ImgDpyRegionPtr
XCreateImageRegion(ImageDestDescPtr desc, InternalReasonPtr *pIrp)
{
    long size;
    XImageRegionPtr ximg;
   
    size = sizeof(XImageRegion);
    ximg = (XImageRegionPtr) AllocNamedMem(size, "X image region", pIrp);
    if (ximg == NULL)
        return(NULL);
 
    ximg->super.init = InitXImageRegion;
    ximg->super.allocCopyBuffer = AllocXCopyBuffer;
    ximg->super.free = FreeXImageRegion;
    ximg->render.putImage = NullPutImage;

    /*
     * Try out shared memory to start with.
     */
    ximg->shm.useShm = True;

    return(&ximg->super.pub);
}

static ImageDestDesc XWinDesc = {
    XCreateImageRegion
};

ImageDestDescPtr XWindowImageDesc = &XWinDesc;

