/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	jpegcomp.c - library interface for jpeg compressor/decompressor
 *
 * DESCRIPTION
 *	<overall description of what the c code does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/jpegcomp.c,v 1.23 1994/10/02 17:30:45 toddb Exp $
 */
#define NeedFunctionPrototypes 1
#include "nvrString.h"
#include "process.h"
#include "protoparseP.h"
#include "mcs-serverP.h"
#include "jpegcompP.h"

#define JpegServerName	"plxjpeg"

static void
EofNotify(ParseInfoPtr pParse)
{
    McsJpegPrivPtr pJpeg = (McsJpegPrivPtr)GetParsePrivateData(pParse);
    McsInputNotify inputNotify = pJpeg->u.generic.required.inputNotify;
    _McsEofNotify(pParse);
    
    (*inputNotify)((McsServer)pJpeg, pJpeg->io.inFd,
		   False, pJpeg->u.generic.optional.arg);
    (*pParse->config.inputNotify)(pParse);
}

/*
 * Decompressor
 */
static boolean
JpegOpenServer(McsJpegPrivPtr pJpeg, boolean compressor)
{
    Argvec serverArgs;
    ParseConfig config;

    pJpeg->compressor = compressor;

    memset((char *)&serverArgs, 0, sizeof(serverArgs));
    serverArgs.path = JpegServerName;
    AddArgv(&serverArgs, serverArgs.path, False);
    pJpeg->io = PipeToAndFrom(&serverArgs, (VoidFuncPtr) NULL);

    config.inFileHandle = pJpeg->io.inFd;
    config.outFileHandle = pJpeg->io.outFd;
    config.inputNotify = _McsInputNotify;
    config.eofNotify = EofNotify;
    config.outputNotify = NULL;	/* this is optional */
    config.pointer = (PrivPointer)pJpeg;
    config.server = False;
    config.asynchronous = False;
    config.u.client.eventNotify = _McsEventNotify;
    pJpeg->base.pParse = InitParsedConnection(&config);
    return (True);
}

static boolean
JpegDecompOpenServer(McsServer server)
{
    return( JpegOpenServer((McsJpegPrivPtr)server, False) );
}

static boolean
JpegCompOpenServer(McsServer server)
{
    return( JpegOpenServer((McsJpegPrivPtr)server, True) );
}

static boolean
JpegFlushParam(McsJpegPrivPtr pJpeg)
{
    ULong valueMask = pJpeg->notFlushed;
    boolean success = True;

    if (valueMask == 0)
	return(True);
    if (valueMask & JXcoord)
	_McsSendAssign(pJpeg->base.pParse, STR_X, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.required.x);
    if (valueMask & JYcoord)
	_McsSendAssign(pJpeg->base.pParse, STR_Y, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.required.y);
    if (valueMask & JWidth)
	_McsSendAssign(pJpeg->base.pParse, STR_WIDTH, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.required.width);
    if (valueMask & JHeight)
	_McsSendAssign(pJpeg->base.pParse, STR_HEIGHT, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.required.height);
    if (valueMask & JDisplayName)
	_McsSendAssign(pJpeg->base.pParse, STR_DISPLAY, MCS_TYPE_STRING,
		      (ParseValuePtr)&pJpeg->u.generic.required.displayName);
    if (valueMask & JPath)
	_McsSendAssign(pJpeg->base.pParse, STR_PATH, MCS_TYPE_STRING,
		      (ParseValuePtr)&pJpeg->u.generic.required.path);
    if (valueMask & JVisualId)
	_McsSendAssign(pJpeg->base.pParse, STR_VISUALID, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.required.visualId);
    if (valueMask & JWindowId)
	_McsSendAssign(pJpeg->base.pParse, STR_WINDOW, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.required.windowId);
    if (valueMask & JMode) {
	char *mode;

	switch(pJpeg->u.generic.required.mode) {
	case SYM_CAPTURE:      mode = STR_CAPTURE; break;
	case SYM_PLAYBACK:     mode = STR_PLAYBACK; break;
	default:	       success = False; break;
	}
	if (success)
	    _McsSendAssign(pJpeg->base.pParse, STR_MODE, MCS_TYPE_SYMBOL,
			   (ParseValuePtr)&mode);
    }
    if (valueMask & JMethod) {
	char *method;

	switch(pJpeg->u.generic.required.method) {
	case SYM_NATIVE: method = STR_NATIVE; break;
	case SYM_FAST_X: method = STR_FAST_X; break;
	case SYM_DIRECT: method = STR_DIRECT; break;
	default:	 success = False; break;
	}
	if (success)
	    _McsSendAssign(pJpeg->base.pParse, STR_METHOD, MCS_TYPE_SYMBOL,
			  (ParseValuePtr)&method);
    }
    if (valueMask & JAudioEnabled)
	_McsSendAssign(pJpeg->base.pParse, STR_AUDIOENABLE, MCS_TYPE_BOOL,
		      (ParseValuePtr)&pJpeg->u.generic.required.audioEnabled);
    if (valueMask & JVideoEnabled)
	_McsSendAssign(pJpeg->base.pParse, STR_VIDEOENABLE, MCS_TYPE_BOOL,
		      (ParseValuePtr)&pJpeg->u.generic.required.videoEnabled);

    if (valueMask & JQFactor)
	_McsSendAssign(pJpeg->base.pParse, STR_QFACTOR, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.required.qFactor);
    if (valueMask & JFrameRate)
	_McsSendAssign(pJpeg->base.pParse, STR_FRAME_RATE, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.required.frameRate);
    if (valueMask & JInTime)
	_McsSendAssign(pJpeg->base.pParse, STR_INTIME, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.optional.inTime);
    if (valueMask & JOutTime)
	_McsSendAssign(pJpeg->base.pParse, STR_OUTTIME, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.optional.outTime);
    if (valueMask & JInFrame)
	_McsSendAssign(pJpeg->base.pParse, STR_INFRAME, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.optional.inFrame);
    if (valueMask & JOutFrame)
	_McsSendAssign(pJpeg->base.pParse, STR_OUTFRAME, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.optional.outFrame);
    if (valueMask & JTolerance)
	_McsSendAssign(pJpeg->base.pParse, STR_TOLERANCE, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.optional.tolerance);
    if (valueMask & JPreloadSize)
	_McsSendAssign(pJpeg->base.pParse, STR_PRELOADSIZE, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.optional.preloadSize);
    if (valueMask & JAudioDelay)
	_McsSendAssign(pJpeg->base.pParse, STR_AUDIODELAY, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.optional.audioDelay);
    if (valueMask & JAudioSource)
	_McsSendAssign(pJpeg->base.pParse, STR_AUDIOSOURCE, MCS_TYPE_STRING,
		      (ParseValuePtr)&pJpeg->u.generic.optional.audioSource);
    if (valueMask & JFrameFrequency)
	_McsSendAssign(pJpeg->base.pParse, STR_FRAMEFREQ, MCS_TYPE_INT,
		      (ParseValuePtr)&pJpeg->u.generic.optional.frameFrequency);
    if (valueMask & JPostInit)
	_McsOutputMessage(pJpeg->base.pParse, "%s;",
		      (ParseValuePtr)pJpeg->u.generic.optional.postInit);
    pJpeg->notFlushed = 0;
    _McsSendCommand(pJpeg->base.pParse, STR_CONFIGURE);
    _McsFlushMessages(pJpeg->base.pParse);
    return (True);
}


static boolean
JpegSetWideParam(McsServer server, PrivPointer params, BitMaskPtr bp)
{
    Msg("JpegSetWideParam: Invalid. Must use JpegSetParam\n");
    return(True);
}

static boolean
JpegSetParam(McsServer server, PrivPointer params, ULong valueMask)
{
    McsJpegCompPtr jparam = (McsJpegCompPtr)params;
    McsJpegPrivPtr pJpeg = (McsJpegPrivPtr)server;

    if (valueMask & JXcoord)
	pJpeg->u.generic.required.x = jparam->required.x;
    if (valueMask & JYcoord)
	pJpeg->u.generic.required.y = jparam->required.y;
    if (valueMask & JWidth)
	pJpeg->u.generic.required.width = jparam->required.width;
    if (valueMask & JHeight)
	pJpeg->u.generic.required.height = jparam->required.height;
    if (valueMask & JDisplayName)
	pJpeg->u.generic.required.displayName = jparam->required.displayName;
    if (valueMask & JPath)
	pJpeg->u.generic.required.path = jparam->required.path;
    if (valueMask & JVisualId)
	pJpeg->u.generic.required.visualId = jparam->required.visualId;
    if (valueMask & JWindowId)
	pJpeg->u.generic.required.windowId = jparam->required.windowId;
    if (valueMask & JMode)
	pJpeg->u.generic.required.mode = jparam->required.mode;
    if (valueMask & JMethod)
	pJpeg->u.generic.required.method = jparam->required.method;
    if (valueMask & JAudioEnabled)
	pJpeg->u.generic.required.audioEnabled = jparam->required.audioEnabled;
    if (valueMask & JVideoEnabled)
	pJpeg->u.generic.required.videoEnabled = jparam->required.videoEnabled;
    if (valueMask & JInputNotify)
	pJpeg->u.generic.required.inputNotify = jparam->required.inputNotify;

    if (valueMask & JQFactor)
	pJpeg->u.generic.required.qFactor = jparam->required.qFactor;
    if (valueMask & JFrameRate)
	pJpeg->u.generic.required.frameRate = jparam->required.frameRate;
    if (valueMask & JInTime)
	pJpeg->u.generic.optional.inTime = jparam->optional.inTime;
    if (valueMask & JOutTime)
	pJpeg->u.generic.optional.outTime = jparam->optional.outTime;
    if (valueMask & JInFrame)
	pJpeg->u.generic.optional.inFrame = jparam->optional.inFrame;
    if (valueMask & JOutFrame)
	pJpeg->u.generic.optional.outFrame = jparam->optional.outFrame;
    if (valueMask & JTolerance)
	pJpeg->u.generic.optional.tolerance = jparam->optional.tolerance;
    if (valueMask & JPreloadSize)
	pJpeg->u.generic.optional.preloadSize = jparam->optional.preloadSize;
    if (valueMask & JAudioDelay)
	pJpeg->u.generic.optional.audioDelay = jparam->optional.audioDelay;
    if (valueMask & JFrameFrequency)
	pJpeg->u.generic.optional.frameFrequency =
	    jparam->optional.frameFrequency;
    if (valueMask & JAudioSource)
	pJpeg->u.generic.optional.audioSource = jparam->optional.audioSource;
    if (valueMask & JArg)
	pJpeg->u.generic.optional.arg = jparam->optional.arg;
    if (valueMask & JPreInit) {
	pJpeg->u.generic.optional.preInit = jparam->optional.preInit;
	_McsOutputMessage(pJpeg->base.pParse, "%s;", jparam->optional.preInit);
    }
    if (valueMask & JPostInit)
	pJpeg->u.generic.optional.postInit = jparam->optional.postInit;

    if ((pJpeg->assigned & JInputNotify) == 0 && (valueMask & JInputNotify)) {
	McsInputNotify inputNotify = pJpeg->u.generic.required.inputNotify;
	(*inputNotify)(server, pJpeg->io.inFd,
		       True, pJpeg->u.generic.optional.arg);
    }

    pJpeg->assigned |= valueMask;
    pJpeg->notFlushed |= valueMask;

    /*
     * Flush only if all the required stuff is in place.
     */
    if (pJpeg->compressor) {
	if ((pJpeg->assigned & JCRequired) != JCRequired)
	    return(True);
    } else {
	if ((pJpeg->assigned & JDRequired) != JDRequired)
	    return(True);
    }
    return( JpegFlushParam(pJpeg) );
}

static int
MaskToSymbol(int valueBit)
{
    int symbol;

    switch (valueBit) {
    case JXcoord:		symbol = SYM_X; break;
    case JYcoord:		symbol = SYM_Y; break;
    case JWidth:		symbol = SYM_WIDTH; break;
    case JHeight:		symbol = SYM_HEIGHT; break;
    case JDisplayName:		symbol = SYM_DISPLAY; break;
    case JPath:			symbol = SYM_PATH; break;
    case JVisualId:		symbol = SYM_VISUALID; break;
    case JWindowId:		symbol = SYM_WINDOW; break;
    case JMode:			symbol = SYM_MODE; break;
    case JMethod: 		symbol = SYM_METHOD; break;
    case JAudioEnabled:		symbol = SYM_AUDIOENABLE; break;
    case JVideoEnabled:		symbol = SYM_VIDEOENABLE; break;
    case JQFactor:		symbol = SYM_QFACTOR; break;
    case JFrameRate:		symbol = SYM_FRAME_RATE; break;
    case JInTime:		symbol = SYM_INTIME; break;
    case JOutTime:		symbol = SYM_OUTTIME; break;
    case JInFrame:		symbol = SYM_INFRAME; break;
    case JOutFrame:		symbol = SYM_OUTFRAME; break;
    case JTolerance:		symbol = SYM_TOLERANCE; break;
    case JPreloadSize:		symbol = SYM_PRELOADSIZE; break;
    case JAudioDelay:		symbol = SYM_AUDIODELAY; break;
    case JAudioSource:		symbol = SYM_AUDIOSOURCE; break;
    case JFrameFrequency:	symbol = SYM_FRAMEFREQ; break;
    default:			symbol = -1;
    }
    return symbol;
}

static boolean
JpegUnsetWideParam(McsServer server, BitMaskPtr bp)
{
    Msg("JpegUnsetWideParam: Invalid. Must use JpegUnsetParam\n");
    return(False);
}

static boolean
JpegUnsetParam(McsServer server, ULong valueMask)
{
    int i, symbol;
    McsJpegPrivPtr pJpeg = (McsJpegPrivPtr)server;

    pJpeg->assigned &= ~valueMask;
    if (valueMask & JArg)
	pJpeg->u.generic.optional.arg = NULL;
    for (i = 0; i < N_MCS_SYMBOLS; i++) {
	if ((1<<i) & valueMask) {
	    if ((symbol = MaskToSymbol(1<<i)) < 0)
		continue;
	    _McsSendAssign(pJpeg->base.pParse, STR_UNINIT, MCS_TYPE_SYMBOL,
			  (ParseValuePtr)&McsSymbolStrings[ symbol ]);
	}
    }
    _McsSendCommand(pJpeg->base.pParse, STR_CONFIGURE);
    _McsFlushMessages(pJpeg->base.pParse);
    return (True);
}

static McsEventPtr
JpegGetNextEvent(McsServer server)
{
    return(NULL);
}

static boolean
JpegStartOperation(McsServer server)
{
    McsJpegPrivPtr pJpeg = (McsJpegPrivPtr)server;

    if (pJpeg->compressor) {
	if ((pJpeg->assigned & JCRequired) != JCRequired)
	    return(False);
    } else {
	if ((pJpeg->assigned & JDRequired) != JDRequired)
	    return(False);
    }

    if (! JpegFlushParam(pJpeg))
	return (False);
    _McsSendCommand(server->pParse, STR_START);
    _McsFlushMessages(server->pParse);
    return(True);
}

static boolean
JpegStopOperation(McsServer server)
{
    _McsSendCommand(server->pParse, STR_STOP);
    _McsFlushMessages(server->pParse);
    return(True);
}

static void
JpegCloseServer(McsServer server)
{
}

static McsServerClass _JpegCompClass = {
    McsJpegCompressor,
    sizeof(McsJpegPriv),
    JpegCompOpenServer,
    JpegSetParam,
    JpegUnsetParam,
    JpegGetNextEvent,
    JpegStartOperation,
    JpegStopOperation,
    JpegCloseServer
};

static McsServerClass _JpegDecompClass = {
    McsJpegDecompressor,
    sizeof(McsJpegPriv),
    JpegDecompOpenServer,
    JpegSetParam,
    JpegUnsetParam,
    JpegGetNextEvent,
    JpegStartOperation,
    JpegStopOperation,
    JpegCloseServer,
    JpegSetWideParam,
    JpegUnsetWideParam
};

McsServerClassPtr McsJpegCompClass = &_JpegCompClass;
McsServerClassPtr McsJpegDecompClass = &_JpegDecompClass;
