/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	jpegcomp.h - public interface to jpeg compressor/decompressor
 *
 * DESCRIPTION
 *	<overall description of what the header file does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/jpegcomp.h,v 1.12 1993/09/09 02:23:42 toddb Exp $
 */
#ifndef _JPEGCOMP_H_
#define _JPEGCOMP_H_

#include "nvrTypes.h"
#include "nvrTimer.h"

/*
 *	DEFINES
 */
#define SYM_NATIVE	(1<<1)
#define SYM_FAST_X	(2<<1)
#define SYM_DIRECT	(3<<1)

#define SYM_CAPTURE	0
#define SYM_PLAYBACK	1

#define JXcoord			0x0000001
#define JYcoord			0x0000002
#define JWidth			0x0000004
#define JHeight			0x0000008
#define JDisplayName		0x00000010
#define JPath			0x00000020
#define JVisualId		0x00000040
#define JWindowId		0x00000080
#define JMode			0x00000100
#define JMethod			0x00000200
#define JAudioEnabled		0x00000400
#define JVideoEnabled		0x00000800
#define JInputNotify		0x00001000
#define JQFactor		0x00002000 /* comp only */
#define JFrameRate		0x00004000 /* comp, required; decomp, option */
#define JInTime			0x00008000
#define JOutTime		0x00010000
#define JInFrame		0x00020000
#define JOutFrame		0x00040000
#define JTolerance		0x00080000
#define JPreloadSize		0x00100000
#define JAudioDelay		0x00200000
#define JFrameFrequency		0x00400000
#define JAudioSource		0x00800000
#define JArg			0x01000000
#define JPreInit		0x02000000
#define JPostInit		0x04000000

#define JRequired \
    (JXcoord | JYcoord | JWidth | JHeight | JDisplayName | JPath \
   | JVisualId | JWindowId | JMode | JMethod | JAudioEnabled \
   | JVideoEnabled | JInputNotify)

#define JOptional \
    (JInTime | JOutTime | JInFrame | JOutFrame | JTolerance \
   | JPreloadSize | JAudioDelay | JFrameFrequency | JArg | JAudioSource \
   | JPreInit | JPostInit)

#define JDRequired	JRequired
#define JDOptional	(JOptional | JFrameRate)
#define JCRequired	(JRequired | JQFactor | JFrameRate)
#define JCOptional	JOptional
    

/*
 *	TYPEDEFS
 */
typedef struct {
    struct {
	int x, y;
	int width;
	int height;
	char *displayName;
	char *path;
	ULong visualId;
	ULong windowId;
	int mode;		/* capture or playback */
	int method;		/* fastx, native or direct */
	boolean audioEnabled;	/* whether we are capturing/playing audio */
	boolean videoEnabled;	/* whether we are capturing/playing video */
	McsInputNotify inputNotify;/* called to tell about file descriptors  */
	ULong frameRate;	/* ... in (integer) frames per second */
	ULong qFactor;
    } required;

    struct {
	TimeSpec inTime;	/* time to start*/
	TimeSpec outTime;	/* time when we are done */
	long inFrame;		/* specified starting frame */
	long outFrame;		/* specified frame which we *don't* play */
	long tolerance;		/* tolerance of frame lateness (in microsecs) */
	long preloadSize;	/* number of frames to read ahead */
	long audioDelay;	/* delay until start of audio */
	long frameFrequency;	/* frequency of frame events */
	char *audioSource;	/* where to get audio */
	PrivPointer arg;	/* arg to inputNotify */
	char *preInit;		/* string to send to mcs before anything else */
	char *postInit;		/* sent to mcs after init, before commands */
    } optional;
} McsJpegComp, *McsJpegCompPtr;

/*
 *	TYPEDEFS
 */
typedef struct {
    struct {
	int x, y;
	int width;
	int height;
	char *displayName;
	char *path;
	ULong visualId;
	ULong windowId;
	int mode;		/* capture or playback */
	int method;		/* fastx, native or direct */
	boolean audioEnabled;	/* whether we are capturing/playing audio */
	boolean videoEnabled;	/* whether we are capturing/playing video */
	McsInputNotify inputNotify;/* called when events available from server */
    } required;

    struct {
	int frameRate;		/* ... in (integer) frames per second */
	long pad;		/* this makes comp and decomp isometric */
	TimeSpec inTime;	/* time to start*/
	TimeSpec outTime;	/* time when we are done */
	long inFrame;		/* specified starting frame */
	long outFrame;		/* specified frame which we *don't* play */
	long tolerance;		/* tolerance of frame lateness (in microsecs) */
	long preloadSize;	/* number of frames to read ahead */
	long audioDelay;	/* delay until start of audio */
	long frameFrequency;	/* frequency of frame events */
	char *audioSource;	/* where to get audio */
	PrivPointer arg;	/* arg to inputNotify */
	char *preInit;		/* string to send to mcs before anything else */
	char *postInit;		/* sent to mcs after init, before commands */
    } optional;
} McsJpegDecomp, *McsJpegDecompPtr;

/*
 *	EXTERNS
 */
CExternal McsServerClassPtr McsJpegCompClass;
CExternal McsServerClassPtr McsJpegDecompClass;

#endif /* _JPEGCOMP_H_ */
