/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mcs-server.h -  public, general interface definitions for starting
 *			servers
 *
 * DESCRIPTION
 *	<overall description of what the header file does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mcs-server.h,v 1.25 1994/06/18 06:19:23 toddb Exp $
 */
#ifndef _MCS_SERVER_H_
#define _MCS_SERVER_H_

#include "nvrTypes.h"
#include "nvrBitTest.h"

/*
 *	DEFINES
 */

/*
 * Event value types.
 */
typedef enum {
    MCS_TYPE_NONE,
    MCS_TYPE_INT,
    MCS_TYPE_BOOL,
    MCS_TYPE_STRING,
    MCS_TYPE_REAL,
    MCS_TYPE_SYMBOL	/* used internally */
} McsTypeValues;

/*
 * Event types
 * Note: the order of these symbols must match the order of McsEventStrings[]
 * defined in protoparse.y exactly.
 */
typedef enum {
    MCS_ACTION_OK,

    MCS_BAD_VALUE,
    MCS_BAD_WINDOW,
    MCS_BAD_DISPLAY,
    MCS_BAD_VISUAL,
    MCS_BAD_ASSIGNMENT,
    MCS_BAD_CONFIGURE,
    MCS_BAD_HARDWARE,
    MCS_BAD_FILE,
    MCS_BAD_WRITE,
    MCS_BAD_READ,
    MCS_BAD_RUNNING,
    MCS_BAD_ALLOC,
    MCS_BAD_SYNTAX,
    MCS_BAD_INTIME,
    MCS_BAD_AUDIO,
    MCS_BAD_PRELOAD,
    MCS_BAD_IMPLEMENTATION,
    MCS_BAD_MATCH,
    MCS_BAD_LICENSE,
    MCS_BAD_GEOMETRY,

    /* parse symbol events */
    MCS_EVENT_REJECT_ASSIGN,
    MCS_EVENT_REJECT_COMMAND,
    MCS_EVENT_LOADING,
    MCS_EVENT_STOPPED,
    MCS_EVENT_PAUSED,
    MCS_EVENT_RUNNING,
    MCS_EVENT_INFO,
    MCS_EVENT_READY,
    MCS_EVENT_AUDIOBUSY,
    MCS_EVENT_FRAME,
    MCS_EVENT_ICOMPRATIO,
    MCS_EVENT_BCOMPRATIO,
    MCS_EVENT_PCOMPRATIO,
    MCS_EVENT_COMPRATIO,
    MCS_EVENT_BITRATE,
    MCS_EVENT_SKIPPED,
    MCS_EVENT_LATEFRAMES,
    MCS_EVENT_FRAMERATE,
    MCS_EVENT_QUANTIZER,
    MCS_EVENT_SERVER_EXIT,

    MCS_EVENTS		/* number of events: must be last */
} McsEventConstants;

/*
 *	TYPEDEFS
 */
typedef struct _McsServer *McsServer;
typedef struct _McsServerClass *McsServerClassPtr;

typedef struct _McsEvent {
    UShort eventType;
    UShort valueType; 
    union _uMcsEvent {
	char *string;
	long number;
	double real;
    } u;
} McsEvent, *McsEventPtr;

typedef void (*McsInputNotify)(
#if NeedFunctionPrototypes
    McsServer,		/* server */
    int,		/* file descriptor */
    boolean,		/* true=>add, false=>delete */
    PrivPointer		/* optional argument */
#endif
);

/*
 *	EXTERNS
 */

CExternal McsServer McsOpenServer(
#if NeedFunctionPrototypes
    McsServerClassPtr		/* service name */
#endif
);

CExternal boolean McsSetParam(
#if NeedFunctionPrototypes
    McsServer,			/* handle to server */
    PrivPointer,		/* pointer to server-specific param struct */
    ULong			/* value mask */
#endif
);

CExternal boolean McsSetWideParam(
#if NeedFunctionPrototypes
    McsServer,			/* handle to server */
    PrivPointer,		/* pointer to server-specific param struct */
    BitMaskPtr			/* value mask */
#endif
);

CExternal boolean McsUnsetParam(
#if NeedFunctionPrototypes
    McsServer,			/* handle to server */
    ULong			/* value mask */
#endif
);

CExternal boolean McsUnsetWideParam(
#if NeedFunctionPrototypes
    McsServer,			/* handle to server */
    BitMaskPtr			/* value mask */
#endif
);

CExternal McsEventPtr McsGetNextEvent(
#if NeedFunctionPrototypes
    McsServer			/* handle to server */
#endif
);

CExternal void McsFreeEvent(
#if NeedFunctionPrototypes
    McsServer,			/* handle to server */
    McsEventPtr			/* event to be freed */
#endif
);

CExternal boolean McsStartOperation(
#if NeedFunctionPrototypes
    McsServer			/* handle to server */
#endif
);

CExternal boolean McsStopOperation(
#if NeedFunctionPrototypes
    McsServer			/* handle to server */
#endif
);

CExternal void McsCloseServer(
#if NeedFunctionPrototypes
    McsServer			/* handle to server */
#endif
);

CExternal void McsFormatEvent(
#if NeedFunctionPrototypes
    McsServer,			/* handle to server */
    McsEventPtr,		/* event to be formated */
    char *			/* where to put the human-readable event */
#endif
);

CExternal void McsDebugEnable(
#if NeedFunctionPrototypes
    McsServer,			/* handle to server */
    boolean,			/* show events from server on stderr */
    boolean			/* show commands to server on stderr */
#endif
);
extern char *McsSymbolStrings[];
extern char *McsEventStrings[];

#endif /* _MCS_SERVER_H_ */
