/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mcs-serverP.h - internal structures for mcs library routines
 *
 * DESCRIPTION
 *	<overall description of what the header file does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mcs-serverP.h,v 1.14 1994/08/30 15:37:45 toddb Exp $
 */
#ifndef _MCS_SERVER_P_H_
#define _MCS_SERVER_P_H_

#include "mcs-server.h"
#include "protoparseP.h"
#include "nvrQueue.h"
#include "nvrBitTest.h"

/*
 *	DEFINES
 */

#define	MCS_EVENT_QINCREMENT	20

#define McsMpegCompressor		"mpeg audio/video compressor"
#define McsMpegDecompressor		"mpeg audio/video decompressor"
#define McsJpegCompressor		"jpeg video/audio compressor"
#define McsJpegDecompressor		"jpeg video/audio decompressor"

/*
 *	TYPEDEFS
 */

/*
 * The Event queue
 */
typedef struct {
    Queue undelivered;		/* circular queue of events not yet delivered */
    Queue delivered;		/* circular queue of unused events */
} McsEventQueue, *McsEventQueuePtr;

/*
 * A class record of this type is declared by all services at compile time
 */
typedef struct _McsServerClass {
    char *serviceName;
    int objectSize;
    boolean (*openServer)(McsServer);
    boolean (*setParam)(McsServer, PrivPointer, ULong);
    boolean (*unsetParam)(McsServer, ULong);
    McsEventPtr (*getNextEvent)(McsServer);
    boolean (*startOperation)(McsServer);
    boolean (*stopOperation)(McsServer);
    void (*closeServer)(McsServer);
    boolean (*setWideParam)(McsServer, PrivPointer, BitMaskPtr);
    boolean (*unsetWideParam)(McsServer, BitMaskPtr);
} McsServerClass;

/*
 * This is the base server instance record
 */
typedef struct _McsServer {
    McsServerClassPtr pClass;	/* pointer to class record */

    McsEventQueue eventQ;	/* event queue */
    ParseInfoPtr pParse;	/* parser information */

    /* server-specific (sub-class) stuff follows this */

} McsServerRec;

/*
 * Prototypes for internally used functions.
 */
CExternal void _McsInputNotify(
    ParseInfoPtr		/* parser info */
);

CExternal void _McsEofNotify(
    ParseInfoPtr		/* parser info */
);

CExternal void _McsInitEvents(
    McsServer
);

CExternal void _McsFreeEvents(
    McsServer
);

CExternal void _McsEventNotify(
    ParseInfoPtr,		/* parser info */
    int,			/* event type */
    int,			/* value type */
    ParseValuePtr		/* value */
);

CExternal void _McsFreeEvent(
    McsServer,			/* server */
    McsEventPtr			/* event to free */
);

CExternal void _McsSendCommand(
    ParseInfoPtr,		/* server */
    char *			/* command name */
);

CExternal void _McsSendAssign(
    ParseInfoPtr,		/* server */
    char *,			/* symbol string */
    int,			/* value type */
    ParseValuePtr		/* value */
);

CExternal void _McsOutputMessage(
    ParseInfoPtr,		/* parser info */
    char *,			/* format */
    ...
);

CExternal void _McsFlushMessages(
    ParseInfoPtr		/* parser info */
);

CExternal void AddConnectionIO(
    ParseInfoPtr pParse,
    ParseConfigPtr pConfig
);

#endif /* _MCS_SERVER_P_H_ */
