/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mcspub.c - definitions of public-access routines
 *
 * DESCRIPTION
 *	<overall description of what the c code does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mcspub.c,v 1.19 1994/07/24 18:03:41 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "mcs-serverP.h"
#include "osio.h"
#include "nvrString.h"
#include "nvrAlloc.h"

McsServer
McsOpenServer(McsServerClassPtr serverClass)
{
    McsServer new;

    new = (McsServer) InsecureAlloc(serverClass->objectSize);
    if (new == NULL)
	return(NULL);
    memset((char *)new, 0, serverClass->objectSize);

    new->pClass = serverClass;
    _McsInitEvents(new);
    if (! (*new->pClass->openServer)(new)) {
	_McsFreeEvents(new);
	free((char *)new);
	return(NULL);
    }

    return ((McsServer) new);
}

boolean
McsUnsetParam(McsServer server, ULong valueMask)
{
    return( (*server->pClass->unsetParam)(server, valueMask) );
}

boolean
McsSetParam(McsServer server, PrivPointer params, ULong valueMask)
{
    return( (*server->pClass->setParam)(server, params, valueMask) );
}

boolean
McsUnsetWideParam(McsServer server, BitMaskPtr valueMask)
{
    return( (*server->pClass->unsetWideParam)(server, valueMask) );
}

boolean
McsSetWideParam(McsServer server, PrivPointer params, BitMaskPtr valueMask)
{
    return( (*server->pClass->setWideParam)(server, params, valueMask) );
}

boolean
McsStartOperation(McsServer server)
{
    return( (*server->pClass->startOperation)(server) );
}

McsEventPtr
McsGetNextEvent(McsServer server)
{
    QueuePtr q = &server->eventQ.undelivered;
    McsEventPtr pev;

    if (QueueEmpty(q)) {
	server->pParse->inputWaiting = True;	/* lie to it */
	HandleConnectionInput(server->pParse);

	if (QueueEmpty(q))
	    return (NULL);
    }

    pev = TailOfQueue(q, McsEventPtr);
    RemoveQueueTail(q);
    return (pev);
}

void
McsFreeEvent(McsServer server, McsEventPtr pev)
{
    QueuePtr q = &server->eventQ.delivered;

    if (QueueFull(q))
	Fatal("McsFreeEvent: delivered queue full\n");
    InsertQueueHead(q, pev);
}

boolean
McsStopOperation(McsServer server)
{
    return( (*server->pClass->stopOperation)(server) );
}

void
McsCloseServer(McsServer server)
{
    (*server->pClass->closeServer)(server);
}

void McsFormatEvent(McsServer server, McsEventPtr evp, char *buf)
{
    sprintf(buf, "%s", McsEventStrings[ evp->eventType ]);
    buf += strlen(buf);
    switch(evp->valueType) {
    case MCS_TYPE_BOOL:
        sprintf(buf, ": %s", evp->u.number ? "true" : "false");
        break;
    case MCS_TYPE_INT:
        sprintf(buf, ": %ld", evp->u.number);
        break;
    case MCS_TYPE_REAL:
        sprintf(buf, ": %f", evp->u.real);
        break;
    case MCS_TYPE_STRING:
        sprintf(buf, ": \"%s\"", evp->u.string);
        break;
    case MCS_TYPE_NONE:
        break;
    default:
        sprintf(buf, ": type?\n");
        break;
    }
}

void
McsDebugEnable(McsServer server, boolean showEvents, boolean showCommands)
{
    server->pParse->showInput = showEvents;
    server->pParse->showOutput = showCommands;
}
