/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mcsutil.c - utilities for dealing with mcs on client side.
 *
 * DESCRIPTION
 *	<overall description of what the c code does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mcsutil.c,v 1.11 1994/06/18 06:24:59 toddb Exp $
 */
#define NeedFunctionPrototypes 1

#include "nvrString.h"
#include "nvrAlloc.h"
#include "mcs-serverP.h"

void
_McsInitEvents(McsServer server)
{
    McsEventPtr pev;
    QueuePtr q;

    if (server->eventQ.delivered.array)	/* already done */
	return;

    InitQueue(q = &server->eventQ.delivered, MCS_EVENT_QINCREMENT);
    InitQueue(&server->eventQ.undelivered, MCS_EVENT_QINCREMENT);
    while (! QueueFull(q)) {
	pev = (McsEventPtr) SecureAlloc(sizeof(McsEvent));
	memset((char *)pev, 0, sizeof(McsEvent));
	InsertQueueHead(q, pev);
    }
}

void
_McsFreeEvents(McsServer server)
{
    McsEventPtr pev;
    QueuePtr q;

    /*
     * clear the undelivered queue
     */
    q = &server->eventQ.undelivered;
    while (! QueueEmpty(q)) {
	pev = TailOfQueue(q, McsEventPtr);
	RemoveQueueTail(q);
	McsFreeEvent(server, pev);
    }
    free((char *)q->array);

    /*
     * Free the events
     */
    q = &server->eventQ.delivered;
    if (! QueueFull(q))
	Msg("Warning: some events delivered for %s have not been freed\n",
	    server->pClass->serviceName);
    while (! QueueEmpty(q)) {
	pev = TailOfQueue(q, McsEventPtr);
	RemoveQueueTail(q);
	if (pev->valueType == MCS_TYPE_STRING)
	    free((void *)pev->u.string);
	free((void *)pev);
    }
    free((void *)q->array);
}

void
_McsInputNotify(ParseInfoPtr pParse)
{
    /* nothing to do */
}

void
_McsEofNotify(ParseInfoPtr pParse)
{
    ParseEventNotifier eventNotify = pParse->config.u.client.eventNotify;

    (*eventNotify)(pParse,
		   MCS_EVENT_SERVER_EXIT,
		   MCS_TYPE_NONE,
		   (ParseValue *)NULL);
}

/*
 * The server-library writer should use this for the event notify/queing
 * operation by the parser.
 */
void
_McsEventNotify(ParseInfoPtr pParse,
	        int eventType,
	        int valueType,
	        ParseValuePtr value)
{
    int i;
    McsServer server = (McsServer)GetParsePrivateData(pParse);
    QueuePtr q = &server->eventQ.delivered;
    QueuePtr qun = &server->eventQ.undelivered;
    McsEventPtr pev;

    if (QueueEmpty(q)) {
	StretchQueue(qun, qun->nitems + MCS_EVENT_QINCREMENT);
	StretchQueue(q, q->nitems + MCS_EVENT_QINCREMENT);
	for (i = 0; i < MCS_EVENT_QINCREMENT; i++) {
	    pev = (McsEventPtr) SecureAlloc(sizeof(McsEvent));
	    memset((char *)pev, 0, sizeof(McsEvent));
	    InsertQueueHead(q, pev);
	}
    }

    /*
     * Get a new event
     */
    pev = TailOfQueue(q, McsEventPtr);
    RemoveQueueTail(q);

    /*
     * fill in the event
     */
    pev->eventType = eventType;
    if (pev->valueType == MCS_TYPE_STRING)
	free((void *)pev->u.string);
    switch(pev->valueType = valueType) {
    case MCS_TYPE_INT:    pev->u.number = value->intvalue; break;
    case MCS_TYPE_REAL:   pev->u.real = value->real; break;
    case MCS_TYPE_STRING: pev->u.string = value->string; break;
    case MCS_TYPE_NONE:   break;
    }
    InsertQueueHead(qun, pev);
}
