/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpegcomp.h - user-visible portion of filter/mpeg compressor
 *
 * DESCRIPTION
 *	This is the interface that is visible to the user of the
 *	filter/mpeg compressor pair.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpegcomp.h,v 1.24 1995/08/11 00:13:48 russ Exp $
 */
#ifndef _MPEGCOMP_H_
#define _MPEGCOMP_H_

#include "nvrImage.h"

/*
 *	DEFINES
 */

/*
 * Output frame rates
 */
#define MCVAL_RATE_23_976	23.976
#define	MCVAL_RATE_24		24.0
#define	MCVAL_RATE_25		25.0
#define	MCVAL_RATE_29_97	29.97
#define	MCVAL_RATE_30		30.0
#define	MCVAL_RATE_50		50.0
#define	MCVAL_RATE_59_94	59.94
#define	MCVAL_RATE_60		60.0

/*
 * NVR private frame rates
 */
#define	MCVAL_RATE_5		5.0
#define	MCVAL_RATE_6		6.0
#define	MCVAL_RATE_10		10.0
#define	MCVAL_RATE_15		15.0
#define	MCVAL_RATE_20		20.0

/*
 * Quantization levels.
 */
#define MCVAL_MINQUANT		1
#define MCVAL_MAXQUANT		31

/*
 * Motion estimation algorithms
 */
typedef enum {
    MCVAL_EXMAD_FULLPEL = 0,
    MCVAL_JAIN_N_JAIN = 1,
    /* external = 2*/
    MCVAL_MPEXMAD_FULLPEL = 3
/*     MCVAL_EXMAD_HALFPEL = 4 */
} MestAlgorithms;

/*
 * Termination methods.
 */
#define	MCVAL_TERM_IMMED	0
#define	MCVAL_TERM_ASAP		1

/*
 * value masks
 */
/* Compression Parameter mask bits */
#define	MCBitrate		0
#define MCBufferSize		1
#define MCInitFullness		2
#define MCInitIcomplex		3
#define MCInitPcomplex		4
#define MCInitBcomplex		5
#define MCSceneEnds		6
#define MCSliceStart		7
#define MCKbitIcomplex		8
#define MCKbitPcomplex		9
#define MCKbitBcomplex		10
#define	MCPpicture		11
#define MCBpicture		12
#define	MCMotion		13
#define	MCMotionRadius		14
#define MCCenteredQscale	16
#define MCFixedQscale		17
#define MCPictureRate		20
#define MCNumProcessors		21
#define MCReactionParam		22
#define MCSpatialMaskGain	23
#define MCVbvZero		24

/* Input Source mask bits */
#define MCVInputSample		30
#define MCVInputColor		31
#define MCVInputName		32
#define MCVInputPrefix		33
#define MCVInputSuffix		34
#define MCVInputNDigits		35
#define MCVInputStart		36
#define MCVInputStop		37
#define MCVResampleFps		38

/* Filter mask bits */
#define MCClipX			50
#define MCClipY			51
#define MCClipWidth		52
#define MCClipHeight		53
#define MCResizeWidth		54
#define MCResizeHeight		55
#define MCResizeMethod		56
#if 0
#define MCMaintainAspect	57
#endif /* 0 */
#define MCDeinterlaceWidth	58
#define MCSmoothLevel		59


/* Misc mask bits */
#define	MCTermMethod		80
#define MCSystemStream		81

/* Debug mask bits */
#define	MPEG_DEBUG		126
#define	MPEG_FULL_DEBUG		127

/*
 *	TYPEDEFS
 */
typedef struct {
    struct {
	ULong	width;
	ULong	height;
	char	*outputfile;
	McsInputNotify inputNotify;
	ImageFileType inputImageType;
	SampleType inputSampleType;
	ColorSpace inputColorSpace;
	boolean	interlaced;
    } required;
    
    struct {
	/* compression variables */
	ULong	bitrate;
	ULong	bufferSize;
	float	initFullness;
	float	initIcomplex;
	float	initPcomplex;
	float	initBcomplex;
	ULong	sceneEnds;
	ULong	sliceStart;
	float	kbitIcomplex;
	float	kbitPcomplex;
	float	kbitBcomplex;
	ULong	nppictures;
	ULong	nbpictures;
	ULong	motionAlgorithm;
	ULong	motionRadius;
	float	centeredQuantLevel;
	ULong	fixedQuantLevel;
	float	pictureRate;
	ULong	numProcessors;
	float	reactionParam;
	float	spatialMaskGain;
	float	vbvZero;

	/* input variables */
	char	*vInputName;	/* required (or the folowing 4) */

	char	*vInputPrefix;	/* required or above */
	char	*vInputSuffix;	/* required or above */
	ULong	vInputNDigits;	/* required or above */
	ULong	vInputStart;	/* required or above */
	ULong	vInputStop;	/* required or above */

	/* filter variables */
	ULong	clipX;
	ULong	clipY;
	ULong	clipWidth;	/* !MCClipWdith => dont */
	ULong	clipHeight;
	ULong	resizeWidth;	/* !MCResizeWidth => dont */
	ULong	resizeHeight;
	ResampleMethodName
		resizeMethod;
	ULong	deinterlaceWidth;	/* !MCDeinterlaceWdidth => dont */
	ULong	smoothLevel;		/* !MCSmoothLevel => dont */
	float	resampleFps;
	
	/* misc variables */
	ULong	terminationMethod;
    } optional;
} MpegCompParams, *MpegCompParamsPtr;

/*
 *	EXTERNS
 */

extern	McsServerClassPtr MpegCompClass; 

#endif /* _MPEGCOMP */
