/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpegdecomp.c - library interface for mpeg decompressor
 *
 * DESCRIPTION
 *	<overall description of what the c code does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpegdecomp.c,v 1.12 1994/06/27 17:01:32 russ Exp $
 */
#define NeedFunctionPrototypes 1
#include "process.h"
#include "nvrString.h"
#include "protoparseP.h"
#include "mcs-serverP.h"
#include "mpegdecompP.h"

#define MpegServerName	"vmpegdecomp"

static void
EofNotify(ParseInfoPtr pParse)
{
    McsMpegDecPrivPtr pMpeg = (McsMpegDecPrivPtr)GetParsePrivateData(pParse);
    McsInputNotify inputNotify = pMpeg->param.inputNotify;
    _McsEofNotify(pParse);
    
    (*inputNotify)((McsServer)pMpeg, pMpeg->io.inFd,
		   False, pMpeg->param.arg);
    (*pParse->config.inputNotify)(pParse);
}

static boolean
MpegDecompOpenServer(McsServer server)
{
    McsMpegDecPrivPtr pMpeg = (McsMpegDecPrivPtr)server;
    Argvec serverArgs;
    ParseConfig config;

    memset((char *)&serverArgs, 0, sizeof(serverArgs));
    serverArgs.path = MpegServerName;
    AddArgv(&serverArgs, serverArgs.path, False);
    pMpeg->io = PipeToAndFrom(&serverArgs, (VoidFuncPtr) NULL);

    config.inFileHandle = pMpeg->io.inFd;
    config.outFileHandle = pMpeg->io.outFd;
    config.inputNotify = _McsInputNotify;
    config.eofNotify = EofNotify;
    config.outputNotify = NULL;	/* this is optional */
    config.pointer = (PrivPointer)pMpeg;
    config.server = False;
    config.asynchronous = False;
    config.u.client.eventNotify = _McsEventNotify;
    pMpeg->base.pParse = InitParsedConnection(&config);
    return (True);
}

static boolean
MpegFlushParam(McsMpegDecPrivPtr pMpeg)
{
    ULong valueMask = pMpeg->notFlushed;
    boolean success = True;

    if (valueMask == 0)
	return(True);
    if (valueMask & MDXcoord)
	_McsSendAssign(pMpeg->base.pParse, STR_X, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.x);
    if (valueMask & MDYcoord)
	_McsSendAssign(pMpeg->base.pParse, STR_Y, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.y);
    if (valueMask & MDWidth)
	_McsSendAssign(pMpeg->base.pParse, STR_WIDTH, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.width);
    if (valueMask & MDHeight)
	_McsSendAssign(pMpeg->base.pParse, STR_HEIGHT, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.height);
    if (valueMask & MDDisplayName)
	_McsSendAssign(pMpeg->base.pParse, STR_DISPLAY, MCS_TYPE_STRING,
		      (ParseValuePtr)&pMpeg->param.displayName);
    if (valueMask & MDWindowId)
	_McsSendAssign(pMpeg->base.pParse, STR_WINDOW, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.windowId);
    if (valueMask & MDColor) {
	char *color;

	switch(pMpeg->param.color) {
	case SYM_GRAY1:    color = STR_GRAY1; break;
	case SYM_GRAY7:    color = STR_GRAY7; break;
	case SYM_GRAY8:    color = STR_GRAY8; break;
	case SYM_RGB_232:  color = STR_RGB_232; break;
	case SYM_RGB_332:  color = STR_RGB_332; break;
	case SYM_RGB_888:  color = STR_RGB_888; break;
	case SYM_RGB_444:  color = STR_RGB_444; break;
	case SYM_RGB_555:  color = STR_RGB_555; break;
	case SYM_RGB_C555:  color = STR_RGB_C555; break;
	case SYM_RGB_C666:  color = STR_RGB_C666; break;
	default:	     success = False; break;
	}
	if (success)
	    _McsSendAssign(pMpeg->base.pParse, STR_COLOR, MCS_TYPE_SYMBOL,
			  (ParseValuePtr)&color);
    }
    if (valueMask & MDDestType) {
	char *destType;

	switch(pMpeg->param.destType) {
	case SYM_DPY_TO_WIN:  destType = STR_DPY_TO_WIN; break;
	case SYM_DPY_TO_FB:   destType = STR_DPY_TO_FB; break;
	case SYM_YCTOFILE:    destType = STR_YCTOFILE; break;
	case SYM_DPY_TO_YCFB: destType = STR_DPY_TO_YCFB; break;
	default:	   success = False; break;
	}
	if (success)
	    _McsSendAssign(pMpeg->base.pParse, STR_DESTTYPE, MCS_TYPE_SYMBOL,
			  (ParseValuePtr)&destType);
    }
    if (valueMask & MDAudioEnabled)
	_McsSendAssign(pMpeg->base.pParse, STR_AUDIOENABLE, MCS_TYPE_BOOL,
		      (ParseValuePtr)&pMpeg->param.audioEnabled);
    if (valueMask & MDVideoEnabled)
	_McsSendAssign(pMpeg->base.pParse, STR_VIDEOENABLE, MCS_TYPE_BOOL,
		      (ParseValuePtr)&pMpeg->param.videoEnabled);

    if (valueMask & MDFrameRate)
	_McsSendAssign(pMpeg->base.pParse, STR_FRAME_RATE, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.frameRate);
    if (valueMask & MDInTime)
	_McsSendAssign(pMpeg->base.pParse, STR_INTIME, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.inTime);
    if (valueMask & MDOutTime)
	_McsSendAssign(pMpeg->base.pParse, STR_OUTTIME, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.outTime);
    if (valueMask & MDInFrame)
	_McsSendAssign(pMpeg->base.pParse, STR_INFRAME, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.inFrame);
    if (valueMask & MDOutFrame)
	_McsSendAssign(pMpeg->base.pParse, STR_OUTFRAME, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.outFrame);
    if (valueMask & MDTolerance)
	_McsSendAssign(pMpeg->base.pParse, STR_TOLERANCE, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.tolerance);
    if (valueMask & MDPreloadSize)
	_McsSendAssign(pMpeg->base.pParse, STR_PRELOADSIZE, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.preloadSize);
    if (valueMask & MDAudioDelay)
	_McsSendAssign(pMpeg->base.pParse, STR_AUDIODELAY, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.audioDelay);
    if (valueMask & MDAudioSource)
	_McsSendAssign(pMpeg->base.pParse, STR_AUDIOSOURCE, MCS_TYPE_STRING,
		      (ParseValuePtr)&pMpeg->param.audioSource);
    if (valueMask & MDFrameFrequency)
	_McsSendAssign(pMpeg->base.pParse, STR_FRAMEFREQ, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.frameFrequency);
    if (valueMask & MDInPath)
	_McsSendAssign(pMpeg->base.pParse, STR_INPATH, MCS_TYPE_STRING,
		      (ParseValuePtr)&pMpeg->param.inPath);
    if (valueMask & MDOutPath)
	_McsSendAssign(pMpeg->base.pParse, STR_OUTPATH, MCS_TYPE_STRING,
		      (ParseValuePtr)&pMpeg->param.outPath);
    if (valueMask & MDAudioInPath)
	_McsSendAssign(pMpeg->base.pParse, STR_AUDIOINPATH, MCS_TYPE_STRING,
		      (ParseValuePtr)&pMpeg->param.aInPath);
    if (valueMask & MDBitServer)
	_McsSendAssign(pMpeg->base.pParse, STR_BITSERVER, MCS_TYPE_STRING,
		      (ParseValuePtr)&pMpeg->param.bitServer);
    if (valueMask & MDAudioQuality)
	_McsSendAssign(pMpeg->base.pParse, STR_AUDIOQUALITY, MCS_TYPE_INT,
		      (ParseValuePtr)&pMpeg->param.audioQuality);
    if (valueMask & MDPostInit)
	_McsOutputMessage(pMpeg->base.pParse, "%s;",
		      (ParseValuePtr)pMpeg->param.postInit);
    pMpeg->notFlushed = 0;
    _McsSendCommand(pMpeg->base.pParse, STR_CONFIGURE);
    _McsFlushMessages(pMpeg->base.pParse);
    return (True);
}

static boolean
MpegSetWideParam(McsServer server, PrivPointer params, BitMaskPtr bp)
{
    Msg("MpegSetWideParam: Invalid. Must use MpegSetParam\n");
    return(False);
}

static boolean
MpegSetParam(McsServer server, PrivPointer params, ULong valueMask)
{
    McsMpegDecompPtr mparam = (McsMpegDecompPtr)params;
    McsMpegDecPrivPtr pMpeg = (McsMpegDecPrivPtr)server;

    if (valueMask & MDXcoord)
	pMpeg->param.x = mparam->x;
    if (valueMask & MDYcoord)
	pMpeg->param.y = mparam->y;
    if (valueMask & MDWidth)
	pMpeg->param.width = mparam->width;
    if (valueMask & MDHeight)
	pMpeg->param.height = mparam->height;
    if (valueMask & MDDisplayName)
	pMpeg->param.displayName = mparam->displayName;
    if (valueMask & MDWindowId)
	pMpeg->param.windowId = mparam->windowId;
    if (valueMask & MDDestType)
	pMpeg->param.destType = mparam->destType;
    if (valueMask & MDColor)
	pMpeg->param.color = mparam->color;
    if (valueMask & MDAudioEnabled)
	pMpeg->param.audioEnabled = mparam->audioEnabled;
    if (valueMask & MDVideoEnabled)
	pMpeg->param.videoEnabled = mparam->videoEnabled;
    if (valueMask & MDInputNotify)
	pMpeg->param.inputNotify = mparam->inputNotify;
    if (valueMask & MDInTime)
	pMpeg->param.inTime = mparam->inTime;
    if (valueMask & MDOutTime)
	pMpeg->param.outTime = mparam->outTime;
    if (valueMask & MDInFrame)
	pMpeg->param.inFrame = mparam->inFrame;
    if (valueMask & MDOutFrame)
	pMpeg->param.outFrame = mparam->outFrame;
    if (valueMask & MDTolerance)
	pMpeg->param.tolerance = mparam->tolerance;
    if (valueMask & MDPreloadSize)
	pMpeg->param.preloadSize = mparam->preloadSize;
    if (valueMask & MDAudioDelay)
	pMpeg->param.audioDelay = mparam->audioDelay;
    if (valueMask & MDFrameFrequency)
	pMpeg->param.frameFrequency = mparam->frameFrequency;
    if (valueMask & MDAudioSource)
	pMpeg->param.audioSource = mparam->audioSource;
    if (valueMask & MDArg)
	pMpeg->param.arg = mparam->arg;
    if (valueMask & MDInPath)
	pMpeg->param.inPath = mparam->inPath;
    if (valueMask & MDOutPath)
	pMpeg->param.outPath = mparam->outPath;
    if (valueMask & MDAudioInPath)
	pMpeg->param.aInPath = mparam->aInPath;
    if (valueMask & MDBitServer)
	pMpeg->param.bitServer = mparam->bitServer;
    if (valueMask & MDAudioQuality)
	pMpeg->param.audioQuality = mparam->audioQuality;
    if (valueMask & MDPreInit) {
	pMpeg->param.preInit = mparam->preInit;
	_McsOutputMessage(pMpeg->base.pParse, "%s;", mparam->preInit);
    }
    if (valueMask & MDPostInit)
	pMpeg->param.postInit = mparam->postInit;

    if ((pMpeg->assigned & MDInputNotify) == 0 && (valueMask & MDInputNotify)) {
	McsInputNotify inputNotify = pMpeg->param.inputNotify;
	(*inputNotify)(server, pMpeg->io.inFd,
		       True, pMpeg->param.arg);
    }

    pMpeg->assigned |= valueMask;
    pMpeg->notFlushed |= valueMask;

    /*
     * Flush only if all the required stuff is in place.
     */
    if ((pMpeg->assigned & MDDestType) == 0)
	return(True);

    switch(pMpeg->param.destType) {
    case SYM_DPY_TO_YCFB:
	if ((pMpeg->assigned & MDFileRequired) != MDFileRequired)
	    return(True);
	break;
    case SYM_YCTOFILE:
	if ((pMpeg->assigned & MDVideoFbRequired) != MDVideoFbRequired)
	    return(True);
	break;
    case SYM_DPY_TO_WIN:
    case SYM_DPY_TO_FB:
	if ((pMpeg->assigned & MDScreenRequired) != MDScreenRequired)
	    return(True);
	break;
    }

    return( MpegFlushParam(pMpeg) );
}

static int
MaskToSymbol(int valueBit)
{
    int symbol;

    switch (valueBit) {
    case MDXcoord:		symbol = SYM_X; break;
    case MDYcoord:		symbol = SYM_Y; break;
    case MDWidth:		symbol = SYM_WIDTH; break;
    case MDHeight:		symbol = SYM_HEIGHT; break;
    case MDDisplayName:		symbol = SYM_DISPLAY; break;
    case MDInPath:		symbol = SYM_INPATH; break;
    case MDAudioInPath:		symbol = SYM_AUDIOINPATH; break;
    case MDBitServer:		symbol = SYM_BITSERVER; break;
    case MDOutPath:		symbol = SYM_OUTPATH; break;
    case MDWindowId:		symbol = SYM_WINDOW; break;
    case MDColor: 		symbol = SYM_COLOR; break;
    case MDDestType: 		symbol = SYM_DESTTYPE; break;
    case MDAudioEnabled:	symbol = SYM_AUDIOENABLE; break;
    case MDVideoEnabled:	symbol = SYM_VIDEOENABLE; break;
    case MDFrameRate:		symbol = SYM_FRAME_RATE; break;
    case MDInTime:		symbol = SYM_INTIME; break;
    case MDOutTime:		symbol = SYM_OUTTIME; break;
    case MDInFrame:		symbol = SYM_INFRAME; break;
    case MDOutFrame:		symbol = SYM_OUTFRAME; break;
    case MDTolerance:		symbol = SYM_TOLERANCE; break;
    case MDPreloadSize:		symbol = SYM_PRELOADSIZE; break;
    case MDAudioDelay:		symbol = SYM_AUDIODELAY; break;
    case MDAudioSource:		symbol = SYM_AUDIOSOURCE; break;
    case MDFrameFrequency:	symbol = SYM_FRAMEFREQ; break;
    case MDAudioQuality:	symbol = SYM_AUDIOQUALITY; break;
    default:			symbol = -1;
    }
    return symbol;
}

static boolean
MpegUnsetWideParam(McsServer server, BitMaskPtr bp)
{
    Msg("MpegUnsetWideParam: Invalid. Must use MpegUnsetParam\n");
    return(False);
}

static boolean
MpegUnsetParam(McsServer server, ULong valueMask)
{
    int i, symbol;
    McsMpegDecPrivPtr pMpeg = (McsMpegDecPrivPtr)server;

    pMpeg->assigned &= ~valueMask;
    if (valueMask & MDArg)
	pMpeg->param.arg = NULL;
    for (i = 0; i < N_MCS_SYMBOLS; i++) {
	if ((1<<i) & valueMask) {
	    if ((symbol = MaskToSymbol(1<<i)) < 0)
		continue;
	    _McsSendAssign(pMpeg->base.pParse, STR_UNINIT, MCS_TYPE_SYMBOL,
			  (ParseValuePtr)&McsSymbolStrings[ symbol ]);
	}
    }
    _McsSendCommand(pMpeg->base.pParse, STR_CONFIGURE);
    _McsFlushMessages(pMpeg->base.pParse);
    return (True);
}

static McsEventPtr
MpegGetNextEvent(McsServer server)
{
    return(NULL);
}

static boolean
MpegStartOperation(McsServer server)
{
    McsMpegDecPrivPtr pMpeg = (McsMpegDecPrivPtr)server;

    if ((pMpeg->assigned & MDDestType) == 0)
	return(False);

    switch(pMpeg->param.destType) {
    case SYM_YCTOFILE:
	if ((pMpeg->assigned & MDFileRequired) != MDFileRequired)
	    return(False);
	break;
    case SYM_DPY_TO_YCFB:
	if ((pMpeg->assigned & MDVideoFbRequired) != MDVideoFbRequired)
	    return(False);
	break;
    case SYM_DPY_TO_WIN:
    case SYM_DPY_TO_FB:
	if ((pMpeg->assigned & MDScreenRequired) != MDScreenRequired)
	    return(False);
	break;
    }

    if (! MpegFlushParam(pMpeg))
	return (False);
    _McsSendCommand(server->pParse, STR_START);
    _McsFlushMessages(server->pParse);
    return(True);
}

static boolean
MpegStopOperation(McsServer server)
{
    _McsSendCommand(server->pParse, STR_STOP);
    _McsFlushMessages(server->pParse);
    return(True);
}

static void
MpegCloseServer(McsServer server)
{
}

static McsServerClass _MpegDecompClass = {
    McsMpegDecompressor,
    sizeof(McsMpegDecPriv),
    MpegDecompOpenServer,
    MpegSetParam,
    MpegUnsetParam,
    MpegGetNextEvent,
    MpegStartOperation,
    MpegStopOperation,
    MpegCloseServer,
    MpegSetWideParam,
    MpegUnsetWideParam,
};

McsServerClassPtr McsMpegDecompClass = &_MpegDecompClass;
