/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpegdecomp.h - public interface to mpeg decompressor
 *
 * DESCRIPTION
 *	<overall description of what the header file does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpegdecomp.h,v 1.9 1994/06/27 17:01:32 russ Exp $
 */
#ifndef _MPEGDECOMP_H_
#define _MPEGDECOMP_H_

#include "nvrTypes.h"
#include "nvrTimer.h"

/*
 *	DEFINES
 */
typedef enum {
    SYM_DPY_TO_YCFB,
    SYM_DPY_TO_WIN,
    SYM_YCTOFILE,
    SYM_DPY_TO_FB,

    SYM_RGB_888,
    SYM_RGB_444,
    SYM_RGB_555,
    SYM_RGB_232,
    SYM_RGB_332,
    SYM_RGB_C555,
    SYM_RGB_C666,
    SYM_GRAY8,
    SYM_GRAY7,
    SYM_GRAY1
} MpegDecSymbol;

#define STR_DPY_TO_WIN  "normalDpy"
#define STR_DPY_TO_FB   "fastDpy"
#define STR_YCTOFILE	"ycToFile"
#define STR_DPY_TO_YCFB	"ycToFb"

#define STR_YCTOFB	"ycToFb"
#define STR_YCTOFILE	"ycToFile"
#define STR_GRAY1	"gray1"
#define STR_GRAY7	"gray7"
#define STR_GRAY8	"gray8"
#define STR_RGB_232	"rgb232"
#define STR_RGB_332	"rgb332"
#define STR_RGB_888	"rgb888"
#define STR_RGB_444	"rgb444"
#define STR_RGB_555	"rgb555"
#define STR_RGB_C555	"rgbC555"
#define STR_RGB_C666	"rgbC666"

#define MDXcoord		0x00000001
#define MDYcoord		0x00000002
#define MDWidth			0x00000004
#define MDHeight		0x00000008
#define MDDisplayName		0x00000010
#define MDInPath		0x00000020
#define MDAudioInPath		0x00000040
#define MDOutPath		0x00000080
#define MDColor			0x00000100
#define MDWindowId		0x00000200
#define MDDestType		0x00000400
#define MDAudioEnabled		0x00000800
#define MDVideoEnabled		0x00001000
#define MDInputNotify		0x00002000
#define MDFrameRate		0x00004000
#define MDInTime		0x00008000
#define MDOutTime		0x00010000
#define MDInFrame		0x00020000
#define MDOutFrame		0x00040000
#define MDTolerance		0x00080000
#define MDPreloadSize		0x00100000
#define MDAudioDelay		0x00200000
#define MDFrameFrequency	0x00400000
#define MDAudioSource		0x00800000
#define MDArg			0x01000000
#define MDPreInit		0x02000000
#define MDPostInit		0x04000000
#define MDBitServer		0x08000000
#define MDAudioQuality		0x10000000

#define MDFileRequired \
    ( MDVideoEnabled | MDOutPath | MDInPath | MDDestType)

#define MDVideoFbRequired \
    ( MDVideoEnabled | MDAudioEnabled | MDInPath | MDDestType)

#define MDScreenRequired \
    (MDXcoord | MDYcoord | MDWidth | MDHeight | MDDisplayName \
   | MDWindowId | MDDestType | MDAudioEnabled \
   | MDVideoEnabled | MDInputNotify | MDInPath | MDColor)

#define MDOptional \
    (MDInTime | MDOutTime | MDInFrame | MDOutFrame | MDTolerance \
   | MDPreloadSize | MDAudioDelay | MDFrameFrequency | MDArg | MDAudioSource \
   | MDPreInit | MDPostInit | MDFrameRate | MDAudioInPath \
   | MDOutPath)

/*
 *	TYPEDEFS
 */
typedef struct {
    int x, y;
    int width;
    int height;
    char *displayName;
    ULong windowId;
    int mode;			/* capture or playback */
    int method;			/* fastx, native or direct */
    int destType;		/* destination type */
    int color;			/* color rendering type */
    boolean audioEnabled;	/* whether we are capturing/playing audio */
    boolean videoEnabled;	/* whether we are capturing/playing video */
    McsInputNotify inputNotify;	/* called when events available from srvr */
    int frameRate;		/* ... in (integer) frames per second */
    long pad;			/* this makes comp and decomp isometric */
    TimeSpec inTime;		/* time to start */
    TimeSpec outTime;		/* time when we are done */
    long inFrame;		/* specified starting frame */
    long outFrame;		/* specified frame which we *don't* play */
    long tolerance;		/* tolerance of frame lateness (in microsecs) */
    long preloadSize;		/* number of frames to read ahead */
    long audioDelay;		/* delay until start of audio */
    long frameFrequency;	/* frequency of frame events */
    char *audioSource;		/* where to get audio */
    PrivPointer arg;		/* arg to inputNotify */
    char *inPath;
    char *aInPath;
    char *outPath;
    char *preInit;		/* string to send to mcs before anything else */
    char *postInit;		/* sent to mcs after init, before commands */
    char *bitServer;
    int audioQuality;		/* 1 .. 32 */
} McsMpegDecomp, *McsMpegDecompPtr;

/*
 *	EXTERNS
 */
CExternal McsServerClassPtr McsMpegDecompClass;

#endif /* _MPEGDECOMP_H_ */
