%{
/* $Header: /usr/local/src/nvrToMIT/RCS/protoparse.y,v 2.50 1995/03/10 05:37:35 toddb Exp $ */

/*
 * The yacc grammar below accepts something like:
 */

#define NeedFunctionPrototypes 1

#include "nvrTypes.h"
#include "nvrAlloc.h"
#include "nvrReason.h"
#include "nvrString.h"
#include "osio.h"
#include "protoparseP.h"
#include "mcs-serverP.h"

#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>

extern int yyparse(YaccContextPtr);

#ifndef YYDEBUG
#define YYDEBUG 0
#endif

#ifdef DEBUG_ENABLED
# define DebugToken(args)					\
    if (ycp->yydebug>0) {					\
	Msg("<Token: ");					\
	Msg args;						\
	Msg(">, ");						\
    }
#else
# define DebugToken(args)
#endif

#define Expect(string)	TokenExpected = string
#define GotIt	TokenExpected = 0
int TokenExpected = 0;

%}

%token            YYEOF YYSUSPEND
%token		  SERVER_TOKEN CLIENT_TOKEN CONFIGURATION_TOKEN
%token            SOURCE_TOKEN
%token            STATEMENT_SEPARATOR BAD_TOKEN EVENT_TOKEN
%token <token>    REAL_VARIABLE INT_VARIABLE STRING_VARIABLE SYMBOL_VARIABLE
%token <token>    VARIABLE_VARIABLE
%token <token>    COMMAND_TOKEN ERROR_TYPE_TOKEN EVENT_TYPE_TOKEN
%token <token>    SYMBOL_VALUE
%type  <token>    SymbolValue VariableValue
%type  <intvalue> SignedValue IntValue
%type  <real>     SignedReal RealValue
%token <real>     REAL_VALUE
%token <intvalue> INT_VALUE SIGNED_VALUE
%token <string>   STRING_VALUE UNKNOWN_TOKEN
%type  <string>   StringValue

%%

Input:
    SERVER_TOKEN Statements
  | CLIENT_TOKEN Events
    ;

Events:
    Events Event
  | Event
    ;

Statements:
    Statements Statement
  | Statement
    ;

Statement:
    Assignment STATEMENT_SEPARATOR
  | SOURCE_TOKEN StringValue STATEMENT_SEPARATOR
    { 
	OpenSourceFile(ycp->pParse, $2);
    }
  | Command STATEMENT_SEPARATOR
  | STATEMENT_SEPARATOR
  | ErrorSyntax
    ;

Assignment:
    INT_VARIABLE Equals SignedValue
    { (*ycp->pParse->config.u.server.assignNotify)(ycp->pParse, $1->id, 
      (ParseValuePtr)&$3); }
  | REAL_VARIABLE Equals SignedReal
    { (*ycp->pParse->config.u.server.assignNotify)(ycp->pParse, $1->id, 
      (ParseValuePtr)&$3); }
  | STRING_VARIABLE Equals StringValue
    { (*ycp->pParse->config.u.server.assignNotify)(ycp->pParse, $1->id, 
      (ParseValuePtr)&$3); }
  | SYMBOL_VARIABLE Equals SymbolValue
    { (*ycp->pParse->config.u.server.assignNotify)(ycp->pParse,
						       $1->id, (ParseValuePtr)&$3->id); }
  | VARIABLE_VARIABLE Equals VariableValue
    { (*ycp->pParse->config.u.server.assignNotify)(ycp->pParse,
						       $1->id, (ParseValuePtr)&$3->id); }
    ;

Command:
    COMMAND_TOKEN
    { (*ycp->pParse->config.u.server.commandNotify)(ycp->pParse, $1->id, 0); }
    ;

Event:
    BasicEvent STATEMENT_SEPARATOR
  | STATEMENT_SEPARATOR
  | ErrorSyntax
    ;

BasicEvent:
    EVENT_TOKEN EVENT_TYPE_TOKEN
    {
	(*ycp->pParse->config.u.client.eventNotify)(ycp->pParse,
						      $2->id,
						      MCS_TYPE_NONE,
						      NULL);
    }
  | EVENT_TOKEN EVENT_TYPE_TOKEN ':' StringValue
    {
	(*ycp->pParse->config.u.client.eventNotify)(ycp->pParse,
						      $2->id,
						      MCS_TYPE_STRING,
						      (ParseValuePtr)&$4);
    }
  | EVENT_TOKEN EVENT_TYPE_TOKEN ':' SignedValue
    {
	(*ycp->pParse->config.u.client.eventNotify)(ycp->pParse,
						      $2->id,
						      MCS_TYPE_INT,
						      (ParseValuePtr)&$4);
    }
  | EVENT_TOKEN EVENT_TYPE_TOKEN ':' SignedReal
    {
	(*ycp->pParse->config.u.client.eventNotify)(ycp->pParse,
						      $2->id,
						      MCS_TYPE_REAL,
						      (ParseValuePtr)&$4);
    }
    ;

ErrorSyntax:
    error
    {
	yyerrok;
	yyclearin;
	yyerror(ycp, "syntax error");
    }
    ;

VariableValue:
    INT_VARIABLE { $$ = $1; }
  | STRING_VARIABLE { $$ = $1; }
  | SYMBOL_VARIABLE { $$ = $1; }
    ;

SignedValue:
    '-' IntValue { $$ = - $2; }
  | '+' IntValue { $$ = $2; }
  | IntValue { $$ = $1; }
    ;

SignedReal:
    '-' RealValue { $$ = - $2; }
  | '+' RealValue { $$ = $2; }
  | RealValue { $$ = $1; }
    ;

Equals: { Expect('='); } '=' { GotIt; } ;
IntValue: { Expect(INT_VALUE); } INT_VALUE { $$ = $2; GotIt; } ;
RealValue: { Expect(REAL_VALUE); } REAL_VALUE { $$ = $2; GotIt; } ;
StringValue: { Expect(STRING_VALUE); } STRING_VALUE { $$ = $2; GotIt; } ;
SymbolValue: { Expect(SYMBOL_VALUE); } SYMBOL_VALUE { $$ = $2; GotIt; } ;
%%

/*
 * XXX this changes the usage of yychar, yyval and yylval to point
 * into the ycp structure.  This is necessary because yacc generates
 * these references.
 */
#define yyval	ycp->_yyval
#define yylval	ycp->_yylval
#define yychar	ycp->_yychar

/*
 * Redefine these...
 */
#undef	yyclearin
#undef	yyerrok
#define yyclearin ycp->_yychar = -1
#define yyerrok ycp->yyerrflag = 0


typedef struct _TokenEntry {
    char *token;
    int tokenLength;
    int type;
    int id;
    VoidFuncPtr function;
} TokenEntry;

typedef struct {
    char accelerators[256];	/* accelerator given first character of token */
    int *sortedIndex;		/* dereference through this for sorted access */
    int numEntries;
    TokenEntry *table;		/* the table of tokens */
    char **symbolStrings;
    char **eventStrings;
    char unknownToken[256];
} TokenInfo;

char *McsSymbolStrings[ N_MCS_SYMBOLS ] =
{
    "x",
    "y",
    "width",
    "height",
    "dpy",
    "visid",
    "win",
    "path",
    "audioInPath",
    "inPath",
    "outPath",
    "q",
    "inTime",
    "outTime",
    "inFrame",
    "outFrame",
    "fps",
    "tolerance",
    "mode",
    "method",
    "preloadSize",
    "audioEnable",
    "videoEnable",
    "audioDelay",
    "frameFreq",
    "audioSource",
    "bitRate",
    "nPframes",
    "nBframes",
    "motionRadius",
    "sliceStart",
    "color",
    "destType",
    "bitserver",
    "audioQuality",

    "start",
    "stop",
    "quit",
    "config",

    "configuration",
    "event",
    "server",
    "client",
    "source",

    "logFile",
    "uninit",
};

/*
 * parse symbol strings
 * Note: the order of these symbols must match the order of McsSymbolConstants
 * defined in protoparseP.h exactly.
 */

char *McsEventStrings[] = {
    "ok",
    "badValue",
    "badWindow",
    "badDisplay",
    "badVisual",
    "badAssign",
    "badConfig",
    "badHardware",
    "badFile",
    "badWrite",
    "badRead",
    "alreadyRunning",
    "badAlloc",
    "badSyntax",
    "tooLate",
    "badAudio",
    "badPreloadSize",
    "badImplementation",
    "badMatch",
    "noLicense",
    "badGeometry",
    "rejectAssign",
    "rejectCommand",
    "loading",
    "stopped",
    "paused",
    "running",
    "info",
    "ready",
    "audioBusy",
    "frameMark",
    "iratio",
    "bratio",
    "pratio",
    "ratio",
    "bitrate",
    "skippedFrames",
    "lateFrames",
    "frameRate",
    "quantizer",
    "serverExit",
};

static TokenInfo Tokens = {
    { 0 },	/* accelerators */
    NULL,	/* sortedIndex */
    0,		/* numEntries */
    NULL,	/* table */
    McsSymbolStrings,
    McsEventStrings,
};

int
NewToken(char *name,
	 int type,
	 int id)
{
    int i;
    char *shortToken, *longToken;
    int shortLength;
    int tokenLength;

    tokenLength = strlen(name);
    if (Tokens.table) {
	for (i = 0; i < Tokens.numEntries; i++) {
	    if (tokenLength == Tokens.table[ i ].tokenLength) {
		if (strcmp(name, Tokens.table[ i ].token) == 0)
		    Fatal("Token \"%s\" redefined\n", name);
		continue;
	    } else if (tokenLength > Tokens.table[ i ].tokenLength) {
		shortToken = Tokens.table[ i ].token;
		shortLength = Tokens.table[ i ].tokenLength;
		longToken = name;
	    } else {
		shortToken = name;
		shortLength = tokenLength;
		longToken = Tokens.table[ i ].token;
	    }
	    /*
	     * if (strncmp(shortToken, longToken, shortLength) == 0)
	     *     Fatal("Token \"%s\" and \"%s\" are ambiguous\n",
	     *         shortToken, longToken);
	     */
	}
    } else
	i = 0;

    Tokens.table = (TokenEntry *)NullRealloc((char *) Tokens.table,
				  (unsigned) (sizeof(TokenEntry) * (i + 1)));
    Tokens.table[i].token = name; /* assume space is already allocated */
    Tokens.table[i].tokenLength = tokenLength;
    Tokens.table[i].type = type;
    Tokens.table[i].id = id;

    Tokens.numEntries = i+1;

    return (i);
}

#if YYDEBUG || defined(DEBUG_ENABLED)
/*VARARGS1*/
static int
ShowToken(char *fmt,
	  int x0, int x1, int x2, int x3, int x4,
	  int x5, int x6, int x7, int x8, int x9)
{
    Msg("<Token: ");
    Msg(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
    Msg(">, ");
}
#endif

static void
OpenSourceFile(ParseInfoPtr pParse, char *path)
{
    /*
     * we can't recurse yet on sourcing files, so just close it.
     */
    if (pParse->sourceFp) {
    	Msg("warning: closing previous source before complete\n");
    	CloseBufferedFile(pParse->sourceFp);
    }
    pParse->sourceFp = OpenBufferedFileForReading(path);
    if (pParse->sourceFp == NULL) {
        Msg("warning: can't open \"%s\"\n", path);
    } else {
	pParse->inputWaiting = True;
	(*pParse->config.inputNotify)(pParse);	/* ask to be read again */
        Msg("reading \"%s\"...\n", path);
    }
}

static void
FillParseBuffer(ParseInfoPtr pParse)
{
    int length;
    int bytesIn = 0;	/* the init value is for lint */

    /*
     * Don't bother reading more if there's no more to read
     */
    if (pParse->terminate) {
	pParse->inputWaiting = False;
	return;
    }

    /*
     * Copy down after we use up half the buffer 
     */
    if (pParse->pStart - pParse->buffer > PARSE_BUFFER_SIZE/2) {
	length = pParse->pNext - pParse->pStart;
	memcpy(pParse->buffer, pParse->pStart, length);
	pParse->pNext = pParse->buffer + length;
	pParse->pStart = pParse->buffer;
    }

    length = &pParse->buffer[ PARSE_BUFFER_SIZE ] - pParse->pNext;

    if (length == 0) {
	pParse->inputWaiting = True;
	return;
    }

    if (pParse->sourceFp) {
        bytesIn = BufferedReadData(pParse->sourceFp, pParse->pNext, length);
        if (bytesIn == 0) {
            CloseBufferedFile(pParse->sourceFp);
            pParse->sourceFp = NULL;
	    Msg("sourcing done.\n");
        }
    }

    if (pParse->sourceFp == NULL) {
        bytesIn = NonBlockingRead(pParse->config.inFileHandle,
			      pParse->pNext, length);
    }
    
    if (bytesIn < 0) {
	pParse->terminate = True;
	(*pParse->config.eofNotify)(pParse);
    } else if (bytesIn == 0) {
	pParse->inputWaiting = False;
    } else {
	if (pParse->showInput) {
	    /* This is where the PARSE_BUFFER_SIZE+1 is necessary */
	    pParse->pNext[ bytesIn ] = '\0';
	    Msg("%s in: <%s>\n",
		pParse->config.server ? "srv" : "clnt", pParse->pNext);
	}

	pParse->pNext = pParse->pNext + bytesIn;
	pParse->inputWaiting = True;
	(*pParse->config.inputNotify)(pParse);	/* ask to be read again */
    }
}

static void
ClientInputNotify(FileHandle fd, ParseInfoPtr pParse)
{
    if (pParse->active) {
	pParse->inputWaiting = True;
    } else {
	FillParseBuffer(pParse);
    }
    (*pParse->config.inputNotify)(pParse);
}
    
void
CloseAndFreeClientConnection(ParseInfoPtr pParse)
{
    FreeAsyncHandler(pParse->config.inFileHandle);
    CloseFile(pParse->config.inFileHandle);
    free((void *)pParse);
}

PrivPointer
GetParsePrivateData(ParseInfoPtr pParse)
{
    return (pParse->config.pointer);
}

void
SetParserState(ParseInfoPtr pParse, boolean suspendParsing)
{
    if (! (pParse->suspendParsing = suspendParsing)) {
	(*pParse->config.inputNotify)(pParse);
    }
}

char *
SymbolToString(int symbol)
{
    int i;

    for (i = 0; i < Tokens.numEntries; i++) {
	if (Tokens.table[ i ].id == symbol)
	    return (Tokens.table[ i ].token);
    }
    return ("??");
}

/*
 * This is the central routine for delivering bytes from client to server
 * or server to client.  Here we write out messages, separating each with
 * a ';' and newlines.
 */
void
_McsOutputMessage(ParseInfoPtr pParse, char *fmt, ...)
{
    char msgBuf[ 1024 ];
    long msgLen;
    va_list ap;

    va_start(ap, fmt);
    vsprintf(msgBuf, fmt, ap);
    msgLen = strlen(msgBuf);
    va_end(ap);
    
    BufferedWriteData(pParse->outFp, msgBuf, msgLen);

    PutBufferedChar('\n', pParse->outFp); /* newlines are statement separators */
    if (pParse->showOutput) {
	Msg("%s out: <%s>;\n",
	    pParse->config.server ? "srv" : "clnt",
	    msgBuf);
    }
    if (pParse->config.outputNotify)
        (*pParse->config.outputNotify)(pParse);
    va_end(ap);
}

void
_McsFlushMessages(ParseInfoPtr pParse)
{
    FlushBufferedFile(pParse->outFp);
}

/*
 * For clients... 
 */
void
_McsSendCommand(ParseInfoPtr pParse, char *symName)
{
    _McsOutputMessage(pParse, "%s", symName);
}

void
_McsSendAssign(ParseInfoPtr pParse,
	      char *symName,
	      int valueType,
	      ParseValuePtr value)
{
    switch(valueType) {
    case MCS_TYPE_BOOL:
	_McsOutputMessage(pParse, "%s=%d", symName, value->bool);
	break;
    case MCS_TYPE_INT:
	_McsOutputMessage(pParse, "%s=%ld", symName, value->intvalue);
	break;
    case MCS_TYPE_REAL:
	_McsOutputMessage(pParse, "%s=%f", symName, value->real);
	break;
    case MCS_TYPE_STRING:
	_McsOutputMessage(pParse, "%s=\"%s\"", symName, value->string);
	break;
    case MCS_TYPE_SYMBOL:
	_McsOutputMessage(pParse, "%s=%s", symName, value->string);
	break;
    default:
	Fatal("MscSendAssign: bad value type\n");
	/*NOTREACHED*/
    }
}

/*
 * For servers...
 */
void
RejectAssignment(pParse, id, reason)
    ParseInfoPtr pParse;
    int id;
    int reason;
{
    _McsOutputMessage(pParse, "%s %s: \"%s\"", 
	Tokens.symbolStrings[ SYM_EVENT ],
	Tokens.eventStrings[ reason ],
	Tokens.symbolStrings[ id ]);
}

void
RejectCommand(pParse, id, reason)
    ParseInfoPtr pParse;
    int id;
    int reason;
{
    _McsOutputMessage(pParse, "%s %s: \"%s %s\"", 
	Tokens.symbolStrings[ SYM_EVENT ],
	Tokens.eventStrings[ MCS_EVENT_REJECT_COMMAND ],
	Tokens.symbolStrings[ id ],
	Tokens.eventStrings[ reason ]);
}

void
ReasonToClient(ParseInfoPtr pParse, InternalReasonPtr reason)
{
    int mcsEvent;

    switch(reason->hint) {
    case IR_InternalProblem:	mcsEvent = MCS_BAD_IMPLEMENTATION; break;
    case IR_NotImplemented:	mcsEvent = MCS_BAD_IMPLEMENTATION; break;
    case IR_Unknown:		mcsEvent = MCS_EVENT_INFO; break;
    case IR_PrematureEof:	mcsEvent = MCS_BAD_FILE; break;
    case IR_CantWrite:		mcsEvent = MCS_BAD_WRITE; break;
    case IR_UnknowFileType:	mcsEvent = MCS_BAD_FILE; break;
    case IR_ShortRead:		mcsEvent = MCS_BAD_READ; break;
    case IR_BadSampleRate:	mcsEvent = MCS_BAD_MATCH; break;
    case IR_BadColor:		mcsEvent = MCS_BAD_MATCH; break;
    case IR_BadAlloc:		mcsEvent = MCS_BAD_ALLOC; break;
    case IR_BadConversion:	mcsEvent = MCS_BAD_MATCH; break;
    case IR_BadDimensions:	mcsEvent = MCS_BAD_VALUE; break;
    case IR_BadIOFile:		mcsEvent = MCS_BAD_FILE; break;
    case IR_BadDisplay:		mcsEvent = MCS_BAD_DISPLAY; break;
    case IR_BadWindow:		mcsEvent = MCS_BAD_WINDOW; break;
    case IR_BadVisual:		mcsEvent = MCS_BAD_VISUAL; break;
    case IR_BadHardware:	mcsEvent = MCS_BAD_HARDWARE; break;
    case IR_Eof:		mcsEvent = MCS_EVENT_INFO; break;
    case IR_Warning:		mcsEvent = MCS_EVENT_INFO; break;
    default:			mcsEvent = MCS_EVENT_INFO; break;
    }

    EventToClient(pParse,
		  mcsEvent,
		  (ParseValue *)&reason->msg,
		  MCS_TYPE_STRING);
}

void
EventToClient(ParseInfoPtr pParse,
	      int event,
	      ParseValue *value,
	      int valueType)
{
    switch (valueType) {
    case MCS_TYPE_INT:
	_McsOutputMessage(pParse, "%s %s: %ld", 
	    Tokens.symbolStrings[ SYM_EVENT ],
	    Tokens.eventStrings[ event ],
	    value->intvalue);
	break;

    case MCS_TYPE_REAL:
	_McsOutputMessage(pParse, "%s %s: %f",
	    Tokens.symbolStrings[ SYM_EVENT ],
	    Tokens.eventStrings[ event ],
	    value->real);
	break;

    case MCS_TYPE_STRING:
	_McsOutputMessage(pParse, "%s %s: \"%s\"", 
	    Tokens.symbolStrings[ SYM_EVENT ],
	    Tokens.eventStrings[ event ],
	    value->string);
	break;

    case MCS_TYPE_NONE:
	_McsOutputMessage(pParse, "%s %s", 
	    Tokens.symbolStrings[ SYM_EVENT ],
	    Tokens.eventStrings[ event ]);
	break;
    }
}

void
AddConnectionIO(ParseInfoPtr pParse, ParseConfigPtr pConfig)
{
    pParse->config.inFileHandle = pConfig->inFileHandle;
    pParse->config.outFileHandle = pConfig->outFileHandle;
    if (pParse->config.inFileHandle >= 0) {
        if (pParse->config.asynchronous) {
            AllocAsyncHandler(pParse->config.inFileHandle,
                              (VoidFuncPtr)ClientInputNotify,
			      False, (void *)pParse);
        } else {
            AllocSelectHandler(pParse->config.inFileHandle,
                               (VoidFuncPtr)ClientInputNotify, (void *)pParse);
            SetNonBlocking(pParse->config.inFileHandle);
         }
    }
    if (pParse->config.outFileHandle >= 0) {
	if (! FdValid(pParse->config.outFileHandle)) {
	    Msg("mcs output file (%d) is not valid... switching to stdout\n",
		pParse->config.outFileHandle);
	useStdout:
	    pParse->config.outFileHandle = 1;
	}
        pParse->outHandleIsTty = FileHandleIsTty(pParse->config.outFileHandle);
        pParse->outFp = (PrivPointer)
            ReopenBufferedFileForWriting(pParse->config.outFileHandle);
    } else {
        goto useStdout;
    }
}

ParseInfoPtr
InitParsedConnection(ParseConfigPtr pConfig)
{
    ParseInfoPtr pParse;
    int startSymIndex;
    static serverSymbolsDone, clientSymbolsDone;

    if ((pParse = (ParseInfoPtr) malloc(sizeof(ParseInfo))) == NULL)
	return (NULL);
    memset((char *)pParse, 0, sizeof(ParseInfo));
    if ((pParse->ycp = yyinit(pParse)) == NULL) {
	free((void *)pParse);
	return (NULL);
    }

    pParse->terminate = False;
    pParse->inputWaiting = False;
    pParse->pQuote = NULL;
    pParse->pStart = pParse->buffer;
    pParse->config = *pConfig;
    if (pParse->config.server) {
	startSymIndex = SYM_SERVER;
	if (! serverSymbolsDone) {
	    serverSymbolsDone = True;
	    NewToken(Tokens.symbolStrings[ startSymIndex ],
		     SERVER_TOKEN, SERVER_TOKEN);
	    NewToken(Tokens.symbolStrings[ SYM_SOURCE ],
		     SOURCE_TOKEN, SOURCE_TOKEN);
	}
    } else {
	startSymIndex = SYM_CLIENT;
	if(! clientSymbolsDone) {
	    int i;
	    int nEvents = NumberOfCells(McsEventStrings);
	    char **events = Tokens.eventStrings;

	    clientSymbolsDone = True;
	    for (i = 0; i < nEvents; i++)
		NewToken(events[ i ], EVENT_TYPE_TOKEN, i);
	    NewToken(Tokens.symbolStrings[ SYM_EVENT ], EVENT_TOKEN, 0);
	    NewToken(Tokens.symbolStrings[ startSymIndex ],
		     CLIENT_TOKEN, CLIENT_TOKEN);
	}
    }

    strcpy(pParse->buffer, Tokens.symbolStrings[ startSymIndex ]);
    pParse->pNext = pParse->pStart + strlen(pParse->pStart);
    *pParse->pNext++ = ' ';

    AddConnectionIO(pParse, pConfig);

    return (pParse);
}

void
SetServerParserNotifiers(ParseInfoPtr pParse,
			      VoidFuncPtr inputNotify,
			      ParseCommandNotifier commandNotify,
			      VoidFuncPtr assignNotify)
{
    pParse->config.inputNotify = inputNotify;
    pParse->config.u.server.assignNotify = assignNotify;
    pParse->config.u.server.commandNotify = commandNotify;
}

void
SetClientParserNotifyiers(ParseInfoPtr pParse,
			       VoidFuncPtr inputNotify,
			       ParseEventNotifier eventNotify)
{
    pParse->config.inputNotify = inputNotify;
    pParse->config.u.client.eventNotify = eventNotify;
}

void
HandleConnectionInput(ParseInfoPtr pParse)
{
    pParse->active = True;	/* lock out */
    if (pParse->inputWaiting)
	FillParseBuffer(pParse);

    (void) yyparse(pParse->ycp);
    pParse->active = False;	/* unlock */
}

static int
atox(char *ptr)
{
    int value = 0;
    char c;

    c = *ptr++;
    while (isxdigit(c)) {
	value <<= 4;
	if (c >= 'a')		/* then its one of [abcdef] */
	    value += c - 'a' + 10;
	else if (c >= 'A')
	    value += c - 'A' + 10;
	else
	    value += c - '0';
	c = *ptr++;
    }
    return (value);
}

static int
IntegerValue(YaccContextPtr ycp, char *ptr)
{
    ParseInfoPtr pParse = ycp->pParse;

    DebugToken(("%ld", ycp->_yylval.intvalue));
    pParse->pStart = ptr;
    return(INT_VALUE);
}

static double
RealValue(YaccContextPtr ycp, char *ptr)
{
    ParseInfoPtr pParse = ycp->pParse;

    DebugToken(("%g", ycp->_yylval.real));
    pParse->pStart = ptr;
    return(REAL_VALUE);
}

static int
yylex(YaccContextPtr ycp)
{
    ParseInfoPtr pParse = ycp->pParse;
    char *ptr = pParse->pStart;
    char *pStart;
    char c, quote;
    int i;
    boolean isReal;

    if (pParse->suspendParsing) {
	return (YYSUSPEND);
    }
    if (pParse->skipStatement) {
	while (ptr != pParse->pNext) {
	    c = *ptr++;
	    if (c == ';' || c == '\n') {
		pParse->pStart = ptr;
		pParse->skipStatement = False;
		return (STATEMENT_SEPARATOR);
	    }
	}
	pParse->pStart = pParse->pNext;
	return (YYSUSPEND);
    }

    while (ptr != pParse->pNext) {
	c = *ptr;

	switch (c) {
	case '\n':
	case ';':
	    DebugToken(("%c\n", c));
	    pParse->pStart = ptr+1;
	    return (STATEMENT_SEPARATOR);

	case ':':
	case '=':
	case '+':
	case '-':
	    DebugToken(("%c", c));
	    pParse->pStart = ptr+1;
	    return (c);

	case ' ':
	case '\t':
	    pParse->pStart = ++ptr;
	    continue;

	case '0': /* hexadecimalNumber */
	    if (pParse->pNext - ptr < 2)
		return (YYSUSPEND);
	    if (ptr[1] != 'x')
		goto decimalNumber;
	    c = *(pStart = (ptr += 2));
	    for (;;) {
		if (ptr == pParse->pNext)
		    return (YYSUSPEND);
		if (! isxdigit(c))
		    break;
		c = *++ptr;
	    }
	    ycp->_yylval.intvalue = atox(pStart);
	    return(IntegerValue(ycp, ptr));

	case '.':
	    if (ptr == pParse->pNext)
		return (YYSUSPEND);
	    if (! isdigit(ptr[1])) {
		DebugToken(("%c", c));
		pParse->pStart = ptr+1;
		return (c);
	    }
	    /* fall thru... */

	case '1': case '2': case '3': case '4':
	case '5': case '6': case '7': case '8': case '9':
	decimalNumber:
	    isReal = False;
	    pStart = ptr;
	    for (;;) {
		if (ptr == pParse->pNext)
		    return (YYSUSPEND);
		if (c == '.')
		    isReal = True;
		else if (! isdigit(c))
		    break;
		c = *++ptr;
	    }
	    if (isReal) {
		ycp->_yylval.real = atof(pStart);
		return( RealValue(ycp, ptr) );
	    } else {
		ycp->_yylval.intvalue = atoi(pStart);
		return( IntegerValue(ycp, ptr) );
	    }
	    break;

	case '\'':
	case '"':
	    quote = c;
	    c = *(pStart = ++ptr);
	    for (;;) {
		if (ptr == pParse->pNext)
		    return (YYSUSPEND);
		if (c == quote)
		    break;
		c = *++ptr;
	    }
	    *ptr = '\0';
	    ycp->_yylval.string = CopyOfString(pStart);
	    DebugToken(("%s", ycp->_yylval.string));
	    *ptr++ = quote;
	    pParse->pStart = ptr;
	    return (STRING_VALUE);

	default:
	    /*
	     * if it doesn't look like a symbol, then return it as an
	     * unknown symbol
	     */
	    if (! isalpha(c) && c != '_') {
		Tokens.unknownToken[0] = c;
		Tokens.unknownToken[1] = '\0';
		ycp->_yylval.string = Tokens.unknownToken;
	        pParse->pStart = ptr+1;
		return (UNKNOWN_TOKEN);
	    }

	    /*
	     * gather up the token, look it up and return
	     */
	    pStart = ptr;
	    for (;;) {
		if (ptr == pParse->pNext)
		    return (YYSUSPEND);
		if (! isalnum(c) && c != '_')
		    break;
		c = *++ptr;
	    }
	    *ptr = '\0';
	    pParse->pStart = ptr;
	    for (i = 0; i < Tokens.numEntries; i++) {
		if (strcmp(pStart, Tokens.table[ i ].token) == 0) {
		    ycp->_yylval.token = &Tokens.table[ i ];
		    *ptr = c;
		    DebugToken(("%s",
			Tokens.table[ i ].token, Tokens.table[ i ].type));
		    return (Tokens.table[ i ].type);
		}
	    }

	    /*
	     * Unknown token.  Record it locally and return
	     */
	    strncpy(Tokens.unknownToken,
		    pStart,
		    NumberOfCells(Tokens.unknownToken)-1);
	    ycp->_yylval.string = Tokens.unknownToken;
	    *ptr = c;
	    return (UNKNOWN_TOKEN);
	}
    }
    if (pParse->terminate)
	return (YYEOF);
    return (YYSUSPEND);
}

static void
yyerror(YaccContextPtr ycp, char *msg)
{
    ParseInfoPtr pParse = ycp->pParse;
    ParseValue got;
    char wantedName[ 100 ], *wanted, gotName[ 100 ];

    /*
     * don't deliver more than one syntax error per statement
     */
    got.string = gotName;
    if (! pParse->skipStatement) {
	pParse->skipStatement = True;

	switch(ycp->_yychar) {
	case UNKNOWN_TOKEN:       got.string = ycp->_yylval.string; break;
	case STATEMENT_SEPARATOR: got.string = ";"; break;
	case STRING_VALUE:        got.string = ycp->_yylval.string; break;
	case '+':                 got.string = "+"; break;
	case '-':                 got.string = "-"; break;
	case '=':                 got.string = "="; break;

	default:                 
	    sprintf(gotName, "got <token %d>", ycp->_yychar);
	    break;
	case REAL_VALUE:
	    sprintf(gotName, "real number (%f)", ycp->_yylval.real);
	    break;
	case INT_VALUE:
	    sprintf(gotName, "integer (%ld)", ycp->_yylval.intvalue);
	    break;
	}

	if (pParse->config.server)
	    EventToClient(pParse, MCS_BAD_SYNTAX, &got, MCS_TYPE_STRING);
	else
	    (*pParse->config.u.client.eventNotify)(pParse,
						   MCS_BAD_SYNTAX,
						   MCS_TYPE_STRING,
						   &got);
    }

    switch (TokenExpected) {
    case '=':          wanted = "'='"; break;
    case REAL_VALUE:   wanted = "a real"; break;
    case INT_VALUE:    wanted = "an integer"; break;
    case STRING_VALUE: wanted = "a string"; break;
    case SYMBOL_VALUE: wanted = "a symbol"; break;
    case 0:            wanted = "<unknown>"; break;
    default:
	sprintf(wantedName, "<token %d>", TokenExpected);
	wanted = wantedName;
	break;
    }

#ifdef DEBUG_ENABLED
    Msg("\nsyntax error: got %s, expected %s\n", got.string, wanted);
#endif
}
