/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	protoparse.h - header for parsing ascii protocol
 *
 * DESCRIPTION
 *	<overall description of what the header file does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/protoparseP.h,v 2.29 1994/10/02 23:55:51 toddb Exp $
 */
#ifndef _PROTOPARSE_H_
#define _PROTOPARSE_H_

#include "nvrTypes.h"
#include "nvrReason.h"
#include "osio.h"
#include "parsedefines.h"

/*
 *	DEFINES
 */
#define PARSE_BUFFER_SIZE	1024

/* reason (for something bad) strings */
#define	STR_BAD_VALUE		McsEventStrings[ MCS_BAD_VALUE ]
#define	STR_BAD_WINDOW		McsEventStrings[ MCS_BAD_WINDOW ]
#define	STR_BAD_DISPLAY		McsEventStrings[ MCS_BAD_DISPLAY ]
#define	STR_BAD_VISUAL		McsEventStrings[ MCS_BAD_VISUAL ]
#define	STR_BAD_ASSIGNMENT	McsEventStrings[ MCS_BAD_ASSIGNMENT ]
#define	STR_BAD_CONFIGURE	McsEventStrings[ MCS_BAD_CONFIGURE ]
#define	STR_BAD_HARDWARE	McsEventStrings[ MCS_BAD_HARDWARE ]
#define	STR_BAD_FILE		McsEventStrings[ MCS_BAD_FILE ]
#define	STR_BAD_WRITE		McsEventStrings[ MCS_BAD_WRITE ]
#define	STR_BAD_RUNNING		McsEventStrings[ MCS_BAD_RUNNING ]
#define	STR_BAD_ALLOC		McsEventStrings[ MCS_BAD_ALLOC ]
#define	STR_BAD_SYNTAX		McsEventStrings[ MCS_BAD_SYNTAX ]
#define	STR_BAD_INTIME		McsEventStrings[ MCS_BAD_INTIME ]
#define	STR_BAD_AUDIO		McsEventStrings[ MCS_BAD_AUDIO ]
#define	STR_BAD_PRELOAD		McsEventStrings[ MCS_BAD_PRELOAD ]
#define	STR_BAD_IMPLEMENTATION	McsEventStrings[ MCS_BAD_IMPLEMENTATION ]
#define	STR_BAD_MATCH		McsEventStrings[ MCS_BAD_MATCH ]
#define	STR_BAD_LICENSE		McsEventStrings[ MCS_BAD_LICENSE ]
#define	STR_BAD_GEOMETRY	McsEventStrings[ MCS_BAD_GEOMETRY ]

/*
 * parse symbol identifiers
 * Note: the order of these symbols must match the order of McsSymbolStrings[]
 * defined in protoparse.y exactly.
 */
typedef enum {
    SYM_X,
    SYM_Y,
    SYM_WIDTH,
    SYM_HEIGHT,
    SYM_DISPLAY,
    SYM_VISUALID,
    SYM_WINDOW,
    SYM_PATH,
    SYM_AUDIOINPATH,
    SYM_INPATH,
    SYM_OUTPATH,
    SYM_QFACTOR,
    SYM_INTIME,
    SYM_OUTTIME,
    SYM_INFRAME,
    SYM_OUTFRAME,
    SYM_FRAME_RATE,
    SYM_TOLERANCE,
    SYM_MODE,
    SYM_METHOD,
    SYM_PRELOADSIZE,
    SYM_AUDIOENABLE,
    SYM_VIDEOENABLE,
    SYM_AUDIODELAY,
    SYM_FRAMEFREQ,
    SYM_AUDIOSOURCE,
    SYM_BITRATE,
    SYM_NPFRAMES,
    SYM_NBFRAMES,
    SYM_MOTIONRAD,
    SYM_SLICESTART,
    SYM_COLOR,
    SYM_DESTTYPE,
    SYM_BITSERVER,
    SYM_AUDIOQUALITY,

    /* parse symbol commands */
    SYM_START,
    SYM_STOP,
    SYM_QUIT,
    SYM_CONFIGURE,

    SYM_CONFIGURATION,
    SYM_EVENT,
    SYM_SERVER,
    SYM_CLIENT,
    SYM_SOURCE,

    /* optional things */
    SYM_LOGFILE,
    SYM_UNINIT,

    N_MCS_SYMBOLS	/* if this is the last item in the list, its correct */
} McsSymbolConstants;

/* parse strings */
#define	STR_X			McsSymbolStrings[ SYM_X ]
#define	STR_Y			McsSymbolStrings[ SYM_Y ]
#define	STR_WIDTH		McsSymbolStrings[ SYM_WIDTH ]
#define	STR_HEIGHT		McsSymbolStrings[ SYM_HEIGHT ]
#define	STR_DISPLAY		McsSymbolStrings[ SYM_DISPLAY ]
#define	STR_VISUALID		McsSymbolStrings[ SYM_VISUALID ]
#define	STR_WINDOW		McsSymbolStrings[ SYM_WINDOW ]
#define	STR_PATH		McsSymbolStrings[ SYM_PATH ]
#define	STR_AUDIOINPATH		McsSymbolStrings[ SYM_AUDIOINPATH ]
#define	STR_INPATH		McsSymbolStrings[ SYM_INPATH ]
#define	STR_OUTPATH		McsSymbolStrings[ SYM_OUTPATH ]
#define	STR_QFACTOR		McsSymbolStrings[ SYM_QFACTOR ]
#define	STR_INTIME		McsSymbolStrings[ SYM_INTIME ]
#define	STR_OUTTIME		McsSymbolStrings[ SYM_OUTTIME ]
#define	STR_INFRAME		McsSymbolStrings[ SYM_INFRAME ]
#define	STR_OUTFRAME		McsSymbolStrings[ SYM_OUTFRAME ]
#define	STR_FRAME_RATE		McsSymbolStrings[ SYM_FRAME_RATE ]
#define	STR_TOLERANCE		McsSymbolStrings[ SYM_TOLERANCE ]
#define	STR_LOGFILE		McsSymbolStrings[ SYM_LOGFILE ]
#define	STR_MODE		McsSymbolStrings[ SYM_MODE ]
#define	STR_METHOD		McsSymbolStrings[ SYM_METHOD ]
#define	STR_PRELOADSIZE		McsSymbolStrings[ SYM_PRELOADSIZE ]
#define	STR_AUDIOENABLE		McsSymbolStrings[ SYM_AUDIOENABLE ]
#define	STR_VIDEOENABLE		McsSymbolStrings[ SYM_VIDEOENABLE ]
#define	STR_AUDIODELAY		McsSymbolStrings[ SYM_AUDIODELAY ]
#define	STR_FRAMEFREQ		McsSymbolStrings[ SYM_FRAMEFREQ ]
#define	STR_AUDIOSOURCE		McsSymbolStrings[ SYM_AUDIOSOURCE ]
#define	STR_UNINIT		McsSymbolStrings[ SYM_UNINIT ]
#define	STR_BITRATE		McsSymbolStrings[ SYM_BITRATE ]
#define	STR_NPFRAMES		McsSymbolStrings[ SYM_NPFRAMES ]
#define	STR_NBFRAMES		McsSymbolStrings[ SYM_NBFRAMES ]
#define	STR_MOTIONRAD		McsSymbolStrings[ SYM_MOTIONRAD ]
#define	STR_SLICESTART		McsSymbolStrings[ SYM_SLICESTART ]
#define	STR_COLOR		McsSymbolStrings[ SYM_COLOR ]
#define	STR_DESTTYPE		McsSymbolStrings[ SYM_DESTTYPE ]
#define	STR_BITSERVER		McsSymbolStrings[ SYM_BITSERVER ]
#define	STR_AUDIOQUALITY       	McsSymbolStrings[ SYM_AUDIOQUALITY ]

#define	STR_START		McsSymbolStrings[ SYM_START ]
#define	STR_STOP		McsSymbolStrings[ SYM_STOP ]
#define	STR_QUIT		McsSymbolStrings[ SYM_QUIT ]
#define	STR_CONFIGURE		McsSymbolStrings[ SYM_CONFIGURE ]

#define	STR_CONFIGURATION	McsSymbolStrings[ SYM_CONFIGURATION ]
#define	STR_EVENT		McsSymbolStrings[ SYM_EVENT ]
#define	STR_SERVER		McsSymbolStrings[ SYM_SERVER ]
#define	STR_CLIENT		McsSymbolStrings[ SYM_CLIENT ]

#define	STR_REJECT_ASSIGN	McsSymbolStrings[ EVENT_REJECT_ASSIGN ]
#define	STR_REJECT_COMMAND	McsSymbolStrings[ EVENT_REJECT_COMMAND ]
#define	STR_LOADING		McsSymbolStrings[ EVENT_LOADING ]
#define	STR_STOPPED		McsSymbolStrings[ EVENT_STOPPED ]
#define	STR_RUNNING		McsSymbolStrings[ EVENT_RUNNING ]
#define	STR_INFO		McsSymbolStrings[ EVENT_INFO ]
#define	STR_READY		McsSymbolStrings[ EVENT_READY ]
#define	STR_AUDIOBUSY		McsSymbolStrings[ EVENT_AUDIOBUSY ]
#define	STR_FRAME		McsSymbolStrings[ EVENT_FRAME ]

/*
 * Forward declarations of structures
 */
typedef struct _parseInfo ParseInfo, *ParseInfoPtr;
typedef union _parseValue ParseValue, *ParseValuePtr;

typedef void (*ParserNotifier)(ParseInfoPtr);

/*
 *	TYPEDEFS
 */
typedef struct _YaccContext YaccContext, *YaccContextPtr;

typedef void (*ParseEventNotifier)(
    ParseInfoPtr,		/* parser info */
    int,			/* event type */
    int,			/* value type */
    ParseValuePtr		/* value */
);

typedef void (*ParseCommandNotifier)(
    ParseInfoPtr,		/* parser info */
    int,			/* event type */
    int				/* value type */
);

typedef void (*ParseAssignNotifier)(
    ParseInfoPtr,		/* parser info */
    int,			/* event type */
    ParseValuePtr		/* value */
);

typedef struct {
    int inFileHandle;		/* file descriptor */
    int outFileHandle;		/* file descriptor */
    PrivPointer pointer;	/* handle for data */
    boolean asynchronous;	/* true if we want interrupts when io ready */
    boolean server;		/* true if this is a server, else client */
    ParserNotifier inputNotify;	/* called when input is available */
    ParserNotifier eofNotify;	/* called when input eof detected */
    ParserNotifier outputNotify;/* optional: called when output is unflushed */
    union {
	struct {
	    ParseAssignNotifier assignNotify;
	    ParseCommandNotifier commandNotify;
	} server;
	struct {
	    ParseEventNotifier eventNotify;
	} client;
    } u;
} ParseConfig, *ParseConfigPtr;

struct _parseInfo {
    ParseConfig config;
    char buffer[ PARSE_BUFFER_SIZE+1 ];	/* see FillParseBuffer() for the +1 */
    PrivPointer outFp;		/* buffered config.outFileHandle */
    char *pStart;		/* start of current, unparsed input statement */
    char *pNext;		/* pointer after last-read byte of input */
    char *pQuote;		/* if defined, pointer to unmatched quote */
    boolean active;		/* true if the stream is being parsed */
    boolean inputWaiting;	/* true if the stream needs reading */
    boolean suspendParsing;	/* true if we should suspend parsing */
    boolean terminate;		/* true if the stream is ready to close */
    boolean skipStatement;	/* true if we should skip to next ';' */
    boolean asynchronous;	/* true if we want interrupts when io ready */
    boolean outHandleIsTty;	/* true if outFileHandle connected to tty */
    boolean showInput;		/* if true, show bytes read */
    boolean showOutput;		/* if true, show bytes written */
    BufFileHndl sourceFp;	/* file pointer for sourcing files */

    YaccContextPtr ycp;		/* for yacc and lex */
}; /* typedef: ParseInfo, *ParseInfoPtr */

/*
 * This is really for yacc, and is the type of argument passed to callback
 * routines.
 */
typedef struct _TokenEntry *TokenEntryPtr;

union _parseValue {
    boolean bool;
    long intvalue;
    char *string;
    TokenEntryPtr token;
    double real;
}; /* typedef: ParseValue, *ParseValuePtr */

typedef ParseValue YYSTYPE;     /* for the sake of the yacc code */
    
/*
 * XXX not in use just now... this is for the context-switchable yacc
 */
struct _YaccContext {
    /*
     * public variables
     */
    int yydebug;	/* set to 1 to get debugging */
    int yynerrs;	/* number of errors */
    int yyerrflag;	/* error recovery flag */
    int _yychar;	/* current input token number */
    YYSTYPE _yyval;
    YYSTYPE _yylval;

    /* back pointer for the sake of yylex, and code called through yacc */
    ParseInfoPtr pParse;

    /*
     * yacc private variables
     */
    YYSTYPE *yyv;	/* value stack */
    int *yys;		/* state stack */
    YYSTYPE *yypv;	/* top of value stack */
    YYSTYPE *yypvt;     /* context copy of top of value stack for $vars */
    int yyn;		/* internal state number info */
    int *yyps;		/* top of state stack */
    int yystate;	/* current state */
    int yytmp;		/* extra var (lasts between blocks) */
    unsigned yymaxdepth;
    int entryPoint;	/* where to jump back into yyparse */
};


/*
 *	EXTERNS
 */
CExternal int NewToken(
    char *,		/* symbol string */
    int,		/* symbol type */
    int			/* symbol identifier */
);

CExternal ParseInfoPtr InitParsedConnection(
    ParseConfigPtr	/* configuration data */
);

CExternal void HandleConnectionInput(
    ParseInfoPtr	/* parse context */
);

CExternal void CloseAndFreeClientConnection(
    ParseInfoPtr
);

CExternal PrivPointer GetParsePrivateData(
    ParseInfoPtr	/* parse context */
);

CExternal YaccContextPtr yyinit(
    ParseInfoPtr	/* parse context */
);

CExternal char *SymbolToString(
    int			/* symbol identifier */
);

CExternal void RejectCommand(
    ParseInfoPtr,	/* parse context */
    int,		/* symbol identifier */
    int			/* reason */
);

CExternal void RejectAssignment(
    ParseInfoPtr,	/* parse context */
    int,		/* symbol identifier */
    int			/* reason */
);

CExternal void yyfree(
    YaccContextPtr	/* parse context */
);

CExternal void ReasonToClient(
    ParseInfoPtr,	/* parse context */
    InternalReasonPtr	/* descriptive reason */
);

CExternal void EventToClient(
    ParseInfoPtr,	/* parse context */
    int,		/* event type */
    ParseValue *,		/* optional value for event */
    int			/* value type (0==>no value) */
);

CExternal void SetParserState(
    ParseInfoPtr,	/* parse context */
    boolean		/* new setting for ->suspendParsing */
);

CExternal void _McsOutputMessage(
    ParseInfoPtr pParse,
    char *fmt,
    ...
);

CExternal void _McsFlushMessages(
    ParseInfoPtr	/* parse context */
);

CExternal void _McsSendCommand(
    ParseInfoPtr pParse,
    char *symName
);

CExternal void _McsSendAssign(
    ParseInfoPtr pParse,
    char *symName,
    int valueType,
    ParseValuePtr value
);

CExternal void SetServerParserNotifiers(
    ParseInfoPtr pParse,
    VoidFuncPtr inputNotify,
    ParseCommandNotifier commandNotify,
    VoidFuncPtr assignNotify
);

CExternal void SetClientParserNotifyiers(
    ParseInfoPtr pParse,
    VoidFuncPtr inputNotify,
    ParseEventNotifier eventNotify
);
			       
#endif /* _PROTOPARSE_H_ */
