/*
 * (c) Copyright 1992-1995 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	NvrDisplay.c nvr display widget -- *must* be subclassed
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/NvrDisplay.c,v 1.12 1995/05/08 22:49:21 toddb Exp $
 */

#include "nvrTypes.h"
#include "nvrDebug.h"
#include "osio.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <Xm/Xm.h>
#include <Xm/XmP.h>
#include "NvrDisplay.h"
#include "NvrDisplayP.h"

static void
nvrClassPartInitialize(WidgetClass widget_class)
{
    XnvrDisplayWidgetClass wc, super;

    WhenDebugEnabled(
	NormalOutput("XnvrDisplayWidgetClass::nvrClassPartInitialize\n") );

    wc = (XnvrDisplayWidgetClass) widget_class;
    super = (XnvrDisplayWidgetClass) wc->core_class.superclass;
    if (wc->nvrDisplay_class.updateDisplay == XnvrInheritUpdateDisplay)
    {
	wc->nvrDisplay_class.updateDisplay =
		super->nvrDisplay_class.updateDisplay;
    }
    if (wc->nvrDisplay_class.renderDisplay == XnvrInheritRenderDisplay)
    {
	wc->nvrDisplay_class.renderDisplay =
		super->nvrDisplay_class.renderDisplay;
    }
    if (wc->nvrDisplay_class.panDisplay == XnvrInheritPanDisplay)
    {
	wc->nvrDisplay_class.panDisplay =
		super->nvrDisplay_class.panDisplay;
    }
}

static void
nvrInitialize(Widget pubReq,
	      Widget pubNew,
	      ArgList args,
	      Cardinal *numArgs)
{
    XnvrDisplayWidget new = (XnvrDisplayWidget)pubNew;

    WhenDebugEnabled( NormalOutput("XnvrDisplayWidget::nvrInitialize\n") );
    /* let subclass determine the initial size */
    new->nvrDisplay.src_x = 0;
    new->nvrDisplay.src_y = 0;
    new->nvrDisplay.src_width = 0;
    new->nvrDisplay.src_height = 0;
    new->nvrDisplay.dest_x = 0;
    new->nvrDisplay.dest_y = 0;
    new->nvrDisplay.dest_width = 0;
    new->nvrDisplay.dest_height = 0;
    new->nvrDisplay.alreadyRedisplayed = False;

    /*
     * really oughtn't be doing this -- application's responsibility
     *
     * new->core.border_width = 0;
     */
}

static void
nvrDestroy(Widget w)
{
    WhenDebugEnabled( NormalOutput("XnvrDisplayWidget::nvrDestroy\n") );
}

static void
nvrStopDisplay (Widget w)
{
    WhenDebugEnabled( NormalOutput("XnvrDisplayWidget::StopDisplay\n") );
}


static void
nvrRealize(Widget w,
	   XtValueMask* value_mask,
	   XSetWindowAttributes* attributes)
{
    /* This routine must be implemented by each subclass */

    WhenDebugEnabled( NormalOutput("XnvrDisplayWidget::nvrRealize\n") );
}


/* The renderProc argument has been added to allow the superclass to invoke
 * a procedure implemented in the subclass.  I tried to get to the class
 * structure of the widget, w, to invoke the RenderDisplay routine stored
 * in the class structure but couldn't get it to work (I could get the
 * class pointer, using XtClass for example, but it pointed to the Core
 * class and not the real class of w <or so it seemed>).
 */
static void
nvrUpdateDisplay (XnvrDisplayWidget w, XnvrRenderProc renderProc)
{
    float width, height, whRatio;
    int base;
    Position newX, newY;
    Dimension newWidth, newHeight;
    XSetWindowAttributes attributes;
    Boolean windowWasResized;
    Display *dpy;
    Window win;

    WhenDebugEnabled( NormalOutput("XnvrDisplayWidget::nvrUpdateDisplay\n") );

    /* background_pixmap futzing--
     * This odd hackery is to cure the problem of the intrinsics automatically
     * clearing the display whenever nvrSetValues returns True (i.e. generating
     * and expose event to cause Redisplay to be called to refresh the
     * window).	 Setting to "None" tells the server not to bother clearing due
     * to there being no background pixmap.
     */
    attributes.background_pixmap = None;
    XChangeWindowAttributes(XtDisplay(w), XtWindow(w), CWBackPixmap,
			    &attributes);

    windowWasResized = False;

    if (w->nvrDisplay.resizeMode == XnvrResizeModeClipAndPan)
    {
	/* RenderDisplay(w); */
	(renderProc)(w);
    }
    else if (w->nvrDisplay.resizeMode == XnvrResizeModeScale4to3)
    {
	/* restrict the display to the largest region within the display
	 * space that is of width to height ratio 4:3
	 */
	width = w->core.width;		/* coerce to float */
	height = w->core.height;	/* coerce to float */
	whRatio = width / height;
	if (whRatio > 1.33)
	{   /* window is too wide... base new size on max height */
	    base = (w->core.height - (w->core.height % 3)) / 3;
	}
	else
	{   /* window is too tall... base new size on max width */
	    base = (w->core.width - (w->core.width % 4)) / 4;
	}
	newWidth = base * 4;
	newHeight = base * 3;
	newX = /* w->core.x + */ (w->core.width - newWidth) / 2;
	newY = /* w->core.y + */ (w->core.height - newHeight) / 2;

	/* if this routine is being called because the window was resized,
	 * or the mode was changed, set up the new display location,
	 * stop the current render operation, clear the window (as this mode
	 * may not display over the entire display space), and start up
	 * the Render with the new parameters.
	 */
	if (newWidth != w->nvrDisplay.dest_width ||
	    newHeight != w->nvrDisplay.dest_height ||
	    newX != w->nvrDisplay.dest_x ||
	    newY != w->nvrDisplay.dest_y)
	{
	    w->nvrDisplay.dest_x = newX;
	    w->nvrDisplay.dest_y = newY;
	    w->nvrDisplay.dest_width = newWidth;
	    w->nvrDisplay.dest_height = newHeight;
	    nvrStopDisplay((Widget)w);
	    windowWasResized = True;
	}
	/* RenderDisplay(w); */
	(renderProc)(w);
	if (windowWasResized)
	{
	    dpy = XtDisplay(w);
	    win = XtWindow(w);
	    attributes.background_pixmap = ParentRelative;
	    XChangeWindowAttributes(XtDisplay(w), XtWindow(w), CWBackPixmap,
				&attributes);
	    /* erase the rectangles that aren't covered by the image */
	    if (newY > 0)
	    {	/* erase a rectangle at the top of the window */
		XClearArea(dpy, win,
			0, 0,
			w->core.width, newY,
			False);
	    }
	    if (newX > w->core.x)
	    {	/* erase a rectangle to the left of the window */
		XClearArea(dpy, win,
			0, newY,
			0, newY + newHeight,
			False);
	    }
	    if (newX + newWidth < w->core.width)
	    {	/* erase a rectangle to the right of the window */
		XClearArea(dpy, win,
			newX + newWidth, newY,
			w->core.width, newY + newHeight,
			False);
	    }
	    if (newY + newHeight < w->core.height)
	    {	/* erase a rectangle at the bottom of the window */
		XClearArea(dpy, win,
			0, newY + newHeight,
			w->core.width, w->core.height,
			False);
	    }
	    attributes.background_pixmap = None;
	    XChangeWindowAttributes(XtDisplay(w), XtWindow(w), CWBackPixmap,
				    &attributes);
	}
    }
    else  /* assume XnvrResizeModeScale */
    {
	if (w->core.width > w->nvrDisplay.src_width)
	{
	    w->nvrDisplay.dest_x =
		w->core.x + (w->core.width - w->nvrDisplay.src_width) / 2;
	    w->nvrDisplay.dest_width = w->nvrDisplay.src_width;
	}
	else
	{
	    /* this is a symptom assault (the problem remains).
	     * the core x and y are getting changed... and I don't know why.
	     * it appears to be after Initialize and unrelated to nvrSetValues.
	     */
	    /* x = w->core.x; */
	    w->nvrDisplay.dest_x = 0;
	    w->nvrDisplay.dest_width = w->core.width;
	}
	if (w->core.height > w->nvrDisplay.src_height)
	{
	    w->nvrDisplay.dest_y =
		w->core.y + (w->core.height - w->nvrDisplay.src_height) / 2;
	    w->nvrDisplay.dest_height = w->nvrDisplay.src_height;
	}
	else
	{
	    /* See above symptom assault note */
	    /* y = w->core.y; */
	    w->nvrDisplay.dest_y = 0;
	    w->nvrDisplay.dest_height = w->core.height;
	}
	(renderProc)(w);
    }
}

static void
nvrRenderDisplay(XnvrDisplayWidget w)
{
    Msg("XnvrDisplayWidget::nvrRenderDisplay\n");
    Msg("	This routine MUST be implemented by the subclass!!\n");
}


static void
nvrRedisplay(Widget pubw, XEvent *event, Region region)
{
    XnvrDisplayWidget w = (XnvrDisplayWidget)pubw;

    WhenDebugEnabled( NormalOutput("XnvrDisplayWidget::nvrRedisplay\n") );
    if (XtIsRealized(w) && !w->core.being_destroyed)
    {
	XtCallCallbackList((Widget)w, w->nvrDisplay.exposeCallback, NULL);
    }
    if (XtIsRealized(w) && !w->core.being_destroyed && w->nvrDisplay.isActive)
    {
	if (!w->nvrDisplay.alreadyRedisplayed)
	{
	    w->nvrDisplay.alreadyRedisplayed = True;
	    nvrUpdateDisplay(w, nvrRenderDisplay);
	}
    }
}

static void
nvrResize(Widget pubw)
{
    XnvrDisplayWidget w = (XnvrDisplayWidget)pubw;
    int origin_x, origin_y, extent_x, extent_y;

    WhenDebugEnabled(
	NormalOutput("XnvrDisplayWidget::nvrResize\n");
	NormalOutput("\t(%d,%d) %dx%d\n",
	    w->core.x, w->core.y, w->core.width, w->core.height);
    )
    origin_x = w->nvrDisplay.dest_x;
    origin_y = w->nvrDisplay.dest_y;
    extent_x = w->nvrDisplay.dest_x + w->nvrDisplay.dest_width;
    extent_y = w->nvrDisplay.dest_y + w->nvrDisplay.dest_height;

    if (extent_x < w->core.width)
    {
	w->nvrDisplay.dest_x += w->core.width - extent_x;
    }
    if (origin_x > w->core.x)
    {
	w->nvrDisplay.dest_x = w->core.x;
    }
    if (extent_y < w->core.height)
    {
	w->nvrDisplay.dest_y += w->core.height - extent_y;
    }
    if (origin_y > w->core.y)
    {
	w->nvrDisplay.dest_y = w->core.y;
    }
    w->nvrDisplay.alreadyRedisplayed = False;
}

static Boolean
nvrSetValues (Widget pubCurrent,
	      Widget pubReq,
	      Widget pubNew,
	      ArgList args,
	      Cardinal *numArgs)
{
    XnvrDisplayWidget new = (XnvrDisplayWidget)pubNew;

    WhenDebugEnabled( NormalOutput("XnvrDisplayWidget::nvrSetValues\n") );

    /* Video control resources */

    /* keep these within the range */
    if (new->nvrDisplay.hue.cur < new->nvrDisplay.hue.min) 
	new->nvrDisplay.hue.cur = new->nvrDisplay.hue.min;
    if (new->nvrDisplay.hue.cur > new->nvrDisplay.hue.count)
	new->nvrDisplay.hue.cur = new->nvrDisplay.hue.count;
    if (new->nvrDisplay.sat.cur < new->nvrDisplay.sat.min) 
	new->nvrDisplay.sat.cur = new->nvrDisplay.sat.min;
    if (new->nvrDisplay.sat.cur > new->nvrDisplay.sat.count)
	new->nvrDisplay.sat.cur = new->nvrDisplay.sat.count;
    if (new->nvrDisplay.contrast.cur < new->nvrDisplay.contrast.min) 
	new->nvrDisplay.contrast.cur = new->nvrDisplay.contrast.min;
    if (new->nvrDisplay.contrast.cur > new->nvrDisplay.contrast.count)
	new->nvrDisplay.contrast.cur = new->nvrDisplay.contrast.count;
    if (new->nvrDisplay.bright.cur < new->nvrDisplay.bright.min) 
	new->nvrDisplay.bright.cur = new->nvrDisplay.bright.min;
    if (new->nvrDisplay.bright.cur > new->nvrDisplay.bright.count)
	new->nvrDisplay.bright.cur = new->nvrDisplay.bright.count;

    return(False);
}



/* The renderProc argument is added only to pass it to UpdateDisplay */
static void
nvrPanDisplay (XnvrDisplayWidget w,
	       XEvent *event,
	       XnvrRenderProc renderProc)
{
    Display *dpy;
    Window root, window, child;
    unsigned int mask;
    int px, py;		/* x,y of pointer location (relative to this window) */
    int cx, cy;		/* amount of x,y movement of the pointer */
    int wx, wy;		/* x,y relative to this widget's window origin */
    int rx, ry;		/* x,y relative to the root window origin */
    GC gc =  w->nvrDisplay.renderGC;
    int origin_x, origin_y, extent_w, extent_h;

    WhenDebugEnabled( NormalOutput("XnvrDisplayWidget::nvrPanDisplay\n") );

    dpy = XtDisplay(w);
    window = XtWindow(w);
    px = event->xbutton.x;
    py = event->xbutton.y;
    while (1)
    {
	XQueryPointer(dpy,window,&root,&child,&rx,&ry, &wx,&wy,&mask);
	if (!(mask & Button1Mask)) break;
	if (w->nvrDisplay.resizeMode != XnvrResizeModeClipAndPan) break;

	/* This is the amount of movement requested (i.e. the amount
	 * of x,y movement of the pointer.
	 */
	cx = wx - px;
	cy = wy - py;

	/* Restrict the movement such that there is never blank space
	 * around the image (i.e. grant only a reduced amount of actual
	 * movement).
	 */
	origin_x = w->nvrDisplay.src_x + cx;
	origin_y = w->nvrDisplay.src_y + cy;
	extent_w = w->nvrDisplay.src_x + cx + w->nvrDisplay.src_width;
	extent_h = w->nvrDisplay.src_y + cy + w->nvrDisplay.src_height;
	if (w->nvrDisplay.src_width > w->core.width)
	{
	    if (extent_w < w->core.width) cx += w->core.width - extent_w;
	    if (origin_x > w->core.x) cx -= origin_x - w->core.x;
	}
	else
	{
	    if (extent_w > w->core.width) cx -= extent_w - w->core.width;
	    if (origin_x < w->core.x) cx += w->core.x - origin_x;
	}
	if (w->nvrDisplay.src_height > w->core.height)
	{
	    if (extent_h < w->core.height) cy += w->core.height - extent_h;
	    if (origin_y > w->core.y) cy -= origin_y - w->core.y;
	}
	else
	{
	    if (extent_h > w->core.height) cy -= extent_h - w->core.height;
	    if (origin_y < w->core.y) cy += w->core.y - origin_y;
	}

	if (cx || cy || !w->nvrDisplay.isActive)
	{
	    w->nvrDisplay.dest_x = w->nvrDisplay.src_x + cx;
	    w->nvrDisplay.dest_y = w->nvrDisplay.src_y + cy;
	    w->nvrDisplay.dest_width = w->nvrDisplay.src_width;
	    w->nvrDisplay.dest_height = w->nvrDisplay.src_height;
	    nvrUpdateDisplay(w, renderProc);
	}

	/* Now erase any areas where the image was but ain't no more. */
	if (cx > 0)
	{
	    XFillRectangle(dpy, window, gc,
			w->nvrDisplay.src_x, w->nvrDisplay.src_y,
			cx, w->nvrDisplay.src_height);
	}
	else if (cx < 0)
	{
	    XFillRectangle(dpy, window, gc,
		    w->nvrDisplay.src_width + w->nvrDisplay.src_x + cx-1,
		    w->nvrDisplay.src_y,
		    -cx+2, w->nvrDisplay.src_height);
	}
	if (cy > 0)
	{
	    XFillRectangle(dpy, window, gc,
		    w->nvrDisplay.src_x, w->nvrDisplay.src_y,
		    w->nvrDisplay.src_width, cy);
	}
	else if (cy < 0)
	{
	    XFillRectangle(dpy, window, gc,
		    w->nvrDisplay.src_x,
		    w->nvrDisplay.src_height + w->nvrDisplay.src_y + cy-1,
		    w->nvrDisplay.src_width, -cy+1);
	}

	w->nvrDisplay.src_x += cx;
	w->nvrDisplay.src_y += cy;
	px = wx;
	py = wy;
    }
}



/* Default translations and action records */
static char nvrDefaultTranslation[] =
    "<Btn1Down>:    pan()";

static XtActionsRec nvrActionsList[] =
{
    {"pan",	    (XtActionProc)nvrPanDisplay}
};

/* Resource definitions for NvrDisplay Widget */
static XtResource nvrResources[] =
{
    {   XnvrNexposeCallback, XnvrCexposeCallback, XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrDisplayWidget, nvrDisplay.exposeCallback),
	XmRImmediate, (XtPointer) NULL },
    {   XnvrNreasonCallback, XnvrCReasonCallback, XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrDisplayWidget, nvrDisplay.reasonCallback),
	XmRImmediate, (XtPointer) NULL },
    {   XnvrNrealizeCallback, XnvrCrealizeCallback, XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrDisplayWidget, nvrDisplay.realizeCallback),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNhue, XnvrChue, XmRInt,
	sizeof (int), XtOffset(XnvrDisplayWidget,nvrDisplay.hue.cur),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNminHue, XnvrCminHue, XmRInt,
	sizeof (int), XtOffset (XnvrDisplayWidget, nvrDisplay.hue.min),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNmaxHue, XnvrCmaxHue, XmRInt,
	sizeof (int), XtOffset (XnvrDisplayWidget, nvrDisplay.hue.count),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNnormalHue, XnvrCnormalHue, XmRInt,
	sizeof (int), XtOffset(XnvrDisplayWidget,nvrDisplay.hue.norm),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNsaturation, XnvrCsaturation, XmRInt,
	sizeof (int), XtOffset(XnvrDisplayWidget,nvrDisplay.sat.cur),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNminSaturation, XnvrCminSaturation, XmRInt,
	sizeof (int), XtOffset (XnvrDisplayWidget, nvrDisplay.sat.min),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNmaxSaturation, XnvrCmaxSaturation, XmRInt,
	sizeof (int), XtOffset (XnvrDisplayWidget, nvrDisplay.sat.count),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNnormalSaturation, XnvrCnormalSaturation, XmRInt,
	sizeof (int), XtOffset (XnvrDisplayWidget, nvrDisplay.sat.norm),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNcontrast, XnvrCcontrast, XmRInt,
	sizeof (int),
	XtOffset(XnvrDisplayWidget, nvrDisplay.contrast.cur),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNminContrast, XnvrCminContrast, XmRInt,
	sizeof (int),
	XtOffset (XnvrDisplayWidget, nvrDisplay.contrast.min),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNmaxContrast, XnvrCmaxContrast, XmRInt,
	sizeof (int),
	XtOffset (XnvrDisplayWidget, nvrDisplay.contrast.count),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNnormalContrast, XnvrCnormalContrast, XmRInt, sizeof (int),
	XtOffset (XnvrDisplayWidget, nvrDisplay.contrast.norm),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNbrightness, XnvrCbrightness, XmRInt,
	sizeof (int),
	XtOffset (XnvrDisplayWidget, nvrDisplay.bright.cur),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNminBrightness, XnvrCminBrightness, XmRInt,
	sizeof (int),
	XtOffset (XnvrDisplayWidget, nvrDisplay.bright.min),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNmaxBrightness, XnvrCmaxBrightness, XmRInt,
	sizeof (int),
	XtOffset (XnvrDisplayWidget, nvrDisplay.bright.count),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNnormalBrightness, XnvrCnormalBrightness, XmRInt,
	sizeof (int),
	XtOffset (XnvrDisplayWidget, nvrDisplay.bright.norm),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNresizeMode, XnvrCresizeMode, XmRInt,
	sizeof (int),
	XtOffset (XnvrDisplayWidget, nvrDisplay.resizeMode),
	XmRImmediate, (XtPointer) XnvrDefaultResizeMode },

    {	XnvrNisEnabled, XnvrCisEnabled, XmRBool,
	sizeof (char),
	XtOffset (XnvrDisplayWidget, nvrDisplay.isEnabled),
	XmRImmediate, (XtPointer) True },

    {	XnvrNisActive, XnvrCisActive, XmRBool,
	sizeof (char),
	XtOffset (XnvrDisplayWidget, nvrDisplay.isActive),
	XmRImmediate, (XtPointer) XnvrDefaultIsActive },

    {	XnvrNnumberOfPorts, XnvrCnumberOfPorts, XmRInt,
	sizeof (int), XtOffset (XnvrDisplayWidget, nvrDisplay.n_ports),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNportIndex, XnvrCportIndex, XmRInt,
	sizeof (int),
	XtOffset (XnvrDisplayWidget, nvrDisplay.port_index),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNportNames, XnvrCportNames, XmRPointer,
	sizeof (int),
	XtOffset (XnvrDisplayWidget, nvrDisplay.portNames),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNengineCount, XnvrCengineCount, XmRInt,
	sizeof (int),
	XtOffset (XnvrDisplayWidget, nvrDisplay.n_engines),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNengine, XnvrCengine, XmRInt,
	sizeof (int),
	XtOffset (XnvrDisplayWidget, nvrDisplay.engine),
	XmRImmediate, (XtPointer) NULL },

    {	XmNvisual, XmCVisual, XmRPointer,
	sizeof (Visual *),
	XtOffset (XnvrDisplayWidget, nvrDisplay.visual),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNdisplayType, XnvrCDisplayType, XmRInt,
	sizeof (int), XtOffset(XnvrDisplayWidget,nvrDisplay.type),
	XmRImmediate, (XtPointer) XnvrUnknownDpy },
};

XnvrDisplayClassRec xnvrDisplayClassRec =
{
    {
	/* superclass	    */	    (WidgetClass) &coreClassRec,
	/* class_name	    */	    "XnvrDisplay",
	/* widget_size	    */	    sizeof(XnvrDisplayRec),
	/* class_initialize */	    NULL,
	/* chained class init */    nvrClassPartInitialize,
	/* class_inited	    */	    FALSE,
	/* initialize	    */	    nvrInitialize,
	/* initialize hook  */	    NULL,
	/* realize	    */	    nvrRealize,
	/* actions	    */	    nvrActionsList,
	/* num_actions	    */	    XtNumber(nvrActionsList),
	/* resources	    */	    nvrResources,
	/* num_resources    */	    XtNumber(nvrResources),
	/* xrm_class	    */	    NULLQUARK,
	/* compress_motion  */	    TRUE,
	/* compress_exposure*/	    TRUE,
	/* compress enter/exit */   TRUE,
	/* visible_interest */	    FALSE,
	/* destroy	    */	    nvrDestroy,
	/* resize	    */	    nvrResize,
	/* expose	    */	    nvrRedisplay,
	/* set_values	    */	    nvrSetValues,
	/* set values hook  */	    NULL,
	/* set values almost*/	    XtInheritSetValuesAlmost,
	/* get values hook  */	    NULL,
	/* accept_focus	    */	    NULL,
	/* version	    */	    XtVersion,
	/* callback offset 1st */   NULL,
	/* default trans    */	    nvrDefaultTranslation,
	/* query geo proc   */	    NULL,
	/* display accelerator */   NULL,
	/* reserved	    */	    NULL	/* extension record */
    },

    {					    /* XnvrDisplay */
	/* updateDisplay    */	    nvrUpdateDisplay,
	/* renderDisplay    */	    nvrRenderDisplay,
	/* panDisplay	    */	    nvrPanDisplay,
	NULL,					/* extension */
    }
};

WidgetClass xnvrDisplayWidgetClass = (WidgetClass) &xnvrDisplayClassRec;

