/*
 * (c) Copyright 1992-1995 North Valley Research, Inc.
 *      All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *      NvrMpegDisplay.c - Implementation of mpeg playback window
 *			   without controls
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/NvrMpegDisplay.c,v 1.9 1996/01/17 00:08:10 toddb Exp $
 */

/*
 * Header Files
 */
#include "nvrAlloc.h"
#include "nvrString.h"
#include "WidgetDialog.h"
#include "NvrMpegDisplayP.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <Xm/Xm.h>
#include <Xm/XmP.h>

/*
 * Resource List
 */
static XtResource resources[] =
{
    {
	XnvrNStateCallback, XnvrCStateCallback, XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrMpegDisplayWidget, mpeg.pub.stateCallback),
	XmRImmediate, (XtPointer) NULL
    },
    {
	XmNexposeCallback, XmCExposeCallback, XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrMpegDisplayWidget, mpeg.pub.exposeCallback),
	XmRImmediate, (XtPointer) NULL
    },
    {
	XnvrNAction, XnvrCAction, XtRInt,
	sizeof(NvrPlayerAction),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.action),
	XmRImmediate, (XtPointer)0
    },
    {
	XnvrNVideoChannel, XnvrCVideoChannel, XtRInt,
	sizeof(int),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.vChan),
	XmRImmediate, (XtPointer)FirstAvailableChannel
    },
    {
	XnvrNAudioChannel, XnvrCAudioChannel, XtRInt,
	sizeof(int),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.aChan),
	XmRImmediate, (XtPointer)FirstAvailableChannel
    },
    {
	XnvrNAudioQuality, XnvrCAudioQuality, XtRInt,
	sizeof(int),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.aQual),
	XmRImmediate, (XtPointer)0
    },
    {
	XnvrNAudioQualitySteps, XnvrCAudioQualitySteps, XtRInt,
	sizeof(int),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.aQualSteps),
	XmRImmediate, (XtPointer)0	/* minimum quality */
    },
    {
	XnvrNDecodeWindow, XnvrCDecodeWindow, XtRInt,
	sizeof(int),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.decodeWindow),
	XmRImmediate, (XtPointer)0
    },
    {
	XnvrNDepth, XnvrCDepth, XtRInt,
	sizeof(NvrDepth),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.depth),
	XmRImmediate, (XtPointer)NvrGray7
    },
    {
	XnvrNDepths, XnvrCDepths, XtRInt,
	sizeof(NvrDepth),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.depths),
	XmRImmediate, (XtPointer)0
    },
    {
	XnvrNInputFile, XnvrNInputFile, XtRInt,
	sizeof(NvrInputSpecPtr),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.pInput),
	XmRImmediate, (XtPointer)0
    },
    {
	XnvrNSeekPoint, XnvrCSeekPoint, XtRInt,
	sizeof(int),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.seekPoint),
	XmRImmediate, (XtPointer)0
    },
    {
	XnvrNSizeScale, XnvrCSizeScale, XtRInt,
	sizeof(NvrSizeScale),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.sizeScale),
	XmRImmediate, (XtPointer)NvrSize1x
    },
    {
	XnvrNState, XnvrCState, XtRInt,
	sizeof(NvrPlayerState),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.cb.playState),
	XmRImmediate, (XtPointer)NvrStopped
    },
    {	XmNvisual, XmCVisual, XmRPointer,
	sizeof (Visual *),
	XtOffset (XnvrMpegDisplayWidget, mpeg.pub.visual),
	XmRImmediate, (XtPointer) NULL
    },
    {
	XnvrNGamma, XnvrCGamma, XtRInt,
	sizeof(int),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.gamma),
	XmRImmediate, (XtPointer)NvrDefaultGamma
    },
    {
	XnvrNTrueColor12Bit, XnvrCTrueColor12Bit, XmRBool,
	sizeof(Bool),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.trueColor12Bit),
	XmRImmediate, (XtPointer)False
    },
    {
	XnvrNAlignWindow, XnvrCAlignWindow, XmRBool,
	sizeof(Bool),
	XtOffset(XnvrMpegDisplayWidget, mpeg.pub.alignWindow),
	XmRImmediate,
#ifdef sun
	(XtPointer)True
#else
	(XtPointer)False
#endif
    },
};


/*
 * Actions Table
 */
#ifdef ACTIONS
static XtActionsRec ActionsList[] =
{
	{"junk",	(XtActionProc)Junk}
};
#endif /* ACTIONS */


char *DepthTypeNames[] = { 
    "DepthUnspecified",
    "Rgb888",
    "Rgb444",
    "Rgb555",
    "Rgb232",
    "Rgb332",
    "GrayScale8",
    "GrayScale7",
    "GrayScale1",
    "RgbC555",
    "RgbC666",
};

char *ColorClasses[] = { 
    "StaticGray",
    "GrayScale",
    "StaticColor",
    "PseudoColor",
    "TrueColor",
    "DirectColor" };

/*
 * Necessary forward declaration
 */
static Boolean CheckAcceleration(Display *dpy, XVisualInfo *pvi);

static void
ChangeContrast(XnvrMpegDisplayWidget cw, int value)
{
    ViewAttributes viewAttr;
    MpegPerDepthPtr pd = &cw->mpeg.priv.perDepth[ cw->mpeg.priv.depthType ];

    cw->mpeg.pub.contrast = viewAttr.contrast = value;
    SetViewAttributes(&pd->view, &viewAttr, CONTRAST_MASK);
}

static void
ChangeBrightness(XnvrMpegDisplayWidget cw, int value)
{
    ViewAttributes viewAttr;
    MpegPerDepthPtr pd = &cw->mpeg.priv.perDepth[ cw->mpeg.priv.depthType ];

    cw->mpeg.pub.brightness = viewAttr.brightness = value;
    SetViewAttributes(&pd->view, &viewAttr, BRIGHTNESS_MASK);
}

static void
ChangeGamma(XnvrMpegDisplayWidget cw, int value)
{
    ViewAttributes viewAttr;
    MpegPerDepthPtr pd = &cw->mpeg.priv.perDepth[ cw->mpeg.priv.depthType ];

    cw->mpeg.pub.gamma = value;
    viewAttr.dpyGamma = (double)value/NvrGammaDivisor;
    SetViewAttributes(&pd->view, &viewAttr, DISPLAY_GAMMA_MASK);
}

static void
ChangeWindowSize(XnvrMpegDisplayWidget w)
{
    XWindowChanges changes;
    UInt mask = CWWidth | CWHeight;

    if (w->mpeg.pub.decodeWindow == 0)
	return;
    changes.width = w->core.width;
    changes.height = w->core.height;
    XConfigureWindow(XtDisplay((Widget)w), 
		     w->mpeg.pub.decodeWindow,
		     mask,
		     &changes);
}

static DualInputSpecPtr
AllocInputSpec(NvrInputSpecPtr input, InternalReasonPtr *pIrp)
{
    int videoLen, audioLen, hostLen, allocSize;
    DualInputSpecPtr dual;
    char *p;

    hostLen = videoLen = audioLen = 0;
    switch (input->type) {
    case NvrInputUnspecified:
	*pIrp = BadValueReason("input specified as \"unspecified\".\n");;
	return(NULL);
	break;

    case NvrInputSinglePath:
	videoLen = strlen(input->single.path) + 1;
	break;

    case NvrInputPathPair:
	videoLen = strlen(input->pathPair.video);
	audioLen = strlen(input->pathPair.audio);
	break;

    case NvrInputTcpPort:
	if (input->net.host)
	    hostLen = strlen(input->net.host);
	break;

    case NvrInputDescriptorPair:
    case NvrInputDescriptor:
	break;
    }

    allocSize = sizeof(DualInputSpec) + videoLen + audioLen + hostLen;
    dual = (DualInputSpecPtr)AllocNamedMem(allocSize,
					   "widget input spec", pIrp);
    if (dual == NULL)
        return(NULL);

    p = (char *)(dual + 1);
    switch (input->type) {
    default:
	break;		/* handled above */

    case NvrInputSinglePath:
	dual->widgetSpec.single.path = p;
	strcpy(dual->widgetSpec.single.path, input->single.path);
	dual->schedSpec.type = MSIT_SinglePath;
	dual->schedSpec.single.path = dual->widgetSpec.single.path;
	break;

    case NvrInputPathPair:
	dual->widgetSpec.pathPair.video = p;
	p += videoLen;
	dual->widgetSpec.pathPair.audio = p;
	strcpy(dual->widgetSpec.pathPair.video, input->pathPair.video);
	strcpy(dual->widgetSpec.pathPair.audio, input->pathPair.audio);
	dual->schedSpec.type = MSIT_PathPair;
	dual->schedSpec.pathPair.audio = dual->widgetSpec.pathPair.audio;
	dual->schedSpec.pathPair.video = dual->widgetSpec.pathPair.video;
	break;

    case NvrInputDescriptor:
	dual->widgetSpec.descriptor.fd = input->descriptor.fd;
	dual->schedSpec.type = MSIT_Descriptor;
	dual->schedSpec.descriptor.fd = dual->widgetSpec.descriptor.fd;
	break;

    case NvrInputDescriptorPair:
	dual->widgetSpec.descriptorPair.afd = input->descriptorPair.afd;
	dual->widgetSpec.descriptorPair.vfd = input->descriptorPair.vfd;
	dual->schedSpec.type = MSIT_DescriptorPair;
	dual->schedSpec.descriptorPair.afd = input->descriptorPair.afd;
	dual->schedSpec.descriptorPair.vfd = input->descriptorPair.vfd;
	break;

    case NvrInputTcpPort:
	if (input->net.host) {
	    dual->widgetSpec.net.host = p;
	    dual->schedSpec.net.host = p;
	    strcpy(p, input->net.host);
	}
	dual->widgetSpec.net.port = input->net.port;
	dual->schedSpec.net.port = input->net.port;
	dual->schedSpec.type = MSIT_TcpPort;
	break;
    }
    dual->widgetSpec.type = input->type;

    return(dual);
}

static void
ConfigInputSpec(XnvrMpegDisplayWidget wnew,
		XnvrMpegDisplayWidget wcur,
		DualInputSpecPtr dual)
{
    InternalReasonPtr irp;
    MpegSchedConf conf;

    conf.input = &dual->schedSpec;
    irp = (*wnew->mpeg.priv.sched->config)(wnew->mpeg.priv.sched,
					   MSM_Input, &conf);
    if (irp) {
	AppendReason(&wnew->mpeg.priv.irList, irp);
	free((char *)dual);
	return;
    }

    if (wcur->mpeg.priv.dualSpec)
	free((char *)wcur->mpeg.priv.dualSpec);
    wcur->mpeg.priv.dualSpec = dual;

    /*
     * Then reset pInput back to our idea of the current sequence
     */
    wnew->mpeg.pub.pInput = &wnew->mpeg.priv.dualSpec->widgetSpec;
}

static void
ChangeDepthType(XnvrMpegDisplayWidget cw)
{
    ViewAttributes viewAttr;
    MpegPerDepthPtr pd = &cw->mpeg.priv.perDepth[ cw->mpeg.priv.depthType ];
    Display *dpy = XtDisplay(cw);
    InternalReasonPtr irp;

    if (cw->mpeg.pub.decodeWindow)
	XUnmapWindow(dpy, cw->mpeg.pub.decodeWindow);

    cw->mpeg.pub.decodeWindow = pd->win;

    if (0)
	Msg("switch to %s, ", DepthTypeNames[ cw->mpeg.priv.depthType ]);
    if (0 && pd->win)
	Msg("win=0x%x", pd->win);
    else if (0)
	Msg("no win\n");

    if (pd->win) {
	ChangeWindowSize(cw);
	SetViewAttributes(&pd->view, &viewAttr, LOAD_COLORS_MASK);
	XMapWindow(dpy, pd->win);
	cw->mpeg.priv.dpyParam.u.wsys.win = (void *)pd->win;
	cw->mpeg.priv.dpyParam.u.wsys.dpy = (void *)XtDisplay(cw);
	cw->mpeg.priv.dpyParam.color = cw->mpeg.priv.depthType;
	irp = InitImageRegion(cw->mpeg.priv.dpy,
			      &cw->mpeg.priv.dpyParam,
			      ID_DepthType | ID_Window);
	if (irp)
	    AppendReason(&cw->mpeg.priv.irList, irp);
    }
}

static void
SetPerDisplay(XnvrMpegDisplayWidget w,
	      RgbDepthType type,
	      XVisualInfo *pvi,
	      int depth,
	      Boolean accelerated)
{
    MpegPerDepthPtr pd = &w->mpeg.priv.perDepth[ type ];

    if (0) {
	Msg("pick: vis=0x%x, class=%s, depth=%d%s%s\n",
	    pvi->visual->visualid, ColorClasses[ pvi->visual->class ], depth,
	    accelerated ? ", accelerated" : "",
	    pd->view.visual ? " (override previous)" : "");
    }

    pd->view.visual = pvi->visual;
    pd->depth = depth;
    pd->accelerated = accelerated;
}

#if 0
static int
FindFirstSet(long value)
{
    int i, bit;

    for (i = 0, bit = 1; i < 32; i++, bit <<= 1) {
        if (value & bit)
            return(bit);
    }
    return(0);
}
#endif

static void
SelectChannels(XnvrMpegDisplayWidget w)
{
    InternalReasonPtr irp;
    MpegSchedConf conf;
    ULong mask;

    conf.vChan = w->mpeg.priv.vChanReq;
    conf.aChan = w->mpeg.priv.aChanReq;

    mask = MSM_VideoChan | MSM_AudioChan;
    irp = (*w->mpeg.priv.sched->config)(w->mpeg.priv.sched, mask, &conf);
    if (irp)
	AppendReason(&w->mpeg.priv.irList, irp);
}

static Boolean
BackgroundProc(XtPointer workData)
{
    XnvrMpegDisplayWidget w = (XnvrMpegDisplayWidget)workData;

    if (w->mpeg.pub.cb.cbMask) {
	XtCallCallbackList((Widget)w, w->mpeg.pub.stateCallback,
			   &w->mpeg.pub.cb);
	w->mpeg.pub.cb.cbMask = 0;
    }

    if (w->mpeg.priv.irList) {
	ReasonDialog((Widget)w, NULL, w->mpeg.priv.irList);
	w->mpeg.priv.irList = NULL;
    }

    if (w->mpeg.priv.sched->needToProcess)
	(*w->mpeg.priv.sched->process)(w->mpeg.priv.sched);

    return(False);       /* never done */
}

static void
InstallNewAudioForm(XnvrMpegDisplayWidget w, MpegSchedCbPtr scbData)
{
    InternalReasonPtr irp;
    MpegSchedConf conf;
    AudioForm dstAu;

    w->mpeg.priv.auInForm = scbData->auInput;
    irp = ChooseAudioForm(&w->mpeg.priv.auInForm,
			  w->mpeg.priv.auOutput->u.devFormats,
			  &dstAu);
    if (irp) {
	AppendReason(&w->mpeg.priv.irList, irp);
	return;
    }

    /*
     * XXX at this point, we should flush the current audio.
     * Set the device to the new hardware settings
     */
    (void) InitAudioDevice(w->mpeg.priv.auOutput, &dstAu);

    conf.auFile = w->mpeg.priv.auOutput;
    conf.aQual = w->mpeg.pub.aQual;
    irp = (*w->mpeg.priv.sched->config)(w->mpeg.priv.sched,
					MSM_AudioOutput | MSM_AudioQuality,
					&conf);
    if (irp)
	AppendReason(&w->mpeg.priv.irList, irp);

    w->mpeg.pub.cb.sampleRate = w->mpeg.priv.auOutput->au.rate;
    w->mpeg.pub.cb.channels = w->mpeg.priv.auOutput->au.channels;
    w->mpeg.pub.cb.cbMask |= NvrAudioChange;
}

static void
SchedCb(MpegSchedPtr sched, MpegSchedCbPtr scbData, MS_CallbackType mask)
{
    XnvrMpegDisplayWidget w = (XnvrMpegDisplayWidget)scbData->cbData;
    InternalReasonPtr irp;

    w->mpeg.priv.schedCbSeen |= mask;

    if (mask & MSCB_NewVideo) {
	w->mpeg.pub.cb.cbMask |= NvrNewVideo;
    }

    if (mask & MSCB_VidChange) {
	w->mpeg.priv.dpyParam.src.width = scbData->width;
	w->mpeg.priv.dpyParam.src.height = scbData->height;
	w->mpeg.priv.dpyParam.src.lineBytes = scbData->width;
	w->mpeg.priv.dpyParam.src.chromaLineBytes = scbData->width >> 1;
	switch (w->mpeg.pub.sizeScale) {
	case NvrSize1x:
	    w->mpeg.priv.dpyParam.dst.doubleSize = False;
	    break;
	case NvrSize4x:
	    w->mpeg.priv.dpyParam.dst.doubleSize = True;
	    break;
	case NvrSizeQuarter:
	    break;		/* XXX unsupported just now */
	}

	irp = InitImageRegion(w->mpeg.priv.dpy,
			      &w->mpeg.priv.dpyParam, ID_SrcInfo);
	if (irp)
	    AppendReason(&w->mpeg.priv.irList, irp);

	w->mpeg.pub.cb.width = scbData->width;
	w->mpeg.pub.cb.height = scbData->height;
	w->mpeg.pub.cb.cbMask |= NvrVideoChange;
    }

    if (mask & MSCB_NewAudio) {
	w->mpeg.pub.cb.cbMask |= NvrNewAudio;
    }

    if (mask & MSCB_AuChange) {
	AudioFormPtr oldAu = &w->mpeg.priv.auInForm;

	if (scbData->auInput.dataForm != oldAu->dataForm
	 || scbData->auInput.channels != oldAu->channels
	 || scbData->auInput.rate     != oldAu->rate) {
	    InstallNewAudioForm(w, scbData);
        }
    }

    if (mask & MSCB_PlayState) {
        switch (scbData->playState) {
        case MSPS_Playing: w->mpeg.pub.cb.playState = NvrPlaying; break;
        case MSPS_Stopped: w->mpeg.pub.cb.playState = NvrStopped; break;
        case MSPS_Seeking: w->mpeg.pub.cb.playState = NvrSeeking; break;
        }
	w->mpeg.pub.cb.cbMask |= NvrNewPlayState;
    }

    if (mask & MSCB_Reason) {
	AppendReason(&w->mpeg.priv.irList, scbData->reason);
    }

    if (mask & MSCB_Position) {
        Msg("frame %d, time %.3f\n", scbData->frame, scbData->time);
    }

    if (mask & MSCB_Eof) {
	w->mpeg.pub.cb.cbMask |= NvrEofReached;
        Msg("eof reached\n", scbData->frame, scbData->time);
    }

    if (mask & MSCB_Channels) {
	w->mpeg.pub.cb.cbMask |= NvrEofReached;
	w->mpeg.pub.aChan = scbData->aChan;
	w->mpeg.pub.vChan = scbData->vChan;
    }
}

static void
InitScheduler(XnvrMpegDisplayWidget w)
{
    MpegSchedConf conf;
    InternalReasonPtr irp;

    conf.cb = SchedCb;
    conf.cbData = (void *)w;
    conf.dpy = w->mpeg.priv.dpy;
    conf.warning = Msg;
    conf.auFile = w->mpeg.priv.auOutput;

    w->mpeg.priv.schedMask = MSM_Callback
			   | MSM_Display
			   | MSM_WarningFunc
			   | MSM_AudioOutput;

    irp = (*w->mpeg.priv.sched->config)(w->mpeg.priv.sched,
					w->mpeg.priv.schedMask, &conf);
    if (irp)
	AppendReason(&w->mpeg.priv.irList, irp);
}

static void
InitImageDisplay(XnvrMpegDisplayWidget w)
{
    MpegPerDepthPtr pd = &w->mpeg.priv.perDepth[ w->mpeg.priv.depthType ];
    InternalReasonPtr irp;

    w->mpeg.priv.dpyParam.u.wsys.win = (void *)pd->win;
    w->mpeg.priv.dpyParam.u.wsys.dpy = (void *)XtDisplay(w);
    w->mpeg.priv.dpyParam.color = w->mpeg.priv.depthType;
    w->mpeg.priv.dpyParam.src.colorSpace = ColorYC;
    w->mpeg.priv.dpyParam.src.sample = Sample4_2_0;
    w->mpeg.priv.dpyParam.src.width = DefaultWidth;
    w->mpeg.priv.dpyParam.src.height = DefaultHeight;
    w->mpeg.priv.dpyParam.src.lineBytes = w->mpeg.priv.dpyParam.src.width;
    w->mpeg.priv.dpyParam.src.chromaLineBytes
	= w->mpeg.priv.dpyParam.src.height >> 1;
    w->mpeg.priv.dpyParam.dst.x = 0;
    w->mpeg.priv.dpyParam.dst.y = 0;
    w->mpeg.priv.dpyParam.dst.doubleSize = False;
    w->mpeg.priv.dpyParam.dst.winClipped = False;

    irp = InitImageRegion(w->mpeg.priv.dpy, &w->mpeg.priv.dpyParam, ID_AllWin);
    if (irp)
	AppendReason(&w->mpeg.priv.irList, irp);
}

static void
InitializeAudio(XnvrMpegDisplayWidget w)
{
    AudioForm srcAu, dstAu;
    InternalReasonPtr irp;

    w->mpeg.priv.auOutput = OpenAudioDevice(NULL, &irp);
    if (w->mpeg.priv.auOutput == NULL) {
	AppendReason(&w->mpeg.priv.irList, irp);
    }
    (void) AudioDeviceMode(w->mpeg.priv.auOutput, DevModeNonBlocking);

    /*
     * Start with the CD rate and find the closest thing that
     * the hardware can do.
     */
    srcAu.rate = Audio44_1Khz;
    srcAu.channels = AudioStereo;
    srcAu.dataForm = Audio16BitPcm;
    irp = ChooseAudioForm(&srcAu, w->mpeg.priv.auOutput->u.devFormats, &dstAu);
    if (irp) {
	AppendReason(&w->mpeg.priv.irList, irp);
	return;
    }

    (void) InitAudioDevice(w->mpeg.priv.auOutput, &dstAu);
}

static void
SetPrivDepthType(XnvrMpegDisplayWidget wcur, XnvrMpegDisplayWidget wnew)
{
    switch (wnew->mpeg.pub.depth) {
    default:
	if (wcur) {
	    wnew->mpeg.pub.depth = wcur->mpeg.pub.depth;
	    SetPrivDepthType(NULL, wnew);
	}
	break;

    case NvrRgb888:	wnew->mpeg.priv.depthType = Rgb888; break;
    case NvrRgb444:	wnew->mpeg.priv.depthType = Rgb444; break;
    case NvrRgb332:	wnew->mpeg.priv.depthType = Rgb332; break;
    case NvrRgb232:	wnew->mpeg.priv.depthType = Rgb232; break;
    case NvrGray8:	wnew->mpeg.priv.depthType = GrayScale8; break;
    case NvrGray7:	wnew->mpeg.priv.depthType = GrayScale7; break;
    case NvrRgbC555:	wnew->mpeg.priv.depthType = RgbC555; break;
    case NvrRgbC666:	wnew->mpeg.priv.depthType = RgbC666; break;
    }
}

/*
 * Method Function Definitions
 */
static void
Initialize(XnvrMpegDisplayWidget wreq,
	   XnvrMpegDisplayWidget wnew)
{
    XVisualInfo *pvi, visualInfoTmpl;
    int visCount, i;
    MpegPerDepthPtr pd, selectedPd;
    Screen *screen = XtScreen(wnew);
    Display *dpy = XtDisplay(wnew);
    int scrNumber = XScreenNumberOfScreen( screen );
    Boolean accelerated;
    InternalReasonPtr irp;
    XtAppContext context = XtDisplayToApplicationContext(XtDisplay(wnew));

    /*
     * This stops purify from complaining about uninitialized memory
     * reads.
     */
    memset(&wnew->mpeg.priv, 0, sizeof(wnew->mpeg.priv));
    wnew->mpeg.pub.cb.cbMask = 0;

    /* by this time, the superclass' Initialize has already been called */
    wnew->mpeg.priv.dpy = CreateImageRegion(XWindowImageDesc, &irp);
    if (wnew->mpeg.priv.dpy == NULL)
	AppendReason(&wnew->mpeg.priv.irList, irp);

    /*
     * Set up the work proc for this widget
     */
    wnew->mpeg.priv.workId = XtAppAddWorkProc(context,
					      BackgroundProc, (XtPointer)wnew);

    if (NULL == (wnew->mpeg.priv.sched = CreateMpegScheduler(&irp))) {
	AppendReason(&wnew->mpeg.priv.irList, irp);
    }

    InitializeAudio(wnew);
    InitScheduler(wnew);

    /*
     * override the core's idea of width and height
     */
    {
	XtGeometryResult result;
	Dimension width, height;

	result = XtMakeResizeRequest((Widget)wnew, DefaultWidth, DefaultHeight,
				     &width, &height);
	if (result == XtGeometryAlmost)
	    result = XtMakeResizeRequest((Widget)wnew, width, height,
					 &width, &height);
	(*XtClass(wnew)->core_class.resize) ((Widget)wnew);
    }

    visualInfoTmpl.screen = scrNumber;
    pvi = XGetVisualInfo(dpy, VisualScreenMask, &visualInfoTmpl, &visCount);

    pd = wnew->mpeg.priv.perDepth;
    memset((char *)pd, 0, sizeof(wnew->mpeg.priv.perDepth));
    for (i = 0; i < visCount; i++) {
	accelerated = CheckAcceleration(dpy, &pvi[ i ]);
	switch (pvi[i].depth) {
	case 1:
	    SetPerDisplay(wnew, GrayScale1, &pvi[ i ], 1, accelerated);
	    break;
	
	case 8:
	    switch (pvi[i].class) {
	    case PseudoColor:
		/*
		 * Take the first one.  (the second one on the
		 * SGI machines only yields 255 colors, instead of 256).
		 */
		if (pd[ Rgb232 ].view.visual
		 && (pd[ Rgb232 ].accelerated || !accelerated))
		    break;

		SetPerDisplay(wnew, RgbC555, &pvi[ i ], 8, accelerated);
		SetPerDisplay(wnew, RgbC666, &pvi[ i ], 8, accelerated);
		SetPerDisplay(wnew, Rgb232, &pvi[ i ], 8, accelerated);
		SetPerDisplay(wnew, Rgb332, &pvi[ i ], 8, accelerated);
		SetPerDisplay(wnew, GrayScale7, &pvi[ i ], 8, accelerated);
		SetPerDisplay(wnew, GrayScale8, &pvi[ i ], 8, accelerated);
		break;

	    case GrayScale:
		/*
		 * We prefer the pseudo color
		 */
		if (pd[ GrayScale7 ].view.visual
		 && (pd[ GrayScale7 ].accelerated || !accelerated))
		    break;
		SetPerDisplay(wnew, GrayScale7, &pvi[ i ], 8, accelerated);
		SetPerDisplay(wnew, GrayScale8, &pvi[ i ], 8, accelerated);
		break;

	    default:
		break;	/* nothing useful */
	    }
	    break;

	case 12:
	    switch (pvi[i].class) {
	    case PseudoColor:
		if (wnew->mpeg.pub.trueColor12Bit)
		    break;
		if (pd[ Rgb444 ].view.visual) {
		    if (pd[ Rgb444 ].view.visual->visualid == DirectColor
		     && pd[ Rgb444 ].accelerated)
			break; /* we prefer DirectColor */
		    if (pd[ Rgb444 ].accelerated || ! accelerated)
			break; /* we prefer accelerated */
		}
		SetPerDisplay(wnew, Rgb444, &pvi[ i ], 12, accelerated);
		break;

	    case TrueColor:
		if (! wnew->mpeg.pub.trueColor12Bit)
		    break;
		if (pd[ Rgb444 ].view.visual) {
		    if (pd[ Rgb444 ].view.visual->visualid == DirectColor
		     && pd[ Rgb444 ].accelerated)
			break; /* we prefer DirectColor */
		    if (pd[ Rgb444 ].view.visual->visualid == PseudoColor
		     && pd[ Rgb444 ].accelerated)
			break; /* we prefer PseudoColor */
		    if (pd[ Rgb444 ].accelerated || ! accelerated)
			break; /* we prefer accelerated */
		}
		/* fall through ... */

	    case DirectColor:
		if (pd[ Rgb444 ].view.visual) {
		    if (pd[ Rgb444 ].accelerated || ! accelerated)
			break; /* we prefer accelerated */
		}
		SetPerDisplay(wnew, Rgb444, &pvi[ i ], 12, accelerated);
		break;

	    default:
		break;	/* nothing useful */
	    }
	    break;

	case 32:
	case 24:
	    switch (pvi[i].class) {
	    case TrueColor:
		if (pd[ Rgb888 ].view.visual) {
		    if (pd[ Rgb888 ].view.visual->visualid == DirectColor
		     && pd[ Rgb888 ].accelerated)
			break; /* we prefer DirectColor */
		    if (pd[ Rgb888 ].accelerated || ! accelerated)
			break; /* we prefer accelerated */
		}
		/* fall through */

	    case DirectColor:
		if (pd[ Rgb888 ].view.visual) {
		    if (pd[ Rgb888 ].accelerated || ! accelerated)
			break; /* we prefer accelerated */
		}
		SetPerDisplay(wnew, Rgb888, &pvi[ i ],
			      pvi[i].depth, accelerated);
		break;

	    default:
		break;	/* nothing useful */
	    }
	    break;
	}
    }

    if (pvi)
	XtFree((char *)pvi);

    /*
     * Initialize the view data.
     */
    for (i = 0; i < XtNumber(wnew->mpeg.priv.perDepth); i++) {
	if (pd[ i ].view.visual == NULL)
	    continue;
	pd[ i ].view.dpy       = dpy;
	pd[ i ].view.scrNumber = scrNumber;
	pd[ i ].view.depthType = i;
	pd[ i ].view.dpyGamma  = (double)wnew->mpeg.pub.gamma/NvrGammaDivisor;
	if (! InitViewCmapInfo(&pd[ i ].view)) {
	    /*
	     * For some reason we can't support this
	     */
	    pd[ i ].view.visual = NULL;
	    continue;
	}
    }

    /*
     * Decide what is available
     */
    wnew->mpeg.priv.availableDepths =
	((pd[ Rgb888 ].view.visual != NULL) << Rgb888)
      | ((pd[ Rgb444 ].view.visual != NULL) << Rgb444)
      | ((pd[ Rgb232 ].view.visual != NULL) << Rgb232)
      | ((pd[ Rgb332 ].view.visual != NULL) << Rgb332)
      | ((pd[ Rgb332 ].view.visual != NULL) << RgbC555)
      | ((pd[ Rgb332 ].view.visual != NULL) << RgbC666)
      | ((pd[ GrayScale1 ].view.visual != NULL) << GrayScale1)
      | ((pd[ GrayScale7 ].view.visual != NULL) << GrayScale7)
      | ((pd[ GrayScale8 ].view.visual != NULL) << GrayScale8);

    wnew->mpeg.pub.depths = 0;
    if (wnew->mpeg.priv.availableDepths & Rgb888) 
	wnew->mpeg.pub.depths |= NvrRgb888;
    if (wnew->mpeg.priv.availableDepths & Rgb444) 
	wnew->mpeg.pub.depths |= NvrRgb444;
    if (wnew->mpeg.priv.availableDepths & Rgb232) 
	wnew->mpeg.pub.depths |= NvrRgb232;
    if (wnew->mpeg.priv.availableDepths & Rgb332) 
	wnew->mpeg.pub.depths |= NvrRgb332;
    if (wnew->mpeg.priv.availableDepths & RgbC555) 
	wnew->mpeg.pub.depths |= NvrRgbC555;
    if (wnew->mpeg.priv.availableDepths & RgbC666) 
	wnew->mpeg.pub.depths |= NvrRgbC666;
    if (wnew->mpeg.priv.availableDepths & GrayScale7) 
	wnew->mpeg.pub.depths |= NvrGray7;
    if (wnew->mpeg.priv.availableDepths & GrayScale8) 
	wnew->mpeg.pub.depths |= NvrGray8;

    SetPrivDepthType(wreq, wnew);   
    selectedPd = &wnew->mpeg.priv.perDepth[ wnew->mpeg.priv.depthType ];
    wnew->mpeg.pub.visual = selectedPd->view.visual;
    wnew->mpeg.pub.contrast = NvrDefaultContrast;
    wnew->mpeg.pub.brightness = NvrDefaultBrightness;
}

static int
ClampValue(int value, int minValue, int maxValue)
{
    if (value > maxValue)
	value = maxValue;
    else if (value < minValue)
	value = minValue;
    return(value);
}

static Boolean
ValueSpecified(char *rName, ArgList args, Cardinal *nargs)
{
    int i;

    for (i = 0; i < *nargs; i++) {
	if (strcmp(rName, args[i].name) == 0)
	    return(True);
    }
    return(False);
}

static Boolean
SetValues(Widget pubcur, Widget pubreq, Widget pubnew,
	  ArgList args, Cardinal *nargs)
{
    XnvrMpegDisplayWidget wcur = (XnvrMpegDisplayWidget) pubcur;
    XnvrMpegDisplayWidget wnew = (XnvrMpegDisplayWidget) pubnew;
    boolean selectChannels = False;
    InternalReasonPtr irp;
    DualInputSpecPtr dual;


    if (ValueSpecified(XnvrNAudioChannel, args, nargs)) {
	selectChannels = True;
	wnew->mpeg.priv.aChanReq = wnew->mpeg.pub.aChan;
    }

    if (ValueSpecified(XnvrNVideoChannel, args, nargs)) {
	selectChannels = True;
	wnew->mpeg.priv.vChanReq = wnew->mpeg.pub.vChan;
    }

    if (selectChannels)
	SelectChannels(wnew);

    /*
     * Query-only resources -- don't allow them to be changed.
     */
    wnew->mpeg.pub.aChan        = wcur->mpeg.pub.aChan;
    wnew->mpeg.pub.vChan        = wcur->mpeg.pub.vChan;
    wnew->mpeg.pub.cb.playState = wcur->mpeg.pub.cb.playState;
    wnew->mpeg.pub.aQualSteps   = wcur->mpeg.pub.aQualSteps;
    wnew->mpeg.pub.decodeWindow = wcur->mpeg.pub.decodeWindow;
    wnew->mpeg.pub.depths       = wcur->mpeg.pub.depths;
    wnew->mpeg.pub.visual       = wcur->mpeg.pub.visual;

    /*
     * now filter the requested values
     */
    wnew->mpeg.pub.gamma = ClampValue(wnew->mpeg.pub.gamma,
				 NvrGammaLow, NvrGammaHigh);

    switch (wnew->mpeg.pub.sizeScale) {
    case NvrSize1x:
    case NvrSize4x:
    case NvrSizeQuarter:
	break;

    default:
	wnew->mpeg.pub.sizeScale = wcur->mpeg.pub.sizeScale;
	break;
    }

    /*
     * if the input spec is non-null, don't allow it to be set to
     * null.
     */
    if (wnew->mpeg.pub.pInput == NULL) {
	wnew->mpeg.pub.pInput = wcur->mpeg.pub.pInput;
    } else if (wnew->mpeg.pub.pInput != wcur->mpeg.pub.pInput) {
	if (NULL == (dual = AllocInputSpec(wnew->mpeg.pub.pInput, &irp))) {
	    AppendReason(&wnew->mpeg.priv.irList, irp);
	} else {
	    ConfigInputSpec(wnew, wcur, dual);
	}
    }

    SetPrivDepthType(wcur, wnew);   

    if (wnew->mpeg.pub.depth != wcur->mpeg.pub.depth) {
	ChangeDepthType(wnew);
    } else {
	/*
	 * Brightness
	 */
	if (wnew->mpeg.pub.brightness != wcur->mpeg.pub.brightness) {
	    ChangeBrightness(wcur, wnew->mpeg.pub.brightness);
	}

	/*
	 * Contrast
	 */
	if (wnew->mpeg.pub.contrast != wcur->mpeg.pub.contrast) {
	    ChangeContrast(wcur, wnew->mpeg.pub.contrast);
	}

	/*
	 * Gamma
	 */
	if (wnew->mpeg.pub.gamma != wcur->mpeg.pub.gamma) {
	    ChangeGamma(wcur, wnew->mpeg.pub.gamma);
	}
    }

    if (ValueSpecified(XnvrNAction, args, nargs)) {
	MpegSchedPtr sched = wnew->mpeg.priv.sched;
	InternalReasonPtr irp;

	switch(wnew->mpeg.pub.action) {
	case NvrStop:    irp = (*sched->stop)(sched); break;
	case NvrPlayFwd: irp = (*sched->playFwd)(sched); break;
	case NvrPlayAll: irp = (*sched->playAll)(sched); break;
	case NvrFastFwd: irp = (*sched->fastFwd)(sched); break;
	case NvrPlayBak: irp = (*sched->playBak)(sched); break;
	case NvrFastBak: irp = (*sched->fastBak)(sched); break;
	case NvrStepFwd: irp = (*sched->stepFwd)(sched); break;
	case NvrStepBak: irp = (*sched->stepBak)(sched); break;
	default:	 irp = BadValueReason("unknown action: %d\n",
				wnew->mpeg.pub.action);
			 break;
	}
	if (irp)
	    AppendReason(&wnew->mpeg.priv.irList, irp);
    }

    return(False);
}


static void
Destroy(XnvrMpegDisplayWidget w)
{
}

#ifdef Solaris1
/**************************************************************************
 *
 * The soaris one implementation uses an older motif library that does
 * not include XtRegisterDrawable().
 *
 **************************************************************************/
static void
XtRegisterDrawable(Display *dpy, Window win, Widget w)
{
    extern void _XtRegisterWindow(Window window, Widget widget);

    _XtRegisterWindow(win, w);
}
#endif /* Solaris1 */

static void
Realize(XnvrMpegDisplayWidget w,
	XtValueMask* widgetValueMask,
	XSetWindowAttributes* attributes)
{
    /* by this time, the superclass' Realize has already been called */
    int i;
    Display *dpy = XtDisplay(w);
    MpegPerDepthPtr pd;
    XtValueMask decodeValueMask;
    XSetWindowAttributes decodeAttr;
    Window widgetWin;

    /*
     * Create the widget's window.
     */
    XtCreateWindow((Widget)w,
		   InputOutput,
		   (Visual *)NULL,
		   *widgetValueMask,
		   attributes);
    widgetWin = w->core.window;

    /*
     * Create a subwindow for each supportable depth.
     */
    pd = w->mpeg.priv.perDepth;
    decodeValueMask = CWBorderPixel
		    | CWColormap
		    | CWBackPixmap
		    | CWBackingStore
		    | CWEventMask;
    decodeAttr.border_pixel = 0;
    decodeAttr.background_pixmap = None;
    decodeAttr.backing_store = NotUseful;
    decodeAttr.event_mask = ExposureMask;

    for (i = 0; i < XtNumber(w->mpeg.priv.perDepth); i++) {
	if ((w->mpeg.priv.availableDepths & (1<<i)) == 0)
	    continue;
	decodeAttr.colormap =  pd[ i ].view.cmap;
	pd[i].win = XCreateWindow(dpy,
				  widgetWin,
				  0,0,		/* x, y */
				  (unsigned)w->core.width,
				  (unsigned)w->core.height,
				  0,		/* border width */
				  pd[ i ].depth,
				  InputOutput,
				  pd[ i ].view.visual,
				  decodeValueMask,
				  &decodeAttr);
	/*
	 * Since the windows created in this loop are not widgets,
	 * we must register an event handler to get expose events.
	 */
	XtRegisterDrawable(dpy, pd[i].win, (Widget)w);
    }

    InitImageDisplay(w);
    ChangeDepthType(w);
}

static void
Resize(XnvrMpegDisplayWidget w)
{
    ChangeWindowSize(w);
}

static void
Exposure(XnvrMpegDisplayWidget w,
		     XExposeEvent *event,
		     Region region)
{
    NvrMpegExposeCbInfo cbInfo;
    InternalReasonPtr irp;
    Display *dpy = XtDisplay(w);

    if (!XtIsRealized(w)) {
	return;
    }

    /*
     * XXX try to align the application with the octword boundary to
     * get the whole picture...
     */
    if (w->mpeg.pub.alignWindow) {
	static Window *parents;
	static int nParents = 0;
	static Screen *screen;
	Window root, child;
	int x, y;
	UInt width, height;
	UInt borderWidth;
	UInt depth;
	int moveToLeft;
	int success;
	XWindowAttributes attr;
	XSetWindowAttributes setAttr;
	
	if (parents == NULL) {
	    Window root, current, parent, *children;
	    unsigned int nchildren;

	    screen = XtScreen(w);
	    parent = XtWindow(w);
	    root = 0;
	    for (;;) {
		current = parent;
		XQueryTree(dpy, current,
			   &root, &parent, &children, &nchildren);
		XFree((void *)children);
		if (parent == root)
		    break;
		if (parent == 0) /* the output window *is* the root window */
		    break;
		nParents++;
		parents = (Window *)
		    NullRealloc((char *)parents, nParents * sizeof(Window));
		parents[nParents-1] = parent;
	    }
	    nParents = nParents;
	}

        success = XTranslateCoordinates(dpy, XtWindow(w),
					RootWindowOfScreen(screen),
					0, 0, &x, &y, &child);
	if (!success)
	    return;
	if ((moveToLeft = (x & 0x7)) == 0)
	    return;
	success = XGetGeometry(dpy, parents[ nParents-1 ],
	    &root, &x, &y, &width, &height, &borderWidth, &depth);
	if (!success)
	    return;
	if (width >= WidthOfScreen(screen))
	    return;	/* don't try it with virtual window managers. */

	/*
	 * remember the current state of the override_redirect
	 */
	success = XGetWindowAttributes(dpy, parents[ nParents-1 ], &attr);
	if (! success)
	    return;

	/*
	 * Make our XMoveWindow stick
	 */
	setAttr.override_redirect = True;
	XChangeWindowAttributes(dpy,
				parents[ nParents-1 ],
				CWOverrideRedirect,
				&setAttr);
	Msg("aligning window ");
	if (x - moveToLeft < 0) {
	    Msg("-%d...\n", 8-moveToLeft);
	    XMoveWindow(dpy, parents[ nParents-1 ], x + (8-moveToLeft), y);
	} else {
	    Msg("+%d...\n", moveToLeft);
	    XMoveWindow(dpy, parents[ nParents-1 ], x - moveToLeft, y);
	}

	/*
	 * Restore the state to the window
	 */
	if (! attr.override_redirect) {
	    setAttr.override_redirect = False;
	    XChangeWindowAttributes(dpy,
				    parents[ nParents-1 ],
				    CWOverrideRedirect,
				    &setAttr);
	}
    }

    if (w->mpeg.pub.cb.playState != NvrPlaying
     && NULL != (irp = (*w->mpeg.priv.dpy->refreshImage)(w->mpeg.priv.dpy))) {
	AppendReason(&w->mpeg.priv.irList, irp);
    }

    cbInfo.reason = XmCR_EXPOSE;
    cbInfo.event = (XEvent *)event;
    cbInfo.window = w->mpeg.pub.decodeWindow;
    XtCallCallbackList((Widget)w, w->mpeg.pub.exposeCallback, &cbInfo);
}

static XtGeometryResult
QueryGeometry(Widget w, XtWidgetGeometry *intended, XtWidgetGeometry *preferred)
{
    preferred->request_mode = CWWidth | CWHeight;
    preferred->width = w->core.width;
    preferred->height = w->core.height;
    if (  ((intended->request_mode & (CWWidth | CWHeight))
                == (CWWidth | CWHeight)) &&
          intended->width == preferred->width &&
          intended->height == preferred->height)
        return XtGeometryYes;
    else if (preferred->width == w->core.width &&
             preferred->height == w->core.height)
        return XtGeometryNo;
    else
        return XtGeometryAlmost;
}

/* #define AccelerationDefined */
#if defined(sgi) && defined(AccelerationDefined)

#include <gl/glws.h>

/*
**  Used to get a value out of the configuration structure returned by
**  GLXgetconfig.  Scan through the GLX config structure, and, when we
**  find the proper buffer-mode combination, return the argument relevant
**  for that buffer type.
*/
static ULong
GLXgetvalue(GLXconfig* conf, int buffer, int mode)
{
    int i;
    for (i = 0; conf[i].buffer; i++)
        if (conf[i].buffer == buffer && conf[i].mode == mode)
            return conf[i].arg;
    return 0;
}

/*
 * Dorky little helper function used to build up a GLXconfig array.
 */
static void
SetGlEntry (GLXconfig* ptr, int b, int m, int a)
{
    ptr->buffer = b;
    ptr->mode = m;
    ptr->arg = a;
}

static Boolean
CheckGlxConfig(Display *dpy, XVisualInfo *pvi, GLXconfig *params)
{
    GLXconfig *conf;
    VisualID vis;

    conf = GLXgetconfig(dpy, pvi->screen, params);
    if (conf) {
        vis = GLXgetvalue(conf, GLX_NORMAL, GLX_VISUAL);
        free(conf);

        if (vis == pvi->visualid)
            return(True);
    }
    return(False);
}

static Boolean
CheckAcceleration(Display *dpy, XVisualInfo *pvi)
{
    GLXconfig params[50];
    GLXconfig* next;

    /*
     * This builds an array in "params" that describes for GLXgetconfig(3G)
     * the type of GL drawing that will be done.  Ask first for RGB double
     * buffered.
     */
    next = params;
    SetGlEntry(next++, GLX_NORMAL, GLX_RGB, True);
    SetGlEntry(next++, GLX_NORMAL, GLX_DOUBLE, True);
    SetGlEntry(next, 0, 0, 0); /* input to GLXgetconfig is null terminated */

    if (CheckGlxConfig(dpy, pvi, params))
	return(True);

    /*
     * Now ask first for RGB single buffered.
     */
    next = params;
    SetGlEntry(next++, GLX_NORMAL, GLX_RGB, True);
    SetGlEntry(next++, GLX_NORMAL, GLX_DOUBLE, False);
    SetGlEntry(next, 0, 0, 0);

    if (CheckGlxConfig(dpy, pvi, params))
	return(True);

    /*
     * Now ask first for index double buffered.
     */
    next = params;
    SetGlEntry(next++, GLX_NORMAL, GLX_RGB, False);
    SetGlEntry(next++, GLX_NORMAL, GLX_DOUBLE, True);
    SetGlEntry(next, 0, 0, 0); /* input to GLXgetconfig is null terminated */

    if (CheckGlxConfig(dpy, pvi, params))
	return(True);

    /*
     * Now ask first for index single buffered.
     */
    next = params;
    SetGlEntry(next++, GLX_NORMAL, GLX_RGB, False);
    SetGlEntry(next++, GLX_NORMAL, GLX_DOUBLE, False);
    SetGlEntry(next, 0, 0, 0);

    if (CheckGlxConfig(dpy, pvi, params))
	return(True);

    return(False);
}
#endif /* defined(sgi) && defined(AccelerationDefined) */

#ifndef AccelerationDefined
static Boolean CheckAcceleration(Display *dpy, XVisualInfo *pvi)
{
    return(False);
}
#endif

#ifdef DoinTranslations
/*
 * Default Translation Table
 */
static char DefaultTranslation[] =
	"<Btn1Down>:	Junk()";
#endif

/*
 * Class Record Initialization
 */
XnvrMpegDisplayClassRec xnvrMpegDisplayClassRec = {
    {		/* Core class part */
	/* superclass		*/	(WidgetClass) &coreClassRec,
	/* class_name		*/	"XnvrMpegDisplay",
	/* widget_size		*/	sizeof(XnvrMpegDisplayRec),
	/* class_initialize	*/	NULL,
	/* chained class init	*/	NULL,
	/* class_inited		*/	FALSE,
	/* initialize		*/	(XtInitProc)Initialize,
	/* initialize hook	*/	NULL,
	/* realize		*/	(XtRealizeProc)Realize,
	/* actions		*/	NULL, /* ActionsList, */
	/* num_actions		*/	0, /* XtNumber(ActionsList), */
	/* resources		*/	resources,
	/* num_resources	*/	XtNumber(resources),
	/* xrm_class		*/	NULLQUARK,
	/* compress_motion	*/	TRUE,
	/* compress_exposure*/		TRUE,
	/* compress enter/exit */	TRUE,
	/* visible_interest	*/	FALSE,
	/* destroy		*/	(XtWidgetProc)Destroy,
	/* resize		*/	(XtWidgetProc)Resize,
	/* expose		*/	(XtExposeProc)Exposure,
	/* set_values		*/	SetValues,
	/* set values hook	*/	NULL,
	/* set values almost	*/	XtInheritSetValuesAlmost,
	/* get values hook	*/	NULL,
	/* accept_focus		*/	NULL,
	/* version		*/	XtVersion,
	/* callback offset 1st	*/	NULL,
	/* default trans	*/	NULL, /* DefaultTranslation, */
	/* query geo proc	*/	QueryGeometry,
	/* display accelerator */	NULL,
	/* reserved		*/	NULL	/* extension record */
    },

    {		/* XnvrMpegDisplay class part */
	/* dummy field		*/	0,
    }
};

/*
 * Class Record Pointer Declaration
 */
WidgetClass xnvrMpegDisplayWidgetClass = (WidgetClass) &xnvrMpegDisplayClassRec;

