/*
 * (c) Copyright 1992-1995 North Valley Research, Inc.
 *      All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *      NvrMpegDisplayP.h - private definitions for for nvr mpeg display widget
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/NvrMpegDisplayP.h,v 1.4 1995/12/24 07:06:02 toddb Exp $
 */

#ifndef _NvrMpegDisplayP_h_
#define _NvrMpegDisplayP_h_

#include <Xm/XmP.h>
#include "NvrMpegDisplay.h"
#include "viewcmap.h"
#include "mpegsched.h"

enum {
    DefaultWidth = 320,
    DefaultHeight = 240
};

/*
 * The class definition for the MpegDisplay Widget
 */
typedef struct _XnvrMpegDisplayClassPart {
    int make_compiler_happy;	/* need dummy field */
} XnvrMpegDisplayClassPart;

typedef struct _XnvrMpegDisplayClassRec {
    CoreClassPart		core_class;
    XnvrMpegDisplayClassPart	mpegDisplay_class;
} XnvrMpegDisplayClassRec;

typedef struct _mpegPerDepth {
    ViewCmapInfo	view;
    int			depth;		/* actual depth of window */
    Window		win;		/* window id */
    Boolean		accelerated;	/* true if rendering is accelerated */
} MpegPerDepth, *MpegPerDepthPtr;

/*
 * This structure contains both the widget version of the input spec
 * and the MPEG scheduler's version of the input spec.
 */
typedef struct _dualInputSpec {
    NvrInputSpec widgetSpec;		/* public widget version */
    MpegSchedInputSpec schedSpec;	/* scheduler version */
} DualInputSpec, *DualInputSpecPtr;

/*
 * Instance record for the MpegDisplay Widget
 */
typedef struct _XnvrMpegDisplayPart {
    /*
     * resource variables consulted only at SetValues time
     */
    struct {
	NvrPlayerAction action;		/* next action to take */
	int aChan;			/* audio channels (for GetValues) */
	int aQual;			/* audio quality 1 to aQualSteps */
	int aQualSteps;			/* audio quality steps */
	int vChan;			/* video channels (for GetValues) */
	NvrDepth depth;			/* selected depth */
	NvrDepth depths;		/* available depths */
	NvrInputSpecPtr pInput;		/* pointer to an NvrInputSpec */
	int seekPoint;			/* requested seek offset */
	NvrSizeScale sizeScale;		/* picture scaling */
	NvrMpegDisplayCbInfo cb;	/* argument for XnvrNStateCallback */
	int gamma;			/* gamma specification for color */
	Bool alignWindow;		/* whether we try to align window */
	Bool trueColor12Bit;		/* true if true-color 12-bit used */
	XtCallbackList stateCallback;	/* callback for state changes */
	XtCallbackList exposeCallback;	/* callback for exposure */
	UChar contrast;
	UChar brightness;
	Visual *visual;			/* visual for selected depth */
	Window decodeWindow;		/* used by app to set colormap win */
    } pub;

    struct {
	/*
	 * one for each value of RgbDepthType
	 */
	MpegPerDepth perDepth[ NRgbDepthTypes ];
	int aChanReq;			/* requested audio channel */
	int vChanReq;			/* requested video channel */
	ULong availableDepths;		/* discovered from X server */
	int depthType;			/* selected depth type */
	ImgDpyRegionPtr dpy;		/* image display info */
	ImgDpyParam dpyParam;		/* kept for the ImgDpyRegion */
	AudioFilePtr auOutput;		/* output device */
	AudioForm auInForm;		/* form of the inut audio */
	ULong schedCbSeen;		/* callbacks we've seen */
	MpegSchedPtr sched;		/* the mpeg scheduler */
	ULong schedMask;		/* items already set in scheduler */
	InternalReasonPtr irList;	/* list of reasons */
	XtWorkProcId workId;		/* work id for work proc */
	DualInputSpecPtr dualSpec;	/* public and internal input spec */
    } priv;

} XnvrMpegDisplayPart;

typedef struct _XnvrMpegDisplayRec
{
    CorePart			core;
    XnvrMpegDisplayPart		mpeg;
} XnvrMpegDisplayRec;

extern XnvrMpegDisplayClassRec xnvrMpegDisplayClassRec;

#endif /* _MpegDisplayP_h_ */
