/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	PlayerCntl.h - interfaces for the player callbacks
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/PlayerCntl.h,v 1.14 1995/08/08 14:35:44 toddb Exp $
 */
#ifndef _PLAYERCNTL_H_
#define _PLAYERCNTL_H_

#include "nvrDeckCntl.h"

/*
 * Forward declaration of structures
 */
typedef struct _playerRunConfig PlayerRunConfig, *PlayerRunConfigPtr;

/*
 * Values for the changedCB() callback that signal which values
 * have changed.
 */
enum {
    kPlayerAbilityChanged        = 0x0001,
    kPlayerWidthChanged          = 0x0002,
    kPlayerHeightChanged         = 0x0004,
    kPlayerFrameChanged          = 0x0008,
    kPlayerFrameLabelChanged     = 0x0010,
    kPlayerFrameFormatChanged    = 0x0020,
    kPlayerMinFrameChanged       = 0x0040,
    kPlayerMaxFrameChanged       = 0x0080,
    kPlayerSpeedChanged          = 0x0100,
    kPlayerMinSpeedChanged       = 0x0200,
    kPlayerMaxSpeedChanged       = 0x0400,
    kPlayerRunningChanged        = 0x0800,
    kPlayerMediaAvailableChanged = 0x1000
};

/*
 * This is what is used to create each source (player) type
 */
struct _playerClass {
    ULong magic;	/* used when storing structure in widget user data */

    /*
     * Activated by UI control buttons
     */
    InternalReasonPtr (*fastRev)(PlayerClass);
    InternalReasonPtr (*stepRev)(PlayerClass);
    InternalReasonPtr (*playRev)(PlayerClass);
    InternalReasonPtr (*play)   (PlayerClass, boolean record);
    InternalReasonPtr (*pause)  (PlayerClass);
    InternalReasonPtr (*stepFwd)(PlayerClass);
    InternalReasonPtr (*fastFwd)(PlayerClass);
    InternalReasonPtr (*seek)   (PlayerClass, long frame);
    InternalReasonPtr (*confPlayer)(PlayerClass, PlayerRunConfigPtr);

    /*
     * Called to free the PlayerClass for this controller
     */
    void (*free)(PlayerClass);

    /*
     * Widget that contains the display window.  This should be created as
     * an Unmanaged child of PlayerConfig.dpyBox;
     */
    Widget wDpy;

    /*
     * Data specific to the controller.
     */
    void *priv;

    PlaySegmentPtr edit;		/* segment used for recording */
    char *prodName;			/* path name, tape name, etc */
    ULong ability;			/* mask of DeckAbilityBits */
    int width, height;			/* size of the window */
    long frame;				/* the current frame number */
    long minFrame, maxFrame;		/* the range of frame numbers */
    char *frameLabel;			/* the label used for the frame # */
    void (*frameFormat)(DeckCntlPtr, char *buf, int length);
    long speed;
    long minSpeed, maxSpeed;		/* the range of speed values */
    Boolean running;
    Boolean mediaAvailable;		/* true: tape in, file defined, etc */
    PlayerClass recorder;		/* designated recorder for this */
    PlayerClass player;			/* designated player for this */
}; /* typedef: PlayerClassRec, *PlayerClass */

typedef struct _playerConfig {
    void (*changedCB)(PlayerClass, ULong changeMask);
    Widget dpyParent;			/* parent of display window */
    int width, height;			/* default width, height */
    DeckMedium medium;			/* see nvrDeckCntl.h */
    ULong ability;			/* kDeckIsDevice &| kDeckCanRecord */
} PlayerConfig, *PlayerConfigPtr;

/*
 * This structure and enum set is used to do run-time configuration of
 * a player.
 */
typedef enum {
    kChangePlayerMedia     = 0x001,
    kChangePlayerRecorder  = 0x002,
    kChangePlayerSelect    = 0x004,
    kChangePlayerSpeed     = 0x008,
    kChangePlayerSegment   = 0x010
} PlayerRunConfigType;

struct _playerRunConfig {
    ULong configType;           /* set of DeckRunConfigType bits */
    PlayerClass recorder;       /* recorder for this player */
    boolean select;             /* whether the player is selected */
    long speed;			/* relative speed of the deck */
    PlaySegmentPtr seg;		/* edit segment */
}; /* typedef: PlayerRunConfig, *PlayerRunConfigPtr */

/*
 * Finally, information for creating, managing and 
 * destroying players and recorders.
 */
CExternal PlayerClass AllocDeckPlayer(PlayerConfigPtr); 
CExternal PlayerClass CreateDeckPlayer(PlayerConfigPtr); 
CExternal PlayerClass CreateTitlePlayer(PlayerConfigPtr);

#endif /* _PLAYERCNTL_H_ */
