/*
 * Copyright (c) 1992-1995 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	SgiDisplay.c - sgi video display stuff
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/SgiDisplay.c,v 1.6 1995/07/11 23:03:52 toddb Exp $
 */

#include "nvrAlloc.h"
#include "nvrReason.h"
#include "nvrString.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <Xm/Xm.h>
#include <Xm/XmP.h>
#include "SgiDisplayP.h"
#include "WidgetUtil.h"

#include "osio.h"

static InternalReason CantOpenVideoServer =
    { "Can't open SGI video server\n", IR_BadValue };

static void
sgiInitialize(Widget pubReq, Widget pubNew, ArgList argList, Cardinal *numArgs)
{
    /* by this time, the superclass' Initialize has already been called */
    XsgiDisplayWidget req = (XsgiDisplayWidget)pubReq;
    XsgiDisplayWidget new = (XsgiDisplayWidget)pubNew;
    int i;
    Display *dpy;
    int screen;
    XVisualInfo *pVisualInfo, visualInfoTmpl;
    Visual *vis, *defaultVisual;
    int visCount;
    int depth;

    dpy = XtDisplay(req);
    screen = XScreenNumberOfScreen( XtScreen(req) );

    new->nvrDisplay.type = XnvrSgiVideo;
    new->sgiDisplay.svr = NULL;
    new->sgiDisplay.src = NULL;
    new->sgiDisplay.drn = NULL;
    new->sgiDisplay.path = NULL;

    new->nvrDisplay.src_x = new->sgiDisplay.xIn;
    new->nvrDisplay.src_y = new->sgiDisplay.yIn;
    new->nvrDisplay.src_width = new->sgiDisplay.widthIn;
    new->nvrDisplay.src_height = new->sgiDisplay.heightIn;
    new->nvrDisplay.dest_x = 0;
    new->nvrDisplay.dest_y = 0;
    new->nvrDisplay.dest_width = new->sgiDisplay.widthIn;
    new->nvrDisplay.dest_height = new->sgiDisplay.heightIn;

    new->nvrDisplay.hue.count = 256;
    new->nvrDisplay.hue.min = 0;
    new->nvrDisplay.hue.cur = 128;
    new->nvrDisplay.hue.norm = 128;

    new->nvrDisplay.sat = new->nvrDisplay.hue;
    new->nvrDisplay.contrast = new->nvrDisplay.hue;
    new->nvrDisplay.bright = new->nvrDisplay.hue;

    /* set the default size to the size of the default vin */
    if (new->core.width == 0)
	new->core.width = new->sgiDisplay.widthIn;
    if (new->core.height == 0)
	new->core.height = new->sgiDisplay.heightIn;

    /* find the visual to use --
     * The best visual to use would be one that both has video capability
     * that matches the default visual.
     * If that can't be, take anything that can do video.
     */
    defaultVisual = DefaultVisual(dpy, screen);

    visualInfoTmpl.screen = screen;
    pVisualInfo =
	XGetVisualInfo(dpy, VisualScreenMask, &visualInfoTmpl, &visCount);

    depth = 24;	/* XXX do we know this?? */

    for (i = 0; i < visCount; i++) {
	if (depth == pVisualInfo[i].depth) {
	    vis = pVisualInfo[ i ].visual;
	    if (vis->visualid == defaultVisual->visualid)
		break;
	}
    }

    new->core.depth = depth;
    new->nvrDisplay.visual = vis;

    /*
     * Figure out the colormap
     */
    if (vis->visualid != defaultVisual->visualid) {
	new->core.colormap = XCreateColormap(dpy,
					     RootWindow(dpy, screen),
					     vis, AllocNone);
    }
}

static void
RemoveVideoPath(XsgiDisplayWidget w)
{
    XsgiDisplayPart *sgiPart = &w->sgiDisplay;

    if (sgiPart->src) {
	vlRemoveNode(sgiPart->svr, sgiPart->path, sgiPart->src);
	sgiPart->src = NULL;
    }
    if (sgiPart->drn) {
	vlRemoveNode(sgiPart->svr, sgiPart->path, sgiPart->drn);
	sgiPart->drn = NULL;
    }
    if (sgiPart->path) {
	vlDestroyPath(sgiPart->svr, sgiPart->path);
	sgiPart->path = NULL;
    }
}

static void
FreezeVideo(XsgiDisplayWidget w)
{
    XsgiDisplayPart *sgiPart = &w->sgiDisplay;
    VLControlValue val;

    if (sgiPart->svr == NULL || sgiPart->path == NULL)
	return;

    vlSetControl(sgiPart->svr, sgiPart->path, sgiPart->drn, VL_FREEZE, &val);
}

static void
ShellMovedCB(Widget topLevel,
	     XtPointer closure,
	     XEvent *event,
	     Boolean *continueToDispatch)
{
    Window dummyWin, win;
    Display *dpy;
    VLControlValue val;
    XsgiDisplayWidget w = (XsgiDisplayWidget)closure;
    XsgiDisplayPart *sgiPart = &w->sgiDisplay;

    if (event->type != ConfigureNotify)
	return;
    if (sgiPart->svr == NULL || sgiPart->path == NULL)
	return;
    win = XtWindow((Widget)w);
    if (win == 0)
	return;	/* not yet realized */

    dpy = XtDisplay((Widget)w);
    XTranslateCoordinates(dpy, win,
                          RootWindow(dpy, DefaultScreen(dpy)),
                          0, 0,
                          &val.xyVal.x, &val.xyVal.y,
                          &dummyWin);

    vlSetControl(sgiPart->svr, sgiPart->path, sgiPart->drn, VL_ORIGIN, &val);
}

static void
InitSgiVideoPath(XsgiDisplayWidget w)
{
    VLControlValue val;
    XnvrDisplayWidget nvrw = (XnvrDisplayWidget)w;
    XsgiDisplayPart *sgiPart = &w->sgiDisplay;
    int status;
    Widget toplevel;

    if (! nvrw->nvrDisplay.isEnabled)
	return;
    if (sgiPart->svr == NULL)
	sgiPart->svr = vlOpenVideo("");
    if (sgiPart->svr == NULL)
	return;

    toplevel = FindTopLevel((Widget)w);
    XtAddEventHandler(toplevel, SubstructureNotifyMask, FALSE,
		      ShellMovedCB, (XtPointer)w);

    if (sgiPart->src == NULL) {
	/* Create a source node on a video device */
	sgiPart->src = vlGetNode(sgiPart->svr, VL_SRC, VL_VIDEO, VL_ANY);
	if (sgiPart->src == -1) {
	    sgiPart->src = NULL;
	    RemoveVideoPath(w);
	    return;
	}
    }

    if (sgiPart->drn == NULL) {
	/* Create a drain node on the screen */
	sgiPart->drn = vlGetNode(sgiPart->svr, VL_DRN, VL_SCREEN, VL_ANY);
	if (sgiPart->drn == -1) {
	    sgiPart->drn = NULL;
	    RemoveVideoPath(w);
	    return;
	}
    }

    if (sgiPart->path == NULL) {
	/* Create a path on the first device that supports it */
	sgiPart->path = vlCreatePath(sgiPart->svr, VL_ANY,
				     sgiPart->src, sgiPart->drn);
	if (sgiPart->path == -1) {
	    sgiPart->path = NULL;
	    RemoveVideoPath(w);
	    return;
	}

	/* Set up the hardware for and define the usage of the path */
	status = vlSetupPaths(sgiPart->svr, (VLPathList)&sgiPart->path, 1,
			      VL_SHARE, VL_SHARE);
	if (status < 0) {
	    vlDestroyPath(sgiPart->svr, sgiPart->path);
	    sgiPart->path = NULL;
	    RemoveVideoPath(w);
	    return;
	}
    }

    if (w->core.window) {
	Display *dpy = XtDisplay((Widget)w);
	int x, y;
	int screenIndex = XScreenNumberOfScreen(w->core.screen);
	Window dummyWin;
	Window root = RootWindow(dpy, screenIndex);

	/* Set the X window to be the drain */
	val.intVal = w->core.window;
	status = vlSetControl(sgiPart->svr, sgiPart->path, sgiPart->drn,
			      VL_WINDOW, &val);

	/* Get X and VL into the same coordinate system */
	XTranslateCoordinates(dpy, w->core.window,
	    root, 0, 0,&x, &y, &dummyWin);
	   
	/* Set the live video to the same location and size as the X window */
	val.xyVal.x = x;
	val.xyVal.y = y;
	status = vlSetControl(sgiPart->svr, sgiPart->path, sgiPart->drn,
			      VL_ORIGIN, &val);
    }
    FreezeVideo(w);
}

static void
ShutdownSgiVideo(XsgiDisplayWidget sgiw)
{
    XsgiDisplayPart *sgiPart = &sgiw->sgiDisplay;

    if (sgiPart->svr) {
	if (sgiPart->path)
	    vlDestroyPath(sgiPart->svr, sgiPart->path);
	vlCloseVideo(sgiPart->svr);
    }
    sgiPart->svr = NULL;
    sgiPart->path = NULL;
    sgiPart->src = NULL;
    sgiPart->drn = NULL;
}


static void
MapEventCB(Widget w,
	   XtPointer closure,
	   XEvent *event,
	   Boolean *continueToDispatch)
{
    XsgiDisplayWidget sgiw = (XsgiDisplayWidget)w;
    XnvrDisplayWidget nvrw = (XnvrDisplayWidget)w;

    switch (event->type) {
    case MapNotify:
	/*
	 * Start up the video server.
	 */
	nvrw->nvrDisplay.isEnabled = True;
	InitSgiVideoPath(sgiw);
	break;

    case UnmapNotify:
	/*
	 * Shut down the video server.
	 */
	ShutdownSgiVideo(sgiw);
	break;

    default:
	break;
    }
}

static void
sgiDestroy(Widget w)
{
    Widget toplevel = FindTopLevel((Widget)w);

    XtRemoveEventHandler(toplevel, SubstructureNotifyMask, FALSE,
		      ShellMovedCB, (XtPointer)w);
    XtRemoveEventHandler(w, StructureNotifyMask, FALSE,
		      MapEventCB, (XtPointer)w);
    RemoveVideoPath((XsgiDisplayWidget)w);
}

static void
sgiRealize(XsgiDisplayWidget w,
	   XtValueMask* value_mask,
	   XSetWindowAttributes* attributes)
{
    /* by this time, the superclass' Realize has already been called */
    Window window;
    Display *dpy;
    XGCValues gcValues;
    Pixel foreground;
    XColor colorDef, rgbDef;

    dpy = XtDisplay(w);

    /*
     * Figure out the foreground pixel
     */

    /* XXX should really use the name in the resource database */
    XAllocNamedColor(dpy, w->core.colormap, "black", &colorDef, &rgbDef);
    foreground = colorDef.pixel;

    /*
     * Fix up the rest of the attributes...
     */
    attributes->background_pixmap = None;
    attributes->event_mask = ExposureMask | VisibilityChangeMask |
			ButtonPressMask | ButtonReleaseMask | KeyPressMask;
    attributes->backing_store = NotUseful;
    *value_mask |= CWBackPixmap | CWEventMask | CWBackingStore;

    XtCreateWindow((Widget)w, InputOutput,
		   w->nvrDisplay.visual, *value_mask, attributes);
    window = XtWindow((Widget)w);

    gcValues.foreground = foreground;
    gcValues.fill_style = FillSolid;
    gcValues.function = GXcopy;

    w->nvrDisplay.renderGC =
        XCreateGC(dpy, window,
                  GCFunction | GCFillStyle | GCForeground, &gcValues);

    XtAddEventHandler((Widget) w, StructureNotifyMask, FALSE,
                      MapEventCB, (XtPointer)w);

    XtCallCallbackList((Widget)w, w->nvrDisplay.realizeCallback, NULL);
}

static void
sgiUpdateDisplay(XsgiDisplayWidget w, XnvrRenderProc renderProc)
{
    XnvrUpdateDisplayProc update =
	xsgiDisplayClassRec.nvrDisplay_class.updateDisplay;

    /* call the superclass's routine */
    (*update)((XnvrDisplayWidget)w, renderProc);
}

static void
sgiRenderDisplay(XnvrDisplayWidget nvrw)
{
    XsgiDisplayWidget w = (XsgiDisplayWidget)nvrw;
    XsgiDisplayPart *sgiPart = &w->sgiDisplay;
    VLControlValue val;

    if (sgiPart->svr == NULL || sgiPart->path == NULL)
	return;

    vlBeginTransfer(sgiPart->svr, sgiPart->path, 0, NULL);
    val.boolVal = False;
    vlSetControl(sgiPart->svr, sgiPart->path, sgiPart->drn, VL_FREEZE, &val);
}


static void
sgiStopDisplay(XsgiDisplayWidget w)
{
    FreezeVideo(w);
}


static void
sgiRedisplay(Widget pubw, XEvent *event, Region region)
{
    XsgiDisplayWidget w = (XsgiDisplayWidget)pubw;

    w->sgiDisplay.firstRedisplay = True;
    if (XtIsRealized(w) && !w->core.being_destroyed)
    {
	XtCallCallbackList((Widget)w, w->nvrDisplay.exposeCallback, NULL);
    }
    if (XtIsRealized(w) && !w->core.being_destroyed) {
	sgiUpdateDisplay(w, sgiRenderDisplay);
	if (! w->nvrDisplay.isActive)
	    sgiStopDisplay (w);
    }
}

static void
sgiResize(Widget w)
{
    /* pass the buck to the superclass */
    (*xsgiDisplayClassRec.core_class.superclass->core_class.resize)(w);
}

static Boolean
sgiSetValues (XsgiDisplayWidget cur,
	      XsgiDisplayWidget req,
	      XsgiDisplayWidget new)
{
    boolean updateDisplay = False;

    /* Query-only resources -- don't allow them to be changed.
       They are gotten from sgi and can't be meaningfully altered.
     */
    new->nvrDisplay.hue.count = cur->nvrDisplay.hue.count;
    new->nvrDisplay.hue.min = cur->nvrDisplay.hue.min;
    new->nvrDisplay.sat.count = cur->nvrDisplay.sat.count;
    new->nvrDisplay.sat.min = cur->nvrDisplay.sat.min;
    new->nvrDisplay.contrast.count = cur->nvrDisplay.contrast.count;
    new->nvrDisplay.contrast.min = cur->nvrDisplay.contrast.min;
    new->nvrDisplay.bright.count = cur->nvrDisplay.bright.count;
    new->nvrDisplay.bright.min = cur->nvrDisplay.bright.min;
    new->nvrDisplay.portNames = cur->nvrDisplay.portNames;
    new->nvrDisplay.n_ports = cur->nvrDisplay.n_ports;
    new->nvrDisplay.n_engines = cur->nvrDisplay.n_engines;
    new->nvrDisplay.visual = cur->nvrDisplay.visual;

    new->sgiDisplay.svr = cur->sgiDisplay.svr;

    /* Video control resources */

    if (new->nvrDisplay.src_x != cur->nvrDisplay.src_x
     || new->nvrDisplay.src_y != cur->nvrDisplay.src_y
     || new->nvrDisplay.src_width != cur->nvrDisplay.src_width
     || new->nvrDisplay.src_height != cur->nvrDisplay.src_height
     || new->nvrDisplay.dest_x != cur->nvrDisplay.dest_x
     || new->nvrDisplay.dest_y != cur->nvrDisplay.dest_y
     || new->nvrDisplay.dest_width != cur->nvrDisplay.dest_width
     || new->nvrDisplay.dest_height != cur->nvrDisplay.dest_height)
    {
	updateDisplay = True;
    }

    if (new->nvrDisplay.resizeMode != cur->nvrDisplay.resizeMode)
    {
	new->nvrDisplay.src_x = new->sgiDisplay.xIn;
	new->nvrDisplay.src_y = new->sgiDisplay.yIn;
	new->nvrDisplay.src_width = new->sgiDisplay.widthIn;
	new->nvrDisplay.src_height = new->sgiDisplay.heightIn;
	new->nvrDisplay.dest_x = 0;
	new->nvrDisplay.dest_y = 0;
	new->nvrDisplay.dest_width = new->sgiDisplay.widthIn;
	new->nvrDisplay.dest_height = new->sgiDisplay.heightIn;
	updateDisplay = True;
    }

    if (new->nvrDisplay.engine != cur->nvrDisplay.engine)
    {
	updateDisplay = True;
    }

    if (new->nvrDisplay.hue.cur != cur->nvrDisplay.hue.cur)
	updateDisplay = True;
    if (new->nvrDisplay.sat.cur != cur->nvrDisplay.sat.cur)
	updateDisplay = True;
    if (new->nvrDisplay.contrast.cur != cur->nvrDisplay.contrast.cur)
	updateDisplay = True;
    if (new->nvrDisplay.bright.cur != cur->nvrDisplay.bright.cur)
	updateDisplay = True;

    if (new->nvrDisplay.port_index != cur->nvrDisplay.port_index)
    {
	new->nvrDisplay.isActive = False;
	updateDisplay = True;
    }

    /* always turn it on, if that's what's been asked.  Only turn
     * it off if we think its on.
     */
    if (new->nvrDisplay.isActive
     || new->nvrDisplay.isActive != cur->nvrDisplay.isActive)
	updateDisplay = True;

    if (! new->nvrDisplay.isEnabled
     && new->nvrDisplay.isEnabled != cur->nvrDisplay.isEnabled) {
	ShutdownSgiVideo((XsgiDisplayWidget) new);
    }

    if (updateDisplay)
    {
	/*
	 * Note that we are calling UpdateDisplay() with 'new'.
	 * Thats because the superclass actually changes some values
	 * in the widget, and according to the Xt documentation, cur
	 * and req are only for reference.
	 *
	 * At this point, we are all done changing the widget as
	 * a result of the SetValues call, so everything should be
	 * ok.  Calling another subroutine from this SetValues would
	 * normally be a no-no, because the subroutine may do other
	 * things to the widget, or worse, it may perform callbacks.
	 * Callbacks may in turn do a SetValues, and then we're in
	 * an infinite loop.
	 */
	new->nvrDisplay.alreadyRedisplayed = True;
	if (new->nvrDisplay.isActive) {
	    sgiUpdateDisplay(new, sgiRenderDisplay);
	} else
	    sgiStopDisplay(new);
	return(False);
    }

    new->nvrDisplay.alreadyRedisplayed = False;
    return(False);
}


static void
sgiPanDisplay (Widget w, XEvent *event, String *params, Cardinal *nParams)
{
    XnvrPanDisplayProc pan = xsgiDisplayClassRec.nvrDisplay_class.panDisplay;

    (*pan)((XnvrDisplayWidget)w, event, sgiRenderDisplay);
}


/**************************************************************************
 *
 * Global definitions
 *
 **************************************************************************/

/* Default translations and action records */
static char sgiDefaultTranslation[] =
    "<Btn1Down>:    pan()";

static XtActionsRec sgiActionsList[] =
{
    {"pan",	    sgiPanDisplay},
};

static XtResource nvrResources[] =
{
    {   XsgiNvideoServer, XsgiCVideoServer, XmRPointer,
        sizeof (int),
        XtOffset (XsgiDisplayWidget, sgiDisplay.svr),
        XmRImmediate, (XtPointer) NULL },
};

XsgiDisplayClassRec xsgiDisplayClassRec =
{
    {
	/* superclass	    */	    (WidgetClass) &xnvrDisplayClassRec,
	/* class_name	    */	    "XsgiDisplay",
	/* widget_size	    */	    sizeof(XsgiDisplayRec),
	/* class_initialize */	    NULL,
	/* chained class init */    NULL,
	/* class_inited	    */	    FALSE,
	/* initialize	    */	    sgiInitialize,
	/* initialize hook  */	    NULL,
	/* realize	    */	    (XtRealizeProc)sgiRealize,
	/* actions	    */	    sgiActionsList,
	/* num_actions	    */	    XtNumber(sgiActionsList),
	/* resources	    */	    nvrResources /* resources */,
	/* num_resources    */	    XtNumber(nvrResources),
	/* xrm_class	    */	    NULLQUARK,
	/* compress_motion  */	    TRUE,
	/* compress_exposure*/	    TRUE,
	/* compress enter/exit */   TRUE,
	/* visible_interest */	    FALSE,
	/* destroy	    */	    sgiDestroy,
	/* resize	    */	    sgiResize,
	/* expose	    */	    sgiRedisplay,
	/* set_values	    */	    (XtSetValuesFunc) sgiSetValues,
	/* set values hook  */	    NULL,
	/* set values almost*/	    XtInheritSetValuesAlmost,
	/* get values hook  */	    NULL,
	/* accept_focus	    */	    NULL,
	/* version	    */	    XtVersion,
	/* callback offset 1st */   NULL,
	/* default trans    */	    sgiDefaultTranslation,
	/* query geo proc   */	    NULL,
	/* display accelerator */   NULL,
	/* reserved	    */	    NULL	/* extension record */
    },

    {					    /* XnvrDisplay */
	/* updateDisplay    */	    XnvrInheritUpdateDisplay,
	/* renderDisplay    */	    sgiRenderDisplay,
	/* panDisplay	    */	    XnvrInheritPanDisplay,
	0,				    /* extension */
    },

    {					    /* XsgiDisplay */
	0,				    /* extension */
    }
};

WidgetClass xsgiDisplayWidgetClass = (WidgetClass) &xsgiDisplayClassRec;

