/*
 * Copyright (c) 1992-1995 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	WidgetConf.c - video display widget configuration... a platform-
 *                     independent API for choosing a video display widget
 *                    
 * $Header: /usr/local/src/nvrToMIT/RCS/WidgetConf.c,v 1.5 1995/04/14 22:18:32 toddb Exp $
 */
#include "NvrDisplay.h"
#include "nvrAlloc.h"

typedef struct _dpyWidgetDesc DpyWidgetDesc, *DpyWidgetDescPtr;
struct _dpyWidgetDesc {
    XnvrDisplayType type;
    WidgetClass wclass;
    Widget (*create)(char *name,
		     Widget parent,
		     DpyWidgetDescPtr pList,
		     ArgList,
		     int nArgs,
		     InternalReasonPtr *pReason);
}; /* typedef: DpyWidgetDesc, *DpyWidgetDescPtr */

/***************************************************************************
 *
 * Sgi create routines
 *
 ***************************************************************************/
#ifdef sgi
#include "SgiDisplay.h"
#include <vl/vl.h>
#include <gl.h>
#include <gl/glws.h>
#include <Sgm/GlxMDraw.h>
#include <GL/GLwMDrawA.h>
#include <GL/GLwDrawA.h>

static Widget
SgiGlxCreate(char *name,
	     Widget parent,
	     DpyWidgetDescPtr pList,
	     ArgList argList,
	     int nArgs,
	     InternalReasonPtr *pReason)
{
    Widget glw;
    ArgList sgiArgs;
    int allocSize = sizeof(Arg) * (nArgs+10);
    static GLXconfig glxConfig [] = {
	{ GLX_NORMAL, GLX_DOUBLE, FALSE },
	{ GLX_NORMAL, GLX_RGB, TRUE },
	{ GLX_NORMAL, GLX_ZSIZE,  GLX_NOCONFIG },
	{ 0, 0, 0 }
    };

    sgiArgs = (ArgList)InsecureAlloc( allocSize );
    if (sgiArgs == NULL) {
	*pReason = AllocReason("arg list", allocSize);
	return(NULL);
    }
    memcpy((char *)sgiArgs, (char *)argList, sizeof(Arg) * nArgs);
#if 0
    XtSetArg(sgiArgs[nArgs], GlxNglxConfig, glxConfig); nArgs++;
#endif
    XtSetArg(sgiArgs[nArgs], XmNresizePolicy, XmRESIZE_NONE); nArgs++;
    XtSetArg(sgiArgs[nArgs], GLwNrgba, True); nArgs++;
    XtSetArg(sgiArgs[nArgs], GLwNdoublebuffer, False); nArgs++;
    XtSetArg(sgiArgs[nArgs], GLwNdepthSize, 24); nArgs++;
    XtSetArg(sgiArgs[nArgs], GLwNredSize, 8); nArgs++;
    XtSetArg(sgiArgs[nArgs], GLwNgreenSize, 8); nArgs++;
    XtSetArg(sgiArgs[nArgs], GLwNblueSize, 8); nArgs++;

    glw = XtCreateWidget(name, pList->wclass, parent, sgiArgs, nArgs);
    free((char *)sgiArgs);
    if (glw == NULL) {
	*pReason = InternalProblem("Can't create GLX drawing widget\n");
	return(NULL);
    }

    return (glw);
}

static int
InitSgiTypes(DpyWidgetDescPtr pList, int listIndex)
{
    pList[ listIndex   ].wclass = xsgiDisplayWidgetClass;
    pList[ listIndex   ].type   = XnvrSgiVideo;
    pList[ listIndex++ ].create = NULL;
    pList[ listIndex   ].wclass = glwMDrawingAreaWidgetClass;
    pList[ listIndex   ].type   = XnvrSgiJpeg;
    pList[ listIndex++ ].create = SgiGlxCreate;
    return(listIndex);
}

#else /* sgi */

#define InitSgiTypes(pList, listIndex)	listIndex

#endif


/***************************************************************************
 *
 * Sun init routines
 *
 ***************************************************************************/
#define InitSunTypes(pList, listIndex)	listIndex

/***************************************************************************
 *
 * External interfaces
 *
 ***************************************************************************/
static int
InitWidgetTypes(DpyWidgetDescPtr pList)
{
    int nDesc = 0;
    nDesc = InitSgiTypes(pList + nDesc, nDesc);
    nDesc = InitSunTypes(pList + nDesc, nDesc);
    return(nDesc);
}

Widget
CreateDisplayWidget(char *name,
		    Widget parent,
		    XnvrDisplayType type,
		    ArgList argList,
		    int nArgs,
		    InternalReasonPtr *pReason)
{
    int i, nDesc;
    DpyWidgetDesc list[ 100 ];		/* more than enough */

    *pReason = NULL;

    nDesc = InitWidgetTypes(list);
    for (i = 0; i < nDesc; i++) {
	if (list[i].wclass == NULL)
	    continue;
	if (list[i].type == type)
	    break;
	if ((list[i].type & XnvrDisplayMask) == type)
	    break;
    }

    if (i >= nDesc) {
	char *displayName = NULL;

	switch (type & XnvrDisplayMask) {
	default:
        case XnvrUnknownDpy:
            *pReason = InternalProblem("Can't find display type: 0x%x\n", type);
	    break;

        case XnvrHwVideo: displayName = "video hardware"; break;
        case XnvrHwJpeg:  displayName = "JPEG/video hardware"; break;
        case XnvrHwMpeg:  displayName = "MPEG/video hardware"; break;
        case XnvrSoftDisplay: displayName = "Sofware display"; break;
	}
	if (displayName)
            *pReason = BadValueReason("No %s widget is available\n",
		displayName);
	return(NULL);
    }

    if (list[i].create)
	return((*list[i].create)(name, parent, &list[i],
				 argList, nArgs, pReason));
    else
	return(XtCreateWidget(name, list[i].wclass, parent, argList, nArgs));
}
