/*
 *	Copyright (c) 1992, North Valley Research, Inc.
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		WidgetDebug.c - menus for debug control
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/WidgetDebug.c,v 1.1 1995/07/10 22:04:22 toddb Exp $
 */

#define DEBUG_ENABLED	/* always */

#include "nvrTypes.h"
#include "nvrDebug.h"
#include "osio.h"
#include "WidgetDebug.h"

#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleB.h>
#include <Xm/MenuShell.h>
#include <Xm/Separator.h>
#include <Xm/Label.h>
#include <Xm/CascadeB.h>

static Widget DebugMenu;

static void
DebugWindowEvent(Widget dpyWidget,
                 XtPointer closure,
                 XEvent *event,
                 Boolean *continueToDispatch)
{
    if (event->type == ButtonPress
     && event->xbutton.state == (ShiftMask | ControlMask | Mod1Mask)) {
	XmMenuPosition(DebugMenu, &event->xbutton);
	XtManageChild(DebugMenu);
    }
}

static void
DebugButtonProc(Widget w, XtPointer indexData, XtPointer callData)
{
    XmToggleButtonCallbackStruct *tbs =
	(XmToggleButtonCallbackStruct *)callData;
    int index = (int)indexData;

    if (tbs->set)
	SetDebugBit(index);
    else
	GetDebugBit(index);
}

void
InitDebugUI(Widget w, FlagLabelsPtr desc)
{
    Widget button, menu;
    int i;
    XmString str;
    FlagNamesPtr flags;
    char *title;

    XtAddEventHandler(w, ButtonPress, FALSE, DebugWindowEvent, NULL);

    if (DebugMenu == NULL) {
	DebugMenu = XmCreatePopupMenu(w, "debugMenu", NULL, 0);
	XtVaCreateManagedWidget("Debug Control", xmLabelWidgetClass,
	    DebugMenu, NULL);
	XtVaCreateManagedWidget("sep", xmSeparatorWidgetClass,
	    DebugMenu, NULL);
    }

    for (; desc->flags; desc++) {
	title = desc->title;
	flags = desc->flags;
	menu = XmCreatePulldownMenu(DebugMenu, "debugPulldown", NULL, 0);
	str = XmStringCreateSimple(title);
	XtVaCreateManagedWidget(title,
	    xmCascadeButtonWidgetClass, DebugMenu,
	    XmNsubMenuId, menu,
	    XmNlabelString, str,
	    NULL);
	XmStringFree(str);

	for (i = 0; i < flags->nNames; i++) {
	    button = XtVaCreateManagedWidget(flags->names[ i ],
		xmToggleButtonWidgetClass, menu, NULL);
	    XtAddCallback(button, XmNvalueChangedCallback, DebugButtonProc,
		(XtPointer) (i + flags->baseIndex));
	}
    }
}
