/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *      WidgetDialog.c - Simplified notifier dialogs
 *
 * $Header: /usr/local/src/nvr/library/RCS/SelectionBox.c,v 1.7 1994/10/02 23:58
:07 toddb Exp $
 */
#include "WidgetUtil.h"
#include "WidgetDialog.h"
#include "nvrAlloc.h"
#include "osio.h"

#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/MessageB.h>
#include <Xm/MwmUtil.h>
#include <stdio.h>
#include <stdarg.h>

typedef struct _privDialogInfo {
    Widget w;
    Widget ok;
    Widget cancel;
    Widget help;
} PrivDialogInfo, *PrivDialogInfoPtr;

typedef struct _messageBoxCbInfo {
    MessageBoxInfo mbi;
    DialogResponse response;
    Boolean synchronous;	/* true if dialog is synchronous */
    PrivDialogInfoPtr dialog;
} MessageBoxCbInfo, *MessageBoxCbInfoPtr;

/*************************************************************************
 *
 * private interfaces for message box dialogs
 *
 *************************************************************************/
static MessageBoxCbInfoPtr
AllocMessageBoxInfo(void)
{
    MessageBoxCbInfoPtr mbi;
    int allocSize = sizeof(MessageBoxCbInfo);

    mbi = (MessageBoxCbInfoPtr)InsecureAlloc(allocSize);
    if (mbi == NULL) {
	Msg("Can't allocate %d bytes for message dialog\n", allocSize);
	return(NULL);
    }
    memset((char *)mbi, 0, allocSize);
    return(mbi);
}

static void
MessageBoxPopdownCB(Widget w, XtPointer userData, XtPointer widgetData)
{
    MessageBoxCbInfoPtr mbcbi = (MessageBoxCbInfoPtr)userData;

    free((char *)mbcbi);
    XtRemoveAllCallbacks(w, XmNpopdownCallback);
}

static void
DismissPopup(PrivDialogInfoPtr dialog)
{
    Widget popup = XtParent(dialog->w);

    XtPopdown(popup);
    XtDestroyWidget(popup);
    dialog->w = NULL;
}

static void
MessageBoxCB(Widget w, XtPointer userData, XtPointer widgetData)
{
    MessageBoxCbInfoPtr mbcbi = (MessageBoxCbInfoPtr)userData;
    MessageBoxInfoPtr mbi = &mbcbi->mbi;
    PrivDialogInfoPtr dialog = mbcbi->dialog;
    XmSelectionBoxCallbackStruct *cbs =
	(XmSelectionBoxCallbackStruct *)widgetData;

    switch (cbs->reason) {
    case XmCR_OK:
	if (mbi->okCB != NULL)
	    (*mbi->okCB)(w, mbi->okData);
	DismissPopup(dialog);
	mbcbi->response = kResponseOk;
	break;

    case XmCR_CANCEL:
	if (mbi->cancelCB != NULL)
	    (*mbi->cancelCB)(w, mbi->cancelData);
	DismissPopup(dialog);
	mbcbi->response = kResponseCancel;
	break;

    case XmCR_HELP:
	/*
	 * This is really an XmManager callback.
	 */
	if (mbi->helpCB != NULL)
	    (*mbi->helpCB)(w, mbi->helpData);
	break;

    case XmCR_MAP:
	/*
	 * This is really an XmBulletinBoard callback (only when it
	 * is a child of a DialogShell).
	 */
	if (mbi->mapCB != NULL)
	    (*mbi->mapCB)(w, mbi->helpData);
	break;
    }
}

#define MwmDialogFunctions	(MWM_FUNC_RESIZE \
			       | MWM_FUNC_MOVE \
			       | MWM_FUNC_MINIMIZE \
			       | MWM_FUNC_MAXIMIZE)
static PrivDialogInfoPtr
GetWorkingDialog(Widget w)
{
    static PrivDialogInfo work;

    if (work.w == NULL) {
        work.w = XmCreateWorkingDialog(FindTopLevel(w),
				       "working", (ArgList)NULL, 0);
        XtVaSetValues(work.w,
            XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
            NULL);
        XtVaSetValues(XtParent(work.w),
            XmNmwmFunctions, MwmDialogFunctions,
            XmNtitle, "Working...",
	    NULL);
	work.ok = XmMessageBoxGetChild(work.w, XmDIALOG_OK_BUTTON);
	work.cancel = XmMessageBoxGetChild(work.w, XmDIALOG_CANCEL_BUTTON);
	work.help = XmMessageBoxGetChild(work.w, XmDIALOG_HELP_BUTTON);
    }
    return(&work);
}

static PrivDialogInfoPtr
GetWarningDialog(Widget w)
{
    static PrivDialogInfo warn;

    if (warn.w == NULL) {
        warn.w = XmCreateWarningDialog(FindTopLevel(w),
				       "warning", (ArgList)NULL, 0);
        XtVaSetValues(warn.w,
            XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
            NULL);
        XtVaSetValues(XtParent(warn.w),
            XmNmwmFunctions, MwmDialogFunctions,
            XmNtitle, "Warning!",
	    NULL);
	warn.ok = XmMessageBoxGetChild(warn.w, XmDIALOG_OK_BUTTON);
	warn.cancel = XmMessageBoxGetChild(warn.w, XmDIALOG_CANCEL_BUTTON);
	warn.help = XmMessageBoxGetChild(warn.w, XmDIALOG_HELP_BUTTON);
    }
    return(&warn);
}

static void
UpdateMessageDialog(Widget w, char *message)
{
    XmString msgString;
    
    msgString = XmStringCreateLtoR(message, XmSTRING_DEFAULT_CHARSET);

    /* determine whether dialog is to determine the default position */
    XtVaSetValues(w, XmNmessageString, msgString, NULL);
    XmStringFree(msgString);
}

static void
PopupMessageDialog(PrivDialogInfoPtr dialog,
		   MessageBoxCbInfoPtr mbcbi,
		   char *message)
{
    MessageBoxInfoPtr mbi = &mbcbi->mbi;

    if (XtIsManaged(XtParent(dialog->w))) {
	Msg("Error: PopupMessageDialog() already managed!\n\n%s\n",
	    message);
    }

    UpdateMessageDialog(dialog->w, message);

    /* determine whether dialog is to determine the default position */
    XtVaSetValues(dialog->w, XmNdefaultPosition, mbi->mapCB == NULL, NULL);

    /*
     * Clean up from previous use
     */
    XtRemoveAllCallbacks(dialog->w, XmNokCallback);
    XtRemoveAllCallbacks(dialog->w, XmNcancelCallback);
    XtRemoveAllCallbacks(dialog->w, XmNhelpCallback);
    XtRemoveAllCallbacks(dialog->w, XmNmapCallback);

    if (mbi->showOkButton || mbi->okCB != NULL) {
	XtAddCallback(dialog->w, XmNokCallback, MessageBoxCB, mbcbi);
	XtManageChild(dialog->ok);
    } else {
	XtUnmanageChild(dialog->ok);
    }

    if (mbi->showCancelButton || mbi->cancelCB != NULL) {
	XtAddCallback(dialog->w, XmNcancelCallback, MessageBoxCB, mbcbi);
	XtManageChild(dialog->cancel);
    } else {
	XtUnmanageChild(dialog->cancel);
    }

    if (mbi->helpCB != NULL) {
	XtAddCallback(dialog->w, XmNhelpCallback, MessageBoxCB, mbcbi);
	XtManageChild(dialog->help);
    } else {
	XtUnmanageChild(dialog->help);
    }

    if (mbi->mapCB != NULL)
	XtAddCallback(dialog->w, XmNmapCallback,
		      MessageBoxCB, (XtPointer)mbcbi);
    if (! mbcbi->synchronous)
	XtAddCallback(XtParent(dialog->w), XmNpopdownCallback,
		      MessageBoxPopdownCB, (XtPointer)mbcbi);

    if (XtIsManaged(dialog->w))
	XtPopup(XtParent(dialog->w), XtGrabExclusive);
    else
	XtManageChild(dialog->w);
}

/*************************************************************************
 *
 * public interfaces for message box dialogs
 *
 *************************************************************************/
void
ReasonDialog(Widget w, MessageBoxInfoPtr mbi, InternalReasonPtr reason)
{
    char msg[ 4 * 1024 ];

    FormatReason(reason, msg);
    WarningDialog(w, mbi, "Sorry: %s", msg);
    FreeReason(reason);
}

void
InternalProblemDialog(Widget w, char *fmt, ...)
{
    va_list ap;
    char msg[ 1024 ], *p = msg;

    strcpy(p, "Internal problem: ");
    p += strlen(p);

    /*
     * Format the message, first.
     */
    va_start(ap, fmt);
    vsprintf(p, fmt, ap);
    va_end(ap);

    (void) SimpleWarningDialog(w, NULL, p);
}

void
AllocFailDialog(Widget w, MessageBoxInfoPtr mbi, char *what, long size)
{
    WarningDialog(w, mbi,
		  "Sorry: Can't allocate %d bytes for %s.", size, what);
}

DialogResponse
WarningDialog(Widget w, MessageBoxInfoPtr mbi, char *fmt, ...)
{
    va_list ap;
    char msg[ 1024 ];

    /*
     * Format the message, first.
     */
    va_start(ap, fmt);
    vsprintf(msg, fmt, ap);
    va_end(ap);

    return( SimpleWarningDialog(w, mbi, msg) );
}

void
StdWarningCancelCB(Widget w, XtPointer userData)
{
    return;	/* the mbi->response will contain kResponseCancel */
}

DialogResponse
SimpleWarningDialog(Widget w, MessageBoxInfoPtr mbi, char *message)
{
    MessageBoxCbInfoPtr mbcbi;
    PrivDialogInfoPtr warn = GetWarningDialog(w);
    DialogResponse response;

    if (NULL == (mbcbi = AllocMessageBoxInfo()))
	return(kNoResponse);

    if (mbi)
	mbcbi->mbi = *mbi;
    mbcbi->mbi.showOkButton = True;	/* force it */
    mbcbi->synchronous = True;
    mbcbi->dialog = warn;

    PopupMessageDialog(warn, mbcbi, message);

    /*
     * Very important!  For these dialogs to appear synchronous, we
     * must call ProcessEvents() here
     */
    while ((response = mbcbi->response) == kNoResponse)
	ProcessEvents(warn->w);
    free((char *)mbcbi);
    
    return(response);
}

void
SimpleWorkingDialog(Widget w, MessageBoxInfoPtr mbi, char *message)
{
    MessageBoxCbInfoPtr mbcbi;
    PrivDialogInfoPtr work = GetWorkingDialog(w);

    if (NULL == (mbcbi = AllocMessageBoxInfo()))
	return;

    if (mbi)
	mbcbi->mbi = *mbi;
    mbcbi->dialog = work;

    PopupMessageDialog(work, mbcbi, message);
}

void
WorkingDialog(Widget w, MessageBoxInfoPtr mbi, char *fmt, ...)
{
    va_list ap;
    char msg[ 1024 ];

    /*
     * Format the message, first.
     */
    va_start(ap, fmt);
    vsprintf(msg, fmt, ap);
    va_end(ap);

    SimpleWorkingDialog(w, mbi, msg);
}

void
CloseWorkingDialog(Widget w)
{
    DismissPopup(GetWorkingDialog(w));
}

void
UpdateWorkingDialog(Widget w, char *fmt, ...)
{
    va_list ap;
    char msg[ 1024 ];
    PrivDialogInfoPtr work = GetWorkingDialog(w);

    /*
     * Format the message, first.
     */
    va_start(ap, fmt);
    vsprintf(msg, fmt, ap);
    va_end(ap);

    UpdateMessageDialog(work->w, msg);
}

