/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	WidgetDialog.h - Motif dialogs for warnings, etc.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/WidgetDialog.h,v 1.3 1995/06/06 19:48:50 toddb Exp $
 */
#ifndef _WIDGETDIALOG_H_
#define _WIDGETDIALOG_H_

#include "nvrReason.h"
#include <Intrinsic.h>

typedef enum {
    kNoResponse,
    kResponseCancel,
    kResponseOk
} DialogResponse;

typedef void (*msgBoxCB)(Widget w, XtPointer);

/*
 * This can be passed to the dialog routines below.  If it is null,
 * a default structure is composed.  If the okCB, cancelCB or helpCB
 * are specified, then those buttons will be shown.  If you don't want
 * a callback for the ok or cancel, then simply set the showOkButton or
 * showCancelButton.
 *
 * All of the warning dialogs (WarningDialog, ReasonDialog, etc) force
 * the ok button to be shown.  The working dialog (PopupWorkingDialog)
 * forces the cancel button to be shown.
 */
typedef struct _messageBoxInfo {
    msgBoxCB okCB;		/* callback for ok button */
    XtPointer okData;		/* callback data for ok button */
    Boolean showOkButton;	/* if true, show ok button */

    msgBoxCB cancelCB;		/* callback for cancel button */
    XtPointer cancelData;	/* callback data for cancel button */
    Boolean showCancelButton;	/* if true, show cancel button */

    msgBoxCB helpCB;		/* callback for help button */
    XtPointer helpData;		/* callback data for help button */

    msgBoxCB mapCB;		/* dialog mapping callback */
    XtPointer mapData;		/* callback data for mapping callback */
} MessageBoxInfo, *MessageBoxInfoPtr;

/*************************************************************************
 *
 * Warning dialogs
 *
 *************************************************************************/
CExternal DialogResponse SimpleWarningDialog(
    Widget w,
    MessageBoxInfoPtr,
    char *message
);

CExternal void InternalProblemDialog(
    Widget w,
    char *format,
    ...
);

CExternal DialogResponse WarningDialog(
    Widget w,
    MessageBoxInfoPtr,
    char *format,
    ...
);

CExternal void ReasonDialog(
    Widget w,
    MessageBoxInfoPtr,
    InternalReasonPtr
);

CExternal void AllocFailDialog(
    Widget w,
    MessageBoxInfoPtr,
    char *what,
    long size
);

CExternal void StdWarningCancelCB(
    Widget w,
    XtPointer userData
);

/*************************************************************************
 *
 * Working dialogs
 *
 *************************************************************************/
CExternal void SimpleWorkingDialog(
    Widget w,
    MessageBoxInfoPtr,
    char *message
);

CExternal void WorkingDialog(
    Widget w,
    MessageBoxInfoPtr,
    char *format,
    ...
);

CExternal void CloseWorkingDialog(
    Widget w
);

CExternal void UpdateWorkingDialog(
    Widget w,
    char *fmt,
    ...
);

#endif /* _WIDGETDIALOG_H_ */
