/*
 * Copyright (c) 1992, 1993, 1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	WidgetInternal.c - debugging, diagnostic tools for widgets
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/WidgetInternal.c,v 1.10 1995/06/06 19:49:17 toddb Exp $
 */
#include "WidgetUtil.h"
#include "WidgetDebug.h"
#include "WidgetDialog.h"
#include "osio.h"
#include "nvrString.h"
#include "nvrAlloc.h"

#include <IntrinsicP.h>
#include <StringDefs.h>
#include <Xm/Xm.h>
#include <Xm/XmStrDefs.h>
#include <CoreP.h>
#include <CompositeP.h>
#include <stdio.h>

char *
GetClassPath(Widget w, char *path, int pathLen)
{
    char *name;
    char *ptr;
    Widget pw, top;
    int length, totalLength;
    WidgetClass wclass;

    totalLength = 0;
    *path = '\0';
    top = NULL;
    for (pw = w; pw; pw = XtParent(pw)) {
	wclass = XtClass(pw);
	name = XrmClassToString(wclass->core_class.xrm_class);
	length = strlen(name) + 1;
	if (totalLength + length >= pathLen) {
	    top = w;
	    break;
	}
	totalLength += length;
    }
    if (totalLength == 0)
	return(path);

    ptr = path + totalLength;
    *ptr = '\0';
    for (pw = w; pw != top; pw = XtParent(pw)) {
	wclass = XtClass(pw);
	name = XrmClassToString(wclass->core_class.xrm_class);
	ptr -= strlen(name);
	strncpy(ptr, name, strlen(name));
	*--ptr = '.';
    }
    return(ptr+1);
}

void
ShowClassPath(Widget w)
{
    char path[1024];

    Msg("class \"%s\"\n", GetClassPath(w, path, sizeof(path)) );
}

char *
GetWidgetPath(Widget w, char *path, int pathLen)
{
    char *name;
    char *ptr;
    Widget pw, top;
    int length, totalLength;

    totalLength = 0;
    *path = '\0';
    top = NULL;
    for (pw = w; pw; pw = XtParent(pw)) {
	name = XtName(pw);
	length = strlen(name) + 1;
	if (totalLength + length >= pathLen) {
	    top = w;
	    break;
	}
	totalLength += length;
    }
    if (totalLength == 0)
	return(path);

    ptr = path + totalLength;
    *ptr = '\0';
    for (pw = w; pw != top; pw = XtParent(pw)) {
	name = XtName(pw);
	ptr -= strlen(name);
	strncpy(ptr, name, strlen(name));
	*--ptr = '.';
    }
    return(ptr+1);
}

void
ShowWidgetPath(Widget w)
{
    char path[1024];

    Msg("widget \"%s\"\n", GetWidgetPath(w, path, sizeof(path)) );
}

static void
ShowWidgetSubtree(Widget node, int indent)
{
    char *name = XtName(node);
    WidgetClass wc = XtClass(node);
    char *className = XrmClassToString(wc->core_class.xrm_class);
    char *separator;
    int nameLen = strlen(name);
    int i, count;
    
    if (indent == 0) {
	separator = "";
    } else {
	separator = ".";
	nameLen++;
    }
    Msg("%*s%s%s (%s)\n", indent, "", separator, name, className);

    indent += nameLen;
    if (XtIsComposite(node)) {
	count = ((CompositeWidget) node)->composite.num_children;
	for (i = 0; i < count; i++) {
	    ShowWidgetSubtree(((CompositeWidget) node)->composite.children[i],
			indent);
	}
    }

    if (node->core.popup_list) {
	count = node->core.num_popups;
	for (i = 0; i < count; i++) {
	    ShowWidgetSubtree(node->core.popup_list[i],
			indent);
	}
    }
}

void
ShowWidgetTree(Widget root)
{
    ShowWidgetSubtree(root, 0);
}

static void
SimpleCvt(XtPointer value, int size, char *string)
{
    switch(size) {
    case 1: sprintf(string, "%u", *((unsigned char *)&value)); break;
    case 2: sprintf(string, "%u", *((unsigned short *)&value)); break;
    case 4: sprintf(string, "%u", *((unsigned long *)&value)); break;
    default: sprintf(string, "0x%x", (int)value); break;
    }
}

static void
StringCvt(XtPointer value, int size, char *string)
{
    if (value == NULL)
	strcpy(string, "null");
    else
	strcpy(string, (char *)value);
}

static void
FloatCvt(XtPointer value, int size, char *string)
{
    sprintf(string, "%g", *(float *)value);
}

static void
FontListCvt(XtPointer value, int size, char *string)
{
    XmFontContext context;
    XmFontList fontList = (XmFontList)value;
    XmStringCharSet charset;
    XFontStruct *font;
    int count;

    if (! XmFontListInitFontContext(&context, fontList)) {
	sprintf(string, "Can't allocate font context");
	return;
    }
    count = 0;
    while (XmFontListGetNextFont(context, &charset, &font)) {
	sprintf(string, "\n%d: charset %s, %d props",
	    ++count, charset, font->n_properties);
	string += strlen(string);
	free(charset);
    }
}

static void
VoidCvt(XtPointer value, int size, char *string)
{
    sprintf(string, "0x%x", (int) value);
}

static void
SelectionPolicyCvt(XtPointer value, int size, char *string)
{
    UChar policy;

    switch( policy = *UCharP(&value) ) {
    case XmSINGLE_SELECT:   strcpy(string, "single"); break;
    case XmMULTIPLE_SELECT: strcpy(string, "multiple"); break;
    case XmEXTENDED_SELECT: strcpy(string, "extended"); break;
    case XmBROWSE_SELECT:   strcpy(string, "browse"); break;
    default: sprintf(string, "unknown (0x%x)", policy); break;
    }
}

static void
DialogTypeCvt(XtPointer value, int size, char *string)
{
    UChar type;

    switch( type = *UCharP(&value) ) {
    case XmDIALOG_ERROR:       strcpy(string, "error"); break;
    case XmDIALOG_INFORMATION: strcpy(string, "information"); break;
    case XmDIALOG_MESSAGE:     strcpy(string, "message"); break;
    case XmDIALOG_QUESTION:    strcpy(string, "question"); break;
    case XmDIALOG_TEMPLATE:    strcpy(string, "template"); break;
    case XmDIALOG_WARNING:     strcpy(string, "warning"); break;
    case XmDIALOG_WORKING:     strcpy(string, "working"); break;
    default: sprintf(string, "unknown (0x%x)", type); break;
    }
}

static void
DialogStyleCvt(XtPointer value, int size, char *string)
{
    UChar style;
    char *s = NULL;

    switch( style = *UCharP(&value) ) {
    case XmDIALOG_MODELESS:                  s = "modeless"; break;
    case XmDIALOG_FULL_APPLICATION_MODAL:    s = "full app modal"; break;
    case XmDIALOG_APPLICATION_MODAL:         s = "app modal"; break;
    case XmDIALOG_SYSTEM_MODAL:              s = "system modal"; break;
    default: sprintf(string, "unknown (0x%x)", style); break;
    }

    if (s)
	strcpy(string, s);
}

static void
SizePolicyCvt(XtPointer value, int size, char *string)
{
    UChar policy;

    switch( policy = *UCharP(&value) ) {
    case XmVARIABLE: strcpy(string, "variable"); break;
    case XmCONSTANT: strcpy(string, "constant"); break;
    case XmRESIZE_IF_POSSIBLE: strcpy(string, "resize if possible"); break;
    default: sprintf(string, "unknown (0x%x)", policy); break;
    }
}

#define NXtTypes	65

typedef struct _xtTypeInfo {
    char *name;
    int size;
    void (*converter)(XtPointer value, int size, char *string);
} XtTypeInfo, *XtTypeInfoPtr;

static XtTypeInfo XtTypes[ NXtTypes ];

static boolean
InitTypeLists(void)
{
    XtTypeInfoPtr t = &XtTypes[0];

#define AddType(typeInfo,string,type,converterProc)			\
    typeInfo->name = string;						\
    typeInfo->size = sizeof(type);					\
    typeInfo->converter = converterProc;				\
    if (++typeInfo - XtTypes >= NXtTypes)				\
	return(False);

    AddType(t, XmRFontList, void *, FontListCvt);
    AddType(t, XmRVerticalDimension, short, SimpleCvt);
    AddType(t, XmRHorizontalDimension, short, SimpleCvt);

    AddType(t, XtRAcceleratorTable, XtAccelerators, VoidCvt);
    AddType(t, XtRAtom, Atom, VoidCvt);
    AddType(t, XtRBitmap, Pixmap, VoidCvt);
    AddType(t, XtRBool, Bool, SimpleCvt);
    AddType(t, XtRBoolean, Boolean, SimpleCvt);
    AddType(t, XtRCallback, void *, VoidCvt);
    AddType(t, XtRCardinal, Cardinal, SimpleCvt);
    AddType(t, XtRColor, XColor, VoidCvt);
    AddType(t, XtRColormap, Colormap, VoidCvt);
    AddType(t, XtRCursor, Cursor, VoidCvt);
    AddType(t, XtRDimension, Dimension, SimpleCvt);
    AddType(t, XtRDisplay, Display *, VoidCvt);
    AddType(t, XtREditMode, void *, SimpleCvt);
    AddType(t, XtREnum, XtEnum, SimpleCvt);
    AddType(t, XtRFile, FILE *, VoidCvt);
    AddType(t, XtRFloat, float, FloatCvt);
    AddType(t, XtRFont, Font, VoidCvt);
    AddType(t, XtRFontStruct, XFontStruct *, VoidCvt);
    AddType(t, XtRFunction, void *, VoidCvt);
    AddType(t, XtRGeometry, void *, VoidCvt);
    AddType(t, XtRImmediate, int, SimpleCvt);
    AddType(t, XtRInitialState, int, SimpleCvt);
    AddType(t, XtRInt, int, SimpleCvt);
    AddType(t, XtRJustify, void *, SimpleCvt);
    AddType(t, XtRLongBoolean, long, SimpleCvt);
    AddType(t, XtRObject, Object, VoidCvt);
    AddType(t, XtROrientation, void *, VoidCvt);
    AddType(t, XtRPixel, Pixel, VoidCvt);
    AddType(t, XtRPixmap, Pixmap, VoidCvt);
    AddType(t, XtRPointer, XtPointer, VoidCvt);
    AddType(t, XtRPosition, Position, SimpleCvt);
    AddType(t, XtRScreen, Screen *, VoidCvt);
    AddType(t, XtRShort, short, VoidCvt);
    AddType(t, XtRString, char *, StringCvt);
    AddType(t, XtRStringArray, String *, VoidCvt);
    AddType(t, XtRStringTable, char **, VoidCvt);
    AddType(t, XtRUnsignedChar, UChar, SimpleCvt);
    AddType(t, XtRTranslationTable, void *, VoidCvt);
    AddType(t, XtRVisual, Visual *, VoidCvt);
    AddType(t, XtRWidget, Widget, VoidCvt);
    AddType(t, XtRWidgetClass, WidgetClass, VoidCvt);
    AddType(t, XtRWidgetList, WidgetList, VoidCvt);
    AddType(t, XtRWindow, Window, VoidCvt);

    AddType(t, XmRListSizePolicy, UChar, SizePolicyCvt);
    AddType(t, XmRSelectionPolicy, UChar, SelectionPolicyCvt);
    AddType(t, XmRDialogType, UChar, DialogTypeCvt);
    AddType(t, XmRDialogStyle, UChar, DialogStyleCvt);

    return(True);

#undef AddType
}

static void
ShowResourceValue(Widget w, XtResourceList rsc)
{
    Arg arg;
    XtTypeInfoPtr t;
    int i;

    XtPointer value = NULL;
    char msgBuf[ 1024 ];
    char *valueMsg = msgBuf;

    XtSetArg(arg, rsc->resource_name, (XtPointer) &value);
    XtGetValues(w, &arg, 1);

    t = &XtTypes[0];
    if (t->name == NULL && ! InitTypeLists())
	WarningDialog(w, NULL, "not enough storage for Xt types");

    valueMsg[0] = '?';
    valueMsg[1] = '\0';
    for (i = 0; i < NXtTypes; i++, t++) {
	if (t->name == NULL)
	    break;
	if (strcmp(rsc->resource_type, t->name) == 0) {
	    (*t->converter)(value, t->size, valueMsg);
	    break;
	}
    }
    if (i == NXtTypes || t->name == NULL)
	VoidCvt(value, t->size, valueMsg);

    Msg("    %s (%s) = %s\n", rsc->resource_name, rsc->resource_type, valueMsg);
}

/*
 * Show one resource (in all the classes)
 */
void
ShowWidgetResource(Widget w, char *rscName)
{
    WidgetClass wclass;
    XtResourceList rscList;
    Cardinal nResources;
    int i, nFound;
    char *className;

    wclass = XtClass(w);
    nFound = 0;
    while (wclass != NULL) {
	XtGetResourceList(wclass, &rscList, &nResources);
	for (i = 0; i < nResources; i++) {
	    if (strcmp(rscList[i].resource_name, rscName) == 0) {
		Msg("\n");
		className = XrmClassToString(wclass->core_class.xrm_class);
		Msg("  %s class:", className);
		ShowResourceValue(w, &rscList[i]);
		nFound++;
		break;
	    }
	}
	if (nResources)
	    XtFree((char *)rscList);
        wclass = wclass->core_class.superclass;
    }
    if (nFound == 0) {
	XtPointer value = 0;
	XtVaGetValues(w, rscName, &value, NULL);
	Msg("can't find %s, GetValues return 0x%x\n", rscName, value);
    }
}

typedef struct _classResources {
    XtResourceList rscList;
    Cardinal nResources;
    WidgetClass wclass;
} ClassResources, *ClassResourcesPtr;

static boolean
InClassResourceList(ClassResourcesPtr classResources,
		    int nClasses,
		    XtResource *rsc)
{
    int i, j;
    Cardinal nResources;
    XtResourceList list;

    for (i = 0; i < nClasses; i++) {
	nResources = classResources[ i ].nResources;
	list = classResources[ i ].rscList;
	for (j = 0; j < nResources; list++, j++) {
	    if (strcmp(list->resource_name, rsc->resource_name) == 0)
		return(True);
	}
    }
    return(False);
}

/*
 * Show all resources
 */
void
ShowWidgetResources(Widget w, int level)
{
    WidgetClass wclass;
    XtResourceList rscList, rsc;
    Cardinal nResources;
    int i, j;
    char *className;
    ClassResources classResources[ 20 ];
    ClassResourcesPtr superClassResources;
    int nClasses, nSuperClasses;

    if (level == 0)
	level = 0x7fffffff;

    ShowWidgetPath(w);
    wclass = XtClass(w);
    nClasses = 0;
    while (--level >= 0 && wclass != NULL) {
	classResources[ nClasses ].wclass = wclass;
	XtGetResourceList(wclass, &rscList, &nResources);
	classResources[ nClasses ].rscList = rscList;
	classResources[ nClasses ].nResources = nResources;
        wclass = wclass->core_class.superclass;
	nClasses++;
    }

    for (i = nClasses - 1; i >= 0; i--) {
	wclass = classResources[ i ].wclass;
	rscList = classResources[ i ].rscList;
	superClassResources = &classResources[ i+1 ];
	nSuperClasses = nClasses - (i+1);
	nResources = classResources[ i ].nResources;

	className = XrmClassToString(wclass->core_class.xrm_class);
	Msg("  %s class:\n", className);
	for (j = 0, rsc = &rscList[0]; j < nResources; j++, rsc++) {
	    /*
	     * Check that this resource is not already listed in
	     * a super class
	     */
	    if (nSuperClasses > 0
	     && InClassResourceList(superClassResources, nSuperClasses, rsc))
		continue;
	    ShowResourceValue(w, rsc);
	}
    }

    for (i = 0; i < nClasses; i++) {
	if (classResources[ i ].nResources)
	    XtFree((char *)classResources[ i ].rscList);
    }
}

WidgetClass
XtSuperClass(WidgetClass wclass)
{
    return( wclass->core_class.superclass );
}

char *
XtClassToString(WidgetClass wclass)
{
    return( XrmClassToString(wclass->core_class.xrm_class) );
}
