/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *      WidgetUtil.c - X widget creation utils
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvr/applications/RCS/OnePlay.c,v 1.5 1994/10/03 02:00
:35 toddb Exp $
 */
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/AtomMgr.h>
#include <Xm/Protocols.h>
#include <Xm/MwmUtil.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/DrawingA.h>
#include <Xm/ToggleB.h>
#include <Xm/Scale.h>
#include <Xm/CascadeB.h>
#include <Xm/ScrolledW.h>
#include <Xm/List.h>
#include <Xm/FileSB.h>
#include <Xm/Text.h>
#include <Xm/DialogS.h>
#include <Xm/ArrowB.h>
#include <Xm/Separator.h>
#include <ctype.h>

#include "WidgetUtil.h"
#include "WidgetDebug.h"
#include "WidgetDialog.h"

typedef struct _instanceInfo {
    char *picFile;
    char *insensitiveFile;
} InstanceInfo;

XtResource ResTemplate[] = {
    {
	"pic", "Pic", XtRString, sizeof(String),
	XtOffsetOf(InstanceInfo, picFile), XtRString, (caddr_t)NULL
    },
    {
	"insensitivePic", "InsensitivePic", XtRString, sizeof(String),
	XtOffsetOf(InstanceInfo, insensitiveFile), XtRString, (caddr_t)NULL
    }
};

char *
WidgetClassNames[] = {
    "TopLevelShell",
    "PushButton",
    "RowColumn",
    "Form",
    "DrawingArea",
    "Label",
    "Toggle",
    "Frame",
    "Scale",
    "Cascade",
    "MenuBar",
    "Menu",
    "List",
    "ScrolledWindow",
};

static void
CantCreateWidget(char *name, WidgetClass wclass, Widget parent)
{
    char widgetPath[ 1024 ];

    InternalProblemDialog(parent, "can't create %s: %s.%s\n",
	wclass ? XtClassToString(wclass) : "(internal class)",
	GetWidgetPath(parent, widgetPath, sizeof(widgetPath)),
	name);
}

static Widget
CreateManagedWidget(char *name,
		    WidgetClass wclass,
		    Widget parent,
		    Boolean mapped)
{
    Widget w;

    w = XtVaCreateManagedWidget(name, wclass, parent,
				XmNmappedWhenManaged, mapped,
				NULL);
    if (w == NULL)
	CantCreateWidget(name, wclass, parent);
    return(w);
}

static Widget
CreateMenu(Widget parent, char *name)
{
    Widget menu, cascade;
    char cascadeName[ 256 ];

    menu = XmCreateSimplePulldownMenu(parent, name, NULL, 0);
    if (menu == NULL) {
	CantCreateWidget(name, NULL, parent);
	return(NULL);
    }

    sprintf(cascadeName, "cascade%s", name);
    if (islower(cascadeName[7]))
	cascadeName[7] = toupper(cascadeName[7]);
    cascade = CreateManagedWidget(cascadeName,
				  xmCascadeButtonWidgetClass, parent, True);
    if (cascade == NULL)
	return(NULL);
    XtVaSetValues(cascade, XmNsubMenuId, menu, NULL);

    return(menu);
}

Widget
CreateWidgetTree(Widget treeParent,
		 UserWidgetDescPtr descList,
		 long nDesc,
		 PixmapResourcePtr pixRes)
{
    InstanceInfo instance;
    UserWidgetDescPtr desc = descList;
    Widget w, parent, tree;
    WidgetClass wclass;
    Boolean mapped, callbackInstalled;
    WParamPtr param;
    XmString string;
    Arg arg;
    int descIndex;

    if (desc[0].parent)
	InternalProblemDialog(treeParent,
	    "First entry for description list must have NULL parent");

    tree = NULL;
    for (descIndex = 0; descIndex < nDesc; descIndex++, desc++) {
	if (tree) {
	    if (desc->parent)
		parent = XtNameToWidget(tree, desc->parent);
	    else
		parent = tree;
	    mapped = True;
	} else {
	    parent = treeParent;
	    mapped = False;
	}
	if (parent == NULL) {
	    InternalProblemDialog(treeParent, "Can't find %s\n", desc->parent);
	    continue;
	}

	/*
	 * First we decide which widget class to use.  Some of the WC_
	 * values are abstract in order to specify the convenience functions.
	 * The result of this switch is either:
	 *   - wclass gets set to the widget class
	 *   - the widget w is created through a convenience function
	 *     and wclass is set to NULL.
	 */
	switch (desc->wclass) {
	case WC_TopLevelShell: wclass = topLevelShellWidgetClass; break;
	case WC_DialogShell:   wclass = xmDialogShellWidgetClass; break;
	case WC_PushButton:    wclass = xmPushButtonWidgetClass; break;
	case WC_ArrowButton:   wclass = xmArrowButtonWidgetClass; break;
	case WC_RowColumn:     wclass = xmRowColumnWidgetClass; break;
	case WC_Form:          wclass = xmFormWidgetClass; break;
	case WC_DrawingArea:   wclass = xmDrawingAreaWidgetClass; break;
	case WC_Label:         wclass = xmLabelWidgetClass; break;
	case WC_Toggle:        wclass = xmToggleButtonWidgetClass; break;
	case WC_Frame:         wclass = xmFrameWidgetClass; break;
	case WC_Scale:         wclass = xmScaleWidgetClass; break;
	case WC_Cascade:       wclass = xmCascadeButtonWidgetClass; break;
	case WC_ScrolledWin:   wclass = xmScrolledWindowWidgetClass; break;
	case WC_List:          wclass = xmListWidgetClass; break;
	case WC_Text:          wclass = xmTextWidgetClass; break;
	case WC_Separator:     wclass = xmSeparatorWidgetClass; break;
	case WC_BulletinBoard: wclass = xmBulletinBoardWidgetClass; break;

	case WC_FormDialog:
	    if (tree) {
		InternalProblemDialog(treeParent,
		    "form dialog \"%s\" must be first in the list.",
		    desc->name);
		XtDestroyWidget(tree);
		return(NULL);
	    }
	    w = XmCreateFormDialog(parent, desc->name, (ArgList)NULL, 0);
	    wclass = NULL;
	    mapped = True;
	    tree = XtParent(w);
	    break;

	case WC_ScrolledList:
	    w = XmCreateScrolledList(parent, desc->name, (ArgList)NULL, 0);
	    XtManageChild(w);
	    wclass = NULL;
	    break;

	case WC_Menu:
	    w = CreateMenu(parent, desc->name);
	    wclass = NULL;
	    break;

	case WC_MenuBar:
	    w = XmVaCreateSimpleMenuBar(parent, desc->name, NULL);
	    if (w)
		XtManageChild(w);
	    wclass = NULL;
	    break;

	case WC_FileSelect:
	    /*
	     * Note that we don't map it because we assume that
	     * that happens as a result of some menu button being pressed
	     */
	    wclass = NULL;
	    XtSetArg(arg, XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL);
	    w = XmCreateFileSelectionDialog(parent, desc->name, &arg, 1);
	    break;

	default:
	    InternalProblemDialog(parent, "unknown wclass: %d\n", desc->wclass);
	    continue;
	}

	if (wclass)
	    w = CreateManagedWidget(desc->name, wclass, parent, mapped);

	if (w == NULL) {
	    if (wclass == NULL)
		CantCreateWidget(desc->name, NULL, parent);
	    continue;
	}

	/*
	 * This is the top level for this tree
	 */
	if (! mapped)
	    tree = w;

	/*
	 * If there are parameters attached to this widget, set them here.
	 */
	callbackInstalled = False;
	if (param = desc->param) {
	    while (param) {
		switch(param->any.type) {
		case WP_Callback:
		    callbackInstalled = True;
		    XtAddCallback(w, param->cb.name,
			    param->cb.callback, param->cb.arg);
		    param = param->cb.next;
		    break;

		case WP_FileSelectProc:
		    if (param->fsb.defaultDir)
			string = XmStringCreateSimple(param->fsb.defaultDir);
		    else
			string = NULL;	/* this makes it use "." */
		    XtVaSetValues(w,
			XmNfileSearchProc, param->fsb.fileSearchProc,
			XmNdirectory, string,
			NULL);
		    if (string)
			XmStringFree(string);
		    param = param->fsb.next;
		    break;
		}
	    }
	}

	/*
	 * If no callback was installed, and this is a toggle or a push
	 * button, then the user interface is not yet fully implemented
	 * and we set the widget to be insensitive (so its obvious).
	 */
	if (! callbackInstalled) {
	    switch (desc->wclass) {
	    case WC_PushButton:
	    case WC_Toggle:
	    case WC_Scale:
		XtVaSetValues(w, XmNsensitive, False, NULL);
		break;

	    default:
		break;	/* its not important */
	    }
	}

	instance.picFile = NULL;
	instance.insensitiveFile = NULL;
	XtGetApplicationResources(w, &instance, ResTemplate,
				  XtNumber(ResTemplate), NULL, 0);
	if (instance.picFile) {
	    SetLabelPixmap(pixRes, w,
		instance.picFile, instance.insensitiveFile);
	}
    }

    if (tree) {
	/* XtMapWidget(tree); */
	XtVaSetValues(tree, XmNmappedWhenManaged, True);
    }
    return(tree);
}

/*
 * This function searches for a top level widget.  It should be
 * a topLevelShellWidgetClass, but may be something else in which
 * case this probably won't yield the desired results.  Nevertheless,
 * it *never* returns null.
 */
Widget
FindTopLevel(Widget w)
{
    WidgetClass wclass;
    Widget parent;

    for (;;) {
	parent = XtParent(w);
	wclass = XtClass(w);
	if (parent == NULL || wclass == topLevelShellWidgetClass)
	    return(w);
	w = parent;
    }
}

/*
 * This function searches for the parent shell widget
 */
Widget
FindShell(Widget w)
{
    Widget parent;
    WidgetClass wclass;

    for (;;) {
	parent = XtParent(w);
	wclass = XtClass(w);
	if (parent == NULL)
	    return(w);
	while (wclass) {
	    if (strcmp(XtClassToString(wclass), "Shell") == 0)
		return(w);
	    wclass = XtSuperClass(wclass);
	}
	w = parent;
    }
}

/*
 * This function searches for the top-most widget.
 */
Widget
FindAppShell(Widget w)
{
    Widget parent;

    for (;;) {
	parent = XtParent(w);
	if (parent == NULL)
	    return(w);
	w = parent;
    }
}

/*
 * This function searches for a widget named 'targetName' starting
 * from widget w.  Each time a search fails, it moves up one level
 * in the widget instance hierarchy and tries again.
 */
Widget
FindWidget(Widget w, char *targetName)
{
    Widget parent, target;

    parent = w;
    for (;;) {
	target = XtNameToWidget(parent, targetName);
	if (target)
	    return(target);
	parent = XtParent(parent);
	if (parent == NULL) {
	    char wpath[ WidgetPathLength ];

	    InternalProblemDialog(w, "%s \"%s\"\n starting from %s",
		"Internal Problem: can't find widget named",
		targetName, GetWidgetPath(w, wpath, WidgetPathLength));
	    return(NULL);
	}
    }
}

/*
 * Call this function in order to popup the widget named in
 * 'widgetName.  This should be called from within button callback such
 * as on a menu.
 */
void
StdShellPopup(Widget w, char *widgetName)
{
    Widget popup;

    popup = FindWidget(w, widgetName);
    if (popup)
	XtManageChild(popup);
}

/*
 * Specify this callback in order to pop down the widget w.  This
 * should be attached to the XmNcancelCallback.
 */
void
StdCancelCB(Widget w, XtPointer userArg, XtPointer widgetArg)
{
    XtUnmanageChild(w);
}

void
UnmapWidgetList(Widget w, char **wNames)
{
    char *wName;
    Widget target;

    while (wName = *wNames++) {
	target = FindWidget(w, wName);
	if (target == NULL)
	    continue;
	if (XtWindow(target) == 0)
	    XtVaSetValues(target, XmNmappedWhenManaged, True, NULL);
	else
	    XtUnmapWidget(target);
    }
}

void
MapWidgetList(Widget w, char **wNames)
{
    char *wName;
    Widget target;

    while (wName = *wNames++) {
	target = FindWidget(w, wName);
	if (target == NULL)
	    continue;
	if (XtWindow(target) == 0)
	    XtVaSetValues(target, XmNmappedWhenManaged, False, NULL);
	else
	    XtMapWidget(target);
    }
}

/*
 * This sets a resource value for a named widget.
 */
void
SetResource(Widget w, char *wName, char *rscName, XtPointer rscValue)
{
    Arg arg;
    Widget target = FindWidget(w, wName);

    if (target == NULL)
	return;		/* FindWidget() failed */
    XtSetArg(arg, rscName, rscValue);
    XtSetValues(target, &arg, 1);
}

/*
 * Similar to SetResource, except that this sets a boolean
 * value for a named widget after checking its current state.  This
 * can prevent a widget from flashing by avoiding an unnecessary SetValue call.
 */
void
SetResourceState(Widget w, char *wName, char *rscName, Boolean state)
{
    Arg arg;
    Widget target = FindWidget(w, wName);
    Boolean currentState = False;

    if (target == NULL)
	return;		/* FindWidget() failed */
    XtSetArg(arg, rscName, &currentState);
    XtGetValues(target, &arg, 1);

    if (currentState == state)
	return;

    XtSetArg(arg, rscName, state);
    XtSetValues(target, &arg, 1);
}

Boolean
GetResourceState(Widget w, char *wName, char *rscName)
{
    Arg arg;
    Widget target = FindWidget(w, wName);
    Boolean currentState = False;

    if (target == NULL)
	return(False);		/* FindWidget() failed */
    XtSetArg(arg, rscName, &currentState);
    XtGetValues(target, &arg, 1);
    return(currentState);
}

void
SetResourceList(Widget w, char **wNames, char *rscName, XtPointer rscValue)
{
    Arg arg;
    char *wName;
    Widget target;

    XtSetArg(arg, rscName, rscValue);

    while (wName = *wNames++) {
	target = FindWidget(w, wName);
	if (target == NULL)
	    continue;
	XtSetValues(target, &arg, 1);
    }
}

void
SetToggleState(Widget w, char *wName, Boolean state)
{
    Widget target = FindWidget(w, wName);

    if (target == NULL)
        return;         /* FindWidget() failed */

    XmToggleButtonSetState(target, state, state);
}

void
SetWidgetRscString(Widget w, char *resource, char *string)
{
    Arg arg;
    XmString rscString;

    if (w == NULL)
	return;		/* FindWidget() failed */

    rscString = XmStringCreateSimple(string);
    if (rscString == NULL)
	return;
    XtSetArg(arg, resource, rscString);
    XtSetValues(w, &arg, 1);
    XmStringFree(rscString);
}

void
SetWidgetLabel(Widget w, char *label)
{
    SetWidgetRscString(w, XmNlabelString, label);
}

void
SetNumberedWidgetLabel(Widget w, long number)
{
    char numberBuf[32];

    sprintf(numberBuf, "%d", number);

    /*
     * For updating something like a frame counter,
     * this is far too painful, and should be a set of direct X calls.
     * Otherwise, this is just fine the way it is.
     */
    SetWidgetLabel(w, numberBuf);
}

void
SetWidgetSensitivity(Widget w, char *wName, Boolean sensitive)
{
    Widget target = FindWidget(w, wName);

    if (target == NULL)
        return;         /* FindWidget() failed */

    if (XtIsSensitive(target) != sensitive)
	XtSetSensitive(target, sensitive);
}

/*
 * When storing structures in widget user data, it is safest to have
 * a magic number in the structure so that you don't collide with other
 * software using the widget user data.  This converts a (presumably unique)
 * string into a magic number
 */
ULong
StringToMagic(char *string)
{
    ULong magic = 0;
    int shift = 0, byte;

    while (byte = *string++) {
	magic += byte << shift;
	shift = (shift + 8) & 0x1f;
    }
    return(magic);
}

/*
 * This function stores the pointer onto the specified widget in
 * the XmNuserData resource.  If something appears to already be
 * there, it reports the (internal) error and doesn't change things.
 *
 * User data can be stored on any widget subclassed from an XmManager.
 * A form is good enough.
 */
void
StoreUserData(Widget w, XtPointer data)
{
    XtPointer oldData;
    char path[ 1024 ];

    if (w == NULL)
        return;		/* probably FindWidget() failed */

    oldData = (XtPointer)1;
    XtVaGetValues(w, XmNuserData, &oldData, NULL);
    if (oldData == (XtPointer)1) {
	InternalProblemDialog(w,
	    "It seems widget \"%s\"\ndoes not have a %s resource.",
	    GetWidgetPath(w, path, sizeof(path)), XmNuserData);
	return;
    }
    if (oldData) {
	InternalProblemDialog(w, "Widget \"%s\"\nalready has user data @0x%x.",
	    GetWidgetPath(w, path, sizeof(path)), oldData);
	return;
    }

    XtVaSetValues(w, XmNuserData, data, NULL);
}

XtPointer
GetUserData(Widget w, char *magicName)
{
    char path[ 1024 ];
    XtPointer data;
    ULong *pMagic, magic;

    if (w == NULL)
        return(NULL);   /* probably FindWidget() failed */

    XtVaGetValues(w, XmNuserData, &data, NULL);
    if (data == NULL) {
	InternalProblemDialog(w, "No user data stored on \"%s\".",
	    GetWidgetPath(w, path, sizeof(path)));
	return(NULL);
    }
    pMagic = (ULong *)data;
    magic = StringToMagic(magicName);
    if (*pMagic != magic) {
	InternalProblemDialog(w, "\"%s\":\nExpected magic 0x%x, got 0x%x.",
	    GetWidgetPath(w, path, sizeof(path)), *pMagic, magic);
	return(NULL);
    }

    return(data);
}

void
ProcessEvents(Widget w)
{
    XtAppContext context = XtDisplayToApplicationContext(XtDisplay(w));
    XEvent event;

    XtAppNextEvent(context, &event);
    XtDispatchEvent(&event);
}


Boolean
CheckAppEvents(Widget w)
{
    XtAppContext context = XtDisplayToApplicationContext(XtDisplay(w));
    XtInputMask mask;

    mask = XtAppPending(context);
    if (mask)
	XtAppProcessEvent(context, mask);
    return(mask != 0);
}

static void
DescendWidgetTree(Widget w, void (*func)(Widget, XtPointer), XtPointer data)
{
    WidgetList kids = NULL;
    Cardinal nKids = 0, kid;

    XtVaGetValues(w, XmNchildren, &kids,
		     XmNnumChildren, &nKids,
		     NULL);
    for (kid = 0; kid < nKids; kid++)
	DescendWidgetTree(kids[ kid ], func, data);

    (*func)(w, data);
}

static void
InstallWindowAccelerators(Widget w, XtPointer data)
{
    Widget button = (Widget)data;

    if (w != button) {
	XtInstallAccelerators(w, button);
    }
}

void
SetDefaultButton(Widget button)
{
    char *defaultTranslations = "#override <Key>Return: ArmAndActivate()";
    XtTranslations trans;
    XtAccelerators accel;

    if (button == NULL)
	return;

    trans = XtParseTranslationTable(defaultTranslations);
    XtOverrideTranslations(button, trans);

    accel = XtParseAcceleratorTable(defaultTranslations);
    XtVaSetValues(button, XmNaccelerators, accel, NULL);

    DescendWidgetTree(FindShell(button), InstallWindowAccelerators, button);
}
