/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	WidgetUtil.h - X widget creation utils
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/WidgetUtil.h,v 1.14 1995/06/30 19:03:46 toddb Exp $
 */
#ifndef _WIDGETUTIL_H_
#define _WIDGETUTIL_H_

#include "nvrTypes.h"
#include "nvrReason.h"
#include <Intrinsic.h>
#include "WidgetPixmap.h"

#define WidgetPathLength	256	/* should be long enough */

typedef enum {
    WC_TopLevelShell,
    WC_DialogShell,
    WC_PushButton,
    WC_ArrowButton,
    WC_RowColumn,
    WC_Form,
    WC_FormDialog,
    WC_DrawingArea,
    WC_Label,
    WC_Toggle,
    WC_Separator,
    WC_Frame,
    WC_Scale,
    WC_Cascade,
    WC_MenuBar,
    WC_Menu,
    WC_List,
    WC_ScrolledList,
    WC_ScrolledWin,
    WC_FileSelect,
    WC_Text,
    WC_BulletinBoard
} WidgetClassEnum;

/*
 * These param types and matching structures are used to specify
 * parameters for particular widget classes.  They are collected below
 * in the WParam union.
 */
typedef enum {
    WP_Callback,
    WP_FileSelectProc
} WParamType;

typedef union _wiParam WParam, *WParamPtr;

typedef struct _widgetAnyParam {
    WParamType type;		/* must be first */
} WidgetAnyParam, *WidgetAnyParamPtr;

typedef struct _widgetFsbDesc {
    WParamType type;		/* must be first */
    void (*fileSearchProc)(Widget w, XtPointer cbStruct);
    char *defaultDir;
    WParamPtr next;
} WidgetFsbDesc, *WidgetFsbDescPtr;

typedef struct _widgetCallback {
    WParamType type;		/* must be first */
    char *name;
    XtCallbackProc callback;
    XtPointer arg;
    WParamPtr next;
} WidgetCbDesc, *WidgetCbDescPtr;

#define WidgetCbMacro(cbname,cbfunc,cbarg,next)		\
    {							\
	WP_Callback,					\
	Concat3(XmN,cbname,Callback),			\
	cbfunc,						\
	(XtPointer) (cbarg),				\
	(WParamPtr) (next)				\
    }

/*
 * Typedef for fast widget label
 * ----------------------------------
 * This is a blind data type used to store information for very fast
 * label widget update
 */
typedef struct _fastLabelInfo FastLabelInfo, *FastLabelInfoPtr;

/*
 * Typedef for widget param
 * ----------------------------------
 * These are the core types for creating a "window item" that
 * can be manipulated, updated and handled by these functions.
 */
union _wiParam {
    WidgetAnyParam any;
    WidgetCbDesc cb;
    WidgetFsbDesc fsb;
}; /* typedef: WParam, *WParamPtr */

typedef struct _userWidgetDesc {
    char *parent;
    char *name;
    WidgetClassEnum wclass;
    WParamPtr param;
} UserWidgetDesc, *UserWidgetDescPtr;

CExternal Widget CreateWidgetTree(
    Widget treeParent,
    UserWidgetDescPtr,
    long nDesc,
    PixmapResourcePtr
);

CExternal Widget FindTopLevel(	/* guaranteed to return non-null */
    Widget w
);

CExternal Widget FindShell(	/* guaranteed to return non-null */
    Widget w
);

CExternal Widget FindAppShell(	/* guaranteed to return non-null */
    Widget w
);

CExternal Widget FindWidget(
    Widget w,
    char *name
);

CExternal void StdShellPopup(
    Widget w,
    char *widgetName
);

CExternal void StdCancelCB(
    Widget w,
    XtPointer userArg,
    XtPointer widgetArg
);

CExternal void ProcessEvents(
    Widget
);

CExternal Boolean CheckAppEvents(
    Widget
);

CExternal void UnmapWidgetList(
    Widget w,			/* current widget (used for searching) */
    char **wNames		/* null-terminated list of widget names */
);

CExternal void MapWidgetList(
    Widget w,			/* current widget (used for searching) */
    char **wNames		/* null-terminated list of widget names */
);

CExternal Boolean GetResourceState(
    Widget w,			/* current widget (used for searching) */
    char *wName,		/* target widget */
    char *rscName		/* boolean resource name */
);

CExternal void SetResourceState(
    Widget w,			/* current widget (used for searching) */
    char *wName,		/* target widget */
    char *rscName,		/* boolean resource name */
    Boolean rscValue		/* desired state */
);

CExternal void SetResource(
    Widget w,			/* current widget (used for searching) */
    char *wName,		/* target widget */
    char *rscName,		/* resource name */
    XtPointer rscValue		/* resource value */
);

CExternal void SetResourceList(
    Widget w,			/* current widget (used for searching) */
    char **wNames,		/* null-terminated list of widget names */
    char *rscName,		/* resource name */
    XtPointer rscValue		/* resource value */
);

CExternal void SetToggleState(
    Widget w,
    char *wName,
    Boolean state
);

CExternal void SetWidgetSensitivity(
    Widget w,
    char *wName,
    Boolean sensitive
);

CExternal void SetWidgetRscString(
    Widget w,
    char *resource,
    char *string
);

CExternal void SetWidgetLabel(
    Widget w,
    char *label
);

CExternal void SetNumberedWidgetLabel(
    Widget w,
    long number
);

CExternal ULong StringToMagic(
    char *
);

CExternal void StoreUserData(
    Widget w,
    XtPointer data
);

CExternal XtPointer GetUserData(
    Widget w,
    char *magicName
);

CExternal void SetDefaultButton(
    Widget button
);

CExternal FastLabelInfoPtr AllocFastLabel(
    Widget label
);

CExternal void UpdateFastLabel(
    FastLabelInfoPtr flip,
    char *label
);

CExternal void FreeFastLabel(
    FastLabelInfoPtr flip
);

#endif /* _WIDGETUTIL_H_ */
