/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *      deckTapeCtl.c - configuration panel for decks
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/deckControl.c,v 1.7 1995/08/08 14:36:09 toddb Exp $
 */
#include "fileAccess.h"
#include "nvrString.h"
#include "nvrAlloc.h"
#include "osio.h"
#include "WidgetUtil.h"
#include "WidgetFile.h"
#include "WidgetDialog.h"
#include "deckPlayerI.h"

#include <Xm/XmStrDefs.h>
#include <Xm/List.h>

typedef enum {
    kSerialConfig,
    kStripeTape,
    kEject,
    kValidate,
    kCloseWindow,
    kNewSegment,
    kDeleteSegment,
    kSaveCollection,
    kSaveCollectionAs,
    kLoadCollection
} ControlButtons;

typedef enum {
    kBrowseCallback
} ListCallbacks;

static void   ButtonCB(Widget w, XtPointer appArg, XtPointer buttonData);
static void     ListCB(Widget w, XtPointer appArg, XtPointer listData);

static WidgetCbDesc
  SerialCb   = WidgetCbMacro(activate, ButtonCB, kSerialConfig,     NULL),
  EjectCb    = WidgetCbMacro(activate, ButtonCB, kEject,            NULL),
  ValidateCb = WidgetCbMacro(activate, ButtonCB, kValidate,         NULL),
  StripeCb   = WidgetCbMacro(activate, ButtonCB, kStripeTape,       NULL),
  CloseCb    = WidgetCbMacro(activate, ButtonCB, kCloseWindow,      NULL),
  NewCb      = WidgetCbMacro(activate, ButtonCB, kNewSegment,       NULL),
  DeleteCb   = WidgetCbMacro(activate, ButtonCB, kDeleteSegment,    NULL),
  SaveCb     = WidgetCbMacro(activate, ButtonCB, kSaveCollection,   NULL),
  SaveAsCb   = WidgetCbMacro(activate, ButtonCB, kSaveCollectionAs, NULL),
  LoadCb     = WidgetCbMacro(activate, ButtonCB, kLoadCollection,   NULL),
  BrowseCb   = WidgetCbMacro(browseSelection,ListCB,kBrowseCallback,NULL);

static UserWidgetDesc ControlList[] = {
#ifdef thisWorksToo
    { NULL,          "panel",           WC_DialogShell },
    { NULL,          "deckPanel",       WC_Form },
#endif
#ifdef thisWorksToo
    { NULL,          "deckPanel",       WC_FormDialog },
#endif
    { NULL,          "panel",           WC_DialogShell },
    { NULL,          "bb",              WC_BulletinBoard },
    { "*bb",         "deckPanel",       WC_Form },

    { "*deckPanel",  "serialButton",    WC_PushButton, (WParamPtr)&SerialCb },
    { "*deckPanel",  "serialName",      WC_Label  },
    { "*deckPanel",  "serialLabel",     WC_Label  },
    { "*deckPanel",  "stripeButton",    WC_PushButton, (WParamPtr)&StripeCb },
    { "*deckPanel",  "mediaName",       WC_Label  },
    { "*deckPanel",  "mediaLabel",      WC_Label  },
    { "*deckPanel",  "validateButton",  WC_PushButton, (WParamPtr)&ValidateCb },
    { "*deckPanel",  "ejectButton",     WC_PushButton, (WParamPtr)&EjectCb },

    { "*deckPanel",  "box1",            WC_Frame },
    { "*box1",       "collection",      WC_Form },
    { "*collection", "collectionName",  WC_Text  },
    { "*collection", "collectionLabel", WC_Label  },
    { "*collection", "list",            WC_ScrolledList,(WParamPtr)&BrowseCb },
    { "*collection", "saveButton",      WC_PushButton, (WParamPtr)&SaveCb },
    { "*collection", "saveAsButton",    WC_PushButton, (WParamPtr)&SaveAsCb },
    { "*collection", "loadButton",      WC_PushButton, (WParamPtr)&LoadCb },
    { "*collection", "newButton",       WC_PushButton, (WParamPtr)&NewCb },
    { "*collection", "deleteButton",    WC_PushButton, (WParamPtr)&DeleteCb },
    { "*collection", "currentName",     WC_Text  },
    { "*collection", "currentLabel",    WC_Label  },

    { "*deckPanel",   "closeFrame",     WC_Frame },
    { "*closeFrame",  "closeButton",    WC_PushButton, (WParamPtr)&CloseCb },
};

static char *NmDeckBB         = "*bb";
static char *NmDeckForm       = "*deckPanel";
static char *NmSerialName     = "*deckPanel*serialName";
static char *NmMediaName      = "*deckPanel*mediaName";
static char *NmCollectionName = "*deckPanel*collectionName";
static char *NmSegmentList    = "*deckPanel*list";
static char *NmCurrentName    = "*deckPanel*currentName";
static char *NmSerialButton   = "*deckPanel*serialButton";
static char *NmSaveButton     = "*deckPanel*saveButton";
static char *NmSaveAsButton   = "*deckPanel*saveAsButton";
static char *NmLoadButton     = "*deckPanel*loadButton";
static char *NmNewButton      = "*deckPanel*newButton";
static char *NmDeleteButton   = "*deckPanel*deleteButton";
static char *NmCloseButton    = "*deckPanel*closeButton";
static char *NmStripeButton   = "*deckPanel*stripeButton";
static char *NmValidateButton = "*deckPanel*validateButton";
static char *NmEjectButton    = "*deckPanel*ejectButton";

/*
 * Since collections reside in the file system, the collection number
 * must be global to the application.
 */
static int CollectionNumber = 1;

/****************************************************************************
 *
 * private interface
 *
 ****************************************************************************/

static void
UpdatePanelItems(Widget w, PlayerClass player)
{
    DeckPlayerInfoPtr deckUI = (DeckPlayerInfoPtr)player->priv;
    DeckCntlPtr deck = deckUI->deck;

    SetWidgetSensitivity(w, NmStripeButton,
			 (deck->ability & kDeckCanStripe) != 0);
    SetWidgetSensitivity(w, NmEjectButton,
			 (deck->ability & kDeckCanEject) != 0);
    SetWidgetSensitivity(w, NmValidateButton,
			 (deck->ability & kDeckCanValidate) != 0
			 && ! deck->running);

    if (deckUI->validating && ! deck->running) {
        CloseWorkingDialog(w);
	deckUI->validating = False;
    }
}

static void
InitPanelItems(Widget w, PlayerClass player)
{
    DeckPlayerInfoPtr deckUI = (DeckPlayerInfoPtr)player->priv;
    DeckCntlPtr deck = deckUI->deck;

    SetWidgetLabel(FindWidget(w, NmSerialName), deck->deckName);
    SetWidgetLabel(FindWidget(w, NmMediaName), deck->mediaName);

    /*
     * For now, most of the buttons are disabled
     */
    SetWidgetSensitivity(w, NmSaveButton, False);
    SetWidgetSensitivity(w, NmSaveAsButton, False);
    SetWidgetSensitivity(w, NmLoadButton, False);
    SetWidgetSensitivity(w, NmNewButton, False);
    SetWidgetSensitivity(w, NmDeleteButton, False);
    SetWidgetSensitivity(w, NmSerialButton, False);

    SetDefaultButton( FindWidget(w, NmCloseButton) );
    XtInstallAccelerators( FindWidget(w, NmSerialName),
			   FindWidget(w, NmCloseButton) );

    UpdatePanelItems(w, player);
}

static void
DeckChanged(PlayerClass player, ULong changed)
{
    DeckPlayerInfoPtr deckUI = (DeckPlayerInfoPtr)player->priv;

    UpdatePanelItems(deckUI->control, player);
}

static void
NewSegmentItem(DeckPlayerInfoPtr deckUI)
{
    XmString string;
    char segName[ MaxPathLength ];
    Widget listw;
    Widget textw;

    if (NULL == (textw = FindWidget(deckUI->control, NmCurrentName)))
	return;
    if (NULL == (listw = FindWidget(deckUI->control, NmSegmentList)))
	return;

    sprintf(segName, "segment %d", ++deckUI->segmentNum);
    string = XmStringCreateSimple(segName);
    if (string == NULL)
        return;

    XmListAddItem(listw, string, 0);	/* adds at the end of the list */
    XtVaSetValues(textw, XmNvalue, segName, NULL);
    XmStringFree(string);
}

static void
ListCB(Widget w, XtPointer appArg, XtPointer listData)
{
    DeckPlayerInfoPtr deckUI;
    PlayerClass player;
    XmListCallbackStruct *listCb = (XmListCallbackStruct *)listData;
    Widget textw;
    char *selection;

    player = (PlayerClass) GetUserData(FindWidget(w, NmDeckForm),
				       DeckMagicName);
    if (player == NULL)
	return;
    deckUI = (DeckPlayerInfoPtr)player->priv;

    if (NULL == (textw = FindWidget(deckUI->control, NmCurrentName)))
	return;
    if (listCb->selected_item_count != 1) {
	InternalProblemDialog(w, NULL, "list cb yields %d items, expected 1.",
	    listCb->selected_item_count);
	if (listCb->selected_item_count < 1)
	    return;
    }
    XmStringGetLtoR(listCb->item, XmFONTLIST_DEFAULT_TAG, &selection);
    if (selection) {
	XtVaSetValues(textw, XmNvalue, selection, NULL);
	XtFree(selection);
    }
}

static void
ButtonCB(Widget w, XtPointer appArg, XtPointer buttonData)
{
    DeckPlayerInfoPtr deckUI;
    DeckCntlPtr deck;
    PlayerClass player;
    InternalReasonPtr irp;

    player = (PlayerClass) GetUserData(FindWidget(w, NmDeckForm),
				       DeckMagicName);
    if (player == NULL)
	return;
    deckUI = (DeckPlayerInfoPtr)player->priv;
    deck = deckUI->deck;

    switch((ControlButtons)appArg) {
    case kSerialConfig:
	Msg("serial config\n");
	break;

    case kStripeTape:
	PopupDeckStriper(w, deckUI);
	break;

    case kEject:
        if (NULL != (irp = (*deck->eject)(deck))) {
            ReasonDialog(w, NULL, irp);
            break;
        }
	break;

    case kValidate:
        if (NULL != (irp = (*deck->validate)(deck))) {
            ReasonDialog(w, NULL, irp);
            break;
        }
	deckUI->validating = True;
	WorkingDialog(w, NULL,
	    "Validating %s tape striping...", deck->prodName);

	break;

    case kCloseWindow:
	XtPopdown(XtParent(deckUI->control));
	break;

    case kSaveCollection:
	Msg("save collection\n");
	break;

    case kSaveCollectionAs:
	Msg("save as...\n");
	break;

    case kNewSegment:
	Msg("new seg\n");
	NewSegmentItem(deckUI);
	break;

    case kDeleteSegment:
	Msg("delete seg\n");
	break;

    case kLoadCollection:
	Msg("load seg\n");
	break;
    }
}

/****************************************************************************
 *
 * public interface
 *
 ****************************************************************************/
void
PopupDeckControl(PlayerClass player)
{
    DeckPlayerInfoPtr deckUI = (DeckPlayerInfoPtr)player->priv;
    DeckCntlPtr deck = deckUI->deck;
    Widget dialog;
    Widget ctl = deckUI->control;
    Widget collection;
    Widget appShell = FindTopLevel(player->wDpy);
    char winName[ 128 ];

    if (ctl == NULL) {
	dialog = CreateWidgetTree(appShell, ControlList,
			       NumberOfCells(ControlList), NULL);
	if (dialog == NULL) {
	    WarningDialog(appShell, NULL, "Can't create deck control panel.");
	    return;
	}
	if (NULL == (ctl = FindWidget(dialog, NmDeckBB)))
	    return;

	sprintf(winName, "%s Configuration", deck->prodName);
	XtVaSetValues(dialog, XmNtitle, winName, NULL);
	if (NULL != (collection = FindWidget(dialog, NmCollectionName)))
	    XtVaSetValues(collection, XmNvalue, "untitled", NULL);
	deckUI->control = ctl;
	NewSegmentItem(deckUI);

	StoreUserData(FindWidget(ctl, NmDeckForm), player);
	deckUI->controlNotify = DeckChanged;
    } else {
	dialog = XtParent(ctl);
    }
    InitPanelItems(ctl, player);
    XtManageChild(ctl);
    XtPopup(dialog, XtGrabNone);
}
