/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *      deckSelectConf.c - selection configuration panel for decks
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/deckSelect.c,v 1.2 1995/06/12 17:36:49 toddb Exp $
 */
#include "fileAccess.h"
#include "nvrString.h"
#include "nvrAlloc.h"
#include "osio.h"
#include "WidgetUtil.h"
#include "WidgetFile.h"
#include "WidgetDialog.h"
#include "deckPlayerI.h"

#include <Xm/XmStrDefs.h>
#include <Xm/List.h>

typedef enum {
    kNoButton,
    kConnect,
    kDisconnect,
    kCancel
} ControlButton;

typedef enum {
    kDevList,
    kSerialList,
    kBaudList
} ListCallbacks;

static void   ButtonCB(Widget w, XtPointer appArg, XtPointer buttonData);
static void     ListCB(Widget w, XtPointer appArg, XtPointer listData);

static WidgetCbDesc
  ConnectCb    = WidgetCbMacro(activate,       ButtonCB, kConnect,    NULL),
  DisconnectCb = WidgetCbMacro(activate,       ButtonCB, kDisconnect, NULL),
  CancelCb     = WidgetCbMacro(activate,       ButtonCB, kCancel,     NULL),
  DevListCb    = WidgetCbMacro(browseSelection,ListCB,   kDevList,    NULL),
  SerialListCb = WidgetCbMacro(browseSelection,ListCB,   kSerialList, NULL),
  BaudListCb   = WidgetCbMacro(browseSelection,ListCB,   kBaudList,   NULL);

static UserWidgetDesc SerialConfList[] = {
    { NULL,          "deckSelect",  WC_DialogShell },
    { NULL,          "bb",          WC_BulletinBoard },
    { "*bb",         "form",        WC_Form },

    { "*form",       "deviceLabel", WC_Label  },
    { "*form",       "deviceList",  WC_ScrolledList, (WParamPtr)&DevListCb },
    { "*form",       "serialLabel", WC_Label  },
    { "*form",       "serialList",  WC_ScrolledList, (WParamPtr)&SerialListCb },
    { "*form",       "baudLabel",   WC_Label  },
    { "*form",       "baudList",    WC_ScrolledList, (WParamPtr)&BaudListCb },
    { "*form",       "buttons",     WC_Form },
    { "*buttons",    "connectFrame",WC_Frame },
    { "*connectFrame","connect",    WC_PushButton,   (WParamPtr)&ConnectCb },
    { "*buttons",    "disconnect",  WC_PushButton,   (WParamPtr)&DisconnectCb },
    { "*buttons",    "cancel",      WC_PushButton,   (WParamPtr)&CancelCb  },
};

static char *NmSerialForm  = "*form";
static char *NmSerialBB    = "*bb";
static char *NmDeviceList  = "*deckSelect*deviceList";
static char *NmSerialList  = "*deckSelect*serialList";
static char *NmSerialLabel = "*deckSelect*serialLabel";
static char *NmBaudList    = "*deckSelect*baudList";
static char *NmBaudLabel   = "*deckSelect*baudLabel";
static char *NmConnect     = "*deckSelect*connect";
static char *NmDisconnect  = "*deckSelect*disconnect";

static char *SelectConfMagicName = "deck select config";
static DeckConfig PrivateDeckConf;
static char PrivateDeckPath[ 64 ];

typedef struct _selectInfo {
    ULong magic;        /* used when storing structure in widget user data */

    ControlButton click;
    Widget dialog;
    DeckChoicePtr choices;
    DeckChoicePtr choice;
    int serialChoice;
    int baudChoice;
    InternalReasonPtr *pReason;
    DeckConfigPtr deckConf;
} SelectInfo, *SelectInfoPtr;

static void
ButtonCB(Widget w, XtPointer appArg, XtPointer buttonData)
{
    SelectInfoPtr select;

    select = (SelectInfoPtr) GetUserData(FindWidget(w, NmSerialForm),
                                         SelectConfMagicName);
    if (select == NULL)
	return;

    select->click = (ControlButton)appArg;
}

static void
AddListItem(Widget listw, char *item)
{
    XmString string;

    string = XmStringCreateSimple(item);
    XmListAddItem(listw, string, 0);
    XmStringFree(string);
}

static void
InitSerialLines(SelectInfoPtr select)
{
    int i;
    Widget listw, w;
    ULong bauds;
    DeckChoicePtr dc = select->choice;

    if (dc->desc->ability & kDeckMediaIsFile) {
	if (NULL != (w = FindWidget(select->dialog, NmSerialList)))
	    XtUnmanageChild(w);
	if (NULL != (w = FindWidget(select->dialog, NmSerialLabel)))
	    XtUnmanageChild(w);
	if (NULL != (w = FindWidget(select->dialog, NmBaudList)))
	    XtUnmanageChild(w);
	if (NULL != (w = FindWidget(select->dialog, NmBaudLabel)))
	    XtUnmanageChild(w);
	return;
    }

    if (NULL != (listw = FindWidget(select->dialog, NmSerialList))) {
	XmListDeleteAllItems(listw);
	for (i = 0; i < dc->nNames; i++)
	    AddListItem(listw, dc->deckNames[ i ]);
	XmListSelectPos(listw, 1, False);
    }

    if (NULL != (listw = FindWidget(select->dialog, NmBaudList))) {
	bauds = dc->desc->baudAvailable;
	XmListDeleteAllItems(listw);
	if (bauds == 0)
	    AddListItem(listw, "no baud rates!");
	if (bauds & kBaudAvail2400)
	    AddListItem(listw, "2400");
	if (bauds & kBaudAvail4800)
	    AddListItem(listw, "4800");
	if (bauds & kBaudAvail9600)
	    AddListItem(listw, "9600");
	if (bauds & kBaudAvail19200)
	    AddListItem(listw, "19200");
	if (bauds & kBaudAvail38400)
	    AddListItem(listw, "38400");
	XmListSelectPos(listw, 1, False);
	select->baudChoice = 1;
    }
}

static void
ListCB(Widget w, XtPointer appArg, XtPointer listData)
{
    XmListCallbackStruct *listCb = (XmListCallbackStruct *)listData;
    SelectInfoPtr select;
    DeckChoicePtr dc;
    int i;

    select = (SelectInfoPtr) GetUserData(FindWidget(w, NmSerialForm),
                                         SelectConfMagicName);
    if (select == NULL)
        return;
    switch((ListCallbacks)appArg) {
    case kDevList:
	dc = select->choices;
	for (i = 1; dc->next && i < listCb->item_position; i++)
	    dc = dc->next;
	if (dc != select->choice) {
	    select->choice = dc;
	    InitSerialLines(select);
	}
        break;

    case kSerialList:
	select->serialChoice = listCb->item_position;
	break;

    case kBaudList:
	select->baudChoice = listCb->item_position;
	break;
    }
}

static void
InitSerialDialog(Widget ctl, SelectInfoPtr select)
{
    DeckChoicePtr dc;
    Widget listw, button;

    select->choice = select->choices;
    InitSerialLines(select);
    if (NULL != (listw = FindWidget(ctl, NmDeviceList))) {
	for (dc = select->choices; dc; dc = dc->next)
	    AddListItem(listw, dc->desc->prodName);
	XmListSelectPos(listw, 1, False);
	select->serialChoice = 1;
    }

    /*
     * We're not ready to support this.
     */
    if (NULL != (button = FindWidget(ctl, NmDisconnect)))
	XtSetSensitive(button, False);

    SetDefaultButton( FindWidget(ctl, NmConnect) );
}

static long
GetBaudChoice(SelectInfoPtr select)
{
    Widget listw;
    XmStringTable items = NULL;
    char *baudString;

    if (NULL == (listw = FindWidget(select->dialog, NmBaudList)))
	return(0);
    XtVaGetValues(listw, XmNitems, &items, NULL);
    if (items == NULL)
	return(0);
    XmStringGetLtoR(items[ select->baudChoice-1 ],
		    XmFONTLIST_DEFAULT_TAG, &baudString);
    return(atoi(baudString));
}

static void
ResolveDeckChoice(SelectInfoPtr select)
{
    DeckChoicePtr choice = select->choice;
    int allocSize = sizeof(DeckConfig);

    if (*select->pReason || choice == NULL || select->click == kCancel) {
	select->deckConf = NULL;
    } else {
	select->deckConf = &PrivateDeckConf;
	memset((char *)select->deckConf, 0, allocSize);
	if ((choice->desc->ability & kDeckMediaIsFile) == 0) {
	    select->deckConf->deckName = PrivateDeckPath;
	    strcpy(select->deckConf->deckName,
		   choice->deckNames[ select->serialChoice-1 ]);
	    select->deckConf->baudrate = GetBaudChoice(select);
	}
	select->deckConf->desc = choice->desc;
    }
    
    FreeDeckChoices(select->choices);
    XtPopdown(select->dialog);
    XtDestroyWidget(select->dialog);
}

/****************************************************************************
 *
 * public interface
 *
 ****************************************************************************/
DeckConfigPtr
PopupDeckSelectConf(Widget w,
		    DeckMedium medium,
		    ULong ability,
		    InternalReasonPtr *pReason)
{
    Widget form, bb;
    Widget appShell = FindTopLevel(w);
    SelectInfo selectData;

    selectData.choices = FindDeckChoices(medium, ability, pReason);
    if (selectData.choices == NULL)
        return(NULL);
    selectData.magic = StringToMagic(SelectConfMagicName);
    selectData.click = kNoButton;
    selectData.pReason = pReason;

    selectData.dialog = CreateWidgetTree(appShell, SerialConfList,
			      NumberOfCells(SerialConfList), NULL);
    if (selectData.dialog == NULL) {
	*pReason =
	    InternalProblem("Can't create deck select config panel.");
	goto complete;
    }
    if (NULL == (form = FindWidget(selectData.dialog, NmSerialForm))
     || NULL == (bb = FindWidget(selectData.dialog, NmSerialBB))) {
	*pReason = InternalProblem("Serial config panel missing widgets.");
	goto complete;
    }
    XtVaSetValues(bb,
      XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
      NULL);

    StoreUserData(form, &selectData);
    InitSerialDialog(form, &selectData);

    XtManageChild(bb);
    XtPopup(selectData.dialog, XtGrabExclusive);

    /*
     * This dialog is synchronous!
     */
    while (selectData.click == kNoButton)
	ProcessEvents(form);

complete:
    ResolveDeckChoice(&selectData);
    return(selectData.deckConf);
}

