/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *      deckStripe.c - dialog for striping tape decks
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/deckStripe.c,v 1.6 1995/08/08 14:38:05 toddb Exp $
 */
#include "fileAccess.h"
#include "nvrString.h"
#include "nvrAlloc.h"
#include "osio.h"
#include "WidgetUtil.h"
#include "WidgetFile.h"
#include "WidgetDialog.h"
#include "deckPlayerI.h"

#include <stdlib.h>
#include <Xm/XmStrDefs.h>
#include <Xm/List.h>

typedef enum {
    kArrowDone  = 0,
    kButtonDown = 0x800,
    kTapeUp     = kButtonDown + 1,
    kTapeDn     = kButtonDown + 2,
    kStartUp    = kButtonDown + 3,
    kStartDn    = kButtonDown + 4,
    kEndUp      = kButtonDown + 5,
    kEndDn      = kButtonDown + 6,
    kAction     = 7,
    kCancel     = 8
} ControlButton;

static void   ButtonCB(Widget w, XtPointer appArg, XtPointer buttonData);
static void   ActionCB(Widget w, XtPointer appArg, XtPointer buttonData);

static WidgetCbDesc
  ArrowDoneCb  = WidgetCbMacro(disarm,  ButtonCB,  kArrowDone,  NULL),
  TapeUpCb     = WidgetCbMacro(arm,     ButtonCB,  kTapeUp,     &ArrowDoneCb),
  TapeDnCb     = WidgetCbMacro(arm,     ButtonCB,  kTapeDn,     &ArrowDoneCb),
  StartUpCb    = WidgetCbMacro(arm,     ButtonCB,  kStartUp,    &ArrowDoneCb),
  StartDnCb    = WidgetCbMacro(arm,     ButtonCB,  kStartDn,    &ArrowDoneCb),
  EndUpCb      = WidgetCbMacro(arm,     ButtonCB,  kEndUp,      &ArrowDoneCb),
  EndDnCb      = WidgetCbMacro(arm,     ButtonCB,  kEndDn,      &ArrowDoneCb),
  ActionCb     = WidgetCbMacro(activate,ActionCB,  kAction,     NULL),
  CancelCb     = WidgetCbMacro(activate,ActionCB,  kCancel,     NULL);

static UserWidgetDesc StripeList[] = {
    { NULL,          "deckStripe",       WC_FormDialog },


    { "*deckStripe",  "tapeIdUp",     WC_ArrowButton, (WParamPtr)&TapeUpCb },
    { "*deckStripe",  "tapeIdDn",     WC_ArrowButton, (WParamPtr)&TapeDnCb },
    { "*deckStripe",  "tapeId",       WC_Label },
    { "*deckStripe",  "tapeIdLabel",  WC_Label },

    { "*deckStripe",  "startLabel",   WC_Label },
    { "*deckStripe",  "start",        WC_Label },
    { "*deckStripe",  "startUp",      WC_ArrowButton, (WParamPtr)&StartUpCb },
    { "*deckStripe",  "startDn",      WC_ArrowButton, (WParamPtr)&StartDnCb },

    { "*deckStripe",  "endLabel",     WC_Label },
    { "*deckStripe",  "end",          WC_Label },
    { "*deckStripe",  "endUp",        WC_ArrowButton, (WParamPtr)&EndUpCb },
    { "*deckStripe",  "endDn",        WC_ArrowButton, (WParamPtr)&EndDnCb },

    { "*deckStripe",  "counterLabel", WC_Label },
    { "*deckStripe",  "counter",      WC_Label },

    { "*deckStripe",  "dubToggle",    WC_Toggle },
    { "*deckStripe",  "separator",    WC_Separator },

    { "*deckStripe",  "buttonForm",   WC_Form },
    { "*buttonForm",  "actionFrame",  WC_Frame },
    { "*actionFrame", "actionButton", WC_PushButton, (WParamPtr)&ActionCb },
    { "*buttonForm",  "cancelButton", WC_PushButton, (WParamPtr)&CancelCb },
};

static char *NmStripeForm     = "deckStripe";
static char *NmTapeId         = "deckStripe*tapeId";
static char *NmTapeIdUp       = "deckStripe*tapeIdUp";
static char *NmTapeIdDn       = "deckStripe*tapeIdDn";
static char *NmStart          = "deckStripe*start";
static char *NmStartUp        = "deckStripe*startUp";
static char *NmStartDn        = "deckStripe*startDn";
static char *NmEnd            = "deckStripe*end";
static char *NmEndUp          = "deckStripe*endUp";
static char *NmEndDn          = "deckStripe*endDn";
static char *NmCounter        = "deckStripe*counter";
static char *NmActionButton   = "deckStripe*actionButton";
static char *NmCancelButton   = "deckStripe*cancelButton";
static char *NmDubToggle      = "deckStripe*dubToggle";

/****************************************************************************
 *
 * private interface
 *
 ****************************************************************************/

static char *StripeMagicName = "deck striping dialog";

enum {
    StartingButtonInterval = MilliSecPerSecond,
    CounterInterval = MilliSecPerSecond / 10,
    MinimumInterval = MilliSecPerSecond / 30,
    /* MinimumStripeLength = 3 * 60 * 30 */
    MinimumStripeLength = 30 * 30
};

typedef struct _stripeInfo {
    ULong magic;		/* must be first */
    Widget dialog;		/* this dialog */
    PlaySegment seg;		/* segment to stripe */
    long tapeId;		/* end timecode */
    DeckPlayerInfoPtr deckUI;	/* pointer to the deck UI structure */
    XtAppContext appContext;	/* used to set timers */
    boolean dialogUp;		/* whether dialog is still up on the screen */
    boolean dub;		/* whether dialog is still up on the screen */
    long counter;		/* current tape counter */
    ControlButton button;	/* current, pressed button */
    XtIntervalId intervalId;
    long interval;		/* size of interval */
    long increment;		/* size of increment */
    long startValue;		/* start value when button pressed */

    FastLabelInfoPtr tapeIdFlip;
    FastLabelInfoPtr startFlip;
    FastLabelInfoPtr endFlip;
    FastLabelInfoPtr counterFlip;

    boolean running;		/* whether we think striping is going on */
} StripeInfo, *StripeInfoPtr;

static void
UpdateTapeId(StripeInfoPtr stripe, long delta)
{
    long newTapeId;
    char buf[ 32 ];

    newTapeId = stripe->tapeId + delta;
    if (newTapeId < 0)
	newTapeId = 0;
    if (newTapeId == stripe->tapeId)
	return;
    stripe->tapeId = newTapeId;
    sprintf(buf, "%d", newTapeId);

    UpdateFastLabel(stripe->tapeIdFlip, buf);
}

static void
UpdateEndTime(StripeInfoPtr stripe, long delta)
{
    DeckCntlPtr deck = stripe->deckUI->deck;
    long newEnd;
    DeckTimecode tc;
    char buf[ 128 ];

    newEnd = stripe->seg.outFrame + delta;
    if (newEnd <= deck->minFrame + MinimumStripeLength)
        newEnd = deck->minFrame + MinimumStripeLength;
    if (newEnd <= stripe->seg.inFrame + MinimumStripeLength)
        newEnd = stripe->seg.inFrame + MinimumStripeLength;
    if (newEnd > deck->maxFrame)
	newEnd = deck->maxFrame + 1;
    if (newEnd == stripe->seg.outFrame)
        return;

    stripe->seg.outFrame = newEnd;
    FrameToTimecode(stripe->seg.outFrame, deck->tcType, &tc);
    FormatTimecode(&tc, buf, sizeof(buf));
    if (newEnd > deck->maxFrame)
	strcat(buf, " (EOT)");
    UpdateFastLabel(stripe->endFlip, buf);
}

static void
UpdateStartTime(StripeInfoPtr stripe, long delta)
{
    DeckCntlPtr deck = stripe->deckUI->deck;
    long newStart, overlapPoint;
    DeckTimecode tc;
    char buf[ 128 ];

    newStart = stripe->seg.inFrame + delta;
    if (newStart < deck->minFrame) {
	if (delta > 0)
	    newStart = deck->minFrame;
	else
	    newStart = 0;		/* set it to BOT */
    }

    overlapPoint = stripe->seg.outFrame - MinimumStripeLength;
    if (newStart > overlapPoint) {
	UpdateEndTime(stripe, newStart - overlapPoint);
	newStart = stripe->seg.outFrame - MinimumStripeLength;
    }
    if (newStart == stripe->seg.inFrame)
	return;
    stripe->seg.inFrame = newStart;

    FrameToTimecode(stripe->seg.inFrame, deck->tcType, &tc);
    FormatTimecode(&tc, buf, sizeof(buf));
    if (stripe->seg.inFrame == 0)
	strcat(buf, " (BOT)");
    UpdateFastLabel(stripe->startFlip, buf);
}

static void
InitStripeDialog(StripeInfoPtr stripe)
{
    Widget button;
    DeckCntlPtr deck = stripe->deckUI->deck;
    Widget w = stripe->dialog;
    DeckTimecode tc;
    char buf[ 128 ];

    stripe->seg.outFrame = deck->maxFrame;
    UpdateEndTime(stripe, 1);

    stripe->seg.inFrame = deck->frame - 1;
    UpdateStartTime(stripe, 1);

    stripe->tapeId = deck->tapeId - 1;
    UpdateTapeId(stripe, 1);

    sprintf(buf, "%s Tape Striping", deck->prodName);
    XtVaSetValues(w, XmNtitle, buf, NULL);
    SetDefaultButton( FindWidget(w, NmActionButton) );

    stripe->counter = deck->frame;
    FrameToTimecode(stripe->counter, deck->tcType, &tc);
    FormatTimecode(&tc, buf, sizeof(buf));
    UpdateFastLabel(stripe->counterFlip, buf);

    if (NULL != (button = FindWidget(stripe->dialog, NmDubToggle))) {
	XtSetSensitive(button,
		       deck->mediaAvailable
		       && (deck->ability & kDeckCanDubTimecode) != 0);
	XtVaSetValues(button, XmNset, False, NULL);
    }
}

static void
StripeWindowEvent(Widget dpyWidget,
                  XtPointer closure,
                  XEvent *event,
                  Boolean *continueToDispatch)
{
    StripeInfoPtr stripe = (StripeInfoPtr) closure;

    switch (event->type) {
    case DestroyNotify:
	stripe->dialog = NULL;
	/* fall through... */

    case UnmapNotify:
	stripe->dialogUp = False;
	break;
    }
}

static void
TimedButtonUpdate(caddr_t data, XtIntervalId id)
{
    StripeInfoPtr stripe = (StripeInfoPtr)data;
    long newInterval, change;

    if ((stripe->button & kButtonDown) == 0) {
	stripe->intervalId = NULL;
	return;
    }
    if (id == 0) {
	switch(stripe->button) {
	case kTapeUp:
	case kTapeDn:
	    stripe->startValue = stripe->tapeId;
	    break;

	case kStartUp:
	case kStartDn:
	    stripe->startValue = stripe->seg.inFrame;
	    break;

	case kEndUp:
	case kEndDn:
	    stripe->startValue = stripe->seg.outFrame;
	    break;
	}
    }

    newInterval = stripe->interval >> 1;
    if (newInterval < MinimumInterval) {
	newInterval = MinimumInterval;
	switch(stripe->button) {
	case kTapeUp:
	case kTapeDn:
	    stripe->increment = 1;
	    break;

	case kStartUp:
	case kStartDn:
	    change = abs(stripe->seg.inFrame - stripe->startValue);
	    goto incrementFrame;

	case kEndUp:
	case kEndDn:
	    change = abs(stripe->seg.outFrame - stripe->startValue);

	incrementFrame:
	    if (change < 90)
		stripe->increment = 1;
	    else if (change < 2700)
		stripe->increment = 30 + 1;
	    else
		stripe->increment = 60*30 + 30 + 1;
	    break;
	}
    }
    stripe->interval = newInterval;

    switch(stripe->button) {
    case kTapeUp:  UpdateTapeId   (stripe, (int) stripe->increment); break;
    case kTapeDn:  UpdateTapeId   (stripe, (int)-stripe->increment); break;
    case kStartUp: UpdateStartTime(stripe, (int) stripe->increment); break;
    case kStartDn: UpdateStartTime(stripe, (int)-stripe->increment); break;
    case kEndUp:   UpdateEndTime  (stripe, (int) stripe->increment); break;
    case kEndDn:   UpdateEndTime  (stripe, (int)-stripe->increment); break;
    }

    stripe->intervalId =
	XtAppAddTimeOut(stripe->appContext,
			stripe->interval,
			(XtTimerCallbackProc) TimedButtonUpdate,
			(caddr_t) stripe);

}

static void
ButtonCB(Widget w, XtPointer appArg, XtPointer buttonData)
{
    StripeInfoPtr stripe;

    stripe = (StripeInfoPtr) GetUserData(FindWidget(w, NmStripeForm),
				         StripeMagicName);
    if (stripe == NULL)
	return;

    switch(stripe->button = (ControlButton)appArg) {
    case kArrowDone:
    case kTapeUp:
    case kTapeDn:
    case kStartUp:
    case kStartDn:
    case kEndUp:
    case kEndDn:
	break;
    }

    if (stripe->button & kButtonDown) {
	stripe->interval = StartingButtonInterval;
	stripe->increment = 1;
	TimedButtonUpdate((caddr_t)stripe, 0);
    } else if (stripe->intervalId) {
	XtRemoveTimeOut(stripe->intervalId);
	stripe->intervalId = NULL;
    }
}

static void
UpdateButtonState(StripeInfoPtr stripe)
{
    Widget w = stripe->dialog;

    SetWidgetSensitivity(w, NmTapeIdUp, ! stripe->running);
    SetWidgetSensitivity(w, NmTapeIdDn, ! stripe->running);
    SetWidgetSensitivity(w, NmEndUp, ! stripe->running);
    SetWidgetSensitivity(w, NmEndDn, ! stripe->running);
    SetWidgetSensitivity(w, NmStartUp, ! stripe->running);
    SetWidgetSensitivity(w, NmStartDn, ! stripe->running);
    SetWidgetSensitivity(w, NmActionButton, ! stripe->running);
    SetWidgetSensitivity(w, NmDubToggle, ! stripe->running);
}

static void
DeckChanged(PlayerClass player, ULong changed)
{
    DeckPlayerInfoPtr deckUI = (DeckPlayerInfoPtr)player->priv;
    DeckCntlPtr deck = deckUI->deck;
    StripeInfoPtr stripe;

    if ((changed & kPlayerRunningChanged) == 0)
	return;

    stripe = (StripeInfoPtr)
	GetUserData(FindWidget(deckUI->stripe, NmStripeForm), StripeMagicName);
    if (stripe == NULL)
	return;

    stripe->running = deck->running;
    UpdateButtonState(stripe);
}

static void
TimedStripeUpdate(caddr_t data, XtIntervalId id)
{
    StripeInfoPtr stripe = (StripeInfoPtr)data;
    DeckCntlPtr deck = stripe->deckUI->deck;
    long newCounter;
    DeckTimecode tc;
    char buf[ 128 ];

    newCounter = deck->frame;
    if (newCounter != stripe->counter) {
	stripe->counter = newCounter;
	FrameToTimecode(stripe->counter, deck->tcType, &tc);
	FormatTimecode(&tc, buf, sizeof(buf));
	UpdateFastLabel(stripe->counterFlip, buf);
    }

    if (stripe->running) {
	stripe->intervalId =
	    XtAppAddTimeOut(stripe->appContext,
			    CounterInterval,
			    (XtTimerCallbackProc) TimedStripeUpdate,
			    (caddr_t) stripe);
    } else {
	stripe->intervalId = NULL;
    }
}

static void
ActionCB(Widget w, XtPointer appArg, XtPointer buttonData)
{
    StripeInfoPtr stripe;
    InternalReasonPtr irp;
    DeckCntlPtr deck;

    stripe = (StripeInfoPtr) GetUserData(FindWidget(w, NmStripeForm),
				         StripeMagicName);
    if (stripe == NULL)
	return;
    deck = stripe->deckUI->deck;

    switch((ControlButton)appArg) {
    case kAction:
	stripe->dub = GetResourceState(w, NmDubToggle, XmNset);
	irp = (*deck->stripe)(deck, &stripe->seg, stripe->dub, stripe->tapeId);
	if (irp) {
	    ReasonDialog(w, NULL, irp);
	    break;
	}
	stripe->running = True;
	UpdateButtonState(stripe);
	TimedStripeUpdate((caddr_t)stripe, (XtIntervalId)0);
	break;

    case kCancel:
	if (stripe->running) {
	    stripe->running = False;
	    UpdateButtonState(stripe);
	    if (NULL != (irp = (*deck->pause)(deck)))
		ReasonDialog(w, NULL, irp);
	} else {
	    stripe->dialogUp = False;
	    XtPopdown(stripe->dialog);
	}
	break;
    }
}

/****************************************************************************
 *
 * public interface
 *
 ****************************************************************************/
void
PopupDeckStriper(Widget w, DeckPlayerInfoPtr deckUI)
{
    Widget form;
    StripeInfo stripe;

    memset((char *)&stripe, 0, sizeof(stripe));
    stripe.dialogUp = True;
    stripe.magic = StringToMagic(StripeMagicName);
    stripe.deckUI = deckUI;
    stripe.dub = (deckUI->deck->ability & kDeckCanDubTimecode);
    stripe.dialog = CreateWidgetTree(w, StripeList,
				     NumberOfCells(StripeList), NULL);
    if (stripe.dialog == NULL) {
        InternalProblemDialog(w, "Can't create tape striping panel.");
        return;
    }
    if (NULL == (form = FindWidget(stripe.dialog, NmStripeForm)))
        goto complete;
    XtVaSetValues(form,
	XmNdialogStyle, XmDIALOG_APPLICATION_MODAL,
	NULL);

    XtRealizeWidget(form);
    stripe.appContext = XtDisplayToApplicationContext(XtDisplay(w));
    stripe.tapeIdFlip = AllocFastLabel(FindWidget(stripe.dialog, NmTapeId));
    stripe.startFlip = AllocFastLabel(FindWidget(stripe.dialog, NmStart));
    stripe.endFlip = AllocFastLabel(FindWidget(stripe.dialog, NmEnd));
    stripe.counterFlip = AllocFastLabel(FindWidget(stripe.dialog, NmCounter));
    if (! stripe.tapeIdFlip || ! stripe.startFlip
     || ! stripe.endFlip    || ! stripe.counterFlip)
	goto complete;

    /*
     * This is if the window manager unmaps us
     */
    XtAddEventHandler(stripe.dialog,
                      StructureNotifyMask,
                      FALSE, StripeWindowEvent, (XtPointer)&stripe);

    StoreUserData(form, (XtPointer)&stripe);
    deckUI->stripeNotify = DeckChanged;
    deckUI->stripe = stripe.dialog;

    InitStripeDialog(&stripe);

    XtManageChild(form);
    XtPopup(stripe.dialog, XtGrabExclusive);

    /*
     * This dialog is synchronous!
     */
    while (stripe.dialogUp)
        ProcessEvents(form);

complete:
    deckUI->stripe = NULL;
    deckUI->stripeNotify = NULL;
    if (stripe.dialog)
	XtDestroyWidget(stripe.dialog);
    if (stripe.tapeIdFlip)
	FreeFastLabel(stripe.tapeIdFlip);
    if (stripe.startFlip)
	FreeFastLabel(stripe.startFlip);
    if (stripe.endFlip)
	FreeFastLabel(stripe.endFlip);
    if (stripe.counterFlip)
	FreeFastLabel(stripe.counterFlip);
    XtRemoveEventHandler(stripe.dialog,
			 StructureNotifyMask,
			 FALSE, StripeWindowEvent, (XtPointer)&stripe);
}

