/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1aLayer1.c - frame level decoding for mpeg audio
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1aLayer1.c,v 1.3 1995/11/02 20:53:07 toddb Exp $
 */

#include "mpeg1adecI.h"
#include "mpeg1aproto.h"

static InternalReason BadCrc =
    { "Bad CRC checksum\n" };

void
InitLayer1Tables(Mpeg1ADecPrivPtr adec)
{
    int nb;
    unsigned short sample;
    double fraction;
    FloatType *requantTable = adec->decode.requantTable;;

    for (nb = 2; nb < 16; nb++) {
        for (sample = 0; sample <= MaskOf(nb); sample++) {
            /* step 1 */
            if ((sample >> (nb-1)) & 1) {
                 fraction = 0.0;
            } else {
                fraction = -1.0;
            }

            /* step 2 */
            fraction +=
                (double) (sample & MaskOf(nb-1)) /
                              (double) (1L<<(nb-1));

            /* step 3 */
            fraction =
                 (double) (fraction+1.0/(double)(1L<<(nb-1))) *
                 (double) (1L<<nb) / (double) MaskOf(nb);


	    requantTable[ (1<<nb) - 4 + sample ] = fraction;
        }
    }
}

static UShort
I_CheckCRC(Mpeg1ADecPrivPtr adec, void *allocBits, boolean stereo)
{
    register int	sb;
    int			*alloc_bits = (int *)allocBits;
    int			bound;
    DeclareCrc(mask, crc);

    UpdateCrc(mask, 4, adec->parse.curHdr.b.bitrateIndex, crc);
    UpdateCrc(mask, 2, adec->parse.curHdr.b.freqIndex, crc);
    UpdateCrc(mask, 1, adec->parse.curHdr.b.paddingBit, crc);
    UpdateCrc(mask, 1, adec->parse.curHdr.b.privateBit, crc);
    UpdateCrc(mask, 2, adec->parse.curHdr.b.mode, crc);
    UpdateCrc(mask, 2, adec->parse.curHdr.b.modeExtension, crc);
    UpdateCrc(mask, 1, adec->parse.curHdr.b.copyright, crc);
    UpdateCrc(mask, 1, adec->parse.curHdr.b.original_home, crc);
    UpdateCrc(mask, 2, adec->parse.curHdr.b.emphasis, crc);
    
    switch(adec->parse.curHdr.b.mode) {
    case ModeSingleChannel:
    case ModeDualChannel:
    case ModeStereo:
	for (sb = 0; sb < (32 << (stereo == True)); sb++) {
	    UpdateCrc(mask, 4, alloc_bits[sb], crc);
	}
	break;
    case ModeJointStereo:
	bound = Mpeg1A_Bound[adec->parse.curHdr.b.modeExtension];
	for (sb = 0; sb < (bound << 1); sb++) {
	    UpdateCrc(mask, 4, alloc_bits[sb], crc);
	}
	for (sb = (bound << 1); sb < (32 << 1); sb += 2) {
	    UpdateCrc(mask, 4, alloc_bits[sb], crc);
	}
    }
    return(crc);
}


/*
 * for Layer1{Mono, Stereo, Intensity}
 *	pSrc points to the beginning of the frame data and
 *	and is guaranteed to contain a full frame of data 
 */
static InternalReasonPtr
Layer1Mono(Mpeg1ADecPrivPtr adec, UChar *pSrc)
{
    register int	sb, slot;
    unsigned int	nb;
    short		i_scale_factor[32];
    int 		alloc_bits[32];
    UShort		inputSample;
    FloatType		requantized_sample[32];
    FloatType		*requantTable = adec->decode.requantTable;
    DeclareBitParseVars(cache, nCacheBits, pBuf);

    InitBitParseMacro(&adec->parse.bitParse, pSrc);
    FillCacheMacro(&adec->parse.bitParse, cache, nCacheBits, pBuf);


    /*
     * DECODE ALLOCATION
     */
    for (sb = 0; sb < 32; sb++) {
	ReadBitsMacro(alloc_bits[sb], cache, nCacheBits, pBuf, 4); 
    }
    DebugTask(N_AllocationData,
	Msg("allocation:    ");
	for (sb = 0; sb < 32; sb++) {
	    if (alloc_bits[sb]) Msg("%2d", alloc_bits[sb]);
	    else                Msg(" .");
	}
	Msg("\n");
    )

    /*
     * DECODE SCALE FACTOR INDICES
     */
    for (sb = 0; sb < 32; sb++) {
	if (alloc_bits[sb]) {
	    ReadBitsMacro(i_scale_factor[sb], cache, nCacheBits, pBuf, 6);
	}
    }
    DebugTask(N_ScaleData,
	Msg("scale:         ");
	for (sb = 0; sb < 32; sb++) {
	    if (alloc_bits[sb]) Msg("%3d", i_scale_factor[sb]);
	    else                Msg("  .");
	}
	Msg("\n");
    )



    /*
     * CRC CHECK
     */
    if (adec->parse.curHdr.b.protectionBit == 0) {
	if ( I_CheckCRC(adec, alloc_bits, False) != adec->decode.crcCheck ) {
	    return(&BadCrc);
	}
    }

    /*
     * DECODE SAMPLE
     */
    for (slot = 0; slot < 12; slot++) {
	/*
	 * BUFFER AND REQUANTIZE SAMPLES
	 */
	for (sb = 0; sb < 32; sb++) {
	    nb = alloc_bits[sb];
	    if (nb) {
		nb++;
		ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
		requantized_sample[sb] =
			requantTable[ (1<<nb) - 4 + inputSample ]
			* Mpeg1A_ScaleFactor[i_scale_factor[sb]];
	    } else {
		requantized_sample[sb] = 0.0;
	    }
	}

	DebugTask(N_SampleData,
	    Msg("slt%3d coef:", slot);
	    for (sb = 0; sb < 32; sb += 2) {
		if (alloc_bits[sb]) Msg("%6.1f", requantized_sample[sb]);
		else                Msg("    . ");
	    }
	    Msg("\n               ");
	    for (sb = 1; sb < 32; sb += 2) {
		if (alloc_bits[sb]) Msg("%6.1f", requantized_sample[sb]);
		else                Msg("    . ");
	    }
	    Msg("\n");
	)

	/*
	 * SYNTHESIS SUBBAND FILTER
	 */
	(*adec->decode.filter.mono)(adec, requantized_sample,
	    (UShort *) &adec->decode.sampleBuf[adec->decode.sampleCount << 1]);
        adec->decode.sampleCount += adec->decode.samplesPerSlot;
    }
    return(NULL);
}

static InternalReasonPtr
Layer1Stereo(Mpeg1ADecPrivPtr adec, UChar *pSrc)
{
    register int	sb, slot;
    unsigned int	nb;
    short		i_scale_factor[32][2];
    int 		alloc_bits[32][2];
    UShort		inputSample;
    FloatType		requantized_sample[2][32];
    FloatType		*requantTable = adec->decode.requantTable;
    DeclareBitParseVars(cache, nCacheBits, pBuf);

    InitBitParseMacro(&adec->parse.bitParse, pSrc);
    FillCacheMacro(&adec->parse.bitParse, cache, nCacheBits, pBuf);


    /*
     * DECODE ALLOCATION
     */
    for (sb = 0; sb < 32; sb++) {
	ReadBitsMacro(alloc_bits[sb][0], cache, nCacheBits, pBuf, 4); 
	ReadBitsMacro(alloc_bits[sb][1], cache, nCacheBits, pBuf, 4); 
    }
    DebugTask(N_AllocationData,
        int ch;
	for (ch = 0; ch < 2; ch++) {
	    Msg("alloc ch%d:   ", ch);
	    for (sb = 0; sb < 32; sb++) {
		if (alloc_bits[sb][ch]) Msg("%2d", alloc_bits[sb][ch]);
		else                Msg(" .");
	    }
	}
	Msg("\n");
    )

    /*
     * DECODE SCALE FACTOR INDICES
     */
    for (sb = 0; sb < 32; sb++) {
	if (alloc_bits[sb][0]) {
	    ReadBitsMacro(i_scale_factor[sb][0], cache, nCacheBits, pBuf, 6);
	}
	if (alloc_bits[sb][1]) {
	    ReadBitsMacro(i_scale_factor[sb][1], cache, nCacheBits, pBuf, 6);
	}
    }
    DebugTask(N_ScaleData,
        int ch;
	for (ch = 0; ch < 2; ch++) {
            Msg("scale ch%d:   ", ch);
	    for (sb = 0; sb < 32; sb++) {
		if (i_scale_factor[sb][ch]) Msg("%3d", i_scale_factor[sb][ch]);
		else                Msg(" .");
	    }
	}
	Msg("\n");
    )



    /*
     * CRC CHECK
     */
    if (adec->parse.curHdr.b.protectionBit == 0) {
	if ( I_CheckCRC(adec, alloc_bits, True) != adec->decode.crcCheck ) {
	    return(&BadCrc);
	}
    }

    /*
     * DECODE SAMPLE
     */
    for (slot = 0; slot < 12; slot++) {
	/*
	 * BUFFER AND REQUANTIZE SAMPLES
	 */
	for (sb = 0; sb < 32; sb++) {
	    if (alloc_bits[sb][0]) {
		nb = alloc_bits[sb][0] + 1;
		ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
		requantized_sample[0][sb] =
			requantTable[ (1<<nb) - 4 + inputSample ]
			* Mpeg1A_ScaleFactor[i_scale_factor[sb][0]];
	    } else {
		requantized_sample[0][sb] = 0.0;
	    }								

	    if (alloc_bits[sb][1]) {
		nb = alloc_bits[sb][1] + 1;
		ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
		requantized_sample[1][sb] =
			requantTable[ (1<<nb) - 4 + inputSample ]
			* Mpeg1A_ScaleFactor[i_scale_factor[sb][1]];
	    } else {
		requantized_sample[1][sb] = 0.0;
	    }								
	}

	DebugTask(N_SampleData,
	    Msg("slt%3d coef:", slot);
	    int ch;
	    for (ch = 0; ch < 2; ch++) {
                Msg("sample ch%d:   ", ch);
		for (sb = 0; sb < 32; sb += 2) {
		    if (alloc_bits[sb][ch])
                        Msg("%6.1f", requantized_sample[ch][sb]);
		    else
                        Msg("    . ");
		}
		Msg("\n               ");
		for (sb = 1; sb < 32; sb += 2) {
		    if (alloc_bits[sb][ch])
                        Msg("%6.1f", requantized_sample[ch][sb]);
		    else
                        Msg("    . ");
		}
		Msg("\n");
	    }
	)

	/*
	 * SYNTHESIS SUBBAND FILTER
	 */
	(*adec->decode.filter.stereo)(adec, requantized_sample,
	    (UShort *) &adec->decode.sampleBuf[adec->decode.sampleCount << 1]);
        adec->decode.sampleCount += adec->decode.samplesPerSlot;
    }
    return(NULL);
}

static InternalReasonPtr
Layer1Intensity(Mpeg1ADecPrivPtr adec, UChar *pSrc)
{

    register int	sb, slot;
    int			bound;
    unsigned int	nb;
    short		i_scale_factor[32][2];
    int 		alloc_bits[32][2];
    UShort		inputSample;
    FloatType		requantized_sample[2][32];
    FloatType		*requantTable = adec->decode.requantTable;
    DeclareBitParseVars(cache, nCacheBits, pBuf);

    InitBitParseMacro(&adec->parse.bitParse, pSrc);
    FillCacheMacro(&adec->parse.bitParse, cache, nCacheBits, pBuf);


    /*
     * DECODE ALLOCATION
     */
    bound = Mpeg1A_Bound[adec->parse.curHdr.b.modeExtension];
    for (sb = 0; sb < bound; sb++) {
	ReadBitsMacro(alloc_bits[sb][0], cache, nCacheBits, pBuf, 4); 
	ReadBitsMacro(alloc_bits[sb][1], cache, nCacheBits, pBuf, 4); 
    }
    for (sb = bound; sb < 32; sb++) {
	ReadBitsMacro(alloc_bits[sb][0], cache, nCacheBits, pBuf, 4); 
	alloc_bits[sb][1] = alloc_bits[sb][0];
    }
    DebugTask(N_AllocationData,
        int ch;
	for (ch = 0; ch < 2; ch++) {
	    Msg("alloc ch%d bound%d:   ", ch, bound);
	    for (sb = 0; sb < 32; sb++) {
		if (alloc_bits[sb][ch]) Msg("%2d", alloc_bits[sb][ch]);
		else                Msg(" .");
	    }
	}
	Msg("\n");
    )

    /*
     * DECODE SCALE FACTOR INDICES
     */
    for (sb = 0; sb < bound; sb++) {
	if (alloc_bits[sb][0]) {
	    ReadBitsMacro(i_scale_factor[sb][0], cache, nCacheBits, pBuf, 6);
	}
	if (alloc_bits[sb][1]) {
	    ReadBitsMacro(i_scale_factor[sb][1], cache, nCacheBits, pBuf, 6);
	}
    }
    for (sb = bound; sb < 32; sb++) {
	if (alloc_bits[sb][0]) {
	    ReadBitsMacro(i_scale_factor[sb][0], cache, nCacheBits, pBuf, 6);
	    ReadBitsMacro(i_scale_factor[sb][1], cache, nCacheBits, pBuf, 6);
	}
    }
    DebugTask(N_ScaleData,
        int ch;
	for (ch = 0; ch < 2; ch++) {
            Msg("scale ch%d:   ", ch);
	    for (sb = 0; sb < 32; sb++) {
		if (i_scale_factor[sb][ch]) Msg("%3d", i_scale_factor[sb][ch]);
		else                Msg(" .");
	    }
	}
	Msg("\n");
    )



    /*
     * CRC CHECK
     */
    if (adec->parse.curHdr.b.protectionBit == 0) {
	if ( I_CheckCRC(adec, alloc_bits, True) != adec->decode.crcCheck ) {
	    return(&BadCrc);
	}
    }

    /*
     * DECODE SAMPLE
     */
    for (slot = 0; slot < 12; slot++) {
	/*
	 * BUFFER AND REQUANTIZE SAMPLES
	 */
	for (sb = 0; sb < bound; sb++) {
	    if (alloc_bits[sb][0]) {
		nb = alloc_bits[sb][0] + 1;
		ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
		requantized_sample[0][sb] =
			requantTable[ (1<<nb) - 4 + inputSample ]
			* Mpeg1A_ScaleFactor[i_scale_factor[sb][0]];
	    } else {
		requantized_sample[0][sb] = 0.0;
	    }								

	    if (alloc_bits[sb][1]) {
		nb = alloc_bits[sb][1] + 1;
		ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
		requantized_sample[1][sb] =
			requantTable[ (1<<nb) - 4 + inputSample ]
			* Mpeg1A_ScaleFactor[i_scale_factor[sb][1]];
	    } else {
		requantized_sample[1][sb] = 0.0;
	    }								
	}

	for (sb = bound; sb < 32; sb++) {
	    if (alloc_bits[sb][0]) {
		nb = alloc_bits[sb][0] + 1;
		ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
		requantized_sample[0][sb] =
			requantTable[ (1<<nb) - 4 + inputSample ]
			* Mpeg1A_ScaleFactor[i_scale_factor[sb][0]];
		requantized_sample[1][sb] =
			requantTable[ (1<<nb) - 4 + inputSample ]
			* Mpeg1A_ScaleFactor[i_scale_factor[sb][0]];
	    } else {
		requantized_sample[0][sb] = 0.0;
		requantized_sample[1][sb] = 0.0;
	    }								
	}

	DebugTask(N_SampleData,
	    Msg("slt%3d coef:", slot);
	    int ch;
	    for (ch = 0; ch < 2; ch++) {
                Msg("sample ch%d:   ", ch);
		for (sb = 0; sb < 32; sb += 2) {
		    if (alloc_bits[sb][ch])
                        Msg("%6.1f", requantized_sample[ch][sb]);
		    else
                        Msg("    . ");
		}
		Msg("\n               ");
		for (sb = 1; sb < 32; sb += 2) {
		    if (alloc_bits[sb][ch])
                        Msg("%6.1f", requantized_sample[ch][sb]);
		    else
                        Msg("    . ");
		}
		Msg("\n");
	    }
	)

	/*
	 * SYNTHESIS SUBBAND FILTER
	 */
	(*adec->decode.filter.stereo)(adec, requantized_sample,
	    (UShort *) &adec->decode.sampleBuf[adec->decode.sampleCount << 1]);
        adec->decode.sampleCount += adec->decode.samplesPerSlot;
    }
    return(NULL);
}

#ifdef CheckMpeg1ASyntax

Mpeg1ASynthMethodRec Layer1SyntaxMethodData = {
    Layer1Mono,
    Layer1Stereo,
    Layer1Intensity
};

#else

Mpeg1ASynthMethodRec Layer1MethodData = {
    Layer1Mono,
    Layer1Stereo,
    Layer1Intensity
};

#endif
