/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1aLayer1.c - frame level decoding for mpeg audio
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1aLayer2.c,v 1.3 1995/11/02 20:53:07 toddb Exp $
 */

#include "mpeg1adecI.h"

static InternalReason BadCrc =
    { "Bad CRC checksum\n" };
 
static void
PrepareSampleReading(int sb, int mode, int allocation, int chBitrate,
		     FloatType **pGroup, FloatType *pFactor)
{
    FloatType factor;
    FloatType *grouping;

    if (mode != ModeSingleChannel) {
	if( chBitrate == 4 ) {
	    chBitrate = 1;
	} else {
	    chBitrate -= 4;
	}
    }
    if (chBitrate == 1 || chBitrate == 2) {
	/* table 3-B.2c or 3-B.2d */
	grouping = Mpeg1A_table_cd_groupingTables[allocation];
	factor = Mpeg1A_table_cd_factor[allocation];
    } else {
	if (sb <= 2) {
	    grouping = Mpeg1A_table_ab1_groupingTables[allocation];
	    factor = Mpeg1A_table_ab1_factor[allocation];
	} else {
	    grouping = Mpeg1A_table_ab234_groupingTables[allocation];
	    if (sb <= 10) {
		factor = Mpeg1A_table_ab2_factor[allocation];
	    } else if (sb <= 22) {
		factor = Mpeg1A_table_ab3_factor[allocation];
	    } else {
		factor = Mpeg1A_table_ab4_factor[allocation];
	    }
	}
    }
    *pGroup = grouping;
    *pFactor = factor;
}

static UShort
II_CheckCRC(Mpeg1ADecPrivPtr adec, void *allocIndex, void *scfsi)
{
    register int	sb;
    int			sblimit;
    int			bound;
    UChar		*nbalTbl;
    int			*pIndex, *pIndex2;
    short		*pScfsi, *pScfsi2;
    DeclareCrc(mask, crc);

    UpdateCrc(mask, 4, adec->parse.curHdr.b.bitrateIndex, crc);
    UpdateCrc(mask, 2, adec->parse.curHdr.b.freqIndex, crc);
    UpdateCrc(mask, 1, adec->parse.curHdr.b.paddingBit, crc);
    UpdateCrc(mask, 1, adec->parse.curHdr.b.privateBit, crc);
    UpdateCrc(mask, 2, adec->parse.curHdr.b.mode, crc);
    UpdateCrc(mask, 2, adec->parse.curHdr.b.modeExtension, crc);
    UpdateCrc(mask, 1, adec->parse.curHdr.b.copyright, crc);
    UpdateCrc(mask, 1, adec->parse.curHdr.b.original_home, crc);
    UpdateCrc(mask, 2, adec->parse.curHdr.b.emphasis, crc);

    sblimit = adec->decode.sblimit;
    nbalTbl = adec->decode.nbalTable;
    switch(adec->parse.curHdr.b.mode) {
    case ModeSingleChannel:
	pIndex = (int *)allocIndex;
	pScfsi = (short *)scfsi;
	for (sb = 0; sb < sblimit; sb++ ) {
	    UpdateCrc(mask, nbalTbl[sb], pIndex[sb], crc);
	    if (pIndex[sb]) {
		UpdateCrc(mask, 2, pScfsi[sb], crc);
	    }
	}
	break;
    case ModeStereo:
    case ModeDualChannel:
	pIndex = (int *)allocIndex;
	pIndex2 = pIndex + 32;
	pScfsi = (short *)scfsi;
	pScfsi2 = pScfsi + 32;
	for (sb = 0; sb < sblimit; sb++) {
	    UpdateCrc(mask, nbalTbl[sb], pIndex[sb], crc);
	    UpdateCrc(mask, nbalTbl[sb], pIndex2[sb], crc);
	}
	for (sb = 0; sb < sblimit; sb++) {
	    if (pIndex[sb]) {
		UpdateCrc(mask, 2, pScfsi[sb], crc);
	    }
	    if (pIndex2[sb]) {
		UpdateCrc(mask, 2, pScfsi2[sb], crc);
	    }
	}
	break;
    case ModeJointStereo:
	pIndex = (int *)allocIndex;
	pIndex2 = pIndex + 32;
	pScfsi = (short *)scfsi;
	pScfsi2 = pScfsi + 32;
	bound = Mpeg1A_Bound[adec->parse.curHdr.b.modeExtension];
	for (sb = 0; sb < bound; sb++) {
	    UpdateCrc(mask, nbalTbl[sb], pIndex[sb], crc);
	    UpdateCrc(mask, nbalTbl[sb], pIndex2[sb], crc);
	}
	for (sb = bound; sb < sblimit; sb++) {
	    UpdateCrc(mask, nbalTbl[sb], pIndex[sb], crc);
	}
	for (sb = 0; sb < bound; sb++) {
	    if (pIndex[sb]) {
		UpdateCrc(mask, 2, pScfsi[sb], crc);
	    }
	    if (pIndex2[sb]) {
		UpdateCrc(mask, 2, pScfsi2[sb], crc);
	    }
	}
	for (sb = bound; sb < sblimit; sb++) {
	    if (pIndex[sb]) {
		UpdateCrc(mask, 2, pScfsi[sb], crc);
		UpdateCrc(mask, 2, pScfsi2[sb], crc);
	    }
	}
	break;
    }
    return(crc);
}

static InternalReasonPtr
Layer2Mono(Mpeg1ADecPrivPtr adec, UChar *pSrc)
{
    register int	sb, granule;
    int 		allocIndex[32];
    int			sblimit;
    int			iPart;
    UInt		nb;
    UInt		c;
    UInt		scaleFactor[32][3];
    short		scfsi[32];
    UShort		inputSample;
    UShort		*pSample;
    char		*allocTable;
    UChar		*nbalTable;
    FloatType		requantSample[3][32];
    FloatType		*groupingTable[32];
    FloatType		factorTable[32];
    Mpeg1A_Alloc	*pRec;
    DeclareBitParseVars(cache, nCacheBits, pBuf);

    InitBitParseMacro(&adec->parse.bitParse, pSrc);
    FillCacheMacro(&adec->parse.bitParse, cache, nCacheBits, pBuf);


    sblimit = adec->decode.sblimit;
    nbalTable = adec->decode.nbalTable;
    allocTable = adec->decode.allocTable;

    /*
     * DECODE ALLOCATION
     */
    for (sb = 0; sb < sblimit; sb++) {
	ReadBitsMacro(allocIndex[sb], cache, nCacheBits, pBuf, nbalTable[sb]); 
    }
    DebugTask(N_AllocationData,
	Msg("allocation Index:    ");
	for (sb = 0; sb < sblimit; sb++) {
	    if (allocIndex[sb]) Msg("%2d", allocIndex[sb]);
	    else                Msg(" .");
	}
	Msg("\n");
    )

    /*
     * DECODE SCALE FACTOR INDICES AND SCALE FACTORS
     */
    for (sb = 0; sb < sblimit; sb++) {
	if (allocIndex[sb]) {
	    ReadBitsMacro(scfsi[sb], cache, nCacheBits, pBuf, 2);
	}
    }
    DebugTask(N_ScaleData,
	Msg("scaleFactorIndex:         ");
	for (sb = 0; sb < sblimit; sb++) {
	    if (allocIndex[sb]) Msg("%3d", scfsi[sb]);
	    else                Msg("  .");
	}
	Msg("\n");
    )

    for (sb = 0; sb < sblimit; sb++) {
	if (allocIndex[sb]) {
	    switch (scfsi[sb]) {
	      case 0:
		/*
		 * three scaleFactors transmitted ...
		 *	for parts 0,1,2 respectively
		 */
		ReadBitsMacro(scaleFactor[sb][0], cache, nCacheBits, pBuf, 6);
		ReadBitsMacro(scaleFactor[sb][1], cache, nCacheBits, pBuf, 6);
		ReadBitsMacro(scaleFactor[sb][2], cache, nCacheBits, pBuf, 6);
		break;
	      case 1:
		/*
		 * two scaleFactors transmitted ...
		 *	first for parts 0 and 1
		 *	second for part 2
		 */
		ReadBitsMacro(scaleFactor[sb][0], cache, nCacheBits, pBuf, 6);
		ReadBitsMacro(scaleFactor[sb][2], cache, nCacheBits, pBuf, 6);
		scaleFactor[sb][1] = scaleFactor[sb][0];
		break;
	      case 2:
		/*
		 * one scaleFactor transmitted ...
		 *	valid for all three parts
		 */
		ReadBitsMacro(scaleFactor[sb][0], cache, nCacheBits, pBuf, 6);
		scaleFactor[sb][2] = scaleFactor[sb][1] = scaleFactor[sb][0];
		break;
	      case 3:
		/*
		 * two scaleFactors transmitted ...
		 *	first for part 0
		 *	second for parts 1 and 2
		 */
		ReadBitsMacro(scaleFactor[sb][0], cache, nCacheBits, pBuf, 6);
		ReadBitsMacro(scaleFactor[sb][1], cache, nCacheBits, pBuf, 6);
		scaleFactor[sb][2] = scaleFactor[sb][1];
		break;
	      default:
		break;
	    }

	    PrepareSampleReading(sb, adec->parse.curHdr.b.mode,
				 allocIndex[sb],
				 adec->parse.curHdr.b.bitrateIndex,
				 &groupingTable[sb],
				 &factorTable[sb]);

	}
    }

    /*
     * CRC CHECK
     */
    if (adec->parse.curHdr.b.protectionBit == 0) {
	if ( II_CheckCRC(adec, allocIndex, scfsi) != adec->decode.crcCheck ) {
	    return(&BadCrc);
	}
    }

    /*
     * DECODE SAMPLE
     */
    for (granule = 0; granule < 12; granule++) {
	/*
	 * BUFFER AND REQUANTIZE SAMPLES
	 */
	iPart = granule / 4;
	for (sb = 0; sb < sblimit; sb++) {
	    if (allocIndex[sb]) {
		pRec =
	    &Mpeg1A_AllocRecords[(int)*(allocTable + sb*16 + allocIndex[sb])];
		nb = pRec->n_bits_per_codeword;
		if (pRec->n_samples_per_codeword == 1)  {
		    ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
		    WhenDebugEnabled(
		       if (inputSample == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
		       }
		    )

		    requantSample[0][sb] =
			(FloatType) ((inputSample * factorTable[sb] - 1.0
				      + pRec->D) * pRec->C);

		    ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
		    WhenDebugEnabled(
		       if (inputSample == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
		       }
		    )
		    requantSample[1][sb] =
			(FloatType) ((inputSample * factorTable[sb] - 1.0
				      + pRec->D) * pRec->C);

		    ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
		    WhenDebugEnabled(
		       if (inputSample == (1 << nb) - 1) {
		  Msg("WARNING: stream contains an illegal subband sample!\n");
		       }
		    )
		    requantSample[2][sb] =
			(FloatType) ((inputSample * factorTable[sb] - 1.0
				      + pRec->D) * pRec->C);

		} else {
		    ReadBitsMacro(c, cache, nCacheBits, pBuf, nb);
		    WhenDebugEnabled(
		       if (c == (1 << nb) - 1) {
		 Msg("WARNING: stream contains an illegal subband sample!\n");
		       }
		     )
		    c += c << 1;
		    requantSample[0][sb] = groupingTable[sb][c];
		    requantSample[1][sb] = groupingTable[sb][c+1];
		    requantSample[2][sb] = groupingTable[sb][c+2];
		}
		requantSample[0][sb] *=
			Mpeg1A_ScaleFactor[scaleFactor[sb][iPart]];
		requantSample[1][sb] *=
			Mpeg1A_ScaleFactor[scaleFactor[sb][iPart]];
		requantSample[2][sb] *=
			Mpeg1A_ScaleFactor[scaleFactor[sb][iPart]];
		WhenDebugEnabled(
		    int i;
		    for(i = 0; i < 3; i++) {
			if (   requantSample[i][sb] < -1.0
			    || requantSample[i][sb] > 1.0) {
	       Msg("WARNING: rescaled subband sample is not in [-1.0, 1.0]\n");
			}
		    }
                )

	    } else {
		requantSample[0][sb] = 0.0;
		requantSample[1][sb] = 0.0;
		requantSample[2][sb] = 0.0;
	    }
	}

	/*
	 * Initialize other subband samples to 0.0
	 */
	for (sb = sblimit; sb < 32; sb++) {
	    requantSample[0][sb] = 0.0;
	    requantSample[1][sb] = 0.0;
	    requantSample[2][sb] = 0.0;
	}

	/*
	 * SYNTHESIS SUBBAND FILTER
	 */
	pSample = ((UShort *)
		  &adec->decode.sampleBuf[adec->decode.sampleCount << 1]);
	for (iPart = 0; iPart < 3; iPart++) {
	    (*adec->decode.filter.mono)(adec, requantSample[iPart], pSample);
	    pSample += adec->decode.samplesPerSlot;
	}
	adec->decode.sampleCount += adec->decode.samplesPerSlot * 3;
	DebugTask(N_Granules, Msg(".%d", granule) )
    }

    return(NULL);
}

static InternalReasonPtr
Layer2Stereo(Mpeg1ADecPrivPtr adec, UChar *pSrc)
{
    register int	sb, granule;
    int 		allocIndex[2][32];
    int			sblimit;
    int			iPart;
    int			ch;
    UInt		nb;
    UInt		c;
    UInt		scaleFactor[2][32][3];
    short		scfsi[2][32];
    UShort		inputSample;
    UShort		*pSample;
    char		*allocTable;
    UChar		*nbalTable;
    FloatType		requantSample[3][2][32];
    FloatType		*groupingTable[2][32];
    FloatType		factorTable[2][32];
    Mpeg1A_Alloc	*pRec;
    DeclareBitParseVars(cache, nCacheBits, pBuf);

    InitBitParseMacro(&adec->parse.bitParse, pSrc);
    FillCacheMacro(&adec->parse.bitParse, cache, nCacheBits, pBuf);


    sblimit = adec->decode.sblimit;
    nbalTable = adec->decode.nbalTable;
    allocTable = adec->decode.allocTable;

    /*
     * DECODE ALLOCATION
     */
    for (sb = 0; sb < sblimit; sb++) {
       ReadBitsMacro(allocIndex[0][sb], cache, nCacheBits, pBuf, nbalTable[sb]);
       ReadBitsMacro(allocIndex[1][sb], cache, nCacheBits, pBuf, nbalTable[sb]);
    }
    DebugTask(N_AllocationData,
        int ch;
        for (ch = 0; ch < 2; ch++) {
	    Msg("alloIndex ch%d:    ",ch);
	    for (sb = 0; sb < sblimit; sb++) {
	        if (allocIndex[ch][sb]) Msg("%2d", allocIndex[ch][sb]);
	        else                Msg(" .");
	    }
        }
	Msg("\n");
    )

    /*
     * DECODE SCALE FACTOR INDICES AND SCALE FACTORS
     */
    for (sb = 0; sb < sblimit; sb++) {
	if (allocIndex[0][sb]) {
	    ReadBitsMacro(scfsi[0][sb], cache, nCacheBits, pBuf, 2);
	}
	if (allocIndex[1][sb]) {
	    ReadBitsMacro(scfsi[1][sb], cache, nCacheBits, pBuf, 2);
	}
    }
    DebugTask(N_ScaleData,
        int ch;
        for (ch = 0; ch < 2; ch++) {
            Msg("scfsi ch%d:   ", ch);
	    for (sb = 0; sb < sblimit; sb++) {
		if (allocIndex[ch][sb]) Msg("%3d", scfsi[ch][sb]);
		else                Msg("  .");
	    }
	}
	Msg("\n");
    )

    for (sb = 0; sb < sblimit; sb++) {
	for (ch = 0; ch < 2; ch++) {
	    if (allocIndex[ch][sb]) {
		switch (scfsi[ch][sb]) {
		case 0:
		    /*
		     * three scaleFactor[ch]s transmitted ...
		     *	for parts 0,1,2 respectively
		     */
		    ReadBitsMacro(scaleFactor[ch][sb][0], cache, nCacheBits,
				  pBuf, 6);
		    ReadBitsMacro(scaleFactor[ch][sb][1], cache, nCacheBits,
				  pBuf, 6);
		    ReadBitsMacro(scaleFactor[ch][sb][2], cache, nCacheBits,
				  pBuf, 6);
		    break;
		case 1:
		    /*
		     * two scaleFactor[ch]s transmitted ...
		     *	first for parts 0 and 1
		     *	second for part 2
		     */
		    ReadBitsMacro(scaleFactor[ch][sb][0], cache, nCacheBits,
				  pBuf, 6);
		    ReadBitsMacro(scaleFactor[ch][sb][2], cache, nCacheBits,
				  pBuf, 6);
		    scaleFactor[ch][sb][1] = scaleFactor[ch][sb][0];
		    break;
		case 2:
		    /*
		     * one scaleFactor[ch] transmitted ...
		     *	valid for all three parts
		     */
		    ReadBitsMacro(scaleFactor[ch][sb][0], cache, nCacheBits,
				  pBuf, 6);
		    scaleFactor[ch][sb][2] = scaleFactor[ch][sb][1] =
			scaleFactor[ch][sb][0];
		    break;
		case 3:
		    /*
		     * two scaleFactor[ch]s transmitted ...
		     *	first for part 0
		     *	second for parts 1 and 2
		     */
		    ReadBitsMacro(scaleFactor[ch][sb][0], cache, nCacheBits,
				  pBuf, 6);
		    ReadBitsMacro(scaleFactor[ch][sb][1], cache, nCacheBits,
				  pBuf, 6);
		    scaleFactor[ch][sb][2] = scaleFactor[ch][sb][1];
		    break;
		default:
		    break;
		}

		PrepareSampleReading(sb, adec->parse.curHdr.b.mode,
				     allocIndex[ch][sb],
				     adec->parse.curHdr.b.bitrateIndex,
				     &groupingTable[ch][sb],
				     &factorTable[ch][sb]);

	    }
	}
    }

    /*
     * CRC CHECK
     */
    if (adec->parse.curHdr.b.protectionBit == 0) {
	if (   II_CheckCRC(adec, allocIndex, scfsi)
	    != adec->decode.crcCheck ) {
	    return(&BadCrc);
	}
    }

    /*
     * DECODE SAMPLE
     */
    for (granule = 0; granule < 12; granule++) {
	/*
	 * BUFFER AND REQUANTIZE SAMPLES
	 */
	iPart = granule / 4;
	for (sb = 0; sb < sblimit; sb++) {
	    for (ch = 0; ch < 2 ; ch++) {
		if (allocIndex[ch][sb]) {
		    pRec =
	  &Mpeg1A_AllocRecords[(int)*(allocTable + sb*16 + allocIndex[ch][sb])];
		    nb = pRec->n_bits_per_codeword;
		    if (pRec->n_samples_per_codeword == 1)  {
			ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
			WhenDebugEnabled(
			    if (inputSample == (1 << nb) - 1) {
		   Msg("WARNING: stream contains an illegal subband sample!\n");
			    }
		        )

			requantSample[0][ch][sb] =
			(FloatType) ((inputSample * factorTable[ch][sb] - 1.0
				      + pRec->D) * pRec->C);

			ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
			WhenDebugEnabled(
			    if (inputSample == (1 << nb) - 1) {
		   Msg("WARNING: stream contains an illegal subband sample!\n");
			    }
			)
			requantSample[1][ch][sb] =
			(FloatType) ((inputSample * factorTable[ch][sb] - 1.0
				      + pRec->D) * pRec->C);

			ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
			WhenDebugEnabled(
			    if (inputSample == (1 << nb) - 1) {
	           Msg("WARNING: stream contains an illegal subband sample!\n");
			    }
			)
			requantSample[2][ch][sb] =
			(FloatType) ((inputSample * factorTable[ch][sb] - 1.0
				      + pRec->D) * pRec->C);

		    } else {
			ReadBitsMacro(c, cache, nCacheBits, pBuf, nb);
			WhenDebugEnabled(
			    if (c == (1 << nb) - 1) {
		   Msg("WARNING: stream contains an illegal subband sample!\n");
			    }
			)
			c += c << 1;
			requantSample[0][ch][sb] = groupingTable[ch][sb][c];
			requantSample[1][ch][sb] = groupingTable[ch][sb][c+1];
			requantSample[2][ch][sb] = groupingTable[ch][sb][c+2];
		    }
		    requantSample[0][ch][sb] *=
		    Mpeg1A_ScaleFactor[scaleFactor[ch][sb][iPart]];
		    requantSample[1][ch][sb] *=
		    Mpeg1A_ScaleFactor[scaleFactor[ch][sb][iPart]];
		    requantSample[2][ch][sb] *=
		    Mpeg1A_ScaleFactor[scaleFactor[ch][sb][iPart]];
		    WhenDebugEnabled(
		        int i;
			for(i = 0; i < 3; i++) {
			    if (   requantSample[i][ch][sb] < -1.0
				|| requantSample[i][ch][sb] > 1.0) {
		Msg("WARNING: rescaled subband sample is not in [-1.0, 1.0]\n");
			    }
			}
	            )

		} else {
		    requantSample[0][ch][sb] = 0.0;
		    requantSample[1][ch][sb] = 0.0;
		    requantSample[2][ch][sb] = 0.0;
		}
	    }
	}

	/*
	 * Initialize other subband samples to 0.0
	 */
	for (sb = sblimit; sb < 32; sb++) {
	    requantSample[0][0][sb] = 0.0;
	    requantSample[0][1][sb] = 0.0;
	    requantSample[1][0][sb] = 0.0;
	    requantSample[1][1][sb] = 0.0;
	    requantSample[2][0][sb] = 0.0;
	    requantSample[2][1][sb] = 0.0;
	}

	/*
	 * SYNTHESIS SUBBAND FILTER
	 */
	pSample = ((UShort *)
		  &adec->decode.sampleBuf[adec->decode.sampleCount << 1]);
	for (iPart = 0; iPart < 3; iPart++) {
	    (*adec->decode.filter.stereo)(adec, requantSample[iPart], pSample);
	    pSample += adec->decode.samplesPerSlot;
	}
	adec->decode.sampleCount += adec->decode.samplesPerSlot * 3;
	DebugTask(N_Granules, Msg(".%d", granule) )
    }

    return(NULL);
}

static InternalReasonPtr
Layer2Intensity(Mpeg1ADecPrivPtr adec, UChar *pSrc)
{
    register int	sb, granule;
    int 		allocIndex[2][32];
    int			sblimit;
    int			iPart;
    int			bound;
    int			ch;
    UInt		nb;
    UInt		c;
    UInt		scaleFactor[2][32][3];
    short		scfsi[2][32];
    UShort		inputSample;
    UShort		*pSample;
    char		*allocTable;
    UChar		*nbalTable;
    FloatType		requantSample[3][2][32];
    FloatType		*groupingTable[2][32];
    FloatType		factorTable[2][32];
    Mpeg1A_Alloc	*pRec;
    DeclareBitParseVars(cache, nCacheBits, pBuf);

    InitBitParseMacro(&adec->parse.bitParse, pSrc);
    FillCacheMacro(&adec->parse.bitParse, cache, nCacheBits, pBuf);


    sblimit = adec->decode.sblimit;
    nbalTable = adec->decode.nbalTable;
    allocTable = adec->decode.allocTable;

    /*
     * DECODE ALLOCATION
     */
    bound = Mpeg1A_Bound[adec->parse.curHdr.b.modeExtension];
    for (sb = 0; sb < bound; sb++) {
       ReadBitsMacro(allocIndex[0][sb], cache, nCacheBits, pBuf, nbalTable[sb]);
       ReadBitsMacro(allocIndex[1][sb], cache, nCacheBits, pBuf, nbalTable[sb]);
    }
    for (sb = bound; sb < sblimit; sb++) {
       ReadBitsMacro(allocIndex[0][sb], cache, nCacheBits, pBuf, nbalTable[sb]);
       allocIndex[1][sb] = allocIndex[0][sb];
    }
    DebugTask(N_AllocationData,
        int ch;
        for (ch = 0; ch < 2; ch++) {
	    Msg("alloIndex ch%d:    ",ch);
	    for (sb = 0; sb < sblimit; sb++) {
	        if (allocIndex[ch][sb]) Msg("%2d", allocIndex[ch][sb]);
	        else                Msg(" .");
	    }
        }
	Msg("\n");
    )

    /*
     * DECODE SCALE FACTOR INDICES AND SCALE FACTORS
     */
    for (sb = 0; sb < bound; sb++) {
	if (allocIndex[0][sb]) {
	    ReadBitsMacro(scfsi[0][sb], cache, nCacheBits, pBuf, 2);
	}
	if (allocIndex[1][sb]) {
	    ReadBitsMacro(scfsi[1][sb], cache, nCacheBits, pBuf, 2);
	}
    }
    for (sb = bound; sb < sblimit; sb++) {
	if (allocIndex[0][sb]) {
	    ReadBitsMacro(scfsi[0][sb], cache, nCacheBits, pBuf, 2);
	    ReadBitsMacro(scfsi[1][sb], cache, nCacheBits, pBuf, 2);
	}
    }
    DebugTask(N_ScaleData,
        int ch;
        for (ch = 0; ch < 2; ch++) {
            Msg("scfsi ch%d:   ", ch);
	    for (sb = 0; sb < sblimit; sb++) {
		if (allocIndex[ch][sb]) Msg("%3d", scfsi[ch][sb]);
		else                Msg("  .");
	    }
	}
	Msg("\n");
    )

    for (sb = 0; sb < bound; sb++) {
	for (ch = 0; ch < 2; ch++) {
	    if (allocIndex[ch][sb]) {
		switch (scfsi[ch][sb]) {
		case 0:
		    /*
		     * three scaleFactor[ch]s transmitted ...
		     *	for parts 0,1,2 respectively
		     */
		    ReadBitsMacro(scaleFactor[ch][sb][0], cache, nCacheBits,
				  pBuf, 6);
		    ReadBitsMacro(scaleFactor[ch][sb][1], cache, nCacheBits,
				  pBuf, 6);
		    ReadBitsMacro(scaleFactor[ch][sb][2], cache, nCacheBits,
				  pBuf, 6);
		    break;
		case 1:
		    /*
		     * two scaleFactor[ch]s transmitted ...
		     *	first for parts 0 and 1
		     *	second for part 2
		     */
		    ReadBitsMacro(scaleFactor[ch][sb][0], cache, nCacheBits,
				  pBuf, 6);
		    ReadBitsMacro(scaleFactor[ch][sb][2], cache, nCacheBits,
				  pBuf, 6);
		    scaleFactor[ch][sb][1] = scaleFactor[ch][sb][0];
		    break;
		case 2:
		    /*
		     * one scaleFactor[ch] transmitted ...
		     *	valid for all three parts
		     */
		    ReadBitsMacro(scaleFactor[ch][sb][0], cache, nCacheBits,
				  pBuf, 6);
		    scaleFactor[ch][sb][2] = scaleFactor[ch][sb][1] =
			scaleFactor[ch][sb][0];
		    break;
		case 3:
		    /*
		     * two scaleFactor[ch]s transmitted ...
		     *	first for part 0
		     *	second for parts 1 and 2
		     */
		    ReadBitsMacro(scaleFactor[ch][sb][0], cache, nCacheBits,
				  pBuf, 6);
		    ReadBitsMacro(scaleFactor[ch][sb][1], cache, nCacheBits,
				  pBuf, 6);
		    scaleFactor[ch][sb][2] = scaleFactor[ch][sb][1];
		    break;
		default:
		    break;
		}

		PrepareSampleReading(sb, adec->parse.curHdr.b.mode,
				     allocIndex[ch][sb],
				     adec->parse.curHdr.b.bitrateIndex,
				     &groupingTable[ch][sb],
				     &factorTable[ch][sb]);

	    }
	}
    }

    for (sb = bound; sb < sblimit; sb++) {
	if (allocIndex[0][sb]) {
	    for (ch = 0; ch < 2; ch++) {
		switch (scfsi[ch][sb]) {
		case 0:
		    /*
		     * three scaleFactor[ch]s transmitted ...
		     *	for parts 0,1,2 respectively
		     */
		    ReadBitsMacro(scaleFactor[ch][sb][0], cache, nCacheBits,
				  pBuf, 6);
		    ReadBitsMacro(scaleFactor[ch][sb][1], cache, nCacheBits,
				  pBuf, 6);
		    ReadBitsMacro(scaleFactor[ch][sb][2], cache, nCacheBits,
				  pBuf, 6);
		    break;
		case 1:
		    /*
		     * two scaleFactor[ch]s transmitted ...
		     *	first for parts 0 and 1
		     *	second for part 2
		     */
		    ReadBitsMacro(scaleFactor[ch][sb][0], cache, nCacheBits,
				  pBuf, 6);
		    ReadBitsMacro(scaleFactor[ch][sb][2], cache, nCacheBits,
				  pBuf, 6);
		    scaleFactor[ch][sb][1] = scaleFactor[ch][sb][0];
		    break;
		case 2:
		    /*
		     * one scaleFactor[ch] transmitted ...
		     *	valid for all three parts
		     */
		    ReadBitsMacro(scaleFactor[ch][sb][0], cache, nCacheBits,
				  pBuf, 6);
		    scaleFactor[ch][sb][2] = scaleFactor[ch][sb][1] =
			scaleFactor[ch][sb][0];
		    break;
		case 3:
		    /*
		     * two scaleFactor[ch]s transmitted ...
		     *	first for part 0
		     *	second for parts 1 and 2
		     */
		    ReadBitsMacro(scaleFactor[ch][sb][0], cache, nCacheBits,
				  pBuf, 6);
		    ReadBitsMacro(scaleFactor[ch][sb][1], cache, nCacheBits,
				  pBuf, 6);
		    scaleFactor[ch][sb][2] = scaleFactor[ch][sb][1];
		    break;
		default:
		    break;
		}

	    }
	    PrepareSampleReading(sb, adec->parse.curHdr.b.mode,
				 allocIndex[0][sb],
				 adec->parse.curHdr.b.bitrateIndex,
				 &groupingTable[0][sb],
				 &factorTable[0][sb]);

	}
    }

    /*
     * CRC CHECK
     */
    if (adec->parse.curHdr.b.protectionBit == 0) {
	if (   II_CheckCRC(adec, allocIndex, scfsi)
	    != adec->decode.crcCheck ) {
	    return(&BadCrc);
	}
    }

    /*
     * DECODE SAMPLE
     */
    for (granule = 0; granule < 12; granule++) {
	/*
	 * BUFFER AND REQUANTIZE SAMPLES
	 */
	iPart = granule / 4;
	for (sb = 0; sb < bound; sb++) {
	    for (ch = 0; ch < 2 ; ch++) {
		if (allocIndex[ch][sb]) {
		    pRec =
	  &Mpeg1A_AllocRecords[(int)*(allocTable + sb*16 + allocIndex[ch][sb])];
		    nb = pRec->n_bits_per_codeword;
		    if (pRec->n_samples_per_codeword == 1)  {
			ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
			WhenDebugEnabled(
			    if (inputSample == (1 << nb) - 1) {
		   Msg("WARNING: stream contains an illegal subband sample!\n");
			    }
		        )

			requantSample[0][ch][sb] =
			(FloatType) ((inputSample * factorTable[ch][sb] - 1.0
				      + pRec->D) * pRec->C);

			ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
			WhenDebugEnabled(
			    if (inputSample == (1 << nb) - 1) {
		   Msg("WARNING: stream contains an illegal subband sample!\n");
			    }
			)
			requantSample[1][ch][sb] =
			(FloatType) ((inputSample * factorTable[ch][sb] - 1.0
				      + pRec->D) * pRec->C);

			ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
			WhenDebugEnabled(
			    if (inputSample == (1 << nb) - 1) {
	           Msg("WARNING: stream contains an illegal subband sample!\n");
			    }
			)
			requantSample[2][ch][sb] =
			(FloatType) ((inputSample * factorTable[ch][sb] - 1.0
				      + pRec->D) * pRec->C);

		    } else {
			ReadBitsMacro(c, cache, nCacheBits, pBuf, nb);
			WhenDebugEnabled(
			    if (c == (1 << nb) - 1) {
		   Msg("WARNING: stream contains an illegal subband sample!\n");
			    }
			)
			c += c << 1;
			requantSample[0][ch][sb] = groupingTable[ch][sb][c];
			requantSample[1][ch][sb] = groupingTable[ch][sb][c+1];
			requantSample[2][ch][sb] = groupingTable[ch][sb][c+2];
		    }
		    requantSample[0][ch][sb] *=
			Mpeg1A_ScaleFactor[scaleFactor[ch][sb][iPart]];
		    requantSample[1][ch][sb] *=
			Mpeg1A_ScaleFactor[scaleFactor[ch][sb][iPart]];
		    requantSample[2][ch][sb] *=
			Mpeg1A_ScaleFactor[scaleFactor[ch][sb][iPart]];
		    WhenDebugEnabled(
		        int i;
			for(i = 0; i < 3; i++) {
			    if (   requantSample[i][ch][sb] < -1.0
				|| requantSample[i][ch][sb] > 1.0) {
		Msg("WARNING: rescaled subband sample is not in [-1.0, 1.0]\n");
			    }
			}
	            )

		} else {
		    requantSample[0][ch][sb] = 0.0;
		    requantSample[1][ch][sb] = 0.0;
		    requantSample[2][ch][sb] = 0.0;
		}
	    }
	}

	for (sb = bound; sb < sblimit; sb++) {
	    if (allocIndex[0][sb]) {
		pRec =
	  &Mpeg1A_AllocRecords[(int)*(allocTable + sb*16 + allocIndex[0][sb])];
		nb = pRec->n_bits_per_codeword;
		if (pRec->n_samples_per_codeword == 1)  {
		    ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
		    WhenDebugEnabled(
		        if (inputSample == (1 << nb) - 1) {
		 Msg("WARNING: stream contains an illegal subband sample!\n");
			}
		    )
		    requantSample[0][0][sb] =
		    requantSample[0][1][sb] =
		    (FloatType) ((inputSample * factorTable[0][sb] - 1.0
				  + pRec->D) * pRec->C);

		    ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
		    WhenDebugEnabled(
		        if (inputSample == (1 << nb) - 1) {
		 Msg("WARNING: stream contains an illegal subband sample!\n");
			}
		    )
		    requantSample[1][0][sb] =
		    requantSample[1][1][sb] =
		    (FloatType) ((inputSample * factorTable[0][sb] - 1.0
				  + pRec->D) * pRec->C);
		    
		    ReadBitsMacro(inputSample, cache, nCacheBits, pBuf, nb);
		    WhenDebugEnabled(
		        if (inputSample == (1 << nb) - 1) {
		 Msg("WARNING: stream contains an illegal subband sample!\n");
			}
		    )
		    requantSample[2][0][sb] =
		    requantSample[2][1][sb] =
		    (FloatType) ((inputSample * factorTable[0][sb] - 1.0
				  + pRec->D) * pRec->C);
		    
		} else {
		    ReadBitsMacro(c, cache, nCacheBits, pBuf, nb);
		    WhenDebugEnabled(
				     if (c == (1 << nb) - 1) {
					 Msg("WARNING: stream contains an illegal subband sample!\n");
				     }
				     )
		    c += c << 1;
		    requantSample[0][0][sb] = 
		    requantSample[0][1][sb] = groupingTable[0][sb][c];
		    requantSample[1][0][sb] = 
		    requantSample[1][1][sb] = groupingTable[0][sb][c+1];
		    requantSample[2][0][sb] = 
		    requantSample[2][0][sb] = groupingTable[0][sb][c+2];
		}
		requantSample[0][0][sb] *=
			Mpeg1A_ScaleFactor[scaleFactor[0][sb][iPart]];
		requantSample[0][1][sb] *=
			Mpeg1A_ScaleFactor[scaleFactor[1][sb][iPart]];
		requantSample[1][0][sb] *=
			Mpeg1A_ScaleFactor[scaleFactor[0][sb][iPart]];
		requantSample[1][1][sb] *=
			Mpeg1A_ScaleFactor[scaleFactor[1][sb][iPart]];
		requantSample[2][0][sb] *=
			Mpeg1A_ScaleFactor[scaleFactor[0][sb][iPart]];
		requantSample[2][1][sb] *=
			Mpeg1A_ScaleFactor[scaleFactor[1][sb][iPart]];
		WhenDebugEnabled(
		    int i;
		    for(i = 0; i < 3; i++) {
			if (   requantSample[i][0][sb] < -1.0
			    || requantSample[i][0][sb] > 1.0
			    || requantSample[i][1][sb] > 1.0
			    || requantSample[i][1][sb] > 1.0 ) {
	     Msg("WARNING: rescaled subband sample is not in [-1.0, 1.0]\n");
			}
		    }
		)
	    } else {
		requantSample[0][0][sb] = 0.0;
		requantSample[0][1][sb] = 0.0;
		requantSample[1][0][sb] = 0.0;
		requantSample[1][1][sb] = 0.0;
		requantSample[2][0][sb] = 0.0;
		requantSample[2][1][sb] = 0.0;
	    }

	}

	/*
	 * Initialize other subband samples to 0.0
	 */
	for (sb = sblimit; sb < 32; sb++) {
	    requantSample[0][0][sb] = 0.0;
	    requantSample[0][1][sb] = 0.0;
	    requantSample[1][0][sb] = 0.0;
	    requantSample[1][1][sb] = 0.0;
	    requantSample[2][0][sb] = 0.0;
	    requantSample[2][1][sb] = 0.0;
	}

	/*
	 * SYNTHESIS SUBBAND FILTER
	 */
	pSample = ((UShort *)
		  &adec->decode.sampleBuf[adec->decode.sampleCount << 1]);
	for (iPart = 0; iPart < 3; iPart++) {
	    (*adec->decode.filter.stereo)(adec, requantSample[iPart], pSample);
	    pSample += adec->decode.samplesPerSlot;
	}
	adec->decode.sampleCount += adec->decode.samplesPerSlot * 3;
	DebugTask(N_Granules, Msg(".%d", granule) )
    }

    return(NULL);
}

#ifdef CheckMpeg1ASyntax

static Mpeg1ASynthMethodRec Layer2SyntaxMethodData = {
    Layer2Mono,
    Layer2Stereo,
    Layer2Intensity
};
extern Mpeg1ASynthMethodRec Layer1SyntaxMethodData;
static Mpeg1AOpsInitRec Mpeg1ADecodeSynOpsData = {
    Mpeg1AInit,
    &Layer1SyntaxMethodData,
    &Layer2SyntaxMethodData
};
Mpeg1AOpsInit Mpeg1ADecodeSynOps = &Mpeg1ADecodeSynOpsData;

#else

static Mpeg1ASynthMethodRec Layer2MethodData = {
    Layer2Mono,
    Layer2Stereo,
    Layer2Intensity
};
extern Mpeg1ASynthMethodRec Layer1MethodData;
static Mpeg1AOpsInitRec Mpeg1ADecodeOpsData = {
    Mpeg1AInit,
    &Layer1MethodData,
    &Layer2MethodData
};
Mpeg1AOpsInit Mpeg1ADecodeOps = &Mpeg1ADecodeOpsData;

#endif
