/*
 *	(c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		mpeg1adec.h - mpeg 1 video decoder (public interface)
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/mpeg1adec.h,v 1.5 1995/11/02 20:53:07 toddb Exp $
 */
#ifndef _MPEG1ADEC_H
#define _MPEG1ADEC_H

#include "nvrTypes.h"
#include "nvrReason.h"
#include "nvrAudio.h"
#include "nvrBitparse.h"
#include "nvrPacket.h"
#include "mpeg1aproto.h"

typedef enum {
    /*
     * Callbacks that are always available
     */
    Mpeg1ASampleCb,	/* decoded samples are ready for presentation */
    Mpeg1AFormCb,	/* form of output samples has changed */

    /*
     * Callbacks that only available with Mpeg1ADecodeSynOps
     * or Mpeg1ADecodeSynFmtOps
     */
    Mpeg1AHeaderCb,	/* an audio header has been fully parsed */
    Mpeg1APacketFree,	/* 1st packet on the queue is about to be freed */
    Mpeg1ADecodeStartCb,/* a frame is about to be decoded */
    Mpeg1ADecodeEndCb,	/* a frame has been fully decoded */
    Mpeg1APtsSeekCb,	/* 1st packet on queue has a certain pts */
    
    /*
     * non-callbacks this requests information for the callbacks
     * under Mpeg1ADecodeSynOps and Mpeg1ADecodeSynFmtOps
     */
    Mpeg1AFormat	/* format tokens into readable form (syntax only) */
} Mpeg1ACbType;

typedef enum {
    Mpeg1AHeaderMask	  = 1<<Mpeg1AHeaderCb,
    Mpeg1APacketFreeMask  = 1<<Mpeg1APacketFree,
    Mpeg1ASampleMask	  = 1<<Mpeg1ASampleCb,
    Mpeg1AFormMask	  = 1<<Mpeg1AFormCb,
    Mpeg1ADecodeStartMask = 1<<Mpeg1ADecodeStartCb,
    Mpeg1ADecodeEndMask	  = 1<<Mpeg1ADecodeEndCb,
    Mpeg1APtsSeekMask	  = 1<<Mpeg1APtsSeekCb,
    Mpeg1AFormatMask	  = 1<<Mpeg1AFormat	
} Mpeg1ACbMask;

typedef enum {
    Mpeg1ANoFormatting,		/* no formatted items are produced */
    Mpeg1AHeaderLevel,		/* frame headers */
    Mpeg1ASampleLevel,		/* sample detail */
    Mpeg1ANLevels		/* number of levels in this list */
} Mpeg1AFmtLevel;

typedef struct _mpeg1AudioTokens {
    Mpeg1AudioHdr frameHdr;	/* valid for Mpeg1AHeaderCb: frame header */
} Mpeg1AudioTokens, *Mpeg1AudioTokensPtr;

/*
 * This structure is passed to the caller when one of the tokens
 * has been fully parsed and the callback has been selected.
 */
typedef struct _Mpeg1ACbInfo {
    Mpeg1AudioTokensPtr pTokens;	/* tokens for callback */
    PacketDataPtr packetDown;		/* packet from below mpeg1 layer */
    ULong tokenOffset;			/* stream offset for start of token */
    ULong tokenFileOffset;		/* file offset for start of token */
    void *cbData;			/* user-supplied cb data */
    FormattedItemPtr itemList;		/* list of formatted items for token */

    /* for Mpeg1APtsSeekCb */
    long seekTimecode;		 	/* pts for Mpeg1APtsSeekCb */

    /* for Mpeg1ASampleCb callback */
    AudioForm sampleForm;		/* FormCb: form of output samples */
    int nBytesPerFrame;			/* FormCb: # of bytes per frame */
    int nSamples;			/* SampleCb: # of samples ready */
    ULong timecode;			/* timecode of samples */
} Mpeg1ACbInfo, *Mpeg1ACbInfoPtr;

typedef struct _mpeg1ASynthMethodRec *Mpeg1ASynthMethod;

/*
 * This (blind) structure describes the functions that can initialize the
 * audio decoder for certain functions.  Some describe
 * the functions used for the level of syntax checking.  Others
 * describe the functions used for synthesizing audio samples for output.
 */
typedef struct _mpeg1AOpsInit *Mpeg1AOpsInit;

typedef void (*Mpeg1ATokenCb)(PacketLayerPtr, Mpeg1ACbType, Mpeg1ACbInfoPtr);

/*
 * This structure is passed to OpenMpeg1AudioStream().  All items must
 * be defined.  They can be reconfigured at runtime with a call to
 * Mpeg1AConfigure().  See below.
 */
typedef struct _mpeg1AudioConf {
    PacketLayerPtr media;		/* defined only for Open... */
    void (*warning)(char *fmt, ...);	/* warning function */
    Mpeg1ATokenCb tokenCb;		/* callback function */
    void *cbData;			/* user callback data */
    ULong cbSelect;			/* set of Mpeg1ACbMask bits */
    Mpeg1AOpsInit opsInit;		/* operational functions */
    Mpeg1AFmtLevel  formatLevel;	/* syntax only: level of formatting */
    UChar *sampleBuf;			/* where to put samples */
    int bufLength;			/* length of 'sampleBuf' */
    boolean quarterSample;		/* for quality changes */
    boolean monoSample;
    long seekTimecode;      		/* for looking for frames >= timecode */
} Mpeg1AudioConf, *Mpeg1AudioConfPtr;

CExternal PacketLayerPtr OpenMpeg1AudioStream(
    Mpeg1AudioConfPtr,
    InternalReasonPtr *pReason
);

/*
 * These masks are used to determine which items changed in the
 * Mpeg1AConfigure() call.  Note that Mpeg1AConfMediaMask is
 * present for completeness but cannot be specified to Mpeg1AConfigure().
 */
typedef enum {
    Mpeg1AConfMediaMask       = 0x001,
    Mpeg1AConfWarningMask     = 0x002,
    Mpeg1AConfTokenCbMask     = 0x004,
    Mpeg1AConfCbDataMask      = 0x008,
    Mpeg1AConfCbSelectMask    = 0x010,
    Mpeg1AConfOpsInitMask     = 0x020,
    Mpeg1AConfFormatMask      = 0x040,
    Mpeg1AConfSamplesMask     = 0x080,
    Mpeg1AConfQualityMask     = 0x100,
    Mpeg1AConfTimecodeMask    = 0x200,
    Mpeg1AConfAllMask         = 0x1FE	/* excludes Mpeg1AConfMediaMask
					   excludes Mpeg1AConfTimecodeMask */
} Mpeg1AConfigMask;

CExternal InternalReasonPtr Mpeg1AConfigure(
    PacketLayerPtr,
    Mpeg1AudioConfPtr,
    long confMasks
);

CExternal void Mpeg1AFreeFormatItems(
    FormattedItemPtr
);

extern Mpeg1AOpsInit Mpeg1ADecodeNoop;		/* parse frame headers only */
extern Mpeg1AOpsInit Mpeg1ADecodeOps;		/* decode only */
extern Mpeg1AOpsInit Mpeg1ADecodeSynOps;	/* ...check syntax */

#endif /* _MPEG1ADEC_H */
