/*
 *	(c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		mpeg1adecI.h - mpeg 1 audio internal decoding structures
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/mpeg1adecI.h,v 1.10 1996/01/14 20:04:38 toddb Exp $
 */
#ifndef _MPEG1ADECI_H_
#define _MPEG1ADECI_H_

#include "mpeg1adec.h"
#include "nvrPacketI.h"

#define DeclareCrc(mask, crc)					\
    register UShort _data;					\
    UInt mask;							\
    UShort crc = ~0;
#define UpdateCrc(mask, nbits, data, crc) \
    {								\
        _data = data;						\
        mask = 1 << nbits;					\
	while(mask >>= 1) {					\
	    crc = (!((crc) & 0x8000) ^ !(_data & mask))		\
		?						\
		((crc) << 1) ^ 0x8005				\
		:						\
		(crc) << 1;					\
        }							\
    }

#define NMonoSamplesPerSlot(adec)					\
    ((adec)->decode.quarterSample? 8 : 32)
#define NStereoSamplesPerSlot(adec)				        \
    (((adec)->decode.quarterSample ? 8 : 32)				\
   * ((adec)->decode.monoSample ? 1 : 2))

/*
   We need to determine which Mpeg1 Audio header transitions
   indciate that a ConfigureDecoder() should be called

   Valid Sync Word
   |		  ID
   |              |Layer
   |		  || Protection Bit
   |		  || | Bitrate Index
   |		  || | |    Frequency Index
   |		  || | |    | Padding Bit
   |		  || | |    | |Private Bit
   |              || | |    | || Mode
   |		  || | |    | || | Mode Extension
   |              || | |    | || | |  Copyright
   |              || | |    | || | |  |Original Home
   |------------| || | |    | || | |  ||Emphasis
   |            | || | |    | || | |  |||
   |            | |TT| |--| TT|| TTTT ||TT
   XXXX XXXX XXXX XXXX XXXX XXXX XXXX XXXX

   0000 0000 0000 1110 1111 1110 1100 0000
*/

#define ReconfigureMask 0x000efec0
#define PaddingMask 	0x00000200

typedef struct _mpeg1ADecPriv Mpeg1ADecPriv, *Mpeg1ADecPrivPtr;

typedef InternalReasonPtr (*Synthesizer)(Mpeg1ADecPrivPtr, UChar *pSrc);

typedef enum {
    kHeaderSearch,		/* scanning for frame sync word */
    kHeaderSearchPts,		/* same but w/ certain PTS value */
    kTokenCopyNewHeader,	/* copying header in a known position */
    kTokenCopyHeader,		/* copying partial header */
    kTokenScanBody,		/* scanning frame body */
    kTokenCopyBody		/* copying frame body */
} Mpeg1AToken;

typedef union _mpeg1AudioHdrView {
    Mpeg1AudioHdr b;
    ULong composite;
} Mpeg1AudioHdrView;
    
typedef struct _mpeg1ASynthMethodRec {
    Synthesizer mono;
    Synthesizer stereo;
    Synthesizer intensity;
} Mpeg1ASynthMethodRec;

enum {
    CopyBufferInc = 128		/* this can be relatively small for audio */
};

/*
 * Speed is better and allocation size is smaller with FloatType defined
 * as 'float'
 */
typedef float		FloatType;

typedef struct _Mpeg1A_Alloc {
    unsigned short	n_steps;
    unsigned short	n_bits_per_sample;
    unsigned short	n_bits_per_codeword;
    unsigned short	n_samples_per_codeword;
    FloatType		C;
    FloatType		D;
    FloatType		mult_coeff;
} Mpeg1A_Alloc, *Mpeg1A_AllocPtr;

/*
 * This structure defines the interface to the subband filter mechanism
 */
typedef struct _mpeg1ASubbandFilter {
    void (*init)  (Mpeg1ADecPrivPtr);
    void (*stereo)(Mpeg1ADecPrivPtr, FloatType[2][32], UShort *);
    void (*mono)  (Mpeg1ADecPrivPtr, FloatType[32], UShort *);
} Mpeg1ASubbandFilter, *Mpeg1ASubbandFilterPtr;

/****************************************************************************
 *
 * This is the private description of the packet layer for MPEG1 audio.
 *
 ****************************************************************************/
struct _mpeg1ADecPriv {
    PacketLayer pub;		/* public data */

    Mpeg1AudioTokens tokens;

    void (*warning)(char *fmt, ...);

    struct {
        int offset;		/* current offset into source packet */
    } packet;
    
    struct {
        UChar *pToken;		/* pointer into packet */
        UChar *copyBuf;		/* allocated buffer */
        UChar *copyBufEnd;	/* end of copyBuf */
        boolean scanned;	/* whether token is scanned or copied */
        ULong bufSize;
        short offset;		/* current source token offset */
        short length;		/* token length */
        short frameLength;	/* computed length of current frame */
        short padLengths[ 2 ];	/* padded and unpadded lengths */
        Mpeg1AudioHdrView curHdr;/* Current frame header */
        Mpeg1AudioHdrView nxtHdr;/* frame header following the current one */
        ULong timeout;		/* time to give up parsing */
        Mpeg1AToken token;	/* current token being parsed */
	BitParseInfo bitParse;	/* for the bit parsing macros */
	float frameTime;	/* ms play time for current audio frame */
	double timecode;	/* calculated timecode for indiv. frames */
	double pktTimecode;	/* timecode at last change:
				   packet timecode for system stream
				   gop timecode for elem video */
	/*
         * methods for initializing and updating ->timecode and ->pktTimecode.
	 * Both are different, depending on whether this is system or
	 * elem stream
	 */
	void (*initTc)(Mpeg1ADecPrivPtr, PacketDataPtr);
	void (*updateTc)(Mpeg1ADecPrivPtr);
    } parse;

    struct {
	long timecode;		/* reference timecode */	
	PacketDataPtr queue;	/* queue for holding packet w/o delivery */
    } seek;

    /*
     * Variables used to do user callbacks
     */
    struct {
	Mpeg1ACbInfo arg;
	void (*tokenCb)(PacketLayerPtr, Mpeg1ACbType, Mpeg1ACbInfoPtr);
	ULong select;			/* set of (1<<Mpeg1ACbType) bits */
	ULong lastPacketEnd;		/* offset to end of previous packet */
	ULong lastPacketFileEnd;	/* file offset of end of prev packet */
        long nextTokenFileOffset;
        long nextTokenOffset;
        FormattedItemPtr lastItem;	/* last item in arg.itemList */
        Mpeg1AFmtLevel curLevel;	/* current relative item level */
        Mpeg1AFmtLevel maxLevel;	/* highest format item level */
    } cb;

    struct {
        Mpeg1ASynthMethod layer1;	/* layer 1 synthesizer methods */
        Mpeg1ASynthMethod layer2;	/* layer 2 synthesizer methods */
        Synthesizer synth;		/* selected synthesizer for samples */
	Mpeg1ASubbandFilter filter;	/* selected subband filter functions */

        UChar *sampleBuf;		/* user's sample buffer */
        int bufLength;  		/* length of sampleBuf */
	int sampleCount;        	/* # 16-bit samples in buffer */
	boolean quarterSample;		/* quarter sample decoding */
	boolean monoSample;		/* mono sample decoding */
	short samplesPerSlot;		/* filled in when configuring */

	struct {			/* state machine for the synthesis */
	    int actual_write_pos;	/*	filter                     */
	    int highest_subband;
	    int offset1;
	    int offset2;
	    int remaining_offset;
	    FloatType scalefactor;
	    FloatType *actual_v;
	    FloatType v1[512];
	    FloatType v2[512];
	} synthFilter[2];

	UShort crcCheck;

	FloatType requantTable[(1<<16)-1];
					/* layer 1 specific */

        unsigned char *nbalTable;	/* layer 2 specific */
        char *allocTable;		/* layer 2 specific */
        int sblimit;			/* layer 2 specific */
    } decode;
    
}; /* typedef: Mpeg1ADecPriv, *Mpeg1ADecPrivPtr */

/*
 * This is the structure for the blind, public pointer type: Mpeg1ASynthMethod
 */
typedef struct _mpeg1AOpsInit Mpeg1AOpsInitRec;
struct _mpeg1AOpsInit {
    void (*init)(Mpeg1ADecPrivPtr, Mpeg1AOpsInit);
    Mpeg1ASynthMethod layer1;
    Mpeg1ASynthMethod layer2;
};

CExternal void Mpeg1AInit(
    Mpeg1ADecPrivPtr,
    Mpeg1AOpsInit
);
CExternal InternalReasonPtr Mpeg1AudioProcessFirst(
    PacketLayerPtr
);
CExternal void InitLayer1Tables(
    Mpeg1ADecPrivPtr adec
);

CExternal void Mpeg1AConfigureQuality(
    Mpeg1ADecPrivPtr adec
);

extern short      Mpeg1A_BitRateTable[4][15];
extern int        Mpeg1A_SamplingFrequencyTable[4];
extern int        Mpeg1A_Bound[4];
extern char      *Mpeg1A_ModeNames[];
extern char      *Mpeg1A_AbbrevModeNames[];
extern char      *Mpeg1A_JointStereoBandNames[];
extern char      *Mpeg1A_EmphasisNames[];
extern FloatType  Mpeg1A_ScaleFactor[64];

extern UChar	    Mpeg1A_Table0_nbal[];
extern UChar	    Mpeg1A_Table1_nbal[];
extern UChar	    Mpeg1A_Table2_nbal[];
extern UChar	    Mpeg1A_Table3_nbal[];
extern char	    Mpeg1A_AllocTable0[];
extern char	    Mpeg1A_AllocTable1[];
extern char	    Mpeg1A_AllocTable2[];
extern char	    Mpeg1A_AllocTable3[];
extern Mpeg1A_Alloc Mpeg1A_AllocRecords[];


/* used by LayerII audio only */
extern FloatType    Mpeg1A_table_cd_factor[16];
extern FloatType    Mpeg1A_table_ab4_factor[8];
extern FloatType    Mpeg1A_table_ab3_factor[8];
extern FloatType    Mpeg1A_table_ab2_factor[16];
extern FloatType    Mpeg1A_table_ab1_factor[16];
extern FloatType   *Mpeg1A_table_cd_groupingTables[16];
extern FloatType   *Mpeg1A_table_ab234_groupingTables[16];
extern FloatType   *Mpeg1A_table_ab1_groupingTables[16];
extern FloatType    Mpeg1A_grouping_7bits[125 * 3];
extern FloatType    Mpeg1A_grouping_5bits[27 * 3];

extern Mpeg1ASubbandFilterPtr Mpeg1AHighQualFilter;
extern Mpeg1ASubbandFilterPtr Mpeg1ALowQualFilter;
extern Mpeg1ASubbandFilterPtr Mpeg1ANoopFilter;

#endif /* _MPEG1ADECI_H_ */
