/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1adecode.c - frame level decoding for mpeg audio
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1adecode.c,v 1.15 1996/01/14 20:04:54 toddb Exp $
 */

#include "mpeg1adecI.h"
#include "nvrAlloc.h"

#ifdef CheckMpeg1ASyntax

static void
FreeCallbackPacket(Mpeg1ADecPrivPtr adec)
{
    PacketDataPtr packet = adec->pub.queue;

    if (packet == NULL)
	return;
    adec->cb.lastPacketEnd = packet->pos + packet->dataBytes;
    if (adec->cb.select & Mpeg1APacketFreeMask)
        (*adec->cb.tokenCb)(&adec->pub, Mpeg1APacketFree, &adec->cb.arg);
}

#else /* CheckMpeg1ASyntax */

#define FreeCallbackPacket(adec)	/* as nothing */

#endif /* CheckMpeg1ASyntax */

static InternalReasonPtr
StretchDest(Mpeg1ADecPrivPtr adec, int minSize)
{
    UChar *newCopyBuf;
    int allocSize = adec->parse.bufSize;
    
    while (allocSize < minSize)
        allocSize += CopyBufferInc;

    newCopyBuf = (UChar *)InsecureAlloc(allocSize);
    if (newCopyBuf == NULL)
        return( AllocReason("temp MPEG audio token", allocSize) );
        
    if (adec->parse.copyBuf) {
        if (adec->parse.offset)
            memcpy(newCopyBuf, adec->parse.copyBuf, adec->parse.offset);
        free( (char *)adec->parse.copyBuf );
    }
    adec->parse.copyBuf = newCopyBuf;
    adec->parse.bufSize = allocSize;
    adec->parse.copyBufEnd = newCopyBuf + allocSize;

    return(NULL);
}

static InternalReasonPtr
CopyToken(Mpeg1ADecPrivPtr adec, UChar *pSrc, int fragmentLength)
{
    InternalReasonPtr irp;
    int tokenLength = fragmentLength + adec->parse.offset;

    if (tokenLength > adec->parse.bufSize) {
	if (NULL != (irp = StretchDest(adec, tokenLength)))
	    return(irp);
    }
    memcpy((char *)adec->parse.copyBuf + adec->parse.offset,
	   (char *)pSrc,
	   fragmentLength);
    adec->parse.offset += fragmentLength;

    return (NULL);
}

static InternalReasonPtr
ConfigureDecoder(Mpeg1ADecPrivPtr adec, ULong newCompositeHdr)
{
    Mpeg1AudioHdrView newHdr;
    int bitsPerChan, bitrate, bitrateIndex, layer;
    int frequency, freqIndex;
    AudioForm *sampleForm;
    ULong newLook, curLook;
    
    curLook = adec->parse.curHdr.composite & ReconfigureMask;
    newLook = newCompositeHdr & ReconfigureMask;
    adec->parse.curHdr.composite = newCompositeHdr;
    if (curLook == newLook)
	return(NULL);

    newHdr.composite = newCompositeHdr;

    /*
     * if just the padding has changed, then change the length and return
     */
    curLook |= PaddingMask;
    newLook |= PaddingMask;
    if (curLook == newLook) {
	adec->parse.frameLength = adec->parse.padLengths[ newHdr.b.paddingBit ];
	return(NULL);
    }

    freqIndex = newHdr.b.freqIndex;
    frequency = Mpeg1A_SamplingFrequencyTable[ freqIndex ];
    layer = newHdr.b.layer;
    bitrateIndex = newHdr.b.bitrateIndex;
    if (bitrateIndex == 15) {
	return( BadValueReason("Invalid bitrate index: 15\n") );
    }
    bitrate = Mpeg1A_BitRateTable[ layer ][ bitrateIndex ];

    switch (layer) {
    case Layer1:
	/* for padding frames, length gets 1 more slot (4 bytes) */
	adec->parse.padLengths[0] = 4 * ( 12 * bitrate * 1000 / frequency );
	adec->parse.padLengths[1] = adec->parse.padLengths[0] + 4;
	adec->parse.frameLength = adec->parse.padLengths[newHdr.b.paddingBit];
	adec->parse.frameTime = 384.0 * 1000.0 / frequency;

	switch (newHdr.b.mode) {
	case ModeSingleChannel:
	    adec->decode.synth = adec->decode.layer1->mono;
	    break;
	case ModeStereo:
	case ModeDualChannel:
	    adec->decode.synth = adec->decode.layer1->stereo;
	    break;
	case ModeJointStereo:
	    adec->decode.synth = adec->decode.layer1->intensity;
	    break;
	}
	break;

    case Layer2:
	/* for padding frames, length gets 1 more slot (1 byte) */
	adec->parse.padLengths[0] = 144 * bitrate * 1000 / frequency;
	adec->parse.padLengths[1] = adec->parse.padLengths[0] + 1;
	adec->parse.frameLength = adec->parse.padLengths[ newHdr.b.paddingBit ];

	adec->parse.frameTime = 1152.0 * 1000.0 / frequency;

	/* test validity of mode and bitrate */
	if (newHdr.b.mode == ModeSingleChannel) {
	    switch (bitrate) {
	    case 224: case 256: case 320: case 384:
	        return( BadValueReason("%dkbits/sec is illegal with %s mode\n",
	        	    bitrate, Mpeg1A_ModeNames[ newHdr.b.mode ]));
	    }
	} else { /* newHdr.b.mode != ModeSingleChannel */
	    switch (bitrate) {
	    case 32: case 48: case 56:
	        return( BadValueReason("%dkbits/sec is illegal with %s mode\n",
	        	    bitrate, Mpeg1A_ModeNames[ newHdr.b.mode ]));
	    }
	}

	switch (newHdr.b.mode) {
	case ModeSingleChannel:
	    adec->decode.synth = adec->decode.layer2->mono;
	    bitsPerChan = bitrate;
	    break;
	case ModeStereo:
	case ModeDualChannel:
	    adec->decode.synth = adec->decode.layer2->stereo;
	    bitsPerChan = bitrate >> 1;
	    break;
	case ModeJointStereo:
	    adec->decode.synth = adec->decode.layer2->intensity;
	    bitsPerChan = bitrate >> 1;
	}

	if ((freqIndex == RateIndex48000HZ && bitsPerChan >= 56) 
		|| (bitsPerChan >= 56 &&  bitsPerChan <= 80)) {
	    adec->decode.sblimit = 27;
	    adec->decode.nbalTable = Mpeg1A_Table0_nbal;
	    adec->decode.allocTable = Mpeg1A_AllocTable0;
	} else if (freqIndex != RateIndex48000HZ && bitsPerChan >= 96) {
	    adec->decode.sblimit = 30;
	    adec->decode.nbalTable = Mpeg1A_Table1_nbal;
	    adec->decode.allocTable = Mpeg1A_AllocTable1;
	} else if (freqIndex != RateIndex32000HZ && bitsPerChan <= 48) {
	    adec->decode.sblimit = 8;
	    adec->decode.nbalTable = Mpeg1A_Table2_nbal;
	    adec->decode.allocTable = Mpeg1A_AllocTable2;
	} else {
	    adec->decode.sblimit = 12;
	    adec->decode.nbalTable = Mpeg1A_Table3_nbal;
	    adec->decode.allocTable = Mpeg1A_AllocTable3;
	}

	break;
    }

    Mpeg1AConfigureQuality(adec);

    return (NULL);
}

static int
FindNextHeader(Mpeg1ADecPrivPtr adec, UChar *pSrc)
{
    ULong frameHdr = adec->parse.nxtHdr.composite;
    PacketDataPtr packet = adec->pub.queue;
    UChar *pSrcEnd = packet->buffer + packet->dataBytes;
    UChar *pTokenEnd;

    pTokenEnd = pSrc;
    while (pTokenEnd < pSrcEnd) {
	frameHdr = (frameHdr << 8) | *pTokenEnd++;
	if ((frameHdr >> 20) == ValidSyncWord)
	    break;
    }

    adec->parse.nxtHdr.composite = frameHdr;
    return( pTokenEnd - pSrc );
}

/*
 * Timecodes usage:
 *	parse.timecode:	calculated timecode for individual frames
 *	parse.pktTimecode: saved timecode value (system streams only)
 *
 * NOTE: it is required that the packet->timecode values in packets comming
 *	from an elementary audio stream are the same on *all* packets.
 * NOTE: initialization prerequisites:
 *	adec->parse.curHdr.b.syncWord != ValidSyncWord
 *	system stream: parse.pktTimecode != first packet->timecode
 *	elem stream:   parse.timecode = 0;
 * NOTE: the value parse.timecode is valid for the entire life of the
 *	audio frame [first bytes of header, first byte of next header)
 */
static InternalReasonPtr
Mpeg1AudioProcess(PacketLayerPtr adecPub)
{
    Mpeg1ADecPrivPtr adec = (Mpeg1ADecPrivPtr)adecPub;
    InternalReasonPtr irp = NULL;
    UChar *pSrc, *pSrcEnd, *pToken;
    boolean timeToYield = False;
    int available;
    ULong hdr;
    PacketDataPtr packet;

    pSrc = pSrcEnd = NULL;
    packet = NULL;
    
    while (! timeToYield) {
	if (packet == NULL)
	    packet = adec->pub.queue;
	else {
	    FreeCallbackPacket(adec);
	    adec->pub.queue = packet->next;
	    adec->packet.offset = 0;
	    (*packet->pLayer->ops.free)(packet);
	    packet = adec->pub.queue;
	    if (packet == NULL)
		goto parsingComplete;
	}

	/*
	 * If the last byte of the frame header for the current frame
	 * fell in the last packet and the first byte of the frame
	 * data falls in this packet, then we need to update pToken,
	 * because it still points at the last byte of the last
	 * packet.
	 */
	if (adec->packet.offset == 0 && adec->parse.token == kTokenScanBody)
	    adec->parse.pToken = packet->buffer;
        
        pSrc = packet->buffer + adec->packet.offset;
        pSrcEnd = packet->buffer + packet->dataBytes;
        if (pSrc >= pSrcEnd)
	    continue;

	while (!timeToYield && pSrc < pSrcEnd) {
	    switch (adec->parse.token) {
	    case kHeaderSearch:
		adec->parse.length = FindNextHeader(adec, pSrc);
		pSrc += adec->parse.length;
		if (adec->parse.nxtHdr.b.syncWord != ValidSyncWord)
		    continue;
		hdr = adec->parse.nxtHdr.composite;
		goto newHeaderComplete;

	    case kTokenScanBody:
		/*
		 * If the full frame is available in this packet... we can
		 * simply decode it right away.
		 */
		available = pSrcEnd - pSrc;
		if (available >= adec->parse.length) {
		    if (adec->parse.curHdr.b.protectionBit == 0) {
			adec->decode.crcCheck = (pSrc[0]<<8) | pSrc[1];
			irp = (*adec->decode.synth)(adec, pSrc+2);
		    } else {
			irp = (*adec->decode.synth)(adec, pSrc);
		    }
		    pSrc += adec->parse.length;
		    if (irp) {
			goto parsingComplete;
		    }
		    adec->parse.token = kTokenCopyNewHeader;
		} else {
		    adec->parse.offset = 0;
		    irp = CopyToken(adec, pSrc, available);
		    adec->parse.token = kTokenCopyBody;
		    pSrc += available;
		    if (irp) {
			goto parsingComplete;
		    }
		}
		break;

	    case kTokenCopyBody:
		/*
		 * If the rest of the frame has finally arrived, we
		 * can now decode it..
		 */

		adec->parse.length = adec->parse.frameLength
		                     - adec->parse.offset - 4;
		available = pSrcEnd - pSrc;
		if (available > adec->parse.length)
		    available = adec->parse.length;
		irp = CopyToken(adec, pSrc, available);
		pSrc += available;
		if (irp)
		    goto parsingComplete;
		if (available == adec->parse.length) {
		    if (adec->parse.curHdr.b.protectionBit == 0) {
			adec->decode.crcCheck = (  adec->parse.copyBuf[0]<<8)
						 | adec->parse.copyBuf[1];

			irp = (*adec->decode.synth)(adec,
						    adec->parse.copyBuf+2);
			
		    } else {
			irp = (*adec->decode.synth)(adec,
						    adec->parse.copyBuf);
		    }
		    if (irp)
			goto parsingComplete;
		    adec->parse.token = kTokenCopyNewHeader;
		}
		break;

	    case kTokenCopyNewHeader:
	        /*
	         * Simplest case: we are done with the previous frame and
	         * we need the next frame header.  If the bytes available
	         * is at least four, then this is very fast.
	         */
		hdr = (pSrc[0] << 24)
		    | (pSrc[1] << 16)
		    | (pSrc[2] << 8)
		    | (pSrc[3] << 0);
		available = pSrcEnd - pSrc;
		if (available >= 4) {
		    if ((hdr >> 20) != ValidSyncWord) {
			adec->parse.token = kHeaderSearch;
			break;
		    }
		    pSrc += 4;

		newHeaderComplete:
		    /*
		     * We're done!  Update the callback timecode here because
		     * we have just found the header.  Immediately, after,
		     * we update the timecode to the current packet/
		     */
		    if (hdr != adec->parse.curHdr.composite) {
			irp = ConfigureDecoder(adec, hdr);
			if (irp) {
			    goto parsingComplete;
			}
		    }

		    /*
		     * now that we have seen the next frame header, we
		     * can decide if there is room for the callback
		     * samples.  If not, we deliver it here.  Note that
		     * the last frame of samples must be flushed out
		     */
		    if (adec->cb.select & Mpeg1ASampleMask) {
			int room = adec->decode.bufLength
				 - (adec->decode.sampleCount << 1);
			if (room < adec->cb.arg.nBytesPerFrame) {
			    adec->cb.arg.nSamples = adec->decode.sampleCount;
			    adec->decode.sampleCount = 0;

			    (*adec->cb.tokenCb)(&adec->pub, Mpeg1ASampleCb,
						&adec->cb.arg);
			    /*
			     * update the next timecode
			     */
			    adec->cb.arg.timecode = adec->parse.timecode;
			    timeToYield = True;
			}
		    }

		    (*adec->parse.updateTc)(adec);

		    adec->parse.token = kTokenScanBody;
		    adec->parse.length = adec->parse.frameLength - 4;
		} else {
		    adec->parse.nxtHdr.composite = hdr;
		    adec->parse.offset = available;
		    pSrc += available;
		    adec->parse.token = kTokenCopyHeader;
		}
		break;

	    case kTokenCopyHeader:
		pToken = (UChar *)&adec->parse.nxtHdr.composite
			 + adec->parse.offset;
		adec->parse.length = 4 - adec->parse.offset;
		available = pSrcEnd - pSrc;
		if (available > adec->parse.length)
		    available = adec->parse.length;
		switch (available) {
		case 3: *pToken++ = *pSrc++;
		case 2: *pToken++ = *pSrc++;
		case 1: *pToken++ = *pSrc++;
		}
		
		if (4 == (adec->parse.offset += available)) {
		    hdr = adec->parse.nxtHdr.composite;
		    if ((hdr >> 20) == ValidSyncWord) {
			goto newHeaderComplete;
		    }
		    adec->parse.token = kHeaderSearch;
		}
		break;
	    
	    default:	/* not possible */
		break;
	    }
        }
    }

parsingComplete:
    if (irp) {
        adec->parse.token = kHeaderSearch;
    }
    if (packet) {
        adec->packet.offset = pSrc - packet->buffer;
    }

    return(irp);
}

/*
 * This is the first process function called so that we get an opportunity
 * to do some initialization.
 */
InternalReasonPtr
Mpeg1AudioProcessFirst(PacketLayerPtr pLayer)
{
    Mpeg1ADecPrivPtr adec = (Mpeg1ADecPrivPtr)pLayer;

    /*
     * Set the process function to the normal one, then initialize the
     * timecode stuff.
     */
    pLayer->ops.process = Mpeg1AudioProcess;

    (*adec->parse.initTc)(adec, adec->pub.queue);

    return( (*pLayer->ops.process)(pLayer) );
}
