/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1apacket.c - main control for mpeg audio decoder
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1apacket.c,v 1.13 1996/01/17 00:08:59 toddb Exp $
 */

#include "mpeg1adecI.h"
#include "nvrAlloc.h"


static InternalReason CantChangeMedia =
    { "Mpeg1AConfigure: cannot change media\n", IR_InternalProblem };

static InternalReason BadMediaValue =
    { "OpenMpeg1AudioStream: media is NULL\n", IR_InternalProblem };

static InternalReason BadOpsInitValue =
    { "OpenMpeg1AudioStream: synOpsInit or procOpsInit is NULL\n",
      IR_InternalProblem };

static InternalReasonPtr
Mpeg1AudioDeliver(PacketLayerPtr pLayer, PacketDataPtr packet)
{
    QueuePacket(&pLayer->queue, packet);
    return(NULL);
}

		     
static int
FindNextHeader(Mpeg1ADecPrivPtr adec, PacketDataPtr packet, UChar *pSrc)
{
    ULong frameHdr = adec->parse.nxtHdr.composite;
    UChar *pSrcEnd = packet->buffer + packet->dataBytes;
    UChar *pTokenEnd;

    pTokenEnd = pSrc;
    while (pTokenEnd < pSrcEnd) {
	frameHdr = (frameHdr << 8) | *pTokenEnd++;
	if ((frameHdr >> 20) == ValidSyncWord)
	    break;
    }

    adec->parse.nxtHdr.composite = frameHdr;
    return( pTokenEnd - pSrc );
}

static InternalReasonPtr
ConfigureDecoderInSeek(Mpeg1ADecPrivPtr adec, ULong newCompositeHdr)
{
    Mpeg1AudioHdrView newHdr;
    int bitrate, bitrateIndex, layer;
    int frequency, freqIndex;
    
    newHdr.composite = newCompositeHdr;

    freqIndex = newHdr.b.freqIndex;
    if (freqIndex == 3) {
	return( BadValueReason("Invalid frequency index: 3\n") );
    }
    frequency = Mpeg1A_SamplingFrequencyTable[ freqIndex ];
    layer = newHdr.b.layer;
    if (layer == 0) {
	return( BadValueReason("Invalid layer token: 0\n") );
    }
    bitrateIndex = newHdr.b.bitrateIndex;
    if (bitrateIndex == 15) {
	return( BadValueReason("Invalid bitrate index: 15\n") );
    }
    bitrate = Mpeg1A_BitRateTable[ layer ][ bitrateIndex ];

    switch (layer) {
    case Layer1:
	adec->parse.frameLength = 4 * ( 12 * bitrate * 1000 / frequency );
	/* for padding frames, length gets 1 more slot (4 bytes) */
	adec->parse.frameLength += newHdr.b.paddingBit << 2;
        adec->parse.frameTime = 384.0 * 1000.0 / frequency;

	break;

    case Layer2:
	adec->parse.frameLength = ( 144 * bitrate * 1000 / frequency );
	/* for padding frames, length gets 1 more slot (1 byte) */
	adec->parse.frameLength += newHdr.b.paddingBit;
        adec->parse.frameTime = 1152.0 * 1000.0 / frequency;

	/* test validity of mode and bitrate */
	if (newHdr.b.mode == ModeSingleChannel) {
	    switch (bitrate) {
	    case 224: case 256: case 320: case 384:
	        return( BadValueReason("%dkbits/sec is illegal with %s mode\n",
	        	    bitrate, Mpeg1A_ModeNames[ newHdr.b.mode ]));
	    }
	} else { /* newHdr.b.mode != ModeSingleChannel */
	    switch (bitrate) {
	    case 32: case 48: case 56:
	        return( BadValueReason("%dkbits/sec is illegal with %s mode\n",
	        	    bitrate, Mpeg1A_ModeNames[ newHdr.b.mode ]));
	    }
	}
	break;
    default:
	return( BadValueReason("invalid layer designation 0x%x\n", layer) );
    }

    adec->parse.curHdr.composite = newCompositeHdr;
    
    return (NULL);
}
 
static boolean
SeekDone(Mpeg1ADecPrivPtr adec, PacketDataPtr packet)
{
    if (adec->parse.timecode + adec->parse.frameTime > adec->seek.timecode)
	return(True);
    (*adec->parse.updateTc)(adec);
    return(False);
}

/*
 * Return with the packet/packet.offset of top of queue whose timecode
 *	is <= seek.timecode.
 * Timecodes usage:
 *	parse.timecode:	calculated timecode for individual frames
 *	parse.pktTimecode: saved timecode value (system streams only)
 *
 * NOTE: it is required that the packet->timecode values in packets comming
 *	from an elementary audio stream are the same on *all* packets.
 * NOTE: initialization prerequisites:
 *	adec->parse.curHdr.b.syncWord != ValidSyncWord
 *	system stream: parse.pktTimecode != first packet->timecode
 *	elem stream:   parse.timecode = 0;
 * NOTE: the value parse.timecode is valid for the entire life of the
 *	audio frame [first bytes of header, first byte of next header)
 */
static InternalReasonPtr
Mpeg1AudioDeliverInSeek(PacketLayerPtr pLayer, PacketDataPtr packet)
{
    Mpeg1ADecPrivPtr adec = (Mpeg1ADecPrivPtr)pLayer;
    InternalReasonPtr reason = NULL;
    UChar *pSrc, *pSrcEnd, *pToken;
    boolean done = False;
    int available;
    ULong hdr;


    pSrc = packet->buffer;
    pSrcEnd = packet->buffer + packet->dataBytes;

    while (pSrc < pSrcEnd) {
	switch (adec->parse.token) {
	case kHeaderSearch:
	    adec->parse.length = FindNextHeader(adec, packet, pSrc);
	    pSrc += adec->parse.length;
	    if (adec->parse.nxtHdr.b.syncWord != ValidSyncWord) {
		continue;
	    }
	    hdr = adec->parse.nxtHdr.composite;
	    goto newHeaderComplete;

	case kTokenScanBody:
	    /*
	     * If the full frame is available in this packet... we can
	     * simply decode it right away.
	     */
	    available = pSrcEnd - pSrc;
	    if (available >= adec->parse.length) {
		pSrc += adec->parse.length;
		adec->parse.token = kTokenCopyNewHeader;

		/* frame available */
		if ((done = SeekDone(adec,packet)) == True) {
		    goto parsingComplete;
		}
	    } else {
		/*
		 * parse.offset here is different than for decoding because
		 * adec->parse.offset = 0; CopyToken(adec, pSrc, available);
		 * is not being called which would set the parse.offset to
		 * available
		 */
		adec->parse.offset = available;
		adec->parse.token = kTokenCopyBody;
		pSrc += available;
	    }
	    break;
	    
	case kTokenCopyBody:
	    /*
	     * If the rest of the frame has finally arrived, we
	     * can now decode it..
	     */

	    adec->parse.length = adec->parse.frameLength
		                 - adec->parse.offset - 4;
	    available = pSrcEnd - pSrc;
	    if (available > adec->parse.length) {
		available = adec->parse.length;
	    }
	    pSrc += available;
	    /*
	     * parse.offset here is different than for decoding because
	     *	CopyToken(adec, pSrc, available) is not being called
	     *	which would increment parse.offset by available
	     */
	    adec->parse.offset += available;
	    if (available == adec->parse.length) {
		adec->parse.token = kTokenCopyNewHeader;

		/* frame available */
		if ((done = SeekDone(adec,packet)) == True) {
		    goto parsingComplete;
		}
	    }
	    break;

	case kTokenCopyNewHeader:
	    /*
	     * Simplest case: we are done with the previous frame and
	     * we need the next frame header.  If the bytes available
	     * is at least four, then this is very fast.
	     */
	    hdr = (pSrc[0] << 24)
	           | (pSrc[1] << 16)
	           | (pSrc[2] << 8)
		   | (pSrc[3] << 0);
	    available = pSrcEnd - pSrc;
	    if (available >= 4) {
		if ((hdr >> 20) != ValidSyncWord) {
		    adec->parse.token = kHeaderSearch;
		    break;
		}
		pSrc += 4;
		
	    newHeaderComplete:
		/*
		 * We're done!
		 */
		if (hdr != adec->parse.curHdr.composite) {
		    reason = ConfigureDecoderInSeek(adec, hdr);
		    if (reason) {
			FreeReason(reason);
			adec->parse.token = kHeaderSearch;
			break;
		    }
		}

		/*
		 * free any saved packets on the data queue
		 * save offset state
		 */
		DiscardQueuedPackets(packet->pLayer);
		adec->packet.offset = pSrc - packet->buffer;
		
		adec->parse.token = kTokenScanBody;
		adec->parse.length = adec->parse.frameLength - 4;
	    } else {
		adec->parse.nxtHdr.composite = hdr;
		adec->parse.offset = available;
		pSrc += available;
		adec->parse.token = kTokenCopyHeader;
	    }
	    break;

	case kTokenCopyHeader:
	    pToken = (UChar *)&adec->parse.nxtHdr.composite
		      + adec->parse.offset;
	    adec->parse.length = 4 - adec->parse.offset;
	    available = pSrcEnd - pSrc;
	    if (available > adec->parse.length)
	        available = adec->parse.length;
	    switch (available) {
	    case 3: *pToken++ = *pSrc++;
	    case 2: *pToken++ = *pSrc++;
	    case 1: *pToken++ = *pSrc++;
	    }
	    
	    if (4 == (adec->parse.offset += available)) {
		hdr = adec->parse.nxtHdr.composite;
		if ((hdr >> 20) == ValidSyncWord) {
		    goto newHeaderComplete;
		}
		adec->parse.token = kHeaderSearch;
	    }
	    break;
	    
	default:	/* not possible */
	    break;
	}
    }


 parsingComplete:
    if (reason) {
	(*packet->pLayer->ops.free)(packet);
	pLayer->ops.deliver = Mpeg1AudioDeliver;
	adec->parse.token = kHeaderSearch;
	return(reason);
    }

    QueuePacket(&pLayer->queue, packet);
	
    if (done) {
	adec->parse.token = kTokenCopyHeader;
	adec->parse.offset = 0;
	adec->parse.nxtHdr.composite = adec->parse.curHdr.composite;
	pLayer->ops.deliver = Mpeg1AudioDeliver;

	if (adec->cb.select & Mpeg1APtsSeekMask) {
	    adec->cb.arg.seekTimecode = adec->parse.timecode;
	    (*adec->cb.tokenCb)(&adec->pub, Mpeg1APtsSeekCb, &adec->cb.arg);
	}
    }

    return(NULL);
}

static InternalReasonPtr
Mpeg1AudioFirstDeliverInSeek(PacketLayerPtr pLayer, PacketDataPtr packet)
{
    Mpeg1ADecPrivPtr adec = (Mpeg1ADecPrivPtr)pLayer;
    InternalReasonPtr reason = NULL;

    /*
     * Note that we immediately change the deliver function...
     */
    pLayer->ops.deliver = Mpeg1AudioDeliverInSeek;

    (*adec->parse.initTc)(adec, packet);
		
    adec->parse.token = kHeaderSearch;
    adec->parse.curHdr.b.syncWord = ~ValidSyncWord;
    QueuePacket(&adec->seek.queue, packet);
    while (! reason && NULL != (packet = adec->seek.queue)) {
	RemovePacket(&adec->seek.queue, packet);
	reason = (*pLayer->ops.deliver)(pLayer, packet);
    }
    if (reason) {
	while (NULL != (packet = adec->seek.queue)) {
	    RemovePacket(&adec->seek.queue, packet);
	    (*packet->pLayer->ops.free)(packet);
	}
    }

    return(reason);
}

static InternalReasonPtr
Mpeg1AudioFlush(PacketLayerPtr pLayer)
{
    Mpeg1ADecPrivPtr adec = (Mpeg1ADecPrivPtr)pLayer;

    if (   adec->cb.select & Mpeg1ASampleMask
	&& adec->decode.sampleCount > 0 ) {
	adec->cb.arg.nSamples = adec->decode.sampleCount;
	adec->decode.sampleCount = 0;
	(*adec->cb.tokenCb)(&adec->pub, Mpeg1ASampleCb,
			    &adec->cb.arg);
    }
    return( NULL );
}

/*
 * It is required that at least one access unit has been parsed before
 *	seeking
 */
static InternalReasonPtr
Mpeg1AudioSeek(PacketLayerPtr pLayer, ULong absOffset)
{
    Mpeg1ADecPrivPtr adec = (Mpeg1ADecPrivPtr)pLayer;
    Mpeg1AudioConf conf;

    /*
     * Need to reinitialize the audio decoder
     * Reconfigure with the current quality settings
     */
    conf.monoSample = adec->decode.monoSample;
    conf.quarterSample = adec->decode.quarterSample;
    Mpeg1AConfigure(pLayer, &conf, Mpeg1AConfQualityMask);

    if (adec->parse.curHdr.b.syncWord != ValidSyncWord) {
	return(NULL);
    }
    if (! pLayer->down->timeCodes) {
	pLayer->ops.deliver = Mpeg1AudioFirstDeliverInSeek;
	adec->seek.timecode = (absOffset / adec->parse.frameLength) *
			       adec->parse.frameTime;
    }

    return( NULL );
}

static void
Mpeg1AudioClose(PacketLayerPtr pLayer)
{
    Mpeg1ADecPrivPtr adec = (Mpeg1ADecPrivPtr)pLayer;
    FreePacketList(&pLayer->freeList);
    pLayer->freeList = NULL;
    FreePacketList(&pLayer->queue);
    pLayer->queue = NULL;
    if (adec->parse.copyBuf)
	free((char *)adec->parse.copyBuf);

    (*pLayer->down->ops.close)(pLayer->down);
    free((char *)pLayer);
}

static void
Mpeg1AudioFree(PacketDataPtr packet)
{
    /* we don't allocate any packets */
}

void
Mpeg1AConfigureQuality(Mpeg1ADecPrivPtr adec)
{
    AudioForm *sampleForm = &adec->cb.arg.sampleForm;
    int frequency, freqIndex;

    if (adec->parse.curHdr.b.syncWord != ValidSyncWord) {
	return;
    }
    freqIndex = adec->parse.curHdr.b.freqIndex;
    frequency = Mpeg1A_SamplingFrequencyTable[ freqIndex ];
    if (adec->decode.monoSample
     || adec->parse.curHdr.b.mode == ModeSingleChannel) {
        sampleForm->channels = AudioMono;
    } else {
        sampleForm->channels = AudioStereo;
    }

    switch(adec->decode.quarterSample == True ? frequency >> 2 : frequency) {
    case 48000:
	sampleForm->rate = Audio48Khz;
	break;
    case 12000:
	sampleForm->rate = Audio12Khz;
	break;
    case 44100:
	sampleForm->rate = Audio44_1Khz;
	break;
    case 11025:
	sampleForm->rate = Audio11_025Khz;
	break;
    case 32000:
	sampleForm->rate = Audio32Khz;
	break;
    case 8000:
	sampleForm->rate = Audio8Khz;
	break;
    }
    sampleForm->dataForm = Audio16BitPcm;

    switch (adec->parse.curHdr.b.mode) {
    case ModeSingleChannel:
	adec->decode.samplesPerSlot = NMonoSamplesPerSlot(adec);
	break;
    case ModeStereo:
    case ModeDualChannel:
    case ModeJointStereo:
	adec->decode.samplesPerSlot = NStereoSamplesPerSlot(adec);
	break;
    }

    switch (adec->parse.curHdr.b.layer) {
    case Layer1:
        adec->cb.arg.nBytesPerFrame = adec->decode.samplesPerSlot * 12 * 2;
	break;

    case Layer2:
        adec->cb.arg.nBytesPerFrame = adec->decode.samplesPerSlot * 3 * 12 * 2;
	break;
    }

    if (adec->cb.select & Mpeg1AFormMask)
        (*adec->cb.tokenCb)(&adec->pub, Mpeg1AFormCb, &adec->cb.arg);
}

InternalReasonPtr
Mpeg1AConfigure(PacketLayerPtr pLayer, Mpeg1AudioConfPtr conf, long confMasks)
{
    PacketDataPtr packet;
    InternalReasonPtr reason = NULL;
    Mpeg1ADecPrivPtr adec = (Mpeg1ADecPrivPtr) pLayer;

    if (confMasks & Mpeg1AConfMediaMask)
	return( &CantChangeMedia );
    if (confMasks & Mpeg1AConfWarningMask)
	adec->warning = conf->warning;
    if (confMasks & Mpeg1AConfTokenCbMask)
	adec->cb.tokenCb = conf->tokenCb;
    if (confMasks & Mpeg1AConfCbDataMask)
	adec->cb.arg.cbData = conf->cbData;
    if (confMasks & Mpeg1AConfOpsInitMask)
	(*conf->opsInit->init)(adec, conf->opsInit);
    if (confMasks & Mpeg1AConfCbSelectMask)
	adec->cb.select = conf->cbSelect;
    if (confMasks & Mpeg1AConfFormatMask)
	adec->cb.maxLevel = conf->formatLevel;
    if (confMasks & Mpeg1AConfSamplesMask) {
	adec->decode.sampleBuf = conf->sampleBuf;
	adec->decode.bufLength = conf->bufLength;
	if (conf->bufLength < adec->cb.arg.nBytesPerFrame) {
	    reason = BadValueReason("%s %d bytes\n",
		"audio sample buffer  is too small.\nMust be at least",
		adec->cb.arg.nBytesPerFrame);
	} else if (((int)conf->sampleBuf) & 1) {
	    reason = BadValueReason(
		"audio sample buffer is not 16-bit aligned\n");
	}
    }
    if (! reason && (confMasks & Mpeg1AConfTimecodeMask)) {
	while (NULL != (packet = pLayer->queue)) {
	    RemovePacket(&pLayer->queue, packet);
	    QueuePacket(&adec->seek.queue, packet);
	}
	adec->seek.timecode = conf->seekTimecode;
	pLayer->ops.deliver = Mpeg1AudioFirstDeliverInSeek;
    }
    if (! reason && (confMasks & Mpeg1AConfQualityMask)) {
	adec->decode.monoSample = conf->monoSample;
	adec->decode.quarterSample = conf->quarterSample;
	adec->decode.sampleCount = 0;
	if (adec->decode.monoSample)
	    adec->decode.filter = *Mpeg1ALowQualFilter;
	else
	    adec->decode.filter = *Mpeg1AHighQualFilter;
	(*adec->decode.filter.init)(adec);
	Mpeg1AConfigureQuality(adec);
    }
    
    return(reason);
}

static void
InitSystemTc(Mpeg1ADecPrivPtr adec, PacketDataPtr packet)
{
    adec->parse.timecode = packet->timecode;
    adec->parse.pktTimecode = packet->timecode;
    adec->cb.arg.timecode = packet->timecode;
}

static void
UpdateSystemTc(Mpeg1ADecPrivPtr adec)
{
    PacketDataPtr packet = adec->pub.queue;

    if (adec->parse.pktTimecode != packet->timecode) {
	adec->parse.timecode = packet->timecode;
	adec->parse.pktTimecode = packet->timecode;
    } else {
	adec->parse.timecode += adec->parse.frameTime;
    }
}

static void
InitElementaryTc(Mpeg1ADecPrivPtr adec, PacketDataPtr packet)
{
    ULong realOffset;

    /*	timecode initialization for elementary streams
     *
     * The best we can do is assume that the packet length
     *	doesn't change and that there are no padding
     *	bytes at the end of a frame. The only deterministic
     *	method is to scan the headers instead of really seeking
     */
    realOffset = GetFileOffsetOfPacket(packet, NULL);
    adec->parse.timecode = (realOffset / adec->parse.frameLength) *
			   adec->parse.frameTime;
    if (realOffset % adec->parse.frameLength) {
	adec->parse.timecode += adec->parse.frameTime;
    }
    adec->parse.pktTimecode = 0;
    adec->cb.arg.timecode = adec->parse.timecode;
}

static void
UpdateElementaryTc(Mpeg1ADecPrivPtr adec)
{
    adec->parse.timecode += adec->parse.frameTime;
}

PacketLayerPtr
OpenMpeg1AudioStream(Mpeg1AudioConfPtr conf, InternalReasonPtr *pReason)
{
    Mpeg1ADecPrivPtr adec;
    PacketLayerPtr pLayer;
    int allocSize;

   if (conf->opsInit == NULL) {
        *pReason = &BadOpsInitValue;
        return(NULL);
    }
    if (conf->media == NULL) {
        *pReason = &BadMediaValue;
        return(NULL);
    }
    
    allocSize = sizeof(Mpeg1ADecPriv);
    pLayer = AllocPacketLayer(PacketMpeg1Audio, conf->media, allocSize);
    if (pLayer == NULL) {
	*pReason = AllocReason("Mpeg 1 Audio packet layer",
	                       sizeof(PacketLayer) + allocSize);
	return(NULL);
    }
    adec = (Mpeg1ADecPrivPtr)pLayer;

    adec->cb.arg.pTokens = &adec->tokens;

    (void) Mpeg1AConfigure(pLayer, conf, Mpeg1AConfAllMask);

    InitLayer1Tables(adec);

    pLayer->ops.deliver = Mpeg1AudioDeliver;
    pLayer->ops.process = Mpeg1AudioProcessFirst;
    pLayer->ops.flush   = Mpeg1AudioFlush;
    pLayer->ops.close   = Mpeg1AudioClose;
    pLayer->ops.seek    = Mpeg1AudioSeek;
    pLayer->ops.free    = Mpeg1AudioFree;

    /* timecode initialization */
    if (pLayer->down->timeCodes) {
	adec->parse.initTc = InitSystemTc;
	adec->parse.updateTc = UpdateSystemTc;
    } else {
	adec->parse.initTc = InitElementaryTc;
	adec->parse.updateTc = UpdateElementaryTc;
    }

    return(pLayer);
}

void
Mpeg1AInit(Mpeg1ADecPrivPtr adec, Mpeg1AOpsInit opsInit)
{
    adec->decode.layer1 = opsInit->layer1;
    adec->decode.layer2 = opsInit->layer2;
}

