/*
 *	(c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		mpeg1aproto.h - mpeg 1 audio protocol structures
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/mpeg1aproto.h,v 1.3 1995/09/27 02:10:42 toddb Exp $
 */
#ifndef _MPEG1APROTO_H_
#define _MPEG1APROTO_H_

#include "mpeg1Proto.h"

/*************************************************************************
 *
 * mpeg audio tokens
 *
 *************************************************************************/
enum {
    Layer1		= 3,
    Layer2		= 2,
    Layer3		= 1,

    /* MPEG audio Modes */
    ModeStereo		= 0,
    ModeJointStereo	= 1,	/* intensity_stereo  and/or ms_stereo */
    ModeDualChannel	= 2,
    ModeSingleChannel	= 3,
    ModeMono		= 3,

    /* MPEG Audio MODE Extention values for Intensity Stereo */
    ModeExt0		= 0,	/* subband  4-31, bound =  4 */
    ModeExt1		= 1,	/* subband  8-31, bound =  8 */
    ModeExt2		= 2,	/* subband 12-31, bound = 12 */
    ModeExt3		= 3,	/* subband 16-31, bound = 16 */

    /* MPEG Audio Emphasis */
    EmphasisNone	= 0,
    Emphasis50Div15Msec	= 1,
    EmphasisReserved	= 2,
    EmphasisCcittJ17	= 3,

    /* MPEG Samping Rate Indices */
    RateIndex44100HZ	= 0,
    RateIndex48000HZ	= 1,
    RateIndex32000HZ	= 2,
    RateIndexReserved	= 3,
    
    ValidSyncWord	= 0xfff,

    /* The the samples for a frame */
    NLayer1Samples	= 384,	/* Layer I */
    NLayer2Samples	= 1152,	/* Layer II */
    NLayer3Samples	= 1152	/* Layer III */
};

typedef struct _mpeg1AudioHdr {
    ULong syncWord         :12;     /* 0xfff */

    /*
     * Audio Frame Header Information Structure
     */
    ULong ID                :1;		/* 1 = MPEG, 0 = reserved */
    ULong layer             :2;		/* 3,2,1 for layer 1,2,3; 0=reserved */
    ULong protectionBit     :1;		/* 0 ==> includes redundancy */
    ULong bitrateIndex      :4;		/* BitRateTable index */
    ULong freqIndex         :2;		/* SamplingFrequencyTable index */
    ULong paddingBit        :1;		/* 1==> has added slot to adjust mean */
					/*  bitrate to sampling frequency  */
					/* only necessary for 44.1 kHz */
    ULong privateBit        :1;		/* for private use */
    ULong mode              :2;		/* ModeStereo, ModeJointStereo, */
					/* ModeDualChannel,ModeSingleChannel */
    ULong modeExtension     :2;		/* Layer I & II: Joint Stereo mode */
					/* see MODE_EXT_# and MA_Bound[]  */
					/* Layer III: encoding for */
					/* intensity_stereo */
					/* (bit 1) and ms_stereo (bit 0). */
    ULong copyright         :1;		/* 0 = no copyright, 1 = copyrighted */
    ULong original_home     :1;		/* 0 = is a copy, 1 = original */
    ULong emphasis          :2;		/* See EMPHASIS_* */
} Mpeg1AudioHdr, *Mpeg1AudioHdrPtr;

extern short      Mpeg1A_BitRateTable[4][15];
extern int        Mpeg1A_SamplingFrequencyTable[4];
extern char      *Mpeg1A_ModeNames[];

#endif /* _MPEG1APROTO_H_ */
