/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1vError.c - Error reporting functions for mpeg1.
 *			The intent of this is to keep the complexity of
 *			error reporting out of the main line of mpeg1 parsing.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1vError.c,v 1.2 1995/05/06 19:40:25 toddb Exp $
 */

#include "mpeg1vdecI.h"

NameEnumPair TokenNames[] = {
    { "kSeqSearch",		kSeqSearch },
    { "kGopSearch",		kGopSearch },
    { "kBPicSearch",		kBPicSearch },
    { "kIPPicSearch",		kIPPicSearch },
    { "kSeqHeader",		kSeqHeader },
    { "kPreGopSearch",		kPreGopSearch },
    { "kSeqExtData",		kSeqExtData },
    { "kSeqUserData",		kSeqUserData },
    { "kCopySeqExtData",	kCopySeqExtData },
    { "kCopySeqUserData",	kCopySeqUserData },
    { "kCopySeqIQM",		kCopySeqIQM },
    { "kCopySeqNIQM",		kCopySeqNIQM },
    { "kGopHeader",		kGopHeader },
    { "kPrePicSearch",		kPrePicSearch },
    { "kGopExtData",		kGopExtData },
    { "kGopUserData",		kGopUserData },
    { "kCopyGopExtData",	kCopyGopExtData },
    { "kCopyGopUserData",	kCopyGopUserData },
    { "kPicHeader",		kPicHeader },
    { "kPreSliceSearch",	kPreSliceSearch },
    { "kPicExtData",		kPicExtData },
    { "kPicUserData",		kPicUserData },
    { "kCopyPicExtData",	kCopyPicExtData },
    { "kCopyPicUserData",	kCopyPicUserData },
    { "kCopySliceData",		kCopySliceData },
    { "kScanSliceData",		kScanSliceData },
    { NULL,                     0 }
};

static char *
TokenToName(Mpeg1VToken token)
{
    char *name = EnumToName(token, TokenNames);
    if (name == NULL)
        name = "??";
    return(name);
}

InternalReasonPtr
TokenTooLarge(int available, Mpeg1VToken curToken)
{
    return( InternalProblem("fill %d bytes on token %s?\n",
                            available, TokenToName( curToken )));
}

InternalReasonPtr
MissingTooSmall(int missing, Mpeg1VToken curToken)
{
    return( InternalProblem("token %s missing %d bytes!\n",
                        TokenToName( curToken ), missing));
}

InternalReasonPtr
UnknownToken(Mpeg1VToken curToken)
{
    return( InternalProblem("unknown token: %d\n", curToken) );
}

static char *
StartCodeName(ULong startCode)
{
    char *name;

    switch( startCode ) {
    default:
	if (startCode >= Slice00StartCode
	 && startCode <= SliceAFStartCode) {
	    static char sliceName[ 24 ];
	    sprintf(name = sliceName, "slice %d", startCode & 0xff);
	} else {
	    name = "unknown";
	}
	break;

    case PictureStartCode:	name = "picture start"; break;
    case UserDataStartCode:	name = "user data start"; break;
    case SeqHeaderStartCode:	name = "seq header start"; break;
    case SeqErrorCode:		name = "seq error"; break;
    case ExtensionStartCode:	name = "extension start"; break;
    case SeqenceEndCode:	name = "seq end"; break;
    case GroupStartCode:	name = "group start"; break;

    /*
     * System level start codes
     */
    case PackStartCode:		name = "pack start"; break;
    case SysStartCode:		name = "system start"; break;
    case SysEndCode:		name = "system end"; break;
    }

    return( name );
}

InternalReasonPtr
UnexpectedVideoStartCode(Mpeg1VDecPrivPtr vdec,
			 UChar *pSrc,
                         ULong expected,
                         ULong got)
{
    InternalReasonPtr reason;
    
    reason = BadSyntaxReason(
	"byte %d: Expected start code 0x%x (%s), got 0x%x (%s)\n",
	GetFileOffsetOfPacket(vdec->pub.queue, pSrc),
	expected, StartCodeName(expected), got, StartCodeName(got));
    return(reason);
}

