/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *      mpeg1vdecode.c - start-code level decoding for mpeg video
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1vcb.c,v 1.2 1995/05/02 20:19:17 toddb Exp $
 */

#ifndef Mpeg1VCallbacks
#define Mpeg1VCallbacks
#endif /* Mpeg1VCallbacks */

#ifndef FormatMpeg1V
#define FormatMpeg1V
#endif /* FormatMpeg1V */

#include "mpeg1vdecI.h"
#include "nvrAlloc.h"

void
CallCallback(Mpeg1VDecPrivPtr vdec, Mpeg1VCbType cbType)
{
    if (vdec->cb.select & (1<<cbType))
        (*vdec->cb.tokenCb)(&vdec->pub, cbType, &vdec->cb.arg);
    Mpeg1VFreeFormatItems(vdec->cb.arg.itemList);
    vdec->cb.arg.itemList = NULL;
}

/*
 * This should only get called when an old packet gets freed and
 * a new one is about to be processed.  This is so that we can
 * keep track of the previous packet offset when a token (a startcode) spans
 * two packets.
 */
void
FreeCallbackPacket(Mpeg1VDecPrivPtr vdec)
{
    PacketDataPtr packet = vdec->pub.queue;

    if (packet == NULL)
	return;
    vdec->cb.lastPacketEnd = packet->pos + packet->dataBytes;
    if (vdec->cb.select & Mpeg1VPacketFreeMask)
        (*vdec->cb.tokenCb)(&vdec->pub, Mpeg1VPacketFree, &vdec->cb.arg);
}

void
SetCallbackTokenStart(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    int bytesInThisPacket;

    bytesInThisPacket = pSrc - vdec->pub.queue->buffer;
    vdec->cb.arg.tokenOffset = vdec->cb.nextTokenOffset;
    vdec->cb.arg.tokenFileOffset = vdec->cb.nextTokenFileOffset;
    if (bytesInThisPacket < 4) {
	vdec->cb.nextTokenOffset     = vdec->cb.lastPacketEnd
				     - (4 - bytesInThisPacket);
	vdec->cb.nextTokenFileOffset = vdec->cb.lastPacketFileEnd
				     - (4 - bytesInThisPacket);
    } else {
	vdec->cb.nextTokenOffset     = vdec->pub.queue->pos
				     + (pSrc - vdec->pub.queue->buffer)
				     - 4;
	vdec->cb.nextTokenFileOffset =
	    GetFileOffsetOfPacket(vdec->pub.queue, pSrc) - 4;
    }
}

void
Mpeg1VFreeFormatItems(FormattedItemPtr itemList)
{
    FormattedItemPtr item;
    FormattedItemPtr next = NULL;
    extern void DumpAllocations(void);

    for (item = itemList; item; item = next) {
        next = item->next;
        free((char *)item);
    }
}

