/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1vdata.c - global data declarations.
 *
 * DESCRIPTION
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1vdata.c,v 1.5 1995/10/12 20:20:16 toddb Exp $
 */

#define NeedFunctionPrototypes 1

#include "mpeg1huffman.h"
#include "mpeg1vproto.h"
#include "binarybits.h"
#include "nvrDebug.h"

float FrameRateTable[16] = {
    0.0,	/* 0 */
    23.976,	/* 1 */
    24,		/* 2 */
    25,		/* 3 */
    29.97,	/* 4 */
    30,		/* 5 */
    50,		/* 6 */
    59.94,	/* 7 */
    60,		/* 8 */
    15,		/* 9 */
    6,		/* 10 */
    10,		/* 11 */
    15,		/* 12 */
    20,		/* 13 */
    5,		/* 14 */
    0,		/* 15 */
};

char IPBDchars[5] = { '?', 'I', 'P', 'B', 'D' };

char *MpegBlockNames[] = { "CR", "CB", "Y3", "Y2", "Y1", "Y0" };

HuffCodeMap MBAI_InitData[ MBAI_INIT_CELLS ] = {
  /* This is column 1 from table B1 */
  /* huffman code              longest  nBits  inc value */
  {  Binary1(1,			11),	(1<<6)|1 },
  {  Binary3(011,		11),	(3<<6)|2 },
  {  Binary3(010,		11),	(3<<6)|3 },
  {  Binary4(0011,		11),	(4<<6)|4 },
  {  Binary4(0010,		11),	(4<<6)|5 },

  {  Binary5(0001,1,		11),	(5<<6)|6 },
  {  Binary5(0001,0,		11),	(5<<6)|7 },
  {  Binary7(0000,111,		11),	(7<<6)|8 },
  {  Binary7(0000,110,		11),	(7<<6)|9 },
  {  Binary8(0000,1011,		11),	(8<<6)|10 },

  {  Binary8(0000,1010,		11),	(8<<6)|11 },
  {  Binary8(0000,1001,		11),	(8<<6)|12 },
  {  Binary8(0000,1000,		11),	(8<<6)|13 },
  {  Binary8(0000,0111,		11),	(8<<6)|14 },
  {  Binary8(0000,0110,		11),	(8<<6)|15 },

  { Binary10(0000,0101,11,	11),	(10<<6)|16},

  /* This is column 2 from table B1 */
  /* huffman code              longest  nBits  inc value */
  { Binary10(0000,0101,10,	11),	(10<<6)|17 },
  { Binary10(0000,0101,01,	11),	(10<<6)|18 },
  { Binary10(0000,0101,00,	11),	(10<<6)|19 },
  { Binary10(0000,0100,11,	11),	(10<<6)|20 },
  { Binary10(0000,0100,10,	11),	(10<<6)|21 },

  { Binary11(0000,0100,011,	11),	(11<<6)|22 },
  { Binary11(0000,0100,010,	11),	(11<<6)|23 },
  { Binary11(0000,0100,001,	11),	(11<<6)|24 },
  { Binary11(0000,0100,000,	11),	(11<<6)|25 },
  { Binary11(0000,0011,111,	11),	(11<<6)|26 },

  { Binary11(0000,0011,110,	11),	(11<<6)|27 },
  { Binary11(0000,0011,101,	11),	(11<<6)|28 },
  { Binary11(0000,0011,100,	11),	(11<<6)|29 },
  { Binary11(0000,0011,011,	11),	(11<<6)|30 },
  { Binary11(0000,0011,010,	11),	(11<<6)|31 },

  { Binary11(0000,0011,001,	11),	(11<<6)|32 },
  { Binary11(0000,0011,000,	11),	(11<<6)|33 },
};

HuffCodeMap MBT_B_InitData[ MBT_B_INIT_CELLS ] = {
  /* huffman code     longest   nBits  type value */
  {  Binary2(10,	6),	(2<<5)|MBT_MFWD|MBT_MBAK },
  {  Binary2(11,	6),	(2<<5)|MBT_MFWD|MBT_MBAK|MBT_PATTERN },
  {  Binary3(010,	6),	(3<<5)|MBT_MBAK },
  {  Binary3(011,	6),	(3<<5)|MBT_MBAK|MBT_PATTERN },
  {  Binary4(0010,	6),	(4<<5)|MBT_MFWD },
  {  Binary4(0011,	6),	(4<<5)|MBT_MFWD|MBT_PATTERN },
  {  Binary5(0001,1,	6),	(5<<5)|MBT_INTRA },
  {  Binary5(0001,0,	6),	(5<<5)|MBT_QUANT|MBT_MFWD|MBT_MBAK|MBT_PATTERN},
  {  Binary6(0000,11,	6),	(6<<5)|MBT_QUANT|MBT_MFWD|MBT_PATTERN },
  {  Binary6(0000,10,	6),	(6<<5)|MBT_QUANT|MBT_MBAK|MBT_PATTERN },
  {  Binary6(0000,01,	6),	(6<<5)|MBT_QUANT|MBT_INTRA }
};

HuffCodeMap MBT_P_InitData[ MBT_P_INIT_CELLS ] = {
  /* huffman code     longest   nBits  type value */
  {  Binary1(1,		6),	(1<<5)|MBT_MFWD|MBT_PATTERN },
  {  Binary2(01,	6),	(2<<5)|MBT_PATTERN },
  {  Binary3(001,	6),	(3<<5)|MBT_MFWD },
  {  Binary5(0001,1,	6),	(5<<5)|MBT_INTRA },
  {  Binary5(0001,0,	6),	(5<<5)|MBT_QUANT|MBT_MFWD|MBT_PATTERN },
  {  Binary5(0000,1,	6),	(5<<5)|MBT_QUANT|MBT_PATTERN },
  {  Binary6(0000,01,	6),	(6<<5)|MBT_QUANT|MBT_INTRA }
};

HuffCodeMap MotionVectorInitData[ MOTION_INIT_CELLS ] = {
  /* huffman code             longest   nBits  vector value */
  {  Binary11(0000,0011,001,	11),	(11<<6)|0 },	/* -16 */
  {  Binary11(0000,0011,011,	11),	(11<<6)|1 },	/* -15 */
  {  Binary11(0000,0011,101,	11),	(11<<6)|2 },	/* -14 */
  {  Binary11(0000,0011,111,	11),	(11<<6)|3 },	/* -13 */
  {  Binary11(0000,0100,001,	11),	(11<<6)|4 },	/* -12 */
  {  Binary11(0000,0100,011,	11),	(11<<6)|5 },	/* -11 */
  {  Binary10(0000,0100,11,	11),	(10<<6)|6 },	/* -10 */
  {  Binary10(0000,0101,01,	11),	(10<<6)|7 },	/* -9 */
  {  Binary10(0000,0101,11,	11),	(10<<6)|8 },	/* -8 */
  {   Binary8(0000,0111,	11),	( 8<<6)|9 },	/* -7 */
  {   Binary8(0000,1001,	11),	( 8<<6)|10 },	/* -6 */
  {   Binary8(0000,1011,	11),	( 8<<6)|11 },	/* -5 */
  {   Binary7(0000,111,		11),	( 7<<6)|12 },	/* -4 */
  {   Binary5(0001,1,		11),	( 5<<6)|13 },	/* -3 */
  {   Binary4(0011,		11),	( 4<<6)|14 },	/* -2 */
  {   Binary3(011,		11),	( 3<<6)|15 },	/* -1 */
  {   Binary1(1,		11),	( 1<<6)|16 },	/* 0 */
  {   Binary3(010,		11),	( 3<<6)|17 },	/* 1 */
  {   Binary4(0010,		11),	( 4<<6)|18 },	/* 2 */
  {   Binary5(0001,0,		11),	( 5<<6)|19 },	/* 3 */
  {   Binary7(0000,110,		11),	( 7<<6)|20 },	/* 4 */
  {   Binary8(0000,1010,	11),	( 8<<6)|21 },	/* 5 */
  {   Binary8(0000,1000,	11),	( 8<<6)|22 },	/* 6 */
  {   Binary8(0000,0110,	11),	( 8<<6)|23 },	/* 7 */
  {  Binary10(0000,0101,10,	11),	(10<<6)|24 },	/* 8 */
  {  Binary10(0000,0101,00,	11),	(10<<6)|25 },	/* 9 */
  {  Binary10(0000,0100,10,	11),	(10<<6)|26 },	/* 10 */
  {  Binary11(0000,0100,010,	11),	(11<<6)|27 },	/* 11 */
  {  Binary11(0000,0100,000,	11),	(11<<6)|28 },	/* 12 */
  {  Binary11(0000,0011,110,	11),	(11<<6)|29 },	/* 13 */
  {  Binary11(0000,0011,100,	11),	(11<<6)|30 },	/* 14 */
  {  Binary11(0000,0011,010,	11),	(11<<6)|31 },	/* 15 */
  {  Binary11(0000,0011,000,	11),	(11<<6)|32 },	/* 16 */
};

HuffCodeMap CodedBlockInitData[ CODED_BLOCK_INIT_CELLS ] = {
  /* column 1, table B3 */
  /* huffman code             longest   nBits   coded block value */
  {   Binary3(111,		9),	( 3<<6)|60 },
  {   Binary4(1101,		9),	( 4<<6)|4 },
  {   Binary4(1100,		9),	( 4<<6)|8 },
  {   Binary4(1011,		9),	( 4<<6)|16 },
  {   Binary4(1010,		9),	( 4<<6)|32 },

  {   Binary5(1001,1,		9),	( 5<<6)|12 },
  {   Binary5(1001,0,		9),	( 5<<6)|48 },
  {   Binary5(1000,1,		9),	( 5<<6)|20 },
  {   Binary5(1000,0,		9),	( 5<<6)|40 },
  {   Binary5(0111,1,		9),	( 5<<6)|28 },

  {   Binary5(0111,0,		9),	( 5<<6)|44 },
  {   Binary5(0110,1,		9),	( 5<<6)|52 },
  {   Binary5(0110,0,		9),	( 5<<6)|56 },
  {   Binary5(0101,1,		9),	( 5<<6)|1 },
  {   Binary5(0101,0,		9),	( 5<<6)|61 },

  {   Binary5(0100,1,		9),	( 5<<6)|2 },
  {   Binary5(0100,0,		9),	( 5<<6)|62 },
  {   Binary6(0011,11,		9),	( 6<<6)|24 },
  {   Binary6(0011,10,		9),	( 6<<6)|36 },
  {   Binary6(0011,01,		9),	( 6<<6)|3 },

  {   Binary6(0011,00,		9),	( 6<<6)|63 },
  {   Binary7(0010,111,		9),	( 7<<6)|5 },
  {   Binary7(0010,110,		9),	( 7<<6)|9 },
  {   Binary7(0010,101,		9),	( 7<<6)|17 },
  {   Binary7(0010,100,		9),	( 7<<6)|33 },

  {   Binary7(0010,011,		9),	( 7<<6)|6 },
  {   Binary7(0010,010,		9),	( 7<<6)|10 },
  {   Binary7(0010,001,		9),	( 7<<6)|18 },
  {   Binary7(0010,000,		9),	( 7<<6)|34 },
  {   Binary8(0001,1111,	9),	( 8<<6)|7 },

  {   Binary8(0001,1110,	9),	( 8<<6)|11 },
  {   Binary8(0001,1101,	9),	( 8<<6)|19 },

  /* column 2, table B3 */
  /* huffman code             longest   nBits   coded block value */
  {   Binary8(0001,1100,	9),	( 8<<6)|35 },
  {   Binary8(0001,1011,	9),	( 8<<6)|13 },
  {   Binary8(0001,1010,	9),	( 8<<6)|49 },
  {   Binary8(0001,1001,	9),	( 8<<6)|21 },
  {   Binary8(0001,1000,	9),	( 8<<6)|41 },

  {   Binary8(0001,0111,	9),	( 8<<6)|14 },
  {   Binary8(0001,0110,	9),	( 8<<6)|50 },
  {   Binary8(0001,0101,	9),	( 8<<6)|22 },
  {   Binary8(0001,0100,	9),	( 8<<6)|42 },
  {   Binary8(0001,0011,	9),	( 8<<6)|15 },

  {   Binary8(0001,0010,	9),	( 8<<6)|51 },
  {   Binary8(0001,0001,	9),	( 8<<6)|23 },
  {   Binary8(0001,0000,	9),	( 8<<6)|43 },
  {   Binary8(0000,1111,	9),	( 8<<6)|25 },
  {   Binary8(0000,1110,	9),	( 8<<6)|37 },

  {   Binary8(0000,1101,	9),	( 8<<6)|26 },
  {   Binary8(0000,1100,	9),	( 8<<6)|38 },
  {   Binary8(0000,1011,	9),	( 8<<6)|29 },
  {   Binary8(0000,1010,	9),	( 8<<6)|45 },
  {   Binary8(0000,1001,	9),	( 8<<6)|53 },

  {   Binary8(0000,1000,	9),	( 8<<6)|57 },
  {   Binary8(0000,0111,	9),	( 8<<6)|30 },
  {   Binary8(0000,0110,	9),	( 8<<6)|46 },
  {   Binary8(0000,0101,	9),	( 8<<6)|54 },
  {   Binary8(0000,0100,	9),	( 8<<6)|58 },

  {   Binary9(0000,0011,1,	9),	( 9<<6)|31 },
  {   Binary9(0000,0011,0,	9),	( 9<<6)|47 },
  {   Binary9(0000,0010,1,	9),	( 9<<6)|55 },
  {   Binary9(0000,0010,0,	9),	( 9<<6)|59 },
  {   Binary9(0000,0001,1,	9),	( 9<<6)|27 },

  {   Binary9(0000,0001,0,	9),	( 9<<6)|39 },
};

HuffCodeMap SizeLuminanceInitData[ SIZE_YC_INIT_CELLS ] = {
  /* huffman code             longest   nBits   size luma */
  {   Binary3(100,		7),	( 3<<4)|0 },
  {   Binary2(00,		7),	( 2<<4)|1 },
  {   Binary2(01,		7),	( 2<<4)|2 },
  {   Binary3(101,		7),	( 3<<4)|3 },
  {   Binary3(110,		7),	( 3<<4)|4 },
  {   Binary4(1110,		7),	( 4<<4)|5 },
  {   Binary5(1111,0,		7),	( 5<<4)|6 },
  {   Binary6(1111,10,		7),	( 6<<4)|7 },
  {   Binary7(1111,110,		7),	( 7<<4)|8 },
};

HuffCodeMap SizeChrominanceInitData[ SIZE_YC_INIT_CELLS ] = {
  /* huffman code             longest   nBits   size luma */
  {   Binary2(00,		8),	( 2<<4)|0 },
  {   Binary2(01,		8),	( 2<<4)|1 },
  {   Binary2(10,		8),	( 2<<4)|2 },
  {   Binary3(110,		8),	( 3<<4)|3 },
  {   Binary4(1110,		8),	( 4<<4)|4 },
  {   Binary5(1111,0,		8),	( 5<<4)|5 },
  {   Binary6(1111,10,		8),	( 6<<4)|6 },
  {   Binary7(1111,110,		8),	( 7<<4)|7 },
  {   Binary8(1111,1110,	8),	( 8<<4)|8 },
};

/*
 * Note that the nBits includes an extra bit for the sign, except for
 * escape and end_of_block
 */
HuffCodeMap DCTCoeff1InitData[ DCT_COEFF1_INIT_CELLS ] = {
  /* huffman code             longest     nBits    run    level */
  {   Binary2(10,		10),	( 2<<11) | 0<<6  |  0 }, /*end_of_blk*/
  {   Binary2(11,		10),	( 3<<11) | 0<<6  |  1 },
  {   Binary3(011,		10),	( 4<<11) | 1<<6  |  1 },
  {   Binary4(0100,		10),	( 5<<11) | 0<<6  |  2 },
  {   Binary4(0101,		10),	( 5<<11) | 2<<6  |  1 },
  {   Binary5(0010,1,		10),	( 6<<11) | 0<<6  |  3 },
  {   Binary5(0011,1,		10),	( 6<<11) | 3<<6  |  1 },
  {   Binary5(0011,0,		10),	( 6<<11) | 4<<6  |  1 },
  {   Binary6(0001,10,		10),	( 7<<11) | 1<<6  |  2 },
  {   Binary6(0001,11,		10),	( 7<<11) | 5<<6  |  1 },
  {   Binary6(0001,01,		10),	( 7<<11) | 6<<6  |  1 },
  {   Binary6(0001,00,		10),	( 7<<11) | 7<<6  |  1 },
  {   Binary7(0000,110,		10),	( 8<<11) | 0<<6  |  4 },
  {   Binary7(0000,100,		10),	( 8<<11) | 2<<6  |  2 },
  {   Binary7(0000,111,		10),	( 8<<11) | 8<<6  |  1 },
  {   Binary7(0000,101,		10),	( 8<<11) | 9<<6  |  1 },
  {   Binary6(0000,01,		10),	( 6<<11) | 0<<6  |  0 }, /* escape */
  {   Binary8(0010,0110,	10),	( 9<<11) | 0<<6  |  5 },
  {   Binary8(0010,0001,	10),	( 9<<11) | 0<<6  |  6 },
  {   Binary8(0010,0101,	10),	( 9<<11) | 1<<6  |  3 },
  {   Binary8(0010,0100,	10),	( 9<<11) | 3<<6  |  2 },
  {   Binary8(0010,0111,	10),	( 9<<11) |10<<6  |  1 },
  {   Binary8(0010,0011,	10),	( 9<<11) |11<<6  |  1 },
  {   Binary8(0010,0010,	10),	( 9<<11) |12<<6  |  1 },
  {   Binary8(0010,0000,	10),	( 9<<11) |13<<6  |  1 },
  {  Binary10(0000,0010,10,	10),	(11<<11) | 0<<6  |  7 },
  {  Binary10(0000,0011,00,	10),	(11<<11) | 1<<6  |  4 },
  {  Binary10(0000,0010,11,	10),	(11<<11) | 2<<6  |  3 },
  {  Binary10(0000,0011,11,	10),	(11<<11) | 4<<6  |  2 },
  {  Binary10(0000,0010,01,	10),	(11<<11) | 5<<6  |  2 },
  {  Binary10(0000,0011,10,	10),	(11<<11) |14<<6  |  1 },
  {  Binary10(0000,0011,01,	10),	(11<<11) |15<<6  |  1 },
  {  Binary10(0000,0010,00,	10),	(11<<11) |16<<6  |  1 },

/*
 * These two entries flag the rest of the variable length codes.
 * Note that these two eight bit codes are prefixes to the rest
 * the the vlc's.  However, we only keep seven of the bits (7<<11),
 * and run the remaining 9 bits throught the second table, below.
 */
  {   Binary8(0000,0000,	10),	( 7<<11) | 0<<6  |  0 },
  {   Binary8(0000,0001,	10),	( 7<<11) | 0<<6  |  0 },
};

/*
 * Again, note that the nBits includes an extra bit for the sign.
 * The values here are only significant past the 7th bit.
 */
HuffCodeMap DCTCoeff2InitData[ DCT_COEFF2_INIT_CELLS ] = {
  /* huffman code                     longest     nBits     run    level */
  {  Binary12(0000,0001,1101,		16),	( 13<<11) | 0<<6  |  8 },
  {  Binary12(0000,0001,1000,		16),	( 13<<11) | 0<<6  |  9 },
  {  Binary12(0000,0001,0011,		16),	( 13<<11) | 0<<6  | 10 },
  {  Binary12(0000,0001,0000,		16),	( 13<<11) | 0<<6  | 11 },
  {  Binary12(0000,0001,1011,		16),	( 13<<11) | 1<<6  |  5 },
  {  Binary12(0000,0001,0100,		16),	( 13<<11) | 2<<6  |  4 },
  {  Binary12(0000,0001,1100,		16),	( 13<<11) | 3<<6  |  3 },
  {  Binary12(0000,0001,0010,		16),	( 13<<11) | 4<<6  |  3 },
  {  Binary12(0000,0001,1110,		16),	( 13<<11) | 6<<6  |  2 },
  {  Binary12(0000,0001,0101,		16),	( 13<<11) | 7<<6  |  2 },
  {  Binary12(0000,0001,0001,		16),	( 13<<11) | 8<<6  |  2 },
  {  Binary12(0000,0001,1111,		16),	( 13<<11) |17<<6  |  1 },
  {  Binary12(0000,0001,1010,		16),	( 13<<11) |18<<6  |  1 },
  {  Binary12(0000,0001,1001,		16),	( 13<<11) |19<<6  |  1 },
  {  Binary12(0000,0001,0111,		16),	( 13<<11) |20<<6  |  1 },
  {  Binary12(0000,0001,0110,		16),	( 13<<11) |21<<6  |  1 },
  {  Binary13(0000,0000,1101,0,		16),	( 14<<11) | 0<<6  | 12 },
  {  Binary13(0000,0000,1100,1,		16),	( 14<<11) | 0<<6  | 13 },
  {  Binary13(0000,0000,1100,0,		16),	( 14<<11) | 0<<6  | 14 },
  {  Binary13(0000,0000,1011,1,		16),	( 14<<11) | 0<<6  | 15 },
  {  Binary13(0000,0000,1011,0,		16),	( 14<<11) | 1<<6  |  6 },
  {  Binary13(0000,0000,1010,1,		16),	( 14<<11) | 1<<6  |  7 },
  {  Binary13(0000,0000,1010,0,		16),	( 14<<11) | 2<<6  |  5 },
  {  Binary13(0000,0000,1001,1,		16),	( 14<<11) | 3<<6  |  4 },
  {  Binary13(0000,0000,1001,0,		16),	( 14<<11) | 5<<6  |  3 },
  {  Binary13(0000,0000,1000,1,		16),	( 14<<11) | 9<<6  |  2 },
  {  Binary13(0000,0000,1000,0,		16),	( 14<<11) |10<<6  |  2 },
  {  Binary13(0000,0000,1111,1,		16),	( 14<<11) |22<<6  |  1 },
  {  Binary13(0000,0000,1111,0,		16),	( 14<<11) |23<<6  |  1 },
  {  Binary13(0000,0000,1110,1,		16),	( 14<<11) |24<<6  |  1 },
  {  Binary13(0000,0000,1110,0,		16),	( 14<<11) |25<<6  |  1 },
  {  Binary13(0000,0000,1101,1,		16),	( 14<<11) |26<<6  |  1 },

  /* huffman code                     longest     nBits     run    level */
  {  Binary14(0000,0000,0111,11,	16),	( 15<<11) | 0<<6  | 16 },
  {  Binary14(0000,0000,0111,10,	16),	( 15<<11) | 0<<6  | 17 },
  {  Binary14(0000,0000,0111,01,	16),	( 15<<11) | 0<<6  | 18 },
  {  Binary14(0000,0000,0111,00,	16),	( 15<<11) | 0<<6  | 19 },
  {  Binary14(0000,0000,0110,11,	16),	( 15<<11) | 0<<6  | 20 },
  {  Binary14(0000,0000,0110,10,	16),	( 15<<11) | 0<<6  | 21 },
  {  Binary14(0000,0000,0110,01,	16),	( 15<<11) | 0<<6  | 22 },
  {  Binary14(0000,0000,0110,00,	16),	( 15<<11) | 0<<6  | 23 },
  {  Binary14(0000,0000,0101,11,	16),	( 15<<11) | 0<<6  | 24 },
  {  Binary14(0000,0000,0101,10,	16),	( 15<<11) | 0<<6  | 25 },
  {  Binary14(0000,0000,0101,01,	16),	( 15<<11) | 0<<6  | 26 },
  {  Binary14(0000,0000,0101,00,	16),	( 15<<11) | 0<<6  | 27 },
  {  Binary14(0000,0000,0100,11,	16),	( 15<<11) | 0<<6  | 28 },
  {  Binary14(0000,0000,0100,10,	16),	( 15<<11) | 0<<6  | 29 },
  {  Binary14(0000,0000,0100,01,	16),	( 15<<11) | 0<<6  | 30 },
  {  Binary14(0000,0000,0100,00,	16),	( 15<<11) | 0<<6  | 31 },
  {  Binary15(0000,0000,0011,000,	16),	( 16<<11) | 0<<6  | 32 },
  {  Binary15(0000,0000,0010,111,	16),	( 16<<11) | 0<<6  | 33 },
  {  Binary15(0000,0000,0010,110,	16),	( 16<<11) | 0<<6  | 34 },
  {  Binary15(0000,0000,0010,101,	16),	( 16<<11) | 0<<6  | 35 },
  {  Binary15(0000,0000,0010,100,	16),	( 16<<11) | 0<<6  | 36 },
  {  Binary15(0000,0000,0010,011,	16),	( 16<<11) | 0<<6  | 37 },
  {  Binary15(0000,0000,0010,010,	16),	( 16<<11) | 0<<6  | 38 },
  {  Binary15(0000,0000,0010,001,	16),	( 16<<11) | 0<<6  | 39 },
  {  Binary15(0000,0000,0010,000,	16),	( 16<<11) | 0<<6  | 40 },
  {  Binary15(0000,0000,0011,111,	16),	( 16<<11) | 1<<6  |  8 },
  {  Binary15(0000,0000,0011,110,	16),	( 16<<11) | 1<<6  |  9 },
  {  Binary15(0000,0000,0011,101,	16),	( 16<<11) | 1<<6  | 10 },
  {  Binary15(0000,0000,0011,100,	16),	( 16<<11) | 1<<6  | 11 },
  {  Binary15(0000,0000,0011,011,	16),	( 16<<11) | 1<<6  | 12 },
  {  Binary15(0000,0000,0011,010,	16),	( 16<<11) | 1<<6  | 13 },
  {  Binary15(0000,0000,0011,001,	16),	( 16<<11) | 1<<6  | 14 },

  /* huffman code                     longest     nBits     run    level */
  {  Binary16(0000,0000,0001,0011,	16),	( 17<<11) | 1<<6  | 15 },
  {  Binary16(0000,0000,0001,0010,	16),	( 17<<11) | 1<<6  | 16 },
  {  Binary16(0000,0000,0001,0001,	16),	( 17<<11) | 1<<6  | 17 },
  {  Binary16(0000,0000,0001,0000,	16),	( 17<<11) | 1<<6  | 18 },
  {  Binary16(0000,0000,0001,0100,	16),	( 17<<11) | 6<<6  |  3 },
  {  Binary16(0000,0000,0001,1010,	16),	( 17<<11) |11<<6  |  2 },
  {  Binary16(0000,0000,0001,1001,	16),	( 17<<11) |12<<6  |  2 },
  {  Binary16(0000,0000,0001,1000,	16),	( 17<<11) |13<<6  |  2 },
  {  Binary16(0000,0000,0001,0111,	16),	( 17<<11) |14<<6  |  2 },
  {  Binary16(0000,0000,0001,0110,	16),	( 17<<11) |15<<6  |  2 },
  {  Binary16(0000,0000,0001,0101,	16),	( 17<<11) |16<<6  |  2 },
  {  Binary16(0000,0000,0001,1111,	16),	( 17<<11) |27<<6  |  1 },
  {  Binary16(0000,0000,0001,1110,	16),	( 17<<11) |28<<6  |  1 },
  {  Binary16(0000,0000,0001,1101,	16),	( 17<<11) |29<<6  |  1 },
  {  Binary16(0000,0000,0001,1100,	16),	( 17<<11) |30<<6  |  1 },
  {  Binary16(0000,0000,0001,1011,	16),	( 17<<11) |31<<6  |  1 },
};

/**************************************************************************
 *
 * Data below this point is active during operation.  Above this should
 * only be data used during startup.
 ***************************************************************************/

float PelAspectRatio[] = {		/* pixel aspect ratio (height/width) */
    -1.000,	/* forbidden */
    1.0000,	/* VGA, etc. */
    0.6735,
    0.7175,
    0.7615,
    0.8055,
    0.8495,
    0.8995,
    0.9375,	/* CCIR 601, 625 line */
    0.9815,
    1.0255,
    1.0695,
    1.1135,	/* CCIR601, 525 line */
    1.1575,
    1.2015,
    -1.000,	/* reserved */
};

UChar DefaultIntraQMatrix[64] = {
     8, 16, 19, 22, 26, 27, 29, 34,
    16, 16, 22, 24, 27, 29, 34, 37,
    19, 22, 26, 27, 29, 34, 34, 38,
    22, 22, 26, 27, 29, 34, 37, 40,
    22, 26, 27, 29, 32, 35, 40, 48,
    26, 27, 29, 32, 35, 40, 48, 58,
    26, 27, 29, 34, 38, 46, 56, 69,
    27, 29, 35, 38, 46, 56, 69, 83,
};

UChar  DefaultNonIntraQMatrix[64] = {       /* Default delta quantizers */
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
    16, 16, 16, 16, 16, 16, 16, 16,
};

/*
 * given the index implied by the protocol (derived from the run) for a single
 * dct coefficient, this array maps it to an index of the reconstructed
 * dct block.  
 */
UChar  ZigZag[] = {                    /* block indices in output scan order */
         0,
	 1, 8,
        16,  9,  2,
         3, 10, 17, 24,
        32, 25, 18, 11,  4,
         5, 12, 19, 26, 33, 40,
        48, 41, 34, 27, 20, 13,  6,
         7, 14, 21, 28, 35, 42, 49, 56,
            57, 50, 43, 36, 29, 22, 15,
                23, 30, 37, 44, 51, 58,
                    59, 52, 45, 38, 31,
                        39, 46, 53, 60,
                            61, 54, 47,
                                55, 62,
                                    63 };

UChar		*SignedClamp;
HuffLookupInfo	HuffLookup;

#if defined(DEBUG_ENABLED)

static char *Mpeg1VdecDebugStrings[] = {
    "pic alloc",
    "pic position",
    "decode cb",
    "display cb",
};

FlagMacroList Mpeg1VdecDebugMacros = { NULL, 0 };

FlagNames Mpeg1VdecDebugNames = {
    DBI_Vmpegdecomp,
    Mpeg1VdecDebugStrings,
    NumberOfCells(Mpeg1VdecDebugStrings)
};

#else /* DEBUG_ENABLED */

FlagMacroList Mpeg1VdecDebugMacros = { NULL, 0 };
FlagNames Mpeg1VdecDebugNames = { DBI_Vmpegdecomp, NULL, 0 };

#endif /* DEBUG_ENABLED */
