/*
 *	(c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		mpeg1vdec.h - mpeg 1 video decoder (public interface)
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/mpeg1vdec.h,v 1.16 1996/01/03 02:41:31 toddb Exp $
 */
#ifndef _MPEG1VDEC_H
#define _MPEG1VDEC_H

#include "nvrTypes.h"
#include "nvrReason.h"
#include "mpeg1vproto.h"
#include "nvrBitparse.h"
#include "nvrPacket.h"
#include "nvrIdct.h"
#include "imageDpy.h"

typedef enum {
    /*
     * callbacks called only with 
     *   - Mpeg1VDecodeSyntaxOps
     *   - Mpeg1VDecodeSyntaxFormatOps
     *   - Mpeg1VDecodeCallbackOps
     */
    Mpeg1VSeqCb,	/* a video sequence header has been fully parsed */
    Mpeg1VGopCb,	/* a video gop header has been fully parsed */
    Mpeg1VPicCb,	/* a video picture header has been fully parsed */
    Mpeg1VSliceCb,	/* a video slice header has been fully parsed */
    Mpeg1VEndCodeCb,	/* a video end code seen */
    Mpeg1VPacketFree,	/* 1st packet on the queue is about to be freed */
    Mpeg1VMbCb,		/* a macroblock has been fully parsed */

    /*
     * callbacks always called (when requested)
     */
    Mpeg1VNewSeqCb,	/* a new (possibly different) seq header has arrived */
    Mpeg1VDisplayCb,	/* a display-order picture is ready for display */
    Mpeg1VDecodeStartCb,/* a picture is about to be decoded */
    Mpeg1VDecodeEndCb,	/* a picture has been fully decoded */
    Mpeg1VPtsSeekCb,	/* 1st packet on queue has a certain pts */
    
    /* non-callbacks... these request information for the callbacks above */
    Mpeg1VFormat	/* format tokens into readable form (syntax only) */
} Mpeg1VCbType;

typedef enum {
    Mpeg1VSeqMask	  = 1<<Mpeg1VSeqCb,
    Mpeg1VGopMask	  = 1<<Mpeg1VGopCb,
    Mpeg1VPicMask	  = 1<<Mpeg1VPicCb,
    Mpeg1VSliceMask	  = 1<<Mpeg1VSliceCb,
    Mpeg1VEndCodeMask	  = 1<<Mpeg1VEndCodeCb,
    Mpeg1VPacketFreeMask  = 1<<Mpeg1VPacketFree,
    Mpeg1VMbMask	  = 1<<Mpeg1VMbCb,
    Mpeg1VNewSeqMask      = 1<<Mpeg1VNewSeqCb,
    Mpeg1VDisplayMask	  = 1<<Mpeg1VDisplayCb,
    Mpeg1VDecodeStartMask = 1<<Mpeg1VDecodeStartCb,
    Mpeg1VDecodeEndMask	  = 1<<Mpeg1VDecodeEndCb,
    Mpeg1VPtsSeekMask	  = 1<<Mpeg1VPtsSeekCb,
    Mpeg1VFormatMask	  = 1<<Mpeg1VFormat
} Mpeg1VCbMask;

typedef enum {
    Mpeg1VNoFormatting,		/* no formatted items are produced */
    Mpeg1VSeqLevel,		/* sequence and gop headers */
    Mpeg1VPicSummaryLevel,	/* sequence, gop detail, pic summary */
    Mpeg1VPicLevel,		/* sequence, gop, pic detail */
    Mpeg1VSliceLevel,		/* picture detail, including slice headers */
    Mpeg1VMbSummaryLevel,	/* slice detail, macroblock summary */
    Mpeg1VMacroblockLevel,	/* slice detail, including macroblock info */
    Mpeg1VBlockLevel,		/* block detail */
    Mpeg1VCoeffLevel,		/* run lengths and coefficient levels */
    Mpeg1VNLevels		/* number of levels in this list */
} Mpeg1VFmtLevel;

typedef struct _mpeg1VideoTokens {
    Mpeg1VideoHdr seqHdr;	/* valid for Mpeg1VSeqCb: seq header */
    Mpeg1PictureHdr picHdr;	/* valid for Mpeg1VPicCb: picture header */
    Mpeg1GopHdr gopHdr;		/* valid for Mpeg1VGopCb: picture header */
} Mpeg1VideoTokens, *Mpeg1VideoTokensPtr;

/*
 * This (blind) structure describes the functions that can initialize the
 * video decoder for certain functions.
 */
typedef struct _mpeg1VOpsInit *Mpeg1VOpsInitPtr;

/*
 * This structure is passed to the caller when one of the tokens
 * has been fully parsed and the callback has been selected.
 */
typedef struct _Mpeg1VCbInfo {
    Mpeg1VideoTokensPtr pTokens;	/* tokens for callback */
    PacketDataPtr packetDown;		/* packet from below mpeg1 layer */
    ULong tokenOffset;			/* stream offset for start of token */
    ULong tokenFileOffset;		/* file offset for start of token */
    void *cbData;			/* user-supplied cb data */
    FormattedItemPtr itemList;		/* list of formatted items for token */
    IdctParamPtr idctp;			/* pointer to dct structures */
    YcImagePtr image;			/* picture for Mpeg1VDisplayCb */
    long seekTimecode;			/* pts for Mpeg1VPtsSeekCb */
} Mpeg1VCbInfo, *Mpeg1VCbInfoPtr;

typedef void (*Mpeg1VTokenCb)(PacketLayerPtr, Mpeg1VCbType, Mpeg1VCbInfoPtr);

/*
 * This structure is passed to OpenMpeg1VideoStream().  All items must
 * be defined.  They can be reconfigured at runtime with the Mpeg1VideoRunConf
 * structure and a call to Mpeg1VRunConfigure().  See below.
 */
typedef struct _mpeg1VideoConf {
    Mpeg1VOpsInitPtr opsInit;		/* operational functions */
    IdctMethodPtr idctMethod;		/* idct method */
    void (*warning)(char *fmt, ...);	/* warning function */
    Mpeg1VTokenCb tokenCb;		/* callback function */
    void *cbData;			/* user callback data */
    ULong cbSelect;			/* set of Mpeg1VCbMask bits */
    Mpeg1VFmtLevel  formatLevel;	/* syntax only: level of formatting */
    PacketLayerPtr media;		/* defined only for Open... */
} Mpeg1VideoConf, *Mpeg1VideoConfPtr;

CExternal PacketLayerPtr OpenMpeg1VideoStream(
    Mpeg1VideoConfPtr,
    InternalReasonPtr *pReason
);

/*
 * This structure is passed to Mpeg1VRunConfigure().  Which items are
 * defined is specified with the confMasks parameter.
 */
typedef struct _mpeg1VideoRunConf {
    Mpeg1VideoConf core;
    float weight;			/* target fraction of frame rate */
} Mpeg1VideoRunConf, *Mpeg1VideoRunConfPtr;

/*
 * These masks are used to determine which items changed in the
 * Mpeg1VRunConfigure() call.  Note that Mpeg1VConfMediaMask is
 * present for completeness but cannot be specified to Mpeg1VConfigure().
 */
typedef enum {
    Mpeg1VConfMediaMask     = 0x001,
    Mpeg1VConfOpsInitMask   = 0x002,
    Mpeg1VConfIdctMethodMask= 0x004,
    Mpeg1VConfWarningMask   = 0x008,
    Mpeg1VConfTokenCbMask   = 0x010,
    Mpeg1VConfCbDataMask    = 0x028,
    Mpeg1VConfCbSelectMask  = 0x040,
    Mpeg1VConfFormatMask    = 0x080,
    Mpeg1VConfWeightMask    = 0x100,

    Mpeg1VConfOpenMask      = 0x0FF	/* excludes Weight */
} Mpeg1VConfigMask;

CExternal InternalReasonPtr Mpeg1VRunConfigure(
    PacketLayerPtr,
    Mpeg1VideoRunConfPtr,
    long confMasks
);

CExternal void Mpeg1VFreeFormatItems(
    FormattedItemPtr
);

extern Mpeg1VOpsInitPtr Mpeg1VDecodeOps;		/* decode the stream */
extern Mpeg1VOpsInitPtr Mpeg1VDecodeSyntaxOps;		/* and check syntax */
extern Mpeg1VOpsInitPtr Mpeg1VDecodeSyntaxFormatOps;	/* and format bits */
extern Mpeg1VOpsInitPtr Mpeg1VDecodeCallbackOps;	/* or do callbacks */

#endif /* _MPEG1VDEC_H */
