/*
 *	(c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		mpeg1vdecI.h - mpeg 1 video decoder (internal)
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/mpeg1vdecI.h,v 1.19 1996/01/03 02:41:56 toddb Exp $
 */
#ifndef _MPEG1VDECI_H
#define _MPEG1VDECI_H

#include "mpeg1vdec.h"
#include "nvrPacketI.h"
#include "nvrDebug.h"

typedef struct _mpeg1VDecPriv Mpeg1VDecPriv, *Mpeg1VDecPrivPtr;
typedef InternalReasonPtr (*TokenDispatch)(Mpeg1VDecPrivPtr, UChar *pSrc);
typedef InternalReasonPtr (*SliceInit)(Mpeg1VDecPrivPtr, UChar *pDest);
typedef InternalReasonPtr (*MacroblockCopier)(Mpeg1VDecPrivPtr, int nMb);

/*
 * Debug bit indices
 */
typedef enum {
    VMD_PicAlloc = DBI_Vmpegdecomp,
    VMD_PicPosition,
    VMD_DecodeCb,
    VMD_DisplayCb
} DebugBitIndices;

enum {
    kSearchToken = 0x8000
};

typedef enum {
    kTokenTypeMask      = 0x003,
    
    kTokenStartCode     = 0x000,    /* scan for start code */
    kTokenLenCopy       = 0x001,    /* size known, copy the token */
    kTokenScan          = 0x002,    /* size unknown, scan to next start code */
    kTokenCopy          = 0x003,    /* size unknonw, copy to next start code */
    
    kSeqSearch          = 0x000<<2, /* look for a sequence start code */
    kGopSearch          = 0x001<<2, /* look for a gop start code */
    kBPicSearch         = 0x002<<2, /* look for a B picture or better */
    kIPPicSearch        = 0x003<<2, /* look for a I or P picture or Gop start */
    kSeqHeader          =(0x004<<2) | kTokenLenCopy,
       kPreGopSearch    = 0x005<<2, /* look for ext, user data or gop start */
       kSeqExtData      =(0x006<<2) | kTokenScan,
       kCopySeqExtData  =(0x006<<2) | kTokenCopy,
       kSeqUserData     =(0x007<<2) | kTokenScan,
       kCopySeqUserData =(0x007<<2) | kTokenCopy,
       kCopySeqIQM      =(0x008<<2) | kTokenLenCopy,
       kCopySeqNIQM     =(0x009<<2) | kTokenLenCopy,
    kGopHeader          =(0x00a<<2) | kTokenLenCopy,
       kPrePicSearch    = 0x00b<<2, /* look for ext, user data or pic start */
       kGopExtData      =(0x00c<<2) | kTokenScan,
       kCopyGopExtData  =(0x00c<<2) | kTokenCopy,
       kGopUserData     =(0x00d<<2) | kTokenScan,
       kCopyGopUserData =(0x00d<<2) | kTokenCopy,
    kPicHeader          =(0x00e<<2) | kTokenLenCopy,
       kPreSliceSearch  = 0x00f<<2, /* look for ext, user data or slice start */
       kPicExtData      =(0x010<<2) | kTokenScan,
       kCopyPicExtData  =(0x010<<2) | kTokenCopy,
       kPicUserData     =(0x011<<2) | kTokenScan,
       kCopyPicUserData =(0x011<<2) | kTokenCopy,
    kCopySliceData      =(0x012<<2) | kTokenCopy, /* copy the slice data */
    kScanSliceData      =(0x012<<2) | kTokenScan /* scan the slice data */
} Mpeg1VToken;

#define TokenIsCopied(_vdec)					\
    ((_vdec->parse.token & kTokenTypeMask) == kTokenCopy)
#define TokenIsScanned(_vdec)					\
    ((_vdec->parse.token & kTokenTypeMask) == kTokenScan)

/*
 * This structure describes the function set that defines the operation
 * of the video decoder.  The function sets that can be used are
 * declared in mpeg1vdec.h.  One for straight decoding, one for syntax checking
 * and one for syntax checking and formatting the bit-level parsing, etc.
 */
typedef struct _mpeg1VOpFuncs {
    PacketReasonFunc startCodeProcess;	/* start-code level processing */
    PacketReasonFunc sliceProcess;	/* slice level processing */
    PacketReasonFunc dctProcess;	/* dct level processing */
    TokenDispatch seqDispatch;		/* look for next sequence */
    TokenDispatch gopDispatch;		/* look for next GOP */
    TokenDispatch picDispatch;		/* look for next picture */
    TokenDispatch sliceDispatch;	/* look for next slice */
    SliceInit sliceInit;		/* slice init */
} Mpeg1VOpFuncs, *Mpeg1VOpFuncsPtr;

/*
 * This is a private version of the YcImagePtr structure seen by
 * the public.  We use it to track references and keep track of image
 * size changes.
 */
typedef struct _ycMpegImage YcMpegImage, *YcMpegImagePtr;
struct _ycMpegImage {
    YcImage pub;
    Mpeg1VDecPrivPtr vdec;	/* back pointer */
    short refCount;		/* reference count for this image */
    boolean delivered;		/* whether it has been delivered or not */
    int validate;		/* used to validate against dimensions, etc */
};

/*************************************************************************
 *
 * Decoder's picture structure
 *
 *************************************************************************/
#define NFreePictures	3	/* past, future and current */

typedef struct _picinfo PictureInfo, *PictureInfoPtr;
struct _picinfo {
    UChar	type;		/* Picture coding type (B_?_PIC) */
    UShort	timeRef;	/* temporal reference */
    UShort	vbvDelay;	/* initial occupancy of the decoder's buffer */
    UChar	fullPel[ 2 ];	/* if true, then vectors are integer offsets */
    UChar	mvSize[ 2 ];	/* # of bits for motion vectors */
    short	maxMotion[ 2 ];	/* used to clamp motion vectors */
    short	minMotion[ 2 ];	/* used to clamp motion vectors */
    short	pastMotion[ 4 ];/* running values: previous motion value */
    UChar	backwardSize;	/* size in bits of backward motion numbers */
    boolean	dropped;	/* true if this picture has been dropped */
    YcMpegImagePtr image;	/* pointer to the allocated yc image */
}; /* typedef: PictureInfo, *PictureInfoPtr */

/*
 * This structure contains a byte aligned pointer to the data in a single
 * slice.  pEnd is pointer to the end of the buffer for sanity sake.
 * Since we may return from a routine before we completely parse all
 * of the data, nBits contains the number of bits not yet parsed in
 * the byte pointed to pBuf.
 */
typedef struct _sliceInfo {
    BitParseInfo bitParse;
    UChar *pEnd;		/* pointer after last byte of data */
    UChar *pBufferBase;		/* start of slice data */
    short nBytes;		/* length of slice data in bytes */
    UChar vertPos;		/* from the slice start code */
    UChar mbType;		/* contains 5 bits about current macroblock */
    short mbAddr;		/* index to current macro block */
    short pastIntraMbAddr;	/* previous intra macroblock */
    short pastMbAddr;		/* previous macroblock (intra or delta) */
    short dcYPast;		/* dctRecon[0] of previous intra-coded Y */
    short dcCrPast;		/* dctRecon[0] of previous intra-coded Cr */
    short dcCbPast;		/* dctRecon[0] of previous intra-coded Cb */
    UChar codedBlockPattern;	/* current block pattern */
    UChar qScale;		/* current quantizer scale */
    IdctParamPtr idct;		/* dct structure */
    DctListInfoPtr curDctList;	/* current dct list from 'idct' */
} SliceInfo, *SliceInfoPtr;

#define CopyBufferInc	2048

#define SetTokenMacro(vdec, _token, _dispatch)			\
    (vdec)->parse.token = _token;				\
    (vdec)->parse.dispatch = _dispatch;

#define ScanTokenToCopyToken(_token)				\
    (Mpeg1VToken) ((_token & ~kTokenTypeMask) | kTokenCopy)

/*
 * This is the private version of the public packet interface for the
 * mpeg1 video decoder.
 */
struct _mpeg1VDecPriv {
    PacketLayer pub;		/* public data */
    
    struct {
	short width;		/* actual width */
	short height;		/* actual height */
        short mbWidth;		/* width in macroblocks */
        short mbHeight;		/* height in macroblocks */
        short lastMacroBlock;	/* the last possible macroblock address */
	short pictureRate;	/* frame rate index from sequence header */
	SampleType sample;	/* 4:2:0 or 4:2:2 */
	float weight;		/* configured weight */
	float dropRate;		/* 1.0 / weight */
	float keepCounter;	/* used with dropRate to pick frames to drop */
	boolean dropFrames;	/* true if we've been configured to drop */
    } decode;
    
    PictureInfoPtr currentPict;	/* I, P or B picture */
    PictureInfoPtr futurePict;	/* I or P picture */
    PictureInfoPtr pastPict;	/* I or P picture */

    PictureInfoPtr picArray;	/* array of pics (not allocated separately) */
    YcMpegImagePtr ycFree;	/* list of allocated, unused images */
    int imageValidate;		/* unique validation per picture size, etc */

    SliceInfo slice;		/* current slice parsing info */
    
    struct {
        int offset;		/* current offset into source packet */
    } packet;
    
    struct {
        UChar *pToken;		/* pointer into packet or tokens.something */
        UChar *copyBuf;		/* allocated buffer */
        UChar *copyBufEnd;	/* end of copyBuf */
        ULong bufSize;
        short offset;		/* current source token offset */
        short length;		/* token length (if known) */
        ULong startCode;       	/* current start code */
        ULong timeout;		/* time to give up parsing */
        MpegSmuxCodes nextStartCode; /* start code following current token */
	boolean seqHeaderSeen;	/* a sequence header has been seen */
	boolean timeCodes;	/* timecodes valid on packets delivered from
				   below? */
	double timecode;	/* calculated timecode of each picture */
	double pktTimecode;	/* timecode at last change:
				   packet timecode for system stream
				   gop timecode for elem video */
	float msPerPicture;	/* 1000.0/framerate */
	boolean newTimeRef;	/* new packet PTS encountered */
	ULong firstPic;		/* time ref of first access unit in packet */

        /*
         * These two should be assigned with the
         * SetTokenMacro() macro.
         */
        Mpeg1VToken token;	/* current token being parsed */
        TokenDispatch dispatch;	/* called after token fully parsed */
    } parse;

    struct {
	PacketDataPtr queue;	/* for saving packets during seek op */
        boolean found;		/* found correct starting point for decode */
	ULong startCode;	/* startcode of starting point */
    } seek;	
    
    Mpeg1VideoTokens tokens;
    

    void (*warning)(char *fmt, ...);

    Mpeg1VOpFuncs ops;
    
    /*
     * Variables used to do user callbacks
     */
    struct {
	Mpeg1VCbInfo arg;
	void (*tokenCb)(PacketLayerPtr, Mpeg1VCbType, Mpeg1VCbInfoPtr);
	ULong select;		/* selection of (1<<Mpeg1VCbType) bits */
	ULong lastPacketEnd;	/* offset of the end of the previous packet */
	ULong lastPacketFileEnd;/* file offset of end of previous packet */
        long nextTokenFileOffset;
        long nextTokenOffset;
        FormattedItemPtr lastItem;/* last formatted item in arg.itemList */
        Mpeg1VFmtLevel curLevel;/* current relative item level */
        Mpeg1VFmtLevel maxLevel;/* don't format beyond this item level */
        boolean wantNextSeqHdr; /* want callback for next sequence header */
    } cb;

    /*
     * Variables used in syntax checking
     */
    struct {
        short firstMbInSlice;	/* first macroblock in current slice */
        short prevMb;		/* last parsed macroblock */
        boolean sliceStart;	/* true for first mb in slice */
	short minTimeRef;	/* min picture time ref for gop */
	short curTimeRef;	/* current picture time ref */
	short maxTimeRef;	/* max picture time ref for gop */
	boolean gopStart;	/* true for first picture of gop */
    } syntax;
}; /* typedef: Mpeg1VDecPriv, *Mpeg1VDecPrivPtr */

typedef struct _mpeg1VOpsInit Mpeg1VOpsInit;
struct _mpeg1VOpsInit {
    void (*init)(Mpeg1VDecPrivPtr);
};

#include "mpeg1vsyntax.h"
#include "mpeg1vformat.h"
#include "mpeg1vcb.h"

/*************************************************************************
 *
 * Internal Utility functions
 *
 *************************************************************************/
CExternal InternalReasonPtr InitMpeg1VData(void);
CExternal void Mpeg1VDeliverImage(Mpeg1VDecPrivPtr, YcMpegImagePtr);
CExternal YcMpegImagePtr Mpeg1VAllocYcImage(Mpeg1VDecPrivPtr,
					    InternalReasonPtr *);

/*************************************************************************
 *
 * slice op init functions
 *
 *************************************************************************/
CExternal void Mpeg1VInitSliceOps(Mpeg1VDecPrivPtr);
CExternal void Mpeg1VInitSliceOpsSyntax(Mpeg1VDecPrivPtr);
CExternal void Mpeg1VInitSliceNoop(Mpeg1VDecPrivPtr);

/*************************************************************************
 *
 * block-level functions
 *
 *************************************************************************/
CExternal InternalReasonPtr Mpeg1VDctProcess(PacketLayerPtr);

/*************************************************************************
 *
 * Internal Error functions
 *
 *************************************************************************/
CExternal InternalReasonPtr TokenTooLarge(int available, Mpeg1VToken);
CExternal InternalReasonPtr MissingTooSmall(int missing, Mpeg1VToken);
CExternal InternalReasonPtr UnknownToken(Mpeg1VToken);

CExternal InternalReasonPtr UnexpectedVideoStartCode(
    Mpeg1VDecPrivPtr,
    UChar *pSrc,
    ULong expected,
    ULong got
);

/*************************************************************************
 *
 * global data symbols
 *
 *************************************************************************/
extern char IPBDchars[5];
extern UChar ZigZag[64];
extern char *MpegBlockNames[];

#endif /* _MPEG1VDECI_H */
