/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1vdecode.c - start-code level decoding for mpeg video
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1vdecode.c,v 1.25 1996/01/03 02:42:23 toddb Exp $
 */

#include "mpeg1vdecI.h"
#include "nvrAlloc.h"

static InternalReasonPtr
StretchDest(Mpeg1VDecPrivPtr vdec, int tokenLength)
{
    UChar *newCopyBuf;
    int allocSize = vdec->parse.bufSize + CopyBufferInc;
    
    while (allocSize < tokenLength)
	allocSize += CopyBufferInc;

    newCopyBuf = (UChar *)InsecureAlloc(allocSize);
    if (newCopyBuf == NULL)
        return( AllocReason("temp MPEG video token", allocSize) );
        
    if (vdec->parse.copyBuf) {
        if (vdec->parse.offset)
            memcpy(newCopyBuf, vdec->parse.copyBuf, vdec->parse.offset);
        free( (char *)vdec->parse.copyBuf );
    }
    vdec->parse.copyBuf = newCopyBuf;
    vdec->parse.bufSize = allocSize;
    vdec->parse.copyBufEnd = newCopyBuf + allocSize;

    return(NULL);
}

static InternalReasonPtr
CopyToken(Mpeg1VDecPrivPtr vdec, UChar *pSrc, int fragmentLength)
{
    InternalReasonPtr reason;
    int tokenLength = fragmentLength + vdec->parse.offset;

    if (tokenLength > vdec->parse.bufSize) {
	if (NULL != (reason = StretchDest(vdec, tokenLength)))
	    return(reason);
    }
    vdec->parse.pToken = vdec->parse.copyBuf;
    memcpy((char *)vdec->parse.copyBuf + vdec->parse.offset,
	   (char *)pSrc,
	   fragmentLength);
    vdec->parse.offset += fragmentLength;

    return (NULL);
}

static int
FindNextStart(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    ULong startCode = vdec->parse.startCode;
    PacketDataPtr packet = vdec->pub.queue;
    UChar *pSrcEnd = packet->buffer + packet->dataBytes;
    UChar *pTokenEnd;
    ULong prefix;

    pTokenEnd = pSrc;
    while (pTokenEnd < pSrcEnd) {
	prefix = startCode << 8;
	startCode = prefix | *pTokenEnd++;
	if (prefix == (StartCodePrefix << 8)) {
	    vdec->parse.nextStartCode = (MpegSmuxCodes) startCode;
	    break;
	}
    }

    vdec->parse.startCode = startCode;
    return( pTokenEnd - pSrc );
}

static InternalReasonPtr
Mpeg1VideoProcess(PacketLayerPtr vdecPub)
{
    Mpeg1VDecPrivPtr vdec = (Mpeg1VDecPrivPtr)vdecPub;
    InternalReasonPtr reason = NULL;
    UChar *pSrc, *pSrcEnd;
    int tokenLength, available;
    PacketDataPtr packet;

    pSrc = pSrcEnd = NULL;
    packet = NULL;
    
    for (;;) {
	if (packet == NULL)
	    packet = vdec->pub.queue;
	else {
	    FreeCallbackPacket(vdec);
	    vdec->pub.queue = packet->next;
	    vdec->packet.offset = 0;
	    (*packet->pLayer->ops.free)(packet);
	    packet = vdec->pub.queue;
	    if (packet == NULL)
		goto parsingComplete;
	}
	/*
	 * If the last byte of the start code for the current slice
	 * fell in the last packet and the first byte of the slice
	 * data falls in this packet, then we need to update pToken,
	 * because it still points at the last byte of the last
	 * packet.
	 */
	if (vdec->packet.offset == 0 && TokenIsScanned(vdec))
	    vdec->parse.pToken = packet->buffer;
        
        pSrc = packet->buffer + vdec->packet.offset;
        pSrcEnd = packet->buffer + packet->dataBytes;
        if (pSrc >= pSrcEnd)
	    continue;

	if (vdec->parse.timeCodes
	 && vdec->parse.pktTimecode != packet->timecode) {
	    vdec->parse.pktTimecode = packet->timecode;
	    vdec->parse.newTimeRef = True;
	    vdec->parse.firstPic = 0;
	}

	while (pSrc < pSrcEnd) {
	    switch (vdec->parse.token & kTokenTypeMask) {
	    case kTokenCopy:
	    case kTokenStartCode:
	    case kTokenScan:
		tokenLength = FindNextStart(vdec, pSrc);

		if ((vdec->parse.startCode >> 8) != StartCodePrefix
		 && TokenIsScanned(vdec))
		    vdec->parse.token = ScanTokenToCopyToken(vdec->parse.token);

		if (TokenIsCopied(vdec)) {
		    reason = CopyToken(vdec, pSrc, tokenLength);
		    if (reason)
			goto parsingComplete;
		}

		pSrc += tokenLength;

		if ((vdec->parse.startCode >> 8) == StartCodePrefix) {
		    SetCallbackTokenStart(vdec, pSrc);
		    reason = (*vdec->parse.dispatch)(vdec, pSrc);
		    if (reason || vdec->pub.ops.process != Mpeg1VideoProcess)
			goto parsingComplete;
		}
		break;
	    
	    case kTokenLenCopy:
		tokenLength = vdec->parse.length - vdec->parse.offset;
		available = pSrcEnd - pSrc;
		if (available > tokenLength)
		    available = tokenLength;
		memcpy((char *)vdec->parse.pToken + vdec->parse.offset,
		       (char *)pSrc,
		       available);
		vdec->parse.offset += available;
		pSrc += available;
		if (vdec->parse.offset >= vdec->parse.length) {
		    SetCallbackTokenStart(vdec, pSrc);
		    reason = (*vdec->parse.dispatch)(vdec, pSrc);
		    if (reason || vdec->pub.ops.process != Mpeg1VideoProcess)
			goto parsingComplete;
		}
		break;
	    
	    default:	/* not possible */
		break;
	    }
        }
    }

parsingComplete:
    if (reason) {
        SetTokenMacro(vdec, kSeqSearch, vdec->ops.seqDispatch);
        vdec->parse.startCode = ~0;
    }
    if (packet) {
        vdec->packet.offset = pSrc - packet->buffer;
    }

    return(reason);
}

static InternalReasonPtr
DispatchSliceData(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    InternalReasonPtr irp;
    UChar *pDest;
    
    CallCallback(vdec, Mpeg1VSliceCb);

    switch (vdec->parse.nextStartCode) {
    case SeqenceEndCode:
	CallCallback(vdec, Mpeg1VEndCodeCb);
	goto processCurrentSlice;

    default:
        if (NULL != (irp = CheckSliceAddr(vdec)))
            return(irp);
        /* fall through ... */
        
    case PictureStartCode:
    case GroupStartCode:
    case SeqHeaderStartCode:
    processCurrentSlice:
        vdec->pub.ops.process = vdec->ops.sliceProcess;
        if (TokenIsScanned(vdec)) {
	    pDest = pSrc;
	} else {
            pDest = vdec->parse.pToken + vdec->parse.offset;
            vdec->parse.token = kScanSliceData;	/* reset to scanned */
	}

        if (NULL != (irp = (*vdec->ops.sliceInit)(vdec, pDest)))
	    return(irp);
        break;
    }
    vdec->parse.pToken = pSrc;
    vdec->parse.offset = 0;	/* don't include the start code */

    return(NULL);
}

static void
StartSliceDispatch(Mpeg1VDecPrivPtr vdec)
{
    vdec->parse.offset = 0;	/* don't include the start code */
    /*
     * This sets the vertical position for the next call to Mpeg1SliceInit()
     */
    vdec->slice.vertPos = vdec->parse.nextStartCode;
    SetTokenMacro(vdec, kScanSliceData, DispatchSliceData);
}

static InternalReasonPtr
DispatchPreSlice(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    InternalReasonPtr irp;

    switch (vdec->parse.nextStartCode) {
    case ExtensionStartCode: 
        if (NULL != (irp = CheckPicExtStartCode(vdec, pSrc)))
            return(irp);
        SetTokenMacro(vdec, kPicExtData, DispatchPreSlice);
        vdec->parse.offset = 4;
        vdec->parse.pToken = pSrc;
        break;

    case UserDataStartCode: 
        SetTokenMacro(vdec, kPicUserData, DispatchPreSlice);
        vdec->parse.offset = 4;
        break;
    
    default:
        StartSliceDispatch(vdec);
        break;
    }
    vdec->parse.pToken = pSrc;

    return(NULL);
}

/*
 * Note that we ignore the extra picture info
 */
static InternalReasonPtr
DispatchPicHdr(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    PictureInfoPtr lastCurrent, newCurrent, tmpPic;
    int forwardSize, backwardSize;
    InternalReasonPtr irp;
    float keepCounter;
    boolean dropped;
    IdctParamPtr idctp;
    YcMpegImagePtr image;
    
    /*
     * There may be P- and B-specific data that still needs to be
     * fetched...
     */
    if (vdec->tokens.picHdr.type == Mpeg1_P_Pic
     || vdec->tokens.picHdr.type == Mpeg1_B_Pic) {
        if (vdec->parse.offset == Mpeg1PictureHdrLen) {
            vdec->parse.length = Mpeg1PB_PictureHdrLen;
            return(NULL);
        }
    }
    if (NULL != (irp = CheckCurPicHeader(vdec)))
	return(irp);
    FormatPicHdr(vdec);
    CallCallback(vdec, Mpeg1VPicCb);

    lastCurrent = vdec->currentPict;
    if (lastCurrent->type == Mpeg1_B_Pic) {
        /*
         * If the previous current picture is a B picture, then it can be
         * reused for whatever the new picture is.
         */
        newCurrent = lastCurrent;
        
	if (NULL != (image = lastCurrent->image)) {
	    Mpeg1VDeliverImage(vdec, image);
	    (*image->pub.free)(&image->pub);
	    lastCurrent->image = NULL;
	}
    } else {
        /*
         * The previous current picture is a P or and I, so
         * it will new become the new future picture.  We can
	 * accomplish this by simply exchanging the current
	 * and future picture.
         * new current <-- past <-- future <-- prev current
         */
        newCurrent = vdec->futurePict;
        vdec->futurePict = lastCurrent;

        /*
         * If the new current picture has a time reference greater
         * than the previous current picture, then the previous current
	 * picture can be delivered.
         */
        if (lastCurrent
         && lastCurrent->timeRef < vdec->tokens.picHdr.timeRef
	 && NULL != (image = lastCurrent->image))
	    Mpeg1VDeliverImage(vdec, image);
    }

    /*
     * Adjust the frame "keep" counter here.  It is only used
     * when frames are dropped, but this is cheaper than a test.
     * It is decremented here to reflect input frames.  It is incremented
     * below by decode->dropRate when a frame goes out.
     */
    keepCounter = vdec->decode.keepCounter - 1;

    /*
     * finally, if the new picture is a P or an I, then move the future
     * picture to the past.  Since, at this point, the future picture
     * is not referenced, put the past picture in the future pointer.
     * The net effect is:
     *		past <===> future
     */
    if (vdec->tokens.picHdr.type != Mpeg1_B_Pic) {
	/*
	 * Now that the past picture is going to be replaced by the
	 * future picture, we can free it.
	 */
	if (NULL != (image = vdec->pastPict->image)) {
	    (*image->pub.free)(&image->pub);
	    vdec->pastPict->image = NULL;
	}

	/*
	 * The future picture is about to become the past picture 
	 * so deliver it now.
	 */
	if (NULL != (image = vdec->futurePict->image)) {
	    Mpeg1VDeliverImage(vdec, image);
	}

	tmpPic = vdec->pastPict;
	vdec->pastPict = vdec->futurePict;
	vdec->futurePict = tmpPic;

	/*
	 * If the keepCounter is positive when a P or I frame arrives,
	 * we will still process the frame as long as it is not greater than
	 * decode->dropRate.  Twice that for I frames.
	 */
	if (keepCounter >= vdec->decode.dropRate) {
	    if (vdec->tokens.picHdr.type == Mpeg1_I_Pic
	     && keepCounter < (float)2.0 * vdec->decode.dropRate) {
		dropped = False;
	    } else {
		dropped = True;
	    }
	} else {
	    /*
	     * We may be forced to drop it if the previous I/P frame is
	     * missing.
	     */
	    if (vdec->tokens.picHdr.type == Mpeg1_P_Pic
	     && vdec->pastPict->dropped) {
		dropped = True;
	    } else {
		dropped = False;
	    }
	}
    } else { /* this is a B picture */
	/*
	 * If the keepCounter is positive when a B frame arrives,
	 * it is dropped.
	 */
	dropped = ((vdec->pastPict->dropped + vdec->futurePict->dropped)
		|| keepCounter > (float)0.0);
    }

    DebugTask(VMD_PicPosition,
	Msg("past: %c%2d  current: %c%2d  future: %c%2d\n",
	    IPBDchars[ vdec->pastPict->type ],
	    vdec->pastPict->timeRef,
	    IPBDchars[ vdec->tokens.picHdr.type ],
	    vdec->tokens.picHdr.timeRef,
	    IPBDchars[ vdec->futurePict->type ],
	    vdec->futurePict->timeRef);
    )

    vdec->currentPict = newCurrent;
    newCurrent->type = vdec->tokens.picHdr.type;
    newCurrent->timeRef = vdec->tokens.picHdr.timeRef;
    newCurrent->vbvDelay = vdec->tokens.picHdr.vbvdelay;
    newCurrent->dropped = dropped;
    if (! dropped)
	keepCounter += vdec->decode.dropRate;
    vdec->decode.keepCounter = keepCounter;

    if (vdec->parse.timeCodes) {
	double timecode;
	long tcDiff;

	if (vdec->parse.newTimeRef) {
	    vdec->parse.newTimeRef = False;
	    vdec->parse.firstPic = vdec->tokens.picHdr.timeRef;
	    timecode = vdec->parse.pktTimecode;
	} else {
	    tcDiff = vdec->tokens.picHdr.timeRef - vdec->parse.firstPic;
	    timecode = vdec->parse.pktTimecode
		     + tcDiff * vdec->parse.msPerPicture;
	}
	vdec->parse.timecode = timecode;
    } else {
	vdec->parse.timecode = vdec->parse.pktTimecode
		     + vdec->tokens.picHdr.timeRef * vdec->parse.msPerPicture;
    }

    if (dropped) {
	/*
	 * If the current picture must be dropped,
	 * we take a different route for token dispatching
	 * because we are going to skip to some picture farther
	 * down the input stream.
	 */
	SetTokenMacro(vdec, kBPicSearch, vdec->ops.picDispatch);
	return(NULL);
    }
    SetTokenMacro(vdec, kPreSliceSearch, DispatchPreSlice);

    /*
     * allocate picture storage and give it the current schedule
     */
    image = newCurrent->image = Mpeg1VAllocYcImage(vdec, &irp);
    if (image == NULL)
	return(irp);
    image->pub.frame = vdec->tokens.picHdr.timeRef;
    image->delivered = False;
    image->pub.sched = vdec->parse.timecode;
    
    
    /*
     * We do this regardless of the picture type (even though it
     * may be wrong.
     */
    newCurrent->fullPel[ kIndexFwd ] = vdec->tokens.picHdr.fullPelFwd;
    newCurrent->fullPel[ kIndexBak ] = vdec->tokens.picHdr.fullPelBak;

    forwardSize = ForwardSizeMacro(&vdec->tokens.picHdr);
    backwardSize = vdec->tokens.picHdr.bakFCode - 1;
    
    /*
     * pre-calculate the various values used during motion vector calculation
     */
    {
#	define MaxMotion(_size)		((1 << ((_size) + 4)) - 1)
#	define MinMotion(_size)		(-(MaxMotion(_size)+1))

        UChar *mvSizes = newCurrent->mvSize;

        mvSizes[ kIndexFwd ] = forwardSize;
        mvSizes[ kIndexBak ] = backwardSize;

        newCurrent->maxMotion[ kIndexFwd ] = MaxMotion(forwardSize);
        newCurrent->maxMotion[ kIndexBak ] = MaxMotion(backwardSize);
        newCurrent->minMotion[ kIndexFwd ] = MinMotion(forwardSize);
        newCurrent->minMotion[ kIndexBak ] = MinMotion(backwardSize);
    }

    /*
     * Now allocate one arena from the free list for the next round
     * of fun in the slice processor.  Note that this should never
     * fail because there is one allocated at dct creation time;
     * also, after blocks have been processed, everything ends up
     * on the free list.
     */
    irp = NULL;
    idctp = vdec->slice.idct;
    vdec->slice.curDctList = AllocDctArena(idctp, &irp);
    (*idctp->newPicture)(idctp,
                         vdec->currentPict->fullPel,
                         &vdec->currentPict->image->pub,
                         &vdec->pastPict->image->pub,
                         &vdec->futurePict->image->pub);

    return(irp);
}

static void
StartPicDispatch(Mpeg1VDecPrivPtr vdec)
{
    SetTokenMacro(vdec, kPicHeader, DispatchPicHdr);
    vdec->parse.length = Mpeg1PictureHdrLen;
    vdec->parse.offset = 4;
    vdec->parse.pToken = (UChar *)&vdec->tokens.picHdr;
}

static InternalReasonPtr
DispatchPrePicture(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    InternalReasonPtr irp;

    switch (vdec->parse.nextStartCode) {
    case ExtensionStartCode: 
        if (NULL != (irp = CheckGopExtStartCode(vdec, pSrc-4)))
            return(irp);
        SetTokenMacro(vdec, kGopExtData, DispatchPrePicture);
        vdec->parse.offset = 4;
        vdec->parse.pToken = pSrc;
        break;

    case UserDataStartCode: 
        SetTokenMacro(vdec, kGopUserData, DispatchPrePicture);
        vdec->parse.offset = 4;
        vdec->parse.pToken = pSrc;
        break;
    
    case PictureStartCode:
        StartPicDispatch(vdec);
        break;

    default:
        return( UnexpectedVideoStartCode(vdec, pSrc, PictureStartCode,
		     vdec->parse.nextStartCode));
        break;
    }
    return(NULL);
}

static void
CalculateGopTimecode(Mpeg1VDecPrivPtr vdec)
{
    Mpeg1GopHdrPtr gop = &vdec->tokens.gopHdr;
    long seconds = 3600 * gop->hours + 60 * gop->minutes + gop->seconds;

    vdec->parse.pktTimecode = seconds * MilliSecPerSecond
			    + gop->pictures * vdec->parse.msPerPicture;
}

static InternalReasonPtr
DispatchGopHdr(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    InternalReasonPtr irp;

    SetTokenMacro(vdec, kPrePicSearch, DispatchPrePicture);
    FormatGopHdr(vdec);
    if (NULL != (irp = CheckGopHdr(vdec)))
	return(irp);

    if (! vdec->parse.timeCodes) {
	CalculateGopTimecode(vdec);
    }
    CallCallback(vdec, Mpeg1VGopCb);
    return(NULL);
}

static void
StartGopDispatch(Mpeg1VDecPrivPtr vdec)
{
    SetTokenMacro(vdec, kGopHeader, DispatchGopHdr);
    vdec->parse.length = Mpeg1GroupHdrLen;
    vdec->parse.offset = 4;
    vdec->parse.pToken = (UChar *)&vdec->tokens.gopHdr;
}

static InternalReasonPtr
DispatchPreGOP(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    InternalReasonPtr irp;

    switch (vdec->parse.nextStartCode) {
    case ExtensionStartCode: 
        if (NULL != (irp = CheckSeqExtStartCode(vdec, pSrc-4)))
            return(irp);
        SetTokenMacro(vdec, kSeqExtData, DispatchPreGOP);
        vdec->parse.offset = 4;
        vdec->parse.pToken = pSrc;
        break;

    case UserDataStartCode: 
        SetTokenMacro(vdec, kSeqUserData, DispatchPreGOP);
        vdec->parse.offset = 4;
        vdec->parse.pToken = pSrc;
        break;
    
    case GroupStartCode:
        StartGopDispatch(vdec);
        break;

    default:
        return( UnexpectedVideoStartCode(vdec, pSrc-4, GroupStartCode,
		     vdec->parse.nextStartCode));
        break;
    }
    return(NULL);
}

static void
Unzigzag(UChar matrix[ 64 ])
{
    UChar copy[ 64 ];
    ULong *pSrc, *pDst;
    int i;

    pSrc = (ULong *)matrix; 
    pDst = (ULong *)copy; 
    for (i = 16; i > 0; i--)
	*pDst++ = *pSrc++;
    for (i = 0; i < 64; i++)
	matrix[ ZigZag[ i ] ] = copy[ i];
}

static InternalReasonPtr
DispatchNIQM(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    Unzigzag(vdec->tokens.seqHdr.niqm);
    (*vdec->slice.idct->newQuantMatrices)(vdec->slice.idct,
                                          NULL, vdec->tokens.seqHdr.niqm);
    SetTokenMacro(vdec, kPreGopSearch, DispatchPreGOP);
    SetCallbackTokenStart(vdec, pSrc);
    FormatSeqHdr(vdec);
    CallCallback(vdec, Mpeg1VSeqCb);
    return(NULL);
}

static InternalReasonPtr
DispatchIQM(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    boolean iqmPresent;
    UChar *p, *pIqm;

    /*
     * The iqm is loaded unaligned by one bit... realign it here.
     */
    iqmPresent = vdec->tokens.seqHdr.iqm[63] & 0x1;
    pIqm = &vdec->tokens.seqHdr.iqm[0];
    for (p = &pIqm[63]; p >= &pIqm[0]; p--)
        p[0] = (p[-1] << 7) | (p[0] >> 1);
    Unzigzag(vdec->tokens.seqHdr.iqm);
    vdec->tokens.seqHdr.loadQmatrix = (vdec->tokens.seqHdr.loadQmatrix & 2)
                                    | iqmPresent;
    (*vdec->slice.idct->newQuantMatrices)(vdec->slice.idct,
                                          vdec->tokens.seqHdr.iqm, NULL);

    /* check for NIQM */
    if (iqmPresent) {
        SetTokenMacro(vdec, kCopySeqNIQM, DispatchNIQM);
        vdec->parse.length = Mpeg1VideoHdrLenWithTwoQM;
    } else {
        SetTokenMacro(vdec, kPreGopSearch, DispatchPreGOP);
	SetCallbackTokenStart(vdec, pSrc);
        FormatSeqHdr(vdec);
	CallCallback(vdec, Mpeg1VSeqCb);
    }
    return(NULL);
}

static InternalReasonPtr
InitPictureData(Mpeg1VDecPrivPtr vdec)
{
    PictureInfoPtr pic;
    YcMpegImagePtr yci, next;
    int i, imageValidate, mbArea, chromaSize;
    DctPictGeom dctGeom;
    InternalReasonPtr reason;

    /*
     * Reallocate (if necessary) images for the sequence.
     * if more are needed for the difference in latency between
     * decode and display, they will be allocated as needed.
     */
    imageValidate = (vdec->decode.mbWidth << 8)
		  |  vdec->decode.mbHeight
		  | ((vdec->decode.sample == Sample4_2_0) << 16);
    if (imageValidate != vdec->imageValidate) {
	next = NULL;
	/*
	 * Free pictures on the free list.
	 */
	for (yci = vdec->ycFree; yci; yci = next) {
	    next = (YcMpegImagePtr) yci->pub.next;
	    free((char *)yci);
	}

	/*
	 * This will force any call to Mpeg1VFreeYcImage() with
	 * an old value of ->imageValidate to be freed.
	 */
	vdec->imageValidate = imageValidate;

	pic = vdec->picArray;
	for (i = 0; i < NFreePictures; i++, pic++) {
	    if (pic->image)
		(*pic->image->pub.free)(&pic->image->pub);
	    pic->image = NULL;
	}
    }

    /*
     * reconfigure the dct interface
     */
    dctGeom.mbWidth = vdec->decode.mbWidth;
    dctGeom.mbHeight = vdec->decode.mbHeight;
    dctGeom.nBlocks = 6;	/* MPEG 1 */
    
    dctGeom.pic[ Dct444Pic ].width = vdec->decode.mbWidth << 4;
    dctGeom.pic[ Dct444Pic ].height = vdec->decode.mbHeight << 4;
    dctGeom.pic[ Dct420Pic ].width = vdec->decode.mbWidth << 3;
    dctGeom.pic[ Dct420Pic ].height = vdec->decode.mbHeight << 3;

    dctGeom.picType[ CR ] = Dct420Pic;
    dctGeom.picType[ CB ] = Dct420Pic;
    dctGeom.picType[ Y0 ] = Dct444Pic;
    dctGeom.picType[ Y1 ] = Dct444Pic;
    dctGeom.picType[ Y2 ] = Dct444Pic;
    dctGeom.picType[ Y3 ] = Dct444Pic;

    mbArea = vdec->decode.lastMacroBlock + 1;
    chromaSize = mbArea * 8 * 8;

    dctGeom.blkOffset[ CR ] = 0;
    dctGeom.blkOffset[ CB ] = chromaSize;
    dctGeom.blkOffset[ Y0 ] = 0;
    dctGeom.blkOffset[ Y1 ] = 8;
    dctGeom.blkOffset[ Y2 ] = vdec->decode.mbWidth * 16 * 8;
    dctGeom.blkOffset[ Y3 ] = vdec->decode.mbWidth * 16 * 8 + 8;

    reason =  (*vdec->slice.idct->newPictureSize)(vdec->slice.idct, &dctGeom);

    return(reason);    
}

static InternalReasonPtr
DispatchSeqHdr(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    InternalReasonPtr reason;
    boolean realloc = False;
    int mbWidth = (vdec->tokens.seqHdr.width+15) >> 4;
    int mbHeight = (vdec->tokens.seqHdr.height+15) >> 4;

    realloc = (mbWidth != vdec->decode.mbWidth
            || mbHeight != vdec->decode.mbHeight);
    vdec->decode.mbWidth = mbWidth;
    vdec->decode.mbHeight = mbHeight;
    vdec->decode.lastMacroBlock = vdec->decode.mbWidth * vdec->decode.mbHeight
                                - 1;
    vdec->decode.sample = Sample4_2_0;

    if (vdec->decode.width != vdec->tokens.seqHdr.width
     || vdec->decode.height != vdec->tokens.seqHdr.height
     || vdec->decode.pictureRate != vdec->tokens.seqHdr.pictureRate) {
	vdec->decode.width = vdec->tokens.seqHdr.width;
	vdec->decode.height = vdec->tokens.seqHdr.height;
	vdec->decode.pictureRate = vdec->tokens.seqHdr.pictureRate;
	vdec->cb.wantNextSeqHdr = True;
	vdec->parse.msPerPicture = MilliSecPerSecond
			         / FrameRateTable[ vdec->decode.pictureRate ];
    }

    if (vdec->cb.wantNextSeqHdr && (vdec->cb.select & Mpeg1VNewSeqMask)) {
	vdec->cb.wantNextSeqHdr = False;
	(*vdec->cb.tokenCb)(&vdec->pub, Mpeg1VNewSeqCb, &vdec->cb.arg);
    }

    if (realloc && NULL != (reason = InitPictureData(vdec)))
        return(reason);

    vdec->parse.seqHeaderSeen = True;

    /* check for IQM */
    switch (vdec->tokens.seqHdr.loadQmatrix) {
    case 2:
    case 3:
        SetTokenMacro(vdec, kCopySeqIQM, DispatchIQM);
        vdec->parse.length = Mpeg1VideoHdrLenWithOneQM;
        break;
    case 1:
        SetTokenMacro(vdec, kCopySeqNIQM, DispatchNIQM);
	vdec->parse.pToken = (UChar *)&vdec->tokens.seqHdr.niqm;
	vdec->parse.length = 64;
	vdec->parse.offset = 0;
        break;
    case 0:
        SetTokenMacro(vdec, kPreGopSearch, DispatchPreGOP);
	SetCallbackTokenStart(vdec, pSrc);
        FormatSeqHdr(vdec);
	CallCallback(vdec, Mpeg1VSeqCb);
	break;
    }
    return(NULL);
}

static void
StartSeqDispatch(Mpeg1VDecPrivPtr vdec)
{
    SetTokenMacro(vdec, kSeqHeader, DispatchSeqHdr);
    vdec->parse.length = Mpeg1VideoHdrBaseLen;
    vdec->parse.offset = 4;
    vdec->parse.pToken = (UChar *)&vdec->tokens.seqHdr;
}

static InternalReasonPtr
FoundEndcode(Mpeg1VDecPrivPtr vdec)
{
    CallCallback(vdec, Mpeg1VEndCodeCb);
    SetTokenMacro(vdec, kSeqSearch, vdec->ops.seqDispatch);
    return(NULL);
}

static InternalReasonPtr
DispatchFindNextSeq(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    if (vdec->parse.nextStartCode == SeqenceEndCode)
	return(FoundEndcode(vdec));
    if (vdec->parse.nextStartCode == SeqHeaderStartCode)
        StartSeqDispatch(vdec);
    return(NULL);
}

static InternalReasonPtr
DispatchFindNextGop(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    if (vdec->parse.nextStartCode == SeqenceEndCode)
	return(FoundEndcode(vdec));
    if (vdec->parse.nextStartCode == SeqHeaderStartCode)
        StartSeqDispatch(vdec);
    if (vdec->parse.nextStartCode == GroupStartCode)
        StartGopDispatch(vdec);
    return(NULL);
}

static InternalReasonPtr
DispatchFindNextPic(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    if (vdec->parse.nextStartCode == SeqenceEndCode)
	return(FoundEndcode(vdec));
    if (vdec->parse.nextStartCode == SeqHeaderStartCode)
        StartSeqDispatch(vdec);
    if (vdec->parse.nextStartCode == GroupStartCode)
        StartGopDispatch(vdec);
    if (vdec->parse.nextStartCode == PictureStartCode)
        StartPicDispatch(vdec);
    return(NULL);
}

static InternalReasonPtr
DispatchFindNextSlice(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
    InternalReasonPtr irp = NULL;
    
    if (vdec->parse.nextStartCode == SeqenceEndCode)
	return(FoundEndcode(vdec));
    else if (vdec->parse.nextStartCode == SeqHeaderStartCode)
        StartSeqDispatch(vdec);
    else if (vdec->parse.nextStartCode == GroupStartCode)
        StartGopDispatch(vdec);
    else if (vdec->parse.nextStartCode == PictureStartCode)
        StartPicDispatch(vdec);
    else if (NULL == (irp = CheckSliceAddr(vdec)))
        StartSliceDispatch(vdec);
    return(irp);
}

static void
Mpeg1VInitStartCodeOps(Mpeg1VDecPrivPtr vdec)
{
    vdec->ops.startCodeProcess = Mpeg1VideoProcess;
    vdec->ops.seqDispatch = DispatchFindNextSeq;
    vdec->ops.gopDispatch = DispatchFindNextGop;
    vdec->ops.picDispatch = DispatchFindNextPic;
    vdec->ops.sliceDispatch = DispatchFindNextSlice;
    vdec->ops.dctProcess = Mpeg1VDctProcess;
#   ifdef CheckMpeg1VSyntax
        (*Mpeg1VInitSliceOpsSyntax)(vdec);
#   else
#       ifdef Mpeg1VCallbacks
	    (*Mpeg1VInitSliceNoop)(vdec);
#       else
	    (*Mpeg1VInitSliceOps)(vdec);
#       endif
#   endif
}

static Mpeg1VOpsInit Mpeg1VDecodeOpsData = {
    Mpeg1VInitStartCodeOps
};

#ifdef CheckMpeg1VSyntax
    Mpeg1VOpsInitPtr Mpeg1VDecodeSyntaxOps = &Mpeg1VDecodeOpsData;
#else
#   ifdef Mpeg1VCallbacks
	Mpeg1VOpsInitPtr Mpeg1VDecodeCallbackOps = &Mpeg1VDecodeOpsData;
#   else
	Mpeg1VOpsInitPtr Mpeg1VDecodeOps = &Mpeg1VDecodeOpsData;
#   endif
#endif
