/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1vformat.c - function declaration for bitstream formatting
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1vformat.c,v 1.13 1995/11/21 20:34:56 toddb Exp $
 */

/*
 * Force definition here so that we always see the function prototypes
 */
#ifndef FormatMpeg1V
#define FormatMpeg1V
#endif

#include "mpeg1vdecI.h"
#include "mpeg1vformat.h"
#include "mpeg1huffman.h"
#include "nvrAlloc.h"

#define LongestLabel	22

static char *
BitFormat(char *buf, int value, int nBits)
{
    int i, bitsPrinted;
    char *p = buf;

    bitsPrinted = 0;
    for (i = nBits-1; i >= 0; i--) {
	*p++ = '0' + ((value & (1<<i)) != 0);
	if ((++bitsPrinted & 0x3) == 0)
	    *p++ = ' ';
    }
    *p = '\0';
    return (buf);
}

static boolean
NewFmtItem(Mpeg1VDecPrivPtr vdec,
           char *name,
           char *value,
           char *bits,
           int size)
{
    InternalReasonPtr irp;
    int allocSize = sizeof(FormattedItem);
    FormattedItemPtr item;
    char *end;
    
    if (name)
        allocSize += strlen(name) + 1;
    if (bits)
        allocSize +=  strlen(bits) + 1;
    if (value)
        allocSize +=  strlen(value) + 1;
    item = (FormattedItemPtr) InsecureAlloc(allocSize);
    if (item == NULL) {
        irp = AllocReason("formated item", allocSize);
        ShowReason(vdec->warning, irp);
        FreeReason(irp);
        return(False);
    }
    memset((char *)item, 0, allocSize);
    if (vdec->cb.arg.itemList == NULL) {
        vdec->cb.arg.itemList = vdec->cb.lastItem = item;
    } else {
        vdec->cb.lastItem->next = item;
        vdec->cb.lastItem = item;
    }
    item->size = size;
    item->streamOffset = vdec->cb.arg.tokenOffset;
    item->fileOffset = vdec->cb.arg.tokenFileOffset;
    item->level = vdec->cb.curLevel;
    end = (char *)(item + 1);
    if (name) {
        item->name = end;
        end += strlen(name) + 1;
        strcpy(item->name, name);
    }
        
    if (bits) {
        item->bits = end;
        end += strlen(bits) + 1;
        strcpy(item->bits, bits);
    }

    if (value) {
        item->value = end;
        strcpy(item->value, value);
    }
    return(True);
}

static boolean
NewLevel(Mpeg1VDecPrivPtr vdec, Mpeg1VFmtLevel newLevel)
{
    vdec->cb.curLevel = newLevel;
    return (vdec->cb.curLevel <= vdec->cb.maxLevel);
}

void
FormatSeqHdr(Mpeg1VDecPrivPtr vdec)
{
    int seqHdrSize, i;
    char buf[ 128 ];
    UChar *q;
    Mpeg1VideoHdrPtr seq = &vdec->tokens.seqHdr;
    
    if (! NewLevel(vdec, Mpeg1VSeqLevel))
	return;
    
    switch (seq->loadQmatrix) {
    default:	/* for lint */
    case 0: seqHdrSize = Mpeg1VideoHdrBaseLen; break;
    case 1: seqHdrSize = Mpeg1VideoHdrLenWithOneQM; break;
    case 2: seqHdrSize = Mpeg1VideoHdrLenWithOneQM; break;
    case 3: seqHdrSize = Mpeg1VideoHdrLenWithTwoQM; break;
    }

    NewFmtItem(vdec, "sequence header", NULL, NULL, seqHdrSize);

    if (! NewLevel(vdec, Mpeg1VPicLevel))
	return;
    
    sprintf(buf, "%d", seq->width);
    NewFmtItem(vdec, "width", buf, NULL, 0);

    sprintf(buf, "%d", seq->height);
    NewFmtItem(vdec, "height", buf, NULL, 0);

    sprintf(buf, "0x%x", seq->pelAspect);
    NewFmtItem(vdec, "pel aspect", buf, NULL, 0);

    sprintf(buf, "0x%x (%.3ffps)",
        seq->pictureRate, FrameRateTable[ seq->pictureRate ]);
    NewFmtItem(vdec, "picture rate", buf, NULL, 0);

    if (seq->bitrate == 0x3ffff)
	sprintf(buf, "0x%x (variable)", seq->bitrate);
    else
	sprintf(buf, "0x%x (%d/sec)", seq->bitrate, seq->bitrate*400);
    NewFmtItem(vdec, "bitrate", buf, NULL, 0);
    
    sprintf(buf, "%d (%d bytes)", seq->bufSize , seq->bufSize<<11);
    NewFmtItem(vdec, "bufSize", buf, NULL, 0);
    NewFmtItem(vdec, "constrained",
               seq->constrained ? "true" : "false", NULL, 0);
    if (seq->loadQmatrix & 0x2) {
        NewFmtItem(vdec, "intra quantizer matrix", NULL, NULL, 0);
        q = seq->iqm;
        for (i = 0; i < 64; i += 8) {
            sprintf(buf, "%3d%3d%3d%3d%3d%3d%3d%3d",
                    q[i+0], q[i+1], q[i+2], q[i+3],
                    q[i+4], q[i+5], q[i+6], q[i+7]);
            NewFmtItem(vdec, NULL, buf, NULL, 0);
        }
    }
    if (seq->loadQmatrix & 0x1) {
        NewFmtItem(vdec, "non-intra quantizer matrix", NULL, NULL, 0);
        q = seq->niqm;
        for (i = 0; i < 64; i += 8) {
            sprintf(buf, "%3d%3d%3d%3d%3d%3d%3d%3d",
                    q[i+0], q[i+1], q[i+2], q[i+3],
                    q[i+4], q[i+5], q[i+6], q[i+7]);
            NewFmtItem(vdec, NULL, buf, NULL, 0);
        }
    }
}

void
FormatGopHdr(Mpeg1VDecPrivPtr vdec)
{
    char buf[ 128 ];
    Mpeg1GopHdrPtr gop = &vdec->tokens.gopHdr;

    /*
     * since this item has a known length, there is no lookahead
     * token... so the next token offset is the offset of the
     * gop header
     */
    vdec->cb.arg.tokenOffset = vdec->cb.nextTokenOffset;
    vdec->cb.arg.tokenFileOffset = vdec->cb.nextTokenFileOffset;

    if (! NewLevel(vdec, Mpeg1VSeqLevel))
	return;
    
    sprintf(buf, "%02d:%02d:%02d.%2d%s%s%s",
        gop->hours,
        gop->minutes,
        gop->seconds,
        gop->pictures,
        gop->dropFrame ? " (drop frame)" : "",
        gop->closedGop ? ", closed" : "",
        gop->brokenLink ? ", broken" : "");
    NewFmtItem(vdec, "gop header", buf, NULL, Mpeg1GroupHdrLen);
}

void
FormatPicHdr(Mpeg1VDecPrivPtr vdec)
{
    char buf[ 128 ], *p;
    Mpeg1PictureHdrPtr pic = &vdec->tokens.picHdr;
    int size;

    /*
     * since this item has a known length, there is no lookahead
     * token... so the next token offset is the offset of the
     * picture header
     */
    vdec->cb.arg.tokenOffset = vdec->cb.nextTokenOffset;
    vdec->cb.arg.tokenFileOffset = vdec->cb.nextTokenFileOffset;

    if (! NewLevel(vdec, Mpeg1VPicSummaryLevel))
	return;

    if (vdec->tokens.picHdr.type == Mpeg1_P_Pic
     || vdec->tokens.picHdr.type == Mpeg1_B_Pic)
        size = Mpeg1PB_PictureHdrLen;
    else
        size = Mpeg1PictureHdrLen;
    sprintf(p = buf, "%c picture #%d", IPBDchars[ pic->type ], pic->timeRef);

    if (NewLevel(vdec, Mpeg1VPicLevel)) {
	NewFmtItem(vdec, buf, NULL, NULL, size);
    } else {
	p += strlen(p);
	sprintf(p, " vbv=%.3f", pic->vbvdelay / 90000.0);
	p += strlen(p);
	if (vdec->tokens.picHdr.type == Mpeg1_P_Pic) {
	    if (! pic->fullPelFwd || ! pic->fullPelBak) {
		sprintf(p, " half-pel %s", !pic->fullPelFwd ? "fwd" : "");
		p += strlen(p);
	    }
	} else if (vdec->tokens.picHdr.type == Mpeg1_B_Pic) {
	    if (! pic->fullPelFwd || ! pic->fullPelBak) {
		sprintf(p, " half-pel %s%s%s",
		    !pic->fullPelFwd ? "fwd" : "",
		    !pic->fullPelBak && !pic->fullPelFwd ? "," : "",
		    !pic->fullPelBak ? "bak" : "");
		p += strlen(p);
	    }
	} else if (vdec->tokens.picHdr.type == Mpeg1_B_Pic) {
	    if (! pic->fullPelFwd || ! pic->fullPelBak) {
		strcpy(p, " half-pel");
		p += strlen(p);
	    }
	}
	NewFmtItem(vdec, buf, NULL, NULL, size);

	return;
    }

    if (pic->vbvdelay == 0xffff)
	sprintf(buf, "%d (variable)", pic->vbvdelay);
    else {
	Mpeg1VideoHdrPtr seq = &vdec->tokens.seqHdr;
	double time = pic->vbvdelay / 90000.0;

	sprintf(buf, "%d (%.3f sec, %.0f bytes)",
	    pic->vbvdelay, time,
	    (time * seq->bitrate * 400) / 8);
    }
    NewFmtItem(vdec, "vbv delay", buf, NULL, 0);
    
    if (size != Mpeg1PB_PictureHdrLen) {
        NewFmtItem(vdec, "extra pic info",
               pic->fullPelFwd ? "true" : "false", NULL, 0);
        return;
    }

    NewFmtItem(vdec, "full pel fwd",
               pic->fullPelFwd ? "true" : "false", NULL, 0);
    
    sprintf(buf, "%d", (pic->fwdFCode1<<1) | pic->fwdFCode2);
    NewFmtItem(vdec, "fwd f code", buf, NULL, 0);

    if (vdec->tokens.picHdr.type != Mpeg1_B_Pic) {
        NewFmtItem(vdec, "extra pic info",
               pic->fullPelBak ? "true" : "false", NULL, 0);
        return;
    }

    NewFmtItem(vdec, "full pel bak",
               pic->fullPelBak ? "true" : "false", NULL, 0);
    
    sprintf(buf, "%d", pic->bakFCode);
    NewFmtItem(vdec, "bak f code", buf, NULL, 0);

    NewFmtItem(vdec, "extra pic info",
               pic->extraPicInfo ? "true" : "false", NULL, 0);
}
    
static void
NewSliceItem(Mpeg1VDecPrivPtr vdec,
             char *name,
             int bits,
             int bitLength,
             int value,
             UChar *pBuf,
             int nCacheBits)
{
    char valString[ 128 ];
    char bitString[ 128 ];
    FormattedItemPtr item;
    int nCacheBytes = (nCacheBits + 7) >> 3;

    BitFormat(bitString, bits, bitLength);
    sprintf(valString, "%d", value);
    if (! NewFmtItem(vdec, name, valString, bitString, 0))
        return;
    item = vdec->cb.lastItem;
    item->tokenOffset = 4 + pBuf - vdec->slice.pBufferBase - nCacheBytes;
    if (nCacheBits & 0x7)
        item->bitOffset = 8 - (nCacheBits & 0x7);
    else
        item->bitOffset = 0;
}

static void
NewComplexSliceItem(Mpeg1VDecPrivPtr vdec,
                    char *name,
                    int bits,
                    int bitLength,
                    char *value,
                    UChar *pBuf,
                    int nCacheBits)
{
    char bitString[ 128 ];
    FormattedItemPtr item;
    int nCacheBytes = (nCacheBits+7) >> 3;

    BitFormat(bitString, bits, bitLength);
    if (! NewFmtItem(vdec, name, value, bitString, 0))
        return;
    item = vdec->cb.lastItem;
    item->tokenOffset = 4 + pBuf - vdec->slice.pBufferBase - nCacheBytes;
    if (nCacheBits & 0x7)
        item->bitOffset = 8 - (nCacheBits & 0x7);
    else
        item->bitOffset = 0;
}

void
FormatSliceHdr(Mpeg1VDecPrivPtr vdec, boolean extraSliceInfo)
{
    char buf[ 128 ];

    if (! NewLevel(vdec, Mpeg1VSliceLevel))
	return;
    sprintf(buf, "%d", vdec->slice.vertPos);
    NewFmtItem(vdec, "slice", buf, NULL, vdec->slice.nBytes + 4);

    if (! NewLevel(vdec, Mpeg1VMacroblockLevel))
	return;

    NewSliceItem(vdec, "q scale", vdec->slice.qScale,
                 5, vdec->slice.qScale, vdec->slice.pBufferBase, 0);

    NewSliceItem(vdec, "extra slice info", extraSliceInfo,
                 1, extraSliceInfo, vdec->slice.pBufferBase+1, 3);
}

void
FormatMbStuffing(Mpeg1VDecPrivPtr vdec, UChar *pBuf, int nCacheBits)
{
    if (! NewLevel(vdec, Mpeg1VMacroblockLevel))
	return;
    NewFmtItem(vdec, "mb stuffing", NULL, NULL, 0);
}

void
FormatMbAddr(Mpeg1VDecPrivPtr vdec,
             int addrIncBits,
             UChar *pBuf,
             int nCacheBits)
{
    char valString[ 64 ];
    int hashValue, inc, bitLength;
    
    if (! NewLevel(vdec, Mpeg1VMacroblockLevel))
	return;
    bitLength = MBAIBits(hashValue = HuffLookup.mbAddressInc[ addrIncBits ]);
    inc = MBAIncrementValue(hashValue);
    sprintf(valString, "%d (mbaddr=%d)", inc, vdec->slice.mbAddr);
    NewComplexSliceItem(vdec, "mb addr inc", addrIncBits >> (11-bitLength),
                 bitLength, valString, pBuf, nCacheBits);
}

void
FormatMbType(Mpeg1VDecPrivPtr vdec,
             int bits,
             int bitLength,
             UChar *pBuf,
             int nCacheBits)
{
    char valString[ 64 ], *p;
    char *separator = "";
    char *comma = ",";

    if (! NewLevel(vdec, Mpeg1VMacroblockLevel))
	return;
    p = valString;

    if (vdec->slice.mbType & MBT_QUANT) {
	separator = comma;
	strcpy(p, "quant");
	p += strlen(p);
    }

    if (vdec->slice.mbType & MBT_MFWD) {
	sprintf(p, "%sfwd", separator);
	separator = comma;
	p += strlen(p);
    }

    if (vdec->slice.mbType & MBT_MBAK) {
	sprintf(p, "%sbak", separator);
	separator = comma;
	p += strlen(p);
    }

    if (vdec->slice.mbType & MBT_PATTERN) {
	sprintf(p, "%spat", separator);
	separator = comma;
	p += strlen(p);
    }

    if (vdec->slice.mbType & MBT_INTRA) {
	sprintf(p, "%sintra", separator);
    }

    NewComplexSliceItem(vdec, "mb type", bits >> (6-bitLength),
                 bitLength, valString, pBuf, nCacheBits);
}

void
FormatMbQuant(Mpeg1VDecPrivPtr vdec,
             UChar *pBuf,
             int nCacheBits)
{
    if (! NewLevel(vdec, Mpeg1VMacroblockLevel))
	return;
    NewSliceItem(vdec, "mb quant", vdec->slice.qScale,
                 5, vdec->slice.qScale, pBuf, nCacheBits);
}

void
FormatBlockPattern(Mpeg1VDecPrivPtr vdec,
             int bits,
             int bitLength,
             UChar *pBuf,
             int nCacheBits)
{
    if (! NewLevel(vdec, Mpeg1VMacroblockLevel))
	return;
    NewSliceItem(vdec, "blk pattern", bits >> (9-bitLength), bitLength,
                 vdec->slice.codedBlockPattern, pBuf, nCacheBits);
}

void
FormatMotionCode(Mpeg1VDecPrivPtr vdec,
                 int bits,
                 IdctMotionIndex mvi,
                 UChar *pBuf,
                 int nCacheBits)
{
    char name[ 32 ];
    int hashValue, bitLength, code;

    if (! NewLevel(vdec, Mpeg1VMacroblockLevel))
	return;

    hashValue = HuffLookup.motionVector[ bits ];
    bitLength = MotionBits(hashValue);
    code = MotionValue(hashValue);

    sprintf(name, "%s code", IdctMotionIndexNames[ mvi ]);
    NewSliceItem(vdec, name, bits >> (11 - bitLength),
	         bitLength, code, pBuf, nCacheBits);
}

void
FormatMotionR(Mpeg1VDecPrivPtr vdec,
              int r,
              IdctMotionIndex mvi,
              UChar *pBuf,
              int nCacheBits)
{
    char name[ 32 ];
    UChar *mvSizes = vdec->currentPict->mvSize;

    if (! NewLevel(vdec, Mpeg1VMacroblockLevel))
	return;

    sprintf(name, "%s r", IdctMotionIndexNames[ mvi ]);
    NewSliceItem(vdec, name, r, mvSizes[ mvi>>1 ], r, pBuf, nCacheBits);
}

void
FormatBlkStart(Mpeg1VDecPrivPtr vdec,
               int blkIdx,
               UChar *pBuf,
               int nCacheBits)
{
    char name[ 32 ];
    char *type;
    
    if (! NewLevel(vdec, Mpeg1VMacroblockLevel))
	return;

    if (((1<<blkIdx) & vdec->slice.codedBlockPattern) == 0)
        type = "copied";
    else if (vdec->slice.mbType & MBT_INTRA)
        type = "intra-coded";
    else
        type = "non-intra-coded";
    
    sprintf(name, "%s block %s", MpegBlockNames[ blkIdx ], type);
    /* NewComplexSliceItem(vdec, name, 0, 0, type, pBuf, nCacheBits); */
    NewFmtItem(vdec, name, NULL, NULL, 0);
}

void
FormatDcChromaSize(Mpeg1VDecPrivPtr vdec,
                   int bits,
                   UChar *pBuf,
                   int nCacheBits)
{
    int hashValue, bitLength, dcSize;

    if (! NewLevel(vdec, Mpeg1VBlockLevel))
	return;

    hashValue = HuffLookup.sizeChrominance[ bits ];
    bitLength = SizeYCBits(hashValue);
    dcSize = SizeYCValue(hashValue);

    NewSliceItem(vdec, "dc size chroma", bits >> (8-bitLength),
                 bitLength, dcSize, pBuf, nCacheBits);
}

void
FormatDcLumaSize(Mpeg1VDecPrivPtr vdec,
                 int bits,
                 UChar *pBuf,
                 int nCacheBits)
{
    int hashValue, bitLength, dcSize;

    if (! NewLevel(vdec, Mpeg1VBlockLevel))
	return;

    hashValue = HuffLookup.sizeLuminance[ bits ];
    bitLength = SizeYCBits(hashValue);
    dcSize = SizeYCValue(hashValue);

    NewSliceItem(vdec, "dc size luma", bits >> (7-bitLength),
                 bitLength, dcSize, pBuf, nCacheBits);
}

void
FormatDcLevel(Mpeg1VDecPrivPtr vdec,
              int bits,
              int bitLength,
              int dcLevel,
              UChar *pBuf,
              int nCacheBits)
{
    char valString[ 64 ];
    
    if (! NewLevel(vdec, Mpeg1VCoeffLevel))
	return;
    sprintf(valString, "dcLevel=%d", dcLevel);
    NewComplexSliceItem(vdec, "dc diff", bits, bitLength, valString,
                        pBuf, nCacheBits);
}

void
FormatDctCoeff(Mpeg1VDecPrivPtr vdec,
              int bits,
              int bitLength,
              int run,
              boolean negative,
              int level,
              UChar *pBuf,
              int nCacheBits)
{
    char valString[ 64 ];
    
    if (! NewLevel(vdec, Mpeg1VCoeffLevel))
	return;
    sprintf(valString, "run=%d, level=%d", run, negative ? -level : level);
    NewComplexSliceItem(vdec, "dc coeff next", bits, bitLength, valString,
                        pBuf, nCacheBits);
}

void
FormatDctCoeffFirst(Mpeg1VDecPrivPtr vdec,
                    boolean negative,
                    UChar *pBuf,
                    int nCacheBits)
{
    char valString[ 64 ];
    
    if (! NewLevel(vdec, Mpeg1VCoeffLevel))
	return;
    sprintf(valString, "run=0, level=%d", negative ? -1 : 1);
    NewComplexSliceItem(vdec, "dc coeff first", 2+negative, 2, valString,
                        pBuf, nCacheBits);
}

void
FormatEndOfBlock(Mpeg1VDecPrivPtr vdec, UChar *pBuf, int nCacheBits)
{
    if (! NewLevel(vdec, Mpeg1VCoeffLevel))
	return;
    NewComplexSliceItem(vdec, "end of block", 0x2, 2, "", pBuf, nCacheBits);
}

void
FormatMbEscape(Mpeg1VDecPrivPtr vdec, UChar *pBuf, int nCacheBits)
{
}

void
FormatMbSummary(Mpeg1VDecPrivPtr vdec, DctMacroblockPtr idctMb)
{
    char valString[ 256 ], *p;
    boolean halfPel;
    PictureInfoPtr pic = vdec->currentPict;

    if (! NewLevel(vdec, Mpeg1VMbSummaryLevel))
	return;

    p = valString;
    sprintf(p, "mb #%d", (int)vdec->slice.mbAddr);
    p += strlen(p);

    if (idctMb->flags.b.intra)
	strcpy(p, ", intra");
    else
	strcpy(p, ", delta");
    p += strlen(p);

    sprintf(p, ", q=%d", idctMb->flags.b.quant);
    p += strlen(p);

    if (idctMb->flags.b.fwdMv) {
        halfPel = ! pic->fullPel[ kIndexFwd ];
	sprintf(p, ", fwd=(%d%s,%d%s)",
	    idctMb->motion[ kIndexFwdX ] >> halfPel,
	    idctMb->motion[ kIndexFwdX ] & halfPel ? "+h" : "",
	    idctMb->motion[ kIndexFwdY ] >> halfPel,
	    idctMb->motion[ kIndexFwdY ] & halfPel ? "+h" : "");
	p += strlen(p);
    }

    if (idctMb->flags.b.bakMv) {
        halfPel = ! pic->fullPel[ kIndexBak ];
	sprintf(p, ", bak=(%d%s,%d%s)",
	    idctMb->motion[ kIndexBakX ] >> halfPel,
	    idctMb->motion[ kIndexBakX ] & halfPel ? "+h" : "",
	    idctMb->motion[ kIndexBakY ] >> halfPel,
	    idctMb->motion[ kIndexBakY ] & halfPel ? "+h" : "");
	p += strlen(p);
    }

    NewFmtItem(vdec, valString, NULL, NULL, 0);
}

void
FormatSliceEnd(Mpeg1VDecPrivPtr vdec, UChar *pBuf, int nCacheBits)
{
    int nPadBits, nPadBytes, i;
    char buf[ 128 ], *p;
    FormattedItemPtr item;

    nPadBits = nCacheBits & 0x7;
    pBuf -= nCacheBits >> 3;

    if (NewLevel(vdec, Mpeg1VMacroblockLevel)) {
	if (nPadBits) {
	    pBuf--;
	    NewComplexSliceItem(vdec, "alignment padding",
				*pBuf & ((1<<nPadBits)-1), nPadBits, NULL,
				pBuf, nPadBits);
	    pBuf++;
	}
    }

    if (! NewLevel(vdec, Mpeg1VSliceLevel))
	return;
    nPadBytes = vdec->slice.pEnd - pBuf;
    if (nPadBytes > 0) {
        NewFmtItem(vdec, "pad bytes", NULL, NULL, 0);
	item = vdec->cb.lastItem;
	item->tokenOffset = pBuf - vdec->slice.pBufferBase + 4;

        for (i = 0; i+8 < nPadBytes; i += 8, pBuf += 8) {
            sprintf(buf, "%3d: %02x %02x %02x %02x %02x %02x %02x %02x",
		    i, 
                    pBuf[0], pBuf[1], pBuf[2], pBuf[3],
                    pBuf[4], pBuf[5], pBuf[6], pBuf[7]);
            NewFmtItem(vdec, NULL, buf, NULL, 0);
        }
	if (i < nPadBytes) {
            sprintf(p = buf, "%3d: ", i);
	    p += strlen(p);
	    for (; i < nPadBytes; i++, pBuf++, p += 3)
		sprintf(p, "%02x ", *pBuf);
            NewFmtItem(vdec, NULL, buf, NULL, 0);
	}
    }
}
