/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1vformat.h - macro and function declaration for formatting output
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1vformat.h,v 1.5 1995/10/20 20:10:37 toddb Exp $
 */

#ifndef MPEG1VFORMAT_H
#define MPEG1VFORMAT_H

#if defined(FormatMpeg1V) || defined(CheckMpeg1VSyntax)

#define WhenFormatting(code)	code

CExternal void FormatSeqHdr(
    Mpeg1VDecPrivPtr vdec
);

CExternal void FormatGopHdr(
    Mpeg1VDecPrivPtr vdec
);

CExternal void FormatPicHdr(
    Mpeg1VDecPrivPtr vdec
);

CExternal void FormatSliceHdr(
    Mpeg1VDecPrivPtr vdec,
    boolean extraSliceInfo
);

CExternal void FormatMbStuffing(
    Mpeg1VDecPrivPtr vdec,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatMbAddr(
    Mpeg1VDecPrivPtr vdec,
    int addrIncBits,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatMbType(
    Mpeg1VDecPrivPtr vdec,
    int bits,
    int nBits,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatMbQuant(
    Mpeg1VDecPrivPtr vdec,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatBlockPattern(
    Mpeg1VDecPrivPtr vdec,
    int bits,
    int nBits,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatMotionCode(
    Mpeg1VDecPrivPtr vdec,
    int bits,
    IdctMotionIndex,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatMotionR(
    Mpeg1VDecPrivPtr vdec,
    int r,
    IdctMotionIndex,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatBlkStart(
    Mpeg1VDecPrivPtr vdec,
    int blkIdx,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatDcChromaSize(
    Mpeg1VDecPrivPtr vdec,
    int bits,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatDcLumaSize(
    Mpeg1VDecPrivPtr vdec,
    int bits,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatDcLevel(
    Mpeg1VDecPrivPtr vdec,
    int bits,
    int bitLength,
    int dcLevel,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatDctCoeffFirst(
    Mpeg1VDecPrivPtr vdec,
    boolean negative,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatDctCoeff(
    Mpeg1VDecPrivPtr vdec,
    int bits,
    int bitLength,
    int run,
    boolean negative,
    int level,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatEndOfBlock(
    Mpeg1VDecPrivPtr vdec,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatMbEscape(
    Mpeg1VDecPrivPtr vdec,
    UChar *pBuf,
    int nCacheBits
);

CExternal void FormatMbSummary(
    Mpeg1VDecPrivPtr vdec,
    DctMacroblockPtr idctMb
);

CExternal void FormatSliceEnd(
    Mpeg1VDecPrivPtr vdec,
    UChar *pBuf,
    int nCacheBits
);

#else /* FormatMpeg1V */

#define WhenFormatting(code)				/* as nothing */

#define FormatSeqHdr(_vdec)				/* as nothing */
#define FormatGopHdr(_vdec)				/* as nothing */
#define FormatPicHdr(_vdec)				/* as nothing */
#define FormatSliceHdr(_vdec, _extraSliceInfo)		/* as nothing */
#define FormatMbStuffing(_vdec, _pBuf, _nCacheBits)	/* as nothing */
#define FormatMbAddr(_vdec, _bits, _pBuf, _nCacheBits)	/* as nothing */
#define FormatMbType(_vdec, _bits, _nbits, _pBuf, _nCacheBits)	/* as nothing */
#define FormatMbQuant(_vdec, _pBuf, _nCacheBits)	/* as nothing */
#define FormatBlockPattern(_vdec, _bits, _nbits, _pBuf, _nCacheBits)
#define FormatMotionCode(_vdec, _bits, _i, _pBuf, _nCacheBits)	/* as nothing */
#define FormatMotionR(_vdec, _r, _i, _pBuf, _nCacheBits)	/* as nothing */
#define FormatBlkStart(_vdec, _blkIdx, _pBuf, _nCacheBits)	/* as nothing */
#define FormatDcChromaSize(_vdec, _bits, _pBuf, _nCacheBits)	/* as nothing */
#define FormatDcLumaSize(_vdec, _bits, _pBuf, _nCacheBits)	/* as nothing */
#define FormatDcLevel(_vdec, _bits, _nbits, _dclvl, _pBuf, _ncb)
#define FormatDctCoeff(_vdec, _bits, _nbits, _run, _neg, _lvl, _pBuf, _ncb)
#define FormatDctCoeffFirst(_vdec, _neg, _pBuf, _ncb)	/* as nothing */
#define FormatEndOfBlock(_vdec, _pBuf, _ncb)		/* as nothing */
#define FormatMbEscape(_vdec, _pBuf, _nCacheBits)	/* as nothing */
#define FormatMbSummary(_vdec, _idctMb)			/* as nothing */
#define FormatSliceEnd(_vdec, _pBuf, _ncb)		/* as nothing */

#define FreeFormatItems(_vdec)				/* as nothing */

#endif /* FormatMpeg1V */

#endif /* MPEG1VFORMAT_H */
