/*
 *	(c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		mpeg1vproto.h - mpeg 1 video protocol structures
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/mpeg1vproto.h,v 1.3 1995/08/02 06:26:11 toddb Exp $
 */
#ifndef _MPEG1VPROTO_H_
#define _MPEG1VPROTO_H_

#include "mpeg1Proto.h"

/*************************************************************************
 *
 * mpeg video tokens
 *
 *************************************************************************/
typedef struct _mpeg1SeqHdr {
    ULong seqHdrCode;
    ULong width       :12;
    ULong height      :12;
    ULong pelAspect   : 4;
    ULong pictureRate : 4;
    ULong bitrate     :18;
    ULong marker      : 1;
    ULong bufSize     :10;
    ULong constrained : 1;
    ULong loadQmatrix : 2;
    UChar iqm[ 64 ];
    UChar niqm[ 64 ];
} Mpeg1VideoHdr, *Mpeg1VideoHdrPtr;

#define Mpeg1VideoHdrBaseLen		12
#define Mpeg1VideoHdrLenWithOneQM	(Mpeg1VideoHdrBaseLen + 64)
#define Mpeg1VideoHdrLenWithTwoQM	(Mpeg1VideoHdrLenWithOneQM + 64)

typedef struct _mpeg1PictureHdr {
    ULong picStartCode;
    ULong timeRef     :10;
    ULong type        : 3;
    ULong vbvdelay    :16;
    
    /*
     * defined only for P and B pictures
     */
    ULong fullPelFwd  : 1;	/* extra picture info marker for I picture */
    ULong fwdFCode1   : 2;
    ULong fwdFCode2   : 1;	/* 3rd long-word begins here */

    /*
     * defined only for B pictures
     */
    ULong fullPelBak  : 1;	/* extra picture info marker for P picture */
    ULong bakFCode    : 3;
    ULong extraPicInfo: 1;	/* extra picture info marker for B picture */
    ULong pad         :26;
} Mpeg1PictureHdr, *Mpeg1PictureHdrPtr;

#define Mpeg1PictureHdrLen		8	/* excludes fwdFCode2 and on */
#define Mpeg1PB_PictureHdrLen		9	/* includes fwdFCode2 and on */

#define ForwardSizeMacro(pic)					\
    ((((pic)->fwdFCode1 << 1) | (pic)->fwdFCode2) - 1)

typedef struct {
    ULong gopStartCode;
    ULong dropFrame :1;
    ULong hours     :5;
    ULong minutes   :6;
    ULong marker    :1;
    ULong seconds   :6;
    ULong pictures  :6;
    ULong closedGop :1;
    ULong brokenLink:1;
    ULong pad       :5;
} Mpeg1GopHdr, *Mpeg1GopHdrPtr;

#define Mpeg1GroupHdrLen		8

/*
 * Picture types in Mpeg1PictureHdr.type
 */
typedef enum {
    Mpeg1_I_Pic = 1,
    Mpeg1_P_Pic = 2,
    Mpeg1_B_Pic = 3,
    Mpeg1_D_Pic = 4
} Mpeg1VPicType;

/*
 * Start codes
 */
typedef enum {
    Mpeg1PicStart       = 0x100,
    Mpeg1SliceMinStart  = 0x101,
    Mpeg1SliceMaxStart  = 0x1AF,
    Mpeg1UserDataStart  = 0x1B2,
    Mpeg1SeqHeaderStart = 0x1B3,
    Mpeg1SeqErrorStart  = 0x1B4,
    Mpeg1ExtensionStart = 0x1B5,
    Mpeg1SeqEndCode     = 0x1B7
} Mpeg1VideoStartCodes;

/*
 * macroblock tokens
 */
typedef enum {
    MacroblockStuffing = 0x00f,	/* 11-bit code for padding */
    MacroblockEscape   = 0x008	/* 11-bit code for macroblock escape */
} MacroblockTokens;

/*
 * macroblock type constants
 */
#define MBT_QUANT_BitPosition	0
#define MBT_QUANT	0x01	/* Override quantizer scale for this macroblk */
#define	MBT_MFWD	0x02	/* Macroblock motion forward */
#define	MBT_MBAK	0x04	/* Macroblock motion backward */
#define	MBT_PATTERN	0x08	/* DCT coefficients to follow */
#define	MBT_INTRA	0x10	/* Intra-frame coding */

/*
 * Protocol Picture rates
 */
typedef enum {
    Mpeg1Rate_23_976	= 0x01,
    Mpeg1PicRate_24	= 0x02,
    Mpeg1PicRate_25	= 0x03,
    Mpeg1PicRate_29_97	= 0x04,
    Mpeg1PicRate_30	= 0x05,
    Mpeg1PicRate_50	= 0x06,
    Mpeg1PicRate_59_94	= 0x07,
    Mpeg1PicRate_60	= 0x08,

    /*
     * NVR picture rates
     */
    Mpeg1PicRate_XING30	= 0x09,
    Mpeg1PicRate_6	= 0x0a,
    Mpeg1PicRate_10	= 0x0b,
    Mpeg1PicRate_15	= 0x0c,
    Mpeg1PicRate_20	= 0x0d,
    Mpeg1PicRate_5	= 0x0e
} Mpeg1PicRate;

/*
 * Macroblock block indices
 */
typedef enum {
    Y0     = 5,		/* Block index of Y0 */
    Y1     = 4,		/* Block index of Y1 */
    Y2     = 3,		/* Block index of Y2 */
    Y3     = 2,		/* Block index of Y3 */
    CB     = 1,		/* Block index of CB */
    CR     = 0,		/* Block index of CR */

    Y0Mask = (1<<Y0),	/* Upper left Y block included in macroblock */
    Y1Mask = (1<<Y1),	/* Upper right Y block included in macroblock */
    Y2Mask = (1<<Y2),	/* Lower left Y block included in macroblock */
    Y3Mask = (1<<Y3),	/* Lower right Y block included in macroblock */
    CBMask = (1<<CB),	/* B-Y block included in macroblock */
    CRMask = (1<<CR),	/* R-Y block included in macroblock */
    
    AllBlocksMask = Y0Mask | Y1Mask | Y2Mask | Y3Mask | CRMask | CBMask
} MbBlockConstant;

extern float    FrameRateTable[16];
extern UChar    DefaultIntraQMatrix[];
extern UChar    DefaultNonIntraQMatrix[];

#endif /* _MPEG1VPROTO_H_ */
