/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1vslice.c - slice level mpeg video decoding
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1vslice.c,v 1.19 1995/10/27 22:19:43 toddb Exp $
 */

#include "mpeg1vdecI.h"
#include "mpeg1huffman.h"
#include "nvrAlloc.h"

static void
CancelSliceDecode(Mpeg1VDecPrivPtr vdec)
{
    vdec->pub.ops.process = vdec->ops.startCodeProcess;
    vdec->slice.mbAddr = vdec->decode.lastMacroBlock;
    (*vdec->ops.sliceDispatch)(vdec, NULL);
}

static void
ConsumeExtraSliceInfo(Mpeg1VDecPrivPtr vdec)
{
    int bits;
    DeclareBitParseVars(cache, nCacheBits, pBuf);

    FillCacheMacro(&vdec->slice.bitParse, cache, nCacheBits, pBuf);
    
    /*
     * Here we read in (and ignore) the extra slice info.
     */
    for (;;) {
        ReadBitsMacro(bits, cache, nCacheBits, pBuf, 9);
	if ((bits & 1) == 0)
	    break;
    }
    EmptyCacheMacro(&vdec->slice.bitParse, cache, nCacheBits, pBuf);
}

static InternalReasonPtr
Mpeg1SliceInit(Mpeg1VDecPrivPtr vdec, UChar *pDest)
{
    int i, mbAddr, bits;
    InternalReasonPtr irp;
    short *pPastMotion = vdec->currentPict->pastMotion;

    /*
     * The vertical offset is usually redundant.  It points to the same
     * row that the first macroblock address increment in the slice
     * points to.  However, if a slice starts in the middle of a row,
     * the next address increment should assume that the mbaddr has
     * been reset to the beginning of the row, minus one.
     *
     * For example, if the the picture is 22 macroblocks wide and
     * the first macroblock goes from mb 0 through mb 26, then the
     * processed macroblocks looks like:
     *
     *    x x x x x x x x x x x x x x x x x x x x x
     *    x x x x x
     *
     * Since there is guaranteed to be no gap in the
     * macro block addresses between slices, the address of the first
     * macroblock in the next slice (27) is always one more
     * than the previous mbaddr (26).  But since we have to reset
     * the mb address to the beginning of the row, minus one (or 21),
     * the next address increment must put us back to the right place
     * i.e. mb addr must be 6 (21+6=27).
     */
    mbAddr = -1;
    for (i = vdec->slice.vertPos-1; i > 0; i--)
        mbAddr += vdec->decode.mbWidth;
    vdec->slice.mbAddr = mbAddr;
    vdec->slice.pastIntraMbAddr = -2;		/* per protocol */
    vdec->slice.dcYPast = 128;			/* per protocol */
    vdec->slice.dcCrPast = 128;			/* per protocol */
    vdec->slice.dcCbPast = 128;			/* per protocol */
    
    
    /*
     * Per protocol: reset past motion vectors for B and P pictures
     * at the beginning of every slice.
     */
    pPastMotion[ kIndexFwdX ] = 0;		/* per protocol */
    pPastMotion[ kIndexFwdY ] = 0;		/* per protocol */
    pPastMotion[ kIndexBakX ] = 0;		/* per protocol */
    pPastMotion[ kIndexBakY ] = 0;		/* per protocol */

    /*
     * Note that the "slice" token contains all of the slice bits plus
     * the following start code.  We exclude those bits from the parsing
     * by subtracting 4 here.
     */
    vdec->slice.nBytes = (pDest - vdec->parse.pToken) - 4;
    vdec->slice.pBufferBase = vdec->parse.pToken;
    vdec->slice.pEnd = vdec->slice.pBufferBase + vdec->slice.nBytes;

    InitBitParseMacro(&vdec->slice.bitParse, vdec->slice.pBufferBase)

    /*
     * The vast majority of the time, there is no extra slice info.
     * These few instructions read the first six bits of the slice,
     * which usually contain the quantizer scale (5 bits) and the
     * extra_bit_slice (1 bit) of 0.
     */
    vdec->slice.bitParse.nBits = 2;	/* we know it starts with 8 bits */
    bits = *vdec->slice.bitParse.pBuf >> 2;
    vdec->slice.qScale = bits >> 1;
    if (bits & 1)
        ConsumeExtraSliceInfo(vdec);
    if (NULL != (irp = CheckSliceQuantizer(vdec)))
	CancelSliceDecode(vdec);
    FormatSliceHdr(vdec, bits & 1);

    return(irp);
}

static InternalReasonPtr
LocalAllocDctArena(Mpeg1VDecPrivPtr vdec, DctMacroblockPtr idctMb)
{
    InternalReasonPtr irp = NULL;
    
    idctMb->flags.b.endOfList = True;
    QueueDctArena(vdec->slice.idct, vdec->slice.curDctList);
    vdec->slice.curDctList = AllocDctArena(vdec->slice.idct, &irp);
    return(irp);
}

static InternalReasonPtr
Mpeg1SliceProcess(PacketLayerPtr vdecPub)
{
    Mpeg1VDecPrivPtr vdec = (Mpeg1VDecPrivPtr)vdecPub;
    DctMacroblockPtr idctMb = vdec->slice.curDctList->curIdctMb;
    DctBlockPtr dctBlk;
    DctMbFlags mbFlags;
    DctBlockHdr blkHdr;
    UShort *pCoeff;
    InternalReasonPtr reason = NULL;
    int blkIdx, bitLength, inc, copied, blocksMask;
    int run, level, dctZZIndex, dctIndex;
    boolean negative;
    ULong bits, bit, mbAddrInc, hashValue;
    short mbAddr;
    DeclareBitParseVars(cache, nCacheBits, pBuf);

    FillCacheMacro(&vdec->slice.bitParse, cache, nCacheBits, pBuf);

    for (;;) {
        bits = NextBitsMacro(cache, 11);
        while (bits == MacroblockStuffing) {
            FormatMbStuffing(vdec, pBuf, nCacheBits);
            ConsumeBitsMacro(cache, nCacheBits, pBuf, 11);
            bits = NextBitsMacro(cache, 11);
        }
        
        mbAddrInc = 0;
        while (bits == MacroblockEscape) {
            mbAddrInc += 33;        /* as per section 3.2.6 */
            FormatMbEscape(vdec, pBuf, nCacheBits);
            ConsumeBitsMacro(cache, nCacheBits, pBuf, 11);
            bits = NextBitsMacro(cache, 11);
        }

        /*
         * Parse the MB address incdement.  Note that
         * 11 bits are already in 'bits'. At least 23 bits in cache.
         */
        bitLength = MBAIBits(hashValue = HuffLookup.mbAddressInc[ bits ]);
        inc = MBAIncrementValue(hashValue);
        mbAddrInc += inc;
        mbAddr = (vdec->slice.mbAddr += mbAddrInc);
        FormatMbAddr(vdec, bits, pBuf, nCacheBits);
        if (NULL != (reason = CheckMbAddr(vdec, bitLength, inc)))
            break;
        ConsumeBitsNoFillMacro(cache, nCacheBits, bitLength);
        /* now at least 12 bits in cache */
        
        copied = mbAddr - vdec->slice.pastMbAddr - 1;
        vdec->slice.pastMbAddr = mbAddr;

        if (NeedNewDctArena(vdec->slice.curDctList, idctMb)) {
            if (NULL != (reason = LocalAllocDctArena(vdec, idctMb)))
                break;
            idctMb = vdec->slice.curDctList->curIdctMb;
        }

        if (copied > 0) {
            short *pastMV = vdec->currentPict->pastMotion;
            long mvFlags;
            
            /*
             * Per protocol, reset P picture motion vectors and
             * set motion vectors to zero.  For B pictures, use the
             * past motion.
             */
            if (vdec->currentPict->type == Mpeg1_P_Pic) {
                pastMV[ kIndexFwdX ] = 0;
                pastMV[ kIndexFwdY ] = 0;
                mvFlags = DctMotionFwd;
            } else {
                switch (vdec->slice.mbType & (MBT_MBAK | MBT_MFWD)) {
                case MBT_MBAK: mvFlags = DctMotionBak; break;
                case MBT_MFWD: mvFlags = DctMotionFwd; break;
                case MBT_MFWD|MBT_MBAK: mvFlags = DctMotionFwd | DctMotionBak;
					break;
		case 0: mvFlags = 0; break; /* for lint */
                }
            }
            
            for (; copied > 0; copied--) {
                idctMb->flags.composite = mvFlags;
                if (mvFlags & DctMotionFwd) {
                    idctMb->motion[ kIndexFwdX ] = pastMV[ kIndexFwdX ];
                    idctMb->motion[ kIndexFwdY ] = pastMV[ kIndexFwdY ];
                }
                if (mvFlags & DctMotionBak) {
                    idctMb->motion[ kIndexBakX ] = pastMV[ kIndexBakX ];
                    idctMb->motion[ kIndexBakY ] = pastMV[ kIndexBakY ];
                }
                idctMb = DctBlockPtrToNextMb(&idctMb->blk[0]);
            }
        }

        /*
         * decode the macroblock type depending on the picture type.
         */
        bits = NextBitsMacro(cache, 6);
        switch (vdec->tokens.picHdr.type) {
        case Mpeg1_B_Pic:
            bitLength = MBTBits(hashValue = HuffLookup.mb_B_Type[ bits ]);
            vdec->slice.mbType = MBTypeValue(hashValue);
            break;
        
        case Mpeg1_P_Pic:
            bitLength = MBTBits(hashValue = HuffLookup.mb_P_Type[ bits ]);
            vdec->slice.mbType = MBTypeValue(hashValue);
            break;
        
        default: /* Mpeg1_I_Pic or Mpeg1_D_Pic */
            /*
             * For I pictures we only have to consider the first of the
	     * six bits.  If its 1, then the bitlength is 1 and the type
	     * is MBT_INTRA; if 0, then the bitlength is 2 (code is 01) and
	     * the type is MBT_INTRA|MBT_QUANT.
             * For D pictures, it is always a one bit and is yields the same
             * type as an I picture with a one bit.
             * This gobbledy gook code does that without branches.
             */
            bit = bits >> 5;
            bitLength = 2 - bit;
            vdec->slice.mbType = MBT_INTRA
		| (((~bit) << MBT_QUANT_BitPosition) & MBT_QUANT);
            break;
        }
        if (NULL != (reason = CheckMbType(vdec, bitLength)))
            break;
        FormatMbType(vdec, bits, bitLength, pBuf, nCacheBits);
        ConsumeBitsMacro(cache, nCacheBits, pBuf, bitLength);
        
        if (vdec->slice.mbType & MBT_QUANT) {
            ReadBitsMacro(vdec->slice.qScale, cache, nCacheBits, pBuf, 5);
            FormatMbQuant(vdec, pBuf, nCacheBits+5);
        }

	/*
	 * This initializes mbFlags, too.
	 */
        mbFlags.composite = vdec->slice.qScale << DctMbQuantShift;

        /*
         * if this is an intra coded macro block, then MBT_MFWD, MBT_MBAK and
         * MBT_PATTERN will never be set.
         */
        if (vdec->slice.mbType & MBT_INTRA) {
            vdec->slice.codedBlockPattern = AllBlocksMask;
	    mbFlags.b.intra = True;
        } else {
            PictureInfoPtr pic = vdec->currentPict;
            int motion, pastMotion;
            int complementR;
            int big, little;
            int mvSize, r, code;
            IdctMotionIndex mvi, mvIdxStart, mvIndexLast;
            
            vdec->slice.dcYPast = 128;                     /* per protocol */
            vdec->slice.dcCrPast = 128;                    /* per protocol */
            vdec->slice.dcCbPast = 128;                    /* per protocol */
            
            if (vdec->slice.mbType & MBT_MFWD) {
                mvIdxStart = kIndexFwdX;
                mbFlags.b.fwdMv = True;
            } else {
                mvIdxStart = kIndexBakX;
		if (vdec->tokens.picHdr.type == Mpeg1_P_Pic) {
		    /*
		     * Non-intra coded macroblocks in P pictures without
		     * motion vectors have their forward motion vectors set
		     * to 0
		     */
                    idctMb->motion[ kIndexFwdX ] = 0;
		    pic->pastMotion[ kIndexFwdX ] = 0;
                    idctMb->motion[ kIndexFwdY ] = 0;
		    pic->pastMotion[ kIndexFwdY ] = 0;
		    mbFlags.b.fwdMv = True;
		}
            }
            if (vdec->slice.mbType & MBT_MBAK) {
                mvIndexLast = kIndexBakY;
                mbFlags.b.bakMv = True;
            } else {
                mvIndexLast = kIndexFwdY;
            }
            for (mvi = mvIdxStart; mvi <= mvIndexLast; mvi++) {
                bits = NextBitsMacro(cache, 11);
                hashValue = HuffLookup.motionVector[ bits ];
                bitLength = MotionBits(hashValue);
                FormatMotionCode(vdec, bits, mvi, pBuf, nCacheBits);
                ConsumeBitsMacro(cache, nCacheBits, pBuf, bitLength)
                
                pastMotion = pic->pastMotion[ mvi ];
                if (0 == (code = MotionValue(hashValue))) {
                    idctMb->motion[ mvi ] = pastMotion;
                } else {
                    if ((mvSize = pic->mvSize[ mvi>>1 ]) > 0) {
                        ReadBitsNoFillMacro(r, cache, nCacheBits, mvSize);
                        FormatMotionR(vdec, r, mvi, pBuf, nCacheBits+mvSize);
                    }
                    
                    /*
                     * compute the motion vector
                     */
                    if (mvSize == 0) {        /* same as forward_f == 1 */
                	complementR = 0;
                	/*
                	 * same code as below, except takes advantage of size==0
                	 */
                	if (0 != (little = code)) {
                	    if (little > 0)
                		big = little - (1<<5);
                	    else
                		big = little + (1<<5);
                	} else
                	    big = 0;
                    } else {
                	if (code == 0)
                	    complementR = 0;
                	else
                	    complementR = (1<<mvSize) - 1 - r;
                
                	if (0 != (little = code << mvSize)) {
                	    if (little > 0) {
                		little -= complementR;
                		big = little - (1<<(mvSize+5));
                	    } else {
                		little += complementR;
                		big = little + (1<<(mvSize+5));
                	    }
                	} else
                	    big = 0;
                    }
                
                    motion = pastMotion + little;
                    if (motion < pic->minMotion[ mvi>>1 ]
                     || motion > pic->maxMotion[ mvi>>1 ])
                	motion += big - little;
                    idctMb->motion[ mvi ] = pic->pastMotion[ mvi ] = motion;
                }
            }
            if (NULL != (reason = CheckMotion(vdec, idctMb)))
                break;

            if (vdec->slice.mbType & MBT_PATTERN) {
                bits = NextBitsMacro(cache, 9);
                hashValue = HuffLookup.codedBlock[ bits ];
                bitLength = CodedBlockBits(hashValue);
                vdec->slice.codedBlockPattern =
                    CodedBlockValue(hashValue);
                FormatBlockPattern(vdec, bits, bitLength, pBuf, nCacheBits);
                ConsumeBitsMacro(cache, nCacheBits, pBuf, bitLength);
            } else {
                vdec->slice.codedBlockPattern = 0;
            }
        }

        dctBlk = &idctMb->blk[0];
        blocksMask = 0;
        for (blkIdx = Y0; blkIdx >= CR; blkIdx--) {
	    int blockBit = 1 << blkIdx;

            FormatBlkStart(vdec, blkIdx, pBuf, nCacheBits);
            if ((blockBit & vdec->slice.codedBlockPattern) == 0)
                continue;
            
            blocksMask |= blockBit;
            blkHdr.composite = blkIdx;	/* initialize and set index */
            
            if (vdec->slice.mbType & MBT_INTRA) {
                short dcSize, dcLevel;
                UChar dcDifferential;
                
                /*
                 * For intra blocks, we have to do some special work to
                 * get the DC coefficient.
                 */
                if (blkIdx > CB) {	/* luma vs. chroma */
                    bits = NextBitsMacro(cache, 7);
                    hashValue = HuffLookup.sizeLuminance[ bits ];
                    FormatDcLumaSize(vdec, bits, pBuf, nCacheBits);
                } else {
                    bits = NextBitsMacro(cache, 8);
                    hashValue = HuffLookup.sizeChrominance[ bits ];
                    FormatDcChromaSize(vdec, bits, pBuf, nCacheBits);
                }
                bitLength = SizeYCBits(hashValue);
                ConsumeBitsNoFillMacro(cache, nCacheBits, bitLength);
                if (0 != (dcSize = SizeYCValue(hashValue))) {
                    ReadBitsMacro(dcDifferential, cache, nCacheBits,
                                  pBuf, dcSize);
                    if (dcDifferential & (1 << (dcSize-1)) )
                        dcLevel = dcDifferential;
                    else
                        dcLevel = (~0 << dcSize) | (dcDifferential + 1);
                    dcLevel <<= 3; /* *8 as per protocol */;
                    FormatDcLevel(vdec, dcDifferential, dcSize, dcLevel,
                                  pBuf, nCacheBits+dcSize);
                } else {
                    dcLevel = 0;
                }
                
                /*
                 * Now reconstruct the coefficient.
                 */
                switch(blkIdx) {
                case Y0:
                    if (vdec->slice.mbAddr - vdec->slice.pastIntraMbAddr > 1)
                        dcLevel += 128 * 8;
                    else
                        dcLevel += vdec->slice.dcYPast;
                    vdec->slice.dcYPast = dcLevel;
                    break;
                
                case Y1: case Y2: case Y3:
                    dcLevel += vdec->slice.dcYPast;
                    vdec->slice.dcYPast = dcLevel;
                    break;
                
                case CR:
                    if (vdec->slice.mbAddr - vdec->slice.pastIntraMbAddr > 1)
                        dcLevel += 128 * 8;
                    else
                        dcLevel += vdec->slice.dcCrPast;
                    vdec->slice.dcCrPast = dcLevel;
                    break;
                
                case CB:
                    if (vdec->slice.mbAddr - vdec->slice.pastIntraMbAddr > 1)
                        dcLevel += 128 * 8;
                    else
                        dcLevel += vdec->slice.dcCbPast;
                    vdec->slice.dcCbPast = dcLevel;
                    break;
                }
                blkHdr.s.dcCoeff = dcLevel;
                dctZZIndex = 0;
                pCoeff = &dctBlk->coefList[0].composite;
            } else {	/* non-intra-coded block */
                pCoeff = &dctBlk->coefList[0].composite;
                /* do a dct_coeff_first */
                if (NextBitsMacro(cache, 1)) { /* NOTE 2 for dct_coeff first */
                    negative = (NextBitsMacro(cache, 2) & 1);
                    FormatDctCoeffFirst(vdec, negative, pBuf, nCacheBits);
                    ConsumeBitsNoFillMacro(cache, nCacheBits, 2);
                    dctZZIndex = 0;
                    *pCoeff++ = ComposeDctCoeff(0, negative, 1);
                } else {
                    dctZZIndex = -1;	/* decode the dc coeff below */
                }
            }

            for(;;) {
                /* do a dct_coeff_next */
            
                bits = NextBitsMacro(cache, DCT_COEFF1_LOOKUP_BITS);
                hashValue = HuffLookup.dctCoeff1[ bits ];
            
                /*
                 * If the level is 0, it may be an end_of_block,
                 * an escape or a long code.
                 */
                if ((level = DCTCoeffLevel(hashValue)) == 0) {
                    if (cache>>30 == 0x2) {
                        FormatEndOfBlock(vdec, pBuf, nCacheBits);
                        ConsumeBitsMacro(cache, nCacheBits, pBuf, 2);
                        break;
		    }
                    if (cache>>26 == 1) {
			WhenFormatting( long code; )

			/*
			 * An escape sequence:
			 *
			 *    0000 01  xx xxxx  xxxx xxxx xxxx xxxx
			 *    escape     run      level    level
			 *
			 * First read escape and run
			 */
                        ReadBitsMacro(run, cache, nCacheBits, pBuf, 12);
			WhenFormatting( code = run; )
			run &= 0x3f;	/* remove the escape code */

			level = NextBitsMacro(cache, 16);
                        if ((level >> 8) == 0x80) {
			    WhenFormatting( code = (code << 16) | level; )
                            bitLength = 16;
			    level = 256 - (level & 0xff);
                            negative = True;
                        } else if ((level >> 8) == 0) {
			    WhenFormatting( code = (code << 16) | level; )
                            bitLength = 16;
                            negative = False;
                        } else {
                            bitLength = 8;
			    level >>= 8;	/* don't need the top 8 bits */
			    WhenFormatting( code = (code << 8) | level; )
                            negative = level >> 7;
                            if (negative) {
                                level |= (~0)<<8; /* make it negative */
                                level = -level;   /* and then positive again */
                            }
                        }
                        FormatDctCoeff(vdec, code, bitLength+12, run,
                                       negative, level, pBuf, nCacheBits);
                    } else {
                        /* really only 9 bits... the top 7 are zero */
                        bits = NextBitsMacro(cache, 16);
                        hashValue = HuffLookup.dctCoeff2[ bits ];
                        level = DCTCoeffLevel(hashValue);
                        bitLength = DCTCoeffBits(hashValue);
                        negative = NextBitsMacro(cache, bitLength) & 1;
                        run = DCTCoeffRun(hashValue);
                        FormatDctCoeff(vdec,
                                       NextBitsMacro(cache, bitLength),
                                       bitLength, run,
                                       negative, level, pBuf, nCacheBits);
                    }
                } else {
                    bitLength = DCTCoeffBits(hashValue);
                    run = DCTCoeffRun(hashValue);
                    negative = NextBitsMacro(cache, bitLength) & 1;
                    FormatDctCoeff(vdec, NextBitsMacro(cache, bitLength),
                                   bitLength, run, negative,
                                   level, pBuf, nCacheBits);
                }
            
                dctZZIndex += run + 1;
                if (NULL != (reason = CheckDctIndex(vdec, dctZZIndex)))
                    goto cancelBlockDecode;
                dctIndex = ZigZag[ dctZZIndex ];
                *pCoeff++ = ComposeDctCoeff(dctIndex, negative, level);

                ConsumeBitsMacro(cache, nCacheBits, pBuf, bitLength);
            }
            blkHdr.s.nCoeff = pCoeff - UShortP(&dctBlk->coefList[0]);
            dctBlk->hdr.composite = blkHdr.composite;
            dctBlk = DctCoeffPtrToNextBlock(pCoeff);
        }
        mbFlags.b.blocksMask = blocksMask;
        idctMb->flags.composite = mbFlags.composite;
	FormatMbSummary(vdec, idctMb);
        idctMb = DctBlockPtrToNextMb(dctBlk);
       
        if (vdec->slice.mbType & MBT_INTRA) {
            short *pPastMotion = vdec->currentPict->pastMotion;

            vdec->slice.pastIntraMbAddr = vdec->slice.mbAddr;/* per protocol */
            
            /*
             * per protocol: reset past motion vectors when the
             * last macroblock type was intra
             */
            pPastMotion[ kIndexFwdX ] = 0;		/* per protocol */
            pPastMotion[ kIndexFwdY ] = 0;		/* per protocol */
            pPastMotion[ kIndexBakX ] = 0;		/* per protocol */
            pPastMotion[ kIndexBakY ] = 0;		/* per protocol */
        }
        if (NextBitsMacro(cache, 23) == 0) {
	    FormatSliceEnd(vdec, pBuf, nCacheBits);
	    if (NULL != (reason = CheckEndOfSlice(vdec, pBuf, nCacheBits)))
		goto cancelBlockDecode;
	    if (vdec->parse.nextStartCode < Slice00StartCode
	     || vdec->parse.nextStartCode > SliceAFStartCode) {
                idctMb->flags.b.endOfList = True;
                QueueDctArena(vdec->slice.idct, vdec->slice.curDctList);
	        vdec->slice.curDctList = NULL;
	        vdec->pub.ops.process = vdec->ops.dctProcess;
	    } else {
	        vdec->pub.ops.process = vdec->ops.startCodeProcess;
		vdec->slice.curDctList->curIdctMb = idctMb;
	    }
	    (void) (*vdec->ops.sliceDispatch)(vdec, NULL);
            break;
        }
    }
    
cancelBlockDecode:
    if (reason)
        CancelSliceDecode(vdec);
    return(reason);
}

#ifdef CheckMpeg1VSyntax
#define InitSliceOps Mpeg1VInitSliceOpsSyntax
#else 
#define InitSliceOps Mpeg1VInitSliceOps
#endif 

void
InitSliceOps(Mpeg1VDecPrivPtr vdec)
{
    vdec->ops.sliceInit = Mpeg1SliceInit;
    vdec->ops.sliceProcess = Mpeg1SliceProcess;
}
