/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1vNopSlice.c - noop video slice decoding
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1vsliceNoop.c,v 1.5 1995/08/02 06:27:27 toddb Exp $
 */

#include "mpeg1vdecI.h"
#include "nvrAlloc.h"

static void
ConsumeExtraSliceNoopInfo(Mpeg1VDecPrivPtr vdec)
{
    int bits;
    DeclareBitParseVars(cache, nCacheBits, pBuf);

    FillCacheMacro(&vdec->slice.bitParse, cache, nCacheBits, pBuf);
    
    /*
     * Here we read in (and ignore) the extra slice info.
     */
    for (;;) {
        ReadBitsMacro(bits, cache, nCacheBits, pBuf, 9);
	if ((bits & 1) == 0)
	    break;
    }
    EmptyCacheMacro(&vdec->slice.bitParse, cache, nCacheBits, pBuf);
}

static InternalReasonPtr
Mpeg1SliceNoopInit(Mpeg1VDecPrivPtr vdec, UChar *pDest)
{
    int bits;

    vdec->slice.nBytes = (pDest - vdec->parse.pToken) - 4;
    vdec->slice.pBufferBase = vdec->parse.pToken;
    vdec->slice.pEnd = vdec->slice.pBufferBase + vdec->slice.nBytes;

    InitBitParseMacro(&vdec->slice.bitParse, vdec->slice.pBufferBase)

    /*
     * The vast majority of the time, there is no extra slice info.
     * These few instructions read the first six bits of the slice,
     * which usually contain the quantizer scale (5 bits) and the
     * extra_bit_slice (1 bit) of 0.
     */
    vdec->slice.bitParse.nBits = 2;	/* we know it starts with 8 bits */
    bits = *vdec->slice.bitParse.pBuf >> 2;
    vdec->slice.qScale = bits >> 1;
    if (bits & 1)
        ConsumeExtraSliceNoopInfo(vdec);
    return(NULL);
}

static InternalReasonPtr
Mpeg1SliceNoopProcess(PacketLayerPtr vdecPub)
{
    Mpeg1VDecPrivPtr vdec = (Mpeg1VDecPrivPtr)vdecPub;

    if (vdec->parse.nextStartCode < Mpeg1SliceMinStart
     || vdec->parse.nextStartCode > Mpeg1SliceMaxStart) {
        QueueDctArena(vdec->slice.idct, vdec->slice.curDctList);
	vdec->slice.curDctList = NULL;
	vdec->pub.ops.process = vdec->ops.dctProcess;
	(void) (*vdec->ops.sliceDispatch)(vdec, NULL);
    } else {
        vdec->pub.ops.process = vdec->ops.startCodeProcess;
    }

    return(NULL);
}

void
Mpeg1VInitSliceNoop(Mpeg1VDecPrivPtr vdec)
{
    vdec->ops.sliceInit = Mpeg1SliceNoopInit;
    vdec->ops.sliceProcess = Mpeg1SliceNoopProcess;
}
