/*
 * Copyright (c) 1992,1993,1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1vsyntax.c - macro and function declaration for syntax checking
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1vsyntax.c,v 1.11 1995/10/15 23:48:55 toddb Exp $
 */

/*
 * Force definition here so that we always see the function prototypes
 */
#ifndef CheckMpeg1VSyntax
#define CheckMpeg1VSyntax
#endif

#include "mpeg1vdecI.h"
#include "mpeg1vsyntax.h"

static char *
StartCodeName(ULong startCode)
{
    char *sliceName = "Slice xxxxxxx";
    
    switch (startCode) {
    case PictureStartCode:	return("Picture");
    case UserDataStartCode:	return("UserData");
    case SeqHeaderStartCode:	return("SeqHeader");
    case SeqErrorCode:		return("SeqError");
    case ExtensionStartCode:	return("Extension");
    case SeqenceEndCode:	return("SeqenceEnd");
    case GroupStartCode:	return("Group");
    default:
        if (startCode >= Slice00StartCode
         && startCode <= SliceAFStartCode) {
            sprintf(sliceName + 6, "%d", (int)(startCode & 0xff));
            return (sliceName);
        }
    	return("unknown");
    }
}

InternalReasonPtr
CheckPicExtStartCode(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
   if (vdec->parse.token == kPicUserData) {
       return( UnexpectedVideoStartCode(vdec, pSrc,
		    Mpeg1SliceMinStart, ExtensionStartCode));
   }
   return(NULL);
}

InternalReasonPtr
CheckGopExtStartCode(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
   if (vdec->parse.token == kGopUserData) {
       return( UnexpectedVideoStartCode(vdec, pSrc,
		    PictureStartCode, ExtensionStartCode));
   }
   return(NULL);
}

InternalReasonPtr
CheckSeqExtStartCode(Mpeg1VDecPrivPtr vdec, UChar *pSrc)
{
   if (vdec->parse.token == kSeqUserData) {
       return( UnexpectedVideoStartCode(vdec, pSrc,
		    GroupStartCode, ExtensionStartCode));
   }
   return(NULL);
}

InternalReasonPtr
CheckGopHdr(Mpeg1VDecPrivPtr vdec)
{
    vdec->syntax.gopStart = True;

    return(NULL);
}

static char *
PicString(Mpeg1VDecPrivPtr vdec, char *buf)
{
    Mpeg1PictureHdrPtr pic = &vdec->tokens.picHdr;

    sprintf(buf, "%c picture #%d @ byte %d",
	IPBDchars[ pic->type ], pic->timeRef,
	(int)vdec->cb.arg.tokenFileOffset);
    return(buf);
}

InternalReasonPtr
CheckCurPicHeader(Mpeg1VDecPrivPtr vdec)
{
    char addr[ 256 ];
    short timeRef;
    static InternalReason dpicNotSupported = {
        "D picture type not supported\n", IR_NotImplemented
    };
    InternalReasonPtr irp = NULL;
    Mpeg1PictureHdrPtr pic = &vdec->tokens.picHdr;
    
    if (vdec->syntax.gopStart) {
	if (vdec->tokens.picHdr.type != Mpeg1_I_Pic) {
	    return( BadSyntaxReason("%s:\n  %s\n",
		    PicString(vdec, addr),
		    "First picture in GOP must be an I picture"));
	}
	vdec->syntax.gopStart = False;
	vdec->syntax.minTimeRef = -1;
	vdec->syntax.maxTimeRef = -1;
    }

    if ((int)pic->timeRef <= vdec->syntax.minTimeRef) {
	return( BadSyntaxReason("%s:\n  %s\n",
		PicString(vdec, addr),
		"picture time reference moving backwards"));
    }

    switch (vdec->tokens.picHdr.type) {
    case Mpeg1_P_Pic:
    case Mpeg1_I_Pic:
	vdec->syntax.minTimeRef = vdec->syntax.maxTimeRef;
	vdec->syntax.maxTimeRef = pic->timeRef;
	vdec->syntax.curTimeRef = pic->timeRef;
	break;
	
    case Mpeg1_B_Pic:
	timeRef = vdec->syntax.minTimeRef + 1;
	if (pic->timeRef != timeRef) {
	    irp = BadSyntaxReason("%s:\n  %s %d\n",
		    PicString(vdec, addr),
		    "picture time referece should be", timeRef);
	}
	vdec->syntax.minTimeRef = timeRef;
	if (pic->timeRef >= vdec->syntax.maxTimeRef) {
	    irp = BadSyntaxReason("%s:\n  %s\n",
		    PicString(vdec, addr),
		    "too many B pictures for this sub-GOP");
	}
	break;
	
    case Mpeg1_D_Pic:
	irp = &dpicNotSupported;
	break;

    default:
	irp = BadValueReason("bad picture type: %d\n",
				vdec->tokens.picHdr.type);
	break;
    }

    return(irp);
}

static char *
SliceString(Mpeg1VDecPrivPtr vdec, char *buf)
{
    Mpeg1PictureHdrPtr pic = &vdec->tokens.picHdr;

    sprintf(buf, "%c picture #%d, slice #%d @ byte %d",
	IPBDchars[ pic->type ], pic->timeRef,
	vdec->slice.vertPos, (int)vdec->cb.arg.tokenFileOffset);
    return(buf);
}

InternalReasonPtr
CheckSliceQuantizer(Mpeg1VDecPrivPtr vdec)
{
    InternalReasonPtr irp;
    char addr[ 128 ];

    vdec->syntax.sliceStart = True;
    if (vdec->slice.mbAddr == -1)
	vdec->syntax.firstMbInSlice = 0;
    else
	vdec->syntax.firstMbInSlice = vdec->syntax.prevMb + 1;
    vdec->syntax.prevMb = vdec->slice.mbAddr;

    if (vdec->slice.qScale == 0) {
        irp = BadSyntaxReason("%s:\n  invalid 0 quantizer\n",
	    SliceString(vdec, addr));
	return(irp);
    }

    return(NULL);
}

InternalReasonPtr
CheckSliceAddr(Mpeg1VDecPrivPtr vdec)
{
    InternalReasonPtr reason = NULL;
    char addr[ 128 ];

    if (vdec->parse.nextStartCode < Slice00StartCode
     || vdec->parse.nextStartCode > SliceAFStartCode) {
        reason = BadSyntaxReason(
            "%s:\n  Expected slice start code (0x%x - 0x%x), got 0x%x (%s)\n",
	    SliceString(vdec, addr),
            Slice00StartCode, SliceAFStartCode, vdec->parse.nextStartCode,
            StartCodeName(vdec->parse.nextStartCode));
    }
    return(reason);
}

static char *
MbAddrString(Mpeg1VDecPrivPtr vdec, char *buf)
{
    Mpeg1PictureHdrPtr pic = &vdec->tokens.picHdr;

    sprintf(buf, "%c picture #%d, slice #%d @ byte %d, mb %d (mb %d in slice)",
	IPBDchars[ pic->type ], pic->timeRef,
	vdec->slice.vertPos, (int)vdec->cb.arg.tokenFileOffset,
	(int)vdec->slice.mbAddr,
	(int)(vdec->slice.mbAddr - vdec->syntax.firstMbInSlice));
    return(buf);
}

InternalReasonPtr
CheckMbAddr(Mpeg1VDecPrivPtr vdec, int bitLength, int inc)
{
    InternalReasonPtr reason = NULL;
    char *name = "macroblock address";
    char addr[ 128 ];

    if (inc > 33) {
        reason = BadSyntaxReason("%s:\n  %s increment is > 33: %d\n",
		    MbAddrString(vdec, addr), name, inc);
    } else if (bitLength > 11 || bitLength < 0) {
        reason = BadSyntaxReason(
            "%s:\n  %s bitlength (%d) is invalid (must be 0-11)\n",
            MbAddrString(vdec, addr), name, bitLength);
    } else if (vdec->slice.mbAddr > vdec->decode.lastMacroBlock) {
        reason = BadSyntaxReason("%s:\n  %s %d is past last macroblock (%d)\n",
				 MbAddrString(vdec, addr),
                                 name, vdec->slice.mbAddr,
				 vdec->decode.lastMacroBlock);
    } else if (vdec->syntax.sliceStart) {
        if (vdec->slice.mbAddr != vdec->syntax.firstMbInSlice) {
            reason = BadSyntaxReason(
                "%s:\n  %d skipped macroblocks starting at mb %d\n",
		MbAddrString(vdec, addr),
                vdec->slice.mbAddr - vdec->syntax.firstMbInSlice,
                vdec->syntax.firstMbInSlice);
        }
        vdec->syntax.sliceStart = False;
    } else if (vdec->slice.mbAddr - vdec->syntax.prevMb > 1) {
        /*
         * Make sure skipped macroblocks in B pictures do not occur
         * following an intra-coded macroblock.
         */
        if (vdec->tokens.picHdr.type == Mpeg1_B_Pic
         && (vdec->slice.mbType & MBT_INTRA)) {
            reason = BadSyntaxReason(
                "%s:\n  %d skipped macroblocks after an intra coded block\n",
		MbAddrString(vdec, addr),
                vdec->slice.mbAddr - vdec->syntax.prevMb);
        }
    }
    
    vdec->syntax.prevMb = vdec->slice.mbAddr;

    return(reason);
}

InternalReasonPtr
CheckMbType(Mpeg1VDecPrivPtr vdec, int bitLength)
{
    InternalReasonPtr reason;
    int min, max;
    char addr[ 128 ];

    switch (vdec->tokens.picHdr.type) {
    case Mpeg1_B_Pic: min=2; max=6; break;    
    case Mpeg1_P_Pic: min=1; max=6; break;
    default: /* Mpeg1_I_Pic or Mpeg1_D_Pic */ min=1; max=2; break;
    }
    if (bitLength < min || bitLength > max) {
        reason = BadSyntaxReason("%s:\n  %s (%d) for %s %d and %d, inclusive\n",
		MbAddrString(vdec, addr), "Bad bitlength", bitLength,
                "macroblock type.\n  Must be between", min, max);
        return( reason );
    }
    
    if ((vdec->slice.mbType & MBT_INTRA)
     && (vdec->slice.mbType & (MBT_MFWD|MBT_MBAK|MBT_PATTERN))) {
        reason = BadSyntaxReason(
                "%s:\n  %s (0x%x).\n",
		MbAddrString(vdec, addr),
                "Bad intra macroblock type bits", vdec->slice.mbType);
        return( reason );
    }
    return(NULL);
}

InternalReasonPtr
CheckMotion(Mpeg1VDecPrivPtr vdec, DctMacroblockPtr idctMb)
{
    PictureInfoPtr pic = vdec->currentPict;
    IdctMotionIndex mvi, mvIdxStart, mvIndexLast;
    int x = vdec->slice.mbAddr % vdec->decode.mbWidth << 4;
    int y = vdec->slice.mbAddr / vdec->decode.mbWidth << 4;
    int width = vdec->decode.mbWidth << 4;
    int height = vdec->decode.mbHeight << 4;
    int delta, i;
    boolean halfPel;
    short final[ 4 ], ref[ 4 ];
    boolean oob;
    short dim[ 4 ];
    char errMsg[ 256 ], *pErrMsg = errMsg;
    char *oobParts[ 16 ];
    char addr[ 128 ];
    enum {
        kFwdXMask  = 1<<kIndexFwdX,
        kFwdYMask  = 1<<kIndexFwdY,
        kFwdXYMask = kFwdXMask | kFwdYMask,

        kBakXMask  = 1<<kIndexBakX,
        kBakYMask  = 1<<kIndexBakY,
        kBakXYMask = kBakXMask | kBakYMask
    };
    
    oob = 0;
    for (i = 0; i < NumberOfCells(oobParts); i++)
        oobParts[ i ] = "??";
    
    if (vdec->slice.mbType & MBT_MFWD) {
        mvIdxStart = kIndexFwdX;
        oobParts[ kFwdXMask ] = "fwd x";
        oobParts[ kFwdYMask ] = "fwd y";
        oobParts[ kFwdXYMask ] = "fwd x and y";
    } else {
        mvIdxStart = kIndexBakX;
    }
    if (vdec->slice.mbType & MBT_MBAK) {
        mvIndexLast = kIndexBakY;
        oobParts[ kBakXMask ] = "bak x";
        oobParts[ kBakYMask ] = "bak y";
        oobParts[ kBakXYMask ] = "bak x and y";
    } else {
        mvIndexLast = kIndexFwdY;
    }
    
    ref[ kIndexFwdX ] = ref[ kIndexBakX ] = x;
    ref[ kIndexFwdY ] = ref[ kIndexBakY ] = y;
    dim[ kIndexFwdX ] = dim[ kIndexBakX ] = width;
    dim[ kIndexFwdY ] = dim[ kIndexBakY ] = height;

    for (mvi = mvIdxStart; mvi <= mvIndexLast; mvi++) {
        delta = idctMb->motion[ mvi ];
        if (! pic->fullPel[ mvi>>1 ]) {
            halfPel = delta & 1;
            delta >>= 1;
        } else {
            halfPel = False;
        }
        final[ mvi ] = ref[ mvi ] + delta;
        if (final[ mvi ] < 0 || final[ mvi ] + halfPel >= dim[ mvi ])
            oob |= 1<<mvi;
    }
    
    if (! oob)
        return(NULL);

    for (mvi = kIndexFwd; mvi <= kIndexBak; mvi++) {
        int base = mvi << 2;
        int mask = (1<<base) | (1<<(base+1));
        int deltax, deltay;
        char *halfx, *halfy;

        halfPel = ! pic->fullPel[ mvi ];
        if (oob & mask) {
            deltax = idctMb->motion[ base ] >> halfPel;
            halfx = (idctMb->motion[ base ] & halfPel) ? "+h" : "";
            deltay = idctMb->motion[ base+1 ] >> halfPel;
            halfy = (idctMb->motion[ base+1 ] & halfPel) ? "+h" : "";
            
            sprintf(pErrMsg, "   %s out of bounds: ", oobParts[ oob & mask ]);
            pErrMsg += strlen(pErrMsg);
            sprintf(pErrMsg, "ref=(%d,%d)+delta=(%d%s,%d%s) %s pel ",
                x, y, deltax, halfx, deltay, halfy, halfPel ? "half" : "full");
            pErrMsg += strlen(pErrMsg);
            sprintf(pErrMsg, "==> (%d,%d).\n", final[ base ], final[ base+1 ]);
            pErrMsg += strlen(pErrMsg);
        }
    }

    return( BadSyntaxReason("%s:\n%s", MbAddrString(vdec, addr), errMsg) );
}

InternalReasonPtr
CheckDctIndex(Mpeg1VDecPrivPtr vdec, int dctZZIndex)
{
    InternalReasonPtr reason = NULL;
    char addr[ 128 ];

    if (dctZZIndex < 0 || dctZZIndex > 63)
        reason = BadSyntaxReason("%s:\n  Invalid dct index: %d\n",
		    MbAddrString(vdec, addr), dctZZIndex);
    return(reason);
}

InternalReasonPtr
CheckEndOfSlice(Mpeg1VDecPrivPtr vdec, UChar *pSrc, int nCacheBits)
{
    int nonZeroBytes = 0, nPadBytes;
    InternalReasonPtr reason = NULL;
    char addr[ 128 ];

    pSrc -= nCacheBits >> 3;
    nPadBytes = vdec->slice.pEnd - pSrc;
    while (pSrc < vdec->slice.pEnd)
	if (*pSrc++)
	    nonZeroBytes++;
    if (nonZeroBytes) {
        reason = BadSyntaxReason("%s:\n  %d of %d %s\n",
		    MbAddrString(vdec, addr), nonZeroBytes, nPadBytes,
		    "padding bytes at slice end are non-null");
    }
    return(reason);
}
