/*
 *	(c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		mpegsched.c - top-level mpeg scheduler
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/mpegsched.c,v 1.17 1996/01/18 23:45:54 toddb Exp $
 */
#include "mpegschedI.h"
#include "schedStat.h"

static struct {
    boolean quarterSample;
    boolean monoSample;
} AQualities[ MSC_NQualitySteps ] = {
    { True,  True },
    { True,  False },
    { False, True },
    { False, False }
};

static char *CommandNames[] = {
    "play fwd",
    "play all",
    "play bak",
    "fast fwd",
    "fast bak",
    "step fwd",
    "step bak",
    "stop",
    "seek",
    "open seq",
    "select channel",
    "load firstFrame",
};

/****************************************************************************
 *
 * Debug functions and data
 *
 ****************************************************************************/
#if defined(DEBUG_ENABLED)

static void
FlushSchedStats(MpegSchedPrivPtr msp)
{
    if (msp->schedStat.fp)
	FlushBufferedFile(msp->schedStat.fp);
}

static void
OpenSchedStats(MpegSchedPrivPtr msp)
{
    BufFileHndl fp;
    SchedSequence seq;
    ClockKeepPtr clock = msp->sched.clock;
    ULong *ticks = clock->clocks;

    if (msp->schedStat.fp == NULL)
	msp->schedStat.fp = OpenBufferedFileForWriting("schedStats");
    fp = msp->schedStat.fp;

    if (fp == NULL)
	return;

    seq.type = SR_sequence;
    seq.channels = msp->input.au.channels;
    seq.width = msp->input.width;
    seq.height = msp->input.height;
    seq.fps = FrameRateTable[ msp->input.seqHdr.pictureRate ];
    seq.msPerAFrame = msp->sched.rtMsPerAFrame;
    seq.sample = msp->input.au.rate;
    BufferedWriteData(fp, (char *)&seq, sizeof(seq));

    msp->schedStat.input.nFrames = msp->input.nFrames;
    msp->schedStat.output.nFrames = msp->output.nFrames;
    msp->schedStat.input.nSamples = msp->input.nSamples;
    msp->schedStat.output.nSamples = msp->output.nSamples;

    msp->schedStat.dpy.last = ticks[ DisplayClockIdx ];
    msp->schedStat.vid.last = ticks[ VideoClockIdx ];
    msp->schedStat.au.last = ticks[ AudioClockIdx ];

    msp->schedStat.samplesPerFrameRecip = 1.0 / msp->output.samplesPerFrame;
}

static void
UpdateSchedStats(MpegSchedPrivPtr msp, long now)
{
    BufFileHndl fp = msp->schedStat.fp;
    SchedRecord rec;
    int nSamples;
    ClockKeepPtr clock = msp->sched.clock;
    ULong *ticks = clock->clocks;
    boolean nothing = True;

    if (fp == NULL)
	return;

    nSamples    = msp->input.nSamples - msp->schedStat.input.nSamples;
    if (nSamples) {
	nothing = False;
	rec.auIn    = nSamples * msp->schedStat.samplesPerFrameRecip;
	rec.auCost = msp->schedStat.au.cost
	           + (ticks[ AudioClockIdx ] - msp->schedStat.au.last);
	msp->schedStat.au.cost = 0;
	msp->schedStat.au.last = ticks[ AudioClockIdx ];
	msp->schedStat.input.nSamples = msp->input.nSamples;
    } else {
	rec.auIn = 0;
    }

    nSamples    = msp->output.nSamples - msp->schedStat.output.nSamples;
    if (nSamples) {
	nothing = False;
	rec.auOut   = nSamples * msp->schedStat.samplesPerFrameRecip;
	msp->schedStat.output.nSamples = msp->output.nSamples;
    } else {
	rec.auOut = 0;
    }

    rec.pictIn  = msp->input.nFrames   - msp->schedStat.input.nFrames;
    if (rec.pictIn) {
	nothing = False;
	rec.pictInCost = msp->schedStat.vid.cost
		       + (ticks[ VideoClockIdx ] - msp->schedStat.vid.last);
	msp->schedStat.vid.cost = 0;
	msp->schedStat.vid.last = ticks[ VideoClockIdx ];
	msp->schedStat.input.nFrames = msp->input.nFrames;
    }

    rec.pictOut = msp->output.nFrames  - msp->schedStat.output.nFrames;
    if (rec.pictOut) {
	nothing = False;
	rec.pictLate = msp->sched.pictLate;
	rec.pictOutCost = msp->schedStat.dpy.cost
		        + (ticks[ DisplayClockIdx ] - msp->schedStat.dpy.last);
	msp->schedStat.dpy.cost = 0;
	msp->schedStat.dpy.last = ticks[ DisplayClockIdx ];
	msp->schedStat.output.nFrames = msp->output.nFrames;
    } else {
	rec.pictLate = 0;
    }

    if (nothing)
	return;

    rec.type = SR_record;
    rec.time = now;

    BufferedWriteData(fp, (char *)&rec, sizeof(rec));
}

static void
UpdateBalanceStats(MpegSchedPrivPtr msp, long now)
{
    YcImagePtr yci;
    long qSize = 0;
    SchedBalance bal;
    BufFileHndl fp = msp->schedStat.fp;
    ClockKeepPtr clock = msp->sched.clock;
    ULong *ticks = clock->clocks;

    msp->schedStat.dpy.cost +=
	ticks[ DisplayClockIdx ] - msp->schedStat.dpy.last;
    msp->schedStat.vid.cost +=
	ticks[ VideoClockIdx ] - msp->schedStat.vid.last;
    msp->schedStat.au.cost +=
	ticks[ AudioClockIdx ] - msp->schedStat.au.last;
    msp->schedStat.dpy.last = 0;
    msp->schedStat.vid.last = 0;
    msp->schedStat.au.last = 0;
    msp->schedStat.input.nFrames = 0;
    msp->schedStat.input.nSamples = 0;
    msp->schedStat.output.nFrames = 0;
    msp->schedStat.output.nSamples = 0;

    bal.vinWeight = msp->sched.vinWeight;
    bal.audioFrac = msp->balanceStat.audioFrac;
    bal.videoFrac = msp->balanceStat.videoFrac;
    bal.targetFps = msp->sched.targetFps;
    bal.dropped = msp->balanceStat.dropped;
    msp->balanceStat.dropped = 0;
    for (yci = msp->output.imgQ; yci; yci = yci->next)
	qSize++;
    bal.qSize = qSize;
    bal.time = now;
    bal.type = SR_balance;

    if (qSize > 15)
	Msg("q=%d ", qSize);
    if (fp)
	BufferedWriteData(fp, (char *)&bal, sizeof(bal));
}

static char *SchedDebugStrings[] = {
    "balance info",
    "sched stats",
    "frame info",
};

FlagMacroList SchedDebugMacros = { NULL, 0 };

FlagNames SchedDebugNames = {
    DBI_MpegSched,
    SchedDebugStrings,
    NumberOfCells(SchedDebugStrings)
};

#else /* DEBUG_ENABLED */

FlagMacroList SchedDebugMacros = { NULL, 0 };
FlagNames SchedDebugNames = { DBI_MpegSched, NULL, 0 };

#endif /* DEBUG_ENABLED */

/****************************************************************************
 *
 * scheduler functions
 *
 ****************************************************************************/
void
MpegSchedFlushCommands(MpegSchedPrivPtr msp)
{
    while (! QueueEmpty(&msp->sched.cmdq)) {
        RemoveQueueTail(&msp->sched.cmdq);
    }
}

static ComplexCmdPtr
AllocComplexCmd(MpegSchedPrivPtr msp,
		SchedCommand cmd,
		int extra,
		InternalReasonPtr *pIrp)
{
    ComplexCmdPtr cp;
    int allocSize;

    /*
     * If there is an available command buffer, use it
     */
    if (msp->sched.freeCmd) {
	cp = msp->sched.freeCmd;
	if (cp->extra >= extra) {
	    msp->sched.freeCmd = cp->next;
	    cp->next = NULL;	/* paranoid */
	    cp->cmd = cmd;
	    return(cp);
	}
    }

    allocSize = sizeof(ComplexCmd) + extra;

    cp = (ComplexCmdPtr) AllocNamedMem(allocSize, "scheduler command", pIrp);
    if (cp == NULL)
	return(cp);
    cp->next = NULL;
    cp->cmd = cmd;
    cp->extra = extra;

    return(cp);
}

static void
FreeComplexCmd(MpegSchedPrivPtr msp, ComplexCmdPtr cmd)
{
    cmd->next = msp->sched.freeCmd;
    msp->sched.freeCmd = cmd;
}

static InternalReasonPtr
QueueComplexCommand(MpegSchedPrivPtr msp, ComplexCmdPtr cmd)
{
    if (QueueFull(&msp->sched.cmdq)) {
	if ((long)cmd < kCommandCount) {
	    Msg("cmd q full, dropped %s\n", CommandNames[ (long)cmd ]);
	} else {
	    Msg("cmd q full, dropped %s\n", CommandNames[ cmd->cmd ]);
	    FreeComplexCmd(msp, cmd);
	}
        return(NULL);
    }
    
    InsertQueueHead(&msp->sched.cmdq, cmd);
    msp->pub.needToProcess = True;
    return(NULL);
}

static InternalReasonPtr
QueueCommand(MpegSchedPrivPtr ms, SchedCommand cmd)
{
    return( QueueComplexCommand(ms, (ComplexCmdPtr)cmd) );
}

static InternalReasonPtr
QueueOpenCommand(MpegSchedPrivPtr msp, MpegSchedInputSpecPtr input)
{
    InternalReasonPtr irp;
    ComplexCmdPtr cp;
    int videoLen, audioLen;
    static InternalReason noSpec = { "mpeg input unspecified\n",
    					IR_BadValue };

    videoLen = audioLen = 0;
    switch (input->type) {
    case MSIT_PathPair:
	videoLen = strlen(input->pathPair.video) + 1;
	audioLen = strlen(input->pathPair.audio) + 1;
	break;

    case MSIT_Unspecified:    return( &noSpec );
    case MSIT_SinglePath:     videoLen = strlen(input->single.path) + 1; break;
    case MSIT_Descriptor:     break;
    case MSIT_DescriptorPair: break;
    case MSIT_TcpPort:        break;
    }
    

    if (NULL == (cp = AllocComplexCmd(msp, kOpenSeq, videoLen+audioLen, &irp)))
        return(irp);

    cp->u.input.type = input->type;
    switch (input->type) {
    case MSIT_PathPair:
	cp->u.input.pathPair.video = (char *)(cp + 1);
	cp->u.input.pathPair.audio = cp->u.input.pathPair.video + videoLen;
	strcpy(cp->u.input.pathPair.video, input->pathPair.video);
	strcpy(cp->u.input.pathPair.audio, input->pathPair.audio);
	break;

    case MSIT_SinglePath:
	cp->u.input.single.path = (char *)(cp + 1);
	strcpy(cp->u.input.single.path, input->single.path);
	break;

    default:
	break;	/* handled elsewhere */
    }

    return( QueueComplexCommand(msp, cp) );
}

static InternalReasonPtr
MpsPlayFwd(MpegSchedPtr ms)
{
    return( QueueCommand((MpegSchedPrivPtr)ms, kPlayFwd) );
}

static InternalReasonPtr
MpsPlayAll(MpegSchedPtr ms)
{
    return( QueueCommand((MpegSchedPrivPtr)ms, kPlayAll) );
}

static InternalReasonPtr
MpsPlayBak(MpegSchedPtr ms)
{
    return( QueueCommand((MpegSchedPrivPtr)ms, kPlayBak) );
}

static InternalReasonPtr
MpsFastFwd(MpegSchedPtr ms)
{
    return( QueueCommand((MpegSchedPrivPtr)ms, kFastFwd) );
}

static InternalReasonPtr
MpsFastBak(MpegSchedPtr ms)
{
    return( QueueCommand((MpegSchedPrivPtr)ms, kFastBak) );
}

static InternalReasonPtr
MpsStepFwd(MpegSchedPtr ms)
{
    return( QueueCommand((MpegSchedPrivPtr)ms, kStepFwd) );
}

static InternalReasonPtr
MpsStepBak(MpegSchedPtr ms)
{
    return( QueueCommand((MpegSchedPrivPtr)ms, kStepBak) );
}

static InternalReasonPtr
MpsStop(MpegSchedPtr ms)
{
    return( QueueCommand((MpegSchedPrivPtr)ms, kStop) );
}

static InternalReasonPtr
MpsSeek(MpegSchedPtr ms, MpegSchedSeekTargetPtr target)
{
    MpegSchedPrivPtr msp = (MpegSchedPrivPtr)ms;
    InternalReasonPtr irp;
    ComplexCmdPtr cp;

    if (NULL == (cp = AllocComplexCmd(msp, kSeek, 0, &irp)))
	return(irp);
    
    switch (target->type) {
    default:
	FreeComplexCmd(msp, cp);
        return( BadValueReason("invalid seek type specified: %d\n",
			       target->type));

    case MSST_FileOffset:
    case MSST_ScanTimecode:
    case MSST_RelativeTimecode:
    case MSST_FilePairOffset:
        break;
    }

    cp->u.seek = *target;
    return( QueueComplexCommand(msp, cp) );
}

static void
MpegSystemTokenCb(PacketLayerPtr pLayer,
                  Mpeg1CbType cbtype,
                  Mpeg1MuxCbInfoPtr cbData)
{
    switch(cbtype) {
    case Mpeg1PacketCb:
    case Mpeg1PackCb:
    case Mpeg1SysHeaderCb:
    case Mpeg1EndCodeCb:
    case Mpeg1PacketDoneCb:
    default:
        break;  /* don't care */
    }
}

static void
InstallSequenceHeader(MpegSchedPrivPtr msp, Mpeg1VideoHdrPtr newSeq)
{
    Mpeg1VideoHdrPtr oldSeq = &msp->input.seqHdr;
    boolean different = False;
    
    /*
     * First, note changes and make appropriate changes
     */
    if (oldSeq->width != newSeq->width
     || oldSeq->height != newSeq->height) {
	msp->cb.arg.width = newSeq->width;
	msp->cb.arg.height = newSeq->height;
	msp->cb.mask |= MSCB_VidChange;
	different = True;
    }

    if (oldSeq->pictureRate != newSeq->pictureRate) {
	msp->cb.arg.frameRate = FrameRateTable[ newSeq->pictureRate ];
	msp->sched.rtMsPerVFrame = MilliSecPerSecond / msp->cb.arg.frameRate;
	msp->cb.mask |= MSCB_VidChange;
        different = True;
    }

    /*
     * Finally, if there are changes, install the new sequence header
     */
    if (different) {
	msp->cb.mask |= MSCB_NewVideo;
	msp->input.seqHdr = *newSeq;
    }
}

static void
TransitFunction(MpegSchedPrivPtr msp, SchedState remove)
{
    SchedState state = msp->sched.state & ~remove;

    switch (state) {
    case kPending | kActive:
	/*
	 * This was one of the Wait states, with no waiting left
	 */
	state = kStopped;
	break;

    case kWaitVideoHdr:
    case kWaitAudioHdr:
    case kWaitAVHdr:
	break;

    case kWaitVideo:
    case kWaitAudio:
    case kWaitAV:
	break;

    default:
    case kStopped:
    case kPlaying:
	break;
    }
    msp->sched.state = state;
}

static short
PushClock(MpegSchedPrivPtr msp, short clockIdx)
{
    ClockKeepPtr clock = msp->sched.clock;
    short curClockIdx = clock->curClock;

    if (clockIdx != curClockIdx)
	(*clock->update)(clock, clockIdx);
    return(curClockIdx);
}

static void
PopClock(MpegSchedPrivPtr msp, short clockIdx)
{
    ClockKeepPtr clock = msp->sched.clock;

    if (clockIdx != clock->curClock)
	(*clock->update)(clock, clockIdx);
}

static void
DeliverReason(MpegSchedPrivPtr msp, InternalReasonPtr irp)
{
    ULong mask = msp->cb.mask | MSCB_Reason;
    short clockIdx;

    msp->cb.mask = 0;
    msp->sched.timeToYield = True;
    MpegSchedFlushCommands(msp);
    msp->sched.state = kStopped;

    clockIdx = PushClock(msp, OtherClockIdx);
    msp->cb.arg.reason = irp;
    (*msp->cb.cb)(&msp->pub, &msp->cb.arg, mask);
    msp->cb.arg.reason = NULL;
    PopClock(msp, clockIdx);
}

static void
CallMpegSchedCallback(MpegSchedPrivPtr msp)
{
    ULong mask = msp->cb.mask;

    if (mask) {
	msp->cb.mask = 0;
	(*msp->cb.cb)(&msp->pub, &msp->cb.arg, mask);
    }
}

static void
QueueDisplayFrame(MpegSchedPrivPtr msp, YcImagePtr yci)
{
    if (msp->output.imgQ == NULL) {
	msp->output.imgQLast = msp->output.imgQ = yci;
	msp->output.imgQ->next = NULL;
    } else {
	msp->output.imgQLast->next = yci;
	yci->next = NULL;
	msp->output.imgQLast = yci;
    }
}

static void
VideoTokenCb(PacketLayerPtr pLayer, Mpeg1VCbType cbType, Mpeg1VCbInfoPtr cb)
{
    MpegSchedPrivPtr msp = (MpegSchedPrivPtr)cb->cbData;
    short clockIdx;

    switch (cbType) {
    case Mpeg1VNewSeqCb:
	InstallSequenceHeader(msp, &cb->pTokens->seqHdr);
	if (msp->sched.state & kWaitVideoHdrBit) {
	    msp->cb.mask |= MSCB_NewVideo;
	    TransitFunction(msp, kWaitVideoHdrBit);
	}
	break;

    case Mpeg1VGopCb:
    case Mpeg1VPicCb:
    case Mpeg1VSliceCb:
    case Mpeg1VEndCodeCb:
    case Mpeg1VPacketFree:
    default:
        break;

    case Mpeg1VDecodeStartCb:
        break;

    case Mpeg1VDecodeEndCb:
        break;

    case Mpeg1VDisplayCb:
	msp->input.nFrames++;
	QueueDisplayFrame(msp, cb->image);
	if (msp->sched.state & kWaitVideoBit) {
	    clockIdx = PushClock(msp, DisplayClockIdx);
	    msp->output.nFrames++;
	    (*msp->output.dpy->displayYcImage)(msp->output.dpy,
					       msp->output.imgQ);
	    TransitFunction(msp, kWaitVideoBit);
	    PopClock(msp, clockIdx);
	}
        break;
    }
}

static InternalReasonPtr
AllocAudioBuf(MpegSchedPrivPtr msp, boolean configAudio)
{
    AudioBufPtr abuf;
    InternalReasonPtr irp = NULL;
    Mpeg1AudioConf audioConf;

    /*
     * If there is a current audio buffer, put it on the delivery queue
     */
    if (msp->output.audQCurrent) {
	msp->output.audQCurrent->next = NULL;
	if (msp->output.audQ) {
	    msp->output.audQLast->next = msp->output.audQCurrent;
	    msp->output.audQLast = msp->output.audQCurrent;
	} else {
	    msp->output.audQ = msp->output.audQCurrent;
	    msp->output.audQLast = msp->output.audQ;
	}
	msp->output.audQCurrent = NULL;
    }

    if (msp->output.audQFree) {
	msp->output.audQCurrent = msp->output.audQFree;
	msp->output.audQFree = msp->output.audQFree->next;
    } else {
	long allocSize = msp->output.nBytesPerBuffer + sizeof(AudioBuf);
	abuf = (AudioBufPtr) AllocNamedMem(allocSize,
					   "scheduler's audio buffer",
					   &irp);
	if (abuf == NULL)
	    return(irp);
	abuf->buffer = (UChar *)(abuf + 1);
	abuf->fullLength = msp->output.nBytesPerBuffer;
	abuf->inputLength = msp->output.nBytesPerInputBuffer;
	msp->output.audQCurrent = abuf;
    }
    msp->output.audQCurrent->pBuf = msp->output.audQCurrent->buffer;

    if (configAudio) {
	audioConf.sampleBuf = msp->output.audQCurrent->buffer;
	audioConf.bufLength = msp->output.audQCurrent->inputLength;
	irp = Mpeg1AConfigure(msp->input.aLayer, &audioConf,
			      Mpeg1AConfSamplesMask);
    }

    return( irp );
}

/*
 * This either puts the available buffer on the free queue, or, if
 * it is the wrong size, it frees it.
 */
static void
FreeAudioBuf(MpegSchedPrivPtr msp, AudioBufPtr ap)
{
    if (ap->fullLength == msp->output.nBytesPerBuffer
     && ap->inputLength == msp->output.nBytesPerInputBuffer) {
	ap->next = msp->output.audQFree;
	msp->output.audQFree = ap;
    } else {
	ap->next = NULL;	/* paranoid */
	free((char *)ap);
    }
}

/*
 * This Calls FreeAudioBuf for the current and all unused buffers,
 * then calls AllocAudioBuf().  If the sizes are all correct, the
 * net effect will be that nothing gets freed or allocated.
 */
static InternalReasonPtr
ReallocAudioBuf(MpegSchedPrivPtr msp)
{
    AudioBufPtr abuf, abufNext;

    /*
     * Since FreeAudioBuf() may put it back on the free list, take
     * all the buffers off the queue.
     */
    abuf = msp->output.audQFree;
    msp->output.audQFree = NULL;
    
    while (abuf) {
	abufNext = abuf->next;
	FreeAudioBuf(msp, msp->output.audQCurrent);
	abuf = abufNext;
    }

    if (NULL != (abuf = msp->output.audQCurrent)) {
	msp->output.audQCurrent = NULL;
	FreeAudioBuf(msp, abuf);
    }

    if (msp->input.aLayer)
	return( AllocAudioBuf(msp, True) );
    return(NULL);
}

static void
CalcAudioBufferSize(MpegSchedPrivPtr msp) 
{
    long samplesPerFrame, framesPerBuffer;
    float msPerAFrame;
    InternalReasonPtr irp;
    long dataShift = DataFormToShift[ msp->input.au.dataForm ];

    irp = InitAudioConversion(&msp->output.auCvt,
			      &msp->input.au,
			      &msp->output.au);
    if (irp) {
	DeliverReason(msp, irp);
	return;
    }

    samplesPerFrame = msp->output.nBytesPerFrame >> dataShift;
    msp->output.samplesPerFrame = samplesPerFrame;

    msPerAFrame = MilliSecPerSecond * (float)samplesPerFrame
		/ (msp->input.au.rate * msp->input.au.channels);
    framesPerBuffer = AudioBufferMs / msPerAFrame + (float)0.5;
    msp->sched.rtMsPerASample = msPerAFrame / samplesPerFrame;
    msp->sched.rtMsPerAFrame = msPerAFrame;

    msp->output.nBytesPerInputBuffer =
	framesPerBuffer * msp->output.nBytesPerFrame;
    msp->output.nBytesPerBuffer =
	CalcAudioConversionSizes(&msp->output.auCvt,
				 framesPerBuffer * samplesPerFrame);
}

static void
InstallAudioForm(MpegSchedPrivPtr msp, Mpeg1ACbInfoPtr cb)
{
    AudioFormPtr newForm = &cb->sampleForm;
    AudioFormPtr oldForm = &msp->cb.arg.auInput;
    
    /*
     * First, note changes and make appropriate changes
     */
    if (oldForm->rate != newForm->rate
     || oldForm->channels != newForm->channels
     || oldForm->dataForm != newForm->dataForm
     || cb->nBytesPerFrame != msp->output.nBytesPerFrame) {
	msp->cb.mask |= MSCB_AuChange;
	msp->cb.arg.auInput = *newForm;
	msp->input.au = *newForm;
	if (msp->output.auFile == NULL)
	    msp->output.au = *newForm;
	msp->output.nBytesPerFrame = cb->nBytesPerFrame;
	CalcAudioBufferSize(msp);
	ReallocAudioBuf(msp);
	CallMpegSchedCallback(msp);
    }
}

static void
AudioTokenCb(PacketLayerPtr pLayer, Mpeg1ACbType cbType, Mpeg1ACbInfoPtr cb)
{
    MpegSchedPrivPtr msp = (MpegSchedPrivPtr)cb->cbData;
    InternalReasonPtr irp;
    int nSamples;
    short clockIdx;

    switch (cbType) {
    case Mpeg1ASampleCb:
	nSamples = cb->nSamples;
	msp->input.nSamples += nSamples;
	if (msp->output.auCvt.convIndex != NoConversion) {
	    /*
	     * convert the samples now so that they are ready
	     * for delivery.  For timing, this is charged against the
	     * input processing.
	     */
	    nSamples = ConvertNewAudioBuffer(&msp->output.auCvt,
					     msp->output.audQCurrent->buffer,
					     nSamples,
					     &irp);
	    if (nSamples <= 0) {
		DeliverReason(msp, irp);
		break;
	    }
	}
	msp->output.audQCurrent->nSamples = nSamples;
	msp->output.audQCurrent->sched = cb->timecode;

	/*
	 * This puts the old buffer on the delivery queue and
	 * allocates a new buffer.
	 */
	if (NULL != (irp = AllocAudioBuf(msp, True))) {
	    DeliverReason(msp, irp);
	    return;
	}

	if (msp->sched.state & kWaitAudioBit)
	    TransitFunction(msp, kWaitAudioBit);
        break;

    case Mpeg1AFormCb:
	if (msp->sched.state & kWaitAudioHdrBit) {
	    msp->cb.mask |= MSCB_NewAudio;
	    TransitFunction(msp, kWaitAudioHdrBit);
	}
	clockIdx = PushClock(msp, OtherClockIdx);
	InstallAudioForm(msp, cb);
	PopClock(msp, clockIdx);
	break;

    default:
        break;
    }
}

/*
 * This will check out the newly specified sequence, opening it if
 * possible.
 */
static InternalReasonPtr
OpenNewSequence(MpegSchedPrivPtr msp)
{
    static InternalReason noPair = { "mpeg from path pair not implemented\n",
    					IR_NotImplemented };
    static InternalReason noDesc = { "mpeg from descriptor not implemented\n",
    					IR_NotImplemented };
    static InternalReason noTcp  = { "mpeg from tcp port not implemented\n",
    					IR_NotImplemented };
    Mpeg1PacketConf sysConf;
    Mpeg1PacketInfoPtr mpegInfo;
    InternalReasonPtr irp;
    MpegSchedInputSpecPtr spec = &msp->input.spec->u.input;

    mpegInfo = &msp->input.mpegInfo;

    memset((char *)mpegInfo, 0, sizeof(*mpegInfo));
    memset((char *)&sysConf, 0, sizeof(sysConf));

    switch (spec->type) {
    case MSIT_SinglePath:  sysConf.path = spec->single.path; break;
    case MSIT_PathPair:	   return(&noPair); break;
    case MSIT_Descriptor:  return(&noDesc); break;
    case MSIT_TcpPort:	   return(&noTcp); break;
    default:		   break;	/* filtered in QueueOpenCommand() */
    }
    
    sysConf.warning = NormalOutput;
    sysConf.mpegOps = Mpeg1SysOps;
    sysConf.mpegCbSelect = 0;
    sysConf.mpegCbData = (void *)msp;
    sysConf.mpegTokenCb = MpegSystemTokenCb;

    if (NULL != (irp = OpenMpeg1PacketStreams(&sysConf, mpegInfo)))
	return(irp);

    msp->cb.arg.aChan = mpegInfo->audio;
    msp->cb.arg.vChan = mpegInfo->video;
    msp->cb.mask |= MSCB_Channels;

    return( NULL );
}

/*
 * This scheduler delivers all data whenever it is ready
 */
void
MpegSchedAdaptiveScheduler(MpegSchedPrivPtr msp)
{
    YcImagePtr yci, yciNext;
    AudioBufPtr ap, apNext;

    /*
     * Deliver the video
     */
    yciNext = NULL;	/* for lint */
    for (yci = msp->output.imgQ; yci; yci = yciNext) {
	yciNext = yci->next;
	msp->output.nFrames++;
	(*msp->output.dpy->displayYcImage)(msp->output.dpy, yci);
    }
    msp->output.imgQ = msp->output.imgQLast = NULL;

    /*
     * Simply discard the audio, if any
     */
    apNext = NULL;	/* for lint */
    for (ap = msp->output.audQ; ap; ap = apNext) {
	apNext = ap->next;
	FreeAudioBuf(msp, ap);
    }
    msp->output.audQ = msp->output.audQLast = NULL;
}

/*
 * This scheduler is used to do nothing while we are in a transition
 * state and not doing any playback.
 */
void
MpegSchedNullScheduler(MpegSchedPrivPtr msp)
{
}

void
BalanceSchedule(MpegSchedPrivPtr msp, long now)
{
    ClockKeepPtr clock = msp->sched.clock;
    ULong *ticks = clock->clocks;
    float ticksPerVdec, ticksPerVdpy, audioFrac, videoFrac;
    float targetFps;
    Mpeg1VideoRunConf vconf;
    InternalReasonPtr irp;

    /*
     * From the video and audio that have been processed so far, figure
     * out the bandwidth leftover for the video after accounting for the
     * audio.  Note that we incorporate the most recent 'pictLate' value
     * as the cost of decoding video, so that we may amortize it over the
     * next set of pictures.
     */
    if (msp->input.nFrames == 0) {
	/* video is disabled */
	ticksPerVdec = (float)0.0;
    } else {
	ticksPerVdec = (float) (ticks[ VideoClockIdx ] + msp->sched.pictLate)
		     / msp->input.nFrames;
    }
    if (msp->output.nFrames == 0) {
	ticksPerVdpy = ticksPerVdec;	/* a default value to work from */
    } else {
	ticksPerVdpy = (float) ticks[ DisplayClockIdx ]
		     / msp->output.nFrames;
    }
    if (msp->input.nSamples == 0) {
	audioFrac = 0.0;
    } else {
	audioFrac = ticks[ AudioClockIdx ]
		    / (msp->sched.rtMsPerASample * msp->input.nSamples);
	/*
	 * This prevents the audio weighting from surpassing 1.0
	 */
	if (audioFrac >= msp->sched.weight)
	    audioFrac = (float)0.9 * msp->sched.weight;
    }
    videoFrac = msp->sched.weight - audioFrac;
    targetFps = (MilliSecPerSecond * videoFrac)
	      / (ticksPerVdec + ticksPerVdpy);

    /*
     * Clamp the calculated target frame rate to rational values.
     */
    if (targetFps <= (float)0.0)
	targetFps = (float)0.0;
    else if (targetFps >= msp->cb.arg.frameRate)
	targetFps = msp->cb.arg.frameRate;

    /*
     * Finally, if the target frame rate is increasing, it increases
     * by a moving average.  If it decreases, it decreases immediately.
     * this means we respond quickly to changes in load, but if we
     * found a few frames that decode quickly, we don't increase the
     * frame rate too quickly.
     */
    if (targetFps > msp->sched.targetFps) {
	targetFps = msp->sched.targetFps * ((float)1.0 - FpsMovingAverage)
		  + (targetFps * FpsMovingAverage);
    }
    msp->sched.targetFps = targetFps;

    msp->sched.vinWeight = targetFps / msp->cb.arg.frameRate;

    /*
     * this is used to tell if a frame is "late" or not
     */
    msp->sched.msPerVFrame = (float)MilliSecPerSecond / targetFps;

    vconf.weight = msp->sched.vinWeight;
    irp = Mpeg1VRunConfigure(msp->input.vLayer, &vconf, Mpeg1VConfWeightMask);
    if (irp)
	DeliverReason(msp, irp);

    DebugTask(VMS_BalanceInfo,
	NormalOutput("video    =%8d for %d frames (%.1ffps)\n",
	    ticks[ VideoClockIdx ], msp->input.nFrames,
	    MilliSecPerSecond / ticksPerVdec);
	NormalOutput("audio    =%8d for %d samples (%.3f%% of cost)\n",
	    ticks[ AudioClockIdx ], msp->input.nSamples, audioFrac);
	NormalOutput("display  =%8d for %d frames\n",
	    ticks[ DisplayClockIdx ], msp->output.nFrames);
	NormalOutput("au del,media,other=%d,%d,%d, target %.1f%% (%.1ffps)\n",
	    ticks[ AuDeliverClockIdx ],
	    ticks[ MediaClockIdx ],
	    ticks[ OtherClockIdx ],
	    msp->sched.vinWeight * (float)100.0, targetFps);
    )

    DebugTask(VMS_SchedStats,
	msp->balanceStat.audioFrac = audioFrac;
	msp->balanceStat.videoFrac = videoFrac;
	UpdateBalanceStats(msp, now);
    )

    msp->input.nFrames = 0;
    msp->input.nSamples = 0;
    msp->output.nFrames = 0;
    msp->output.nSamples = 0;
    ticks[ VideoClockIdx ] = 0;
    ticks[ AudioClockIdx ] = 0;
    ticks[ DisplayClockIdx ] = 0;

    msp->sched.balanceTime = now + BalanceTime;
}

/*
 * This scheduler keeps a real-time clock and tries to deliver
 * data as close as possible to its targeted time.
 */
void
MpegSchedRealtimeScheduler(MpegSchedPrivPtr msp)
{
    YcImagePtr yci, yciNext;
    AudioBufPtr ap, apNext;
    ClockKeepPtr clock = msp->sched.clock;
    long now = (*clock->gettime)(clock) - msp->sched.seqClockOffset;
    long late;

    if (now < msp->sched.schedTime)
	return;
    msp->sched.schedTime = now + msp->sched.minSchedWait;

    /*
     * Deliver the video
     */
    yci = msp->output.imgQ;
    if (yci && 0 <= (late = now - yci->sched)) {

	yciNext = yci->next;

	/*
	 * Drop all but the last of the late frames
	 */
	while (yciNext
	       && late > msp->sched.msPerVFrame
	       && yciNext->sched <= now)
	{
	    DebugTask(VMS_SchedStats, msp->balanceStat.dropped++);
	    (*yci->free)(yci);
	    yci = yciNext;
	    yciNext = yci->next;
	    late = now - yci->sched;
	}

	msp->sched.pictLate = late;
	msp->output.nFrames++;
	(*clock->update)(clock, DisplayClockIdx);
	(*msp->output.dpy->displayYcImage)(msp->output.dpy, yci);

	if (yciNext)
	    msp->output.imgQ = yciNext;
	else
	    msp->output.imgQ = msp->output.imgQLast = NULL;
    }

    /*
     * Deliver the audio
     */
    apNext = NULL;	/* for lint */
    ap = msp->output.audQ;
    if (ap) {
	long written;

	late = now - ap->sched + msp->sched.auLeadMs;
	if (late >= 0) {
	    /*
	     * Now we can reset the audio lead time to a large amount
	     * so that we will deliver the audio when it is available
	     * instead of waiting to the scheduled time.  This keeps
	     * the audio hardware buffer full.
	     */
	    msp->sched.auLeadMs = MilliSecPerSecond;

	    (*clock->update)(clock, AudioClockIdx);
	    for (ap = msp->output.audQ; ap; ap = apNext) {
		written = WriteAudioSamples(msp->output.auFile,
					    ap->buffer, ap->nSamples);
		
		msp->output.nSamples += written;
		if (written < ap->nSamples) {
		    ap->nSamples -= written;
		    ap->pBuf += written << 1;	/* we know they are words */

		    /*
		     * keep this buffer on the list
		     */
		    apNext = ap;
		    break;
		}
		apNext = ap->next;
		FreeAudioBuf(msp, ap);
	    }

	    if (apNext)
		msp->output.audQ = apNext;
	    else
		msp->output.audQ = msp->output.audQLast = NULL;
	}
    }

    DebugTask(VMS_SchedStats, UpdateSchedStats(msp, now));

    if (now >= msp->sched.balanceTime)
	BalanceSchedule(msp, now);
}

static void
InitScheduler(MpegSchedPrivPtr msp)
{
    /*
     * Reset the clocks
     */
    msp->input.nFrames = 0;
    msp->input.nSamples = 0;
    (*msp->sched.clock->init)(msp->sched.clock);
}

static void
StartAdaptiveScheduler(MpegSchedPrivPtr msp)
{
    Mpeg1VideoRunConf vconf;
    InternalReasonPtr irp;

    msp->sched.scheduler = MpegSchedAdaptiveScheduler;

    /*
     * Should also re-seek so that no frames are missing from
     * the queue.  Should turn off audio here, too.
     */
    vconf.weight = (float)1.0;
    irp = Mpeg1VRunConfigure(msp->input.vLayer, &vconf, Mpeg1VConfWeightMask);
    if (irp)
	DeliverReason(msp, irp);
}

static void
StartRealtimeScheduler(MpegSchedPrivPtr msp)
{
    Mpeg1VideoRunConf vconf;
    InternalReasonPtr irp;
    ClockKeepPtr clock = msp->sched.clock;
    long now = (*clock->gettime)(clock);
    long clockOffset;
    YcImagePtr yci;
    AudioBufPtr ap;

    msp->sched.scheduler = MpegSchedRealtimeScheduler;

    /*
     * This should also compensate for the difference between audio
     * delivery delay and video delivery delay.
     */
    yci = msp->output.imgQ;
    ap = msp->output.audQ;
    if (yci && ap) {
	if (yci->sched < ap->sched - msp->output.auHwDelayMs)
	    clockOffset = yci->sched;
	else
	    clockOffset = ap->sched - msp->output.auHwDelayMs;
    } else if (yci) {
	clockOffset = yci->sched;
    } else if (ap) {
	clockOffset = ap->sched;
    } else {
	clockOffset = now;
    }
    msp->sched.seqClockOffset = now - clockOffset;
    msp->sched.auLeadMs = msp->output.auHwDelayMs;

    /*
     * force load balancing and scheduling to kick in immediately
     */
    msp->sched.balanceTime = clockOffset;
    msp->sched.schedTime = clockOffset;

    /*
     * make a guess at the ms per video frame.  This is
     * set more accurately later on in the Balance routine.
     */
    msp->sched.msPerVFrame = (float)MilliSecPerSecond
			   / (msp->sched.rtMsPerVFrame / msp->sched.vinWeight);

    /*
     * Set scheduler maximum frequency to 1/4 of frame rate
     */
    msp->sched.minSchedWait = msp->sched.msPerVFrame >> 2;

    msp->sched.pictLate = 0;
    msp->sched.targetFps = msp->cb.arg.frameRate;

    /*
     * Set the current known weighting.  If this is the first time around,
     * then the load balancing will set this correctly.  Otherwise, it
     * makes sure the balance is set to a reasonable value.
     */
    vconf.weight = msp->sched.vinWeight;
    irp = Mpeg1VRunConfigure(msp->input.vLayer, &vconf, Mpeg1VConfWeightMask);
    if (irp)
	DeliverReason(msp, irp);

    DebugTask(VMS_SchedStats, OpenSchedStats(msp));
}

static ULong
FindFirstSet(ULong mask)
{
    int i;
    
    for (i = 0; i < 31; i++)
        if (mask & (1 << i))
            return(1 << i);
    return(0);
}

static InternalReasonPtr
SelectSequenceChannels(MpegSchedPrivPtr msp, ComplexCmdPtr cp)
{
    Mpeg1VideoConf videoConf;
    Mpeg1VideoConfPtr pvc;
    Mpeg1AudioConf audioConf;
    Mpeg1AudioConfPtr pac;
    PacketLayerPtr elayer;
    ULong video, audio;
    Mpeg1PacketInfoPtr mpegInfo = &msp->input.mpegInfo;
    InternalReasonPtr irp;
    SchedState callbacksWanted = 0;

    msp->input.vChan = cp->u.chan.video;
    msp->input.configMask |= MSM_VideoChan;
    video = FindFirstSet(mpegInfo->video & msp->input.vChan);

    if (video) {
        pvc = &videoConf;    
        memset((char *)&videoConf, 0, sizeof(videoConf));
        videoConf.media = mpegInfo->media;
        videoConf.warning = NormalOutput;
        videoConf.tokenCb = VideoTokenCb;
        videoConf.cbData = (void *)msp;
        videoConf.cbSelect = Mpeg1VDisplayMask | Mpeg1VNewSeqMask;
        videoConf.opsInit = Mpeg1VDecodeOps;
        videoConf.idctMethod = FloatLeeIdctMethod;
	callbacksWanted |= kWaitVideoHdr;
	if (msp->output.dpy == NULL)
	    msp->input.configMask &= ~MSM_Display;
    } else {
        pvc = NULL;
	/*
	 * If there's no video, don't insist on display settings
	 */
	msp->input.configMask |= MSM_Display;
    }

    msp->input.aChan = cp->u.chan.audio;
    msp->input.configMask |= MSM_AudioChan;
    audio = FindFirstSet(mpegInfo->audio & msp->input.aChan);

    if (audio) {
        pac = &audioConf;      
        memset((char *)&audioConf, 0, sizeof(audioConf));
        audioConf.media = mpegInfo->media;
        audioConf.warning = NormalOutput;
        audioConf.tokenCb = AudioTokenCb;
        audioConf.cbData = (void *)msp;
        audioConf.cbSelect = Mpeg1ASampleMask
                           | Mpeg1AFormMask;
        audioConf.opsInit = Mpeg1ADecodeOps;
        audioConf.formatLevel = Mpeg1ANoFormatting;
	if (NULL != (irp = AllocAudioBuf(msp, False)))
	    return (irp);
        audioConf.sampleBuf = msp->output.audQCurrent->buffer;
        audioConf.bufLength = msp->output.audQCurrent->inputLength;
	callbacksWanted |= kWaitAudioHdr;
	if (msp->output.auFile == NULL)
	    msp->input.configMask &= ~MSM_AudioOutput;
    } else {
        pac = NULL;
	/*
	 * If there's no audio, don't insist on output and quality settings
	 */
	msp->input.configMask |= MSM_AudioQuality | MSM_AudioOutput;
    }

    msp->sched.state = kStopped;
    if (video + audio == 0)
        return( BadValueReason("No audio or video configured\n") );

    irp = ConfigureMpeg1Streams(mpegInfo, pvc, video, pac, audio, 0, 0);
    if (irp)
	return(irp);

    msp->input.vLayer = msp->input.aLayer = NULL;
    for (elayer = mpegInfo->list; elayer; elayer = elayer->sibling) {
	if (video && elayer->type == PacketMpeg1Video)
	    msp->input.vLayer = elayer;
	if (audio && elayer->type == PacketMpeg1Audio)
	    msp->input.aLayer = elayer;
    }
    if (video && msp->input.vLayer == NULL)
	return( InternalProblem("sched: no video layer\n") );
    if (audio && msp->input.aLayer == NULL)
	return( InternalProblem("sched: no audio layer\n") );

    msp->sched.state = callbacksWanted;

    return(NULL);
}

static boolean
NeedToProcess(MpegSchedPrivPtr msp)
{
    if (! QueueEmpty(&msp->sched.cmdq))
	return(True);

    if (0 != (msp->sched.state & (kActive | kPending)))
	return(True);

    DebugTask(VMS_SchedStats, FlushSchedStats(msp));
    return(False);
}

static InternalReasonPtr
MpsConfig(MpegSchedPtr ms, long mask, MpegSchedConfPtr conf)
{
    InternalReasonPtr irp;
    MpegSchedPrivPtr msp = (MpegSchedPrivPtr)ms;
    boolean newSequence = False;
    ComplexCmdPtr selectCp = NULL;
    Mpeg1AudioConf audioConf;
    
    /*
     * Halt the decompressor
     */
    (*msp->sched.halt)(msp);

    /*
     * Mark everything specified as unconfigured.
     */
    msp->input.configMask &= ~mask;
    
    /*
     * make all the assignments
     */
    if (mask & MSM_Input) {
        /* open a new input stream */
	if (NULL != (irp = QueueOpenCommand(msp, conf->input)))
	    return(irp);
	newSequence = True;
    }
    if (mask & MSM_AudioQuality) {
        /* set the decoded audio quality */
        msp->input.configMask |= MSM_AudioQuality;
	if (conf->aQual >= MSC_NQualitySteps)
	    conf->aQual = MSC_NQualitySteps-1;
	else if (conf->aQual < 0)
	    conf->aQual = 0;
        msp->input.aQual = conf->aQual;
	audioConf.monoSample = AQualities[ msp->input.aQual ].monoSample;
	audioConf.quarterSample = AQualities[ msp->input.aQual ].quarterSample;
	irp = Mpeg1AConfigure(msp->input.aLayer, &audioConf,
			      Mpeg1AConfQualityMask);
	if (irp)
	    DeliverReason(msp, irp);
    }
    if (mask & MSM_Callback) {
        /* change the callback and associated data */
        msp->input.configMask |= MSM_Callback;
        msp->cb.cb = conf->cb;
        msp->cb.arg.cbData = conf->cbData;
    }
    if (mask & MSM_Display) {
        /* set new display description */
        msp->input.configMask |= MSM_Display;
        msp->output.dpy = conf->dpy;
    }
    if (mask & MSM_AudioOutput) {
        /* set the audio output device/file */
        msp->output.auFile = conf->auFile;
	msp->output.au = conf->auFile->au;
	msp->output.auHwDelayMs = AudioDelayMs(msp->output.auFile);
	if (msp->input.au.rate != AudioRateUnspecified) {
	    CalcAudioBufferSize(msp);
	    ReallocAudioBuf(msp);
	}
        msp->input.configMask |= MSM_AudioOutput;
    }
    if (mask & MSM_WarningFunc) {
        /* set the warning function */
        msp->sched.warning = conf->warning;
        msp->input.configMask |= MSM_WarningFunc;
    }
    if (mask & MSM_AudioChan) {
        /* set the audio channels */
	selectCp = AllocComplexCmd(msp, kSelectChan, 0, &irp);
	if (NULL == selectCp)
	    return(irp);
	selectCp->u.chan.audio = conf->aChan;
	selectCp->u.chan.video = msp->input.vChan;
    }
    if (mask & MSM_VideoChan) {
        /* set the video channels */
	if (selectCp == NULL) {
	    selectCp = AllocComplexCmd(msp, kSelectChan, 0, &irp);
	    if (NULL == selectCp)
		return(irp);
	    selectCp->u.chan.audio = msp->input.aChan;
	}
	selectCp->u.chan.video = conf->aChan;
    }

    if (mask & MSM_Weight) {
        /* set the cpu usage */
	msp->sched.weight = conf->weight;
    }

    if (newSequence && selectCp == NULL) {
	selectCp = AllocComplexCmd(msp, kSelectChan, 0, &irp);
	if (NULL == selectCp)
	    return(irp);
	selectCp->u.chan.audio = msp->input.aChan;
	selectCp->u.chan.video = msp->input.vChan;
    }

    if (selectCp && NULL != (irp = QueueComplexCommand(msp, selectCp)))
	return (irp);

    if (newSequence && NULL != (irp = QueueCommand(msp, kLoadFirstFrame)))
	return (irp);

    msp->pub.needToProcess = NeedToProcess(msp);

    if (msp->cb.mask) {
	ULong mask = msp->cb.mask;

	msp->cb.mask = 0;
        (*msp->cb.cb)(&msp->pub, &msp->cb.arg, mask);
    }

    return(NULL);
}

static InternalReasonPtr
MpsQuery(MpegSchedPtr ms, long mask, MpegSchedConfPtr conf)
{
    return(NULL);
}

static void
NoopHaltDecoder(MpegSchedPrivPtr msp)
{
}

static void
NullCallback(MpegSchedPtr sched, MpegSchedCbPtr cbData, MS_CallbackType type)
{
    MpegSchedPrivPtr msp = (MpegSchedPrivPtr)sched;

    switch(type) {
    case MSCB_VidChange:
    case MSCB_AuChange:
    case MSCB_PlayState:
    case MSCB_Position:
        break;
        
    case MSCB_Reason:
        ShowReason(msp->sched.warning, cbData->reason);
        break;

    default:
	break;	/* ignore */
    }
}

static void
ProcessNextCommand(MpegSchedPrivPtr msp)
{
    ComplexCmdPtr cmdp;
    SchedCommand cmd;
    QItem item;
    InternalReasonPtr irp;
    SchedState newState;
    
    if (QueueEmpty(&msp->sched.cmdq))
	return;

    /*
     * We extract a QItem (which is an unsigned long) instead of
     * SchedCommand in case the compiler decides to store the enum
     * in something smaller than a long, because the command may
     * turn out to be a ComplexCmdPtr.
     */
    msp->sched.scheduler = MpegSchedNullScheduler;
    item = TailOfQueue(&msp->sched.cmdq, SchedCommand);
    RemoveQueueTail(&msp->sched.cmdq);
    if (item > kCommandCount) {
	cmdp = (ComplexCmdPtr)item;
	cmd = cmdp->cmd;
    } else {
	cmd = item;
	cmdp = NULL;	/* for lint */
    }

    switch (cmd) {
    default:
    case kPlayBak:
    case kFastFwd:
    case kFastBak:
    case kStepFwd:
    case kStepBak:
    case kSeek:
	break;		/* ignore these for now */

    case kPlayFwd:
	StartRealtimeScheduler(msp);
	msp->sched.scheduler = MpegSchedRealtimeScheduler;
	msp->sched.balanceTime = 0;	/* force load balancing to kick in */
	msp->sched.state = kPlaying;
	break;

    case kPlayAll:
	StartAdaptiveScheduler(msp);
	msp->sched.state = kPlaying;
	break;

    case kStop:
	msp->sched.state = kStopped;
	break;

    case kOpenSeq:
	if (msp->input.spec)
	    FreeComplexCmd(msp, msp->input.spec);
	msp->input.spec = cmdp;
	msp->sched.state = kStopped;
	if (NULL != (irp = OpenNewSequence(msp)))
	    DeliverReason(msp, irp);
	InitScheduler(msp);
	break;

    case kSelectChan:
	if (NULL != (irp = SelectSequenceChannels(msp, cmdp)))
	    DeliverReason(msp, irp);
	break;

    case kLoadFirstFrame:
	newState = kNoState;
	if (msp->output.imgQ == NULL) {
	    newState |= kWaitVideo;
	} else {
	    (*msp->sched.clock->update)(msp->sched.clock, DisplayClockIdx);
	    msp->output.nFrames++;
	    (*msp->output.dpy->displayYcImage)(msp->output.dpy,
					       msp->output.imgQ);
	    (*msp->sched.clock->update)(msp->sched.clock, OtherClockIdx);
	}
	if (msp->output.audQ == NULL)
	    newState |= kWaitAudio;
	msp->sched.state = newState;
	break;
    }

    if (cmdp && cmdp->cmd != kOpenSeq)
	FreeComplexCmd(msp, cmdp);
}

static void
MpsProcess(MpegSchedPtr ms)
{
    MpegSchedPrivPtr msp = (MpegSchedPrivPtr)ms;
    Mpeg1PacketInfoPtr mpegInfo = &msp->input.mpegInfo;
    PacketLayerPtr elayer;
    InternalReasonPtr irp;
    ClockKeepPtr clock = msp->sched.clock;

    do {
	/*
	 * read and process packets.
	 */
	if (msp->sched.state & kActive) {
	    (*clock->update)(clock, MediaClockIdx);
	    if (NULL != (irp = ReadNextPacket(mpegInfo->media))) {
		if (irp->hint == IR_Eof) {
		    msp->sched.state = kStopped;
		    msp->cb.mask |= MSCB_Eof;
		} else {
		    DeliverReason(msp, irp);
		}
	    }
	    for (elayer = mpegInfo->list; elayer; elayer = elayer->sibling) {
		while (irp == NULL && elayer->queue) {
		    (*clock->update)(clock, elayer->type);
		    if (NULL != (irp = (*elayer->ops.process)(elayer)))
			DeliverReason(msp, irp);
		    if (msp->output.imgQ || msp->output.audQ)
			(*msp->sched.scheduler)(msp);
		    if (msp->sched.timeToYield)
			goto finishLoop;
		}
	    }
	finishLoop:
	    (*clock->update)(clock, OtherClockIdx);
	}

	/*
	 * process requests
	 */
	if ((msp->sched.state & kPending) == 0
	 && ! QueueEmpty(&msp->sched.cmdq)) {
	    ProcessNextCommand(msp);
	    /*
	     * This allows the audio and video to be delivered if any is 
	     * ready (which is true when this is called just after a "play"
	     * command
	     */
	    if ((msp->sched.state & kActive)
	     && (msp->output.imgQ || msp->output.audQ))
		(*msp->sched.scheduler)(msp);
	}

	msp->pub.needToProcess = NeedToProcess(msp);
	CallMpegSchedCallback(msp);
    } while (! msp->sched.timeToYield && msp->pub.needToProcess);

    msp->sched.timeToYield = False;
    msp->pub.needToProcess = NeedToProcess(msp);
    CallMpegSchedCallback(msp);
}

MpegSchedPtr
CreateMpegScheduler(InternalReasonPtr *pIrp)
{
    MpegSchedPrivPtr msp;
    int allocSize = sizeof(MpegSchedPriv);
    ClockKeepPtr clock;

    clock = CreateClockKeep(NClockIndices, MilliSecPerSecond, pIrp);
    if (clock == NULL)
	return(NULL);
    
    msp = (MpegSchedPrivPtr) AllocNamedMem(allocSize, "MPEG scheduler", pIrp);
    if (msp == NULL) {
	(*clock->free)(clock);
        return(NULL);
    }
    *pIrp = InitQueue(&msp->sched.cmdq, CommandQueueLength);
    if (*pIrp) {
        free((char *)msp);
	(*clock->free)(clock);
        return(NULL);
    }
    msp->pub.playFwd = MpsPlayFwd;
    msp->pub.playAll = MpsPlayAll;
    msp->pub.playBak = MpsPlayBak;
    msp->pub.fastFwd = MpsFastFwd;
    msp->pub.fastBak = MpsFastBak;
    msp->pub.stepFwd = MpsStepFwd;
    msp->pub.stepBak = MpsStepBak;
    msp->pub.stop    = MpsStop;
    msp->pub.seek    = MpsSeek;
    msp->pub.config  = MpsConfig;
    msp->pub.query   = MpsQuery;
    msp->pub.process = MpsProcess;
    
    /*
     * Set up reasonable defaults for internal use.
     */
    msp->cb.cb             = NullCallback;
    msp->sched.clock       = clock;
    msp->sched.warning     = Msg;
    msp->sched.halt        = NoopHaltDecoder;
    msp->sched.weight      = DefaultCPUWeighting;
    msp->sched.vinWeight   = DefaultCPUWeighting;
    msp->sched.state       = kStopped;
    msp->sched.scheduler   = MpegSchedNullScheduler;
    msp->input.au.rate     = Audio44_1Khz;
    msp->input.au.channels = AudioStereo;
    msp->input.au.dataForm = Audio16BitPcm;
    msp->output.au         = msp->input.au;
    msp->input.aChan       = MSC_FirstAvailable;
    msp->input.vChan       = MSC_FirstAvailable;
    msp->output.nBytesPerFrame = DefaultBytesPerFrame;

    CalcAudioBufferSize(msp);

    return(&msp->pub);
}
