/*
 *	(c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		mpegsched.h - mpeg scheduler (public interface)
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/mpegsched.h,v 1.7 1996/01/17 00:05:02 toddb Exp $
 */
#ifndef _MPEGSCHED_H
#define _MPEGSCHED_H

#include "nvrTypes.h"
#include "nvrReason.h"
#include "nvrAudio.h"
#include "imageDpy.h"

typedef struct _mpegSched MpegSched, *MpegSchedPtr;
typedef struct _mpegSchedConf MpegSchedConf, *MpegSchedConfPtr;

/*
 * Mpeg Sched Config types, used to specify various values
 */
typedef enum {
    MSC_Unknown,
    MSC_ExactLength,		/* ->length is exact number of frames */
    MSC_ApproxLength,		/* ->length is approximate number of frames */
    MSC_NQualitySteps = 4,	/* that is, 0 <= MpegSchedConf->aQual < 4 */

    MSC_FirstAvailable = -1	/* special value for ->aChan and ->vChan */
} MS_ConfigType;

typedef enum {
    MSM_Input	      = 0x001,	/* sets ->input and ->inputType */
    MSM_AudioQuality  = 0x002,	/* sets aQual, queries nAQualSteps */
    MSM_Callback      = 0x004,	/* sets ->cb and ->cbData */
    MSM_Display       = 0x008,	/* sets ->dpy */
    MSM_AudioOutput   = 0x010,	/* sets ->auFile */
    MSM_WarningFunc   = 0x020,	/* sets ->warning */
    MSM_AudioChan     = 0x040,	/* sets, queries ->aChan */
    MSM_VideoChan     = 0x080,	/* sets, queries ->vChan */

    MSM_AllSet        = 0x0ff,	/* all required parameters specified */

    /*
     * optional settings
     */
    MSM_Weight        = 0x100,	/* sets, queries ->vChan */

    /*
     * Queries only
     */
    MSM_Length        = 0x008	/* queries ->length and ->lengthType */
} MS_Mask;

typedef enum {
    MSCB_NewVideo   = 0x001,	/* New video sequence */
    MSCB_VidChange  = 0x002,	/* ->width, ->height changed */
    MSCB_NewAudio   = 0x004,	/* New audio sequence */
    MSCB_AuChange   = 0x008,	/* ->auInput changed */
    MSCB_PlayState  = 0x010,	/* ->playState changed */
    MSCB_Reason     = 0x020,	/* ->reason has some information in it */
    MSCB_Position   = 0x040,	/* ->frame and ->time changed */
    MSCB_Eof        = 0x080,	/* current sequence is done */
    MSCB_Channels   = 0x100	/* ->aChan or ->vChan changed */
} MS_CallbackType;

typedef enum {
    MSPS_Playing,
    MSPS_Stopped,
    MSPS_Seeking
} MSchedPlayState;

typedef struct _mpegSchedCb {
    int width;			/* VInputSize: input size */
    int height;			/* VInputSize: input size */
    long aChan;			/* available audio channels */
    long vChan;			/* available video channels */
    AudioForm auInput;		/* input audio attributes  */
    MSchedPlayState playState;	/* playback state changed */
    InternalReasonPtr reason;	/* reason generated internally */
    float frameRate;		/* input video frame rate */
    long frame;			/* current frame number */
    long time;			/* current timecode (in milliseconds) */
    void *cbData;		/* specified via MSM_Callback */
} MpegSchedCb, *MpegSchedCbPtr;

typedef void (*MpsCallback)(MpegSchedPtr, MpegSchedCbPtr, MS_CallbackType);

/*
 * This enum set and structure is used to describe the input file.
 */
typedef enum {
    MSIT_Unspecified = 0,    	/* illegal */
    MSIT_SinglePath,		/* single pathname */
    MSIT_PathPair,		/* pair of pathnames: audio and video */
    MSIT_Descriptor,		/* file descriptor */
    MSIT_DescriptorPair,	/* pair of file descriptors */
    MSIT_TcpPort		/* tcp port number for listening */
} MpegSchedInputType;

typedef union _mpegSchedInputSpec {
    MpegSchedInputType type;
    struct {
        MpegSchedInputType type;/* must be MSIT_SinglePath */
        char *path;		/* audio, video or multiplexed file */
    } single;
    struct {
        MpegSchedInputType type;/* must be MSIT_PathPair */
        char *audio;		/* path to separate audio file */
        char *video;		/* path to separate video file */
    } pathPair;
    struct {
        MpegSchedInputType type;/* must be MSIT_TcpPort */
        char *host;		/* if specified, this host only; else any */
        int port;
    } net;
    struct {
        MpegSchedInputType type;/* must be MSIT_Descriptor */
        int fd;			/* file descriptor of open file */
    } descriptor;
    struct {
        MpegSchedInputType type;/* must be MSIT_Descriptor */
        int afd;		/* file descriptor of open audio file */
        int vfd;		/* file descriptor of open video file */
    } descriptorPair;
} MpegSchedInputSpec, *MpegSchedInputSpecPtr;

struct _mpegSchedConf {
    MpegSchedInputSpecPtr input;/* description of input */
    long aChan;			/* audio channels available, channel selected */
    long vChan;			/* video channels available, channel selected */
    int aQual;			/* audio quality */
    MpsCallback cb;		/* call back function */
    void *cbData;		/* assigned to MpegSchedCbPtr->cbData */
    float weight;		/* 0.0 < weight <= 1.0: percentage of cpu */
    ImgDpyRegionPtr dpy;	/* configured output image display */
    AudioFilePtr auFile;	/* audio output */
    void (*warning)(char *fmt, ...); /* warning function to call */
};

typedef enum {
  MSST_Unknown,			/* 0 value: invalid */
  MSST_FileOffset,		/* jump to a specific byte offset */
  MSST_FilePairOffset,		/* byte offsets for video/audio */
  MSST_ScanTimecode,		/* scan for a timecode specified in millisec */
  MSST_RelativeTimecode		/* scan for relative time (millisec) */
} MS_SeekType;

typedef union _mpegSchedSeekTarget {
    MS_SeekType type;
    struct {
	MS_SeekType type;	/* must be FileOffset */
	long offset;
    } file;
    struct {
	MS_SeekType type;	/* must be FilePairOffset */
	long vOffset;
	long aOffset;
    } pair;
    struct {
	MS_SeekType type;	/* must be ScanTimecode or RelativeTimecode */
	long tc;
    } tc;
} MpegSchedSeekTarget, *MpegSchedSeekTargetPtr;
	
struct _mpegSched {
    InternalReasonPtr (*playFwd)(MpegSchedPtr);
    InternalReasonPtr (*playBak)(MpegSchedPtr);
    InternalReasonPtr (*playAll)(MpegSchedPtr);
    InternalReasonPtr (*fastFwd)(MpegSchedPtr);
    InternalReasonPtr (*fastBak)(MpegSchedPtr);
    InternalReasonPtr (*stepFwd)(MpegSchedPtr);
    InternalReasonPtr (*stepBak)(MpegSchedPtr);
    InternalReasonPtr (*stop)   (MpegSchedPtr);
    InternalReasonPtr (*seek)   (MpegSchedPtr, MpegSchedSeekTargetPtr);
    InternalReasonPtr (*config)(MpegSchedPtr, long mask, MpegSchedConfPtr);
    InternalReasonPtr (*query)(MpegSchedPtr, long mask, MpegSchedConfPtr);

    void (*process)(MpegSchedPtr);
    boolean needToProcess;	/* ->process needs to be called */
};

CExternal MpegSchedPtr CreateMpegScheduler(InternalReasonPtr *);

#endif /* _MPEGSCHED_H */
