/*
 *	(c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *		All Rights Reserved
 *
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of North Valley Research, Inc.
 *
 *	This program is the property of North Valley Research, Inc. or others
 *	from whom North Valley Research has obtained a licensing right, and is
 *	considered by North Valley Research to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 *	NAME
 *		mpegschedI.h - internal mpeg scheduler header file
 *
 *	REVISION
 *		$Header: /usr/local/src/nvrToMIT/RCS/mpegschedI.h,v 1.13 1996/01/18 23:45:54 toddb Exp $
 */
#ifndef _MPEGSCHEDI_H
#define _MPEGSCHEDI_H

#include "mpegsched.h"
#include "imageDpy.h"
#include "nvrMpegConf.h"
#include "nvrQueue.h"
#include "nvrAlloc.h"
#include "nvrClock.h"
#include "nvrDebug.h"

/*
 * Debug bit indices
 */
typedef enum {
    VMS_BalanceInfo = DBI_MpegSched,
    VMS_SchedStats,
    VMS_FrameInfo
} DebugBitIndices;

typedef enum {
    kPlayFwd,
    kPlayAll,
    kPlayBak,
    kFastFwd,
    kFastBak,
    kStepFwd,
    kStepBak,
    kStop,
    kSeek,
    kOpenSeq,
    kSelectChan,
    kLoadFirstFrame,
    kCommandCount
} SchedCommand;

typedef enum {
    kPending         = 0x8000,	/* part way done in a multi-step state */
    kActive          = 0x4000,	/* active state, requires processing */

    kWaitVideoHdrBit = 0x0001,
    kWaitVideoHdr    = kWaitVideoHdrBit | kActive | kPending,
    kWaitAudioHdrBit = 0x0002,
    kWaitAudioHdr    = kWaitAudioHdrBit | kActive | kPending,
    kWaitAVHdr       = 0x0003 | kActive | kPending,

    kWaitVideoBit    = 0x0004,
    kWaitVideo       = kWaitVideoBit | kActive | kPending,
    kWaitAudioBit    = 0x0008,
    kWaitAudio       = kWaitAudioBit | kActive | kPending,
    kWaitAV          = 0x000c | kActive | kPending,

    kStopped         = 0x0100,
    kPlaying         = 0x0200 | kActive,

    kNoState         = 0	/* special state */
} SchedState;

#define DefaultCPUWeighting	((float)1.0)

/*
 * This number is the weighting of the arithmetic moving average of
 * an *increasing* frame rate.  That is, it the frame rate is increasing,
 * it only increases by
 *
 *    old * FpsMovingAverage + new * (1 - FpsMovingAverage)
 */
#define FpsMovingAverage	((float)0.25)

enum {
    CommandQueueLength = 10,
    AudioBufferMs = 100,	/* length of each audio buffer in ms */
    DefaultBytesPerFrame = (NLayer2Samples << 2),	/* stereo, 16 bit */
    BalanceTime = 500		/* # of ms between each balancing act */
};

typedef struct _complexCmd ComplexCmd, *ComplexCmdPtr;
struct _complexCmd {
    ComplexCmdPtr next;		/* for linked lists */
    SchedCommand cmd;		/* current command */
    int extra;			/* when extra storage is attached */

    union {
	MpegSchedSeekTarget seek;	/* when cmd is kSeek */
	MpegSchedInputSpec input;	/* when cmd is kOpenSeq */
	struct {
	    ULong audio;		/* when cmd is kSelectAudioChan */
	    ULong video;		/* when cmd is kSelectVideoChan */
	} chan;
    } u;
};

/*
 * The ClockKeepPtr holds an array of clocks where we record the overhead
 * of each system component.  The audio and video and media packet layers
 * are indexed by the PacketLayerType of the layer.  Other subsystems
 * are indexed by a value that does not intersect with a PacketLayerType
 * (hence the use of NPacketLayerTypes).  These values are clock indices for
 * those other subsystems.
 */
typedef enum {
    OtherClockIdx = 0,		/* overhead for all other activity */
    VideoClockIdx = PacketMpeg1Video,
    AudioClockIdx = PacketMpeg1Audio,
    DisplayClockIdx = NPacketLayerTypes,
    AuDeliverClockIdx,
    MediaClockIdx,
    NClockIndices
} ClockIndex;
    
/*
 * Description of internal audio buffering
 */
typedef struct _audioBuf AudioBuf, *AudioBufPtr;
struct _audioBuf {
    long sched;			/* timecode associated with first sample */
    int nSamples;		/* number of samples available in buffer */
    int fullLength;		/* length of entire buffer in bytes */
    int inputLength;		/* number of input bytes (before conversion) */
    UChar *buffer;		/* the buffer itself */
    UChar *pBuf;		/* pointer to ->buffer for partial writes */
    AudioBufPtr next;		/* for linked list */
}; 

typedef struct _mpegSchedPriv MpegSchedPriv, *MpegSchedPrivPtr;

struct _mpegSchedPriv {
    MpegSched pub;
    
    struct {
	ComplexCmdPtr spec;	/* current input description */
	Mpeg1VideoHdr seqHdr;	/* current video sequence header */
        int width;		/* video input width */
        int height;		/* video input height */
        AudioForm au;		/* form of input audio */
        Mpeg1PacketInfo mpegInfo;/* description of elementary streams */
        ULong configMask;	/* config items successfully configured */
        long aChan;		/* selected audio channel */
        long vChan;		/* selected video channel */
	int aQual;		/* specified audio quality */
	PacketLayerPtr vLayer;	/* pointer to video packet layer */
	PacketLayerPtr aLayer;	/* pointer to audio packet layer */
	ULong nFrames;		/* input frames processed */
	ULong nSamples;		/* input samples processed */
    } input;
    
    struct {
	YcImagePtr imgQ;	/* queue of images to be displayed */
	YcImagePtr imgQLast;	/* pointer to last in imgQ */
        ImgDpyRegionPtr dpy;	/* specified display mechanism */
	ULong nFrames;		/* output frames displayed */
	ULong nSamples;		/* output samples processed */
        int aQual;		/* specified audio quality */
	long auHwDelayMs;	/* hardware audio output delay in millisec */
        AudioFilePtr auFile;	/* specified output file */
        AudioForm au;		/* copy of auFile->au or input->au */
        AudioConversion auCvt;	/* conversion data for auFile */
	AudioBufPtr audQ;	/* queue of audio buffers */
	AudioBufPtr audQLast;	/* pointer to last in audQ */
	AudioBufPtr audQCurrent;/* current audio buffer */
	AudioBufPtr audQFree;	/* list of unused audio buffers */
	int nBytesPerFrame;	/* from adec callback */
	int samplesPerFrame;	/* calculated from adec callback */
	int nBytesPerBuffer;	/* total bytes in buffer */
	int nBytesPerInputBuffer;/* # of bytes available for input */
        Mpeg1PacketInfoPtr mpegInfo;/* description of streams */
    } output;
    
    struct {
	ClockKeepPtr clock;	/* timing mechanism */
	long seqClockOffset;	/* skew between rt clock and seq clock */
	ClockIndex mediaIdx;	/* used to record overhead of media layer */
        Queue cmdq;		/* queue of commands to be processed */
	ComplexCmdPtr freeCmd;	/* free, complex command structures */
	SchedState state;	/* scheduler state */
        boolean timeToYield;	/* time to return to caller */
	float weight;		/* configured cpu weighting */
	float vinWeight;	/* balanced, video input weight */
	float targetFps;	/* target frame rate for video in/out */
	long balanceTime;	/* time when next load balancing done */
	long schedTime;		/* time when next scheduling done */
	long minSchedWait;	/* min time before next scheduling */
	long auLeadMs;		/* amount of time to lead the schedule */
	long pictLate;		/* latency of last picture wrt schedule */
	double rtMsPerASample;	/* real-time ms per audio sample */
	double rtMsPerAFrame;	/* real-time ms per audio frame */
	double rtMsPerVFrame;	/* real-time ms per video frame */
	long msPerVFrame;	/* ms per video frame at balanced fps */
        void (*halt)(MpegSchedPrivPtr); /* state transition function */
        void (*warning)(char *fmt, ...);/* warning function */
	void (*scheduler)(MpegSchedPrivPtr);
    } sched;
    
    struct {
        MpsCallback cb;		/* specified callback */
        MpegSchedCb arg;	/* mpeg sched data for callback */
	ULong mask;		/* mask of callbacks accumulated */
    } cb;

#ifdef DEBUG_ENABLED
    struct {
	BufFileHndl fp;
	struct {
	    ULong nFrames;	/* last known i/o video frames processed */
	    ULong nSamples;	/* last known i/o audio samples processed */
	} input, output;
	struct {
	    long last;		/* last known clock value */
	    long cost;		/* accumulated cost for decoding */
	} au, vid, dpy;
	float samplesPerFrameRecip;
    } schedStat;

    struct {
	short dropped;		/* frames dropped during last balancing */
	float videoFrac;	/* allocated cpu frac for video i/o */
	float audioFrac;	/* allocated cpu for audio */
    } balanceStat;
#endif /* DEBUG_ENABLED */

};

CExternal void MpegSchedFlushCommands(MpegSchedPrivPtr);
CExternal void MpegSchedAdaptiveScheduler(MpegSchedPrivPtr);
CExternal void MpegSchedRealtimeScheduler(MpegSchedPrivPtr);

#endif /* _MPEGSCHEDI_H */
