/*
 *      (c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              schedStat.c - program to parse binary scheduler statistics
 *
 *      REVISION
 *              $Header: /nvr/share/src/nvr/prod/library/mpegsched/RCS/mpegsched
I.h,v 1.7 1995/12/24 07:04:29 toddb Exp toddb $
 */
#include "schedStat.h"
#include "osio.h"
#include "nvrAlloc.h"
#include "nvrString.h"
#include "fileAccess.h"

static void
PrintBalanceHdr(void)
{
    NormalOutput("\n");
    NormalOutput("                    v in   video video video audio\n");
    NormalOutput("   time  qsize drop weight fps   ms/fr frac  frac\n");
    NormalOutput("--------------------------------------------------\n");
}

static void
PrintRecordHdr(void)
{
    NormalOutput("\n");
    NormalOutput("               pin   pout  aud\n");
    NormalOutput("   time  late  cost  cost  cost pin pou ain aou\n");
    NormalOutput("-----------------------------------------------\n");
}

static void
ListData(UChar *buf, long fileSize)
{
    UChar *last, *p;
    SchedRecordPtr rec;
    SchedSequence seq;
    SchedBalance bal;
    SchedAnyPtr any;
    SRType curType = 0;
    long recLength;

    last = buf + fileSize;
    for (p = buf; p < last; p += recLength) {
	any = (SchedAnyPtr) p;
	if (curType != any->type) {
	    curType = any->type;
	    switch(any->type) {
	    case SR_record:
		PrintRecordHdr();
		break;

	    case SR_balance:
		PrintBalanceHdr();
		break;

	    case SR_sequence:
		break;

	    default:
		break;
	    }
	}

	switch(any->type) {
	case SR_record:
	    recLength = sizeof(SchedRecord);
	    rec = &any->rec;

	    NormalOutput("%7d", rec->time);
	    if (rec->pictOut)
		NormalOutput("%6d", rec->pictLate);
	    else
		NormalOutput("      ");
	    if (rec->pictIn)
		NormalOutput("%6d", rec->pictInCost);
	    else
		NormalOutput("      ");
	    if (rec->pictOut)
		NormalOutput("%6d", rec->pictOutCost);
	    else
		NormalOutput("      ");
	    if (rec->auIn)
		NormalOutput("%6d", rec->auCost);
	    else
		NormalOutput("      ");
	    if (rec->pictIn)
		NormalOutput("%4d", rec->pictIn);
	    else
		NormalOutput("    ");
	    if (rec->pictOut)
		NormalOutput("%4d", rec->pictOut);
	    else
		NormalOutput("    ");
	    if (rec->auIn)
		NormalOutput("%4d", rec->auIn);
	    else
		NormalOutput("    ");
	    if (rec->auOut)
		NormalOutput("%4d", rec->auOut);
	    else
		NormalOutput("    ");
	    NormalOutput("\n");
	    
	    break;

	case SR_balance:
	    recLength = sizeof(SchedBalance);
	    bal = any->bal;
	    NormalOutput("%7d%7d%5d%7.3f%6.3f%6d%6.3f%6.3f\n",
		bal.time, bal.qSize, bal.dropped,
		bal.vinWeight, bal.targetFps, (int)(1000 / bal.targetFps),
		bal.videoFrac, bal.audioFrac);
	    break;

	case SR_sequence:
	    recLength = sizeof(SchedSequence);
	    seq = any->seq;
	    break;

	default:
	    Fatal("unknown scheduler record type %d @ byte%d\n",
		any->type, p - buf);
	    break;
	}
    }
}

static void
InterpolateData(UChar *buf, long fileSize)
{
    UChar *p, *last;
    long recLength, lastClock;
    BufFileHndl ppsFp;		/* picture/sec file */
    BufFileHndl afpsFp;		/* audio frames/sec file */
    BufFileHndl picLateFp;	/* picture lateness file */
    BufFileHndl schedFp;	/* scheduler latency file */
    SchedRecordPtr rec;
    SchedSequence seq;
    SchedBalance bal;
    SchedAnyPtr any;

    ppsFp = OpenBufferedFileForWriting("pps.data");
    afpsFp = OpenBufferedFileForWriting("afps.data");
    picLateFp = OpenBufferedFileForWriting("piclate.data");
    schedFp = OpenBufferedFileForWriting("shed.data");

    if (ppsFp == NULL || afpsFp == NULL
     || picLateFp == NULL || schedFp == NULL)
	Fatal("Can't create one of the data files\n");

    recLength = 0;	/* for lint */
    lastClock = 0;
    last = buf + fileSize;
    for (p = buf; p < last; p += recLength) {
	any = (SchedAnyPtr) p;
	switch(any->type) {
	case SR_record:
	    recLength = sizeof(SchedRecord);
	    rec = &any->rec;

	    if (rec->pictIn) {
		float rtCost = 1000 / seq.fps;
		float costRatio = (rec->pictInCost / rec->pictIn) / rtCost;
		BufPrintf(ppsFp, "%d %.3f\n", rec->time, costRatio);
	    }
	    if (rec->pictOut) {
		float lateRatio = rec->pictLate / (1000.0 / seq.fps);
		BufPrintf(picLateFp, "%d %.3f\n", rec->time, lateRatio);
	    }
	    if (rec->auIn) {
		float costRatio = (rec->auCost / rec->auIn) / seq.msPerAFrame;
		BufPrintf(afpsFp, "%d %.3f\n", rec->time, costRatio);
	    }
	    BufPrintf(schedFp, "%d %d\n", rec->time, rec->time - lastClock);
	    lastClock = rec->time;
	    break;

	case SR_sequence:
	    recLength = sizeof(SchedSequence);
	    seq = any->seq;
	    break;

	case SR_balance:
	    recLength = sizeof(SchedBalance);
	    bal = any->bal;
	    break;

	default:
	    Fatal("unknown scheduler record type %d @ byte%d\n",
		any->type, p - buf);
	    break;
	}
    }

    CloseBufferedFile(ppsFp);
    CloseBufferedFile(afpsFp);
    CloseBufferedFile(picLateFp);
    CloseBufferedFile(schedFp);
}

int
main(int argc, char **argv)
{
    FileHandle fdIn;
    long fileSize;
    UChar *buf;
    boolean list = False;

    for (argc--, argv++; argc; argc--, argv++) {
	if (strcmp(*argv, "-l") == 0)
	    list = True;
    }

    fdIn = OpenFileForReading("schedStats");
    if (fdIn < 0)
	Fatal("can't open schedStats\n");

    fileSize = FdFileSize(fdIn);
    buf = (UChar *)InsecureAlloc(fileSize);
    if (buf == NULL)
	Fatal("can't allocate %d bytes \n", fileSize);

    if (ReadData(fdIn, (char *)buf, fileSize) != fileSize)
	Fatal("can't read %d bytes \n", fileSize);
    CloseFile(fdIn);

    if (list)
	ListData(buf, fileSize);
    else
	InterpolateData(buf, fileSize);

    return(0);
}
