/*
 *      (c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              mpegtransit.c - transit function for mpeg scheduler
 *
 *      REVISION
 *              $Header: /usr/local/src/nvr/library/RCS/mpegsched.c,v 1.10 1995/
08/17 21:41:13 toddb Exp $
 */
#include "mpegsched.h"
#include "nvrAlloc.h"
#include "process.h"
#include "nvrDebug.h"

#ifdef DEBUG_ENABLED

extern FlagNames Mpeg1VdecDebugNames;
extern FlagMacroList Mpeg1VdecDebugMacros;
extern FlagNames PacketDebugNames;
extern FlagMacroList PacketDebugMacros;
extern FlagNames IDctDebugNames;
extern FlagMacroList IDctDebugMacros;

FlagLabels DebugFlagDesc[] = {
    { &PacketDebugNames, &PacketDebugMacros },
    { &Mpeg1VdecDebugNames, &Mpeg1VdecDebugMacros },
    { &IDctDebugNames, &IDctDebugMacros },
    { NULL },
};

#else /* DEBUG_ENABLED */

FlagLabels DebugFlagDesc[1];

#endif /* DEBUG_ENABLED */

#define UsageMacro(args)	{ Msg args; Usage(); }

typedef struct _sequence Sequence, *SequencePtr;
typedef struct _mpegPlayer MpegPlayer, *MpegPlayerPtr;

struct _sequence {
    char *video;
    char *audio;
    SequencePtr next;
};

struct _mpegPlayer {
    SequencePtr list;
    SequencePtr current;
    MpegSchedPtr sched;
    struct {
	char *video;
	char *audio;
    } output;

    ImgDpyRegionPtr dpy;
    ImgDpyParam dpyParam;

    AudioFilePtr auFile;
    AudioFileConfig auConf;

    boolean done;		/* true when current sequence done */
    boolean finished;		/* true when all processing is finished */
};

char *ProgramName;

static void
Usage(void)
{
    Msg("Usage: %s [video=name] [audio=name] sequence1 [sequence2 ...]\n",
	ProgramName);
    exit(1);
}

static SequencePtr
AllocSequence(char *video, char *audio, InternalReasonPtr *pIrp)
{
    int videoLen, audioLen, allocSize;
    SequencePtr seq;

    if (audio == NULL)
	audio = video;
    if (video == NULL)
	video = audio;
    videoLen = strlen(video) + 1;
    audioLen = strlen(audio) + 1;
    allocSize = sizeof(Sequence) + videoLen + audioLen;

    seq = (SequencePtr)AllocNamedMem(allocSize, "Sequence description", pIrp);
    if (seq == NULL)
	return(NULL);
    seq->video = (char *)(seq + 1);
    seq->audio = seq->video + videoLen;
    strcpy(seq->video, video);
    strcpy(seq->audio, audio);

    return(seq);
}

static void
AppendSequence(SequencePtr *list, SequencePtr seq)
{
    SequencePtr s;

    if (*list == NULL) {
	*list = seq;
    } else {
	for (s = *list; s->next; s = s->next)
	    ;
	s->next = seq;
    }
}
    
static void
ExitReason(InternalReasonPtr irp)
{
    ShowReason(Msg, irp);
    FreeReason(irp);
    exit(0);
}

static InternalReasonPtr
OpenNextSequence(MpegPlayerPtr player)
{
    static InternalReason noPair = { "mpeg from path pair not implemented\n",
                                        IR_NotImplemented };
    static InternalReason noDesc = { "mpeg from descriptor not implemented\n",
                                        IR_NotImplemented };
    static InternalReason noTcp  = { "mpeg from tcp port not implemented\n",
                                        IR_NotImplemented };
    static InternalReason noSpec = { "mpeg input unspecified\n",
                                        IR_BadValue };
    MpegSchedConf conf;
    MpegSchedInputSpec input;

    if (player->current == NULL) {
	player->current = player->list;
    } else {
	if (player->current->next == NULL) {
	    player->finished = True;
	    return( NULL );
	}
	player->current = player->current->next;
    }

    player->done = False;

    input.single.type = MSIT_SinglePath;
    input.single.path = player->current->video;
    conf.input = &input;

    return( (*player->sched->config)(player->sched, MSM_Input, &conf) );
}

static int
FindFirstSet(long value)
{
    int i, bit;

    for (i = 0, bit = 1; i < 32; i++, bit <<= 1) {
	if (value & bit)
	    return(bit);
    }
    return(0);
}

static InternalReasonPtr
SelectChannels(MpegPlayerPtr player, MpegSchedCbPtr scbData)
{
    MpegSchedConf conf;
    ULong mask;

    if (player->output.video)
	conf.vChan = FindFirstSet(scbData->vChan);
    else
	conf.vChan = 0;

    if (player->output.audio)
	conf.aChan = FindFirstSet(scbData->aChan);
    else
	conf.aChan = 0;

    mask = MSM_VideoChan | MSM_AudioChan;
    return( (*player->sched->config)(player->sched, mask, &conf) );
}

static void
SchedCb(MpegSchedPtr sched, MpegSchedCbPtr scbData, MS_CallbackType mask)
{
    MpegPlayerPtr player = (MpegPlayerPtr)scbData->cbData;
    InternalReasonPtr irp;

    if (mask & MSCB_NewVideo) {
	;
    }

    if (mask & MSCB_VidChange) {
	if (player->output.video) {
	    player->dpyParam.u.file.path = player->output.video;
	    player->dpyParam.color = DepthUnspecified;
	    player->dpyParam.src.colorSpace = ColorYC;
	    player->dpyParam.src.sample = Sample4_2_0;
	    player->dpyParam.src.width = scbData->width;
	    player->dpyParam.src.height = scbData->height;
	    player->dpyParam.src.lineBytes = scbData->width;
	    player->dpyParam.src.chromaLineBytes = scbData->width >> 1;

	    irp = InitImageRegion(player->dpy, &player->dpyParam, ID_AllFile);
	    if (irp)
		ExitReason(irp);
	}
    }

    if (mask & MSCB_NewAudio) {
	;
    }

    if (mask & MSCB_AuChange) {
	if (player->output.audio) {
	    if (player->auFile) {
		if (scbData->auInput.dataForm != player->auConf.au.dataForm
		 && scbData->auInput.channels != player->auConf.au.channels
		 && scbData->auInput.rate != player->auConf.au.rate) {
		    ExitReason( BadValueReason("audio seq %s changed rate!\n",
			player->current->audio));
		}
	    } else {
		MpegSchedConf conf;
		ULong confMask;

		player->auConf.au = scbData->auInput;
		player->auConf.type = FT_AiffAudio;
		player->auFile = CreateAudioFile(player->output.audio,
						 &player->auConf,
						 &irp);
		if (irp)
		    ExitReason(irp);

		conf.auFile = player->auFile;
		conf.aQual = MSC_NQualitySteps-1;
		confMask = MSM_AudioOutput | MSM_AudioQuality;
		irp = (*player->sched->config)(player->sched, confMask, &conf);
		if (irp)
		    ExitReason(irp);
	    }
	}
    }

    if (mask & MSCB_PlayState) {
	switch (scbData->playState) {
	case MSPS_Playing: Msg("playing\n"); break;
	case MSPS_Stopped: Msg("playing\n"); break;
	case MSPS_Seeking: Msg("playing\n"); break;
	}
    }

    if (mask & MSCB_Reason) {
	ExitReason(scbData->reason);
    }

    if (mask & MSCB_Position) {
	Msg("frame %d, time %.3f\n", scbData->frame, scbData->time);
    }

    if (mask & MSCB_Eof) {
	Msg("frame %d, time %.3f\n", scbData->frame, scbData->time);
	player->done = True;
    }

    if (mask & MSCB_Channels) {
	if (NULL != (irp = SelectChannels(player, scbData)))
	    ExitReason(scbData->reason);
    }
}

static InternalReasonPtr
ConfigInitialSettings(MpegPlayerPtr player)
{
    MpegSchedConf conf;
    ULong mask;

    conf.cb = SchedCb;
    conf.cbData = (void *)player;
    conf.dpy = player->dpy;
    conf.warning = Msg;

    mask = MSM_Callback | MSM_Display | MSM_WarningFunc;

    return( (*player->sched->config)(player->sched, mask, &conf) );
}

int
main(int argc, char **argv)
{
    InternalReasonPtr irp;
    MpegPlayer player;
    SequencePtr seq;

    ProgramName = *argv;

    memset((char *)&player, 0, sizeof(player));

    for (argc--, argv++; argc; argc--, argv++) {
	if (strncmp(*argv, "vout=", 5) == 0) {
	    player.output.video = *argv + 5;
	} else if (strncmp(*argv, "aout=", 5) == 0) {
	    player.output.audio = *argv + 5;
	} else {
	    if (NULL == (seq = AllocSequence(*argv, NULL, &irp)))
		ExitReason(irp);
	    AppendSequence(&player.list, seq);
	}
    }

    if (player.list == NULL) {
	UsageMacro(("no input specified\n"));
    }

    if (player.output.video == NULL && player.output.audio == NULL) {
	UsageMacro(("no output specified\n"));
    }

    if (player.output.video) {
	player.dpy = CreateImageRegion(FileImageDesc, &irp);
	if (player.dpy == NULL)
	    ExitReason(irp);
    }

    if (NULL == (player.sched = CreateMpegScheduler(&irp)))
	ExitReason(irp);

    if (NULL != (irp = ConfigInitialSettings(&player)))
	ExitReason(irp);

    if (NULL != (irp = OpenNextSequence(&player)))
	ExitReason(irp);

    if (NULL != (irp = (*player.sched->playAll)(player.sched)))
	ExitReason(irp);

    while (! player.done) {
	(*player.sched->process)(player.sched);
    }

    exit (0);
}

