/*
 *      (c) Copyright 1992 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              mpegSysDecode.h
 *
 *      DESCRIPTION
 *		Interface declaration for MPEG 1 system layer
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/mpegSysDecode.h,v 1.7 1994/12/17 13:04:35 russ Exp $
 */

#ifndef _MPEGSYSDECODE_H_
#define _MPEGSYSDECODE_H_

#include "nvrTypes.h"
#include "nvrReason.h"
#include "nvrTimer.h"
#include "osio.h"

#define	L_BYTE		3	/* Log 2 of the number of bits per byte */
#define	L_LONG		5	/* Log 2 of the number of bits per long */

#define	pwr2(x)		(1 << (x))
#define msk2(x)		((1 << (x)) - 1)

typedef enum {
    VideoBitstream,
    AudioBitstream,
    DataBitstream,
    PrivateBitstream
} BitstreamType;

typedef FileHandle  (*SystemOpenFunc)(char *);
typedef int  (*SystemReadFunc)(FileHandle, char *, long);
typedef void (*SystemCloseFunc)(FileHandle);
typedef long  (*SystemSeekFunc)(FileHandle, long offset, SeekWhere sk);

typedef struct _mpegStreamInfo {
    TimeSpec    currentPts;	/* presentation timestamp for AU in .buf */
    TimeSpec    currentDts;	/* decode timestamp for access unit in .buf */
    PrivPointer streamState;	/* Stream state ptr for system layer */
    UChar       *buf;		/* First byte of a range on data */
    int		length;		/* Number of bytes in buffer */
    int         pos;	 	/* Byte offset in system stream to .buf[0] */
} MpegStreamInfo, *MpegStreamInfoPtr;

typedef struct _mpegSystemInfo {
    SystemOpenFunc openFunc;	/* Function for opening the stream */
    SystemReadFunc readFunc;	/* Function for reading data */
    SystemCloseFunc closeFunc;	/* Function for closing */
    SystemSeekFunc seekFunc;	/* Function for seeking */

    int            nVideo;	/* Number of video elementary streams */
    int            nAudio;	/* Number of audio elementary streams */
    int            nData;	/* Number of data  elementary streams */
    double  	   timescale;	/* Real time scale factor else negative */
    double	   maxbehind;	/* limit on how far behend schedule */
    void         (*scheduler)(  /* Temporal/data-ready callback sched */
	    void (*)(),         /* Callback function pointer */
	    PrivPointer,        /* Argument to the callback function */
	    TimeSpecPtr,        /* Callback at this time or earlier */
	    FileHandle);        /* Callback when this file is readable */
    PrivPointer    systemState;	/* Data structures used by system layer */
    void         (*startAccessUnit)/* start of access unit callback */
           (MpegStreamInfoPtr);
} MpegSystemInfo, *MpegSystemInfoPtr;
    
/*
 * returns system state pointer
 */

CExternal InternalReasonPtr SystemOpen(
    char *,		/* in: file name */
    MpegSystemInfoPtr   /* out: where to put the system-specific state data */
);

CExternal void SystemClose(
    MpegSystemInfoPtr,		/* system state */
    boolean			/* free storage if true */
);

/*
 * returns stream-specific state pointer
 */

CExternal InternalReasonPtr StreamOpen(
    MpegSystemInfoPtr,		/* in: system state */
    BitstreamType,		/* in: which type */
    int,			/* in: bitstream index */
    MpegStreamInfoPtr		/* out: individual stream state */
);

CExternal void StreamRead(
    MpegStreamInfoPtr		/* in/out: stream state */
);
    
CExternal void StreamClose(
    MpegStreamInfoPtr,		/* in/out: stream state */
    boolean			/* Free storage if true */
);

CExternal int SystemSeek(
    MpegSystemInfoPtr,
    long offset,
    SeekWhere
);

CExternal void SystemClose1(
    MpegSystemInfoPtr msys,
    MpegStreamInfoPtr mstr,
    boolean freeStg
);

extern	MpegSystemInfoPtr	MpegSystem;

#endif /* _MPEGSYSDECODE_H_ */
