/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 *      All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *      mpeg1Packet.c - mpeg system demultiplexer
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1Packet.c,v 1.22 1995/10/02 00:39:59 toddb Exp $
 */
#include "nvrPacketI.h"
#include "mpeg1PacketI.h"
#include "nvrAlloc.h"

static InternalReason BadOpValue =
    { "OpenMpeg1SystemStream: ops is NULL\n", IR_InternalProblem };
static InternalReason BadMediaValue =
    { "OpenMpeg1SystemStream: media is NULL\n", IR_InternalProblem };

static char TokenSize[] = {
     4, /* MT_StartCodeSearch1 */
     4, /* MT_StartCodeSearchSeek */
     4, /* MT_StartCodeSearch2 */
     4, /* MT_StartCodeSearch3 */
    12, /* MT_PackHdr */
    12, /* MT_SysHeader */
     3, /* MT_StreamId */
     6, /* MT_PacketHdr */
     1, /* MT_PacketBranch1 */
     1, /* MT_PacketBranch2 */
     1, /* MT_PacketBranch3 */
     2, /* MT_BufferSize */
     5, /* MT_Pts */
    10, /* MT_PtsDts */
     0, /* MT_PacketData */
};

static char *TokenNames[] = {
    "MT_StartCodeSearch1",
    "MT_StartCodeSearchSeek",
    "MT_StartCodeSearch2",
    "MT_StartCodeSearch3",
    "PackHdr",
    "SysHeader",
    "StreamId",
    "PacketHdr",
    "PacketBranch1",
    "PacketBranch2",
    "PacketBranch3",
    "BufferSize",
    "Pts",
    "PtsDts",
    "PacketData"
};

InternalReasonPtr
AllocMpeg1IdList(Mpeg1PrivPtr priv)
{
    StreamId *newList;
    int nAllocIds = (priv->cb.sysh.headerLength - 6 + 2) / 3;
    int allocSize;

    priv->curId = 0;
    if (nAllocIds > 32 + 16 + 16 + 4) {
	(*priv->warning)("Error %d: %s: %d; offset=%d\n",
	    ++priv->errorCount,
	    "mpeg system header length appears incorrect",
	    priv->cb.sysh.headerLength,
	    priv->cb.syshOffset);
	nAllocIds = 32 + 16 + 16 + 4;
    }
    priv->cb.nids = nAllocIds;
    if (nAllocIds <= priv->nAllocIds)
	return(NULL);
    allocSize = sizeof(StreamId) * nAllocIds;

    newList = (StreamIdPtr)InsecureAlloc(allocSize);
    if (newList == NULL)
	return(AllocReason("mpeg stream ids", allocSize));
    if (priv->cb.idList) {
	memcpy((char *)newList,
	       (char *)priv->cb.idList,
	       allocSize - sizeof(StreamId));
	free((char *)priv->cb.idList);
    }
    priv->cb.idList = newList;
    priv->nAllocIds = nAllocIds;
    return(NULL);
}

PacketLayerPtr
OpenMpeg1SystemStream(Mpeg1StreamConfPtr conf, InternalReasonPtr *pReason)
{
    Mpeg1PrivPtr priv;
    PacketLayerPtr pLayer;

    if (conf->ops == NULL) {
        *pReason = &BadOpValue;
        return(NULL);
    }
    if (conf->media == NULL) {
        *pReason = &BadMediaValue;
        return(NULL);
    }
    pLayer = AllocPacketLayer(PacketMpeg1Sys, conf->media, sizeof(Mpeg1Priv));
    if (pLayer == NULL) {
	*pReason = &IRBadAllocation;
	return(NULL);
    }
    priv = (Mpeg1PrivPtr)pLayer;
    priv->curToken = MT_StartCodeSearch1;	/* this is where we start */
    priv->warning = conf->warning;
    priv->tokenCb = conf->tokenCb;
    priv->cb.cbData = conf->cbData;
    priv->cbSelect = conf->cbSelect;
    priv->tokenSize = TokenSize;
    priv->tokenNames = TokenNames;
    pLayer->ops = *conf->ops;
    pLayer->timeCodes = True;

    /*
     * If the syntax-checking version of the mpeg demultiplexer was
     * chosen, then we give it the chance to ask for packet alignment
     */
    if (pLayer->ops.alignPackets)
	(*pLayer->ops.alignPackets)(pLayer);

    /*
     * pre-register padding streams
     */
    BM_SET(&priv->streams, MSId_Padding & 0x7f);

    /*
     * Now parse enough packets to get through the system header.
     */
    return(pLayer);
}

void
DoubleToScr(double scr, MpegPackHdrPtr pack)
{
    long intScr;

    intScr = (scr / 2.0) * 90000.0;	/* convert to a 33 bit number >> 1 */
    pack->scr1 = intScr >> 29;
    pack->scr2 = intScr >> 14;
    pack->scr3 = intScr >> 6;
    pack->scr4 = intScr << 1;
    pack->scr4 |= ((long)scr) & 1;
}
