/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1Packet.h - public interface for mpeg 1 packet-level transport
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1Packet.h,v 1.15 1995/09/29 21:18:21 toddb Exp $
 */
#ifndef _MPEG1PACKET_H_
#define _MPEG1PACKET_H_

#include "nvrPacket.h"
#include "mpeg1Proto.h"

typedef enum {
    Mpeg1PackCb,	/* a pack header has been fully parsed */
    Mpeg1PacketCb,	/* a packet header has been fully parsed */
    Mpeg1SysHeaderCb,	/* a system header has been fully parsed */
    Mpeg1GapCb,		/* A gap was encountered before a start code */
    Mpeg1EndCodeCb,	/* an ISO 11172 end code seen */
    Mpeg1PacketDoneCb	/* a packet from below has been fully parsed */
} Mpeg1CbType;

typedef enum {
    Mpeg1PackMask	= 1<<Mpeg1PackCb,	/* called for syntax ops only */
    Mpeg1PacketMask	= 1<<Mpeg1PacketCb,	/* called for syntax ops only */
    Mpeg1SysHeaderMask	= 1<<Mpeg1SysHeaderCb,
    Mpeg1GapMask	= 1<<Mpeg1GapCb,	/* called for syntax ops only */
    Mpeg1EndCodeMask	= 1<<Mpeg1EndCodeCb,	/* called for syntax ops only */
    Mpeg1PacketDoneMask = 1<<Mpeg1PacketDoneCb
} Mpeg1CbMask;

typedef enum {
    MpegPacketPts    = 0x001,
    MpegPacketDts    = 0x002,
    MpegPacketBuffer = 0x004
} MpegPacketParts;

typedef struct _mpegPacketInfo MpegPacketInfo, *MpegPacketInfoPtr;
struct _mpegPacketInfo {
    MpegPacketHdr hdr;
    int nStuffBytes;            /* number of stuffing bytes */
    int subHdrBytes;            /* total non-data bytes after length field */
    StdBuffer buffer;           /* the buffer scale and size, if present */
    union {
        PtsProto pts;           /* if pts.marker == 0010 */
        PtsDtsProto ptsDts;     /* if ptdDst.marker == 0011 */
        UChar marker;           /* if marker == 0000 1111 */
    } u;
    int dataLength;             /* this is hdr.length - buffer, pts and dts */
    ULong parts;                /* mask of MpegPacketParts */
};

/*
 * This structure is passed to the caller when one of the tokens
 * has been fully parsed and the callback has been selected.
 */
typedef struct _Mpeg1MuxCbInfo {
    /* valid for Mpeg1PackCb callbacks */
    MpegPackHdr pack;                   /* current pack header */
    ULong packOffset;                   /* offset for start of pack */

    /* valid for Mpeg1SysHeaderCb callbacks */
    MpegSystemHdr sysh;                 /* current system header */
    ULong syshOffset;                   /* offset for start of system header */
    StreamId *idList;			/* stream ids listed with cur pack */
    int nids;				/* # if ids in idList */

    /* valid for Mpeg1PacketCb callbacks */
    MpegPacketInfo packet;              /* current packet header */
    ULong packetOffset;                 /* offset for start of packet header */

    /* valid for Mpeg1GapCb callbacks */
    ULong gap;                          /* current gap */
    ULong gapOffset;                    /* offset for start of gap */

    /* valid for Mpeg1EndCodeCb callbacks */
    ULong endCodeOffset;                /* offset for system endcode */

    /* valid for all callbacks */
    void *cbData;			/* user-supplied cb data */
    PacketDataPtr packetDown;		/* packet from below mpeg1 layer */
    boolean prevPacketDown;		/* true if token in prev packetDown */
} Mpeg1MuxCbInfo, *Mpeg1MuxCbInfoPtr;

/*
 * Generally, you don't need to call this directly; instead,
 * OpenMpeg1PacketStreams() (in nvrMpegConf.h) and ConfigureMpeg1Streams() will
 * handle these for you.
 */
typedef struct _mpeg1StreamConf {
    PacketLayerPtr media;
    void (*warning)(char *fmt, ...);
    void (*tokenCb)(PacketLayerPtr, Mpeg1CbType, Mpeg1MuxCbInfoPtr);
    void *cbData;
    ULong cbSelect;	/* selection of (1<<Mpeg1CbType) bits */
    PacketOpsPtr ops;	/* set of ops to use (see below) */
} Mpeg1StreamConf, *Mpeg1StreamConfPtr;

CExternal PacketLayerPtr OpenMpeg1SystemStream(
    Mpeg1StreamConfPtr,
    InternalReasonPtr *pReason
);

CExternal PacketLayerPtr OpenMpeg1SystemStream(
    Mpeg1StreamConfPtr,
    InternalReasonPtr *pReason
);

CExternal void DoubleToScr(
    double scr,
    MpegPackHdrPtr pack
);

extern PacketOpsPtr Mpeg1SysOps;	/* standard mpeg1 system ops */
extern PacketOpsPtr Mpeg1SysSyntaxOps;	/* system ops w/ syntax checking */

#endif /* _MPEG1PACKET_H_ */
