/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	mpeg1Proto.h - mpeg1 protocol structures
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1Proto.h,v 1.12 1995/10/01 01:07:26 toddb Exp $
 */
#ifndef _MPEG1PROTO_H_
#define _MPEG1PROTO_H_

#include "nvrTypes.h"

typedef enum {
    StartCodePrefix  = 0x000001,

    /*
     * Video start codes
     */
    PictureStartCode   = 0x00000100,
    Slice00StartCode   = 0x00000101,
    /* ... */
    SliceAFStartCode   = 0x000001af,
    UserDataStartCode  = 0x000001b2,
    SeqHeaderStartCode = 0x000001b3,
    SeqErrorCode       = 0x000001b4,
    ExtensionStartCode = 0x000001b5,
    SeqenceEndCode     = 0x000001b7,
    GroupStartCode     = 0x000001b8,

    /*
     * System level start codes
     */
    PackStartCode      = 0x000001ba,
    SysStartCode       = 0x000001bb,
    SysEndCode         = 0x000001b9
} MpegSmuxCodes;

/*************************************************************************
 *
 * The Pack header
 *
 *************************************************************************/
typedef struct _mpegPack {
    ULong packHdrStart  :32;
    ULong marker1	: 4;	/* 0010 */
    ULong scr1		: 3;	/* first system clock reference */
    ULong markerA	: 1;	/* 1 */
    ULong scr2		:15;	/* scr continued... */
    ULong markerB	: 1;	/* 1 */
    ULong scr3		: 8;	/* scr continued... */

    ULong scr4		: 7;	/* scr continued... */
    ULong markerC	: 1;	/* 1 */
    ULong markerD	: 1;	/* 1 */
    ULong muxRate	:22;	/* system mux rate */
    ULong markerE	: 1;	/* 1 */
} MpegPackHdr, *MpegPackHdrPtr;

#define PackHdrLength         12

/*
 * This will convert the scr to a signed floating point number
 * Note that if the high-bit is set, then it looks like a
 * negative number and comparisons will work after the scr
 * rolls over past 0.  However, if it is rolling over the other way,
 * this won't work.
 */
#define ScrToDouble(pack)						\
  (									\
   (									\
    (long)(((pack)->scr4 >> 1)						\
         + ((pack)->scr3 << 6)						\
         + ((pack)->scr2 << 14)						\
         + ((pack)->scr1 << 29)) * 2.0					\
         + ((pack)->scr4 & 1)						\
   ) / 90000.0					\
  )

/*************************************************************************
 *
 * The System header
 *
 *************************************************************************/
typedef struct _mpegSystemHdr {
    ULong sysHdrStart   :32;
    ULong headerLength	:16;
    ULong marker1	: 1;	/* 1 */
    ULong rateBoundHi   :15;
    ULong rateBoundLo   : 7;
    ULong marker2	: 1;	/* 1 */
    ULong audioBound	: 6;
    ULong fixed		: 1;
    ULong csps		: 1;
    ULong audioLock	: 1;
    ULong videoLock	: 1;
    ULong marker3	: 1;	/* 1 */
    ULong videoBound	: 5;
    ULong reserved	: 8;
} MpegSystemHdr, *MpegSystemHdrPtr;

#define SystemHdrLength		12
#define GetRateBound(sys) (((sys)->rateBoundHi << 7) + (sys)->rateBoundLo)

#define StuffingMarker		1

typedef struct _stuffingByte {
    ULong marker        : 1;	/* 1 */
    ULong stuffing      : 7;
    ULong pad           :24;
} StuffingByte;

#define EmptyMarker		0x0f

/*************************************************************************
 *
 * Std buffer token in mpeg system packet
 *
 *************************************************************************/
#define StdBufferMarker		1
#define CopyStdBuffer(src, dst)					\
    {								\
	UChar *pDst = UCharP(dst);				\
	pDst[0] = UCharP(src)[0];				\
	pDst[1] = UCharP(src)[1];				\
	src = UCharP(src)+2;					\
    }

#define StdBufferToBytes(buf)					\
    ((buf)->scale ? (buf)->size * 1024 : (buf)->size * 128)

typedef struct _stdBuffer {
    ULong marker        : 2;	/* 01 */
    ULong scale         : 1;
    ULong size          :13;
    ULong pad           :16;
} StdBuffer, *StdBufferPtr;

/*************************************************************************
 *
 * Pts token in mpeg system packet
 *
 *************************************************************************/
#define PtsMarker		2
#define CopyTsInfo(src, ts)					\
    {								\
	(ts)->tsLow = ((UCharP(src)[0] & 0x6) << 29)		\
		    |  (UCharP(src)[1] << 22)			\
		    | ((UCharP(src)[2] & 0xfe) << 14)		\
		    |  (UCharP(src)[3] << 7)			\
		    |  (UCharP(src)[4] >> 1);			\
	(ts)->tsHigh = ((UCharP(src)[0] >> 3) & 1);		\
	src = UCharP(src)+5;					\
    }

typedef struct _mpegTimeStamp {
    ULong tsHigh;
    ULong tsLow;
} MpegTimeStamp, *MpegTimeStampPtr;

typedef struct _ptsProto {
    ULong marker1       : 4;	/* 0010 */
    ULong pts1		: 3;	/* first system clock reference */
    ULong markerA	: 1;	/* 1 */
    ULong pts2		:15;	/* pts continued... */
    ULong markerB	: 1;	/* 1 */
    ULong pts3		: 8;	/* pts continued... */

    ULong pts4		: 7;	/* pts continued... */
    ULong markerC	: 1;	/* 1 */
    ULong pad           :24;
} PtsProto, *PtsProtoPtr;

#define PtsDtsMarker1		3
#define PtsDtsMarker2		1

typedef struct _ptsDtsProto {
    ULong marker1       : 4;	/* 0011 */
    ULong pts1		: 3;	/* first system clock reference */
    ULong markerA	: 1;	/* 1 */
    ULong pts2		:15;	/* pts continued... */
    ULong markerB	: 1;	/* 1 */
    ULong pts3		: 8;	/* pts continued... */

    ULong pts4		: 7;	/* pts continued... */
    ULong markerC	: 1;	/* 1 */
    ULong marker2       : 4;	/* 0010 */
    ULong dts1		: 3;	/* first system clock reference */
    ULong markerD	: 1;	/* 1 */
    ULong dts2		:15;	/* dts continued... */
    ULong markerE	: 1;	/* 1 */

    ULong dts3		:15;	/* dts continued... */
    ULong markerF	: 1;	/* 1 */
    ULong pad           :16;
} PtsDtsProto, *PtsDtsProtoPtr;

#define PtsToDouble(pts)						\
  (									\
   (									\
    (long)(((pts)->pts4 >> 1)						\
	 + ((pts)->pts3 << 6)						\
	 + ((pts)->pts2 << 14)						\
	 + ((pts)->pts1 << 29)) * 2.0					\
	 + ((pts)->pts4 & 1)						\
   ) / 90000.0								\
  )

#define DtsToDouble(dts)						\
  (									\
   (									\
    (long)(((dts)->dts3 >> 1)						\
	 + ((dts)->dts2 << 14)						\
	 + ((dts)->dts1 << 29)) * 2.0					\
	 + ((dts)->dts3 & 1)						\
   ) / 90000.0								\
  )

#if 0
#define PtsToDouble(pts)						\
   (((pts)->pts4							\
   + ((pts)->pts3 << 7)							\
   + ((pts)->pts2 << 15)						\
   + ((pts)->pts1 * (double)(1<<30))) / 90000.0)

#define DtsToDouble(dts)						\
   (((dts)->dts3							\
   + ((dts)->dts2 << 15)						\
   + ((dts)->dts1 * (double)(1<<30))) / 90000.0)
#endif


typedef struct _streamId {
    ULong streamId      : 8;
    ULong marker        : 2;	/* 11 */
    ULong bufBoundScale : 1;
    ULong bufSizeBound  :13;
    ULong pad		: 8;	/* XXX this is not part of the protocol */
} StreamId, *StreamIdPtr;

#define StreamIdLength		3

#define StreamIdToBufferSize(id)					\
    ((id)->bufBoundScale ? (id)->bufSizeBound * 1024 : (id)->bufSizeBound * 128)

/*************************************************************************
 *
 * mpeg system packet header
 *
 *************************************************************************/
typedef struct _mpegPacketHdr {
    ULong startCode     :24;    /* start code prefix: 0x000001 */
    ULong id            : 8;    /* stream id */
    UShort length;              /* packet length */
    UChar data[2];              /* probably longer... */
} MpegPacketHdr, *MpegPacketHdrPtr;

#define PacketHdrLength		6

/*************************************************************************
 *
 * mpeg system stream ids
 *
 *************************************************************************/
typedef enum {
    MSId_Reserved = 0xbc,
    MSId_Private1 = 0xbd,
    MSId_Padding  = 0xbe,
    MSId_Private2 = 0xbf,
    MSId_Audio00  = 0xc0,
    MSId_Audio01, MSId_Audio02, MSId_Audio03, MSId_Audio04, MSId_Audio05,
    MSId_Audio06, MSId_Audio07, MSId_Audio08, MSId_Audio09, MSId_Audio10,
    MSId_Audio11, MSId_Audio12, MSId_Audio13, MSId_Audio14, MSId_Audio15,
    MSId_Audio16, MSId_Audio17, MSId_Audio18, MSId_Audio19, MSId_Audio20,
    MSId_Audio21, MSId_Audio22, MSId_Audio23, MSId_Audio24, MSId_Audio25,
    MSId_Audio26, MSId_Audio27, MSId_Audio28, MSId_Audio29, MSId_Audio30,
    MSId_Audio31,

    MSId_Video00  = 0xe0,
    MSId_Video01, MSId_Video02, MSId_Video03, MSId_Video04, MSId_Video05,
    MSId_Video06, MSId_Video07, MSId_Video08, MSId_Video09, MSId_Video10,
    MSId_Video11, MSId_Video12, MSId_Video13, MSId_Video14, MSId_Video15,

    MSId_Data00  = 0xf0,
    MSId_Data01, MSId_Data02, MSId_Data03, MSId_Data04, MSId_Data05,
    MSId_Data06, MSId_Data07, MSId_Data08, MSId_Data09, MSId_Data10,
    MSId_Data11, MSId_Data12, MSId_Data13, MSId_Data14, MSId_Data15
} MpegStreamId;

#endif /* _MPEG1PROTO_H_ */
