/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 *      All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *      mpeg1Packet.c - mpeg system demultiplexer
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/mpeg1Sys.c,v 1.9 1995/10/02 00:39:59 toddb Exp $
 */
#include "nvrPacketI.h"
#include "mpeg1PacketI.h"
#include "nvrAlloc.h"

static InternalReason CantProcess =
    { "Process function called for mpeg1 system packet layer.\n",
      IR_InternalProblem };

#ifdef CheckMpeg1SysSyntax

#define WhenCheckingSyntax(statement)	statement

static void
Mpeg1ForceAligment(PacketLayerPtr pLayer)
{
    if (pLayer->down->ops.alignPackets)
	(*pLayer->down->ops.alignPackets)(pLayer->down);
}

static void
CheckPacketPadBytes(Mpeg1PrivPtr priv,
		    UChar *pSrc,
		    int available)
{
    int badPadBytes = 0;
    char offBuf[ OffsetBufSize ];
    PacketDataPtr packet = priv->cb.packetDown;

    while ((*pSrc & 0x80) && --available >= 0) {
	badPadBytes += (*pSrc != 0xff);
	pSrc++;
    }
    if (badPadBytes) {
	(*priv->warning)("Err #%d: %d %s at %s\n",
	    ++priv->errorCount, badPadBytes,
	    "packet pad bytes did not have the value 0xff",
	    FormatOffset(offBuf, packet, pSrc));
    }
}

static void
CallCallback(Mpeg1PrivPtr priv, Mpeg1CbType type, PacketDataPtr packet)
{
    if (priv->cbSelect & (1<<type))
	(*priv->tokenCb)(&priv->pub, type, &priv->cb);
}

static void
CheckPacketMarker(Mpeg1PrivPtr priv, UChar *pSrc)
{
    char offBuf[ OffsetBufSize ];
    PacketDataPtr packet = priv->cb.packetDown;

    if (*pSrc != 0x0f) {
	(*priv->warning)("Err #%d: %s %d at %s\n%s\n",
	    ++priv->errorCount,
	    "Invalid 8-bit pts/dts packet token", *pSrc >> 4,
	    FormatOffset(offBuf, packet, pSrc),
	    "Must be 0x0f (no pts), 0x2N (pts) or 0x3N (pts/dts)");
    }
}

static void
ReportExtraId(Mpeg1PrivPtr priv, UChar *pSrc)
{
    PacketDataPtr packet = priv->cb.packetDown;
    char offBuf[ OffsetBufSize ];

    (*priv->warning)("Err #%d: %s %s at %s\n",
	++priv->errorCount,
	"stream id following system header",
	"not included in header length",
	FormatOffset(offBuf, packet, pSrc));
}

static void
CheckTokenAtSysStart(Mpeg1PrivPtr priv,
		     MpegToken curToken,
		     UChar *pSrc)
{
    PacketDataPtr packet = priv->cb.packetDown;
    char offBuf[ OffsetBufSize ];

    if (curToken != MT_StartCodeSearch2) {
	(*priv->warning)("Err #%d: %s (0x%x) %s\n",
	    ++priv->errorCount,
	    "Unexpected mpeg sys header start code",
	    SysStartCode,
	    FormatOffset(offBuf, packet, pSrc-4));
    }
}

static void
CheckPacketStart(Mpeg1PrivPtr priv,
		 MpegToken curToken,
		 int systemOffset,
		 UChar *pSrc)
{
    PacketDataPtr packet = priv->cb.packetDown;
    char offBuf[ OffsetBufSize ];
    ULong startCode = *ULongP(&priv->cb.packet.hdr);

    if (! BM_ISSET(&priv->streams, startCode & 0x7f)) {
	(*priv->warning)("Err #%d: %s 0x%x: %s\n",
	    ++priv->errorCount,
	    "Packet from unregistered stream",
	    startCode & 0xff,
	    FormatOffset(offBuf, packet, pSrc-4));
	BM_SET(&priv->streams, startCode & 0x7f);
    }
    if (curToken == MT_StartCodeSearch1) {
	(*priv->warning)("Err #%d: %s 0x%x: %s\n",
	    ++priv->errorCount,
	    "Unexpected mpeg packet start code", startCode,
	    FormatOffset(offBuf, packet, pSrc-4));
    }
    priv->cb.packet.nStuffBytes = 0;
}

static void
UpdateEndCodeOffset(Mpeg1PrivPtr priv, int systemOffset)
{
    priv->cb.endCodeOffset = systemOffset;
    priv->cb.prevPacketDown = (systemOffset < priv->pos);
}

static void
UpdateSysHdrOffset(Mpeg1PrivPtr priv, int systemOffset)
{
    priv->cb.syshOffset = systemOffset;
    priv->cb.prevPacketDown = (systemOffset < priv->pos);
}

static void
UpdatePackOffset(Mpeg1PrivPtr priv, int systemOffset)
{
    priv->cb.packOffset = systemOffset;
    priv->cb.prevPacketDown = (systemOffset < priv->pos);
}

static void
UpdatePacketOffset(Mpeg1PrivPtr priv, int systemOffset)
{
    priv->cb.packetOffset = systemOffset;
    priv->cb.prevPacketDown = (systemOffset < priv->pos);
}

static void
UpdateGap(Mpeg1PrivPtr priv,
          UChar *pSrc,
	  UChar *pStartSearch,
	  boolean report)
{
    PacketDataPtr packet = priv->cb.packetDown;
    int gap = pSrc - pStartSearch;

    /*
     * report gaps in the stream
     */
    if (priv->cbSelect & Mpeg1GapMask) {
	priv->cb.gap += gap;
	if (report && priv->cb.gap) {
	    priv->cb.gapOffset = priv->pos
			       + (pSrc - packet->buffer)
			       - priv->cb.gap;
	    priv->cb.prevPacketDown = (priv->cb.gapOffset < priv->pos);
	    (*priv->tokenCb)(&priv->pub, Mpeg1GapCb, &priv->cb);
	    priv->cb.gap = 0;
	}
    }
}

#else /* CheckMpeg1SysSyntax */

#define UpdateGap(_priv, _pSrc, _pStartSearch, report)	/* as nothing */
#define UpdateSysHdrOffset(_priv, _systemOffset)	/* as nothing */
#define UpdateEndCodeOffset(_priv, _systemOffset)	/* as nothing */
#define UpdatePackOffset(_priv, _systemOffset)		/* as nothing */
#define UpdatePacketOffset(_priv, _systemOffset)	/* as nothing */
#define CheckTokenAtSysStart(_priv, _curToken, _pSrc)	/* as nothing */
#define CheckPacketStart(_priv, _tok, _sysOff, _pSrc)	/* as nothing */
#define CheckPacketMarker(_priv, _pSrc)			/* as nothing */
#define CheckPacketPadBytes(_priv, _pSrc, _available)	/* as nothing */
#define CallCallback(_priv, _type, _packet)		/* as nothing */
#define ReportExtraId(_priv, _pSrc)			/* as nothing */
#define WhenCheckingSyntax(statement)			/* as nothing */

#endif /* CheckMpeg1SysSyntax */

static InternalReasonPtr
Mpeg1Deliver(PacketLayerPtr pLayer, PacketDataPtr packet)
{
    Mpeg1PrivPtr priv = (Mpeg1PrivPtr) pLayer;
    PacketDataPtr new;
    InternalReasonPtr reason = NULL;
    UChar *pSrc, *pSrcEnd, *pToken;
    ULong startCode;
    int tokenLength, tokenOff, missing, available;
    MpegToken curToken;

    WhenCheckingSyntax(UChar *pStartSearch;)
    WhenCheckingSyntax(long systemOffset;)

    priv->cb.packetDown = packet;

    /*
     * This prevents the packet from being freed by callbacks called by
     * us before we are through parsing it.
     */
    packet->refCount++;

    pSrc = packet->buffer;
    pSrcEnd = pSrc + packet->dataBytes;
    tokenOff = priv->tokenOff;
    curToken = priv->curToken;
    startCode = priv->startCode;
    while (pSrc < pSrcEnd) {

	switch (curToken) {
	case MT_StartCodeSearch1:
	case MT_StartCodeSearchSeek:
	case MT_StartCodeSearch2:
	case MT_StartCodeSearch3:
	    WhenCheckingSyntax(pStartSearch = pSrc);

	    /*
	     * This is special... we don't know where we are and
	     * we are looking for a start code.
	     */
	    while (pSrc < pSrcEnd) {
		startCode = (startCode << 8) | *pSrc++;
		if ((startCode >> 8) == StartCodePrefix) {

		    UpdateGap(priv, pSrc-4, pStartSearch, True);
		    WhenCheckingSyntax(
			systemOffset = priv->pos - 4 + (pSrc - packet->buffer));

		    switch (startCode) {
		    case PackStartCode:
			curToken = MT_PackHdr;
			WhenCheckingSyntax(
			    priv->cb.pack.packHdrStart = PackStartCode;
			)
			UpdatePackOffset(priv, systemOffset);
			break;

		    case SysStartCode:
			CheckTokenAtSysStart(priv, curToken, pSrc) ;
			UpdateSysHdrOffset(priv, systemOffset);
			priv->cb.sysh.sysHdrStart = SysStartCode;
			curToken = MT_SysHeader;
			priv->curId = 0;
			break;

		    case SysEndCode:
			UpdateEndCodeOffset(priv, systemOffset);
			CallCallback(priv, Mpeg1EndCodeCb, packet);
			WhenCheckingSyntax(pStartSearch = pSrc);
			continue;

		    default:
			if ( curToken == MT_StartCodeSearchSeek) {
			    continue;
			}
			if ((startCode & 0xff) < MSId_Reserved
			 || (startCode & 0xff) > MSId_Data15)
			    continue;	/* must be a video start code */

			*ULongP(&priv->cb.packet.hdr) = startCode;
			CheckPacketStart(priv, curToken, systemOffset, pSrc);
		        curToken = MT_PacketHdr;
			UpdatePacketOffset(priv, systemOffset);
			priv->cb.packet.subHdrBytes = 0;
			priv->cb.packet.parts = 0;
			break;
		    }

		    if (reason)
			goto parsingComplete;
		    tokenOff = 4;
		    startCode = ~0;
		    goto knownToken;
		}
	    }

	    UpdateGap(priv, pSrc, pStartSearch, False);
	    goto parsingComplete;

	default:
	    break;	/* Handled below */
	}

    knownToken:
	available = pSrcEnd - pSrc;

	/*
	 * the packet data is special in that we do not copy it to
	 * to the token buffer, we just turn it into a packet and
	 * ship it.
	 */
	if (curToken == MT_PacketData) {
	    if (available == 0)
		continue;
	    new = AllocPacket(&pLayer->freeList, 0);
	    if (new == NULL) {
		reason = AllocReason("mpeg packet data", sizeof(PacketData));
		goto parsingComplete;
	    }

	    new->buffer = pSrc;
	    tokenLength = priv->cb.packet.hdr.length
			- priv->cb.packet.subHdrBytes
			- tokenOff;
	    if (available > tokenLength)
		available = tokenLength;
	    new->dataBytes = available;
	    new->pLayer = pLayer;
	    new->down = packet;
	    new->packetType = priv->cb.packet.hdr.id & 0x7f;
	    new->pos = priv->streamPos[ new->packetType ];
	    new->timecode = priv->timecode[ new->packetType ];
	    priv->streamPos[ new->packetType ] += available;
	    pSrc += available;
	    packet->refCount++;

	    if (pLayer->pkiList) {
		reason = DeliverLayersAbove(pLayer, new);
	    } else {
		QueuePacket(&pLayer->queue, new);
	    }

	    if (tokenLength == available) {
		tokenOff = 0;
		curToken = MT_StartCodeSearch3;
		if (priv->cbSelect & Mpeg1PacketMask)
		    (*priv->tokenCb)(&priv->pub, Mpeg1PacketCb, &priv->cb);
	    } else {
		tokenOff += available;
	    }

	    if (reason)
		goto parsingComplete;
	    continue;
	}

	/*
	 * Figure out which token it is we are parsing, point the
	 * 'pToken' at the right place;
	 */
	switch(curToken) {
	case MT_PackHdr:    pToken = (UChar*)&priv->cb.pack; break;
	case MT_Pts:        pToken = (UChar*)&priv->cb.packet.u.pts; break;
	case MT_PtsDts:     pToken = (UChar*)&priv->cb.packet.u.ptsDts;break;
	case MT_PacketHdr:  pToken = (UChar*)&priv->cb.packet.hdr; break;
	case MT_SysHeader:  pToken = (UChar*)&priv->cb.sysh; break;

	case MT_StreamId:
	    if (tokenOff == 0) {
		if ((*pSrc & 0x80) == 0) {
		    curToken = MT_StartCodeSearch3;
		    if (priv->cbSelect & Mpeg1SysHeaderMask)
			(*priv->tokenCb)(&priv->pub,
					 Mpeg1SysHeaderCb, &priv->cb);
		    continue;
		}
	    }
	    if (priv->curId >= priv->nAllocIds) {
		pToken = (UChar*) &priv->idOverflow;
		ReportExtraId(priv, pSrc);
	    } else {
		pToken = (UChar*) &priv->cb.idList[ priv->curId ];
	    }
	    break;

	case MT_PacketBranch1:
	    if (*pSrc & 0x80) {
		/*
		 * This sub-token is variable length, so we take
		 * care of it here rather than do this entire
		 * loop for each pad byte
		 */
		CheckPacketPadBytes(priv, pSrc, available);
		while (*pSrc & 0x80) {
		    tokenOff++;
		    pSrc++;
		    if (--available == 0)
			break;
		}
		priv->cb.packet.nStuffBytes = tokenOff;
		priv->cb.packet.subHdrBytes = tokenOff;
		if (available == 0)
		    goto parsingComplete;
	    }
	    tokenOff = 0;
	    curToken = MT_PacketBranch2;
	    /* fall through ... */

	case MT_PacketBranch2:
	    /*
	     * Since we know (*pSrc & 0x80 == 0 (from MT_PacketBranch1),
	     * then to test for nextbits() == '01', we can simply
	     * say...
	     */
	    if (*pSrc >> 6) {
		priv->cb.packet.parts |= MpegPacketBuffer;
		curToken = MT_BufferSize;
		pToken = (UChar*)&priv->cb.packet.buffer;
		priv->cb.packet.subHdrBytes += priv->tokenSize[ MT_BufferSize ];
		break;
	    }
	    curToken = MT_PacketBranch3;
	    /* fall through ... */

	case MT_PacketBranch3:
	    /*
	     * Decide which of pts, pts/dts or '00001111' happens next.
	     */
	    switch (*pSrc >> 4) {
	    case 2:
		priv->cb.packet.parts |= MpegPacketPts;
		pToken = (UChar*)&priv->cb.packet.u.pts;
		priv->cb.packet.subHdrBytes += priv->tokenSize[ MT_Pts ];
		curToken = MT_Pts;
		break;

	    case 3:
		priv->cb.packet.parts |= MpegPacketPts | MpegPacketDts;
		pToken = (UChar*)&priv->cb.packet.u.ptsDts;
		curToken = MT_PtsDts;
		priv->cb.packet.subHdrBytes += priv->tokenSize[ MT_PtsDts ];
		break;

	    default: /* no pts or dts */
		CheckPacketMarker(priv, pSrc);
		curToken = MT_PacketData;
		tokenOff = 0;
		priv->cb.packet.u.marker = *pSrc++;
		priv->cb.packet.subHdrBytes++;
		continue;
	    }
	    break;

	default:
	    reason = InternalProblem("Mpeg1Deliver: token %d\n",
			curToken);
	    goto parsingComplete;
	}

	/*
	 * Figure out how much is missing from the current token.
	 */
	tokenLength = priv->tokenSize[ curToken ];
	missing = tokenLength - tokenOff;
	if (missing <= 0) {
	    reason = InternalProblem("token %s missing 0 bytes!\n",
			priv->tokenNames[ curToken ]);
	    goto parsingComplete;
	}
	if (available > missing)
	    available = missing;

	pToken += tokenOff;
	switch (available) {
	default:
	    reason = InternalProblem("fill %d bytes on token %s?\n",
			available, priv->tokenNames[ curToken ]);
	    goto parsingComplete;

	case 10: *pToken++ = *pSrc++;
	case 9:  *pToken++ = *pSrc++;
	case 8:  *pToken++ = *pSrc++;
	case 7:  *pToken++ = *pSrc++;
	case 6:  *pToken++ = *pSrc++;
	case 5:  *pToken++ = *pSrc++;
	case 4:  *pToken++ = *pSrc++;
	case 3:  *pToken++ = *pSrc++;
	case 2:  *pToken++ = *pSrc++;
	case 1:  *pToken++ = *pSrc++;
	case 0:  ;	/* nothing available */
	}

	/*
	 * Finally, if we have the entire token, then pick the correct
	 * next token to parse
	 */
	tokenOff += available;
	if (tokenOff == tokenLength) {
	    tokenOff = 0;
	    switch(curToken) {
	    case MT_BufferSize: curToken = MT_PacketBranch3; break;

	    case MT_Pts:
	    case MT_PtsDts:
		/*
		 * Convert the pts from a 90khz clock to a 1khz clock
		 */
		priv->timecode[ priv->cb.packet.hdr.id & 0x7f ] =
		    PtsToDouble(&priv->cb.packet.u.pts) * MilliSecPerSecond;
		curToken = MT_PacketData;
		break;

	    case MT_PackHdr:
		curToken = MT_StartCodeSearch2;
		if (priv->cbSelect & Mpeg1PackMask)
		    (*priv->tokenCb)(&priv->pub, Mpeg1PackCb, &priv->cb);
		break;

	    case MT_SysHeader:
		{
		    int hdrLen = priv->cb.sysh.headerLength + 6;
		    if (hdrLen > priv->tokenSize[ MT_SysHeader ]) {
			curToken = MT_StreamId;
			if (NULL != (reason = AllocMpeg1IdList(priv)))
			    goto parsingComplete;
		    } else {
			curToken = MT_StartCodeSearch3;
			if (priv->cbSelect & Mpeg1SysHeaderMask)
			    (*priv->tokenCb)(&priv->pub,
					     Mpeg1SysHeaderCb, &priv->cb);
		    }
		}
		break;

	    case MT_PacketHdr:
		if (priv->cb.packet.hdr.id == MSId_Private2)
		    curToken = MT_PacketData;
		else
		    curToken = MT_PacketBranch1;
		break;

	    case MT_StreamId:
		{
		    int idIndex;

		    idIndex = priv->cb.idList[ priv->curId++ ].streamId;
		    BM_SET(&priv->streams, idIndex & 0x7f);
		}
		break; /* don't change the current token */

	    default:            /* don't change it */
		break;
	    }
	} else {
	    /*
	     * we ran out of data in this packet, so we move on.
	     */
	    goto parsingComplete;
	}
    }

parsingComplete:
    /*
     * We are through parsing the packet... this will alow it to be freed.
     */
    if (priv->cbSelect & Mpeg1PacketDoneMask)
	(*priv->tokenCb)(&priv->pub, Mpeg1PacketDoneCb, &priv->cb);
    if (--packet->refCount == 1)
	(*packet->pLayer->ops.free)(packet);

    priv->pos += packet->dataBytes;
    if (reason) {
	priv->tokenOff = 0;
	priv->curToken = MT_StartCodeSearch3;
	priv->startCode = ~0;
    } else {
	priv->tokenOff = tokenOff;
	priv->curToken = curToken;
	priv->startCode = startCode;
    }

    return(reason);
}

static InternalReasonPtr
Mpeg1Process(PacketLayerPtr pLayer)
{
    return( &CantProcess );
}

static InternalReasonPtr
Mpeg1Flush(PacketLayerPtr pLayer)
{
    return( NULL );
}

static InternalReasonPtr
Mpeg1Seek(PacketLayerPtr pLayer, ULong absOffset)
{
    Mpeg1PrivPtr priv = (Mpeg1PrivPtr) pLayer;
    void *cbData;
    StreamId *idList;

    idList = priv->cb.idList;
    cbData = priv->cb.cbData;
    memset((char *)&priv->cb, 0, sizeof(priv->cb));
    memset((char *)&priv->streamPos, 0, sizeof(priv->streamPos));
    priv->cb.cbData = cbData;
    priv->cb.idList = idList;

    priv->startCode = ~0;
    priv->curToken = MT_StartCodeSearchSeek;	/* this is where we start */
    priv->tokenOff = 0;
    priv->pos = 0;	/* hard to do better */
    priv->cb.gap = 0;

    return( NULL );
}

static void
Mpeg1Close(PacketLayerPtr pLayer)
{
    if (--pLayer->refCount)
        return;

    FreePacketList(&pLayer->freeList);
    FreePacketList(&pLayer->queue);
    if (pLayer->down)
        (*pLayer->down->ops.close)(pLayer->down);
    free((char *)pLayer);
}

static void
Mpeg1Free(PacketDataPtr packet)
{
    PacketDataPtr nextPacket = packet->down;

    if (--packet->refCount)
	return;
    (*nextPacket->pLayer->ops.free)(nextPacket);
    QueuePacket(&packet->pLayer->freeList, packet);
}

static PacketOps sysOps = {
    Mpeg1Deliver,
    Mpeg1Process,
    Mpeg1Flush,
    Mpeg1Close,
    Mpeg1Seek,
    Mpeg1Free,
#ifdef CheckMpeg1SysSyntax
    Mpeg1ForceAligment
#endif
};

#ifdef CheckMpeg1SysSyntax
    PacketOpsPtr Mpeg1SysSyntaxOps = &sysOps;
#else
    PacketOpsPtr Mpeg1SysOps = &sysOps;
#endif
