/*
 * (c) Copyright 1992 North Valley Research, Inc.
 *         All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *     nvrBitparse.h - bit-level parsing
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrBitparse.h,v 1.3 1995/05/04 16:28:02 toddb Exp $
 */
#ifndef _NVRBITPARSE_H_
#define _NVRBITPARSE_H_
#include "nvrTypes.h"

#define KeepBits(nBits, value)  ((value) & ((1<<(nBits)) - 1))

#define NextBitsMacro(_cache, _nBits)	((_cache) >> (32 - (_nBits)))

#define ConsumeBitsNoFillMacro(_cache, _nCacheBits, _nConsume)		\
    {									\
	_cache <<= _nConsume;						\
	_nCacheBits -= _nConsume;					\
    }

/*
 * ConsumeBitsMacro() is the most crucial macro because it is used in the
 * innermost loop when decoding dct run and level.  There are a variety of
 * methods here.
 */
#ifndef ConsumeBitsMethod
#define ConsumeBitsMethod 1
#endif

#if ConsumeBitsMethod == 1
#define ConsumeBitsMacro(_cache, _nCacheBits, _pBuf, _nConsume)		\
    {									\
	_cache <<= _nConsume;						\
	_nCacheBits -= _nConsume;					\
	switch (_nCacheBits >> 3) {					\
	case 0: _cache |= *(_pBuf)++ << (24 - _nCacheBits); _nCacheBits += 8; \
	case 1: _cache |= *(_pBuf)++ << (24 - _nCacheBits); _nCacheBits += 8; \
	case 2: _cache |= *(_pBuf)++ << (24 - _nCacheBits); _nCacheBits += 8; \
	}                                                               \
    }
#endif

#if ConsumeBitsMethod == 2
#define ConsumeBitsMacro(_cache, _nCacheBits, _pBuf, _nConsume)		\
    {									\
	int _newBytes, _newBits;					\
	_cache <<= _nConsume;						\
	_nCacheBits -= _nConsume;					\
	_newBytes = 3 - (_nCacheBits >> 3);				\
	if (_newBytes == 0) { /* most common, do nothing */		\
	    ;								\
	} else if (_newBytes == 1) {					\
	    _cache |= *(_pBuf)++ << (24 - _nCacheBits);			\
	    _nCacheBits += 8;						\
	} else if (_newBytes == 2) {					\
	    _newBits  = *(_pBuf)++ << 8;				\
	    _newBits |= *(_pBuf)++;					\
	    _cache |= _newBits << (16 - _nCacheBits);			\
	    _nCacheBits += 16;						\
	} else if (_newBytes == 3) {					\
	    _newBits  = *(_pBuf)++ << 16;				\
	    _newBits |= *(_pBuf)++ << 8;				\
	    _newBits |= *(_pBuf)++;					\
	    _cache |= _newBits << (8 - _nCacheBits);			\
	    _nCacheBits += 24;						\
	}								\
    }
#endif

#define FillCacheMacro(_pParse, _cache, _nCacheBits, _pBuf)		\
    {									\
        _pBuf = (_pParse)->pBuf;					\
        _nCacheBits = (_pParse)->nBits;					\
        _cache = *(_pBuf)++ << (32-_nCacheBits);			\
        ConsumeBitsMacro(_cache, _nCacheBits, _pBuf, 0);		\
    }

#define ReadBitsMacro(_value, _cache, _nCacheBits, _pBuf, _nConsume)	\
    {									\
        _value = NextBitsMacro(_cache, _nConsume);			\
        ConsumeBitsMacro(_cache, _nCacheBits, _pBuf, _nConsume);	\
    }

#define ReadBitsNoFillMacro(_value, _cache, _nCacheBits, _nConsume)	\
    {									\
        _value = NextBitsMacro(_cache, _nConsume);			\
        ConsumeBitsNoFillMacro(_cache, _nCacheBits, _nConsume);		\
    }

#define InitBitParseMacro(_pParse, _pBuf)				\
    {									\
        (_pParse)->pBuf = _pBuf;					\
        (_pParse)->nBits = 8;						\
    }

#define EmptyCacheMacro(_pParse, _cache, _nCacheBits, _pBuf)		\
    {									\
	switch (_nCacheBits >> 3) {					\
	case 3: *(_pBuf)-- = _cache >> 24; _cache <<= 8; _nCacheBits -= 8; \
	case 2: *(_pBuf)-- = _cache >> 24; _cache <<= 8; _nCacheBits -= 8; \
	case 1: *(_pBuf)-- = _cache >> 24; _cache <<= 8; _nCacheBits -= 8; \
	case 0: *(_pBuf)-- = _cache >> 24;				\
	}								\
        (_pParse)->pBuf = _pBuf;					\
        (_pParse)->nBits = _nCacheBits;					\
    }

#define DeclareBitParseVars(_cache, _nCacheBits, _pBuf)			\
    UChar *_pBuf;							\
    int _nCacheBits;							\
    ULong _cache

#define DeclareBitParseArgs(_cache, _nCacheBits, _pBuf)			\
    UChar *_pBuf,							\
    int _nCacheBits,							\
    ULong _cache

/*
 * This structure describes the holding area for bit parsing while
 * parsing is *not* going on.  FillCacheMacro() takes a pointer
 * to this and initializes the local variables.  EmptyCacheMacro()
 * puts the cache variables back into this array
 */
typedef struct _bitParseInfo {
    UChar *pBuf;		/* current pointer into array of bits */
    short nBits;		/* bits not used out of the *pBuf byte */
} BitParseInfo, *BitParseInfoPtr;

#endif /* _NVRBITPARSE_H_ */
