/*
 * Copyright (c) 1992, 1993, 1994 North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	nvrMpegConf.c - open and configure mpeg streams
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrMpegConf.c,v 1.19 1995/11/15 00:21:24 toddb Exp $
 */
#include "nvrPacketI.h"
#include "mpeg1PacketI.h"
#include "nvrMpegConf.h"
#include "nvrAlloc.h"
#include "fileContent.h"
#include "fileAccess.h"

static InternalReason PipesNotImplemented =
    { "fifo and pipe support not implemented yet", IR_NotImplemented };

static boolean
HasDataPackets(PacketLayerPtr mpeg)
{
    PacketDataPtr packet;

    for (packet = mpeg->queue; packet; packet = packet->next) {
	if (packet->packetType >= (MSId_Audio00 & 0x7f)
	 && packet->packetType <= (MSId_Data15 & 0x7f))
	    return(True);
    }
    return(False);
}

static InternalReasonPtr
OpenSystemStream(Mpeg1PacketInfoPtr info,
		 PacketLayerPtr media,
		 Mpeg1PacketConfPtr sConf,
		 BitMaskPtr interest)
{
    PacketLayerPtr mpegLayer;
    Mpeg1StreamConf mpegConf;
    InternalReasonPtr irp;
    Mpeg1PrivPtr priv;
    int streamId;

    /*
     * This is a system layer, open it.  Note that we don't
     * include the callback function because after figuring out what
     * streams are present, we'll seek back to the beginning.
     */
    memset((char *)&mpegConf, 0, sizeof(mpegConf));
    mpegConf.media = media;
    mpegConf.warning = sConf->warning;
    mpegConf.ops = sConf->mpegOps;

    mpegLayer = OpenMpeg1SystemStream(&mpegConf, &irp);
    if (mpegLayer == NULL) {
	(*media->ops.close)(media);
	return(irp);
    }
    info->media = mpegLayer;
    priv = (Mpeg1PrivPtr)mpegLayer;

    /*
     * Express interest in all the video and audio packets
     * comming from the media layer below the mpeg system layer.
     */
    irp = RegisterPacketInterest(mpegLayer, interest);
    if (irp) {
	(*mpegLayer->ops.close)(media);
	return(irp);
    }

    for (;;) {
	/*
	 * We don't want to continue until we have seen the first
	 * non-padding packet.  The last
	 * condition suggests that we have seen all the system headers
	 * (there are one or more in video cd and cdi)
	 */
	if (HasDataPackets(mpegLayer))
	    break;

	if (mpegLayer->eof) {
	    irp = BadValueReason("%s does not have any data packets.\n",
		    ExtractPathSuffix(sConf->path));
	} else {
	    irp = ReadNextPacket(mpegLayer);
	}
	if (irp) {
	    (*mpegLayer->ops.close)(mpegLayer);
	    return(irp);
	}
    }

    for (streamId = MSId_Reserved; streamId <= MSId_Data15; streamId++) {
	if (! BM_ISSET(&priv->streams, streamId & 0x7f))
	    continue;

	switch (streamId) {
	case MSId_Reserved:
	case MSId_Private1:
	case MSId_Padding:
	case MSId_Private2:
	    info->other |= (1<<(streamId & 0x3));
	    break;

	case MSId_Audio00: case MSId_Audio01: case MSId_Audio02:
	case MSId_Audio03: case MSId_Audio04: case MSId_Audio05:
	case MSId_Audio06: case MSId_Audio07: case MSId_Audio08:
	case MSId_Audio09: case MSId_Audio10: case MSId_Audio11:
	case MSId_Audio12: case MSId_Audio13: case MSId_Audio14:
	case MSId_Audio15: case MSId_Audio16: case MSId_Audio17:
	case MSId_Audio18: case MSId_Audio19: case MSId_Audio20:
	case MSId_Audio21: case MSId_Audio22: case MSId_Audio23:
	case MSId_Audio24: case MSId_Audio25: case MSId_Audio26:
	case MSId_Audio27: case MSId_Audio28: case MSId_Audio29:
	case MSId_Audio30: case MSId_Audio31:
	    info->audio |= (1<<(streamId & 0x1f));
	    break;

	case MSId_Video00: case MSId_Video01: case MSId_Video02:
	case MSId_Video03: case MSId_Video04: case MSId_Video05:
	case MSId_Video06: case MSId_Video07: case MSId_Video08:
	case MSId_Video09: case MSId_Video10: case MSId_Video11:
	case MSId_Video12: case MSId_Video13: case MSId_Video14:
	case MSId_Video15:
	    info->video |= (1<<(streamId & 0xf));
	    break;

	case MSId_Data00: case MSId_Data01: case MSId_Data02:
	case MSId_Data03: case MSId_Data04: case MSId_Data05:
	case MSId_Data06: case MSId_Data07: case MSId_Data08:
	case MSId_Data09: case MSId_Data10: case MSId_Data11:
	case MSId_Data12: case MSId_Data13: case MSId_Data14:
	case MSId_Data15:
	    info->data |= (1<<(streamId & 0xf));
	    break;

	default:
	    (*priv->warning)("Error #%d: Unknown stream type: 0x%x.\n",
		++priv->errorCount, streamId);
	    break;
	}
    }

    priv->tokenCb = sConf->mpegTokenCb;
    priv->cb.cbData = priv->cbData = sConf->mpegCbData;
    priv->cbSelect = sConf->mpegCbSelect;

    return( SeekPacketOffset(media, 0) );
}

static InternalReasonPtr
PipeOpenMpeg1PacketStreams(char *path, Mpeg1PacketInfoPtr info)
{
    return(&PipesNotImplemented);
}

static InternalReasonPtr
FsOpenMpeg1PacketStreams(char *path,
			 Mpeg1PacketConfPtr sConf,
			 Mpeg1PacketInfoPtr info)
{
    InternalReasonPtr irp;
    FsPacketConf fsConf;
    CdPacketConf vcdConf;
    PacketLayerPtr mediaLayer, vcdLayer;
    FileInfo fileInfo;
    BitMask mask;
    BitMaskPtr interest;

    /*
     * Open up the fs packet layer.
     */
    memset((char *)&fsConf, 0, sizeof(fsConf));
    fsConf.file = sConf->path;
    fsConf.warning = sConf->warning;
    fileInfo.type = FT_MpegSystem;

    switch(sConf->format) {
    case Mpeg1VcdFormat:	   fsConf.packetSize = 2324 * 4; break;
    case Mpeg1VcdSegFormat:	   fsConf.packetSize = 2324 * 4; break;
    case Mpeg1CdiFormat:	   fsConf.packetSize = 2324 * 4; break;
    case Mpeg1VcdWithSubheaders:   fsConf.packetSize = 2336 * 4; break;
    case Mpeg1VcdSegWithSubheaders:fsConf.packetSize = 2336 * 4; break;
    case Mpeg1CdiWithSubheaders:   fsConf.packetSize = 2336 * 4; break;

    case Mpeg1GenericFormat:
	fsConf.packetSize = sConf->bufSize;
	irp = GetFileContentInfo(sConf->path, &fileInfo);
	if (irp)
	    return(irp);

	switch (fileInfo.type) {
	case FT_MpegAudio: info->audio = 1; break;
	case FT_MpegVideo: info->video = 1; break;
	case FT_MpegSystem: break;
	default:
	    if (fileInfo.type) {
		irp = BadValueReason("%s is a %s,\nbut %s.\n",
		    ExtractPathSuffix(sConf->path),
		    EnumToName(fileInfo.type, AllFileTypes),
		    "but should should be an MPEG video, audio or system file");
		return(irp);
	    }
	    break;
	}

	break;

    default:
	irp = InternalProblem("Invalid mpeg system format: 0x%x\n",
		sConf->format);
	return(irp);
    }

    mediaLayer = OpenFsPacket(&fsConf, &irp);
    if (mediaLayer == NULL)
	return(irp);

    /*
     * If we are reading a cd with subheaders, stick the cd sector
     * layer in there just above the fs layer.
     */
    switch (sConf->format) {
    case Mpeg1VcdWithSubheaders:
    case Mpeg1VcdSegWithSubheaders:
    case Mpeg1CdiWithSubheaders:
	vcdConf.sectorCb = sConf->sectorCb;
	vcdConf.sectorCbData = sConf->sectorCbData;
	vcdConf.warning = sConf->warning;
	vcdLayer = OpenCdPacket(&vcdConf, mediaLayer, &irp);
	if (vcdLayer == NULL) {
	    (*mediaLayer->ops.close)(mediaLayer);
	    return(irp);
	}
	mediaLayer = vcdLayer;

        /*
         * Ask for all net packets
         */
	irp = RegisterPacketInterest(vcdLayer, NULL);
	if (irp) {
	    (*mediaLayer->ops.close)(mediaLayer);
	    return(irp);
	}

	interest = &mask;
	BM_ZERO(interest);
	BM_SET(interest, CdAudioSector);
	BM_SET(interest, CdVideoSector);

	break;

    default:
	interest = NULL;	/* interested in everything */
	break;
    }

    /*
     * most of the following code should be common to all
     * implementations: net, file system, cd, etc.
     */
    if (fileInfo.type == FT_MpegAudio || fileInfo.type == FT_MpegVideo) {
	/*
	 * Nothing to do... we can let the media layer handle
	 * all of the packet delivery.
	 */
	info->media = mediaLayer;
        return(NULL);
    }

    return( OpenSystemStream(info, mediaLayer, sConf, interest) );
}

static InternalReasonPtr
NetOpenMpeg1PacketStreams(char *path,
			 Mpeg1PacketConfPtr sConf,
			 Mpeg1PacketInfoPtr info)
{
    InternalReasonPtr irp;
    NetPacketConf netConf;
    CdPacketConf vcdConf;
    PacketLayerPtr mediaLayer, vcdLayer;
    FileContentType type;
    BitMask mask;
    BitMaskPtr interest;

    /*
     * Open up the net packet layer.
     */
    memset((char *)&netConf, 0, sizeof(netConf));
    netConf.source= sConf->path;
    netConf.warning = sConf->warning;
    switch(sConf->format) {
    case Mpeg1GenericFormat:	   netConf.packetSize = sConf->bufSize; break;
    case Mpeg1VcdFormat:	   netConf.packetSize = 2324 * 4; break;
    case Mpeg1VcdSegFormat:	   netConf.packetSize = 2324 * 4; break;
    case Mpeg1CdiFormat:	   netConf.packetSize = 2324 * 4; break;
    case Mpeg1VcdWithSubheaders:   netConf.packetSize = 2336 * 4; break;
    case Mpeg1VcdSegWithSubheaders:netConf.packetSize = 2336 * 4; break;
    case Mpeg1CdiWithSubheaders:   netConf.packetSize = 2336 * 4; break;
    default:
	irp = InternalProblem("Invalid mpeg system format: 0x%x\n",
		sConf->format);
	return(irp);
    }
    
    mediaLayer = OpenNetPacket(&netConf, &irp);
    if (mediaLayer == NULL)
	return(irp);
    
    irp = ReadNextPacket(mediaLayer);
    if (irp) {
	(*mediaLayer->ops.close)(mediaLayer);
	return(irp);
    }
    
    type = GetFileType((char *)mediaLayer->queue->buffer,
		       mediaLayer->queue->dataBytes);
    switch (type) {
    case FT_MpegAudio: info->audio = 1; break;
    case FT_MpegVideo: info->video = 1; break;
    case FT_MpegSystem: break;
    default:
	if (type) {
	    irp = BadValueReason("input is a %s,\nbut %s.\n",
		    EnumToName(type, AllFileTypes),
		    "but should be an MPEG video, audio or system file");
	    return(irp);
	}
	break;
    }

    /*
     * If we are reading a cd with subheaders, stick the cd sector
     * layer in there just above the fs layer.
     */
    switch (sConf->format) {
    case Mpeg1VcdWithSubheaders:
    case Mpeg1VcdSegWithSubheaders:
    case Mpeg1CdiWithSubheaders:
	vcdConf.sectorCb = sConf->sectorCb;
	vcdConf.sectorCbData = sConf->sectorCbData;
	vcdConf.warning = sConf->warning;
	vcdLayer = OpenCdPacket(&vcdConf, mediaLayer, &irp);
	if (vcdLayer == NULL) {
	    (*mediaLayer->ops.close)(mediaLayer);
	    return(irp);
	}

	/*
	 * Ask for all net packets
	 */
	irp = RegisterPacketInterest(vcdLayer, NULL);
	if (irp) {
	    (*mediaLayer->ops.close)(mediaLayer);
	    return(irp);
	}
	mediaLayer = vcdLayer;

	interest = &mask;
	BM_ZERO(interest);
	BM_SET(interest, CdAudioSector);
	BM_SET(interest, CdVideoSector);

	break;

    default:
	interest = NULL;	/* interested in everything */
	break;
    }

    /*
     * most of the following code should be common to all
     * implementations: net, file system, cd, etc.
     */
    if (type == FT_MpegAudio || type == FT_MpegVideo) {
	/*
	 * Nothing to do... we can let the media layer handle
	 * all of the packet delivery.
	 */
	info->media = mediaLayer;
        return(NULL);
    }

    return( OpenSystemStream(info, mediaLayer, sConf, interest) );
}

InternalReasonPtr
OpenMpeg1PacketStreams(Mpeg1PacketConfPtr conf, Mpeg1PacketInfoPtr info)
{
    InternalReasonPtr irp = NULL;
    int type;

    memset((char *)info, 0, sizeof(Mpeg1PacketInfo));

    type = FileType(conf->path);
    switch (type & ~FILE_SYMLINK) {
    case FILE_DIRECTORY:
    case FILE_BLOCK:
    case FILE_CHAR:
	irp = CreateReason("%s is a %s\n", conf->path, TypeName(type));
	irp->hint =IR_BadIOFile;
	break;

    case FILE_REGULAR:
	irp = FsOpenMpeg1PacketStreams(conf->path, conf, info);
	break;

    case 0:
	irp = NetOpenMpeg1PacketStreams(conf->path, conf, info);
	break;

    case FILE_SOCKET:
    case FILE_FIFO:
	irp = PipeOpenMpeg1PacketStreams(conf->path, info);
	break;
    }

    return(irp);
}

static InternalReasonPtr
Mpeg1ElemDeliver(PacketLayerPtr pLayer, PacketDataPtr packet)
{
    QueuePacket(&pLayer->queue, packet);
    return(NULL);
}

static InternalReasonPtr
Mpeg1ElemProcess(PacketLayerPtr pLayer)
{
    PacketDataPtr packet;
    
    while (NULL != (packet = pLayer->queue)) {
        pLayer->queue = packet->next;
        (*packet->pLayer->ops.free)(packet);
    }
    
    return( NULL );
}

static InternalReasonPtr
Mpeg1ElemFlush(PacketLayerPtr pLayer)
{
    return( NULL );
}

static InternalReasonPtr
Mpeg1ElemSeek(PacketLayerPtr pLayer, ULong absOffset)
{
    return( NULL );
}

static void
Mpeg1ElemClose(PacketLayerPtr pLayer)
{
    FreePacketList(&pLayer->freeList);
    FreePacketList(&pLayer->queue);
    (*pLayer->down->ops.close)(pLayer->down);
}

static void
Mpeg1ElemFree(PacketDataPtr packet)
{
    /* we don't allocate any packets */
}

static InternalReasonPtr
InitElemPacketLayer(Mpeg1PacketInfoPtr info, PacketLayerType type, int id)
{
    BitMask mask;
    PacketLayerPtr pLayer;

    pLayer = AllocPacketLayer(type, info->media, 0);
    if (pLayer == NULL)
	return(&IRBadAllocation);
    pLayer->ops.deliver = Mpeg1ElemDeliver;
    pLayer->ops.process = Mpeg1ElemProcess;
    pLayer->ops.flush   = Mpeg1ElemFlush;
    pLayer->ops.close   = Mpeg1ElemClose;
    pLayer->ops.seek    = Mpeg1ElemSeek;
    pLayer->ops.free    = Mpeg1ElemFree;
    pLayer->sibling     = info->list;
    info->list          = pLayer;

    BM_ZERO(&mask);
    BM_SET(&mask, id & 0x7f);
    return( RegisterPacketInterest(pLayer, &mask) );
}

static InternalReasonPtr
InitAudioPacketLayer(Mpeg1PacketInfoPtr info, Mpeg1AudioConfPtr aConf, int id)
{
    BitMask mask;
    PacketLayerPtr pLayer;
    InternalReasonPtr reason = NULL;

    aConf->media = info->media;
    pLayer = OpenMpeg1AudioStream(aConf, &reason);
    if (pLayer == NULL)
	return(reason);
    pLayer->sibling = info->list;
    info->list      = pLayer;

    BM_ZERO(&mask);
    /*
     * If there is a system stream layer below us, then express interest
     * in the stream id, otherwise ask for all the packets.
     */
    if (info->media->type == PacketFs) {
        return( RegisterPacketInterest(pLayer, NULL) );
    } else { 
        BM_SET(&mask, id & 0x7f);
        return( RegisterPacketInterest(pLayer, &mask) );
    }
}

static InternalReasonPtr
InitVideoPacketLayer(Mpeg1PacketInfoPtr info, Mpeg1VideoConfPtr vConf, int id)
{
    BitMask mask;
    PacketLayerPtr pLayer;
    InternalReasonPtr reason = NULL;

    vConf->media = info->media;
    pLayer = OpenMpeg1VideoStream(vConf, &reason);
    if (pLayer == NULL)
	return(reason);
    pLayer->sibling = info->list;
    info->list      = pLayer;

    BM_ZERO(&mask);
    /*
     * If there is a system stream layer below us, then express interest
     * in the stream id, otherwise ask for all the packets.
     */
    if (info->media->type == PacketFs) {
        return( RegisterPacketInterest(pLayer, NULL) );
    } else { 
        BM_SET(&mask, id & 0x7f);
        return( RegisterPacketInterest(pLayer, &mask) );
    }
}

InternalReasonPtr
ConfigureMpeg1Streams(Mpeg1PacketInfoPtr info,
		      Mpeg1VideoConfPtr vConf, ULong video,
		      Mpeg1AudioConfPtr aConf, ULong audio,
		      ULong data,
		      ULong other)
{
    int i;
    InternalReasonPtr reason = NULL;

    for (i = 0; data && i < 16; i++) {
	if ((data & (1<<i)) == 0)
	    continue;
	data &= ~(1<<i);
	reason = InitElemPacketLayer(info, PacketMpeg1Data, MSId_Data00 + i);
	if (reason)
	    goto configurationDone;
    }

    for (i = 0; other && i < 16; i++) {
	if ((other & (1<<i)) == 0)
	    continue;
	other &= ~(1<<i);
	reason = InitElemPacketLayer(info, PacketMpeg1Other, MSId_Reserved + i);
	if (reason)
	    goto configurationDone;
    }

    for (i = 0; audio && i < 32; i++) {
	if ((audio & (1<<i)) == 0)
	    continue;
	audio &= ~(1<<i);
	reason = InitAudioPacketLayer(info, aConf, MSId_Audio00 + i);
	if (reason)
	    goto configurationDone;
    }

    for (i = 0; video && i < 16; i++) {
	if ((video & (1<<i)) == 0)
	    continue;
	video &= ~(1<<i);
	reason = InitVideoPacketLayer(info, vConf, MSId_Video00 + i);
	if (reason)
	    goto configurationDone;
    }

configurationDone:
    DiscardQueuedPackets(info->media);

    return(reason);
}

void
FreeMpeg1PacketInfo(Mpeg1PacketInfoPtr info)
{
    PacketLayerPtr pLayer, next = NULL;

    for (pLayer = info->list; pLayer; pLayer = next) {
	next = pLayer->sibling;
	(*pLayer->ops.close)(pLayer);
    }
    info->list = NULL;
}
