/*
 * (c) Copyright 1992, 1993, 1994 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	nvrMpegConf.h - public interface to configure mpeg parsing
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/nvrMpegConf.h,v 1.6 1995/10/01 17:42:24 toddb Exp $
 */
#ifndef _NVRMPEGCONF_H_
#define _NVRMPEGCONF_H_

#include "mpeg1Packet.h"
#include "mpeg1vdec.h"
#include "mpeg1adec.h"
#include "vcdPacket.h"
#include "fsPacket.h"
#include "netPacket.h"

typedef enum {
    Mpeg1GenericFormat   = 0x0000,

    /*
     * basic formats
     */
    Mpeg1VcdFormat       = 0x0001,
    Mpeg1VcdSegFormat    = 0x0002,
    Mpeg1CdiFormat       = 0x0003,

    /*
     * format modifiers
     */
    Mpeg1SubheaderFormat = 0x0010,

    /*
     * combined formats
     */
    Mpeg1VcdWithSubheaders    = 0x0011,
    Mpeg1VcdSegWithSubheaders = 0x0012,
    Mpeg1CdiWithSubheaders    = 0x0013
} Mpeg1MuxFormat;

/*
 * This structure is filled in by OpenMpeg1PacketStreams() so that
 * the caller can tell what kind of streams are available
 */
typedef struct _mpeg1PacketInfo {
    ULong video;	/* mask of available video streams */
    ULong audio;	/* mask of available audio streams */
    ULong data;		/* mask of available data streams */
    ULong other;	/* mask of other streams (including padding) */
    PacketLayerPtr media; /* Mpeg system layer, or fs if not system stream */
    PacketLayerPtr list;/* elemetary mpeg packet streams; video, audio, etc */
} Mpeg1PacketInfo, *Mpeg1PacketInfoPtr;

/*
 * After opening an mpeg audio/video/system stream with
 * OpenMpeg1PacketStreams().  This is used to (re)configure
 * the list of audio, video streams we subscribe to and
 * put them in info->list.
 */
CExternal InternalReasonPtr ConfigureMpeg1Streams(
    Mpeg1PacketInfoPtr info,
    Mpeg1VideoConfPtr vConf, ULong video,
    Mpeg1AudioConfPtr aConf, ULong audio,
    ULong data,
    ULong other
);

/*
 * This is called first to open a stream.  It returns enough info
 * to decide how many streams there are.
 */
typedef struct _mpeg1PacketConf {
    char *path;                 /* network name, path name, cd track, etc */
    Mpeg1MuxFormat format;	/* format to be parsed (unless implicit) */
    long bufSize;		/* zero or preferred buffer size to use */
    PacketOpsPtr mpegOps;	/* see mpeg1Packet.h */
    void (*warning)(char *fmt, ...);

    /*
     * This function is called after four major mpeg system layer tokens
     * have been fully parsed: pack header, system header, packet and
     * end code.  If a Mpeg1PacketCb callback, then a corresponding
     * packet has been delivered.
     */
    void (*mpegTokenCb)(PacketLayerPtr, Mpeg1CbType, Mpeg1MuxCbInfoPtr);
    void *mpegCbData;
    ULong mpegCbSelect;	/* selection of (1<<Mpeg1CbType) bits */

    /*
     * This function is called before each cd-layer packet is
     * delivered to the mpeg system layer above it.  This is only
     * used by diagnostic routines that need to check cdrom-xa sector
     * subheaders.
     */
    void (*sectorCb)(PacketLayerPtr, PacketDataPtr, void *cdxaCbData);
    void *sectorCbData;
} Mpeg1PacketConf, *Mpeg1PacketConfPtr;

CExternal InternalReasonPtr OpenMpeg1PacketStreams(
    Mpeg1PacketConfPtr conf,
    Mpeg1PacketInfoPtr info
);

CExternal void FreeMpeg1PacketInfo(
    Mpeg1PacketInfoPtr
);

#endif /* _NVRMPEGCONF_H_ */
