/* $Header: /usr/local/src/nvrToMIT/RCS/DeepDisplay.c,v 1.3 1993/09/09 02:21:06 toddb Exp $ */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <Xm/Xm.h>
#include <Xm/XmP.h>
#include "DeepDisplay.h"
#include "DeepDisplayP.h"
#include "nvrTypes.h"
#include "nvrDebug.h"

/* Core class functions */
static void Realize();
static void Destroy();
static void Redisplay();
static void Resize();
static Boolean SetValues();
static XtGeometryResult QueryGeometry();

/* Resource definitions for DeepDisplay Widget */
static XtResource resources[] =
{
    {	XmNvisual, XmCVisual, XmRPointer,
	sizeof (Visual *),
	XtOffset (XnvrDeepDisplayWidget, deepDisplay.visual),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNinputCallback, XnvrCInputCallback, XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrDeepDisplayWidget, deepDisplay.input_callback),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNresizeCallback, XnvrCResizeCallback, XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrDeepDisplayWidget, deepDisplay.resize_callback),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNexposeCallback, XnvrCExposeCallback, XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrDeepDisplayWidget, deepDisplay.expose_callback),
	XmRImmediate, (XtPointer) NULL },

};

XnvrDeepDisplayClassRec xnvrDeepDisplayClassRec =
{
    {
	/* superclass	    */	    (WidgetClass) &coreClassRec,
	/* class_name	    */	    "XnvrDeepDisplay",
	/* widget_size	    */	    sizeof(XnvrDeepDisplayRec),
	/* class_initialize */	    NULL,
	/* chained class init */    NULL,
	/* class_inited	    */	    FALSE,
	/* initialize	    */	    NULL,
	/* initialize hook  */	    NULL,
	/* realize	    */	    (XtRealizeProc)Realize,
	/* actions	    */	    NULL,
	/* num_actions	    */	    0,
	/* resources	    */	    resources,
	/* num_resources    */	    XtNumber(resources),
	/* xrm_class	    */	    NULLQUARK,
	/* compress_motion  */	    TRUE,
	/* compress_exposure*/	    TRUE,
	/* compress enter/exit */   TRUE,
	/* visible_interest */	    FALSE,
	/* destroy	    */	    NULL,
	/* resize	    */	    NULL,
	/* expose	    */	    Redisplay,
	/* set_values	    */	    NULL,
	/* set values hook  */	    NULL,
	/* set values almost*/	    NULL,
	/* get values hook  */	    NULL,
	/* accept_focus	    */	    NULL,
	/* version	    */	    XtVersion,
	/* callback offset 1st */   NULL,
	/* default trans    */	    NULL,
	/* query geo proc   */	    NULL, /*(XtGeometryHandler)QueryGeometry,*/
	/* display accelerator */   NULL,
	/* reserved	    */	    NULL	/* extension record */
    },

    {					    	/* XnvrDeepDisplay */
	NULL,					/* extension */
    }
};

WidgetClass xnvrDeepDisplayWidgetClass = (WidgetClass) &xnvrDeepDisplayClassRec;

static void Realize(w, value_mask, attributes)
    XnvrDeepDisplayWidget w;
    XtValueMask* value_mask;
    XSetWindowAttributes* attributes;
{
    /* by this time, the superclass' Realize has already been called */

    attributes->backing_store = Always;
    *value_mask |= CWBackingStore;

    XtCreateWindow((Widget)w, InputOutput,
		   w->deepDisplay.visual, *value_mask, attributes);
}

static void Redisplay(w)
    XnvrDeepDisplayWidget w;
{
    WhenDebugEnabled( NormalOutput("XnvrDeepDisplayWidget::Redisplay\n") );
    if (XtIsRealized(w) && !w->core.being_destroyed)
    {
	XtCallCallbacks((Widget)w, XnvrNexposeCallback, NULL);
    }
}

