// $Header: /usr/local/src/nvrToMIT/RCS/GenericDialogs.C,v 1.6 1993/08/19 18:11:10 toddb Exp $

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/Frame.h>
#include <Xm/MessageB.h>
#include <Xm/MwmUtil.h>
#include <Xm/RowColumn.h>
#include "GenericDialogs.H"



void AskUser(Widget parent, char* question,
	     XtCallbackProc YesCB, XtPointer YesData,
	     XtCallbackProc NoCB,  XtPointer NoData,
	     XtCallbackProc HelpCB, XtPointer HelpData,
	     XtCallbackProc MapCB, XtPointer MapData)
{
    XmString messageString;
    Arg args[2];
    static Widget askUserDialog = NULL;

    /* If the dialog hasn't yet been created, construct it.
     * This set of widgets is reused -- do I need to make sure they aren't
     * already in use???
     */
    if (askUserDialog == NULL)
    {
	askUserDialog = XmCreateQuestionDialog
			    (parent, "Question", (ArgList)NULL, 0);
	XmString yesString = XmStringCreateSimple("Yes");
	XmString noString = XmStringCreateSimple("No");
	XtVaSetValues(askUserDialog,
	    XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
	    XmNokLabelString, yesString,
	    XmNcancelLabelString, noString,
	    NULL);
	XmStringFree(yesString);
	XmStringFree(noString);
    }

    /* tell the dialog whether to determine where to popup */
    if (MapCB)
    {
	XtVaSetValues(askUserDialog,
	    XmNdefaultPosition, False,
	    NULL);
    }
    else
    {
	XtVaSetValues(askUserDialog,
	    XmNdefaultPosition, True,
	    NULL);
    }

    /* OK - set the question to be asked */
    messageString = XmStringCreateLtoR(question, XmSTRING_DEFAULT_CHARSET);
    XtVaSetValues(askUserDialog,
	XmNmessageString, messageString,
	NULL);
    XmStringFree(messageString);

    /* Clean up from previous use. */
    if (XtHasCallbacks(askUserDialog, XmNokCallback) == XtCallbackHasSome)
    {
	XtRemoveAllCallbacks(askUserDialog, XmNokCallback);
    }
    if (XtHasCallbacks(askUserDialog, XmNcancelCallback) == XtCallbackHasSome)
    {
	XtRemoveAllCallbacks(askUserDialog, XmNcancelCallback);
    }
    if (XtHasCallbacks(askUserDialog, XmNhelpCallback) == XtCallbackHasSome)
    {
	XtRemoveAllCallbacks(askUserDialog, XmNhelpCallback);
    }
    if (XtHasCallbacks(askUserDialog, XmNmapCallback) == XtCallbackHasSome)
    {
	XtRemoveAllCallbacks(askUserDialog, XmNmapCallback);
    }

    /* Set up the callback routines (if they were passed in). */
    if (YesCB != NULL)
    {
	XtAddCallback(askUserDialog, XmNokCallback, YesCB, YesData);
    }
    if (NoCB != NULL)
    {
	XtAddCallback(askUserDialog, XmNcancelCallback, NoCB, NoData);
    }
    if (HelpCB == NULL)
    {
	XtSetSensitive(XmMessageBoxGetChild(askUserDialog, XmDIALOG_HELP_BUTTON),
			False);
    }
    else
    {
	XtSetSensitive(XmMessageBoxGetChild(askUserDialog, XmDIALOG_HELP_BUTTON),
			True);
	XtAddCallback(askUserDialog, XmNhelpCallback, HelpCB, HelpData);
    }
    if (MapCB)
    {
	XtAddCallback(askUserDialog, XmNmapCallback, MapCB, MapData);
    }

    XtManageChild(askUserDialog);
};




void WarnUser(Widget parent, char* warning,
	    XtCallbackProc OkCB, XtPointer OkData,
	    XtCallbackProc CancelCB,  XtPointer CancelData,
	    XtCallbackProc HelpCB, XtPointer HelpData,
	    XtCallbackProc MapCB, XtPointer MapData)
{
    XmString messageString;
    Arg args[2];
    int i;
    static Widget warnUserDialog = NULL;

    /* If the dialog hasn't yet been created, construct it.  */
    if (warnUserDialog == NULL)
    {
	warnUserDialog = XmCreateWarningDialog
			    (parent, "warning", (ArgList)NULL, 0);
	XtVaSetValues(warnUserDialog,
	    XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
	    NULL);
    }

    /* determine whether dialog is to determine the default position */
    if (MapCB)
    {
	XtVaSetValues(warnUserDialog,
	    XmNdefaultPosition, False,
	    NULL);
    }
    else
    {
	XtVaSetValues(warnUserDialog,
	    XmNdefaultPosition, True,
	    NULL);
    }

    /* OK - set the warning message */
    messageString = XmStringCreateLtoR(warning, XmSTRING_DEFAULT_CHARSET);
    i = 0;
    XtSetArg(args[i], XmNmessageString, messageString); i++;
    XtSetValues(warnUserDialog, args, i);
    XmStringFree(messageString);

    /* Clean up from previous use, then set up the callback routines
     * (if they were passed in).
     */
    if (XtHasCallbacks(warnUserDialog, XmNokCallback) == XtCallbackHasSome)
    {
	XtRemoveAllCallbacks(warnUserDialog, XmNokCallback);
    }
    if (XtHasCallbacks(warnUserDialog, XmNcancelCallback) == XtCallbackHasSome)
    {
	XtRemoveAllCallbacks(warnUserDialog, XmNcancelCallback);
    }
    if (XtHasCallbacks(warnUserDialog, XmNhelpCallback) == XtCallbackHasSome)
    {
	XtRemoveAllCallbacks(warnUserDialog, XmNhelpCallback);
    }
    if (XtHasCallbacks(warnUserDialog, XmNmapCallback) == XtCallbackHasSome)
    {
	XtRemoveAllCallbacks(warnUserDialog, XmNmapCallback);
    }
    if (OkCB != NULL)
    {
	XtAddCallback(warnUserDialog, XmNokCallback, OkCB, OkData);
    }
    if (CancelCB != NULL)
    {
	XtAddCallback(warnUserDialog, XmNcancelCallback, CancelCB, CancelData);
    }
    if (HelpCB == NULL)
    {
	XtSetSensitive(
	    XmMessageBoxGetChild(warnUserDialog, XmDIALOG_HELP_BUTTON),
	    False);
    }
    else
    {
	XtSetSensitive(
	    XmMessageBoxGetChild(warnUserDialog, XmDIALOG_HELP_BUTTON),
	    True);
	XtAddCallback(warnUserDialog, XmNhelpCallback, HelpCB, HelpData);
    }
    if (MapCB)
    {
	XtAddCallback(warnUserDialog, XmNmapCallback, MapCB, MapData);
    }

    XtManageChild(warnUserDialog);
};



void BlindInformUser(Widget parent, char* message)
{
    InformUser(parent, message,
            (XtCallbackProc)NULL, (XtPointer)NULL,
            (XtCallbackProc)NULL, (XtPointer)NULL,
            (XtCallbackProc)NULL, (XtPointer)NULL);
}

void InformUser(Widget parent, char* message,
	    XtCallbackProc OkCB, XtPointer OkData,
	    XtCallbackProc HelpCB, XtPointer HelpData,
	    XtCallbackProc MapCB, XtPointer MapData)
{
    XmString messageString;
    Arg args[2];
    int i;
    static Widget informUserDialog = NULL;

    /* If the dialog hasn't yet been created, construct it.  */
    if (informUserDialog == NULL)
    {
	i = 0;
	informUserDialog = XmCreateWarningDialog
			    (parent, "Information", (ArgList)NULL, i);
	XtUnmanageChild(XmMessageBoxGetChild(informUserDialog,
					    XmDIALOG_CANCEL_BUTTON));
	XmString moreString = XmStringCreateSimple("More");
	XtVaSetValues(informUserDialog,
	    XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
	    XmNhelpLabelString, moreString,
	    NULL);
	XmStringFree(moreString);
    }

    /* determine whether dialog is to determine the default position */
    if (MapCB)
    {
	XtVaSetValues(informUserDialog,
	    XmNdefaultPosition, False,
	    NULL);
    }
    else
    {
	XtVaSetValues(informUserDialog,
	    XmNdefaultPosition, True,
	    NULL);
    }

    /* OK - set the message */
    messageString = XmStringCreateLtoR(message, XmSTRING_DEFAULT_CHARSET);
    i = 0;
    XtSetArg(args[i], XmNmessageString, messageString); i++;
    XtSetValues(informUserDialog, args, i);
    XmStringFree(messageString);

    /* Clean up from previous use, then set up the callback routines
     * (if they were passed in).
     */
    if (XtHasCallbacks(informUserDialog, XmNokCallback) == XtCallbackHasSome)
    {
	XtRemoveAllCallbacks(informUserDialog, XmNokCallback);
    }
    if (XtHasCallbacks(informUserDialog, XmNhelpCallback) == XtCallbackHasSome)
    {
	XtRemoveAllCallbacks(informUserDialog, XmNhelpCallback);
    }
    if (XtHasCallbacks(informUserDialog, XmNmapCallback) == XtCallbackHasSome)
    {
	XtRemoveAllCallbacks(informUserDialog, XmNmapCallback);
    }
    if (OkCB != NULL)
    {
	XtAddCallback(informUserDialog, XmNokCallback, OkCB, OkData);
    }
    if (HelpCB == NULL)
    {
	XtSetSensitive(
	    XmMessageBoxGetChild(informUserDialog, XmDIALOG_HELP_BUTTON),
	    False);
    }
    else
    {
	XtSetSensitive(
	    XmMessageBoxGetChild(informUserDialog, XmDIALOG_HELP_BUTTON),
	    True);
	XtAddCallback(informUserDialog, XmNhelpCallback, HelpCB, HelpData);
    }
    if (MapCB)
    {
	XtAddCallback(informUserDialog, XmNmapCallback, MapCB, MapData);
    }

    XtManageChild(informUserDialog);
};


/*ARGSUSED*/
void gdNoCB(Widget w, Widget shell, XtPointer alsoUnused)
{
    XtPopdown(shell);
}


void MakeWidgetDialog(Widget parent, char* title,
	    Widget *shellReturn, Widget *widgetParentReturn,
	    XtCallbackProc YesCB, XtPointer YesData, char* YesLabel,
	    XtCallbackProc NoCB,  XtPointer NoData, char* NoLabel,
	    XtCallbackProc Custom1CB, XtPointer Custom1Data, char* Custom1Label,
	    XtCallbackProc Custom2CB, XtPointer Custom2Data, char* Custom2Label,
	    XtCallbackProc HelpCB, XtPointer HelpData, char* HelpLabel,
	    XtCallbackProc MapCB, XtPointer MapData)
{
    /* unique names for widgets... necessary? */
    *shellReturn =
	XtVaCreatePopupShell(title, topLevelShellWidgetClass, parent,
	XmNtitle, title,
	XmNmwmDecorations, MWM_DECOR_TITLE | MWM_DECOR_BORDER,
	NULL);
    Widget rowCol =
	XtVaCreateManagedWidget("rc", xmRowColumnWidgetClass, *shellReturn,
	NULL);
    *widgetParentReturn =
	XtVaCreateManagedWidget("wf", xmFrameWidgetClass, rowCol,
	NULL);
    Widget buttonFrame =
	XtVaCreateManagedWidget("bf", xmRowColumnWidgetClass, rowCol,
	XmNborderWidth, 0,
	XmNorientation, XmHORIZONTAL,
	NULL);
    Widget yesButton =
	XtVaCreateManagedWidget(YesLabel, xmPushButtonWidgetClass, buttonFrame,
	NULL);

    /* determine whether dialog is to determine the default position */
    if (MapCB)
    {
	XtVaSetValues(*shellReturn,
	    XmNdefaultPosition, False,
	    NULL);
    }
    else
    {
	XtVaSetValues(*shellReturn,
	    XmNdefaultPosition, True,
	    NULL);
    }

    if (Custom1CB != NULL)
    {
	Widget custom1Button =
	    XtVaCreateManagedWidget(Custom1Label, xmPushButtonWidgetClass,
		buttonFrame,
		NULL);
	XtAddCallback(custom1Button, XmNactivateCallback,
			Custom1CB, Custom1Data);
    }
    if (Custom2CB != NULL)
    {
	Widget custom2Button =
	    XtVaCreateManagedWidget(Custom2Label, xmPushButtonWidgetClass,
		buttonFrame,
		NULL);
	XtAddCallback(custom2Button, XmNactivateCallback,
			Custom2CB, Custom2Data);
    }
    Widget noButton =
	XtVaCreateManagedWidget(NoLabel, xmPushButtonWidgetClass, buttonFrame,
	    NULL);
    Widget helpButton =
	XtVaCreateManagedWidget(HelpLabel, xmPushButtonWidgetClass, buttonFrame,
	    NULL);
    if (YesCB == NULL)
    {
	XtSetSensitive(yesButton, False);
    }
    else
    {
	XtAddCallback(yesButton, XmNactivateCallback, YesCB, YesData);
    }
    if (NoCB == NULL)
    {
	XtSetSensitive(noButton, False);
    }
    else
    {
	XtAddCallback(noButton, XmNactivateCallback, NoCB, NoData);
    }
    /* XtAddCallback(noButton, XmNactivateCallback, gdNoCB, *shellReturn); */
    if (HelpCB == NULL)
    {
	XtSetSensitive(helpButton, False);
    }
    else
    {
	XtAddCallback(helpButton, XmNactivateCallback, HelpCB, HelpData);
    }
    if (MapCB)
    {
	XtAddCallback(*shellReturn, XmNmapCallback, MapCB, MapData);
    }
    return;
}


