/* $Header: /usr/local/src/nvrToMIT/RCS/MpegDisplay.c,v 1.20 1994/12/17 13:02:55 russ Exp $ */

/*
 * Header Files
 */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <Xm/Xm.h>
#include <Xm/XmP.h>
#include "MpegDisplayP.h"
#include "nvrAlloc.h"
#include "nvrString.h"
#include "mcs-server.h"
#include "mpegdecomp.h"

#ifndef MIN
# define MIN(x,y) ((x) > (y) ? (y) : (x))
#endif /* MIN */
#ifndef MAX
# define MAX(x,y) ((x) > (y) ? (x) : (y))
#endif /* MAX */

/*
 * Resource List
 */
static XtResource resources[] =
{
    {   XnvrNconfigCallback,
	XnvrCConfigCallback,
	XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrMpegDisplayWidget, mpeg.config_callback),
	XmRImmediate,
	(XtPointer) NULL
    },
    {
	XnvrNdecodeWindow,
	XnvrCDecodeWindow,
	XtRInt,
	sizeof(int),
	XtOffset(XnvrMpegDisplayWidget, mpeg.decodeWindow),
	XmRImmediate,
	(XtPointer)0
    },
    {
	XnvrNavailableDepths,
	XnvrCAvailableDepths,
	XtRInt,
	sizeof(int),
	XtOffset(XnvrMpegDisplayWidget, mpeg.availableDepths),
	XmRImmediate,
	(XtPointer)0
    },
    {
	XnvrNdepthType,
	XnvrCDepthType,
	XtRInt,
	sizeof(int),
	XtOffset(XnvrMpegDisplayWidget, mpeg.depthType),
	XmRImmediate,
	(XtPointer)GrayScale7
    },
    {
	XnvrNgamma,
	XnvrCGamma,
	XtRInt,
	sizeof(float),
	XtOffset(XnvrMpegDisplayWidget, mpeg.gamma),
	XmRImmediate,
	(XtPointer)XnvrDefaultGamma
    },
    {
	"debug",
	"Debug",
	XmRBool,
	sizeof(Bool),
	XtOffset(XnvrMpegDisplayWidget, mpeg.debug),
	XmRImmediate,
	(XtPointer)False
    },
    {
	XnvrNactive,
	XnvrCActive,
	XmRBool,
	sizeof(Bool),
	XtOffset(XnvrMpegDisplayWidget, mpeg.active),
	XmRImmediate,
	(XtPointer)False	/* should ALWAYS start out this way */
    },
    {
	XnvrNtrueColor12Bit,
	XnvrCTrueColor12Bit,
	XmRBool,
	sizeof(Bool),
	XtOffset(XnvrMpegDisplayWidget, mpeg.trueColor12Bit),
	XmRImmediate,
	(XtPointer)False
    },
    {
	XnvrNalignWindow,
	XnvrCAlignWindow,
	XmRBool,
	sizeof(Bool),
	XtOffset(XnvrMpegDisplayWidget, mpeg.alignWindow),
	XmRImmediate,
#ifdef sun
	(XtPointer)True
#else
	(XtPointer)False
#endif
    },
    {   XnvrNmcsServer,
	XnvrCMcsServer,
	XmRPointer,
        sizeof (XtPointer),
        XtOffset (XnvrMpegDisplayWidget, mpeg.mcs_server),
        XmRImmediate,
	(XtPointer) NULL
    },
};

/*
 * Action Function Declarations
 */
static void ChangeContrast();
static void ChangeBrightness();
static void ChangeGamma();
static void ChangeDepthType();


/*
 * Actions Table
 */
#ifdef ACTIONS
static XtActionsRec ActionsList[] =
{
	{"junk",	(XtActionProc)Junk}
};
#endif /* ACTIONS */


char *DepthTypeNames[] = { 
    "DepthUnspecified",
    "Rgb888",
    "Rgb444",
    "Rgb232",
    "Rgb332",
    "GrayScale8",
    "GrayScale7",
    "GrayScale1",
    "RgbC555",
    "RgbC666",
};

char *ColorClasses[] = { 
    "StaticGray",
    "GrayScale",
    "StaticColor",
    "PseudoColor",
    "TrueColor",
    "DirectColor" };
/*
 * Default Translation Table
 */
#ifdef DEFAULT_TRANSLATIIONS
static char DefaultTranslation[] =
	"<Btn1Down>:	Junk()";
#endif /* DEFAULT_TRANSLATIIONS */


/*
 * Method Function Declarations
 */

    /*
     * Core class functions
     *
     * following Core Class Functions not used
     *
     */

static void Initialize();
static void Realize();
static void Destroy();
static void Resize();
static void Exposure();
static Boolean SetValues();

    /*
     * VideoDisplay Class Functions
     *
     * following VideoDisplay Class Functions not used
     *	static void UpdateDisplay();
     *	static void RenderDisplay();
     *	static void PanDisplay();
     */

    /*
     * MpegDisplay class functions
     */
 
/*
 * Class Record Initialization
 */
XnvrMpegDisplayClassRec xnvrMpegDisplayClassRec = {
    {		/* Core class part */
	/* superclass		*/	(WidgetClass) &xnvrVideoDisplayClassRec,
	/* class_name		*/	"XnvrMpegDisplay",
	/* widget_size		*/	sizeof(XnvrMpegDisplayRec),
	/* class_initialize	*/	NULL,
	/* chained class init	*/	NULL,
	/* class_inited		*/	FALSE,
	/* initialize		*/	Initialize,
	/* initialize hook	*/	NULL,
	/* realize		*/	(XtRealizeProc)Realize,
	/* actions		*/	NULL, /* ActionsList, */
	/* num_actions		*/	0, /* XtNumber(ActionsList), */
	/* resources		*/	resources,
	/* num_resources	*/	XtNumber(resources),
	/* xrm_class		*/	NULLQUARK,
	/* compress_motion	*/	TRUE,
	/* compress_exposure*/		TRUE,
	/* compress enter/exit */	TRUE,
	/* visible_interest	*/	FALSE,
	/* destroy		*/	Destroy,
	/* resize		*/	Resize,
	/* expose		*/	Exposure,
	/* set_values		*/	(XtSetValuesFunc) SetValues,
	/* set values hook	*/	NULL,
	/* set values almost	*/	XtInheritSetValuesAlmost,
	/* get values hook	*/	NULL,
	/* accept_focus		*/	NULL,
	/* version		*/	XtVersion,
	/* callback offset 1st	*/	NULL,
	/* default trans	*/	NULL, /* DefaultTranslation, */
	/* query geo proc	*/	NULL,
					/*(XtGeometryHandler)QueryGeometry,*/
	/* display accelerator */	NULL,
	/* reserved		*/	NULL	/* extension record */
    },

    {		/* XnvrVideoDisplay class part */
	/* updateDisplay	*/	NULL,	/*XnvrInheritUpdateDisplay, */
	/* renderDisplay	*/	NULL,	/*(XtWidgetProc)RenderDisplay,*/
	/* panDisplay		*/	NULL,	/*XnvrInheritPanDisplay, */
	/* extension		*/	0,			
    },

    {		/* XnvrMpegDisplay class part */
	/* dummy field		*/	0,
    }
};


static Boolean CheckAcceleration(Display *dpy, XVisualInfo *pvi);

/*
 * Class Record Pointer Declaration
 */

WidgetClass xnvrMpegDisplayWidgetClass = (WidgetClass) &xnvrMpegDisplayClassRec;

static void
SetPerDisplay(XnvrMpegDisplayWidget w,
	      RgbDepthType type,
	      XVisualInfo *pvi,
	      int depth,
	      Boolean accelerated)
{
    MpegPerDepthPtr pd = &w->mpeg.perDepth[ type ];

    if (w->mpeg.debug)
	ProgMsg("pick: vis=0x%x, class=%s, depth=%d%s%s\n",
	    pvi->visual->visualid, ColorClasses[ pvi->visual->class ], depth,
	    accelerated ? ", accelerated" : "",
	    pd->view.visual ? " (override previous)" : "");

    pd->view.visual = pvi->visual;
    pd->depth = depth;
    pd->accelerated = accelerated;
}

/*
 * Method Function Definitions
 */

static void
Initialize(XnvrMpegDisplayWidget req,
	   XnvrMpegDisplayWidget new)
{
    XVisualInfo *pvi, visualInfoTmpl;
    int visCount, i;
    MpegPerDepthPtr pd;
    Screen *screen = XtScreen(new);
    Display *dpy = XtDisplay(new);
    int scrNumber = XScreenNumberOfScreen( screen );
    Boolean accelerated;

    /* by this time, the superclass' Initialize has already been called */

    new->mpeg.mcs_server = (XtPointer)McsOpenServer(McsMpegDecompClass);
    if (new->mpeg.mcs_server == NULL) {
        Msg("XnvrMpegDisplayWidget::Initialize:: ");
        Error("Cannot connect to MCS-MPEG server.\n");
        return;
    }

    visualInfoTmpl.screen = scrNumber;
    pvi = XGetVisualInfo(dpy, VisualScreenMask, &visualInfoTmpl, &visCount);

    pd = new->mpeg.perDepth;
    memset((char *)pd, 0, sizeof(new->mpeg.perDepth));
    for (i = 0; i < visCount; i++) {
	accelerated = CheckAcceleration(dpy, &pvi[ i ]);
	switch (pvi[i].depth) {
	case 1:
	    SetPerDisplay(new, GrayScale1, &pvi[ i ], 1, accelerated);
	    break;
	
	case 8:
	    switch (pvi[i].class) {
	    case PseudoColor:
		/*
		 * Take the first one.  (the second one on the
		 * SGI machines only yields 255 colors, instead of 256).
		 */
		if (pd[ Rgb232 ].view.visual
		 && (pd[ Rgb232 ].accelerated || !accelerated))
		    break;

		SetPerDisplay(new, RgbC555, &pvi[ i ], 8, accelerated);
		SetPerDisplay(new, RgbC666, &pvi[ i ], 8, accelerated);
		SetPerDisplay(new, Rgb232, &pvi[ i ], 8, accelerated);
		SetPerDisplay(new, Rgb332, &pvi[ i ], 8, accelerated);
		SetPerDisplay(new, GrayScale7, &pvi[ i ], 8, accelerated);
		SetPerDisplay(new, GrayScale8, &pvi[ i ], 8, accelerated);
		break;

	    case GrayScale:
		/*
		 * We prefer the pseudo color
		 */
		if (pd[ GrayScale7 ].view.visual
		 && (pd[ GrayScale7 ].accelerated || !accelerated))
		    break;
		SetPerDisplay(new, GrayScale7, &pvi[ i ], 8, accelerated);
		SetPerDisplay(new, GrayScale8, &pvi[ i ], 8, accelerated);
		break;

	    default:
		break;	/* nothing useful */
	    }
	    break;

	case 12:
	    switch (pvi[i].class) {
	    case PseudoColor:
		if (new->mpeg.trueColor12Bit)
		    break;
		if (pd[ Rgb444 ].view.visual) {
		    if (pd[ Rgb444 ].view.visual->visualid == DirectColor
		     && pd[ Rgb444 ].accelerated)
			break; /* we prefer DirectColor */
		    if (pd[ Rgb444 ].accelerated || ! accelerated)
			break; /* we prefer accelerated */
		}
		SetPerDisplay(new, Rgb444, &pvi[ i ], 12, accelerated);
		break;

	    case TrueColor:
		if (! new->mpeg.trueColor12Bit)
		    break;
		if (pd[ Rgb444 ].view.visual) {
		    if (pd[ Rgb444 ].view.visual->visualid == DirectColor
		     && pd[ Rgb444 ].accelerated)
			break; /* we prefer DirectColor */
		    if (pd[ Rgb444 ].view.visual->visualid == PseudoColor
		     && pd[ Rgb444 ].accelerated)
			break; /* we prefer PseudoColor */
		    if (pd[ Rgb444 ].accelerated || ! accelerated)
			break; /* we prefer accelerated */
		}
		/* fall through ... */

	    case DirectColor:
		if (pd[ Rgb444 ].view.visual) {
		    if (pd[ Rgb444 ].accelerated || ! accelerated)
			break; /* we prefer accelerated */
		}
		SetPerDisplay(new, Rgb444, &pvi[ i ], 12, accelerated);
		break;

	    default:
		break;	/* nothing useful */
	    }
	    break;

	case 32:
	case 24:
	    switch (pvi[i].class) {
	    case TrueColor:
		if (pd[ Rgb888 ].view.visual) {
		    if (pd[ Rgb888 ].view.visual->visualid == DirectColor
		     && pd[ Rgb888 ].accelerated)
			break; /* we prefer DirectColor */
		    if (pd[ Rgb888 ].accelerated || ! accelerated)
			break; /* we prefer accelerated */
		}
		/* fall through */

	    case DirectColor:
		if (pd[ Rgb888 ].view.visual) {
		    if (pd[ Rgb888 ].accelerated || ! accelerated)
			break; /* we prefer accelerated */
		}
		SetPerDisplay(new, Rgb888, &pvi[ i ],
			      pvi[i].depth, accelerated);
		break;

	    default:
		break;	/* nothing useful */
	    }
	    break;
	}
    }

    if (pvi)
	XtFree((char *)pvi);

    /*
     * Initialize the view data.
     */
    for (i = 0; i < XtNumber(new->mpeg.perDepth); i++) {
	if (pd[ i ].view.visual == NULL)
	    continue;
	pd[ i ].view.dpy       = dpy;
	pd[ i ].view.scrNumber = scrNumber;
	pd[ i ].view.depthType = i;
	pd[ i ].view.dpyGamma  = (double)new->mpeg.gamma/XnvrGammaDivisor;
	if (! InitViewCmapInfo(&pd[ i ].view)) {
	    /*
	     * For some reason we can't support this
	     */
	    pd[ i ].view.visual = NULL;
	    continue;
	}
    }

    /*
     * Decide what is available
     */
    new->mpeg.availableDepths =
	((pd[ Rgb888 ].view.visual != NULL) << Rgb888)
      | ((pd[ Rgb444 ].view.visual != NULL) << Rgb444)
      | ((pd[ Rgb232 ].view.visual != NULL) << Rgb232)
      | ((pd[ Rgb332 ].view.visual != NULL) << Rgb332)
      | ((pd[ Rgb332 ].view.visual != NULL) << RgbC555)
      | ((pd[ Rgb332 ].view.visual != NULL) << RgbC666)
      | ((pd[ GrayScale1 ].view.visual != NULL) << GrayScale1)
      | ((pd[ GrayScale7 ].view.visual != NULL) << GrayScale7)
      | ((pd[ GrayScale8 ].view.visual != NULL) << GrayScale8);

    new->video.visual = new->mpeg.perDepth[ new->mpeg.depthType ].view.visual;

    /*
     * Check instance values set by resources that may be invalid.
     */
    if (new->mpeg.gamma < XnvrDefaultGammaLow) {
	XtWarning("XnvrMpegDisplay: gamma value too low, \
		using XnvrDefaultGammaLow");
	new->mpeg.gamma = XnvrDefaultGammaLow;
    }
    if (new->mpeg.gamma > XnvrDefaultGammaHigh) {
	XtWarning("XnvrMpegDisplay: gamma value too high, \
		using XnvrDefaultGammaHigh");
	new->mpeg.gamma = XnvrDefaultGammaHigh;
    }

    /*
     * Set super-class resources
     */

    new->video.contrast.count = MAX_CONTRAST - MIN_CONTRAST;
    new->video.contrast.min = MIN_CONTRAST;
    new->video.contrast.normal = DEFAULT_CONTRAST;
    if (new->video.contrast.current == 0) {
	new->video.contrast.current = DEFAULT_CONTRAST;
    }
    new->video.bright.count = MAX_BRIGHTNESS - MIN_BRIGHTNESS;
    new->video.bright.min = MIN_BRIGHTNESS;
    new->video.bright.normal = DEFAULT_BRIGHTNESS;
    if (new->video.bright.current == 0) {
	new->video.bright.current = DEFAULT_BRIGHTNESS;
    }
}

static Boolean
SetValues (XnvrMpegDisplayWidget current,
	   XnvrMpegDisplayWidget req,
	   XnvrMpegDisplayWidget new)
{
    XnvrVideoDisplayPart *newVdpy = &new->video;
    XnvrVideoDisplayPart *curVdpy = &current->video;

    /*
     * Query-only resources -- don't allow them to be changed.
     */
    newVdpy->contrast.normal = curVdpy->contrast.normal;
    newVdpy->contrast.normal = curVdpy->contrast.normal;
    newVdpy->contrast.min = curVdpy->contrast.min;
    newVdpy->bright.normal = curVdpy->bright.normal;
    newVdpy->bright.min = curVdpy->bright.min;
    new->mpeg.availableDepths = current->mpeg.availableDepths;

    if (! new->mpeg.active && new->mpeg.decodeWindow)
	XUnmapWindow(XtDisplay(new), new->mpeg.decodeWindow);

    switch (new->mpeg.depthType) {
    default:
	new->mpeg.depthType = current->mpeg.depthType;
	break;

    case DepthUnspecified:
    case GrayScale1:
	newVdpy->bright.current = curVdpy->bright.current;
	newVdpy->contrast.current = curVdpy->contrast.current;
	new->mpeg.gamma = current->mpeg.gamma;
	break;
	
    case Rgb888:
    case Rgb444:
    case Rgb232:
    case Rgb332:
    case RgbC555:
    case RgbC666:
    case GrayScale8:
    case GrayScale7:
	/*
	 * Brightness
	 */
	if (newVdpy->bright.current != curVdpy->bright.current) {
	    ChangeBrightness(current, newVdpy->bright.current);
	}

	/*
	 * Contrast
	 */
	if (newVdpy->contrast.current != curVdpy->contrast.current) {
	    ChangeContrast(current, newVdpy->contrast.current);
	}

	/*
	 * Gamma
	 */
	new->mpeg.gamma = MAX(XnvrDefaultGammaLow, new->mpeg.gamma);
	new->mpeg.gamma = MIN(XnvrDefaultGammaHigh, new->mpeg.gamma);
	if (new->mpeg.gamma != current->mpeg.gamma) {
	    ChangeGamma(current, new->mpeg.gamma);
	}
	break;
    }
    ChangeDepthType(new);

    return(False);
}


static void
Destroy(XnvrMpegDisplayWidget w)
{
}


static void
Realize(XnvrMpegDisplayWidget w,
	XtValueMask* widgetValueMask,
	XSetWindowAttributes* attributes)
{
    /* by this time, the superclass' Realize has already been called */
    int i;
    Display *dpy = XtDisplay(w);
    int scrNumber = XScreenNumberOfScreen(XtScreen(w));
    MpegPerDepthPtr pd;
    XtValueMask decodeValueMask;
    XSetWindowAttributes decodeAttr;
    Window widgetWin;

    /*
     * Create the widget's window.
     */
    XtCreateWindow((Widget)w,
		   InputOutput,
		   (Visual *)NULL,
		   *widgetValueMask,
		   attributes);
    widgetWin = w->core.window;

    /*
     * Create a subwindow for each supportable depth.
     */
    pd = w->mpeg.perDepth;
    decodeValueMask = CWBorderPixel
		    | CWColormap
		    | CWBackPixmap
		    | CWBackingStore;
    decodeAttr.border_pixel = 0;
    decodeAttr.background_pixmap = None;
    decodeAttr.backing_store = NotUseful;

    for (i = 0; i < XtNumber(w->mpeg.perDepth); i++) {
	if ((w->mpeg.availableDepths & (1<<i)) == 0)
	    continue;
	decodeAttr.colormap =  pd[ i ].view.cmap;
	pd[i].win = XCreateWindow(dpy,
				  widgetWin,
				  0,0,		/* x, y */
				  (unsigned)w->core.width,
				  (unsigned)w->core.height,
				  0,		/* border width */
				  pd[ i ].depth,
				  InputOutput,
				  pd[ i ].view.visual,
				  decodeValueMask,
				  &decodeAttr);
    }

    ChangeDepthType(w);
    XtCallCallbackList((Widget)w, w->video.realizeCallback, NULL);
}

static void
ChangeWindowSize(XnvrMpegDisplayWidget w)
{
    XWindowChanges changes;
    UInt mask = CWWidth | CWHeight;

    if (w->mpeg.decodeWindow == 0)
	return;
    changes.width = w->core.width;
    changes.height = w->core.height;
    XConfigureWindow(XtDisplay((Widget)w), 
		     w->mpeg.decodeWindow,
		     mask,
		     &changes);
}

static void
Resize(XnvrMpegDisplayWidget w)
{
    ChangeWindowSize(w);
    XtCallCallbackList((Widget)w, w->mpeg.config_callback, NULL);
}

static void
Exposure(XnvrMpegDisplayWidget w,
		     XExposeEvent *event,
		     Region region)
{
    Display *dpy = XtDisplay(w);

    if (!XtIsRealized(w)) {
	return;
    }

#ifdef old
    if (region) {

	XClipBox(region, &rect);
	XSetWindowBackground(dpy,
		XtWindow(w), w->core.background_pixel);
	XClearArea(dpy, XtWindow(w),
		rect.x, rect.y, rect.width, rect.height, False);
	XSetWindowBackgroundPixmap(dpy, XtWindow(w), None);
    }
#endif

    /*
     * XXX try to align the application with the octword boundary to
     * get the whole picture...
     */
    if (w->mpeg.alignWindow) {
	static Window *parents;
	static int nParents = 0;
	static Screen *screen;
	Window root, child;
	int x, y;
	UInt width, height;
	UInt borderWidth;
	UInt depth;
	int moveToLeft;
	int success;
	XWindowAttributes attr;
	XSetWindowAttributes setAttr;
	
	if (parents == NULL) {
	    Window root, current, parent, *children;
	    unsigned int nchildren;

	    screen = XtScreen(w);
	    parent = XtWindow(w);
	    root = 0;
	    for (;;) {
		current = parent;
		XQueryTree(dpy, current,
			   &root, &parent, &children, &nchildren);
		XFree((void *)children);
		if (parent == root)
		    break;
		if (parent == 0) /* the output window *is* the root window */
		    break;
		nParents++;
		parents = (Window *)
		    NullRealloc((char *)parents, nParents * sizeof(Window));
		parents[nParents-1] = parent;
	    }
	    nParents = nParents;
	}

        success = XTranslateCoordinates(dpy, XtWindow(w),
					RootWindowOfScreen(screen),
					0, 0, &x, &y, &child);
	if (!success)
	    return;
	if ((moveToLeft = (x & 0x7)) == 0)
	    return;
	success = XGetGeometry(dpy, parents[ nParents-1 ],
	    &root, &x, &y, &width, &height, &borderWidth, &depth);
	if (!success)
	    return;
	if (width >= WidthOfScreen(screen))
	    return;	/* don't try it with virtual window managers. */

	/*
	 * remember the current state of the override_redirect
	 */
	success = XGetWindowAttributes(dpy, parents[ nParents-1 ], &attr);
	if (! success)
	    return;

	/*
	 * Make our XMoveWindow stick
	 */
	setAttr.override_redirect = True;
	XChangeWindowAttributes(dpy,
				parents[ nParents-1 ],
				CWOverrideRedirect,
				&setAttr);
	Msg("aligning window ");
	if (x - moveToLeft < 0) {
	    Msg("-%d...\n", 8-moveToLeft);
	    XMoveWindow(dpy, parents[ nParents-1 ], x + (8-moveToLeft), y);
	} else {
	    Msg("+%d...\n", moveToLeft);
	    XMoveWindow(dpy, parents[ nParents-1 ], x - moveToLeft, y);
	}

	/*
	 * Restore the state to the window
	 */
	if (! attr.override_redirect) {
	    setAttr.override_redirect = False;
	    XChangeWindowAttributes(dpy,
				    parents[ nParents-1 ],
				    CWOverrideRedirect,
				    &setAttr);
	}
    }

    if (XtIsRealized(w) && !w->core.being_destroyed)
    {
        XtCallCallbackList((Widget)w, w->video.exposeCallback, NULL);
    }
}


/*
 * Action Function Definitions
 */

static void
ChangeContrast(XnvrMpegDisplayWidget cw, int value)
{
    ViewAttributes viewAttr;
    MpegPerDepthPtr pd = &cw->mpeg.perDepth[ cw->mpeg.depthType ];

    cw->video.contrast.current = viewAttr.contrast = value;
    SetViewAttributes(&pd->view, &viewAttr, CONTRAST_MASK);
}

static void
ChangeBrightness(XnvrMpegDisplayWidget cw, int value)
{
    ViewAttributes viewAttr;
    MpegPerDepthPtr pd = &cw->mpeg.perDepth[ cw->mpeg.depthType ];

    cw->video.bright.current = viewAttr.brightness = value;
    SetViewAttributes(&pd->view, &viewAttr, BRIGHTNESS_MASK);
}

static void
ChangeGamma(XnvrMpegDisplayWidget cw, int value)
{
    ViewAttributes viewAttr;
    MpegPerDepthPtr pd = &cw->mpeg.perDepth[ cw->mpeg.depthType ];

    cw->mpeg.gamma = value;
    viewAttr.dpyGamma = (double)value/XnvrGammaDivisor;
    SetViewAttributes(&pd->view, &viewAttr, DISPLAY_GAMMA_MASK);
}

static void
ChangeDepthType(XnvrMpegDisplayWidget cw)
{
    ViewAttributes viewAttr;
    MpegPerDepthPtr pd = &cw->mpeg.perDepth[ cw->mpeg.depthType ];
    Display *dpy = XtDisplay(cw);

    if (cw->mpeg.decodeWindow)
	XUnmapWindow(dpy, cw->mpeg.decodeWindow);

    cw->mpeg.decodeWindow = pd->win;

    if (cw->mpeg.debug) {
	ProgMsg("switch to %s, ", DepthTypeNames[ cw->mpeg.depthType ]);
	if (pd->win)
	    Msg("win=0x%x", pd->win);
	else
	    Msg("no win");
	Msg(", %smapped\n", cw->mpeg.active ? "" : "not ");
    }

    if (pd->win) {
	ChangeWindowSize(cw);
	SetViewAttributes(&pd->view, &viewAttr, LOAD_COLORS_MASK);
	if (cw->mpeg.active) {
	    XMapWindow(dpy, pd->win);
	}
	/*
	 * Hack to stop background from being refreshed after this routine exits
	XSetWindowBackgroundPixmap(XtDisplay(cw), XtWindow(cw), None);
	 */
    }
}

#ifdef sgi
#define AccelerationDefined

#include <gl/glws.h>

/*
**  Used to get a value out of the configuration structure returned by
**  GLXgetconfig.  Scan through the GLX config structure, and, when we
**  find the proper buffer-mode combination, return the argument relevant
**  for that buffer type.
*/
static ULong
GLXgetvalue(GLXconfig* conf, int buffer, int mode)
{
    int i;
    for (i = 0; conf[i].buffer; i++)
        if (conf[i].buffer == buffer && conf[i].mode == mode)
            return conf[i].arg;
    return 0;
}

/*
 * Dorky little helper function used to build up a GLXconfig array.
 */
static void
SetGlEntry (GLXconfig* ptr, int b, int m, int a)
{
    ptr->buffer = b;
    ptr->mode = m;
    ptr->arg = a;
}

static Boolean
CheckGlxConfig(Display *dpy, XVisualInfo *pvi, GLXconfig *params)
{
    GLXconfig *conf;
    VisualID vis;

    conf = GLXgetconfig(dpy, pvi->screen, params);
    if (conf) {
        vis = GLXgetvalue(conf, GLX_NORMAL, GLX_VISUAL);
        free(conf);

        if (vis == pvi->visualid)
            return(True);
    }
    return(False);
}

static Boolean
CheckAcceleration(Display *dpy, XVisualInfo *pvi)
{
    GLXconfig params[50];
    GLXconfig* next;

    /*
     * This builds an array in "params" that describes for GLXgetconfig(3G)
     * the type of GL drawing that will be done.  Ask first for RGB double
     * buffered.
     */
    next = params;
    SetGlEntry(next++, GLX_NORMAL, GLX_RGB, True);
    SetGlEntry(next++, GLX_NORMAL, GLX_DOUBLE, True);
    SetGlEntry(next, 0, 0, 0); /* input to GLXgetconfig is null terminated */

    if (CheckGlxConfig(dpy, pvi, params))
	return(True);

    /*
     * Now ask first for RGB single buffered.
     */
    next = params;
    SetGlEntry(next++, GLX_NORMAL, GLX_RGB, True);
    SetGlEntry(next++, GLX_NORMAL, GLX_DOUBLE, False);
    SetGlEntry(next, 0, 0, 0);

    if (CheckGlxConfig(dpy, pvi, params))
	return(True);

    /*
     * Now ask first for index double buffered.
     */
    next = params;
    SetGlEntry(next++, GLX_NORMAL, GLX_RGB, False);
    SetGlEntry(next++, GLX_NORMAL, GLX_DOUBLE, True);
    SetGlEntry(next, 0, 0, 0); /* input to GLXgetconfig is null terminated */

    if (CheckGlxConfig(dpy, pvi, params))
	return(True);

    /*
     * Now ask first for index single buffered.
     */
    next = params;
    SetGlEntry(next++, GLX_NORMAL, GLX_RGB, False);
    SetGlEntry(next++, GLX_NORMAL, GLX_DOUBLE, False);
    SetGlEntry(next, 0, 0, 0);

    if (CheckGlxConfig(dpy, pvi, params))
	return(True);

    return(False);
}
#endif /* sgi */

#ifndef AccelerationDefined
static Boolean CheckAcceleration(Display *dpy, XVisualInfo *pvi)
{
    return(False);
}
#endif
