
/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	MpegViewer.c
 *
 * DESCRIPTION
 *	Implementation for the MpegViewer widget.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/MpegViewer.c,v 1.2 1994/12/17 13:16:37 russ Exp $
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <Xm/Xm.h>
#include <Xm/XmP.h>
#include "MpegViewerP.h"

extern void DefaultDuplexPipeRewriter(Argvec*, int, int);

#ifndef MIN
#define MIN(x,y) ((x) > (y) ? (y) : (x))
#endif MIN
#ifndef MAX
#define MAX(x,y) ((x) > (y) ? (x) : (y))
#endif MAX


/*
 *	Resource List
 */
static XtResource resources[] =
{
    {
	XnvrNplayState, XnvrCplayState, XtRInt, sizeof(int),
	XtOffset(XnvrMpegViewerWidget, mpegViewer.playState), XmRImmediate,
	(XtPointer)XnvrReadyState
    },
    {
	XnvrNmpegFile, XnvrCmpegFile, XmRString, sizeof(String),
	XtOffset(XnvrMpegViewerWidget, mpegViewer.mpegFile), XmRImmediate,
	(XtPointer)NULL
    },
    {
	XnvrNmpegStream, XnvrCmpegStream, XtRInt, sizeof(int),
	XtOffset(XnvrMpegViewerWidget, mpegViewer.mpegStream), XmRImmediate,
	(XtPointer)0
    },
    {
	XnvrNframeRate, XnvrCframeRate, XtRInt, sizeof(int),
	XtOffset(XnvrMpegViewerWidget, mpegViewer.frameRate), XmRImmediate,
	(XtPointer)10
    },
    {
	XnvrNcurrentFrame, XnvrCcurrentFrame, XtRInt, sizeof(int),
	XtOffset(XnvrMpegViewerWidget, mpegViewer.currentFrame), XmRImmediate,
	(XtPointer)0
    },
    {
	XnvrNfirstFrame, XnvrCfirstFrame, XtRInt, sizeof(int),
	XtOffset(XnvrMpegViewerWidget, mpegViewer.firstFrame), XmRImmediate,
	(XtPointer)0
    },
    {
	XnvrNlastFrame, XnvrClastFrame, XtRInt, sizeof(int),
	XtOffset(XnvrMpegViewerWidget, mpegViewer.lastFrame), XmRImmediate,
	(XtPointer)0
    },
    {
	XnvrNframeChangeCallback, XnvrCframeChangeCallback, XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrMpegViewerWidget, mpegViewer.frameChangedCallbacks),
	XmRImmediate, (XtPointer) NULL
    },
    {
	XnvrNfailureCallback, XnvrCfailureCallback, XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrMpegViewerWidget, mpegViewer.failureCallbacks),
	XmRImmediate, (XtPointer) NULL
    }
};


/*
 * Action Function Declarations
 */


/*
 * Actions Table
 */
#ifdef ACTIONS
static XtActionsRec ActionsList[] =
{
    {"junk",	(XtActionProc)Junk}
};
#endif /* ACTIONS */


/*
 * Default Translation Table
 */
#ifdef DEFAULT_TRANSLATIONS
static char DefaultTranslation[] =
	"<Btn1Down>:	Junk()";
#endif /* DEFAULT_TRANSLATIONS */


/*
 * Method Function Declarations
 */

    /*
     * Core class functions
     *
     * following Core Class Functions not used
     *
     */

static void Initialize();
static void Realize();
static void Destroy();
static void Resize();
static void Exposure();
static Boolean SetValues();
static XtGeometryResult QueryGeometry();

    /*
     * VideoDisplay Class Functions
     */

    /*
     * MpegDisplay class functions
     */

    /*
     * MpegViewer class functions
     */

    /*
     * MpegViewer support functions
     */
static void     VVSetAppData(XnvrMpegViewerWidget w, VVAppData *p_app_data);
static void	VVRunZip(XnvrMpegViewerWidget w);
static void	VVReallyUseX(XnvrMpegViewerWidget w, boolean useX);
static void	VVStart(XnvrMpegViewerWidget w);
static void	VVRestart(XnvrMpegViewerWidget w);
static void	VVPause(XnvrMpegViewerWidget w);
static void	VVResume(XnvrMpegViewerWidget w);
static void	VVSingleForward(XnvrMpegViewerWidget w);
static void     VVQuit(XnvrMpegViewerWidget w);

static void	MARunAudioDecode(XnvrMpegViewerWidget w, Boolean wait);
static void	MAGo(XnvrMpegViewerWidget w);
static void	MAStart(XnvrMpegViewerWidget w, Boolean wait);
static void	MARestart(XnvrMpegViewerWidget w, Boolean wait);
static void	MAPause(XnvrMpegViewerWidget w);
static void	MAResume(XnvrMpegViewerWidget w);
static void	MAHalt(XnvrMpegViewerWidget w);
static void     MAQuit(XnvrMpegViewerWidget w);

static void	StateManager(XnvrMpegViewerWidget w,
			PlayerState new_current_state, PlayerState req_target);
static void	MpegVideoStateUpdate(XnvrMpegViewerWidget w,
			MpegVideoState new_video_state);
static void	AudioStateUpdate(XnvrMpegViewerWidget w,
			AudioState new_audio_state);

static void	WaitForZip(XnvrMpegViewerWidget w);
static void	HandleVideoServerInput(XtPointer clientData, int *source,
					XtInputId *id);
static void	KillAudioDecodeAndWait(XnvrMpegViewerWidget w);
static void	HandleAudioServerInput(XtPointer clientData, int *source,
					XtInputId *id);
static int	CheckFilePath(XnvrMpegViewerWidget w, char *path_in);
static int	OpenFile(XnvrMpegViewerWidget w, char *filepath);

static void	MpegConfigVideoDisplay(XnvrMpegViewerWidget w);


/*
 * Class Record Initialization
 */
XnvrMpegViewerClassRec xnvrMpegViewerClassRec = {
    {		/* Core class part */
	/* superclass		*/	(WidgetClass) &xnvrMpegDisplayClassRec,
	/* class_name		*/	"XnvrMpegViewer",
	/* widget_size		*/	sizeof(XnvrMpegViewerRec),
	/* class_initialize	*/	NULL,
	/* chained class init	*/	NULL,
	/* class_inited		*/	FALSE,
	/* initialize		*/	Initialize,
	/* initialize hook	*/	NULL,
	/* realize		*/	(XtRealizeProc)Realize,
	/* actions		*/	NULL, /* ActionsList, */
	/* num_actions		*/	0, /* XtNumber(ActionsList), */
	/* resources		*/	resources,
	/* num_resources	*/	XtNumber(resources),
	/* xrm_class		*/	NULLQUARK,
	/* compress_motion	*/	TRUE,
	/* compress_exposure*/		TRUE,
	/* compress enter/exit */	TRUE,
	/* visible_interest	*/	FALSE,
	/* destroy		*/	Destroy,
	/* resize		*/	Resize,
	/* expose		*/	Exposure,
	/* set_values		*/	(XtSetValuesFunc) SetValues,
	/* set values hook	*/	NULL,
	/* set values almost	*/	XtInheritSetValuesAlmost,
	/* get values hook	*/	NULL,
	/* accept_focus		*/	NULL,
	/* version		*/	XtVersion,
	/* callback offset 1st	*/	NULL,
	/* default trans	*/	NULL, /* DefaultTranslation, */
	/* query geo proc	*/	NULL,
					/*(XtGeometryHandler)QueryGeometry,*/
	/* display accelerator */	NULL,
	/* reserved		*/	NULL	/* extension record */
    },

    {		/* XnvrVideoDisplay class part */
	/* updateDisplay	*/	NULL,	/*XnvrInheritUpdateDisplay, */
	/* renderDisplay	*/	NULL,	/*(XtWidgetProc)RenderDisplay,*/
	/* panDisplay		*/	NULL,	/*XnvrInheritPanDisplay, */
	/* extension		*/	0,			
    },

    {		/* XnvrMpegDisplay class part */
	/* dummy field		*/	0,
    },

    {		/* XnvrMpegViewer class part */
	/* dummy field		*/	0,
    }
};


/*
 * Class Record Pointer Declaration
 */

WidgetClass xnvrMpegViewerWidgetClass = (WidgetClass) &xnvrMpegViewerClassRec;

/*
 * Method Function Definitions
 */

static void
Initialize(req, new)
    XnvrMpegViewerWidget req;
    XnvrMpegViewerWidget new;
{
    int fileType;

    /* by this time, the superclass' Initialize has already been called */

#ifdef debug
    printf("XnvrMpegViewerWidget::Initialize\n");
#endif

    /*
     * Check instance values set by resources that may be invalid.
     */
	/* make sure size isn't smaller than minimum */
    new->core.width = MAX(new->core.width, MIN_VIDEODISPLAY_WIDTH);
    new->core.height = MAX(new->core.height, MIN_VIDEODISPLAY_HEIGHT);

    /*
     * Set super-class resources
     */

    /*
     * Set instance variable defaults 
     */
    new->mpegViewer.playerState = X_NoFile;
    new->mpegViewer.targetState = X_NoFile;
    new->mpegViewer.videoState = VideoCompleted;
    new->mpegViewer.audioState = AudioCompleted;
    new->mpegViewer.MpegAudioDelay = 0;
    new->mpegViewer.MA_Quality = DEFAULT_QUALITY;
    new->mpegViewer.AudioDecodeLaunching = False;
    new->mpegViewer.ZipLaunching = False;
    new->mpegViewer.fileState = File_None;

    new->mpegViewer.Zip.rscData.mpegBig = False;

    /*
     *  Init internal state as warranted by the resource values
     */
    if (new->mpegViewer.mpegStream > 0)
    {
	if (new->mpegViewer.mpegFile != NULL)
	{
	    printf("XnvrMpegViewerWidget::Initialize:: both a file and stream specified...\n	file will be ignored\n");
	}
    }
    else if (new->mpegViewer.mpegFile != NULL)
    {
	/* This forces a state change requiring window info... window
	 * not yet open.
	 * fileType = OpenFile(new, new->mpegViewer.mpegFile);
	 */
	fileType = CheckFilePath(new, new->mpegViewer.mpegFile);
	if (fileType < 0)
	{
	    printf("XnvrMpegViewerWidget::Initialize:: invalid mpeg file.\n");
	    printf("     CheckFilePath on %s returns %d\n",
		new->mpegViewer.mpegFile,
		fileType);
	    /* call error callbacks ??? */
	}
    }
    else
    {
	printf("XnvrMpegViewerWidget::Initialize:: no mpeg file specified.\n");
    }
}

static Boolean
SetValues (current, req, new)
    XnvrMpegViewerWidget current;
    XnvrMpegViewerWidget req;
    XnvrMpegViewerWidget new;
{
#ifdef debug
    printf("XnvrMpegViewerWidget::SetValues\n");
#endif


    /*
     * Query-only resources -- don't allow them to be changed.
     *
     * ??? should be moved to super-class ???
     */

    return(False);
}


static void
Destroy(w)
    XnvrMpegViewerWidget w;
{
#ifdef debug
    printf("XnvrMpegViewerWidget::Destroy\n");
#endif
}


static void
Realize(w, value_mask, attributes)
    XnvrMpegViewerWidget w;
    XtValueMask* value_mask;
    XSetWindowAttributes* attributes;
{
    /* by this time, the superclass' Realize has already been called */

#ifdef debug
    printf("XnvrMpegViewerWidget::Realize\n");
#endif
/*
    dpy = XtDisplay(w);
    screen = XScreenNumberOfScreen(XtScreen(w));
*/
}


static void Resize(w)
	XnvrMpegViewerWidget w;
{
#ifdef debug
	printf("XnvrMpegViewerWidget::Resize\n");
#endif
}


static void Exposure(w, event, region)
    XnvrMpegViewerWidget w;
    XExposeEvent *event;
    Region region;
{
#ifdef debug
    printf("XnvrMpegViewerWidget::Exposure\n");
#endif

/*
    if (!XtIsRealized(w)) {
	return;
    }

    if (region) {
    }
*/
}


/*
 * Action Function Definitions
 */


/*
 * Support Functions
 */



void
StateManager(XnvrMpegViewerWidget w,
		PlayerState new_current_state, PlayerState req_target) 
{
    static PlayerState saved_target;
    static PlayerState saved_state;

    if (req_target ==  X_Selecting ||
	w->mpegViewer.playerState ==  X_Selecting) {
	printf(
	"XnvrMpegViewerWidget::StateManager:: X_Selecting invalid state\n");
    }

    if (new_current_state != X_None) {
	w->mpegViewer.playerState = new_current_state;
    } else {
	new_current_state = w->mpegViewer.playerState;
    }


    if (req_target != X_None) {
	w->mpegViewer.targetState = req_target;
    }

    if (new_current_state == w->mpegViewer.targetState) {
	return;
    }

    switch (new_current_state) {
      case X_NoFile :
	break;
      case AV_NewFile :
	switch (w->mpegViewer.targetState) {
	  case AV_Paused :
	  case AV_Running :
	    if (w->mpegViewer.videoState == VideoLoading) {
		/* ignore ... waiting for loaded state */
		return;
	    }
	    /*
	     * Start vmpegdecomp and AudioDecode
	     */
	    VVStart(w);
	    MAStart(w, True);
	    break;
	  case AV_Completed :
	    /* Ignore */
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected state change\n");
	    return;
	}
	break;
      case AV_ALoaded :
	switch (w->mpegViewer.targetState) {
	  case AV_Completed :
	    w->mpegViewer.targetState = AV_Paused;
	  case AV_Running :
	    if (w->mpegViewer.videoState == VideoCompleted) {
		/* need to get it reloaded */
		VVRestart(w);
	    }
	  case AV_Paused :
	    /* Ignore */
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case AV_VLoaded :
	switch (w->mpegViewer.targetState) {
	  case AV_Completed :
	    w->mpegViewer.targetState = AV_Paused;
	  case AV_Running :
	  case AV_Paused :
	    /* Ignore */
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case AV_Loaded :
	switch (w->mpegViewer.targetState) {
	  case AV_Completed :
	    w->mpegViewer.targetState = AV_Loaded;
	    break;
	  case AV_Paused :
	    VVSingleForward(w);
	    break;
	  case AV_Running :
	    VVResume(w);
	    MAGo(w);
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case AV_ARunning :
	switch (w->mpegViewer.targetState) {
	  case AV_Paused :
	    MAPause(w);
	  case AV_Running :
	  case AV_Completed :
	    /* Ignore */
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case AV_VRunning :
	switch (w->mpegViewer.targetState) {
	  case AV_Loaded :
	    /* ??? Ignore initial single forward */
	  case AV_Running :
	  case AV_Paused :
	  case AV_Completed :
	    /* Ignore */
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case AV_Running :
	switch (w->mpegViewer.targetState) {
	  case AV_Running :
	    /* Achieved */
	    break;
	  case AV_Paused :
	    VVPause(w);
	    MAPause(w);
	    break;
	  case AV_Completed :
	    VVPause(w);
	    MAPause(w);
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case AV_APaused :
	switch (w->mpegViewer.targetState) {
	  case AV_Running :
	  case AV_Paused :
	  case AV_Completed :
	    /* Ignore */
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case AV_VPaused :
	switch (w->mpegViewer.targetState) {
	  case AV_Paused :
	  case AV_Running :
	  case AV_Completed :
	    /* Ignore */
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case AV_Paused :
	switch (w->mpegViewer.targetState) {
	  case AV_Running :
	    VVResume(w);
	    if (w->mpegViewer.audioState == AudioLoaded) {
		MAGo(w);
	    } else {
		MAResume(w);
	    }
	    break;
	  case AV_Paused :
	    /* Achieved */
	    break;
	  case AV_Completed :
	    w->mpegViewer.playerState = AV_Completed;
	    break;
	  case AV_Loaded : /* ??? */
	    w->mpegViewer.playerState = AV_Loaded;
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case AV_ACompleted :
	switch (w->mpegViewer.targetState) {
	  case AV_Running :
	    /* ignore */
	    break;
	  case AV_ACompleted :
	    /* ignore */
	    break;
	  case AV_Paused :
	  case AV_xVPaused :
	  case AV_Completed :
	    VVPause(w);
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case AV_xVPaused :
	switch (w->mpegViewer.targetState) {
	  case AV_Completed :
	    w->mpegViewer.playerState = AV_Completed;
	    break;
	  case AV_Running :
	    w->mpegViewer.targetState = AV_ACompleted;
	  case AV_ACompleted :
	    VVResume(w);
	    break;
	  case AV_Paused :
	    w->mpegViewer.targetState = AV_xVPaused;
	  case AV_xVPaused :
	    /* Achieved */
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case AV_VCompleted :
	switch (w->mpegViewer.targetState) {
	  case AV_Running :
	    /* ignore */
	    break;
	  case AV_Paused :
	    w->mpegViewer.targetState = AV_AxPaused;
	  case AV_AxPaused :
	    MAPause(w);
	    break;
	  case AV_Completed :
	    MAHalt(w);
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case AV_AxPaused :
	switch (w->mpegViewer.targetState) {
	  case AV_Completed :
	    MAHalt(w);
	    break;
	  case AV_Running :
	    w->mpegViewer.targetState = AV_VCompleted;
	  case AV_VCompleted :
	    MAResume(w);
	    break;
	  case AV_Paused :
	    w->mpegViewer.targetState = AV_AxPaused;
	  case AV_AxPaused :
	    /* Achieved */
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case AV_Completed :
	switch (w->mpegViewer.targetState) {
	  case AV_Running :
	    VVRestart(w);
	    MARestart(w, True);
	    break;
	  case AV_Paused :
	    w->mpegViewer.targetState = AV_Completed;
	  case AV_Completed :
	    /* Achieved */
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case A_NewFile :
	switch (w->mpegViewer.targetState) {
	  case A_Running :
	    /*
	     * Start AudioDecode
	     */
	    MAStart(w, True);
	    break;
	  case A_Paused :
	  case A_Completed :
	    /* Ignore */
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected state change\n");
	    return;
	}
	break;
      case A_Loaded :
	switch (w->mpegViewer.targetState) {
	  case A_Running :
	  case A_Paused :
	  case A_Completed :
	    MAGo(w);
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case A_Running :
	switch (w->mpegViewer.targetState) {
	  case A_Running :
	    /* Achieved */
	    break;
	  case A_Paused :
	    MAPause(w);
	    break;
	  case A_Completed :
	    MAPause(w);
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case A_Paused :
	switch (w->mpegViewer.targetState) {
	  case A_Running :
	    MAResume(w);
	    break;
	  case A_Paused :
	    /* Achieved */
	    break;
	  case A_Completed :
	    w->mpegViewer.playerState = A_Completed;
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case A_Completed :
	switch (w->mpegViewer.targetState) {
	  case A_Running :
	    MARestart(w, True);
	    break;
	  case A_Paused :
	    /* Ignore */
	    break;
	  case A_Completed :
	    /* Achieved */
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case V_NewFile :
	switch (w->mpegViewer.targetState) {
	  case V_Completed :
	    /* Ignore */
	    break;
	  case V_Paused :
	  case V_Running :
	    if (w->mpegViewer.videoState == VideoLoading) {
		/* ignore ... waiting for loaded state */
		return;
	    }
	    VVStart(w);
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case V_Loaded :
	switch (w->mpegViewer.targetState) {
	  case V_Paused :
	    VVSingleForward(w);
	    break;
	  case V_Running :
	  case V_Completed :
	    VVResume(w);
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case V_Running :
	switch (w->mpegViewer.targetState) {
	  case V_Running :
	    /* Achieved */
	    break;
	  case V_Paused :
	    VVPause(w);
	    break;
	  case V_Completed :
	    VVPause(w);
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case V_Paused :
	switch (w->mpegViewer.targetState) {
	  case V_Running :
	    VVResume(w);
	    break;
	  case V_Paused :
	    /* Achieved */
	    break;
	  case V_Completed :
	    w->mpegViewer.playerState = V_Completed;
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
      case V_Completed :
	switch (w->mpegViewer.targetState) {
	  case V_Running :
	    VVRestart(w);
	    break;
	  case V_Completed :
	    /* Achieved */
	  case V_Paused :
	    /* Ignore */
	    break;
	  default :
	    INTERNAL_ERROR("StateManager() unexpected target state");
	    return;
	}
	break;
    }
}



static void
MpegVideoStateUpdate(XnvrMpegViewerWidget w, MpegVideoState new_video_state)
{
    static Widget stateNotify;
    static MpegVideoState old_video_state = VideoCompleted;
    PlayerState new_player_state;

    w->mpegViewer.ZipLaunching = False;

    switch(new_video_state) {
      case VideoLoading :
#ifdef PROTOCOL_TRACE
	fprintf(stderr, "XnvrMpegViewerWidget:: received l from vmpegdecomp\n");
#endif /* PROTOCOL_TRACE */
	switch (w->mpegViewer.playerState) {
	  case AV_NewFile :
	  case AV_ALoaded :
	  case AV_Completed :
	  case V_NewFile :
	  case V_Completed :
	    /* !!! Post Message Dialog */
	    new_player_state = X_None;
	    if (!stateNotify) {
/*
		stateNotify = XtVaCreateManagedWidget(
			"  Loading video data...  ",
			xmLabelWidgetClass, XtParent(videoDisplay),
			XmNx,		50,
			XmNy,		50,
			NULL);
*/
		printf("  Loading video data...\n");
	    }
	    XtManageChild(stateNotify);
	    break;
	  default :
	    INTERNAL_ERROR("MpegVideoStateUpdate() unexpected Video state");
	    return;
	}
	break;
      case VideoPaused :
#ifdef PROTOCOL_TRACE
	fprintf(stderr, "XnvrMpegViewerWidget:: received s from vmpegdecomp\n");
#endif /* PROTOCOL_TRACE */
	if (old_video_state == VideoLoading) {
	    /* Video Decompressor now in VideoLoaded State */
	    switch (w->mpegViewer.playerState) {
	      case AV_NewFile :
		new_player_state = AV_VLoaded;
		break;
	      case AV_ALoaded :
	      case AV_VRunning :
		new_player_state = AV_Loaded;
		break;
	      case V_NewFile :
	      case V_Completed :
		new_player_state = V_Loaded;
		break;
	      default :
		INTERNAL_ERROR("MpegVideoStateUpdate() unexpected Video state");
		return;
	    }
	} else {
	    /* Video Decompressor was in Running State */
	    switch (w->mpegViewer.playerState) {
	      case AV_Loaded :
	      case AV_Running :
		new_player_state = AV_VPaused;
		break;
	      case AV_ALoaded :
	      case AV_APaused :
	      case AV_VRunning :
		new_player_state = AV_Paused;
		break;
	      case AV_ACompleted :
		new_player_state = AV_xVPaused;
		break;
	      case V_Loaded :
	      case V_Running :
		new_player_state = V_Paused;
		break;
	      case AV_AxPaused :
		/*
		 * Ok, if we receive s from vmpegdecomp at when playerState
		 * is AV_AxPaused, it tells me that vmpegdecomp has completed
		 * and paused('s').  Just ignore it!
		 */
	      case AV_Completed :
	      case V_Completed :
		/* ignore */
		return;
	      default :
		INTERNAL_ERROR("MpegVideoStateUpdate() unexpected Video state");
		return;
	    }
	}
	break;
      case VideoRunning :
#ifdef PROTOCOL_TRACE
	fprintf(stderr, "XnvrMpegViewerWidget:: received r from vmpegdecomp\n");
#endif /* PROTOCOL_TRACE */
	switch (w->mpegViewer.playerState) {
	  case AV_Loaded :
	    new_player_state = AV_VRunning;
	    break;
	  case AV_ARunning :
	    new_player_state = AV_Running;
	    break;
	  case AV_VPaused :
	    new_player_state = AV_Running;
	    break;
	  case AV_Paused :
	    new_player_state = AV_APaused;
	    break;
	  case AV_xVPaused :
	    new_player_state = AV_ACompleted;
	    break;
	  case V_Loaded :
	  case V_Paused :
	    new_player_state = V_Running;
	    break;
	  default :
	    INTERNAL_ERROR("MpegVideoStateUpdate() unexpected Video state");
	    return;
	}
	XtUnmanageChild(stateNotify);
	break;
      case VideoCompleted :
#ifdef PROTOCOL_TRACE
	fprintf(stderr, "XnvrMpegViewerWidget:: received c from vmpegdecomp\n");
#endif /* PROTOCOL_TRACE */
	switch (w->mpegViewer.playerState) {
	  case AV_Running :
	    new_player_state = AV_VCompleted;
	    break;
	  case AV_APaused :
	    new_player_state = AV_AxPaused;
	    break;
	  case AV_Paused :
	    new_player_state = AV_VCompleted;
	    break;
	  case AV_ACompleted :
	  case AV_xVPaused :
	    w->mpegViewer.targetState = new_player_state = AV_Completed;
	    break;
	  case V_Running :
	  case V_Paused :
	    w->mpegViewer.targetState = new_player_state = V_Completed;
	    break;
	  default :
	    INTERNAL_ERROR("MpegVideoStateUpdate() unexpected Video state");
	    return;
	}
	/* adequate ??? */
	w->videoDisplay.isActive = True;
	break;
      default:
	INTERNAL_ERROR("MpegVideoStateUpdate() invalid new state");
	return;
    }

    w->mpegViewer.videoState = old_video_state = new_video_state;

    if (new_player_state != X_None) {
	StateManager(w, new_player_state, X_None);
    }
}


/* ARGSUSED */
void
AudioStateUpdate(XnvrMpegViewerWidget w, AudioState new_audio_state)
{
    PlayerState new_player_state;

    w->mpegViewer.AudioDecodeLaunching = False;

    w->mpegViewer.audioState = new_audio_state;

    switch(new_audio_state) {
      case AudioLoading :
	/* Ignore */
	break;
      case AudioLoaded :
#ifdef PROTOCOL_TRACE
	fprintf(stderr, "XnvrMpegViewerWidget:: received l from AudioDecode\n");
#endif /* PROTOCOL_TRACE */
	switch (w->mpegViewer.playerState) {
	  case AV_NewFile :
	  case AV_Completed :
	    new_player_state = AV_ALoaded;
	    break;
	  case AV_VLoaded :
	    new_player_state = AV_Loaded;
	    break;
	  case A_NewFile :
	  case A_Completed :
	    new_player_state = A_Loaded;
	    break;
	  default :
	    INTERNAL_ERROR("AudioStateUpdate() unexpected audio state");
	    return;
	}
	break;
      case AudioRunning :
#ifdef PROTOCOL_TRACE
	fprintf(stderr, "XnvrMpegViewerWidget:: received r from AudioDecode\n");
#endif /* PROTOCOL_TRACE */
	switch (w->mpegViewer.playerState) {
	  case AV_Loaded :
	    new_player_state = AV_ARunning;
	    break;
	  case AV_VRunning :
	    new_player_state = AV_Running;
	    break;
	  case AV_APaused :
	    new_player_state = AV_Running;
	    break;
	  case AV_Paused :
	    new_player_state = AV_VPaused;
	    break;
	  case AV_AxPaused :
	    new_player_state = AV_VCompleted;
	    break;
	  case A_Loaded :
	  case A_Paused :
	    new_player_state = A_Running;
	    break;
	  case AV_Completed :
	  case A_Completed :
	  case AV_NewFile :
	  case A_NewFile :
	  case V_NewFile :
	    /*
	     * Hack attack ... some routine is just trying to reset the 
	     * audio device to its playing state.
	     * So ... just ignore !!
	     */
	    return;
	  default :
	    INTERNAL_ERROR("AudioStateUpdate() unexpected audio state");
	    return;
	}
	break;
      case AudioPaused :
#ifdef PROTOCOL_TRACE
	fprintf(stderr, "XnvrMpegViewerWidget:: received s from AudioDecode\n");
#endif /* PROTOCOL_TRACE */
	switch (w->mpegViewer.playerState) {
	  case AV_Running :
	    new_player_state = AV_APaused;
	    break;
	  case AV_VPaused :
	    new_player_state = AV_Paused;
	    break;
	  case AV_VCompleted :
	    new_player_state = AV_AxPaused;
	    break;
	  case A_Running :
	    new_player_state = A_Paused;
	    break;
	  default :
	    INTERNAL_ERROR("AudioStateUpdate() unexpected audio state");
	    return;
	}
	break;
      case AudioCompleted :
#ifdef PROTOCOL_TRACE
	fprintf(stderr, "XnvrMpegViewerWidget:: received c from AudioDecode\n");
#endif /* PROTOCOL_TRACE */
	switch (w->mpegViewer.playerState) {
	  case AV_NewFile :
	  case A_NewFile :
	    /* ignore */
	    return;
	  case AV_Running :
	    new_player_state = AV_ACompleted;
	    break;
	  case AV_Paused :
	    new_player_state = AV_ACompleted;
	    break;
	  case AV_VCompleted :
	  case AV_AxPaused :
	    w->mpegViewer.targetState = new_player_state = AV_Completed;
	    break;
	  case AV_VPaused :
	    new_player_state = AV_xVPaused;
	    break;
	  case A_Running :
	  case A_Paused :
	    w->mpegViewer.targetState = new_player_state = A_Completed;
	    break;
	  default :
	    INTERNAL_ERROR("AudioStateUpdate() unexpected audio state");
	    return;
	}
	break;
      default:
	INTERNAL_ERROR("AudioStateUpdate() invalid new state");
	return;
    }

    StateManager(w, new_player_state, X_None);
}


static int
CheckFilePath(XnvrMpegViewerWidget w, char *path_in)
{
    /*
     *	Return values:
     *		0 - file type unknown
     *		1 - mpeg audio
     *		2 - mpeg video
     *		3 - mpeg audio+video (i.e. matching file also found)
     *
     *		-1 - cannot open file
     *		-2 - file spec undefined
     */
    char	buf[MAX_FILENAME_LENGTH];
    char	*ext;
    char	*p_tmp;
    char	*p_videodir;
    struct stat	stat_struct;
    ULong	startoffile;
    int		fd;
    int		bytesread;
    int		mpeg_file_type = 0; /* 0=unknown, 1=audio, 2=video */

    if (*path_in == '\0') {
	return(-2);
    }

    fd = open(path_in, O_RDONLY);
    if (fd < 0) {
	*path_in == '\0';
	printf("XnvrMpegViewerWidget::CheckFilePath::\n	Cannot open selected file.");
	return(-1);
    }
    bytesread = read(fd, (char*)&startoffile, 4);
    close(fd);
    if ((startoffile & 0xfff00000) == 0xfff00000) {
	/* file contains an MPEG audio stream */
	mpeg_file_type = 1;
	strncpy(w->mpegViewer.AFilePath, path_in, MAX_PATH_LEN);
    } else if (startoffile == 0x000001b3) {
	/* file contains an MPEG video stream */
	mpeg_file_type = 2;
	strncpy(w->mpegViewer.VFilePath, path_in, MAX_PATH_LEN);
    } else {
	printf("XnvrMpegViewerWidget::CheckFilePath::\n	The selected file is not an MPEG file.");
	return(-1);
    }

    strncpy(buf, path_in, MAX_FILENAME_LENGTH-1);
    ext = strrchr(buf, '.');
    if (ext == NULL) {
	/* file does not have ".vmpeg" or ".ampeg" extension */
	return(mpeg_file_type);
    }

    if (mpeg_file_type == 1) {
	/* File specified is a MPEG audio file.  Find its video counterpart. */
        if (strcmp(ext, ".ampeg") == 0) {
	    strcpy(ext, ".vmpeg");
	    if (stat(buf, &stat_struct) != -1) {
		strncpy(w->mpegViewer.VFilePath, buf, MAX_PATH_LEN);
		return(3);
	    }
	    strcpy(ext, ".mpeg");
	    if (stat(buf, &stat_struct) != -1) {
		strncpy(w->mpegViewer.VFilePath, buf, MAX_PATH_LEN);
		return(3);
	    }
	}
	/* no counterpart found */
	strcpy(w->mpegViewer.VFilePath, "");
	return(1);
    }

    if (mpeg_file_type == 2) {
	/* File specified is a MPEG video file.  Find its audio counterpart. */
	if (strcmp(ext, ".vmpeg") == 0) {
	    strcpy(ext, ".ampeg");
	    if (stat(buf, &stat_struct) != -1) {
		strncpy(w->mpegViewer.AFilePath, buf, MAX_PATH_LEN);
		return(3);
	    }
	} else if (strcmp(ext, ".mpeg") == 0) {
	    strcpy(ext, ".ampeg");
	    if (stat(buf, &stat_struct) != -1) {
		strncpy(w->mpegViewer.AFilePath, buf, MAX_PATH_LEN);
		return(3);
	    }
	}
	/* no counterpart expected */
	strcpy(w->mpegViewer.AFilePath, "");
	return(2);
    }

    return(-1);
}


FileState
SetFileState(XnvrMpegViewerWidget w, FileState new_file_state)
{
    FileState old_file_state = w->mpegViewer.fileState;

    w->mpegViewer.fileState = new_file_state;
    return(old_file_state);
}


static int 
OpenFile(XnvrMpegViewerWidget w, char* filepath)
{
    PlayerState *p_saved_state = &(w->mpegViewer.playerState);	/* ??? */
    PlayerState new_current_state;
    PlayerState target_state;
    int file_state = 0;
    int width = 0;
    int height = 0;

    file_state = CheckFilePath(w, filepath);

    switch (file_state) {
      case 3 :
	new_current_state = AV_NewFile;
	target_state = AV_Paused;
	break;
      case 2 :
	new_current_state = V_NewFile;
	target_state = V_Paused;
	break;
      case 1 :
	new_current_state = A_NewFile;
	target_state = A_Paused;
	break;
      case -2 :
	new_current_state = target_state = X_NoFile;
	break;
      default :
	printf("XnvrMpegViewerWidget::OpenFile:: unexpected target state\n");
	/* continue or bomb out??? */
      case -1 :
	StateManager(w, X_None, *p_saved_state);
	return(file_state);
    }

    if (*(w->mpegViewer.VFilePath) != '\0')
    {
	FileInfo file_info;
	GetFileInfo(w->mpegViewer.VFilePath, &file_info);
	w->core.width = file_info.width;
	w->core.height = file_info.height;
    }

    /* MASetFile(w->mpegViewer.AFilePath); */
    if (filepath != NULL)
    {
	strcpy(w->mpegViewer.AudioDecode.file, filepath);
    }
    /* VVSetFile(w->mpegViewer.VFilePath, w->core.width, w->core.height); */
    if (filepath != NULL)
    {
	strcpy(w->mpegViewer.Zip.args.file, filepath);
	w->mpegViewer.Zip.args.width = w->core.width;
	w->mpegViewer.Zip.args.height = w->core.height;
	if (*filepath != '\0')
	{
	    MpegConfigVideoDisplay(w);
	}
    }

    if (*(w->mpegViewer.AFilePath) == '\0' && *(w->mpegViewer.VFilePath) == '\0') {
	SetFileState(w, File_None);
    } else if (*(w->mpegViewer.AFilePath) == '\0') {
	SetFileState(w, File_VideoOnly);
    } else if (*(w->mpegViewer.VFilePath) == '\0') {
	SetFileState(w, File_AudioOnly);
    } else {
	SetFileState(w, File_AudioVideo);
    }

    /*
     * We changed files ..
     *   kill current audio/video processes
     */
    StateManager(w, new_current_state, target_state);

    return(file_state);
}



void
MpegConfigVideoDisplay(XnvrMpegViewerWidget w)
{
    /* how should resize really be done ??? */
    if (w->mpegViewer.Zip.rscData.mpegBig) {
	/* 4x mode */
	w->core.width = w->mpegViewer.Zip.args.width * 2;
	w->core.height = w->mpegViewer.Zip.args.height * 2;
    } else {
	/* 1x mode */
	w->core.width = w->mpegViewer.Zip.args.width;
	w->core.height = w->mpegViewer.Zip.args.height;
    }
}


/*************************************
 * VV routines
 *************************************/

static void
VVStart(XnvrMpegViewerWidget w)
{
    if (!w->mpegViewer.ZipLaunching) {
	w->mpegViewer.ZipLaunching = True;
	VVRunZip(w);
    }
}

static void
VVRestart(XnvrMpegViewerWidget w)
{
    if (! w->mpegViewer.Zip.decoder.pid) {
	return;
    }

#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: sent r to vmpegdecomp\n");
#endif /* PROTOCOL_TRACE */

/*
    XtVaSetValues(w->mpegViewer.Zip.videoDisplay,
	XnvrNisActive, True,
	NULL);
*/
    /* probably not sufficient to kick the widget active */
    w->videoDisplay.isActive = True;
    write(w->mpegViewer.Zip.decoder.outFd, "r", 1);
}

static void
VVPause(XnvrMpegViewerWidget w)
{
    if (! w->mpegViewer.Zip.decoder.pid)
	return;
#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: sent P to vmpegdecomp\n");
#endif /* PROTOCOL_TRACE */

/*
    XtVaSetValues(w->mpegViewer.Zip.videoDisplay,
	XnvrNisActive, True,
	NULL);
*/
    /* probably not sufficient to kick the widget active */
    w->videoDisplay.isActive = True;
    write(w->mpegViewer.Zip.decoder.outFd, "P", 1);
}

static void
VVResume(XnvrMpegViewerWidget w)
{
    if (! w->mpegViewer.Zip.decoder.pid)
	return;
#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: sent p to vmpegdecomp\n");
#endif /* PROTOCOL_TRACE */

/*
    XtVaSetValues(w->mpegViewer.Zip.videoDisplay,
	XnvrNisActive, True,
	NULL);
*/
    /* probably not sufficient to kick the widget active */
    w->videoDisplay.isActive = True;
    write(w->mpegViewer.Zip.decoder.outFd, "p", 1);
}

static void
VVSingleForward(XnvrMpegViewerWidget w)
{
    if (! w->mpegViewer.Zip.decoder.pid)
	return;
#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: sent > to vmpegdecomp\n");
#endif /* PROTOCOL_TRACE */

    write(w->mpegViewer.Zip.decoder.outFd, ">", 1);
}

static void
VVQuit(XnvrMpegViewerWidget w)
{
    if (w->mpegViewer.Zip.decoder.pid == 0) {
        return;
    }

#  ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: sent q to vmpegdecomp\n");
#  endif /* PROTOCOL_TRACE */

/*
    XtVaSetValues(w->mpegViewer.Zip.videoDisplay,
	XnvrNisActive, True,
	NULL);
*/
    /* probably not sufficient to kick the widget active */
    w->videoDisplay.isActive = True;

    write(w->mpegViewer.Zip.decoder.outFd, "q", 1);
    w->mpegViewer.Zip.decoder.pid = 0;
    XtRemoveInput(w->mpegViewer.Zip.inputid);
    close(w->mpegViewer.Zip.decoder.inFd);
    close(w->mpegViewer.Zip.decoder.outFd);

    WaitForZip(w);
}


static void
VVRunZip(XnvrMpegViewerWidget w)
{
    int    i, pid;
    Argvec argvec;
    char	buf[256];
    XWindowAttributes win_attr;

    XGetWindowAttributes(XtDisplay(w), XtWindow(w), &win_attr);
    if (win_attr.map_state != IsViewable) {
        return;
    }

    argvec.length = 0;
    AddArgv(&argvec, "vmpegdecomp", False);
    if (w->mpegViewer.Zip.rscData.gray)
        AddArgv(&argvec, "-y", False);
    if (w->mpegViewer.Zip.rscData.loop)
        AddArgv(&argvec, "-l", False);
    AddFmtArgv(&argvec, "-%c%s",
	w->mpegViewer.Zip.rscData.useX ? 'X' : 'x',
	(int) DisplayString(XtDisplay(w)));
    AddFmtArgv(&argvec, "-w%d", XtWindow(w));

    if (w->mpegViewer.Zip.rscData.gray) {
	if (w->mpegViewer.Zip.rscData.grayFrameRate)
	    AddFmtArgv(&argvec, "-f%d", w->mpegViewer.Zip.rscData.grayFrameRate);
    } else {
	if (w->mpegViewer.Zip.rscData.colorFrameRate)
	    AddFmtArgv(&argvec, "-f%d", w->mpegViewer.Zip.rscData.colorFrameRate);
    }

    if (w->mpegViewer.Zip.rscData.preloadSize)
        AddFmtArgv(&argvec, "-p%d", w->mpegViewer.Zip.rscData.preloadSize);
    if (w->mpegViewer.Zip.rscData.queueSize)
        AddFmtArgv(&argvec, "-q%d", w->mpegViewer.Zip.rscData.queueSize);
    if (w->mpegViewer.Zip.rscData.verbose)
        AddArgv(&argvec, "-v", False);

    AddArgv(&argvec, "-geometry", False);
    sprintf(w->mpegViewer.Zip.args.geometry, "%dx%d+%d+%d",
            w->core.width, w->core.height, 0, 0);
    AddArgv(&argvec, w->mpegViewer.Zip.args.geometry, False);

    if (w->mpegViewer.Zip.rscData.displayDevice) {
	AddArgv(&argvec, "-dev", False);
	AddArgv(&argvec, w->mpegViewer.Zip.rscData.displayDevice, False);
    }

    switch (w->core.depth) {
    case 24:
        AddArgv(&argvec, "-24", False);
        break;
    case 8:
        if (w->mpegViewer.Zip.rscData.overlay)
            AddArgv(&argvec, "-o", False);
        else
            AddArgv(&argvec, "-7", False);
        break;
    case 1:
        AddArgv(&argvec, "-1", False);
        break;
    }

    if (w->mpegViewer.Zip.args.dindex) {
        for (i = 0; i < w->mpegViewer.Zip.args.dindex; i++)
            AddArgv(&argvec, w->mpegViewer.Zip.args.dflags[ i ], False);
    }
    AddArgv(&argvec, w->mpegViewer.Zip.args.file, False);
    argvec.path = argvec.argv[0];

/*
    XtVaSetValues(w->mpegViewer.Zip.videoDisplay,
	XnvrNisActive, True,
	NULL);
*/
    w->videoDisplay.isActive = True;

    w->mpegViewer.Zip.decoder = PipeToAndFrom(&argvec, DefaultDuplexPipeRewriter);
    w->mpegViewer.Zip.inputid =
	XtAppAddInput(XtWidgetToApplicationContext((Widget)w),
			w->mpegViewer.Zip.decoder.inFd,
			(XtPointer)XtInputReadMask,
			HandleVideoServerInput,
			(XtPointer)w);

#ifdef PROTOCOL_TRACE
    PrintArgv(&argvec);
#endif /* PROTOCOL_TRACE */
    FreeArgv(&argvec);
}


/*********************************
 *  MA Routines
 *********************************/

static void
MARunAudioDecode(XnvrMpegViewerWidget w, Boolean wait)
{
    int    i, pid;
    Argvec argvec;
    char	buf[256];
    int		i_chars;

    argvec.length = 0;
    AddArgv(&argvec, "AudioDecode", False);
    AddFmtArgv(&argvec, "-q%d", w->mpegViewer.MA_Quality);
    if (wait == False) {
	AddArgv(&argvec, "-n", False);
    }

    AddArgv(&argvec, w->mpegViewer.AudioDecode.file, False);
    argvec.path = argvec.argv[0];

    w->mpegViewer.AudioDecode.decoder =
	PipeToAndFrom(&argvec, DefaultDuplexPipeRewriter);
    w->mpegViewer.AudioDecode.inputid =
	XtAppAddInput(XtWidgetToApplicationContext((Widget)w),
			w->mpegViewer.AudioDecode.decoder.inFd,
			(XtPointer)XtInputReadMask,
			HandleAudioServerInput,
			(XtPointer)w);

#ifdef PROTOCOL_TRACE
    PrintArgv(&argvec);
#endif /* PROTOCOL_TRACE */
    FreeArgv(&argvec);
}

static void
MAStart(XnvrMpegViewerWidget w, Boolean wait)
{
    if (!w->mpegViewer.AudioDecodeLaunching) {
	w->mpegViewer.AudioDecodeLaunching = True;
	MARunAudioDecode(w, wait);
	AudioSetPlayState(RESUME, 0);
    }
}

static void
MARestart(XnvrMpegViewerWidget w, Boolean wait)
{
    if (wait) {
	write(w->mpegViewer.AudioDecode.decoder.outFd, "w", 1);
    } else {
	write(w->mpegViewer.AudioDecode.decoder.outFd, "n", 1);
    }
    write(w->mpegViewer.AudioDecode.decoder.outFd, "r", 1);
#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: sent r(restart) to audio decoder\n");
#endif /* PROTOCOL_TRACE */
    AudioSetPlayState(RESUME, 0);
}

static void
MAGo(XnvrMpegViewerWidget w)
{ 
    if (! w->mpegViewer.AudioDecode.decoder.pid)
	return;
    if (w->mpegViewer.MpegAudioDelay > 0) {
	usleep(w->mpegViewer.MpegAudioDelay);
    }
    write(w->mpegViewer.AudioDecode.decoder.outFd, "g", 1);
#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: sent go to audio decoder\n");
#endif /* PROTOCOL_TRACE */
}


static void
MAPause(XnvrMpegViewerWidget w)
{
    if (! w->mpegViewer.AudioDecode.decoder.pid)
	return;
#ifdef LOCAL_AUDIO_CONTROL

    write(w->mpegViewer.AudioDecode.decoder.outFd, "P", 1);
#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: sent P to audio decoder\n");
#endif /* PROTOCOL_TRACE */

#else /* LOCAL_AUDIO_CONTROL*/

    AudioSetPlayState(PAUSE, 0);

#endif /* LOCAL_AUDIO_CONTROL */
}


static void
MAResume(XnvrMpegViewerWidget w)
{
    if (! w->mpegViewer.AudioDecode.decoder.pid)
	return;
#ifdef LOCAL_AUDIO_CONTROL

    write(w->mpegViewer.AudioDecode.decoder.outFd, "p", 1);
#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: sent p to audio decoder\n");
#endif /* PROTOCOL_TRACE */

#else  /* LOCAL_AUDIO_CONTROL */

    AudioSetPlayState(RESUME, 0);

#endif /* LOCAL_AUDIO_CONTROL */
}

static void
MAHalt(XnvrMpegViewerWidget w)
{
    if (! w->mpegViewer.AudioDecode.decoder.pid)
	return;
    write(w->mpegViewer.AudioDecode.decoder.outFd, "h", 1);
#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: sent h to audio decoder\n");
#endif /* PROTOCOL_TRACE */
}

static void
MAQuit(XnvrMpegViewerWidget w)
{
    KillAudioDecodeAndWait(w);
#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: killing audio decoder\n");
#endif /* PROTOCOL_TRACE */
}


/***************************************************
 *  Handlers for input from Audio/Video processes
 ***************************************************/


static void
WaitForZip(XnvrMpegViewerWidget w)
{
    union wait wstatus;
    int wpid;
    extern int errno;

#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: WaitForZip()\n");
#endif /* PROTOCOL_TRACE */
    for (;;) {
#ifdef Solaris2
	wpid = waitpid(-1, &wstatus, WNOHANG);
#else
        wpid = wait3(&wstatus, WNOHANG, 0);
#endif
        if (wpid == 0) {
            sleep(1);
            continue;
        }
        if (wpid < 0) {
            if (errno == ECHILD)
                break;
            Error("Error while waiting for child\n");
            break;
        }
        if (WIFEXITED(wstatus)) {
            if (wstatus.w_retcode != 0) {
                Msg("Process %d exit code=%d\n",
                    wpid, wstatus.w_retcode);
            }
	    break;
        }
        else if (WIFSTOPPED(wstatus)) {
            Msg("Process %d stopped with signal %d\n",
                wpid, wstatus.w_stopsig);
        } else if (WIFSIGNALED(wstatus) && wstatus.w_termsig != SIGTERM) {
            Msg("Process %d exit with signal %d\n",
                wpid, wstatus.w_termsig);
        }
    }
}


static void
HandleVideoServerInput(XtPointer clientData, int *source, XtInputId *id)
{
    int inFd = *source;
    char buf[ 1000 ], *p;
    long bytesIn;
    XnvrMpegViewerWidget w = (XnvrMpegViewerWidget)clientData;

    bytesIn = read(inFd, buf, 1000);
    if (bytesIn < 0)
        Fatal("can't read from client on fd %d\n", inFd);
    if (bytesIn == 0) {
	XtRemoveInput(w->mpegViewer.Zip.inputid);
	close(inFd);
	/* Call error callbacks ??? */
	/* VideoFailedCB(NULL, NULL, NULL); */
	/*NOTREACHED*/
    }

    p = buf;
    while (bytesIn--) {
        switch(*p++) {
        case 'l':
	    MpegVideoStateUpdate(w, VideoLoading);
	    break;
        case 'r':
	    MpegVideoStateUpdate(w, VideoRunning);
	    break;
        case 's':
	    MpegVideoStateUpdate(w, VideoPaused);
	    break;
        case 'c':
	    MpegVideoStateUpdate(w, VideoCompleted);
	    break;
        case 'g':
            break;

        case 'X': break;
        case 'x': break;
        case 'w': Msg("bad window specified\n"); break;
        default:  Msg("protocol '%c'?\n", p[-1]); break;
        }
    }
}



static void
KillAudioDecodeAndWait(XnvrMpegViewerWidget w)
{
    union wait wstatus;
    int wpid;
    extern int errno;

#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: entered KillAudioDecodeAndWait()\n");
#endif /* PROTOCOL_TRACE */

    if (w->mpegViewer.AudioDecode.decoder.pid == 0) {
        return;
    }


    write(w->mpegViewer.AudioDecode.decoder.outFd, "q", 1);
#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: sent q to audio decoder\n");
#endif /* PROTOCOL_TRACE */

    for (;;) {
#ifdef Solaris2
        wpid = waitpid(-1, &wstatus, WNOHANG);
#else
        wpid = wait3(&wstatus, WNOHANG, 0);
#endif
        if (wpid == 0) {
            sleep(1);
            continue;
        }
        if (wpid < 0) {
            if (errno == ECHILD)
                break;
            Error("Error while waiting for child\n");
            break;
        }
        if (WIFEXITED(wstatus)) {
#ifdef PROTOCOL_TRACE
                Msg("Process %d exit code=%d\n",
                    wpid, wstatus.w_retcode);
#endif /* PROTOCOL_TRACE */
	    break;
        } else if (WIFSTOPPED(wstatus)) {
#ifdef PROTOCOL_TRACE
            Msg("Process %d stopped with signal %d\n",
                wpid, wstatus.w_stopsig);
#endif /* PROTOCOL_TRACE */
        } else if (WIFSIGNALED(wstatus) && wstatus.w_termsig != SIGTERM) {
#ifdef PROTOCOL_TRACE
            Msg("Process %d exit with signal %d\n",
                wpid, wstatus.w_termsig);
#endif /* PROTOCOL_TRACE */
        }
    }

    XtRemoveInput(w->mpegViewer.AudioDecode.inputid);
    w->mpegViewer.AudioDecode.decoder.pid = 0;
    close(w->mpegViewer.AudioDecode.decoder.inFd);
    close(w->mpegViewer.AudioDecode.decoder.outFd);

#ifdef PROTOCOL_TRACE
    fprintf(stdout, "XnvrMpegViewerWidget:: exiting KillAudioDecodeAndWait()\n");
#endif /* PROTOCOL_TRACE */
}


static void
HandleAudioServerInput(XtPointer clientData, int *source, XtInputId *id)
{
    int inFd = *source;
    char buf[ 1000 ], *p;
    long bytesIn;
    XnvrMpegViewerWidget w = (XnvrMpegViewerWidget)clientData;


    bytesIn = read(inFd, buf, 1000);

    if (bytesIn < 0) {
        Fatal("can't read from client on fd %d\n", inFd);
    }

    if (bytesIn == 0) {
	XtRemoveInput(w->mpegViewer.AudioDecode.inputid);
	close(inFd);
	/* Call error callbacks ??? */
	return;
    }

    p = buf;
    while (bytesIn--) {
        switch(*p++) {
        case 'r':
	    AudioStateUpdate(w, (AudioState)AudioRunning);
	    break;
        case 'l':
	    AudioStateUpdate(w, AudioLoaded);
	    break;
        case 's':
	    AudioStateUpdate(w, AudioPaused);
	    break;
        case 'h':
        case 'c':
	    AudioStateUpdate(w, AudioCompleted);
	    break;
        case 'B':
	    printf("XnvrMpegViewer:: Audio device busy. Try again later.\n");
	    AudioStateUpdate(w, AudioCompleted);
	    break;
        case 'F':
	    printf("XnvrMpegViewer:: Cannot open specified audio file.\n");
	    AudioStateUpdate(w, AudioCompleted);
	    break;
        case 'O':
	    printf("XnvrMpegViewer:: Cannot open audio device /dev/audio.\n");
	    AudioStateUpdate(w, AudioCompleted);
	    break;
        case 'I':
	    printf("XnvrMpegViewer:: Specified audio file contain invalid data.\n");
	    AudioStateUpdate(w, AudioCompleted);
	    break;
        case 'W':
	    printf("XnvrMpegViewer:: Unable to write to audio device.\n");
	    AudioStateUpdate(w, AudioCompleted);
	    break;
        case 'C':
	    printf("XnvrMpegViewer:: Version of MPEG Audio Decoder can handle monaural data only.\n");
	    AudioStateUpdate(w, AudioCompleted);
	    break;
        case 'M':
	    printf("XnvrMpegViewer:: Version of MPEG Audio Decoder can handle 32 kHz data only.\n");
	    AudioStateUpdate(w, AudioCompleted);
	    break;
        default:
	    Msg("protocol '%c'?\n", p[-1]);
	    break;
        }
    }

}
