
/*
 * (c) Copyright 1992, 1993 North Valley Research, Inc.
 * 	All Rights Reserved
 *
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	MpegViewerP.h
 *
 * DESCRIPTION
 *	Private header file for the MpegViewer widget.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/MpegViewerP.h,v 1.2 1993/09/09 02:21:11 toddb Exp $
 */
#ifndef _MPEG_VIEWERP_H_
#define _MPEG_VIEWERP_H_

#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <sys/wait.h>
#include <sys/signal.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <bitstream.h>
#include "fileContent.h"
#include "osio.h"
#include "sunaudio.h"
#include "process.h"
#include "nvrTypes.h"

#include "MpegViewer.h"
#include "MpegDisplayP.h"

/*
 *	DEFINES
 */

#ifdef debug
# define    INTERNAL_ERROR(m)   InternalError(m)
#else
# define    INTERNAL_ERROR(m)
#endif


#define MAX_FILENAME_LENGTH    256
#define MAX_PATH_LEN    160

#define DEFAULT_QUALITY 8
#define MIN_QUALITY 1
#ifdef NOT_SPARC
#define MAX_QUALITY 32
#else
#define MAX_QUALITY 10
#endif
#define MIN_VIDEODISPLAY_WIDTH  320
#define MIN_VIDEODISPLAY_HEIGHT 240


#define X_None		0
#define X_NoFile	1
#define X_Selecting	2
#define AV_NewFile	3
#define AV_ALoaded	4
#define AV_VLoaded	5
#define AV_Loaded	6
#define AV_ARunning	7
#define AV_VRunning	8
#define AV_Running	9
#define AV_APaused	10
#define AV_VPaused	11
#define AV_Paused	12
#define AV_ACompleted	13
#define AV_xVPaused	14
#define AV_VCompleted	15
#define AV_AxPaused	16
#define AV_Completed	17
#define A_NewFile	18
#define A_Loaded	19
#define A_Running	20
#define A_Paused	21
#define A_Completed	22
#define V_NewFile	23
#define V_Loaded	24
#define V_Running	25
#define V_Paused	26
#define V_Completed	27
typedef int PlayerState;

#define File_None	0
#define File_AudioOnly	1
#define File_VideoOnly	2
#define File_AudioVideo 3
typedef int FileState;

#define AudioLoading	0
#define AudioLoaded	1
#define AudioPaused	2
#define AudioRunning	3
#define AudioCompleted	4
typedef int AudioState;

typedef struct {
    char file[MAX_FILENAME_LENGTH];
    TwoWayPipe decoder;
    XtInputId inputid;
} AudioDecodeInfo;

#define VideoLoading	0
#define VideoLoaded	1
#define VideoRunning	2
#define VideoPaused	3
#define VideoCompleted	4
typedef int MpegVideoState;

typedef struct {
    char    *displayDevice;
    Boolean gray;
    float   gamma;
    Boolean loop;
    Boolean useX;
    Boolean mpegBig;
    int     grayFrameRate;
    int     colorFrameRate;
    int     preloadSize;
    int     queueSize;
    Boolean verbose;
    Boolean overlay;
} VVAppData;


typedef struct _ZipArgs {
    char geometry[64];
    char *dflags[100];
    int dindex;
    char file[MAX_FILENAME_LENGTH];
    Dimension width;
    Dimension height;
} ZipArgs;

typedef struct _ZipInfo {
    VVAppData rscData;
    ZipArgs args;
    ViewCmapInfo *xInfo;
    ViewAttributes viewAttr;
    Boolean exposed;
    TwoWayPipe  decoder;
    XtInputId inputid;
    XRectangle geometry;
    XRectangle oldGeometry;
} ZipInfo;


/*
 *	TYPEDEFS
 */

/* The class definition for the MpegDisplay Widget */
typedef struct _XnvrMpegViewerClassPart
{
    int make_compiler_happy;    /* need dummy field */
} XnvrMpegViewerClassPart;

typedef struct _XnvrMpegViewerClassRec
{
    CoreClassPart           	core_class;
    XnvrVideoDisplayClassPart   videoDisplay_class;
    XnvrMpegDisplayClassPart    mpegDisplay_class;
    XnvrMpegViewerClassPart	mpegViewer_class;
} XnvrMpegViewerClassRec;

/* Instance record for the MpegViewer Widget */
typedef struct _XnvrMpegViewerPart
{
    /* resource variables */
    Boolean connected;
    int playState;
    char *mpegFile;
    int mpegStream;
    int frameRate;
    int currentFrame, firstFrame, lastFrame;
    XtCallbackList frameChangedCallbacks;
    XtCallbackList failureCallbacks;

    /* private state */
    FileInfo fileInfo;

    /* mpeg state */
    PlayerState      playerState;
    PlayerState      targetState;
    MpegVideoState   videoState;
    AudioState       audioState;
    FileState        fileState;
    int              MpegAudioDelay;
    char             VFilePath[MAX_PATH_LEN];
    char             AFilePath[MAX_PATH_LEN];

    /* audio vars */
    int MA_Quality;
    AudioDecodeInfo AudioDecode;
    Boolean AudioDecodeLaunching;

    /* video vars */
    ZipInfo Zip;
    Boolean ZipLaunching;


} XnvrMpegViewerPart;

typedef struct _XnvrMpegViewerRec
{
    CorePart            core;
    XnvrVideoDisplayPart        videoDisplay;
    XnvrMpegDisplayPart     mpegDisplay;
    XnvrMpegViewerPart		mpegViewer;
} XnvrMpegViewerRec;


/*
 *	EXTERNS
 */
extern XnvrMpegViewerClassRec xnvrMpegViewerClassRec;

#ifdef debug
extern void InternalError(char *);
#endif

#endif /* _MPEG_VIEWERP_H_ */
