/* $Header: /usr/local/src/nvrToMIT/RCS/MvexDisplay.c,v 1.13 1994/02/21 04:01:02 toddb Exp $ */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <Xm/Xm.h>
#include <Xm/XmP.h>
#include <MVEXlib.h>
#include "MvexDisplayP.h"
#include "nvrAlloc.h"
#include "nvrTypes.h"
#include "nvrDebug.h"

/* forward data references */
static char *ViolationAction();
static char *ViolationState();
static char *OverrideState();
static char *MvexEventName();

/* Core class functions */
static void Initialize();
static void Realize();
static void Destroy();
static void Redisplay();
static void Resize();
static Boolean SetValues();
static XtGeometryResult QueryGeometry();

/* VideoDisplay Class Functions */
static void UpdateDisplay();
static void RenderDisplay();
static void PanDisplay();

/* MvexDisplay class functions */
static void StopDisplay();
static void Sync();
static void Override();

/* Default translations and action records */
/*  What I wanted to do...
static char DefaultTranslation[] =
    "<MVEXSync>:    sync() \n\
     <MVEXOverride>: override() \n\
     <Btn1Down>:    pan() \n\
     <Key>q:	    quit()";
*/
static char DefaultTranslation[] =
    "<Btn1Down>:    pan()";

static XtActionsRec ActionsList[] =
{
    {"pan",	    (XtActionProc)PanDisplay},
    {"sync",	    (XtActionProc)Sync},
    {"override",    (XtActionProc)Override}
};

/* Resource definitions for VideoDisplay Widget */
static XtResource resources[] =
{
    {	XnvrNoverrideMode, XnvrCoverrideMode, XmRInt,
	sizeof (int), XtOffset (XnvrMvexDisplayWidget, mvexDisplay.overrideMode),
	XmRImmediate, (XtPointer) XnvrDefaultOverrideMode },

    {	XnvrNpriority, XnvrCpriority, XmRInt,
	sizeof (int), XtOffset (XnvrMvexDisplayWidget, mvexDisplay.priority),
	XmRImmediate, (XtPointer) XnvrDefaultPriority },

    {	XnvrNmvexEvent, XnvrCmvexEvent, XmRPointer,
	sizeof (int), XtOffset (XnvrMvexDisplayWidget, mvexDisplay.mvexEvent),
	XmRImmediate, (XtPointer) NULL },
};

XnvrMvexDisplayClassRec xnvrMvexDisplayClassRec =
{
    {
	/* superclass	    */	    (WidgetClass) &xnvrVideoDisplayClassRec,
	/* class_name	    */	    "XnvrMvexDisplay",
	/* widget_size	    */	    sizeof(XnvrMvexDisplayRec),
	/* class_initialize */	    NULL,
	/* chained class init */    NULL,
	/* class_inited	    */	    FALSE,
	/* initialize	    */	    Initialize,
	/* initialize hook  */	    NULL,
	/* realize	    */	    (XtRealizeProc)Realize,
	/* actions	    */	    ActionsList,
	/* num_actions	    */	    XtNumber(ActionsList),
	/* resources	    */	    resources,
	/* num_resources    */	    XtNumber(resources),
	/* xrm_class	    */	    NULLQUARK,
	/* compress_motion  */	    TRUE,
	/* compress_exposure*/	    TRUE,
	/* compress enter/exit */   TRUE,
	/* visible_interest */	    FALSE,
	/* destroy	    */	    Destroy,
	/* resize	    */	    Resize,
	/* expose	    */	    Redisplay,
	/* set_values	    */	    (XtSetValuesFunc) SetValues,
	/* set values hook  */	    NULL,
	/* set values almost*/	    XtInheritSetValuesAlmost,
	/* get values hook  */	    NULL,
	/* accept_focus	    */	    NULL,
	/* version	    */	    XtVersion,
	/* callback offset 1st */   NULL,
	/* default trans    */	    DefaultTranslation,
	/* query geo proc   */	    XtInheritQueryGeometry, /* (XtGeometryHandler)QueryGeometry,*/
	/* display accelerator */   NULL,
	/* reserved	    */	    NULL	/* extension record */
    },

    {					    /* XnvrVideoDisplay */
	/* updateDisplay    */	    XnvrInheritUpdateDisplay,
	/* renderDisplay    */	    (XtWidgetProc) RenderDisplay,
	/* panDisplay	    */	    XnvrInheritPanDisplay,
	0,				    /* extension */
    },

    {					    /* XnvrMvexDisplay */
	0,				    /* extension */
    }
};

WidgetClass xnvrMvexDisplayWidgetClass = (WidgetClass) &xnvrMvexDisplayClassRec;


static void Initialize(req, new)
    XnvrMvexDisplayWidget req;
    XnvrMvexDisplayWidget new;
{
    /* by this time, the superclass' Initialize has already been called */
    int i;
    int nvins;
    int intUnused;
    XVisualInfo *p_vis_info, vis_info_tmpl;
    Visual* vis, *def_vis;
    MVEXRenderModel *pModel, *tmp_pModel;
    MVEXVin *pXvin;
    MVEXVin *pXvins;
    MVEXFraction lower, higher;
    MVEXFraction *hueList, *satList, *conList, *briList;
    int iHue, nHue, iSat, nSat, iCon, nCon, iBri, nBri;
    int ii;
    Display *dpy;
    int screen;
    
    WhenDebugEnabled( NormalOutput("XnvrMvexDisplayWidget::Initialize\n") );
    dpy = XtDisplay(req);
    screen = XScreenNumberOfScreen( XtScreen(req) );

    new->mvexDisplay.mvexBase = MVEXBaseEvent(dpy);
    pXvins = MVEXGetVins( dpy, screen, &nvins);
	/* previously... videoDisplay->engine was mvexDisplay->vin */

    /* new->videoDisplay.engine = 1; */
    new->videoDisplay.n_engines = 1; /* XXX lazy!! */
    pXvin = &(pXvins[new->videoDisplay.engine]);
    if (!nvins)
	Fatal("XnvrMvexDisplayWidget::Initialize:: no video inputs\n");

    new->mvexDisplay.pXvin = pXvin;
    new->mvexDisplay.vid = MVEXCreateVideo(dpy, pXvin->vref);
    if (new->videoDisplay.port_index >= 0 &&
	new->videoDisplay.port_index <= pXvin->n_ports)
    {
	new->mvexDisplay.port = pXvin->ports[ new->videoDisplay.port_index ];
    }
    else
	Fatal("XnvrMvexDisplayWidget::Initialize:: invalid port number\n");

    new->videoDisplay.n_ports = pXvin->n_ports;
    new->mvexDisplay.portNameArray =
	(char **) SecureAlloc(pXvin->n_ports * sizeof(char *));
    for (i = 0; i < pXvin->n_ports; i++)
    {
	new->mvexDisplay.portNameArray[i] = XGetAtomName(dpy, pXvin->ports[i]);
    }
    new->videoDisplay.portNames = &(new->mvexDisplay.portNameArray[0]);

    new->videoDisplay.visual = NULL;

    new->mvexDisplay.vin_x = 0;
    new->mvexDisplay.vin_y = 0;
    new->mvexDisplay.vin_width = pXvin->geometry.width;
    new->mvexDisplay.vin_height = pXvin->geometry.height;
    new->videoDisplay.src_x = new->mvexDisplay.vin_x;
    new->videoDisplay.src_y = new->mvexDisplay.vin_y;
    new->videoDisplay.src_width = new->mvexDisplay.vin_width;
    new->videoDisplay.src_height = new->mvexDisplay.vin_height;
    new->videoDisplay.dest_x = new->mvexDisplay.vin_x;
    new->videoDisplay.dest_y = new->mvexDisplay.vin_y;
    new->videoDisplay.dest_width = new->mvexDisplay.vin_width;
    new->videoDisplay.dest_height = new->mvexDisplay.vin_height;

    new->mvexDisplay.hueList = NULL;
    new->mvexDisplay.satList = NULL;
    new->mvexDisplay.contrastList = NULL;
    new->mvexDisplay.brightList = NULL;
    new->mvexDisplay.abilityMask = 0;
    if (pXvin->ability.n_hue)
    {
	hueList = MVEXFractionList(pXvin->ability.hue,
				pXvin->ability.n_hue,
				&nHue);
	MVEXMatchFractionInList(&(pXvin->ability.normal_hue),
				hueList, nHue, &lower, &higher, &iHue);
	new->mvexDisplay.hueList = hueList;
	new->videoDisplay.hue.count = nHue;
	new->videoDisplay.hue.min = 0;
	new->videoDisplay.hue.current = iHue;
	new->videoDisplay.hue.normal = iHue;
	new->mvexDisplay.abilityMask |= VRHue;
    }
    if (pXvin->ability.n_saturation)
    {
	satList = MVEXFractionList(pXvin->ability.saturation,
				pXvin->ability.n_saturation,
				&nSat);
	MVEXMatchFractionInList(&(pXvin->ability.normal_saturation),
				satList, nSat, &lower, &higher, &iSat);
	new->mvexDisplay.satList = satList;
	new->videoDisplay.sat.count = nSat;
	new->videoDisplay.sat.min = 0;
	new->videoDisplay.sat.current = iSat;
	new->videoDisplay.sat.normal = iSat;
	new->mvexDisplay.abilityMask |= VRSaturation;
    }
    if (pXvin->ability.n_contrast)
    {
	conList = MVEXFractionList(pXvin->ability.contrast,
				pXvin->ability.n_contrast,
				&nCon);
	MVEXMatchFractionInList(&(pXvin->ability.normal_contrast),
				conList, nCon, &lower, &higher, &iCon);
	new->mvexDisplay.contrastList = conList;
	new->videoDisplay.contrast.count = nCon;
	new->videoDisplay.contrast.min = 0;
	new->videoDisplay.contrast.current = iCon;
	new->videoDisplay.contrast.normal = iCon;
	new->mvexDisplay.abilityMask |= VRContrast;
    }
    if (pXvin->ability.n_bright)
    {
	briList = MVEXFractionList(pXvin->ability.bright,
				pXvin->ability.n_bright,
				&nBri);
	MVEXMatchFractionInList(&(pXvin->ability.normal_bright),
				briList, nBri, &lower, &higher, &iBri);
	new->mvexDisplay.brightList = briList;
	new->videoDisplay.bright.count = nBri;
	new->videoDisplay.bright.min = 0;
	new->videoDisplay.bright.current = iBri;
	new->videoDisplay.bright.normal = iBri;
	new->mvexDisplay.abilityMask |= VRBrightness;
    }

    /* set the default size to the size of the default vin */
    if (new->core.width == 0)
	new->core.width = new->mvexDisplay.vin_width;
    if (new->core.height == 0)
	new->core.height = new->mvexDisplay.vin_height;

    /* find the visual to use --
     * The best visual to use would be one that both has video capability
     * that matches the default visual.
     * If that can't be, take anything that can do video.
     */

    vis = def_vis = DefaultVisual(dpy, screen);
    tmp_pModel = &pXvin->model_list[ 0 ];
    pModel = NULL;
    for (ii=0; ii < pXvin->n_models; ii++, tmp_pModel++)
    {
	WhenDebugEnabled(
	    NormalOutput("XnvrMvexDisplayWidget::Realize:: checking visual\n"));
	if (tmp_pModel->model_mask & MVEXWindowModelMask)
	{
	    WhenDebugEnabled(
		NormalOutput("XnvrMvexDisplayWidget::Realize:: ");
		NormalOutput("found video visual\n");
	    )

	    if (tmp_pModel->visualid == def_vis->visualid)
	    {
		pModel = tmp_pModel;
		WhenDebugEnabled(
		    NormalOutput("XnvrMvexDisplayWidget::Realize:: ");
		    NormalOutput("found optimum visual\n");
		)
		break;
	    }
	    /* keep the first non-optimum just in case */
	    if (pModel == NULL) pModel = tmp_pModel;
	}
    }
    if (pModel)
    {
	new->core.depth = pModel->depth;
    }
    else
	Fatal("XnvrMvexDisplayWidget::Initialize failed to find visual\n");

    vis_info_tmpl.visualid = pModel->visualid;
    p_vis_info = XGetVisualInfo(dpy, VisualIDMask, &vis_info_tmpl, &intUnused);
    if (!p_vis_info)
	Fatal("XnvrMvexDisplayWidget::Realize:: can't find visual #%x\n",
		pModel->visualid);

    vis = new->videoDisplay.visual = p_vis_info->visual;

    if (vis->visualid != def_vis->visualid) {
	new->core.colormap =
	    XCreateColormap(dpy, RootWindow(dpy, screen), vis, AllocNone);
    }
}


static void Destroy(w)
    XnvrMvexDisplayWidget w;
{
    WhenDebugEnabled( NormalOutput("XnvrMvexDisplayWidget::Destroy\n") );
}


static void Realize(w, value_mask, attributes)
    XnvrMvexDisplayWidget w;
    XtValueMask* value_mask;
    XSetWindowAttributes* attributes;
{
    /* by this time, the superclass' Realize has already been called */
    Window window;
    XtValueMask new_value_mask;
    XGCValues gcValues;
    Pixel foreground;
    Display *dpy;
    XColor colorDef, rgbDef;

    WhenDebugEnabled( NormalOutput("XnvrMvexDisplayWidget::Realize\n") );

    dpy = XtDisplay(w);

    /* XXX should really use the name in the resource database */
    XAllocNamedColor(dpy, w->core.colormap, "black", &colorDef, &rgbDef);
    foreground = colorDef.pixel;

    attributes->background_pixmap = None;
    attributes->event_mask = ExposureMask | VisibilityChangeMask |
			ButtonPressMask | ButtonReleaseMask | KeyPressMask;
    attributes->backing_store = Always;
    *value_mask |= CWColormap | CWBackPixmap | CWEventMask
		 | CWBorderPixel | CWBackingStore;
    XtCreateWindow((Widget)w, InputOutput,
		    w->videoDisplay.visual, *value_mask, attributes);
    window = XtWindow((Widget)w);

    MVEXSelectEvents(dpy, window, MVEXOverrideMask | MVEXViolationMask);
    MVEXSelectEvents(dpy, w->mvexDisplay.vid, MVEXSyncMask);

    gcValues.foreground = foreground;
    gcValues.fill_style = FillSolid;
    gcValues.function = GXcopy;

    w->videoDisplay.renderGC =
        XCreateGC(dpy, window,
                  GCFunction | GCFillStyle | GCForeground, &gcValues);
    XtCallCallbackList((Widget)w, w->videoDisplay.realizeCallback, NULL);
}


static void Redisplay(w, event, region)
    XnvrMvexDisplayWidget w;
    XEvent event;
    Region region;
{
    WhenDebugEnabled( NormalOutput("XnvrMvexDisplayWidget::Redisplay\n") );

    /* pass the buck to the superclass */
    /*
    (*xnvrMvexDisplayClassRec.core_class.superclass->core_class.expose)
	((XnvrVideoDisplayWidget)w, event, region);
    */
    if (XtIsRealized(w) && !w->core.being_destroyed)
    {
	XtCallCallbackList((Widget)w, w->videoDisplay.exposeCallback, NULL);
    }
    if (XtIsRealized(w) && !w->core.being_destroyed && w->videoDisplay.isActive)
    {
	if (!w->videoDisplay.alreadyRedisplayed)
	{
	    w->videoDisplay.alreadyRedisplayed = True;
	    UpdateDisplay(w, RenderDisplay);
	}
    }
}

static void Resize(w)
    XnvrMvexDisplayWidget w;
{
    XtWidgetProc resize;

    WhenDebugEnabled( NormalOutput("XnvrMvexDisplayWidget::Resize\n") );

    /* pass the buck to the superclass */
    resize = xnvrMvexDisplayClassRec.core_class.superclass->core_class.resize;
    (*resize) ((Widget)w);
}

static Boolean SetValues (current, req, new)
    XnvrMvexDisplayWidget current;
    XnvrMvexDisplayWidget req;
    XnvrMvexDisplayWidget new;
{
    XEvent *mvexEvent;
    int mvexBase;

    WhenDebugEnabled( NormalOutput("XnvrMvexDisplayWidget::SetValues\n") );

    /* Query-only resources -- don't allow them to be changed.
       They are gotten from mvex and can't be meaningfully altered.
     */
    new->videoDisplay.hue.count = current->videoDisplay.hue.count;
    new->videoDisplay.hue.min = current->videoDisplay.hue.min;
    new->videoDisplay.sat.count = current->videoDisplay.sat.count;
    new->videoDisplay.sat.min = current->videoDisplay.sat.min;
    new->videoDisplay.contrast.count = current->videoDisplay.contrast.count;
    new->videoDisplay.contrast.min = current->videoDisplay.contrast.min;
    new->videoDisplay.bright.count = current->videoDisplay.bright.count;
    new->videoDisplay.bright.min = current->videoDisplay.bright.min;
    new->videoDisplay.portNames = current->videoDisplay.portNames;
    new->videoDisplay.n_ports = current->videoDisplay.n_ports;
    new->videoDisplay.n_engines = current->videoDisplay.n_engines;
    /* new->videoDisplay.engine = current->videoDisplay.engine; */
    new->videoDisplay.visual = current->videoDisplay.visual;

    /* Video control resources */

    if (new->videoDisplay.resizeMode != current->videoDisplay.resizeMode)
    {
	new->videoDisplay.src_x = new->mvexDisplay.vin_x;
	new->videoDisplay.src_y = new->mvexDisplay.vin_y;
	new->videoDisplay.src_width = new->mvexDisplay.vin_width;
	new->videoDisplay.src_height = new->mvexDisplay.vin_height;
	new->videoDisplay.dest_x = new->mvexDisplay.vin_x;
	new->videoDisplay.dest_y = new->mvexDisplay.vin_y;
	new->videoDisplay.dest_width = new->mvexDisplay.vin_width;
	new->videoDisplay.dest_height = new->mvexDisplay.vin_height;
    }

    if (new->videoDisplay.port_index != current->videoDisplay.port_index)
    {
	new->mvexDisplay.port =
	    (new->mvexDisplay.pXvin)->ports[new->videoDisplay.port_index];
	new->videoDisplay.isActive = False;
    }

    if (new->mvexDisplay.mvexEvent != NULL)
    {
	/* The event has to be copied into a temp and cleared before
	 * processing the event, lest an infinite loop ensue
	 */
	mvexEvent = new->mvexDisplay.mvexEvent;
	new->mvexDisplay.mvexEvent = NULL;
	mvexBase = MVEXBaseEvent(XtDisplay(current));
	switch(mvexEvent->type - mvexBase)
	{
	    case MVEXOverride:
		Override (current, mvexEvent);
		return(False);
		break;
	    case MVEXSync:
		Sync (current, mvexEvent);
		return(False);
		break;
	}
    }

    if (new->videoDisplay.isActive != current->videoDisplay.isActive)
    {
	if (new->videoDisplay.isActive)
	{
	    UpdateDisplay(current, RenderDisplay);
	    return(False);
	}
	else
	{
	    StopDisplay(current);
	    return(False);
	}
    }

    new->videoDisplay.alreadyRedisplayed = False;
    return(True);
}


static void PanDisplay (w, event)
    XnvrMvexDisplayWidget w;
    XEvent *event;
{
    WhenDebugEnabled(
	NormalOutput("XnvrMvexDisplayWidget::PanDisplay ");
	NormalOutput("inherit from superclass\n");
    )

    /* call the superclass's routine */
    (*xnvrMvexDisplayClassRec.videoDisplay_class.panDisplay)
	(w, event, RenderDisplay);
}


static void Sync (w, event)
    XnvrMvexDisplayWidget w;
    XEvent *event;
{
    WhenDebugEnabled( NormalOutput("XnvrMvexDisplayWidget::Sync\n") );
}


static void Override (w, event)
    XnvrMvexDisplayWidget w;
    XEvent *event;
{
    union {
	MVEXViolationEvent *violate;
	MVEXSetupEvent *setup;
	MVEXSyncEvent *sync;
	MVEXOverrideEvent *override;
	MVEXRequestEvent *request;
	XEvent *core;
    } u;
    WhenDebugEnabled( NormalOutput("XnvrMvexDisplayWidget::Override\n") );
    u.core = event;
    if (u.override->state == MVEXRequestOverride &&
	u.override->override_id != w->mvexDisplay.vid)
    {
	WhenDebugEnabled(
	    NormalOutput("XnvrMvexDisplayWidget::Override client override\n"));

	if (w->mvexDisplay.overrideMode == XnvrOverrideModeNormal)
	{
	    w->videoDisplay.isActive = False;
	}
	else
	{
	    UpdateDisplay(w, RenderDisplay);
	}
    }
}


static void UpdateDisplay(w, renderProc)
    XnvrMvexDisplayWidget w;
    XtWidgetProc renderProc;
{
    WhenDebugEnabled(
	NormalOutput("XnvrMvexDisplayWidget::UpdateDisplay ");
	NormalOutput("calling superclass's routine\n");
    )

    /* call the superclass's routine */
    (*xnvrMvexDisplayClassRec.videoDisplay_class.updateDisplay)
	((Widget)w, renderProc);
    /* RenderDisplay(w); */
}


static void RenderDisplay(w)
    XnvrMvexDisplayWidget w;
{
    MVEXVideoValues vv;

    WhenDebugEnabled( NormalOutput("XnvrMvexDisplayWidget::RenderDisplay\n") );

    vv.priority = w->mvexDisplay.priority;
    vv.full_motion = (Bool)(w->videoDisplay.fullMotion);
    if (w->mvexDisplay.hueList)
	vv.hue = w->mvexDisplay.hueList[w->videoDisplay.hue.current];
    if (w->mvexDisplay.satList)
	vv.saturation = w->mvexDisplay.satList[w->videoDisplay.sat.current];
    if (w->mvexDisplay.contrastList)
	vv.contrast = w->mvexDisplay.contrastList[w->videoDisplay.contrast.current];
    if (w->mvexDisplay.brightList)
	vv.bright = w->mvexDisplay.brightList[w->videoDisplay.bright.current];
    MVEXRenderVideo(XtDisplay(w), w->mvexDisplay.vid, XtWindow(w),
	    w->videoDisplay.renderGC, w->mvexDisplay.port,
	    w->mvexDisplay.vin_x, w->mvexDisplay.vin_y,
	    w->mvexDisplay.vin_width, w->mvexDisplay.vin_height,
	    w->videoDisplay.dest_x, w->videoDisplay.dest_y,
	    w->videoDisplay.dest_width, w->videoDisplay.dest_height,
	    VRPriority | VRFullMotion | w->mvexDisplay.abilityMask,
	    &vv);
}


static void StopDisplay (w)
    XnvrMvexDisplayWidget w;
{
    Window videoWindow;
    WhenDebugEnabled( NormalOutput("XnvrMvexDisplayWidget::StopDisplay\n") );
    videoWindow = XtWindow(w);
    MVEXStopVideo(XtDisplay(w), MVEXStopRenderVideo, &videoWindow, 1);
}


char *MvexEventNames[] = {
    "Violation",
    "Setup",
    "Sync",
    "Override",
    "Request"
};

char *ViolationActions[] = {
    "scale",
    "placement",
    "clip",
    "overlap",
    "hue",
    "saturation",
    "contrast",
    "brightness",
};

char *ViolationStates[] = {
    "success",
    "fail",
    "subset"
};

char *OverrideStates[] = {
    "stopped",
    "drawable change",
    "marker reached",
    "request override",
    "hardware error",
    "reused",
};

#define AllViolationBits       (MVEXScaleViolation	\
			      | MVEXPlacementViolation	\
			      | MVEXClipViolation	\
			      | MVEXOverlapViolation	\
			      | MVEXHueViolation	\
			      | MVEXSaturationViolation	\
			      | MVEXContrastViolation	\
			      | MVEXBrightnessViolation)

static char *ViolationAction(action)
    int action;
{
    static char buf[ 100 ];  /* BUFSIZ */
    int i, bits;

    buf[0] = '\0';
    bits = 0;
    for (i = 0; (1<<i) < AllViolationBits; i++) {
	if ((1<<i) & action) {
	    sprintf(buf + strlen(buf), "%s%s",
		bits ? "," : "", ViolationActions[ i ]);
	    bits++;
	}
    }
    if (action & (~AllViolationBits))
	    sprintf(buf + strlen(buf), "%s ?? bits", bits ? "and" : "");
    return (buf);
}

static char *ViolationState(state)
    int state;
{
    if (state >= sizeof(ViolationStates) / sizeof(ViolationStates[0]))
	return ("??");
    return (ViolationStates[ state ]);
}

static char *OverrideState(state)
    int state;
{
    if (state >= sizeof(OverrideStates) / sizeof(OverrideStates[0]))
	return ("??");
    return (OverrideStates[ state ]);
}

static char *MvexEventName(mvexType)
    int mvexType;
{
    if (mvexType >= sizeof(MvexEventNames) / sizeof(MvexEventNames[0]))
	return("unknown");
    else
	return(MvexEventNames[ mvexType ]);
}

