/* $Header: /usr/local/src/nvrToMIT/RCS/PlxDisplay.c,v 1.19 1995/04/12 23:10:00 russ Exp $ */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <Xm/Xm.h>
#include <Xm/XmP.h>
#include "PlxDisplayP.h"
#include "viewcmap.h"
#include "nvrAlloc.h"

/* Core class functions */
static void Initialize();
static void Realize();
static void Destroy();
static void Redisplay();
static void Resize();
static Boolean SetValues();
static XtGeometryResult QueryGeometry();

/* VideoDisplay Class Functions */
static void UpdateDisplay();
static void RenderDisplay();
static void PanDisplay();

/* PlxDisplay class functions */
static void StopDisplay();

/* Default translations and action records */
/*  What I wanted to do...
static char DefaultTranslation[] =
     <Btn1Down>:    pan() \n\
     <Key>q:	    quit()";
*/
static char DefaultTranslation[] =
    "<Btn1Down>:    pan()";

static XtActionsRec ActionsList[] =
{
    {"pan",	    (XtActionProc)PanDisplay},
};

static struct PlxPortMap {
    char *name;
    UInt standard;
    UInt format;
    boolean marked;
} PlxPortMaps[] = {
    { "ntsc-composite",	PLX_NTSC,  PLX_COMP },
    { "ntsc-SVideo",	PLX_NTSC,  PLX_YC },
    { "ntsc-BetaCam",	PLX_NTSC,  PLX_YUV },
    { "ntsc-RGB",	PLX_NTSC,  PLX_RGB },
    { "pal-composite",	PLX_PAL,   PLX_COMP },
    { "pal-SVideo",	PLX_PAL,   PLX_YC },
    { "pal-BetaCam",	PLX_PAL,   PLX_YUV },
    { "pal-RGB",		PLX_PAL,   PLX_RGB },
    { "secam-composite",	PLX_SECAM, PLX_COMP },
    { "secam-SVideo",	PLX_SECAM, PLX_YC },
    { "secam-BetaCam",	PLX_SECAM, PLX_YUV },
    { "secam-RGB",	PLX_SECAM, PLX_RGB },
};

/* Resource definitions for VideoDisplay Widget */
/*
 *static XtResource resources[] =
 *{
 *    {	XnvrNdeactivateCallback, XnvrCdeactivateCallback, XmRCallback,
 *	sizeof (int),
 *	XtOffset (XnvrPlxDisplayWidget, plxDisplay.deactivateCBL),
 *	XmRImmediate, (XtPointer) NULL },
 *};
 */
static XtResource resources[] =
{
    {	XnvrNinputChannel, XnvrCinputChannel, XmRInt,
	sizeof (int),
	XtOffset (XnvrPlxDisplayWidget, plxDisplay.inputChannel),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNinputStandard, XnvrCinputStandard, XmRInt,
	sizeof (int),
	XtOffset (XnvrPlxDisplayWidget, plxDisplay.inputStandard),
	XmRImmediate, (XtPointer) NULL },

};


XnvrPlxDisplayClassRec xnvrPlxDisplayClassRec =
{
    {
	/* superclass	    */	    (WidgetClass) &xnvrVideoDisplayClassRec,
	/* class_name	    */	    "XnvrPlxDisplay",
	/* widget_size	    */	    sizeof(XnvrPlxDisplayRec),
	/* class_initialize */	    NULL,
	/* chained class init */    NULL,
	/* class_inited	    */	    FALSE,
	/* initialize	    */	    Initialize,
	/* initialize hook  */	    NULL,
	/* realize	    */	    (XtRealizeProc)Realize,
	/* actions	    */	    ActionsList,
	/* num_actions	    */	    XtNumber(ActionsList),
	/* resources	    */	    resources,
	/* num_resources    */	    XtNumber(resources),
	/* xrm_class	    */	    NULLQUARK,
	/* compress_motion  */	    TRUE,
	/* compress_exposure*/	    TRUE,
	/* compress enter/exit */   TRUE,
	/* visible_interest */	    FALSE,
	/* destroy	    */	    Destroy,
	/* resize	    */	    Resize,
	/* expose	    */	    Redisplay,
	/* set_values	    */	    (XtSetValuesFunc) SetValues,
	/* set values hook  */	    NULL,
	/* set values almost*/	    XtInheritSetValuesAlmost,
	/* get values hook  */	    NULL,
	/* accept_focus	    */	    NULL,
	/* version	    */	    XtVersion,
	/* callback offset 1st */   NULL,
	/* default trans    */	    DefaultTranslation,
	/* query geo proc   */	    NULL, /*(XtGeometryHandler)QueryGeometry,*/
	/* display accelerator */   NULL,
	/* reserved	    */	    NULL	/* extension record */
    },

    {					    /* XnvrVideoDisplay */
	/* updateDisplay    */	    XnvrInheritUpdateDisplay,
	/* renderDisplay    */	    (XtWidgetProc) RenderDisplay,
	/* panDisplay	    */	    XnvrInheritPanDisplay,
	0,				    /* extension */
    },

    {					    /* XnvrPlxDisplay */
	0,				    /* extension */
    }
};

WidgetClass xnvrPlxDisplayWidgetClass = (WidgetClass) &xnvrPlxDisplayClassRec;

/*
 * Assign widget attributes that are governed by the signal standard
 */
static void
AssignSignalInfo(new)
    XnvrPlxDisplayWidget new;
{
    plx_IO_list *list = new->plxDisplay.plxio->inputs;
    int engine = new->videoDisplay.engine;
    plx_signal *pSignal;
    plx_signal_list *pSignalList;

    while( engine-- ) {
	list = list->next_IO;
    }

    pSignalList = list->signal_list;
    while (pSignalList) {
	pSignal = pSignalList->signal;
	if (pSignal->standard == new->plxDisplay.inputStandard) {
	    break;
	}
	pSignalList = pSignalList->next_signal;
    }
    if (!pSignalList) {
	pSignal = list->signal_list->signal;
    }
    /*
     * set the defaults for this engine... the first in every set
     */
    /*
     * We want a signal size that is 4:3.  Parallax gives a height that
     * is the entire vertical signal.  Clip it here to be exactly
     * 3/4 of the width.
     */
    new->plxDisplay.heightIn = (pSignal->w * 3) >> 2;

    /*
     * The starting point in the signal may be different depending
     * on the source material.  We have observed that 46 works nicely
     * for the Dick Tracy Laser Disk (hence the +1).
     */
    new->plxDisplay.yIn =
	pSignal->h - new->plxDisplay.heightIn + 1;

    new->plxDisplay.input = list;
    new->plxDisplay.inputChannel = list->channel;
    new->plxDisplay.xIn = 0;
    new->plxDisplay.widthIn = pSignal->w;
    
    new->videoDisplay.src_x = new->plxDisplay.xIn;
    new->videoDisplay.src_y = new->plxDisplay.yIn;
    new->videoDisplay.src_width = new->plxDisplay.widthIn;
    new->videoDisplay.src_height = new->plxDisplay.heightIn;
}


static void AssignPortIndex(new)
    XnvrPlxDisplayWidget new;
{
    plx_IO_list *list = new->plxDisplay.plxio->inputs;
    int defaultMap;
    int i, j;

    /*
     * If the port_index is out of range, bring it back in
     */
    j = 0;
    for (i = 0; i < NumberOfCells(PlxPortMaps); i++) {
	if (! PlxPortMaps[ i ].marked)
	    continue;
	if (new->videoDisplay.port_index == j) {
	    new->plxDisplay.inputStandard = PlxPortMaps[i].standard;
	    new->plxDisplay.pixelFormat = PlxPortMaps[i].format;
	    AssignSignalInfo(new);
	    return;
	}
	if (j++ == 0)
	    defaultMap = i;
    }
    new->videoDisplay.port_index = defaultMap;
    new->plxDisplay.inputStandard = PlxPortMaps[ defaultMap ].standard;
    new->plxDisplay.pixelFormat = PlxPortMaps[ defaultMap ].format;

}

static void SelectEngine(new)
    XnvrPlxDisplayWidget new;
{
    plx_IO_list *list = new->plxDisplay.plxio->inputs;
    plx_signal *pSignal;
    plx_signal_list *pSignalList;
    plx_format_list *pFormat;
    plx_type_list *pType;
    UInt standard, format;
    int i, j;
    int engine;

    if (list == NULL)
	Fatal("XnvrPlxDisplayWidget::SelectEngine:: no video inputs\n");

    engine = new->videoDisplay.engine;
    while (--engine >= 0)
	list = list->next_IO;
#if 0

    /*
     * set the defaults for this engine... the first in every set
     */
    /*
     * We want a signal size that is 4:3.  Parallax gives a height that
     * is the entire vertical signal.  Clip it here to be exactly
     * 3/4 of the width.
     */
    new->plxDisplay.heightIn = (list->signal_list->signal->w * 3) / 4;

    /*
     * The starting point in the signal may be different depending
     * on the source material.  We have observed that 46 works nicely
     * for the Dick Tracy Laser Disk (hence the +1).
     */
    new->plxDisplay.yIn =
	list->signal_list->signal->h - new->plxDisplay.heightIn + 1;

    new->plxDisplay.input = list;
    new->plxDisplay.inputChannel = list->channel;
    new->plxDisplay.xIn = 0;
    new->plxDisplay.widthIn = list->signal_list->signal->w;
#else /* 0 */
    AssignSignalInfo(new);
#endif /* 0 */

    /*
     * Unmark all of the mappings.
     */
    for (i = 0; i < NumberOfCells(PlxPortMaps); i++)
	PlxPortMaps[ i ].marked = False;

    j = 0;
    new->plxDisplay.portNameArray =
	(char **) NullRealloc((char *)new->plxDisplay.portNameArray,
			    NumberOfCells(PlxPortMaps) * sizeof(char *) );

    pSignalList = list->signal_list;
    while (pSignalList) {
	pSignal = pSignalList->signal;
	standard = pSignal->standard;
	pFormat = list->format_list;
	while (pFormat) {
	    format = pFormat->format;
	    for (i = 0; i < NumberOfCells(PlxPortMaps); i++) {
		if (PlxPortMaps[ i ].format != format
		 || PlxPortMaps[ i ].standard != standard
		 || PlxPortMaps[ i ].marked)
		    continue;
		new->plxDisplay.portNameArray[ j++ ] = PlxPortMaps[ i ].name;
		PlxPortMaps[ i ].marked = True;
		    
	    }
            pFormat = pFormat->next_format;
        }

	pSignalList = pSignalList->next_signal;
    }

    new->videoDisplay.n_ports = j;
    new->videoDisplay.portNames = new->plxDisplay.portNameArray;

    AssignPortIndex(new);
}

static void Initialize(req, new)
    XnvrPlxDisplayWidget req;
    XnvrPlxDisplayWidget new;
{
    /* by this time, the superclass' Initialize has already been called */
    int i;
    int nEngines;
    int nvins;
    Display *dpy;
    int screen;
    plx_IO_list *list;
    XVisualInfo *pVisualInfo, visualInfoTmpl;
    Visual *vis, *defaultVisual;
    int visCount;
    int depth;
    plx_type_list *pixelTypes;
    extern plx_IO *XPlxQueryConfig();

    dpy = XtDisplay(req);
    screen = XScreenNumberOfScreen( XtScreen(req) );

    new->plxDisplay.firstRedisplay = True;
    new->plxDisplay.plxio = XPlxQueryConfig(dpy,
					    RootWindow(dpy, screen),
					    DefaultGC(dpy, screen));
    list = new->plxDisplay.plxio->inputs;
    nEngines = 0;
    while (list) {
	list = list->next_IO;
	nEngines++;
    }
    new->videoDisplay.n_engines = nEngines;
    new->plxDisplay.portNameArray = NULL;
    SelectEngine(new);

    new->videoDisplay.src_x = new->plxDisplay.xIn;
    new->videoDisplay.src_y = new->plxDisplay.yIn;
    new->videoDisplay.src_width = new->plxDisplay.widthIn;
    new->videoDisplay.src_height = new->plxDisplay.heightIn;
    new->videoDisplay.dest_x = 0;
    new->videoDisplay.dest_y = 0;
    new->videoDisplay.dest_width = new->plxDisplay.widthIn;
    new->videoDisplay.dest_height = new->plxDisplay.heightIn;

    new->videoDisplay.hue.count = 256;
    new->videoDisplay.hue.min = 0;
    new->videoDisplay.hue.current = 128;
    new->videoDisplay.hue.normal = 128;

    new->videoDisplay.sat = new->videoDisplay.hue;
    new->videoDisplay.contrast = new->videoDisplay.hue;
    new->videoDisplay.bright = new->videoDisplay.hue;

    /* set the default size to the size of the default vin */
    if (new->core.width == 0)
	new->core.width = new->plxDisplay.widthIn;
    if (new->core.height == 0)
	new->core.height = new->plxDisplay.heightIn;

    /* find the visual to use --
     * The best visual to use would be one that both has video capability
     * that matches the default visual.
     * If that can't be, take anything that can do video.
     */
    defaultVisual = DefaultVisual(dpy, screen);

    visualInfoTmpl.screen = screen;
    pVisualInfo =
	XGetVisualInfo(dpy, VisualScreenMask, &visualInfoTmpl, &visCount);

    for (i = 0; i < visCount; i++) {
	pixelTypes = new->plxDisplay.input->type_list;
	while (pixelTypes) {
	    switch (pixelTypes->type) {
	    case PLX_RGB24: depth = 24; break;
	    case PLX_YUV7:
	    case PLX_RGB8:
	    case PLX_GRAY5:
	    case PLX_GRAY7:
	    case PLX_GRAY8: depth = 8; break;
	    default:
		Msg("XnvrPlxDisplayWidget::Realize:: ");
		Fatal("unknown parallax pixel type=%d\n", pixelTypes->type);
		/* NOTREACHED */
	    }

	    /*
	     * Some versions of parallax software return depth 32...
	     */
	    if (depth == 24 && pVisualInfo[i].depth == 32)
		depth = 32;

	    if (depth == pVisualInfo[i].depth) {
		vis = pVisualInfo[ i ].visual;
		new->plxDisplay.pixelType = pixelTypes->type;
		if (vis->visualid == defaultVisual->visualid)
		    break;
	    }
	    pixelTypes = pixelTypes->next_type;
	}
    }

    new->core.depth = depth;
    new->videoDisplay.visual = vis;

    /*
     * Figure out the colormap
     */
    if (vis->visualid != defaultVisual->visualid) {
	new->core.colormap = XCreateColormap(dpy,
					     RootWindow(dpy, screen),
					     vis, AllocNone);
    }
}

static void SelectVideoContext(dpy, window, w, gc)
    Display *dpy;
    Window window;
    XnvrPlxDisplayWidget w;
    GC gc;
{
    XPlxVideoInputSelect(dpy,
			 window, 
			 gc,
			 w->plxDisplay.inputChannel,
			 w->plxDisplay.inputStandard,
			 w->plxDisplay.pixelFormat,
			 w->plxDisplay.pixelType);
    XPlxVideoValueLoad(dpy,
		       window,
		       gc,
		       w->videoDisplay.sat.current,
		       w->videoDisplay.contrast.current,
		       w->videoDisplay.hue.current,
		       w->videoDisplay.bright.current);

}

static void Destroy(w)
    XnvrPlxDisplayWidget w;
{
    free((char *)w->plxDisplay.portNameArray);
}


static void Realize(w, value_mask, attributes)
    XnvrPlxDisplayWidget w;
    XtValueMask* value_mask;
    XSetWindowAttributes* attributes;
{
    /* by this time, the superclass' Realize has already been called */
    Window window;
    Display *dpy;
    int screen;
    int i;
    XGCValues gcValues;
    Pixel foreground;
    XColor colorDef, rgbDef;

    dpy = XtDisplay(w);
    screen = XScreenNumberOfScreen( XtScreen(w) );

    /*
     * Figure out the foreground pixel
     */

    /* XXX should really use the name in the resource database */
    XAllocNamedColor(dpy, w->core.colormap, "black", &colorDef, &rgbDef);
    foreground = colorDef.pixel;

    /*
     * Fix up the rest of the attributes...
     */
    attributes->background_pixmap = None;
    attributes->event_mask = ExposureMask | VisibilityChangeMask |
			ButtonPressMask | ButtonReleaseMask | KeyPressMask;
    attributes->backing_store = NotUseful;
    *value_mask |= CWBackPixmap | CWEventMask | CWBackingStore;

    XtCreateWindow((Widget)w, InputOutput,
		   w->videoDisplay.visual, *value_mask, attributes);
    window = XtWindow((Widget)w);

    gcValues.foreground = foreground;
    gcValues.fill_style = FillSolid;
    gcValues.function = GXcopy;

    w->videoDisplay.renderGC =
        XCreateGC(dpy, window,
                  GCFunction | GCFillStyle | GCForeground, &gcValues);
    XtCallCallbackList((Widget)w, w->videoDisplay.realizeCallback, NULL);
}

static void Redisplay(w, event, region)
    XnvrPlxDisplayWidget w;
    XEvent event;
    Region region;
{
    /* pass the buck to the superclass */
    /*
     * (*xnvrPlxDisplayClassRec.core_class.superclass->core_class.expose)
     *	(w, event, region);
     */
    w->plxDisplay.firstRedisplay = True;
    if (XtIsRealized(w) && !w->core.being_destroyed)
    {
	XtCallCallbackList((Widget)w, w->videoDisplay.exposeCallback, NULL);
    }
    if (XtIsRealized(w) && !w->core.being_destroyed) {
	UpdateDisplay(w, RenderDisplay);
	if (! w->videoDisplay.isActive)
	    StopDisplay (w);
    }
}

static void Resize(w)
    XnvrPlxDisplayWidget w;
{
    /* pass the buck to the superclass */
    (*xnvrPlxDisplayClassRec.core_class.superclass->core_class.resize) ((Widget)w);
}

static Boolean SetValues (current, req, new)
    XnvrPlxDisplayWidget current;
    XnvrPlxDisplayWidget req;
    XnvrPlxDisplayWidget new;
{
    XEvent *plxEvent;
    int plxBase;
    boolean updateDisplay = False;

    /* Query-only resources -- don't allow them to be changed.
       They are gotten from plx and can't be meaningfully altered.
     */
    new->videoDisplay.hue.count = current->videoDisplay.hue.count;
    new->videoDisplay.hue.min = current->videoDisplay.hue.min;
    new->videoDisplay.sat.count = current->videoDisplay.sat.count;
    new->videoDisplay.sat.min = current->videoDisplay.sat.min;
    new->videoDisplay.contrast.count = current->videoDisplay.contrast.count;
    new->videoDisplay.contrast.min = current->videoDisplay.contrast.min;
    new->videoDisplay.bright.count = current->videoDisplay.bright.count;
    new->videoDisplay.bright.min = current->videoDisplay.bright.min;
    new->videoDisplay.portNames = current->videoDisplay.portNames;
    new->videoDisplay.n_ports = current->videoDisplay.n_ports;
    new->videoDisplay.n_engines = current->videoDisplay.n_engines;
    new->videoDisplay.visual = current->videoDisplay.visual;

    /* Video control resources */

    if (new->videoDisplay.src_x != current->videoDisplay.src_x
     || new->videoDisplay.src_y != current->videoDisplay.src_y
     || new->videoDisplay.src_width != current->videoDisplay.src_width
     || new->videoDisplay.src_height != current->videoDisplay.src_height
     || new->videoDisplay.dest_x != current->videoDisplay.dest_x
     || new->videoDisplay.dest_y != current->videoDisplay.dest_y
     || new->videoDisplay.dest_width != current->videoDisplay.dest_width
     || new->videoDisplay.dest_height != current->videoDisplay.dest_height)
    {
	updateDisplay = True;
    }

    if (new->videoDisplay.resizeMode != current->videoDisplay.resizeMode)
    {
	new->videoDisplay.src_x = new->plxDisplay.xIn;
	new->videoDisplay.src_y = new->plxDisplay.yIn;
	new->videoDisplay.src_width = new->plxDisplay.widthIn;
	new->videoDisplay.src_height = new->plxDisplay.heightIn;
	new->videoDisplay.dest_x = 0;
	new->videoDisplay.dest_y = 0;
	new->videoDisplay.dest_width = new->plxDisplay.widthIn;
	new->videoDisplay.dest_height = new->plxDisplay.heightIn;
	updateDisplay = True;
    }

    if (new->videoDisplay.engine != current->videoDisplay.engine)
    {
	SelectEngine(new);
	updateDisplay = True;
    }

    if (new->videoDisplay.hue.current != current->videoDisplay.hue.current)
	updateDisplay = True;
    if (new->videoDisplay.sat.current != current->videoDisplay.sat.current)
	updateDisplay = True;
    if (new->videoDisplay.contrast.current != current->videoDisplay.contrast.current)
	updateDisplay = True;
    if (new->videoDisplay.bright.current != current->videoDisplay.bright.current)
	updateDisplay = True;

#if 1
    AssignPortIndex(new);
    if (new->videoDisplay.port_index != current->videoDisplay.port_index)
    {
	new->videoDisplay.isActive = False;
	updateDisplay = True;
    }
#else /* 0 */
    if (new->videoDisplay.port_index != current->videoDisplay.port_index)
    {
	AssignPortIndex(new);
	new->videoDisplay.isActive = False;
	updateDisplay = True;
    }
#endif /* 0 */

    /* always turn it on, if that's what's been asked.  Only turn
     * it off if we think its on.
     */
    if (new->videoDisplay.isActive
     || new->videoDisplay.isActive != current->videoDisplay.isActive)
	updateDisplay = True;

    if (updateDisplay)
    {
	/*
	 * Note that we are calling UpdateDisplay() with 'new'.
	 * Thats because the superclass actually changes some values
	 * in the widget, and according to the Xt documentation, current
	 * and req are only for reference.
	 *
	 * At this point, we are all done changing the widget as
	 * a result of the SetValues call, so everything should be
	 * ok.  Calling another subroutine from this SetValues would
	 * normally be a no-no, because the subroutine may do other
	 * things to the widget, or worse, it may perform callbacks.
	 * Callbacks may in turn do a SetValues, and then we're in
	 * an infinite loop.
	 */
	new->videoDisplay.alreadyRedisplayed = True;
	if (new->videoDisplay.isActive) {
	    UpdateDisplay(new, RenderDisplay);
	} else
	    StopDisplay(new);
	return(False);
    }

    new->videoDisplay.alreadyRedisplayed = False;
    return(False);
}


static void PanDisplay (w, event)
    XnvrPlxDisplayWidget w;
    XEvent *event;
{
    /* call the superclass's routine */
    (*xnvrPlxDisplayClassRec.videoDisplay_class.panDisplay)
	(w, event, RenderDisplay);
}


static void UpdateDisplay(w, renderProc)
    XnvrPlxDisplayWidget w;
    XtWidgetProc renderProc;
{
    /* call the superclass's routine */
    (*xnvrPlxDisplayClassRec.videoDisplay_class.updateDisplay)
	((Widget)w, renderProc);
}

static void RenderDisplay(w)
    XnvrPlxDisplayWidget w;
{
    Display *dpy = XtDisplay(w);
    Window window = XtWindow(w);
    GC gc = w->videoDisplay.renderGC;

    SelectVideoContext(dpy, window, w, gc);

    /*
     * Here we initialize the window pixels such that they are tagged
     * as video pixels.  Backing store *must* be turned off, otherwise
     * the parallax server redraws exposed pixels, they are not
     * marked as 'video' pixels, and we never get an expose event to
     * fix things.  This flag is also set in Redisplay.
     */
    if (w->plxDisplay.firstRedisplay)
	XPlxVideoTag(dpy, window, gc, PLX_VIDEO);

    if (w->plxDisplay.widthIn == w->videoDisplay.dest_width
     && w->plxDisplay.heightIn == w->videoDisplay.dest_height) {
	XPlxVideoLive(dpy, window, gc,
		      w->plxDisplay.xIn,
		      w->plxDisplay.yIn,
		      w->videoDisplay.dest_x,
		      w->videoDisplay.dest_y,
		      w->videoDisplay.dest_width,
		      w->videoDisplay.dest_height);
    } else {
	XPlxVideoSqueezeLive(dpy, window, gc,
			     w->plxDisplay.xIn,
			     w->plxDisplay.yIn,
			     w->plxDisplay.widthIn,
			     w->plxDisplay.heightIn,
			     w->videoDisplay.dest_x,
			     w->videoDisplay.dest_y,
			     w->videoDisplay.dest_width,
			     w->videoDisplay.dest_height);
    }

    if (w->plxDisplay.firstRedisplay) {
	/*
	 * If the tag value is PLX_VIDEO, the screen flashes everytime
	 * a new XPlxVideoLive() is done.
	 */
	XPlxVideoTag(dpy, window, gc, PLX_VIDEO_OVR);
	w->plxDisplay.firstRedisplay = False;
    }
}


static void StopDisplay (w)
    XnvrPlxDisplayWidget w;
{
    Display *dpy = XtDisplay(w);
    Window window = XtWindow(w);

    XPlxVideoStop(dpy, window, w->videoDisplay.renderGC);
}
