/* $Header: /usr/local/src/nvrToMIT/RCS/PlxJpegDisplay.c,v 1.5 1994/02/21 04:01:02 toddb Exp $ */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <Xm/Xm.h>
#include <Xm/XmP.h>
#include "PlxJpegDisplayP.h"
#include "XPlxExt.h"
#include "nvrAlloc.h"
#include "mcs-server.h"
#include "jpegcomp.h"

/* Core class functions */
static void Initialize();
static void Realize();
static void Destroy();
static void Redisplay();
static void Resize();
static Boolean SetValues();
static XtGeometryResult QueryGeometry();

/* VideoDisplay Class Functions */

/* PlxJpegDisplay class functions */

/* Default translations and action records */



/* Resource definitions for PlxJpegDisplay Widget */
static XtResource resources[] =
{
    {   XnvrNconfigCallback, XnvrCConfigCallback, XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrPlxJpegDisplayWidget, plxJpegDisplay.config_callback),
	XmRImmediate, (XtPointer) NULL },
    {   XnvrNmcsServer, XnvrCMcsServer, XmRPointer,
	sizeof (XtPointer),
	XtOffset (XnvrPlxJpegDisplayWidget, plxJpegDisplay.mcs_server),
	XmRImmediate, (XtPointer) NULL },
};



/* Class Record definitions for VideoDisplay Widget */

XnvrPlxJpegDisplayClassRec xnvrPlxJpegDisplayClassRec =
{
    {
	/* superclass	    */	    (WidgetClass) &xnvrVideoDisplayClassRec,
	/* class_name	    */	    "XnvrPlxJpegDisplay",
	/* widget_size	    */	    sizeof(XnvrPlxJpegDisplayRec),
	/* class_initialize */	    NULL,
	/* chained class init */    NULL,
	/* class_inited	    */	    FALSE,
	/* initialize	    */	    Initialize,
	/* initialize hook  */	    NULL,
	/* realize	    */	    (XtRealizeProc)Realize,
	/* actions	    */	    NULL,
	/* num_actions	    */	    0,
	/* resources	    */	    resources,
	/* num_resources    */	    XtNumber(resources),
	/* xrm_class	    */	    NULLQUARK,
	/* compress_motion  */	    TRUE,
	/* compress_exposure*/	    TRUE,
	/* compress enter/exit */   TRUE,
	/* visible_interest */	    FALSE,
	/* destroy	    */	    Destroy,
	/* resize	    */	    Resize,
	/* expose	    */	    Redisplay,
	/* set_values	    */	    (XtSetValuesFunc) SetValues,
	/* set values hook  */	    NULL,
	/* set values almost*/	    XtInheritSetValuesAlmost,
	/* get values hook  */	    NULL,
	/* accept_focus	    */	    NULL,
	/* version	    */	    XtVersion,
	/* callback offset 1st */   NULL,
	/* default trans    */	    NULL,
	/* query geo proc   */	    NULL, /*(XtGeometryHandler)QueryGeometry,*/
	/* display accelerator */   NULL,
	/* reserved	    */	    NULL	/* extension record */
    },

    {					    /* XnvrVideoDisplay */
	/* updateDisplay    */	    NULL,
	/* renderDisplay    */	    NULL,
	/* panDisplay	    */	    NULL,
	/* extension	    */      0,
    },

    {					    /* XnvrPlxJpegDisplay */
	0,				    /* extension */
    }
};

WidgetClass xnvrPlxJpegDisplayWidgetClass = (WidgetClass) &xnvrPlxJpegDisplayClassRec;


static void Initialize(req, new)
    XnvrPlxJpegDisplayWidget req;
    XnvrPlxJpegDisplayWidget new;
{
    /* by this time, the superclass' Initialize has already been called */
    int			i;
    Display		*dpy;
    int			screen;
    plx_IO		*p_plx_IO;
    plx_IO_list		*p_plx_IO_list;
    plx_type_list	*p_plx_type_list;
    extern plx_IO	*XPlxQueryConfig();
    XVisualInfo *pVisualInfo, visualInfoTmpl;
    Visual *vis, *defaultVisual;
    int visCount;

    new->plxJpegDisplay.mcs_server = (XtPointer)McsOpenServer(McsJpegDecompClass);

    if (new->plxJpegDisplay.mcs_server == NULL) {
	Msg("XnvrPlxJpegDisplayWidget::Initialize:: ");
	Error("Cannot connect to MCS-Jpeg server.\n");
	return;
    }

    dpy = XtDisplay(req);
    screen = XScreenNumberOfScreen(XtScreen(req));


    new->plxJpegDisplay.firstRedisplay = True;
    p_plx_IO = XPlxQueryConfig(dpy, RootWindow(dpy, screen),
	    DefaultGC(dpy, screen));
    for (p_plx_IO_list = p_plx_IO->inputs; p_plx_IO_list;
	    p_plx_IO_list = p_plx_IO_list->next_IO) {
	for (p_plx_type_list = p_plx_IO_list->type_list;
		p_plx_type_list; p_plx_type_list = p_plx_type_list->next_type) {
	    switch (p_plx_type_list->type) {
	      case PLX_RGB24:
		  goto found_PLX_RGB24;
		  break;
	      case PLX_YUV7:
	      case PLX_RGB8:
	      case PLX_GRAY5:
	      case PLX_GRAY7:
	      case PLX_GRAY8:
		  /* not yet supported */
		  break;
	      default:
		  break;
	    }
	}
    }
    Msg("XnvrPlxJpegDisplayWidget::Initialize:: ");
    Fatal("Parallax display board does not support 24-bit TrueColor\n");
    return;


found_PLX_RGB24:

    /*
     * set the default size
     */
    if (new->core.width == 0)
	new->core.width = XnvrPlxJpegDefaultWidth;
    if (new->core.height == 0)
	new->core.height = XnvrPlxJpegDefaultHeight;

    new->videoDisplay.dest_x = new->core.x;
    new->videoDisplay.dest_y = new->core.y;
    new->videoDisplay.dest_width = new->core.width;
    new->videoDisplay.dest_height = new->core.height;

    /*
     * Find the visual to use --
     * The best visual to use would be one that both has video capability
     * that matches the default visual.
     * If that can't be, take anything that can do video.
     */


    defaultVisual = DefaultVisual(dpy, screen);

    visualInfoTmpl.screen = screen;
    visualInfoTmpl.depth = 24;
    visualInfoTmpl.class = TrueColor;

    pVisualInfo = XGetVisualInfo(dpy,
	    VisualScreenMask | VisualDepthMask | VisualClassMask,
	    &visualInfoTmpl,
	    &visCount);

    if (visCount == 0) {
	visualInfoTmpl.depth = 32;
	pVisualInfo = XGetVisualInfo(dpy,
		VisualScreenMask | VisualDepthMask | VisualClassMask,
		&visualInfoTmpl,
		&visCount);

	if (visCount == 0) {
	    Msg("XnvrPlxJpegDisplayWidget::Initialize:: ");
	    Fatal("24-bit or 32-bit TrueColor visual type not found.\n");
	    return;
	}
    }

    new->core.depth = visualInfoTmpl.depth;
    new->videoDisplay.visual = pVisualInfo[0].visual;
    XFree(pVisualInfo);

    /*
     * Figure out the colormap
     */
    if (new->videoDisplay.visual->visualid != defaultVisual->visualid) {
	new->core.colormap = XCreateColormap(dpy, RootWindow(dpy, screen),
		new->videoDisplay.visual, AllocNone);
    }
}


static void Destroy(w)
    XnvrPlxJpegDisplayWidget w;
{
    /* ???? */
}


static void Realize(w, value_mask, attributes)
    XnvrPlxJpegDisplayWidget w;
    XtValueMask* value_mask;
    XSetWindowAttributes* attributes;
{
    /* by this time, the superclass' Realize has already been called */
    Window window;
    Display *dpy;
    int screen;
    int i;
    XGCValues gcValues;
    Pixel foreground;
    XColor colorDef, rgbDef;

    dpy = XtDisplay(w);
    screen = XScreenNumberOfScreen( XtScreen(w) );

    /*
     * Figure out the foreground pixel
     */

    /* XXX should really use the name in the resource database */
    XAllocNamedColor(dpy, w->core.colormap, "black", &colorDef, &rgbDef);
    foreground = colorDef.pixel;

    /*
     * Fix up the rest of the attributes...
     */
    attributes->background_pixmap = None;
    attributes->event_mask = ExposureMask | VisibilityChangeMask ;
    attributes->backing_store = NotUseful;
    *value_mask |= CWBackPixmap | CWEventMask | CWBackingStore;

    XtCreateWindow((Widget)w, InputOutput, w->videoDisplay.visual,
	    *value_mask, attributes);
    XtCallCallbackList((Widget)w, w->videoDisplay.realizeCallback, NULL);
}

static void
Resize(w)
    XnvrPlxJpegDisplayWidget w;
{
    /* pass the buck to the superclass */
    (*xnvrPlxJpegDisplayClassRec.core_class.superclass->core_class.resize) ((Widget)w);
    XtCallCallbackList((Widget)w, w->plxJpegDisplay.config_callback, NULL);
}

static Boolean SetValues (current, req, new)
    XnvrPlxJpegDisplayWidget current;
    XnvrPlxJpegDisplayWidget req;
    XnvrPlxJpegDisplayWidget new;
{
    XEvent *plxEvent;
    int plxBase;
    boolean config_changed = False;
    McsJpegComp	jpeg_params;
    ULong	jpeg_value_mask = 0;


    /* Query-only resources -- don't allow them to be changed.
       They are gotten from plx and can't be meaningfully altered.
     */
    new->videoDisplay.visual = current->videoDisplay.visual;
    new->plxJpegDisplay.mcs_server = current->plxJpegDisplay.mcs_server;

    /* Video control resources */

    if ( new->core.x != current->core.x) {
	config_changed = True;
    }
    if ( new->core.x != current->core.x
	    || new->core.y != current->core.y
	    || new->core.width != current->core.width
	    || new->core.height != current->core.height)
    {
	config_changed = True;
    }

    if (config_changed)
    {
	XtCallCallbackList((Widget)new, new->plxJpegDisplay.config_callback,
		NULL);
    }

    return(False);
}

static void
Redisplay(w, event, region)
    XnvrPlxJpegDisplayWidget w;
    XEvent event;
    Region region;
{
    /* pass the buck to the superclass */
    /*
     * (*xnvrPlxJpegDisplayClassRec.core_class.superclass->core_class.expose)
     *	(w, event, region);
     */
    w->plxJpegDisplay.firstRedisplay = True;
    if (XtIsRealized(w) && !w->core.being_destroyed)
    {
	XtCallCallbackList((Widget)w, w->videoDisplay.exposeCallback, NULL);
    }
}
