/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	SelectionBox
 *
 * DESCRIPTION
 *	Used for File Selection Box activities
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/SelectionBox.c,v 1.7 1994/10/02 23:58:07 toddb Exp $
 */
#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include "fileAccess.h"
#include "fileContent.h"
#include "nvrImage.h"
#include "nvrAlloc.h"
#include "SelectionBox.h"

typedef struct _dirInfo {
    char	**names;
    XmString	*xmNames;
    int		length;
    int		nNames;
} DirInfo, *DirInfoPtr;

static DirInfo DirData;

static void
StretchNames(DirInfoPtr dir, int index)
{
    int oldLength = dir->length;
    int i;

    if (index + 2 > dir->length) {
	dir->length = index + 2;	/* allow for name & null at the end */
	dir->names = (char **)
	    NullRealloc((char *)dir->names, sizeof(char *) * dir->length);
	dir->xmNames = (XmString *)
	    NullRealloc((char *)dir->xmNames, sizeof(XmString *) * dir->length);
	for (i = oldLength; i < dir->length; i++) {
	    dir->names[ i ] = NULL;
	    dir->xmNames[ i ] = NULL;
	}
    }
}

static int
SortDir(char **file1, char **file2)
{
    return(strcmp(*file1, *file2));
}

char **
GetFileList(char *directory, BitMaskPtr types)
{
    char	*file,
		buf[256],
		*bp;
    int		i,
		nbuf,
		systemType;
    FileInfo	fileInfo;
    InternalReasonPtr reason;
    ImageFileInfo imageInfo;
    void	*vp;

    for (i = 0; i < DirData.length; i++) {
	XtFree(DirData.names[i]);
	DirData.names[i] = NULL;
	if (DirData.xmNames[i])
	    XmStringFree(DirData.xmNames[i]);
	DirData.xmNames[i] = NULL;
    }
    DirData.nNames = 0;

    nbuf = 256;
    vp = OpenDir(directory);
    if( vp ) {
	i = strlen(directory);
	if( i > 250 )
	    nbuf = 0;
	else
	    strncpy(buf,directory,i);
    }
    if( nbuf == 0 || vp == NULL )
    	return(NULL);
    bp = buf+i;
    *bp++ = '/';
    nbuf -= i+1;

    i = 0;
    while( (file = ReadDir(vp)) ) {
	strncpy(bp,file,nbuf);
	systemType = FileType(buf);
	if ((systemType & FILE_REGULAR) == 0)
	    continue;
	reason = GetFileContentInfo(buf, &fileInfo);
	if (reason) {
	    FreeReason(reason);
	    continue;
	}
	if( ! BM_ISSET(types, fileInfo.type) )
	    continue;
	if( fileInfo.type == FT_Jmovie ) {
	    reason = GetImageFileInfo(buf, fileInfo.type, &imageInfo);
	    if (reason) {
		FreeReason(reason);
		continue;
	    }
	    if (!imageInfo.frameCount )
		continue;
	}

	StretchNames(&DirData, i);
	DirData.names[ i++ ] = CopyOfString(buf);
    }
    CloseDir(vp);

    DirData.nNames = i;
    qsort((void *)DirData.names, DirData.nNames, sizeof(char *), SortDir);
    for (i = 0; i < DirData.nNames; i++) {
	DirData.xmNames[i] = XmStringCreateSimple(DirData.names[i]);
	if( DirData.xmNames[i] == NULL )
	    return(NULL);
    }
    return(DirData.names);
}

char **
FileSearch(Widget fs, XmFileSelectionBoxCallbackStruct *cbs, BitMaskPtr types)
{
    char *mask;

    if (!XmStringGetLtoR(cbs->mask, (XmStringCharSet)XmSTRING_DEFAULT_CHARSET, 
	    &mask)) {
	return(NULL); /* can't do a thing */
    }
    if (GetFileList(mask, types) == NULL)
	return(NULL);
    XtFree(mask);

    if (DirData.nNames) {
	XtVaSetValues(fs,
		XmNfileListItems,	DirData.xmNames,
		XmNfileListItemCount,	DirData.nNames,
		XmNdirSpec,		DirData.xmNames[0],
		XmNlistUpdated,		True,
		NULL);
    } else {
	XtVaSetValues(fs,
		XmNfileListItems,	NULL,
		XmNfileListItemCount,	0,
		XmNlistUpdated,		True,
		NULL);
    }
    return(DirData.names);
}
