/* $Header: /usr/local/src/nvrToMIT/RCS/VideoDisplay.c,v 1.9 1994/02/21 04:01:02 toddb Exp $ */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <Xm/Xm.h>
#include <Xm/XmP.h>
#include "VideoDisplay.h"
#include "VideoDisplayP.h"
#include "viewcmap.h"  /* ??? */
#include "nvrDebug.h"
#include "osio.h"

/* Core class functions */
static void ClassPartInitialize();
static void Initialize();
static void Realize();
static void Destroy();
static void Redisplay();
static void Resize();
static Boolean SetValues();

/* VideoDisplay class functions */
static void UpdateDisplay();
static void RenderDisplay();
static void StopDisplay();
static void PanDisplay();

/* Default translations and action records */
static char DefaultTranslation[] =
    "<Btn1Down>:    pan()";

static XtActionsRec ActionsList[] =
{
    {"pan",	    (XtActionProc)PanDisplay}
};

/* Resource definitions for VideoDisplay Widget */
static XtResource resources[] =
{
    {   XnvrNexposeCallback, XnvrCexposeCallback, XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.exposeCallback),
	XmRImmediate, (XtPointer) NULL },
    {   XnvrNrealizeCallback, XnvrCrealizeCallback, XmRCallback,
	sizeof (XtCallbackList),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.realizeCallback),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNhue, XnvrChue, XmRInt,
	sizeof (int), XtOffset(XnvrVideoDisplayWidget,videoDisplay.hue.current),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNminHue, XnvrCminHue, XmRInt,
	sizeof (int), XtOffset (XnvrVideoDisplayWidget, videoDisplay.hue.min),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNmaxHue, XnvrCmaxHue, XmRInt,
	sizeof (int), XtOffset (XnvrVideoDisplayWidget, videoDisplay.hue.count),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNnormalHue, XnvrCnormalHue, XmRInt,
	sizeof (int), XtOffset(XnvrVideoDisplayWidget,videoDisplay.hue.normal),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNsaturation, XnvrCsaturation, XmRInt,
	sizeof (int), XtOffset(XnvrVideoDisplayWidget,videoDisplay.sat.current),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNminSaturation, XnvrCminSaturation, XmRInt,
	sizeof (int), XtOffset (XnvrVideoDisplayWidget, videoDisplay.sat.min),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNmaxSaturation, XnvrCmaxSaturation, XmRInt,
	sizeof (int), XtOffset (XnvrVideoDisplayWidget, videoDisplay.sat.count),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNnormalSaturation, XnvrCnormalSaturation, XmRInt,
	sizeof (int), XtOffset (XnvrVideoDisplayWidget, videoDisplay.sat.normal),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNcontrast, XnvrCcontrast, XmRInt,
	sizeof (int),
	XtOffset(XnvrVideoDisplayWidget, videoDisplay.contrast.current),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNminContrast, XnvrCminContrast, XmRInt,
	sizeof (int),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.contrast.min),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNmaxContrast, XnvrCmaxContrast, XmRInt,
	sizeof (int),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.contrast.count),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNnormalContrast, XnvrCnormalContrast, XmRInt, sizeof (int),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.contrast.normal),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNbrightness, XnvrCbrightness, XmRInt,
	sizeof (int),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.bright.current),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNminBrightness, XnvrCminBrightness, XmRInt,
	sizeof (int),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.bright.min),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNmaxBrightness, XnvrCmaxBrightness, XmRInt,
	sizeof (int),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.bright.count),
	XmRImmediate, (XtPointer) NULL },
    {	XnvrNnormalBrightness, XnvrCnormalBrightness, XmRInt,
	sizeof (int),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.bright.normal),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNdisplayMode, XnvrCdisplayMode, XmRInt,
	sizeof (int),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.displayMode),
	XmRImmediate, (XtPointer) XnvrDefaultDisplayMode },

    {	XnvrNresizeMode, XnvrCresizeMode, XmRInt,
	sizeof (int),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.resizeMode),
	XmRImmediate, (XtPointer) XnvrDefaultResizeMode },

    {	XnvrNisActive, XnvrCisActive, XmRBool,
	sizeof (char),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.isActive),
	XmRImmediate, (XtPointer) XnvrDefaultIsActive },

    {	XnvrNfullMotion, XnvrCfullMotion, XmRBool,
	sizeof (char),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.fullMotion),
	XmRImmediate, (XtPointer) XnvrDefaultFullMotion },

    {	XnvrNnumberOfPorts, XnvrCnumberOfPorts, XmRInt,
	sizeof (int), XtOffset (XnvrVideoDisplayWidget, videoDisplay.n_ports),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNportIndex, XnvrCportIndex, XmRInt,
	sizeof (int),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.port_index),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNportNames, XnvrCportNames, XmRPointer,
	sizeof (int),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.portNames),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNengineCount, XnvrCengineCount, XmRInt,
	sizeof (int),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.n_engines),
	XmRImmediate, (XtPointer) NULL },

    {	XnvrNengine, XnvrCengine, XmRInt,
	sizeof (int),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.engine),
	XmRImmediate, (XtPointer) NULL },

    {	XmNvisual, XmCVisual, XmRPointer,
	sizeof (Visual *),
	XtOffset (XnvrVideoDisplayWidget, videoDisplay.visual),
	XmRImmediate, (XtPointer) NULL },

};

XnvrVideoDisplayClassRec xnvrVideoDisplayClassRec =
{
    {
	/* superclass	    */	    (WidgetClass) &coreClassRec,
	/* class_name	    */	    "XnvrVideoDisplay",
	/* widget_size	    */	    sizeof(XnvrVideoDisplayRec),
	/* class_initialize */	    NULL,
	/* chained class init */    (XtWidgetClassProc) ClassPartInitialize,
	/* class_inited	    */	    FALSE,
	/* initialize	    */	    Initialize,
	/* initialize hook  */	    NULL,
	/* realize	    */	    (XtRealizeProc)Realize,
	/* actions	    */	    ActionsList,
	/* num_actions	    */	    XtNumber(ActionsList),
	/* resources	    */	    resources,
	/* num_resources    */	    XtNumber(resources),
	/* xrm_class	    */	    NULLQUARK,
	/* compress_motion  */	    TRUE,
	/* compress_exposure*/	    TRUE,
	/* compress enter/exit */   TRUE,
	/* visible_interest */	    FALSE,
	/* destroy	    */	    Destroy,
	/* resize	    */	    Resize,
	/* expose	    */	    Redisplay,
	/* set_values	    */	    (XtSetValuesFunc) SetValues,
	/* set values hook  */	    NULL,
	/* set values almost*/	    XtInheritSetValuesAlmost,
	/* get values hook  */	    NULL,
	/* accept_focus	    */	    NULL,
	/* version	    */	    XtVersion,
	/* callback offset 1st */   NULL,
	/* default trans    */	    DefaultTranslation,
	/* query geo proc   */	    NULL, /*(XtGeometryHandler)QueryGeometry,*/
	/* display accelerator */   NULL,
	/* reserved	    */	    NULL	/* extension record */
    },

    {					    /* XnvrVideoDisplay */
	/* updateDisplay    */	    (XtWidgetProc) UpdateDisplay,
	/* renderDisplay    */	    (XtWidgetProc) RenderDisplay,
	/* panDisplay	    */	    (XtWidgetProc) PanDisplay,
	NULL,					/* extension */
    }
};

WidgetClass xnvrVideoDisplayWidgetClass = (WidgetClass) &xnvrVideoDisplayClassRec;

static void ClassPartInitialize(widget_class)
    WidgetClass widget_class;
{
    XnvrVideoDisplayWidgetClass wc, super;

    WhenDebugEnabled(
	NormalOutput("XnvrVideoDisplayWidgetClass::ClassPartInitialize\n") );

    wc = (XnvrVideoDisplayWidgetClass) widget_class;
    super = (XnvrVideoDisplayWidgetClass) wc->core_class.superclass;
    if (wc->videoDisplay_class.updateDisplay == XnvrInheritUpdateDisplay)
    {
	wc->videoDisplay_class.updateDisplay =
		super->videoDisplay_class.updateDisplay;
    }
    if (wc->videoDisplay_class.renderDisplay == XnvrInheritRenderDisplay)
    {
	wc->videoDisplay_class.renderDisplay =
		super->videoDisplay_class.renderDisplay;
    }
    if (wc->videoDisplay_class.panDisplay == XnvrInheritPanDisplay)
    {
	wc->videoDisplay_class.panDisplay =
		super->videoDisplay_class.panDisplay;
    }
}

static void Initialize(req, new)
    XnvrVideoDisplayWidget req;
    XnvrVideoDisplayWidget new;
{
    WhenDebugEnabled( NormalOutput("XnvrVideoDisplayWidget::Initialize\n") );
    /* let subclass determine the initial size */
    new->videoDisplay.src_x = 0;
    new->videoDisplay.src_y = 0;
    new->videoDisplay.src_width = 0;
    new->videoDisplay.src_height = 0;
    new->videoDisplay.dest_x = 0;
    new->videoDisplay.dest_y = 0;
    new->videoDisplay.dest_width = 0;
    new->videoDisplay.dest_height = 0;
    new->videoDisplay.alreadyRedisplayed = False;

    /*
     * really oughtn't be doing this -- application's responsibility
     *
     * new->core.border_width = 0;
     */
}

static void Destroy(w)
    XnvrVideoDisplayWidget w;
{
    WhenDebugEnabled( NormalOutput("XnvrVideoDisplayWidget::Destroy\n") );
}


static void Realize(w, value_mask, attributes)
    XnvrVideoDisplayWidget w;
    XtValueMask* value_mask;
    XSetWindowAttributes* attributes;
{
    /* This routine must be implemented by each subclass */

    WhenDebugEnabled( NormalOutput("XnvrVideoDisplayWidget::Realize\n") );
}


static void Redisplay(w)
    XnvrVideoDisplayWidget w;
{
    WhenDebugEnabled( NormalOutput("XnvrVideoDisplayWidget::Redisplay\n") );
    if (XtIsRealized(w) && !w->core.being_destroyed)
    {
	XtCallCallbackList((Widget)w, w->videoDisplay.exposeCallback, NULL);
    }
    if (XtIsRealized(w) && !w->core.being_destroyed && w->videoDisplay.isActive)
    {
	if (!w->videoDisplay.alreadyRedisplayed)
	{
	    w->videoDisplay.alreadyRedisplayed = True;
	    UpdateDisplay(w, RenderDisplay);
	}
    }
}

static void Resize(w)
    XnvrVideoDisplayWidget w;
{
    int origin_x, origin_y, extent_x, extent_y;

    WhenDebugEnabled(
	NormalOutput("XnvrVideoDisplayWidget::Resize\n");
	NormalOutput("\t(%d,%d) %dx%d\n",
	    w->core.x, w->core.y, w->core.width, w->core.height);
    )
    origin_x = w->videoDisplay.dest_x;
    origin_y = w->videoDisplay.dest_y;
    extent_x = w->videoDisplay.dest_x + w->videoDisplay.dest_width;
    extent_y = w->videoDisplay.dest_y + w->videoDisplay.dest_height;

    if (extent_x < w->core.width)
    {
	w->videoDisplay.dest_x += w->core.width - extent_x;
    }
    if (origin_x > w->core.x)
    {
	w->videoDisplay.dest_x = w->core.x;
    }
    if (extent_y < w->core.height)
    {
	w->videoDisplay.dest_y += w->core.height - extent_y;
    }
    if (origin_y > w->core.y)
    {
	w->videoDisplay.dest_y = w->core.y;
    }
    w->videoDisplay.alreadyRedisplayed = False;
}

static Boolean SetValues (current, req, new)
    XnvrVideoDisplayWidget current;
    XnvrVideoDisplayWidget req;
    XnvrVideoDisplayWidget new;
{
    WhenDebugEnabled( NormalOutput("XnvrVideoDisplayWidget::SetValues\n") );

    /* Video control resources */

    /* keep these within the range */
    if (new->videoDisplay.hue.current < new->videoDisplay.hue.min) 
	new->videoDisplay.hue.current = new->videoDisplay.hue.min;
    if (new->videoDisplay.hue.current > new->videoDisplay.hue.count)
	new->videoDisplay.hue.current = new->videoDisplay.hue.count;
    if (new->videoDisplay.sat.current < new->videoDisplay.sat.min) 
	new->videoDisplay.sat.current = new->videoDisplay.sat.min;
    if (new->videoDisplay.sat.current > new->videoDisplay.sat.count)
	new->videoDisplay.sat.current = new->videoDisplay.sat.count;
    if (new->videoDisplay.contrast.current < new->videoDisplay.contrast.min) 
	new->videoDisplay.contrast.current = new->videoDisplay.contrast.min;
    if (new->videoDisplay.contrast.current > new->videoDisplay.contrast.count)
	new->videoDisplay.contrast.current = new->videoDisplay.contrast.count;
    if (new->videoDisplay.bright.current < new->videoDisplay.bright.min) 
	new->videoDisplay.bright.current = new->videoDisplay.bright.min;
    if (new->videoDisplay.bright.current > new->videoDisplay.bright.count)
	new->videoDisplay.bright.current = new->videoDisplay.bright.count;

    return(False);
}



/* The renderProc argument is added only to pass it to UpdateDisplay */
static void PanDisplay (w, event, renderProc)
    XnvrVideoDisplayWidget w;
    XEvent *event;
    XtWidgetProc renderProc;
{
    Display *dpy;
    Window root, window, child;
    unsigned int mask;
    int px, py;		/* x,y of pointer location (relative to this window) */
    int cx, cy;		/* amount of x,y movement of the pointer */
    int wx, wy;		/* x,y relative to this widget's window origin */
    int rx, ry;		/* x,y relative to the root window origin */
    GC gc =  w->videoDisplay.renderGC;
    int origin_x, origin_y, extent_w, extent_h;

    WhenDebugEnabled( NormalOutput("XnvrVideoDisplayWidget::PanDisplay\n") );

    dpy = XtDisplay(w);
    window = XtWindow(w);
    px = event->xbutton.x;
    py = event->xbutton.y;
    while (1)
    {
	XQueryPointer(dpy,window,&root,&child,&rx,&ry, &wx,&wy,&mask);
	if (!(mask & Button1Mask)) break;
	if (w->videoDisplay.resizeMode != XnvrResizeModeClipAndPan) break;

	/* This is the amount of movement requested (i.e. the amount
	 * of x,y movement of the pointer.
	 */
	cx = wx - px;
	cy = wy - py;

	/* Restrict the movement such that there is never blank space
	 * around the image (i.e. grant only a reduced amount of actual
	 * movement).
	 */
	origin_x = w->videoDisplay.src_x + cx;
	origin_y = w->videoDisplay.src_y + cy;
	extent_w = w->videoDisplay.src_x + cx + w->videoDisplay.src_width;
	extent_h = w->videoDisplay.src_y + cy + w->videoDisplay.src_height;
	if (w->videoDisplay.src_width > w->core.width)
	{
	    if (extent_w < w->core.width) cx += w->core.width - extent_w;
	    if (origin_x > w->core.x) cx -= origin_x - w->core.x;
	}
	else
	{
	    if (extent_w > w->core.width) cx -= extent_w - w->core.width;
	    if (origin_x < w->core.x) cx += w->core.x - origin_x;
	}
	if (w->videoDisplay.src_height > w->core.height)
	{
	    if (extent_h < w->core.height) cy += w->core.height - extent_h;
	    if (origin_y > w->core.y) cy -= origin_y - w->core.y;
	}
	else
	{
	    if (extent_h > w->core.height) cy -= extent_h - w->core.height;
	    if (origin_y < w->core.y) cy += w->core.y - origin_y;
	}

	if (cx || cy || !w->videoDisplay.isActive)
	{
	    w->videoDisplay.dest_x = w->videoDisplay.src_x + cx;
	    w->videoDisplay.dest_y = w->videoDisplay.src_y + cy;
	    w->videoDisplay.dest_width = w->videoDisplay.src_width;
	    w->videoDisplay.dest_height = w->videoDisplay.src_height;
	    UpdateDisplay(w, renderProc);
	}

	/* Now erase any areas where the image was but ain't no more. */
	if (cx > 0)
	{
	    XFillRectangle(dpy, window, gc,
			w->videoDisplay.src_x, w->videoDisplay.src_y,
			cx, w->videoDisplay.src_height);
	}
	else if (cx < 0)
	{
	    XFillRectangle(dpy, window, gc,
		    w->videoDisplay.src_width + w->videoDisplay.src_x + cx-1,
		    w->videoDisplay.src_y,
		    -cx+2, w->videoDisplay.src_height);
	}
	if (cy > 0)
	{
	    XFillRectangle(dpy, window, gc,
		    w->videoDisplay.src_x, w->videoDisplay.src_y,
		    w->videoDisplay.src_width, cy);
	}
	else if (cy < 0)
	{
	    XFillRectangle(dpy, window, gc,
		    w->videoDisplay.src_x,
		    w->videoDisplay.src_height + w->videoDisplay.src_y + cy-1,
		    w->videoDisplay.src_width, -cy+1);
	}

	w->videoDisplay.src_x += cx;
	w->videoDisplay.src_y += cy;
	px = wx;
	py = wy;
    }
}


/* The renderProc argument has been added to allow the superclass to invoke
 * a procedure implemented in the subclass.  I tried to get to the class
 * structure of the widget, w, to invoke the RenderDisplay routine stored
 * in the class structure but couldn't get it to work (I could get the
 * class pointer, using XtClass for example, but it pointed to the Core
 * class and not the real class of w <or so it seemed>).
 */
static void UpdateDisplay (w, renderProc)
    XnvrVideoDisplayWidget w;
    XtWidgetProc renderProc;
{
    float width, height, whRatio;
    int base;
    Position newX, newY;
    Dimension newWidth, newHeight;
    XSetWindowAttributes attributes;
    Boolean windowWasResized;
    Display *dpy;
    Window win;

    WhenDebugEnabled( NormalOutput("XnvrVideoDisplayWidget::UpdateDisplay\n") );

    /* background_pixmap futzing--
     * This odd hackery is to cure the problem of the intrinsics automatically
     * clearing the display whenever SetValues returns True (i.e. generating
     * and expose event to cause Redisplay to be called to refresh the
     * window).	 Setting to "None" tells the server not to bother clearing due
     * to there being no background pixmap.
     */
    attributes.background_pixmap = None;
    XChangeWindowAttributes(XtDisplay(w), XtWindow(w), CWBackPixmap,
			    &attributes);

    windowWasResized = False;

    if (w->videoDisplay.resizeMode == XnvrResizeModeClipAndPan)
    {
	/* RenderDisplay(w); */
	(renderProc)((Widget)w);
    }
    else if (w->videoDisplay.resizeMode == XnvrResizeModeScale4to3)
    {
	/* restrict the display to the largest region within the display
	 * space that is of width to height ratio 4:3
	 */
	width = w->core.width;		/* coerce to float */
	height = w->core.height;	/* coerce to float */
	whRatio = width / height;
	if (whRatio > 1.33)
	{   /* window is too wide... base new size on max height */
	    base = (w->core.height - (w->core.height % 3)) / 3;
	}
	else
	{   /* window is too tall... base new size on max width */
	    base = (w->core.width - (w->core.width % 4)) / 4;
	}
	newWidth = base * 4;
	newHeight = base * 3;
	newX = /* w->core.x + */ (w->core.width - newWidth) / 2;
	newY = /* w->core.y + */ (w->core.height - newHeight) / 2;

	/* if this routine is being called because the window was resized,
	 * or the mode was changed, set up the new display location,
	 * stop the current render operation, clear the window (as this mode
	 * may not display over the entire display space), and start up
	 * the Render with the new parameters.
	 */
	if (newWidth != w->videoDisplay.dest_width ||
	    newHeight != w->videoDisplay.dest_height ||
	    newX != w->videoDisplay.dest_x ||
	    newY != w->videoDisplay.dest_y)
	{
	    w->videoDisplay.dest_x = newX;
	    w->videoDisplay.dest_y = newY;
	    w->videoDisplay.dest_width = newWidth;
	    w->videoDisplay.dest_height = newHeight;
	    StopDisplay(w);
	    windowWasResized = True;
	}
	/* RenderDisplay(w); */
	(renderProc)((Widget)w);
	if (windowWasResized)
	{
	    dpy = XtDisplay(w);
	    win = XtWindow(w);
	    attributes.background_pixmap = ParentRelative;
	    XChangeWindowAttributes(XtDisplay(w), XtWindow(w), CWBackPixmap,
				&attributes);
	    /* erase the rectangles that aren't covered by the image */
	    if (newY > 0)
	    {	/* erase a rectangle at the top of the window */
		XClearArea(dpy, win,
			0, 0,
			w->core.width, newY,
			False);
	    }
	    if (newX > w->core.x)
	    {	/* erase a rectangle to the left of the window */
		XClearArea(dpy, win,
			0, newY,
			0, newY + newHeight,
			False);
	    }
	    if (newX + newWidth < w->core.width)
	    {	/* erase a rectangle to the right of the window */
		XClearArea(dpy, win,
			newX + newWidth, newY,
			w->core.width, newY + newHeight,
			False);
	    }
	    if (newY + newHeight < w->core.height)
	    {	/* erase a rectangle at the bottom of the window */
		XClearArea(dpy, win,
			0, newY + newHeight,
			w->core.width, w->core.height,
			False);
	    }
	    attributes.background_pixmap = None;
	    XChangeWindowAttributes(XtDisplay(w), XtWindow(w), CWBackPixmap,
				    &attributes);
	}
    }
    else  /* assume XnvrResizeModeScale */
    {
	if (w->core.width > w->videoDisplay.src_width)
	{
	    w->videoDisplay.dest_x =
		w->core.x + (w->core.width - w->videoDisplay.src_width) / 2;
	    w->videoDisplay.dest_width = w->videoDisplay.src_width;
	}
	else
	{
	    /* this is a symptom assault (the problem remains).
	     * the core x and y are getting changed... and I don't know why.
	     * it appears to be after Initialize and unrelated to SetValues.
	     */
	    /* x = w->core.x; */
	    w->videoDisplay.dest_x = 0;
	    w->videoDisplay.dest_width = w->core.width;
	}
	if (w->core.height > w->videoDisplay.src_height)
	{
	    w->videoDisplay.dest_y =
		w->core.y + (w->core.height - w->videoDisplay.src_height) / 2;
	    w->videoDisplay.dest_height = w->videoDisplay.src_height;
	}
	else
	{
	    /* See above symptom assault note */
	    /* y = w->core.y; */
	    w->videoDisplay.dest_y = 0;
	    w->videoDisplay.dest_height = w->core.height;
	}
	/* RenderDisplay(w); */
	(renderProc)((Widget)w);
    }
}


static void RenderDisplay(w)
    XnvrVideoDisplayWidget w;
{
    Msg("XnvrVideoDisplayWidget::RenderDisplay\n");
    Msg("	This routine MUST be implemented by the subclass!!\n");
}


static void StopDisplay (w)
    XnvrVideoDisplayWidget w;
{
    WhenDebugEnabled( NormalOutput("XnvrVideoDisplayWidget::StopDisplay\n") );
}

#if 0
static void ClearDisplay (w)
    XnvrVideoDisplayWidget w;
{
    XSetWindowAttributes attributes;
    WhenDebugEnabled(
	NormalOutput("XnvrVideoDisplayWidget::ClearDisplay\n") );
    attributes.background_pixmap = ParentRelative;
    XChangeWindowAttributes(XtDisplay(w), XtWindow(w), CWBackPixmap,
			    &attributes);
    XClearArea (XtDisplay(w), XtWindow(w),
		w->core.x, w->core.y, w->core.width, w->core.height, False);
}

static XtGeometryResult QueryGeometry (w, intended, preferred)
    XnvrVideoDisplayWidget w;
    XtWidgetGeometry *intended;
    XtWidgetGeometry *preferred;
{
    WhenDebugEnabled(
	NormalOutput("XnvrVideoDisplayWidget::QueryGeometry intended %dx%d\n",
	    intended->width, intended->height);
    )
    return(XtGeometryYes);
}
#endif

