// $Header: /usr/local/src/nvrToMIT/RCS/cbrd.C,v 1.1 1994/05/10 23:27:10 russ Exp $

/*
 * $Id: cbrd.C,v 1.1 1994/05/10 23:27:10 russ Exp $
 *
 * cbrd.C
 *
 * Copyright Pencom Software, 1994
 */

#include <Cout.h>
#include <IFhand.h>
#include <SchedProd.h>
#include <SchedCons.h>
#include <Feed.h>
#include <ulocks.h>

const int MaxFeeds = 16;

static Logger log( "cbrd" );
	
static Feed* feed = NULL;
static Cout* new_cout = NULL;
static IFhand* if_hand = NULL;
static SchedProd* prod = NULL;
static SchedCons* cons = NULL;

static int nfeeds = 0;
static int max_speed = 0;
static char* media_dir = NULL;

void
cleanup( int signo, ... )
{
	if( signo != -1 )
	{	cout << log << "trapped signal " << signo << ", exiting..."
		     << endl;
	}
	
	if( signo == SIGHUP )
	{	cout << log << "Dumping feed information..." << endl;
		dump_feeds( "/tmp/cbrd.dump", feed, nfeeds );
		return;
	}

	delete cons;
	delete prod;
	delete if_hand;

	sleep( 1 );	// let the consumer and producer die

	cout << log << "bye, bye" << endl;

	delete feed;
	delete new_cout;

	exit( 1 );
}

void
usage( char* msg = NULL )
{	if( msg ) cout << msg << endl;
	cout << "usage: cbrd -f <max_feeds> -m <media_dir> -s <max_speed> " 
	     << "-d <zones>" << endl
	     << "  where arguments are:" << endl
	     << "  -d <zones>      Bit vector of debug zones." << endl
	     << "  -f <max_feeds>  Maximun number of feeds to handle" << endl
	     << "  -m <media_dir>  Location of media files" << endl
	     << "  -s <max_speed>  Maximun CBR speed in Kbytes/sec" << endl
	     << endl;
	max_speed = 0;
}

void
parse_args( int ac, char* av[] )
{
	cbrDebugZones = CBR_DEBUG_ALL_1 | CBR_DEBUG_RING_2;

	int c;
	while( ( c = getopt( ac, av, "d:f:m:s:" ) ) != -1 )
	{
		switch( c )
		{
		case 'd':
			cbrDebugZones = atoi( optarg );
			break;
		case 'f':
			nfeeds = atoi( optarg );
			break;
		case 'm':
			media_dir = optarg;
			break;
		case 's':
			max_speed = atoi( optarg ) * 1024; // conv to bytes/sec
			break;
		case '?':
			usage();
		}
	}
	if( max_speed <= 0 )
	{	usage( "-s <max_speed> should be > 0" );
	}
	if( nfeeds <= 0 )
	{	usage( "-f <max_feeds> should be > 0" );
	}
	else
	if( nfeeds > MaxFeeds )
	{	usage( "-f <max_feeds> should be <= MaxFeeds(16)" );
	}
	if( !media_dir )
	{	usage( "-m <media_dir> must be specified" );
	}	
	if( access( av[0], F_OK ) == -1 )
	{	usage( "argv[0] must be a full pathname" );
	}	
}

int
main( int argc, char* argv[] )
{
	signal( SIGHUP, cleanup );
	signal( SIGINT, cleanup );
	signal( SIGQUIT, cleanup );
	signal( SIGSEGV, cleanup );
	signal( SIGTERM, cleanup );
/*
 * Keep this line before any output to cout or cerr, so that all output will go 
 * to the console instead of being sent back through the TCP socket
 */
	new_cout = new Cout( "/dev/console" );

	cout << log << "argv[0]=" << argv[0] << endl;
 
	parse_args( argc, argv );

	feed = new Feed[nfeeds];
	if( !feed )
	{	cout << log << "out of memory" << endl;
		return 0;
	}

	if( usconfig( CONF_INITUSERS, MaxFeeds*2+1 ) < 0 )
	{	cout << log << "usconfig() failed, errno=" << ::errno << endl;
		return 0;
	}
	
	cout << log << "Started..." << endl
	     << log << "Starting the producer and consumer..." << endl;

	if_hand = new IFhand( media_dir, argv[0], max_speed, feed, nfeeds, log);

	perr_t perr = if_hand->run();
	if( perr != pNoErr )
	{	cout << "ERROR: if_hand->run() (perr=" << perr << ")" << endl;
	}

	cleanup( -1 );

}


