// $Header: /usr/local/src/nvrToMIT/RCS/CBRclient.C,v 1.2 1994/05/10 19:32:31 russ Exp $

/*
 * $Id: CBRclient.C,v 1.2 1994/05/10 19:32:31 russ Exp $
 *
 * CBRclient.C
 *
 * Copyright Pencom Software, 1994
 */

#include <CBRclient.h>
#include <Stopwatch.h>
#include <Feed.h>
#include <cbr.h>

static Logger log( "CBRclient" );

CBRclient::
CBRclient( char* server )
 : vtd( server ), my_addr_len( sizeof( my_addr ) )
{
	cd = new Cdeck[MaxDecks];
	if( !cd )
	{	cout << log << "Unable to create Cdeck array" << endl;
		exit( 1 );
	}
/*
 * make sure the VTD object was created correctly
 */	
	if( vtd.get_cn_state() == VTD::not_connected )
	{	cout << log << "Unable to connect to the server." << endl;
		exit( 1 );
	}
/*
 * Determine who I am
 */
	my_sockfd = vtd.get_sockfd();
	int rc = getsockname( my_sockfd, (sockaddr*)&my_addr, 
		&my_addr_len );
	if( rc < 0 )
	{	cout << log  << "Unable to determine who I am." << endl;
		exit( 1 );
	}
	else
	{	cout << log << "is " << inet_ntoa( my_addr.sin_addr )
			    << "[" << my_addr.sin_port << "]" << endl;
	}

}

CBRclient::
~CBRclient()
{	Logger log( "CBRclient::~CBRclient()" );
	delete[] cd;
}

tdhand_t
CBRclient::
open( char* filename, size_t netbufsz, speed_t rate )
{	
/*
 * An open gets translated into a load(), which returns a tdhand_t
 */
	tdhand_t tdh;
	err_t rc = vtd.load( filename, netbufsz, &tdh );
	if( rc == 0 )
	{	cd[tdh].speed = rate;
	}
	else
	{	::errno = (int)rc;
		return -1;
	}
/*
 * start using the new tape deck
 */
	if( vtd.swtch( tdh ) != VS_ER_OK )
	{	::errno = EINVAL;
		return -1;
	}
/*
 * reset info
 */
	cd[tdh].first = TRUE;
	cd[tdh].num_reads = 0;
	cd[tdh].eom = 0;
	cd[tdh].total_reads.set( 0 , 0 );
	cd[tdh].min_read.set( MaxSecond, MaxMicrosecond );
	cd[tdh].max_read.set( 0 , 0 );

	return tdh;
}

int
CBRclient::
read( tdhand_t tdh, void* buffer, size_t size )
{
	Logger log( "CBRclient::read()" );
#if 0
	cout << "size=" << size << endl;
#endif
/*
 * start using the specified tape deck
 */
	if( vtd.swtch( tdh ) != VS_ER_OK )
	{	::errno = EINVAL;
		return -1;
	}
/*
 * The very frist time read() is called, a Ring needs to be allocated and
 * a CBRprod object created.  Then a play is communicated to the server to
 * start the CBR stream flowing.  These resources are freed when the tdhand_t
 * is closed.
 */
	if( !cd[tdh].ring )
	{	cd[tdh].ring = alloc_ring( cd[tdh].speed, vtd.get_td_netbufsz(),
					   vtd.get_td_sock(), 1);
		cd[tdh].prod = new CBRprod( cd[tdh].ring, vtd.get_td_sock(), 
			my_sockfd, cd[tdh].speed, cd[tdh].eom );
		if( !cd[tdh].ring || !cd[tdh].prod )
		{	::errno = ENOMEM;
			return -1;
		}
		int rc = vtd.play( cd[tdh].speed );
		if( rc != VS_ER_OK )
		{	::errno = EINVAL;
			return -1;
		}
	}
/*
 * if we're paused, restart the feed flowing
 */
	if( vtd.get_td_state() == VTD::paused )
	{	int rc = vtd.play( cd[tdh].speed );
		if( rc != VS_ER_OK )
		{	::errno = EINVAL;
			return -1;
		}
	}
/*
 * If this is the first read after and open() or seek(), wait until the Ring
 * becomes half full, or EOM, which ever comes first.  This keeps the user
 * calling this read() from starving on future reads.
 */
	if( cd[tdh].first )
	{	
#ifdef TCPEOM
		while( !vtd.eom() && 
#else
		while( !cd[tdh].eom && 
#endif
		       cd[tdh].ring->num_full() < cd[tdh].ring->max_full()>>2 )
		{	// wait for eom or the ring to half fill
		}
	}

	cd[tdh].read_stat.start();
	bool_t some = FALSE;
	char* tp = (char*)buffer;
	size_t total = 0;
	bool_t starving = FALSE;
	Stopwatch sw;
	Time timeout( 10, 0 );	// 10 second starvation timeout

	while( size > 0 )
	{
	/*
	 * get a new buffer if we haven't used up the current
	 */
		if( cd[tdh].sz == 0 )
		{	cd[tdh].id = cd[tdh].ring->get_full( &cd[tdh].buf, 
							     &cd[tdh].sz );
		/*
		 * if we didn't get a buffer, check for eom or starvation
		 */
			if( cd[tdh].id == -1 )
#ifdef TCPEOM
			{	if( vtd.eom() )
#else
			{	if( cd[tdh].eom )
#endif
				{	size = 0;
				}
				else
				if( starving )
				{	Time dif = sw.diff();
					if( dif > timeout )
					{	cout << log 
						     << "Consumer starved "
						     << "longer than "
						     << timeout.seconds() 
						     << " seconds.  "
						     << "Assume eom." << endl;
						size = 0;
						total = 0;
					}
				}
				else 
				if( !cd[tdh].first )
				{	cout << log << "Consumer starved!!!"
					     << endl;
					sw.start();
					starving = TRUE;
				}
				continue;
			}
		}
	/*
	 * must have a buffer to use
 	 *
	 * if we were starving, meaning the Ring was completely full for 
	 * some duration, print out the amount of starvation time.
	 */
		if( starving )
		{	sw.stop();
			cout << log << "Consumer starved for " << sw.diff() 
			     << endl;
			starving = FALSE;
		}
		some = TRUE;
	/*
	 * use the contents of the buffer
	 */
		if( size < cd[tdh].sz )
		{	// got more data from the Ring than needed
			memcpy( tp, cd[tdh].buf, size );
			cd[tdh].sz -= size;
			cd[tdh].buf += size;	// start from here next time
			total += size;
			size = 0;
		}
		else
		{	// use up rest of data from Ring and ask for more
			memcpy( tp, cd[tdh].buf, cd[tdh].sz );
			tp += cd[tdh].sz;	// continue to fill
			size -= cd[tdh].sz;	// with more data from Ring
			total += cd[tdh].sz;
			cd[tdh].sz = 0;
			cd[tdh].ring->now_empty( cd[tdh].id );
		}
	}

	if( starving )	// only if eom encountered while starving
	{	sw.stop();
		cout << log << "Consumer starved for " << sw.diff() << endl;
		starving = FALSE;
	}
/*
 * compute some statistics
 */
	cd[tdh].read_stat.stop();
	Time dif = cd[tdh].read_stat.diff();

	if( cd[tdh].first )
	{	cd[tdh].first = FALSE;
		cd[tdh].first_read = dif;
	}

	cd[tdh].total_reads += dif;
	cd[tdh].num_reads++;
	if( cd[tdh].min_read > dif )
	{	cd[tdh].min_read = dif;
	}
	if( cd[tdh].max_read < dif )
	{	cd[tdh].max_read = dif;
	}
	
	return total;	
}

int
CBRclient::
preview( tdhand_t tdh, void* buffer, size_t* size, offset_t offset )
{
	if( vtd.swtch( tdh ) != VS_ER_OK )
	{	::errno = EINVAL;
		return -1;
	}
	return vtd.preview( buffer, size, offset );
}

int
CBRclient::
catalog( char** buffer, size_t* size )
{
	return vtd.catalog( buffer, size );
}

int 
CBRclient::
seek( tdhand_t tdh, long offset )
{
	if( vtd.swtch( tdh ) != VS_ER_OK )
	{	::errno = EINVAL;
		return -1;
	}
	
/*
 * if EOM, reset the eom flag because more data will be streamed, and
 * reset the first flag to treat the next read() as the first read, because
 * the Ring is most likely empty and needs to be reprimed
 */
#ifdef TCPEOM
	if( vtd.eom() )
#else
	if( cd[tdh].eom )
#endif
	{	cd[tdh].first = TRUE;
		cd[tdh].eom = FALSE;
	}
	err_t rc = vtd.seek( offset );
	switch( rc )
	{
	case VS_ER_OK:
		return 0;
	case VS_ER_NOTAPE:
		::errno = EINVAL;
		return -1;
	default:
		::errno = EINVAL;
		return -1;
	}

}

int 
CBRclient::
pause( tdhand_t tdh )
{
	if( vtd.swtch( tdh ) != VS_ER_OK )
	{	::errno = EINVAL;
		return -1;
	}
	err_t rc = vtd.pause();
	switch( rc )
	{
	case VS_ER_OK:
		return 0;
	case VS_ER_NOTAPE:
		::errno = EINVAL;
		return -1;
	default:
		::errno = EINVAL;
		return -1;
	}

}

int 
CBRclient::
close( tdhand_t tdh )
{
	if( vtd.swtch( tdh ) != VS_ER_OK )
	{	::errno = EINVAL;
		return -1;
	}

	delete cd[tdh].ring;
	delete cd[tdh].prod;

	cd[tdh].ring = NULL;
	cd[tdh].prod = NULL;

	err_t rc = vtd.eject();
	switch( rc )
	{
	case VS_ER_OK:
		return 0;
	case VS_ER_NOTAPE:
		::errno = EINVAL;
		return -1;
	default:
		return -1;
	}

}

/*****************************************************************************/

/*
 * C interface into a CBR client object
 */

static CBRclient* cbr = NULL;

CBRclient*
cbr_object()
{	return cbr;
}

void
sig_handler( int sig, ... )
{       
#if 0
    	fprintf(stderr, "client: trapped signal %d; exiting...\n", sig );
#endif /* 0 */
	delete cbr;
        exit( 1 );
}

void
cbr_init( char* server )
{
	signal( SIGHUP, SIG_IGN );
	signal( SIGINT, sig_handler );
	signal( SIGQUIT, sig_handler );
	signal( SIGTERM, sig_handler );

	cbr = new CBRclient( server );
}

tdhand_t
cbr_open( char* filename, size_t netbufsz, speed_t rate )
{	if( cbr == NULL )
	{	::errno = EINVAL;
		return -1;
	}
	return cbr->open( filename, netbufsz, rate );
}

int
cbr_preview( tdhand_t tdh, void* buffer, size_t* size, off_t offset )
{	if( cbr == NULL )
	{	::errno = EINVAL;
		return -1;
	}
	return cbr->preview( tdh, buffer, size, offset );
}

int
cbr_catalog( char** buffer, size_t* size )
{	if( cbr == NULL )
	{	::errno = EINVAL;
		return -1;
	}
	return cbr->catalog( buffer, size );
}

int
cbr_read( tdhand_t tdh, void* buffer, size_t size )
{	if( cbr == NULL )
	{	::errno = EINVAL;
		return -1;
	}
	return cbr->read( tdh, buffer, size );
}

int
cbr_seek( tdhand_t tdh, long offset )
{	if( cbr == NULL )
	{	::errno = EINVAL;
		return -1;
	}
	return cbr->seek( tdh, offset );
}

int
cbr_pause( tdhand_t tdh )
{	if( cbr == NULL )
	{	::errno = EINVAL;
		return -1;
	}
	return cbr->pause( tdh );
}

int
cbr_close( tdhand_t tdh )
{	if( cbr == NULL )
	{	::errno = EINVAL;
		return -1;
	}
	return cbr->close( tdh );
}

void
cbr_done()
{	delete cbr;
	cbr = NULL;
}

