// $Header: /usr/local/src/nvrToMIT/RCS/CBRprod.C,v 1.2 1994/05/10 19:32:31 russ Exp $

/*
 * $Id: CBRprod.C,v 1.2 1994/05/10 19:32:31 russ Exp $
 *
 * CBRprod.C
 *
 * Copyright Pencom Software, 1994
 */

#include <CBRprod.h>
#include <vtd.h>
#include <NetHead.h>
#include <VTDprot.h>

static Logger log( "CBRprod" );

CBRprod::
CBRprod( Ring* r, int fd, int ctrl_fd, speed_t& speed, bool_t& eom )
 : ring( r ), fd( fd ), total( 0 ), ctrl_fd( ctrl_fd ), eom( eom ), seq( 0 ), 
   speed( speed )
{
	mid = ring->max_full() / 2; 	// half full
	hwm = ring->max_full() - 2;	// high water mark
	lwm = ring->max_full() / 3;	// low water mark
	need = 0;

	pid = sproc( CBRprod_entry, PR_SALL, this );
	if( pid == -1 )
	{	perror( "sproc()" );
	}
	assert( pid != -1 );

}

CBRprod::
~CBRprod()
{
	if( pid != -1 )
	{	kill( pid, SIGTERM );
	}
}

void
CBRprod_entry( void* arg )
{	
	CBRprod* p = (CBRprod*)arg;
	assert( p );
	p->run();
}

void
CBRprod::
run()
{
	cout << log << "started" << endl;

	double total_time = 0.0;
	fd_set fdvar;
	timeval tv;

	FD_ZERO( &fdvar );
	tv.tv_sec = 5;
	tv.tv_usec = 0;

	sockaddr_in l_addr;
	int len = sizeof( l_addr );
        getsockname( fd, (sockaddr*)&l_addr, &len );
        cout << log << "from " << inet_ntoa( l_addr.sin_addr )
	     << "[" << l_addr.sin_port << "]..." << endl;
	port = ntohs( l_addr.sin_port );
        getpeername( ctrl_fd, (sockaddr*)&l_addr, &len );
        cout << log << "flows sent to " << inet_ntoa( l_addr.sin_addr)
	     << "[" << l_addr.sin_port << "]..." << endl;

/*
 * first flow message should almost fill the buffer to the high water mark.
 * filling to the hwm cause unnecessary flow control (ping ponging).
 */
	need = hwm - 2;
	send_flow();

	for(;;)
	{
	/*
	 * do flow control, and get a buffer to fill from the Ring
	 */
		bool_t once = TRUE;
		do
		{
			count_t nfb = ring->num_full();
			if( !need && nfb >= hwm ) // slow down
			{	need = -mid;
				send_flow();
			}
			else
			if( !need && nfb <= lwm ) // speed up
			{	need = hwm - nfb - 2;
				send_flow();
			}

			if( need > 0 ) need--;	// idle back to need == 0
			else
			if( need < 0 ) need++;

			id = ring->get_empty( &ptr, &size );
			if( id == -1 )
			{	delay();	// don't let it run wild
			}
			if( getppid() == 1 )
			{	// owned by inetd, parent must have died
				return;
			}
		}
		while( id == -1 );
	/*
	 * wait for available information on the UDP socket
	 */
		do
		{	FD_SET( fd, &fdvar );
			int rc = select( fd+1, &fdvar, NULL, NULL, &tv );
			if( rc == -1 )
			{	perror( "select()" );
				cout << log << "bye, bye" << endl;
				return;
			}
			if( getppid() == 1 )
			{	// owned by inetd, parent must have died
				return;
			}
		} while( !FD_ISSET( fd, &fdvar ) );
	/*
	 * to get here, we must have a buffer and some data ready, so let's
	 * read the data and process it.
	 */
		int rc = produce( ptr, size );
		if( rc == -1 )
		{	::perror( "produce():" );
			return;
		}
		else
		if( rc == 0 )
		{	ring->back_to_empty( id );
		}
		else	
		{	ring->now_full( id, rc );
		}
		total += rc;
	}
}

size_t
CBRprod::
produce( void* buffer, size_t size )
{
	Logger log( "CBRprod::produce()" );

	NetHead* rnh;
	static bool_t once = TRUE;
	static count_t cnt = 0;
#if 0
/*
 * read the NetHead 
 */
	int rc = recvn( fd, rnh(), rnh.size(), 0 );
	if( rc != rnh.size() )
	{	cout << log << "rc=" << rc << " != rnh.size()=" 
			<< rnh.size() << endl;
	}
	if( rc == -1 )
	{	return rc;
	}
/*
 * read the UDP packet
 */
	rc = recvn( fd, buffer, size, 0 );
	if( rc != size )
	{	cout << log << "rc=" << rc << " != size=" << size 
			<< endl;
	}
	if( rc == -1 )
	{	return rc;
	}
#else
	static char lbuf[MaxUDP];
	int rc = recvn( fd, lbuf, size + sizeof( NetHead ), 0 );
	if( rc != size + sizeof( NetHead ) )
	{	cout << log << "rc=" << rc << " != size=" 
		     << size + sizeof( NetHead ) << endl;
	}
	if( rc == -1 )
	{	return rc;
	}
	rnh = (NetHead*)lbuf;
	memcpy( buffer, &lbuf[sizeof( NetHead )], size );

#endif
/*
 * verify the check sum of the NetHead + Data
 */
	ulong read_csum = rnh->get_checksum();
	rnh->set_checksum( 0 );
	ulong csum = comp_checksum( rnh, sizeof( NetHead ) );
	csum += comp_checksum( buffer, (size_t)rnh->get_size() );
	if( csum != read_csum )
	{	cout << log << "read_csum=" << read_csum << ",csum=" << csum 
		     << endl << log << "corrupt data..." << endl;
		send_retry();
		once = FALSE;
		return 0;
	}
/*
 * validate the NetHead stuff, note EOM if encountered, and throw out 
 * unknow typed packets
 */
	switch( rnh->get_type() )
	{
	case NetHead::Data:
		break;
	case NetHead::EOM:
		eom = TRUE;
		need = 0;
		cout << log << "eom encountered." << endl;
		break;
	default:
		cout << log << "skipping unknown NetHead type (type="
		     << rnh->get_type() << ",rc=" << rc << ")" << endl;
		size_t i;
		char *p = (char*)rnh;
		for( i = 0; i < rnh->get_size(); i++ )
		{	
#if 0
		    	if( i && i % 16 == 0 ) fprintf(stderr, "\n" );
			fprintf(stderr, "%2x ", p[i] );
#endif /* 0 */
		}
#if 0
		fprintf(stderr, "\n" );
#endif /* 0 */
		if( once )
		{	send_retry();
			once = FALSE;
		}
		return 0;
	}
#if 0
	// fake some packe loss for testing purposes
	static int next_skip = 10;
	if( rnh->get_seq() == next_skip )
	{	rnh++;
		next_skip += 10;
	}
#endif
/*
 * Check the UDP packet's sequence and ask for a retry if necessary
 */
	if( rnh->get_seq() < seq )
	{	// duplicate message, so discard it
		rc = 0;
		cout << log << "received duplicate message (got seq=" 
		     << rnh->get_seq() << ", wanted seq=" << seq << ")" 
		     << endl;
	}
	else
	if( rnh->get_seq() > seq )
	{	// missed a message, instruct server to retry packet
		rc = 0;
		cout << log << "missed message (got seq="
		     << rnh->get_seq() << ", wanted seq=" << seq << ")" 
		     << endl;
		if( once && cnt == 0 )
		{	send_retry();
			once = FALSE;
			cnt = 20;
		}
		else
		{	if( cnt )
			{	cnt--;
			}
			else
			{	once = TRUE;
			}
		}
	}
	else
	{	// proper sequence, wait for next one
		once = TRUE;
		cnt = 0;
		seq++;
		rc = (size_t)rnh->get_size();	
#if 0
		cout << log << "produced " << rc << " bytes (seq="
		     << rnh->get_seq() << ")" << endl;
#endif
        }

	return rc;
}

void
CBRprod::
send_retry()
{
	cout << log << "resetting server back to seq " << seq << endl;

	err_t rc = vsRetryBuffer( ctrl_fd, fd, seq );
	if( rc != VS_ER_OK )
	{	cout << log << "failed sending retry (errno="
		     << ::errno << ")" << endl;
	}
}

void
CBRprod::
delay()
{	
        double buf_rate = (double)speed / (double)ring->full_size();
        double interval = 1 / buf_rate;
	Time time_val = Time( interval );
#if 0
	cout << log << "delaying " << time_val << endl;
#endif
	timeval tv = time_val;
	int rc = select( 0, NULL, NULL, NULL, &tv );
	if( rc == -1 )
	{	perror( "delay(): select()" );
	}
}

void
CBRprod::
send_flow()
{
	if( eom )
	{	return;
	}

	err_t err = vsFlowControl( ctrl_fd, fd, need );
	if( err !=  VS_ER_OK )
	{	::perror( "vsFlowControl()" );
	}
	Time t;
	cout << log << "Sent flow( " << need << " ) at " << t.cur_time() 
	     << endl;
}

