// $Header: /usr/local/src/nvrToMIT/RCS/Feed.C,v 1.2 1994/12/17 12:58:00 russ Exp $

/*
 * $Id: Feed.C,v 1.2 1994/12/17 12:58:00 russ Exp $
 *
 * Feed.C
 *
 * Copyright Pencom Software, 1994
 */

#include <Feed.h>
#include <Time.h>

Feed::
Feed()
{	init();
}

Feed::
~Feed()
{

}

void
Feed::
init()
{
	state = idle;
	p_eom = c_eom = FALSE;
	speed = 0;
	netbufsz = ConsSize;
	next_p = next_c = 0;
	nbufs = 0;
	rseq = -1;
	rseq_off = 0;
	seq = 0;
	ring = NULL;
	ctrl_if = -1;
	source[0] = '\0';
	source_fd = -1;
	memset( (char*)&destin, 0, sizeof( destin ) );
	destin_fd = -1;
	memset( (char*)&l_addr, 0, sizeof( l_addr ) );
}

void
Feed::
reset()
{
	delete ring;

	init();
}

perr_t
Feed::
load( int ctrl_if, char* source, sockaddr_in destin, size_t netbufsz )
{	
	Logger log( "Feed::load()" );

	if( state != idle )
	{	::errno = 0;
		return pBusy;
	}

        int n = 1;
        int rc = setsockopt( ctrl_if, IPPROTO_TCP, TCP_NODELAY, (char*)&n,
                sizeof( n ) );
        if( rc < 0 )
        {       return pInvalCtrlIf;
        }
/*
 * open the source media file
 */
	source_fd = ::open( source, O_RDONLY );
	if( source_fd == -1 )
	{	return pNSfile;
	}
/* 
 * create/bind the local UDP socket 
 */
	destin_fd = socket( AF_INET, SOCK_DGRAM, 0 );
	if( destin_fd == -1 )
	{	return pInvalDestin;
	}

	rc = bind( destin_fd, (sockaddr*)&l_addr, sizeof( l_addr ) );
	if( rc == -1 )
	{	return pInvalDestin;
	}
/*
 * find out what port we are locally bound to
 */
	int len = sizeof( l_addr );
	rc = getsockname( destin_fd, (sockaddr*)&l_addr, &len );
	if( rc == -1 )
	{	return pInvalDestin;
	}

CBR_DEBUG( CBR_DEBUG_FEED_1,
        cout << log << "local  " << inet_ntoa( l_addr.sin_addr )
             << "[" << l_addr.sin_port << "]..." << endl;
        cout << log << "destin " << inet_ntoa( destin.sin_addr )
             << "[" << destin.sin_port << "]..." << endl;
);

/*
 * Connecting a UPD socket allows us to use recv() and send()
 */
	rc = connect( destin_fd, (sockaddr*)&destin, sizeof( destin ) );
	if( rc == -1 )
	{	return pInvalDestin;
	}

	this->ctrl_if = ctrl_if;
	strcpy( this->source, source );
	this->destin = destin;
	this->netbufsz = netbufsz;


	state = loaded;

	return pNoErr;
}

perr_t
Feed::
play( speed_t s )
{
	::errno = 0;

	switch( state )
	{
	case idle:
		return pIdle;
	case loaded:
		if( s == 0 )
		{	return pInval;
		}
		if( !ring )
		{	ring = alloc_ring( s, netbufsz, destin_fd, vsEtoFratio);
			if( !ring )
			{	return pNoMem;
			}
		}
		// break left out on purpose
	case paused:
		if( s )
		{	speed = s;
		}
		state = playing;
		break;
	case playing:
		if( s )
		{	speed = s;
		}
		break;
	default: 
		return pBadState;
	}

	return pNoErr;
}

perr_t
Feed::
seek( count_t offset )
{
	::errno = 0;

	if( offset < 0 )
	{	return pBadOff;
	}
	
	switch( state )
	{
	case idle:
		return pIdle;
	case loaded:
	case paused:
	case playing:
		next_p = offset;
		next_c = offset;
		rseq = -1;
		if( ring ) ring->clear();
		p_eom = FALSE;
		c_eom = FALSE;
		break;
	default: 
		return pBadState;
	}

	return pNoErr;
}

perr_t
Feed::
pause( count_t& offset )
{
	::errno = 0;

	switch( state )
	{
	case idle:
		return pIdle;
	case loaded:
	case paused:
	case playing:
		state = paused;
		offset = next_c;
		break;
	default: 
		return pBadState;
	}

	return pNoErr;
}

perr_t
Feed::
flow( count_t nbufs )
{
	::errno = 0;

	if( state == idle )
	{	return pIdle;
	}

	this->nbufs = nbufs;

	return pNoErr;
}

perr_t
Feed::
retry( count_t rseq, offset_t off )
{
	::errno = 0;

	if( state == idle )
	{	return pIdle;
	}

	if( rseq < 0 || rseq > MaxSequence )
	{	return pBadSeq;
	}
	if( off < 0 )
	{	return pBadOff;
	}

	this->rseq = rseq;
	rseq_off = off;

	return pNoErr;
}

perr_t
Feed::
eject()
{
	::errno = 0;
	reset();
	return pNoErr;
}

void
Feed::
lock()
{
}

void
Feed::
unlock()
{
}

ostream&
operator<<( ostream& o, Feed& f )
{
	o << "state     = " << f.state << endl
	  << "ctrl_if   = " << f.ctrl_if << endl
	  << "source    = " << f.source << endl
	  << "source_fd = " << f.source_fd << endl
	  << "destin    = " << inet_ntoa( f.destin.sin_addr )
          << "[" << f.destin.sin_port << "]" << endl
	  << "destin_fd = " << f.destin_fd << endl
	  << "l_addr    = " << inet_ntoa( f.l_addr.sin_addr )
          << "[" << f.l_addr.sin_port << "]" << endl
	  << "speed     = " << f.speed << endl
	  << "p_eom     = " << (short)f.p_eom << endl
	  << "c_eom     = " << (short)f.c_eom << endl
	  << "next_p    = " << f.next_p << endl
	  << "next_c    = " << f.next_c << endl
	  << "nbufs     = " << f.nbufs << endl
	  << "rseq      = " << f.rseq << endl
	  << "seq       = " << f.seq << endl
	  << "ring      = ";

	if( f.ring ) o << endl << *f.ring; else o << "NULL" << endl;

	return o;
}

static char* s = "############################################################"
		 "###################";

void
dump_feeds( char* file, Feed* f, count_t n )
{	Logger log( "dump_feeds()" );

	unlink( file );
	ofstream dmp( file );

	if( !dmp )
	{	cout <<log << "unable to open file (errno=" << ::errno << endl;
	}

	Time t;
	t.set();

	dmp << t.cur_time() << endl;

	for( count_t i = 0; i < n; i++ )
	{	dmp << s << endl
		    << "feed[" << i << "]:" << endl
		    << f[i];
	}

	dmp.close();

}

