// $Header: /usr/local/src/nvrToMIT/RCS/IFhand.C,v 1.2 1994/12/17 12:57:53 russ Exp $

/*
 * $Id: IFhand.C,v 1.2 1994/12/17 12:57:53 russ Exp $
 *
 * IFhand.C
 *
 * Copyright Pencom Software, 1994
 */

#include <IFhand.h>
#include <vtd.h>
#include <Cout.h>
#include <Time.h>
#include <Debug.h>

void
sigpipe_handler( int sig, ... )
{	Logger log( "sigpipe_handler()" );
	cout << log << "Received SIGPIPE(" << sig << "), ignoring it..."
	     << endl;
	
}

IFhand::
IFhand( char* media_dir, char* cntr_dir, speed_t max_speed, Feed* feed, 
	count_t nfeeds, Logger glog )
 : media_dir( media_dir ), cntr_dir( cntr_dir ), max_speed( max_speed ), 
   feed( feed ), nfeeds( nfeeds ), ifeed( NULL ), in_sockfd( 0 ), 
   cur_feed( -1 ), glog( glog ), cntr( NULL )
{
	Logger log( "IFhand::IFhand()" );

	signal( SIGPIPE, sigpipe_handler );
/*
 * find out who I am
 */
	int len = sizeof( l_ctrl );
	memset( (char*)&l_ctrl, 0, len );
	if( getsockname( in_sockfd, &l_ctrl, &len ) < 0 )
	{	cout << log << "Unable to determine who I am." << endl;
	}
/*
 * find out who's my client
 */
	len = sizeof( r_ctrl );
	memset( (char*)&r_ctrl, 0, len );
	if( getpeername( in_sockfd, &r_ctrl, &len ) < 0 )
	{	cout << log << "Unable to determine my client." << endl;
	}

	int v = 1;
	len = sizeof( v );
	if( setsockopt(in_sockfd,SOL_SOCKET,SO_KEEPALIVE,(char*)&v,len) < 0 )
	{	cout << log << "Unable to set SO_KEEPALIVE." << endl;
	}
	getsockopt( in_sockfd, SOL_SOCKET, SO_KEEPALIVE, (char*)&v, &len );

/*
 * create a Counter object to keep track of the number of feeds on the system
 */
	cntr = new Counter( cntr_dir, FALSE );

}

IFhand::
~IFhand()
{	Logger log( "IFhand::~IFhand()" );
	delete[] ifeed;
	delete cntr;
CBR_DEBUG( CBR_DEBUG_IFHAND_1,
	cout << log << "bye, bye" << endl;
);
}


perr_t
IFhand::
run()
{	Logger log( "IFhand::run()" );

	ifeed = new IFhandFeed[nfeeds];
/*
 * check the initial constructor parameters and inform the client
 * about our status
 */
	if( !ifeed )
	{	send_status( VS_ER_BADSERV, ENOMEM );
	}
	else
	if( max_speed <= 0 )
	{	send_status( VS_ER_BADSERV );
	}
	else
	{	send_status( VS_ER_OK );     // let client know I'm ready to go
	}

	timeval tv = Time( 5, 0 );
	fd_set fdvar;
	FD_ZERO( &fdvar );
/*
 * wait for input from the client and process it
 */
	for(;;)
	{
		FD_SET( in_sockfd, &fdvar );

		int rc = select( in_sockfd+1, &fdvar, NULL, NULL, &tv );

		if( rc == -1 )
		{	cout << log << "select() failed (errno="
				    << ::errno << ")" << endl;
			continue;
		}	

		int v = -1;
		int len = sizeof( v );
		getsockopt( in_sockfd, SOL_SOCKET, SO_ERROR, (char*)&v, &len );
		if( v )
		{	cout << log << "SO_ERROR=" << v << endl;
			break;
		}
		
		if( FD_ISSET( in_sockfd, &fdvar ) )
		{	int n = handle_input();
			if( n <= 0 )
			{	break;
			}
		}

	}
	
	for( count_t i = 0; i < nfeeds; i++ )
	{	feed[0].reset();
	}

	return pNoErr;
}

int
IFhand::
handle_input()
{	Logger log( "IFhand::handle_input()" );

	vtdprot_t vprot;
	int rc;

	int n = recvn( in_sockfd, &vprot, sizeof( vprot ), 0 );
	if( n < 0 )
	{	cout << log << "Unable to receive from socket." << endl;
	}
	if( n == 0 )
	{	return n;
	}


CBR_DEBUG( CBR_DEBUG_IFHAND_2, 
	char* buf = (char*)&vprot;
	char nm[32];
	for( int i = 0; i < 32; i++ )
	{
		sprintf( nm, "%.2x ", (int)buf[i] );
		cout << nm;
		if( (i + 1) % 16 == 0 ) cout << endl;

	}
);


	VTDprot cmd( vprot );

CBR_DEBUG( CBR_DEBUG_IFHAND_1, 
	if( cmd.get_type() != VS_PROT_RETRY ) 
		cout << log << "received " << n << " bytes" << endl;
);

/*
 * determine what type of message we're dealing with and execute it's 
 * handler function
 */

	if( cmd.get_type() != VS_PROT_CATALOG )
	{	cur_feed = map_port( cmd.get_port() );
	}

	switch( cmd.get_type() )
	{
	case VS_PROT_LOAD:
		rc = do_load( cmd );
		break;
	case VS_PROT_PREVIEW:
		rc = do_preview( cmd );
		break;
	case VS_PROT_CATALOG:
		rc = do_catalog( cmd );
		break;
	case VS_PROT_PLAY:
		rc = do_play( cmd );
		break;
	case VS_PROT_PAUSE:
		rc = do_pause( cmd );
		break;
	case VS_PROT_SEEK:
		rc = do_seek( cmd );
		break;
	case VS_PROT_FLOW:
		rc = do_flow( cmd );
		break;
	case VS_PROT_EJECT:
		rc = do_eject( cmd );
		break;
	case VS_PROT_RETRY:
		rc = do_retry( cmd );
		break;
	case VS_PROT_DUMP:
		rc = do_dump( cmd );
		break;
	default:
		cout << log << "unknown command" << endl;
		return n;
	}

	if( rc == -1 )
	{	return -1;
	}

	return n;
}

int 
IFhand::
do_load( VTDprot& cmd )
{
	Logger log( "IFhand::do_load()" );

CBR_DEBUG( CBR_DEBUG_IFHAND_1,
	cout << log << cmd;
);

	if( cur_feed != -1 )
	{	send_status( VS_ER_BUSY );
		return 0;
	}
/*
 * reserve a new feed
 */
	cur_feed = new_feed( cmd.get_port() );
	if( cur_feed == -1 || cntr->count() == nfeeds )
	{	cout << log << "No more feeds." << endl;
		send_status( VS_ER_OUT );
		return 0;
	}

	sockaddr_in destin;
	memset( (char*)&destin, 0, sizeof( destin ) );
	destin = r_ctrl;
	destin.sin_port = htons( (ushort)cmd.get_port() );
/*
 * Load the new feed with the given information
 */
	perr_t perr = feed[cur_feed].load( in_sockfd, cmd.get_file(), destin,
					   (size_t)cmd.get_netbufsz() );
	if( perr != pNoErr )
	{	cout << log << "error loading feed (perr=" << perr << ",errno="
		     << ::errno << ")" << endl;
		send_status( perr, ::errno );
		return 0;
	}
/*
 * start up a SchedProd object to read data from the disk
 */
	ifeed[cur_feed].p = new SchedProd( max_speed, &feed[cur_feed], 1,glog);
	if( !ifeed[cur_feed].p )
	{	cout << log << "error creating producer, errno="
		     << ::errno << ")" << endl;
		feed[cur_feed].eject();
		send_status( VS_ER_BADSERV );
	}
/*
 * start up a SchedCons to write data to the network
 */
	ifeed[cur_feed].c = new SchedCons( cntr_dir, max_speed, 
					   &feed[cur_feed], 1, glog );
	if( !ifeed[cur_feed].c )
	{	cout << log << "error creating consumer, errno="
		     << ::errno << ")" << endl;
		delete ifeed[cur_feed].p;
		feed[cur_feed].eject();
		send_status( VS_ER_BADSERV );
	}
		
	send_status( VS_ER_OK );

	return 0;
}

int 
IFhand::
do_preview( VTDprot& cmd )
{	Logger log( "IFhand::do_preview()" );

CBR_DEBUG( CBR_DEBUG_IFHAND_1,
	cout << log << cmd;
);

	if( cur_feed == -1 )
	{	cout << log << "no such unit" << endl;
		send_status( VS_ER_NSUNIT );
		return 0;
	}

	char* p = new char[cmd.get_preview_len()];
	if( !p )
	{	send_status( VS_ER_NOMEM, ::errno );
		return 0;
	}

	int fd = dup( feed[cur_feed].get_source_fd() );
	if( fd == -1 )
	{	send_status( VS_ER_NSUNIT, ::errno );
		delete[] p;
		return 0;
	}

	off_t off = lseek( fd, cmd.get_preview_offset(), SEEK_SET );
	if( off == -1 )
	{	send_status( VS_ER_NOSEEK, ::errno );
		delete[] p;
		close( fd );
		return 0;
	}

	size_t rc = read( fd, p, cmd.get_preview_len() );
	if( rc == -1 )
	{	send_status( VS_ER_NOREAD, ::errno );
		delete[] p;
		close( fd );
		return 0;
	}
	close( fd );

	send_status( VS_STAT_PREVIEW, VS_ER_OK, p, rc );

	delete[] p;

	return 0;
}

int 
IFhand::
do_catalog( VTDprot& cmd )
{	Logger log( "IFhand::do_catalog()" );

CBR_DEBUG( CBR_DEBUG_IFHAND_1,
	cout << log << cmd;
);

	char sh[256];

	sprintf( sh, "find %s -type f -print | sort", media_dir );

/*
 * do it once to count the number if bytes
 */
	FILE* fp = popen( sh, "r" );
	if( fp == NULL )
	{	send_status( VS_ER_INVAL, ::errno );
		return 0;
	}
	int len = 0;
	int c;
	while( ( c = fgetc( fp ) ) != EOF )
	{	len++;
	}
	pclose( fp );
/*
 * read it again to actually use the data
 */
	fp = popen( sh, "r" );
	if( fp == NULL )
	{	send_status( VS_ER_INVAL, ::errno );
		return 0;
	}

	char* p = new char[len+1+sizeof(long)]; // +1 for \0, 
						// +sizeof(long) for word align
	if( !p )
	{	send_status( VS_ER_NOMEM, ::errno );
		pclose( fp );
		return 0;
	}
	fread( p, len, 1, fp );
	pclose( fp );
/*
 * make it a string
 */
	p[len] = '\0';
/*
 * do word alignment for net transmission
 */
	while( len % sizeof(long) != 0  )
	{	p[len] = '\0';
		len++;
	}

CBR_DEBUG( CBR_DEBUG_IFHAND_2,
	cout << log << "cataloged " << len << " bytes" << endl;
);

	send_status( VS_STAT_CATALOG, VS_ER_OK, p, len );

	delete[] p;

	return 0;
}

int 
IFhand::
do_play( VTDprot& cmd )
{	Logger log( "IFhand::do_play()" );

CBR_DEBUG( CBR_DEBUG_IFHAND_1,
	cout << log << cmd;
);

	if( cur_feed == -1 )
	{	cout << log << "no such unit" << endl;
		send_status( VS_ER_NSUNIT );
		return 0;
	}

	if( cmd.get_speed() > max_speed )
	{	cout << log << "maximum speed of " << max_speed << " exceeded"
		     << endl;
		send_status( VS_ER_MAXSPEED );
		return 0;
	}

	perr_t perr = feed[cur_feed].play( cmd.get_speed() );
	if( perr != pNoErr )
	{	cout << log << "ERROR playing feed (perr=" << perr << ",errno="
		     << ::errno << ")" << endl;
		send_status( perr, ::errno );
		return 0;
	}

	send_status( VS_ER_OK );

	return 0;
}

int 
IFhand::
do_pause( VTDprot& cmd )
{	Logger log( "IFhand::do_pause()" );

CBR_DEBUG( CBR_DEBUG_IFHAND_1,
	cout << log << cmd;
);

	if( cur_feed == -1 )
	{	cout << log << "no such unit" << endl;
		send_status( VS_ER_NSUNIT );
		return 0;
	}

	count_t off;
	perr_t perr = feed[cur_feed].pause( off );
	if( perr != pNoErr )
	{	cout << log << "ERROR pausing feed (perr=" << perr << ",errno="
		     << ::errno << ")" << endl;
		send_status( perr, ::errno, VS_STAT_PAUSE, -1 );
		return 0;
	}

	send_status( VS_ER_OK, 0, VS_STAT_PAUSE, off );

	return 0;
}

int 
IFhand::
do_seek( VTDprot& cmd )
{	Logger log( "IFhand::do_seek()" );

CBR_DEBUG( CBR_DEBUG_IFHAND_1,
	cout << log << cmd;
);

	if( cur_feed == -1 )
	{	cout << log << "no such unit" << endl;
		send_status( VS_ER_NSUNIT );
		return 0;
	}

	perr_t perr = feed[cur_feed].seek( cmd.get_offset() );
	if( perr != pNoErr )
	{	cout << log << "ERROR seeking feed (perr=" << perr << ",errno="
		     << ::errno << ")" << endl;
		send_status( perr, ::errno );
		return 0;
	}

	send_status( VS_ER_OK );


	return 0;
}

int 
IFhand::
do_flow( VTDprot& cmd )
{	Logger log( "IFhand::do_flow()" );

CBR_DEBUG( CBR_DEBUG_IFHAND_1,
	cout << log << cmd;
);

	if( cur_feed == -1 )
	{	cout << log << "no such unit" << endl;
		return 0;
	}

	feed[cur_feed].inc_nbufs( cmd.get_nbufs() );
	return 0;
}

int 
IFhand::
do_eject( VTDprot& cmd )
{	Logger log( "IFhand::do_eject()" );

CBR_DEBUG( CBR_DEBUG_IFHAND_1,
	cout << log << cmd;
);

	if( cur_feed == -1 )
	{	cout << log << "no such unit" << endl;
		send_status( VS_ER_NSUNIT );
		return 0;
	}

	ifeed[cur_feed].port = -1;
	perr_t perr = feed[cur_feed].eject();
	if( perr != pNoErr )
	{	cout << log << "ERROR ejecting feed (perr=" << perr <<",errno="
		     << ::errno << ")" << endl;
		send_status( perr, ::errno );
		return 0;
	}

	delete ifeed[cur_feed].p;
	delete ifeed[cur_feed].c;

	ifeed[cur_feed].p = NULL;
	ifeed[cur_feed].c = NULL;

	send_status( VS_ER_OK );

	return 0;
}

int 
IFhand::
do_retry( VTDprot& cmd )
{	Logger log( "IFhand::do_retry()" );

CBR_DEBUG( CBR_DEBUG_IFHAND_1,
	cout << log << cmd;
);

	if( cur_feed == -1 )
	{	cout << log << "no such unit" << endl;
		return 0;
	}

	feed[cur_feed].retry( cmd.get_seq(), cmd.get_retry_offset() );

CBR_DEBUG( CBR_DEBUG_IFHAND_2,
	cout << log << "nbufs=" << feed[cur_feed].get_nbufs() << endl;
);

	return 0;
}

int 
IFhand::
do_dump( VTDprot& cmd )
{	Logger log( "IFhand::do_retry()" );

	::dump_feeds( cmd.get_dump_file(), feed, nfeeds );
	return 0;

}

void
IFhand::
send_status( int value, int errno, long type, offset_t off )
{	Logger log( "IFhand::send_status()" );

	VTDprot cmd( VS_PROT_STATUS );
	cmd.set_status( value );
	cmd.set_status_errno( errno );
	cmd.set_status_type( type  );
	cmd.set_status_pause_offset( off  );

CBR_DEBUG( CBR_DEBUG_IFHAND_1,
	cout << log << cmd;
);
	
	vtdprot_t* buf = cmd.send();

	int rc = send( in_sockfd, (char*)buf, vtdprot_sizeof( buf ), 0 );
	if( rc < 0 )
	{	cout << log << "failed sending status (errno="
		     << ::errno << ")" << endl;
	}
}

void
IFhand::
send_status( int type, int value, char* buffer, size_t len )
{	Logger log( "IFhand::send_status()" );

	VTDprot cmd( VS_PROT_STATUS );
	cmd.set_status( value );
	cmd.set_status_errno( 0 );
	cmd.set_status_type( type  );
	switch( type )
	{
	case VS_STAT_PREVIEW:
		cmd.set_status_preview_len( len  );
		break;
	case VS_STAT_CATALOG:
		cmd.set_status_catalog_len( len  );
		break;
	}

CBR_DEBUG( CBR_DEBUG_IFHAND_1,
	cout << log << cmd;
);
	
	vtdprot_t* buf = cmd.send();

CBR_DEBUG( CBR_DEBUG_IFHAND_2,
	cout << log << "size=" << vtdprot_sizeof( buf ) << endl;
);
	int rc = send( in_sockfd, (char*)buf, vtdprot_sizeof( buf ), 0 );
	if( rc < 0 )
	{	cout << log << "failed sending status (errno="
		     << ::errno << ")" << endl;
	}
	rc = send( in_sockfd, buffer, len, 0 );
	if( rc < 0 )
	{	cout << log << "failed sending status (errno="
		     << ::errno << ")" << endl;
	}
}

count_t
IFhand::
map_port( port_t p )
{
	Logger log( "IFhand::map_port()" );

	for( count_t i = 0; i < nfeeds; i++ )
	{	if( ifeed[i].port == p )
		{	break;
		}
	}
	count_t rc = i < nfeeds ? i : -1;
CBR_DEBUG( CBR_DEBUG_IFHAND_2,
	cout << log << "mapped port " << p << " to " << rc << " feed " << endl;
);
	return rc;
}

count_t
IFhand::
new_feed( port_t p )
{
	Logger log( "IFhand::new_feed()" );

	for( count_t i = 0; i < nfeeds; i++ )
	{	if( ifeed[i].port == -1 )
		{	ifeed[i].port = p;
			break;
		}
	}
	count_t rc = i < nfeeds ? i : -1;
CBR_DEBUG( CBR_DEBUG_IFHAND_2,
	cout << log << "from port " << p << " to " << rc << " got new feed " 
	     << endl;
);
	return rc;
}
