// $Header: /usr/local/src/nvrToMIT/RCS/SchedProd.C,v 1.1 1994/03/10 23:45:14 russ Exp $

/*
 * $Id: SchedProd.C,v 1.1 1994/03/10 23:45:14 russ Exp $
 *
 * SchedProd.C
 *
 * Copyright Pencom Software, 1994
 */

#include <SchedProd.h>

static Logger glog( "/dev/null" );

SchedProd::
SchedProd( speed_t max_speed, Feed* feed, count_t nfeeds, Logger& log )
 : max_speed( max_speed ), Scheduler( feed, nfeeds )
{
        pid = sproc( SchedProd_entry, PR_SALL, this );
        if( pid == -1 )
	{	cout << log << "unable to create producer (errno=" << ::errno
		     << ")" << endl;
	}
	glog = log;

}

static
void
cleanup( int signo, ... )
{
	cout << glog << "producer trapped signal " << signo 
	     << ", exiting..." << endl;
	exit( 1 );
}

void
SchedProd_entry( void* arg )
{
	signal( SIGHUP, SIG_IGN );
	signal( SIGINT, cleanup );
	signal( SIGQUIT, cleanup );
	signal( SIGSEGV, cleanup );
	signal( SIGTERM, cleanup );

	SchedProd* p = (SchedProd*)arg;
	p->run();
}

perr_t
SchedProd::
run()
{	
	Logger log( "SchedProd::run()" );

CBR_DEBUG( CBR_DEBUG_PROD_1,
	cout << log << "start scheduling..." << endl;
);

	timeval tv = Time( 0, 500000 );

	for( bool_t stop = FALSE; !stop; )
	{
		for( count_t i = 0; i < nfeeds; i++ )
		{
			switch( feed[i].get_state() )
			{
			case Feed::idle:
			case Feed::loaded:
				break;
			case Feed::playing:
			case Feed::paused:
				schedule( feed[i] );
				break;
			default:
				cout << log << "bad state, bye bye" << endl;
				stop = TRUE;
				break;
			}
		}
		sleep( 1 );
	}

	cout << log << "SchedProd::run(): leaving...should never get here" 
	     << endl;

	return pInval;		// should never return
}

void
SchedProd::
schedule( Feed& f )
{
	Logger log( "SchedProd::schedule()", Logger::silent );

	Ring* ring = f.get_ring();

	assert( ring );
	
	for(;;)
	{
		adjust_speed( f, ring );

		switch( f.get_state() )
		{
		case Feed::idle:
		case Feed::paused:
		case Feed::loaded:
			break;
		case Feed::playing:
			doit( ring, f );
			break;
		}

		release();
	}

}

void
SchedProd::
doit( Ring* ring, Feed& f )
{
	Logger log( "SchedProd::doit()", Logger::silent );
/*
 * Check for done Producers
 */
	Buffer* b;
	while( ( b = ring->get_filling() ) != NULL )
	{	aiocb* cb = b->getcb();
		if( aio_error( cb ) == EINPROGRESS )
		{	break;
		}
		else
		{	if( aio_error( cb ) != 0 )
			{	cout << log << "producer failed (errno=" 
				     << aio_error( cb ) << ")" << endl;
				::errno = aio_error( cb );
				int rc = produce( f, cb );
				assert( rc != -1 );
			}
			size_t n = aio_return( cb );
CBR_DEBUG( CBR_DEBUG_PROD_2,
			cout << log << "produced " << n << " bytes" << endl;
);
			if( n == 0 )
			{	ring->pstop();
				if( !f.get_p_eom() )
				{
CBR_DEBUG( CBR_DEBUG_PROD_1,
					cout << log << "p_eom reached" << endl;
);
					f.set_p_eom( TRUE );
				}
			}
			ring->now_full( n );
		}
	}
/*
 * If we've produced all the data possible, then stop here
 */
	if( f.get_p_eom() )
	{	return; 
	}
/*
 * Start one Producer for each empty buffer
 */
	while( ( b = ring->get_empty() ) != NULL )
	{	aiocb* cb = b->getcb( f.get_source_fd(), f.get_next_p() );
		if( produce( f, cb ) == -1 )
		{	break;
		}
	}
}

int
SchedProd::
produce( Feed& f, aiocb* cb )
{	Logger log ( "SchedProd::produce()" );
CBR_DEBUG( CBR_DEBUG_PROD_2,
	cout << log << "producing " << cb->aio_nbytes << " bytes at "
	     << cb->aio_offset << " offset" << endl;
);
#if DO_AIO
	int rc = ::aio_read( cb );
#else
	::lseek( cb->aio_fildes, cb->aio_offset, SEEK_SET );
	int rc = ::read( cb->aio_fildes, (char*)cb->aio_buf, cb->aio_nbytes );
#endif
	if( rc == -1 )
	{	cout << log << "ERROR: running producer "
		     << "(errno=" << ::errno << ")" << endl;
		cout << log << "fd=" << cb->aio_fildes
		     << ",buf=0x" << hex << (long)cb->aio_buf
		     << dec << ",offset=" << cb->aio_offset
		     << ",size=" << cb->aio_nbytes << endl;
		f.get_ring()->back_to_empty();
	}
	else
	{	f.inc_next_p( cb->aio_nbytes );
#if DO_AIO
#else
		cb->aio_errno = 0;
		cb->aio_ret = rc;
		cb->aio_nobytes = rc;
#endif
	}
	return rc;
}

void
SchedProd::
adjust_speed( Feed& f, Ring* ring )
{
	speed_t s = f.get_speed();
//	set_speed( (f.get_nbufs() > 0)?max_speed:f.get_speed(), ring );
	set_speed( (f.get_nbufs() > 0)?s*2:s, ring );
}

