// $Header: /usr/local/src/nvrToMIT/RCS/Util.C,v 1.1 1994/05/10 03:01:52 russ Exp $

/*
 * $Id: Util.C,v 1.1 1994/05/10 03:01:52 russ Exp $
 *
 * util.C
 *
 * Copyright Pencom Software, 1994
 */

#include <Util.h>

/* 
 * Read "n" bytes from a descriptor.
 * Use in place of read() when fd is a stream socket.
 */
extern "C"
int 
readn( int fd, void* ptr, size_t nbytes )
{
	char* p = (char*)ptr;
	int nleft, nread;

	nleft = nbytes;
	while( nleft > 0 )
	{
		nread = read( fd, p, nleft );
		if( nread < 0 )
		{	return nread;
		}
		else
		if( nread == 0 )
		{	break;		// EOF
		}
		nleft -= nread;
		p += nread;
	}

	return nbytes - nleft;
}

/* 
 * Write "n" bytes to a descriptor.
 * Use in place of write() when fd is a stream socket.
 */
extern "C"
int 
writen( int fd, void* ptr, size_t nbytes )
{
	char* p = (char*)ptr;
	int nleft, nwritten;

	nleft = nbytes;
	while( nleft > 0 )
	{
		nwritten = write( fd, p, nleft );
		if( nwritten < 0 )
		{	return nwritten;
		}
		nleft -= nwritten;
		p += nwritten;
	}

	return nbytes - nleft;

}


/* 
 * Receive "n" bytes from a descriptor.
 * Use in place of recv() when fd is a stream socket.
 */
extern "C"
int 
recvn( int fd, void* ptr, size_t nbytes, int flags )
{
	char* p = (char*)ptr;
	int nleft, nrecv;

	nleft = nbytes;
	while( nleft > 0 )
	{
		nrecv = recv( fd, p, nleft, flags );
		if( nrecv < 0 )
		{	return nrecv;
		}
		else
		if( nrecv == 0 )
		{	break;		// EOF
		}
		nleft -= nrecv;
		p += nrecv;
	}

	return nbytes - nleft;
}

/* 
 * Send "n" bytes to a descriptor.
 * Use in place of send() when fd is a stream socket.
 */
extern "C"
int 
sendn( int fd, void* ptr, size_t nbytes, int flags )
{
	char* p = (char*)ptr;
	int nleft, nwritten;

	nleft = nbytes;
	while( nleft > 0 )
	{
		nwritten = send( fd, p, nleft, flags );
		if( nwritten < 0 )
		{	return nwritten;
		}
		nleft -= nwritten;
		p += nwritten;
	}

	return nbytes - nleft;

}

extern "C"
ulong
comp_checksum( void* buf, size_t size )
{
	register char* p = (char*)buf;
	register ulong sum = 0;
	register size_t sz = size;

	for( ; sz ; sz--, p++ )
	{	sum += *p;
	}

	return sum;
}
