// $Header: /usr/local/src/nvrToMIT/RCS/VTD.C,v 1.1 1994/03/10 23:46:47 russ Exp $

/*
 * $Id: VTD.C,v 1.1 1994/03/10 23:46:47 russ Exp $
 *
 * VTD.C
 *
 * Copyright Pencom Software, 1994
 */

#include <VTD.h>
#include <vtd.h>
#include <NetHead.h>

VTD::
VTD( char* hostname ) : cn_state( not_connected ), sockfd( -1 )
{
	strcpy( this->hostname, hostname );

	err_t err = vsPowerOn( hostname, &sockfd );
	if( err != VS_ER_OK )
	{	cerr << vsErrorStr( err ) << " (errno=" << ::errno << ")" <<
		     endl;
		perror( "vsPowerOn()" );
		return;
	}

	serv_addr = ::serv_addr;
	serv_info = ::serv_info;
	host_info = ::host_info;

	cn_state = connected;

	cout << "Connected with " << CBRserviceName << ": "
	     << inet_ntoa( serv_addr.sin_addr ) << "[" << serv_addr.sin_port
	     << "]" << endl;

	current_td = 0;
}

VTD::
~VTD()
{
	vsPowerOff( sockfd );
	sockfd = -1;
}

err_t
VTD::
load( char* filename, size_t netbufsz, tdhand_t* td )
{
	if( netbufsz + sizeof( NetHead ) > MaxUDP )
	{	return VS_ER_INVAL;
	}

	for( tdhand_t ltd = 0; ltd < MaxDecks; ltd++ )
	{
		if( decks[ltd].state == idle )
		{	break;
		}
	}

	if( td ) *td = ltd;
	strcpy( decks[ltd].file, filename );
	decks[ltd].eom = FALSE;
	decks[ltd].speed = 0;
	decks[ltd].netbufsz = netbufsz;

	err_t err = vsLoadTape( sockfd, filename, netbufsz,&decks[ltd].sockfd );
	if( err == VS_ER_OK )
	{	decks[ltd].state = loaded;
	}
	else
	{	cerr << "ERROR: VTD::load( filename=" << filename
		     << ", *td=" << decks[ltd].sockfd << " ) failed" << endl
		     << "       " << vsErrorStr( err ) << endl;
	}
	return err;

}

err_t
VTD::
swtch( tdhand_t td )
{	
	if( td < 0 || td >= MaxDecks )
	{	return VS_ER_NOTAPE;
	}

	if( decks[td].state == idle )
	{	return VS_ER_NOTAPE;
	}
	current_td = td;
	return VS_ER_OK;
}

err_t
VTD::
preview( void* buffer, size_t* len, offset_t offset )
{
	if( decks[current_td].state == idle )
	{	return VS_ER_NOTAPE;
	}

	err_t err =  vsPreview( sockfd, decks[current_td].sockfd, offset, 
		buffer, len );
	if( err != VS_ER_OK )
	{	cerr << "ERROR: VTD::preview( buffer=0x" << hex << (long)buffer 
		     << ", *len=" << dec << *len << ", offset=" << offset 
		     << " ) failed" << endl
		     << "       " << vsErrorStr( err ) << endl;
	}
	return err;
}

err_t
VTD::
catalog( char** buffer, size_t* len )
{
	err_t err =  vsCatalog( sockfd, buffer, len );
	if( err != VS_ER_OK )
	{	cerr << "ERROR: VTD::catalog( buffer=0x" << hex <<(long)*buffer
		     << ", *len=" << dec << *len << " ) failed" << endl
		     << "       " << vsErrorStr( err ) << endl;
	}
	return err;
}

err_t
VTD::
play( speed_t speed )
{
	if( decks[current_td].state == idle )
	{	return VS_ER_NOTAPE;
	}

	decks[current_td].speed = speed;

	err_t err = vsPlay( sockfd, decks[current_td].sockfd, speed );
	if( err == VS_ER_OK )
	{	decks[current_td].state = playing;
	}
	else
	{	cerr << "ERROR: VTD::play( speed=" << speed << " ) failed"<<endl
		     << "       " << vsErrorStr( err ) << endl;
	}
	return err;
}

err_t
VTD::
pause( offset_t* off )
{
	if( decks[current_td].state == idle )
	{	return VS_ER_NOTAPE;
	}

	offset_t o;
	err_t err = vsPause( sockfd, decks[current_td].sockfd, &o );
	if( err == VS_ER_OK )
	{	decks[current_td].state = paused;
		if( off ) *off = o;
	}
	else
	{	cerr << "ERROR: VTD::pause(...) failed"<<endl
		     << "       " << vsErrorStr( err ) << endl;
	}
	return err;
}

err_t
VTD::
seek( offset_t off )
{
	switch( decks[current_td].state )
	{
	case idle:
		return VS_ER_NOTAPE;
	}

	if( decks[current_td].eom )
	{	decks[current_td].eom = FALSE;
	}

	err_t err = vsSeek( sockfd, decks[current_td].sockfd, off );
	if( err != VS_ER_OK )
	{	cerr << "ERROR: VTD::seek( off=" << off <<" ) failed" << endl
		     << "       " << vsErrorStr( err ) << endl;
	}
	return err;
}

err_t
VTD::
flow( count_t nbufs )
{
	if( decks[current_td].state == idle )
	{	return VS_ER_NOTAPE;
	}

	err_t err = vsFlowControl( sockfd, decks[current_td].sockfd, nbufs );
	if( err != VS_ER_OK )
	{	cerr << "ERROR: VTD::flow( nbufs=" << nbufs << " ) failed"<<endl
		     << "       " << vsErrorStr( err ) << endl;
	}
	return err;
}

err_t
VTD::
retry( count_t seq )
{
	if( decks[current_td].state == idle )
	{	return VS_ER_NOTAPE;
	}

	err_t err = vsFlowControl( sockfd, decks[current_td].sockfd, seq );
	if( err != VS_ER_OK )
	{	cerr << "ERROR: VTD::retry( seq=" << seq << " ) failed"<<endl
		     << "       " << vsErrorStr( err ) << endl;
	}
	return err;
}

err_t
VTD::
eject()
{
	if( decks[current_td].state == idle )
	{	return VS_ER_NOTAPE;
	}

	err_t err = vsEject( sockfd, decks[current_td].sockfd );
	if( err == VS_ER_OK )
	{	decks[current_td].state = idle;
		decks[current_td].eom = FALSE;
		decks[current_td].speed = 0;
		decks[current_td].sockfd = -1;
		decks[current_td].file[0] = '\0';
	}
	else
	{	cerr << "ERROR: VTD::eject() failed"<<endl
		     << "       " << vsErrorStr( err ) << endl;
	}
	return err;
}

bool_t 
VTD::
eom()
{
	if( decks[current_td].state == idle )
	{	return VS_ER_NOTAPE;
	}

	if( decks[current_td].eom )
	{	return TRUE;
	}

	bool_t eom;
	err_t err = vsEndOfMedia( sockfd, decks[current_td].sockfd, &eom );
	if( err == VS_ER_OK )
	{	decks[current_td].eom = eom;
	}
	else
	{	cerr << "ERROR: VTD::eom() failed"<<endl
		     << "       " << vsErrorStr( err ) << endl;
	}
	return decks[current_td].eom;
}

err_t
VTD::
dump( char* filename )
{
	cout << "dump not implemented..." << filename << endl;
	return VS_ER_OK;
}

static char* VTD_cn_state_str[] = { "connected", "not connected" };
static char* VTD_td_state_str[] = { "idle", "loaded", "playing", "paused" };

char*
VTD::
get_cn_state_str()
{	return VTD_cn_state_str[decks[current_td].state];
}

char*
VTD::
get_td_state_str()
{	return VTD_td_state_str[decks[current_td].state];
}

ostream&
operator<<( ostream& ostr, VTD& i )
{
	ostr << endl
	     << "hostname: " << i.hostname << endl
	     << endl;

	return ostr;

}
