/* $Header: /usr/local/src/nvrToMIT/RCS/vtd.c,v 1.3 1994/12/17 12:57:55 russ Exp $ */

/*
 * $Id: vtd.c,v 1.3 1994/12/17 12:57:55 russ Exp $
 *
 * vtd.c
 *
 * Copyright Pencom Software, 1994
 */

#include <vtd.h>
#include <vtdprot.h>
#include <sys/select.h>

struct servent serv_info;
struct hostent host_info;
struct sockaddr_in serv_addr;

static err_t rcv_stat( sock_t sockfd, vtdprot_t* buf );
static err_t parse_stat( vtdprot_t* buf );
static port_t conv_sock_to_port( sock_t sockfd );
static fd_set eoms[256];

#define CONV_SOCK_TO_PORT( buf, out ) \
{	(buf).port = conv_sock_to_port( (out) ); \
	if( (buf).port == -1 ) \
	{	return VS_ER_OUT; \
	} \
	(buf).port = htonl( (buf).port ); \
}

err_t
vsPowerOn( char* hostname, sock_t* decksocket )
{
	struct servent* s;
	struct hostent* h;
	u_long inaddr;
	int n, rc;
	vtdprot_t buf;

	memset( (char*)&serv_addr, 0, sizeof( serv_addr ) );
	serv_addr.sin_family = AF_INET;

#if 1
	s = getservbyname( CBRserviceName, "tcp" );
#else
	s = getservbyname( "CBRtest", "tcp" );
#endif
	if( !s )
	{	*decksocket = -1;
		return VS_ER_NOSERV;
	}
	serv_info = *s;
	serv_addr.sin_port = htons( s->s_port ); 
/*
 * First try to use the hostname as a IP address 
 */
	inaddr = inet_addr( hostname );
	if( inaddr != INADDR_NONE )
        {       memcpy( (char*)&serv_addr.sin_addr, (char*)&inaddr,
			sizeof( inaddr ) );
		host_info.h_name = 0;
	}
/*
 * Then try it as a regular hostname
 */
	else
	{	h = gethostbyname( hostname );
		if( !h )
		{	return VS_ER_BADSERV;	
		}
		host_info = *h;
                memcpy((char*)&serv_addr.sin_addr, h->h_addr,
                       h->h_length );
	}

	*decksocket = socket( AF_INET, SOCK_STREAM, 0 );
	if( *decksocket < 0 )
	{	return VS_ER_NOSOCK;
	}
/*
 * Set to socket to NODELAY so the commands get sent ASAP
 */
	n = 1;
	rc = setsockopt( *decksocket, IPPROTO_TCP, TCP_NODELAY, (char*)&n,
		sizeof( n ) );
	if( rc < 0 )
	{	close( *decksocket );
		*decksocket = -1;
		return VS_ER_NOOPT;
	}
		
/*
 * Connect to the server
 */
	rc = connect( *decksocket, (struct sockaddr*)&serv_addr, 
		sizeof(serv_addr) );
	if( rc < 0 )
	{	close( *decksocket );
		*decksocket = -1;
		return VS_ER_NOCONN;
	}

	rc = rcv_stat( *decksocket, &buf );
	if( rc != VS_ER_OK )
	{	close( *decksocket );
		*decksocket = -1;
		return rc;
	}

	FD_ZERO( &eoms );

	return VS_ER_OK;

}

err_t
vsLoadTape( sock_t vs, char* filename, size_t netbufsz, sock_t* out )
{
	int rc;
	vtdprot_t buf;
	struct sockaddr_in serv_addr;
	int len;

	if( !filename )
	{	errno = EINVAL;
		return VS_ER_NSFILE;
	}

	*out = socket( AF_INET, SOCK_DGRAM, 0 );
	if( *out == -1 )
	{	return VS_ER_NOSOCK;
	}
	
	memset( (char*)&serv_addr, 0, sizeof( serv_addr ) );
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = htonl( INADDR_ANY );
	serv_addr.sin_port = htons( 0 );
#if 0
	fprintf(stderr, "%s[%d]\n",inet_ntoa( serv_addr.sin_addr ),serv_addr.sin_port);
#endif /* 0 */
 
/*
 * bind to the socket and let the system pick a port
 */
	rc = bind( *out, (struct sockaddr*)&serv_addr, sizeof( serv_addr ) );
	if( rc < 0 )
	{	close( *out );
		*out = -1;
		return VS_ER_NOBIND;
	}
	len = sizeof( serv_addr );
	rc = getsockname( *out, (struct sockaddr*)&serv_addr, &len );
#if 0
	fprintf(stderr, "%s[%d]\n",inet_ntoa( serv_addr.sin_addr ),serv_addr.sin_port);
#endif /* 0 */

	VS_ADD_ID( buf );
	buf.type = htonl( VS_PROT_LOAD );
	CONV_SOCK_TO_PORT( buf.p.load, *out );
	strcpy( buf.p.load.file, filename );
	buf.p.load.netbufsz = htonl( netbufsz );
/*
 * Send queue request
 */
	rc = send( vs, &buf, vtdprot_sizeof( &buf ), 0 );
	if( rc < 0 )
	{	close( *out );
		*out = -1;
		return VS_ER_NSUNIT;
	}
/*
 * wait for ack form server
 */
	rc = rcv_stat( vs, &buf );
	if( rc != VS_ER_OK )
	{	close( *out );
		*out = -1;
		return rc;
	}

#if 0
	FD_CLR( *out, &eoms );
#endif

	return VS_ER_OK;
}

err_t
vsPreview( sock_t vs, sock_t out, offset_t offset, void* buffer, size_t* len )
{
	vtdprot_t buf;
	int rc;
	err_t err;
        struct iovec iov[2];
        struct msghdr mh;
	size_t l;

	VS_ADD_ID( buf );
	buf.type = htonl( VS_PROT_PREVIEW );
	CONV_SOCK_TO_PORT( buf.p.preview, out );
	buf.p.preview.offset = htonl( offset );
	buf.p.preview.len = htonl( *len );

	rc = send( vs, &buf, vtdprot_sizeof( &buf ), 0 );
	if( rc < 0 )
	{	return VS_ER_NSUNIT;
	}
/*
 * wait for ack form server
 */
	rc = rcv_stat( vs, &buf );
	if( rc != VS_ER_OK )
	{	return rc;
	}

	l = buf.p.status.data.preview.len;
#if 0
	fprintf(stderr, "getting preview data, len=%d\n", l );
#endif /* 0 */

	if( l > *len )
	{	
#if 0
	    	fprintf(stderr, "oops, got more data than asked for(%d)\n", *len );
#endif /* 0 */
		l = *len;
	}
	else
	if( l < *len )
	{	
#if 0
	    	fprintf(stderr, "oops, got less data than asked for(%d)\n", *len );
#endif /* 0 */
		*len = l;
	}

	rc = recvn( vs, buffer, l , 0 );
	if( rc < 0 )
	{	return VS_ER_NORECV;
	}
	
#if 0
	fprintf(stderr, "got %d bytes of preview data\n", rc );
#endif /* 0 */
	return VS_ER_OK;
}

err_t
vsCatalog( sock_t vs, char** buffer, size_t* len )
{
	vtdprot_t buf;
	int rc;

	*buffer = NULL;
	*len = 0;

	VS_ADD_ID( buf );
	buf.type = htonl( VS_PROT_CATALOG );

	rc = send( vs, &buf, vtdprot_sizeof( &buf ), 0 );
	if( rc < 0 )
	{	return VS_ER_NSUNIT;
	}
/*
 * wait for ack form server
 */
	rc = rcv_stat( vs, &buf );
	if( rc != VS_ER_OK )
	{	return rc;
	}

	*len = buf.p.status.data.catalog.len;
#if 0
	fprintf(stderr, "getting catalog data, *len=%d\n", *len );
#endif /* 0 */

	*buffer = malloc( *len );
	if( !*buffer )
	{	return VS_ER_NOMEM;
	}

	rc = recvn( vs, *buffer, *len , 0 );
	if( rc < 0 )
	{	return VS_ER_NORECV;
	}
	
#if 0
	fprintf(stderr, "got %d bytes of catalog data\n", rc );
#endif /* 0 */
	return VS_ER_OK;
}

err_t
vsPlay( sock_t vs, sock_t out, speed_t speed )
{
	vtdprot_t buf;
	int rc;

	VS_ADD_ID( buf );
	buf.type = htonl( VS_PROT_PLAY );
	CONV_SOCK_TO_PORT( buf.p.play, out );
	buf.p.play.speed = htonl( speed );

	rc = send( vs, &buf, vtdprot_sizeof( &buf ), 0 );
	if( rc < 0 )
	{	return VS_ER_NSUNIT;
	}
/*
 * wait for ack form server
 */
	return rcv_stat( vs, &buf );
	
}

err_t
vsPause( sock_t vs, sock_t out, offset_t* offset )
{
	vtdprot_t buf;
	err_t rc;

	VS_ADD_ID( buf );
	buf.type = htonl( VS_PROT_PAUSE );
	CONV_SOCK_TO_PORT( buf.p.pause, out );

	rc = send( vs, &buf, vtdprot_sizeof( &buf ), 0 );
	if( rc < 0 )
	{	return VS_ER_NSUNIT;
	}
/*
 * wait for ack form server
 */
	rc = rcv_stat( vs, &buf );
	*offset = buf.p.status.data.pause_offset;
	return rc;
	
}

err_t
vsSeek( sock_t vs, sock_t out, offset_t offset )
{
	vtdprot_t buf;
	int rc;

	VS_ADD_ID( buf );
	buf.type = htonl( VS_PROT_SEEK );
	CONV_SOCK_TO_PORT( buf.p.seek, out );
	buf.p.seek.offset = htonl( offset );

	rc = send( vs, &buf, vtdprot_sizeof( &buf ), 0 );
	if( rc < 0 )
	{	return VS_ER_NSUNIT;
	}
/*
 * wait for ack form server
 */
	return rcv_stat( vs, &buf );
}

err_t
vsFlowControl( sock_t vs, sock_t out, count_t nbufs )
{
	vtdprot_t buf;
	int rc;

	VS_ADD_ID( buf );
	buf.type = htonl( VS_PROT_FLOW );
	CONV_SOCK_TO_PORT( buf.p.flow, out );
	buf.p.flow.nbufs = htonl( nbufs );

	rc = send( vs, &buf, vtdprot_sizeof( &buf ), 0 );
	if( rc < 0 )
	{	return VS_ER_NSUNIT;
	}
	return VS_ER_OK;
}

err_t
vsRetryBuffer( sock_t vs, sock_t out, count_t seq )
{
	vtdprot_t buf;
	int rc;

	VS_ADD_ID( buf );
	buf.type = htonl( VS_PROT_RETRY );
	CONV_SOCK_TO_PORT( buf.p.retry, out );
	buf.p.retry.seq = htonl( seq );

	rc = send( vs, &buf, vtdprot_sizeof( &buf ), 0 );
	if( rc < 0 )
	{	return VS_ER_NSUNIT;
	}
	return VS_ER_OK;
}

err_t
vsEject( sock_t vs, sock_t out )
{
	vtdprot_t buf;
	int rc;
	err_t err;

	VS_ADD_ID( buf );
	buf.type = htonl( VS_PROT_EJECT );
	CONV_SOCK_TO_PORT( buf.p.eject, out );

	rc = send( vs, &buf, vtdprot_sizeof( &buf ), 0 );
	if( rc < 0 )
	{	return VS_ER_NSUNIT;
	}
/*
 * wait for ack form server
 */
	err = rcv_stat( vs, &buf );
/*
 * clean up tape deck port
 */
	close( out );

	return err;

}

err_t
vsEndOfMedia( sock_t vs, sock_t out, bool_t* eom )
{
	fd_set fdvar;
	struct timeval tv;
	vtdprot_t buf;
	int rc;

#if 0
	if( FD_ISSET( out, &eoms ) )
	{	*eom = TRUE;
		return VS_ER_OK;
	}
#endif

	*eom = FALSE;

	FD_ZERO( &fdvar );
	FD_SET( vs, &fdvar );

	tv.tv_sec = 0;
	tv.tv_usec = 0;

	rc = select( vs+1, &fdvar, NULL, NULL, &tv );
	if( rc == -1 )
	{	return VS_ER_NOSELECT;
	}

	if( FD_ISSET( vs, &fdvar ) )
	{	rc = recvn( vs, &buf, vtdprot_sizeof( &buf ), 0 );
		if( rc == -1 )
		{	return VS_ER_NSUNIT;
		}
#if 0
		fprintf(stderr, "rcvd %d bytes, looking for eom (type=%d)\n", rc, 
			VS_PROT_EOM );
#endif /* 0 */
		buf.type = ntohl( buf.type );
#if 0
		fprintf(stderr, "rcv buf.type=%d\n", buf.type );
#endif /* 0 */
		*eom = buf.type == VS_PROT_EOM;
	}

	return VS_ER_OK;

}

err_t
vsPowerOff( sock_t vs )
{	
	int rc = close( vs );
	if( rc == -1 )
	{	return VS_ER_NSUNIT;
	}
}

static
err_t
rcv_stat( sock_t sockfd, vtdprot_t* buf )
{
	int rc;

	rc = recvn( sockfd, buf, sizeof( *buf ), 0 );
	if( rc < 0 )
	{	return VS_ER_NORECV;
	}

#if 0
	fprintf(stderr, "rcv_stat(): rcvd %d bytes\n", rc );
#endif /* 0 */
	return parse_stat( buf );
}

static
err_t
parse_stat( vtdprot_t* buf )
{
	char* p;
	buf->type = ntohl( buf->type );

	if( buf->type != VS_PROT_STATUS )
	{	errno = 0;
		p = (char*)buf;
#if 0
		fprintf(stderr, "rcv buf->type=%d\n", buf->type );
#endif /* 0 */
		return VS_ER_BADSERV;
	}
	
	buf->p.status.status = ntohl( buf->p.status.status );
	buf->p.status.errno = ntohl( buf->p.status.errno );
	buf->p.status.type = ntohl( buf->p.status.type );
	
#if 0
	fprintf(stderr, "rcv buf->type           =%d\n", buf->type );
	fprintf(stderr, "rcv buf->p.status.status=%d\n", buf->p.status.type );
	fprintf(stderr, "rcv buf->p.status.errno =%d\n", buf->p.status.type );
	fprintf(stderr, "rcv buf->p.status.type  =%d\n", buf->p.status.type );
#endif /* 0 */

	switch( buf->p.status.type )
	{	
	case VS_STAT_PAUSE:
		buf->p.status.data.pause_offset = ntohl( 
			buf->p.status.data.pause_offset );
		break;
	case VS_STAT_PREVIEW:
		buf->p.status.data.preview.len = ntohl( 
			buf->p.status.data.preview.len );
		break;
	case VS_STAT_CATALOG:
		buf->p.status.data.catalog.len = ntohl( 
			buf->p.status.data.catalog.len );
		break;
	}
	if( buf->p.status.status != VS_ER_OK )
	{	errno = buf->p.status.errno;
		return buf->p.status.status;;
	}

	return VS_ER_OK;
}

static
port_t
conv_sock_to_port( sock_t sockfd )
{
	struct sockaddr_in addr;
	int len = sizeof( addr );
	int rc = getsockname( sockfd, &addr, &len );
	if( rc < 0 )
	{	return -1;
	}
	return addr.sin_port;
}

size_t
vtdprot_sizeof( vtdprot_t* buf )
{
	return sizeof( *buf );

}

static char* vs_err_str[VS_ER_LASTERR+1] = {
/* 0*/  "No error",
/* 1*/  "No server found in /etc/services",
/* 2*/  "Unable to create a socket",
/* 3*/  "Unable to create a connection",
/* 4*/  "Unable to get the options of a socket",
/* 5*/  "Unable to receive from a socket",
/* 6*/  "Something is wrong with the server",
/* 7*/  "Something is wrong with the server",
/* 8*/  "No such tape unit found",
/* 9*/  "Unable to bind an address to a socket",
/*10*/  "Unable to select on a socket",
/*11*/  "No such file",
/*12*/  "Output socket is invalid",
/*13*/  "Tape unit is unable to load another tape",
/*14*/  "There is no tape loaded for this output socket",
/*15*/  "Tape unit is idle",
/*16*/  "Tape unit is in a bad state",
/*17*/  "Tape unit cannot accpet negative offsets",
/*18*/  "Sequence is out of range",
/*19*/  "Not enough memory to complete the task",
/*20*/  "Tape unit is not ready",
/*21*/  "Unable to create a process",
/*22*/  "Invalid parameter passed in command",
/*23*/  "Invalid TCP descriptor given Tape Unit",
/*24*/  "Invalid file given Tape Unit",
/*25*/  "Invalid destination given Tape Unit",
/*26*/  "Unable to seek within the tape",
/*27*/  "Unable to read from the tape",
/*28*/  "Unable to write to the tape",
/*29*/  "Maximum CBR speed exceeded",
/*30*/  0
};

char*
vsErrorStr( err_t err )
{
	char* p;

	if( err >= VS_ER_OK && err < VS_ER_LASTERR )
	{	p = vs_err_str[err];
	}
	else
	{	p = "Invalid error code";
	}
	return p ? p : "Undefined error string";
}

static char* vs_prot_str[] =
{	"first", "status", "load", "preview", "play", "seek", "pause", 
	"flow", "retry", "eom", "eject", "dump", "catalog", "unknown", 0

};

char*
vtdprot_type( long type )
{
	if( type > VS_PROT_FIRST && type < VS_PROT_LAST )
	{	return vs_prot_str[type];
	}
	else
	{	return "Invalid protocol type";
	}
}


