/* $Header: /usr/local/src/nvrToMIT/RCS/Buffer.h,v 1.1 1994/05/10 23:29:17 russ Exp $ */

#ifndef __h_Buffer__
#define __h_Buffer__

/*
 * $Id: Buffer.h,v 1.1 1994/05/10 23:29:17 russ Exp $
 *
 * Buffer.h
 *
 * Copyright Pencom Software, 1994
 */

#include <Vodoo.h>

typedef enum bufstat_t { EMPTY, EMPTYING, PARTIAL, FULL, FILLING } bufstat_t;

/*
 * The Buffer class represents a contiguous area of memory, that has a 
 * particular state, total allocated size, amount used, and some extra 
 * information used for scheduling it's filling, etc.
 *
 * It is used to create the Ring class.
 */
class Buffer
{
public:
	Buffer();

	void init( char* p = NULL, size_t s = 0, bufstat_t st = EMPTY, 
	 	   seq_t seq = 0 );
/*
 * State determination member functions
 */
	bufstat_t get_state() { return state; }
	bool_t isempty() { return state == EMPTY; }
	bool_t isfull() { return state == FULL; }
	bool_t ispartial() { return state == PARTIAL; }
/*
 * Access member functions
 */
	char* getptr() { return ptr; }
	size_t getsize() { return size; }
	size_t getused() { return used; }
	aiocb* getcb( int fd, off_t offset );
	aiocb* getcb() { return &cb; }
	seq_t get_seq() { return seq; }
	size_t get_total() { return total; }
	size_t get_offset() { return cb.aio_offset; }
/*
 * Update member functions	
 */
	void setptr( char* p ) { ptr = p; cb.aio_buf = p; }
	void setsize( size_t s ) { size = s; cb.aio_nbytes = 0; }
	void setused( size_t s ) { used = s; }
	void set_seq( seq_t s ) { seq = s; }
	void set_total( size_t t ) { total = t; cb.aio_nbytes = t; }

	void now_empty() { setstate( EMPTY ); setused( 0 ); }
	void now_emptying() { setstate( EMPTYING ); }
	void now_full() { setstate( FULL ); }
	void now_full( size_t s ) { setused( s ) ; setstate( FULL ); }
	void now_partial( size_t s ) { setused( s ) ; setstate( PARTIAL ); }
	void now_filling() { setstate( FILLING ); }
/*
 * Misc member functions
 */
	char* strstate();	// convert current state to a char*
	friend ostream& operator<<( ostream& o, Buffer& b );

protected:
	void setstate( bufstat_t s ) { state = s; }

public:
	bufstat_t state;	// state of this Buffer
	char* ptr;		// location in memory
	size_t size;		// Buffer size in bytes
	size_t used;		// Number of bytes used in the buffer
	size_t total;		// Total bytes in group of buffers
	aiocb cb;		// AIO control block for filling
	seq_t seq;		// sequence for emptying

};

#endif
