/* $Header: /usr/local/src/nvrToMIT/RCS/CBRclient.h,v 1.1 1994/05/10 23:29:19 russ Exp $ */

#ifndef __h_CBRclient__
#define __h_CBRclient__

/*
 * $Id: CBRclient.h,v 1.1 1994/05/10 23:29:19 russ Exp $
 *
 * CBRclient.h
 *
 * Copyright Pencom Software, 1994
 */

#include <Vodoo.h>
#include <VTDprot.h>
#include <VTD.h>
#include <Ring.h>
#include <Stopwatch.h>
#include <CBRprod.h>

class Cdeck;

/*
 * CBRclient provides a simple open/read/close interface to the CBR server.
 * It's intent is allow programs that usually open/read/close files located
 * on a local file system to be easily ported to used the CBR server.
 *
 * This class used a VTD class to communicate with the server.
 * 
 * Users of this class have to keep track to a tape deck handle for each
 * virtual tape deck being used.  This is analogous to the file descriptor
 * returned by the normal ::open() call
 */
class CBRclient
{
public:
	CBRclient( char* server );
	~CBRclient();

	tdhand_t open( 		// opens a CBR stream with given file
		char* filename, // full path of file to open
		size_t netbutsz,// size of producers's network buffers
		speed_t rate 	// Constant bit-rate in bytes/sec
	);
	int preview( 		// used for non-CBR reading of the file
		tdhand_t tdh, 	// handle gotten from open()
		void* buffer, 	// buffer, size bytes long, to place the data
		size_t* size, 	// size of buffer in bytes
		offset_t offset	// offset into the file which to begin reading
	);
	int catalog( 		// get a \n separated list of media available
		char** buffer, 	// media list, caller responsible for freeing
		size_t* size 	// size of buffer in bytes
	);
	int read( 		// reads from the CBR stream
		tdhand_t tdh, 	// handle gotten from open()
		void* buffer, 	// buffer, size bytes long, to place the data
		size_t size 	// size of buffer in bytes
	);
	int seek( 		// seeks around in the CBR stream
		tdhand_t tdh, 	// handle gotten from open()
		long offset 	// offset to seek to in the stream
	);
	/*
	 * NOTE:  seek() is inherantly non-CBR, so seek only works if:
	 *	  1) read() hasn't been called yet, 
	 *	  2) after EOM has been received,
	 *	  3) or, paused
	 */
	int pause( 		// pauses the CBR stream
		tdhand_t tdh	// handle gotten from open()
	);
	int close( 		// closes the CBR stream and releases resources
		tdhand_t tdh 	// handle gotten from open()
	);

private:
	VTD vtd;			// Interface connection to CBR server
	Cdeck *cd;			// array of tapes being played
	
	sock_t my_sockfd;		// local socket information
	struct sockaddr_in my_addr;
	int my_addr_len;

};

class Cdeck
{
public:
	friend class CBRclient;

	Cdeck() : eom( FALSE ), ring( NULL ), prod( NULL ), sz( 0 ) {}
	~Cdeck() { delete prod; delete ring; }

	Time get_avg_read()
	{	if( max_read == first_read )
		{	return ( total_reads - first_read ) / num_reads; 
		}
		else
		{	return total_reads / num_reads; 
		}
	}
	int get_num_reads() { return num_reads; }
	Time get_first_read() { return first_read; }
	Time get_min_read() { return min_read; }
	Time get_max_read() { return max_read; }

private:
	bool_t first;		// TRUE if first read not yet done
	bool_t eom;		// TRUE if end of media, shared with the
				//  CBRprod class
	Ring* ring;		// Ring buffer to consume buffer from
	CBRprod* prod;		// Producer process that get's media data
				//  from the network and fills the Ring 
	speed_t speed;		// Constant bit-rate speed in bytes/sec
	bufid_t id;		// Current Ring buffer chunk being used
	char* buf;		// Current place within the buffer being 
				//  consumed
	size_t sz;		// Bytes left in buffer to consume
/*
 * Statistical information about read() activity
 */
	Time first_read;
	Stopwatch read_stat;
	Time total_reads;
	int num_reads;
	Time min_read;
	Time max_read;
	
};


#endif
