/* $Header: /usr/local/src/nvrToMIT/RCS/CBRprod.h,v 1.1 1994/05/10 23:29:21 russ Exp $ */

#ifndef __h_CBRprod__
#define __h_CBRprod__

/*
 * $Id: CBRprod.h,v 1.1 1994/05/10 23:29:21 russ Exp $
 *
 * CBRprod.h
 *
 * Copyright Pencom Software, 1994
 */

#include <Vodoo.h>
#include <Ring.h>
#include <Stopwatch.h>
#include <NetHead.h>

/*
 * The CBRprod class is designed to produce information from a network
 * and fill a Ring buffer.  When consturcted, it sproc()'s itself and
 * begins to run as it's own process.
 *
 * It handles flow control and retry messages back to the server to affect
 * the incoming data.
 *
 * One instance of this class is created for every CBRclient::open(), or 
 * cbr_open().
 */
class CBRprod 
{
public:
	CBRprod( Ring* r, sock_t fd, sock_t ctrl_fd, speed_t& speed, 
		 bool_t& eom );
	~CBRprod();

	virtual void run();	// sproc() entry point

	size_t produce( 	// get a buffer's worth of data for the net
		void* buf, 
		size_t size 
	);
	void delay(); 		// delay for speed's interval
	void send_flow();	// send flow control message
	void send_retry();	// send retry message

	friend void CBRprod_entry( void* arg );

private:

	Ring* ring;		// Ring buffer to produce into
	sock_t fd;		// UPD socket descriptor (media data)
	sock_t ctrl_fd;		// TCP socket descriptor (control data)
	speed_t& speed;		// speed which we're being consumed, hopefully
	bool_t& eom;		// set to TRUE when EOM is encountered

	char* ptr;		// buffer to fill
	size_t size;		// size of ptr buffer in bytes
	bufid_t id;		// Ring buffer id for the buffer
	size_t total;		// total bytes produced so far
	short port;

	long need;		// current flow control need 
				//  0 ok 
				//  < 0 asked server to slow down
				//  > 0 asked server to slow up	
	long mid;		// half full Ring buffer
	long hwm;		// high water mark
	long lwm;		// low water mark

	pid_t pid;		// process id of this object
	seq_t seq;		// next seq number to get of the net

};

#endif

