/* $Header: /usr/local/src/nvrToMIT/RCS/Feed.h,v 1.1 1994/05/10 23:29:30 russ Exp $ */


#ifndef __h_Feed__
#define __h_Feed__

/*
 * $Id: Feed.h,v 1.1 1994/05/10 23:29:30 russ Exp $
 *
 * Feed.h
 *
 * Copyright Pencom Software, 1994
 */

#include <Vodoo.h>
#include <Ring.h>
#include <vtd.h>

const seq_t MaxSequence = ~0;
//const size_t ChunksPerFull = 60;	// 60K, assumes 1K chunks
//const size_t ChunksPerEmpty = 60;	// 60K, assumes 1K chunks

class Feed
{
public:
	enum state_t { idle, loaded, playing, paused };

	Feed();
	~Feed();

	void init();
	void reset();

	perr_t load(
		int ctrl_if,		// control interface file descriptor
		char* source,		// source of media
		sockaddr_in destin,	// destination of media
		size_t netbufsz
	);

	perr_t play(
		speed_t speed		// in frames per second
	);

	perr_t seek( 
		count_t offset		// byte offset to seek to
	);

	perr_t pause( 
		count_t& offset 	// the next byte to be consumed
	);

	perr_t flow( 
		count_t nbufs 		// number of buffers to speed up or
					// slow down
	);

	perr_t retry(
		count_t rseq,		// retry sending starting with this
					// sequence number
		offset_t off		// reset to this offset in the media
	);

	perr_t eject();

	void lock();			// gain access to this feed
	void unlock();			// release access to this feed

	state_t get_state() { return state; }

	int get_ctrl_if() { return ctrl_if; }
	char* get_source() { return source; }
	int get_source_fd() { return source_fd; }
	sockaddr_in get_destin() { return destin; }
	int get_destin_fd() { return destin_fd; }

	speed_t get_speed() { return speed; }
	size_t get_netbufsz() { return netbufsz; }

	bool_t get_p_eom() { return p_eom; }
	bool_t get_c_eom() { return c_eom; }

	offset_t get_next_p() { return next_p; }
	offset_t get_next_c() { return next_c; }

	count_t get_nbufs() { return nbufs; }

	count_t get_rseq() { return rseq; }
	offset_t get_rseq_off() { return rseq_off; }
	seq_t get_seq() { return seq; }

	Ring* get_ring() { return ring; }

	void set_state( state_t s ) { state = s; }

	void set_p_eom( bool_t v ) { p_eom = v; }
	void set_c_eom( bool_t v ) { c_eom = v; }

	void set_next_p( offset_t n ) { next_p = n; }
	void inc_next_p( offset_t n ) { next_p += n; }

	void set_next_c( offset_t n ) { next_c = n; }
	void inc_next_c( offset_t n ) { next_c += n; }

	void set_nbufs( count_t n ) { nbufs = n; }
	void inc_nbufs( count_t n ) { nbufs += n; }

	void set_rseq( count_t s ) { rseq = s; }
	void set_seq( seq_t s ) { seq = s; }
	void inc_seq() { seq++; }

	friend ostream& operator<<( ostream& o, Feed& f );
	friend void dump_feeds( char* file, Feed* f, count_t n );

private:

	state_t state;

	int ctrl_if;			// control interface
	char source[MAXPATHLEN];	// source of media
	int source_fd;
	sockaddr_in destin;		// destination of media
	int destin_fd;
	sockaddr_in l_addr;

	speed_t speed;		// in bytes per second
	size_t netbufsz;	// size of Consumer's network packets

	bool_t p_eom;		// TRUE if producers reached end of media
	bool_t c_eom;		// TRUE if consumers reached end of media

	offset_t next_p;	// next byte to be produced from the media
	offset_t next_c;	// next byte to be consumed the media

	count_t nbufs;		// current client's need
				// 0, current rate is fine
				// >0, speed up
				// <0, slow down
	count_t rseq;		// reset sequence to this number
	offset_t rseq_off;	// offset in media stream related to rseq
	seq_t seq;		// next sequence number for buffer to be
				// consumed
	Ring* ring;		// Ring buffer 
//	Mutex m;		// mutual exclusion for multi-process
				// accessing
};

#endif
