/* $Header: /usr/local/src/nvrToMIT/RCS/Ring.h,v 1.1 1994/05/10 23:29:39 russ Exp $ */

#ifndef __h_Ring__
#define __h_Ring__

#include <Vodoo.h>
#include <Buffer.h>
#include <Stopwatch.h>
#include <Semaphore.h>

typedef long bufid_t;

struct RingInfo
{
	char* parent_buffer;	// the original contiguous area of memory
	size_t pbsize;		// parent buffer size in Kbytes
	size_t cbsize;		// child buffer size in Kbytes

	Buffer* bufs;
	count_t nbufs;		// number of buffers

	count_t bpf;		// buffers per full buffer
	count_t bpe;		// buffers per empty buffer

	bufid_t neb;		// next empty buffer
	bufid_t nFb;		// next filling buffer
	bufid_t nfb;		// next full buffer
	bufid_t nEb;		// next emptying buffer

	bufid_t leb;		// last buffer returned by get_empty()
	bufid_t lfb;		// last buffer returned by get_full()

	int error;		// last error to occur

	count_t nempty;
	count_t nemptying;
	count_t nfull;
	count_t nfilling;
	count_t npartial;

	count_t max_empty;	// nbufs / bpe, maximum empty buffers
	count_t max_full;	// nbufs / bpf, maximum fulle buffers

	count_t max_filling;	// max chunks in filling state
	count_t max_emptying;	// max chunks in emptying state
				// if==0 no auto consume

	size_t produced;
	size_t consumed;

	Stopwatch psw;
	Stopwatch csw;

};

class Ring
{
public:
	enum type { Private, Shared };

	Ring( 
		speed_t speed,
		size_t netbufsz,
		int empty_full_ratio,
		Logger& log,
		Ring::type rtype = Private // type of ring to create
	);

	Ring( 
		size_t size, 	// the buffer size in Kbytes
		count_t chunks,	// divide the buffer into this many chunks
		count_t bpe, 	// Number of chunks that constitute an empty 
				// buffer.
		count_t bpf, 	// Number of chunks that constitute a full
				// buffer.
		Ring::type rtype = Private // type of ring to create
	);

	~Ring();

	void alloc( size_t size, count_t chunks, count_t bpe, count_t bpf );

	void clear();

	Buffer* get_empty();
	void now_empty();
	void back_to_empty();

	Buffer* get_filling();
	void set_filling_to_empty();

	Buffer* get_full();
	void now_full( size_t used );
	void back_to_full();

	Buffer* get_emptying();
	void set_emptying_to_full();

	int errno() { return ri->error; }

	count_t max_empty() { return ri->max_empty; }
	count_t max_full() { return ri->max_full; }
	count_t max_emptying() { return ri->max_emptying / ri->bpf; }
	count_t max_filling() { return ri->max_filling / ri->bpe; }

	void set_max_emptying( count_t s );
	void set_max_filling( count_t s );

	count_t num_empty() { return ri->nempty / ri->bpe; }
	count_t num_full() { return ri->nfull / ri->bpf; }

//	count_t empty_size() { return ri->cbsize * 1024 * ri->bpe ; }
//	count_t full_size() { return ri->cbsize * 1024 * ri->bpf; }
	count_t empty_size() { return ri->cbsize * ri->bpe ; }
	count_t full_size() { return ri->cbsize * ri->bpf; }

	bool_t isfull() { return ri->nfull == ri->nbufs; }
	bool_t isempty() { return ri->nempty == ri->nbufs; }

	void pstop() { ri->psw.stop(); }
	void cstop() { ri->csw.stop(); }
	
	double ptp( Time& t, size_t& s )
	{	t = ri->psw.diff();
		s = ri->produced;
		return (double)s / (double)t;
	}

	double ctp( Time& t, size_t& s )
	{	t = ri->csw.diff();
		s = ri->consumed;
		return (double)s / (double)t;
	}

	friend ostream& operator<<( ostream& o, Ring& r );
	friend Ring* alloc_ring( speed_t s, size_t netbufsz, int fd, 
				 int empty_full_ratio );

// don't uncomment yet!
//protected:

	bufid_t get_empty( char** ptr, size_t* size );
	int now_empty( bufid_t bufid );
	void do_now_empty();
	int back_to_empty( bufid_t bufid );

	bufid_t get_full( char** ptr, size_t* size );
	bufid_t next_full();
	int now_full( bufid_t bufid, size_t used );
	int back_to_full( bufid_t bufid );

//	int now_partial( bufid_t bufid, size_t used );
//	int will_fill( bufid_t bufid, size_t used );

private:
	void update_bufstate( Buffer& b, bufstat_t next_state, size_t s = -1 );

	Ring::type rtype;

	long key;
	int shmid;
	char* image;
	size_t image_size;
	RingInfo* ri;

	Semaphore sem;

	char* alloc_image(); };

#endif
