/* $Header: /usr/local/src/nvrToMIT/RCS/VTD.h,v 1.1 1994/05/10 23:29:54 russ Exp $ */

#ifndef __h_VTD__
#define __h_VTD__

/*
 * $Id: VTD.h,v 1.1 1994/05/10 23:29:54 russ Exp $
 *
 * VTD.h
 *
 * Copyright Pencom Software, 1994
 */

#include <Vodoo.h>

typedef long tdhand_t;

const int MaxDecks = 20;

/*
 * This class represents Virtual Tape Deck objects.  It's the lowest level 
 * interface to the Vodoo servers.
 */
class VTD
{
public:
	friend class SingleTD;

	enum cnstate_t { connected, not_connected };
	enum tdstate_t { idle, loaded, playing, paused };


	VTD(
		char* hostname	// hostname of the CBR server to contact
	);
	~VTD();

	err_t load( 
		char* filename,		// i: full pathname of the media to load
		size_t netbufsz,	// i: size of Network buffers
		tdhand_t* td = NULL	// o: handle of the tape deck chosen to
					//    play the media.  If td == NULL, 
					//    then nothing is returned
	);

	err_t swtch(
		tdhand_t td		// i: makes the given tape deck the 
					//    current tape deck
	);

	err_t preview(
		void* buffer,		// o:  buffer to contain preview data
		size_t* len,		// io: length of buffer going in, length
					//     of preview data going out
		offset_t offset = 0	// i:  offset into the file to obtain
					//     the preview data
	);

	err_t catalog(
		char** buffer,		// o: buffer to contain preview data
					//    caller responsible for freeing it
		size_t* len		// o: length of catalog data going out
	);

	tdhand_t current() { return current_td; }

	err_t play(
		speed_t speed		// i: the speed in bytes/second which
					//    the tape will be played
	);

	err_t seek(
		offset_t off		// i: seek to the given absolute byte
					//    offset into the current tape deck
	);

	err_t pause(
		offset_t* off = NULL	// o: pause the current tape deck, and
					//    if off != NULL return the current
					//    byte offset position into the tape
	);

	err_t flow(
		count_t nbufs		// i: changes the flow of buffers from
					//    the server.  (i.e. -5 causes the
					//    server to skip 5 buffer time
					//    intervals, while 4, cause it to
					//    send 4 buffers as fast as possible
	);

	err_t retry(
		count_t seq		// i: causes the server to retry sending
					//    the buffer associated with the
					//    given sequence number
	);

	err_t eject();			// unload the current tape
	err_t dump( char* filename );	// server dumps to filename

	bool_t eom();			// TRUE if end of media has been reached
/*
 * Access member functions
 */
	cnstate_t get_cn_state() { return cn_state; }
	char* get_cn_state_str();
	char* get_hostname() { return hostname; }
	sock_t get_sockfd() { return sockfd; }
	
	tdstate_t get_td_state() { return decks[current_td].state; }
	char* get_td_state_str();
	char* get_td_file() { return decks[current_td].file; }
	speed_t get_td_speed() { return decks[current_td].speed; }
	size_t get_td_netbufsz() { return decks[current_td].netbufsz; }
	sock_t get_td_sock() { return decks[current_td].sockfd; }

	friend ostream& operator<<( ostream&, VTD&  );

private:
	struct SingleTD
	{
		SingleTD() { state = idle; }
		tdstate_t state;
		char file[MAXPATHLEN];
		speed_t speed;
		size_t netbufsz;
		bool_t eom;
		sock_t sockfd;
	};

	SingleTD decks[MaxDecks];	// all the decks in the system
	tdhand_t current_td;		// current tape deck to affect

	cnstate_t cn_state;		// connection state
	char hostname[MAXHOSTNAMELEN];	// name of the contacted server
	sock_t sockfd;
	sockaddr_in serv_addr;	
	servent serv_info;
	hostent host_info;

};


#endif
