/* $Header: /usr/local/src/nvrToMIT/RCS/main.c,v 1.3 1993/01/22 16:37:57 toddb Exp $ */
#include "characterize.h"

void HoldFloat(f)
    float f;
{
}

void HoldInt(i)
{
}

void ShowOpsPerSec(usage, nops, ninstructions, unitSize)
    CpuUsagePtr usage;
    long nops;
{
    double durationInSecs;
    double durationInMicroSecs;
    double opsPerSecond;
    double nsPerOp;
    double nsPerInstr;

    PrintTimeUsage(usage, False);

    durationInMicroSecs = usage->msUser + usage->msSystem;
    if (usage->msElapsed > durationInMicroSecs)
	NormalOutput("  (%.3fs dead time)\n",
	    (usage->msElapsed - durationInMicroSecs) / MicroSecPerSec);
    durationInSecs = durationInMicroSecs / MicroSecPerSec;
    opsPerSecond = ((double)nops) / durationInSecs;
    nsPerOp = (1000 * durationInMicroSecs) / (double)nops;

    switch (unitSize) {
    case OneMillion:
	NormalOutput("  %d megaops, ", nops/OneMillion);
	break;
    case OneThousand:
	NormalOutput("  %d kiloops, ", nops/OneThousand);
	break;
    default:
	NormalOutput("  %d ops, ", nops);
	break;
    }
    if (ninstructions > 1)
	NormalOutput("%d instr/ea., ", ninstructions);
    if (nsPerOp > 1000000)
	NormalOutput("%.2fms/op, ", nsPerOp / 1000000.);
    else if (nsPerOp > 1000)
	NormalOutput("%.2fus/op, ", nsPerOp / 1000.);
    else
	NormalOutput("%.2fns/op, ", nsPerOp);

    if (ninstructions * unitSize > 1) {
	nsPerInstr = (1000 * durationInMicroSecs)
		   / (double)(nops * ninstructions);
	if (nsPerInstr > 1000000)
	    NormalOutput("%.2fms/instr, ", nsPerInstr / 1000000.);
	else if (nsPerInstr > 1000)
	    NormalOutput("%.2fus/instr, ", nsPerInstr / 1000.);
	else
	    NormalOutput("%.2fns/instr, ", nsPerInstr);
    }
    NormalOutput("or %.3f ops/sec\n", opsPerSecond);
}

char *ProgramName = "addf";

void TimeOp(banner, func, nops, unitSize)
    char *banner;
    IntFuncPtr func;
    long nops;
    long unitSize;
{
    int ninstructions;

    NormalOutput(banner);
    StartTimer();
    ninstructions = func(nops);
    ShowOpsPerSec(StopTimer(), nops, ninstructions, unitSize);
}

void TimeDynamicOp(banner, func, basefunc, nops)
    char *banner;
    IntFuncPtr func;
    IntFuncPtr basefunc;
    long nops;
{
    CpuUsage baseUsage, usage;
    int ninstructions = 0;
    double durationInMicroSecs;

    NormalOutput(banner);
    NormalOutput(" base:\n");
    StartTimer();
    (void) (*basefunc)(nops);
    baseUsage = *StopTimer();
    ShowOpsPerSec(&baseUsage, nops, 0, 1);

    NormalOutput(" together:\n");
    StartTimer();
    nops = (*func)(nops, basefunc);
    usage = *StopTimer();
    ShowOpsPerSec(&usage, nops, 0, 1);
    if (usage.msUser >= baseUsage.msUser*10)
	usage.msUser -= baseUsage.msUser*10;
    else
	usage.msUser = 0;
    if (usage.msSystem >= baseUsage.msSystem*10)
	usage.msSystem -= baseUsage.msSystem*10;
    else
	usage.msSystem = 0;
    if (usage.msElapsed >= baseUsage.msElapsed*10)
	usage.msElapsed -= baseUsage.msElapsed*10;
    else
	usage.msElapsed = 0;

    NormalOutput(" adjusted:\n");
    ShowOpsPerSec(&usage, nops, 0, 1);
}

extern long end;
extern int main();
long *EndPtr = &end;
long *StartPtr = (long *)main;

long PullInEverything()
{
    extern long end;
    long *ptr, *last;
    long i = 0, wordCount = 0;

    for (ptr = StartPtr; ptr < EndPtr; ptr++) {
	wordCount++;
	i += *ptr;
    }
    StoreDoubleMem(OneMega);
    return (i);
}

main()
{
    int i;

    PullInEverything();

    TimeDynamicOp("profile timer:\n", ProfileTimer, AddInt, NumAdds);

    ClockResolution();

    TimeOp("time-of-day:\n", TimeOfDay, 10*TenThousand, 10*TenThousand);
    TimeOp("time-of-day:\n", GetPid, 10*TenThousand, 10*TenThousand);

    TimeOp("swap regs 1:\n", SwapRegisterSets1, ThirtyTwoMillion, OneMillion);
    TimeOp("swap regs 2:\n", SwapRegisterSets2, ThirtyTwoMillion, OneMillion);
    TimeOp("swap regs 3:\n", SwapRegisterSets3, ThirtyTwoMillion, OneMillion);
    TimeOp("swap regs 4:\n", SwapRegisterSets4, FourMillion, OneMillion);
    TimeOp("swap regs 5:\n", SwapRegisterSets5, FourMillion, OneMillion);
    TimeOp("swap regs 6:\n", SwapRegisterSets6, FourMillion, OneMillion);
    TimeOp("swap regs 7:\n", SwapRegisterSets7, OneMillion, OneMillion);
    TimeOp("swap regs 8:\n", SwapRegisterSets8, OneMillion, OneMillion);
    TimeOp("swap regs 9:\n", SwapRegisterSets9, OneMillion, OneMillion);

    for (i = 16; i < 96; i += 16) {
	LoadCode(i);
	TimeOp("code cache:\n", RunCode, NumCode, OneThousand);
    }

    TimeOp("mem write 1 non-cache:\n", StoreByteMem, NumMem, OneMillion);
    TimeOp("mem write 1 cache:\n", StoreByteCache, NumMem, OneMillion);
    TimeOp("mem read 1 non-cache:\n", LoadByteMem, NumMem, OneMillion);
    TimeOp("mem read 1 cache:\n", LoadByteCache, NumCacheMem, OneMillion);
    TimeOp("mem write 2 non-cache:\n", StoreShortMem, NumMem, OneMillion);
    TimeOp("mem write 2 cache:\n", StoreShortCache, NumMem, OneMillion);
    TimeOp("mem read 2 non-cache:\n", LoadShortMem, NumMem, OneMillion);
    TimeOp("mem read 2 cache:\n", LoadShortCache, NumCacheMem, OneMillion);
    TimeOp("mem write 4 non-cache:\n", StoreLongMem, NumMem, OneMillion);
    TimeOp("mem write 4 cache:\n", StoreLongCache, NumMem, OneMillion);
    TimeOp("mem read 4 non-cache:\n", LoadLongMem, NumMem, OneMillion);
    TimeOp("mem read 4 cache:\n", LoadLongCache, NumCacheMem, OneMillion);
    TimeOp("mem write 8 non-cache:\n", StoreDoubleMem, NumMem, OneMillion);
    TimeOp("mem write 8 cache:\n", StoreDoubleMem, NumMem, OneMillion);
    TimeOp("mem read 8 non-cache:\n", LoadDoubleMem, NumMem, OneMillion);
    TimeOp("mem read 8 cache:\n", LoadDoubleCache, NumCacheMem, OneMillion);
    TimeOp("subroutine:\n", Subroutine, NumMem, OneMillion);
    TimeOp("shift left logical:\n", ShiftLL, NumAdds, OneMillion);
    TimeOp("integer add:\n", AddInt, NumAdds, OneMillion);
    TimeOp("integer subtraction:\n", SubInt, NumAdds, OneMillion);
    TimeOp("float division:\n", DivFloat, NumDivs, OneMillion);
    TimeOp("double division:\n", DivDouble, NumDivs, OneMillion);
    TimeOp("float multiply:\n", MultFloat, NumMuls, OneMillion);
    TimeOp("double multiply:\n", MultDouble, NumMuls, OneMillion);
    TimeOp("float subtraction:\n", SubFloat, NumFAdds, OneMillion);
    TimeOp("double subtraction:\n", SubDouble, NumFAdds, OneMillion);
    TimeOp("float add:\n", AddFloat, NumFAdds, OneMillion);
    TimeOp("double add:\n", AddDouble, NumFAdds, OneMillion);
    TimeDynamicOp("frame timer (compare w/ int add):\n",
	FrameTimer, AddInt, NumAdds);

}
