/*
 *      (c) Copyright 1992,1993,1994 North Valley Research, Inc.
 *              All Rights Reserved
 *
 *      Confidential -- All proprietary rights reserved.
 *      Licensed Material -- Property of North Valley Research, Inc.
 *
 *      This program is the property of North Valley Research, Inc. or others
 *      from whom North Valley Research has obtained a licensing right, and is
 *      considered by North Valley Research to be confidential.  It is protected
 *      by U.S. copyright law as an unpublished work and is
 *      furnished pursuant to a written license agreement.  It may
 *      not be used, copied or otherwise reproduced, or disclosed to
 *      others except in accordance with the terms and conditions of
 *      that agreement.
 *
 *      NAME
 *              constantdef.c
 *
 *      DESCRIPTION
 *              <overall description of what the package does>
 *
 *      REVISION
 *              $Header: /usr/local/src/nvrToMIT/RCS/constantdef.c,v 4.5 1994/03/05 00:46:00 russ Exp $
 */

#define NeedFunctionPrototypes 1

#include "osio.h"

#define True	1
#define False	0

char	*ProgramName;
char LeftComment[3] = { '/', '*', '\0' };
char RightComment[3] = { '*', '/', '\0' };

typedef struct _bitCluster {
    int offset;
    int width;
} BitCluster;

BitCluster Clusters[16];

double atof();

void Usage()
{
    Msg("Usage: %s fbits pbits\n", ProgramName);
    Msg("where fbits is the number of fraction bits in a\n");
    Msg("fixed-point number and pbits is the number of fraction\n");
    Msg("bits to use from the input numbers to perform a multiplication.\n");
    Msg("Input is expected on standard input in the form:\n");
    Msg("  name number\n");
    Msg("where number is a floating point number, and name is the\n");
    Msg("base used to form the defined name of constants and macros\n");
    exit (1);
}

char *NtoBinary(int value, int stringLen, int nBits)
{
    int i, bitsPrinted;
    static char buf[ 1024 ];
    char *p;

    sprintf(buf, "Binary%d(", stringLen);
    p = buf + strlen(buf);
    bitsPrinted = 0;
    for (i = stringLen-1; i >= 0; i--) {
	*p++ = '0' + ((value & (1<<i)) != 0);
	if ((++bitsPrinted & 0x3) == 0)
	    *p++ = ',';
    }
    if (bitsPrinted & 0x3)
	*p++ = ',';
    *p = '\0';

    sprintf(p, " %d)", nBits);
    return (buf);
}

void
main(int argc, char **argv)
{
    int     likeClusters, clusterCount[32];
    int     i;
    int     items, value, mask;
    int	    nBits, stringLength, newShift;
    int	    fbits, fractionBits, precisionBits;
    ULong   intPart, fracPart;
    int     intBits;
    double  number;
    char    name[ 100 ], numberString[ 1024 ];
    BitCluster *pCluster;
    int	    sgiMult = 0;

    ProgramName = *argv;

    switch(argc) {
    case 3:
	fractionBits = atoi(argv[1]);
	precisionBits = atoi(argv[2]);
	break;
    case 4:
	if( strcmp(argv[1],"-sgiMult") != 0 ) {
	    Usage();
	}
	sgiMult++;
	fractionBits = atoi(argv[2]);
	precisionBits = atoi(argv[3]);
	break;
    default:
	Usage();
    }

    if (precisionBits > fractionBits)
	Fatal("# of fraction bits (%d) must be >= precision bits(%d)\n",
	    fractionBits, precisionBits);
    NormalOutput("#include \"binarybits.h\"\n\n");
    while ((items = scanf(" %s %s\n", name, numberString)) != -1) {
	number = atof(numberString);
	if (number < 0)
	    Fatal("can't express negative numbers (too messy)\n");
	intPart = number;
	fracPart = (number - intPart) * (1<<precisionBits);
	fracPart <<= fractionBits-precisionBits;

	for (intBits = 31; intBits >= 0; intBits--) {
	    if ((1<<intBits) & intPart)
		break;
	}
	intBits++;

	if ((nBits = intBits + fractionBits) > 32)
	    Fatal("%f with %d fraction bits is bigger than 32 bits\n",
		number, fractionBits);

	/*
	 * increase the bit string length so that the decimal point falls
	 * on a comma (this adds leading 0 bits).
	 */
	if (nBits > 0 && nBits < 32 && (intBits&0x3))
	    nBits += 4 - (intBits&0x3);

	/*
	 * decrease the fraction part (and increase the shift) if 
	 * their are trailing 0 bits.
	 */
	stringLength = nBits;
	fbits = fractionBits;
	value = (intPart<<fractionBits) + fracPart;
	while (fbits > 0) {
	    if (fbits & 0x3)
		newShift = fbits & 0x3;
	    else
		newShift = 4;
	    mask = (1<<newShift) - 1;
	    if (value & mask)
		break;
	    value >>= newShift;
	    stringLength -= newShift;
	    fbits -= newShift;
	}

	NormalOutput("#define Real%s	%s\n", name, numberString);
	NormalOutput("#define Fixed%s	%s\n", name,
	    NtoBinary(value, stringLength, nBits));

	/*
	 * Calculate the most efficient method of multiplication for
	 * this constant.
	 */
	nBits = intBits + fractionBits;
	value = (intPart<<fractionBits) + fracPart;
	pCluster = Clusters;
	for (i = 0; i < 31; i++)
	    clusterCount[ i ] = 0;
	likeClusters = 0;
	for (i=nBits-1; i>=0; i--) {
	    pCluster->width = 0;
	    while (value & (1<<i)) {
		pCluster->width++;
		i--;
	    }
	    if (pCluster->width) {
		pCluster->offset = i+1-fractionBits;
		if (++clusterCount[ pCluster->width ] > 1)
		    likeClusters++;
		pCluster++;
	    }
	}
	pCluster->width = 0;	/* set the end condition */

	NormalOutput("#define %s_Times(storage, value, tmp) \\\n", name);
	if(sgiMult) {
	    sgiCalc(name, fractionBits, precisionBits);
	} else {
	    defaultCalc(clusterCount);
	}
    }

    exit(0);
}

sgiCalc(char *operand, int fractionBits, int precisionBits)
{
    NormalOutput("\t\tstorage = (value * Fixed%s)>>%d;\n\n",
		 operand, fractionBits);
}

defaultCalc(int *clusterCount)
{
    BitCluster *pCluster;
    boolean computed, assigned;
    boolean addStarted;
    int top, i, offsetOfTmp, width;

	/* pass 1: do the multiple bit clusters */
	assigned = False;
	for (i = 2; i < 31; i++) {
	    if (clusterCount[i] > 1) {
		computed = False;
		for (pCluster = Clusters; pCluster->width; pCluster++) {
		    if (pCluster->width != i)
			continue;
		    top = pCluster->offset + pCluster->width;
		    if (! computed) {
			NormalOutput("\t\ttmp = (value");
			if (top != 0)
			    NormalOutput("%s%d", top > 0 ? "<<" : ">>", abs(top));
			top -= pCluster->width;
			NormalOutput(")-(value");
			if (top != 0)
			    NormalOutput("%s%d",
				top > 0 ? "<<" : ">>", abs(top));
			NormalOutput(");\t%s %d bit%s @ %d%s \\\n",
			    LeftComment,
			    i, i > 1 ? "s" : "", pCluster->offset,
			    RightComment);
			NormalOutput("\t\tstorage %s= tmp",
			    assigned ? "+" : "");
			offsetOfTmp = pCluster->offset;
		    } else {
			NormalOutput(" \\\n\t\t\t+ (tmp>>%d) %s %d bit%s @ %d%s",
			    offsetOfTmp - pCluster->offset,
			    LeftComment,
			    i, i > 1 ? "s" : "", pCluster->offset,
			    RightComment);
		    }
		    assigned = computed = True;
		}
		NormalOutput("; \\\n");
		clusterCount[i] = 0;
	    }
	}

	/* pass 2: do what's left */
	NormalOutput("\t\tstorage ");
	addStarted = False;
	for (i = 1; i < 31; i++) {
	    if (clusterCount[i] == 0)
		continue;
	    clusterCount[i] = 0;
	    for (pCluster = Clusters; pCluster->width; pCluster++) {
		if ((width = pCluster->width) != i)
		    continue;

		top = pCluster->offset + width;

		/* one and two bit quantities are not done with subtraction */
		if (width <= 2) {
		    if (width == 2)
			pCluster->offset++;
		    width = 1;
		    top--;
		}

	    anotherBit:
		if (addStarted)
		    NormalOutput(" \\\n\t\t\t+");
		NormalOutput("%s (value", addStarted ? "" :
					assigned ? "+=" : "=");

		if (top != 0)
		    NormalOutput("%s%d", top > 0 ? "<<" : ">>", abs(top));
		if (width > 2) {
		    top -= width;
		    NormalOutput(")-(value");
		    if (top != 0)
			NormalOutput("%s%d", top > 0 ? "<<" : ">>", abs(top));
		}
		NormalOutput(")\t%s %d bit%s @ %d%s",
		    LeftComment,
		    width, width > 1 ? "s" : "", pCluster->offset,
		    RightComment);
		addStarted = assigned = True;

		if (pCluster->width == 2) {
		    top--;
		    pCluster->width--;
		    pCluster->offset--;
		    goto anotherBit;
		}
	    }
	}

	NormalOutput(" \\\n\t\t\t;\n\n");
}
