/*	Copyright (c) 1984, 1985, 1986, 1987 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*	cscope - interactive C symbol or text cross-reference
 *
 *	command functions
 */

/*
 * $Header: /u5/alt/tools/src/cscope/RCS/command.c,v 1.8 1989/11/01 14:11:55 kimr Exp $
 *
 * $Log: command.c,v $
 * Revision 1.8  1989/11/01  14:11:55  kimr
 * Added ^A (function name abbreviation) and ^T (function type information)
 * handling.  Made launching editor automatically optional.
 *
 * Revision 1.7  89/10/26  18:23:32  kimr
 * Added ^F command to toggle between full file name and basename mode.
 * The search command is repeated after the mode is toggled.
 * 
 * Revision 1.6  89/10/26  15:09:09  kimr
 * Modified "Change this text string" to allow you to abort at the "To:" prompt
 * by pressing ESC.  As a side effect, ESC will abort any pattern input in 
 * progress which might tick off someone used to using ESC for command/filename
 * completion.
 * 
 * Revision 1.5  89/07/21  17:59:32  kimr
 * Fixed "!" command to always wait on shell.  Replaced funky '-'
 * routine with two calls to backpage().  Backwards paging now
 * stops at the top of the reference list.
 * 
 * Revision 1.4  89/07/18  17:24:45  kimr
 * Made a pass at cleaning up lint errors and added RCS headers.
 * 
 *
 */

#include "global.h"
#include <ctype.h>	/* isdigit */
#include <fcntl.h>	/* O_RDONLY */

BOOL	*change;		/* change this line */
BOOL	changing = NO;		/* changing text */
char	newpat[PATLEN + 1];	/* new pattern */
char	pattern[PATLEN + 1];	/* symbol or text pattern */

BOOL	changestring(), search();
void	atchange(), editall(), editref(), help(), mark(), scrollbar();

/* execute the command */

BOOL
command(c)
register int	c;
{
	char	msg[MSGLEN + 1];	/* message */
	char	filename[PATHLEN + 1];	/* file path name */
	MOUSE	*p;			/* mouse data */
	register int	i;
	register FILE	*file;

	switch (c) {

	case ctrl('A'):	/* toggle full/short function name display */
		fullfuncname = !fullfuncname;
		backpage();         /* back nextline up to current page */
		seekline(nextline); /* reset refs file to current page */
		return(YES); 	    /* cause to redisplay in new format */
		break;
		
	case ctrl('F'):	/* toggle full/base file name display */
		fullfilename = !fullfilename;
		backpage();         /* back nextline up to current page */
		seekline(nextline); /* reset refs file to current page */
		return(YES); 	    /* cause to redisplay in new format */
		break;

	case ctrl('R'):	/* rebuild the cross reference */
		if (isuptodate == YES) {
			putmsg("The -d option prevents rebuilding the cross-reference");
			return(NO);
		}
		else {	/* remake the source file list */
			while (nsrcfiles > 0) {
				free(srcfiles[--nsrcfiles]);
				free(srcnames[nsrcfiles]);
			}
			makefilelist();
	
			/* rebuild the cross-reference */
			close(symrefs);
			endwin();	/* restore terminal modes */
			build();
			if ((symrefs = open(reffile, O_RDONLY, 0)) == -1) {
				sprintf(msg, "Cannot open file: %s", reffile);
				putmsg(msg);
				errorsfound = YES;
			}
			if (errorsfound == YES) {
				errorsfound = NO;
				askforchar();
			}
			/* revert to the initial display */
			if (refsfound != NULL) {
				fclose(refsfound);
				refsfound = NULL;
			}

			/* Reinitialize the curses display package */
			initscr();
			nonl();
			cbreak();
			noecho();

			clear();
		}
		break;

	case ctrl('T'):	/* toggle display of fcn return type on prev line */
		prevlinetoo = !prevlinetoo;
		if (field == DEFINITION) { /* repeat search to rebuild refs */
		    autolaunch = NO; /* turn off for redisplay */
		    if (*pattern != '\0') { 
			    addstr(pattern);
			    goto repeat;
		    }
		}
		else {
			return(NO); /* don't update display */
		}
		break;

	case ctrl('X'):	/* mouse selection */
		if ((p = getmouse()) == NULL) {
			return(NO);	/* unknown control sequence */
		}
		/* if the button number is a scrollbar tag */
		if (p->button == '0') {
			scrollbar(p);
			break;
		} 
		/* ignore a sweep */
		if (p->x2 >= 0) {
			return(NO);
		}
		/* if this is a line selection */
		if (p->y1 < FLDLINE - 1) {

			/* find the selected line */
			/* note: the selection is forced into range */
			for (i = disprefs - 1; i > 0; --i) {
				if (p->y1 >= displine[i]) {
					break;
				}
			}
			/* display it in the file with the editor */
			editref(i);
		}
		else {	/* this is an input field selection */
			field = p->y1 - FLDLINE;
			
			/* force it into range */
			if (field < 0) {
				field = 0;
			}
			else if (field >= FIELDS) {
				field = FIELDS - 1;
			}
			setfield();
			return(NO);
		}
		break;

	case '\t':	/* go to next input field */
	case '\n':
	case '\r':
	case ctrl('N'):
#if TERMINFO
	case KEY_DOWN:
	case KEY_RIGHT:
#endif
		field = (field + 1) % FIELDS;
		setfield();
		return(NO);

	case ctrl('P'):	/* go to previous input field */
#if TERMINFO
	case KEY_UP:
	case KEY_LEFT:
#endif
		field = (field + (FIELDS - 1)) % FIELDS;
		setfield();
		return(NO);
#if TERMINFO
	case KEY_HOME:	/* go to first input field */
		field = 0;
		setfield();
		return(NO);

	case KEY_LL:	/* go to last input field */
		field = FIELDS - 1;
		setfield();
		return(NO);
#endif
	case ' ':	/* display next page */
	case '+':
	case ctrl('V'):
#if TERMINFO
	case KEY_NPAGE:
#endif
		/* don't redisplay if there are no lines */
		if (totallines == 0) {
			return(NO);
		}
		/* note: seekline() is not used to move to the next 
		 * page because display() leaves the file pointer at
		 * the next page to optimize paging through refs
		 */
		break;

	case '-':	/* display previous page */
#if TERMINFO
	case KEY_PPAGE:
#endif
		/* don't redisplay if there are no lines */
		if (totallines == 0) {
			return(NO);
		}
		/* go back two pages because already at next page */
		backpage();
		backpage();
		seekline(nextline);
		break;

	case '>':	/* append the lines to a file */
		if (totallines == 0) {
			putmsg("There are no lines to print");
		}
		else {	/* get the file name */
			move(PRLINE, 0);
			addstr("Append to file: ");
			if (getline(newpat, sizeof(newpat), '\0') > 0) {

				/* open the file */
				shellpath(filename, sizeof(filename), newpat);
				if ((file = fopen(filename, "a")) == NULL) {
					sprintf(msg, "Cannot open file: %s", filename);
					putmsg(msg);
				}
				else {	/* append the lines */
					seekline(1);
					while ((c = getc(refsfound)) != EOF) {
						putc(c, file);
					}
					seekline(topline);
					fclose(file);
				}
			}
			/* clear the file name prompt */
			move(PRLINE, 0);
			clrtoeol();
		}
		/* return to the previous field */
		return(NO);

	case ctrl('L'):	/* redraw screen */
#if TERMINFO
	case KEY_CLEAR:
#endif
		clearok(curscr, TRUE);
		wrefresh(curscr);
		draw_scrollbar(topline, bottomline);
		return(NO);

	case '!':	/* shell escape */
#if BSD
		move(LINES - 1, 0);	/* to lower left corner */
		clrtoeol();		/* clear bottom line */
		refresh();
#endif
		execute(NO, shell, shell, (char *) 0);
		clear();
		seekline(topline);
		break;

	case '?':	/* help */
		clear();
		help();
		addch('\n');
		askforchar();
		clear();
		seekline(topline);
		break;

	case ctrl('E'):	/* edit all lines */
		editall();
		break;

	case '.':	/* repeat last pattern */
	case ctrl('Y'):
		if (*pattern != '\0') {
			addstr(pattern);
			goto repeat;
		}
		break;

	case '\\':	/* next character is not a command */
	case ctrl('Q'):
		addch('\\');	/* display the quote character */

		/* get a character from the terminal */
		if ((c = mygetch()) == EOF) {
			return(NO);	/* quit */
		}
		addstr("\b \b");	/* erase the quote character */
		goto ispat;

	default:
		/* edit a selected line */
		if (isdigit(c) && c != '0' && mouse == NO) {
			editref(c - '1');
		}
		/* if this is the start of a pattern */
		else if (isprint(c)) {
	ispat:		if (getline(newpat, sizeof(newpat), c) > 0) {
				(void) strcpy(pattern, newpat);
	repeat:
				if (field == CHANGE) {
					
					/* prompt for the new text */
					move(PRLINE, 0);
					addstr("To: (press ESC to abort) ");
					getline(newpat, sizeof(newpat), '\0');
					if (newpat[0] == ESC) { /* aborted */
					    move(PRLINE, 0);
					    clrtoeol();
					    return(YES); /* redisplay */
					}
				}
				/* search for the pattern */
				if (search() == YES) {
					switch (field) {
					case DEFINITION:
					case FILENAME:
						if (totallines > 1) {
							break;
						}
						topline = 1;
						if (autolaunch) {
						    editref(0);
						}
						break;
					case CHANGE:
						return(changestring());
					}
				}
				/* try to edit the file anyway */
				else if (field == FILENAME && 
				    access(newpat, READ) == 0) {
					edit(newpat, "1");
				}
			}
			else {	/* no pattern--the input was erased */
				return(NO);
			}
		}
		else {	/* control character */
			return(NO);
		}
	}
	return(YES);
}
/* change one text string to another */

static	BOOL
changestring()
{
	char	newfile[PATHLEN + 1];	/* new file name */
	char	oldfile[PATHLEN + 1];	/* old file name */
	char	linenum[NUMLEN + 1];	/* file line number */
	char	msg[MSGLEN + 1];	/* message */
	FILE	*script;		/* shell script file */
	BOOL	anymarked = NO;		/* any line marked */
	MOUSE	*p;			/* mouse data */
	int	c, i;
	char	*s;

	/* open the temporary file */
	if ((script = fopen(temp2, "w")) == NULL) {
		putmsg("Cannot open temporary file");
		return(NO);
	}
	/* create the line change indicators */
	change = (BOOL *) mycalloc(totallines, sizeof(BOOL));
	changing = YES;
	mousemenu();

	/* until the quit command is entered */
	for (;;) {
		/* display the current page of lines */
		display();
	same:
		atchange();
		
		/* get a character from the terminal */
		if ((c = mygetch()) == EOF || c == ctrl('D') || c == ctrl('Z')) {
			break;	/* change lines */
		}
		/* see if the input character is a command */
		switch (c) {
		case ' ':	/* display next page */
		case '+':
		case ctrl('V'):
#if TERMINFO
		case KEY_NPAGE:
#endif
		case '-':	/* display previous page */
#if TERMINFO
		case KEY_PPAGE:
#endif
		case '!':	/* shell escape */
		case '?':	/* help */
			command(c);
			break;

		case ctrl('L'):	/* redraw screen */
#if TERMINFO
		case KEY_CLEAR:
#endif
			command(c);
			goto same;

		case ESC:	/* don't change lines */
		case ctrl('G'):
			goto nochange;

		case '*':	/* mark/unmark all displayed lines */
			for (i = 0; topline + i < nextline; ++i) {
				mark(i);
			}
			goto same;

		case 'a':	/* mark/unmark all lines */
			for (i = 0; i < totallines; ++i) {
				if (change[i] == NO) {
					change[i] = YES;
				}
				else {
					change[i] = NO;
				}
			}
			/* show that all have been marked */
			seekline(totallines);
			break;

		case ctrl('X'):	/* mouse selection */
			if ((p = getmouse()) == NULL) {
				goto same;	/* unknown control sequence */
			}
			/* if the button number is a scrollbar tag */
			if (p->button == '0') {
				scrollbar(p);
				break;
			}
			/* find the selected line */
			/* note: the selection is forced into range */
			for (i = disprefs - 1; i > 0; --i) {
				if (p->y1 >= displine[i]) {
					break;
				}
			}
			mark(i);
			goto same;
		default:
			/* if a line was selected */
			if (isdigit(c) && c != '0' && mouse == NO) {
				mark(c - '1');
			}
			goto same;
		}
	}
	/* for each line containing the old text */
	fprintf(script, "ed - <<\\!\n");
	*oldfile = '\0';
	seekline(1);
	for (i = 0; fscanf(refsfound, "%s%*s%s%*[^\n]", newfile, linenum) == 2;
	    ++i) {
		/* see if the line is to be changed */
		if (change[i] == YES) {
			anymarked = YES;
		
			/* if this is a new file */
			if (strcmp(newfile, oldfile) != 0) {
				
				/* make sure it can be changed */
				if (access(newfile, WRITE) != 0) {
					sprintf(msg, "Cannot write to file %s", newfile);
					putmsg(msg);
					anymarked = NO;
					break;
				}
				/* if there was an old file */
				if (*oldfile != '\0') {
					fprintf(script, "w\n");	/* save it */
				}
				/* edit the new file */
				(void) strcpy(oldfile, newfile);
				fprintf(script, "e %s\n", oldfile);
			}
			/* output substitute command */
			fprintf(script, "%ss/", linenum);	/* change */
			for (s = pattern; *s != '\0'; ++s) {	/* old text */
				if (strchr("/\\[.^*$", *s) != NULL) {
					putc('\\', script);
				}
				putc(*s, script);
			}
			putc('/', script);			/* to */
 			for (s = newpat; *s != '\0'; ++s) {	/* new text */
				if (strchr("/\\&", *s) != NULL) {
					putc('\\', script);
				}
				putc(*s, script);
			}
			fprintf(script, "/gp\n");		/* and print */
		}
	}
	fprintf(script, "w\nq\n!\n");	/* write and quit */
	fclose(script);

	/* if any line was marked */
	if (anymarked == YES) {
		
		/* edit the files */
		move(FLDLINE, 0);
		addstr("Changed lines:\n\r");
		clrtobot();
		refresh();
		execute(NO, "sh", "sh", temp2, (char *) 0);
		askforchar();
		clear();
		seekline(1);
	}
	else {
nochange:
		move(PRLINE, 0);
		clrtoeol();
	}
	changing = NO;
	mousemenu();
	free((char *) change);
	return(anymarked);
}

/* mark/unmark this displayed line to be changed */

static	void
mark(i)
int	i;
{
	int	j;
	
	j = i + topline - 1;
	if (j < totallines) {
		move(displine[i], 1);
		if (change[j] == NO) {
			change[j] = YES;
			addch('>');
		}
		else {
			change[j] = NO;
			addch(' ');
		}
	}
}

/* scrollbar actions */

void
scrollbar(p)
MOUSE	*p;
{
	/* reposition list if it makes sense */
	if (totallines == 0) {
		return;
	}
	switch (p->percent) {
		
	case 101: /* scroll down one page */
		if (nextline + mdisprefs > totallines) {
			nextline = totallines - mdisprefs + 1;
		}
		break;
		
	case 102: /* scroll up one page */
		nextline = topline - mdisprefs;
		if (nextline < 1) {
			nextline = 1;
		}
		break;

	case 103: /* scroll down one line */
		nextline = topline + 1;
		break;
		
	case 104: /* scroll up one line */
		if (topline > 1) {
			nextline = topline - 1;
		}
		break;
	default:
		nextline = p->percent * totallines / 100;
	}
	seekline(nextline);
}
