#ifndef LINT
#ifdef RCS_ID
static char *rcsid = "@(#)$Header: /u5/alt/tools/src/cscope/RCS/crossref.c,v 1.3 1991/10/29 18:05:42 marvinw Exp $";
#endif /* RCS_ID */
#endif /* LINT */
/*	Copyright (c) 1984, 1985, 1986, 1987 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*	cscope - interactive C symbol cross-reference
 *
 *	build cross-reference file
 */

#include <ctype.h>	/* isalpha */
#include "global.h"

BOOL	errorsfound = NO;	/* prompt before clearing messages */
int	symbols;		/* number of symbols */

static	char	*filename;	/* file name for warning messages */
static	struct	{		/* symbol data */
	int	type;		/* type */
	int	first;		/* index of first character in text */
	int	last;		/* index of last+1 character in text */
	int	length;		/* symbol length */
} symbol[SYMBOLMAX];

void
crossref(srcfile)
char	*srcfile;
{
	register int	i;
	register int	length;		/* symbol length */
	int	token;			/* current token */
	BOOL	too_many = NO;		/* too many symbols in a line */
	char	*s;
	void	beginbp(), putcrossref(), resetscanner();

	/* open the source file */
	if ((yyin = fopen(srcfile, "r")) == NULL) {
		fprintf(stderr, "cscope: cannot open file %s\n", srcfile);
		errorsfound = YES;
		return;
	}
	resetscanner();	/* minimize carry-over of scanner problem */

	/* see if this is not a C file */
	if ((s = strrchr(srcfile, '.')) != NULL) {
		switch (*++s) {	/* this switch saves time on C files */
		case 'b':
			if (strcmp(s, "bp") == 0) {	/* breakpoint listing */
				beginbp();
			}
			break;
		case 'l':
			if (strcmp(s, "l") == 0) {	/* lex */
				beginlex();
			}
			break;
		case 's':
			if (strcmp(s, "sd") == 0) {	/* sdl */
				beginsdl();
			}
			break;
		}
	}
	filename = srcfile;	/* save the file name for warning messages */
	putfilename(srcfile);	/* output file name */
	fprintf(newrefs, "\n\n");

	/* read the source file */
	symbols = 0;
	for (;;) {
		/* get the next token */
		switch (token = yylex()) {
		default:
			/* see if the symbol is already in the list */
			length = last - first;
			for (i = 0; i < symbols; ++i) {
				if (length == symbol[i].length) {
					if (strncmp(yytext + first,
					    yytext + symbol[i].first, length) == 0) {
						break;
					}
				}
			}
			if (i == symbols) {	/* if not already in list */

				/* save the symbol if there is room */
				if (symbols < SYMBOLMAX) {
					symbol[symbols].type = token;
					symbol[symbols].first = first;
					symbol[symbols].last = last;
					symbol[symbols].length = length;
					++symbols;
				}
				else {	/* too many symbols */
					too_many = YES;
				}
			}
			break;
		case NEWLINE:	/* end of line */
			--yyleng;	/* remove the newline */
			
			/* adjust the line number for the \n; restore later */
			--yylineno;
			
		case LEX_EOF:	/* end of file; last line may not have \n */
			
			/* check for too many symbols */
			if (too_many == YES) {
				too_many = NO;
				warning("too many symbols in this line");
			}
			/* output the symbols and source line */
			if (symbols > 0) {
				putcrossref();
				symbols = 0;
			}
			/* check for end of file */
			if (token == LEX_EOF) {
				fclose(yyin);	/* close the source file */
				yylineno = 1;	/* reset the line count */

				/* output the leading tab expected by the next call */
				putc('\t', newrefs);
				return;
			}
			/* restore the line number */
			++yylineno;
			break;
		}
	}
}

/* output the file name */

void
putfilename(srcfile)
char	*srcfile;
{
	/* check for disk out of space */
	if (fprintf(newrefs, "%c%s", NEWFILE, srcfile) < 0) {
		perror("cscope: write failed");
		fclose(newrefs);
		unlink(newreffile);
		myexit(1);
	}
}
/* output the symbols and source line */

static void
putcrossref()
{
	register int	i, j, c;
	register BOOL	blank;		/* blank indicator */
	register int	symput = 0;	/* symbols output */

	/* output the source line */
	fprintf(newrefs, "%d ", yylineno);
	blank = NO;
	for (i = 0; i < yyleng; ++i) {
		
		/* change a tab to a blank and compress blanks */
		if ((c = yytext[i]) == ' ' || c == '\t') {
			blank = YES;
		}
		/* look for the start of a symbol */
		else if (symput < symbols && i == symbol[symput].first) {

			/* check for compressed blanks */
			if (blank == YES) {
				blank = NO;
				putc(' ', newrefs);
			}
			putc('\n', newrefs);	/* symbols start on a new line */
			
			/* output any symbol type */
#ifdef DEBUG
{ int ii;
switch ( symbol[symput].type) {
case IDENT: {printf("IDENT: "); break;}
case FCNDEF: {printf("FCNDEF: "); break;}
case DEFINE: {printf("DEFINE: "); break;}
case FCNCALL: {printf("FCNCALL: "); break;}
case INCLUDE: {printf("INCLUDE: "); break;}
case NEWFILE: {printf("NEWFILE: "); break;}
case NEWLINE: {printf("NEWLINE: "); break;}
case KEYWORD: {printf("KEYWORD: "); break;}
case LEX_EOF: {printf("LEX_EOF: "); break;}
}
for ( ii = i; ii < symbol[symput].last; ++ii) {
	putchar(yytext[ii]);
}
putchar('\n');
}
#endif
			if (symbol[symput].type != IDENT) {
				putc('\t', newrefs);
				putc(symbol[symput].type, newrefs);
			}
			/* output the symbol */
			for ( ; i < symbol[symput].last; ++i) {

				/* compress digraphs */
				c = yytext[i];
				if (dicode1[c] && dicode2[yytext[i + 1]] &&
				    i < symbol[symput].last - 1) {
					c = (0200 - 2) + dicode1[c] + dicode2[yytext[i + 1]];
					++i;
				}
				putc(c, newrefs);
			}
			putc('\n', newrefs);
			++symput;
			--i;
		}
		else {
			/* check for compressed blanks */
			if (blank == YES) {
				if (dicode2[c]) {
					c = (0200 - 2) + dicode1[' '] + dicode2[c];
				}
				else {
					putc(' ', newrefs);
				}
			}
			/* compress digraphs */
			else if (dicode1[c] && (j = dicode2[yytext[i + 1]]) != 0 && 
			    symput < symbols && i + 1 != symbol[symput].first) {
				c = (0200 - 2) + dicode1[c] + j;
				++i;
			}
			putc(c, newrefs);
			blank = NO;
			
			/* skip compressed characters */
			if (c < ' ') {
				++i;
				
				/* skip blanks before a preprocesor keyword */
				while (yytext[i] == ' ' || yytext[i] == '\t') {
					++i;
				}
				/* skip the rest of the keyword */
				while (isalpha(yytext[i])) {
					++i;
				}
				/* skip space after certain keywords */
				if (keyword[c].delim != '\0') {
					/* note: don't use isspace because \f and \v
					   are used for keywords */
					while (yytext[i] == ' ' || yytext[i] == '\t') {
						++i;
					}
				}
				/* skip a '(' after certain keywords */
				if (keyword[c].delim == '(' && yytext[i] == '(') {
					++i;
				}
				--i;	/* compensate for ++i in for() */
			}
		}
	}
	/* ignore trailing blanks */
	fprintf(newrefs, "\n\n");
}

/* print a warning message with the file name and line number */

void
warning(text)
char *text;
{
	fprintf(stderr, "cscope: \"%s\", line %d: warning: %s\n", filename, 
		yylineno, text);
	errorsfound = YES;
}
