/*	Copyright (c) 1984, 1985, 1986, 1987 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*	cscope - interactive C symbol cross-reference
 *
 *	directory searching functions
 */

/*
 * $Header: /u5/alt/tools/src/cscope/RCS/dir.c,v 1.4 1990/02/12 11:04:21 jrb Exp $
 *
 * $Log: dir.c,v $
 * Revision 1.4  1990/02/12  11:04:21  jrb
 * added ifdef to handle UTEK5 difference in directory(3) routines
 *
 * Revision 1.3  89/11/21  16:55:35  kimr
 * Added recognition of ".cc" extension as a source file.
 * 
 * Revision 1.2  89/07/18  17:25:15  kimr
 * Made a pass at cleaning up lint errors and added RCS headers.
 * 
 *
 */

#include <sys/types.h>	/* needed by stat.h */
#include <sys/stat.h>	/* stat */
#include "global.h"
#include "ndir.h"	/* direct and directory access macros */
#include "vp.h"		/* vpdirs and vpndirs */

#define	DIRSEPS	" ,:"	/* directory list separators */
#define	SRCINC	1000	/* source file list size increment */
			/* note: 15000 files have been tried */

char	currentdir[PATHLEN + 1];/* current directory */
char	**incdirs;		/* #include directories */
char	**srcdirs;		/* source directories */
char	**srcfiles;		/* source files */
char	**srcnames;		/* source file names without view pathing */
int	nincdirs;		/* number of #include directories */
int	nsrcdirs;		/* number of source directories */
int	nsrcfiles;		/* number of source files */
int	msrcfiles = SRCINC;	/* maximum number of source files */

static	char	**incnames;	/* #include directory names without view pathing */
static	int	nvpsrcdirs;	/* number of view path source directories */

BOOL	infilelist(), issrcfile();
char	*inviewpath();
void	addsrcdir(), addincdir();

/* make the view source directory list */

void
makevpsrcdirs()
{
	int	i;

	/* return if this function has already been called */
	if (nsrcdirs > 0) {
		return;
	}
	/* get the current directory name */
	if (getwd(currentdir) == NULL) {
		fprintf(stderr, "cscope: warning: cannot get current directory name\n");
		(void) strcpy(currentdir, "<unknown>");
	}
	/* see if there is a view path and this directory is in it */
	vpinit(currentdir);
	if (vpndirs > 1) {
		nsrcdirs = vpndirs;
	}
	else {
		nsrcdirs = 1;
	}
	/* create the source directory list */
	srcdirs = (char **) mymalloc(nsrcdirs * sizeof(char *));
	*srcdirs = ".";	/* first source dir is always current dir */
	for (i = 1; i < vpndirs; ++i) {
		srcdirs[i] = vpdirs[i];
	}
	/* save the number of original source directories in the view path */
	nvpsrcdirs = nsrcdirs;
}

/* add a source directory to the list for each view path source directory */

void
sourcedir(dirlist)
char	*dirlist;
{
	char	path[PATHLEN + 1];
	register char	*dir;
	register int	i;

	makevpsrcdirs();		/* make the view source directory list */
	dirlist = stralloc(dirlist);	/* don't change environment variable text */
	
	/* parse the directory list */
	dir = strtok(dirlist, DIRSEPS);
	while (dir != NULL) {
		addsrcdir(dir);

		/* if it isn't a full path name and there is a 
		   multi-directory view path */
		if (*dirlist != '/' && vpndirs > 1) {
			
			/* compute its path from higher view path source dirs */
			for (i = 1; i < nvpsrcdirs; ++i) {
				sprintf(path, "%s/%s", srcdirs[i], dir);
				addsrcdir(path);
			}
		}
		dir = strtok((char *) NULL, DIRSEPS);
	}
}

/* add a source directory to the list */

static	void
addsrcdir(dir)
char	*dir;
{
	struct	stat	statstruct;

	/* make sure it is a directory */
	if (stat(compath(dir), &statstruct) == 0 && 
	    (statstruct.st_mode & S_IFDIR)) {

		/* note: there already is a source directory list */
		srcdirs = (char **) myrealloc(srcdirs, (nsrcdirs + 1) * sizeof(char *));
		srcdirs[nsrcdirs++] = stralloc(dir);
	}
}

/* add a #include directory to the list for each view path source directory */

void
includedir(dirlist)
char	*dirlist;
{
	char	path[PATHLEN + 1];
	register char	*dir;
	register int	i;

	makevpsrcdirs();		/* make the view source directory list */
	dirlist = stralloc(dirlist);	/* don't change environment variable text */
	
	/* parse the directory list */
	dir = strtok(dirlist, DIRSEPS);
	while (dir != NULL) {
		addincdir(dir, dir);

		/* if it isn't a full path name and there is a 
		   multi-directory view path */
		if (*dirlist != '/' && vpndirs > 1) {
			
			/* compute its path from higher view path source dirs */
			for (i = 1; i < nvpsrcdirs; ++i) {
				sprintf(path, "%s/%s", srcdirs[i], dir);
				addincdir(dir, path);
			}
		}
		dir = strtok((char *) NULL, DIRSEPS);
	}
}

/* add a #include directory to the list */

static	void
addincdir(name, path)
char	*name;
char	*path;
{
	struct	stat	statstruct;

	/* make sure it is a directory */
	if (stat(compath(path), &statstruct) == 0 && 
	    (statstruct.st_mode & S_IFDIR)) {
		if (incdirs == NULL) {
			incdirs = (char **) mymalloc(sizeof(char *));
			incnames = (char **) mymalloc(sizeof(char *));
		}
		else {
			incdirs = (char **) myrealloc(incdirs, 
				(nincdirs + 1) * sizeof(char *));
			incnames = (char **) myrealloc(incnames, 
				(nincdirs + 1) * sizeof(char *));
		}
		incdirs[nincdirs] = stralloc(path);
		incnames[nincdirs++] = stralloc(name);
	}
}

/* make the source file list */

void
makefilelist()
{
	DIR	*dirfile;		/* directory file descriptor */
#ifdef BLACKBIRD
	struct	dirent	*entry;		/* directory entry pointer */
#else
	struct	direct	*entry;		/* directory entry pointer */
#endif
	FILE	*names;			/* name file pointer */
	register char	*file;
	char	dir[PATHLEN + 1];
	char	path[PATHLEN + 1];
	char	*s;
	int	i;

	makevpsrcdirs();	/* make the view source directory list */

	/* if there are source file arguments */
	if (fileargc > 0) {
		
		/* put them in a list that can be expanded */
		for (i = 0; i < fileargc; ++i) {
			file = fileargv[i];
			if (infilelist(file) == NO) {
				if ((s = inviewpath(file)) != NULL) {
					addsrcfile(file, s);
				}
				else {
					fprintf(stderr, "cscope: cannot find file %s\n",
					    file);
					errorsfound = YES;
				}
			}
		}
		return;
	}
	/* see if a file name file exists */
	if (namefile == NULL && access(NAMEFILE, READ) == 0) {
		namefile = NAMEFILE;
	}
	/* if there is a file of source file names */
	if (namefile != NULL) {
		if ((names = fopen(namefile, "r")) == NULL) {
			fprintf(stderr, "cscope: cannot open file %s\n", namefile);
			myexit(1);
		}
		/* get the names in the file */
		while (fscanf(names, "%s", path) == 1) {
			if (*path == '-') {	/* if an option */
				i = path[1];
				s = path + 2;	/* for "-Ipath" */
				if (*s == '\0') {	/* if "-I path" */
					fscanf(names, "%s", path);
					s = path;
				}
				/* expand $ and ~ */
				shellpath(dir, sizeof(dir), s);
				if (i == 'I') {	/* #include file directory */
					includedir(dir);
				}
				else {
					fprintf(stderr, "%s: only -I options can be in file %s\n", 
						argv0, namefile);
				}
			}
			else if ((s = inviewpath(path)) != NULL) {
				addsrcfile(path, s);
			}
			else {
				fprintf(stderr, "cscope: cannot find file %s\n",
				    path);
				errorsfound = YES;
			}
		}
		fclose(names);
		return;
	}
	/* make a list of all the source files in the directories */
	for (i = 0; i < nsrcdirs; ++i) {

		/* open the directory */
		/* note: failure is allowed because SOURCEDIRS may not exist */
		if ((dirfile = opendir(srcdirs[i])) != NULL) {

			/* read each entry in the directory */
			while ((entry = readdir(dirfile)) != NULL) {
				
				/* if it is a source file not already found */
				file = entry->d_name;
				if (entry->d_ino != 0 && issrcfile(file) &&
				    infilelist(file) == NO) {

					/* add it to the list */
					sprintf(path, "%s/%s", srcdirs[i], file);
					addsrcfile(file, path);
				}
			}
			closedir(dirfile);
		}
	}
}

/* see if this is a source file */

BOOL
issrcfile(file)
char	*file;
{
	char	*s;

	/* if there is a file suffix */
	if ((s = strrchr(file, '.')) != NULL) {
		if (*++s == '\0') {		/* no suffix */
			;
		}
		else if (s[1] == '\0') {	/* 1 character suffix */
			switch (*s) {
			case 'c':
			case 'h':
			case 'l':
			case 'y':
			case 'C':
			case 'G':
			case 'H':
			case 'L':
				return(YES);
			}
		}
		else if (s[2] == '\0') {	/* 2 character suffix */
			if ((*s == 'b' && s[1] == 'p') ||
			    (*s == 's' && s[1] == 'd') ||
			    (*s == 'c' && s[1] == 'c')) {
				return(YES);
			}
		}
	}
	return(NO);
}

/* add an include file to the source file list */

void
incfile(file, type)
char	*file;
char	type;
{
	char	name[PATHLEN + 1];
	char	path[PATHLEN + 1];
	char	*s;
	register int	i;
	
	/* see if the file is already in the source file list */
	if (infilelist(file) == YES) {
		return;
	}
	/* look in current directory if it was #include "file" */
	if (type == '"' && (s = inviewpath(file)) != NULL) {
		addsrcfile(file, s);
	}
	else {
		/* search for the file in the #include directory list */
		for (i = 0; i < nincdirs; ++i) {
			
			/* don't include the file from two directories */
			sprintf(name, "%s/%s", incnames[i], file);
			if (infilelist(name) == YES) {
				break;
			}
			/* make sure it exists and is readable */
			sprintf(path, "%s/%s", incdirs[i], file);
			if (access(compath(path), READ) == 0) {
				addsrcfile(name, path);
				break;
			}
		}
	}
}

/* see if the file is already in the list */

static	BOOL
infilelist(file)
char	*file;
{
	register int	i;
	
	compath(file);
	for (i = 0; i < nsrcfiles; ++i) {
		if (strequal(file, srcnames[i])) {
			return(YES);
		}
	}
	return(NO);
}

/* search for the file in the view path */

static	char *
inviewpath(file)
char	*file;
{
	static	char	path[PATHLEN + 1];
	register int	i;

	/* look for the file */
	if (access(compath(file), READ) == 0) {
		return(file);
	}
	/* if it isn't a full path name and there is a multi-directory view path */
	if (*file != '/' && vpndirs > 1) {

		/* compute its path from higher view path source dirs */
		for (i = 1; i < nvpsrcdirs; ++i) {
			sprintf(path, "%s/%s", srcdirs[i], file);
			if (access(compath(path), READ) == 0) {
				return(path);
			}
		}
	}
	return(NULL);
}

/* add a source file to the list */

void
addsrcfile(name, path)
char	*name;
char	*path;
{
	/* make sure there is room for the file */
	if (nsrcfiles == msrcfiles) {
		msrcfiles += SRCINC;
		srcfiles = (char **) myrealloc(srcfiles, msrcfiles * sizeof(char *));
		srcnames = (char **) myrealloc(srcnames, msrcfiles * sizeof(char *));
	}
	/* add the file to the list */
	srcfiles[nsrcfiles] = stralloc(compath(path));
	srcnames[nsrcfiles++] = stralloc(compath(name));
}
