/*	Copyright (c) 1984, 1985, 1986, 1987 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*	cscope - interactive C symbol cross-reference
 *
 *	display functions
 */

/*
 * $Header: /u5/alt/tools/src/cscope/RCS/display.c,v 1.9 1991/06/28 00:22:01 marvinw Exp $
 *
 * $Log: display.c,v $
 * Revision 1.9  1991/06/28  00:22:01  marvinw
 * Changed MIPS to mips
 *
 * Revision 1.8  1991/05/21  15:45:36  erice
 * Specific calls to printw() that pass more than two parameters (a format
 * string and the first variable) do not work correctly on the current
 * mips implementation of curses.
 *
 * To work around this, these printw() calls are split into multiple
 * statements where possible.  In a few cases, new subroutines fmt_string1()
 * and fmt_string2() had to be created to maintain the same functionality.
 *
 * Revision 1.7  89/11/27  13:38:28  kimr
 * Gave up trying to outdate the references in setfield if you change the
 * field.  The abrupt clearing of the display if you accidentally hit
 * return seems more harmful than showing the incorrect references for a
 * field if you change fields and don't repeat the search.
 * 
 * Revision 1.6  89/11/21  16:56:04  kimr
 * Caused setfield to zero out totallines.  This fixes a bug if
 * you changed the field and then hit space or "+".
 * 
 * 
 * Revision 1.5  89/11/01  14:13:22  kimr
 * Put in handling for ^A (function name abbreviation) and ^F (filename
 * expansion) commands.
 * 
 * Revision 1.4  89/10/26  18:24:27  kimr
 * If the global fullfilename is set, display the full pathname of the
 * file on the reference line, otherwise display just the basename.
 * 
 * Revision 1.3  89/07/21  18:01:16  kimr
 * Revised comments for the variables which control the formatting of
 * references on the screen.  Added a pagestack (size 500) which allows
 * you to backup to the same pages that you saw going forward.  
 * 
 * You can't back up before the first page any more, and if you do have
 * 500 pages, the top entry will stop getting pushed, so backing up will
 * jump back to the 500th page.  Added function backpage() to back up a
 * page.
 * 
 * Fixed a bug that allowed the last reference line to override the paging
 * prompt if the last line wrapped around.
 * 
 * Revision 1.2  89/07/18  17:25:20  kimr
 * Made a pass at cleaning up lint errors and added RCS headers.
 * 
 *
 */

#include "global.h"
#include "version.h"	/* FILEVERSION and FIXVERSION */
#include <ctype.h>	/* toupper */
#include <setjmp.h>	/* jmp_buf */
#include <signal.h>	/* SIGINT */

#if BSD			/* toupper does not test for an upper-case */
#undef	toupper		/* character in BSD */
#define	toupper(c)	(isupper(c) ? (c) : (c)-'a'+'A')	
#endif

/*
 * Following variables keep track of where we are in the references and
 * now the references map onto screen lines.
 */

int	*displine;		/* screen line of displayed reference */
int	disprefs;		/* relative index of displayed references */
int	field;			/* input field */
int	mdisprefs;		/* maximum lines for displayed references */
int	nextline;		/* next reference to be shown */
int	topline = 1;		/* top reference on page */
int	bottomline;		/* bottom reference on page */
int	totallines;		/* total number of references */

/*
 * Keep a stack of which reference line begins each page for use in going 
 * backwards or undoing calls to display(). Unfortunately, we can't tell in 
 * advance how many pages there might be, but it will never be more than 
 * totallines.  There should always be one entry in pagestack for newline=1.
 */

#define NUMPAGES	500
int	pagestack[NUMPAGES + 1] = {1};
int	pagestackindex = 0;

static	int	fldline;		/* input field line */
static	int	fldcolumn;		/* input field column */
static	int	filelen;		/* base file name field length */
static	int	fullfilelen;		/* full file name field length */
static	int	fcnlen;			/* function name display field length */
static	jmp_buf	env;			/* setjmp/longjmp buffer */
static	int	lastdispline;		/* last displayed reference line */
static	char	lastmsg[MSGLEN + 1];	/* last message displayed */
static	int	numlen;			/* line number display field length */
static	int	searchcount;		/* count of files searched */
static	char	helpstring[] = "Press the ? key for help";
static	char	selprompt[] = 
	"Select lines to change (press the ? key for help): ";

#if BSD	/* compiler bug workaround */
#define	Void	char *
#else
#define	Void	void
#endif

#ifdef DEBUG
extern FILE *console;
#endif

#ifdef mips
void fmt_string1(), fmt_string2();
#endif

char	*findstring();
Void	calling(), calledby(), finddef(), findfile(), findinclude(), findsymbol();

typedef char *(*FP)();	/* pointer to function returning a character pointer */

static	struct	{		/* text of input fields */
	char	*text1;
	char	*text2;
	FP	findfcn;
} fields[FIELDS] = {
	"Find this", "C symbol",			(FP) findsymbol,
	"Find this", "function or #define",		(FP) finddef,
	"Find", "functions called by this function",	(FP) calledby,
	"Find", "functions calling this function",	(FP) calling,
	"Find this", "text string",			findstring,
	"Change this", "text string",			findstring,
	"Find this", "egrep pattern",			findregexp,
	"Find this", "file",				(FP) findfile,
	"Find", "files #including this file",		(FP) findinclude,
};
	
/* Page backwards in refs */

void
backpage()
{
	/* Reset nextline so the next call to display will redisplay a
	 * previously displayed page.  Called once, display will redisplay
	 * the same page.  Called twice, display will back up a page.
	 * If we're on the first page, don't back up any more.
	 */
    nextline = pagestack[pagestackindex];
    if (pagestackindex) { /* non-zero */
	pagestackindex--;
    }
}

/* initialize display parameters */

void
dispinit()
{
	/* calculate the maximum displayed reference lines */
	lastdispline = FLDLINE - 2;
	mdisprefs = lastdispline - REFLINE + 1;
	if (mouse == NO && mdisprefs > 9) {
		mdisprefs = 9;
	}
	/* allocate the displayed line array */
	displine = (int *) mymalloc(mdisprefs * sizeof(int));
}
/* display a page of the references */

void
display()
{
	char	file[PATHLEN + 1];	/* file name */
	char	function[PATLEN + 1];	/* function name */
	char	linenum[NUMLEN + 1];	/* line number */
	int	screenline;		/* screen line number */
	int	width;			/* source line display width */
	register int	i, j;
	register char	*s;

	/* see if this is the initial display */
	erase();
	if (refsfound == NULL) {
		printw("Cscope version %d", FILEVERSION);
		printw("%s", FIXVERSION);
		move(0, COLS - (int) sizeof(helpstring));
		addstr(helpstring);
	}
	/* if no references were found */
	else if (totallines == 0) {
		
		/* redisplay the last message */
		addstr(lastmsg);
	}
	else {	/* display the pattern */
		if (changing == YES) {
			printw("Change \"%s\" to ", pattern);
			printw("\"%s\"", newpat);
		}
		else {
			printw("%c", toupper(fields[field].text2[0]));
			printw("%s: ", fields[field].text2 + 1);
			printw("%s", pattern);
		}
		/* display the column headings */
		move(2, 2);
#ifdef mips
		fmt_string1("%-*s ", fullfilename?fullfilelen:filelen, "File");
#else
		printw("%-*s ", fullfilename?fullfilelen:filelen, "File");
#endif
		if (field < STRING) {
#ifdef mips
			fmt_string1("%-*s ", fullfuncname?fcnlen:8, "Function");
#else
			printw("%-*s ", fullfuncname?fcnlen:8, "Function");
#endif
		}
		if (field != FILENAME) {
			if ((field == DEFINITION) && prevlinetoo) {
				addstr("Definition"); /* indicate previous (type) line is showing */
			} else {
				addstr("Line");
			}
		}
		addstr("\n\r");

		/* if at end of file go back to beginning */
		if (nextline > totallines) {
			seekline(1);
		}
		/* calculate the source text column */
		width = COLS - (fullfilename?fullfilelen:filelen) - (fullfuncname?fcnlen:8) - numlen - 5;
		if (field >= STRING) {
			width += (fullfuncname?fcnlen:8) + 1;
		}

		/* 
		 * As long as nextline is greater than 1, push it on pagestack.
		 * If nextline is one, clear the pagestack.
		 */

		if (nextline <= 1) {
		    pagestackindex = 0; /* pop pagestack */
		}
		else { /* push nextline on pagestack */
		    if (pagestackindex < NUMPAGES) {
			pagestack[++pagestackindex] = nextline;
		    }
		}

		/* until the max references have been displayed or 
		   there is no more room */
		topline = nextline;
		for (disprefs = 0, screenline = REFLINE;
		    disprefs < mdisprefs && screenline <= lastdispline;
		    ++disprefs, ++screenline) {
			
			/* read the reference line */
			if (fscanf(refsfound, "%s%s%s %[^\n]", file, function, 
			    linenum, yytext) < 4) {
				break;
			}
			++nextline;
			displine[disprefs] = screenline;
			
			/* if no mouse, display the selection number */
			if (mouse == YES) {
				addch(' ');
			}
			else {
				printw("%d", disprefs + 1);
			}
			/* display any change mark */
			if (changing == YES && 
			    change[topline + disprefs - 1] == YES) {
				addch('>');
			}
			else {
				addch(' ');
			}
			/* display the file name */
			if ((field == FILENAME) || fullfilename) {
				s = file;
			}
			else {
				s = basename(file);
			}
#ifdef mips
			fmt_string1("%-*s ",fullfilename?fullfilelen:filelen, s);
#else
			printw("%-*s ", fullfilename?fullfilelen:filelen, s);
#endif

			/* display the function name */
			if (field < STRING) {
				if (fullfuncname) {
#ifdef mips
				    fmt_string1("%-*s ", fcnlen, function);
#else
				    printw("%-*s ", fcnlen, function);
#endif
				} 
				else { /* abbreviate to strlen("Function") */
#ifdef mips
				    fmt_string2("%-*.*s ", 8, 8, function);
#else
				    printw("%-*.*s ", 8, 8, function);
#endif
				}
			}
			if (field == FILENAME) {
				addstr("\n\r");	/* go to next line */
				continue;
			}
			/* display the line number */
#ifdef mips
			fmt_string1("%*s ", numlen, linenum);
#else
			printw("%*s ", numlen, linenum);
#endif

			/* there may be tabs in egrep output */
			while ((s = strchr(yytext, '\t')) != NULL) {
				*s = ' ';
			}
			/* display the source line */
			s = yytext;
			for (;;) {

				/* see if the source line will fit */
				if ((i = strlen(s)) > width) {
					
					/* find the nearest blank */
					for (i = width; s[i] != ' ' && i > 0; --i) {
						;
					}
					if (i == 0) {
						i = width;	/* no blank */
					}
				}
				/* print up to this point */
#ifdef mips
				fmt_string1("%.*s", i, s);
#else
				printw("%.*s", i, s);
#endif
				s += i;
				
				/* if line didn't wrap around */
				if (i < width) {
					addstr("\n\r");	/* go to next line */
				}
				/* skip blanks */
				while (*s == ' ') {
					++s;
				}
				/* see if there is more text */
				if (*s == '\0') {
					break;
				}
				/* indent the continued source line */
				move(++screenline, COLS - width);
				/* see if there is any room for the rest */
				if (screenline > lastdispline) {
				    break;
				}
			}
		}
		/* position the cursor for the message */
		j = FLDLINE - 1;
		if (screenline < j) {
			addch('\n');
		}
		else {
			move(j, 0);
		}
		/* check for more references */
		i = totallines - nextline + 1;
		bottomline = nextline;
		if (i > 0) {
			printw("* %d more lines - press the space bar to display more *", i);
		}
		/* if this is the last page of references */
		else if (topline > 1 && nextline > totallines) {
			addstr("* Press the space bar to display the first lines again *");
		}
	}
	/* display the input fields */
	move(FLDLINE, 0);
	for (i = 0; i < FIELDS; ++i) {
		printw("%s ", fields[i].text1);
		printw("%s:\n\r", fields[i].text2);
	}
	/* display any prompt */
	if (changing == YES) {
		move(PRLINE, 0);
		addstr(selprompt);
	}
	draw_scrollbar(topline, nextline);	/* display the scrollbar */
}

/* Set the cursor position for the field.  Normally, changing the field does
 * not outdate the references.  The CLEARONFIELDCHANGE code will wipe out the 
 * references and clear the display if you change fields, but that behavior 
 * seems less friendly than occasionally garbling the display.
 */

void
setfield()
{
#ifdef CLEARONFIELDCHANGE
	static int oldfield = 0;

	if (oldfield != field) {
	    totallines = 0;		/* wipe out references */
	    oldfield = field;
	    (void) strcpy(lastmsg, "");	/* wipe out lastmsg */
	    display();
	}
#endif
	fldline = FLDLINE + field;
	fldcolumn = strlen(fields[field].text1) + strlen(fields[field].text2) + 3;
}

/* move to the current input field */

void
atfield()
{
	move(fldline, fldcolumn);
}

/* move to the changing lines prompt */

void
atchange()
{
	move(PRLINE, (int) sizeof(selprompt) - 1);
}

/* search for the symbol or text pattern */

static
jumpback()
{
	longjmp(env, 1);
}

BOOL
search()
{
	char	file[PATHLEN + 1];	/* file name */
	char	function[PATLEN + 1];	/* function name */
	char	linenum[NUMLEN + 1];	/* line number */
	char	*egreperror = NULL;	/* egrep error message */
	BOOL	regcmperror = NO;	/* regcmp error */
	void	(*savesig)();		/* old value of signal */
	FP	f;			/* searching function */
	register int	c, i;
	BOOL	findinit();
	void	findcleanup();
	
	/* open the references found file for writing */
	if (refsfound == NULL) {
		if ((refsfound = fopen(temp1, "w")) == NULL) {
			putmsg("Cannot open temporary file");
			return(NO);
		}
	}
	else {
		freopen(temp1, "w", refsfound);
	}
	/* find the pattern - stop on an interrupt */
	putmsg("Searching");
	searchcount = 0;
	if (setjmp(env) == 0) {
		savesig = signal(SIGINT, jumpback);
		f = fields[field].findfcn;
		if (f == findregexp || f == findstring) {
			egreperror = (*f)(pattern);
		}
		else if ((regcmperror = findinit()) == NO) {
			(*f)();
			findcleanup();
		}
	}
	signal(SIGINT, savesig);

	/* rewind the cross-reference file */
	lseek(symrefs, (long) 0, 0);
	
	/* reopen the references found file for reading */
	freopen(temp1, "r", refsfound);
	nextline = 1;
	totallines = 0;
	
	/* see if it is empty */
	if ((c = getc(refsfound)) == EOF) {
		if (egreperror != NULL) {
			sprintf(lastmsg, "Egrep %s in this pattern: %s", 
				egreperror, pattern);
		}
		else if (regcmperror == YES) {
			sprintf(lastmsg, "Error in this regcmp(3X) regular expression: %s", 
				pattern);
			
		}
		else {
			sprintf(lastmsg, "Could not find the %s: %s", 
				fields[field].text2, pattern);
		}
		return(NO);
	}
	/* put back the character read */
	ungetc(c, refsfound);

	/* count the references found and find the length of the file,
	   function, and line number display fields */
	filelen = 4;	 /* strlen("File") */
	fullfilelen = 4; /* strlen("File") */
	fcnlen = 8;	 /* strlen("Function") */
	numlen = 0;
	while (fscanf(refsfound, "%s%s%s", file, function, linenum) == 3) {
		if ((i = strlen(file)) > fullfilelen) {
			fullfilelen = i;
		}
		if ((i = strlen(basename(file))) > filelen) {
			filelen = i;
		}
		if ((i = strlen(function)) > fcnlen) {
			fcnlen = i;
		}
		if ((i = strlen(linenum)) > numlen) {
			numlen = i;
		}
		/* skip the line text */
		while ((c = getc(refsfound)) != EOF && c != '\n') {
			;
		}
		++totallines;
	}
	rewind(refsfound);
	return(YES);
}

/* display search progress */

void
progress()
{
	static	long	start;
	long	now;
	char	msg[MSGLEN + 1];
	long	time();

	/* save the start time */
	if (searchcount == 0) {
		start = time((long *) NULL);
	}
	/* display the progress every 3 seconds */
	else if ((now = time((long *) NULL)) - start >= 3) {
		start = now;
		sprintf(msg, "%d of %d files searched", 
			searchcount, nsrcfiles);
		putmsg(msg);
	}
	++searchcount;
}

/* print error message on system call failure */

void
myperror(text) 
char	*text; 
{
	extern	int	errno, sys_nerr;
	extern	char	*sys_errlist[];
	char	msg[MSGLEN + 1];	/* message */
	register char	*s;

	s = "Unknown error";
	if (errno < sys_nerr) {
		s = sys_errlist[errno];
	}
	sprintf(msg, "%s: %s", text, s);
	putmsg(msg);
}

/* putmsg clears the message line and prints the message */

void
putmsg(msg) 
char	*msg; 
{
	move(MSGLINE, 0);
	clrtoeol();
	addstr(msg);
	refresh();
	(void) strcpy(lastmsg, msg);
}
/* position references found file at specified line */

void
seekline(line) 
int	line; 
{
	register int	c;

	/* verify that there is a references found file */
	if (refsfound == NULL) {
		return;
	}
	/* go to the beginning of the file */
	rewind(refsfound);
	
	/* find the requested line */
	nextline = 1;
	while (nextline < line && (c = getc(refsfound)) != EOF) {
		if (c == '\n') {
			nextline++;
		}
	}
}

#ifdef mips
/* This code is used to bypass a bug in the mips implementation of
   printw() under curses.  Because of this bug, only the first parameter
   to printw() is correctly used while the others are treated as null.

   This subroutine uses sprintf() to create the correct string which is
   then fed to printw() as one parameter.

   fmt_string1() accepts one length parameter, while fmt_string2() accepts
   two.  Both are used in this file.
*/

void fmt_string1(format, length, string)
char *format;
int length;
char *string;
{
    char *newstr;

    newstr = (char *)malloc(length + 3);

    sprintf(newstr, format, length, string);
    printw("%s", newstr);

    free(newstr);
}

void fmt_string2(format, length1, length2, string)
char *format;
int length1;
int length2;
char *string;
{
    char *newstr;

    newstr = (char *)malloc(length2 + 3);

    sprintf(newstr, format, length1, length2, string);
    printw("%s", newstr);

    free(newstr);
}

#endif
