/*	Copyright (c) 1984, 1985, 1986, 1987 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*	cscope - interactive C symbol cross-reference
 *
 *	file editing functions
 */

/*
 * $Header: /u5/alt/tools/src/cscope/RCS/edit.c,v 1.6 1989/11/27 14:11:42 kimr Exp $
 *
 * $Log: edit.c,v $
 * Revision 1.6  1989/11/27  14:11:42  kimr
 * Turns out that only STRING and CHANGE fields should have metacharacters
 * protected - all others should pass through.
 *
 * Revision 1.5  89/11/07  17:14:12  kimr
 * If preload is 'v', pass a quoted string to the editor in "vi" format:
 * <line>g/<string>/.  This places the cursor at the beginning of <line> and
 * loads <string> in the search buffer.  If the field is any but "find this
 * egrep pattern" or "find this file", all ed metacharacters and slash are
 * quoted with a backslash.  For the remaining fields, only slash characters
 * are quoted.  The pattern is not passed if pager or more is being used, mostly
 * because I don't know anyone who uses them.  Note that in quoted(), metachar
 * scanning is done with a big if statement instead of strchr.  I convinced
 * myself the if statement was faster after I wrote it, but it could just as
 * well use strchr.
 * 
 * Revision 1.4  89/07/21  18:14:12  kimr
 * Revised prompt to continue editing all files and put it on the second
 * message line.  Pass "keep_running" to execute.
 * 
 * Revision 1.3  89/07/18  17:25:25  kimr
 * Made a pass at cleaning up lint errors and added RCS headers.
 * 
 *
 */

#include "global.h"

/* edit this displayed reference */

void
editref(i)
int	i;
{
	char	file[PATHLEN + 1];	/* file name */
	char	linenum[NUMLEN + 1];	/* line number */

	/* verify that there is a references found file */
	if (refsfound == NULL) {
		return;
	}
	/* get the selected line */
	seekline(i + topline);
	
	/* get the file name and line number */
	if (fscanf(refsfound, "%s%*s%s", file, linenum) == 2) {
		edit(file, linenum);	/* edit it */
	}
	seekline(topline);	/* restore the line pointer */
}

/* edit all references */

void
editall()
{
	char	file[PATHLEN + 1];	/* file name */
	char	linenum[NUMLEN + 1];	/* line number */
	int	c;

	/* verify that there is a references found file */
	if (refsfound == NULL) {
		return;
	}
	/* get the first line */
	seekline(1);
	
	/* get each file name and line number */
	while (fscanf(refsfound, "%s%*s%s%*[^\n]", file, linenum) == 2) {
		edit(file, linenum);	/* edit it */
		move(MSGLINE + 1, 0);   /* put prompt on second line */
		clrtoeol();
		addstr("Type ^D to stop editing all files, or any other character to continue: ");
		if ((c = mygetch()) == EOF || c == ctrl('D') || c == ctrl('Z')) {
			break;
		}
	}
	seekline(topline);
}
	
/* call the editor */

void
edit(file, linenum)
char	*file;
char	*linenum;
{
	char	msg[MSGLEN + 1];	/* message */
	char	plusnum[NUMLEN + 2];	/* line number option */
	char	*s;
	char	*quoted();

	sprintf(msg, "%s +%s %s", basename(editor), linenum, file);
	putmsg(msg);
	if (preload == 'v') { /* "vi" string preloading */
	    sprintf(plusnum, "+%sg/%s/", linenum, quoted(pattern));
	} else {
	    sprintf(plusnum, "+%s", linenum);
	}
	
	/* Get pager to pause after displaying a file smaller than the screen 
	 * length by passing /dev/null as the last file.
         */
	if (strcmp(s = basename(editor), "more") == 0 || strcmp(s, "page") == 0) {
	    execute(keep_running, editor, editor, plusnum, file, "/dev/null", (char *) 0);
	} else {
	    if (preload == 'p') { 
		execute(keep_running, editor, editor, quoted(pattern), plusnum, file, (char *) 0);
	    } else {
		execute(keep_running, editor, editor, plusnum, file, (char *)0);
	    }
	}
	if (!keep_running) {
	    clear();	/* redisplay screen */
	}
}


/* Quote the search pattern in the appropriate style for preloading.  If
 * preload style is "vi", protect '/' since slashes delimit the search string.
 * For "find/change this text string" protect all ed/regexp metachars.
 * All other search patterns are regexp.
 */

char *
quoted(pattern)
register char *pattern;
{
    static char buf[PATLEN*2];
    register char *p;

    p = buf;
    while (*pattern) {
	if ((*pattern == '/') && (preload == 'v')) { /* protect '/' if "vi" */
	    *p++ = '\\'; 
	}
	/* protect '\', '*', '.', '[', ']', '^', '$' if not in egrep pattern */
	if ((field == STRING) || (field == CHANGE)) { 
	    if ((*pattern == '\\') || (*pattern == '*') || (*pattern == '.') ||
		(*pattern == '[' ) || (*pattern == ']') || (*pattern == '^') ||
		 (*pattern == '$' )) {
		*p++ = '\\'; /* protect meta-char with backslash */
	    }
	}
	*p++ = *pattern++;
    }
    *p = NULL; /* place terminator in buf */
    return(buf);
}
