/*	Copyright (c) 1984, 1985, 1986, 1987 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*	cscope - interactive C symbol cross-reference
 *
 *	process execution functions
 */

/*
 * $Header: /u5/alt/tools/src/cscope/RCS/exec.c,v 1.8 1990/02/12 11:08:37 jrb Exp $
 *
 * $Log: exec.c,v $
 * Revision 1.8  1990/02/12  11:08:37  jrb
 * fixed for utek88k box
 *
 * Revision 1.7  89/07/21  18:05:45  kimr
 * Added arguments to execute() and myexecvp() to distinquish between
 * processes which should be waited on (shells) and those which shouldn't
 * (window-based editors).
 * 
 * If continueing execution after starting a child, redirect the child's stdout
 * and stderr to /dev/console, if possible.  Change the child to an independent
 * process group so it runs completely independently of cscope.
 * 
 * 
 * Revision 1.6  89/07/18  17:25:28  kimr
 * Made a pass at cleaning up lint errors and added RCS headers.
 * 
 *
 */

#include "global.h"
#include <signal.h>	/* SIGINT and SIGQUIT */
#include <varargs.h>
#include <sys/param.h>
#ifdef BSD
#include <sys/file.h>
#else
#include <fcntl.h>
#endif

static	void	(*savesig)();	/* old value of signal */

/* execute forks and executes a program or shell script. If spawn is false,
 * it waits for the child to finish, and returns its exit code.  Otherwise
 * it continues execution.
 */

#define MAXARGS		20
 
/*VARARGS1*/
execute(spawn, a, va_alist)	/* note: "exec" is already defined on u370 */
BOOL	spawn;
char	*a;
va_dcl
{
	va_list	ap;
	char *args[MAXARGS];
	int argno = 0;
	int	exitcode;
	int	p;

	/* fork and exec the program or shell script */
	if (!spawn) {
	    endwin();	/* restore the terminal modes */
	    mousecleanup();
	    fflush(stdout);
	    }
	if ((p = myfork()) == 0) {
		va_start(ap);
		while((args[argno++] = va_arg(ap, char*)) != (char *) 0)
			;
		va_end(ap);
		myexecvp(spawn, a, args);	/* child */
	}
	else {
		if (spawn)
		    return(0); /* spawn child and continue execution */
		else
		    exitcode = join(p);	/* parent waits for child */
	}

	/* Reinitialize the curses display package */
	initscr();
	nonl();
	cbreak();
	noecho();

	/* the menu and scrollbar may be changed by the command executed */
	mousemenu();
	draw_scrollbar(topline, nextline);
	return(exitcode);
}

/* myexecvp is an interface to the execvp system call to
 * close all files except stdin, stdout, and stderr; and
 * modify argv[0] to reference the last component of its path-name.
 * If spawn is true, we're running currently with our parent so redirect
 * stderr and stdout to /dev/console, if it can be opened.
 */

static
myexecvp(spawn, a, argv)
BOOL	spawn;
char	*a;
char **argv;
/*
va_list	args;
*/
{
	/*
	register char	**argv;
	*/
	register int	i;
	static   int    console;
	
	if (spawn) { /* try to attach stderr, stdout to /dev/console */
	    fflush(stdout);
	    fflush(stderr);
	    if ((console = open("/dev/console", O_WRONLY)) == -1) {
		/* perror("Couldn't open /dev/console") */; 
	    } else {
	    close(1); dup(console);
	    close(2); dup(console);
	    }
	/* change the process group if spawning an independent process */
	setpgrp(0, getpid());
	}

	/* close console and all other files */
	for (i = 3; i < NOFILE && close(i) == 0; ++i) {
		;
	}
	/* modify argv[0] to reference the last component of its path name */
	/*
	argv = (char **) args.stk;
	*/
	argv[0] = basename(argv[0]);

	/* execute the program or shell script */
	execvp(a, argv);	/* returns only on failure */
	perror("\nExec failed");/* display the reason */
	fprintf(stderr, "Press the RETURN key to continue: ");
	getchar();		/* wait until the user sees the message */
	exit(1);		/* exit the child */
}

/* myfork acts like fork but also handles the interrupt and quit signals */

static
myfork() 
{
	int	p;		/* process number */

	p = fork();
	
	/* the parent ignores the interrupt and quit signal */
	if (p > 0) {
		savesig = signal(SIGQUIT, SIG_IGN);
	}
	/* so they can bu used to stop the child */
	else if (p == 0) {
		signal(SIGINT, SIG_DFL);
		signal(SIGQUIT, SIG_DFL);
	}
	/* check for fork failure */
	if (p == -1) {
		myperror("Cannot fork");
	}
	return p;
}

/* join is the compliment of fork */

static
join(p) 
int	p; 
{
	int	status;  
	int	w;
	void    (*stop)()=signal(SIGTSTP, SIG_DFL);

	/* wait for the correct child to exit */
	do {
		w = wait(&status);
	} while (p != -1 && w != p);

	/* restore signal handling */
	signal(SIGTSTP, stop);
	signal(SIGQUIT, savesig);

	/* return the child's exit code */
	return(status >> 8);
}
