/*	Copyright (c) 1984, 1985, 1986, 1987 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*	cscope - interactive C symbol or text cross-reference
 *
 *	searching functions
 */

/*
 * $Header: /u5/alt/tools/src/cscope/RCS/find.c,v 1.9 1989/11/21 16:54:59 kimr Exp $
 *
 * $Log: find.c,v $
 * Revision 1.9  1989/11/21  16:54:59  kimr
 * Fixed bug in putsource that was backing up too far in lseek call.
 *
 * Revision 1.7  89/11/01  13:32:01  kimr
 * If the type line doesn't end with "::", " *", or "\t*", separate
 * the type from the function name with a space.
 * h a space.
 * 
 * Revision 1.6  89/10/26  18:19:01  kimr
 * Added functions putprevline, putlinenum and modified putref and putsource
 * to call them.  If global prevlinetoo is set and the current command
 * is function definition, the source line preceeding the line the function
 * definition appears on is extracted from the source file and inserted
 * into the reference line between the line number and the function definition.
 * This should be used if the function return type appears on the line
 * before the function declaration, and you wish to see the function
 * return type on the reference line.  If the type ends in ":" (as in
 * foo::bar() ), no space is inserted before the function name.
 * 
 * Revision 1.5  89/09/27  17:03:02  kimr
 * Modified findfunction to take a second parameter indicating whether
 * #defines count as hits.  When called from "calledby", #defines don't
 * count.  This was to fix a bug where it would report the rest of an
 * include file as being the functions called by an #defined symbol.
 * 
 * Revision 1.4  89/07/18  17:25:32  kimr
 * Made a pass at cleaning up lint errors and added RCS headers.
 * 
 *
 */

#include "global.h"
#include <ctype.h>	/* isalnum */
#include <varargs.h>

/* most of these functions have been optimized so their innermost loops have
 * only one test for the desired character by putting the char and 
 * an end-of-block marker (\0) at the end of the disk block buffer.
 * When the inner loop exits on the char, an outer loop will see if
 * the char is followed by a \0.  If so, it will read the next block
 * and restart the inner loop.
 */

char	*blockp;			/* pointer to current char in block */
char	block[BUFSIZ + 2];		/* leave room for end-of-block mark */
int	blocklen;			/* length of disk block read */
char	blockmark;			/* mark character to be searched for */

static	char	global[] = "<global>";	/* dummy global function name */
static	char	cpattern[PATLEN + 1];	/* compressed pattern */
static	char	*regexp;		/* regular expression */

void	putline(), putref(), putsource(), putprevline();
BOOL	match(), matchrest();
int	findfunction();

/* find the symbol in the cross-reference */

void
findsymbol()
{
	char	file[PATHLEN + 1];	/* source file name */
	char	function[PATLEN + 1];	/* function name */
	char	symbol[PATLEN + 1];	/* symbol name */
	FILE	*savedrefs;		/* saved references file */
	register int	c;
	register char	*cp;
	char	*s;

	/* open the temporary file */
	if ((savedrefs = fopen(temp2, "w")) == NULL) {
		putmsg("Cannot open temporary file");
		return;
	}
	scanpast('\t');			/* find the end of the header */
	getrefchar();			/* skip the file marker */
	putstring(file);		/* save the file name */
	(void) strcpy(function, global); /* set the dummy global function name */
	
	/* find the next symbol */
	/* note: this code was expanded in-line for speed */
	/* while (scanpast('\n') != NULL) { */
	/* other macros were replaced by code using cp instead of blockp */
	cp = blockp;
	for (;;) {
		setmark('\n');
		do {	/* innermost loop optimized to only one test */
			while (*cp != '\n') {
				++cp;
			}
		} while (*(cp + 1) == '\0' && (cp = readblock()) != NULL);

		/* skip the found character */
		if (cp != NULL && *(++cp + 1) == '\0') {
			cp = readblock();
		}
		if (cp == NULL) {
			break;
		}
		/* look for a source file or function name */
		if (*cp == '\t') {
			blockp = cp;
			switch (getrefchar()) {

			case NEWFILE:		/* file name */
				(void) strcpy(function, global);
				progress();

				/* save the name */
				getrefchar();
				putstring(file);
			
				/* check for the end of the symbols */
				if (*file == '\0') {
					goto endloop;
				}
				goto notmatched;	/* don't match file name */
				
			case FCNDEF:		/* function name */
				s = function;
				break;
			default:		/* other symbol */
				s = symbol;
			}
			/* save the name */
			getrefchar();
			putstring(s);

			/* see if this is a regular expression pattern */
			if (regexp != NULL) {
				if (regex(regexp, s) != NULL) {
					goto matched;
				}
			}
			/* match the symbol to the text pattern */
			else if (strequal(pattern, s)) {
				goto matched;
			}
			goto notmatched;
		}
		/* if this is a regular expression pattern */
		if (regexp != NULL) {
			
			/* if this is a symbol */
			if (isalnum(*cp) || *cp == '_') {
				blockp = cp;
				putstring(symbol);
				
				/* match the symbol to the regular expression */
				if (regex(regexp, symbol) != NULL) {
					goto matched;
				}
				goto notmatched;
			}
		}
		/* match the character to the text pattern */
		else if (*cp == cpattern[0]) {
			blockp = cp;

			/* match the rest of the symbol to the text pattern */
			if (matchrest()) {
		matched:		
				/* output the file, function and source line */
				/* put global references first */
				if (strcmp(function, global) == 0) {
					putref(refsfound, file, function, NO);
				}
				else {
					putref(savedrefs, file, function, NO);
				}
			}
		notmatched:
			cp = blockp;
		}
	}
	blockp = cp;
endloop:	
	/* append the saved references */
	freopen(temp2, "r", savedrefs);
	while ((c = getc(savedrefs)) != EOF) {
		putc(c, refsfound);
	}
	fclose(savedrefs);
}
/* find the function definition or #define */

void
finddef()
{
	char	file[PATHLEN + 1];	/* source file name */
	int	deftype;		/* DEFINE or FCNDEF */

	/* find all definitions */
	while (deftype=findfunction(file, TRUE)) {

		/* output the file, function and source line */
		putref(refsfound, file, pattern, deftype==FCNDEF);
	}
}
/* find the functions calling this function */

void
calling()
{
	char	file[PATHLEN + 1];	/* source file name */
	char	function[PATLEN + 1];	/* function name */

	/* find the next file name or function definition */
	while (scanpast('\t') != NULL) {
		switch (*blockp) {
			
		case NEWFILE:		/* save file name */
			getrefchar();
			putstring(file);
			if (*file == '\0') {	/* if end of symbols */
				return;
			}
			(void) strcpy(function, global);
			progress();
			break;
			
		case FCNDEF:		/* save calling function name */
			getrefchar();
			putstring(function);
			break;
			
		case FCNCALL:		/* match function called to pattern */
			getrefchar();
			if (match()) {
				
				/* output the file, calling function and source */
				putref(refsfound, file, function, NO);
			}
		}
	}
}
/* find the functions called by this function */

void
calledby()
{
	char	file[PATHLEN + 1];	/* source file name */

	/* find the function definition(s) */
	while (findfunction(file, FALSE)) {

		/* find the next function call or the end of this function */
		while (scanpast('\t') != NULL) {
			switch (*blockp) {
			
			case FCNCALL:		/* function call */

				/* output the file name */
				fprintf(refsfound, "%s ", file);

				/* output the function name */
				getrefchar();
				putline(refsfound);
				putc(' ', refsfound);

				/* output the source line */
				putsource(refsfound, NO, file);
				break;
			
			case FCNDEF:		/* function end */
				goto next;
			}
		}
	next:	;
	}
}

/* find the text in the source files */

char *
findstring()
{
	char	egreppat[2 * PATLEN];
	char	*cp, *pp;

	/* translate special characters in the regular expression */
	cp = egreppat;
	for (pp = pattern; *pp != '\0'; ++pp) {
		if (strchr(".*[\\^$+?|()", *pp) != NULL) {
			*cp++ = '\\';
		}
		*cp++ = *pp;
	}
	*cp = '\0';
	
	/* search the source files */
	return(findregexp(egreppat));
}

/* find this regular expression in the source files */

char *
findregexp(egreppat)
char	*egreppat;
{
	register int	i;
	char	*egreperror;

	/* compile the pattern */
	if ((egreperror = egrepinit(egreppat)) == NULL) {

		/* search the files */
		for (i = 0; i < nsrcfiles; ++i) {
			progress();
			if (egrep(srcfiles[i], refsfound, "%s <unknown> %ld ") < 0) {
				move(1, 0);
				clrtoeol();
				printw("Cannot open file %s", srcfiles[i]);
				refresh();
			}
		}
	}
	return(egreperror);
}

/* find matching file names */

void
findfile()
{
	register int	i;

	for (i = 0; i < nsrcfiles; ++i) {
		if (regex(regexp, srcfiles[i]) != NULL) {
			fprintf(refsfound, "%s <unknown> 1 <unknown>\n", 
				srcfiles[i]);
		}
	}
}

/* find files #including this file */

void
findinclude()
{
	char	file[PATHLEN + 1];	/* source file name */

	/* find the next file name or function definition */
	while (scanpast('\t') != NULL) {
		switch (*blockp) {
			
		case NEWFILE:		/* save file name */
			getrefchar();
			putstring(file);
			if (*file == '\0') {	/* if end of symbols */
				return;
			}
			progress();
			break;
			
		case INCLUDE:		/* match function called to pattern */
			getrefchar();
			getrefchar();	/* skip global or local #include marker */
			if (match()) {
				
				/* output the file and source line */
				putref(refsfound, file, global, NO);
			}
		}
	}
}
/* find the function definition or #define in the cross-reference */
  /* Returns zero if nothing found; FCNDEF, or DEFINE		    */

static	int
findfunction(file, defstoo)
char	*file;
BOOL	defstoo;
{
	/* find the next file name or definition */
	while (scanpast('\t') != NULL) {
		switch (*blockp) {
			
		case NEWFILE:		/* save file name */
			getrefchar();
			putstring(file);
			if (*file == '\0') {	/* if end of symbols */
				return(NO);
			}
			progress();
			break;

		case DEFINE:
			if (defstoo) {
			    getrefchar();
			    if (match()) {
				    return(DEFINE);
			    }
			}
			break;
		case FCNDEF:		/* match name to pattern */
			getrefchar();
			if (match()) {
				return(FCNDEF);
			}
			break;
		}
	}
	return(NO);
}

/* initialize */

BOOL
findinit()
{
	register int	c, i;
	register char	*s;

	/* allow a partial match for a file name */
	if (field == FILENAME) {
		if ((regexp = regcmp(pattern, (char *) NULL)) == NULL) {
			return(YES);	/* regcmp error */
		}
	}
	/* see if the pattern is a regular expression */
	else if (strpbrk(pattern, "^.[{*+$") != NULL) {
		/* remove a leading ^ */
		s = pattern;
		if (*s == '^') {
			++s;
		}
		/* remove a trailing $ */
		i = strlen(s) - 1;
		if (s[i] == '$') {
			s = strcpy(newpat, s);
			s[i] = '\0';
		}
		/* must be an exact match */
		if ((regexp = regcmp("^", s, "$", (char *) NULL)) == NULL) {
			return(YES);	/* regcmp error */
		}
	}
	else {	/* compress the string pattern for matching */
		s = cpattern;
		for (i = 0; (c = pattern[i]) != '\0'; ++i) {
			if (dicode1[c] && dicode2[pattern[i + 1]]) {
				c = (0200 - 2) + dicode1[c] + dicode2[pattern[i + 1]];
				++i;
			}
			*s++ = c;
		}
		*s = '\0';
	}
	readblock();	/* read the first cross-reference block */
	return(NO);	/* no error */
}

void
findcleanup()
{
	/* discard any regular expression */
	if (regexp != NULL) {
#ifndef BSD
		free(regexp);
#endif BSD
		regexp = NULL;
	}
}

/* match the pattern to the string */

static	BOOL
match()
{
	char	string[PATLEN + 1];
	
	/* see if this is a regular expression pattern */
	if (regexp != NULL) {
		putstring(string);
		return(regex(regexp, string) ? YES : NO);
	}
	/* it is a string pattern */
	return((BOOL) (*blockp == cpattern[0] && matchrest()));
}

/* match the rest of the pattern to the name */

static	BOOL
matchrest()
{
	int	i = 1;
	
	getrefchar();
	do {
		while (*blockp == cpattern[i]) {
			++blockp;
			++i;
		}
	} while (*(blockp + 1) == '\0' && readblock() != NULL);
	
	if (*blockp == '\n' && cpattern[i] == '\0') {
		return(YES);
	}
	return(NO);
}

/* put the reference into the file */

static	void
putref(output, file, function, functiontype)
FILE	*output;
char	*file;
char	*function;
BOOL	functiontype;
{
	fprintf(output, "%s %s ", file, function);
	putsource(output, functiontype, file);
}

/* put the source line into the file */

static	void
putsource(output, functiontype, file)
FILE	*output;
BOOL	functiontype;
char	*file;
{
	register char	*cp, nextc = '\0';
	
	/* check for old database file format */
	if (fileversion == 5) {
		scanpast(' ');
		putline(output);
		putc('\n', output);
		return;
	}
	/* scan back to the beginning of the source line */
	cp = blockp;
	while (*cp != '\n' || nextc != '\n') {
		nextc = *cp;
		if (--cp < block) {
			
			/* read the previous block */
			lseek(symrefs, (long) -BUFSIZ - blocklen, 1);
			readblock();
			cp = &block[BUFSIZ - 1];
		}
	}
	blockp = cp;
	getrefchar();	/* skip the double newline */
	getrefchar();
	if (functiontype && prevlinetoo) { 
		/* stuff the text from the previous line between the line 
		 * number and the start of this line
		 */
		putprevline(output, file, putlinenum(output));
	}
	/* until a double newline is found */
	do {
		/* skip a symbol type */
		if (*blockp == '\t') {
			getrefchar();
			getrefchar();
		}
		/* output a piece of the source line */
		putline(output);
	} while (getrefchar() != '\n');
	putc('\n', output);
}

static void 
putprevline(output, file, linenum)
FILE 	*output;
char	*file;
int	linenum;
{ /* Copy line linenum from file to output without a terminating newline.
   * If the line doesn't end in a "::" (as in class::func) or " *" (as in
   * char *foo), append a space.
   */
	register int c, curline, targetline;
	int 	old_c, old_c2;
	FILE	*p;

	if (*blockp == '#' || linenum == 1) {
	    return; /* either a define or function def on first line of file */
	}
	if ((p = fopen(file, "r")) == NULL) {
	    return; /* give up silently if can't find file */
	}
	targetline = linenum - 1;
	curline = 1;
	/* scan forward in file to targetline */
	do {
		while ((c = getc(p)) != '\n') ;
	} while ( ++curline < targetline );
	/* copy the source line to output */
        while ((c = getc(p)) != '\n') {
		putc(c, output);
		old_c2 = old_c;
		old_c = c;
	}
	/* If the type line doesn't end with "::", " *", or "\t*", separate 
	 * the type from the function name with a space.
	 */
	if ((old_c != ':') && 
	  !((old_c == '*') && (old_c2 == ' ' || old_c2 == '\t'))) {
		putc(' ', output); 
	}
	fclose(p);
};

static int
putlinenum(output)
FILE *output;
{ /* copy <digit>+ <space> from blockp to output, updating blockp.
   * return the int representing the line number */
	int i;
	i = atoi(blockp);
	for(; *blockp != ' '; blockp++) {
		putc(*blockp, output);
	}
	putc(*blockp++, output); /* move blank too */
	return i;
}

/* put the rest of the cross-reference line into the file */

static	void
putline(output)
FILE	*output;
{
	register char	*cp;
	register unsigned c;
	
	setmark('\n');
	cp = blockp;
	do {
		while ((c = *cp) != '\n') {
			
			/* check for a compressed digraph */
			if (c > '\177') {
				c &= 0177;
				putc(dichar1[c / 8], output);
				putc(dichar2[c & 7], output);
			}
			/* check for a compressed keyword */
			else if (c < ' ') {
				fputs(keyword[c].text, output);
				if (keyword[c].delim != '\0') {
					putc(' ', output);
				}
				if (keyword[c].delim == '(') {
					putc('(', output);
				}
			}
			else {
				putc((int) c, output);
			}
			++cp;
		}
	} while (*(cp + 1) == '\0' && (cp = readblock()) != NULL);
	blockp = cp;
}

/* put the rest of the cross-reference line into the string */

void
putstring(s)
register char	*s;
{
	register char	*cp;
	register unsigned c;
	
	setmark('\n');
	cp = blockp;
	do {
		while ((c = *cp) != '\n') {
			if (c > '\177') {
				c &= 0177;
				*s++ = dichar1[c / 8];
				*s++ = dichar2[c & 7];
			}
			else {
				*s++ = c;
			}
			++cp;
		}
	} while (*(cp + 1) == '\0' && (cp = readblock()) != NULL);
	blockp = cp;
	*s = '\0';
}
/* scan past the next occurence of this character in the cross-reference */

char	*
scanpast(c)
register char	c;
{
	register char	*cp;
	
	setmark(c);
	cp = blockp;
	do {	/* innermost loop optimized to only one test */
		while (*cp != c) {
			++cp;
		}
	} while (*(cp + 1) == '\0' && (cp = readblock()) != NULL);
	blockp = cp;
	if (cp != NULL) {
		getrefchar();	/* skip the found character */
	}
	return(blockp);
}

/* read a block of the cross-reference */

char	*
readblock()
{
	/* read the next block */
	blocklen = read(symrefs, block, BUFSIZ);
	blockp = block;
	
	/* add the search character and end-of-block mark */
	block[blocklen] = blockmark;
	block[blocklen + 1] = '\0';
	
	/* return NULL on end-of-file */
	if (blocklen == 0) {
		return(NULL);
	}
	return(block);
}


#if BSD

char *
regcmp(va_alist)
va_dcl
{
	va_list ap;
	char *p, string[PATLEN + 1];
	int len = 0;

	string[0] = '\0';

	va_start(ap);
	while((p = va_arg(ap, char *))) {
		len += strlen(p);
		if (len > PATLEN)
			return(NULL);
		else
			(void) strcat(string, p);
	}
	va_end(ap);

	if ((p = re_comp(string))) {
		printw("regcmp: re_comp says '%s'\n", p);
		return(NULL);
	}
	else {
		return((char *)1); /* return dummy non-NULL pointer */
	}

}

char *
regex(regexp, s)
char *regexp; /* dummy */
char *s;
{
	int n;

	if ((n = re_exec(s)) == 1) {
		return((char *)1); /* return dummy non-NULL pointer */
	}
	else if (n == 0) {
		return(NULL);
	}
	else {
		printw("regex: re_exec returned %d\n", n);
		return(NULL);
	}
}

#endif BSD

