/*	Copyright (c) 1984, 1985, 1986, 1987 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*
 *	char *getwd(dir)
 *
 *	Get the full pathname of the current directory.
 *
 *	returns: a pointer to dir on success.
 *		 NULL on failure.
 *
 *	Current directory upon return from getwd:
 *
 *		success: same as on entry
 *		failure: If a failure is noted during processing getwd
 *			 backs out of it's traversal and restores the
 *			 current directory to what it was upon entry.
 *			 Of course there are pathological corruptions
 *			 to the file system which can totally confuse
 *			 getwd.
 */

/*
 * $Header: /u5/alt/tools/src/cscope/RCS/getwd.c,v 1.9 1991/06/28 00:10:53 marvinw Exp $
 *
 * $Log: getwd.c,v $
 * Revision 1.9  1991/06/28  00:10:53  marvinw
 * Changed MIPS to mips.
 *
 * Revision 1.8  1991/05/21  15:49:10  erice
 * Added a specific MIPS construct to include fs.h from the proper directory.
 *
 * Revision 1.7  91/01/14  16:44:36  marvinw
 * Commented out call to getcwd().
 * 
 * Revision 1.6  90/02/12  11:10:58  jrb
 * changed to handle utek88k box
 * 
 * Revision 1.5  89/07/18  17:25:37  kimr
 * Made a pass at cleaning up lint errors and added RCS headers.
 * 
 *
 */

#include <sys/param.h>	/* ROOTINO */
#ifdef	S5ROOTINO	/* SVR3 changed the name of ROOTINO */
#define ROOTINO	S5ROOTINO
#endif
#if BSD
#ifdef sun
#include <ufs/fs.h>	/* ROOTINO */
#else
#ifdef mips
#include "/usr/include/sys/fs/ufs_fs.h"
#else
#include <sys/fs.h>	/* ROOTINO */
#endif
#endif
#else
#include <sys/types.h>	/* included by param.h in BSD */
#endif
#include <sys/stat.h>	/* stat */
#include <stdio.h>	/* NULL */
#include "ndir.h"	/* direct and directory access macros */

#define YES	1
#define NO 	0

extern char	*strcat(), *strcpy(), *strncpy();

char *
getwd(dir)
register char	*dir; 	/* destination buffer for name so far */
{
	register DIR *dirfile;		/* directory file descriptor */
	char	namebuf[512+1];		/* buffer for recursivly building path */
	char	cofp[MAXNAMLEN + 5]; 	/* child of parent name */
	char	*child;			/* child name */
	char	*pwd;			/* PWD environment variable value */
	struct	stat	d_sb;  		/* current directory status */
	struct	stat	dd_sb; 		/* parent directory status */
	struct	stat	tmp_sb; 	/* temporary stat buffer */
	int	atmount = NO;		/* at root of a mounted file system */
	struct	direct	*p;
	char	*getenv();
#if APOLLO | BLACKBIRD
	char	*getcwd();
	
	return(getcwd(dir, 512));
#else
/* HACK */
/********
return(getcwd(dir, 512));
********/
	/* get the current directory's status */
	if (stat(".", &d_sb) < 0) {
		return(NULL);
	}
	/* use $PWD if it matches this directory */
	if ((pwd = getenv("PWD")) != NULL && stat(pwd, &tmp_sb) == 0 &&
	    d_sb.st_ino == tmp_sb.st_ino && d_sb.st_dev == tmp_sb.st_dev) {
		(void) strcpy(dir, pwd);
		return(pwd);
	}
	/* open the parent directory */
	if ((dirfile = opendir("..")) == NULL) {
		return(NULL);
	}
	/* get the current and parent directories status */
	if (statdir(dirfile, &dd_sb) < 0) {
		(void) closedir(dirfile);
		return(NULL);
	}
	/*
	 * Determine if at a mount point or at the root of the root 
	 * file system.
	 */
	if (d_sb.st_ino == ROOTINO) {
		atmount = YES;
		if (d_sb.st_ino == dd_sb.st_ino && d_sb.st_dev == dd_sb.st_dev) {
			(void) strcpy(dir, "/"); /* start out right */
			(void) closedir(dirfile);
			return(dir);  		/* start to unwind */
		}
	}
	/*  find the name of "."  in ".." */
	(void) strcpy(cofp, "../");
	child = cofp + 3;
	for (;;) {
		if ((p = readdir(dirfile)) == NULL) {
			(void) closedir(dirfile);
			return(NULL);
		}
		if (p->d_ino == 0)
			continue;
		if (p->d_ino == d_sb.st_ino || atmount == YES) {
			(void) strcpy(child, p->d_name);
			/* 
			 * Must stat all the files in the parent 
			 * directory if the current directory is a mount
			 * point because stat returns a different inode
			 * number than is in the parent's directory.
			 */
			if (stat(cofp, &tmp_sb) < 0)  {
				(void) closedir(dirfile);
				return(NULL);
			}
			if ((tmp_sb.st_ino == d_sb.st_ino)
			    && (tmp_sb.st_dev == d_sb.st_dev))
				break;  /* found it */
		}
	}
	(void) closedir(dirfile);
	if (chdir("..") < 0)
		return(NULL);

	/* namebuf used as destination for invoking child */
	if (getwd(namebuf) == NULL)  {

		/* failure - backout by changing to child directory */
		(void) chdir(child);
		return(NULL);
	} else {
		/* 
		 * As the tail recursion unwinds add the current
		 * directoriy's name to the end of 'namebuf',
		 * and copy 'namebuf' to 'dir', while 
		 * descending the tree to the directory of 
		 * invocation.
		 */
		(void) strcpy(dir, namebuf);
		if (*(namebuf + strlen(namebuf) - 1) != '/') {
			/* previous call didn't find root */
			(void) strcat(dir, "/");
		}
		(void) strcat(dir, child);

		/* return to directory of invocation */
		(void) chdir(child);
		return(dir);
	}
#endif
}
