/*	Copyright (c) 1984, 1985, 1986, 1987 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*	cscope - interactive C symbol cross-reference
 *
 *	global type, data, and function definitions
 */

/*
 * $Header: /u5/alt/tools/src/cscope/RCS/global.h,v 1.5 1989/11/21 16:53:46 kimr Exp $
 *
 * $Log: global.h,v $
 * Revision 1.5  1989/11/21  16:53:46  kimr
 * Added "preload" variable.  If set to "p", pass extra parameter.  If
 * set to "v", use vi-style preloading.
 *
 * Revision 1.4  89/11/01  14:22:11  kimr
 * Added autolaunch for CSCOPEAUTOLAUNCH and fullfuncname for ^A.
 * 
 * Revision 1.3  89/10/26  18:10:25  kimr
 * Added prevlinetoo, which is set by the "-t" option to indicate that
 * reference lines for function defs should have to previous line prefixed
 * to the ref line to capture the function type.
 * 
 * Added fullfilename, which is toggled by the ^F command and controls
 * whether file names in ref lines are the full pathname or the basename.
 * Basename is default.
 * 
 * Revision 1.2  89/07/21  17:58:20  kimr
 * Added global boolean "keep_running" to indicate that editors should
 * run asynchronously with cscope.  Added backpage() to function list.
 * 
 *
 */

#include <curses.h>	/* LINES needed by constants.h */
#include <stdio.h>	/* standard I/O package */
#include "constants.h"	/* misc. constants */
#include "library.h"	/* library function return values */

typedef	enum	{		/* boolean data type */
	NO,
	YES
} BOOL;

typedef	struct {
	int	button;
	int	percent;
	int	x1;
	int	y1;
	int	x2;
	int	y2;
} MOUSE;

/* digraph data for text compression */
extern	char	dichar1[];	/* 16 most frequent first chars */
extern	char	dichar2[];	/* 8 most frequent second chars 
				   using the above as first chars */
extern	char	dicode1[];	/* digraph first character code */
extern	char	dicode2[];	/* digraph second character code */

/* main.c global data */
extern	BOOL	autolaunch;	/* auto edit if one ref for def or file */
extern	char	*editor, *home, *shell;
				/* environment variables */
extern	char	*argv0;		/* command name */
extern	BOOL	compress;	/* compress the characters in the crossref */
extern	BOOL	isuptodate;	/* consider the crossref up-to-date */
extern	int	fileargc;	/* file argument count */
extern	char	**fileargv;	/* file argument values */
extern	int	fileversion;	/* cross-reference file version */
extern	BOOL	fullfilename;	/* display full pathname for files */
extern	BOOL	fullfuncname;	/* display full name for functions */
extern	BOOL	keep_running;	/* continue after spawning $EDITOR */
extern	char	*namefile;	/* file of file names */
extern	char	*newreffile;	/* new cross-reference file name */
extern	FILE	*newrefs;	/* new cross-reference */
extern	BOOL	prevlinetoo;	/* display line preceeding func def also */
extern	char	preload;	/* preload editor search string if 'p' or 'v' */
extern	char	*reffile;	/* cross-reference file path name */
extern	FILE	*refsfound;	/* references found file */
extern	int	symrefs;	/* cross-reference file */
extern	char	temp1[];	/* temporary file name */
extern	char	temp2[];	/* temporary file name */


/* command.c global data */
extern	BOOL	*change;	/* change this line */
extern	BOOL	changing;	/* changing text */
extern	char	newpat[];	/* new pattern */
extern	char	pattern[];	/* symbol or text pattern */

/* crossref.c global data */
extern	BOOL	errorsfound;	/* prompt before clearing error messages */
extern	int	symbols;	/* number of symbols */

/* dir.c global data */
extern	char	currentdir[];	/* current directory */
extern	char	**incdirs;	/* #include directories */
extern	char	**srcdirs;	/* source directories */
extern	char	**srcfiles;	/* source files */
extern	char	**srcnames;	/* source file names */
extern	int	nincdirs;	/* number of #include directories */
extern	int	nsrcdirs;	/* number of source directories */
extern	int	nsrcfiles;	/* number of source files */
extern	int	msrcfiles;	/* maximum number of source files */

/* display.c global data */
extern	int	*displine;	/* screen line of displayed reference */
extern	int	disprefs;	/* displayed references */
extern	int	field;		/* input field */
extern	int	mdisprefs;	/* maximum displayed references */
extern	int	nextline;	/* next line to be shown */
extern	int	topline;	/* top line of page */
extern	int	bottomline;	/* bottom line of page */
extern	int	totallines;	/* total reference lines */

/* find.c global data */
extern	char	block[];	/* cross-reference file block */
extern	char	blockmark;	/* mark character to be searched for */
extern	char	*blockp;	/* pointer to current character in block */
extern	int	blocklen;	/* length of disk block read */

/* lookup.c global data */
extern	struct	keystruct {
	char	*text;
	char	delim;
	struct	keystruct *next;
} keyword[];

/* mouse.c global data */
extern	BOOL	mouse;		/* mouse interface */

/* scanner.l global data */
extern	int	first;		/* buffer index for first char of symbol */
extern	int	last;		/* buffer index for last char of symbol */
extern	FILE	*yyin;		/* input file descriptor */
extern	int	yyleng;		/* length of input line */
extern	int	yylineno;	/* number of input line */
extern	char	yytext[];	/* input line text */

/* cscope functions called from more than one function or between files */ 
char	*findregexp(), *readblock(), *scanpast();
void	addsrcfile(), askforchar(), atfield(), backpage(), build(), 
	beginlex(), beginsdl(), display(), draw_scrollbar(), edit(), 
	incfile(), makefilelist(), mousecleanup(), mousemenu(), mouseinit(), 
	mousereinit(), myexit(), myperror(), progress(), putfilename(), 
	putmsg(), putstring(), seekline(), setfield(), shellpath(), warning();
MOUSE	*getmouse();
