/*	Copyright (c) 1984, 1985, 1986, 1987 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*	cscope - interactive C symbol cross-reference
 *
 *	terminal input functions
 */

#include "global.h"
/*
 * $Header: /u5/alt/tools/src/cscope/RCS/input.c,v 1.3 1989/10/26 15:11:08 kimr Exp $
 *
 * $Log: input.c,v $
 * Revision 1.3  1989/10/26  15:11:08  kimr
 * Cause ESC to kill current input pattern.  Cause TAB to print as space
 * so backspace will keep display cleaned up.
 *
 * Revision 1.2  89/07/18  17:25:42  kimr
 * Made a pass at cleaning up lint errors and added RCS headers.
 * 
 *
 */

#include <ctype.h>	/* iscntrl and isspace */
#include <setjmp.h>	/* jmp_buf */
#include <signal.h>	/* SIGINT */

static	jmp_buf	env;	/* setjmp/longjmp buffer */

/* catch the interrupt signal */

static
catchint()
{
	signal(SIGINT, catchint);
	longjmp(env, 1);
}

/* get a character from the terminal */

mygetch()
{
	void	(*savesig)();		/* old value of signal */
	int	c;

	/* change an interrupt signal to a break key character */
	if (setjmp(env) == 0) {
		savesig = signal(SIGINT, catchint);
		refresh();	/* update the display */
		mousereinit();	/* curses can change the menu number */
		c = getch();	/* get a character from the terminal */
	}
	else {	/* longjmp to here from signal handler */
		c = KEY_BREAK;
	}
	signal(SIGINT, savesig);
	return(c);
}

/* get a line from the terminal in non-canonical mode */

getline(s, limit, firstchar)
register char	s[];
register int	limit;
int	firstchar;
{
	register int	c, i = 0;
	int	j;

	/* if a character already has been typed */
	if (firstchar != '\0') {
		addch(firstchar);	/* display it */
		s[i++] = firstchar;	/* save it */
	}
	/* until the end of the line is reached */
	while ((c = mygetch()) != '\r' && c != '\n' && c != KEY_ENTER && 
	    i < limit) {
		if (c == erasechar() || c == KEY_BACKSPACE) {	/* erase */
			if (i > 0) {
				addstr("\b \b");
				--i;
			}
		}
		else if (c == killchar() || c == KEY_BREAK || c == ESC) {	/* kill */
			for (j = 0; j < i; ++j) {
				addch('\b');
			}
			for (j = 0; j < i; ++j) {
				addch(' ');
			}
			for (j = 0; j < i; ++j) {
				addch('\b');
			}
			i = 0;
			if (c == ESC) {	/* escape aborts getline */
			    s[i] = ESC;
			    return(i);
			}
		}
		else if (isprint(c)) {	/* printable */
			addch(c);	/* display it */
			s[i++] = c;	/* save it */
		}
		else if (c == '\t') {	/* tab */
			addch(' ');	/* display as space so backspace works*/
			s[i++] = c;	/* save it */
		}
		else if (mouse == YES && c == ctrl('X')) {	/* mouse */
			(void) getmouse();	/* ignore it */
		}
		else if (c == EOF) {				/* end-of-file */
			break;
		}
		/* return on an empty line to allow a command to be entered */
		if (firstchar != '\0' && i == 0) {
			break;
		}
	}
	s[i] = '\0';
	return(i);
}

/* ask user to enter a character after reading the message */

void
askforchar()
{
	addstr("Type any character to continue: ");
	mygetch();
}

/* expand the ~ and $ shell meta characters in a path */

void
shellpath(out, limit, in) 
register char	*out;
int	limit; 
register char	*in;
{
	register char	*lastchar;
	char	*s, *v;

	/* skip leading white space */
	while (isspace(*in)) {
		++in;
	}
	lastchar = out + limit - 1;

	/* a tilde (~) by itself represents $HOME; followed by a name it
	   represents the $LOGDIR of that login name */
	if (*in == '~') {
		*out++ = *in++;	/* copy the ~ because it may not be expanded */

		/* get the login name */
		s = out;
		while (s < lastchar && *in != '/' && *in != '\0' && !isspace(*in)) {
			*s++ = *in++;
		}
		*s = '\0';

		/* if the login name is null, then use $HOME */
		if (*out == '\0') {
			v = getenv("HOME");
		}
		else {	/* get the home directory of the login name */
			v = logdir(out);
		}
		/* copy the directory name */
		if (v != NULL) {
			(void) strcpy(out - 1, v);
			out += strlen(v) - 1;
		}
		else {	/* login not found, so ~ must be part of the file name */
			out += strlen(out);
		}
	}
	/* get the rest of the path */
	while (out < lastchar && *in != '\0' && !isspace(*in)) {

		/* look for an environment variable */
		if (*in == '$') {
			*out++ = *in++;	/* copy the $ because it may not be expanded */

			/* get the variable name */
			s = out;
			while (s < lastchar && *in != '/' && *in != '\0' &&
			    !isspace(*in)) {
				*s++ = *in++;
			}
			*s = '\0';
	
			/* get its value */
			if ((v = getenv(out)) != NULL) {
				(void) strcpy(out - 1, v);
				out += strlen(v) - 1;
			}
			else {	/* var not found, so $ must be part of the file name */
				out += strlen(out);
			}
		}
		else {	/* ordinary character */
			*out++ = *in++;
		}
	}
	*out = '\0';
}
