/*	Copyright (c) 1984, 1985, 1986, 1987 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*	cscope - interactive C symbol cross-reference
 *
 *	main functions
 */

/*
 * $Header: /u5/alt/tools/src/cscope/RCS/main.c,v 1.21 1990/02/12 11:15:46 jrb Exp $
 *
 * $Log: main.c,v $
 * Revision 1.21  1990/02/12  11:15:46  jrb
 * ported to utek 88k box
 *
 * Revision 1.20  89/11/22  15:08:57  kimr
 * *** empty log message ***
 * 
 * Revision 1.19  89/11/22  15:04:36  kimr
 * *** empty log message ***
 * 
 * Revision 1.18  89/11/22  15:02:40  kimr
 * Added advice to use stty to set rows and columns if ioctl fails.
 * 
 * Revision 1.17  89/11/21  16:52:51  kimr
 * Added "-t" to usage message.
 * 
 * Revision 1.16  89/11/07  17:19:44  kimr
 * Use environment variable CSCOPEPRELOAD to set value of "preload" global.
 * 
 * Revision 1.15  89/11/01  14:15:02  kimr
 * Added handling for CSCOPEAUTOLAUNCH environment variable.  Change to
 * always rebuild the database file if the FILEVERSION was changed.
 * Used to not rebuild if files were up to date.
 * 
 * Revision 1.14  89/10/26  18:12:48  kimr
 * Added prevlinetoo, which is set by the "-t" option to indicate that
 * reference lines for function defs should have to previous line prefixed
 * to the ref line to capture the function type.
 * 
 * ws_row and ws_col were unsigned, so made "<= 0" be "== 0".
 * 
 * Revision 1.13  89/10/23  15:03:17  kimr
 * Mention file name in errors opening files.  Stop trying to open
 * cscope.out in the user's home directory if the current directory
 * isn't writable - just give a decent error and let the user figure it
 * out.  This is a fix for CRL177 and CRL189.
 * 
 * Revision 1.12  89/07/30  13:23:37  kimr
 * Well, if you try to use cbreak mode before it's set, it doesn't work very
 * well.  Changed "press any character" to "press RETURN."
 * 
 * Revision 1.11  89/07/30  13:14:01  kimr
 * Moved pause for initial messages before curses initialization. It was
 * clearing the screen and then waiting before.
 * 
 * Revision 1.10  89/07/27  10:20:36  kimr
 * Renamed myresize to resize.  Put in resize optimization to not redisplay if
 * the new size is the same as the old size, as well as setting LINES to an
 * invalid value initially to force first screen to display.
 * 
 * Revision 1.9  89/07/24  18:44:31  kimr
 * Added stuff to resize() to not change LINES or COLS until
 * we're positive ioctl got good ones.  If they aren't good from
 * ioctl (like they're zero if "tset" didn't get invoked), ask
 * the user if he wants to continue with a default 24x80.
 * All this was to fix a segmentation fault if you started cscope
 * and tset hadn't been run.
 * 
 * Revision 1.8  89/07/21  17:57:02  kimr
 * Deleted "-w" flag - superceded by CSCOPEWINEDIT.  Changed CSCOPEEDIT to 
 * CSCOPEWINEDIT and made it spawn editors if set.  Fixed resize() to
 * deallocate and reallocate the curses buffers.
 * 
 * Revision 1.7  89/07/20  14:17:34  kimr
 * Added CSCOPEEDIT environment variable which overrides EDITOR/VIEWER.
 * Fixed resize to not page display forward.
 * 
 * Revision 1.6  89/07/19  19:21:34  kimr
 * Now handles SIGWINCH signal.  Gets initial window size from ioctl 
 * instead of TERMCAP, since UTek loginterm apparently lies to curses.
 * Has debugging writes to /dev/console.
 * 
 * Revision 1.5  89/07/18  17:25:50  kimr
 * Made a pass at cleaning up lint errors and added RCS headers.
 * 
 *
 */

/*
Symbol database (cscope.out file) format

	header
	file 1 symbol data
	...
	file n symbol data
	trailer

The header is a single line

	cscope <database format> <current directory> <trailer pointer>

When the database format version number in the cscope program is greater than that
in the database, the entire database will be rebuilt when any part of it is
out-of-date.  The current directory is either a full path or prefixed by
$HOME, allowing the user's login to be moved to a different file system
without rebuilding the database.  The trailer pointer is the lseek(2)
offset of the trailer.

The header is followed by the symbol data for each file in alphabetical
order.  This allows fast updating of the database when only a few files
have changed, because cscope just copies the old database to the new up to
the point of the changed file, builds a new database for it, copies up to
the next changed file, etc.

Note: two data compression techniques are used: (1) keywords and trailing
syntax are converted to control characters, and (2) common digraphs
(character pairs) are compressed to meta-characters.  Since this makes it
hard to read or postprocess the database, you may want to use the -c option
to turn off data compression when the database is built.

The symbol data for each file starts with

	<file tag><file path>
	<empty line>

and for each source line containing a symbol

	<line number><blank><non-symbol text>
	<optional tag><symbol>
	<non-symbol text>
	repeat above 2 lines as necessary
	<empty line>

The symbol data for the last file ends with

	<file tag>
	
A tag is a tab followed by one of these characters:

	Char	Meaning
	@	new file
	#	#define
	~	#include
	$	function definition
	`	function call

I picked the first tag characters from those not appearing in C, but
realized that all can appear in a string constant, so I added the tab
prefix.  I never changed to more meaningful characters.  Cscope changes
tabs to blanks in the non-symbol source line text in the database.

The trailer contains lists of source directories, include directories, and
source files; its format is

	<number of source directories> .
	<second source directory path>
	...
	<last source directory path>
	<number of include directories> <first include directory path>
	<second include directory path>
	...
	/usr/include
	<number of source and include files> <first file path>
	<second file path>
	...
	<last file path>

The trailer was part of the header prior to cscope version 8.  The problem
was that the source file list did not have #include'd files because they
weren't known until the files were read and the symbol data output.  This
list is used by findstring() and findregexp(), so the #include'd files were
not searched if the database was up-to-date.  Outputting the source file
list after the symbol data solved this problem.  The directory lists could
have remained part of the header; it was just easier move all but the first
line of the header to the trailer.
*/

#include "global.h"
#include "version.h"	/* FILEVERSION and FIXVERSION */
#include <ctype.h>	/* isdigit */
#include <fcntl.h>	/* O_RDONLY */
#include <signal.h>	/* SIGINT, SIGQUIT, and SIGWINCH */
#include <sys/types.h>	/* needed by stat.h */
#include <sys/stat.h>	/* stat */
#include <sys/file.h>
#include <sys/ioctl.h>
#ifndef BSD
#include <sys/termios.h>
#endif
#ifdef BLACKBIRD
#include <setjmp.h>
jmp_buf env;
#endif

/* command defaults */
#define	EDITOR	"vi"
#define	PAGER	"more"
#define	SHELL	"sh"

#define	TEMPLEN	23	/* temporary file name length */

/* note: these digraph character frequencies were calculated from possible 
   printable digraphs in the cross-reference for the C compiler */
char	dichar1[] = " teisaprnl(of)=c";	/* 16 most frequent first chars */
char	dichar2[] = " tnerpla";		/* 8 most frequent second chars 
					   using the above as first chars */
char	dicode1[128];		/* digraph first character code */
char	dicode2[128];		/* digraph second character code */

BOOL	autolaunch = YES;	/* auto edit if one def or file */
char	*editor, *home, *shell;	/* environment variables */
char	*argv0;			/* command name */
BOOL	compress = YES;		/* compress the characters in the crossref */
BOOL	isuptodate = NO;	/* consider the crossref up-to-date */
int	fileargc;		/* file argument count */
char	**fileargv;		/* file argument values */
int	fileversion;		/* cross-reference file version */
BOOL	fullfilename = NO;	/* display basename only */
BOOL	fullfuncname = YES;	/* display unabbreviated function name */
BOOL	keep_running = NO;      /* should I continue after spawning $EDITOR? */
char	*namefile;		/* file of file names */
char	*newreffile;		/* new cross-reference file name */
FILE	*newrefs;		/* new cross-reference */
BOOL	prevlinetoo = NO;	/* don't display line preceeding func def */
char    preload = ' ';          /* preload editor search string if 'p' or 'v' */
char	*reffile = REFFILE;	/* cross-reference file path name */
FILE	*refsfound;		/* references found file */
int	symrefs = -1;		/* cross-reference file */
char	temp1[TEMPLEN + 1];	/* temporary file name */
char	temp2[TEMPLEN + 1];	/* temporary file name */

static	BOOL	buildonly = NO;	/* only build the cross-reference */
static	BOOL	incurses = NO;	/* in curses */
static	long	offset;		/* file trailer offset */

#ifdef DEBUG
FILE *console;
#endif

main(argc, argv)
int	argc;
char	**argv;
{
	char	file[PATHLEN + 1];	/* file name */
	register char	*s;
	register int	c, i;
	BOOL	launchstate; 		/* CSCOPEAUTOLAUNCH value */
	BOOL	command();
	char	*mygetenv();
	void	dispinit(), includedir(), init_symtab(), sourcedir();
	
	/* save the command name for messages */
	argv0 = argv[0];
	
	/* set the options */
	while (--argc > 0 && (*++argv)[0] == '-') {
		for (s = argv[0] + 1; *s != '\0'; s++) {
			
			/* look for an input field number */
			if (isdigit(*s)) {
				field = *s - '0';
				if (field > 8) {
					field = 8;
				}
				if (*++s == '\0' && --argc > 0) {
					s = *++argv;
				}
				(void) strcpy(pattern, s);
				goto nextarg;
			}
			switch (*s) {
			case '-':	/* end of options */
				--argc;
				++argv;
				goto lastarg;
			case 'V':	/* print the version number */
				fprintf(stderr, "%s: version %d%s\n", argv0,
					FILEVERSION, FIXVERSION);
				exit(0);
			case 'b':	/* only build the cross-reference */
				buildonly = YES;
				break;
			case 'c':	/* ASCII characters only in crossref */
				compress = NO;
				break;
			case 'd':	/* consider crossref up-to-date */
				isuptodate = YES;
				break;
			case 'f':	/* alternate cross-reference file */
			case 'i':	/* file containing file names */
			case 'I':	/* #include file directory */
			case 's':	/* additional source file directory */
			case 'S':
				c = *s;
				if (*++s == '\0' && --argc > 0) {
					s = *++argv;
				}
				switch (c) {
				case 'f':	/* alternate cross-reference file */
					reffile = s;
					break;
				case 'i':	/* file containing file names */
					namefile = s;
					break;
				case 'I':	/* #include file directory */
					includedir(s);
					break;
				case 's':	/* additional source directory */
				case 'S':
					sourcedir(s);
					break;
				}
				goto nextarg;
			case 't':	/* display func type with func def */
				prevlinetoo = YES;
				break;
			default:
				fprintf(stderr, "%s: unknown option: -%c\n", argv0, *s);
				fprintf(stderr, "Usage:  cscope [-bcdt] [-f cross-reference file] [-i name file]\n");
				fprintf(stderr, "               [-I include dir] [-s source dir] [-[0-8] text] [source files]\n");
				exit(1);
			}
		}
nextarg:	;
	}
lastarg:
	/* read the environment */
	if ((editor = mygetenv("CSCOPEWINEDIT", NULL)) != NULL) {
		keep_running = YES;
	} else {
		editor = mygetenv("EDITOR", EDITOR);
		editor = mygetenv("VIEWER", editor); /* use viewer if set */
	}
	home = getenv("HOME");
	shell = mygetenv("SHELL", SHELL);
	if (strcmp(mygetenv("CSCOPEAUTOLAUNCH", ""), "no") == 0) {
		autolaunch = NO;
	}
	launchstate = autolaunch; /* record initial value */
	{ 
	char	*value;
	value = getenv("CSCOPEPRELOAD");
	    if (value != NULL) { /* environment variable is set */
		if (strcmp(value, "vi") == 0) { 
		    preload = 'v'; /* use "vi-style" preloading */
		} else {  
		    preload = 'p'; /* pass third argument */
		}
	    }
	}
	/* set up the digraph character tables for text compression */
	if (compress == YES) {
		for (i = 0; i < 16; ++i) {
			dicode1[dichar1[i]] = i * 8 + 1;
		}
		for (i = 0; i < 8; ++i) {
			dicode2[dichar2[i]] = i + 1;
		}
	}
	/* if the cross-reference is to be considered up-to-date */
	if (isuptodate == YES) {
		FILE	*oldrefs;	/* old cross-reference */
		
		if ((oldrefs = fopen(reffile, "r")) == NULL) {
			fprintf(stderr, "cscope: cannot open file "); 
			perror(reffile);
			exit(1);
		}
		/* get the crossref file version but skip the current directory */
		fscanf(oldrefs, "cscope %d %*s", &fileversion);
		
		/* get the file trailer offset and seek to it */
		if (fileversion >= 8) {
			fscanf(oldrefs, "%d", &offset);
			fseek(oldrefs, offset, 0);
		}
		/* skip the source directory list */
		fscanf(oldrefs, "%d", &nsrcdirs);
		for (i = 0; i < nsrcdirs; ++i) {
			fscanf(oldrefs, "%*s");
		}
		/* skip the #include directory list */
		fscanf(oldrefs, "%d", &nincdirs);
		for (i = 0; i < nincdirs; ++i) {
			fscanf(oldrefs, "%*s");
		}
		/* get the number of source files */
		fscanf(oldrefs, "%d", &nsrcfiles);

		/* get the source file list */
		srcfiles = (char **) mymalloc(nsrcfiles * sizeof(char *));
		for (i = 0; i < nsrcfiles; ++i) {
			fscanf(oldrefs, "%s", file);
			srcfiles[i] = stralloc(file);
		}
		fclose(oldrefs);
	} else {
		/* save the file arguments */
		fileargc = argc;
		fileargv = argv;
	
		/* get source directories from the environment */
		if ((s = getenv("SOURCEDIRS")) != NULL) {
			sourcedir(s);
		}
		/* make the source file list */
		srcfiles = (char **) mymalloc(msrcfiles * sizeof(char *));
		srcnames = (char **) mymalloc(msrcfiles * sizeof(char *));
		makefilelist();
		if (nsrcfiles == 0) {
			fprintf(stderr, "cscope: no source files found\n");
			exit(1);
		}
		/* get include directories from the environment */
		if ((s = getenv("INCLUDEDIRS")) != NULL) {
			includedir(s);
		}
		/* add /usr/include to the #include directory list */
		includedir("/usr/include");

		/* initialize the C keyword table */
		init_symtab();
	
		/* create the file name used for a new cross-reference */
		newreffile = mymalloc((unsigned) (strlen(reffile) + strlen(home) + 3));
		if ((s = strrchr(reffile, '/')) != NULL) {
			++s;
			(void) strcpy(newreffile, reffile);
			sprintf(newreffile + (s - reffile), "n%s", s);
		}
		else {
			sprintf(newreffile, "n%s", reffile);
		}
		/* build the cross-reference */
		build();
		if (buildonly == YES) {
			exit(0);
		}
	}
#ifdef DEBUG
{ int i;
printf("Source files/names:\n");
for (i=0; i < nsrcfiles; i++) 
	printf("%d\t%s\t%s\n", i, srcfiles[i], srcnames[i]);
}
#endif
	/* open the cross-reference file */
	if ((symrefs = open(reffile, O_RDONLY, 0)) == -1) {
		fprintf(stderr, "cscope: cannot open file ");
		perror(reffile);
		exit(1);
	}
	/* ignore the interrupt signal and if running in the foreground */
	if (signal(SIGINT, SIG_IGN) != SIG_IGN) {
		void myexit();
		void resize();

		/* cleanup on the quit signal */
		signal(SIGQUIT, myexit);
 		signal(SIGWINCH, resize);
	}
#ifdef BLACKBIRD
	/*
	 * job control works on UTEKV but not quite
	 * as berkeley as elsewhere. Unknown to me
	 * why. On a "normal" Berkeley system, the
	 * application doesn't need to catch the job
	 * control signals and do terminal switching say
	 * from half-cooked to raw. Unfortunately that
	 * doesn't seem to be the case here. So we catch
	 * the signals, massage the terminal modes, and 
	 * setjmp/longjmp into the command loop.
	 */
	{
		int onstop();
		int onstart();

		signal(SIGTSTP, onstop);
		signal(SIGCONT, onstart);

	}
#endif
	/* create the temporary files */
	i = getpid();
	sprintf(temp1, "/usr/tmp/cscope%d.1", i);
	sprintf(temp2, "/usr/tmp/cscope%d.2", i);

	/* pause before clearing the screen if there have been error messages */
	if (errorsfound == YES) {
		errorsfound = NO;
		fprintf(stderr, "Press RETURN to continue: ");
		while (getchar() != '\n') ;
	}

#ifdef BLACKBIRD
	setjmp(env);
#endif

	/* initialize the curses display package */
	initscr();		/* initialize the screen */
	nonl();			/* don't translate an output \n to \n\r */
	cbreak();		/* single character input */
	noecho();		/* don't echo input characters */
#ifdef DEBUG
	if ((console = fopen("/dev/console", "a")) == NULL) {
	    perror("open of console failed:");
	    exit(0);
	} 
#endif


	/* vax curses getch() intermittently returns garbage with keypad enabled */
#if TERMINFO && !vax
	/* keypad(stdscr, TRUE);	enable the keypad */
#endif
#if UNIXPC
	standend();		/* turn off reverse video */
#endif
	incurses = YES;

	LINES = -1; /* force redisplay by resize; Don't call curses stuff! */
	resize(0);   /* get size from ioctl to get around UTek loginterm bug */
	mouseinit();  /* initialize any mouse interface */
	
	/* do any optional search */
	if (*pattern != '\0') {
		atfield();	/* move to the input field */
		command('.');	/* search */
		display();	/* update the display */
	}
	for (;;) {
		atfield();	/* move to the input field */
		
		/* exit if the quit command is entered */
		if ((c = mygetch()) == EOF || c == ctrl('D') || c == ctrl('Z')) {
			break;
		}
		/* execute the commmand, updating the display if necessary */
		if (command(c) == YES) {
			display();
		}
	autolaunch = launchstate; /* ^T command may have cleared flag. */
	}
	/* cleanup and exit */
	myexit(0);
	/* NOTREACHED */
}

/* return the environment value or, if not found, the default arg */

char	*
mygetenv(variable, deflt)
char	*variable, *deflt;
{
	char	*value;

	value = getenv(variable);
	if (value == NULL || strlen(value) == 0) {
		return(deflt);
	}
	return(value);
}

/* build the cross-reference */

void
build()
{
	char	*oldfile;		/* file in old cross-reference */
	char	newdir[PATHLEN + 1];	/* directory in new cross-reference */
	char	olddir[PATHLEN + 1];	/* directory in old cross-reference */
	char	oldname[PATHLEN + 1];	/* name in old cross-reference */
	int	oldnum;			/* number in old cross-ref */
	struct	stat statstruct;	/* file status */
	time_t	reftime;		/* old crossref modification time */
	FILE	*oldrefs;		/* old cross-reference */
	int	firstfile;		/* first source file in pass */
	int	lastfile;		/* last source file in pass */
	int	built = 0;		/* built crossref for these files */
	int	copied = 0;		/* copied crossref for these files */
	register char	*cp;
	register int	i;
	int	compare();		/* needed for qsort call */
	void	crossref();
	char	*getoldfile();
	void	buildprogress();

	/* normalize the current directory relative to the home directory so
	   the cross-reference is not rebuilt when the user's login is moved */
	(void) strcpy(newdir, currentdir);
	if (strcmp(currentdir, home) == 0) {
		(void) strcpy(newdir, "$HOME");
	}
	else if (strncmp(currentdir, home, strlen(home)) == 0) {
		sprintf(newdir, "$HOME%s", currentdir + strlen(home));
	}
	/* sort the source file names (needed for rebuilding) */
	qsort((char *) srcfiles, (unsigned) nsrcfiles, sizeof(char *), compare);

	/* if there is an old cross-reference and its current directory matches */
	if ((oldrefs = fopen(reffile, "r")) != NULL &&
	    fscanf(oldrefs, "cscope %d %s", &fileversion, olddir) == 2 &&
	    (strcmp(olddir, currentdir) == 0 || /* remain compatible */
	     strcmp(olddir, newdir) == 0)) {

		/* if the database format has changed, rebuild it all */
		if (fileversion != FILEVERSION) {
			fprintf(stderr, "cscope: converting to new format for cscope.out\n");
			goto force;
		}
		
		/* get the cross-reference file's modification time */
		fstat(fileno(oldrefs), &statstruct);
		reftime = statstruct.st_mtime;
		
		/* get the file trailer offset and seek to it */
		if (fileversion >= 8) {
			fscanf(oldrefs, "%d", &offset);
			fseek(oldrefs, offset, 0);
		}
		/* see if the number of source directories is the same */
		if (fscanf(oldrefs, "%d", &oldnum) < 1 ||
		    oldnum != nsrcdirs) {
			goto outofdate;
		}
		/* see if the source directory list is the same */
		for (i = 0; i < nsrcdirs; ++i) {
			if (fscanf(oldrefs, "%s", oldname) < 1 ||
			    strnotequal(oldname, srcdirs[i])) {
				goto outofdate;
			}
		}
		/* see if the number of #include directories is the same */
		if (fscanf(oldrefs, "%d", &oldnum) < 1 ||
		    oldnum != nincdirs) {
			goto outofdate;
		}
		/* see if the #include directory list is the same */
		for (i = 0; i < nincdirs; ++i) {
			if (fscanf(oldrefs, "%s", oldname) < 1 ||
			    strnotequal(oldname, incdirs[i])) {
				goto outofdate;
			}
		}
		fscanf(oldrefs, "%d", &oldnum);	/* skip the number of files */
		/* see if the list of source files is the same and
		   none have been changed up to the included files */
		for (i = 0; i < nsrcfiles; ++i) {
			if (fscanf(oldrefs, "%s", oldname) < 1 ||
			    strnotequal(oldname, srcfiles[i]) ||
			    stat(srcfiles[i], &statstruct) != 0 ||
			    statstruct.st_mtime > reftime) {
				goto outofdate;
			}
		}
		/* the old cross-reference is up-to-date */
		/* so get the list of included files */
		while (i++ < oldnum) {
			fscanf(oldrefs, "%s", oldname);
			addsrcfile(basename(oldname), oldname);
		}
		fclose(oldrefs);
		return;
		
	outofdate:
		/* reopen the old cross-reference file for fast scanning */
		if ((symrefs = open(reffile, O_RDONLY, 0)) == -1) {
			fprintf(stderr, "cscope: cannot open file ");
			perror(reffile);
			myexit(1);
		}
		/* get the first file name in the old cross-reference */
		readblock();		/* read the first cross-ref block */
		scanpast('\t');		/* skip the header */
		oldfile = getoldfile();
	}
	else {	/* force cross-referencing of all the source files */
	force:	reftime = 0;
		oldfile = NULL;
	}
	/* open the new cross-reference file */
	if ((newrefs = fopen(newreffile, "w")) == NULL) {
		fprintf(stderr, "cscope: cannot open file ");
		perror(newreffile);
		myexit(1);
	}
	/* output the cscope version, current directory, and space for the
	   file trailer offset */
	fprintf(newrefs, "cscope %d %s %10ld\n", FILEVERSION, newdir, 0l);
	fileversion = FILEVERSION;

	/* output the leading tab expected by crossref() */
	putc('\t', newrefs);

	/* make passes through the source file list until the last level of
	   included files is processed */
	fprintf(stderr, "cscope: building symbol cross-reference\n");
	firstfile = 0;
	lastfile = nsrcfiles;
	for (;;) {

		/* get the next source file name */
		for (i = firstfile; i < lastfile; ++i) {
			
			/* report progress every 3 seconds */
			buildprogress(built, copied);
			
			/* if the old file has been deleted get the next one */
			while (oldfile != NULL && strcmp(srcfiles[i], oldfile) > 0) {
				oldfile = getoldfile();
			}
			/* if there isn't an old database or this is a new file */
			if (oldfile == NULL || strcmp(srcfiles[i], oldfile) < 0) {
				crossref(srcfiles[i]); /* crossref it */
				++built;
			}
			/* if this file was modified */
			else if (stat(srcfiles[i], &statstruct) == 0 &&
			    statstruct.st_mtime > reftime) {
				crossref(srcfiles[i]); /* crossref it */
				++built;
				
				/* skip its old crossref so modifying the last
				   source file does not cause all included files
				   to be built.  Unfortunately a new file that is
				   alphabetically last will cause all included
				   files to be build, but this is less likely */
				oldfile = getoldfile();
			}
			else {	/* copy its cross-reference */
				++copied;
				putfilename(srcfiles[i]);
				setmark('\t');
				cp = blockp;
				for (;;) {
					/* copy up to the next \t */
					do {	/* innermost loop optimized to only one test */
						while (*cp != '\t') {
							putc(*cp++, newrefs);
						}
					} while (*++cp == '\0' && (cp = readblock()) != NULL);

					putc('\t', newrefs);	/* copy the \t */
					
					/* get the next character */
					if (*(cp + 1) == '\0') {
						cp = readblock();
					}
					/* exit if at the end of this file's xref */
					if (cp == NULL || *cp == NEWFILE) {
						break;
					}
					/* look for an #included file */
					if (*cp == INCLUDE) {
						int	c, j;
						
						putc(INCLUDE, newrefs);
						blockp = cp;
						getrefchar();
						putstring(oldname);
						incfile(oldname + 1, *oldname);

						/* compress digraphs */
						for (j = 0; (c = oldname[j]) != '\0'; ++j) {
							if (dicode1[c] && dicode2[oldname[j + 1]]) {
								c = (0200 - 2) + dicode1[c] + dicode2[oldname[j + 1]];
								++j;
							}
							putc(c, newrefs);
						}
						setmark('\t');
						cp = blockp;
					}
				}
				blockp = cp;
				oldfile = getoldfile();
			}
		}
		/* see if any included files were found */
		if (lastfile == nsrcfiles) {
			break;
		}
		firstfile = lastfile;
		lastfile = nsrcfiles;
	
		/* sort the included file names */
		qsort((char *) &srcfiles[firstfile], (unsigned) (lastfile - 
			firstfile), sizeof(char *), compare);
	}
	/* add a null file name to the trailing tab */
	fprintf(newrefs, "%c\n", NEWFILE);

	/* get the file offset */
	offset = ftell(newrefs);
	
	/* output the source directory count and names */
	fprintf(newrefs, "%d ", nsrcdirs);
	for (i = 0; i < nsrcdirs; ++i) {
		fprintf(newrefs, "%s\n", srcdirs[i]);
	}
	/* output the #include directory count and names */
	fprintf(newrefs, "%d ", nincdirs);
	for (i = 0; i < nincdirs; ++i) {
		fprintf(newrefs, "%s\n", incdirs[i]);
	}
	/* output the source file count and names */
	fprintf(newrefs, "%d ", nsrcfiles);
	for (i = 0; i < nsrcfiles; ++i) {
		fprintf(newrefs, "%s\n", srcfiles[i]);
	}
	/* rewrite the header with the trailer offset */
	rewind(newrefs);
	fprintf(newrefs, "cscope %d %s %10ld\n", FILEVERSION, newdir, offset);
	fclose(newrefs);
	
	/* close and remove the old cross-reference file */
	if (symrefs >= 0) {
		close(symrefs);
	}
	if (oldrefs != NULL) {
		fclose(oldrefs);
		unlink(reffile);
	}
	/* replace it with the new cross-reference file */
	if (link(newreffile, reffile) == -1) {
		fprintf(stderr, "cscope: cannot link file %s to file %s\n", 
			newreffile, reffile);
		perror("cscope");
		myexit(1);
	}
	if (unlink(newreffile) == -1) {
		fprintf(stderr, "cscope: cannot unlink file %s\n", newreffile);
		perror("cscope");
		errorsfound = YES;
	}
}

/* display search progress */

void
buildprogress(built, copied)
int	built, copied;
{
	static	long	start;
	long	now;
	long	time();

	/* save the start time */
	if (built == 0 && copied == 0) {
		start = time((long *) NULL);
	}
	/* display the progress every 3 seconds */
	else if ((now = time((long *) NULL)) - start >= 3) {
		start = now;
		fprintf(stderr, "cscope: %d files built, %d files copied\n", 
			built, copied);
	}
}

/* get the next file name in the old cross-reference */

static	char *
getoldfile()
{
	static	char	file[PATHLEN + 1];	/* file name in old crossref */

	if (blockp != NULL) {
		do {
			if (*blockp == NEWFILE) {
				getrefchar();
				putstring(file);
				if (file[0] != '\0') {	/* if not end-of-crossref */
					return(file);
				}
				return(NULL);
			}
		} while (scanpast('\t') != NULL);
	}
	return(NULL);
}
	
/* string comparison function for qsort */

static	int
compare(s1, s2)
char	**s1;
char	**s2;
{
	return(strcmp(*s1, *s2));
}

/* cleanup and exit */

void
myexit(sig)
int	sig;
{
	/* remove any temporary files */
	if (temp1[0] != '\0') {
		unlink(temp1);
		unlink(temp2);
	}
	/* restore the terminal to its original mode */
	if (incurses == YES) {
#if BSD || UNIXPC
		clear();
		refresh();
#endif
		endwin();
		mousecleanup();
	}
	/* dump core for debugging on the quit signal */
	if (sig == SIGQUIT) {
		abort();
	}
	exit(sig);
}

void
resize(sig)
{
    int fd;
    struct winsize arg;

    if((fd = open ("/dev/tty", O_WRONLY)) >= 0){
        ioctl (fd, TIOCGWINSZ, &arg);
	close(fd);
#if DEBUG
	fprintf(console, "ttytype = %s, LINES = %d, COLS = %d\n", ttytype, LINES, COLS);
	fflush(console);
#endif
	if ((arg.ws_col == 0) || (arg.ws_row == 0)) {
	    arg.ws_col = 80;
	    arg.ws_row = 24;
#ifndef BLACKBIRD
	/* ioctl fails unless resize utility is run on XD88.
	 * I don't see any reason to inform anybody that we
	 * are setting the default configuration. jrb.
	 */
printf("\ncscope: Couldn't get valid tty size with TIOCGWINSZ ioctl.\n\r");
printf(  "cscope: Execute      \"stty rows <rowcnt> cols <colcnt>\"\n\r");
printf(  "cscope: before starting cscope to set your display size.\n\r");
printf(  "cscope: Hit any character to assume 24x80, ^D to terminate: ");
#endif

	    fflush(stdout);
	    fflush(stdin);
#ifndef BLACKBIRD
	    if (getchar() == ctrl('D')) {
	        printf("\n");
		myexit(1);
	    }
#endif
	}

	if ((LINES == arg.ws_row) && (COLS == arg.ws_col)) {
	    return; /* window didn't change size */
	}

	LINES = arg.ws_row;
	COLS  = arg.ws_col;
#if DEBUG
	fprintf(console, "new LINES = %d, COLS = %d\n", LINES, COLS);
	fflush(console);
#endif

	/*
	 * Use new window size to recreate curwin/stdwin. Following code
	 * copied from initscr() curses routine.
	 */

        if (curscr != NULL) {
                delwin(curscr);
        }
        if ((curscr = newwin(LINES, COLS, 0, 0)) == ERR)
                fprintf(stderr, "Couldn't recreate curscr\n");
        clearok(curscr, TRUE);
#ifdef BSD
#define _FULLLINE 020
        curscr->_flags &= ~_FULLLINE;
#endif
        if (stdscr != NULL) {
                delwin(stdscr);
        }
        stdscr = newwin(LINES, COLS, 0, 0);
	/* === End of borrowed code === */
    } else perror("Couldn't open /dev/tty to get new size");
    clear();
    dispinit(); /* recompute window layout */
    backpage(); /* redisplay same page */
    seekline(nextline);
    display();  /* recompute ref displays */
    setfield(); /* recompute field offsets */
    atfield();	/* reset the cursor position */
    refresh();  /* redraw the window */
    mousereinit();
}

#ifdef BLACKBIRD
/*  
 * trap SIGTSTP on job control terminal stop
 * and undo curses setup.
 * then send SIGTSTP to self to go into proper
 * terminal stop state.
 */
onstop()
{
	if (incurses == YES) {
#if BSD || UNIXPC
		clear();
		refresh();
#endif
		endwin();
		mousecleanup();
	}
	kill(getpid(), SIGTSTP);
	
}

onstart()
{
	signal(SIGTSTP, onstop);
	signal(SIGCONT, onstart);
	longjmp(env,1);
}
#endif
