#ifndef LINT
#ifdef RCS_ID
static char *rcsid = "@(#)$Header: /u5/alt/tools/src/cscope/RCS/mouse.c,v 1.2 1991/10/29 18:06:33 marvinw Exp $";
#endif /* RCS_ID */
#endif /* LINT */
/*	Copyright (c) 1984, 1985, 1986, 1987 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*	cscope - interactive C symbol cross-reference
 *
 *	mouse functions
 */

#include "global.h"

BOOL	mouse = NO;			/* mouse interface */

typedef	struct {			/* menu */
	char	*text;
	char	value;
} MENU;

static	MENU	mainmenu[] = {		/* main menu */
	"Next",		' ',
	"Previous",	'-',
	"Repeat",	'.',
	"Edit All",	ctrl('E'),
	"Append",	'>',
	"Rebuild",	ctrl('R'),
	"Shell",	'!',
	"Redraw",	ctrl('L'),
	"Help",		'?',
	"Exit",		ctrl('D'),
	NULL,		'\0'
};

static	MENU	changemenu[] = {	/* change mode menu */
	"Next",		' ',
	"Previous",	'-',
	"Mark Screen",	'*',
	"Mark All",	'a',
	"Change",	ctrl('D'),
	"No Change",	ESC,
	"Shell",	'!',
	"Redraw",	ctrl('L'),
	"Help",		'?',
	NULL,		'\0'
};

static	MENU	*loaded;		/* menu loaded */
static	BOOL	emacsviterm = NO;	/* terminal type */

void	loadmenu();

/* see if there is a mouse interface */

void
mouseinit()
{
	char	*term;
	char	*mygetenv();

	/* see if this is emacsterm or viterm */
	term = mygetenv("TERM", "");
	if (strcmp(term, "emacsterm") == 0 || 
	    strcmp(term, "viterm") == 0) {
		emacsviterm = YES;
		mouse = YES;
	}
	/* the MOUSE enviroment variable is for 5620 terminal programs that have
	   mouse support but the TERM environment variable is the same as a
	   terminal without a mouse, such as myx */
	else if (strcmp(mygetenv("MOUSE", ""), "myx") == 0) {
		mouse = YES;
	}
	if (mouse == YES) {
		loadmenu(mainmenu);
	}
}

/* load the correct mouse menu */

void
mousemenu()
{
	if (mouse == YES) {
		if (changing == YES) {
			loadmenu(changemenu);
		}
		else {
			loadmenu(mainmenu);
		}
	}
}

/* download a menu */

static	void
loadmenu(menu)
MENU	*menu;
{
	register int	i;

	if (emacsviterm == YES) {
		mousereinit();
#if SCROLLBAR
		printf("\033V1");	/* display the scrollbar */
#endif
		printf("\033M0@%s@%c@", menu[0].text, menu[0].value);
		for (i = 1; menu[i].text != NULL; ++i) {
			printf("\033M@%s@%c@", menu[i].text, menu[i].value);

			/* put Send menu item after Repeat menu item */
			if (menu[i].value == '.') {
				printf("\033M@Send@##\033s##\r@");
			}
		}
	}
	else {	/* myx */
		int	len;
		
		mousecleanup();
		printf("\033[6;1X\033[9;1X");
		for (i = 0; menu[i].text != NULL; ++i) {
			len = strlen(menu[i].text);
			printf("\033[%d;%dx%s%c", len, len + 1, 
				menu[i].text, menu[i].value);
		}
		loaded = menu;
	}
	fflush(stdout);
}

/* reinitialize the mouse in case curses changed the attributes */

void
mousereinit()
{
	if (emacsviterm == YES) {

		/* enable the mouse click and sweep coordinate control sequence */
		/* and switch to menu 2 */
		printf("\033{2\033#2");
		fflush(stdout);
	}
}

/* restore the mouse attributes */

void
mousecleanup()
{
	register int	i;

	if (loaded != NULL) {	/* only true for myx */
		
		/* remove the mouse menu */
		printf("\033[6;0X\033[9;0X");
		for (i = 0; loaded[i].text != NULL; ++i) {
			printf("\033[0;0x");
		}
		loaded = NULL;
	}
}

/* draw the scrollbar */

void
draw_scrollbar(top, bot)
int top, bot;
{
	register p1, p2;

	if (emacsviterm == YES) {
		if (bot > top) {
			p1 = 16 + (top - 1) * 100 / totallines;
			p2 = 16 + (bot - 1) * 100 / totallines;
			if (p2 > 116) {
				p2 = 116;
			}
			if (p1 < 16) {
				p1 = 16;
			}
			/* don't send ^S or ^Q because it will hang a layer using cu(1) */
			if (p1 == ctrl('Q') || p1 == ctrl('S')) {
				++p1;
			}
			if (p2 == ctrl('Q') || p2 == ctrl('S')) {
				++p2;
			}
		}
		else {
			p1 = p2 = 16;
		}
		printf("\033W%c%c", p1, p2);
	}
}

/* get the mouse information */

MOUSE *
getmouse()
{
	static	MOUSE	m;

	if (mouse == NO) {
		return(NULL);
	}
	switch (mygetch()) {
	case ctrl('_'):		/* click */
		if ((m.button = mygetch()) == '0') {	/* if scrollbar */
			m.percent = getpercent();
		}
		else {
			m.x1 = getcoordinate();
			m.y1 = getcoordinate();
			m.x2 = m.y2 = -1;
		}
		break;

	case ctrl(']'):		/* sweep */
		m.button = mygetch();
		m.x1 = getcoordinate();
		m.y1 = getcoordinate();
		m.x2 = getcoordinate();
		m.y2 = getcoordinate();
		break;
	default:
		return(NULL);
	}
	return(&m);
}

/* get a row or column coordinate from a mouse button click or sweep */

getcoordinate()
{
	register int  c, next;

	c = mygetch();
	next = 0;
	if (c == ctrl('A')) {
		next = 96;
		c = mygetch();
	}
	if (c < ' ') {
		return (0);
	}
	return (next + c - ' ');
}

/* get a percentage */

getpercent()
{
	register int c;

	c = mygetch();
	if (c < 16) {
		return(0);
	}
	if (c > 120) {
		return(100);
	}
	return(c - 16);
}
