%{
#ifndef LINT
#ifdef RCS_ID
static char *rcsid = "@(#)$Header: /u5/alt/tools/src/cscope/RCS/scanner.l,v 1.5 1991/10/29 18:06:43 marvinw Exp $";
#endif /* RCS_ID */
#endif /* LINT */
%}
%{
/*	Copyright (c) 1984, 1985, 1986, 1987 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*	cscope - interactive C symbol cross-reference
 *
 *	C symbol scanner
 */
#include <ctype.h>	/* isprint */
#include "global.h"

/* the line counting has been moved from character reading for speed */
#undef	input
#define input()  ((yytchar=yysptr>yysbuf?*--yysptr:getc(yyin))==EOF?0:yytchar)
#undef	unput
#define unput(c) (*yysptr++=yytchar=(c))

#define	MAXIF	10	/* maximum #if nesting */

int	first;	/* buffer index for first char of symbol */
int	last;	/* buffer index for last char of symbol */

static	int	braces;			/* unmatched left brace count */
static	int	iflevel;		/* #if nesting level */
static	int	maxifbraces[MAXIF];	/* maximum brace count within #if */
static	int	preifbraces[MAXIF];	/* brace count before #if */
static	int	token;			/* token found */
static	BOOL	bplisting;		/* breakpoint listing */
static	BOOL	lex;			/* lex file */
static	BOOL	elseelif;		/* #else or #elif found */
static	BOOL	psuedoelif;		/* psuedo-#elif */
static	BOOL	ppdefine;		/* preprocessor define statement */
static	BOOL	rules;			/* lex/yacc rules */
static 	BOOL	struct_seen;		/* saw "struct/class/union" keyword */
static	BOOL	sdl;			/* sdl file */

/* context maintains the current scanner context - outside of any 
 * struct/union/class/body; inside a struct/union/class; or inside a function
 * definition.  The previous context is restored on exit from the current 
 * context. A new context is pushed at each "{" encountered and popped at
 * each "}".
 */

#define MAX_CONTEXT 30
enum	context_t	{global_context, /* outside any definition 	*/
		   	struct_context,  /* inside struct definition 	*/
			body_context};   /* inside function definition 	*/
static	enum context_t next_context = global_context;
static  enum context_t context[MAX_CONTEXT] = {global_context};

#ifdef	DEBUG
static	char *context_name();
#endif
void	scanfor();
%}
identifier	[a-zA-Z_][a-zA-Z_0-9]*
number		"."?[0-9][.0-9a-fA-FlLxX]*
space		[ \t]
%start SDL
%%
"%%"		{	/* lex/yacc rules delimiter */
			braces = 0;
			if (rules == NO) {
				rules = YES;
			}
			else {
				rules = NO;
			}
		}
<SDL>STATE{space}+({identifier}|\*)	{ /* sdl state, treat as function def */
			braces = 1;
			context[braces] = body_context;
			token = FCNDEF;
			goto findident;
		}
<SDL>ENDSTATE{space}	{ /* end of an sdl state, treat as end of a function */
			goto endfcn;
		}
\{		{	/* count unmatched left braces for fcn def detection */
			if (struct_seen) {
			    context[++braces] = struct_context;
			    struct_seen = NO;
			} else {
			    context[++braces] = next_context;
			}
#ifdef DEBUG
printf("In lbrack, braces = %d, old_context = %s, new_context = %s\n", 
braces, context_name(braces-1), context_name(braces));
#endif
			goto more;
		}
\#{space}*endif/.*\n[ \t\n]*#{space}*if	{
			/* attempt to correct erroneous brace count caused by:
			 * 
			 * #if ...
			 * 	... {
			 * #endif
			 * #if ...
			 * 	... {
			 * #endif
			 */
			/* the current #if must not have an #else or #elif */
			if (elseelif == YES) {
				goto endif;
			}
			psuedoelif = YES;
			goto more;
		}
\#{space}*ifn?(def)?	{ /* #if, #ifdef or #ifndef */
			elseelif = NO;
			if (psuedoelif == YES) {
				psuedoelif = NO;
				goto elif;
			}
			if (iflevel < MAXIF) {	/* push the current brace count */
				preifbraces[iflevel] = braces;
				maxifbraces[iflevel++] = 0;
			}
			else {
				warning("#if stack overflow");
			}
			goto more;
		}
\#{space}*el(se|if)	{ /* #elif or #else */
			elseelif = YES;
		elif:
			/* save the maximum brace count for this #if */
			if (braces > maxifbraces[iflevel]) {
				maxifbraces[iflevel - 1] = braces;
			}
			/* restore the brace count to before the #if */
			braces = preifbraces[iflevel - 1];
			goto more;
		}
\#{space}*endif	{	/* #endif */
		endif:
			if (iflevel > 0) {
				
				/* get the max brace count for this #if */
				if (braces < maxifbraces[--iflevel]) {
					braces = maxifbraces[iflevel];
				}
			}
			else {
				warning("too many #endif statements");
			}
			goto more;
		}
\}		{	/* check for the end of a function */
			/* or a struct/union definition in its args */
			if (--braces <= 0) {
		endfcn:
				braces = 0;
			}
			next_context = context[braces]; /* restore after pop */
#ifdef DEBUG
printf("In endfcn, braces = %d, old_context = %s, new_context = %s\n", 
braces, context_name(braces+1), context_name(braces));
#endif
			goto more;
		}
\)		{	/* see if the struct/union was not a definition   */
			/* ")" may not intervene between "struct" and "{" */
			struct_seen = NO;
			goto more;
		}
;		{	/* see if the struct/union was not a definition   */
			/* ";" may not intervene between "struct" and "{" */
			struct_seen = NO;
			goto more;
		}
\#{space}*define{space}+{identifier}	{
				
			/* preprocessor macro or constant definition */
			ppdefine = YES;
			token = DEFINE;
			if (compress == YES) {
				yytext[0] = '\0';	/* compress the keyword */
			}
		findident:
			first = yyleng - 1;
			while (yytext[first] != ' ' && yytext[first] != '\t') {
				--first;
			}
			++first;
			goto fcn;
		}
operator[ \t]*[^( \t]+/[ \t]*\([^"|)]*\)[ \t\n()]*[{/a-zA-Z_] 	{ 
#ifdef DEBUG
printf("In operator, yytext = '%s'\n", yytext);
#endif
			first = yyleng - 1;
			/* parse backwards:  
			 * "operator" [whitespace] <string> [whitespace]
			 */
			while (yytext[first] == ' ' || yytext[first] == '\t')
				--first;
			while (yytext[first] != ' ' && yytext[first] != '\t' && first > 0)
				--first;
			if (first > 0) /* stopped on blank or tab */
				++first;
			if (strncmp(&yytext[first], "operator", 8) == 0) {
				first += 8; /* skip over "operator */
			}
			goto fcndef;
		}
{identifier}/[ \t]*\([^"|)]*\)[ \t\n()]*[{/a-zA-Z_]	{
			
			/* warning: "if (...)" must not overflow yytext, so a
			   function definition is not allowed to contain \n */
			
			/* look for a function definition */
			/* note: "#define a (b)" is not one */
		fcndef:
			if ((context[braces] != body_context) && ppdefine == NO && rules == NO) {
				next_context = body_context;
				token = FCNDEF;
				goto fcn;
			}
			goto fcncal;
		}
{identifier}/[ \t]*\(	{ /* look for a function call */
		fcncal:	if (context[braces] == body_context || ppdefine == YES || rules == YES) {
				token = FCNCALL;
				goto fcn;
			}
			goto ident;
		}
{identifier}	{
		ident:	token = IDENT;
		
			/* check for a long line */
		fcn:	if (yyleng > LINEMAX) {
				int	c;
				
				/* skip to the end of the line */
				warning("line too long");
				while ((c = input()) != LEX_EOF) {
					if (c == '\n') {
						unput(c);
						break;
					}
				}
			}
			/* check for a keyword */
			yymore();
			if (lookup(yytext + first) == KEYWORD) {
				/* check for struct/union in the fcn args */
				if (braces >= 0) {
					char	*s;
					if (compress == YES) {
						s = keyword[*(yytext + first)].text;
					}
					else {
						s = yytext + first;
					}
					if (strequal(s, "struct") ||
					    strequal(s, "union")  ||
					    strequal(s, "class")) {
						struct_seen = YES;
					}
				}
				first = yyleng;
			}
			else {	/* identifier */
				last = yyleng;
				return(token);
			}
		}
\\\n		{	/* preprocessor statement is continued on next line */
			goto eol;
		}
\n		{	/* end of the line */
			ppdefine = NO;
			
			/* skip the first 8 characters of a breakpoint listing line */
			if (bplisting == YES) {
				int	i;

				for (i = 0; i < 8; ++i) {
					if (input() == '\n') {
						unput('\n');
						break;
					}
				}
			}
		eol:	++yylineno;
			first = 0;
			last = 0;
			if (symbols > 0) {
				return(NEWLINE);
			}
		}
\'		{	/* character constant */
			if (sdl == NO) {
				scanfor('\'');
			}
			goto more;
		}
\"		{	/* string constant */
			scanfor('"');
			goto more;
		}
"/*"		{	/* delete comments to prevent line buffer overflow */
			register char	c, lastc;

			yyleng -= 2;	/* delete the leading comment delimiter */
		
			/* find the trailing comment delimiter */
			lastc = '\0';
			while ((c = input()) != LEX_EOF) {
				if (c == '/' && lastc == '*') { /* fewer '/'s */
					break;
				}
				else if (c == '\n') {
					++yylineno;
				}
				lastc = c;
			}
			goto more;	/* this comment may be in a statement */
		}
"//"		{	/* C++ comment */
			register int	c;

			yyleng -= 2;	/* delete the leading comment delimiter */

			/* delete up to the newline */
			while ((c = input()) != LEX_EOF) {
				if (c == '\n') {
					unput(c);
					break;
				}
			}
			goto more;	/* this comment may be in a statement */
		}
^[ \t\f]+	{	/* don't save leading white space */
		}
\#{space}*include{space}*["<][^"> \t\n]+	{ /* #include file */
			char	*s;
			
			s = strpbrk(yytext, "\"<");
			incfile(s + 1, *s);
			first = s - yytext;
			last = yyleng;
			if (compress == YES) {
				yytext[0] = '\1';	/* compress the keyword */
			}
			yymore();
			return(INCLUDE);
		}
\#{space}*{identifier}	|	/* preprocessor keyword */
{number}	|	/* number */
.		{	/* punctuation and operators */
		more:	first = yyleng;
			yymore();
		}
%%

/* don't carry-over scanner problems to new file */
void
resetscanner()
{
	first = 0;		/* buffer index for first char of symbol */
	last = 0;		/* buffer index for last char of symbol */
	braces = 0;		/* unmatched left brace count */
	ppdefine = NO;		/* preprocessor define statement */
	rules = NO;		/* lex/yacc rules */
	next_context = global_context;	/* global context */
	bplisting = lex = sdl = NO;	/* file types */
	BEGIN 0;
}
void
beginbp()
{
	bplisting = YES;
}

void
beginlex()
{
	lex = YES;
}

void
beginsdl()
{
	sdl = YES;
	BEGIN SDL;
}

#ifdef DEBUG
static char *
context_name(i)
	int i;
{
	switch (context[i]) {
	    case global_context: return "global";
	    case struct_context: return "struct";
	    case body_context: return "body";
	    default: return "invalid context stack entry";
	}
}
#endif

static	void
scanfor(terminator)
register char	terminator;
{
	register char	c;

	/* scan until the terminator is found */
	while ((c = yytext[yyleng++] = input()) != terminator) {
		switch (c) {
		case '\\':	/* escape character */
			if ((yytext[yyleng++] = input()) == '\n') {
				++yylineno;
			}
			break;
		case '\t':	/* tab character */
		
			/* if not a lex program, continue */
			if (lex == NO) {
				break;
			}
			/* fall through */
			
		case '\n':	/* illegal character */
			
			/* assume the terminator is missing, so put
			   this character back */
			unput(c);
			yytext[--yyleng] = '\0';
			/* fall through */
			
		case LEX_EOF:	/* end of file */
			return;
			
		default:
			/* change a control character to a blank */
			if (!isprint(c)) {
				yytext[yyleng - 1] = ' ';
			}
		}
		/* see if this token will overflow the line buffer */
		/* note: '\\' may cause yyleng to be > LINEMAX */
		if (yyleng >= LINEMAX) {
			
			/* truncate the token */
			while ((c = input()) != LEX_EOF) {
				if (c == terminator) {
					unput(c);
					break;
				}
				else if (c == '\n') {
					++yylineno;
				}
			}
		}
	}
	yytext[yyleng] = '\0';
}

/* dummy routine to avoid need for lex library on non-UNIX systems */

static
yywrap()
{
	return(1);
}
