/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	<file name> - <one-line description of file contents>
 *
 * DESCRIPTION
 *	<overall description of what the c code does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/filterclick.c,v 1.1 1993/09/15 21:14:39 toddb Exp $
 */
#include "nvrTypes.h"
#include <stdio.h>
#include <ulaw2linear.h>

#define ClickTheshhold	16000

char *ProgramName;

void
FilterChange(FILE *input, FILE *output)
{
    int c, clast;
    int ulawLast, ulaw;
    int diff;
    long offset;
    boolean firstMessage = True;

    clast = getc(input);
    if (clast == EOF)
	exit(0);
    putc(clast, output);
    ulawLast = audio_u2s(clast);

    offset = 1;
    while ((c = getc(input)) != EOF) {
	ulaw = audio_u2s(c);
	diff = ulaw - ulawLast;
	if (diff > ClickTheshhold || diff < -ClickTheshhold) {
	    if (firstMessage) {
		Msg("Filtered samples:\n");
		firstMessage = False;
	    }
	    Msg("*%8d (diff=%d) @ byte %d\n", ulaw, diff, offset);
	    c = clast;
	} else {
	    /* Msg(" %8d (diff=%d) @ byte %d\n", ulaw, diff, offset); */
	}
	ulawLast = ulaw;
	clast = c;
	putc(c, output);

	offset++;
    }
}

void
FilterAvgChange(FILE *input, FILE *output)
{
#   define numComparDeltas	8		/* must be power of 2 */
#   define log2NumComparDeltas	3		/* log2(numComparDeltas) */
#   define filterWidth		(numComparDeltas+2+1)	/* makes it odd */
#   define filterCenter		(filterWidth>>1)
#   define NextIndex(i)		((i+1 == filterWidth) ? 0 : i+1)
#   define PrevIndex(i)		((i == 0) ? filterWidth-1 : i-1)
#   define NewToCenterIndex(i)	(i-filterCenter < 0 \
				 ? i+filterCenter   \
				 : i-filterCenter)
#   define CalcAverageDelta(sum) ((sum)>>log2NumComparDeltas)

    UShort deltaSamples[ filterWidth ];
    UShort samples[ filterWidth ];
    ULong sum;
    int pcm, pcmLast, pcmPrev, i;
    int ulaw;
    int diff, averageDiff;
    long offset;
    int newIndex, centerIndex;
    boolean firstMessage = True;

    ulaw = getc(input);
    if (ulaw == EOF)
	return;
    samples[ 0 ] = ulaw;
    pcmLast = audio_u2s(ulaw);

    offset = 1;
    for (newIndex = 1; newIndex < filterWidth-1; newIndex++) {
	if ((ulaw = getc(input)) == EOF)
	    break;
	pcm = audio_u2s(ulaw);
	diff = pcm - pcmLast;
	if (diff < 0)
	    diff = -diff;
	deltaSamples[ newIndex ] = diff;
	samples[ newIndex ] = ulaw;
	offset++;
    }

    /*
     * output everything up to the center of the filter
     */
    for (i = 0; i < (filterWidth>>1); i++)
	putc(samples[i], output);

    /* XXX if input is smaller than filterWidth, nothing comes out */

    while ((ulaw = getc(input)) != EOF) {
	/*
	 * First, record this sample
	 */
	pcm = audio_u2s(ulaw);
	diff = pcm - pcmLast;
	if (diff < 0)
	    diff = -diff;
	deltaSamples[ newIndex ] = diff;
	samples[ newIndex ] = ulaw;
	pcmLast = pcm;

	/*
	 * Now go back to the center of the filter, and do the
	 * adaptive filter.
	 */
	centerIndex = NewToCenterIndex(newIndex);
	sum = 0;
	for (i = NextIndex(centerIndex);
	     i != PrevIndex(centerIndex);
	     i = NextIndex(i))
	{
	    sum += deltaSamples[i];
	}
	diff = deltaSamples[ centerIndex ];
	ulaw = samples[ centerIndex ];
	averageDiff = CalcAverageDelta(sum);
	if (diff - averageDiff > ClickTheshhold
	 || diff - averageDiff < -ClickTheshhold) {
	    if (firstMessage) {
		Msg("Filtered samples:\n");
		firstMessage = False;
	    }
	    Msg("*%8d (diff=%d) @ byte %d\n",
		audio_u2s(ulaw), diff, offset-filterCenter);
	    i = NextIndex(newIndex);
	    while (i != newIndex) {
		Msg("%c%d%c",
		    i == centerIndex ? '(' : ' ',
		    audio_u2s(samples[ i ]),
		    i == centerIndex ? ')' : ' ');
		i = NextIndex(i);
	    }
	    Msg("\n");
	    pcmPrev = audio_u2s(samples[ PrevIndex(centerIndex) ]);
	    pcm = (pcmPrev + audio_u2s(samples[ NextIndex(centerIndex) ])) >> 1;
	    ulaw = audio_s2u(pcm);
	    diff = pcm - pcmPrev;
	    if (diff < 0)
		diff = -diff;
	    deltaSamples[ centerIndex ] = diff;
	} else {
	    Msg(" %8d (diff=%d) @ byte %d\n",
		audio_u2s(ulaw), diff, offset-filterCenter);
	}

	putc(ulaw, output);
	offset++;
	newIndex = NextIndex(newIndex);
    }
}

main(int argc, char **argv)
{
    FILE *input, *output;

    input = stdin;
    output = stdout;
    ProgramName = *argv;

    /* FilterChange(input, output); */
    FilterAvgChange(input, output);
    exit (0);
}
