/* $Header: /usr/local/src/nvrToMIT/RCS/main.c,v 3.13 1992/07/30 22:37:56 toddb Exp $ */
#include "struct.h"

typedef struct _programmap {
    ProgramId	id;
    char	*string;
} ProgramMap;

static ProgramMap Programs[] = {
    ProgMklinks,	"mklinks",
    ProgHoldRcsDir,	"holdrcsdir",
    ProgRcs,		"rcs",
    ProgRcsdiff,	"rcsdiff",
    ProgRlog,		"rlog",
    ProgCo,		"co",
    ProgCi,		"ci",
    ProgRcsmerge,	"rcsmerge",
    ProgUnknown,	0,
};
    
/*
 * Global variables.
 */
Orchard Farm;
boolean CreateCommands;
boolean ListFiles;
boolean VerboseLevel;
boolean AskToExecute;
boolean ForceCheck;
boolean OneLevel;
boolean ShowTreeChoice;
char    *ProgramName;
char    *RcsBin;
char    *MklinksTreeName = "MklinksTreeName";
char    *ProgramPrefix = "Mklinks subprocess -- ";
Collection CommandSet;
RcsRevision DefaultRcsRevision = { DefaultRevision };
void (*RcsInitFunc)();
void (*RcsRewriteFunc)();


static void MklinksUsage()
{
    Msg("Usage: %s [-A1cfltV] [-TtreeName] [treeName]\n", ProgramName);
    Msg("Where\n");
    Msg(" -A       means ``ask'' before executing a command\n");
    Msg(" -1       means to only do one directory level (don't recurse)\n");
    Msg(" -c       means to create a set of shell commands that would\n");
    Msg("            accomplish the update; produce them on standard out.\n");
    Msg("            no action is taken\n");
    Msg(" -f       means to force a check on the rcsid header info in the\n");
    Msg("            file even if the file modification times look right.\n");
    Msg(" -l       means to list the files that would be updated;\n");
    Msg("            no action is taken\n");
    Msg(" -t       means to show the derived tree of choice, then exit.\n");
    Msg(" -Tname   means to select a specific tree named 'name'. This option\n");
    Msg("            is redundant: it is available only for compatability.\n");
    Msg(" -V       means ``verbose''.   More -V gets more verbose.\n\n");
    Msg("If ``treeName'' is missing, an appropriate tree is selected\n");
    Msg("assuming the current directory\n");
    exit (1);
}

static void GenericUsage()
{
    Msg("Usage: %s [-V] [-A] [-TtreeName] [normal %s arguments]",
	ProgramName, ProgramName);
    Msg("Where\n");
    Msg(" -A       means ``ask'' before executing a command\n");
    Msg(" -Tname  means to select a specific tree named 'name'.\n");
    Msg(" -V       means ``verbose''.   More -V gets more verbose.\n");

    exit (1);
}

static void HoldRcsDirUsage()
{
    Msg("Usage: %s [-V] [-A] [-TtreeName] [file]", ProgramName);
    Msg("Where\n");
    Msg(" -A       means ``ask'' before executing a command\n");
    Msg(" -Tname  means to select a specific tree named 'name'.\n");
    Msg(" -V       means ``verbose''.   More -V gets more verbose.\n");

    exit (1);
}

static int GetTreeIndex(treeName)
    char *treeName;
{
    static int treeIndex = -1;
    int treeToken, i;
    boolean nameFromEnv = FALSE;

    if (treeIndex >= 0)
	Fatal("tree name specified twice\n");

    if (treeName == NULL) {
	treeName = getenv(MklinksTreeName);
	if (treeName == NULL)
	    return(treeIndex);
	nameFromEnv = TRUE;
    }

    if ((treeToken = StringToTokenIndex(treeName)) < 0) {
	if (nameFromEnv)
	    Msg("Environment variable %s=", MklinksTreeName);
	Msg("\"%s\" must be a token name in %s\n", treeName, CONFIGFILE);
    }

    if (treeToken >= 0) {
	for (i=0; i < Farm.nTrees; i++)
	    if (Farm.treeList[i].label == treeToken)
		return (treeIndex = i);
	if (nameFromEnv)
	    Msg("Environment variable %s=", MklinksTreeName);
	Msg("\"%s\" is not a tree name in %s\n", treeName, CONFIGFILE);
    }
	    
    return (treeIndex);
}

static void ParseCommonFlag(arg, pTreeIndex, usage)
    char *arg;
    int *pTreeIndex;
    void (*usage)();
{
    switch(*arg) {
    case 'A':
	AskToExecute++;
	break;

    case 'T':
	if (*pTreeIndex >= 0)
	    Msg("Tree name redefined\n");
	if (arg[1] == '\0' || (*pTreeIndex = GetTreeIndex(arg+1)) < 0)
	    (*usage)();
	break;

    case 'V':
	VerboseLevel++;
	break;
    }
}

/*
 * Rewrite a command line for use with ci.
 */
/*ARGSUSED*/
static void CiRewriteRcs(id, original, new, revision)
    ProgramId id;
    Argvec *original;
    Argvec *new;
    RcsRevision *revision;
{
    int i;
    char *arg;

    new->path = original->path;
    for (i = 0; i < original->index; i++) {
	arg = original->argv[ i ];
	AddArgv(new, arg, FALSE);
    }

    /*
     * force the check in to have the indicated revision.
     */
    switch (revision->type) {
    case NamedLabel:
	AddFmtArgv(new, "-N%s", revision->u.label);
	break;
    case VersionNumber:
	AddFmtArgv(new, "-r%s", revision->u.version);
	break;
    }
}

/*
 * Rewrite a command line for use with co.
 */
static void CoRewriteRcs(id, original, new, revision)
    ProgramId id;
    Argvec *original;
    Argvec *new;
    RcsRevision *revision;
{
    int i;
    char *arg;
    boolean revisionAdded = FALSE;

    new->path = original->path;
    for (i = 0; i < original->index; i++) {
	arg = original->argv[ i ];
	if (revision->type != DefaultRevision && arg[0] == '-') {
	    switch(arg[1]) {
	    case 'p':
	    case 'f':
	    case 'l':
	    case 'q':
	    case 'r':
	    case 'u':
		AddFmtArgv(new, "-%c%s", arg[1], revision->u.label);
		revisionAdded = TRUE;
		continue;
	    }
	}
	AddArgv(new, arg, FALSE);
    }

    if (id == ProgCo && !revisionAdded && revision->type != DefaultRevision)
	AddFmtArgv(new, "-r%s", revision->u.label);
}

/*
 * Rewrite a command line for use with the rcs command "rcs".
 */
/*ARGSUSED*/
static void RcsRewriteRcs(id, original, new, revision)
    ProgramId id;
    Argvec *original;
    Argvec *new;
    RcsRevision *revision;
{
    int i;
    char *arg;

    new->path = original->path;
    for (i = 0; i < original->index; i++) {
	arg = original->argv[ i ];
	if (revision->type != DefaultRevision && arg[0] == '-') {
	    switch(arg[1]) {
	    case 'b':
	    case 'l':
	    case 'u':
		AddFmtArgv(new, "-%c%s", arg[1], revision->u.label);
		continue;
	    }
	}
	AddArgv(new, arg, FALSE);
    }
}

/*ARGSUSED*/
static void RlogRewriteRcs(id, original, new, revision)
    ProgramId id;
    Argvec *original;
    Argvec *new;
    RcsRevision *revision;
{
    int i;

    new->path = original->path;
    for (i = 0; i < original->index; i++)
        AddArgv(new, original->argv[ i ], FALSE);
}

/*ARGSUSED*/
static void RewriteRcsWithRevision(id, original, new, revision)
    ProgramId id;
    Argvec *original;
    Argvec *new;
    RcsRevision *revision;
{
    int i;

    new->path = original->path;
    for (i = 0; i < original->index; i++)
        AddArgv(new, original->argv[ i ], FALSE);
    if (revision->type != DefaultRevision)
        AddFmtArgv(new, "-r%s", revision->u.label);
}

/*
 * This routine initializes itself for use with various rcs commands.
 */
static void GenericInitRcs(argv, argc)
    char **argv;
    int argc;
{
    int i;
    RcsSource *src;

    if (argv == NULL) {	/* reinitialize */
	src = CommandSet.unique;
	for (i = 0; i < CommandSet.nUnique; i++, src++) {
	    src->fromDir = NULL;
	    src->rev = NULL;
	    FreeArgv(&src->cmd);
	}
	CommandSet.nUnique = 0;
    } else {	/* initialize */
	char arg0[ BUFSIZ ];

	sprintf(arg0, "%s%s", ProgramPrefix, ProgramName);
	AddArgv(&CommandSet.command, arg0, TRUE);
	for (argv++, argc--; argc; argv++, argc--) {
	    if (argv[0][0] != '-')
		break;
	    switch (argv[0][1]) {
	    case 'A':
	    case 'T':
	    case 'V':
		continue;
	    }
	    AddArgv(&CommandSet.command, *argv, FALSE);
	}
	RcsInitFunc = GenericInitRcs;
	switch (CommandSet.id) {
	case ProgCo: RcsRewriteFunc = CoRewriteRcs; break;
	case ProgCi: RcsRewriteFunc = CiRewriteRcs; break;
	case ProgRcs: RcsRewriteFunc = RcsRewriteRcs; break;
	case ProgRlog: RcsRewriteFunc = RlogRewriteRcs; break;
	case ProgRcsdiff: RcsRewriteFunc = RewriteRcsWithRevision; break;
	case ProgRcsmerge: RcsRewriteFunc = RewriteRcsWithRevision; break;
	}
    }
}

/*
 * This routine initializes itself for use with mklinks.
 */
/*ARGSUSED*/
static void MklinksInitRcs(argv, argc)
    char **argv;
    int argc;
{
    if (argv == NULL) {	/* reinitialize */
	GenericInitRcs(argv, argc);
    } else {	/* initialize */
	AddArgv(&CommandSet.command, "co", FALSE);
	AddArgv(&CommandSet.command, "-q", FALSE);
	RcsInitFunc = MklinksInitRcs;
	RcsRewriteFunc = RewriteRcsWithRevision;
    }
}

static void MklinksMain(cwd, argc, argv)
    char   *cwd;
    int    argc;
    char    **argv;
{
    int    i, treeIndex = -1;
    boolean treeNameFromEnvironment = FALSE;
    SourceState state;

    for (argv++, argc--; argc; argv++, argc--) {
        if (argv[0][0] == '-') {
            for (i=1; argv[0][i]; i++)
                switch(argv[0][i]) {
                /* Only decend one level in directory tree */
                case '1':
                    OneLevel++;
                    break;

                /* produce command lines instead of doing work */
                case 'c':
                    CreateCommands++;
                    break;

                /* Don't settle for just the dates on the */
                /* files being newer than the rcs files... */
                /* force a check of the revision and date */
                case 'f':
                    ForceCheck++;
                    break;

                /* produce a list of out-of-date files instead of doing work */
                case 'l':
                    ListFiles++;
                    break;

                /* print out the chosen tree */
                case 't':
                    ShowTreeChoice++;
                    break;

		case 'A':
		case 'T':
		case 'V':
		    ParseCommonFlag(*argv + i, &treeIndex, MklinksUsage);
		    i = strlen(*argv) - 1;
		    break;

                default:
		    Msg("Unknown flag %s\n", *argv);
                    MklinksUsage();
                    break;
                }
        } else {
	    if ((treeIndex = GetTreeIndex(*argv)) < 0)
		MklinksUsage();
        }
    }

    if (ListFiles && CreateCommands) {
	Msg("Only one of -l and -c allowed\n");
	MklinksUsage();
    }
    if (! ListFiles && ! CreateCommands && VerboseLevel == 0)
        VerboseLevel++;

    if (treeIndex < 0) {
	treeIndex = GetTreeIndex(NULL);
	treeNameFromEnvironment = TRUE;
    }

    /*
     * If a tree was specified, and it has a root specified, and we are
     * not in the that subdirectory, we may want to go to it.
     */
    if (treeIndex >= 0 && Farm.treeList[ treeIndex ].toRoot.path) {
	DirTree *tree = &Farm.treeList[ treeIndex ];
	GetRealRoot(tree, cwd);
	if (! PathsIntersect(tree->toRoot.realPath, cwd, strlen(cwd))) {
	    if (! Confirm("cd %s? ", tree->toRoot.path))
		exit (0);
	}
    }

    /*
     * Determine where in the source tree we start fetching from.
     */
    state = IdentifyInitialSource(treeIndex, cwd);
    switch (state) {
    case ImplicitRootDir:
	/* 
         * At this point, we have what we think to be a starting point.
         * If the label for the tree was
	 * explicitly requested on the command line, then just do it,
	 * otherwise ask for confirmation.
         */
	if (treeIndex != -1 && !treeNameFromEnvironment)
	    break;
	Msg("%s appears to be the\nroot directory for \"%s\".\n",
	    cwd,
	    TokenIndexToString(Farm.treeList[ Farm.choice ].label));
	Msg("(To avoid this confirmation, specify the tree name with -T,\n%s\n",
	    "or specify a TreeRoot for the tree.)\n");
	if (! Confirm("Do you want to continue?"))
	    exit(0);
	break;

    case AmbiguousTopDir:
    case AmbiguousSubDir:
	AmbiguousPath(state, cwd);
	exit (1);
	/*NOTREACHED*/

    case NoSourceFound:
	Fatal("No usable mappings appear in %s\n", CONFIGFILE);
	/*NOTREACHED*/

    case UniqueSourceFound:
	break;
    }

    if (VerboseLevel > 3)
	ShowTrees(FALSE);
    else if (ShowTreeChoice) {
	ShowTrees(TRUE);
	exit(0);
    }

    /*
     * We are in 'cwd' (of course), but the routines, beginning with LinkTree
     * assume that the working directory is the one above the one
     * passed as an argument.
     */
    MklinksInitRcs(argv, argc);
    if (ChangeNode(Farm.start, TRUE) && ChangeNode(Farm.start->parent, TRUE))
	LinkTree(Farm.start, Farm.start->data->from);
}

typedef struct _digestedpath {
    boolean	absolute;	/* true if the path is absolute */
    boolean	isRcsFile;	/* true if the path ends in ,v */
    boolean	hasRcsDir;	/* true if the path contains "RCS" */
    char	*parent;	/* pointer to the parent directories, if any */
    char	*file;		/* the last component of the path */
    char	*specified;	/* pointer to the original path */
} DigestedPath;

static DigestedPath *DigestPath(path)
    char *path;
{
    DigestedPath *result;
    char *p, *filep;
    int pathLen = strlen(path);
    int fileLen, parentLen;

    result = (DigestedPath *)SecureAlloc(pathLen + 2 + sizeof(DigestedPath));
    result->absolute = (boolean)(*path == '/');
    result->isRcsFile = FALSE;
    result->hasRcsDir = FALSE;
    result->parent = NULL;
    result->specified = path;

    if (pathLen == 0) {
	result->file = "";
	return (result);
    }

    p = path + pathLen - 1;
    if (pathLen > 2 && p[-1] == ',' && p[0] == 'v')
	result->isRcsFile = TRUE;

    /*
     * find the file component.
     */
    for (fileLen = 1, filep = p;;) {
	if (*filep == '/') {
	    filep++; fileLen--;
	    break;
	}
	if (filep == path)
	    break;
	filep--; fileLen++;
    }

    result->file = (char *)(result + 1);
    if (result->isRcsFile) {
	(void) strncpy(result->file, filep, fileLen - 2);
	result->file[ fileLen - 2 ] = '\0';	/* remove the ',v' */
    } else
	(void) strcpy(result->file, filep);

    /*
     * if filep == path, then we found no '/' and this is a simple file name
     */
    if (filep == path)
	return (result);

    /*
     * Note whether the path has an RCS component in it; e.g.
     *
     * path = ".../RCS/file,v"
     *                 ^---- filep
     * which implies that filep[-4 thru -2] would have "RCS"
     */
    parentLen = pathLen - fileLen - 1;
    if (result->isRcsFile && parentLen >= 3) {
	if (filep[-4] == 'R' && filep[-3] == 'C' && filep[-2] == 'S') {
	    result->hasRcsDir = TRUE;
	    if (parentLen == 3)	/* it was just RCS/file,v */
		return(result);
	    parentLen -= 4;	/* remove the RCS/ */
	}
    }

    if (parentLen) {
	result->parent = result->file + fileLen + 1;
	(void) strncpy(result->parent, path, pathLen - fileLen - 1);
	result->parent[ pathLen - fileLen - 1 ] = '\0';
    } else if (result->absolute)
	result->parent = "/";
    else
	result->parent = NULL;

    return (result);
}

/*
 * This routine is called if we are apparently in the top level of some tree.
 * This suggests that the user may be doing things outside of mklinks'
 * purview (where everything looks like a top-level directory).
 */
static boolean ConfirmTopLevelRcsAction(localRcsFile, treeRcsFile)
    char *localRcsFile;
    char *treeRcsFile;
{
    int label = Farm.treeList[ Farm.choice ].label;
    struct stat *st, stTree, *stImplied;

    /*
     * if the rcs file derived from the tree is the same as the rcs file
     * specified or implied by the command line, then we will assume
     * that we are in the tree and we know what's happenin'.
     */
    if ((st = StatFile(stat, treeRcsFile)) == NULL)
	return (FALSE);	/* the tree rcs file doesn't exist */
    stTree = *st;

    if ((stImplied = StatFile(stat, localRcsFile)) == NULL)
	return (TRUE);	/* the local file doesn't exist */

    if (SameFile(&stTree, stImplied))
	return(TRUE);

    Msg("%s\nis in the top level of the %s tree.\nHowever, %s exists.\n",
        treeRcsFile,
        TokenIndexToString(label),
	localRcsFile);
    if (Confirm("  Continue with %s?", treeRcsFile))
	return (TRUE);
    if (Confirm("  Continue with %s?", localRcsFile))
	return (FALSE);

    exit(1);
    /*NOTREACHED*/
}

/*
 * This is used when we masquerade as an rcs command.  If the path fits
 * in some tree, then the returned node will be in that tree and
 * flags & NODE_NEWSOURCE will be true.
 */
static Node *PickSourceNode(treeIndex, cwd, path)
    int treeIndex;
    char *cwd;
    DigestedPath *path;
{
    Node *node, **childList;
    boolean returnToCwd = FALSE;
    char *newCwd, newCwdBuf[ BUFSIZ ];
    char newFromPath[ BUFSIZ ];
    char *localRcsPath, localRcsPathBuf[ BUFSIZ ];
    int i;
    SourceState state;

    /*
     * decide what directory we will look in
     */
    newCwd = cwd;
    if (path->isRcsFile) {
	if (path->parent) {
	    returnToCwd = TRUE;
	    if (! ChangeDir(path->parent))
		Fatal("Can't cd to implied path \"%s\"\n", path->parent);
	    if (! path->absolute)
		ConcatPath(newCwd = newCwdBuf, cwd, path->parent);
	    else
		newCwd = path->parent;
	}
	localRcsPath = path->specified;
    } else
	ComposeRcsFile(localRcsPath = localRcsPathBuf, NullString, path->file);

    /*
     * Check the trees for a match of the implied directory
     */
    state = IdentifyInitialSource(treeIndex, newCwd);
    if (returnToCwd) {
	if (! ChangeDir(cwd))
	    Fatal("Can't get back to '.'\n");
	returnToCwd = FALSE;
    }

    switch (state) {
    case AmbiguousTopDir:
    case AmbiguousSubDir:
	if (StatFile(stat, localRcsPath))
	    return(NULL);
	AmbiguousPath(state, cwd);
	exit (1);
	/*NOTREACHED*/

    case NoSourceFound:
	return (NULL);
    }

    /*
     * This file seems to be one we know about... look into
     * rewriting the command line.
     *
     * Farm.start is the node pointing to the parent of the
     * file uttered on the command line.  Peruse its children
     * looking for a match.  If we find one, make that the
     * starting point.
     */
    if (Farm.start->data) {
	childList = Farm.start->data->inferiors.list;
	for (i = Farm.start->data->inferiors.nNodes - 1; i >= 0; i--) {
	    if (strcmp(childList[ i ]->name, path->file) == 0) {
		Farm.start = node = childList[ i ];
		if ((node->flags & NODE_NEWSOURCE) == 0) {
		    ComposeRcsFile(newFromPath,
				   node->parent->data->from,
				   path->file);
		    AddFromPath(&Farm.treeList[ Farm.choice ],
				node,
				path->specified,
				CopyOfString(newFromPath));
		}
		break;
	    }
	}
	if (i < 0)
	    goto NotInTheTree;

    } else {
	char toPath[ BUFSIZ ];
NotInTheTree:
	/*
	 * At this point, the node is not in the tree... add it.
	 */
	ComposeRcsFile(newFromPath, Farm.start->data->from, path->file);
	
	(void) NodeToPath(toPath, Farm.start, Farm.treeList[ Farm.choice ].top);
	ConcatPath(toPath, toPath, path->file);
	node = AddPathMapToTree(&Farm.treeList[ Farm.choice ],
				newFromPath,
				toPath);
	Farm.start = node;
    }

    /*
     * now then, node is set to the node of interest, but if the tree
     * found was only implicit, then we should make sure this is what
     * the user wants.
     */
    if (state == ImplicitRootDir
     && ! ConfirmTopLevelRcsAction(localRcsPath, node->data->from))
	return (NULL);

    return (node);
}

static boolean CoCiMain(cwd, argc, argv)
    char   *cwd;
    int    argc;
    char    **argv;
{
    int    treeIndex = -1;
    char *workFileArg;
    char *rcsFileArg;
    DigestedPath *path, *rcsPath, *workFilePath;
    Node *node;
    RcsRevision *revision;
    boolean sortPaths;
    boolean addRevisionName = TRUE;
    boolean commandsToRun;

    GenericInitRcs(argv, argc);

    /*
     * Look at the flag arguments to see if any specify an explicit
     * revision.  At the same time collect all of them so that
     * they may be inserted when we rewrite the command.
     */
    for (argv++, argc--; argc; argv++, argc--) {
        if (argv[0][0] == '-') {
	    switch(argv[0][1]) {
	    case 'p':	/* options for co */
		if (CommandSet.id != ProgCo)
		    continue;	/* bad argument? */
		goto CommonOptions;

	    case 'N':	/* options for ci */
	    case 'n':	/* options for ci */
		if (CommandSet.id != ProgCi)
		    continue;	/* bad argument? */
		addRevisionName = FALSE;
		break;

	    case 'k':	/* options for ci */
		if (CommandSet.id != ProgCi)
		    continue;	/* bad argument? */
		goto CommonOptions;

	    case 'f':
	    case 'l':
	    case 'q':
	    case 'r':
	    case 'u':
	    CommonOptions:
		/*
		 * If a revision appears on the command line even once,
		 * don't try to fill one in.
		 */
		if (argv[0][2])
		    addRevisionName = FALSE;
		break;

	    case 'A':
	    case 'T':
	    case 'V':
		ParseCommonFlag(*argv + 1, &treeIndex, GenericUsage);
		break;

	    default:
		break;
	    }
        } else
	    break;
    }

    if (treeIndex < 0)
	treeIndex = GetTreeIndex(NullString);

    /*
     * All that is left is what the "ci" and "co" commands think of as file
     * arguments Interpret each of these according to rcs documentation:
     *   - if a path of a workfile,v file is given, rcs will extract
     *     from the specified file placing it in workfile in the current
     *     directory. Therefore, we look for it in the directory implied by
     *     the argument.
     *   - if a path of a workfile file is given, rcs will look in the current
     *     directory for RCS/workfile,v and then workfile,v, placing the
     *     result in workfile.  Therefore, we look in the current directory.
     *   - if a pair of paths, workfile and workfile,v are given, then
     *     they are used together, but we look in the directory
     *     implied by workfile for the config information.
     */
    commandsToRun = FALSE;
    for (; argc; argv++, argc--) {
	commandsToRun = TRUE;
	path = DigestPath(*argv);

	/*
	 * Check for a pair of files.
	 */
	sortPaths = TRUE;
	if (argc > 1) {
	    rcsPath = DigestPath(argv[1]);
	    if (path->isRcsFile != rcsPath->isRcsFile
	     && strcmp(path->file, rcsPath->file) == 0) {
		if (rcsPath->isRcsFile)
		    workFilePath = path;
		else {
		    workFilePath = rcsPath;
		    rcsPath = path;
		    path = workFilePath;
		}
		argv++, argc--;
		sortPaths = FALSE;
	    } else {
		free((char *) rcsPath);	/* we'll get it on the next iteration */
		rcsPath = NULL;
	    }
	}

	if (sortPaths) {
	    if (path->isRcsFile) {
		rcsPath = path;
		workFilePath = NULL;
	    } else {
		rcsPath = NULL;
		workFilePath = path;
	    }
	}

	node = PickSourceNode(treeIndex,
			      cwd,
			      rcsPath ? rcsPath : workFilePath);
	if (node) {
	    /*
	     * Look for a new source and/or new revision.  If a rcs
	     * file or an rcs label was specified on the command line,
	     * don't override it.
	     */
	    if (addRevisionName)
		revision = GetNodeRevision(node);
	    else
		revision = &DefaultRcsRevision;
	    
	    /*
	     * PickSourceNode has already guaranteed that node will have
	     * the rcsFile filled in (start & NODE_NEWSOURCE) != 0
	     */
	    rcsFileArg = node->data->from;

	    if (rcsPath == NULL) {
		if (workFilePath->parent)
		    workFileArg = workFilePath->specified;
		else
		    workFileArg = NULL;
	    }

	    /*
	     * It must be that rcsPath != NULL.  This is simple: if
	     * workfile was specified, use it, if not don't.
	     */
	    else {
		rcsFileArg = rcsPath->specified;
		if (workFilePath)
		    workFileArg = workFilePath->specified;
		else
		    workFileArg = NULL;
	    }

	    if (workFileArg)
		(void) CollectArguments(&CommandSet,
					NullString, revision, workFileArg);
	    if (rcsFileArg)
		(void) CollectArguments(&CommandSet,
					NullString, revision, rcsFileArg);
	} else {
	    if (workFilePath && rcsPath)
		(void) CollectArguments(&CommandSet,
				    NullString, &DefaultRcsRevision, argv[-1]);
	    (void) CollectArguments(&CommandSet,
				    NullString, &DefaultRcsRevision, argv[0]);
	}
    }

    return (commandsToRun);
}

static void HoldRcsDirMain(cwd, argc, argv)
    char   *cwd;
    int    argc;
    char    **argv;
{
    int treeIndex = -1;
    char srcDir[ BUFSIZ ];
    char *srcParent;
    DigestedPath *path, *rcsPath;
    Node *node;

    for (argv++, argc--; argc; argv++, argc--) {
        if (argv[0][0] == '-') {
	    switch (argv[0][1]) {
	    case 'A':
	    case 'T':
	    case 'V':
		ParseCommonFlag(*argv + 1, &treeIndex, HoldRcsDirUsage);
		break;

	    default:
		HoldRcsDirUsage();
		/*NOTREACHED*/
	    }
        } else
	    break;
    }

    if (argc > 1) {
	Msg("you can specify only one file at a time\n");
	HoldRcsDirUsage();
    }

    if (treeIndex < 0)
	treeIndex = GetTreeIndex(NullString);

    /*
     * If there is a file argument, try to get the RCS directory specific
     * to that file (because there could be several rcs directories
     * represented in the current directory).
     */
    if (argc) {
	path = DigestPath(*argv);

	node = PickSourceNode(treeIndex,
			      cwd,
			      path);
	if (node == NULL)
	    Fatal("No apparent matching tree for %s\n", *argv);

	/*
	 * PickSourceNode has already guaranteed that node will have
	 * the data->from filled in (start & NODE_NEWSOURCE) != 0
	 */
	rcsPath = DigestPath(node->data->from);
	if (!rcsPath->hasRcsDir)
	    Fatal("%s maps to %s, but its not in an RCS directory\n",
		*argv, rcsPath->specified);

	srcParent = rcsPath->parent;
    } else {
	SourceState state;
	/*
	 * Check the trees for a match of the current directory
	 */
	switch (state = IdentifyInitialSource(treeIndex, cwd)) {
	case AmbiguousTopDir:
	case AmbiguousSubDir:
	    AmbiguousPath(state, cwd);
	    exit (1);
	    /*NOTREACHED*/

	case NoSourceFound:
	    Fatal("No apparent matching tree for %s\n", cwd);
	    /*NOTREACHED*/
	}
	srcParent = Farm.start->data->from;
    }

    sprintf(srcDir, "%s/RCS", srcParent);
    (void) MakeSymbolicLink(srcDir, "RCS");
    Msg("%s -> %s\n", srcDir, "RCS");
}

static boolean SimpleRcsMain(cwd, argc, argv)
    char   *cwd;
    int    argc;
    char    **argv;
{
    int treeIndex = -1;
    int rFlags = 0;
    char *workFileArg;
    char *rcsFileArg;
    DigestedPath *path;
    Node *node;
    RcsRevision *revision;
    boolean addRevisionName = TRUE;
    boolean commandsToRun;

    GenericInitRcs(argv, argc);

    /*
     * Look at the flag arguments to see if any specify an explicit
     * revision.  At the same time collect all of them so that
     * they may be inserted when we rewrite the command.
     */
    for (argv++, argc--; argc; argv++, argc--) {
        if (argv[0][0] == '-') {
	    switch (argv[0][1]) {
	    case 'A':
	    case 'T':
	    case 'V':
		ParseCommonFlag(*argv + 1, &treeIndex, GenericUsage);
		break;

	    default:
		switch (CommandSet.id) {
		/*
		 * In the case of rlog we never add
		 * any rcs labels.
		 */
		case ProgRlog:
		    break;

		/*
		 * Count the -r options. If there aren't any, then
		 * we want to add one that has the label specified
		 * in the tree.
		 */
		case ProgRcsdiff:
		case ProgRcsmerge:
		    if (argv[0][1] == 'r')
			rFlags++;
		    break;

		case ProgRcs:
		    switch (argv[0][1]) {
		    case 'N':
		    case 'n':
			addRevisionName = FALSE;
			break;

		    case 'b':
		    case 'l':
		    case 'u':
			/*
			 * If a revision appears on the command line even once,
			 * don't try to fill one in.
			 */
			if (argv[0][2])
			    addRevisionName = FALSE;
			break;
		    }
		    break;
		}
		break;
	    }
        } else
	    break;
    }

    switch (CommandSet.id) {
    case ProgRcsdiff:
    case ProgRcsmerge:
	if (rFlags >= 1)
	    addRevisionName = FALSE;
	break;
    }
	
    if (treeIndex < 0)
	treeIndex = GetTreeIndex(NullString);

    /*
     * All that is left is what the "rcs" think of as file arguments
     * Interpret each of these according to rcs documentation:
     *   - if a path of a workfile,v file is given, rcs will extract
     *     from the specified file placing it in workfile in the current
     *     directory. Therefore, we look for it in the directory implied by
     *     the argument.
     *   - if a path of a workfile file is given, rcs will look in the current
     *     directory for RCS/workfile,v and then workfile,v, placing the
     *     result in workfile.  Therefore, we look in the current directory.
     */
    commandsToRun = FALSE;
    for (; argc; argv++, argc--) {
	commandsToRun = TRUE;
	path = DigestPath(*argv);

	node = PickSourceNode(treeIndex,
			      cwd,
			      path);
	if (node) {
	    /*
	     * Look for a new source and/or new revision.  If a rcs
	     * file or an rcs label was specified on the command line,
	     * don't override it.
	     */
	    if (addRevisionName)
		revision = GetNodeRevision(node);
	    else
		revision = &DefaultRcsRevision;
	    
	    /*
	     * PickSourceNode has already guaranteed that node will have
	     * the rcsFile filled in (start & NODE_NEWSOURCE) != 0
	     */
	    rcsFileArg = node->data->from;

	    if (! path->isRcsFile) {
		if (path->parent)
		    workFileArg = path->specified;
		else
		    workFileArg = NULL;
	    }

	    /*
	     * It must be that path->isRcsFile is true.  This is simple: if
	     * workfile was specified, use it, if not don't.
	     */
	    else {
		rcsFileArg = path->specified;
		workFileArg = NULL;
	    }

	    if (workFileArg)
		(void) CollectArguments(&CommandSet,
					NullString, revision, workFileArg);
	    if (rcsFileArg)
		(void) CollectArguments(&CommandSet,
					NullString, revision, rcsFileArg);
	} else {
	    (void) CollectArguments(&CommandSet,
				    NullString, &DefaultRcsRevision, argv[0]);
	}
    }

    return (commandsToRun);
}

/*
 * Compose a program path.  As a side-effect, assign CommandSet.command.path
 * for use by Execute.  We discover the path by choosing the second
 * occurence of the program in the search path.  If that fails, we
 * settle for the first (with a warning).
 */
static void ComposeProgramPath(programId)
    ProgramId programId;
{
    char firstPathBuf[ BUFSIZ ];
    char secondPathBuf[ BUFSIZ ];
    char *path = firstPathBuf;
    char *execPath, *p;
    boolean anotherPath;
    struct stat *st;

    /*
     * Mklinks does not re-execute itself, it uses "co".
     */
    if (programId == ProgMklinks)
	programId = ProgCo;

    /*
     * HoldRcsDir does not re-execute anything.
     */
    if (programId == ProgHoldRcsDir)
	return;

    if (RcsBin) {
	ConcatPath(path, RcsBin, Programs[ (int)programId ].string);
	CommandSet.command.path = CopyOfString(path);
	return;
    }

    if ((execPath = getenv("PATH")) == NULL)
	Fatal("No PATH in environment?\n");
    while (*execPath) {
	p = execPath-1;
	anotherPath = FALSE;
	while (*(++p)) {
	    if (*p == ':') {
		anotherPath = TRUE;
		break;
	    }
	}
	*p = '\0';

	ConcatPath(path, execPath, Programs[ (int)programId ].string);
	execPath = p;
	if (anotherPath)
	    execPath++;

	if (access(path, X_OK) < 0)
	    continue;
	if (path == secondPathBuf) {
	    CommandSet.command.path = CopyOfString(path);
	    return;
	}
	path = secondPathBuf;
    }

    if (path == secondPathBuf) {
	Msg("Warning: only one copy of %s found in %s.  Continuing...\n",
	    Programs[ (int)programId ].string, firstPathBuf);
	CommandSet.command.path = CopyOfString(firstPathBuf);
    } else {
	Fatal("Can't find an alternate copy of %s\n",
	    Programs[ (int)programId ].string);
    }
}

main(argc, argv)
    int    argc;
    char    **argv;
{
    int	   i, exitValue;
    boolean commandsToRun;
    ProgramId programId;
    char   cwd[ BUFSIZ ];
    DigestedPath *cmdPath = DigestPath(*argv);

    Farm.choice = -1;	/* mark this: nothing chosen yet */
    /*
     * Set up the program name so that it looks right in messages and
     * we know what we are supposed to do.
     */
    ProgramName = *argv; /* something to start with */
    if (strncmp(ProgramName, ProgramPrefix, strlen(ProgramPrefix)) == 0)
	Fatal("two copies of mklinks (named %s)\nare in your execution path\n",
	    ProgramName + strlen(ProgramPrefix));

    programId = ProgUnknown;
    for (i = 0; Programs[ i ].string; i++)
	if (strcmp(Programs[ i ].string, cmdPath->file) == 0) {
	    CommandSet.id = programId = Programs[ i ].id;
	    ProgramName = cmdPath->file;
	    break;
	}

    if (programId == ProgUnknown) {
	ProgNameMsg("This binary should be named one of:\n    ");
	for (i = 0; Programs[ i ].string; i++)
	    Msg("%s%s", i > 0 ? ", " : "", Programs[ i ].string);
	Msg("\n");
	Fatal("You must change the name or change mklinks.\n");
    }

    if (! getwd(cwd))
        Fatal("%s\n", cwd);

    /*
     * Parse the tree specifications.
     */
    ReadTreeSpec();

    /*
     * Figure out the argv0 for execv
     */
    ComposeProgramPath(programId);

    switch(programId) {
    case ProgMklinks:
	MklinksMain(cwd, argc, argv);
	commandsToRun = FALSE;
	break;

    case ProgRcs:
    case ProgRlog:
    case ProgRcsdiff:
    case ProgRcsmerge:
	commandsToRun = SimpleRcsMain(cwd, argc, argv);
	break;

    case ProgHoldRcsDir:
	HoldRcsDirMain(cwd, argc, argv);
	break;

    case ProgCi:
    case ProgCo:
	commandsToRun = CoCiMain(cwd, argc, argv);
	break;
    }

    exitValue = 0;
    if (commandsToRun) {
        for (i = 0; i < CommandSet.nUnique; i++)
            exitValue |= Execute(&CommandSet.unique[ i ].cmd, FALSE);
    } else if (programId != ProgHoldRcsDir && programId != ProgMklinks) {
	/* this will get an rcs usage */
	exitValue = Execute(&CommandSet.command, FALSE);
    }
    exit(exitValue);

    /*NOTREACHED*/
}
