/* $Header: /usr/local/src/nvrToMIT/RCS/mklinks.c,v 3.13 1992/11/18 01:20:30 terryl Exp $ */
#include "struct.h"

/*
 * we assume here that the parent of
 * 'node' is the current working directory (in the destination tree)
 */
static boolean MakeLink(node, pathFrom)
    Node *node;
    char *pathFrom;
{
    short   fromType, toType;
    char    linkName[ BUFSIZ ];
    char    *linkNameShouldBe, linkNameBuf[ BUFSIZ ];
    char    pathTo[ BUFSIZ ];    

    (void) NodeToPath(pathTo, node, NullNode);
    Verbose(3, ("link %s -> %s ", pathFrom, pathTo));

    if (node->parent && (node->parent->flags & NODE_NOTAVAILABLE))
	toType = 0;
    else
	toType = FileType(node->name);
    fromType = node->type;

    if ((toType & NODE_SYMLINK) == 0 || fromType == 0) {
        if (fromType == 0 || (fromType != toType && toType != 0)) {
            SyscallError("%s is a %s, but %s is a %s!\n",
                pathTo, TypeName(toType), pathFrom, TypeName(fromType));
            return(FALSE);
        }
        if (fromType == toType) {
            Verbose(3, ("\n%s is already a %s\n", pathTo, TypeName(toType)));
            return(TRUE);
        }
    }

    switch (fromType) {
    case NODE_DIRECTORY:
        Verbose(3, ("(mkdir)\n"));
        if (! MakeDirectory(node->name, 0700 | permof(pathFrom)))
            return(FALSE);
        break;

    case NODE_REGULAR:
        if (node->flags & NODE_USEONLYRCS) {
            Verbose(3, ("(not linked)\n"));
            break;
        }
	/* fall through... */

    case NODE_SYMLINK:
        Verbose(3, ("(symlink)\n"));
        if (fromType == NODE_SYMLINK) {
            if (! GetLinkName(pathFrom, linkNameBuf, BUFSIZ))
                return(FALSE);
            linkNameShouldBe = linkNameBuf;
        } else
            linkNameShouldBe = pathFrom;
        if (toType != 0) {
            if (! GetLinkName(node->name, linkName, BUFSIZ))
                return(FALSE);
            if (strcmp(linkName, linkNameShouldBe) == 0) {
                Verbose(3, ("symlink %s ok.\n", pathTo));
                return(TRUE);
            }
            Verbose(3, ("symlink %s wrong.\n", pathTo));
            if (! UnlinkPath(node->name, TRUE))
                return(FALSE);
        }
        (void) MakeSymbolicLink(linkNameShouldBe, node->name);
	break;

    default:
        Verbose(3, ("%s: can't work with a %s\n",
	    pathFrom, TypeName(fromType)));
	break;
    }
    return(TRUE);
}

/*
 * This routine is called recursively, and assumes that the parent of
 * 'node' is the current working directory (in the destination tree)
 *
 * we spend all of our time in the destination tree using chdir
 * to walk among the nodes.  This at least caches the destination
 * directory in the inode cache.
 */
void LinkTree(node, pathFrom)
    Node *node;
    char *pathFrom;
{
    static int level = 1;

    if (! MakeLink(node, pathFrom))
	return;

    /*
     * we may have inferior nodes that need to be gotten.
     */
    if (node->type & NODE_DIRECTORY) {
	/*
	 * This is where we descend recursively.  If we only want one
	 * directory level, break out of the loop after the first
	 * directory which we assume is "."
	 */
	if (OneLevel && level > 1)
	    return;

	/*
	 * Get into the directory and work from there.
	 */
	if (! ChangeNode(node, TRUE)) {
	    /*
	     * If we are just trying to check things out, keep going.
	     */
	    if (! ListFiles && ! CreateCommands)
		return;
	}

	/*
	 * let GetInferiors() figure out what to omit, etc.  But it
	 * may call LinkTree() for some of the inferiors.
	 */
	level++;
	GetInferiors(node, pathFrom);
	level--;

	/*
	 * We always have to call ChangeNode() just in case a directory
	 * needs to be created
	 */
	if (! ChangeNode(node->parent, FALSE)
	 && (node->flags & NODE_NOTAVAILABLE) == 0)
            Fatal("can't get back to %s/..\n", pathFrom);
    }
}

static void GetNodeFileType(node, fullFromPath, pComponent)
    Node *node;
    char *fullFromPath;
    char *pComponent;
{
    if (node->flags & NODE_NEWSOURCE)
	node->type = FileType(node->data->from);
    else {
	(void) strcpy(pComponent, node->name);
	node->type = FileType(fullFromPath);
    }
}
    
/*
 * Called to obtain all the immediate inferiors of 'parent'.  We cannot assume
 * that parent->data is non-null.
 */
void GetInferiors(parent, pathFrom)
    Node *parent;
    char *pathFrom;
{
    NodeList acquired, normal, rcs;
    Node *node;
    int i, j;
    int existingNodesUsed;
    char childFromPath[ BUFSIZ ], *fromComponent, *from;
    Node *pExisting, *pNew;
    NodeData *data = parent->data;
    boolean onlyCertainNodes;
    boolean found;
    boolean hasRcsNode;

    (void) strcpy(childFromPath, pathFrom);
    fromComponent = childFromPath + strlen(childFromPath);
    *fromComponent++ = '/';

    /*
     * If there is an 'only' list of inferiors, then
     * we simply use the nodes that are already in the tree.
     */
    onlyCertainNodes = ((parent->flags & NODE_HASONLY) != 0);

    /*
     * If we are only interested in certain nodes, then there is no need to
     * read the contents of the directory.  However, if we are using RCS
     * then we need to read the directory anyway to see if an RCS directory
     * exists.
     */
    rcs.list = normal.list = NULL;
    rcs.nNodes = normal.nNodes = 0;
    if (onlyCertainNodes && (parent->flags & NODE_USERCS) == 0) {
	if (data) {
	    for (i = 0; i < data->inferiors.nNodes; i++) {
		pExisting = data->inferiors.list[ i ];
		if ((pExisting->flags & (NODE_OMIT | NODE_ONLY)) == NODE_ONLY) {
		    GetNodeFileType(pExisting, childFromPath, fromComponent);
		    if (pExisting->flags & NODE_ISRCSFILE)
			AddToNodeList(&rcs, pExisting);
		    else
			AddToNodeList(&normal, pExisting);
		}
	    }
	}
	acquired.nNodes = 0;	/* this says "don't free acquired.list */
    }

    else {
	acquired.list = NULL;
	acquired.nNodes = 0;
	hasRcsNode = ReadFiles(&acquired,
			       pathFrom,
			       (boolean) ((parent->flags & NODE_USERCS) != 0));

	if (hasRcsNode)
	    (void) ReadRcsFiles(&acquired, pathFrom);

	/*
	 * check every file in this directory
	 */
	existingNodesUsed = 0;
	for (i = 0; i < acquired.nNodes; i++) {
	    pNew = acquired.list[ i ];

	    /*
	     * Get the file type.  If we are only interested in RCS,
	     * then only allow rcs files and directories to filter through.
	     */
	    if ((pNew->flags & NODE_ISRCSFILE) == 0) {
		GetNodeFileType(pNew, childFromPath, fromComponent);
		if ((pNew->type & NODE_DIRECTORY) == 0
		 && (parent->flags & NODE_USEONLYRCS))
		    continue;
	    }

	    /*
	     * if we already have this node in our list of inferiors,
	     * then just get the flags
	     */
	    found = FALSE;
	    if (data) {
		for (j = data->inferiors.nNodes - 1; j >= 0; j--) {
		    pExisting = data->inferiors.list[ j ];
		    if (SameNodeName(pNew, pExisting)) {
			existingNodesUsed++;
			pExisting->flags |= (pNew->flags | NODE_USED);
			pExisting->type = pNew->type;
			acquired.list[ i ] = pExisting;
			FreeNode(pNew);
			node = pExisting;
			found = TRUE;
			break;
		    }
		}
	    }

	    if (! found) {
		/* XXX should we add the node to the parent`s list? */
		pNew->parent = parent;
		node = pNew;
	    }

	    if (node->flags & NODE_OMIT)
		continue;
	    if (onlyCertainNodes && (node->flags & NODE_ONLY) == 0)
		continue;

	    /*
	     * Sort the list into rcs-updated files, and other.
	     */
	    if (node->flags & NODE_ISRCSFILE)
		AddToNodeList(&rcs, node);
	    else {
		if (node->type == 0)
		    Msg("Warning: %s doesn't exist\n", childFromPath);
		else
		    AddToNodeList(&normal, node);
	    }
	}

	/*
	 * there may be some nodes in the existing data->inferiors.list that
	 * were not found in the acquired list.  There are two possible reasons
	 * for this:
	 *   1. the config file specified it but it doesn't really exist.
	 *   2. it exists, but not in pathFrom; i.e., it has its own souce
	 * Note that if it *was* on the acquired list and has its own source, it
	 * will be handled correctly below.  For case 1, warn the user.  For
	 * case 2, add it to the list.
	 */
	if (data && existingNodesUsed < data->inferiors.nNodes) {
	    for (i = data->inferiors.nNodes - 1; i >= 0; i--) {
		pExisting = data->inferiors.list[ i ];
		if (pExisting->flags & (NODE_USED | NODE_OMIT))
		    continue;

		if ((pExisting->flags & NODE_NEWSOURCE) == 0) {
		    Msg("Warning: %s/%s was specified but doesn't exist\n",
			pathFrom, pExisting->name);
		    continue;
		}

		GetNodeFileType(pExisting, childFromPath, fromComponent);
		if (pExisting->flags & NODE_ISRCSFILE)
		    AddToNodeList(&rcs, pExisting);
		else
		    AddToNodeList(&normal, pExisting);
	    }
	}
    }

    /*
     * 'rcs' and 'normal' now contain all the inferiors we will update.
     * Obtain all nodes from each list, recursing on the directories.
     */
    if (rcs.nNodes)
	ExtractRcsFiles(&rcs, pathFrom);
    for (i = 0; i < normal.nNodes; i++) {
	pNew = normal.list[ i ];
	if (pNew->flags & NODE_NEWSOURCE) {
	    Verbose(1, ("using source %s...\n", pNew->data->from));
	    from = pNew->data->from;
	} else {
	    (void) strcpy(fromComponent, pNew->name);
	    from = childFromPath;
	}

	LinkTree(pNew, from);
    }

    if (normal.nNodes)
	free((char *)normal.list);
    if (rcs.nNodes)
	free((char *)rcs.list);
    if (acquired.nNodes)
	free((char *)acquired.list);
}

void SetTreeRootToString(label, root)
    int label;
    char *root;
{
    DirTree *dirTree = LabelToTree(label);

    if (dirTree->toRoot.path) {
	if (strcmp(dirTree->toRoot.path, root) == 0)
	    return;

	ConfigError("Root for %s redefined from %s to %s\n",
	    TokenIndexToString(label), dirTree->toRoot.path, root);
    }
    dirTree->toRoot.path = CopyOfString(root);
}


#ifdef notdef
void SetTreeRootToTreeTmp(label, treeTmp, lastComponent)
    int label;
    TreeTmp *treeTmp;
{
    Node *newNode, *node, *top;
    DirTree *dirTree = LabelToTree(label);
    int i;
    char root[ BUFSIZ ];

    dirTree->root = NewNode("", NODE_PERMANENT | NODE_IS_ROOT);
    node = dirTree->root;

    if (treeTmp->absolute)
	(void) strcpy(root, "/");
    else
	root[0] = '\0';
    for (i = 0; i <= lastComponent; i++) {
	newNode = NewNode(treeTmp->list[ i ], NODE_PERMANENT);
	AddInferior(node, newNode);
	ConcatPath(root, root, newNode->name);
	node = newNode;
    }
    dirTree->depthOfTop = lastComponent + 1;
    dirTree->toRoot.path = CopyOfString(root);

    if ((top = dirTree->top) == NULL) {
	node->flags |= NODE_IS_TOP;
	dirTree->top = node;
    } else {
	/*
	 * The simplest way is to throw away the bottom-most node
	 * from the new root, and replace it with the old 'top'.
	 */ 
	node->parent->data->inferiors.list[0] = top;
	top->parent = node->parent;
	free(top->name);
	top->name = node->name;
	node->name = NullString; /* this keeps FreeNode() from freeing it */
	FreeNode(node);
    }
}
#endif

Node *AddPathMapToLabel(label, from, to)
    int label;
    char *from;
    char *to;
{
    return( AddPathMapToTree(LabelToTree(label), from, to) );
}

void AddFromPath(tree, node, to, from)
    DirTree *tree;
    Node *node;
    char *to;
    char *from;
{
    int length = strlen(from);
    char nodeName[ BUFSIZ ];

    if (node->flags & NODE_NEWSOURCE) {
	if (strcmp(from, node->data->from) == 0)
	    return;
	Fatal("source for %s declared twice as %s and %s\n",
	    to, from, node->data->from);
    }
    if (node->data == NULL)
	AddNodeData(node);
    node->data->from = from;
    node->flags |= NODE_NEWSOURCE;
    AddToNodeList(&tree->from, node);

    /*
     * This might be an rcs file... check it out.
     */
    if (length > 3 && from[ length-2 ] == ',' && from[ length-1 ] == 'v') {
	/*
	 * When merging trees, the flags haven't been added yet
	 *
	 * if ((node->flags & NODE_USERCS) == 0) {
	 *     ConfigError("the source for %s\n%s\n%s\n",
	 * 	 to, "looks like an RCS file",
	 * 	"but RCS isn't specified (yet) for this subtree.");
	 *     Msg("Am assuming that this is an RCS file...\n");
	 * }
	 */
	node->flags |= (NODE_ISRCSFILE | NODE_USERCS);
    }
}

/*
 * Add a mapping from --> to.  Create nodes where necessary.  Note that
 * 'from' should be a preallocated string, not something on the stack,
 * so that it can be permanently stored with the node.
 */
Node *AddPathMapToTree(tree, from, to)
    DirTree *tree;
    char *from;
    char *to;
{
    TreeTmp *treeTmp;
    int i, j;
    Node **nodeList;
    Node *node;
    Node *newNode;
    short inheritedFlags, flags;

    if (tree->top == NULL)
	tree->top = NewNode("", NODE_IS_TOP | NODE_PERMANENT);
    node = tree->top;
    inheritedFlags = InheritedFlags(node->flags);

    if (strcmp(to, ".") != 0) {
	treeTmp = PathToTreeTmp(to);
	for (i = 0; i < treeTmp->nNodes; i++) {
	    /*
	     * We either jump to a new level, or add a new child to
	     * this node.
	     */
	    if (node->data) {
		nodeList = node->data->inferiors.list;
		for (j = node->data->inferiors.nNodes - 1; j >= 0; j--) {
		    if (strcmp(nodeList[ j ]->name, treeTmp->list[ i ]) == 0)
			goto NextLevel;
		}
	    }

	    newNode = NewNode(treeTmp->list[ i ],
			      NODE_PERMANENT | inheritedFlags);
	    AddInferior(node, newNode);
	    j = node->data->inferiors.nNodes - 1;

	NextLevel:
	    if (i == 0 && treeTmp->absolute)
		node->flags |= NODE_IS_ROOT;
	    node = node->data->inferiors.list[ j ];
	    inheritedFlags = InheritedFlags(inheritedFlags | node->flags);
	}
	free((char *)treeTmp);
    }

    if (from)
	AddFromPath(tree, node, to, from);

    return (node);
}

void WalkTree(node, func, arg1, arg2)
    Node *node;
    void (*func)();
    caddr_t arg1, arg2;
{
    Node **nodeList;
    int i;

    (*func)(node, arg1, arg2);
    if (node->data) {
	nodeList = node->data->inferiors.list;
	for (i = node->data->inferiors.nNodes - 1; i >= 0; i--)
	    WalkTree(nodeList[ i ], func, arg1, arg2);
    }
}

/*ARGSUSED*/
static void AddFlagsToNode(node, arg1, arg2)
    Node *node;
    caddr_t arg1;
    caddr_t arg2;
{
    short flags = (short)arg1;

    node->flags |= flags;
    if (flags & NODE_ONLY)
	node->parent->flags |= NODE_HASONLY;
    if (flags & NODE_OMIT)
	node->parent->flags |= NODE_HASOMIT;
}

/*ARGSUSED*/
static void RemoveFlagsFromNode(node, arg1, arg2)
    Node *node;
    caddr_t arg1;
    caddr_t arg2;
{
    short flags = (short)arg1;

    /*
     * XXX if we ever use this for NODE_{ONLY,OMIT}, we may have to
     * clear the parent's NODE_HAS{ONLY,OMIT}
     */
    node->flags &= (~flags);
}

/*ARGSUSED*/
static void AddFlaggedNode(flaggedNode, arg1, arg2)
    Node *flaggedNode;
    caddr_t arg1, arg2;
{
    char flaggedPath[ BUFSIZ ];
    int label;
    short flags = (short)arg2;
    Node *newNode;

    if (flaggedNode->flags & flags) {
	label = (int)arg1;
	(void) NodeToPath(flaggedPath, flaggedNode, NullNode);
	newNode = AddPathMapToLabel(label, NullString, flaggedPath);
	newNode->flags |= (flaggedNode->flags & flags);
    }
}

void AddPathToLabel(label, path, flags)
    int label;
    char *path;
    int flags;
{
    AddFlagsToNode( AddPathMapToLabel(label, NullString, path),
		    (caddr_t) flags,
		    (caddr_t) 0);
}

void AddFlagsFromLabel(treeLabel, flaggedLabel, flags)
    int treeLabel;
    int flaggedLabel;
    int flags;
{
    DirTree *flaggedTree = LabelToTree(flaggedLabel);

    WalkTree(flaggedTree->top,
	     AddFlaggedNode,
	     (caddr_t) treeLabel,
	     (caddr_t) flags);
}

static void AddFlagsToSubTree(subTree, flags)
    Node *subTree;
    int flags;
{
    WalkTree(subTree, AddFlagsToNode, (caddr_t) flags, (caddr_t) 0);
}

static void RemoveFlagsFromSubTree(subTree, flags)
    Node *subTree;
    int flags;
{
    WalkTree(subTree, RemoveFlagsFromNode, (caddr_t) flags, (caddr_t) 0);
}

void KeepRcsLink(label, path)
    int label;
    char *path;
{
    AddFlagsToSubTree( AddPathMapToLabel(label, NullString, path),
		       NODE_USERCS | NODE_KEEPRCSLINK );
}

void UseOnlyRcs(label, path)
    int label;
    char *path;
{
    RemoveFlagsFromSubTree( AddPathMapToLabel(label, NullString, path),
			    NODE_DROPRCS );
    AddFlagsToSubTree( AddPathMapToLabel(label, NullString, path),
		       NODE_USERCS | NODE_USEONLYRCS );
}

void UseRcs(label, path)
    int label;
    char *path;
{
    RemoveFlagsFromSubTree( AddPathMapToLabel(label, NullString, path),
			    NODE_DROPRCS | NODE_USEONLYRCS );
    AddFlagsToSubTree( AddPathMapToLabel(label, NullString, path),
		       NODE_USERCS );
}

void DropRcs(label, path)
    int label;
    char *path;
{
    Node *node = AddPathMapToLabel(label, NullString, path);

    RemoveFlagsFromSubTree( node,
			    NODE_KEEPRCSLINK | NODE_USERCS | NODE_USEONLYRCS );
    AddFlagsToSubTree( node, NODE_DROPRCS );
}

void SetRcsRevision(treeLabel, toPath, revision, revisionType)
    int treeLabel;
    char *toPath;
    char *revision;
    RcsRevisionType revisionType;
{
    Node *node;

    node = AddPathMapToLabel(treeLabel, NullString, toPath);
    if (node->data == NULL)
	AddNodeData(node);
    if (node->flags & NODE_REVISIONINFO
     && node->data->rev.type != revisionType
     && strcmp(revision, node->data->rev.u.label) != 0) {
	ConfigError("rcs revision info for \"%s\" tree set twice\n",
	    TokenIndexToString(treeLabel));
	if (node->data->rev.u.version)
	    free(node->data->rev.u.version);
        node->data->rev.u.version = NULL;
    }

    if (revision) {
	node->data->rev.u.label = CopyOfString(revision);
	node->data->rev.stringLength = strlen(revision);
    }
    node->data->rev.type = revisionType;
    node->flags |= NODE_REVISIONINFO;
    UseRcs(treeLabel, toPath);
}

/*ARGSUSED*/
static void AddGenericNode(node, arg1, arg2)
    Node *node;
    caddr_t arg1, arg2;
{
    DirTree *tree = (DirTree *)arg1;
    char *toRoot = (char *)arg2;
    char toPath[ BUFSIZ ];
    Node *addedNode;
    long rootLength = (toRoot == NULL) ? 0 : strlen(toRoot);

    if (toRoot == NULL)
	rootLength = 0;
    else {
	rootLength = strlen(toRoot);
	strcpy(toPath, toRoot);
	toPath[ rootLength++ ] = '/';
    }

    (void) NodeToPath(toPath + rootLength, node, NullNode);
    if (node->flags & NODE_REVISIONINFO)
	SetRcsRevision(tree->label,
		       toPath,
		       node->data->rev.u.label,
		       node->data->rev.type);
    addedNode = AddPathMapToTree(tree,
				 node->data ? node->data->from : NullString,
				 toPath);
    addedNode->flags |= node->flags;
    if (addedNode->flags & NODE_DROPRCS)
	addedNode->flags &= ~(NODE_USERCS|NODE_USEONLYRCS|NODE_KEEPRCSLINK);
    else
	addedNode->flags &= ~NODE_DROPRCS;
}

/*
 * This gets called when one tree has an "Include ..." with some tree
 * label.  Add it here.  Make sure that we start adding into the
 * tree at the current depth, hence the ExpandTreeStack() call.
 */
void CombineTreeLabels(labelTo, labelFrom)
    int labelTo;
    int labelFrom;
{
    DirTree *treeFrom = LabelToTree(labelFrom);
    DirTree *treeTo = LabelToTree(labelTo);
    char *currentDir = ExpandTreeStack(NullString);

    if (strcmp(currentDir, ".") == 0)
	currentDir = NULL;
    WalkTree(treeFrom->top,
	     AddGenericNode,
	     (caddr_t)treeTo,
	     (caddr_t)currentDir);

    if (treeFrom->toRoot.path)
	SetTreeRootToString(labelTo, treeFrom->toRoot.path);
}

/*
 * See if comparison shows that the first nodes of
 * 'treeTmp' * are the same as the bottom most nodes of 'node', and that the
 * pathname implied exists; e.g.,
 *	node->data->from is "/xsrc/myclients/fum"
 *      node is "clients/foo"
 *      nodeIndex points at "clients/foo/bitmaps" in treeTmp
 * then CheckForMappedInferior() add a new starting point to 'Farm' iff
 * /xsrc/myclients/fum/bitmaps exists.
 */
static void CheckForMappedInferior(tree, fromIndex, treeTmp, nodeIndex)
    DirTree *tree;
    int fromIndex;
    TreeTmp *treeTmp;
    int nodeIndex;
{
    Node *node = tree->from.list[ fromIndex ];
    Node *n, **nodeList;
    char pathFrom[ BUFSIZ ];
    char pathTo[ BUFSIZ ];
    int lastCommonNode, i, j, fileType;
    StartingPoint *startingPoint;

    /*
     * the node in the tree may be deeper than the treeTmp we
     * are considering.
     */
    lastCommonNode = nodeIndex + DepthOfNode(node) - 1;
    if (lastCommonNode >= treeTmp->nNodes)
	return;

    if ((node->flags & NODE_NEWSOURCE) == 0)
	Fatal("internal: no from path\n");

    /*
     * First, be sure that from here to the "top" of the tree,
     * all of the common nodes match.
     */
    n = node;
    for (i = lastCommonNode; i >= nodeIndex; i--) {
	if (strcmp(treeTmp->list[i], n->name) != 0)
	    return;
	n = n->parent;
    }

    (void) strcpy(pathFrom, node->data->from);
    fileType = 0;
    (void) NodeToPath(pathTo, node, tree->top);
    if (lastCommonNode + 1 < treeTmp->nNodes) {
	/*
	 * then build a pathname that starts with the 'from' path
	 * and ends with the non-common nodes
	 */
	for (i = lastCommonNode + 1; i < treeTmp->nNodes; i++) {
	    ConcatPath(pathFrom, pathFrom, treeTmp->list[ i ]);
	    ConcatPath(pathTo,   pathTo,   treeTmp->list[ i ]);
	    /*
	     * Keep track of the tree as long as the path we
	     * are building is congruent to some subtree.
	     */
	    if (node == NULL || node->data == NULL) {
		node = NULL;
		continue;
	    }
	    nodeList = node->data->inferiors.list;
	    for (j = node->data->inferiors.nNodes - 1; j >= 0; j--) {
		if (strcmp(treeTmp->list[ i ], nodeList[ j ]->name) == 0) {
		    node = nodeList[ j ];
		    break;
		}
	    }
	    if (j < 0) {	/* couldn't find a matching inferior */
		node = NULL;
		continue;
	    }

	    /*
	     * if this node (which is deeper than the one we were handed)
	     * has an alternate source, then either
	     *  - this node will appear later in the tree's 'from' list, in
	     *    which case a later iteration in IdentifyInitialSource()
	     *    will find it.
	     *  - we've already checked this node out and the pathFrom doesn't 
	     *    exist.
	     * In either case, we're outa here.
	     */
	    if (node->flags & NODE_NEWSOURCE)
		return;
	}
    }
    if (fileType = FileType(pathFrom)) {
	startingPoint = (StartingPoint *) SecureAlloc(sizeof(StartingPoint));
	startingPoint->tree = tree;
	startingPoint->node = node;
	startingPoint->pathTo = CopyOfString(pathTo);
	startingPoint->pathFrom = CopyOfString(pathFrom);
	startingPoint->fileType = fileType;
	startingPoint->treeTopIndex = nodeIndex;

	Farm.starts = (StartingPoint **)
	    NullRealloc((char *)Farm.starts,
			(unsigned)(sizeof(StartingPoint *) * ++Farm.nStarts));
	Farm.starts[ Farm.nStarts-1 ] = startingPoint;
    }
}

/*ARGSUSED*/
static void SetTreeChoice(start, rootTreeTmp)
    StartingPoint *start;
    TreeTmp *rootTreeTmp;
{
    if (start->node == NULL || (start->node->flags & NODE_NEWSOURCE) == 0)
	start->node = AddPathMapToTree(start->tree,
				       CopyOfString(start->pathFrom),
				       CopyOfString(start->pathTo));
    start->node->type = start->fileType;
    Farm.choice = start->tree - Farm.treeList;
    Farm.start = start->node;
}

/*
 * A 'toRoot' has been specified for this tree.  Do a chdir() to it, followed
 * by a 'pwd', and see if they are different.
 */
void GetRealRoot(tree, cwd)
    DirTree *tree;
    char *cwd;
{
    TreeTmp *treeTmp;
    char realPath[ BUFSIZ ];

    if (tree->toRoot.realPath == NULL) {
	tree->toRoot.realPath = tree->toRoot.path;
	if (! ChangeDir(tree->toRoot.path) || ! getwd(realPath)) {
	    /*
	     * SyscallError("root for %s doesn't exist (%s)\n",
	     *	TokenIndexToString(tree->label),
	     *	tree->toRoot.path);
	     */
	    tree->dontConsider = TRUE;
	}
	if (! ChangeDir(cwd))
	    Fatal("Can't get back to %s\n", cwd);
	if (tree->dontConsider)
	    return;

	if (strcmp(tree->toRoot.path, realPath) != 0) {
	    tree->toRoot.realPath = CopyOfString(realPath);
	    treeTmp = PathToTreeTmp(realPath);
	} else
	    treeTmp = PathToTreeTmp(tree->toRoot.path);
	tree->toRoot.depth = treeTmp->nNodes;
	free((char *)treeTmp);
    }
}

/*
 * Use the 'toDir' (an absolute path name) to determine where we
 * are in the source tree specified by fromDir.  Produce from that a
 * root (which comes from the Atlas.mapList, and a subdirectory.
 */
SourceState IdentifyInitialSource(specifiedTreeIndex, cwd)
    int     specifiedTreeIndex;
    char    *cwd;
{
    int     i, j, k, usableTrees;
    StartingPoint startBuf, *start = &startBuf;
    TreeTmp *treeTmp;
    DirTree *tree, *specifiedTree;
    boolean onlyMatchAbsolute;

    /*
     * Previous calls may have identified some starting points...
     * throw those away.
     */
    if (Farm.nStarts) {
	for (i = 0; i < Farm.nStarts; i++) {
	    free(Farm.starts[i]->pathTo);
	    free(Farm.starts[i]->pathFrom);
	    free((char *)Farm.starts[i]);
	}
	free((char *)Farm.starts);
	Farm.starts = NULL;
	Farm.nStarts = 0;
    }


    /*
     * if there is a label defined, find it.
     */
    if (specifiedTreeIndex >= 0) {
	specifiedTree = &Farm.treeList[ specifiedTreeIndex ];
	if (specifiedTree->dontConsider) {
	    Msg("Warning: specified tree %s cannot be considered\n",
		TokenIndexToString(specifiedTree->label));
	    return (NoSourceFound);
	}
    } else
	specifiedTree = NULL;

    /*
     * For each of the trees that have a root specified, consider
     * them only if our working directory intersects with it.
     */
    onlyMatchAbsolute = FALSE;
StartCountingAgain:
    usableTrees = 0;
    for (i=0; i < Farm.nTrees; i++) {
	tree = &Farm.treeList[i];
	if (tree->dontConsider)
	    continue;
	if (specifiedTree && specifiedTree != tree)
	    continue;

	if (tree->toRoot.path) {
	    GetRealRoot(tree, cwd);
	    if (PathsIntersect(tree->toRoot.realPath, cwd, strlen(cwd))) {
		/*
		 * If we match one absolute path, then turn on
		 * 'onlyMatchAbsolute' and start the search again, but
		 * this time don't count the relative paths as usable
		 */
		if (! onlyMatchAbsolute) {
		    onlyMatchAbsolute = TRUE;
		    goto StartCountingAgain;
		}
		usableTrees++;
	    } else
		tree->dontConsider = TRUE;
	} else if (! tree->dontConsider) {
	    /* set for 'omit' & 'only' trees */
	    usableTrees++;
	}
    }

    /*
     * examine our working directory, making it shorter and shorter (from
     * the left hand end).
     */
    Farm.choice = -1;
    treeTmp = PathToTreeTmp(cwd);
    for (i = 0; i < treeTmp->nNodes; i++) {

	/*
	 * look at each of the trees to see if this portion of the working
	 * directory fits.
	 */
	for (j=0; j < Farm.nTrees; j++) {
	    tree = &Farm.treeList[j];

	    /*
	     * if we are only considering a certain tree, forget all others
	     */
	    if (specifiedTreeIndex != j && specifiedTreeIndex >= 0)
		continue;

	    /*
	     * Some trees we may have already eliminated.
	     * Others, if they have a 'toRoot', should not be considered
	     * until 'i' points to the node that would correspond with
	     * the current tree's "top".
	     */
	    if (tree->dontConsider)
		continue;
	    if (onlyMatchAbsolute) {
		if (tree->toRoot.path == NullString)
		    continue;
		if (i != tree->toRoot.depth)
		    continue;
	    }

	    /*
	     * Consider each subdirectory mapping in order.
	     */
	    for (k = 0; k < tree->from.nNodes; k++)
		CheckForMappedInferior(tree, k, treeTmp, i);
	}
    }

    /*
     * From here on out, we decide whether we have decided...
     */
    switch (Farm.nStarts) {
    default:
	free((char *)treeTmp);
	return (AmbiguousSubDir);

    case 1:
	SetTreeChoice(Farm.starts[0], treeTmp);
	free((char *)treeTmp);
	return (UniqueSourceFound);
	break;

    case 0:
	/*
	 * No subtree matched, so this must be a top level
	 * directory.  If there is only one valid tree to pick from
	 * then that must be what the user wants.  First, let's see if
	 * that is so.
	 */
	if ((tree = specifiedTree) == NULL) {
	    switch (usableTrees) {
	    case 0:
		return (NoSourceFound);

	    case 1:
		for (i = 0; i < Farm.nTrees; i++)
		    if (! Farm.treeList[ i ].dontConsider) {
			tree = & Farm.treeList[ i ];
			break;
		    }
		break;

	    default:
		return(AmbiguousTopDir);
	    }
	}
	

	/*
	 * Mark which tree we are going to use.  If the top of the tree
	 * has no defined source then we really can't use this.
	 */
	if ((tree->top->flags & NODE_NEWSOURCE) == 0)
	    return (NoSourceFound);
	start->tree = tree;
	start->node = tree->top;
	start->pathTo = NullString; /* cell unneeded since start->node!=NULL */
	start->pathFrom = NullString; /* not necessary... */
	start->fileType = FileType(tree->top->data->from);
	start->treeTopIndex = treeTmp->nNodes;
	SetTreeChoice(start, treeTmp);
	free((char *)treeTmp);

	return (tree->toRoot.path == NULL
		 ? ImplicitRootDir : UniqueSourceFound);
    }
    /*NOTREACHED*/
}
