.TH MKLINKS 1 "21 Mar 1988"
.SH NAME
.B mklinks
- a program for creating and updating software development trees.
.SH SYNOPSIS
.B mklinks
[ \fIoptions\fP ] [ \fItree_name\fP ]
.SH DESCRIPTION
.I mklinks
is a program that creates and updates one or more \fBlocal\fP development trees
from a selection of one or more \fBremote\fP development trees.
The local development trees are described in a configuration file
using a hierarchical description language.  It is a language that allows
a user to specify a variety of attributes for the subtrees of the local tree,
including the remote trees used for update,
whether and how RCS (Revision Control System) is used, as well as
restrictions and exclusions of specific files and directories.
.PP
The \fImklinks\fP program can also masquerade as any of the RCS commands
to provide RCS label substitution and file name substitution.  See
the \fBRCS COMMANDS\fP section below.
.PP
For a given invocation of \fImklinks\fP, the
particular tree selected for update is determined by a number of factors:
.IP \(bu .2i
The command line and \fIthen\fP the environment
variable \fCMklinksTreeName\fP is examined for a tree name.
If a tree name is specified in either place, then no other tree
is considered.  If none is found, the closest match to all described trees
is made based on the current working directory.
.IP \(bu .2i
If no tree name is specified and the current directory implies a match
with two or more described trees, \fImklinks\fP will point this out and
then exit.
The mechanism for finding the correct match with a tree
is by "looking down" the from each tree's remote root
and "looking up" from the current working directory
until matching relative local and remote paths are found.
See \fBCONFIGURATION\fP for a definition of the remote root.
.IP \(bu .2i
If it appears that the current directory is a subtree of the local tree,
then only that subtree will be updated.
.IP \(bu .2i
If the configuration specifies a root directory for the
local tree, but the current working directory is not at or below that
node, then \fImklinks\fP changes the working directory to the top
of the local tree before updating.
.PP
See the \fBCONFIGURATION\fP section below for more details about the
update control provided by the configuration file.
.SH MKLINKS OPTIONS
.TP
.B \-1
Create/check-out only one level of the directory structure.
.TP
.B \-A
Ask for confirmation before executing any rcs command.
.TP
.B "\-Tname"
For consistency with the \fImklinks/RCS\fP commands, this option can also
be used to specify a tree name.  See the \fBRCS COMMANDS\fP section below.
.TP
.B \-V
Increase verbosity level of messages.
More -v flags increases message verbosity.  Note that if the -c or -l
options are not specified, a verbose level of one is forced.
.TP
.B \-c
Create a set of shell commands that would perform the update and
place them on the standard output.  No actual update will take place.
.TP
.B \-f
During an update, force a check on file's rcsid header information even if
the file modification times look correct.
.TP
.B \-l
List the file names that would be updated on the standard output,
without actually doing the update.
.TP
.B \-t
For debugging purposes, this option will cause \fImklinks\fP to
print out a descriptive listing of the tree selected for update,
and then exit without performing any update.
.SH CONFIGURATION
The configuration file may contain several directives.  At the top
level four things can be described: sets of subtrees to
be omitted, sets of restrictions for subtree contents, sets of trees
that specify local development trees (and may refer to the former two things),
and finally, the directory containing real rcs commands.  The following
syntax description is similar to a Backus Naur Form (BNF).  Keywords
are in quotes ("").
.PP
.ne 31

\fCConfiguration ::=
    "Omission" <OmitLabel> <OmitList>
  | "Restriction" <OnlyLabel> <OnlyList>
  | "Tree" <TreeLabel> <TreeSpecList>
  | "RcsBin" <absolute path>

TreeSpecList ::=
    <TreeSpec>
  | "{" <TreeSpec> "," ... "," <TreeSpec> "}"

TreeSpec ::=
    <absolute path> "-->" <relative path>
  | "Omit" <OmitItem> | <OmitList>
  | "Only" <OnlyItem> | <OnlyList>
  | "Include" <IncludeList>
  | "TreeRoot" <absolute path>
  | "RcsLabel" <string>
  | "RcsVersion" <version>
  | "DefaultRcsRevision"
  | "UseRcs"
  | "UseOnlyRcs"
  | "DropRcs"
  | "KeepRcsLink"
  | "Template"
  | "SubTree" <relative path> <TreeSpecList>

OmitItem ::= <relative path> | <OmitLabel>
.br
OmitList ::=
    "{" <OmitItem> "," ... "," <OmitItem> "}"
  | Subtree <relative path> OmitItem
  | Subtree <relative path> OmitList

OnlyItem ::= <relative path> | <OnlyLabel>
.br
OnlyList ::=
    "{" <OnlyItem> "," ... "," <OnlyItem> "}"
  | Subtree <relative path> OnlyItem
  | Subtree <relative path> OnlyList

IncludeList ::= <TreeName> | "{" <TreeName> "," ... "," <TreeName> "}" \fP
.PP
The description of the keywords and their affect on the description of
trees follows.
.IP "<absolute path> --> <relative path>" 1i
This specifies a mapping from a remote development tree (the absolute
path) to a local development tree (the relative path).  This means that
for the subtree under <relative path> files are obtained from the
corresponding <absolute path>.  The first mapping should be one of the
form <absolute path> "-->" "."; the "." is the top of the local tree,
and the absolute is the remote root.  See \fBRCS COMMANDS\fP below.
.IP \fCDropRcs\fP 1i
This directive clears all flags marked by the
\fCUseRcs,\fP \fCUseOnlyRcs,\fP \fCKeepRcsLink\fP,
\fCRcsLabel\fP and \fCRcsVersion\fP directives
in the current subtree.
This is also the default state if any of the latter directives
have not been specified.  The method for updating RCS and non-RCS
subtrees is described under \fBRCS COMMANDS\fP.
.IP \fCKeepRcsLink\fP 1i
When \fImklinks\fP updates nodes with RCS, it most often creates a
symbolic link in the local tree that points from "./RCS" to the RCS
directory in the remote tree, and then runs the RCS command \fIco\fP to
obtain the files; afterward the symbolic link is removed.  This
directive causes \fImklinks\fP to retain the symbolic link after the
update so that the user may peruse the remote RCS directory.  Note that
if the files in the directory do not all share the same remote RCS
directory, \fImklinks\fP will refuse to retain the symbolic link
because it would be ambiguous (in fact, \fImklinks\fP won't make the
link in the first place).  Also note that when \fImklinks\fP
masquerades as an RCS command, it never creates a symbolic link.
This directive also implies \fCUseRcs\fP.
.IP \fCInclude\fP 1i
This directive will logically "or" a source \fCTreeSpec\fP specified by the
\fCTreeName\fP with the current, or destination, \fCTreeSpec;\fP i.e., any
attribute in the source is added to the destination.  \fCRcsLabels\fP and
\fCRcsVersions\fP in the
source will override those in the destination.  Note that rcs labels and
rcs versions are recorded on the node for which they are specified.
Rcs labels are "inherited" by inferior nodes; hence, if a source tree
has an rcs label for the same node as the destination,
the entire subtree will be affected.  Rcs versions on the other hand
only affect the specified node; see RcsVersion below.
.IP \fCOmission\fP 1i
This directive creates a tree named by the \fCOmitLabel\fP and with the
specified contents, and all specified nodes are marked "omit".  That is, if
an \fCOmitLabel\fP is included in a \fCTreeSpec,\fP nodes marked "omit" will
not be considered for update.
.IP \fCOnly\fP 1i
This directive will mark specified relative paths as "only" in the
current \fCTreeSpec.\fP  Also, if an \fCOnlyLabel\fP is specified, all
nodes marked "only" in it will be marked "only" in the \fCTreeSpec\fP.
.IP \fCOmit\fP 1i
This directive will mark specified relative paths as "omit" in the
current \fCTreeSpec.\fP  Also, if an \fCOmitLabel\fP is specified, all nodes marked
"omit" in it will be marked "omit" in the \fCTreeSpec.\fP
.IP \fCRcsBin\fP 1i
This directive's argument identifies the path where the "real" RCS
commands are kept.  When a copy of \fImklinks\fP is executed as a
command named co, ci, rcs, rcsdiff, rcsmerge or rlog, \fImklinks\fP
will identify the tree to be updated, substitute labels and file names
as described elsewhere, and then execute the command of the same name
in the directory identified by \fCRcsBin.\fP  If \fCRcsBin\fP is not defined,
\fImklinks\fP will look for the second occurrence of the binary
according to the order of the shell execution path in the \fCPATH\fP
environment variable.  If two do not exist, it will use the first
binary.
.IP \fCRcsLabel\fP 1i
This directive's argument identifies RCS label to be used when
\fImklinks\fP obtains files, and when \fImklinks\fP masquerades as an
RCS command.  If the argument is
the reserved word DefaultRevision, then no RCS label will be specified
when \fImklinks\fP obtains files or masquerades as an RCS command.  This
implies that \fIco\fP will obtain the latest revision on the default
branch.  See \fBRCS COMMANDS\fP below.  This directive implies \fCUseRcs\fP.
.IP \fCRcsVersion\fP 1i
This directive's argument identifies RCS version number to be used when
\fImklinks\fP obtains a node, and when \fImklinks\fP masquerades as an
RCS command.  Note that it is not inherited as RcsLabel is, but is effective
only for the current node.  For example, to specify a version for foo.c
is can be done with "\fCSubTree foo.c RcsVersion 1.1\fP".
See \fBRCS COMMANDS\fP below.  This directive implies \fCUseRcs\fP for the node.
.IP \fCRestriction\fP 1i
This directive creates a tree named by the \fCOnlyLabel\fP and with the
specified contents, and all specified nodes are marked "only".  That is, if
an \fCOnlyLabel\fP is included in a \fCTreeSpec,\fP subtrees that contain
nodes marked "only" will only allow the "only" nodes to be considered
for update.  If an "only" node is also marked "omit" it will not be
considered for update.
.IP \fCSubTree\fP 1i
This directive allows a tree specification to be more succinct by
naming a subtree for which all enclosed specifications will be
interpreted relative to that subtree.  \fCSubTrees\fP may be nested.
.IP \fCTreeRoot\fP 1i
The current tree's local directory is rooted on the argument to this
directive.  For example, with a specification of "\fCTreeRoot\fP /usr/src",
the current tree's top level can only be located at /usr/src.  Without
a \fCTreeRoot\fP specification, the top level of the tree can be located
anywhere... this may be a blessing or a curse.
.IP \fCTree\fP 1i
This directive creates a tree named by the \fCTreeLabel\fP and with the
specified contents.  These trees are the ones that are used to
determine the local development trees to be updated.
.IP \fCTemplate\fP 1i
This directive marks the current tree as a template so that it will
never be considered when looking for a match between a local development tree
and a model from which to update. 
.IP \fCUseOnlyRcs\fP 1i
This directive restricts the actions of \fImklinks\fP for the current
subtree so that only RCS will be used to obtain files; no symbolic
links will be made for files.  This directive implies \fCUseRcs\fP.
.IP \fCUseRcs\fP 1i
This directive marks all existing nodes of the current subtree
such that updates will be performed with RCS commands.  If an \fCRcsLabel\fP
is not in effect for a subtree, then the default revision is used
(normally this is top-of-trunk).
.SH "MKLINKS EXECUTION"
When a subtree has \fInot\fP been marked with \fCUseRcs,\fP \fImklinks\fP
creates local directories but local files are
updated by creating symbolic links to the remote development tree.
This means the RCS directories are treated like any other.
.PP
When a subtree has been marked with \fCUseRcs,\fP RCS files are treated in a
special way.  In the following discussion, "workfile" and "rcsfile"
have the same meaning as described in the RCS\ documentation (see
\fIco(1)\fP).  When \fImklinks\fP executes (as itself) it is guided by
the following rules.
.IP \(bu .2i
The name RCS will never appear in the local tree (unless \fCKeepRcsLink\fP is
in effect for that subtree).
.IP \(bu .2i
If a file "x" exists in the remote subtree "foo", but "foo/RCS/x,v"
does not exist, "foo/x" will be updated using a symbolic link.  On the
other hand, if "foo/RCS/x,v" exists, \fImklinks\fP will create or
update the workfile "foo/x" with \fIco\fP.
.IP \(bu .2i
(If the -f option has been specified or) the modification date/time of
the local RCS workfile is older than the remote rcsfile's date/time,
the file will be considered "old".
.IP \(bu .2i
If the local work file is old and the date or RCS id number and date of
the latest revision in the remote rcsfile is not identical to that in
the $\&Header\&$ string of the local file, the file will be updated
using \fIco\fP.
.IP \(bu .2i
When \fIco\fP is invoked, the \fCRcsLabel\fP inherited by the current subtree
will be provided with the -r command line option.  The filenames
provided to \fIco\fP will be those implied by the rcsfiles discovered
in the RCS subdirectory.  If the label does not exist in the rcsfile, a
separate \fIco\fP command will be generated that will use the default
revision (usually the top of trunk).
.SH "RCS COMMANDS"
When \fImklinks\fP masquerades as \fIco\fP, \fIci\fP, \fIrcs\fP,
\fIrcsdiff\fP, \fIrcsmerge\fP or
\fIrlog\fP, it performs some command line substitution for certain flags and
arguments according to the rules below, and then executes the real
RCS command (see \fCRcsBin\fP above for a description of how it finds the real
command).
.IP \(bu .2i
For the \fIco\fP and \fIci\fP commands, each of the file arguments are
examined.  As described by the RCS documentation, the files may be
specified as a single workfile or a single rcs file or a
workfile/rcsfile pair.  If just a workfile is specified, its path (plus
the current working directory if it is not absolute) will be used to
identify the subtree of interest.  If an rcsfile or an rcsfile/workfile
pair is specified, the path of the rcsfile (plus the working directory)
will be used to identify the subtree of interest.  In all cases the
rcsfile will be changed (or added) so that it points to the remote
tree.
.IP \(bu .2i
For the \fIrcsdiff\fP, \fIrcsmerge\fP and \fIrlog\fP commands, each of
the file arguments are examined.  File names are substituted in the
same manner as with \fIco\fP and \fIci\fP, except that according to the
RCS documentation, an rcsfile/workfile pair may not be specified.
.IP \(bu .2i
For the \fIco\fP command the flags p, f, l, q, r and u are examined for
a revision argument.  If none of the flags specify the optional label
and an \fCRcsLabel\fP is in effect for the current subtree, the \fCRcsLabel\fP is
added to each flags.  If no flags are present, an r flag with the
\fCRcsLabel\fP is added to the command line.  This will force the version
specified by the \fCRcsLabel\fP to be extracted.  If the label does not exist
in the rcsfile, the \fIco\fP command will fail.
.IP \(bu .2i
For the \fIci\fP command the flags N, n, k, f, l, q, r and u are
examined for a revision argument.  If none of the flags specify the
optional label and an \fCRcsLabel\fP is in effect for the current subtree, an
N flag is added to the command line with the \fCRcsLabel.\fP  \fIWARNING: if
the label does not already exist, the added N flag will add it.\fP
This can be circumvented by specifying a revision number with one of
the above flags.
.IP \(bu .2i
For the \fIrcsdiff\fP and \fIrcsmerge\fP commands the r flags are
counted.  If rcsdiff has zero or one r flags, and an \fCRcsLabel\fP is in
effect for the current subtree, then an r flag with the \fCRcsLabel\fP is
added as the last flag.  If rcsmerge has no r flags, and an \fCRcsLabel\fP is
in effect for the current subtree, then an r flag with the \fCRcsLabel\fP is
added as the last flag.
.SH "RCS COMMAND OPTIONS"
In addition to the normal options to the RCS commands, \fImklinks\fP
will recognize three others when it masquerades as an RCS command.
.TP
.B \-A
Ask for confirmation before executing the rcs command.
.TP
.B "\-Tname"
This option specifies a tree name.  If one is not specified
\fCMklinksTreeName\fP is checked.  When masquerading, \fImklinks\fP
uses the same method for identifying the tree of interest as described
in the beginning of the man page.
.TP
.B \-V
Increase verbosity level of messages.  More -v flags increases message verbosity.
.SH "EXAMPLE CONFIGURATION FILE"
The following example demonstrates most of the features of \fImklinks\fP
based on an X development tree.  The items at the beginning are
basic definitions; the trees named XServer and MitSunServer and Clients
at the end define the labels that one might utter on the command line
or assing to the \fCMklinksTreeName\fP environment variable.

    \fC/*****************************************************************/
    /*                                                               */
    /*                        basic "omit" lists                       */
    /*                                                               */
    /*****************************************************************/

    Omission Doc     { mit/doc }
    Omission Contrib { mit/contrib }

    Omission PEX {
        SubTree mit/extensions {
            server/PEX,
            lib/PEX,
            include/PEX,
            test/PEX,
            test/InsPEX,
            fonts/PEX
        }
    }

    Omission Xinput {
        SubTree mit/extensions {
            lib/xinput,
            server/xinput,
            test/xinput
        }
    }

    /*****************************************************************/
    /*                                                               */
    /*                        basic "only" lists                     */
    /*                                                               */
    /*****************************************************************/
    Restriction BasicConfig {
        SubTree mit/config {
            Imake.rules,
            Imake.tmpl,
            Imakefile,
            Library.tmpl,
            Makefile.ini,
            Project.tmpl,
            README,
            Server.rules,
            Server.tmpl,
            Util.tmpl,
            ccimake.c,
            examples.cf,
            generic.cf,
            imake.c,
            imake.man,
            imakefile.man,
            site.def
        }
    }

    /*****************************************************************/
    /*                                                               */
    /* VanillaX gives you most sources, including server and clients */
    /*                                                               */
    /*****************************************************************/
    Tree VanillaX {
	Template,
              /epoch/X/X11/R4_exp --> . ,
        /jenzy/usr/src/video/MVEX --> mit/extensions/vex,
        Only mit,
        Omit { Doc, Contrib, PEX, Xinput }
    }

    /*****************************************************************/
    /*                                                               */
    /* XServer gives you just the sources related to the server      */
    /*                                                               */
    /*****************************************************************/
    Tree XServer {
        Include { VanillaX },
        SubTree mit {
            Only {
                Imakefile, Makefile,
                server, include, lib, config, fonts, util
            },
            SubTree extensions { Only { Imakefile, server, include } },
            SubTree lib { Only { Imakefile, Xdmcp } }
        }
    }

    /*****************************************************************/
    /*                                                               */
    /*               MitSunServer gives a buildable sun server       */
    /*                                                               */
    /*****************************************************************/
    Tree MitSunServer {
        Include { XServer },
        RcsLabel MITcurrent,
        Only { BasicConfig, mit/config/sun.cf },
	SubTree mit/lib/Xt/Display.c RcsVersion 1.4;
        Omit ddx/dec,

        /* this adds in the vex stuff to what's already there */
        SubTree mit/extensions {
            Only vex,
            DefaultRcsRevision
        }
    }\fP
    .SH FILES
/usr/local/lib/mklinks.config	The configuration file
.SH ENVIRONMENT VARIABLES
.TP
\fBMklinksTreeName\fP	Pathname of remote development directory.
.TP
\fBPATH\fP	Execution search path for the real rcs commands.
.SH SEE ALSO
\fIco(1)\fP,
\fIci(1)\fP,
\fIrcs(1)\fP,
\fIrcsdiff(1)\fP,
\fIrcsmerge(1)\fP
or \fIrlog(1)\fP
.SH COPYRIGHT
Copyright (c) 1988, 1989, 1990, 1991, Tektronix, Inc.
.SH AUTHOR
Todd Brunhoff; Video and Electronic Systems Lab; Tektronix, Inc.
