/* $Header: /usr/local/src/nvrToMIT/RCS/rcs.c,v 3.5 1994/12/17 12:55:11 russ Exp $ */
#include "struct.h"

typedef enum _revstate {
    NoLabels,
    VersionMissing,
    LabelMissing,
    FileCorrupt,
    RevFound
} RevState;

typedef enum _updateaction {
    FileIsOk,
    Update,
    UpdateUsingDefault
} UpdateAction;

static RevState GetRevisionInfo(sourceFile, revision, versionInfo)
    char    *sourceFile;
    RcsRevision    *revision;
    char    versionInfo[];
{
    char buf[ BUFSIZ ];
    register char *p, *date;
    static char symbolsString[] = "symbols";
    static char dateString[] = "date";
    int    i;
    RevState result = FileCorrupt;
    FILE    *fd;

    fd = fopen(sourceFile, "r");
    if (fd == NULL) {
        SyscallError("Can't open %s\n", sourceFile);
        goto done;
    }

    /*
     * First, look for the revision of interest.  This is
     *  - the head revision
     *  - a specific revision number
     *  - a symbolic revision name 'revision'
     */
    if (revision->type == VersionNumber)
	strcpy(versionInfo, revision->u.version);
    else if (revision->type == NamedLabel) {
        while (fgets(buf, BUFSIZ, fd)) {
            if (strncmp(buf, symbolsString, sizeof(symbolsString)-1) == 0)
                break;
            else if (*buf == '\n') {
		result = NoLabels;
		rewind(fd);
                goto LookForHead;
            }
        }

        /*
         * the line looks like:
         *    symbols  revname:3.1 anothername:4.6;
         */
        for (p=buf; *p; p++) {
            if ((p == buf || isspace(*(p-1)))
             && strncmp(p, revision->u.label, revision->stringLength) == 0
             && p[ revision->stringLength ] == ':')
                break;
	    if (*p == ';')
		break;
	    else if (*p == '\n') {
		if (fgets(buf, BUFSIZ, fd) == NULL) {
		    SyscallError("Unexpected EOF in %s\n", sourceFile);
		    break;
		}
		p = buf;
	    }
        }
        if (*p == ';') {
	    result = LabelMissing;
	    rewind(fd);
	    goto LookForHead;
        }

        p = p + revision->stringLength + 1;
        i = 0;
        while (!isspace(*p) && *p != ';') {
            if (*p == '\0') {
                ProgNameMsg("symbols field in %s missing ';'\n",
                    sourceFile);
                goto done;
            }
            versionInfo[ i++ ] = *p++;
        }
        versionInfo[ i ] = '\0';
    } else {
LookForHead:
        if (fscanf(fd, "head %s ;", versionInfo) != 1) {
	    ProgNameMsg("No head revision in %s\n", sourceFile);
            goto done;
        }
        versionInfo[strlen(versionInfo) - 1] = '\0';    /* remove the ";" */
    }

    while (fgets(buf, BUFSIZ, fd)) {
        buf[ strlen(buf) - 1 ] = '\0';        /* remove the \n */
        if (strncmp(versionInfo, buf, strlen(versionInfo)) == 0) {
            if (fgets(buf, BUFSIZ, fd) == NULL) {
                SyscallError("Unexpected EOF in %s\n", sourceFile);
                goto done;
            }
            if (strncmp(dateString, buf, sizeof(dateString)-1)) {
                ProgNameMsg("missing date for revision %s in %s\n",
                    versionInfo, sourceFile);
                goto done;
            }
	    p = buf+4;
            while (isspace(*p))
		p++;

            date = p;

	    /* find the year */
	    for (i = 0; *p != '.' && i < 4; p++, i++)
		;
	    if (i == 4) {
		/* we have found 19NN, change to NN */
	       date = date+1; /* include 1 space in what we copy */
	       *date = ' ';
	    }
	    else if (i == 2) {
	       date = date-1; /* include 1 space in what we copy */
	       *date = ' ';
	    }
	    else {
	       ProgNameMsg("Missing year in %s\n", sourceFile);
	       goto done;
	    }
        
            for(i=0; *p && *p != ';'; p++)
                if (*p == '.') {
                    if (++i <= 2)
                        *p = '/';
                    else if (i == 3)
                        *p = ' ';
                    else if (i > 3)
                        *p = ':';
                }
            *p = '\0';
            (void) strcat(versionInfo, date);
	    if (result != LabelMissing)
		result = RevFound;
            goto done;
        }
        else if (strcmp(buf, "desc") == 0) {    /* to far */
            ProgNameMsg ("Missing revision info in %s for %s\n",
                sourceFile, versionInfo);
            goto done;
        }
    }

    switch (revision->type) {
    case NamedLabel:	result = FileCorrupt; break;
    case VersionNumber:	result = VersionMissing; break;
    case DefaultRevision:	result = FileCorrupt; break;
    }

done:
    (void) fclose(fd);
    return (result);
}

static boolean GetFileVersionInfo(file, fileInfo)
    char    *file;
    char    fileInfo[];
{
    boolean result = FALSE;
    int     i;
    char    line[BUFSIZ],
	    date[BUFSIZ],
            *headerPtr,
            *versionPtr,
	    *datePtr;
    FILE    *fd;

    fd = fopen(file, "r");
    if (fd == NULL) {
        SyscallError("Can't open %s\n", file);
        return(FALSE);
    }

    while (fgets(line, BUFSIZ, fd)) {
        if (headerPtr = SubString(line, "$Header")) {
            if (sscanf(headerPtr, "$Header: %*s %s", fileInfo) < 1)
                return(0);
            versionPtr = fileInfo + strlen(fileInfo);
            *versionPtr++ = ' ';
#ifdef notdef
            (void) sscanf(headerPtr, "$Header: %*s %*s %s", versionPtr);
            versionPtr = fileInfo + strlen(fileInfo);
            *versionPtr++ = ' ';
            (void) sscanf(headerPtr, "$Header: %*s %*s %*s %s", versionPtr);
            result = TRUE;
            break;
#endif
	    (void) sscanf(headerPtr, "$Header: %*s %*s %s", date);
	    /* find the year and check for 19NN or NN */

	    for (i = 0, datePtr = date; *datePtr != '/' && i < 4; i++, datePtr++)
		;

	    if ((i == 4) && (*datePtr == '/')) {
		/* must change 19NN to NN */
		(void) strcpy(versionPtr, date+2);
	    }
	    else if (i == 2) {
		(void) strcpy(versionPtr, date);
	    }
	    else {
		ProgNameMsg("Can't find year in %s\n", file);
		return(FALSE);
	    }
	    versionPtr = fileInfo + strlen(fileInfo);
	    *versionPtr++ = ' ';
	    (void) sscanf(headerPtr, "$Header: %*s %*s %*s %s", versionPtr);
	    result = TRUE;
	    break;
        }
    }
    (void) fclose(fd);
    return (result);
}

static UpdateAction RcsFileIsUpToDate(node, revision, sourceFile)
    Node *node;
    RcsRevision *revision;
    char *sourceFile;
{
    int localFileType;
    char versionInfo[ BUFSIZ ];
    char fileInfo[ BUFSIZ ];
    char toPath[ BUFSIZ ];
    char warningBuf[ BUFSIZ ], *warning = warningBuf;
    RevState revState;

    if (node->parent->flags & NODE_NOTAVAILABLE)
	localFileType = 0;
    else {
	/* don't allow symbolic links for rcs files */
	localFileType = FileType(node->name);
	if (localFileType & NODE_SYMLINK) {
	    (void) UnlinkPath(node->name, FALSE);
	    localFileType = 0;
	}
	if (localFileType && localFileType != NODE_REGULAR) {
	    Verbose(1, ("%s is a %s: not updated\n",
		node->name, TypeName(localFileType)));
	    return (FileIsOk);
	}
    }

    if (localFileType == 0)
	Verbose(1, ("\t%s: non-existent\n", node->name));
    else if (!ForceCheck && ModTime(node->name) > ModTime(sourceFile))
	return (FileIsOk);

    /*
     * if a revision name was specified, we need to be sure its there,
     * otherwise 'co' will get upset. If the label is not there,
     * then get top of trunk.  If the revision name
     * wasn't even specified, and the local file doesn't exist, don't
     * bother evel looking at the rcs file.
     */
    if (revision->type == DefaultRevision && localFileType == 0)
	revState = RevFound;
    else
	revState = GetRevisionInfo(sourceFile, revision, versionInfo);

    switch (revState) {
    case FileCorrupt:
	return (FileIsOk);	/* don't try to update it. */
    case RevFound:
	*warning = '\0';
	break;
    case NoLabels:
	sprintf(warning, "\n\n%s in %s\nusing %s\n",
		sourceFile, versionInfo);
	break;
    case LabelMissing:
	sprintf(warning, "\n\nrcs label \"%s\" missing in %s\nusing %s\n",
		revision->u.label, sourceFile, versionInfo);
	break;
    case VersionMissing:
	Msg("\nWarning: version  \"%s\" missing in %s\n",
		revision->u.version, sourceFile);
	Msg("%s not obtained\n", node->name);
	return (FileIsOk);
    }

    if (localFileType) {
	if (!GetFileVersionInfo(node->name, fileInfo))
	    Verbose(1, ("\t%s: no RCS header\n", node->name));
	else if (strcmp(versionInfo, fileInfo) != 0)
	    Verbose(1, ("\t%s: %s\n", node->name, fileInfo));
	else
	    return (FileIsOk);
    }

    if (versionInfo[0])
	Verbose(1, ("%s\t%s: %s\n", warning, sourceFile, versionInfo));
    if (ListFiles)
	NormalOutput("checkout %s\n", NodeToPath(toPath, node, NullNode));

    return(revState == RevFound ? Update : UpdateUsingDefault);
}

/*
 * look upward in the tree for the revision info for this subtree.  Because
 * we are interested in the subtree, version numbers are ignored, because
 * they are useful only for an individual node.  Use GetNodeRevision()
 * to find the revision for an individual node.
 */
RcsRevision *GetSubtreeRevision(node)
    Node *node;
{
    while (node) {
	if (node->flags & NODE_REVISIONINFO)
	    if (node->data->rev.type != VersionNumber)
		return (&node->data->rev);
	node = node->parent;
    }
    return (&DefaultRcsRevision);
}

/*
 * Look upward from the specified node for its RcsRevision structure.
 */
RcsRevision *GetNodeRevision(node)
    Node *node;
{
    if (node->flags & NODE_REVISIONINFO)
	return (&node->data->rev);
    return (GetSubtreeRevision(node));
}

boolean CollectArguments(set, fromDir, revision, fileArg)
    Collection *set;
    char *fromDir;
    RcsRevision *revision;
    char *fileArg;
{
    int i;
    boolean dirIsUnique;
    RcsSource *src;

    /*
     * In some cases, the pointer that we record for the revision and
     * the fromDir will be used in subsequent calls.  If so, then these
     * tests will quickly fall through the loop, otherwise, we will end
     * up doing a string compare.
     */
    dirIsUnique = TRUE;
    for (i = 0, src = set->unique; i < set->nUnique; i++, src++) {
	if (src->fromDir == fromDir)
	    dirIsUnique = FALSE;
	else {
	    if (src->fromDir == NULL || fromDir == NULL)
		continue;
	    if (strcmp(src->fromDir, fromDir) == 0)
		dirIsUnique = FALSE;
	    else
		continue;
	}
	if (src->rev != revision) {
	    if (src->rev->type != revision->type)
		continue;

	    /*
	     * If its DefaultRevision, the label is (effectively) the same.
	     * Otherwise its either a VersionNumber or a NamedLabel:
	     * do a string compare.
	     */
	    if (revision->type != DefaultRevision
	     && strcmp(src->rev->u.label, revision->u.label) != 0)
		continue;
	}

	/*
	 * The revision and fromDir are the same.  Record the file here.
	 */
	AddArgv(&src->cmd, fileArg, TRUE);
	return (FALSE);
    }

    /*
     * Record the fromDir, the revision, and the fileArg.
     */
    if (++set->nUnique > set->nAllocated) {
	set->unique = (RcsSource *)
	    NullRealloc((char *)set->unique,
			(unsigned) (sizeof(RcsSource) * ++set->nAllocated));
        memset((char *) &set->unique[ set->nUnique - 1 ], 0, sizeof(RcsSource));
    }
    src = &set->unique[ set->nUnique - 1 ];
    src->rev = revision;
    src->fromDir = fromDir;
    (*RcsRewriteFunc)(set->id, &set->command, &src->cmd, revision);
    AddArgv(&src->cmd, fileArg, TRUE);

    return ((boolean) (fromDir != NULL && dirIsUnique));
}

/*
 * collect one more file in one of (possibly several) commands.  Note
 * that the parser guarantees that 'from' is either an RCS directory
 * or a file with the suffix ",v".  In both cases, it will be an
 * absolute path.
 *
 * Return true if this file introduces yet another source directory.
 */
static boolean CollectFiles(set, revision, from, toFile)
    Collection *set;
    RcsRevision *revision;
    char *from;		/* could be a file or directory */
    char *toFile;
{
    char *lastComponent, *fromDir, fromDirBuf[ BUFSIZ ];
    char *fileArg;
    int componentLen;
    boolean isRcsFile;

    lastComponent = from + strlen(from) - 1;
    isRcsFile = (boolean) (lastComponent[-1] == ',' && lastComponent[0] == 'v');

    fileArg = toFile;
    if (isRcsFile) {
	/*
	 * find the beginning of the filename component.
	 */
	componentLen = 0;
	while (*lastComponent != '/') {
	    lastComponent--;
	    componentLen++;
	}
	lastComponent++;

        /*
	 * If the 'toFile' is different than the 'from' file, then
	 * we must utter the from file explicitly on the command line.
	 * Hence, 'fromDir' will be null.
	 */
	if (strncmp(toFile, lastComponent, componentLen) != 0) {
	    fromDir = NULL;
	    fileArg = from;
	} else {
	    (void) strcpy(fromDir = fromDirBuf, from);
	    fromDir[ lastComponent - from - 1 ] = '\0';
	}
    } else
	fromDir = from;

    return( CollectArguments(set, fromDir, revision, fileArg) );
}

/*
 * 'list' contains all of the inferiors for a certain directory.
 * 'fromDir' is the parent of where most files should be obtained
 * (e.g. "fromDir/RCS/.").  This routine
 * extracts all files marked with NODE_ISRCSFILE.
 */
void ExtractRcsFiles(list, fromDir)
    NodeList *list;
    char    *fromDir;
{
    Node    *node, *parent = list->list[0]->parent;
    char    *sourceFile, sourceFileBuf[ BUFSIZ ];
    char    *rcsSource, treeRcsSource[ BUFSIZ ];
    RcsSource *src;
    RcsRevision *treeRevision, *revision;
    int     i, sourceDirCount;
    boolean filesToCheckOut;

    /*
     * Be sure we are all set for this pass
     */
    (*RcsInitFunc)(NULL, 0);

    /*
     * Figure out the implicit RCS source for this directory, and the
     * implicit rcs label.  These are used unless the file itself has
     * an explicit override for the label or source.
     */
    ConcatPath(treeRcsSource, fromDir, "RCS");
    treeRevision = GetSubtreeRevision(parent);

    sourceDirCount = 0;
    filesToCheckOut = FALSE;
    for (i=0; i < list->nNodes; i++) {
	node = list->list[ i ];

	if (node->flags & NODE_NEWSOURCE)
	    rcsSource = sourceFile = node->data->from;
	else {
	    sourceFile = sourceFileBuf;
	    rcsSource = treeRcsSource;
	    (void) sprintf(sourceFile, "%s/%s,v", treeRcsSource, node->name);
	}

	if (node->flags & NODE_REVISIONINFO)
	    revision = &node->data->rev;
	else
	    revision = treeRevision;

	switch (RcsFileIsUpToDate(node, revision, sourceFile)) {
	case FileIsOk:		continue;
	case Update:		break;
	case UpdateUsingDefault:	revision = &DefaultRcsRevision; break;
	}

	if (CollectFiles(&CommandSet, revision, rcsSource, node->name))
	    sourceDirCount++;
	filesToCheckOut = TRUE;
    }

    if (filesToCheckOut && ! ListFiles) {
	src = CommandSet.unique;
	for (i = 0; i < CommandSet.nUnique; i++, src++) {
	    if (src->fromDir
	     && ! MakeSymbolicLink(src->fromDir, "RCS")) {
		SyscallError("Can't make link for RCS\n");
		continue;
	    }
	    (void) Execute(&src->cmd, TRUE);
	}
    }
    if ((parent->flags & NODE_KEEPRCSLINK) == 0 || sourceDirCount > 1) {
	(void) UnlinkPath("RCS", FALSE);
	if (sourceDirCount > 1)
	    Msg("Warning: %s RCS link not retained (it would be ambiguous)\n",
		fromDir);
    } else {
	if (! MakeSymbolicLink(treeRcsSource, "RCS"))
	    SyscallError("Can't make link for RCS\n");
    }
}

boolean ReadRcsFiles(list, pathFrom)
    NodeList *list;
    char *pathFrom;
{
    char rcsDir[ BUFSIZ ], *p;
    int newListStart = list->nNodes;
    int i, j;
    Node *node, **nodeList;
    boolean rcsFilesPresent;

    ConcatPath(rcsDir, pathFrom, "RCS");
    if (ReadFiles(list, rcsDir, TRUE))
	Msg("Warning: %s/RCS exists, but will be ignored\n", rcsDir);
    nodeList = list->list;

    /*
     * trim off the ,v and mark them with NODE_ISRCSFILE
     */
    rcsFilesPresent = FALSE;
    for (i = newListStart; i < list->nNodes; i++) {
	node = nodeList[ i ];
	p = node->name + strlen(node->name) - 2;
	if (p[0] != ',' || p[1] != 'v') {
	    RemoveFromNodeList(list, i--);
	    continue;
	}
	p[0] = '\0';	/* remove the ,v */

	for (j = 0; j < newListStart; j++) {
	    if (SameNodeName(node, nodeList[ j ])) {
		RemoveFromNodeList(list, i--);
		node = nodeList[ j ];
		break;
	    }
	}

	node->flags |= NODE_ISRCSFILE;
	rcsFilesPresent = TRUE;
    }
    return (rcsFilesPresent);
}
