/* $Header: /usr/local/src/nvrToMIT/RCS/struct.h,v 3.16 1995/11/24 12:37:29 russ Exp $ */
#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/errno.h>

#ifdef sgi

#include <unistd.h>
#include <dirent.h>
#include <sys/wait.h>

typedef struct dirent    DirectoryEntry;
#define rindex    strrchr
char *getwd(), *getcwd(), *strchr();

#define ForkFunc	fork
typedef int		WaitStatus;

#endif /* sgi */

#if defined(_HPUX_SOURCE)

#include <dirent.h>
#include <sys/file.h>
#include <sys/wait.h>

typedef struct dirent    DirectoryEntry;
#define rindex    strrchr
char *getwd(), *getcwd(), *strchr();

#define ForkFunc	fork
typedef int		WaitStatus;

#endif /* _HPUX_SOURCE */

#ifdef sun

#include <unistd.h>
#ifndef X_OK
#define R_OK    4/* test for read permission */
#define W_OK    2/* test for write permission */
#define X_OK    1/* test for execute (search) permission */
#define F_OK    0/* test for presence of file */
#endif

char *getwd();

#include <sys/wait.h>
#define ForkFunc		vfork
# ifdef Solaris2
# include <dirent.h>
   typedef int			WaitStatus;
   typedef struct dirent	DirectoryEntry;
# else /* Solaris1 */
# include <sys/dir.h>
   union wait			WaitStatus;
   typedef struct direct	DirectoryEntry;
   typedef u_short		mode_t;
# endif

#endif /* sun */

#define ProcessExited(wstat)	WIFEXITED(wstat)
#define ProcessStopped(wstat)	WIFSTOPPED(wstat)
#define ProcessSignaled(wstat)	WIFSIGNALED(wstat)
#define ProcessSignal(wstat)	WTERMSIG(wstat)
#define ProcessExitCode(wstat)	WEXITSTATUS(wstat)

/*
 * probably just for 4319.
 */
#if !defined(isgraph) && !defined(Solaris2)
#define isgraph(c)      ((_ctype_+1)[c]&(_P|_U|_L|_N))
#endif

typedef u_char    boolean;

#define INCREMENT	100	/* a multiplier, when reallocating some things */
#define TRUE		((boolean)1)
#define FALSE		((boolean)0)
#define NullNode	((Node *)NULL)
#define NullString	((char *)NULL)
#define Verbose(level,args) \
		(level<=VerboseLevel) ? VerboseOutput args : VerboseLevel

/*
 * Flag values for Node.flags, below.
 */
#define NODE_ONLY	  0x0001  /* restrict consideration to this node */
#define NODE_HASONLY	  0x0002  /* asserted if some immediate inferiors */
				  /* are marked NODE_ONLY */
#define NODE_OMIT	  0x0004  /* do not consider this node */
#define NODE_HASOMIT	  0x0008  /* asserted if some immediate inferiors */
				  /* are marked NODE_OMIT */
#define NODE_NEWSOURCE	  0x0010  /* asserted if data->from is non-null */
#define NODE_REVISIONINFO 0x0020  /* asserted if data->rev is defined */
#define NODE_IS_TOP	  0x0040  /* true if node is "top" of the source tree*/
#define NODE_PERMANENT	  0x0080  /* asserted if the node should not be freed */
#define NODE_IS_ROOT	  0x0100  /* true if this is an absolute path */
#define NODE_NOTAVAILABLE 0x0200  /* true if the 'to' directory is not there */
#define NODE_DROPRCS	  0x0400  /* if true, rcs flags may not be added */
#define NODE_ISRCSFILE	  0x0800  /* true if node should be updated by rcs */
#define NODE_USERCS	  0x1000  /* true if rcs should be used for update */
#define NODE_USEONLYRCS	  0x2000  /* true if rcs should be used for update */
#define NODE_KEEPRCSLINK  0x4000  /* true if the link to the source rcs */
				  /* directory should be left in the tree */
#define NODE_USED	  0x8000  /* used as a temporary marker */

#define InheritedFlags(parentFlags)					\
	(((parentFlags) & NODE_DROPRCS) ?				\
	    ((parentFlags) & NODE_DROPRCS)				\
	  : ((parentFlags) & (NODE_USERCS|NODE_USEONLYRCS|NODE_KEEPRCSLINK)))

/*
 * These are for Node.type below, but note that the file types
 * are what the node *should* be; i.e. the file type is gotten from
 * the 'from' directory.
 */
#define NODE_DIRECTORY	0x0001	/* true if the node is a directory */
#define NODE_BLOCK	0x0002	/* true if the node is a block special file */
#define NODE_CHAR	0x0004	/* true if the node is a char special file */
#define NODE_REGULAR	0x0008	/* true if the node is a regular file */
#define NODE_FIFO	0x0010	/* true if the node is a fifo (sysv) */
#define NODE_SYMLINK	0x0020	/* true if the node is a symbolic link */
#define NODE_SOCKET	0x0040	/* true if the node is a socket (bsd) */

typedef struct _node Node;

/*
 * Used to describe revision information for a file.
 */
typedef enum _rcsrevisiontype {
    NamedLabel,
    VersionNumber,
    DefaultRevision
} RcsRevisionType;

typedef struct _rcsrevision {
    RcsRevisionType type;
    int stringLength;		/* length of label or version */
    union {
	char *label;
	char *version;
    } u;
} RcsRevision;

/*
 * Each node in the tree that has inferiors builds a list of them with
 * this structure.
 */
typedef struct _nodelist {
    int nNodes;		/* number of nodes in this list */
    Node **list;	/* this list of nodes */
} NodeList;

/*
 * if Node.data is non-null, it points to one of these.
 */
typedef struct _ndata {
    char *from;		/* source for this node and below */
    RcsRevision rev;	/* rcs revision info for this node, and those below */
    NodeList inferiors;	/* list of inferior nodes */
} NodeData;

/*
 * The directory tree itself is defined as a tree of these things.
 */
struct _node {
    char	*name;		/* name for this node in the tree */
    short	flags;		/* more information about the node; see above */
    short	type;		/* file type */
    Node	*parent;	/* parent node */
    NodeData	*data;		/* additional data */
};

/*
 * this hold a pointer to a single tree of nodes.  The organization
 * is always modeled after the destination tree (the tree to be built)
 * because mklinks can gather files from scattered locations.
 */
typedef struct _dirtree {
    int		label;		/* label for this tree */
    boolean	dontConsider;	/* true if we've decided this is not "it" */
/*  short	depthOfTop;	/* number of levels down to 'top' */
    Node	*root;		/* the tree's root */
    Node	*top;		/* subtree of interest (points into 'tree') */
    Node	*start;		/* node to start at (points into 'tree') */
    NodeList	from;		/* list of nodes with an explicit source */
    struct {
	char	*path;		/* non-null if the destination root specified */
	char	*realPath;	/* result of pwd while in 'path' */
	int	depth;		/* number of components in 'realPath' */
    } toRoot;
} DirTree;

/*
 * This is recorded by IsMappedInferior() to identify the starting point
 */
typedef struct _startingpoint {
    DirTree *tree;
    
    Node *node;		/* if the node is in the tree, this is it */
    char *pathTo;
    char *pathFrom;
    short treeTopIndex;
    short fileType;
} StartingPoint;

/*
 * This holds the complete set of trees, and as a pun, is called an Orchard.
 */
typedef struct _orchard {
    int nTrees;			/* length of mapList */
    DirTree	*treeList;	/* the complete list of trees */

    /*
     * after a tree has been selected, this is filled in with a pointer
     * to the start node.
     */
    int		choice;		/* index of the chosen tree */
    Node	*start;		/* place to start */
    StartingPoint **starts;	/* details for starting points */
    int		nStarts;	/* count of things in 'starts' */
} Orchard;

/*
 * Used for intermediate representation of a path before it becomes part
 * of a tree.
 */
typedef struct _treetmp {
    boolean absolute;
    int nNodes;
    char **list;
    char *original;
} TreeTmp;

/*
 * This is used to construct command lines
 */
#define ARGVINCREMENT	10	/* increment for argv */

typedef enum _programid {
    ProgMklinks    = 0,
    ProgHoldRcsDir = 1,
    ProgRcs        = 2,
    ProgRcsdiff    = 3,
    ProgRlog       = 4,
    ProgCo         = 5,
    ProgCi         = 6,
    ProgRcsmerge   = 7,
    ProgUnknown    = 8
} ProgramId;

typedef struct _argvec {
    char   *path;		/* pointer to arg1 in execv(arg1, arg2) */
    char   **argv;		/* pointer to arg2 in execv(arg1, arg2) */
    boolean *allocated;		/* true if corresp. cell in argv can be freed */
    int    index;		/* index to the last+1 argument */
    int    length;		/* actual length of the **argv element */
} Argvec;

typedef struct _rcssource {
    char *fromDir;		/* source directory; if none, then each file */
				/* must have its rcs source fully qualified */
    RcsRevision *rev;		/* rcs revision info for this node */
    Argvec cmd;			/* command to execute */
} RcsSource;

typedef struct _collection {
    Argvec command;		/* basic command to be executed */
    ProgramId id;		/* id of basic command */
    int	nUnique;		/* number of unique command lines */
    int nAllocated;		/* number of RcsSource structs allocated */
    RcsSource *unique;		/* the command lines themselves */
} Collection;

/*
 * Returned by IdentifyInitialSource()
 */
typedef enum _sourcestate {
    AmbiguousTopDir,
    AmbiguousSubDir,
    UniqueSourceFound,
    ImplicitRootDir,
    NoSourceFound
} SourceState;

/*
 * Function declarations
 */
Node *AddPathMapToLabel();
Node *AddPathMapToTree();
Node *NewNode();
DirTree *LabelToTree();
RcsRevision *GetNodeRevision();
RcsRevision *GetSubtreeRevision();
SourceState IdentifyInitialSource();
TreeTmp *PathToTreeTmp();
boolean ChangeDir();
boolean ChangeNode();
boolean CollectArguments();
boolean Confirm();
boolean IsInferior();
boolean MakeDirectory();
boolean MakeSymbolicLink();
boolean ReadFiles();
boolean ReadRcsFiles();
boolean ReadRcsFiles();
boolean ReadTreeSpec();
boolean SameFile();
boolean SameNodeName();
boolean UnlinkPath();
boolean VerboseOutput();
char *ComposeRcsFile();
char *CopyOfString();
char *ExpandTreeStack();
char *NodeToPath();
char *NullRealloc();
char *PathsIntersect();
char *SecureAlloc();
char *SubString();
char *TokenIndexToString();
char *TypeName();
int DepthOfNode();
int StringToTokenIndex();
int TokenIndexToType();
int Execute();
struct stat *StatFile();
void AddArgv();
void AddFlagsFromLabel();
void AddFmtArgv();
void AddFromPath();
void AddInferior();
void AddNodeData();
void AddPathToLabel();
void AddToNodeList();
void AmbiguousPath();
void CombineTreeLabels();
void ConcatPath();
void ConfigError();
void DropRcs();
void SyscallError();
void ExtractRcsFiles();
void Fatal();
void FreeArgv();
void FreeNode();
void GetRealRoot();
void GetInferiors();
void KeepRcsLink();
void Msg();
void NewTree();
void NormalOutput();
void PopSubTree();
void ProgNameMsg();
void PushSubTree();
void RemoveFromNodeList();
void SetRcsRevision();
void SetTreeRootToString();
void ShowTrees();
void TrimArgv();
void UseOnlyRcs();
void UseRcs();
void WalkTree();
void LinkTree();

#if !defined(sgi) && !defined(Solaris2) && !defined(_HPUX_SOURCE)
char *malloc();
char *realloc();
int free();
char *getenv();
char *strcpy();
char *strncpy();
char *strcat();

int mkdir();
int lstat();
int stat();
int readlink();
int symlink();
int close();
void exit();
#endif


/*
 * global flags.
 */
extern boolean CreateCommands;
extern boolean ListFiles;
extern boolean VerboseLevel;
extern boolean AskToExecute;
extern boolean ForceCheck;
extern boolean OneLevel;
extern boolean ShowTreeChoice;

extern char *RcsBin;
extern char *ProgramName;
extern char *MklinksTreeName;
extern char *ProgramPrefix;
extern Orchard Farm;
extern Collection CommandSet;
extern void (*RcsInitFunc)();
extern void (*RcsRewriteFunc)();
extern RcsRevision DefaultRcsRevision;
