/* $Header: /usr/local/src/nvrToMIT/RCS/util.c,v 3.12 1995/04/12 22:50:05 russ Exp $ */
#include "struct.h"
#include "y.tab.h"

extern int    errno;

/*VARARGS1*/
void SyscallError(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9)
    char    *fmt;
{
    extern char    *sys_errlist[];

    (void) Msg("%s: ", ProgramName);
    if (errno) {
        (void) Msg("%s: ", sys_errlist[ errno ]);
	errno = 0;	/* don't report the same error twice */
    }
    (void) Msg(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
}

/*VARARGS1*/
void Fatal(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9)
    char    *fmt;
{
    SyscallError(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
    exit(1);
}

/*VARARGS1*/
void ProgNameMsg(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9)
    char    *fmt;
{
    (void) Msg("%s: ", ProgramName);
    (void) fprintf(stderr, fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
    (void) fflush(stderr);
}

/*VARARGS1*/
void Msg(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9)
    char    *fmt;
{
    (void) ChangeNode(NullNode, TRUE);
    (void) fprintf(stderr, fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
    (void) fflush(stderr);
}

/*VARARGS1*/
boolean VerboseOutput(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9)
    char    *fmt;
{
    Msg(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
    return(TRUE);;
}

/*VARARGS1*/
void NormalOutput(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9)
    char    *fmt;
{
    (void) ChangeNode(NullNode, TRUE);
    (void) printf(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
    (void) fflush(stdout);
}

int Execute(argv, showExitValue)
    Argvec    *argv;
    boolean showExitValue;
{
    int    i, pid, wpid, exitValue;
    WaitStatus wstatus;

    Verbose(1, ("(%s) ", argv->path));
    for (i=0; i<argv->index; i++)
	Msg("%s ", argv->argv[ i ]);
    Msg("\n");
    if (CreateCommands || (AskToExecute && ! Confirm("Execute?")))
	return;

    pid = ForkFunc();
    if (pid < 0)
	Fatal("can't fork\n");
    if (pid == 0) { /* child */
	(void) execv(argv->path, argv->argv);
	Fatal("Can't exec \"%s\"\n", argv->path);
    }

    exitValue = 0;
    for (;;) {
        wpid = wait(&wstatus);
        if (wpid < -1) {
            SyscallError("Error while waiting for \"%s\"\n",
                argv->path);
            break;
        }
	if (ProcessExited(wstatus)) {
	    if (ProcessExitCode(wstatus) != 0) {
		if (showExitValue)
		    ProgNameMsg("\"%s\": Process exit code=%d\n",
			argv->path, ProcessExitCode(wstatus));
		exitValue = 1;
	    }
	}
	else if (ProcessStopped(wstatus)) {
	    ProgNameMsg("\"%s\": Process stopped with signal %d\n",
		argv->path, ProcessSignal(wstatus));
	    exitValue = 1;
	} else if (ProcessSignaled(wstatus)) {
	    ProgNameMsg("\"%s\": Process exit with signal %d\n",
		argv->path, ProcessSignal(wstatus));
	    exitValue = 1;
	}

	if (wpid == pid)
	    break;
    }
    return (exitValue);
}

void AddArgv(argv, arg, makeCopy)
    Argvec  *argv;
    char    *arg;
    boolean makeCopy;
{
    if (argv->index == argv->length) {
        argv->length += ARGVINCREMENT;
        argv->argv = (char **)
	    NullRealloc((char *) argv->argv,
			(unsigned) (sizeof(char *) * (argv->length + 1)));
        argv->allocated = (boolean *)
	    NullRealloc((char *) argv->allocated,
			(unsigned) (sizeof(boolean *) * (argv->length + 1)));
    }

    if (makeCopy) {
	argv->allocated[ argv->index ] = TRUE;
	argv->argv[ argv->index++ ] = CopyOfString(arg);
    } else {
	argv->allocated[ argv->index ] = FALSE;
	argv->argv[ argv->index++ ] = arg;
    }
    argv->argv[ argv->index ] = NullString;
}

/*VARARGS2*/
void AddFmtArgv(argv, fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9)
    Argvec    *argv;
    char    *fmt;
{
    char arg[ BUFSIZ ];

    (void) sprintf(arg, fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
    AddArgv(argv, arg, TRUE);
}

void TrimArgv(argv, length)
    Argvec    *argv;
{
    int    i;
    for (i = length; i < argv->index; i++) {
	if (argv->allocated[ i ])
	    free(argv->argv[i]);
    }
    argv->index = length;
}

void FreeArgv(argv)
    Argvec    *argv;
{
    TrimArgv(argv, 0);
}

/*
 * Obtain the files in a directory; return true if one of them is "RCS".
 */    
boolean ReadFiles(list, pathFrom, useRcs)
    NodeList *list;
    char *pathFrom;
    boolean useRcs;
{
    boolean hasRcsNode = FALSE;
    DIR    *dirfd;
    DirectoryEntry *retval;
    int nodeFlags = useRcs ? NODE_USERCS : 0;
    char rcsDirectory[ BUFSIZ ];

    dirfd = opendir(pathFrom);
    if (dirfd == NULL) {
	Msg("Can't open %s!\n", pathFrom);
	return (FALSE);
    }

    while (retval = readdir(dirfd)) {
        if (strcmp(retval->d_name, ".") == 0
         || strcmp(retval->d_name, "..") == 0)
            continue;
        Verbose(4, ("readdir()=%s\n", retval->d_name));

	if (useRcs && strcmp(retval->d_name, "RCS") == 0) {
	    ConcatPath(rcsDirectory, pathFrom, "RCS");
	    if (FileType(rcsDirectory) == NODE_DIRECTORY) {
		hasRcsNode = TRUE;
		continue;
	    }
	}

	AddToNodeList(list, NewNode(retval->d_name, nodeFlags));
    }

    closedir(dirfd);

    return (hasRcsNode);
}

char *TypeName(type)
    int    type;
{
    switch (type) {
    case NODE_DIRECTORY:
	return "directory";
    case NODE_CHAR:
	return "character special file";
    case NODE_BLOCK:    
	return "block special file";
    case NODE_REGULAR:  
	return "regular file";
    case NODE_SYMLINK:  
	return "symbolic link pointing nowhere";
    case NODE_SOCKET:   
	return "socket";
    case NODE_SYMLINK | NODE_DIRECTORY:
	return "symbolic link pointing to a directory";
    case NODE_SYMLINK | NODE_CHAR:
	return "symbolic link pointing to a character special file";
    case NODE_SYMLINK | NODE_BLOCK:
	return "symbolic link pointing to a block special file";
    case NODE_SYMLINK | NODE_REGULAR:
	return "symbolic link pointing to a regular file";
    case NODE_SYMLINK | NODE_SOCKET:
	return "symbolic link pointing to a socket";
    case 0:
	return "non-existent file";
    default:
	return "unknown file type";
    }
}

struct stat *StatFile(func, path)
    int (*func)();
    char *path;
{
    static char lastLstatPath[ BUFSIZ ];
    static char lastStatPath[ BUFSIZ ];
    static struct stat lstatBuf, statBuf;
    struct stat *st;
    char *lastPath;

    if (path == NullString) {
	lastLstatPath[0] = '\0';
	lastLstatPath[0] = '\0';
	return(NULL);
    }

    if (func == lstat) {
	lastPath = lastLstatPath;
	st = &lstatBuf;
    } else {
	lastPath = lastStatPath;
	st = &statBuf;
    }

    if (strcmp(path, lastPath) == 0)
	return (st);
    if ((*func)(path, st) < 0) {
	if (errno == ENOENT)
	    errno = 0;
	return (NULL);
    }
    strcpy(lastPath, path);
    return (st);
}

FileSize(fd)
    int fd;
{
    struct stat    st;
    int    result;

    result = fstat(fd, &st);
    if (result < 0) {
        errno = 0;
        return 0;
    }
    return(st.st_size);
}

ModTime(path)
    char    *path;
{
    struct stat    *st;

    if (st = StatFile(stat, path))
	return(st->st_mtime);
    return (0);
}

static StatModeToFileType(path, mode)
    char *path;
    mode_t mode;
{
    switch (mode) {
    case S_IFDIR: return (NODE_DIRECTORY);
    case S_IFCHR: return (NODE_CHAR);
    case S_IFBLK: return (NODE_BLOCK);
    case S_IFREG: return (NODE_REGULAR);
    case S_IFIFO: return (NODE_FIFO);
    case S_IFLNK: return (NODE_SYMLINK);
    }

    Fatal("file \"%s\" type 0x%x\n", path, mode);
    /*NOTREACHED*/
}

boolean SameFile(st1, st2)
    struct stat *st1;
    struct stat	*st2;
{
    return((boolean) (st2->st_dev == st1->st_dev && st2->st_ino == st1->st_ino));
}
    
FileType(path)
    char    *path;
{
    struct stat    *st;
    int    type;

    if ((st = StatFile(lstat, path)) == NULL)
	return(0);

    type = StatModeToFileType(path, st->st_mode & S_IFMT);
    if (type & NODE_SYMLINK) {
	if ((st = StatFile(stat, path)) == NULL)
	    return(type);
	type |= StatModeToFileType(path, st->st_mode & S_IFMT);
    }
    return (type);
}

permof(path)
    char    *path;
{
    struct stat    *st;

    if ((st = StatFile(stat, path)) == NULL)
        Fatal("can't get perm for %s\n", path);
    return(st->st_mode & (~S_IFMT));
}

void ConcatPath(fullPath, root, dir)
    char    *fullPath, *root, *dir;
{
    char    *p;

    p = fullPath;
    if (strcmp(root, ".") == 0) {
	*p = '\0';
    } else {
	/*
	 * don't bother copying if they are the same
	 */
	if (fullPath != root)
	    (void) strcpy(p, root);
	p += strlen(p);
    }

    if (*fullPath == '\0' || strcmp(dir, ".") != 0) {
        if (*fullPath && p[-1] != '/')
            *p++ = '/';
        (void) strcpy(p, dir);
        p += strlen(p);
    }
}

/*
 * Test if path1 and path2 intersect.  Path1 is assumed less than or equal in
 * length to path2.  E.g.
 *	PathsIntersect("usr", "usr/tmp", 4) returns "tmp"
 *	PathsIntersect(".", "usr/tmp", 4) returns "usr/tmp"
 */
char *PathsIntersect(path1, path2, path2Length)
    char *path1;
    char *path2;
    int path2Length;
{
    int path1Length;

    if (path1[0] == '.' && path1[1] == '\0')
	return(path2);

    path1Length = strlen(path1);
    if (path1Length > path2Length)
	return(NullString);

    if (strncmp(path1, path2, path1Length) == 0) {
	path2 += path1Length;
        if (*path2 == '/')
	    return (path2+1);
	if (*path2 == '\0')	/* the paths are identical */
	    return (path2);
    }

    return (NullString);
}

/*
 * Generate a path that can be used to go from the "current" working 
 * directory to the "destination" directory.
 */
static char *ChdirPath(destination, current)
    Node *destination, *current;
{
    static char pathTo[ BUFSIZ ];

    /*
     * if 'current' is an inferior of 'destination' then all that is necessary
     * is to generate a number of '..' components.
     */
    if (IsInferior(destination, current)) {
	pathTo[ 0 ] = '\0';
	while (current != destination) {
	    (void) ConcatPath(pathTo, pathTo, "..");
	    current = current->parent;
	}
    }

    /*
     * If, on the other hand, 'destination' is an inferior of 'current', then
     * we simply need to descend to that directory.
     */
    else if (IsInferior(current, destination)) {
	(void) NodeToPath(pathTo, destination, current);
    }

    /*
     * The nodes are in different subtrees.  Find a common root, and
     * then generate the path.
     */
    else {
	Node *root;
	char descent[ BUFSIZ ];

	root = current;
	pathTo[ 0 ] = '\0';
	while ( ! IsInferior(root, destination)) {
	    (void) ConcatPath(pathTo, pathTo, "..");
	    if ((root = root->parent) == NULL)
		Fatal("no common node!\n");
	}
	(void) ConcatPath(pathTo,
			  pathTo,
			  NodeToPath(descent, destination, root));
    }

    return (pathTo);
}

boolean ChangeDir(path)
    char *path;
{
    (void) StatFile(NULL, NullString);
    return ((boolean)(chdir(path) >= 0));
}

boolean ChangeNode(node, complain)
    Node    *node;
    boolean complain;
{
    static Node *lastNode, *lastPrintedNode;
    static int printed;
    char path[ BUFSIZ ], *pPath;

    if (node == NULL) {
        printed++;
        if (printed == 1 && lastPrintedNode != lastNode && lastNode) {
	    pPath = ChdirPath(lastNode, lastPrintedNode);
	    if (CreateCommands)
		NormalOutput("cd %s\n", pPath);
            Verbose(1, ("chdir(%s)\n", pPath));
	    lastPrintedNode = lastNode;
	}
        return(TRUE);
    }

    if (lastNode == NULL) {	/* first time */
	lastPrintedNode = lastNode = node;
    } else {
	printed = 0;
	if (IsInferior(node, lastNode)) {
	    while (lastNode != node) {
		if ((lastNode->flags & NODE_NOTAVAILABLE) == 0
		 && ! ChangeDir(".."))
		    SyscallError("can't get back to .. (%s)\n",
			NodeToPath(path, node, NullNode));
		lastNode = lastNode->parent;
	    }
	} else {
	    (void) NodeToPath(path, node, lastNode);
	    lastNode = node;
	    if ((node->parent->flags & NODE_NOTAVAILABLE)
	     || ! ChangeDir(path)) {
		if (complain) {
		    SyscallError("cannot chdir(%s)\n",
			NodeToPath(path, node, NullNode));
		    if (! CreateCommands)
			printed++;
		}
		node->flags |= NODE_NOTAVAILABLE;
		return(FALSE);
	    }
	}
    }
    return(TRUE);
}

/*
 * returns a pointer within s1 that contains the string s2.
 * e.g. SubString("fee, fie, foe, fum", "foe") returns pointer
 * starting at "foe, fum".
 */
char * SubString(s1, s2)
    char *s1, *s2;
{
    int n = strlen(s2);
    while (*s1) {
        if (strncmp(s1, s2, n) == 0)
            return (s1);
        s1++;
    }
    return (NullString);
}

GetLinkName(path, buf, size)
    char    *path, *buf;
    int    size;
{
    int    cnt;

    cnt = readlink(path, buf, size);
    if (cnt < 0) {
        SyscallError("Cannot readlink(%s,0x%x,%d)\n",
            path, buf, size);
        return(FALSE);
    }
    if (cnt >= size)
        Fatal("Oops: readlink() returned %d bytes\n", cnt);
    buf[ cnt ] = '\0';
    return(TRUE);
}

boolean MakeSymbolicLink(from, to)
    char    *from, *to;
{
    char currentLink[ BUFSIZ ];
    int nameLength;

    if (CreateCommands) {
	NormalOutput("rm -f %s; ln -s %s %s\n", to, from, to);
    } else if (ListFiles) {
	NormalOutput("link     %s\n", to);
    } else {
	if ((nameLength = readlink(to, currentLink, BUFSIZ)) < 0) {
	    if (errno == EINVAL) {	/* its not a symbolic link */
		SyscallError("%s is not a symbolic link!\n", to);
		return (FALSE);
	    }
	    if (errno != ENOENT)
		Fatal("cant readlink(%s)\n", to);
	    errno = 0;
	} else {
	    currentLink[ nameLength ] = '\0';
	    if (strcmp(currentLink, from) == 0)
		return (TRUE);	/* it is already correct */
	    (void) unlink(to);
	}
	if (symlink(from, to) < 0) {
	    SyscallError("Cannot symlink(%s, %s)\n", from, to);
	    return (FALSE);
	}
    }
    return (TRUE);
}

boolean MakeDirectory(path, mode)
    char *path;
    int mode;
{
    if (CreateCommands) {
	NormalOutput("mkdir %s; chmod 0%o %s\n", path, mode, path);
    } else if (ListFiles) {
	NormalOutput("mkdir    %s\n", path);
    } else {
	if ( *path && mkdir(path, mode) < 0) {
	    SyscallError("Cannot mkdir(%s)\n", path);
	    return (FALSE);
	}
    }
    return (TRUE);
}

boolean UnlinkPath(path, complain)
    char *path;
    boolean complain;
{
    if (CreateCommands) {
	NormalOutput("rm -f %s\n", path);
    } else {
	if (unlink(path) < 0 && complain) {
	    SyscallError("Cannot mkdir(%s)\n", path);
	    return (FALSE);
	}
	errno = 0;
    }
    return (TRUE);
}

char *SecureAlloc(size)
    int size;
{
    char *ptr = malloc(size == 0 ? 1 : size);

    if (ptr == NullString)
        Fatal("can't allocate memory\n");
    return (ptr);
}

char *NullRealloc(ptr, size)
    char *ptr;
    unsigned size;
{
    if (ptr == NullString)
        ptr = malloc(size);
    else
	ptr = realloc(ptr, size);
    if (ptr == NullString)
	Fatal("can't allocate memory\n");
    return (ptr);
}

char *CopyOfString(string)
    char *string;
{
    char *newString = SecureAlloc(strlen(string)+1);

    (void) strcpy(newString, string);
    return (newString);
}

char *NodeToPath(path, node, cwd)
    char *path;
    Node *node;
    Node *cwd;
{
    char *psrc, *pdst;
    static level = 0;

    if (node->parent == NULL || node == cwd) {
	pdst = path;
	if (node->flags & NODE_IS_ROOT)
	    *pdst++ = '/';
	else if (level == 0)
	    *pdst++ = '.';
    } else {
	level++;
	pdst = NodeToPath(path, node->parent, cwd);
	psrc = node->name;
	while (*psrc)
	    *pdst++ = *psrc++;
	if (--level > 0)
	    *pdst++ = '/';
    }

    *pdst = '\0';
    if (level > 0)
	return(pdst);
    else
	return(path);
}

TreeTmp *PathToTreeTmp(path)
    char *path;
{
    TreeTmp *new;
    int i, nNodes, pathLength = 1;
    char *p = path;

    /*
     * for now, ignore whether there is a leading '/'
     */
    if (*p == '/') {
	p++;
	pathLength++;
    }

    if (*p) {
	nNodes = 1;
	while (*p) {
	    pathLength++;
	    if (*p++ == '/')
		nNodes++;
	}
    } else
	nNodes = 0;

    /*
     * Allocate space in one fell swoop.
     */
    new = (TreeTmp *)SecureAlloc(sizeof(TreeTmp)
			  + sizeof(char *) * nNodes
			  + pathLength);
    new->original = path;
    new->absolute = ((boolean) (*path == '/'));
    new->nNodes = nNodes;
    new->list = (char **)(new + 1);
    p = (char *)(new->list + nNodes);
    (void) strcpy(p, path);
    if (*p == '/')
	p++;
    for (i = 0; i < nNodes; i++) {
	new->list[ i ] = p;
	while (*p && *p != '/')
	    p++;
	*p++ = '\0';
    }

    return(new);
}

int DepthOfNode(node)
    Node *node;
{
    int depth = 0;

    while (node->parent) {
	node = node->parent;
	depth++;
    }
    return (depth);
}

boolean IsInferior(parent, inferior)
    Node *parent;
    Node *inferior;
{
    while (inferior) {
	if (inferior == parent)
	    return (TRUE);
	inferior = inferior->parent;
    }
    return (FALSE);
}

DirTree *LabelToTree(label)
    int label;
{
    int i;

    for (i=0; i < Farm.nTrees; i++) {
	if (label == Farm.treeList[ i ].label)
	    return(&Farm.treeList[ i ]);
    }
    Fatal("Can't find tree label\n");
    /*NOTREACHED*/
}

void NewTree(label)
    int label;
{
    int i = Farm.nTrees++;

    Farm.treeList = (DirTree *)	
	NullRealloc((char *) Farm.treeList,
		    (unsigned) (sizeof(DirTree) * Farm.nTrees));
    memset(Farm.treeList + i, 0, sizeof (DirTree));
    Farm.treeList[ i ].label = label;

    /*
     * Omit trees are not intended as a source tree, so don't search
     * them as a potential match for the initial source tree.
     */
    switch (TokenIndexToType(label)) {
    case OMITLABEL:
    case ONLYLABEL:
	Farm.treeList[ i ].dontConsider = TRUE;
	break;
    }
}

/*
 * NewNode() and FreeNode() provide the memory management for
 * the Nodes allocated and freed.
 */
Node **NodeSets;
int NodeSetCount;
Node **FreeList;
int FreeNodeCount;
int FreeListLength;

Node *NewNode(name, flags)
    char *name;
    int flags;
{
    int i;
    Node *list, *newNode;

    if (FreeNodeCount == 0) {
	NodeSets = (Node **)
	    NullRealloc((char *)NodeSets,
			(unsigned) (sizeof(Node *) * ++NodeSetCount));
	list = (Node *)SecureAlloc(sizeof(Node) * INCREMENT);
	NodeSets[ NodeSetCount-1 ] = list;
	if (FreeListLength == 0) { /* first time */
	    FreeList = (Node **)SecureAlloc(sizeof(Node *) * INCREMENT);
	    FreeListLength = INCREMENT;
	}
	for (i=0; i<INCREMENT; i++)
	    FreeList[i] = &list[i];
	FreeNodeCount = INCREMENT;
    }
    newNode = FreeList[ --FreeNodeCount ];
    memset(newNode, 0, sizeof(Node));
    newNode->name = CopyOfString(name);
    newNode->flags = flags;

    return (newNode);
}

void FreeNode(node)
    Node *node;
{
    if (node->name)
	free(node->name);
    if (node->data)
	free((char *)node->data);
    if (++FreeNodeCount > FreeListLength) {
	FreeListLength += INCREMENT;
	FreeList = (Node **)NullRealloc((char *) FreeList,
					(unsigned) FreeListLength * INCREMENT);
    }
    FreeList[ FreeNodeCount-1 ] = node;
}

/*
 * AddToNodeList() and handle the list maintenaince
 * for nodes.
 */
void AddToNodeList(list, node)
    NodeList *list;
    Node *node;
{
    list->list = (Node **)
	NullRealloc((char *) list->list,
		    (unsigned) (++list->nNodes * sizeof(Node *)));
    list->list[ list->nNodes - 1 ] = node;
}

void RemoveFromNodeList(list, nodeIndex)
    NodeList *list;
    int nodeIndex;
{
    int i;
    Node **nodeList = list->list;

    /*
     * don't bother reallocating the list, unless the count goes to zero...
     * then we free it.
     */ 
    FreeNode(nodeList[ nodeIndex]);
    if (--list->nNodes == 0) {
	free((char *)nodeList);
	return;
    }

    for (i = nodeIndex; i <= list->nNodes; i++)
	nodeList[ i ] = nodeList[ i+1 ];
}

void AddNodeData(node)
    Node *node;
{
    node->data = (NodeData *) SecureAlloc(sizeof(NodeData));
    memset(node->data, 0, sizeof (NodeData));
}
    
void AddInferior(parent, inferior)
    Node *parent;
    Node *inferior;
{
    if (parent->data == NULL)
	AddNodeData(parent);
    AddToNodeList(&parent->data->inferiors, inferior);
    inferior->parent = parent;
}

boolean SameNodeName(node1, node2)
    Node *node1, *node2;
{
    return (strcmp(node1->name, node2->name) == 0);
}

/*VARARGS1*/
boolean Confirm(msg, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9)
    char *msg;
{
    char buf[ BUFSIZ ], *p;

    for (;;) {
	Msg(msg, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
	if (gets(buf) == NullString)
	    break;
	for (p = buf; isspace(*p); p++)
	    ;
	switch (*p) {
	case 'y':
	    return(TRUE);

	case 'n':
	    return(FALSE);

	default:
	    Msg("enter y or n; ");
	    break;
	}
    }

    Msg("no.\n");	/* only get here on an eof */
    return (FALSE);
}

#if defined(SYSV) || defined(Solaris2)
/*
 *  getwd():  Remap BSD getwd() to POSSIX getcwd().
 *
 *  getcwd(buf, size)
 *  char   *buf;
 *  int    size;
 *
 *  This assumes that cwd is of size BUFSIZ.
 *
 */
char *getwd(cwd)
char    *cwd;
{
    char    *getcwd();

    return (getcwd(cwd, BUFSIZ));
}
#endif /* SYSV */

static void ShowTree(node)
    Node *node;
{
#define Comma	flagCount++ > 0 ? ", " : ""
    char path[ BUFSIZ ];
    int flagCount = 0, i;
    Node **nodeList;

    (void) printf("    ");
    (void) printf("%s\t", NodeToPath(path, node, NullNode));

    if (node->flags & NODE_ONLY)
	(void) printf("%sonly", Comma);
    if (node->flags & NODE_HASONLY)
	(void) printf("%shasonly", Comma);
    if (node->flags & NODE_OMIT)
	(void) printf("%somit", Comma);
    if (node->flags & NODE_HASOMIT)
	(void) printf("%shasomit", Comma);
    if (node->flags & NODE_NEWSOURCE)
	(void) printf("%snewsrc", Comma);
    if (node->flags & NODE_REVISIONINFO) {
	(void) printf("%srcsvers=", Comma);
	switch(node->data->rev.type) {
	case DefaultRevision:
	    (void) printf("default-revision");
	    break;
	case NamedLabel:
	case VersionNumber:
	    (void) printf("%s", node->data->rev.u.label);
	    break;
	}
    }
    if (node->flags & NODE_IS_TOP)
	(void) printf("%stop", Comma);
    if (node->flags & NODE_PERMANENT)
	(void) printf("%sperm", Comma);
    if (node->flags & NODE_IS_ROOT)
	(void) printf("%sisroot", Comma);
    if (node->flags & NODE_ISRCSFILE)
	(void) printf("%srcs file", Comma);
    if (node->flags & NODE_USERCS)
	(void) printf("%suse %srcs", Comma,
			(node->flags & NODE_USEONLYRCS) ? "only " : "");
    if (node->flags & NODE_KEEPRCSLINK)
	(void) printf("%skeep rcs links", Comma);
    if (node->flags & NODE_DROPRCS)
	(void) printf("%sdrop rcs", Comma);
    if (node->type & NODE_DIRECTORY)
	(void) printf("%sdir", Comma);
    if (node->type & NODE_BLOCK)
	(void) printf("%sblock", Comma);
    if (node->type & NODE_CHAR)
	(void) printf("%schar", Comma);
    if (node->type & NODE_REGULAR)
	(void) printf("%sreg", Comma);
    if (node->type & NODE_FIFO)
	(void) printf("%sfifo", Comma);
    if (node->type & NODE_SYMLINK)
	(void) printf("%ssymlink", Comma);
    if (node->type & NODE_SOCKET)
	(void) printf("%ssocket", Comma);
    if (node->flags & NODE_NEWSOURCE)
	(void) printf("\n\tsrc=%s", node->data->from);
    (void) printf("\n");
    if (node->data) {
        nodeList = node->data->inferiors.list;
        for (i = node->data->inferiors.nNodes - 1; i >= 0; i--)
            ShowTree(nodeList[ i ]);
    }
#undef Comma
}

/*
 * This is mostly for debugging
 */
void ShowTrees(showChoiceOnly)
    boolean showChoiceOnly;
{
    int i;
    char startPath[ BUFSIZ ];
    DirTree *tree;

    for (i = 0; i < Farm.nTrees; i++) {
	if (showChoiceOnly && i != Farm.choice)
	    continue;
	tree = & Farm.treeList[ i ];
	(void) printf("%s#%d %s: ", i == Farm.choice ? "*" : " ",
			     i, TokenIndexToString(tree->label));
	if (i == Farm.choice)
	    (void) printf("start = \"%s\" ",
		NodeToPath(startPath, Farm.start, NullNode));
	if (tree->dontConsider)
	    (void) printf("rejected,");
	if (tree->toRoot.path) {
	    (void) printf("\n    root=%s\n", tree->toRoot.path);
	    if (tree->toRoot.path != tree->toRoot.realPath)
		(void) printf("    (really %s)\n", tree->toRoot.realPath);
	}
	(void) printf("\n");
	if (tree->root)
	    ShowTree(tree->root);
	else
	    ShowTree(tree->top);
	(void) printf("\n");
    }
}

void AmbiguousPath(state, cwd)
    SourceState state;
    char *cwd;
{
    int i, labelsPrinted;

    switch (state) {
    case AmbiguousTopDir:
	Msg("%s appears to be the top level directory for the labels\n", cwd);
	labelsPrinted = 0;
	for (i = 0; i < Farm.nTrees; i++)
	    if (! Farm.treeList[ i ].dontConsider)
		Msg("%s%s", labelsPrinted++ ? ", " : "",
			    TokenIndexToString(Farm.treeList[ i ].label));
	Msg("\n");
	break;

    case AmbiguousSubDir:
	Msg("%s is an ambiguous subdirectory for the following labels\n", cwd);
	for (i = 0; i < Farm.nStarts; i++) {
	    Msg("    %s fits \"%s\"\n",
		Farm.starts[i]->pathFrom,
		TokenIndexToString(Farm.starts[i]->tree->label));
	}
	break;
    }

    Msg("Pick one, specify it on the command line");
    if (CommandSet.id != ProgMklinks)
	Msg(" with -T");
    Msg(", or in the\n");
    Msg("environment variable \"%s\", and try again\n", MklinksTreeName);
}

char *ComposeRcsFile(buf, dir, workfile)
    char *buf;
    char *dir;
    char *workfile;
{
    if (dir)
	(void) sprintf(buf, "%s/RCS/%s,v", dir, workfile);
    else
	(void) sprintf(buf, "RCS/%s,v", workfile);
    return (buf);
}
