/*
 * $Header: /usr/local/src/nvrToMIT/RCS/rcsit.c,v 1.17 1995/04/12 23:02:56 russ Exp $
 * $Locker:  $
 */
#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/wait.h>
#include <sys/uio.h>

#if defined(sgi) || defined(_HPUX_SOURCE)

#define ForkFunc	fork
typedef int		WaitStatus;
#define USEGETCWD

#endif

#ifdef sun

#include <unistd.h>
#include <sys/wait.h>
#define ForkFunc	vfork
# ifdef Solaris2
#define USEGETCWD
#  include <fcntl.h>
   typedef int		WaitStatus;
# else
#  include <sys/dir.h>
   typedef union wait	WaitStatus;
# endif

#endif

#define ProcessExited(wstat)	WIFEXITED(wstat)
#define ProcessStopped(wstat)	WIFSTOPPED(wstat)
#define ProcessSignaled(wstat)	WIFSIGNALED(wstat)
#define ProcessSignal(wstat)	WTERMSIG(wstat)
#define ProcessExitCode(wstat)	WEXITSTATUS(wstat)

#define    TRUE    1
#define    FALSE    0
#define    CC_HEADER	0    /* c-source (use static var for $Header) */
#define    CCNO_HEADER	1    /* c-source language file */
#define    TROFF_HEADER	2    /* troff source file */
#define    SH_HEADER	3    /* bourne- or c-shell source */
#define    ASM_HEADER	4    /* assembly language source */
#define    CPLUS_HEADER	5    /* C++ language source */

#define    BINDIRECTORY	""
typedef u_char    boolean;
typedef struct _file {
    char    *buf;
    int    size;
} File;

boolean InsertNothing = FALSE;
boolean InsertLog = FALSE;
boolean InsertLocker = FALSE;
boolean InsertHeader = TRUE;
boolean LockIt = FALSE;
int     DefaultHeaderType = CCNO_HEADER;
char    *Program;
char    *ArgvRevNumber;
char    *CommentString = " * ";
char    *HeaderString;

boolean GoodRevNumber();
boolean MakeDirectory();
boolean Exists();
boolean IsAFile();
void    RcsIt();
void    Error();
void    Fatal();
void    Usage();
void    GotoDirectory();
void    AlreadyRCSed();
void    NotAFile();
void    PrintHeader();
char    *Concat();
char    *GetLine();
char    *FindSubString();
char    *GetRevNumber();
File    *GetFile();
int     SetHeader();

extern char    *malloc();
extern int    errno;

main(argc, argv)
    int    argc;
    char    **argv;
{
    int    filecnt = 0;
    boolean    showHeader = FALSE;

    Program = *argv;
    for (argv++, argc--; argc; argv++, argc--) {
	if (argv[0][0] == '-' || argv[0][0] == '+') {
	    if (strcmp(argv[0], "-troff") == 0) {
		DefaultHeaderType = TROFF_HEADER;
	    }
	    else if (strcmp(argv[0], "-cc") == 0) {
		DefaultHeaderType = CC_HEADER;
	    }
	    else if (strcmp(argv[0], "-ccno") == 0) {
		DefaultHeaderType = CCNO_HEADER;
	    }
	    else if (strcmp(argv[0], "-sh") == 0
		  || strcmp(argv[0], "-csh") == 0
		  || strcmp(argv[0], "-make") == 0) {
		DefaultHeaderType = SH_HEADER;
	    }
	    else if (strcmp(argv[0], "-asm") == 0) {
		DefaultHeaderType = ASM_HEADER;
	    }
	    else if (strcmp(argv[0], "-c++") == 0) {
		DefaultHeaderType = CPLUS_HEADER;
	    }
	    else if (strcmp(argv[0], "-l") == 0)
		LockIt = TRUE;
	    else if (strcmp(argv[0]+1, "log") == 0)
		InsertLog = (argv[0][0] == '+');
	    else if (strcmp(argv[0]+1, "locker") == 0)
		InsertLocker = (argv[0][0] == '+');
	    else if (strcmp(argv[0]+1, "header") == 0)
		InsertHeader = (argv[0][0] == '+');
	    else if (strcmp(argv[0], "-n") == 0)
		InsertNothing = TRUE;
	    else if (strcmp(argv[0], "-s") == 0)
		showHeader = TRUE;
	    else if (argv[0][1] == 'r') {
		if (! GoodRevNumber(argv[0]+2))
		    Fatal("\"%s\" is an incorrect %s\n",
			argv[0]+2, "revision number");
		ArgvRevNumber = argv[0]+2;
	    } else
		Usage();
	} else {
	    filecnt++;
	    if (showHeader)
		PrintHeader(argv[0]);
	    RcsIt(argv[0]);
	}
    }
    if (InsertNothing && (InsertLocker || InsertLog))
	Fatal("the -n flag negates the +locker and +log flag.\n");
    if (showHeader)
	PrintHeader(NULL);
    else if (filecnt == 0)
	Usage();

    exit(0);
}

void RcsIt(file)
    char    *file;
{
    char    *directory;
    char    *rcs1, *rcs2;
    char    *newfile;
    char    *revNumber, *revComment;
    char    command[ BUFSIZ ];
    File    *filep = NULL;
    int    fd, wrote;
    int    headerType;
    boolean useFileInfo;

    GotoDirectory(&file, &directory);

    if (! Exists(file)) {
	Error("%s/%s doesn't exist.\n", directory, file);
	return;
    }

    if (! IsAFile(file)) {
	Error("%s/%s is not a plain file\n", directory, file);
	return;
    }

    rcs1 = Concat(file, ",v", 0);
    rcs2 = Concat("RCS/", file, ",v", 0);
    newfile = Concat("#", file, 0);

    if (Exists(rcs1)) {
	if (IsAFile(rcs1))
	    AlreadyRCSed(directory, file, rcs1);
	else
	    NotAFile(directory, file, rcs1);
	goto allDone_lockit;
    }
    errno = 0;
    if (Exists(rcs2)) {
	if (IsAFile(rcs2))
	    AlreadyRCSed(directory, file, rcs2);
	else
	    NotAFile(directory, file, rcs2);
	goto allDone_lockit;
    }

    if (!MakeDirectory("RCS"))
	goto allDone;

    if ((filep = GetFile(file, directory)) == NULL)
	goto allDone;

    if ((fd = openrw(newfile)) < 0) {
	free(filep);
	goto allDone;
    }

    headerType = SetHeader(filep, file, directory,
			   &revNumber, &revComment, &useFileInfo);
    if (!InsertNothing) {
	/*
	 * If there is a #! then skip it.
	 */
	if (headerType == SH_HEADER
	 && *filep->buf == '#') {
	    char    *line = GetLine(filep);
	    write(fd, line, strlen(line));
	}
	write(fd, HeaderString, strlen(HeaderString));
    }

    wrote = write(fd, filep->buf, filep->size);
    if (wrote != filep->size) {
	Error("Can't write entire file into %s/%s.\n",
	    directory, newfile);
	Error("%s/%s not rcs'ed.\n", directory, file);
	goto allDone;
    }
    close(fd);

    if (rename(newfile, file) < 0) {
	Error("Can't rename %s to %s\n", newfile, file);
	goto allDone;
    }

    if (useFileInfo)
	sprintf(command, "%sci -k %s%s",
	    BINDIRECTORY,
	    LockIt ? "" : "-u ",
	    file);
    else
	sprintf(command, "%sci -r%s %s%s",
	    BINDIRECTORY,
	    revNumber,
	    LockIt ? "" : "-u ",
	    file);

    if (! run(command, revComment))
	goto allDone;

    if (headerType == TROFF_HEADER
     || headerType == SH_HEADER
     || headerType == ASM_HEADER) {
	sprintf(command, "%srcs -c\'%s\' %s",
	    BINDIRECTORY, CommentString, file);
	if (! run(command, NULL))
	    goto allDone;
    }

allDone_lockit:
    if (LockIt) {
	sprintf(command, "%sco -l %s", BINDIRECTORY, file);
	if (! run(command, NULL))
	    goto allDone;
    }

allDone:
    if (filep)
	free(filep);
    free(newfile);
    free(rcs1);
    free(rcs2);
}

run(command, input)
    char    *command;
    char    *input;
{
    char   buf[ BUFSIZ ], *pbuf;
    char   *argv[ 100 ], *pcmd;
    int    pipefd[2];
    int    pid, wpid;
    int    wrote, i;
    char   quote;
    WaitStatus	status;

    i = 0;
    pcmd = command;
    pbuf = buf;
    argv[i++] = pbuf;
    while (*pcmd) {
	if (*pcmd == '"' || *pcmd == '\'') {
	    quote = *pcmd++;
	    while (*pcmd && *pcmd != quote)
		*pbuf++ = *pcmd++;
	    if (*pcmd == '\0')
		Fatal("Missing quote (%c) in\n\t%s\n", quote, command);
	    pcmd++;
	} else if (*pcmd == ' ') {
	    *pbuf++ = '\0';
	    while (isspace(*pcmd))
		pcmd++;
	    printf("%s ", argv[i-1]);
	    argv[i++] = pbuf;
	} else
	    *pbuf++ = *pcmd++;
    }
    *pbuf = '\0';
    printf("%s\n", argv[i-1]);
    argv[i] = NULL;

    if (pipe(pipefd) < 0)
	Fatal("can't make a pipe\n");
    pid = ForkFunc();
    if (pid < 0)
	Fatal("can't fork \n");
    if (pid == 0) {    /* child */
	close(pipefd[1]);
	dup2(pipefd[0], 0);
	close(pipefd[0]);
	execvp(argv[0], argv);
	Fatal("Can't exec \"%s\"\n", command);
    } else {
	struct iovec iovp[ 4 ];
	int vectors = 0;

	close(pipefd[0]);
	if (input) {
	    iovp[ vectors ].iov_base = "Original version.\n.\n";
	    iovp[ vectors ].iov_len = strlen(iovp[ vectors ].iov_base);
	    if (input && *input) {
		iovp[ ++vectors ].iov_base = "Original header:\n    ";
		iovp[ vectors ].iov_len = strlen(iovp[ vectors ].iov_base);
		iovp[ ++vectors ].iov_base = input;
		iovp[ vectors ].iov_len = strlen(input);
	    }
	    iovp[ ++vectors ].iov_base = "\n.\n";
	    iovp[ vectors ].iov_len = 3;
	    wrote = writev(pipefd[1], iovp, vectors);
	} else
	    wrote = 0;
	close(pipefd[1]);
    }

    wpid = 0;
    while (wpid != pid) {
	wpid = wait(&status);
	if (wpid < -1) {
	    Error("Error while waiting for \"%s\"\n", command);
	    return(FALSE);
	}
	if (ProcessExited(status) && ProcessExitCode(status) != 0) {
	    Error("\"%s\": exit 0x%x\n", command, ProcessExitCode(status));
	    return(FALSE);
	}
    }
    if (wrote < 0) {
	Error("Error writing to \"%s\"\n", command);
	return(FALSE);
    }
    return(TRUE);
}

File *GetFile(file, directory)
    char    *file,
	*directory;
{
    int    fd, got, size;
    File    *filep;

    if ((fd = open(file, O_RDONLY)) < 0) {
	Error("Can't open \"%s/%s\" for reading\n", directory, file);
	return(NULL);
    }
    
    size = filesize(file);
    filep = (File *)malloc(size+1+sizeof(File));
    filep->size = size;
    filep->buf = (char *)filep + sizeof(File);
    filep->buf[ size ] = '\0';
    got = read(fd, filep->buf, size);
    close(fd);
    if (got != filep->size) {
	Error("Can't read all of \"%s/%s\"\n", directory, file);
	free(filep);
	filep = NULL;
    }
    return(filep);
}

void AlreadyRCSed(dir, rcsfile)
    char    *rcsfile,
	*dir;
{
    Error("%s/%s has already been rcs'ed.\n", dir, rcsfile);
}

void NotAFile(dir, file, rcsfile)
    char    *dir,
	*file,
	*rcsfile;
{
    Error("%s/%s already exists but is not a file.\n", dir, rcsfile);
    Error("%s/%s not rcs'ed.\n", dir, file);
}

boolean MakeDirectory(name, cwd)
    char    *name;
    char    *cwd;
{
    struct stat st;

    if (stat(name, &st) < 0) {
	if (mkdir(name, 0777) < 0) {
	    Fatal("Can't mkdir(%s)\n", name);
	    return(FALSE);
	}
	return(TRUE);
    }
    errno = 0;
    if ((st.st_mode & S_IFMT) != S_IFDIR) {
	Error("\"%s/%s\" exists but is not a directory\n",
	    cwd, name);
	return(FALSE);
    }
    return(TRUE);
}

char *WorkingDirectory()
{
    static char    *cwd, *wdReturn;
    char    buf[ BUFSIZ ];

    if (cwd == NULL) {
#ifdef USEGETCWD
	char *getcwd();
	wdReturn = getcwd(buf, BUFSIZ);
#else
	char *getwd();
	wdReturn = getwd(buf);
#endif
	if (wdReturn == NULL)
	    Fatal("can't get current working directory: %s\n", buf);
	cwd = malloc(strlen(buf) + 1);
	strcpy(cwd, buf);
    }
    return (cwd);
}

void GotoDirectory(pfile, pdir)
    char    **pfile,
	**pdir;
{
    char    *file;
    for (file = *pfile + strlen(*pfile) - 1; file >= *pfile; file--) {
	if (*file == '/') {
	    *pdir = *pfile;
	    *file++ = '\0';
	    *pfile = file;
	    if (**pdir != '/')
		if (chdir(WorkingDirectory()) < 0)
		    Fatal("can't chdir(%s)\n", WorkingDirectory());
	    if (chdir(*pdir) < 0)
		Fatal("can't chdir(%s)\n", *pdir);
	    return;
	}
    }
    *pdir = ".";
}

boolean GoodRevNumber(revNumber)
    char    *revNumber;
{
    while (*revNumber) {
	if (!isdigit(*revNumber) && *revNumber != '.')
	    return(FALSE);
	revNumber++;
    }

    return(TRUE);
}

void Usage()
{
    fprintf(stderr, "Usage: %s [-cc|-ccno|-make|-sh|-make|-clu] \\\n",
	Program);
    fprintf(stderr, "    [-l] [-s] [-n] [-r#] [+log|-log]");
    fprintf(stderr, " [+locker|-locker] [+header|-header] files...\n");
    fprintf(stderr, "Where\n");
    fprintf(stderr, "      -cc\tmeans c-source language file");
    fprintf(stderr, " (use static var for $Header)\n");
    fprintf(stderr, "      -ccno\tmeans c-source language file");
    fprintf(stderr, " (no static variables)\n");
    fprintf(stderr, "      -troff\tmeans troff source file\n");
    fprintf(stderr, "      -sh\tmeans bourne- or c-shell source file\n");
    fprintf(stderr, "      -make\tmeans Make source (same as -sh)\n");
    fprintf(stderr, "      -asm\tmeans assembly language source file\n");
    fprintf(stderr, "      -c++\tmeans C++ language source file\n");
    fprintf(stderr, "      -l\tmeans to lock the file (a la ``co -l'')\n");
    fprintf(stderr, "      -s\tmeans to show the header to be inserted\n");
    fprintf(stderr, "      -n\tmeans don't insert any keywords ");
    fprintf(stderr, "and try to use existing\n\t\trevision numbers\n");
    fprintf(stderr, "      -r#\tmeans use revision number #\n");
    fprintf(stderr, "      +/-log\tmeans to add/don't add ");
    fprintf(stderr, "$Log to the file\n");
    fprintf(stderr, "      +/-locker\tmeans to add/don't add ");
    fprintf(stderr, "$Locker to the file\n");
    fprintf(stderr, "      +/-header\tmeans to add/don't add ");
    fprintf(stderr, "$Header to the file\n");
    fprintf(stderr, "As a side effect, -s with no file argument will ");
    fprintf(stderr, "print the header to be used\nwith respect to the ");
    fprintf(stderr, "given flags.  Also note that all flags may be\n");
    fprintf(stderr, "mixed with the file arguments.\n");
    exit (1);
}

void Error(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9)
    char    *fmt;
{
    extern char    *sys_errlist[];

    fprintf(stderr, "%s: ", Program);
    if (errno) {
	fprintf(stderr, "%s: ", sys_errlist[ errno ]);
	errno = 0;
    }
    fprintf(stderr, fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
}

void Fatal(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9)
    char    *fmt;
{
    Error(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
    exit(1);
}

char *Concat(a0, a1, a2, a3, a4, a5, a6, a7, a8, a9)
    char    *a0, *a1, *a2, *a3, *a4, *a5, *a6, *a7, *a8, *a9;
{
    char    *argbuf[11], **args = argbuf;
    char    buf[ BUFSIZ ];
    char    *p;

    args[0] = a0; args[1] = a1; args[2] = a2; args[3] = a3; args[4] = a4;
    args[5] = a5; args[6] = a6; args[7] = a7; args[8] = a8; args[9] = a9;

    buf[0] = '\0';
    while (*args) {
	strcat(buf, *args);
	args++;
    }
    p = malloc(strlen(buf)+1);
    strcpy(p, buf);
    return(p);
}

filesize(file)
    char    *file;
{
    struct stat st;

    if (stat(file, &st) < 0)
	return(0);
    return(st.st_size);
}

boolean IsAFile(file)
    char    *file;
{
    struct stat st;

    if (stat(file, &st) < 0)
	return(FALSE);
    return ((st.st_mode & S_IFMT) == S_IFREG);
}

boolean Exists(file)
    char    *file;
{
    struct stat st;

    if (stat(file, &st) < 0) {
	errno = 0;
	return(FALSE);
    }
    return(TRUE);
}

openrw(newfile)
    char    *newfile;
{
    int    fd;

    fd = open(newfile, O_CREAT|O_EXCL|O_WRONLY, 0666);
    if (fd < 0)
	Error("Can't open %s for writing\n", newfile);
    return(fd);
}

char *GetLine(filep)
    File    *filep;
{
    char    *p;
    static char buf[ BUFSIZ ];

    p = buf;
    while (filep->size) {
	filep->size--;
	if ((*p++ = *filep->buf++) == '\n')
	    break;
    }
    *p = '\0';
    return(buf);
}

int pickHeaderType(file)
    char *file;
{
    char *suffix;

    for (suffix = file + strlen(file) - 1; *suffix != '.'; suffix--)
	if (suffix <= file)
	    return(DefaultHeaderType);

    if (strcmp(suffix, ".c") == 0)
	return(DefaultHeaderType == CC_HEADER
	    || DefaultHeaderType == CCNO_HEADER
		? DefaultHeaderType : CCNO_HEADER);
    if (strcmp(suffix, ".sh") == 0)
	return(SH_HEADER);
    if (strcmp(suffix, ".man") == 0)
	return(TROFF_HEADER);
    if (strcmp(suffix, ".1") == 0)
	return(TROFF_HEADER);
    if (strcmp(suffix, ".troff") == 0)
	return(TROFF_HEADER);
    if (strcmp(suffix, ".s") == 0)
	return(ASM_HEADER);
    if (strcmp(suffix, ".C") == 0)
	return(CPLUS_HEADER);
    if (strcmp(suffix, ".h") == 0)
	return(CCNO_HEADER);
    if (strcmp(suffix, ".H") == 0)
	return(CCNO_HEADER);
    if (strcmp(suffix, ".y") == 0)
	return(CCNO_HEADER);
    if (strcmp(suffix, ".l") == 0)
	return(CCNO_HEADER);
    return(DefaultHeaderType);
}

int SetHeader(filep, file, directory, revNumber, revComment, useFileInfo)
    File    *filep;
    char    *file, *directory;
    char    **revNumber, **revComment;
    boolean *useFileInfo;
{
    boolean    insertLog = InsertLog;
    boolean    insertLocker = InsertLocker;
    boolean    insertHeader = InsertHeader;
    static char buf[ BUFSIZ ];
    char    *headerLine;
    char    *lockLogLine;
    char    *p;
    int     headerType = pickHeaderType(file);

    HeaderString = headerLine = "";
    
    if (InsertLog && FindSubString("$Log:", filep->buf))
	insertLog = FALSE;
    if (InsertLocker && FindSubString("$Locker:", filep->buf))
	insertLocker = FALSE;

    /*
     * Always look for revision number so that we know whether to add
     * a Header string or not.
     */
    *revComment = "";
    *revNumber = "1.1";
    *useFileInfo = FALSE;
    if (p = FindSubString("$Header:", filep->buf)) {
	insertHeader = FALSE;
	if (p = GetRevNumber(p, revComment)) {
	    if (ArgvRevNumber) {
		*revNumber = ArgvRevNumber;
		*useFileInfo = FALSE;
	    } else {
		*revNumber = p;
		*useFileInfo = TRUE;
		Error("%s/%s: Using revision %s found in $Header\n",
		    directory, file, *revNumber);
	    }
	} else {
	    Error("%s/%s has $Header but no revision #\n",
		directory, file);
	}
    }

    /*
     * Set up the header strings for each individual file type
     */
    switch (headerType) {
    case CC_HEADER:
	CommentString = " * ";
	headerLine = "static char *rcsid=\"@Header@\";\n";
	if (insertLog && insertLocker)
	    lockLogLine = "%s/* @Locker@\n * \n * @Log@\n */\n";
	else if (insertLog)
	    lockLogLine = "%s/*\n * @Log@\n */\n";
	else if (insertLocker)
	    lockLogLine = "%s/* @Locker@ */\n";
	else
	    lockLogLine = "%s";
	break;
    case CCNO_HEADER:
	CommentString = " * ";
	if (insertHeader)
	    headerLine = " * @Header@\n ";
	if (insertLog && insertLocker)
	    lockLogLine = "/*\n%s* @Locker@\n * \n * @Log@\n */\n";
	else if (insertLog)
	    lockLogLine = "/*\n%s* @Log@\n */\n";
	else if (insertLocker)
	    lockLogLine = "/*\n%s* @Locker@\n */\n";
	else
	    lockLogLine = "/* @Header@ */\n";
	break;
    case CPLUS_HEADER:
	CommentString = "// ";
	if (insertHeader)
	    headerLine = "// @Header@\n";
	if (insertLog && insertLocker)
	    lockLogLine = "%s// @Locker@\n//\n// @Log@\n //\n";
	else if (insertLog)
	    lockLogLine = "%s//\n// @Log@\n//\n";
	else if (insertLocker)
	    lockLogLine = "%s//\n// @Locker@\n//\n";
	else
	    lockLogLine = "%s";
	break;
    case TROFF_HEADER:
	CommentString = ".\\\"";
	if (insertHeader)
	    headerLine = ".\\\" @Header@\n";
	if (insertLog && insertLocker)
	    lockLogLine = "%s.\\\" @Locker@\n.\\\"\n.\\\" @Log@\n.\\\"\n";
	else if (insertLog)
	    lockLogLine = "%s.\\\"\n.\\\" @Log@\n.\\\"\n";
	else if (insertLocker)
	    lockLogLine = "%s.\\\" @Locker@\n";
	else
	    lockLogLine = "%s";
	break;
    case ASM_HEADER:
	CommentString = "; ";
	if (insertHeader)
	    headerLine = "; @Header@\n";
	if (insertLog && insertLocker)
	    lockLogLine = "%s; @Locker@\n;\n; @Log@\n;\n";
	else if (insertLog)
	    lockLogLine = "%s;\n; @Log@\n;\n";
	else if (insertLocker)
	    lockLogLine = "%s; @Locker@\n";
	else
	    lockLogLine = "%s";
	break;
    case SH_HEADER:
	CommentString = "# ";
	if (insertHeader)
	    headerLine = "# @Header@\n";
	if (insertLog && insertLocker)
	    lockLogLine = "%s# @Locker@\n#\n# @Log@\n#\n";
	else if (insertLog)
	    lockLogLine = "%s#\n# @Log@\n#\n";
	else if (insertLocker)
	    lockLogLine = "%s# @Locker@\n";
	else
	    lockLogLine = "%s";
	break;
    }

    if (InsertNothing)
	return(headerType);

    if (InsertLog != insertLog && InsertLocker != insertLocker)
	Error("%s/%s already has %s$Log\n",
	    directory, file, insertHeader ? "" : "$Header, ");
    else if (InsertLog != insertLog)
	Error("%s/%s already has %s$Log.\n",
	    directory, file, insertHeader ? "" : "$Header and ");
    else if (InsertLocker != insertLocker)
	Error("%s/%s already has %s$Locker.\n",
	    directory, file, insertHeader ? "" : "$Header and ");
    else if (insertHeader != InsertHeader)
	Error("%s/%s already has $Header.\n", directory, file);

    if (!insertLocker && !insertLog && !insertHeader)
	return(headerType);

    sprintf(buf, lockLogLine, headerLine);
    for (p=buf; *p; p++)
	if (*p == '@')
	    *p = '$';
    HeaderString = buf;

    return (headerType);
}

char *FindSubString(subString, string)
    char    *subString,
	*string;
{
    int    subStringLength = strlen(subString);
    while (*string) {
	if (*subString == *string) {
	switch (subStringLength) {
	default:
	case 8:    if (subString[7] != string[7]) break;
	case 7:    if (subString[6] != string[6]) break;
	case 6:    if (subString[5] != string[5]) break;
	case 5:    if (subString[4] != string[4]) break;
	case 4:    if (subString[3] != string[3]) break;
	case 3:    if (subString[2] != string[2]) break;
	case 2:    if (subString[1] != string[1]) break;
	case 1:
	    if (subStringLength > 8
	     && strncmp(subString, string, subStringLength) != 0)
	    break;
	    return(string);
	}
	}
	string++;
    }

    return(NULL);
}

void PrintHeader(file)
    char    *file;
{
    File    dummy;

    dummy.size = 0;
    dummy.buf = "";

    SetHeader(&dummy, "", "");
    if (file)
	printf("Header for %s:\n", file);
    printf("%s\n", HeaderString);
}

char *GetRevNumber(string, revComment)
    char    *string;
    char    **revComment;
{
    char    *p, *prevnum;
    static char    revnum[ BUFSIZ ];
    static char    header[ BUFSIZ ];

    
    /* remember this for the revision comment */
    *revComment = header;

    /*
     * we are pointing at the string "$Header:"  so look for
     * the third token down from here.
     */
    p = header;
    while (*string != '\n' && *string && *string != ' ')
	*p++ = *string++;
    while (isspace(*string))
	*p++ = *string++;

    while (*string != '\n' && *string && *string != ' ')
	*p++ = *string++;
    while (isspace(*string))
	*p++ = *string++;

    /*
     * We are now looking at the beginning of the revision number.
     * remember it and look for the end of the Header comment.
     */
    prevnum = string;
    while (*string && *string != '$')
	*p++ = *string++;
    *p++ = *string;
    *p = '\0';

    p = revnum;
    while (isdigit(*prevnum) || *prevnum == '.')
	*p++ = *prevnum++;
    *p = '\0';
    if (*revnum == '\0')
	return(NULL);
    return(revnum);
}
