/* $Header: /usr/local/src/nvrToMIT/RCS/reblock.c,v 1.12 1994/12/17 12:55:44 russ Exp $ */
#include "osio.h"
#include "fileAccess.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/file.h>
#ifdef Solaris2
# include <sys/fcntl.h>
#endif

char	*ProgramName;

struct {
    ULong recordsIn;
    ULong recordsOut;
    ULong megaBytesIn;
    ULong bytesIn;
    ULong megaBytesOut;
    ULong bytesOut;
} Total;

#define IncrementBytesOut(n)					\
    {								\
	if ((Total.bytesOut += n) > 1024*1024) {		\
	    Total.megaBytesOut++;				\
	    Total.bytesOut -= 1024*1024;			\
	}							\
    }

#define IncrementBytesIn(n)					\
    {								\
	if ((Total.bytesIn += n) > 1024*1024) {			\
	    Total.megaBytesIn++;				\
	    Total.bytesIn -= 1024*1024;				\
	}							\
    }

static void localFatal();
void Usage();
void ShowStatus();
boolean IsPrefix();
boolean NewMedia();
char *malloc();

main(argc, argv)
    int	    argc;
    char    **argv;
{
    long inputBlockSize = 1024 * 10;
    long outputBlockSize = 1024 * 10;
    char *buffer, *pInBuf, *pOutBuf;
    char *inputFile = NULL, *outputFile = NULL;
    int inputFd, outputFd;
    long outCount = 0, inCount = 0;
    long bufferSize;
    boolean hitEof;
    boolean fixedInput = False, fixedOutput = False;
    boolean inputIsDevice, outputIsDevice;
    boolean multiInput = False, multiOutput = False;
    boolean finalSummary = True;
    long toBeWritten;
    long red, wrote, bytesInBuf;

    ProgramName = *argv;
    for (argv++, argc--; argc; argv++, argc--) {
	if (argv[0][0] == '-') {
	    switch(argv[0][1]) {
	    case 'z':
		break;
	    default:
		Usage("unknown flag=\"%s\"\n", *argv);
		break;
	    }
	} else if (IsPrefix("bs=", *argv)) {
	    inputBlockSize = outputBlockSize = atoi(*argv+3);
	} else if (IsPrefix("ibs=", *argv)) {
	    inputBlockSize = atoi(*argv+4);
	} else if (IsPrefix("obs=", *argv)) {
	    outputBlockSize = atoi(*argv+4);
	} else if (IsPrefix("icount=", *argv)) {
	    inCount = atoi(*argv+7);
	} else if (IsPrefix("ocount=", *argv)) {
	    outCount = atoi(*argv+7);
	} else if (IsPrefix("if=", *argv)) {
	    inputFile = *argv+3;
	} else if (IsPrefix("of=", *argv)) {
	    outputFile = *argv+3;
	} else if (strcmp("multiinput", *argv) == 0) {
	    multiInput = True;
	} else if (strcmp("multioutput", *argv) == 0) {
	    multiOutput = True;
	} else if (strcmp("fixedinput", *argv) == 0) {
	    fixedInput = True;
	} else if (strcmp("quiet", *argv) == 0) {
	    finalSummary = False;
	} else if (strcmp("fixedoutput", *argv) == 0) {
	    fixedOutput = True;
	} else {
	    Usage("unknown option=\"%s\"\n", *argv);
	}
    }

    if (inputBlockSize == 0)
	localFatal("0 input block size\n");
    if (outputBlockSize == 0)
	localFatal("0 output block size\n");
    bufferSize = inputBlockSize;
    while (bufferSize < outputBlockSize)
	bufferSize += inputBlockSize;
    bufferSize += inputBlockSize;

    buffer = malloc(bufferSize);

    if (inputFile) {
	inputFd = open(inputFile, O_RDONLY);
	if (inputFd < 0)
	    localFatal("can't open %s for reading\n", inputFile);
    } else
	inputFd = 0;
    if (outputFile) {
	outputFd = open(outputFile, O_WRONLY|O_TRUNC|O_CREAT);
	if (outputFd < 0)
	    localFatal("can't open %s for writing\n", outputFile);
    } else
	outputFd = 1;

    inputIsDevice = ((FdFileType(inputFd) & (FILE_BLOCK | FILE_CHAR)) != 0);
    outputIsDevice = ((FdFileType(outputFd) & (FILE_BLOCK | FILE_CHAR)) != 0);

    pInBuf = buffer;
    pOutBuf = buffer;
    hitEof = False;
    bytesInBuf = 0;
    do {
	while (!hitEof && bytesInBuf < outputBlockSize) {
	    red = read(inputFd, pInBuf, inputBlockSize);
	    if (red == 0) {
		if (inputIsDevice && multiInput && NewMedia("input")) {
		    close(inputFd);
		    inputFd = open(inputFile, O_RDONLY);
		    if (inputFd < 0)
			localFatal("can't open %s for reading\n", inputFile);
		    continue;
		}
		hitEof = True;
	    } else if (red < 0) {
		Error("error on read\n");
		hitEof = True;
		red = 0;
	    } else {
		if (red < inputBlockSize && fixedInput)
		    Msg("Short read: expected %d, got %d\n",
			inputBlockSize, red);
		if (++Total.recordsIn >= inCount && inCount) {
		    hitEof = True;
		}
	    }

	    pInBuf += red;
	    bytesInBuf += red;
	    IncrementBytesIn(red);
	}

	while (bytesInBuf >= outputBlockSize) {
	    wrote = write(outputFd, pOutBuf, outputBlockSize);
	    if (wrote < 0)
		localFatal("error on write\n");
	    if (wrote == 0) {
		if (outputFile == NULL)
		    localFatal("Eof on stdout\n");
		if (outputIsDevice && multiOutput && NewMedia("output")) {
		    outputFd = open(outputFile, O_WRONLY|O_TRUNC|O_CREAT);
		    if (outputFd < 0)
			localFatal("can't open %s for writing\n", outputFile);
		    continue;
		}
		bytesInBuf = 0;
		break;
	    }
	    if (wrote < outputBlockSize && fixedOutput)
		Msg("Short write: expected %d, got %d\n",
		    outputBlockSize, wrote);
	    if (++Total.recordsOut >= outCount && outCount) {
		hitEof = True;
		bytesInBuf = 0;
		IncrementBytesOut(wrote);
		break;
	    }
	    pOutBuf += wrote;
	    bytesInBuf -= wrote;
	    IncrementBytesOut(wrote);
	}

	if (bytesInBuf)
	    memcpy(buffer, pOutBuf, bytesInBuf);
	pOutBuf = buffer;
	pInBuf = buffer + bytesInBuf;
    } while (! hitEof);

    while (bytesInBuf) {
	if (fixedOutput)
	    toBeWritten = outputBlockSize;
	else
	    toBeWritten = bytesInBuf;

	wrote = write(outputFd, pOutBuf, toBeWritten);
	if (wrote < 0)
	    localFatal("error on write\n");
	Total.recordsOut++;
	IncrementBytesOut(wrote);
	if (fixedOutput && wrote < toBeWritten) {
	    Msg("Short final write: expected %d, got %d\n",
		toBeWritten, wrote);
	    break;
	}
	if ((bytesInBuf -= wrote) < 0)
	    break;
    }

    close(inputFd);
    close(outputFd);

    if (finalSummary)
	ShowStatus();

    exit(0);
}

static void localFatal(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9)
    char *fmt;
{
    Error(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
    ShowStatus();
    exit(1);
}

static char *ByteCount(nMegaBytes, nBytes)
    int nMegaBytes;
    int nBytes;
{
    if (nMegaBytes == 0)
	Msg("(%.1fKB)\n", nBytes/1024.0 + 0.05);
    else
	Msg("(%.1fMB)\n", nMegaBytes + nBytes/(1024.0*1024.0) + 0.05);
}

void ShowStatus()
{
    Msg("%d records in  ", Total.recordsIn);
    ByteCount(Total.megaBytesIn, Total.bytesIn);
    Msg("%d records out ", Total.recordsOut);
    ByteCount(Total.megaBytesOut, Total.bytesOut);
}

boolean NewMedia(which)
    char *which;
{
    FILE *fp = fopen("/dev/tty", "rw");
    boolean resume;
    char buf[ BUFSIZ ];

    if (isatty(0)) {
	fp = fdopen(0, "rw");
	if (fp == NULL)
	    localFatal("can't fdopen(0, \"rw\") for new %s media\n", which);
    } else {
	fp = fopen("/dev/tty", "rw");
	if (fp == NULL)
	    localFatal("can't open /dev/tty for new %s media\n", which);
    }
    ShowStatus();
    Msg("Eof on %s.  Insert new media and hit return: ", which);
    resume = (fgets(buf, BUFSIZ, fp) != NULL);
    fclose(fp);
    return (resume);
}

boolean IsPrefix(prefix, string)
    char *prefix;
    char *string;
{
    return (strncmp(prefix, string, strlen(prefix)) == 0);
}

void Usage(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9)
    char    *fmt;
{
    Msg(fmt, x0,x1,x2,x3,x4,x5,x6,x7,x8,x9);
    Msg("Usage: %s [options...]\n", ProgramName);
    Msg("  where options are selected from:\n");
    Msg("    bs=count       select the input and output block size\n");
    Msg("    ibs=count      select the input block size\n");
    Msg("    obs=count      select the output block size\n");
    Msg("    icount=count   set a limit on the number of input blocks\n");
    Msg("    ocount=count   set a limit on the number of output blocks\n");
    Msg("    if=file        select an input file name\n");
    Msg("    of=file        select an output file name\n");
    Msg("    fixedinput     force the input reads to ibs (or bs)\n");
    Msg("    fixedoutput    force the output writes to obs (or bs)\n");
    Msg("    multiinput     prompt for new media on eof (devices only)\n");
    Msg("    multioutput    prompt for new media on eof (devices only)\n");
    exit (1);
}
