/* $Header: /usr/local/src/nvrToMIT/RCS/a60ftp.c,v 1.2 1994/12/17 12:55:22 russ Exp $ */
/*
 * This file contains an ftp client to be linked into a program that needs
 * to transfer data to/from an Abekas A60 video disk via ftp.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/ftp.h>
#include <arpa/telnet.h>
#include <stdio.h>
#include <netdb.h>
#include <errno.h>

#define	BYTE(a, o)	(((u_char *)&(a))[o])

#define	XMAX		1280	/* Max width of source RGB image */

#define	D1WIDTH		720	/* Total pixels per line in 422 D1 video */
#define	D1HEIGHT	486	/* Total lines in 422 D1 video per frame */

#define	D1YMIN		16	/* Minimum luminance */
#define	D1YMAX		235	/* Maximum luminance */
#define	D1CMIN		16	/* Minimum chromanance */
#define	D1CMAX		240	/* Maximum chromanance */
#define	D1YBIAS		125	/* Luminance offset to make 0 centered  range */
#define	D1CBIAS		128	/* Chominance offset to make 0 centered range */

/*
 * RGB to Y, Cr, Cb conversion.  The hex constants below are 32-bit fractions
 * (where 0x100000000 is 1.0).  To evaluate Y (or Cr or Cb) execute:
 *
 *      Y  =  (R*YRFactor  + G*YGFactor  + B*YBFactor  + YOffset)  >> Fraction
 *      Cr =  (R*CrRFactor - G*CrGFactor - B*CrBFactor + CrOffset) >> Fraction
 *      Cb = (-R*CbRFactor - G*CbGFactor + B*CbBFactor + CbOffset) >> Fraction
 *
 * The resulting YUV values are positive (biased chromas).
 */

#define Fraction        22

#define YRFactor  ((u_long)0x41700794 >> (32 - Fraction)) /* .299*219/255 */
#define YGFactor  ((u_long)0x8077c38a >> (32 - Fraction)) /* .587*219/255 */
#define YBFactor  ((u_long)0x18f30fbb >> (32 - Fraction)) /* .114*219/255 */
#define YOffset   (0x021 << (Fraction-1)) 		  /* 16.5<<Fraction */

#define CbRFactor ((u_long)0x25bec833 >> (32 - Fraction)) /* .1686*224/255 */
#define CbGFactor ((u_long)0x4a1ff62e >> (32 - Fraction)) /* .3311*224/255 */
#define CbBFactor ((u_long)0x6fdebe62 >> (32 - Fraction)) /* .4997*224/255 */
#define CbOffset  (0x101 << (Fraction-1))		  /* 128.5<<Fraction */

#define CrRFactor ((u_long)0x6fe47991 >> (32 - Fraction)) /* .4998*224/255 */
#define CrGFactor ((u_long)0x5db104fc >> (32 - Fraction)) /* .4185*224/255 */
#define CrBFactor ((u_long)0x12337494 >> (32 - Fraction)) /* .0813*224/255 */
#define CrOffset  (0x101 << (Fraction-1))		  /* 128.5<<Fraction */

char	abekasFtpHost[50];	/* Abekas host name */
int	abekasFtpCmd = -1;	/* Command socket file descriptor */
int	abekasFtpData = -1;	/* Data socket file descriptor */
int	abekasFtpDebug;		/* Debug flag */
int	abekasFtpRcv;		/* non-zero if receiving data */
int	abekasFtpReply;		/* Complete reply code */
int	abekasFtpState;		/* State of FTP protocol */
char	*abekasFtpRcvptr;		/* Abekas receive buffer pointer */
char	*abekasFtpRlast;		/* Abekas receive last */
char	abekasFtpRcvbuf[256];	/* Abekas receive buffer */

struct	sockaddr_in  abekasLocalAddr;	/* Local control socket addr */

char	abekasFtpPut[] = "STOR";	/* FTP put command */
char	abekasFtpGet[] = "RETR";	/* FTP get command */

extern	int	 errno;		/* syscall error code */
extern	char	*sys_errlist[];	/* syscall error strings */

char	*getenv();

abekasFtpOpen(host)
char *host;
{
    struct sockaddr_in sin;
    struct hostent *hp;
    struct servent *sp;
    int n;

    memset((char *)&sin, 0, sizeof sin);
    if ((host == NULL) && ((host = getenv("ABEKAS")) == NULL))
	host = "a60";
    if ((int)(sin.sin_addr.s_addr = inet_addr(host)) != -1)
	sin.sin_family = AF_INET;
    else {
	if ((hp = gethostbyname(host)) == NULL) {
	    fprintf(stderr, "Unknown host: %s\n", host);
	    return(1);
	}
	sin.sin_family = hp->h_addrtype;
#ifdef SYSV
        memcpy((char *)&sin.sin_addr, hp->h_addr_list[0], hp->h_length);
#else
        memcpy((char *)&sin.sin_addr, hp->h_addr, hp->h_length);
#endif
    }
    strncpy(abekasFtpHost, host, sizeof abekasFtpHost);
    if ((sp = getservbyname("ftp", "tcp")) == NULL) {
	fprintf(stderr, "Cannot find ftp port number\n");
	return(1);
    }
    sin.sin_port = sp->s_port;
    if (abekasFtpCmd >= 0)
	close(abekasFtpCmd);
abekasFtpState = 1;
    if ((abekasFtpCmd = socket(sin.sin_family, SOCK_STREAM, 0)) < 0) {
	fprintf(stderr, "Socket creation error: %s\n", sys_errlist[errno]);
	return(1);
    }
abekasFtpState = 2;
    if (connect(abekasFtpCmd, &sin, sizeof sin) < 0) {
	fprintf(stderr, "Cannot connect to %s: %s\n", host,
	    sys_errlist[errno]);
	goto closesock;
    }
abekasFtpState = 3;

    n = sizeof(struct sockaddr_in);
    if (getsockname(abekasFtpCmd, &abekasLocalAddr, &n) < 0) {
	fprintf(stderr, "getsockname() error: %s\n", sys_errlist[errno]);
	goto closesock;
    }
abekasFtpState = 4;
    (void) abekasFtpCommand(NULL);
abekasFtpState = 5;
    if ((n = abekasFtpCommand("USER ftp")) == CONTINUE)
	if ((n = abekasFtpCommand("PASS ftp")) == CONTINUE)
	    n = abekasFtpCommand("ACCT ftp");
    if (n != COMPLETE) {
	fprintf(stderr, "Login failed (%d): %s",
	    abekasFtpReply, abekasFtpRcvbuf);
	goto closesock;
    }
abekasFtpState = 6;
    if (abekasFtpCommand("TYPE I") != COMPLETE) {
	fprintf(stderr, "Type command failed (%d): %s",
	    abekasFtpReply, abekasFtpRcvbuf);
closesock:
abekasFtpState = 33;
	close(abekasFtpCmd);
	abekasFtpCmd = -1;
abekasFtpState = 0;
	return(1);
    }
abekasFtpState = 7;
    return(0);
}


abekasFtpCommand(fmt, x1, x2, x3, x4, x5, x6)
char *fmt;
{
    char *p, buf[1024];
    int n, m, c, retval;

    if (abekasFtpCmd < 0) {
	fprintf(stderr, "Connection closed\n");
	return(-1);
    }
    if (fmt != NULL) {
	sprintf(buf, fmt, x1, x2, x3, x4, x5, x6);
	if (abekasFtpDebug)
	    fprintf(stderr, "> %s\n", buf);
	if ((n = strlen(buf)) >= sizeof buf)
	    n = sizeof buf - 2;
	buf[n++] = '\r';
	buf[n++] = '\n';
	for (p = buf; p < &buf[n]; p += m) {
	    if ((m = write(abekasFtpCmd, p, &buf[n] - p)) < 0) {
		if (errno == EINTR) {
		    m = 0;
		    continue;
		}
		fprintf(stderr, "Write error on %s socket: %s\n",
		    abekasFtpHost, sys_errlist[errno]);
		goto closesock;
	    }
	    if (m == 0) {
		fprintf(stderr, "Zero byte write on %s socket\n", abekasFtpHost);
		goto closesock;
	    }
	}
abekasFtpState = - abekasFtpState;
    }
    abekasFtpReply = m = c = retval = 0;
    while (c != '\n') {
	if (abekasFtpRcvptr >= abekasFtpRlast) {
	    if ((n = read(abekasFtpCmd, abekasFtpRcvbuf,
		sizeof abekasFtpRcvbuf - 1)) < 0) {
		if (errno == EINTR) {
		    n = 0;
		    continue;
		}
		fprintf(stderr, "Read error on %s socket: %s\n",
		    abekasFtpHost, sys_errlist[errno]);
		goto closesock;
	    }
	    if (n == 0) {
		fprintf(stderr, "Lost connection from %s\n", abekasFtpHost);
closesock:
abekasFtpState = 32;
		close(abekasFtpCmd);
		abekasFtpCmd = -1;
		return(-1);
	    }
	    abekasFtpRlast = abekasFtpRcvbuf + n;
	    *abekasFtpRlast = 0;
	    if (abekasFtpDebug)
		fprintf(stderr, "< %s", abekasFtpRcvbuf);
	    abekasFtpRcvptr = abekasFtpRcvbuf;
	}
	c = *abekasFtpRcvptr++;
	if (!retval)
	    retval = c - '0';
	if (!m && isdigit(c))
	   abekasFtpReply = 10 * abekasFtpReply + c - '0';
	else
	   m = 1;
    }
    return(retval);
}


/*
 * abekasSendChannel() opens a data connection for sending data to
 * a particular frame of the A60's disk.
 */

abekasXferSetup(cmd, a60name)
char *cmd, *a60name;
{
    struct sockaddr_in sin;
    int s, n;

    /*
     * Setup a listner socket to receive the connection for the data
     */
    sin = abekasLocalAddr;
    sin.sin_port = 0;			/* chosen by UNIX */
abekasFtpState = 8;
    while ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
	if (errno != EINTR) {
	    fprintf(stderr, "socket(): %s\n", sys_errlist[errno]);
	    return(-1);
	}
    }
abekasFtpState = 9;
    while (bind(s, (struct sockaddr *)&sin, sizeof sin) < 0) {
	if (errno != EINTR) {
	    fprintf(stderr, "bind() error: %s\n", sys_errlist[errno]);
	    goto closesock;
	}
    }
    n = sizeof sin;
abekasFtpState = 10;
    while (getsockname(s, (char *)&sin, &n) < 0) {
	if (errno != EINTR) {
	    fprintf(stderr, "getsockname() error: %s\n", sys_errlist[errno]);
	    goto closesock;
	}
    }
abekasFtpState = 11;
    while (listen(s, 1) < 0) {
	if (errno != EINTR) {
	    fprintf(stderr, "listen() error: %s\n", sys_errlist[errno]);
	    goto closesock;
	}
    }
abekasFtpState = 12;
    if ((n = abekasFtpCommand("PORT %d,%d,%d,%d,%d,%d",  BYTE(sin.sin_addr, 0),
	BYTE(sin.sin_addr, 1), BYTE(sin.sin_addr, 2), BYTE(sin.sin_addr, 3),
	BYTE(sin.sin_port, 0), BYTE(sin.sin_port, 1))) != COMPLETE) {
	fprintf(stderr, "PORT command failed (%d): %s",
	    abekasFtpReply, abekasFtpRcvbuf);
	goto closesock;
    }
abekasFtpState = 13;
    /*
     * Request data transfer
     */
    if ((n = abekasFtpCommand("%s %s", cmd, a60name)) != PRELIM) {
	fprintf(stderr, "Unexpected reponse to %s (%d): %s",
	    cmd, abekasFtpReply, abekasFtpRcvbuf);
	goto closesock;
    }
    /*
     * Accept connection anb close listner
     */
abekasFtpState = 14;
    do {
	n = sizeof sin;
	if ((abekasFtpData = accept(s, (struct sockaddr *)&sin, &n)) >= 0) {
	    abekasFtpRcv = cmd == abekasFtpGet;
abekasFtpState = 15;
	    close(s);
abekasFtpState = 16;
	    return(abekasFtpData);
	}
    } while (errno == EINTR);
    fprintf(stderr, "accept() error: %s\n", sys_errlist[errno]);
closesock:
abekasFtpState = 15;
    close(s);
    return(-1);
}


/*
 * AbekasFtpDone() terminates a file transfer
 */

abekasFtpDone(abortrcv) {
    static char abt0[] = { IAC, IP };
    static char abt1[] = { DM, 'A', 'B', 'O', 'R', '\r', '\n' };
    char buf[1024];

    if (abortrcv) {
abekasFtpState = 19;
	write(abekasFtpCmd, abt0, sizeof abt0);
abekasFtpState = 20;
	send(abekasFtpCmd, abt0, 1, MSG_OOB);
abekasFtpState = 21;
	write(abekasFtpCmd, abt1, sizeof abt1);
abekasFtpState = 22;
	while (read(abekasFtpData, buf, sizeof buf) > 0)
	    ;
abekasFtpState = 23;
	if ((abekasFtpCommand(NULL) == ERROR) && (abekasFtpReply == 552)) {
	    if (abekasFtpData >= 0) {
abekasFtpState = 24;
		close(abekasFtpData);
		abekasFtpData = -1;
	    }
abekasFtpState = 25;
	    (void) abekasFtpCommand(NULL);
	}
    } else if (abekasFtpRcv) {
abekasFtpState = 26;
	while (read(abekasFtpData, buf, sizeof buf) > 0)
	    ;
    }
    if (abekasFtpData >= 0) {
abekasFtpState = 27;
	close(abekasFtpData);
    }
    abekasFtpData = -1;
abekasFtpState = 28;
    if (abekasFtpCommand(NULL) != COMPLETE) {
	fprintf(stderr, "Expected 'File Transfer OK', got (%d): %s",
	    abekasFtpReply, abekasFtpRcvbuf);
abekasFtpState = 7;
	return(1);
    }
abekasFtpState = 7;
    return(0);
}

/*
 * AbekasFtpClose() closes the FTP connection with the A60
 */

abekasFtpClose() {
    static char quit[] = "QUIT\r\n";

    if (abekasFtpData >= 0) {
abekasFtpState = 29;
	abekasFtpDone(abekasFtpRcv);
    }
abekasFtpState = 30;
    write(abekasFtpCmd, quit, sizeof quit);
abekasFtpState = 31;
    close(abekasFtpCmd);
    abekasFtpCmd = -1;
abekasFtpState = 0;
}

/*
 * abekasPutYUV() transfers a RGB frame to the A60.  This routine does
 * the resizing to D1 dimensions and conversion to YUV.  Returns 0 if
 * successful.
 */

abekasFtpPutRGB(fno, red, grn, blu, inc, width, height)
u_char *red, *grn, *blu;
{
    u_char linea[3 * XMAX], lineb[3 * XMAX], yuvline[2 * D1WIDTH];
    double yin, yinc, yinmax, xinc, xinmax, t;
    int winc, y0, y1, r0, r1, g0, g1, b0, b1;
    register u_char *a, *b, *c, *d;
    u_char *line0, *line1;
    register float alpha;
    register int k;

    if (width > XMAX) {
	fprintf(stderr, "RGB source too wide (%d), must be less than %d\n",
	    width, XMAX);
	return(1);
    }
    sprintf(linea, "%d.yuv", fno);
    if (abekasXferSetup(abekasFtpPut, linea) < 0)
	return(1);
    line0 = linea;
    line1 = lineb;
    y0 = y1 = -1;
    yinc = (yinmax = height - 1) / (double)(D1HEIGHT - 1);
    xinc = (xinmax = width - 1) / (double)(D1WIDTH - 1);
    xinmax += 0.5 * xinc;
    yinmax += 0.5 * yinc;
    winc = width * inc;
    for (yin = 0.0; yin < yinmax; yin += yinc) {
	/*
	 * Interpolate to get two RGB lines (line1 & line2) for two
	 * adjacent lines of the original data with an X spacing for
	 * D1 video.
	 */
	if (y0 != (int)yin) {
	    y0 = (int)yin;
	    if (y1 != y0) {
		a = &red[k = y0 * winc];
		b = &grn[k];
		c = &blu[k];
		for (d = line0, t = 0.0; t < xinmax; t += xinc) {
		    k = t;
		    alpha = t - k;
		    k *= inc;
		    d[0] = alpha * ((int)a[k + inc] - (int)a[k]) + a[k];
		    d[1] = alpha * ((int)b[k + inc] - (int)b[k]) + b[k];
		    d[2] = alpha * ((int)c[k + inc] - (int)c[k]) + c[k];
		    d += 3;
		}
	    } else {
		line0 = line1;
		line1 = (line0 == linea) ? lineb : linea;
	    }
	    if ((y1 = y0 + 1) >= height)
		y1 = height - 1;
	    a = &red[k = y1 * winc];
	    b = &grn[k];
	    c = &blu[k];
	    for (d = line0, t = 0.0; t < xinmax; t += xinc) {
		k = t;
		alpha = t - k;
		k *= inc;
		d[0] = alpha * ((int)a[k + inc] - (int)a[k]) + a[k];
		d[1] = alpha * ((int)b[k + inc] - (int)b[k]) + b[k];
		d[2] = alpha * ((int)c[k + inc] - (int)c[k]) + c[k];
		d += 3;
	    }
	}
	/*
	 * Interpolate vertically to find the line (between line0 and line1)
	 * at the height of a video line.  The conversion between RGB and
	 * YUV is done here.
	 */
	alpha = yin - y0;
	a = line0;
	b = line1;
	c = yuvline;
	for (d = c + 2*D1WIDTH; c < d; a += 6, b += 6, c += 4) {
	    /*
	     * (r0, g0, b0) and (r1, g1, b1) are the even and odd consecutive
	     * pixels along a video line.
	     */
	    r0 = alpha * ((int)b[0] - (int)a[0]) + a[0];
	    g0 = alpha * ((int)b[1] - (int)a[1]) + a[1];
	    b0 = alpha * ((int)b[2] - (int)a[2]) + a[2];
	    r1 = alpha * ((int)b[3] - (int)a[3]) + a[3];
	    g1 = alpha * ((int)b[4] - (int)a[4]) + a[4];
	    b1 = alpha * ((int)b[5] - (int)a[5]) + a[5];
	    /*
	     * Convert RGB to YUV.  The chroma values are the average of the
	     * two pixel's chromas.
	     */
	    c[1] = (r0 * YRFactor + g0 * YGFactor + b0 * YBFactor +
		YOffset) >> Fraction;
	    c[3] = (r1 * YRFactor + g1 * YGFactor + b1 * YBFactor +
		YOffset) >> Fraction;
	    c[0] = ((b0 += b1) * CbBFactor - (r0 += r1) * CbRFactor -
		(g0 += g1) * CbGFactor + 2 * CbOffset) >> (Fraction + 1);
	    c[2] = (r0 * CrRFactor - b0 * CrBFactor - g0 * CbGFactor +
		2 * CrOffset) >> (Fraction + 1);
	}
	/*
	 * Write YUV line to A60 FTP data socket
	 */
	for (a = yuvline; a < &yuvline[2*D1WIDTH]; a += k) {
abekasFtpState = 17;
	    if ((k = write(abekasFtpData, a, &yuvline[2*D1WIDTH] - a)) < 0) {
abekasFtpState = 16;
		fprintf(stderr, "Write error on %s data socket: %s\n",
		    abekasFtpHost, sys_errlist[errno]);
		abekasFtpDone(0);
		return(1);
	    }
abekasFtpState = 16;
	    if (k == 0) {
		fprintf(stderr, "Zero byte write on %s data socket\n",
		    abekasFtpHost);
		abekasFtpDone(0);
		return(1);
	    }
	    if (abekasFtpDebug)
		fprintf(stderr, "Wrote %d bytes\n", k);
	}
    }
    abekasFtpDone(0);
    return(0);
}

#if 0
/*
 * Display an image on the hideous framebuffer (debugging)
 */

#include <sys/stat.h>
#include "../../hid/hid/hid.h"
#undef h
#define HID             ((struct hideous *)0xC0000000)
#define HIDSIZE         ((0xD6000000-0xC0000000)>>3)
#define HID_RED         0
#define HID_GRN         1
#define HID_BLU         2
#define HID_TICK        5       /* Tick mark length in pixels */

#define	fatal(f)	{ fprintf f ; exit(1); }

showhid(x, y, red, grn, blu, w, h)
register u_char *red, *grn, *blu;
{
    static int *hid_fb, hid_width, hid_height;
    static char hidfile[] = "/dev/hid";
    struct stat st;
    register int n, *p;

    if (hid_fb == NULL) {
	/*
	 * Open hideous shared memory region
	 */
	if (stat(hidfile, &st) == -1)
	    fatal((stderr, "Cannot fstat %s: %s\n",
		hidfile, sys_errlist[errno]));
	if ((n = shmget(st.st_ino, HIDSIZE, 0)) == -1)
	    fatal((stderr, "Cannot shmget(%d, 0x%x, 0): %s\n",
		st.st_ino, HIDSIZE, sys_errlist[errno]));
	if ((int)(hid_fb = (int *)shmat(n, HID, 0)) == -1)
	    fatal((stderr, "shmat(%d, 0x%x, 0): %s\n",
		n, HID, sys_errlist[errno]));
	hid_fb = (int *)((int)hid_fb + ((HID->h_dpyst - 0xC0000000) >> 3));
	hid_width = HID->h_dinc >> 5;
	hid_height = HID->h_vsblnk - HID->h_veblnk;
	if (!(HID->h_dpyctl & DPYCTL_NIL))
	    hid_height <<= 1;
        memset(hid_fb, 0, hid_width * hid_height * sizeof(int));
	/*
	 * Avoid top & left margins and leave room for tick marks
	 */
	hid_fb += (2 + HID_TICK) * hid_width + 16 + HID_TICK;
    }
    p = &hid_fb[x + y * hid_width];
    n = hid_width - w;
    for (y = 0; y < h; y++) {
	for (x = 0; x < w; x++)
	    *p++ = RGB(*red++, *grn++, *blu++);
	p += n;
    }
}
#endif
