/*
 *	Copyright (c) 1988, Tektronix, Inc.
 *	Confidential -- All proprietary rights reserved.
 *	Licensed Material -- Property of Tektronix, Inc.
 *
 *	This program is the property of Tektronix, Inc. or others
 *	from whom Tektronix has obtained a licensing right, and is
 *	considered by Tektronix to be confidential.  It is protected
 *	by U.S. copyright law as an unpublished work and is
 *	furnished pursuant to a written license agreement.  It may
 *	not be used, copied or otherwise reproduced, or disclosed to
 *	others except in accordance with the terms and conditions of
 *	that agreement.
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/minido.c,v 1.2 1991/10/09 18:32:15 bobe Exp $
 */

static char rcsid[] = " $Header: /usr/local/src/nvrToMIT/RCS/minido.c,v 1.2 1991/10/09 18:32:15 bobe Exp $ ";

#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <pwd.h>
#include <netdb.h>
#include <signal.h>
#include <ctype.h>

#ifdef BSD
#include <sgtty.h>
#else
#include <termio.h>
#endif /* BSD */

#include "minivas.h"

#define DEFAULT_TTY	"/dev/tty-0"		/* Default tty */
#define DEF_TIMEOUT	120			/* Default timeout */
#define	DEFAULT_VTR	"SonyBVW40"		/* Default VTR */

char *getenv();
char *malloc();
char *strchr();

int     minivas_start_frame = 1;	/* in point */
int	minivas_num_frames = 1;		/* number of frames to record */
char	*minivasTty;			/* minivas TTY special file */
int	minivasTtyFd = -1;		/* minivas tty file descriptor */
char	*minivasVTRname;		/* minivas VTR name */
int	minivasVTRcode;			/* VTR command code */
int	minivasVerbose;			/* diagnostics flag */
int	minivasSendOK;			/* OK to send flag */
int	minivasReceiveOK;		/* OK to receive flag */
int	minivasTC;			/* Use SMPTE timecode vs VIFC codes */
int	minivasFGrab;			/* Frame grabbing vs recording */
int	minivasTimeoutSeconds = DEF_TIMEOUT;	/* minivas timeout (seconds) */

extern int errno;
extern char *sys_errlist[];

struct	vtrtable {
	char	*vtr_name;	/* VTR name */
	int	 vtr_code;	/* VTR code */
} minivasVTRtable[] = {
	{  "AmpexVPR-3",	MINIVAS_AMPEX_VPR3	},
	{  "AmpexVPR-6",	MINIVAS_AMPEX_VPR6	},
	{  "AmpexVPR-80",	MINIVAS_AMPEX_VPR80	},
	{  "AmpexVPR-80TC",	MINIVAS_AMPEX_VPR80TC	},
	{  "AmpexCVR-40",	MINIVAS_AMPEX_CVR40	},
	{  "AmpexCVR-75",	MINIVAS_AMPEX_CVR75	},
	{  "JVC-CR850",		MINIVAS_JVC_CR850	},
	{  "JVC-PR900",		MINIVAS_JVC_PR900	},
	{  "PanAU650TC",	MINIVAS_PAN_AU650TC	},
	{  "PanAU650",		MINIVAS_PAN_AU650	},
	{  "PanAG7500",		MINIVAS_PAN_AG7500	},
	{  "SonyBVW40",		MINIVAS_SONY_BVW40	},
	{  "SonyBVW75",		MINIVAS_SONY_BVW75	},
	{  "SonyBVU800",	MINIVAS_SONY_BVU800	},
	{  "SonyBVU800TC",	MINIVAS_SONY_BVU800TC	},
	{  "SonyBVU950",	MINIVAS_SONY_BVU950	},
	{  "SonyBVU950TC",	MINIVAS_SONY_BVU950TC	},
	{  "SonyVO5850",	MINIVAS_SONY_VO5850	},
	{  "SonyVO5850P",	MINIVAS_SONY_VO5850P	},
	{  "SonyBVH2000",	MINIVAS_SONY_BVH2000	},
	{  "SonyBVH2500",	MINIVAS_SONY_BVH2500	},
	{  "SonyBVH3000",	MINIVAS_SONY_VBH3000	},
	{  "SonyDVR10",		MINIVAS_SONY_DVR10	},
	{  "SonyDVR1000",	MINIVAS_SONY_DVR1000	},
	{  NULL,		0			},
};

minivasOpen(p)
char *p;
{
    static char ttyName[50], vtrName[20];
    int n, n1, n2, n3, n4, preblack = 0, noinit = 0, rewind = 0;
    char tc1[10], tc2[10];
    struct vtrtable *vtr;
    char *q;

    if ((minivasTty = getenv("MINIVAS")) == NULL)
	minivasTty = DEFAULT_TTY;
    if ((minivasVTRname = getenv("VTR")) == NULL)
	minivasVTRname = DEFAULT_VTR;
    while (*p) {
	switch (*p++) {

	case 'b':			/* preblack the tape */
	    preblack = 1;
	    break;

	case 'i':			/* initialize minivas */
	    /* minivasInitFlag = 1; 	   obsolete */
	    break;
	
	case 'd':			/* Specify VTR device */
	    if ((p = strchr(q = p, ',')) == NULL) {
		p = "";
		strncpy(vtrName, q, sizeof vtrName);
	    } else {
		n1 = *p;
		*p = 0;
		strncpy(vtrName, q, sizeof vtrName);
		*p++ = n1;
	    }
	    minivasVTRname = vtrName;
	    break;

	case 'l':			/* Specify minivas tty file */
	    if ((p = strchr(q = p, ',')) == NULL) {
		p = "";
		strncpy(ttyName, q, sizeof ttyName);
	    } else {
		n1 = *p;
		*p = 0;
		strncpy(ttyName, q, sizeof ttyName);
		*p++ = n1;
	    }
	    minivasTty = ttyName;
	    break;

	case 'n':			/* Number of frames to record */
	    if (!isdigit(*p)) {
		fprintf(stderr, "Exepcted number of frames after 'r'\n");
		return(1);
	    }
	    minivas_num_frames = strtol(p, &p, 10);
	    break;

	case 's':			/* Set start frame */
	    if (!isdigit(*p)) {
		fprintf(stderr,
		    "Exepcted frame number or time code after 's'\n");
		return(1);
	    }
	    minivas_start_frame = strtol(p, &p, 10);
	    if (*p == ':') {
		n1 = minivas_start_frame;
		n2 = strtol(p + 1, &p, 10);
		if (*p == ':') {
		    n3 = strtol(p + 1, &p, 10);
		    if (*p == ':') {
			n4 = strtol(p + 1, &p, 10);
		    } else {
			n4 = n3; n3 = n2; n2 = n1; n1 = 0;
		    }
		} else {
		    n4 = n2; n3 = n1; n2 = n1 = 0;
		}
		if ((n1 < 0) || (n1 > 23)) {
		    fprintf(stderr,
			"Time code hours field (%d) out-of-range\n", n1);
		    return(1);
		}
		if ((n2 < 0) || (n2 > 59)) {
		    fprintf(stderr,
			"Time code minutes field (%d) out-of-range\n", n2);
		    return(1);
		}
		if ((n3 < 0) || (n3 > 59)) {
		    fprintf(stderr,
			"Time code seconds field (%d) out-of-range\n", n3);
		    return(1);
		}
		if ((n4 < 0) || (n4 > 29)) {
		    fprintf(stderr,
			"Time code frame field (%d) out-of-range\n", n4);
		    return(1);
		}
		/*
		 * Compute the sequential frame number from 00:00:00:00
		 * using the NTSC drop-frame conventions.  Without these
		 * conventions, the calculation is:
		 *
		 * minivas_start_frame = 30 * (60 * (60 * n1 + n2) + n3) + n4;
		 */
		minivas_start_frame = n4 + 30 * n3 +
		    17982 * (6 * n1 + n2 / 10);
		if (n = n2 % 10) {
		    if ((n3 == 0) && (n4 < 2)) {
			fprintf(stderr,
		"Timecode %02d:%02d:%02d:%02d is illegal in drop-frame mode\n",
			    n1, n2, n3, n4);
			return(1);
		    }
		    minivas_start_frame += (1800 - 2) * n;
		}
	    }
	    break;
	
	case 'g':			/* Frame grab mode */
	    minivasFGrab = 1;
	    break;

	case 'r':			/* Rewind tape first */
	    rewind = 1;
	    break;

	case 't':			/* Use time-code addressing */
	    minivasTC = 1;
	    break;

	case 'T':			/* Specify timeout in seconds */
	    if (!isdigit(*p)) {
		fprintf(stderr, "Exepected number following T option\n");
		return(1);
	    }
	    minivasTimeoutSeconds = strtol(p, &p, 10);
	    break;

	case 'v':			/* Activate diagnostics */
	    minivasVerbose = 1;
	    break;

	case '-':			/* Don't initialize for record/grab */
	    noinit = 1;
	    break;

	default:
	    fprintf(stderr, "Unknown option: %s\n", p);
	    return(1);
	}
    }
    /*
     * Find VTR type code by name
     */
    for (vtr = minivasVTRtable; ; vtr++) {
	if (vtr->vtr_name == NULL) {
	    fprintf(stderr, "Unknown VTR name: %s\n", minivasVTRname);
	    return(1);
	}
	if (!strcmp(vtr->vtr_name, minivasVTRname))
	    break;
    }
    minivasVTRcode = vtr->vtr_code;
    if ((minivasTtyFd = minivasOpenTty()) < 0)
	return(1);
    if (minivasSetVTR(minivasVTRcode))
	return(1);
    if (rewind) {
	minivasSendChar(MINIVAS_REWIND);
	do {
	    sleep(1);
	    if (minivasGetTC(tc1))
		return(1);
	    sleep(1);
	    if (minivasGetTC(tc2))
		return(1);
	} while (strcmp(tc1, tc2));
    }
    if (preblack) {
	(void) minivasSendChar(MINIVAS_PREBLACK);
	(void) minivasSendChar(MINIVAS_ENTER);
	minivasClose();
	return(0);
    }
    if (!noinit && minivasInit(minivas_start_frame, minivas_num_frames))
	return(1);
    return(0);
}


minivasGetTC(buf)
char *buf;
{
    int n, m;

    minivasSendChar(MINIVAS_GETTC);
    for (;;) {
	if (minivasRead(buf, 1) <= 0)
	    return(1);
	if (strchr("<=?>", buf[0])) {
	    for (n = 1; n < 9; n += m) {
		if ((m = minivasRead(&buf[n], 9 - n)) <= 0)
		    return(1);
		buf[n + m] = 0;
		/*
		fprintf(stderr, "minivasGetTC='%s' %d %d\n", buf, n, m);
		*/
	    }
	    if (minivasVerbose)
		fprintf(stderr, "minivasGetTC='%s'\n", buf);
	    buf[9] = 0;
	    return(0);
	}
	if (minivasVerbose)
	    fprintf(stderr, "minivasGetTC got %c (0x%x)\n", buf[0], buf[0]);
    }
}

minivasClose() {
    if (minivasTtyFd < 0)
	return;
    minivasShutdown(0);
    close(minivasTtyFd);
    minivasTtyFd = -1;
}

minivasRecord() {
    return(minivasSendChar(MINIVAS_REC));
}

/*
 * Returns frame number which recording was not successful
 */

minivasPollStatus() {
    if (minivasWaitFor(MINIVAS_A_R))   /* Ready for next recording */
        return(minivas_start_frame);
    ++minivas_start_frame;
    return(0);
}

/*
 * Initializes the minivas for the specified VTR
 */

minivasSetVTR(vtr) {
    if (minivasSendCmd(MINIVAS_INITIALIZE, MINIVAS_A_I)) {
	fprintf(stderr, "No acknowledge after minivas initialize\n");
	return(1);
    }
    (void) minivasSendChar(MINIVAS_IFSWITCH);			/* Select VTR */
    (void) minivasSendChar(vtr >> 8);
    if (minivasSendCmd(vtr & 0xFF, MINIVAS_A_I)) {
	fprintf(stderr, "No acknowledge after setting VTR type\n");
	return(1);
    }
}

/*
 * Initializes the minvas/VTR for frame-by-frame recording or grabbing in
 * either VIFC mode or SMPTE time code mode.  On successful completion of the
 * function, the tape has been cued (without waiting for the acknowledge)
 * and is read for the record/grab commands.  The minivasPollStatus()
 * function should be used to verify the VTR is ready prior to issuing the
 * record/grab commands (or modifying the frame buffer in the case of record
 * commands).
 */

minivasInit(start, num) {
    int hours, min, sec, frames;

    if(start)
	minivas_start_frame = start;
    if(num)
        minivas_num_frames = num;
    if(minivas_num_frames < 0)
	return(0); 			/* No recording/grabbing is needed */
    if (minivasTC) {
        /*
         * Use SMPTE time code frame addressing
         */
        if (minivasSendCmd(minivasFGrab ? MINIVAS_GRAB_TC : MINIVAS_RECORD_TC,
	    minivasFGrab ? MINIVAS_A_H : MINIVAS_A_U)) {
	    fprintf(stderr, "No acknowledge after timecode %s command\n",
		minivasFGrab ? "grab" : "record");
	    return(1);
	}
        minivasSendNum(minivas_num_frames);  /* number frames to record */
        (void) minivasSendChar(MINIVAS_ENTER);
	minivasSendTC(minivas_start_frame);
    } else {
	/*
	 * Use VIFC frame addressing
	 */
        if (minivasSendCmd(minivasFGrab ? MINIVAS_GRAB_VIFC :
	    MINIVAS_RECORD_VIFC, MINIVAS_A_P)) {
	    fprintf(stderr, "No acknowledge after timecode %s command\n",
		minivasFGrab ? "grab" : "record");
	    return(1);
	}
        minivasSendNum(minivas_num_frames);  /* number frames to record */
        (void) minivasSendChar(MINIVAS_ENTER, NULL);
        minivasSendNum(minivas_start_frame); /* start frame to record */
    }
    (void) minivasSendChar(MINIVAS_ENTER);
    if (minivasFGrab) {
	minivasSendNum(0);	/* AUX out delay */
	(void) minivasSendChar(MINIVAS_ENTER);
    }
    if (minivasWaitFor(MINIVAS_A_E)) {
	fprintf(stderr, "No acknowledge after record/grab setup\n");
	return(1);
    }
    (void) minivasSendChar(MINIVAS_EE); /* Cue the tape */
    return(0);
}

minivasShutdown(sig) {
    if (sig)
	signal(sig, SIG_IGN);
    if (sig) {
	close(minivasTtyFd);
	exit(0);
    }
}

/*
 * Minivas transcript output routine
 */

minivasShowString(announce, string, length)
char *announce;
char *string;
int length;
{
    fprintf(stderr, "%s \"", announce);
    while (length-- > 0) {
	if (*string >= ' ' && *string <= '~')
	    putc(*string, stderr);
	else {
	    switch (*string) {
	    case '\n': putc('\\', stderr); putc('n', stderr); break;
	    case '\r': putc('\\', stderr); putc('r', stderr); break;
	    case '\b': putc('\\', stderr); putc('b', stderr); break;
	    default: fprintf(stderr, "\\%03o", *string); break;
	    }
	}
	string++;
    }
    putc('"', stderr);
    putc('\n', stderr);
}

/*
 * await something from recorder, fault if error received
 */

minivasWaitFor(code) {
    char buf[ BUFSIZ + 1 ];
    char *pSTR;
    int ret;
    int gotIt = 0;

    if (!minivasReceiveOK)
	return(-1);

    if (minivasVerbose)
	fprintf(stderr,"Waiting for 0x%02x from minivas\n", code);

    for (;;) {
	if ((ret = minivasRead(buf, BUFSIZ)) < 0)
	    return(1);
	buf[ret] = '\0';  /* stuff a null after the last character received */
	if (buf[ret - 1] == code) {
	    if (minivasVerbose)
		minivasShowString("minivasWaitFor(): got string=", buf, ret);
	    return(0);
	}
	if (minivasVerbose)
	    minivasShowString("minivasWaitFor() didn't expect string=", buf, ret);
    }
}

/*
 * minivasRead() read chars from minivas with a timeout & error dectection
 */

minivasRead(buf, size)
char *buf;
{
    struct timeval timeout;
    u_long readFds;
    int ret;

    timeout.tv_sec = minivasTimeoutSeconds;
    timeout.tv_usec = 0;
    readFds = 1<<minivasTtyFd;
    if ((ret = select(minivasTtyFd+1, &readFds, NULL, NULL, &timeout)) == 0) {
	minivasReceiveOK = 0;
	fprintf(stderr, "minivasRead(%s) timeout after %d seconds\n",
	    minivasTty, minivasTimeoutSeconds);
	return(-1);
    }
    if (ret < 0) {
	minivasReceiveOK = 0;
	if (errno == EINTR)
	    return(0);
	fprintf(stderr, "minivasRead(%s) select error: %s\n",
	    minivasTty, sys_errlist[errno]);
	return(-1);
    }

    if ((ret = read(minivasTtyFd, buf, BUFSIZ)) < 0) {
	minivasReceiveOK = 0;
	fprintf(stderr, "Read error %s: %s\n", minivasTty,
	    sys_errlist[errno]);
	return(-1);
    }
    if (ret == 0) {
	minivasReceiveOK = 0;
	fprintf(stderr, "got EOF\n");
	return(-1);
    }
    return(ret);
}

/*
 * minivasOpenTty() opens the special file for the minivas and sets
 * the correct modes
 */

minivasOpenTty() {
    int fd;
#ifdef BSD
    struct sgttyb	tty_params;
#else
    struct termio ttynew;
#endif /* BSD */

    if (strcmp(minivasTty, "none") == 0)
	return(-1);
    signal(SIGINT, minivasShutdown);
    signal(SIGQUIT, minivasShutdown);

#ifdef BSD
    fd = open(minivasTty, O_NDELAY | O_RDWR | O_EXCL, 0777 );
#else
    fd = open(minivasTty, O_RDWR, 0);
#endif /* BSD*/

    if (fd < 0) {
	fprintf(stderr, "Cannot open %s: %s\n", minivasTty,
	    sys_errlist[errno]);
	return(-1);
    }
    
#ifdef BSD
    if (gtty(&tty_params) == -1) {
	fprintf(stderr,"minivasOpenTty: gtty failed: %s\n", sys_errlist[errno]);
	return(-1);
    }
    tty_params.sg_ospeed = B9600;	/* the system setting */
    tty_params.sg_ispeed = B9600;
    tty_params.sg_flags = RAW|TANDEM; /* default: no processing (no parity) */
    if (stty(fd,&tty_params) == -1) {
	fprintf(stderr,"minivasOpenTty: stty failed; %s\n", sys_errlist[errno]);
	return(-1);
    }
#else
    if (ioctl(fd, TCGETA, &ttynew) < 0) {
	fprintf(stderr, "TCGETA error: %s\n", sys_errlist[errno]);
	return(-1);
    }
    ttynew.c_lflag &= ~( ICANON | ECHO );
    ttynew.c_cflag = (unsigned short) (B9600 | CS8 | CLOCAL | CREAD);
    ttynew.c_cc[4] = 1; /* Set to return after single char entered */
    if (ioctl(fd, TCSETAW, &ttynew ) < 0) {
	fprintf(stderr, "TCSETAW error: %s\n", sys_errlist[errno]);
	return(-1);
    }
#endif /* BSD */
    minivasSendOK = 1;
    minivasReceiveOK = 1;
    return (fd);
}


minivasSendChar(c)
    unsigned char c;
{
    int wrote;
    struct timeval timeout;
    int nFound;
    u_long writeFds;

    if (!minivasSendOK)
	return;

    if (minivasVerbose)
	fprintf(stderr,"Sending 0x%02x to minivas\n", c);

    timeout.tv_sec = minivasTimeoutSeconds;
    timeout.tv_usec = 0;
    writeFds = 1<<minivasTtyFd;
    errno = 0;
    nFound = select(minivasTtyFd+1, NULL, &writeFds, NULL, &timeout);
    if (nFound <= 0) {
	minivasSendOK = 0;
	if (errno == EINTR) {
	    if (minivasVerbose)
		fprintf(stderr, "select on write interrupted\n");
	    return(1);
	}
	fprintf(stderr,
	    "Cannot send character/command to %s after %d seconds\n",
	    minivasTty, minivasTimeoutSeconds);
	return(1);
    }
    if ((wrote = write(minivasTtyFd, &c, 1)) < 1) {
	minivasSendOK = 0;
	if (wrote == 0) {
	    fprintf(stderr, "Zero byte write on %s\n", minivasTty);
	    return(1);
	}
	fprintf(stderr, "Write error on %s: %s\n", minivasTty,
	    sys_errlist[errno]);
	return(1);
    }
    return(0);
}

/*
 * Minivas speicialists.
 */

minivasSendCmd(cmd, resp)
char cmd, resp;
{
    int retval;

    (void) minivasSendChar(cmd);
    /*
     * NULL resp means no device input is expected.
     */
    if((resp != NULL) && (retval = minivasWaitFor(resp)))
        return(retval);
    else
        return(NULL);
}

/*
 * Output a decimal number to the minivas
 */

minivasSendNum(c_num)
int c_num;
{
    char buf[32];
    char *pSTR = buf;

    sprintf(buf, "%d", c_num);
    while (*pSTR != '\0') {
        (void) minivasSendChar(*pSTR, NULL);
        pSTR++;
    }
}

/*
 * Output 8 digits of timecode to the minivas using the drop-frame conventions
 * for converting from frame number.
 */

minivasSendTC(frame) {
    int hours, min, sec;

    hours = frame / ((2997 * 60 * 60) / 100);
    frame %= ((2997 * 60 * 60) / 100);
    min = (frame / 17982) * 10;
    frame %= 17982;			/* 17982 is nbr frames in 10 min */
    if (frame >= 1800) {		/* frames in 1min at 30 Hz */
	/*
	 * Drop 2 frames per minute, except every 10th minute.  Codes
	 * hh:mm:00:00 & hh:mm:00:01 never occur if mm is not a multiple
	 * of 10.  This will result in 108 frames per hour less than 30 Hz.
	 */
	frame -= 2;
	min += frame / (1800 - 2);
	frame %= (1800 - 2);
	frame += 2;
    }
    sec = frame / 30;
    frame %= 30;
    if ((hours >= 24) || (hours < 0)) {
	fprintf(stderr, "Time code hours (%d) too big\n", hours);
	return(1);
    }
    (void) minivasSendChar(hours / 10 + '0');
    (void) minivasSendChar(hours % 10 + '0');
    (void) minivasSendChar(min / 10 + '0');
    (void) minivasSendChar(min % 10 + '0');
    (void) minivasSendChar(sec / 10 + '0');
    (void) minivasSendChar(sec % 10 + '0');
    (void) minivasSendChar(frame / 10 + '0');
    (void) minivasSendChar(frame % 10 + '0');
    return(0);
}
