/*
 * Copyright (c) 1992,1993, North Valley Research, Inc.
 * Confidential -- All proprietary rights reserved.
 * Licensed Material -- Property of North Valley Research, Inc.
 *
 * This program is the property of North Valley Research, Inc. or others
 * from whom North Valley Research has obtained a licensing right, and is
 * considered by North Valley Research to be confidential.  It is protected
 * by U.S. copyright law as an unpublished work and is
 * furnished pursuant to a written license agreement.  It may
 * not be used, copied or otherwise reproduced, or disclosed to
 * others except in accordance with the terms and conditions of
 * that agreement.
 *
 * NAME
 *	<file name> - <one-line description of file contents>
 *
 * DESCRIPTION
 *	<overall description of what the c code does>
 *
 * $Header: /usr/local/src/nvrToMIT/RCS/zerorcsid.c,v 1.1 1993/02/08 01:59:47 toddb Exp $
 */
#include "types.h"
#include "alloc.h"
#include "fileAccess.h"
#include "osio.h"
#include <stdio.h>

#define BUFFERSIZE (8*1024)

typedef struct {
    char *data;
    int totalLength;
    int dataLength;
} BufferInfo, *BufferInfoPtr;

char *ProgramName;

char *RcsIds[] = {
    "Author",
    "Date",
    "Header",
    "Id",
    "Locker",
    "Log",
    "RCSfile",
    "Revision",
    "Source",
    "State",
};

static boolean
WasRcsVar(char *data, int length)
{
    char *pFirst, *pLast, *pEnd;
    int i, j;

    pEnd = data + length;
    pFirst = data+1;
    for (pLast = pFirst; pLast != pEnd; pLast++)
	if (*pLast == '$' || *pLast == ':')
	     break;
    if (pFirst == pLast)
	return(False);
    for (i = 0; i < NumberOfCells(RcsIds); i++) {
	if (strncmp(RcsIds[i], pFirst, pLast-pFirst) == 0) {
	    Msg("Zero id: \"%s\"\n", data);
	    for (j = 0; j < length; j++)
		data[ j ] = ' ';
	    return(True);
	}
    }
    return(False);
}

static void
StretchBuf(BufferInfoPtr buf, int dataLength)
{
    if (dataLength > buf->totalLength) {
	buf->totalLength += BUFFERSIZE;
	buf->data = NullRealloc(buf->data, buf->totalLength);
    }
}

static BufferInfoPtr 
GetPossibleRcsVar(FILE *fpIn)
{
    static BufferInfo buf;
    int length = 1;
    int c;
    char *pStart, *pEnd;

    StretchBuf(&buf, BUFFERSIZE);
    pEnd = buf.data;
    *pEnd++ = '$';
    while ((c = getc(fpIn)) >= 0) {
	if (++length >= buf.totalLength) {
	    StretchBuf(&buf, length+1);
	    pEnd = buf.data + length - 1;
	}
	*pEnd++ = c;
	if (c == '$') {
	    *pEnd = '\0';
	    if (WasRcsVar(buf.data, length))
		break;
	    else {
		length--;
		break;
	    }
	} else if (! isprint(c)) {
	    break;
	}
    }

    buf.dataLength = length;
    return (&buf);
}

static void
ZeroRcsIds(char *fileName)
{
    FILE *fpIn, *fpOut;
    int fileType;
    int c;
    BufferInfoPtr buf;
    char newFile[ BUFFERSIZE ];

    switch (fileType = FileType(fileName)) {
    case FILE_REGULAR:
    case FILE_REGULAR | FILE_SYMLINK:
	break;
    default:
	Error("%s is a %s, not a regular file\n",
	    fileName, TypeName(fileType));
	return;
    }

    if ((fpIn = fopen(fileName, "r")) == NULL) {
	Error("can't open %s for reading\n", fileName);
	return;
    }

    sprintf(newFile, "%s.new", fileName);
    if ((fpOut = fopen(newFile, "w")) == NULL) {
	Error("can't open %s for writing\n", newFile);
	goto doneWithFpIn;
    }

    while ((c = getc(fpIn)) >= 0) {
	if (c == '$') {
	    buf = GetPossibleRcsVar(fpIn);
	    if (fwrite(buf->data, buf->dataLength, 1, fpOut) != 1) {
		Error("Can't write %d bytes to %s\n", newFile);
		goto doneWithAll;
	    }
	} else {
	    if (putc(c, fpOut) < 0) {
		Error("Can't write to %s\n", newFile);
		goto doneWithAll;
	    }
	}
    }

    if (unlink(fileName) < 0) {
	Error("can't unlink %s\n", fileName);
	goto doneWithAll;
    }
    if (rename(newFile, fileName) < 0)
	Error("can't rename %s to %s\n", newFile, fileName);

doneWithAll:
    (void) unlink(newFile);
    (void) fclose(fpOut);
doneWithFpIn:
    (void) fclose(fpIn);
    return;
}

main(int argc, char **argv)
{
    ProgramName = *argv;

    for (argc--, argv++; argc; argc--, argv++) {
	ZeroRcsIds(*argv);
    }
}
